@@ -5,7 +5,7 @@
  * Hierarchical layout constraint system
  * Written by: Steven M. Rubin, Static Free Software
  *
- * Copyright (c) 2000 Static Free Software
+ * Copyright (c) 2001 Static Free Software
  *
  * Electric(tm) is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -102,7 +102,7 @@ void cla_layconsetmode(INTSML count, char *par[])
 
 	if (el_curconstraint != cla_constraint)
 	{
-		ttyputerr(_("Must first switch to this solver with 'constraint use'"));
+		ttyputerr(M_("Must first switch to this solver with 'constraint use'"));
 		return;
 	}
 
@@ -113,10 +113,10 @@ void cla_layconsetmode(INTSML count, char *par[])
 	{
 #ifdef DEBUG
 		cla_conlaydebug = 1 - cla_conlaydebug;
-		if (cla_conlaydebug != 0) ttyputmsg(_("Layout constraint debugging on")); else
-			ttyputmsg(_("Layout constraint debugging off"));
+		if (cla_conlaydebug != 0) ttyputmsg(M_("Layout constraint debugging on")); else
+			ttyputmsg(M_("Layout constraint debugging off"));
 #else
-		ttyputmsg(_("Sorry, constraint debugging code has been compiled out"));
+		ttyputmsg(M_("Sorry, constraint debugging code has been compiled out"));
 #endif
 		return;
 	}
@@ -175,6 +175,7 @@ void cla_layconsolve(NODEPROTO *np)
 				cla_computefacet(cc->changefacet, cc->forcedlook);
 	}
 
+	if (curbatch == NOCHANGEBATCH) return;
 	for(c = curbatch->changehead; c != NOCHANGE; c = c->nextchange)
 		switch (c->changetype)
 	{
@@ -201,9 +202,82 @@ void cla_layconsolve(NODEPROTO *np)
 	}
 }
 
-void cla_layconnewobject(INTBIG addr, INTBIG type) {}
+/*
+ * If an export is created, touch all instances of the facet
+ */
+void cla_layconnewobject(INTBIG addr, INTBIG type)
+{
+	REGISTER NODEPROTO *np;
+	REGISTER PORTPROTO *pp;
+	REGISTER NODEINST *ni;
 
-void cla_layconkillobject(INTBIG addr, INTBIG type) {}
+	if (type == VPORTPROTO)
+	{
+		pp = (PORTPROTO *)addr;
+		np = pp->parent;
+		for(ni = np->firstinst; ni != NONODEINST; ni = ni->nextinst)
+		{
+			(void)db_change((INTBIG)ni, OBJECTSTART, VNODEINST, 0, 0, 0, 0, 0);
+			(void)db_change((INTBIG)ni, NODEINSTMOD, ni->lowx, ni->lowy,
+				ni->highx, ni->highy, ni->rotation, ni->transpose);
+			(void)db_change((INTBIG)ni, OBJECTEND, VNODEINST, 0, 0, 0, 0, 0);
+		}
+	}
+}
+
+/*
+ * If an export is deleted, touch all instances of the facet
+ */
+void cla_layconkillobject(INTBIG addr, INTBIG type)
+{
+	REGISTER NODEPROTO *np;
+	REGISTER PORTPROTO *pp;
+	REGISTER NODEINST *ni;
+
+	if (type == VPORTPROTO)
+	{
+		pp = (PORTPROTO *)addr;
+		np = pp->parent;
+		for(ni = np->firstinst; ni != NONODEINST; ni = ni->nextinst)
+		{
+			(void)db_change((INTBIG)ni, OBJECTSTART, VNODEINST, 0, 0, 0, 0, 0);
+			(void)db_change((INTBIG)ni, NODEINSTMOD, ni->lowx, ni->lowy,
+				ni->highx, ni->highy, ni->rotation, ni->transpose);
+			(void)db_change((INTBIG)ni, OBJECTEND, VNODEINST, 0, 0, 0, 0, 0);
+		}
+	}
+}
+
+/*
+ * If an export is renamed, touch all instances of the facet
+ */
+void cla_layconnewvariable(INTBIG addr, INTBIG type, INTBIG skey, INTBIG stype)
+{
+	char *name;
+	REGISTER NODEPROTO *np;
+	REGISTER PORTPROTO *pp;
+	REGISTER NODEINST *ni;
+
+	if (type == VPORTPROTO)
+	{
+		if ((stype&VCREF) != 0)
+		{
+			name = changedvariablename(type, skey, stype);
+			if (strcmp(name, "protoname") == 0)
+			{
+				pp = (PORTPROTO *)addr;
+				np = pp->parent;
+				for(ni = np->firstinst; ni != NONODEINST; ni = ni->nextinst)
+				{
+					(void)db_change((INTBIG)ni, OBJECTSTART, VNODEINST, 0, 0, 0, 0, 0);
+					(void)db_change((INTBIG)ni, NODEINSTMOD, ni->lowx, ni->lowy,
+						ni->highx, ni->highy, ni->rotation, ni->transpose);
+					(void)db_change((INTBIG)ni, OBJECTEND, VNODEINST, 0, 0, 0, 0, 0);
+				}
+			}
+		}
+	}
+}
 
 /*
  * set layout constraints on arc instance "ai" according to "changetype".
@@ -280,7 +354,6 @@ void cla_layconmodifyportproto(PORTPROTO *pp, NODEINST *oni, PORTPROTO *opp) {}
 void cla_layconmodifynodeproto(NODEPROTO *np) {}
 void cla_layconnewlib(LIBRARY *lib) {}
 void cla_layconkilllib(LIBRARY *lib) {}
-void cla_layconnewvariable(INTBIG addr, INTBIG type, INTBIG skey, INTBIG stype) {}
 void cla_layconkillvariable(INTBIG addr, INTBIG type, INTBIG key, INTBIG saddr,
 	INTBIG stype, INTBIG sdes) {}
 void cla_layconmodifyvariable(INTBIG addr, INTBIG type, INTBIG key, INTBIG stype,
@@ -289,7 +362,7 @@ void cla_layconinsertvariable(INTBIG addr, INTBIG type, INTBIG key, INTBIG ainde
 void cla_laycondeletevariable(INTBIG addr, INTBIG type, INTBIG key, INTBIG aindex,
 	INTBIG oldval) {}
 void cla_layconsetvariable(void) {}
-void cla_layconmodifydescript(INTBIG addr, INTBIG type, VARIABLE *var, INTBIG olddes) {}
+void cla_layconmodifydescript(INTBIG addr, INTBIG type, INTBIG key, INTBIG olddes) {}
 
 /******************** NODE MODIFICATION CODE *************************/
 
@@ -347,7 +420,7 @@ INTSML cla_modifynodeinst(NODEINST *ni, INTBIG deltalx, INTBIG deltaly, INTBIG d
 
 #ifdef DEBUG
 	if (cla_conlaydebug)
-		ttyputmsg(_("Change node %s by X(%s %s) Y(%s %s) r=%d t=%d"),
+		ttyputmsg(M_("Change node %s by X(%s %s) Y(%s %s) r=%d t=%d"),
 			describenodeinst(ni), latoa(deltalx), latoa(deltahx), latoa(deltaly),
 				latoa(deltahy), dangle, dtrans);
 #endif
@@ -459,7 +532,7 @@ void cla_modwithin(NODEINST *ni, INTSML dangle, INTSML dtrans)
 		xform(ai->end[1].xpos-ox, ai->end[1].ypos-oy, &onox, &onoy, trans);
 #ifdef	DEBUG
 		if (cla_conlaydebug)
-			ttyputmsg(_("Internal arc %s moves 0:%s,%s  1:%s,%s"),
+			ttyputmsg(M_("Internal arc %s moves 0:%s,%s  1:%s,%s"),
 				describearcinst(ai), latoa(nox), latoa(noy), latoa(onox), latoa(onoy));
 #endif
 		/* move the arcinst */
@@ -553,7 +626,7 @@ INTSML cla_modrigid(NODEINST *ni, INTSML dangle, INTSML dtrans)
 		opt = ai->end[thatend].portarcinst->proto;
 #ifdef	DEBUG
 		if (cla_conlaydebug)
-			ttyputmsg(_("Modify rigid arc %s from %s to %s"),
+			ttyputmsg(M_("Modify rigid arc %s from %s to %s"),
 				describearcinst(ai), describenodeinst(ni), describenodeinst(ono));
 #endif
 
@@ -562,7 +635,7 @@ INTSML cla_modrigid(NODEINST *ni, INTSML dangle, INTSML dtrans)
 			&ay[thatend], trans);
 #ifdef	DEBUG
 		if (cla_conlaydebug)
-			ttyputmsg(_("Other end of arc moves to (%s,%s)"),
+			ttyputmsg(M_("Other end of arc moves to (%s,%s)"),
 				latoa(ax[thatend]), latoa(ay[thatend]));
 #endif
 
@@ -608,7 +681,7 @@ INTSML cla_modrigid(NODEINST *ni, INTSML dangle, INTSML dtrans)
 		/* move the arcinst */
 #ifdef	DEBUG
 		if (cla_conlaydebug)
-			ttyputmsg(_("Now arc runs from (%s,%s) to (%s,%s)"),
+			ttyputmsg(M_("Now arc runs from (%s,%s) to (%s,%s)"),
 				latoa(ax[0]), latoa(ay[0]), latoa(ax[1]), latoa(ay[1]));
 #endif
 		cla_domovearcinst(ai, ax[0],ay[0], ax[1],ay[1], 0);
@@ -698,7 +771,7 @@ INTSML cla_modflex(NODEINST *ni, INTSML dangle, INTSML dtrans)
 #ifdef	DEBUG
 		if (cla_conlaydebug)
 		{
-			ttyputmsg(_("Considering arc %s (clock=%d, curclock=%d)"),
+			ttyputmsg(M_("Considering arc %s (clock=%d, curclock=%d)"),
 				describearcinst(ai), ai->changed, cla_changeclock+1);
 		}
 #endif
@@ -730,7 +803,6 @@ INTSML cla_modflex(NODEINST *ni, INTSML dangle, INTSML dtrans)
 		xform(ai->end[thisend].xpos-ox, ai->end[thisend].ypos-oy,
 			&ax[thisend], &ay[thisend], trans);
 
-#if 1
 		/* make sure the arc end is still in the port */
 		if (poly == NOPOLYGON) poly = allocstaticpolygon(4, db_cluster);
 		shapeportpoly(ai->end[thisend].nodeinst, ai->end[thisend].portarcinst->proto,
@@ -756,7 +828,6 @@ INTSML cla_modflex(NODEINST *ni, INTSML dangle, INTSML dtrans)
 				closestpoint(poly, &ax[thisend], &ay[thisend]);
 			}
 		}
-#endif
 
 		/* get other end of arcinst and its position */
 		ono = ai->end[thatend].nodeinst;
@@ -766,16 +837,13 @@ INTSML cla_modflex(NODEINST *ni, INTSML dangle, INTSML dtrans)
 #ifdef	DEBUG
 		if (cla_conlaydebug)
 		{
-			ttyputmsg(_("Modify flexible arc %s from %s to %s"),
+			ttyputmsg(M_("Modify flexible arc %s from %s to %s"),
 				describearcinst(ai), describenodeinst(ni), describenodeinst(ono));
 		}
 #endif
 
 		/* see if other nodeinst has changed */
 		mangle = 1;
-#if 0
-		if (ono->changed >= cla_changeclock) mangle = 0;
-#endif
 		if ((ai->userbits&FIXANG) == 0) mangle = 0;
 		if (ono->proto->primindex == 0)
 		{
@@ -816,7 +884,7 @@ INTSML cla_modflex(NODEINST *ni, INTSML dangle, INTSML dtrans)
 
 #ifdef	DEBUG
 					if (cla_conlaydebug)
-						ttyputmsg(_("  Moving other end of arc to (%s,%s), other node by (%s,0)"),
+						ttyputmsg(M_("  Moving other end of arc to (%s,%s), other node by (%s,0)"),
 							latoa(ax[thatend]), latoa(ay[thatend]), latoa(dx-odx));
 #endif
 					if (dx != odx)
@@ -847,7 +915,7 @@ INTSML cla_modflex(NODEINST *ni, INTSML dangle, INTSML dtrans)
 
 #ifdef	DEBUG
 				if (cla_conlaydebug)
-					ttyputmsg(_("  Moving other end of arc to (%s,%s), other node by (0,%s)"),
+					ttyputmsg(M_("  Moving other end of arc to (%s,%s), other node by (0,%s)"),
 						latoa(ax[thatend]), latoa(ay[thatend]), latoa(dy-ody));
 #endif
 				if (dy != ody)
@@ -883,7 +951,7 @@ INTSML cla_modflex(NODEINST *ni, INTSML dangle, INTSML dtrans)
 		/* other node has changed or arc is funny, just use its position */
 #ifdef	DEBUG
 		if (cla_conlaydebug)
-			ttyputmsg(_("  ! Moving arc %s to (%s,%s)-(%s,%s)"), describearcinst(ai),
+			ttyputmsg(M_("  ! Moving arc %s to (%s,%s)-(%s,%s)"), describearcinst(ai),
 				latoa(ax[0]), latoa(ay[0]), latoa(ax[1]), latoa(ay[1]));
 #endif
 		cla_domovearcinst(ai, ax[0],ay[0], ax[1],ay[1], 1);
@@ -972,7 +1040,7 @@ void cla_ensurearcinst(ARCINST *ai, INTSML arctyp)
 		getcenter(poly1, &tx, &ty);
 #ifdef	DEBUG
 		if (cla_conlaydebug)
-			ttyputmsg(_("Ensuring rigid arc %s"), describearcinst(ai));
+			ttyputmsg(M_("Ensuring rigid arc %s"), describearcinst(ai));
 #endif
 		cla_domovearcinst(ai, fx, fy, tx, ty, arctyp);
 		return;
@@ -992,7 +1060,7 @@ void cla_ensurearcinst(ARCINST *ai, INTSML arctyp)
 		closestpoint(poly1, &tx, &ty);
 #ifdef	DEBUG
 		if (cla_conlaydebug)
-			ttyputmsg(_("Ensuring manhattan arc %s"), describearcinst(ai));
+			ttyputmsg(M_("Ensuring manhattan arc %s"), describearcinst(ai));
 #endif
 		cla_domovearcinst(ai, fx, fy, tx, ty, arctyp);
 		return;
@@ -1006,7 +1074,7 @@ void cla_ensurearcinst(ARCINST *ai, INTSML arctyp)
 		closestpoint(poly1, &tx, &ty);
 #ifdef	DEBUG
 		if (cla_conlaydebug)
-			ttyputmsg(_("Ensuring manhattan arc %s"), describearcinst(ai));
+			ttyputmsg(M_("Ensuring manhattan arc %s"), describearcinst(ai));
 #endif
 		cla_domovearcinst(ai, fx, fy, tx, ty, arctyp);
 		return;
@@ -1036,7 +1104,7 @@ void cla_updatearc(ARCINST *ai, INTBIG fx, INTBIG fy, INTBIG tx, INTBIG ty, INTS
 	/* set the proper arcinst position */
 #ifdef DEBUG
 	if (cla_conlaydebug)
-		ttyputmsg(_("Change arc %s from (%s,%s)-(%s,%s) to (%s,%s)-(%s,%s)"),
+		ttyputmsg(M_("Change arc %s from (%s,%s)-(%s,%s) to (%s,%s)-(%s,%s)"),
 			describearcinst(ai), latoa(ai->end[0].xpos), latoa(ai->end[0].ypos),
 				latoa(ai->end[1].xpos), latoa(ai->end[1].ypos),
 					latoa(fx), latoa(fy), latoa(tx), latoa(ty));
@@ -1092,7 +1160,7 @@ void cla_domovearcinst(ARCINST *ai, INTBIG fx, INTBIG fy, INTBIG tx, INTBIG ty,
 #ifdef	DEBUG
 	if (cla_conlaydebug)
 	{
-		ttyputmsg(_("Jogging arc %s (0%o) to run from (%s,%s) to (%s,%s)"),
+		ttyputmsg(M_("Jogging arc %s (0%o) to run from (%s,%s) to (%s,%s)"),
 			describearcinst(ai), ai, latoa(fx), latoa(fy), latoa(tx), latoa(ty));
 	}
 #endif
@@ -1115,10 +1183,10 @@ void cla_domovearcinst(ARCINST *ai, INTBIG fx, INTBIG fy, INTBIG tx, INTBIG ty,
 		oldxA = fx;   oldxB = tx;
 		lx = oldxB - psx/2; hx = lx + psx;
 		ly = oldyB - psy/2; hy = ly + psy;
-		no1 = db_newnodeinst(np, lx,hx, ly,hy, 0, 0, pnt);
+		no1 = db_newnodeinst(np, lx, hx, ly, hy, 0, 0, pnt);
 		lx = oldxA - psx/2; hx = lx + psx;
 		ly = oldyA - psy/2; hy = ly + psy;
-		no2 = db_newnodeinst(np, lx,hx, ly,hy, 0, 0, pnt);
+		no2 = db_newnodeinst(np, lx, hx, ly, hy, 0, 0, pnt);
 	} else
 	{
 		/* assume horizontal arcinst */
@@ -1126,10 +1194,10 @@ void cla_domovearcinst(ARCINST *ai, INTBIG fx, INTBIG fy, INTBIG tx, INTBIG ty,
 		oldxA = oldxB = (tx+fx) / 2;
 		lx = oldxB - psx/2; hx = lx + psx;
 		ly = oldyB - psy/2; hy = ly + psy;
-		no1 = db_newnodeinst(np, lx,hx, ly,hy, 0, 0, pnt);
+		no1 = db_newnodeinst(np, lx, hx, ly, hy, 0, 0, pnt);
 		lx = oldxA - psx/2; hx = lx + psx;
 		ly = oldyA - psy/2; hy = ly + psy;
-		no2 = db_newnodeinst(np, lx,hx, ly,hy, 0, 0, pnt);
+		no2 = db_newnodeinst(np, lx, hx, ly, hy, 0, 0, pnt);
 	}
 	if (no1 == NONODEINST || no2 == NONODEINST)
 	{
@@ -1203,9 +1271,9 @@ void cla_adjustmatrix(NODEINST *ni, PORTPROTO *pp, XARRAY trans)
 #ifdef	DEBUG
 		if (cla_conlaydebug)
 		{
-			ttyputmsg(_("Node %s port %s was at (%s,%s), is at (%s,%s)"), describenodeinst(ni),
+			ttyputmsg(M_("Node %s port %s was at (%s,%s), is at (%s,%s)"), describenodeinst(ni),
 				pp->protoname, latoa(onox), latoa(onoy), latoa(dx), latoa(dy));
-			ttyputmsg(_("    Node moved (%s,%s)"), latoa(ox), latoa(oy));
+			ttyputmsg(M_("    Node moved (%s,%s)"), latoa(ox), latoa(oy));
 		}
 #endif
 		trans[2][0] = dx - onox + ox;   trans[2][1] = dy - onoy + oy;
@@ -1370,7 +1438,7 @@ void cla_computefacet(NODEPROTO *facet, INTSML forcedlook)
 
 #ifdef	DEBUG
 	if (cla_conlaydebug)
-		ttyputmsg(_("In computefacet on facet %s (fl=%d)"),
+		ttyputmsg(M_("In computefacet on facet %s (fl=%d)"),
 			facet->cell->cellname, forcedlook);
 #endif
 	/* get current boundary of facet */
@@ -1403,7 +1471,7 @@ void cla_computefacet(NODEPROTO *facet, INTSML forcedlook)
 	mixed = 0;
 	oneparent = NONODEPROTO;
 	lni = NONODEINST;
-	for(ni=facet->firstinst; ni != NONODEINST; ni = ni->nextinst)
+	for(ni = facet->firstinst; ni != NONODEINST; ni = ni->nextinst)
 	{
 		oneparent = ni->parent;
 		if (lni != NONODEINST) if (ni->parent != lni->parent) mixed++;
@@ -1418,12 +1486,12 @@ void cla_computefacet(NODEPROTO *facet, INTSML forcedlook)
 	{
 #ifdef	DEBUG
 		if (cla_conlaydebug)
-			ttyputmsg(_("   Recomputing uniform parents of facet %s"),
+			ttyputmsg(M_("   Recomputing uniform parents of facet %s"),
 				describenodeproto(facet));
 #endif
 		dlx = facet->lowx - flx;   dhx = facet->highx - fhx;
 		dly = facet->lowy - fly;   dhy = facet->highy - fhy;
-		for(ni=facet->firstinst; ni != NONODEINST; ni = ni->nextinst)
+		for(ni = facet->firstinst; ni != NONODEINST; ni = ni->nextinst)
 		{
 			makeangle(ni->rotation, ni->transpose, trans);
 			xform(dhx+dlx, dhy+dly, &offx, &offy, trans);
@@ -1431,7 +1499,7 @@ void cla_computefacet(NODEPROTO *facet, INTSML forcedlook)
 			nly = (dly-dhy+offy)/2;  nhy = offy - nly;
 			if (cla_modifynodeinst(ni, nlx, nly, nhx, nhy, 0, 0, 1) != 0) forcedlook++;
 		}
-		for(ni=facet->firstinst; ni != NONODEINST; ni = ni->nextinst)
+		for(ni = facet->firstinst; ni != NONODEINST; ni = ni->nextinst)
 			if (cla_modnodearcs(ni, 0, 0)) forcedlook++;
 		cla_computefacet(oneparent, forcedlook);
 		return;
@@ -1443,7 +1511,7 @@ void cla_computefacet(NODEPROTO *facet, INTSML forcedlook)
 	 */
 #ifdef	DEBUG
 	if (cla_conlaydebug)
-		ttyputmsg(_("   Performing complex tree examination"));
+		ttyputmsg(M_("   Performing complex tree examination"));
 #endif
 	for(lib = el_curlib; lib != NOLIBRARY; lib = lib->nextlibrary)
 		for(np = lib->firstnodeproto; np != NONODEPROTO; np = np->nextnodeproto)
@@ -1514,7 +1582,7 @@ INTSML cla_lookdown(NODEPROTO *start)
 	}
 #ifdef	DEBUG
 	if (cla_conlaydebug)
-		ttyputmsg(_("      Complex tree search at facet %s"), describenodeproto(start));
+		ttyputmsg(M_("      Complex tree search at facet %s"), describenodeproto(start));
 #endif
 	/* modify the nodes in this facet that changed */
 	forcedlook = 0;
