@@ -5,7 +5,7 @@
  * Linear inequality constraint system
  * Written by: Steven M. Rubin, Static Free Software
  *
- * Copyright (c) 2000 Static Free Software.
+ * Copyright (c) 2001 Static Free Software.
  *
  * Electric(tm) is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -75,7 +75,7 @@ char       *cli_linconops[] = {">=", "==", "<="};
 static INTSML cli_conlindebug;
 static INTSML cli_didsolve;
 
-#define	NOCONNODE ((CONNODE *)-1)
+#define NOCONNODE ((CONNODE *)-1)
 typedef struct Iconnode
 {
 	NODEINST *realn;		/* actual node that is constrained */
@@ -86,7 +86,7 @@ typedef struct Iconnode
 	struct Iconnode *nextconnode;/* next in list of constraint nodes */
 } CONNODE;
 
-#define	NOCONARC ((CONARC *)-1)
+#define NOCONARC ((CONARC *)-1)
 typedef struct Iconarc
 {
 	ARCINST *reala;		/* actual arc that is constrained */
@@ -217,7 +217,7 @@ void cli_linconsetmode(INTSML count, char *par[])
 
 	if (el_curconstraint != cli_constraint)
 	{
-		ttyputerr(_("Must first switch to this solver with 'constraint use'"));
+		ttyputerr(M_("Must first switch to this solver with 'constraint use'"));
 		return;
 	}
 
@@ -226,8 +226,8 @@ void cli_linconsetmode(INTSML count, char *par[])
 	if (namesamen(pp, "debug-toggle", l) == 0 && l >= 1)
 	{
 		cli_conlindebug = 1 - cli_conlindebug;
-		if (cli_conlindebug != 0) ttyputverbose(_("Linear constraint debugging on")); else
-			ttyputverbose(_("Linear constraint debugging off"));
+		if (cli_conlindebug != 0) ttyputverbose(M_("Linear constraint debugging on")); else
+			ttyputverbose(M_("Linear constraint debugging off"));
 		return;
 	}
 
@@ -244,7 +244,7 @@ void cli_linconsetmode(INTSML count, char *par[])
 			np = getcurfacet();
 			if (np == NONODEPROTO)
 			{
-				ttyputerr(_("Must have a current facet to generate names"));
+				ttyputerr(M_("Must have a current facet to generate names"));
 				return;
 			}
 			cli_nameallnodes(np);
@@ -253,14 +253,14 @@ void cli_linconsetmode(INTSML count, char *par[])
 		if (namesamen(pp, "go", l) == 0 && l >= 1)
 		{
 			np = getcurfacet();
-			if (np == NONODEPROTO) ttyputverbose(_("Enabling text/graphics association")); else
+			if (np == NONODEPROTO) ttyputverbose(M_("Enabling text/graphics association")); else
 			{
 				if (cli_texton != 0)
 				{
 					if (np == cli_curfacet)
-						ttyputverbose(_("Reevaluating textual description")); else
-							ttyputverbose(_("Switching association to facet %s"), describenodeproto(np));
-				} else ttyputverbose(_("Text window will mirror facet %s"), describenodeproto(np));
+						ttyputverbose(M_("Reevaluating textual description")); else
+							ttyputverbose(M_("Switching association to facet %s"), describenodeproto(np));
+				} else ttyputverbose(M_("Text window will mirror facet %s"), describenodeproto(np));
 			}
 			cli_maketextfacet(np);
 			return;
@@ -270,7 +270,7 @@ void cli_linconsetmode(INTSML count, char *par[])
 			geom = (GEOM *)askaid(us_aid, "get-object");
 			if (geom == NOGEOM)
 			{
-				ttyputerr(_("Find a single object first"));
+				ttyputerr(M_("Find a single object first"));
 				return;
 			}
 			cli_highlightequivalent(geom);
@@ -286,13 +286,13 @@ void cli_linconsetmode(INTSML count, char *par[])
 		if (namesame(pp, "on") == 0)
 		{
 			cli_manhattan = 1;
-			ttyputverbose(_("Manhattan arcs will be kept that way"));
+			ttyputverbose(M_("Manhattan arcs will be kept that way"));
 			return;
 		}
 		if (namesamen(pp, "off", l) == 0 && l >= 2)
 		{
 			cli_manhattan = 0;
-			ttyputverbose(_("No assumptions will be made about Manhattan arcs"));
+			ttyputverbose(M_("No assumptions will be made about Manhattan arcs"));
 			return;
 		}
 		ttyputusage("constraint tell linear implicit-manhattan (on|off)");
@@ -306,7 +306,7 @@ void cli_linconsetmode(INTSML count, char *par[])
 			np = getcurfacet();
 			if (np == NONODEPROTO)
 			{
-				ttyputerr(_("No facet specified and no current facet"));
+				ttyputerr(M_("No facet specified and no current facet"));
 				return;
 			}
 		} else
@@ -314,12 +314,12 @@ void cli_linconsetmode(INTSML count, char *par[])
 			np = getnodeproto(par[1]);
 			if (np == NONODEPROTO)
 			{
-				ttyputerr(_("No facet named %s"), par[1]);
+				ttyputerr(M_("No facet named %s"), par[1]);
 				return;
 			}
 			if (np->primindex != 0)
 			{
-				ttyputerr(_("Must solve facets, not primitives"));
+				ttyputerr(M_("Must solve facets, not primitives"));
 				return;
 			}
 		}
@@ -392,7 +392,7 @@ void cli_dosolve(NODEPROTO *facet)
 	REGISTER INTBIG dlx, dhx, dly, dhy;
 
 	if (cli_conlindebug != 0)
-		ttyputmsg(_("Re-checking linear constraints for facet %s"), describenodeproto(facet));
+		ttyputmsg(M_("Re-checking linear constraints for facet %s"), describenodeproto(facet));
 	if (cli_didsolve == 0) cli_solvefacet(facet, 0, 0);
 
 	/* get current boundary of facet */
@@ -485,7 +485,7 @@ void cli_linconmodifynodeinst(NODEINST *ni, INTBIG dlx, INTBIG dly, INTBIG dhx,
 		oldhx, oldhy, oldang, oldtrans);
 
 	/* look at all of the arcs on this nodeinst */
-	for(pi=ni->firstportarcinst; pi!=NOPORTARCINST; pi=pi->nextportarcinst)
+	for(pi = ni->firstportarcinst; pi != NOPORTARCINST; pi = pi->nextportarcinst)
 	{
 		ai = pi->conarcinst;
 
@@ -587,14 +587,14 @@ INTSML cli_linconsetobject(INTBIG addr, INTBIG type, INTBIG changetype, INTBIG c
 	if (changetype == 3)
 	{
 		var = getvalkey((INTBIG)ai, VARCINST, VINTEGER|VISARRAY, cli_properties);
-		if (var == NOVARIABLE) ttyputmsg(_("No linear constraint properties")); else
+		if (var == NOVARIABLE) ttyputmsg(M_("No linear constraint properties")); else
 		{
 			len = getlength(var) / LINCONSIZE;
-			ttyputmsg(_("%d %s on this arc:"), len, makeplural(_("constraint"), len));
+			ttyputmsg(M_("%d %s on this arc:"), len, makeplural(M_("constraint"), len));
 			for(i=0; i<len; i++)
 			{
 				conptr = &((LINCON *)var->addr)[i];
-				ttyputmsg(_("Constraint %d: %s %s %s"), i+1, (conptr->variable<=1 ? "x" : "y"),
+				ttyputmsg(M_("Constraint %d: %s %s %s"), i+1, (conptr->variable<=1 ? "x" : "y"),
 					cli_linconops[conptr->oper], frtoa(conptr->value));
 			}
 		}
@@ -607,7 +607,7 @@ INTSML cli_linconsetobject(INTBIG addr, INTBIG type, INTBIG changetype, INTBIG c
 		var = getvalkey((INTBIG)ai, VARCINST, VINTEGER|VISARRAY, cli_properties);
 		if (var == NOVARIABLE)
 		{
-			ttyputerr(_("No constraints on this arc"));
+			ttyputerr(M_("No constraints on this arc"));
 			return(1);
 		}
 		len = getlength(var) / LINCONSIZE;
@@ -640,7 +640,7 @@ INTSML cli_linconsetobject(INTBIG addr, INTBIG type, INTBIG changetype, INTBIG c
 			variable = 2;
 	} else
 	{
-		ttyputerr(_("Cannot parse variable (should be 'X' or 'Y')"));
+		ttyputerr(M_("Cannot parse variable (should be 'X' or 'Y')"));
 		error++;
 	}
 	pt++;
@@ -655,7 +655,7 @@ INTSML cli_linconsetobject(INTBIG addr, INTBIG type, INTBIG changetype, INTBIG c
 		pt += strlen(cli_linconops[i]);
 	} else
 	{
-		ttyputerr(_("Cannot parse operator (must be '>=', '==', or '<=')"));
+		ttyputerr(M_("Cannot parse operator (must be '>=', '==', or '<=')"));
 		error++;
 	}
 
@@ -665,26 +665,26 @@ INTSML cli_linconsetobject(INTBIG addr, INTBIG type, INTBIG changetype, INTBIG c
 		value = atofr(pt);
 		if (value < 0)
 		{
-			ttyputerr(_("Values must be positive"));
+			ttyputerr(M_("Values must be positive"));
 			error++;
 		}
 	} else
 	{
-		ttyputerr(_("Invalid number on right (%s)"), pt);
+		ttyputerr(M_("Invalid number on right (%s)"), pt);
 		error++;
 	}
 
 	if (error != 0)
 	{
-		ttyputmsg(_("Cannot accept linear constraint '%s'"), str);
+		ttyputmsg(M_("Cannot accept linear constraint '%s'"), str);
 		return(1);
 	}
 
 	db_setchangefacet(ai->parent);
 	db_forcehierarchicalanalysis(ai->parent);
 	if (cli_addarcconstraint(ai, (INTSML)variable, oper, value, (INTSML)changetype) != 0) return(1);
-	if (changetype == 0) ttyputverbose(_("Constraint set")); else
-		ttyputverbose(_("Constraint added"));
+	if (changetype == 0) ttyputverbose(M_("Constraint set")); else
+		ttyputverbose(M_("Constraint added"));
 	return(0);
 }
 
@@ -801,7 +801,7 @@ void cli_linconmodifyportproto(PORTPROTO *pp, NODEINST *oni, PORTPROTO *opp)
 }
 
 void cli_linconmodifynodeproto(NODEPROTO *np) {}
-void cli_linconmodifydescript(INTBIG addr, INTBIG type, VARIABLE *var, INTBIG descript) {}
+void cli_linconmodifydescript(INTBIG addr, INTBIG type, INTBIG key, INTBIG descript) {}
 void cli_linconnewlib(LIBRARY *lib) {}
 void cli_linconkilllib(LIBRARY *lib) {}
 
@@ -877,7 +877,7 @@ void cli_solvefacet(NODEPROTO *np, INTSML minimum, INTSML noanchors)
 	/* quit now if all constraints are solved */
 	if (xnotsolved == 0 && ynotsolved == 0 && minimum == 0)
 	{
-		if (cli_conlindebug != 0) ttyputmsg(_("All constraints are met"));
+		if (cli_conlindebug != 0) ttyputmsg(M_("All constraints are met"));
 		cli_deleteconstraints(firstx);
 		cli_deleteconstraints(firsty);
 		return;
@@ -889,29 +889,29 @@ void cli_solvefacet(NODEPROTO *np, INTSML minimum, INTSML noanchors)
 
 	if (cli_conlindebug != 0)
 	{
-		cli_printconstraints(firstx, _("Horizontal constraints:"), "X");
-		if (xnotsolved == 0) ttyputmsg(_("HORIZONTAL CONSTRAINTS ARE ALL MET"));
-		cli_printconstraints(firsty, _("Vertical constraints:"), "Y");
-		if (ynotsolved == 0) ttyputmsg(_("VERTICAL CONSTRAINTS ARE ALL MET"));
+		cli_printconstraints(firstx, M_("Horizontal constraints:"), "X");
+		if (xnotsolved == 0) ttyputmsg(M_("HORIZONTAL CONSTRAINTS ARE ALL MET"));
+		cli_printconstraints(firsty, M_("Vertical constraints:"), "Y");
+		if (ynotsolved == 0) ttyputmsg(M_("VERTICAL CONSTRAINTS ARE ALL MET"));
 	}
 
 	/* solve the networks and determine the success */
 	failure = 0;
 	if (xnotsolved != 0 || minimum != 0)
 	{
-		if (cli_conlindebug != 0) ttyputmsg(_("Solving X constraints:"));
+		if (cli_conlindebug != 0) ttyputmsg(M_("Solving X constraints:"));
 		if (cli_solveconstraints(firstx, minimum) != 0)
 		{
-			ttyputmsg(_("X constraint network cannot be solved!"));
+			ttyputmsg(M_("X constraint network cannot be solved!"));
 			failure = 1;
 		}
 	}
 	if (ynotsolved != 0 || minimum != 0)
 	{
-		if (cli_conlindebug != 0) ttyputmsg(_("Solving Y constraints:"));
+		if (cli_conlindebug != 0) ttyputmsg(M_("Solving Y constraints:"));
 		if (cli_solveconstraints(firsty, minimum) != 0)
 		{
-			ttyputmsg(_("Y constraint network cannot be solved!"));
+			ttyputmsg(M_("Y constraint network cannot be solved!"));
 			failure = 1;
 		}
 	}
@@ -1290,7 +1290,7 @@ INTSML cli_solveconstraints(CONNODE *first, INTSML minimum)
 			cn->solved = 1;
 		}
 
-		if (cli_conlindebug != 0) ttyputmsg(_("  Working from node %s at %ld"),
+		if (cli_conlindebug != 0) ttyputmsg(M_("  Working from node %s at %ld"),
 			describenodeinst(cn->realn), cn->value);
 
 		/* check all constraint arcs to other nodes */
@@ -1315,7 +1315,7 @@ INTSML cli_solveconstraints(CONNODE *first, INTSML minimum)
 			if (ca->other->value == cn->value + ca->value)
 			{
 				if (cli_conlindebug != 0)
-					ttyputmsg(_("    Constraint to %s is met perfectly"),
+					ttyputmsg(M_("    Constraint to %s is met perfectly"),
 						describenodeinst(ca->other->realn));
 				ca->other->solved = 1;
 				continue;
@@ -1325,7 +1325,7 @@ INTSML cli_solveconstraints(CONNODE *first, INTSML minimum)
 			if (satisfied != 0 && minimum == 0)
 			{
 				if (cli_conlindebug != 0)
-					ttyputmsg(_("    Constraint to %s is met adequately"),
+					ttyputmsg(M_("    Constraint to %s is met adequately"),
 						describenodeinst(ca->other->realn));
 				ca->other->solved = 1;
 				continue;
@@ -1336,7 +1336,7 @@ INTSML cli_solveconstraints(CONNODE *first, INTSML minimum)
 			{
 				ca->other->value = cn->value + ca->value;
 				if (cli_conlindebug != 0)
-					ttyputmsg(_("    Constraint to unsolved %s adjusts it to %ld"),
+					ttyputmsg(M_("    Constraint to unsolved %s adjusts it to %ld"),
 						describenodeinst(ca->other->realn), ca->other->value);
 				ca->other->solved = 1;
 				continue;
@@ -1347,7 +1347,7 @@ INTSML cli_solveconstraints(CONNODE *first, INTSML minimum)
 				cno->loopflag = 0;
 			cn->loopflag = 1;
 			if (cli_conlindebug != 0)
-				ttyputmsg(_("    Constraint to solved %s takes some work:"),
+				ttyputmsg(M_("    Constraint to solved %s takes some work:"),
 					describenodeinst(ca->other->realn));
 			if (cli_satisfy(cn, ca) != 0) return(1);
 		}
@@ -1371,7 +1371,7 @@ INTSML cli_satisfy(CONNODE *cn, CONARC *ca)
 
 	/* set new value for that constraint node */
 	cno->value = cn->value + ca->value;
-	if (cli_conlindebug != 0) ttyputmsg(_("      Setting %s to %ld"),
+	if (cli_conlindebug != 0) ttyputmsg(M_("      Setting %s to %ld"),
 		describenodeinst(cno->realn), cno->value);
 
 	/* see if that constraint node is now satisfied */
@@ -1506,10 +1506,10 @@ void cli_printconstraints(CONNODE *first, char *message, char *axis)
 	ttyputmsg("%s", message);
 	for(cn = first; cn != NOCONNODE; cn = cn->nextconnode)
 	{
-		ttyputmsg(_("  Node %s at %ld anchored=%d has these constraints:"),
+		ttyputmsg(M_("  Node %s at %ld anchored=%d has these constraints:"),
 			describenodeinst(cn->realn), cn->value, cn->solved);
 		for(ca = cn->firstconarc; ca != NOCONARC; ca = ca->nextconarc)
-			ttyputmsg(_("    to node %s (%s %s %ld)"), describenodeinst(ca->other->realn), axis,
+			ttyputmsg(M_("    to node %s (%s %s %ld)"), describenodeinst(ca->other->realn), axis,
 				cli_linconops[ca->oper], ca->value);
 	}
 }
@@ -1517,7 +1517,7 @@ void cli_printconstraints(CONNODE *first, char *message, char *axis)
 /*
  * routine to name all of the nodes in facet "np"
  */
-#define	NOABBREV ((ABBREV *)-1)
+#define NOABBREV ((ABBREV *)-1)
 
 typedef struct Iabbrev
 {
@@ -1531,6 +1531,7 @@ void cli_nameallnodes(NODEPROTO *np)
 	REGISTER NODEINST *ni;
 	REGISTER char *pt;
 	REGISTER INTSML upper;
+	REGISTER VARIABLE *var;
 	REGISTER ABBREV *firstabbrev, *a, *nexta;
 	char line[20];
 
@@ -1546,7 +1547,7 @@ void cli_nameallnodes(NODEPROTO *np)
 		/* if the name is null, assign a name */
 		if (strlen(pt) == 0)
 		{
-			if (ni->proto->primindex == 0) pt = _("FACET"); else pt = _("node");
+			if (ni->proto->primindex == 0) pt = M_("FACET"); else pt = M_("node");
 		}
 
 		/* find an abbreviation module for this */
@@ -1567,7 +1568,10 @@ void cli_nameallnodes(NODEPROTO *np)
 		(void)addstringtoinfstr(a->name);
 		(void)sprintf(line, "%d", a->aindex++);
 		(void)addstringtoinfstr(line);
-		(void)setvalkey((INTBIG)ni, VNODEINST, el_node_name, (INTBIG)returninfstr(), VSTRING|VDISPLAY);
+		var = setvalkey((INTBIG)ni, VNODEINST, el_node_name, (INTBIG)returninfstr(), VSTRING|VDISPLAY);
+		if (var != NOVARIABLE)
+			var->textdescript = (var->textdescript & ~VTSIZE) |
+				(defaulttextsize(3) << VTSIZESH);
 	}
 
 	/* free the abbreviation keys */
