@@ -121,7 +121,7 @@ void cli_maketextfacet(NODEPROTO *np)
 		(void)askaid(us_aid, "edit-describe", (INTBIG)&name);
 		(void)initinfstr();
 		(void)addstringtoinfstr(name);
-		(void)addstringtoinfstr(_(" Editor equivalence for facet "));
+		(void)addstringtoinfstr(M_(" Editor equivalence for facet "));
 		(void)addstringtoinfstr(np->cell->cellname);
 		(void)allocstring(&name, returninfstr(), el_tempcluster);
 		cli_curwindow = cli_makeeditorwindow(name, &chars, &lines);
@@ -345,7 +345,7 @@ void cli_eq_killnode(NODEINST *ni)
 		cli_deletecomponentdec(dec);
 		return;
 	}
-	ttyputmsg(_("Warning: cannot find node %s in text"), compname);
+	ttyputmsg(M_("Warning: cannot find node %s in text"), compname);
 }
 
 /* handle the change of node "ni" */
@@ -419,7 +419,7 @@ void cli_eq_modarc(ARCINST *ai)
 	cindex = cli_findtextconn(ai);
 	if (cindex == -1)
 	{
-		ttyputerr(_("Cannot find text equivalent for arc %s"), describearcinst(ai));
+		ttyputerr(M_("Cannot find text equivalent for arc %s"), describearcinst(ai));
 		return;
 	}
 
@@ -532,7 +532,7 @@ void cli_eq_newvar(INTBIG addr, INTBIG type, INTBIG key)
 		if (key == el_node_name)
 		{
 			/* construct the original name */
-			(void)sprintf(line, _("NODE%ld"), (INTBIG)ni);
+			(void)sprintf(line, M_("NODE%ld"), (INTBIG)ni);
 
 			/* get the new node name */
 			var = getvalkey((INTBIG)ni, VNODEINST, VSTRING, el_node_name);
@@ -580,7 +580,7 @@ void cli_eq_killvar(INTBIG addr, INTBIG type, INTBIG key)
 			if (var == NOVARIABLE) return;
 
 			/* construct the new node name */
-			(void)sprintf(line, _("NODE%ld"), (INTBIG)ni);
+			(void)sprintf(line, M_("NODE%ld"), (INTBIG)ni);
 			cli_replacename((char *)var->addr, line);
 			return;
 		}
@@ -705,7 +705,7 @@ void cli_changeequivcomp(NODEINST *ni)
 		cli_deletecomponentdec(dec);
 		return;
 	}
-	ttyputmsg(_("Warning: cannot find node %s in text"), compname);
+	ttyputmsg(M_("Warning: cannot find node %s in text"), compname);
 }
 
 /*
@@ -784,33 +784,33 @@ void cli_replacename(char *oldname, char *newname)
 					switch (e->bits&STATEBITS)
 					{
 						case INPORT :
-							(void)addstringtoinfstr(_("input")); break;
+							(void)addstringtoinfstr(M_("input")); break;
 						case OUTPORT:
-							(void)addstringtoinfstr(_("output")); break;
+							(void)addstringtoinfstr(M_("output")); break;
 						case BIDIRPORT:
-							(void)addstringtoinfstr(_("bidirectional")); break;
+							(void)addstringtoinfstr(M_("bidirectional")); break;
 						case PWRPORT:
-							(void)addstringtoinfstr(_("power")); break;
+							(void)addstringtoinfstr(M_("power")); break;
 						case GNDPORT:
-							(void)addstringtoinfstr(_("ground")); break;
+							(void)addstringtoinfstr(M_("ground")); break;
 						case CLKPORT:
-							(void)addstringtoinfstr(_("clock")); break;
+							(void)addstringtoinfstr(M_("clock")); break;
 						case C1PORT:
-							(void)addstringtoinfstr(_("clock1")); break;
+							(void)addstringtoinfstr(M_("clock1")); break;
 						case C2PORT:
-							(void)addstringtoinfstr(_("clock2")); break;
+							(void)addstringtoinfstr(M_("clock2")); break;
 						case C3PORT:
-							(void)addstringtoinfstr(_("clock3")); break;
+							(void)addstringtoinfstr(M_("clock3")); break;
 						case C4PORT:
-							(void)addstringtoinfstr(_("clock4")); break;
+							(void)addstringtoinfstr(M_("clock4")); break;
 						case C5PORT:
-							(void)addstringtoinfstr(_("clock5")); break;
+							(void)addstringtoinfstr(M_("clock5")); break;
 						case C6PORT:
-							(void)addstringtoinfstr(_("clock6")); break;
+							(void)addstringtoinfstr(M_("clock6")); break;
 						case REFOUTPORT:
-							(void)addstringtoinfstr(_("refout")); break;
+							(void)addstringtoinfstr(M_("refout")); break;
 						case REFINPORT:
-							(void)addstringtoinfstr(_("refin")); break;
+							(void)addstringtoinfstr(M_("refin")); break;
 					}
 				}
 				if ((e->flag&EXPATTR) != 0)
