@@ -5,7 +5,7 @@
  * Linear inequality constraint system: text-to-graphics conversion
  * Written by: Steven M. Rubin, Static Free Software
  *
- * Copyright (c) 2000 Static Free Software.
+ * Copyright (c) 2001 Static Free Software.
  *
  * Electric(tm) is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -102,7 +102,7 @@ INTSML cli_charhandler(WINDOWPART *win, INTSML i)
 	/* M(=) typed: highlight the equivalent line */
 	if (cli_curfacet == NONODEPROTO)
 	{
-		ttyputmsg(_("No valid facet to associate"));
+		ttyputmsg(M_("No valid facet to associate"));
 		return(0);
 	}
 	line = (char *)askaid(us_aid, "edit-getline", (INTBIG)win, -1);
@@ -111,26 +111,26 @@ INTSML cli_charhandler(WINDOWPART *win, INTSML i)
 	switch (type)
 	{
 		case LINECOMM:
-			ttyputmsg(_("No graphic equivalent of comment line"));
+			ttyputmsg(M_("No graphic equivalent of comment line"));
 			break;
 		case LINEBEGIN:
-			ttyputmsg(_("No graphic equivalent of BEGINFACET line"));
+			ttyputmsg(M_("No graphic equivalent of BEGINFACET line"));
 			break;
 		case LINEEND:
-			ttyputmsg(_("No graphic equivalent of ENDFACET line"));
+			ttyputmsg(M_("No graphic equivalent of ENDFACET line"));
 			break;
 		case LINEEXPORT:
-			ttyputmsg(_("No graphic equivalent of EXPORT line"));
+			ttyputmsg(M_("No graphic equivalent of EXPORT line"));
 			break;
 		case LINEUNKN:
-			ttyputmsg(_("No graphic equivalent of this line"));
+			ttyputmsg(M_("No graphic equivalent of this line"));
 			break;
 
 		case LINECONN:
 			ai = cli_findarcname(line);
 			if (ai == NOARCINST)
 			{
-				ttyputmsg(_("Cannot find equivalent arc for this line"));
+				ttyputmsg(M_("Cannot find equivalent arc for this line"));
 				return(0);
 			}
 			(void)askaid(us_aid, "clear");
@@ -141,7 +141,7 @@ INTSML cli_charhandler(WINDOWPART *win, INTSML i)
 			comd = cli_parsecomp(line, 1);
 			if (comd == NOCOMPONENTDEC)
 			{
-				ttyputmsg(_("Cannot parse this declaration line"));
+				ttyputmsg(M_("Cannot parse this declaration line"));
 				return(0);
 			}
 			count = 0;
@@ -155,7 +155,7 @@ INTSML cli_charhandler(WINDOWPART *win, INTSML i)
 			}
 			if (count == 0)
 			{
-				ttyputmsg(_("Cannot find equivalent node(s) for this line"));
+				ttyputmsg(M_("Cannot find equivalent node(s) for this line"));
 				return(0);
 			}
 			break;
@@ -207,7 +207,7 @@ void cli_changehandler(WINDOWPART *win, INTSML nature, char *oldline, char *newl
 	/* comment out an unknown line type */
 	if (ntype == LINEUNKN)
 	{
-		ttyputerr(_("Unrecognized line '%s' commented out"), newline);
+		ttyputerr(M_("Unrecognized line '%s' commented out"), newline);
 		if (otype == LINEUNKN || otype == LINECOMM) cli_commentline((INTSML)changed, win); else
 			cli_restoreline(oldline, (INTSML)changed, win);
 		return;
@@ -237,13 +237,12 @@ void cli_changehandler(WINDOWPART *win, INTSML nature, char *oldline, char *newl
 				{
 					if (namesame(oldname, newname) != 0)
 					{
-						ttyputmsg(_("Facet name changed from %s to %s"), oldname, newname);
-						(void)reallocstring(&cli_curfacet->cell->cellname,
-							newname, cli_curfacet->cell->cluster);
+						(void)setval((INTBIG)cli_curfacet->cell, VCELL, "cellname", (INTBIG)newname, VSTRING);
+						ttyputmsg(M_("Facet name changed from %s to %s"), oldname, newname);
 					}
 				} else if (newname == NOSTRING)
 				{
-					ttyputmsg(_("Erroneous BEGINFACET line"));
+					ttyputmsg(M_("Erroneous BEGINFACET line"));
 					cli_restoreline(oldline, (INTSML)changed, win);
 				}
 				if (oldname != NOSTRING) efree(oldname);
@@ -311,7 +310,7 @@ void cli_changehandler(WINDOWPART *win, INTSML nature, char *oldline, char *newl
 				/* must be the first line entered */
 				if (cli_textlines > 0)
 				{
-					ttyputmsg(_("Already a valid facet definition"));
+					ttyputmsg(M_("Already a valid facet definition"));
 					cli_commentline((INTSML)changed, win);
 					return;
 				}
@@ -320,7 +319,7 @@ void cli_changehandler(WINDOWPART *win, INTSML nature, char *oldline, char *newl
 				newname = cli_parsebeginfacet(newline, 0);
 				if (newname == NOSTRING)
 				{
-					ttyputmsg(_("Erroneous BEGINFACET line removed"));
+					ttyputmsg(M_("Erroneous BEGINFACET line removed"));
 					cli_commentline((INTSML)changed, win);
 					return;
 				}
@@ -401,7 +400,7 @@ void cli_changeexport(char *oldline, char *newline, INTSML changed, WINDOWPART *
 	oldpp = getportproto(cli_curfacet, olde->portname);
 	if (oldpp == NOPORTPROTO)
 	{
-		ttyputerr(_("Cannot parse old port name"));
+		ttyputerr(M_("Cannot parse old port name"));
 		cli_restoreline(oldline, changed, win);
 		cli_deleteexport(olde);   cli_deleteexport(newe);
 		return;
@@ -412,7 +411,7 @@ void cli_changeexport(char *oldline, char *newline, INTSML changed, WINDOWPART *
 	newni = cli_findnodename(newe->component);
 	if (oldni == NONODEINST || newni == NONODEINST)
 	{
-		ttyputerr(_("Cannot parse subnode name"));
+		ttyputerr(M_("Cannot parse subnode name"));
 		cli_restoreline(oldline, changed, win);
 		cli_deleteexport(olde);   cli_deleteexport(newe);
 		return;
@@ -423,7 +422,7 @@ void cli_changeexport(char *oldline, char *newline, INTSML changed, WINDOWPART *
 	newsubpp = cli_getexportsubport(newe, newni);
 	if (oldsubpp == NOPORTPROTO || newsubpp == NOPORTPROTO)
 	{
-		ttyputerr(_("Cannot parse subport name"));
+		ttyputerr(M_("Cannot parse subport name"));
 		cli_restoreline(oldline, changed, win);
 		cli_deleteexport(olde);   cli_deleteexport(newe);
 		return;
@@ -444,7 +443,7 @@ void cli_changeexport(char *oldline, char *newline, INTSML changed, WINDOWPART *
 	{
 		if (getportproto(cli_curfacet, newe->portname) != NOPORTPROTO)
 		{
-			ttyputerr(_("New port name not unique"));
+			ttyputerr(M_("New port name not unique"));
 			cli_restoreline(oldline, changed, win);
 			cli_deleteexport(olde);   cli_deleteexport(newe);
 			return;
@@ -478,6 +477,7 @@ void cli_changedeclaration(char *oldline, char *newline, INTSML changed,
 	REGISTER NODEPROTO *newnp, *oldnp;
 	REGISTER NODEINST *ni;
 	REGISTER INTSML newun, oldun;
+	REGISTER VARIABLE *var;
 	INTBIG newlx, newly, newhx, newhy, oldlx, oldly, oldhx, oldhy, lx, hx, ly, hy;
 	INTSML newrot, oldrot, newtrans, oldtrans, rot, trn;
 
@@ -500,7 +500,7 @@ void cli_changedeclaration(char *oldline, char *newline, INTSML changed,
 	newnp = getnodeproto(newcd->protoname);
 	if (newnp == NONODEPROTO)
 	{
-		ttyputerr(_("Unknown component: %s"), newcd->protoname);
+		ttyputerr(M_("Unknown component: %s"), newcd->protoname);
 		cli_restoreline(oldline, changed, win);
 		cli_deletecomponentdec(oldcd);   cli_deletecomponentdec(newcd);
 		return;
@@ -508,7 +508,7 @@ void cli_changedeclaration(char *oldline, char *newline, INTSML changed,
 	oldnp = getnodeproto(oldcd->protoname);
 	if (oldnp == NONODEPROTO)
 	{
-		ttyputerr(_("Unknown old component: %s"), oldcd->protoname);
+		ttyputerr(M_("Unknown old component: %s"), oldcd->protoname);
 		cli_restoreline(oldline, changed, win);
 		cli_deletecomponentdec(oldcd);   cli_deletecomponentdec(newcd);
 		return;
@@ -617,7 +617,7 @@ void cli_changedeclaration(char *oldline, char *newline, INTSML changed,
 		ni = cli_findnodename(newcomp->name);
 		if (ni != NONODEINST)
 		{
-			ttyputerr(_("Already a component called '%s'"), newcomp->name);
+			ttyputerr(M_("Already a component called '%s'"), newcomp->name);
 			cli_restoreline(oldline, changed, win);
 			cli_deletecomponentdec(oldcd);   cli_deletecomponentdec(newcd);
 			return;
@@ -628,8 +628,11 @@ void cli_changedeclaration(char *oldline, char *newline, INTSML changed,
 		ni = newnodeinst(newnp, lx, hx, ly, hy, trn, rot, cli_curfacet);
 		if (ni != NONODEINST)
 		{
-			(void)setvalkey((INTBIG)ni, VNODEINST, el_node_name, (INTBIG)newcomp->name,
+			var = setvalkey((INTBIG)ni, VNODEINST, el_node_name, (INTBIG)newcomp->name,
 				VSTRING|VDISPLAY);
+			if (var != NOVARIABLE)
+				var->textdescript = (var->textdescript & ~VTSIZE) |
+					(defaulttextsize(3) << VTSIZESH);
 			endobjectchange((INTBIG)ni, VNODEINST);
 		}
 		cli_ownchanges = 0;
@@ -652,7 +655,7 @@ void cli_changedeclaration(char *oldline, char *newline, INTSML changed,
 		{
 			cli_ownchanges = 1;
 			startobjectchange((INTBIG)ni, VNODEINST);
-			ni = replacenodeinst(ni, newnp, 0);
+			ni = replacenodeinst(ni, newnp, 0, 0);
 			endobjectchange((INTBIG)ni, VNODEINST);
 			cli_ownchanges = 0;
 			if (ni == NONODEINST)
@@ -662,7 +665,7 @@ void cli_changedeclaration(char *oldline, char *newline, INTSML changed,
 				 * worked and this one failed.  Then the replacement of the
 				 * original line will not be correct!!!
 				 */
-				ttyputerr(_("New component type cannot be replaced"));
+				ttyputerr(M_("New component type cannot be replaced"));
 				cli_deletecomponentdec(oldcd);   cli_deletecomponentdec(newcd);
 				cli_restoreline(oldline, changed, win);
 				return;
@@ -674,15 +677,18 @@ void cli_changedeclaration(char *oldline, char *newline, INTSML changed,
 		{
 			if (cli_findnodename(newcomp->name) != NONODEINST)
 			{
-				ttyputerr(_("Already a component called '%s'"), newcomp->name);
+				ttyputerr(M_("Already a component called '%s'"), newcomp->name);
 				cli_deletecomponentdec(oldcd);   cli_deletecomponentdec(newcd);
 				cli_restoreline(oldline, changed, win);
 				return;
 			}
 			(void)db_change((INTBIG)ni, OBJECTSTART, VNODEINST, 0,0,0,0,0);
 			cli_ownchanges = 1;
-			(void)setvalkey((INTBIG)ni, VNODEINST, el_node_name, (INTBIG)newcomp->name,
+			var = setvalkey((INTBIG)ni, VNODEINST, el_node_name, (INTBIG)newcomp->name,
 				VSTRING|VDISPLAY);
+			if (var != NOVARIABLE)
+				var->textdescript = (var->textdescript & ~VTSIZE) |
+					(defaulttextsize(3) << VTSIZESH);
 			cli_ownchanges = 0;
 			(void)db_change((INTBIG)ni, OBJECTEND, VNODEINST, 0,0,0,0,0);
 
@@ -725,6 +731,7 @@ void cli_changeconnection(char *oldline, char *newline, INTSML changed, WINDOWPA
 	NODEINST *oldend1, *oldend2, *newend1, *newend2;
 	PORTPROTO *oldport1, *oldport2, *newport1, *newport2;
 	REGISTER INTBIG newxoff, newyoff, lambda;
+	REGISTER VARIABLE *var;
 	REGISTER INTSML resolve, variable, varx, vary, offchanged;
 	INTBIG x1, y1, x2, y2, widnew, widold;
 	INTSML ret;
@@ -749,7 +756,7 @@ void cli_changeconnection(char *oldline, char *newline, INTSML changed, WINDOWPA
 	ai = cli_findarcname(oldline);
 	if (ai == NOARCINST)
 	{
-		ttyputerr(_("Cannot figure out which wire changed"));
+		ttyputerr(M_("Cannot figure out which wire changed"));
 		cli_restoreline(oldline, changed, win);
 		cli_deleteconnection(oldcon);   cli_deleteconnection(newcon);
 		return;
@@ -819,7 +826,7 @@ void cli_changeconnection(char *oldline, char *newline, INTSML changed, WINDOWPA
 	{
 		if (cli_findnodename(newcon->end1) != NONODEINST)
 		{
-			ttyputerr(_("Already a component called '%s'"), newcon->end1);
+			ttyputerr(M_("Already a component called '%s'"), newcon->end1);
 			cli_restoreline(oldline, changed, win);
 			cli_deleteconnection(oldcon);   cli_deleteconnection(newcon);
 			return;
@@ -828,8 +835,11 @@ void cli_changeconnection(char *oldline, char *newline, INTSML changed, WINDOWPA
 		/* make the change */
 		(void)db_change((INTBIG)oldend1, OBJECTSTART, VNODEINST, 0,0,0,0,0);
 		cli_ownchanges = 1;
-		(void)setvalkey((INTBIG)oldend1, VNODEINST, el_node_name, (INTBIG)newcon->end1,
+		var = setvalkey((INTBIG)oldend1, VNODEINST, el_node_name, (INTBIG)newcon->end1,
 			VSTRING|VDISPLAY);
+		if (var != NOVARIABLE)
+			var->textdescript = (var->textdescript & ~VTSIZE) |
+				(defaulttextsize(3) << VTSIZESH);
 		cli_ownchanges = 0;
 		(void)db_change((INTBIG)oldend1, OBJECTEND, VNODEINST, 0,0,0,0,0);
 
@@ -840,7 +850,7 @@ void cli_changeconnection(char *oldline, char *newline, INTSML changed, WINDOWPA
 	{
 		if (cli_findnodename(newcon->end2) != NONODEINST)
 		{
-			ttyputerr(_("Already a component called '%s'"), newcon->end2);
+			ttyputerr(M_("Already a component called '%s'"), newcon->end2);
 			cli_restoreline(oldline, changed, win);
 			cli_deleteconnection(oldcon);   cli_deleteconnection(newcon);
 			return;
@@ -849,8 +859,11 @@ void cli_changeconnection(char *oldline, char *newline, INTSML changed, WINDOWPA
 		/* make the change */
 		(void)db_change((INTBIG)oldend2, OBJECTSTART, VNODEINST, 0,0,0,0,0);
 		cli_ownchanges = 1;
-		(void)setvalkey((INTBIG)oldend2, VNODEINST, el_node_name, (INTBIG)newcon->end2,
+		var = setvalkey((INTBIG)oldend2, VNODEINST, el_node_name, (INTBIG)newcon->end2,
 			VSTRING|VDISPLAY);
+		if (var != NOVARIABLE)
+			var->textdescript = (var->textdescript & ~VTSIZE) |
+				(defaulttextsize(3) << VTSIZESH);
 		cli_ownchanges = 0;
 		(void)db_change((INTBIG)oldend2, OBJECTEND, VNODEINST, 0,0,0,0,0);
 
@@ -869,7 +882,7 @@ void cli_changeconnection(char *oldline, char *newline, INTSML changed, WINDOWPA
 		cli_ownchanges = 0;
 		if (newai == NOARCINST)
 		{
-			ttyputerr(_("Cannot create this new wire"));
+			ttyputerr(M_("Cannot create this new wire"));
 			cli_restoreline(oldline, changed, win);
 			cli_deleteconnection(oldcon);   cli_deleteconnection(newcon);
 			return;
@@ -906,7 +919,7 @@ void cli_changeconnection(char *oldline, char *newline, INTSML changed, WINDOWPA
 		cli_ownchanges = 0;
 		if (ai == NOARCINST)
 		{
-			ttyputerr(_("Cannot change the type of this wire"));
+			ttyputerr(M_("Cannot change the type of this wire"));
 			cli_restoreline(oldline, changed, win);
 			cli_deleteconnection(oldcon);   cli_deleteconnection(newcon);
 			return;
@@ -927,7 +940,7 @@ void cli_changeconnection(char *oldline, char *newline, INTSML changed, WINDOWPA
 		cli_ownchanges = 0;
 		if (ret != 0)
 		{
-			ttyputerr(_("Cannot change the width of this wire"));
+			ttyputerr(M_("Cannot change the width of this wire"));
 			cli_restoreline(oldline, changed, win);
 			cli_deleteconnection(oldcon);   cli_deleteconnection(newcon);
 			return;
@@ -1092,7 +1105,7 @@ void cli_deletedeclaration(char *oldline)
 	np = getnodeproto(dec->protoname);
 	if (np == NONODEPROTO)
 	{
-		ttyputerr(_("Unknown component type: %s"), dec->protoname);
+		ttyputerr(M_("Unknown component type: %s"), dec->protoname);
 		cli_deletecomponentdec(dec);
 		return;
 	}
@@ -1142,7 +1155,7 @@ void cli_deleteexportline(char *oldline)
 	pp = getportproto(cli_curfacet, e->portname);
 	if (pp == NOPORTPROTO)
 	{
-		ttyputerr(_("Unknown port: %s"), e->portname);
+		ttyputerr(M_("Unknown port: %s"), e->portname);
 		cli_deleteexport(e);
 		return;
 	}
@@ -1167,6 +1180,7 @@ INTSML cli_adddeclaration(char *newline, INTSML changed, WINDOWPART *win)
 	REGISTER COMPONENT *compo, *ocomp;
 	REGISTER NODEPROTO *np;
 	REGISTER NODEINST *ni;
+	REGISTER VARIABLE *var;
 	INTBIG lx, hx, ly, hy;
 	INTSML rot, trn;
 
@@ -1182,7 +1196,7 @@ INTSML cli_adddeclaration(char *newline, INTSML changed, WINDOWPART *win)
 	np = getnodeproto(dec->protoname);
 	if (np == NONODEPROTO)
 	{
-		ttyputerr(_("Unknown component type: %s"), dec->protoname);
+		ttyputerr(M_("Unknown component type: %s"), dec->protoname);
 		cli_commentline(changed, win);
 		cli_deletecomponentdec(dec);
 		return(1);
@@ -1195,7 +1209,7 @@ INTSML cli_adddeclaration(char *newline, INTSML changed, WINDOWPART *win)
 		ni = cli_findnodename(compo->name);
 		if (ni != NONODEINST)
 		{
-			ttyputerr(_("Already a component called '%s'"), compo->name);
+			ttyputerr(M_("Already a component called '%s'"), compo->name);
 			cli_commentline(changed, win);
 			cli_deletecomponentdec(dec);
 			return(1);
@@ -1206,7 +1220,7 @@ INTSML cli_adddeclaration(char *newline, INTSML changed, WINDOWPART *win)
 		{
 			if (namesame(compo->name, ocomp->name) == 0)
 			{
-				ttyputerr(_("Duplicate component name '%s'"), compo->name);
+				ttyputerr(M_("Duplicate component name '%s'"), compo->name);
 				cli_commentline(changed, win);
 				cli_deletecomponentdec(dec);
 				return(1);
@@ -1224,8 +1238,11 @@ INTSML cli_adddeclaration(char *newline, INTSML changed, WINDOWPART *win)
 		ni = newnodeinst(np, lx, hx, ly, hy, trn, rot, cli_curfacet);
 		if (ni != NONODEINST)
 		{
-			(void)setvalkey((INTBIG)ni, VNODEINST, el_node_name, (INTBIG)compo->name,
+			var = setvalkey((INTBIG)ni, VNODEINST, el_node_name, (INTBIG)compo->name,
 				VSTRING|VDISPLAY);
+			if (var != NOVARIABLE)
+				var->textdescript = (var->textdescript & ~VTSIZE) |
+					(defaulttextsize(3) << VTSIZESH);
 			endobjectchange((INTBIG)ni, VNODEINST);
 		}
 		cli_ownchanges = 0;
@@ -1273,7 +1290,7 @@ INTSML cli_addwire(char *newline, INTSML changed, WINDOWPART *win)
 	cli_ownchanges = 0;
 	if (ai == NOARCINST)
 	{
-		ttyputerr(_("Cannot make the wire"));
+		ttyputerr(M_("Cannot make the wire"));
 		cli_commentline(changed, win);
 		cli_deleteconnection(dcl);
 		return(1);
@@ -1345,7 +1362,7 @@ void cli_addexport(char *newline, INTSML changed, WINDOWPART *win)
 	pp = getportproto(cli_curfacet, e->portname);
 	if (pp != NOPORTPROTO)
 	{
-		ttyputerr(_("Already an export called %s"), pp->protoname);
+		ttyputerr(M_("Already an export called %s"), pp->protoname);
 		cli_commentline(changed, win);
 		cli_deleteexport(e);
 		return;
@@ -1355,7 +1372,7 @@ void cli_addexport(char *newline, INTSML changed, WINDOWPART *win)
 	ni = cli_findnodename(e->component);
 	if (ni == NONODEINST)
 	{
-		ttyputerr(_("Cannot find node %s"), e->component);
+		ttyputerr(M_("Cannot find node %s"), e->component);
 		cli_commentline(changed, win);
 		cli_deleteexport(e);
 		return;
@@ -1376,7 +1393,7 @@ void cli_addexport(char *newline, INTSML changed, WINDOWPART *win)
 	cli_ownchanges = 0;
 	if (newpp == NOPORTPROTO)
 	{
-		ttyputerr(_("Could not create the port"));
+		ttyputerr(M_("Could not create the port"));
 		cli_commentline(changed, win);
 		cli_deleteexport(e);
 		return;
@@ -1391,13 +1408,13 @@ PORTPROTO *cli_getexportsubport(EXPORT *e, NODEINST *ni)
 	{
 		pp = getportproto(ni->proto, e->subport);
 		if (pp == NOPORTPROTO)
-			ttyputerr(_("No such port %s on node %s"), e->subport, e->component);
+			ttyputerr(M_("No such port %s on node %s"), e->subport, e->component);
 	} else
 	{
 		pp = ni->proto->firstportproto;
 		if (cli_uniqueport(ni, pp) == 0)
 		{
-			ttyputerr(_("Ambiguous port on node %s"), e->component);
+			ttyputerr(M_("Ambiguous port on node %s"), e->component);
 			pp = NOPORTPROTO;
 		}
 	}
