@@ -45,7 +45,7 @@ void db_nullconmodifynodeinst(NODEINST*, INTBIG, INTBIG, INTBIG, INTBIG, INTSML,
 void db_nullconmodifyarcinst(ARCINST*, INTBIG, INTBIG, INTBIG, INTBIG, INTBIG, INTBIG);
 void db_nullconmodifyportproto(PORTPROTO*, NODEINST*, PORTPROTO*);
 void db_nullconmodifynodeproto(NODEPROTO*);
-void db_nullconmodifydescript(INTBIG, INTBIG, VARIABLE*, INTBIG);
+void db_nullconmodifydescript(INTBIG, INTBIG, INTBIG, INTBIG);
 void db_nullconnewlib(LIBRARY*);
 void db_nullconkilllib(LIBRARY*);
 void db_nullconnewvariable(INTBIG, INTBIG, INTBIG, INTBIG);
@@ -127,7 +127,7 @@ void db_nullconmodifynodeinst(NODEINST *ni, INTBIG dlx, INTBIG dly, INTBIG dhx,
 void db_nullconmodifyarcinst(ARCINST *ai, INTBIG oldx0, INTBIG oldy0, INTBIG oldx1, INTBIG oldy1, INTBIG oldwid, INTBIG oldlen) {}
 void db_nullconmodifyportproto(PORTPROTO *pp, NODEINST *oni, PORTPROTO *opp) {}
 void db_nullconmodifynodeproto(NODEPROTO *np) {}
-void db_nullconmodifydescript(INTBIG addr, INTBIG type, VARIABLE *var, INTBIG olddes) {}
+void db_nullconmodifydescript(INTBIG addr, INTBIG type, INTBIG key, INTBIG olddes) {}
 void db_nullconnewlib(LIBRARY *lib) {}
 void db_nullconkilllib(LIBRARY *lib) {}
 void db_nullconnewvariable(INTBIG addr, INTBIG type, INTBIG skey, INTBIG stype) {}
