@@ -48,7 +48,7 @@
  *    INTBIG owid, INTBIG olen) {}
  * aa_modifyportproto(PORTPROTO *pp) {}
  * aa_modifynodeproto(NODEPROTO *np) {}
- * aa_modifydescript(INTBIG addr, INTBIG type, VARIABLE *var, INTBIG old) {}
+ * aa_modifydescript(INTBIG addr, INTBIG type, INTBIG key, INTBIG old) {}
  * aa_newobject(INTBIG addr, INTBIG type) {}
  * aa_killobject(INTBIG addr, INTBIG type) {}
  * aa_newvariable(INTBIG addr, INTBIG type, INTBIG key, INTBIG newtype) {}
@@ -250,7 +250,7 @@ AIDENTRY el_aids[] =
 	/* Quisc "Silicon" Compiler module */
 	{"silicon-compiler", AIDSYNTHESIS,	0,	&sc_silcomp,	NOCLUSTER,
 	sc_init,               sc_done,              sc_set,				/* init/done/set */
-	0,                     0,                    sc_slice,				/* request/examine/slice */
+	sc_request,            0,                    sc_slice,				/* request/examine/slice */
 	0,                     0,											/* startbatch/endbatch */
 	0,                     0,											/* startobject/endobject */
 	0,                     0,                    0,						/* modnode/modarc/modport */
@@ -329,7 +329,7 @@ AIDENTRY el_aids[] =
 	/* network maintenance and comparison */
 	{"network", AIDON | AIDINCREMENTAL, 0, &net_networkp, NOCLUSTER,
 	net_init,              net_done,             net_set,				/* init/done/set */
-	net_request,           net_examinenodeproto, 0,						/* request/examine/slice */
+	net_request,           net_examinenodeproto, net_slice,				/* request/examine/slice */
 	net_startbatch,        net_endbatch,								/* startbatch/endbatch */
 	0,                     0,											/* startobject/endobject */
 	0,                     0,                    net_modifyportproto,	/* modnode/modarc/modport */
@@ -337,7 +337,7 @@ AIDENTRY el_aids[] =
 	net_newobject,         net_killobject,								/* newobject/killobject */
 	net_newvariable,       net_killvariable,							/* newvariable/killvariable */
 	0,                     0,                    0,						/* modvar/insvar/delvar */
-	net_readlibrary,       0,                    0,						/* readlib/eraselib/writelib */
+	net_readlibrary,       net_eraselibrary,     0,						/* readlib/eraselib/writelib */
 	NOVARIABLE, 0},
 
 #if DRCAID
@@ -383,7 +383,7 @@ AIDENTRY el_aids[] =
 	0,                     0,											/* modnodeproto/moddescript */
 	sim_newobject,         sim_killobject,								/* newobject/killobject */
 	sim_newvariable,       sim_killvariable,							/* newvariable/killvariable */
-	0,                     0,                    0,						/* modvar/insvar/delvar */
+	sim_modifyvariable,    sim_insertvariable,   sim_deletevariable,	/* modvar/insvar/delvar */
 	0,                     0,                    0,						/* readlib/eraselib/writelib */
 	NOVARIABLE, 0},
 #endif
@@ -396,10 +396,10 @@ AIDENTRY el_aids[] =
 	proj_startbatch,       0,											/* startbatch/endbatch */
 	0,                     0,											/* startobject/endobject */
 	proj_modifynodeinst,   proj_modifyarcinst,   proj_modifyportproto,	/* modnode/modarc/modport */
-	0,                     0,											/* modnodeproto/moddescript */
+	0,                     proj_modifydescript,							/* modnodeproto/moddescript */
 	proj_newobject,        proj_killobject,								/* newobject/killobject */
-	0,                     0,											/* newvariable/killvariable */
-	0,                     0,                    0,						/* modvar/insvar/delvar */
+	proj_newvariable,      proj_killvariable,							/* newvariable/killvariable */
+	proj_modifyvariable,   proj_insertvariable,  proj_deletevariable,	/* modvar/insvar/delvar */
 	proj_readlibrary,      0,                    0,						/* readlib/eraselib/writelib */
 	NOVARIABLE, 0},
 #endif
