@@ -49,18 +49,6 @@ void db_broadcastchange(CHANGE*, INTSML, INTSML);
 INTSML db_reversechange(CHANGE*);
 char *db_describechange(INTSML, INTBIG, INTBIG, INTBIG, INTBIG, INTBIG, INTBIG, INTBIG);
 
-/* Undo dialog */
-DIALOGITEM db_undodialogitems[] =
-{
- /*  1 */ {0, {468,608,492,680}, BUTTON, N_("OK")},
- /*  2 */ {0, {468,16,492,88}, BUTTON, N_("Undo")},
- /*  3 */ {0, {32,8,455,690}, SCROLL, ""},
- /*  4 */ {0, {8,8,24,241}, MESSAGE, N_("These are the recent changes:")},
- /*  5 */ {0, {8,344,24,456}, CHECK, N_("Show details")},
- /*  6 */ {0, {468,108,492,180}, BUTTON, N_("Redo")}
-};
-DIALOG db_undodialog = {{50,75,555,774}, N_("Change History"), 0, 6, db_undodialogitems};
-
 INTBIG       db_batchtally;					/* number of batches in list */
 INTBIG       db_maximumbatches;				/* limit of number of batches */
 INTSML       db_broadcasting = 0;			/* nonzero if broadcasting */
@@ -73,6 +61,8 @@ CHANGEBATCH *db_changebatchfree;			/* head of free change batches */
 CHANGE      *db_changefree;					/* head of free change modules */
 CHANGEFACET *db_changefacetfree;			/* head of free change facet modules */
 AIDENTRY    *db_currenttool;				/* current tool being given a slice */
+INTSML       db_donextchangequietly = 0;	/* nonzero to do next change quietly */
+INTSML       db_dochangesquietly = 0;		/* nonzero to do changes quietly */
 
 /*
  * Routine to free all memory associated with this module.
@@ -137,7 +127,6 @@ CHANGE *db_change(INTBIG obj, INTSML change, INTBIG a1, INTBIG a2, INTBIG a3, IN
 	REGISTER ARCINST *ai;
 	REGISTER CHANGE *c;
 	REGISTER LIBRARY *lib;
-	REGISTER VARIABLE *var;
 	static char *broadcasttype[] = {N_("nodeinstnew"), N_("nodeinstkill"), N_("nodeinstmod"),
 		N_("arcinstnew"), N_("arcinstkill"), N_("arcinstmod"), N_("portprotonew"),
 		N_("portprotokill"), N_("portprotomod"), N_("nodeprotonew"), N_("nodeprotokill"),
@@ -205,6 +194,10 @@ CHANGE *db_change(INTBIG obj, INTSML change, INTBIG a1, INTBIG a2, INTBIG a3, IN
 			{
 				np = db_whichnodeproto(obj, a1);
 				lib = db_whichlibrary(obj, a1);
+
+				/* special case: when changing text in text-only facet, this is major */
+				if (a1 == VNODEPROTO && a2 == el_facet_message)
+					major = 1;
 			}
 			break;
 		case VARIABLEKILL:
@@ -215,8 +208,7 @@ CHANGE *db_change(INTBIG obj, INTSML change, INTBIG a1, INTBIG a2, INTBIG a3, IN
 			}
 			break;
 		case DESCRIPTMOD:
-			var = (VARIABLE *)a2;
-			if ((var->type&VDONTSAVE) != 0) { lib = NOLIBRARY;   np = NONODEPROTO; } else
+			if ((a3&VDONTSAVE) != 0) { lib = NOLIBRARY;   np = NONODEPROTO; } else
 			{
 				np = db_whichnodeproto(obj, a1);
 				lib = db_whichlibrary(obj, a1);
@@ -277,6 +269,24 @@ CHANGE *db_change(INTBIG obj, INTSML change, INTBIG a1, INTBIG a2, INTBIG a3, IN
 	return(c);
 }
 
+/*
+ * Routine to request that the next change be made "quietly" (i.e. no constraints,
+ * change control, or broadcast).
+ */
+void nextchangequiet(void)
+{
+	db_donextchangequietly = 1;
+}
+
+/*
+ * Routine to set the nature of subsequent changes to be "quiet".  Nonzero
+ * means no constraints, change control, or broadcast are done.
+ */
+void changesquiet(INTSML quiet)
+{
+	db_dochangesquietly = quiet;
+}
+
 /*
  * Routine to broadcast change "c" to all tools that are on.  If "firstchange" is nonzero,
  * this is the first change of a batch, and a "startbatch" change will also be broadcast.
@@ -409,7 +419,7 @@ void db_broadcastchange(CHANGE *c, INTSML firstchange, INTSML undoredo)
 		case DESCRIPTMOD:
 			for(i=0; i<el_maxaid; i++)
 				if ((el_aids[i].aidstate & AIDON) != 0 && el_aids[i].modifydescript != 0)
-					(*el_aids[i].modifydescript)(c->entryaddr, c->p1, (VARIABLE *)c->p2, c->p3);
+					(*el_aids[i].modifydescript)(c->entryaddr, c->p1, c->p2, c->p4);
 			break;
 	}
 	db_broadcasting = oldbroadcasting;
@@ -1035,13 +1045,13 @@ INTSML db_reversechange(CHANGE *c)
 			c->p5 = var->textdescript;
 			c->changetype = VARIABLEKILL;
 			var->type = VINTEGER;		/* fake it out so no memory is deallocated */
-			nextvarchangequiet();
+			nextchangequiet();
 			(void)delvalkey(c->entryaddr, c->p1, c->p2);
 			if (c->p1 == VNODEINST || c->p1 == VARCINST) return(1);
 			break;
 		case VARIABLEKILL:
 			/* args: addr, type, key, oldaddr, oldtype, olddescript */
-			nextvarchangequiet();
+			nextchangequiet();
 			if ((c->p4&VCREF) != 0)
 			{
 				attname = changedvariablename(c->p1, c->p2, c->p4);
@@ -1070,7 +1080,7 @@ INTSML db_reversechange(CHANGE *c)
 			break;
 		case VARIABLEMOD:
 			/* args: addr, type, key, vartype, aindex, oldvalue */
-			nextvarchangequiet();
+			nextchangequiet();
 			if ((c->p3&VCREF) != 0)
 			{
 				attname = changedvariablename(c->p1, c->p2, c->p4);
@@ -1103,7 +1113,7 @@ INTSML db_reversechange(CHANGE *c)
 					c->p4, makename(c->p2), db_namevariable(c->entryaddr, c->p1));
 				break;
 			}
-			nextvarchangequiet();
+			nextchangequiet();
 			(void)delindkey(c->entryaddr, c->p1, c->p2, c->p4);
 			c->changetype = VARIABLEDEL;
 			c->p5 = oldval;
@@ -1111,15 +1121,15 @@ INTSML db_reversechange(CHANGE *c)
 			break;
 		case VARIABLEDEL:
 			/* args: addr, type, key, vartype, aindex, oldvalue */
-			nextvarchangequiet();
+			nextchangequiet();
 			(void)insindkey(c->entryaddr, c->p1, c->p2, c->p4, c->p5);
 			c->changetype = VARIABLEINS;
 			if (c->p1 == VNODEINST || c->p1 == VARCINST) return(1);
 			break;
 		case DESCRIPTMOD:
-			var = (VARIABLE *)c->p2;
+			var = getvalkey(c->entryaddr, c->p1, -1, c->p2);
 			if (var == NOVARIABLE) break;
-			oldval = var->textdescript;   var->textdescript = c->p3;   c->p3 = oldval;
+			oldval = var->textdescript;   var->textdescript = c->p4;   c->p4 = oldval;
 			return(1);
 	}
 	return(0);
@@ -1213,11 +1223,11 @@ void showhistorylist(INTSML which)
 			if (batchreport->batchnumber > hibatch) hibatch = batchreport->batchnumber;
 			if (batchreport->batchnumber != which) continue;
 			(void)initinfstr();
-			(void)formatinfstr(_("Batch %ld from %s aid"), batchreport->batchnumber,
+			(void)formatinfstr(M_("Batch %ld from %s aid"), batchreport->batchnumber,
 				batchreport->aid->aidname);
 			if (batchreport->activity != 0)
-				(void)formatinfstr(_(", caused by '%s'"), batchreport->activity);
-			(void)addstringtoinfstr(_(" is:"));
+				(void)formatinfstr(M_(", caused by '%s'"), batchreport->activity);
+			(void)addstringtoinfstr(M_(" is:"));
 			ttyputmsg("%s", returninfstr());
 
 			for(c = batchreport->changehead; c != NOCHANGE; c = c->nextchange)
@@ -1225,12 +1235,12 @@ void showhistorylist(INTSML which)
 					c->p3, c->p4, c->p5, c->p6));
 			return;
 		}
-		ttyputmsg(_("Batch %d is not in the list (range is %d to %d)"), which, lobatch, hibatch);
+		ttyputmsg(M_("Batch %d is not in the list (range is %d to %d)"), which, lobatch, hibatch);
 		return;
 	}
 
 	/* display the change batches */
-	ttyputmsg(_("%ld batches (limit is %ld):"), db_batchtally-1, db_maximumbatches);
+	ttyputmsg(M_("%ld batches (limit is %ld):"), db_batchtally-1, db_maximumbatches);
 	for(batchreport = db_tailbatch; batchreport != NOCHANGEBATCH;
 		batchreport = batchreport->nextchangebatch)
 	{
@@ -1280,36 +1290,36 @@ void showhistorylist(INTSML which)
 		(void)sprintf(line, "%ld {", batchreport->batchnumber);
 		(void)addstringtoinfstr(line);
 		if (batchreport->activity != 0) (void)addstringtoinfstr(batchreport->activity);
-		(void)addstringtoinfstr(_("} affects"));
+		(void)addstringtoinfstr(M_("} affects"));
 
 		if (node != 0)
 		{
-			(void)sprintf(line, " %d %s", node, makeplural(_("node"), node));
+			(void)sprintf(line, " %d %s", node, makeplural(M_("node"), node));
 			(void)addstringtoinfstr(line);
 		}
 		if (arcinst != 0)
 		{
-			(void)sprintf(line, " %d %s", arcinst, makeplural(_("arc"), arcinst));
+			(void)sprintf(line, " %d %s", arcinst, makeplural(M_("arc"), arcinst));
 			(void)addstringtoinfstr(line);
 		}
 		if (portproto != 0)
 		{
-			(void)sprintf(line, " %d %s", portproto, makeplural(_("port"), portproto));
+			(void)sprintf(line, " %d %s", portproto, makeplural(M_("port"), portproto));
 			(void)addstringtoinfstr(line);
 		}
 		if (nproto != 0)
 		{
-			(void)sprintf(line, " %d %s", nproto, makeplural(_("facet"), nproto));
+			(void)sprintf(line, " %d %s", nproto, makeplural(M_("facet"), nproto));
 			(void)addstringtoinfstr(line);
 		}
 		if (object != 0)
 		{
-			(void)sprintf(line, " %d %s", object, makeplural(_("object"), object));
+			(void)sprintf(line, " %d %s", object, makeplural(M_("object"), object));
 			(void)addstringtoinfstr(line);
 		}
 		if (variable != 0)
 		{
-			(void)sprintf(line, " %d %s", variable, makeplural(_("variable"), variable));
+			(void)sprintf(line, " %d %s", variable, makeplural(M_("variable"), variable));
 			(void)addstringtoinfstr(line);
 		}
 		ttyputmsg("%s", returninfstr());
@@ -1323,7 +1333,6 @@ char *db_describechange(INTSML changetype, INTBIG entryaddr, INTBIG p1, INTBIG p
 	REGISTER ARCINST *ai;
 	REGISTER PORTPROTO *pp;
 	REGISTER NODEPROTO *np;
-	REGISTER VARIABLE *var;
 	VARIABLE myvar;
 
 	switch (changetype)
@@ -1331,144 +1340,140 @@ char *db_describechange(INTSML changetype, INTBIG entryaddr, INTBIG p1, INTBIG p
 		case NODEINSTNEW:
 			ni = (NODEINST *)entryaddr;
 			(void)initinfstr();
-			(void)formatinfstr(_(" Node '%s' created in facet %s"), describenodeinst(ni),
+			(void)formatinfstr(M_(" Node '%s' created in facet %s"), describenodeinst(ni),
 				describenodeproto(ni->parent));
 			return(returninfstr());
 		case NODEINSTKILL:
 			ni = (NODEINST *)entryaddr;
 			(void)initinfstr();
-			(void)formatinfstr(_(" Node '%s' killed in facet %s"), describenodeinst(ni),
+			(void)formatinfstr(M_(" Node '%s' killed in facet %s"), describenodeinst(ni),
 				describenodeproto(ni->parent));
 			return(returninfstr());
 		case NODEINSTMOD:
 			ni = (NODEINST *)entryaddr;
 			(void)initinfstr();
-			(void)formatinfstr(_(" Node '%s' changed in facet %s [was %sx%s, center (%s,%s), rotated %ld"),
+			(void)formatinfstr(M_(" Node '%s' changed in facet %s [was %sx%s, center (%s,%s), rotated %ld"),
 				describenodeinst(ni), describenodeproto(ni->parent), latoa(p3-p1), latoa(p4-p2),
 				latoa((p1+p3)/2), latoa((p2+p4)/2), p5);
-			if (p6 != 0) (void)addstringtoinfstr(_(", transposed"));
+			if (p6 != 0) (void)addstringtoinfstr(M_(", transposed"));
 			(void)addstringtoinfstr("]");
 			return(returninfstr());
 		case ARCINSTNEW:
 			ai = (ARCINST *)entryaddr;
 			(void)initinfstr();
-			(void)formatinfstr(_(" Arc '%s' created in facet %s"), describearcinst(ai),
+			(void)formatinfstr(M_(" Arc '%s' created in facet %s"), describearcinst(ai),
 				describenodeproto(ai->parent));
 			return(returninfstr());
 		case ARCINSTKILL:
 			ai = (ARCINST *)entryaddr;
 			(void)initinfstr();
-			(void)formatinfstr(_(" Arc '%s' killed in facet %s"), describearcinst(ai),
+			(void)formatinfstr(M_(" Arc '%s' killed in facet %s"), describearcinst(ai),
 				describenodeproto(ai->parent));
 			return(returninfstr());
 		case ARCINSTMOD:
 			ai = (ARCINST *)entryaddr;
 			(void)initinfstr();
-			(void)formatinfstr(_(" Arc '%s' modified in facet %s [was %s wide %s long from (%s,%s) to (%s,%s)]"),
+			(void)formatinfstr(M_(" Arc '%s' modified in facet %s [was %s wide %s long from (%s,%s) to (%s,%s)]"),
 				describearcinst(ai), describenodeproto(ai->parent), latoa(p5), latoa(p6), latoa(p1), latoa(p2),
 				latoa(p3), latoa(p4));
 			return(returninfstr());
 		case PORTPROTONEW:
 			pp = (PORTPROTO *)entryaddr;
 			(void)initinfstr();
-			(void)formatinfstr(_(" Port '%s' created in facet %s"), pp->protoname,
+			(void)formatinfstr(M_(" Port '%s' created in facet %s"), pp->protoname,
 				describenodeproto(pp->parent));
 			return(returninfstr());
 		case PORTPROTOKILL:
 			pp = (PORTPROTO *)entryaddr;
 			(void)initinfstr();
-			(void)formatinfstr(_(" Port '%s' killed in facet %s"), pp->protoname,
+			(void)formatinfstr(M_(" Port '%s' killed in facet %s"), pp->protoname,
 				describenodeproto(pp->parent));
 			return(returninfstr());
 		case PORTPROTOMOD:
 			pp = (PORTPROTO *)entryaddr;
 			(void)initinfstr();
-			(void)formatinfstr(_(" Port '%s' moved in facet %s [was on node %s, subport %s]"),
+			(void)formatinfstr(M_(" Port '%s' moved in facet %s [was on node %s, subport %s]"),
 				pp->protoname, describenodeproto(pp->parent), describenodeinst((NODEINST *)p1),
 				((PORTPROTO *)p2)->protoname);
 			return(returninfstr());
 		case NODEPROTONEW:
 			np = (NODEPROTO *)entryaddr;
 			(void)initinfstr();
-			(void)formatinfstr(_(" Facet '%s' created"), describenodeproto(np));
+			(void)formatinfstr(M_(" Facet '%s' created"), describenodeproto(np));
 			return(returninfstr());
 		case NODEPROTOKILL:
 			np = (NODEPROTO *)entryaddr;
 			(void)initinfstr();
-			(void)formatinfstr(_(" Facet %s' killed"), describenodeproto(np));
+			(void)formatinfstr(M_(" Facet %s' killed"), describenodeproto(np));
 			return(returninfstr());
 		case NODEPROTOMOD:
 			np = (NODEPROTO *)entryaddr;
 			(void)initinfstr();
-			(void)formatinfstr(_(" Facet '%s' modified [was from %s<=X<=%s %s<=Y<=%s]"),
+			(void)formatinfstr(M_(" Facet '%s' modified [was from %s<=X<=%s %s<=Y<=%s]"),
 				describenodeproto(np), latoa(p1), latoa(p2), latoa(p3), latoa(p4));
 			return(returninfstr());
 		case OBJECTSTART:
 			/* args: addr, type */
 			(void)initinfstr();
-			(void)formatinfstr(_(" Start change to object %s"), db_namevariable(entryaddr, p1));
+			(void)formatinfstr(M_(" Start change to object %s"), db_namevariable(entryaddr, p1));
 			return(returninfstr());
 		case OBJECTEND:
 			/* args: addr, type */
 			(void)initinfstr();
-			(void)formatinfstr(_(" End change to object %s"), db_namevariable(entryaddr, p1));
+			(void)formatinfstr(M_(" End change to object %s"), db_namevariable(entryaddr, p1));
 			return(returninfstr());
 		case OBJECTNEW:
 			/* args: addr, type */
 			(void)initinfstr();
-			(void)formatinfstr(_(" Object %s created"), db_namevariable(entryaddr, p1));
+			(void)formatinfstr(M_(" Object %s created"), db_namevariable(entryaddr, p1));
 			return(returninfstr());
 		case OBJECTKILL:
 			/* args: addr, type */
 			(void)initinfstr();
-			(void)formatinfstr(_(" Object %s deleted"), db_namevariable(entryaddr, p1));
+			(void)formatinfstr(M_(" Object %s deleted"), db_namevariable(entryaddr, p1));
 			return(returninfstr());
 		case VARIABLENEW:
 			/* args: addr, type, key, newtype */
 			(void)initinfstr();
-			(void)formatinfstr(_(" Variable '%s' created on %s"), changedvariablename(p1, p2, p3),
+			(void)formatinfstr(M_(" Variable '%s' created on %s"), changedvariablename(p1, p2, p3),
 				db_namevariable(entryaddr, p1));
 			return(returninfstr());
 		case VARIABLEKILL:
 			/* args: addr, type, key, oldaddr, oldtype, olddescript */
 			(void)initinfstr();
 			myvar.key = p2;  myvar.type = p4;  myvar.addr = p3;
-			(void)formatinfstr(_(" Variable '%s' killed on %s [was %s]"), changedvariablename(p1, p2, p4),
+			(void)formatinfstr(M_(" Variable '%s' killed on %s [was %s]"), changedvariablename(p1, p2, p4),
 				db_namevariable(entryaddr, p1), describevariable(&myvar, -1, -1));
 			return(returninfstr());
 		case VARIABLEMOD:
 			/* args: addr, type, key, vartype, aindex, oldvalue */
 			(void)initinfstr();
-			(void)formatinfstr(_(" Variable '%s[%ld]' changed on %s"), changedvariablename(p1, p2, p3), p4,
+			(void)formatinfstr(M_(" Variable '%s[%ld]' changed on %s"), changedvariablename(p1, p2, p3), p4,
 				db_namevariable(entryaddr, p1));
 			if ((p3&VCREF) == 0)
 			{
 				myvar.key = p2;  myvar.type = p3 & ~VISARRAY;
 				myvar.addr = p5;
-				(void)formatinfstr(_(" [was '%s']"), describevariable(&myvar, -1, -1));
+				(void)formatinfstr(M_(" [was '%s']"), describevariable(&myvar, -1, -1));
 			}
 			return(returninfstr());
 		case VARIABLEINS:
 			/* args: addr, type, key, vartype, aindex */
 			(void)initinfstr();
-			(void)formatinfstr(_(" Variable '%s[%ld]' inserted on %s"), changedvariablename(p1, p2, p3), p4,
+			(void)formatinfstr(M_(" Variable '%s[%ld]' inserted on %s"), changedvariablename(p1, p2, p3), p4,
 				db_namevariable(entryaddr, p1));
 			return(returninfstr());
 		case VARIABLEDEL:
 			/* args: addr, type, key, vartype, aindex, oldvalue */
 			(void)initinfstr();
-			(void)formatinfstr(_(" Variable '%s[%ld]' deleted on %s"), changedvariablename(p1, p2, p3), p4,
+			(void)formatinfstr(M_(" Variable '%s[%ld]' deleted on %s"), changedvariablename(p1, p2, p3), p4,
 				db_namevariable(entryaddr, p1));
 			return(returninfstr());
 		case DESCRIPTMOD:
-			var = (VARIABLE *)p2;
-			if (var != NOVARIABLE)
-			{
-				(void)initinfstr();
-				(void)formatinfstr(_(" Text descriptor on variable %s changed [was 0%lo]"),
-					makename(var->key), p3);
-				return(returninfstr());
-			}
+			(void)initinfstr();
+			(void)formatinfstr(M_(" Text descriptor on variable %s changed [was 0%lo]"),
+				makename(p2), p4);
+			return(returninfstr());
 	}
 	return("?");
 }
@@ -1599,12 +1604,24 @@ char *db_namevariable(INTBIG addr, INTBIG type)
 	return(returninfstr());
 }
 
-/* routine to display the "undo" menu
- * Change list = 3 (user item)
- * Undo        = 2 (button)
- * Redo        = 6 (button)
- * Show detail = 5 (check box)
- */
+/* Undo dialog */
+DIALOGITEM db_undodialogitems[] =
+{
+ /*  1 */ {0, {468,608,492,680}, BUTTON, N_("OK")},
+ /*  2 */ {0, {468,16,492,88}, BUTTON, N_("Undo")},
+ /*  3 */ {0, {32,8,455,690}, SCROLL, ""},
+ /*  4 */ {0, {8,8,24,241}, MESSAGE, N_("These are the recent changes:")},
+ /*  5 */ {0, {8,344,24,456}, CHECK, N_("Show details")},
+ /*  6 */ {0, {468,108,492,180}, BUTTON, N_("Redo")}
+};
+DIALOG db_undodialog = {{50,75,555,774}, N_("Change History"), 0, 6, db_undodialogitems};
+
+/* special items in the "undo" dialog: */
+#define DUND_UNDO      2		/* Undo (button) */
+#define DUND_CHGLIST   3		/* Change list (scroll) */
+#define DUND_DETAILS   5		/* Show detail (check box) */
+#define DUND_REDO      6		/* Redo (button) */
+
 void db_undodlog(void)
 {
 	INTBIG itemHit;
@@ -1613,33 +1630,33 @@ void db_undodlog(void)
 
 	/* display the undo dialog box */
 	if (DiaInitDialog(&db_undodialog) != 0) return;
-	DiaInitTextDialog(3, DiaNullDlogList, DiaNullDlogItem, DiaNullDlogDone,
+	DiaInitTextDialog(DUND_CHGLIST, DiaNullDlogList, DiaNullDlogItem, DiaNullDlogDone,
 		-1, SCHORIZBAR|SCSMALLFONT);
 	details = 0;
 	db_loadhistorylist(details);
 
 	/* default number of changes to undo */
-	DiaSelectLine(3, 1000);
+	DiaSelectLine(DUND_CHGLIST, 1000);
 
 	/* loop until done */
 	for(;;)
 	{
 		itemHit = DiaNextHit();
 		if (itemHit == OK) break;
-		if (itemHit == 5)
+		if (itemHit == DUND_DETAILS)
 		{
 			details = 1 - details;
-			DiaSetControl(5, details);
+			DiaSetControl(DUND_DETAILS, details);
 			db_loadhistorylist(details);
 			continue;
 		}
-		if (itemHit == 2)
+		if (itemHit == DUND_UNDO)
 		{
 			undoabatch(&aid);
 			db_loadhistorylist(details);
 			continue;
 		}
-		if (itemHit == 6)
+		if (itemHit == DUND_REDO)
 		{
 			redoabatch(&aid);
 			db_loadhistorylist(details);
@@ -1660,7 +1677,7 @@ void db_loadhistorylist(short details)
 	char line[50];
 	CHANGEBATCH *batch;
 
-	DiaLoadTextDialog(3, DiaNullDlogList, DiaNullDlogItem, DiaNullDlogDone, -1);
+	DiaLoadTextDialog(DUND_CHGLIST, DiaNullDlogList, DiaNullDlogItem, DiaNullDlogDone, -1);
 	lineno = 0;
 	curline = -1;
 	for(batch = db_tailbatch; batch != NOCHANGEBATCH; batch = batch->nextchangebatch)
@@ -1698,53 +1715,49 @@ void db_loadhistorylist(short details)
 				case VARIABLEMOD:
 				case VARIABLEINS:
 				case VARIABLEDEL:
-					variable++;
-					break;
 				case DESCRIPTMOD:
-					if ((VARIABLE *)c->entryaddr != NOVARIABLE) variable++; else
-						if ((PORTPROTO *)c->p1 != NOPORTPROTO) portproto++; else
-							node++;
+					variable++;
 					break;
 		}
 
 		(void)initinfstr();
-		if (details != 0) (void)addstringtoinfstr(_("***** BATCH "));
+		if (details != 0) (void)addstringtoinfstr(M_("***** BATCH "));
 		(void)sprintf(line, "%ld {", batch->batchnumber);
 		(void)addstringtoinfstr(line);
 		if (batch->activity != 0) (void)addstringtoinfstr(batch->activity);
-		(void)addstringtoinfstr(_("} affects"));
+		(void)addstringtoinfstr(M_("} affects"));
 
 		if (node != 0)
 		{
-			(void)sprintf(line, " %d %s", node, makeplural(_("node"), node));
+			(void)sprintf(line, " %d %s", node, makeplural(M_("node"), node));
 			(void)addstringtoinfstr(line);
 		}
 		if (arcinst != 0)
 		{
-			(void)sprintf(line, " %d %s", arcinst, makeplural(_("arc"), arcinst));
+			(void)sprintf(line, " %d %s", arcinst, makeplural(M_("arc"), arcinst));
 			(void)addstringtoinfstr(line);
 		}
 		if (portproto != 0)
 		{
-			(void)sprintf(line, " %d %s", portproto, makeplural(_("port"), portproto));
+			(void)sprintf(line, " %d %s", portproto, makeplural(M_("port"), portproto));
 			(void)addstringtoinfstr(line);
 		}
 		if (nproto != 0)
 		{
-			(void)sprintf(line, " %d %s", nproto, makeplural(_("facet"), nproto));
+			(void)sprintf(line, " %d %s", nproto, makeplural(M_("facet"), nproto));
 			(void)addstringtoinfstr(line);
 		}
 		if (object != 0)
 		{
-			(void)sprintf(line, " %d %s", object, makeplural(_("object"), object));
+			(void)sprintf(line, " %d %s", object, makeplural(M_("object"), object));
 			(void)addstringtoinfstr(line);
 		}
 		if (variable != 0)
 		{
-			(void)sprintf(line, " %d %s", variable, makeplural(_("variable"), variable));
+			(void)sprintf(line, " %d %s", variable, makeplural(M_("variable"), variable));
 			(void)addstringtoinfstr(line);
 		}
-		DiaStuffLine(3, returninfstr());
+		DiaStuffLine(DUND_CHGLIST, returninfstr());
 		if (batch == db_donebatch) curline = lineno;
 		lineno++;
 
@@ -1752,13 +1765,13 @@ void db_loadhistorylist(short details)
 		{
 			for(c = batch->changehead; c != NOCHANGE; c = c->nextchange)
 			{
-				DiaStuffLine(3, db_describechange(c->changetype, c->entryaddr, c->p1, c->p2,
+				DiaStuffLine(DUND_CHGLIST, db_describechange(c->changetype, c->entryaddr, c->p1, c->p2,
 					c->p3, c->p4, c->p5, c->p6));
 				lineno++;
 			}
 		}
 	}
-	DiaSelectLine(3, curline);
+	DiaSelectLine(DUND_CHGLIST, curline);
 }
 
 /************************* MEMORY ALLOCATION *************************/
