@@ -33,11 +33,11 @@
 #include "tech.h"
 #include "edialogs.h"
 
-char *el_version = "6.02.1";		/* the current Electric version */
+char *el_version = "6.03";		/* the current Electric version */
 
 /* miscellaneous defaults */
-#define	DEFTECH     "nmos"        /* default technology */
-#define	DEFCONSTR   "layout"      /* default constraint system */
+#define DEFTECH     "nmos"        /* default technology */
+#define DEFCONSTR   "layout"      /* default constraint system */
 
 /* prototypes for local routines */
 void db_freeallmemory(void);
@@ -139,7 +139,6 @@ void osprimaryosinit(void)
 		realtech->term = tech->term;
 		realtech->setmode = tech->setmode;
 		realtech->request = tech->request;
-		realtech->paramnode = tech->paramnode;
 		realtech->nodepolys = tech->nodepolys;
 		realtech->nodeEpolys = tech->nodeEpolys;
 		realtech->shapenodepoly = tech->shapenodepoly;
