@@ -58,7 +58,7 @@ NODEINST *allocnodeinst(CLUSTER *cluster)
 	ni->nextinst = NONODEINST;
 	ni->firstportarcinst = NOPORTARCINST;
 	ni->firstportexpinst = NOPORTEXPINST;
-	ni->textdescript = VTPOSBOXED | (TXTLARGE << VTSIZESH);
+	ni->textdescript = VTPOSBOXED | (defaulttextsize(5) << VTSIZESH);
 	ni->lowx = ni->highx = ni->lowy = ni->highy = 0;
 	ni->transpose = 0;
 	ni->rotation = 0;
@@ -81,12 +81,22 @@ void freenodeinst(NODEINST *ni)
 
 void startobjectchange(INTBIG addr, INTBIG type)
 {
-	(void)db_change(addr, OBJECTSTART, type, 0, 0, 0, 0, 0);
+	/* handle change control and broadcast */
+	if (db_donextchangequietly == 0 && db_dochangesquietly == 0)
+	{
+		(void)db_change(addr, OBJECTSTART, type, 0, 0, 0, 0, 0);
+	}
+	db_donextchangequietly = 0;
 }
 
 void endobjectchange(INTBIG addr, INTBIG type)
 {
-	(void)db_change(addr, OBJECTEND, type, 0, 0, 0, 0, 0);
+	/* handle change control and broadcast */
+	if (db_donextchangequietly == 0 && db_dochangesquietly == 0)
+	{
+		(void)db_change(addr, OBJECTEND, type, 0, 0, 0, 0, 0);
+	}
+	db_donextchangequietly = 0;
 }
 
 /*
@@ -95,15 +105,35 @@ void endobjectchange(INTBIG addr, INTBIG type)
  * in facet "parnt". The address of the nodeinst is returned.
  * If NONODEINST is returned, there is an error creating the nodeinst.
  */
-NODEINST *newnodeinst(NODEPROTO *np, INTBIG lx,INTBIG hx,INTBIG ly,INTBIG hy, INTSML trans,
-	INTSML angle, NODEPROTO *parnt)
+NODEINST *newnodeinst(NODEPROTO *np, INTBIG lx, INTBIG hx, INTBIG ly, INTBIG hy,
+	INTSML trans, INTSML angle, NODEPROTO *parnt)
 {
 	/* make sure values are correct */
-	if (trans != 0 && trans != 1) return((NODEINST *)db_error(DBBADTRANS|DBNEWNODEINST));
-	if (angle < 0 || angle >= 3600) return((NODEINST *)db_error(DBBADROT|DBNEWNODEINST));
-	if (np == NONODEPROTO) return((NODEINST *)db_error(DBBADPROTO|DBNEWNODEINST));
-	if (parnt == NONODEPROTO) return((NODEINST *)db_error(DBBADPARENT|DBNEWNODEINST));
-	if (isachildof(parnt, np)) return((NODEINST *)db_error(DBRECURSIVE|DBNEWNODEINST));
+	if (trans != 0 && trans != 1)
+	{
+		db_donextchangequietly = 0;
+		return((NODEINST *)db_error(DBBADTRANS|DBNEWNODEINST));
+	}
+	if (angle < 0 || angle >= 3600)
+	{
+		db_donextchangequietly = 0;
+		return((NODEINST *)db_error(DBBADROT|DBNEWNODEINST));
+	}
+	if (np == NONODEPROTO)
+	{
+		db_donextchangequietly = 0;
+		return((NODEINST *)db_error(DBBADPROTO|DBNEWNODEINST));
+	}
+	if (parnt == NONODEPROTO)
+	{
+		db_donextchangequietly = 0;
+		return((NODEINST *)db_error(DBBADPARENT|DBNEWNODEINST));
+	}
+	if (isachildof(parnt, np))
+	{
+		db_donextchangequietly = 0;
+		return((NODEINST *)db_error(DBRECURSIVE|DBNEWNODEINST));
+	}
 
 	/* build the nodeinst */
 	return(db_newnodeinst(np, lx,hx,ly,hy, trans, angle, parnt));
@@ -122,7 +152,11 @@ NODEINST *db_newnodeinst(NODEPROTO *np, INTBIG lx, INTBIG hx, INTBIG ly, INTBIG
 
 	/* allocate the nodeinst */
 	ni = allocnodeinst(parnt->cell->cluster);
-	if (ni == NONODEINST) return(NONODEINST);
+	if (ni == NONODEINST)
+	{
+		db_donextchangequietly = 0;
+		return(NONODEINST);
+	}
 
 	/* initialize the nodeinst */
 	ni->proto = np;      ni->rotation = angle;   ni->transpose = trans;
@@ -142,14 +176,19 @@ NODEINST *db_newnodeinst(NODEPROTO *np, INTBIG lx, INTBIG hx, INTBIG ly, INTBIG
 	/* put it in appropriate lists */
 	db_enternodeinst(ni);
 
-	/* set the change facet environment */
-	db_setchangefacet(parnt);
+	/* handle change control, constraint, and broadcast */
+	if (db_donextchangequietly == 0 && db_dochangesquietly == 0)
+	{
+		/* set the change facet environment */
+		db_setchangefacet(parnt);
 
-	/* tell all aids about this nodeinst */
-	ni->changeaddr = (char *)db_change((INTBIG)ni, NODEINSTNEW, 0, 0, 0, 0, 0, 0);
+		/* tell all aids about this nodeinst */
+		ni->changeaddr = (char *)db_change((INTBIG)ni, NODEINSTNEW, 0, 0, 0, 0, 0, 0);
 
-	/* tell constraint system about new node */
-	(*el_curconstraint->newobject)((INTBIG)ni, VNODEINST);
+		/* tell constraint system about new node */
+		(*el_curconstraint->newobject)((INTBIG)ni, VNODEINST);
+	}
+	db_donextchangequietly = 0;
 
 	/* report nodeinst address */
 	return(ni);
@@ -195,15 +234,24 @@ INTSML killnodeinst(NODEINST *ni)
 {
 	/* error if nodeinst is not valid */
 	if (ni == NONODEINST)
+	{
+		db_donextchangequietly = 0;
 		return((INTSML)db_error(DBBADINST|DBKILLNODEINST));
+	}
 
 	/* error if any arcs still connected to this nodeinst */
 	if (ni->firstportarcinst != NOPORTARCINST)
+	{
+		db_donextchangequietly = 0;
 		return((INTSML)db_error(DBHASARCS|DBKILLNODEINST));
+	}
 
 	/* error if any exports */
 	if (ni->firstportexpinst != NOPORTEXPINST)
+	{
+		db_donextchangequietly = 0;
 		return((INTSML)db_error(DBHASPORTS|DBKILLNODEINST));
+	}
 
 	/* remove nodeinst from lists */
 	db_killnodeinst(ni);
@@ -221,14 +269,24 @@ void db_killnodeinst(NODEINST *ni)
 {
 	db_retractnodeinst(ni);
 
-	/* set the change facet environment */
-	db_setchangefacet(ni->parent);
+	/* handle change control, constraint, and broadcast */
+	if (db_donextchangequietly == 0 && db_dochangesquietly == 0)
+	{
+		/* set the change facet environment */
+		db_setchangefacet(ni->parent);
 
-	/* tell all aids about this nodeinst */
-	ni->changeaddr = (char *)db_change((INTBIG)ni, NODEINSTKILL, 0, 0, 0, 0, 0, 0);
+		/* tell all aids about this nodeinst */
+		ni->changeaddr = (char *)db_change((INTBIG)ni, NODEINSTKILL, 0, 0, 0, 0, 0, 0);
 
-	/* tell constraint system about killed node */
-	(*el_curconstraint->killobject)((INTBIG)ni, VNODEINST);
+		/* tell constraint system about killed node */
+		(*el_curconstraint->killobject)((INTBIG)ni, VNODEINST);
+	} else
+	{
+		/* must deallocate now: no change control */
+		freegeom(ni->geom);
+		freenodeinst(ni);
+	}
+	db_donextchangequietly = 0;
 }
 
 /*
@@ -270,11 +328,11 @@ void defaultnodesize(NODEPROTO *np, INTBIG *xs, INTBIG *ys)
 	if (np->primindex == 0) return;
 
 	/* see if there is an override on the node */
-	var = getvalkey((INTBIG)np, VNODEPROTO, VINTEGER|VISARRAY, el_node_default_size);
+	var = getvalkey((INTBIG)np, VNODEPROTO, VINTEGER|VISARRAY, el_node_size_default);
 	if (var != NOVARIABLE)
 	{
-		*xs = ((INTBIG *)var->addr)[0];
-		*ys = ((INTBIG *)var->addr)[1];
+		*xs = ((INTBIG *)var->addr)[0] * el_curlib->lambda[np->tech->techindex] / WHOLE;
+		*ys = ((INTBIG *)var->addr)[1] * el_curlib->lambda[np->tech->techindex] / WHOLE;
 	}
 }
 
@@ -291,10 +349,25 @@ void modifynodeinst(NODEINST *ni, INTBIG deltalx, INTBIG deltaly, INTBIG deltahx
 	if (deltarot < 0) deltarot += 3600;
 	if (deltatrans != 0) deltatrans = 1;
 	if (deltalx == 0 && deltahx == 0 && deltaly == 0 && deltahy == 0 && deltarot == 0 &&
-		deltatrans == 0) return;
+		deltatrans == 0)
+	{
+		db_donextchangequietly = 0;
+		return;
+	}
 
-	/* tell constraint system about modified node */
-	(*el_curconstraint->modifynodeinst)(ni, deltalx, deltaly, deltahx, deltahy, deltarot, deltatrans);
+	if (db_donextchangequietly == 0 && db_dochangesquietly == 0)
+	{
+		/* tell constraint system about modified node */
+		(*el_curconstraint->modifynodeinst)(ni, deltalx, deltaly, deltahx, deltahy, deltarot, deltatrans);
+	} else
+	{
+		/* make the change now: no constraint system */
+		ni->lowx += deltalx;   ni->highx += deltahx;
+		ni->lowy += deltaly;   ni->highy += deltahy;
+		ni->rotation += deltarot;
+		ni->transpose += deltatrans;
+	}
+	db_donextchangequietly = 0;
 
 	/* set the change facet environment */
 	db_setchangefacet(ni->parent);
@@ -315,7 +388,7 @@ NODEINST *dummynode(void)
 	node.firstportarcinst = NOPORTARCINST;
 	node.firstportexpinst = NOPORTEXPINST;
 	node.userbits = 0;
-	node.textdescript = VTPOSBOXED | (TXTLARGE << VTSIZESH);
+	node.textdescript = VTPOSBOXED | (defaulttextsize(5) << VTSIZESH);
 	node.numvar = 0;
 	return(&node);
 }
@@ -471,23 +544,59 @@ ARCINST *newarcinst(ARCPROTO *typ, INTBIG wid, INTBIG initialbits, NODEINST *nA,
 	static POLYGON *poly = NOPOLYGON;
 
 	/* check for missing specifications */
-	if (typ == NOARCPROTO) return((ARCINST *)db_error(DBBADPROTO|DBNEWARCINST));
-	if (parnt == NONODEPROTO) return((ARCINST *)db_error(DBBADPARENT|DBNEWARCINST));
-	if (wid < 0) return((ARCINST *)db_error(DBBADWIDTH|DBNEWARCINST));
-	if (nA == NONODEINST || pA == NOPORTPROTO) return((ARCINST *)db_error(DBBADENDAN|DBNEWARCINST));
-	if (nB == NONODEINST || pB == NOPORTPROTO) return((ARCINST *)db_error(DBBADENDBN|DBNEWARCINST));
+	if (typ == NOARCPROTO)
+	{
+		db_donextchangequietly = 0;
+		return((ARCINST *)db_error(DBBADPROTO|DBNEWARCINST));
+	}
+	if (parnt == NONODEPROTO)
+	{
+		db_donextchangequietly = 0;
+		return((ARCINST *)db_error(DBBADPARENT|DBNEWARCINST));
+	}
+	if (wid < 0)
+	{
+		db_donextchangequietly = 0;
+		return((ARCINST *)db_error(DBBADWIDTH|DBNEWARCINST));
+	}
+	if (nA == NONODEINST || pA == NOPORTPROTO)
+	{
+		db_donextchangequietly = 0;
+		return((ARCINST *)db_error(DBBADENDAN|DBNEWARCINST));
+	}
+	if (nB == NONODEINST || pB == NOPORTPROTO)
+	{
+		db_donextchangequietly = 0;
+		return((ARCINST *)db_error(DBBADENDBN|DBNEWARCINST));
+	}
 
 	/* make sure that the node is in the facet */
-	if (nA->parent != parnt) return((ARCINST *)db_error(DBBADENDAN|DBNEWARCINST));
-	if (nB->parent != parnt) return((ARCINST *)db_error(DBBADENDBN|DBNEWARCINST));
+	if (nA->parent != parnt)
+	{
+		db_donextchangequietly = 0;
+		return((ARCINST *)db_error(DBBADENDAN|DBNEWARCINST));
+	}
+	if (nB->parent != parnt)
+	{
+		db_donextchangequietly = 0;
+		return((ARCINST *)db_error(DBBADENDBN|DBNEWARCINST));
+	}
 
 	/* make sure that the port proto is on the node */
 	for(pp = nA->proto->firstportproto; pp != NOPORTPROTO; pp = pp->nextportproto)
 		if (pp == pA) break;
-	if (pp == NOPORTPROTO) return((ARCINST *)db_error(DBBADENDAN|DBNEWARCINST));
+	if (pp == NOPORTPROTO)
+	{
+		db_donextchangequietly = 0;
+		return((ARCINST *)db_error(DBBADENDAN|DBNEWARCINST));
+	}
 	for(pp = nB->proto->firstportproto; pp != NOPORTPROTO; pp = pp->nextportproto)
 		if (pp == pB) break;
-	if (pp == NOPORTPROTO) return((ARCINST *)db_error(DBBADENDBN|DBNEWARCINST));
+	if (pp == NOPORTPROTO)
+	{
+		db_donextchangequietly = 0;
+		return((ARCINST *)db_error(DBBADENDBN|DBNEWARCINST));
+	}
 
 	/* make sure that the arc can connect to the port */
 	for(i=0; pA->connects[i] != NOARCPROTO; i++)
@@ -498,6 +607,7 @@ ARCINST *newarcinst(ARCPROTO *typ, INTBIG wid, INTBIG initialbits, NODEINST *nA,
 		if (db_printerrors != 0)
 			ttyputmsg(_("Port %s of node %s in facet %s cannot connect to arc %s"), pA->protoname,
 				describenodeinst(nA), describenodeproto(parnt), describearcproto(typ));
+		db_donextchangequietly = 0;
 		return(NOARCINST);
 	}
 	for(i=0; pB->connects[i] != NOARCPROTO; i++)
@@ -508,6 +618,7 @@ ARCINST *newarcinst(ARCPROTO *typ, INTBIG wid, INTBIG initialbits, NODEINST *nA,
 		if (db_printerrors != 0)
 			ttyputmsg(_("Port %s of node %s in facet %s cannot connect to arc %s"), pB->protoname,
 				describenodeinst(nB), describenodeproto(parnt), describearcproto(typ));
+		db_donextchangequietly = 0;
 		return(NOARCINST);
 	}
 
@@ -520,6 +631,7 @@ ARCINST *newarcinst(ARCPROTO *typ, INTBIG wid, INTBIG initialbits, NODEINST *nA,
 		if (db_printerrors != 0)
 			ttyputmsg(_("Point (%s,%s) not inside port %s of node %s in facet %s"), latoa(xA),
 				latoa(yA), pA->protoname, describenodeinst(nA), describenodeproto(parnt));
+		db_donextchangequietly = 0;
 		return(NOARCINST);
 	}
 	shapeportpoly(nB, pB, poly, 0);
@@ -529,6 +641,7 @@ ARCINST *newarcinst(ARCPROTO *typ, INTBIG wid, INTBIG initialbits, NODEINST *nA,
 		if (db_printerrors != 0)
 			ttyputmsg(_("Point (%s,%s) not inside port %s of node %s in facet %s"), latoa(xB),
 				latoa(yB), pB->protoname, describenodeinst(nB), describenodeproto(parnt));
+		db_donextchangequietly = 0;
 		return(NOARCINST);
 	}
 
@@ -551,7 +664,11 @@ ARCINST *db_newarcinst(ARCPROTO *typ, INTBIG wid, INTBIG initialbits, NODEINST *
 	REGISTER PORTARCINST *npi;
 
 	ai = allocarcinst(parnt->cell->cluster);
-	if (ai == NOARCINST) return(NOARCINST);
+	if (ai == NOARCINST)
+	{
+		db_donextchangequietly = 0;
+		return(NOARCINST);
+	}
 	ai->proto = typ;
 	ai->width = wid;
 	ai->length = computedistance(xA,yA, xB,yB);
@@ -583,14 +700,19 @@ ARCINST *db_newarcinst(ARCPROTO *typ, INTBIG wid, INTBIG initialbits, NODEINST *
 	/* enter arcinst in appropriate lists */
 	db_enterarcinst(ai);
 
-	/* set the change facet environment */
-	db_setchangefacet(parnt);
+	/* handle change control, constraint, and broadcast */
+	if (db_donextchangequietly == 0 && db_dochangesquietly == 0)
+	{
+		/* set the change facet environment */
+		db_setchangefacet(parnt);
 
-	/* tell all aids about this arcinst */
-	ai->changeaddr = (char *)db_change((INTBIG)ai, ARCINSTNEW, 0, 0, 0, 0, 0, 0);
+		/* tell all aids about this arcinst */
+		ai->changeaddr = (char *)db_change((INTBIG)ai, ARCINSTNEW, 0, 0, 0, 0, 0, 0);
 
-	/* tell constraint system about new arc */
-	(*el_curconstraint->newobject)((INTBIG)ai, VARCINST);
+		/* tell constraint system about new arc */
+		(*el_curconstraint->newobject)((INTBIG)ai, VARCINST);
+	}
+	db_donextchangequietly = 0;
 
 	/* report arcinst address */
 	return(ai);
@@ -658,7 +780,7 @@ void db_addportarcinst(NODEINST *ni, PORTARCINST *npi)
 	pr = npi->proto;
 	pp = ni->proto->firstportproto;
 	lpi = NOPORTARCINST;
-	for(pi=ni->firstportarcinst; pi!=NOPORTARCINST; pi=pi->nextportarcinst)
+	for(pi = ni->firstportarcinst; pi != NOPORTARCINST; pi = pi->nextportarcinst)
 	{
 		while (pp != pi->proto && pp != pr) pp = pp->nextportproto;
 		if (pp == pr)
@@ -684,7 +806,10 @@ void db_addportarcinst(NODEINST *ni, PORTARCINST *npi)
 INTSML killarcinst(ARCINST *ai)
 {
 	if (ai == NOARCINST)
+	{
+		db_donextchangequietly = 0;
 		return((INTSML)db_error(DBBADINST|DBKILLARCINST));
+	}
 
 	/* remove arcinst from appropriate lists */
 	db_killarcinst(ai);
@@ -697,14 +822,26 @@ void db_killarcinst(ARCINST *ai)
 {
 	db_retractarcinst(ai);
 
-	/* set the change facet environment */
-	db_setchangefacet(ai->parent);
+	/* handle change control, constraint, and broadcast */
+	if (db_donextchangequietly == 0 && db_dochangesquietly == 0)
+	{
+		/* set the change facet environment */
+		db_setchangefacet(ai->parent);
 
-	/* tell all aids about this arcinst */
-	ai->changeaddr = (char *)db_change((INTBIG)ai, ARCINSTKILL, 0, 0, 0, 0, 0, 0);
+		/* tell all aids about this arcinst */
+		ai->changeaddr = (char *)db_change((INTBIG)ai, ARCINSTKILL, 0, 0, 0, 0, 0, 0);
 
-	/* tell constraint system about killed arc */
-	(*el_curconstraint->killobject)((INTBIG)ai, VARCINST);
+		/* tell constraint system about killed arc */
+		(*el_curconstraint->killobject)((INTBIG)ai, VARCINST);
+	} else
+	{
+		/* delete the arc now: no change control */
+		freeportarcinst(ai->end[0].portarcinst);
+		freeportarcinst(ai->end[1].portarcinst);
+		freegeom(ai->geom);
+		freearcinst(ai);
+	}
+	db_donextchangequietly = 0;
 }
 
 void db_retractarcinst(ARCINST *ai)
@@ -767,8 +904,9 @@ INTBIG defaultarcwidth(ARCPROTO *ap)
 	width = ap->nominalwidth;
 
 	/* see if there is an override on the arc */
-	var = getvalkey((INTBIG)ap, VARCPROTO, VINTEGER, el_arc_default_width);
-	if (var != NOVARIABLE) width = var->addr;
+	var = getvalkey((INTBIG)ap, VARCPROTO, VINTEGER, el_arc_width_default);
+	if (var != NOVARIABLE)
+		width = var->addr * el_curlib->lambda[ap->tech->techindex] / WHOLE;
 
 	return(width);
 }
@@ -787,13 +925,22 @@ INTSML modifyarcinst(ARCINST *ai, INTBIG deltawid, INTBIG deltax1, INTBIG deltay
 	REGISTER INTSML e1, e2;
 
 	if (ai == NOARCINST)
+	{
+		db_donextchangequietly = 0;
 		return((INTSML)db_error(DBBADINST|DBMODIFYARCINST));
+	}
 	if (ai->width + deltawid < 0)
+	{
+		db_donextchangequietly = 0;
 		return((INTSML)db_error(DBBADNEWWID|DBMODIFYARCINST));
+	}
 	if (deltax1 != 0 || deltay1 != 0 || deltax2 != 0 || deltay2 != 0)
 	{
 		if ((ai->userbits&(FIXED|CANTSLIDE)) != 0)
+		{
+			db_donextchangequietly = 0;
 			return((INTSML)db_error(DBNOSLIDING|DBMODIFYARCINST));
+		}
 
 		/* temporarily set the width and check for port validity */
 		ai->width += deltawid;
@@ -801,7 +948,10 @@ INTSML modifyarcinst(ARCINST *ai, INTBIG deltawid, INTBIG deltax1, INTBIG deltay
 		e2 = db_stillinport(ai, 1, ai->end[1].xpos+deltax2, ai->end[1].ypos+deltay2);
 		ai->width -= deltawid;
 		if (e1 == 0 || e2 == 0)
+		{
+			db_donextchangequietly = 0;
 			return((INTSML)db_error(DBNOTINPORT|DBMODIFYARCINST));
+		}
 	}
 
 	/* change the arcinst */
@@ -811,19 +961,24 @@ INTSML modifyarcinst(ARCINST *ai, INTBIG deltawid, INTBIG deltax1, INTBIG deltay
 	oldy0 = ai->end[0].ypos;   ai->end[0].ypos += deltay1;
 	oldx1 = ai->end[1].xpos;   ai->end[1].xpos += deltax2;
 	oldy1 = ai->end[1].ypos;   ai->end[1].ypos += deltay2;
-	ai->length = computedistance(ai->end[0].xpos,ai->end[0].ypos, ai->end[1].xpos,ai->end[1].ypos);
+	ai->length = computedistance(ai->end[0].xpos, ai->end[0].ypos, ai->end[1].xpos, ai->end[1].ypos);
 	(void)setshrinkvalue(ai, 1);
 	updategeom(ai->geom, ai->parent);
 
-	/* tell all aids about this arcinst */
-	ai->changeaddr = (char *)db_change((INTBIG)ai, ARCINSTMOD, oldx0, oldy0, oldx1, oldy1,
-		oldwid, oldlen);
+	/* handle change control, constraint, and broadcast */
+	if (db_donextchangequietly == 0 && db_dochangesquietly == 0)
+	{
+		/* tell all aids about this arcinst */
+		ai->changeaddr = (char *)db_change((INTBIG)ai, ARCINSTMOD, oldx0, oldy0, oldx1, oldy1,
+			oldwid, oldlen);
 
-	/* tell constraint system about modified arc */
-	(*el_curconstraint->modifyarcinst)(ai, oldx0, oldy0, oldx1, oldy1, oldwid, oldlen);
+		/* tell constraint system about modified arc */
+		(*el_curconstraint->modifyarcinst)(ai, oldx0, oldy0, oldx1, oldy1, oldwid, oldlen);
 
-	/* set the change facet environment */
-	db_setchangefacet(ai->parent);
+		/* set the change facet environment */
+		db_setchangefacet(ai->parent);
+	}
+	db_donextchangequietly = 0;
 
 	return(0);
 }
@@ -896,7 +1051,7 @@ INTBIG setshrinkvalue(ARCINST *ai, INTSML extend)
  * for nonmanhattan geometry and if so, by what amount.  The
  * routine returns the angle between the nonmanhattan objects.
  */
-#define	MAXANGLES 3
+#define MAXANGLES 3
 INTSML db_checkshortening(NODEINST *ni, PORTPROTO *pp)
 {
 	REGISTER PORTARCINST *pi;
@@ -1016,6 +1171,7 @@ PORTPROTO *allocportproto(CLUSTER *cluster)
 	pp->subportexpinst = NOPORTEXPINST;
 	pp->subportproto = NOPORTPROTO;
 	pp->textdescript = VTPOSBOXED;
+	pp->cachedequivport = NOPORTPROTO;
 	return(pp);
 }
 
@@ -1072,20 +1228,38 @@ PORTPROTO *newportproto(NODEPROTO *np, NODEINST *nodeinst, PORTPROTO *portaddres
 	REGISTER char *ptr;
 
 	/* error checks */
-	if (np == NONODEPROTO) return((PORTPROTO *)db_error(DBBADFACET|DBNEWPORTPROTO));
-	if (*name == 0) return((PORTPROTO *)db_error(DBBADNAME|DBNEWPORTPROTO));
-	for(ptr = name; *ptr != 0; ptr++) if (*ptr <= ' ' || *ptr >= 0177)
+	if (np == NONODEPROTO)
+	{
+		db_donextchangequietly = 0;
+		return((PORTPROTO *)db_error(DBBADFACET|DBNEWPORTPROTO));
+	}
+	if (*name == 0)
+	{
+		db_donextchangequietly = 0;
 		return((PORTPROTO *)db_error(DBBADNAME|DBNEWPORTPROTO));
+	}
+	for(ptr = name; *ptr != 0; ptr++) if (*ptr <= ' ' || *ptr >= 0177)
+	{
+		db_donextchangequietly = 0;
+		return((PORTPROTO *)db_error(DBBADNAME|DBNEWPORTPROTO));
+	}
 
 	/* check the validity of the sub-nodeinst */
-	for(pp=nodeinst->proto->firstportproto; pp!=NOPORTPROTO; pp=pp->nextportproto)
+	for(pp = nodeinst->proto->firstportproto; pp != NOPORTPROTO; pp = pp->nextportproto)
 		if (pp == portaddress) break;
-	if (pp == NOPORTPROTO) return((PORTPROTO *)db_error(DBBADSUBPORT|DBNEWPORTPROTO));
+	if (pp == NOPORTPROTO)
+	{
+		db_donextchangequietly = 0;
+		return((PORTPROTO *)db_error(DBBADSUBPORT|DBNEWPORTPROTO));
+	}
 
 	/* look for duplicate names */
 	for(pp = np->firstportproto; pp != NOPORTPROTO; pp = pp->nextportproto)
 		if (namesame(pp->protoname, name) == 0)
-			return((PORTPROTO *)db_error(DBDUPLICATE|DBNEWPORTPROTO));
+	{
+		db_donextchangequietly = 0;
+		return((PORTPROTO *)db_error(DBDUPLICATE|DBNEWPORTPROTO));
+	}
 
 	return(db_newportproto(np, nodeinst, portaddress, name));
 }
@@ -1101,7 +1275,11 @@ PORTPROTO *db_newportproto(NODEPROTO *np, NODEINST *nodeinst, PORTPROTO *portadd
 
 	/* allocate a new portproto */
 	pp = allocportproto(np->cell->cluster);
-	if (pp == NOPORTPROTO) return(NOPORTPROTO);
+	if (pp == NOPORTPROTO)
+	{
+		db_donextchangequietly = 0;
+		return(NOPORTPROTO);
+	}
 
 	/* load up the port prototype */
 	pp->connects = portaddress->connects;
@@ -1109,20 +1287,30 @@ PORTPROTO *db_newportproto(NODEPROTO *np, NODEINST *nodeinst, PORTPROTO *portadd
 	pp->subnodeinst = nodeinst;
 	pp->subportproto = portaddress;
 	pp->userbits = portaddress->userbits;
-	pp->textdescript = defaulttextdescript(nodeinst->geom);
-	if (allocstring(&pp->protoname, name, np->cell->cluster) != 0) return(NOPORTPROTO);
+	pp->textdescript = (defaulttextdescript(nodeinst->geom) & ~VTSIZE) |
+		(defaulttextsize(1) << VTSIZESH);
+	if (allocstring(&pp->protoname, name, np->cell->cluster) != 0)
+	{
+		db_donextchangequietly = 0;
+		return(NOPORTPROTO);
+	}
 
 	/* link it in and announce it */
 	db_enterportproto(pp);
 
-	/* set the change facet environment */
-	db_setchangefacet(np);
+	/* handle change control, constraint, and broadcast */
+	if (db_donextchangequietly == 0 && db_dochangesquietly == 0)
+	{
+		/* set the change facet environment */
+		db_setchangefacet(np);
 
-	/* announce it */
-	pp->changeaddr = (char *)db_change((INTBIG)pp, PORTPROTONEW, 0, 0, 0, 0, 0, 0);
+		/* announce it */
+		pp->changeaddr = (char *)db_change((INTBIG)pp, PORTPROTONEW, 0, 0, 0, 0, 0, 0);
 
-	/* tell constraint system about new port */
-	(*el_curconstraint->newobject)((INTBIG)pp, VPORTPROTO);
+		/* tell constraint system about new port */
+		(*el_curconstraint->newobject)((INTBIG)pp, VPORTPROTO);
+	}
+	db_donextchangequietly = 0;
 
 	return(pp);
 }
@@ -1148,6 +1336,20 @@ void db_enterportproto(PORTPROTO *pp)
 	/* special case: if port is on node that changes size with connectivity, update its geometry */
 	if ((pp->subnodeinst->proto->userbits&WIPEON1OR2) != 0)
 		updategeom(pp->subnodeinst->geom, pp->parent);
+
+	/* clear cache of port associations in this cell */
+	db_clearportcache(pp->parent);
+}
+
+void db_clearportcache(NODEPROTO *facet)
+{
+	REGISTER NODEPROTO *np, *onp;
+
+	for(np = facet->cell->firstincell; np != NONODEPROTO; np = np->nextincell)
+	{
+		for(onp = np; onp != NONODEPROTO; onp = onp->lastversion)
+			onp->cachedequivfacet = NONODEPROTO;
+	}
 }
 
 /*
@@ -1162,7 +1364,7 @@ void db_addportexpinst(NODEINST *ni, PORTEXPINST *npe)
 	pr = npe->proto;
 	pp = ni->proto->firstportproto;
 	lpe = NOPORTEXPINST;
-	for(pe=ni->firstportexpinst; pe!=NOPORTEXPINST; pe=pe->nextportexpinst)
+	for(pe = ni->firstportexpinst; pe != NOPORTEXPINST; pe = pe->nextportexpinst)
 	{
 		while (pp != pe->proto && pp != pr) pp = pp->nextportproto;
 		if (pp == pr)
@@ -1240,15 +1442,24 @@ INTSML killportproto(NODEPROTO *np, PORTPROTO *pp)
 
 	/* look at all portprotos to ensure it exists */
 	if (np == NONODEPROTO)
+	{
+		db_donextchangequietly = 0;
 		return((INTSML)db_error(DBBADFACET|DBKILLPORTPROTO));
-	for(spt=np->firstportproto; spt != NOPORTPROTO; spt = spt->nextportproto)
+	}
+	for(spt = np->firstportproto; spt != NOPORTPROTO; spt = spt->nextportproto)
 		if (spt == pp) break;
 	if (spt == NOPORTPROTO)
+	{
+		db_donextchangequietly = 0;
 		return((INTSML)db_error(DBBADPROTO|DBKILLPORTPROTO));
+	}
 
 	/* cannot delete port on primitive nodeproto */
 	if (pp->subnodeinst == NONODEINST)
+	{
+		db_donextchangequietly = 0;
 		return((INTSML)db_error(DBPRIMITIVE|DBKILLPORTPROTO));
+	}
 
 	db_killportproto(pp);
 	return(0);
@@ -1290,14 +1501,24 @@ void db_killportproto(PORTPROTO *pp)
 	/* remove the port prototype */
 	db_retractportproto(pp);
 
-	/* set the change facet environment */
-	db_setchangefacet(np);
+	/* handle change control, constraint, and broadcast */
+	if (db_donextchangequietly == 0 && db_dochangesquietly == 0)
+	{
+		/* set the change facet environment */
+		db_setchangefacet(np);
 
-	/* report the change */
-	pp->changeaddr = (char *)db_change((INTBIG)pp, PORTPROTOKILL, 0, 0, 0, 0, 0, 0);
+		/* report the change */
+		pp->changeaddr = (char *)db_change((INTBIG)pp, PORTPROTOKILL, 0, 0, 0, 0, 0, 0);
 
-	/* tell constraint system about killed port */
-	(*el_curconstraint->killobject)((INTBIG)pp, VPORTPROTO);
+		/* tell constraint system about killed port */
+		(*el_curconstraint->killobject)((INTBIG)pp, VPORTPROTO);
+	} else
+	{
+		/* delete the export now: no change control */
+		efree(pp->protoname);
+		freeportproto(pp);
+	}
+	db_donextchangequietly = 0;
 }
 
 void db_retractportproto(PORTPROTO *pp)
@@ -1325,6 +1546,9 @@ void db_retractportproto(PORTPROTO *pp)
 	/* special case: if port is on node that changes size with connectivity, update its geometry */
 	if ((pp->subnodeinst->proto->userbits&WIPEON1OR2) != 0)
 		updategeom(pp->subnodeinst->geom, np);
+
+	/* clear cache of port associations in this cell */
+	db_clearportcache(pp->parent);
 }
 
 /*
