@@ -40,7 +40,7 @@
 #define mmini(a, b) ((a) < (b) ? (a) : (b))
 #define mmaxi(a, b) ((a) > (b) ? (a) : (b))
 
-#define	MAXDEPTH 100			/* maximum depth of tree (for search) */
+#define MAXDEPTH 100			/* maximum depth of tree (for search) */
 /* #define USEFLOATS */			/* uncomment to use floating point */
 
 #define NORSEARCH ((RSEARCH *)-1)
@@ -240,7 +240,7 @@ void linkgeom(GEOM *geom, NODEPROTO *parnt)
 
 #ifndef	USEFLOATS
 		/* compute scaling factor */
-		scaledown = HUGEINT;
+		scaledown = MAXINTBIG;
 		for(i=0; i<rtn->total; i++)
 		{
 			subrtn = (RTNODE *)rtn->pointers[i];
@@ -318,7 +318,7 @@ INTSML db_addtortnode(UINTBIG object, RTNODE *rtn, NODEPROTO *facet)
 		 * also, compute scale and copy node to temporary
 		 */
 #ifndef	USEFLOATS
-		scaledown = HUGEINT;
+		scaledown = MAXINTBIG;
 #endif
 		newdist = 0;
 		temp.flag = rtn->flag;
@@ -761,15 +761,15 @@ void db_printrtree(RTNODE *expectedparent, RTNODE *rtn, INTSML indent)
 
 	(void)initinfstr();
 	for(i=0; i<indent; i++) (void)addtoinfstr(' ');
-	(void)sprintf(line, _("Node X(%s-%s) Y(%s-%s) has %d children:"),
+	(void)sprintf(line, M_("Node X(%s-%s) Y(%s-%s) has %d children:"),
 		latoa(rtn->lowx), latoa(rtn->highx), latoa(rtn->lowy), latoa(rtn->highy), rtn->total);
 	(void)addstringtoinfstr(line);
 
 	/* sensibility check of this R-tree node */
-	if (rtn->parent != expectedparent) (void)addstringtoinfstr(_(" WRONG-PARENT"));
-	if (rtn->total > db_maxrtnodesize) (void)addstringtoinfstr(_(" TOO-MANY"));
+	if (rtn->parent != expectedparent) (void)addstringtoinfstr(M_(" WRONG-PARENT"));
+	if (rtn->total > db_maxrtnodesize) (void)addstringtoinfstr(M_(" TOO-MANY"));
 	if (rtn->total < db_minrtnodesize && rtn->parent != NORTNODE)
-		(void)addstringtoinfstr(_(" TOO-FEW"));
+		(void)addstringtoinfstr(M_(" TOO-FEW"));
 	if (rtn->total != 0)
 	{
 		db_rtnbbox(rtn, 0, &lowestxv, &highestxv, &lowestyv, &highestyv);
@@ -783,7 +783,7 @@ void db_printrtree(RTNODE *expectedparent, RTNODE *rtn, INTSML indent)
 		}
 		if (rtn->lowx != lowestxv || rtn->highx != highestxv ||
 			rtn->lowy != lowestyv || rtn->highy != highestyv)
-				(void)addstringtoinfstr(_(" WRONG-BOUNDS"));
+				(void)addstringtoinfstr(M_(" WRONG-BOUNDS"));
 		rtn->lowx = lowestxv;
 		rtn->highx = highestxv;
 		rtn->lowy = lowestyv;
@@ -958,10 +958,10 @@ void boundobj(GEOM *geom, INTBIG *lx, INTBIG *hx, INTBIG *ly, INTBIG *hy)
 
 #if 0
 		/* include any text on the arc */
-		pts = tech_displayableavars(ai);
+		pts = tech_displayableavars(ai, NOWINDOWPART);
 		for(j=0; j<pts; j++)
 		{
-			var = tech_filldisplayableavar(ai, poly);
+			var = tech_filldisplayableavar(ai, poly, NOWINDOWPART, 0);
 			getbbox(poly, &plx, &phx, &ply, &phy);
 			if (plx < *lx) *lx = plx;
 			if (phx > *hx) *hx = phx;
@@ -1035,7 +1035,7 @@ void boundobj(GEOM *geom, INTBIG *lx, INTBIG *hx, INTBIG *ly, INTBIG *hy)
 				if (var != NOVARIABLE)
 				{
 					makerot(ni, trans);
-					num = nodepolys(ni, 0);
+					num = nodepolys(ni, 0, NOWINDOWPART);
 					for(j=0; j<num; j++)
 					{
 						shapenodepoly(ni, j, poly);
@@ -1073,10 +1073,10 @@ void boundobj(GEOM *geom, INTBIG *lx, INTBIG *hx, INTBIG *ly, INTBIG *hy)
 
 #if 0
 		/* include any text on the node */
-		pts = tech_displayablenvars(ni);
+		pts = tech_displayablenvars(ni, NOWINDOWPART);
 		for(j=0; j<pts; j++)
 		{
-			var = tech_filldisplayablenvar(ni, poly);
+			var = tech_filldisplayablenvar(ni, poly, NOWINDOWPART, 0);
 			getbbox(poly, &plx, &phx, &ply, &phy);
 			if (plx < *lx) *lx = plx;
 			if (phx > *hx) *hx = phx;
