@@ -167,26 +167,26 @@ struct S_EConstraint  { CONSTRAINT  *handle; };
 struct S_EWindowFrame { WINDOWFRAME *handle; };
 
 /* macros for accessing Electric objects from the Lisp structures */
-#define	EELECTRIC(obj)    ((struct S_Electric *)POINTER(obj))
-#define	ENODEINST(obj)    ((struct S_ENodeInst *)POINTER(obj))
-#define	ENODEPROTO(obj)   ((struct S_ENodeProto *)POINTER(obj))
-#define	EPORTARCINST(obj) ((struct S_EPortArcInst *)POINTER(obj))
-#define	EPORTEXPINST(obj) ((struct S_EPortExpInst *)POINTER(obj))
-#define	EPORTPROTO(obj)   ((struct S_EPortProto *)POINTER(obj))
-#define	EARCINST(obj)     ((struct S_EArcInst *)POINTER(obj))
-#define	EARCPROTO(obj)    ((struct S_EArcProto *)POINTER(obj))
-#define	EGEOM(obj)        ((struct S_EGeom *)POINTER(obj))
-#define	ELIBRARY(obj)     ((struct S_ELibrary *)POINTER(obj))
-#define	ETECHNOLOGY(obj)  ((struct S_ETechnology *)POINTER(obj))
-#define	EAID(obj)         ((struct S_EAid *)POINTER(obj))
-#define	ERTNODE(obj)      ((struct S_ERTNode *)POINTER(obj))
-#define	ENETWORK(obj)     ((struct S_ENetwork *)POINTER(obj))
-#define	ECELL(obj)        ((struct S_ECell *)POINTER(obj))
-#define	EVIEW(obj)        ((struct S_EView *)POINTER(obj))
-#define	EWINDOW(obj)      ((struct S_EWindow *)POINTER(obj))
-#define	EGRAPHICS(obj)    ((struct S_EGraphics *)POINTER(obj))
-#define	ECONSTRAINT(obj)  ((struct S_EConstraint *)POINTER(obj))
-#define	EWINDOWFRAME(obj) ((struct S_EWindowFrame *)POINTER(obj))
+#define EELECTRIC(obj)    ((struct S_Electric *)POINTER(obj))
+#define ENODEINST(obj)    ((struct S_ENodeInst *)POINTER(obj))
+#define ENODEPROTO(obj)   ((struct S_ENodeProto *)POINTER(obj))
+#define EPORTARCINST(obj) ((struct S_EPortArcInst *)POINTER(obj))
+#define EPORTEXPINST(obj) ((struct S_EPortExpInst *)POINTER(obj))
+#define EPORTPROTO(obj)   ((struct S_EPortProto *)POINTER(obj))
+#define EARCINST(obj)     ((struct S_EArcInst *)POINTER(obj))
+#define EARCPROTO(obj)    ((struct S_EArcProto *)POINTER(obj))
+#define EGEOM(obj)        ((struct S_EGeom *)POINTER(obj))
+#define ELIBRARY(obj)     ((struct S_ELibrary *)POINTER(obj))
+#define ETECHNOLOGY(obj)  ((struct S_ETechnology *)POINTER(obj))
+#define EAID(obj)         ((struct S_EAid *)POINTER(obj))
+#define ERTNODE(obj)      ((struct S_ERTNode *)POINTER(obj))
+#define ENETWORK(obj)     ((struct S_ENetwork *)POINTER(obj))
+#define ECELL(obj)        ((struct S_ECell *)POINTER(obj))
+#define EVIEW(obj)        ((struct S_EView *)POINTER(obj))
+#define EWINDOW(obj)      ((struct S_EWindow *)POINTER(obj))
+#define EGRAPHICS(obj)    ((struct S_EGraphics *)POINTER(obj))
+#define ECONSTRAINT(obj)  ((struct S_EConstraint *)POINTER(obj))
+#define EWINDOWFRAME(obj) ((struct S_EWindowFrame *)POINTER(obj))
 
 /* equality routine */
 int EElectric_Equal(ELKObject a, ELKObject b)
@@ -1253,20 +1253,19 @@ ELKObject lsp_replacenodeinst(ELKObject oni, ELKObject opr)
 	Ensure_Type(oni, T_ENodeInst);   ni = ENODEINST(oni)->handle;
 	Ensure_Type(opr, T_ENodeProto);  pr = ENODEPROTO(opr)->handle;
 
-	nni = replacenodeinst(ni, pr, 0);
+	nni = replacenodeinst(ni, pr, 0, 0);
 	if (nni == NONODEINST) return(Null);
 	return(Make_EElectric((INTBIG)nni, (INTSML)T_ENodeInst));
 }
 
 ELKObject lsp_nodefunction(ELKObject oni)
 {
-	char *dummy;
 	REGISTER NODEINST *ni;
 
 	/* get inputs from LISP */
 	Ensure_Type(oni, T_ENodeInst);   ni = ENODEINST(oni)->handle;
 
-	return(Make_Integer(nodefunction(ni, &dummy)));
+	return(Make_Integer(nodefunction(ni, 0)));
 }
 
 /****************************** ARCINST ROUTINES ******************************/
