@@ -34,7 +34,6 @@
 #include "global.h"
 #include "dblang.h"
 #include "database.h"
-#include "usr.h"
 #ifdef	USETK
 #  ifdef BLACK
 #    undef BLACK
@@ -47,7 +46,7 @@
 #  include "tkInt.h"
 #endif
 
-#define	BUFSIZE 500
+#define BUFSIZE 500
 
 char *tcl_outputbuffer, *tcl_outputloc, *tcl_inputbuffer, *tcl_inputloc;
 Tcl_Interp *tcl_interp = 0;
@@ -1217,7 +1216,7 @@ int tcl_replacenodeinst(ClientData dummy, Tcl_Interp *interp, int argc, char **a
 	if (ni == NONODEINST || np == NONODEPROTO) return TCL_ERROR;
 
 	/* call Electric */
-	nni = replacenodeinst(ni, np, 0);
+	nni = replacenodeinst(ni, np, 0, 0);
 
 	/* convert result */
 	Tcl_AppendResult(interp, tcl_converttotcl((INTBIG)nni, VNODEINST), (char *) NULL);
@@ -1227,7 +1226,6 @@ int tcl_replacenodeinst(ClientData dummy, Tcl_Interp *interp, int argc, char **a
 int tcl_nodefunction(ClientData dummy, Tcl_Interp *interp, int argc, char **argv)
 {
 	REGISTER NODEINST *ni;
-	char *edummy;
 	REGISTER INTBIG fun;
 
 	/* get input */
@@ -1240,7 +1238,7 @@ int tcl_nodefunction(ClientData dummy, Tcl_Interp *interp, int argc, char **argv
 	if (ni == NONODEINST) return TCL_ERROR;
 
 	/* call Electric */
-	fun = nodefunction(ni, &edummy);
+	fun = nodefunction(ni, 0);
 
 	/* convert result */
 	Tcl_AppendResult(interp, tcl_converttotcl(fun, VINTEGER), (char *) NULL);
