@@ -76,9 +76,13 @@ LIBRARY *alloclibrary(void)
 	lib->libfile = NOSTRING;
 	lib->firstvar = NOVARIABLE;
 	lib->firstnodeproto = NONODEPROTO;
+	lib->tailnodeproto = NONODEPROTO;
 	lib->firstcell = NOCELL;
+	lib->tailcell = NOCELL;
+	lib->numcells = 0;
 	lib->curnodeproto = NONODEPROTO;
 	lib->nextlibrary = NOLIBRARY;
+	lib->cellhashtablesize = 0;
 	return(lib);
 }
 
@@ -92,6 +96,7 @@ void freelibrary(LIBRARY *lib)
 	if (lib == NOLIBRARY) return;
 	if (lib->numvar != 0) db_freevars(&lib->firstvar, &lib->numvar);
 	efree((char *)lib->lambda);
+	if (lib->cellhashtablesize > 0) efree((char *)lib->cellhashtable);
 	clus = lib->cluster;
 	efree((char *)lib);
 	freecluster(clus);
@@ -297,7 +302,7 @@ void eraselibrary(LIBRARY *lib)
 		lnp = np->nextnodeproto;
 
 		/* free the portproto entries */
-		for(pp=np->firstportproto; pp != NOPORTPROTO; pp = lpt)
+		for(pp = np->firstportproto; pp != NOPORTPROTO; pp = lpt)
 		{
 			lpt = pp->nextportproto;
 			efree(pp->protoname);
@@ -308,6 +313,7 @@ void eraselibrary(LIBRARY *lib)
 		freenodeproto(np);
 	}
 	lib->firstnodeproto = NONODEPROTO;
+	lib->tailnodeproto = NONODEPROTO;
 	lib->curnodeproto = NONODEPROTO;
 
 	/* now erase the cells */
@@ -318,6 +324,13 @@ void eraselibrary(LIBRARY *lib)
 		freecell(c);
 	}
 	lib->firstcell = NOCELL;
+	lib->tailcell = NOCELL;
+	lib->numcells = 0;
+	if (lib->cellhashtablesize > 0)
+	{
+		efree((char *)lib->cellhashtable);
+		lib->cellhashtablesize = 0;
+	}
 
 	/* now erase the library information */
 	if (lib->numvar != 0) db_freevars(&lib->firstvar, &lib->numvar);
@@ -376,7 +389,8 @@ void changelambda(INTBIG count, TECHNOLOGY **techarray, INTBIG *newlam,
 		if (how == 1 && whichlib != lib) continue;
 		for(np = lib->firstnodeproto; np != NONODEPROTO; np = np->nextnodeproto)
 			db_scalefacet(np, count, techarray, newlam, oldlam);
-		lib->userbits |= LIBCHANGEDMAJOR;
+		if (lib->firstnodeproto != NONODEPROTO)
+			lib->userbits |= LIBCHANGEDMAJOR;
 	}
 	efree((char *)oldlam);
 }
@@ -596,7 +610,6 @@ void changetechnologylambda(TECHNOLOGY *tech, INTBIG newlambda)
 {
 	REGISTER NODEPROTO *np;
 	REGISTER ARCPROTO *ap;
-	REGISTER VARIABLE *var;
 	REGISTER INTBIG oldlambda;
 
 	oldlambda = tech->deflambda;
@@ -607,9 +620,6 @@ void changetechnologylambda(TECHNOLOGY *tech, INTBIG newlambda)
 	for(ap = tech->firstarcproto; ap != NOARCPROTO; ap = ap->nextarcproto)
 	{
 		ap->nominalwidth = muldiv(ap->nominalwidth, newlambda, oldlambda);
-		var = getvalkey((INTBIG)ap, VARCPROTO, VINTEGER, el_arc_default_width);
-		if (var != NOVARIABLE)
-			var->addr = muldiv(var->addr, newlambda, oldlambda);
 	}
 
 	/* now change the default size of the primitive node prototypes */
@@ -619,12 +629,6 @@ void changetechnologylambda(TECHNOLOGY *tech, INTBIG newlambda)
 		np->highx = muldiv(np->highx, newlambda, oldlambda);
 		np->lowy  = muldiv(np->lowy,  newlambda, oldlambda);
 		np->highy = muldiv(np->highy, newlambda, oldlambda);
-		var = getvalkey((INTBIG)np, VNODEPROTO, VINTEGER|VISARRAY, el_node_default_size);
-		if (var != NOVARIABLE)
-		{
-			((INTBIG *)var->addr)[0] = muldiv(((INTBIG *)var->addr)[0], newlambda, oldlambda);
-			((INTBIG *)var->addr)[1] = muldiv(((INTBIG *)var->addr)[1], newlambda, oldlambda);
-		}
 	}
 
 	/* finally, reset the default lambda value */
@@ -667,9 +671,6 @@ void changeinternalunits(LIBRARY *whichlib, INTBIG oldunits, INTBIG newunits)
 			for(ap = tech->firstarcproto; ap != NOARCPROTO; ap = ap->nextarcproto)
 			{
 				hardscaleerror += db_scaleunits(&ap->nominalwidth, num, den);
-				var = getvalkey((INTBIG)ap, VARCPROTO, VINTEGER, el_arc_default_width);
-				if (var != NOVARIABLE)
-					hardscaleerror += db_scaleunits(&var->addr, num, den);
 			}
 
 			/* scale node sizes */
@@ -679,12 +680,6 @@ void changeinternalunits(LIBRARY *whichlib, INTBIG oldunits, INTBIG newunits)
 				hardscaleerror += db_scaleunits(&np->highx, num, den);
 				hardscaleerror += db_scaleunits(&np->lowy,  num, den);
 				hardscaleerror += db_scaleunits(&np->highy, num, den);
-				var = getvalkey((INTBIG)np, VNODEPROTO, VINTEGER|VISARRAY, el_node_default_size);
-				if (var != NOVARIABLE)
-				{
-					hardscaleerror += db_scaleunits(&((INTBIG *)var->addr)[0], num, den);
-					hardscaleerror += db_scaleunits(&((INTBIG *)var->addr)[1], num, den);
-				}
 			}
 
 			/* finally, scale lambda */
