@@ -63,10 +63,10 @@ static INTBIG maxnumpts = MAXNUMPTS; /* apply default (PJA:920527) */
 # define PTMATCH 2      /* point match */
 
 /* polygon direction and box edge identifiers */
-#define	UP          0
-#define	RIGHT       1
-#define	DOWN        2
-#define	LEFT        3
+#define UP          0
+#define RIGHT       1
+#define DOWN        2
+#define LEFT        3
 
 POLYLISTHEAD db_mrg_polycoordlist;
 POLYCOORD   *db_mrg_free_poly_list;
@@ -1849,7 +1849,6 @@ typedef struct Impoly
 MPOLY  *db_mpolylist = NOMPOLY;			/* the list of active MPOLYs */
 MPOLY  *db_mpolyfree = NOMPOLY;			/* the list of unused MPOLYs */
 MPOLY  *db_mergenewmp = NOMPOLY;		/* temporary MPOLY for merging */
-MPOLY  *db_mergeholemp = NOMPOLY;		/* temporary MPOLY for finding holes */
 
 /* for figuring out what to do at an intersection */
 INTBIG *db_mergeintpol;					/* polygon with intersection */
@@ -1858,7 +1857,7 @@ INTBIG *db_mergeintleaveangle;			/* angle of intersection */
 INTBIG  db_mergeintcount;				/* number of intersection */
 INTBIG  db_mergeinttotal = 0;			/* size of intersection arrays */
 
-INTSML db_mergeaddholetopoly(MPOLY *holemp, MPOLY *newmp);
+INTSML db_mergeaddholetopoly(MPOLY *holemplist, MPOLY *newmp);
 void   db_mergeaddintersectionpoint(INTBIG polygon, INTBIG point);
 INTSML db_mergeaddintersectionpoints(MPOLY *mp, MPOLY *omp);
 MPOLY *db_mergeallocpoly(void);
@@ -1874,10 +1873,12 @@ INTSML db_mergepoly(MPOLY *mp, MPOLY *omp);
 void   db_mergesimplifyandbbox(MPOLY *mp);
 INTSML db_mergewalkedge(MPOLY *mp, INTBIG i, MPOLY *omp, MPOLY *newmp);
 
-/* #define DEBUGWALK    1 */		/* uncomment to debug polygon point traversal */
-/* #define SHOWRESULTS  1 */		/* uncomment to show results of each merge */
-/* #define SHOWTICKS    1 */		/* uncomment to show coordinate values with results */
-/* #define DESIREDLAYER 6 */		/* uncomment to select a specific layer */
+/* #define DEBUGWALK       1 */		/* uncomment to debug polygon point traversal */
+/* #define SHOWRESULTS     1 */		/* uncomment to show results of each merge */
+/* #define SHOWTICKS       1 */		/* uncomment to show coordinate values with results */
+/* #define SHOWHOLEGROWTH  1 */		/* uncomment to show results of hole growth in a merge */
+#define DESIREDLAYER    6		/* uncomment to select a specific layer */
+#define RESULTSPACING 175		/* distance between result graphics */
 
 /*
  * Routine to free all memory associated with this module.
@@ -1895,7 +1896,6 @@ void db_freemergememory(void)
 
 	/* free memory used by polygonal merger */
 	if (db_mergenewmp != NOMPOLY) db_mergefreepoly(db_mergenewmp);
-	if (db_mergeholemp != NOMPOLY) db_mergefreepoly(db_mergeholemp);
 	while (db_mpolylist != NOMPOLY)
 	{
 		mp = db_mpolylist;
@@ -1953,6 +1953,7 @@ void mergestorepolygon(INTBIG layer, TECHNOLOGY *tech, POLYGON *poly)
 	INTBIG lx, hx, ly, hy;
 
 	/* copy the polygon into a local structure */
+	if (poly->count == 0) return;
 	mp = db_mergeallocpoly();
 	if (db_mergeensurespace(mp, poly->count) != 0) return;
 	if (poly->style == FILLEDRECT || poly->style == CLOSEDRECT)
@@ -2032,9 +2033,32 @@ void mergestorepolygon(INTBIG layer, TECHNOLOGY *tech, POLYGON *poly)
 		static INTBIG xoff = 0, yoff = 0;
 		char line[20];
 
-		xoff += el_curlib->lambda[el_curtech->techindex] * 150;
+		xoff += el_curlib->lambda[el_curtech->techindex] * RESULTSPACING;
 		facet = getcurfacet();
 		if (facet == NONODEPROTO) return;
+		prim = getnodeproto("metal-2-node");
+		lx = hx = poly->xv[0];
+		ly = hy = poly->yv[0];
+		for(i=1; i<poly->count; i++)
+		{
+			if (poly->xv[i] < lx) lx = poly->xv[i];
+			if (poly->xv[i] > hx) hx = poly->xv[i];
+			if (poly->yv[i] < ly) ly = poly->yv[i];
+			if (poly->yv[i] > hy) hy = poly->yv[i];
+		}
+		cx = (lx+hx) / 2;   cy = (ly+hy) / 2;
+		ni = newnodeinst(prim, lx+xoff, hx+xoff, ly+yoff, hy+yoff, 0, 0, facet);
+		newlist = (INTBIG *)emalloc(poly->count * 2 * SIZEOFINTBIG, el_tempcluster);
+		if (newlist == 0) return;
+		for(i=0; i<poly->count; i++)
+		{
+			newlist[i*2] = poly->xv[i] - cx;
+			newlist[i*2+1] = poly->yv[i] - cy;
+		}
+		(void)setvalkey((INTBIG)ni, VNODEINST, el_trace, (INTBIG)newlist,
+			VINTEGER|VISARRAY|((poly->count*2)<<VLENGTHSH));
+		endobjectchange((INTBIG)ni, VNODEINST);
+
 		prim = getnodeproto("metal-1-node");
 		for(mp = db_mpolylist; mp != NOMPOLY; mp = mp->nextmpoly)
 		{
@@ -2124,7 +2148,7 @@ void mergestorepolygon(INTBIG layer, TECHNOLOGY *tech, POLYGON *poly)
 						VSTRING|VDISPLAY);
 					if (var != NOVARIABLE)
 						var->textdescript = (var->textdescript & ~(VTSIZE|VTPOSITION)) |
-							(VTPOSDOWN | (1<<VTSIZESH));
+							(VTPOSDOWN | (TXT1L<<VTSIZESH));
 					endobjectchange((INTBIG)ni, VNODEINST);
 				}
 			}
@@ -2152,7 +2176,7 @@ void mergestorepolygon(INTBIG layer, TECHNOLOGY *tech, POLYGON *poly)
 						VSTRING|VDISPLAY);
 					if (var != NOVARIABLE)
 						var->textdescript = (var->textdescript & ~(VTSIZE|VTPOSITION)) |
-							(VTPOSLEFT | (1<<VTSIZESH));
+							(VTPOSLEFT | (TXT1L<<VTSIZESH));
 					endobjectchange((INTBIG)ni, VNODEINST);
 				}
 			}
@@ -2190,8 +2214,8 @@ void mergedone(void (*writepolygon)(INTSML, TECHNOLOGY*, INTBIG*, INTBIG*, INTSM
  */
 INTSML db_mergepoly(MPOLY *mp, MPOLY *omp)
 {
-	REGISTER INTBIG i, oi, startpol, startpt, intpoints;
-	MPOLY *mps[2];
+	REGISTER INTBIG i, oi, j, startpol, startpt, intpoints;
+	MPOLY *mps[2], *holepolylist, *holepoly;
 
 	/* reject immediately if they are on different layers */
 	if (omp->tech != mp->tech || omp->layer != mp->layer) return(0);
@@ -2270,31 +2294,60 @@ INTSML db_mergepoly(MPOLY *mp, MPOLY *omp)
 	if (db_mergewalkedge(mps[startpol], startpt, mps[1-startpol], db_mergenewmp) != 0)
 		return(0);
 
-	/* see if there are any points that didn't get seen and are not in the other polygon */
-	if (db_mergeholemp == NOMPOLY) db_mergeholemp = db_mergeallocpoly();
+	/* look for holes */
+	holepolylist = NOMPOLY;
+
+	/* see if there are any nonintersection points that didn't get seen and are not in the other polygon */
 	for(i=0; i<mp->count; i++)
 	{
 		if (mp->seen[i] != 0) continue;
-#if 0
 		if (mp->intersect[i] != 0) continue;
-#endif
 		if (db_mergeisinside(mp->x[i], mp->y[i], omp) != 0) continue;
-		db_mergeholemp->count = 0;
-		if (db_mergewalkedge(mp, i, omp, db_mergeholemp) != 0) return(0);
-		if (db_mergeaddholetopoly(db_mergeholemp, db_mergenewmp) != 0) return(0);
+		holepoly = db_mergeallocpoly();
+		if (db_mergewalkedge(mp, i, omp, holepoly) != 0) return(0);
+		holepoly->nextmpoly = holepolylist;
+		holepolylist = holepoly;
 	}
 	for(oi=0; oi<omp->count; oi++)
 	{
 		if (omp->seen[oi] != 0) continue;
-#if 0
 		if (omp->intersect[oi] != 0) continue;
-#endif
 		if (db_mergeisinside(omp->x[oi], omp->y[oi], mp) != 0) continue;
-		db_mergeholemp->count = 0;
-		if (db_mergewalkedge(omp, oi, mp, db_mergeholemp) != 0) return(0);
-		if (db_mergeaddholetopoly(db_mergeholemp, db_mergenewmp) != 0) return(0);
+		holepoly = db_mergeallocpoly();
+		if (db_mergewalkedge(omp, oi, mp, holepoly) != 0) return(0);
+		holepoly->nextmpoly = holepolylist;
+		holepolylist = holepoly;
 	}
 
+	/* see if there are any points that didn't get seen and are not in the other polygon */
+	for(i=0; i<mp->count; i++)
+	{
+		if (mp->seen[i] != 0) continue;
+		for(j=0; j<mp->count; j++)
+			if (mp->seen[j] != 0 && mp->x[j] == mp->x[i] && mp->y[j] == mp->y[i]) break;
+		if (j < mp->count) continue;
+		if (db_mergeisinside(mp->x[i], mp->y[i], omp) != 0) continue;
+		holepoly = db_mergeallocpoly();
+		if (db_mergewalkedge(mp, i, omp, holepoly) != 0) return(0);
+		holepoly->nextmpoly = holepolylist;
+		holepolylist = holepoly;
+	}
+	for(oi=0; oi<omp->count; oi++)
+	{
+		if (omp->seen[oi] != 0) continue;
+		for(j=0; j<omp->count; j++)
+			if (omp->seen[j] != 0 && omp->x[j] == omp->x[oi] && omp->y[j] == omp->y[oi]) break;
+		if (j < omp->count) continue;
+		if (db_mergeisinside(omp->x[oi], omp->y[oi], mp) != 0) continue;
+		holepoly = db_mergeallocpoly();
+		if (db_mergewalkedge(omp, oi, mp, holepoly) != 0) return(0);
+		holepoly->nextmpoly = holepolylist;
+		holepolylist = holepoly;
+	}
+
+	/* carve holes from polygon */
+	if (db_mergeaddholetopoly(holepolylist, db_mergenewmp) != 0) return(0);
+
 	/* copy result back to second polygon */
 	if (db_mergecopypoly(db_mergenewmp, omp) != 0) return(0);
 	db_mergesimplifyandbbox(omp);
@@ -2347,41 +2400,214 @@ INTBIG db_mergefindstartingpoint(MPOLY *mp, MPOLY *omp)
  * Finds the closest points on the two and adds the points.
  * Returns nonzero on error.
  */
-INTSML db_mergeaddholetopoly(MPOLY *holemp, MPOLY *newmp)
+INTSML db_mergeaddholetopoly(MPOLY *holemplist, MPOLY *newmp)
 {
-	REGISTER INTBIG j, k, dist, bestdist, hint, nint, nx, ny;
+	REGISTER INTBIG j, k, l, dist, bestdist, hint, nint, nx, ny, hfx, hfy, htx, hty,
+		nfx, nfy, ntx, nty, lastj, lastk, solved;
 	INTBIG bestpos;
+	MPOLY *holemp, *besthole, *lasthole, *bestlasthole;
 
-	for(j=0; j<holemp->count; j++)
+	/* keep adding holes until there are no more */
+	while (holemplist != NOMPOLY)
 	{
+		solved = 0;
+		lasthole = NOMPOLY;
+		besthole = NOMPOLY;
+
+		/* mark points on the main polygon that are duplicated (can't attach holes there) */
+		for(k=0; k<newmp->count; k++) newmp->seen[k] = 0;
 		for(k=0; k<newmp->count; k++)
 		{
-			dist = computedistance(holemp->x[j], holemp->y[j], newmp->x[k], newmp->y[k]);
-			if ((j==0 && k==0) || dist < bestdist)
+			for(j=k+1; j<newmp->count; j++)
 			{
-				bestdist = dist;
-				hint = j;
-				nint = k;
+				if (newmp->x[k] == newmp->x[j] && newmp->y[k] == newmp->y[j])
+					newmp->seen[k] = newmp->seen[j] = 1;
 			}
 		}
-	}
-	bestpos = nint+1;
-	nx = newmp->x[nint];   ny = newmp->y[nint];
-	if (nx != holemp->x[hint] || ny != holemp->y[hint])
-	{
-		if (db_mergeinsertpoint(newmp, &bestpos, holemp->x[hint], holemp->y[hint]) != 0) return(1);
-	}
-	j = hint;
-	for(;;)
-	{
-		j++;
-		if (j >= holemp->count) j = 0;
-		if (db_mergeinsertpoint(newmp, &bestpos, holemp->x[j], holemp->y[j]) != 0) return(1);
-		if (j == hint) break;
-	}
-	if (nx != holemp->x[hint] || ny != holemp->y[hint])
-	{
-		if (db_mergeinsertpoint(newmp, &bestpos, nx, ny) != 0) return(1);
+
+		/* examine every hole to find the next one to add */
+		for(holemp = holemplist; holemp != NOMPOLY; holemp = holemp->nextmpoly)
+		{
+			/* find point on hole that is closest to a polygon point */
+			for(j=0; j<holemp->count; j++)
+			{
+				/* get coordinates of the hole segment */
+				if (j == 0) lastj = holemp->count - 1; else
+					lastj = j - 1;
+				hfx = holemp->x[lastj];   hfy = holemp->y[lastj];
+				htx = holemp->x[j];       hty = holemp->y[j];
+				for(k=0; k<newmp->count; k++)
+				{
+					/* get coordinates of the main polygon segment */
+					if (k == 0) lastk = newmp->count - 1; else
+						lastk = k - 1;
+					nfx = newmp->x[lastk];   nfy = newmp->y[lastk];
+					ntx = newmp->x[k];       nty = newmp->y[k];
+
+					/* see if they are colinear */
+					if (hfx == htx && hfx == nfx && nfx == ntx)
+					{
+						/* vertically aligned */
+						if (mini(hfy, hty) < maxi(nfy, nty) && maxi(hfy, hty) > mini(nfy, nty))
+						{
+							bestpos = k;
+							l = lastj;
+							for(;;)
+							{
+								l++;
+								if (l >= holemp->count) l = 0;
+								if (db_mergeinsertpoint(newmp, &bestpos, holemp->x[l], holemp->y[l]) != 0) return(1);
+#ifdef SHOWHOLEGROWTH
+								ttyputmsg(" Adding carveout point (%s,%s)", latoa(holemp->x[l]), latoa(holemp->y[l]));
+#endif
+								if (l == lastj) break;
+							}
+							besthole = holemp;
+							bestlasthole = lasthole;
+							solved = 1;
+							break;
+						}
+					}
+					if (hfy == hty && hfy == nfy && nfy == nty)
+					{
+						if (mini(hfx, htx) < maxi(nfx, ntx) && maxi(hfx, htx) > mini(nfx, ntx))
+						{
+							bestpos = k;
+							l = lastj;
+							for(;;)
+							{
+								l++;
+								if (l >= holemp->count) l = 0;
+								if (db_mergeinsertpoint(newmp, &bestpos, holemp->x[l], holemp->y[l]) != 0) return(1);
+								if (l == lastj) break;
+							}
+							besthole = holemp;
+							bestlasthole = lasthole;
+							solved = 1;
+							break;
+						}
+					}
+
+					/* not colinear: look for proximity */
+					if (newmp->seen[k] == 0)
+					{
+						dist = computedistance(holemp->x[j], holemp->y[j], newmp->x[k], newmp->y[k]);
+						if (besthole == NOMPOLY || dist < bestdist)
+						{
+							bestdist = dist;
+							hint = j;
+							nint = k;
+							besthole = holemp;
+							bestlasthole = lasthole;
+						}
+					}
+				}
+				if (solved != 0) break;
+			}
+			lasthole = holemp;
+		}
+		if (solved == 0)
+		{
+			if (besthole == NOMPOLY)
+			{
+				ttyputerr("ERROR merging polygon: cannot remove hole");
+				break;
+			}
+			bestpos = nint+1;
+			nx = newmp->x[nint];   ny = newmp->y[nint];
+#ifdef SHOWHOLEGROWTH
+			ttyputmsg("Hole point %d (%s,%s) closest to polygon point %d (%s,%s)",hint, latoa(besthole->x[hint]),
+				latoa(besthole->y[hint]), nint, latoa(nx), latoa(ny));
+#endif
+			if (nx != besthole->x[hint] || ny != besthole->y[hint])
+			{
+#ifdef SHOWHOLEGROWTH
+				ttyputmsg(" Adding bridge point (%s,%s)", latoa(besthole->x[hint]), latoa(besthole->y[hint]));
+#endif
+				if (db_mergeinsertpoint(newmp, &bestpos, besthole->x[hint], besthole->y[hint]) != 0) return(1);
+			}
+			j = hint;
+			for(;;)
+			{
+				j++;
+				if (j >= besthole->count) j = 0;
+				if (db_mergeinsertpoint(newmp, &bestpos, besthole->x[j], besthole->y[j]) != 0) return(1);
+#ifdef SHOWHOLEGROWTH
+				ttyputmsg(" Adding point (%s,%s)", latoa(besthole->x[j]), latoa(besthole->y[j]));
+#endif
+				if (j == hint) break;
+			}
+			if (nx != besthole->x[hint] || ny != besthole->y[hint])
+			{
+#ifdef SHOWHOLEGROWTH
+				ttyputmsg(" Adding bridge point (%s,%s)", latoa(nx), latoa(ny));
+#endif
+				if (db_mergeinsertpoint(newmp, &bestpos, nx, ny) != 0) return(1);
+			}
+		}
+
+#ifdef SHOWHOLEGROWTH
+		{
+			REGISTER NODEPROTO *facet, *prim;
+			REGISTER NODEINST *ni;
+			REGISTER INTBIG i, lx, hx, ly, hy, cx, cy, *newlist;
+			static INTBIG xoff = 0, yoff = 0;
+
+			xoff += el_curlib->lambda[el_curtech->techindex] * RESULTSPACING;
+			facet = getcurfacet();
+			if (facet == NONODEPROTO) return(1);
+			prim = getnodeproto("metal-2-node");
+			lx = hx = besthole->x[0];
+			ly = hy = besthole->y[0];
+			for(i=1; i<besthole->count; i++)
+			{
+				if (besthole->x[i] < lx) lx = besthole->x[i];
+				if (besthole->x[i] > hx) hx = besthole->x[i];
+				if (besthole->y[i] < ly) ly = besthole->y[i];
+				if (besthole->y[i] > hy) hy = besthole->y[i];
+			}
+			cx = (lx+hx) / 2;   cy = (ly+hy) / 2;
+			ni = newnodeinst(prim, lx+xoff, hx+xoff, ly+yoff, hy+yoff, 0, 0, facet);
+			newlist = (INTBIG *)emalloc(besthole->count * 2 * SIZEOFINTBIG, el_tempcluster);
+			if (newlist == 0) return(1);
+			for(i=0; i<besthole->count; i++)
+			{
+				newlist[i*2] = besthole->x[i] - cx;
+				newlist[i*2+1] = besthole->y[i] - cy;
+			}
+			(void)setvalkey((INTBIG)ni, VNODEINST, el_trace, (INTBIG)newlist,
+				VINTEGER|VISARRAY|((besthole->count*2)<<VLENGTHSH));
+			efree((char *)newlist);
+			endobjectchange((INTBIG)ni, VNODEINST);
+
+			prim = getnodeproto("metal-1-node");
+			lx = hx = newmp->x[0];
+			ly = hy = newmp->y[0];
+			for(i=1; i<newmp->count; i++)
+			{
+				if (newmp->x[i] < lx) lx = newmp->x[i];
+				if (newmp->x[i] > hx) hx = newmp->x[i];
+				if (newmp->y[i] < ly) ly = newmp->y[i];
+				if (newmp->y[i] > hy) hy = newmp->y[i];
+			}
+			cx = (lx+hx) / 2;   cy = (ly+hy) / 2;
+			ni = newnodeinst(prim, lx+xoff, hx+xoff, ly+yoff, hy+yoff, 0, 0, facet);
+			newlist = (INTBIG *)emalloc(newmp->count * 2 * SIZEOFINTBIG, el_tempcluster);
+			if (newlist == 0) return(1);
+			for(i=0; i<newmp->count; i++)
+			{
+				newlist[i*2] = newmp->x[i] - cx;
+				newlist[i*2+1] = newmp->y[i] - cy;
+			}
+			(void)setvalkey((INTBIG)ni, VNODEINST, el_trace, (INTBIG)newlist,
+				VINTEGER|VISARRAY|((newmp->count*2)<<VLENGTHSH));
+			efree((char *)newlist);
+			endobjectchange((INTBIG)ni, VNODEINST);
+		}
+#endif
+		if (bestlasthole == NOMPOLY) holemplist = besthole->nextmpoly; else
+			bestlasthole->nextmpoly = besthole->nextmpoly;
+		db_mergefreepoly(besthole);
 	}
 	return(0);
 }
@@ -2489,10 +2715,7 @@ INTSML db_mergewalkedge(MPOLY *mp, INTBIG i, MPOLY *omp, MPOLY *newmp)
 	static INTBIG seen = 0;
 
 #ifdef DEBUGWALK
-	ttyputmsg("Walking around edge, polygon 0 has %d points:", mp->count);
-	for(pt=0; pt<mp->count; pt++) ttyputmsg("  (%s,%s)", latoa(mp->x[pt]), latoa(mp->y[pt]));
-	ttyputmsg("Polygon 1 has %d points:", omp->count);
-	for(pt=0; pt<omp->count; pt++) ttyputmsg("  (%s,%s)", latoa(omp->x[pt]), latoa(omp->y[pt]));
+	ttyputmsg("Walking around edge, polygon 0 has %d points, polygon 1 has %d", mp->count, omp->count);
 #endif
 	seen++;
 	if (seen == 0) seen++;
@@ -2514,7 +2737,7 @@ INTSML db_mergewalkedge(MPOLY *mp, INTBIG i, MPOLY *omp, MPOLY *newmp)
 		if (newmp->count == 1000)
 		{
 			ttyputerr(_("Warning: loop detected while merging polygons on layer %s"),
-				layername(mp->tech, (INTSML)mp->layer));
+				layername(mp->tech, mp->layer));
 			return(1);
 		}
 
