@@ -61,12 +61,21 @@ enum {USEFLOAT = 1, USEINTEGER = 2};
  */
 INTBIG mult(INTBIG a,INTBIG b)
 {
+#if 1
+	INTHUGE c;
+
+	c = a;
+	c *= b;
+	c >>= 30;
+	return((INTBIG)c);
+#else
 	/* if it hasn't been done yet, see which way is faster */
 	if (db_multstyle == 0) db_checkfloat();
 
 	/* do the appropriate version */
 	if (db_multstyle == USEFLOAT) return(db_floatmult(a, b));
 	return(db_intmult(a, b));
+#endif
 }
 
 /*
@@ -108,13 +117,21 @@ INTBIG db_intmult(INTBIG a, INTBIG b)
  */
 INTBIG muldiv(INTBIG ai, INTBIG bi, INTBIG ci)
 {
+#if 1
+	INTHUGE v;
 
+	v = ai;
+	v *= bi;
+	v /= ci;
+	return((INTBIG)v);
+#else
 	/* if it hasn't been done yet, see which way is faster */
 	if (db_muldivstyle == 0) db_checkfloat();
 
 	/* do the appropriate version */
 	if (db_muldivstyle == USEFLOAT) return(db_floatmuldiv(ai, bi, ci));
 	return(db_intmuldiv(ai, bi, ci));
+#endif
 }
 
 /*
