@@ -35,15 +35,20 @@
 
 /* hierarchical traversal data structures */
 NODEINST **db_hierpath;
-INTSML     db_hierdepth = 0;
-INTSML     db_hierlimit = 0;
+INTBIG     db_hierdepth = 0;
+INTBIG     db_hierlimit = 0;
 
 /* prototypes for local routines */
 INTSML db_schematic_pages(NODEPROTO*);
-char *db_getframedesc(NODEPROTO*);
+char  *db_getframedesc(NODEPROTO*);
 INTSML db_copyxlibvars(INTBIG, INTBIG, INTBIG, INTBIG, NODEPROTO*, LIBRARY*);
 INTSML db_portassociate(NODEINST*, NODEINST*, INTSML);
 INTSML db_doesntconnect(ARCPROTO*[], PORTPROTO*);
+void   db_changeallports(PORTPROTO*);
+INTSML db_addtohierpath(NODEINST *ni);
+void   db_insertcell(CELL *cell);
+void   db_retractcell(CELL *cell);
+INTBIG db_namehash(char *name);
 
 /*
  * Routine to free all memory associated with this module.
@@ -126,6 +131,7 @@ NODEPROTO *allocnodeproto(CLUSTER *cluster)
 	np->firstnodeinst = NONODEINST;
 	np->firstarcinst = NOARCINST;
 	np->firstportproto = NOPORTPROTO;
+	np->cachedequivfacet = NONODEPROTO;
 	np->tech = NOTECHNOLOGY;
 	np->nextnodeproto = NONODEPROTO;
 	np->cell = NOCELL;
@@ -182,7 +188,10 @@ NODEPROTO *newnodeproto(char *fname, LIBRARY *lib)
 		nameend = ptr;
 		for(opt = ptr+1; *opt != 0; opt++) if (*opt == '}') break;
 		if (*opt == 0 || opt[1] != 0)
+		{
+			db_donextchangequietly = 0;
 			return((NODEPROTO *)db_error(DBBADNAME|DBNEWNODEPROTO));
+		}
 		*opt = 0;
 		for(v = el_views; v != NOVIEW; v = v->nextview)
 			if (namesame(&ptr[1], v->sviewname) == 0) break;
@@ -194,7 +203,11 @@ NODEPROTO *newnodeproto(char *fname, LIBRARY *lib)
 		{
 			/* create a new schematic page view */
 			cpt = (char *)emalloc((16+strlen(&ptr[2])), el_tempcluster);
-			if (cpt == 0) return(NONODEPROTO);
+			if (cpt == 0)
+			{
+				db_donextchangequietly = 0;
+				return(NONODEPROTO);
+			}
 			(void)strcpy(cpt, "schematic-page-");
 			(void)strcat(cpt, &ptr[2]);
 			v = newview(cpt, &ptr[1]);
@@ -202,7 +215,11 @@ NODEPROTO *newnodeproto(char *fname, LIBRARY *lib)
 		}
 
 		*opt = '}';
-		if (v == NOVIEW) return((NODEPROTO *)db_error(DBBADNAME|DBNEWNODEPROTO));
+		if (v == NOVIEW)
+		{
+			db_donextchangequietly = 0;
+			return((NODEPROTO *)db_error(DBBADNAME|DBNEWNODEPROTO));
+		}
 		view = v;
 	}
 
@@ -213,21 +230,31 @@ NODEPROTO *newnodeproto(char *fname, LIBRARY *lib)
 	{
 		if (nameend == 0 || ptr < nameend) nameend = ptr;
 		explicitversion = atoi(&ptr[1]);
-		if (explicitversion <= 0) return((NODEPROTO *)db_error(DBBADNAME|DBNEWNODEPROTO));
+		if (explicitversion <= 0)
+		{
+			db_donextchangequietly = 0;
+			return((NODEPROTO *)db_error(DBBADNAME|DBNEWNODEPROTO));
+		}
 	}
 	if (nameend == 0) nameend = ptr;
 
 	/* scan main facet name for illegal characters */
-	if (*fname == 0) return((NODEPROTO *)db_error(DBBADNAME|DBNEWNODEPROTO));
+	if (*fname == 0)
+	{
+		db_donextchangequietly = 0;
+		return((NODEPROTO *)db_error(DBBADNAME|DBNEWNODEPROTO));
+	}
 	for(opt = fname; opt != ptr; opt++)
 		if (*opt <= ' ' || *opt == ':' || *opt >= 0177)
-			return((NODEPROTO *)db_error(DBBADNAME|DBNEWNODEPROTO));
+	{
+		db_donextchangequietly = 0;
+		return((NODEPROTO *)db_error(DBBADNAME|DBNEWNODEPROTO));
+	}
 
 	/* see if the cell name exists */
 	save = *nameend;
 	*nameend = 0;
-	for(c = lib->firstcell; c != NOCELL; c = c->nextcell)
-		if (namesame(c->cellname, fname) == 0) break;
+	c = db_findcellname(fname, lib);
 	if (c == NOCELL)
 	{
 		/* new cell name, create it */
@@ -235,6 +262,7 @@ NODEPROTO *newnodeproto(char *fname, LIBRARY *lib)
 		if (c == NOCELL)
 		{
 			*nameend = (char)save;
+			db_donextchangequietly = 0;
 			return(NONODEPROTO);
 		}
 		c->lib = lib;
@@ -242,13 +270,22 @@ NODEPROTO *newnodeproto(char *fname, LIBRARY *lib)
 		(void)allocstring(&c->cellname, fname, c->cluster);
 		(void)strncpy(c->cluster->clustername, fname, 29);
 		*ptr = (char)save;
-		(void)db_change((INTBIG)c, OBJECTNEW, VCELL, 0, 0, 0, 0, 0);
+
+		/* handle change control and broadcast */
+		if (db_donextchangequietly == 0 && db_dochangesquietly == 0)
+		{
+			(void)db_change((INTBIG)c, OBJECTNEW, VCELL, 0, 0, 0, 0, 0);
+		}
 	}
 	*nameend = (char)save;
 
 	/* create the facet */
 	np = allocnodeproto(c->cluster);
-	if (np == NONODEPROTO) return(NONODEPROTO);
+	if (np == NONODEPROTO)
+	{
+		db_donextchangequietly = 0;
+		return(NONODEPROTO);
+	}
 
 	/* initialize */
 	np->primindex = 0;
@@ -272,7 +309,10 @@ NODEPROTO *newnodeproto(char *fname, LIBRARY *lib)
 		np->version = (INTSML)explicitversion;
 		for(onp = c->firstincell; onp != NONODEPROTO; onp = onp->nextincell)
 			if (onp->cellview == view && onp->version == explicitversion)
-				return((NODEPROTO *)db_error(DBBADNAME|DBNEWNODEPROTO));
+		{
+			db_donextchangequietly = 0;
+			return((NODEPROTO *)db_error(DBBADNAME|DBNEWNODEPROTO));
+		}
 	} else
 	{
 		np->version = 1;
@@ -284,15 +324,24 @@ NODEPROTO *newnodeproto(char *fname, LIBRARY *lib)
 	db_insertnodeproto(np);
 
 	/* create initial R-tree data */
-	if (geomstructure(np) != 0) return(NONODEPROTO);
+	if (geomstructure(np) != 0)
+	{
+		db_donextchangequietly = 0;
+		return(NONODEPROTO);
+	}
 
-	/* report the new facet */
-	np->changeaddr = (char *)db_change((INTBIG)np, NODEPROTONEW, 0, 0, 0, 0, 0, 0);
+	/* handle change control, constraint, and broadcast */
+	if (db_donextchangequietly == 0 && db_dochangesquietly == 0)
+	{
+		/* report the new facet */
+		np->changeaddr = (char *)db_change((INTBIG)np, NODEPROTONEW, 0, 0, 0, 0, 0, 0);
 
-	/* tell constraint system about new facet */
-	(*el_curconstraint->newobject)((INTBIG)np, VNODEPROTO);
+		/* tell constraint system about new facet */
+		(*el_curconstraint->newobject)((INTBIG)np, VNODEPROTO);
 
-	db_setchangefacet(np);
+		db_setchangefacet(np);
+	}
+	db_donextchangequietly = 0;
 
 	/* tell which facet it is */
 	return(np);
@@ -395,19 +444,31 @@ INTSML killnodeproto(NODEPROTO *np)
 
 	/* cannot delete primitive nodeprotos */
 	if (np == NONODEPROTO)
+	{
+		db_donextchangequietly = 0;
 		return((INTSML)db_error(DBBADPROTO|DBKILLNODEPROTO));
+	}
 	if (np->primindex != 0)
+	{
+		db_donextchangequietly = 0;
 		return((INTSML)db_error(DBPRIMITIVE|DBKILLNODEPROTO));
+	}
 
 	/* search the library for the nodeproto */
 	for(lnt = np->cell->lib->firstnodeproto; lnt != NONODEPROTO; lnt = lnt->nextnodeproto)
 		if (np == lnt) break;
 	if (lnt == NONODEPROTO)
+	{
+		db_donextchangequietly = 0;
 		return((INTSML)db_error(DBBADFACET|DBKILLNODEPROTO));
+	}
 
 	/* cannot delete nodeproto with any instances */
 	if (np->firstinst != NONODEINST)
+	{
+		db_donextchangequietly = 0;
 		return((INTSML)db_error(DBHASINSTANCES|DBKILLNODEPROTO));
+	}
 
 	/* delete everything in the facet */
 	for(pp = np->firstportproto; pp != NOPORTPROTO; pp = npt)
@@ -421,16 +482,25 @@ INTSML killnodeproto(NODEPROTO *np)
 	/* remove nodeproto from library */
 	db_retractnodeproto(np);
 
-	/* remove any "changefacet" modules for this facet */
-	db_removechangefacet(np);
+	/* handle change control, constraint, and broadcast */
+	if (db_donextchangequietly == 0 && db_dochangesquietly == 0)
+	{
+		/* remove any "changefacet" modules for this facet */
+		db_removechangefacet(np);
 
-	/* record the change to the facet for broadcasting */
-	np->changeaddr = (char *)db_change((INTBIG)np, NODEPROTOKILL, 0, 0, 0, 0, 0, 0);
-	if (np->cell->firstincell == NONODEPROTO)
-		(void)db_change((INTBIG)np->cell, OBJECTKILL, VCELL, 0, 0, 0, 0, 0);
+		/* record the change to the facet for broadcasting */
+		np->changeaddr = (char *)db_change((INTBIG)np, NODEPROTOKILL, 0, 0, 0, 0, 0, 0);
+		if (np->cell->firstincell == NONODEPROTO)
+			(void)db_change((INTBIG)np->cell, OBJECTKILL, VCELL, 0, 0, 0, 0, 0);
 
-	/* tell constraint system about killed facet */
-	(*el_curconstraint->killobject)((INTBIG)np, VNODEPROTO);
+		/* tell constraint system about killed facet */
+		(*el_curconstraint->killobject)((INTBIG)np, VNODEPROTO);
+	} else
+	{
+		/* delete the facet now: no change control */
+		freenodeproto(np);
+	}
+	db_donextchangequietly = 0;
 
 	return(0);
 }
@@ -441,31 +511,18 @@ INTSML killnodeproto(NODEPROTO *np)
 void db_insertnodeproto(NODEPROTO *np)
 {
 	REGISTER NODEPROTO *ent, *onp, *lastnp;
-	REGISTER CELL *oc;
 
 	/* first append to the list of facets in the library */
-	if (np->cell->lib->firstnodeproto == NONODEPROTO)
-	{
-		np->cell->lib->firstnodeproto = np;   ent = NONODEPROTO;
-	} else
-	{
-		ent = np->cell->lib->firstnodeproto;
-		while (ent->nextnodeproto != NONODEPROTO) ent = ent->nextnodeproto;
-		ent->nextnodeproto = np;
-	}
-	np->lastnodeproto = ent;
+	np->lastnodeproto = np->cell->lib->tailnodeproto;
 	np->nextnodeproto = NONODEPROTO;
+	if (np->cell->lib->tailnodeproto == NONODEPROTO) np->cell->lib->firstnodeproto = np; else
+		np->cell->lib->tailnodeproto->nextnodeproto = np;
+	np->cell->lib->tailnodeproto = np;
 
 	/* if this is the first facet in the cell, insert the cell structure */
 	if (np->cell->firstincell == NONODEPROTO)
 	{
-		if (np->cell->lib->firstcell == NOCELL) np->cell->lib->firstcell = np->cell; else
-		{
-			oc = np->cell->lib->firstcell;
-			while (oc->nextcell != NOCELL) oc = oc->nextcell;
-			oc->nextcell = np->cell;
-		}
-		np->cell->nextcell = NOCELL;
+		db_insertcell(np->cell);
 	}
 
 	/* see if there is already a facet with this view */
@@ -518,14 +575,15 @@ void db_insertnodeproto(NODEPROTO *np)
 void db_retractnodeproto(NODEPROTO *np)
 {
 	REGISTER NODEPROTO *onp, *lastnp;
-	REGISTER CELL *c, *lastc, *lc;
+	REGISTER CELL *c;
 
 	/* remove facet from list in the library */
 	if (np->lastnodeproto == NONODEPROTO)
 		np->cell->lib->firstnodeproto = np->nextnodeproto; else
 			np->lastnodeproto->nextnodeproto = np->nextnodeproto;
-	if (np->nextnodeproto != NONODEPROTO)
-		np->nextnodeproto->lastnodeproto = np->lastnodeproto;
+	if (np->nextnodeproto == NONODEPROTO)
+		np->cell->lib->tailnodeproto = np->lastnodeproto; else
+			np->nextnodeproto->lastnodeproto = np->lastnodeproto;
 
 	/* remove facet from list */
 	c = np->cell;
@@ -569,15 +627,167 @@ void db_retractnodeproto(NODEPROTO *np)
 	/* remove cell from library if it now stands alone */
 	if (c->firstincell == NONODEPROTO)
 	{
-		lastc = NOCELL;
-		for(lc = c->lib->firstcell; lc != NOCELL; lc = lc->nextcell)
-		{
-			if (c == lc) break;
-			lastc = lc;
-		}
-		if (lastc == NOCELL) c->lib->firstcell = c->nextcell; else
-			lastc->nextcell = c->nextcell;
+		db_retractcell(c);
 	}
+
+	/* clear cache of port associations in this cell */
+	db_clearportcache(np);
+}
+
+/*
+ * routine to add cell "c" to the library
+ */
+void db_insertcell(CELL *cell)
+{
+	REGISTER INTBIG i, j;
+	REGISTER LIBRARY *lib;
+
+	lib = cell->lib;
+	if (lib->firstcell == NOCELL) lib->firstcell = cell; else
+		lib->tailcell->nextcell = cell;
+	lib->tailcell = cell;
+	cell->nextcell = NOCELL;
+	lib->numcells++;
+
+	/* insert into hash table of cell names */
+	if (lib->numcells < lib->cellhashtablesize/2)
+	{
+		i = db_namehash(cell->cellname) % lib->cellhashtablesize;
+		for(j=1; j<=lib->cellhashtablesize; j += 2)
+		{
+			if (lib->cellhashtable[i] == NOCELL)
+			{
+				lib->cellhashtable[i] = cell;
+				break;
+			}
+			i += j;
+			if (i >= lib->cellhashtablesize) i -= lib->cellhashtablesize;
+		}
+	} else
+	{
+		db_buildcellhashtable(cell->lib);
+	}
+}
+
+/*
+ * routine to remove cell "cell" from the library
+ */
+void db_retractcell(CELL *cell)
+{
+	REGISTER CELL *lastc, *lc;
+
+	lastc = NOCELL;
+	for(lc = cell->lib->firstcell; lc != NOCELL; lc = lc->nextcell)
+	{
+		if (cell == lc) break;
+		lastc = lc;
+	}
+	if (lastc == NOCELL) cell->lib->firstcell = cell->nextcell; else
+		lastc->nextcell = cell->nextcell;
+	if (cell->lib->tailcell == cell) cell->lib->tailcell = lastc;
+	cell->lib->numcells--;
+
+	/* rebuild hash table of cell names */
+	db_buildcellhashtable(cell->lib);
+}
+
+/*
+ * Routine to rebuild the hash table of cell names in library "lib".
+ */
+void db_buildcellhashtable(LIBRARY *lib)
+{
+#ifdef HASHCELLNAMES
+	REGISTER INTBIG i, j, numcells, hashtablesize;
+	REGISTER CELL *c;
+
+	/* free the former table */
+	if (lib->cellhashtablesize > 0)
+	{
+		lib->cellhashtablesize = 0;
+		efree((char *)lib->cellhashtable);
+	}
+
+	/* determine the size of the hash table */
+	numcells = 0;
+	for(c = lib->firstcell; c != NOCELL; c = c->nextcell) numcells++;
+	if (numcells != lib->numcells)
+	{
+		ttyputmsg(_("Warning: number of cells in library %s corrected"), lib->libname);
+		lib->numcells = numcells;
+	}
+	hashtablesize = pickprime(numcells * 4);
+
+	/* create the hash table and clear it */
+	lib->cellhashtable = (CELL **)emalloc(hashtablesize * (sizeof (CELL *)), lib->cluster);
+	if (lib->cellhashtable == 0) return;
+	for(i=0; i<hashtablesize; i++)
+		lib->cellhashtable[i] = NOCELL;
+	lib->cellhashtablesize = hashtablesize;
+
+	/* insert all cells into the table */
+	for(c = lib->firstcell; c != NOCELL; c = c->nextcell)
+	{
+		i = db_namehash(c->cellname) % hashtablesize;
+		for(j=1; j<=hashtablesize; j += 2)
+		{
+			if (lib->cellhashtable[i] == NOCELL)
+			{
+				lib->cellhashtable[i] = c;
+				break;
+			}
+			i += j;
+			if (i >= hashtablesize) i -= hashtablesize;
+		}
+	}
+#endif
+}
+
+/*
+ * Routine to find the cell named "name" in library "lib".
+ * Returns NOCELL if not found.
+ */
+CELL *db_findcellname(char *name, LIBRARY *lib)
+{
+	REGISTER CELL *c;
+	REGISTER INTBIG i, j;
+
+	if (lib->cellhashtablesize > 0)
+	{
+		i = db_namehash(name) % lib->cellhashtablesize;
+		for(j=1; j<=lib->cellhashtablesize; j += 2)
+		{
+			c = lib->cellhashtable[i];
+			if (c == NOCELL) break;
+			if (namesame(name, c->cellname) == 0) return(c);
+			i += j;
+			if (i >= lib->cellhashtablesize) i -= lib->cellhashtablesize;
+		}
+	} else
+	{
+		for (c = lib->firstcell; c != NOCELL; c = c->nextcell)
+			if (namesame(name, c->cellname) == 0) return(c);
+	}
+	return(NOCELL);
+}
+
+#define	HASH_SH		5
+#define	HASH_MSK	((1 << HASH_SH) - 1)
+
+/*
+ * Routine to encode a cell name for the hash table.
+ */
+INTBIG db_namehash(char *name)
+{
+	REGISTER INTBIG hash, shiftout, c;
+	hash = 0;
+	while ((c = *name++ & 0377) != 0)
+	{
+		c = tolower(c);
+		shiftout = (hash >> (31 - HASH_SH)) & HASH_MSK;
+		hash = ((hash << HASH_SH) | shiftout) ^ c;
+	}
+	hash &= 0x7fffffff;
+	return(hash);
 }
 
 /*
@@ -600,14 +810,32 @@ NODEPROTO *getcurfacet(void)
  */
 void gettraversalpath(NODEINST ***nilist, INTSML *depth)
 {
+	REGISTER NODEPROTO *np;
+	REGISTER NODEINST *ni;
+
 	*nilist = db_hierpath;
-	*depth = db_hierdepth;
+	*depth = (INTSML)db_hierdepth;
+	if (*depth == 0)
+	{
+		/* not in an active traversal, see if one exists from edit sequences */
+		np = el_curlib->curnodeproto;
+		for(;;)
+		{
+			if (np == NONODEPROTO) break;
+			ni = descentparent(np);
+			if (ni == NONODEINST) break;
+			if (db_addtohierpath(ni) != 0) break;
+			np = ni->proto;
+		}
+		*depth = (INTSML)db_hierdepth;
+		*nilist = db_hierpath;
+		db_hierdepth = 0;
+	}
 }
 
 /*
- * routine to begin the traversal of the hierarchy, starting at facet "np".  At each level of the
- * traversal, the routine "hierroutine" is called with the facet and the user data.  The
- * initial packet to pass is in "data".
+ * routine to begin the traversal of the hierarchy.  At each level of the
+ * traversal, the routine "hierroutine" is called with the facet and the user data.
  */
 void begintraversehierarchy(void)
 {
@@ -619,33 +847,30 @@ void begintraversehierarchy(void)
  */
 void downhierarchy(NODEINST *ni)
 {
-	REGISTER INTSML i;
+	/* add this node to the traversal stack */
+	if (db_addtohierpath(ni) != 0) return;
+}
+
+INTSML db_addtohierpath(NODEINST *ni)
+{
+	REGISTER INTBIG i, newlimit;
 	REGISTER NODEINST **nilist;
 
 	/* make sure there is room in the list of nodes traversed */
 	if (db_hierdepth >= db_hierlimit)
 	{
-		nilist = (NODEINST **)emalloc((db_hierlimit+10) * (sizeof (NODEINST *)), db_cluster);
-		if (nilist == 0) return;
+		newlimit = db_hierlimit+10;
+		if (newlimit < db_hierdepth) newlimit = db_hierdepth;
+		nilist = (NODEINST **)emalloc(newlimit * (sizeof (NODEINST *)), db_cluster);
+		if (nilist == 0) return(1);
 		for(i=0; i<db_hierdepth; i++) nilist[i] = db_hierpath[i];
 		if (db_hierlimit > 0) efree((char *)db_hierpath);
 		db_hierpath = nilist;
-		db_hierlimit += 10;
+		db_hierlimit = newlimit;
 	}
-
-	/* add this node to the traversal stack */
 	db_hierpath[db_hierdepth] = ni;
 	db_hierdepth++;
-
-	/* invoke parameterization if any */
-	if ((ni->proto->userbits & HASPARAMS) != 0)
-	{
-		if (ni->proto->userbits != 0)
-		{
-			if (ni->proto->tech->paramnode != 0)
-				(*(ni->proto->tech->paramnode))(ni);
-		}
-	}
+	return(0);
 }
 
 /*
@@ -734,28 +959,54 @@ VIEW *newview(char *vname, char *svname)
 	/* error checks */
 	for(ptr = vname; *ptr != 0; ptr++)
 		if (*ptr <= ' ' || *ptr == ':' || *ptr >= 0177)
-			return((VIEW *)db_error(DBBADNAME|DBNEWVIEW));
+	{
+		db_donextchangequietly = 0;
+		return((VIEW *)db_error(DBBADNAME|DBNEWVIEW));
+	}
 	for(ptr = svname; *ptr != 0; ptr++)
 		if (*ptr <= ' ' || *ptr == ':' || *ptr >= 0177)
-			return((VIEW *)db_error(DBBADNAME|DBNEWVIEW));
+	{
+		db_donextchangequietly = 0;
+		return((VIEW *)db_error(DBBADNAME|DBNEWVIEW));
+	}
 	for(v = el_views; v != NOVIEW; v = v->nextview)
 		if (namesame(v->viewname, vname) == 0 || namesame(v->sviewname, svname) == 0)
-			return((VIEW *)db_error(DBDUPLICATE|DBNEWVIEW));
+	{
+		db_donextchangequietly = 0;
+		return((VIEW *)db_error(DBDUPLICATE|DBNEWVIEW));
+	}
 
 	/* create the view */
 	v = allocview();
-	if (v == NOVIEW) return(NOVIEW);
+	if (v == NOVIEW)
+	{
+		db_donextchangequietly = 0;
+		return(NOVIEW);
+	}
 
 	/* initialize view names */
-	if (allocstring(&v->viewname, vname, db_cluster) != 0) return(NOVIEW);
-	if (allocstring(&v->sviewname, svname, db_cluster) != 0) return(NOVIEW);
+	if (allocstring(&v->viewname, vname, db_cluster) != 0)
+	{
+		db_donextchangequietly = 0;
+		return(NOVIEW);
+	}
+	if (allocstring(&v->sviewname, svname, db_cluster) != 0)
+	{
+		db_donextchangequietly = 0;
+		return(NOVIEW);
+	}
 	if (namesamen(vname, "Schematic-Page-", 15) == 0) v->viewstate |= MULTIPAGEVIEW;
 
 	/* insert in list of views */
 	v->nextview = el_views;
 	el_views = v;
 
-	(void)db_change((INTBIG)v, OBJECTNEW, VVIEW, 0, 0, 0, 0, 0);
+	/* handle change control and broadcast */
+	if (db_donextchangequietly == 0 && db_dochangesquietly == 0)
+	{
+		(void)db_change((INTBIG)v, OBJECTNEW, VVIEW, 0, 0, 0, 0, 0);
+	}
+	db_donextchangequietly = 0;
 
 	/* return the view */
 	return(v);
@@ -771,12 +1022,20 @@ INTSML killview(VIEW *view)
 	REGISTER NODEPROTO *np;
 
 	/* make sure it is not one of the permanent (undeletable) views */
-	if ((view->viewstate&PERMANENTVIEW) != 0) return(1);
+	if ((view->viewstate&PERMANENTVIEW) != 0)
+	{
+		db_donextchangequietly = 0;
+		return(1);
+	}
 
 	/* make sure it is not in any facet */
 	for(lib = el_curlib; lib != NOLIBRARY; lib = lib->nextlibrary)
 		for(np = lib->firstnodeproto; np != NONODEPROTO; np = np->nextnodeproto)
-			if (np->cellview == view) return(1);
+			if (np->cellview == view)
+	{
+		db_donextchangequietly = 0;
+		return(1);
+	}
 
 	/* find the view */
 	lastv = NOVIEW;
@@ -791,7 +1050,12 @@ INTSML killview(VIEW *view)
 	if (lastv == NOVIEW) el_views = v->nextview; else
 		lastv->nextview = v->nextview;
 
-	(void)db_change((INTBIG)v, OBJECTKILL, VVIEW, 0, 0, 0, 0, 0);
+	/* handle change control and broadcast */
+	if (db_donextchangequietly == 0 && db_dochangesquietly == 0)
+	{
+		(void)db_change((INTBIG)v, OBJECTKILL, VVIEW, 0, 0, 0, 0, 0);
+	}
+	db_donextchangequietly = 0;
 	return(0);
 }
 
@@ -867,7 +1131,7 @@ NODEPROTO *iconview(NODEPROTO *np)
 
 /*
  * routine to see if facet "np" is an icon or a skeleton, and if so, return its
- * true contents facet.  Returns NONODEPROTO if the facet is not an icon
+ * true contents facet.  Returns NONODEPROTO if the contents facet is not found.
  */
 NODEPROTO *contentsview(NODEPROTO *np)
 {
@@ -963,17 +1227,38 @@ NODEPROTO *anyview(NODEPROTO *np, VIEW *v)
  */
 PORTPROTO *equivalentport(NODEPROTO *np, PORTPROTO *pp, NODEPROTO *enp)
 {
-	REGISTER PORTPROTO *epp;
+	REGISTER PORTPROTO *epp, *opp;
 
 	/* don't waste time searching if the two views are the same */
 	if (np == enp) return(pp);
-	for(epp = enp->firstportproto; epp != NOPORTPROTO; epp = epp->nextportproto)
-		if (namesame(epp->protoname, pp->protoname) == 0) return(epp);
+
+	/* load the cache if not already there */
+	if (enp != np->cachedequivfacet)
+	{
+		for(opp = np->firstportproto; opp != NOPORTPROTO; opp = opp->nextportproto)
+			opp->cachedequivport = NOPORTPROTO;
+		for(opp = np->firstportproto; opp != NOPORTPROTO; opp = opp->nextportproto)
+		{
+			for(epp = enp->firstportproto; epp != NOPORTPROTO; epp = epp->nextportproto)
+			{
+				if (namesame(epp->protoname, opp->protoname) != 0) continue;
+				opp->cachedequivport = epp;
+				break;
+			}
+		}
+		np->cachedequivfacet = enp;
+	}
+	epp = pp->cachedequivport;
+	if (epp != 0 && epp != NOPORTPROTO) return(epp);
 
 	/* don't report errors for global ports not on icons */
-	if (enp->cellview != el_iconview || (pp->userbits&BODYONLY) == 0)
-		ttyputerr(_("Warning: no port in facet %s corresponding to port %s in facet %s"),
-			describenodeproto(enp), pp->protoname, describenodeproto(np));
+	if (epp == NOPORTPROTO)
+	{
+		if (enp->cellview != el_iconview || (pp->userbits&BODYONLY) == 0)
+			ttyputerr(_("Warning: no port in facet %s corresponding to port %s in facet %s"),
+				describenodeproto(enp), pp->protoname, describenodeproto(np));
+	}
+	pp->cachedequivport = (PORTPROTO *)0;
 	return(NOPORTPROTO);
 }
 
@@ -1159,7 +1444,7 @@ void framepoly(INTSML boxnum, POLYGON *poly, NODEPROTO *np)
 		}
 		poly->count = 4;
 		poly->style = TEXTBOX;
-		poly->font = TXTMEDIUM;
+		poly->font = TXT6L;
 		poly->tech = el_curtech;
 		(void)sprintf(line, "%d", xsections-pindex);
 		poly->string = line;
@@ -1180,7 +1465,7 @@ void framepoly(INTSML boxnum, POLYGON *poly, NODEPROTO *np)
 		}
 		poly->count = 4;
 		poly->style = TEXTBOX;
-		poly->font = TXTMEDIUM;
+		poly->font = TXT6L;
 		poly->tech = el_curtech;
 		line[0] = 'A' + pindex;
 		line[1] = 0;
@@ -1209,7 +1494,7 @@ void framepoly(INTSML boxnum, POLYGON *poly, NODEPROTO *np)
 				poly->yv[2] = poly->yv[3] = poly->yv[0] + ylogobox / 6;
 				poly->count = 4;
 				poly->style = TEXTBOX;
-				poly->font = TXTMEDIUM;
+				poly->font = TXT6L;
 				poly->tech = el_curtech;
 				poly->string = ndstring;
 				break;
@@ -1230,7 +1515,7 @@ void framepoly(INTSML boxnum, POLYGON *poly, NODEPROTO *np)
 				poly->yv[2] = poly->yv[3] = poly->yv[0] + ylogobox / 4;
 				poly->count = 4;
 				poly->style = TEXTBOX;
-				poly->font = TXTLARGE;
+				poly->font = TXT6L;
 				poly->tech = el_curtech;
 				poly->string = (char *)var->addr;
 				break;
@@ -1251,7 +1536,7 @@ void framepoly(INTSML boxnum, POLYGON *poly, NODEPROTO *np)
 				poly->yv[2] = poly->yv[3] = poly->yv[0] + ylogobox / 6;
 				poly->count = 4;
 				poly->style = TEXTBOX;
-				poly->font = TXTMEDIUM;
+				poly->font = TXT6L;
 				poly->tech = el_curtech;
 				poly->string = ndstring;
 				break;
@@ -1272,7 +1557,7 @@ void framepoly(INTSML boxnum, POLYGON *poly, NODEPROTO *np)
 				poly->yv[2] = poly->yv[3] = poly->yv[0] + ylogobox / 6;
 				poly->count = 4;
 				poly->style = TEXTBOX;
-				poly->font = TXTMEDIUM;
+				poly->font = TXT6L;
 				poly->tech = el_curtech;
 				poly->string = ndstring;
 				break;
@@ -1293,7 +1578,7 @@ void framepoly(INTSML boxnum, POLYGON *poly, NODEPROTO *np)
 				poly->yv[2] = poly->yv[3] = poly->yv[0] + ylogobox / 4;
 				poly->count = 4;
 				poly->style = TEXTBOX;
-				poly->font = TXTLARGE;
+				poly->font = TXT6L;
 				poly->tech = el_curtech;
 				poly->string = (char *)var->addr;
 				break;
@@ -1338,7 +1623,7 @@ char *db_getframedesc(NODEPROTO *np)
 		var = getvalkey((INTBIG)np, VNODEPROTO, VSTRING, el_schematic_page_size);
 		if (var != NOVARIABLE)
 		{
-			(void)strcpy(line, (char *)var->addr);
+			(void)strncpy(line, (char *)var->addr, 9);
 			for(pt = line; *pt != 0; pt++) if (isupper(*pt)) *pt = tolower(*pt);
 		}
 	}
@@ -1743,9 +2028,13 @@ INTSML db_copyxlibvars(INTBIG fromaddr, INTBIG fromtype, INTBIG toaddr,
 /*
  * routine to replace nodeinst "ni" with one of type "np", leaving all arcs
  * intact.  The routine returns the address of the new replacing nodeinst if
- * successful, NONODEINST if the replacement cannot be done.
+ * successful, NONODEINST if the replacement cannot be done.  If "ignoreportnames"
+ * is nonzero, do not use port names when determining association between old
+ * and new prototype.  if "allowmissingports" is nonzero, allow replacement
+ * to have missing ports and, therefore, delete the arcs that used to be there.
  */
-NODEINST *replacenodeinst(NODEINST *ni, NODEPROTO *np, INTSML ignoreportnames)
+NODEINST *replacenodeinst(NODEINST *ni, NODEPROTO *np, INTSML ignoreportnames,
+	INTSML allowmissingports)
 {
 	REGISTER PORTPROTO *pp, *opt;
 	INTBIG endx[2], endy[2], bx, by, cx, cy, lxo, hxo, lyo, hyo, lx, hx, ly, hy, psx, psy,
@@ -1775,15 +2064,10 @@ NODEINST *replacenodeinst(NODEINST *ni, NODEPROTO *np, INTSML ignoreportnames)
 	if (np->primindex == 0)
 	{
 		/* facet replacement: determine location of replacement */
-#if 0
-		lx = ni->lowx;   hx = lx + np->highx - np->lowx;
-		ly = ni->lowy;   hy = ly + np->highy - np->lowy;
-#else
 		lx = ni->lowx - ni->proto->lowx + np->lowx;
 		hx = ni->highx - ni->proto->highx + np->highx;
 		ly = ni->lowy - ni->proto->lowy + np->lowy;
 		hy = ni->highy - ni->proto->highy + np->highy;
-#endif
 
 		/* adjust position for facet center alignment */
 		varold = getvalkey((INTBIG)ni->proto, VNODEPROTO, VINTEGER|VISARRAY, el_prototype_center);
@@ -1833,6 +2117,7 @@ NODEINST *replacenodeinst(NODEINST *ni, NODEPROTO *np, INTSML ignoreportnames)
 		opt = (PORTPROTO *)pi->proto->temp1;
 		if (opt == NOPORTPROTO)
 		{
+			if (allowmissingports != 0) continue;
 			if (db_printerrors != 0)
 				ttyputmsg(_("No port on new node corresponds to old port: %s"), pi->proto->protoname);
 			db_killnodeinst(newno);
@@ -1845,6 +2130,7 @@ NODEINST *replacenodeinst(NODEINST *ni, NODEPROTO *np, INTSML ignoreportnames)
 			if (opt->connects[i] == ai->proto) break;
 		if (opt->connects[i] == NOARCPROTO)
 		{
+			if (allowmissingports != 0) continue;
 			if (db_printerrors != 0)
 				ttyputmsg(_("%s arc on old port %s cannot connect to new port %s"),
 					describearcinst(ai), pi->proto->protoname, opt->protoname);
@@ -1912,6 +2198,12 @@ NODEINST *replacenodeinst(NODEINST *ni, NODEPROTO *np, INTSML ignoreportnames)
 			{
 				endnodeinst[i] = newno;
 				endportproto[i] = (PORTPROTO *)pi->proto->temp1;
+				if (endportproto[i] == NOPORTPROTO)
+				{
+					if (allowmissingports == 0)
+						ttyputerr(_("Cannot re-connect %s arc"), describearcinst(ai));
+					break;
+				}
 				endx[i] = ai->end[i].xpos;   endy[i] = ai->end[i].ypos;
 				shapeportpoly(newno, endportproto[i], poly, 0);
 				if (isinside(endx[i], endy[i], poly) == 0) getcenter(poly, &endx[i], &endy[i]);
@@ -1924,7 +2216,13 @@ NODEINST *replacenodeinst(NODEINST *ni, NODEPROTO *np, INTSML ignoreportnames)
 		}
 		if (endportproto[0] == NOPORTPROTO || endportproto[1] == NOPORTPROTO)
 		{
-			ttyputerr(_("Cannot re-connect %s arc"), describearcinst(ai));
+			if (allowmissingports == 0)
+			{
+				ttyputerr(_("Cannot re-connect %s arc"), describearcinst(ai));
+			} else
+			{
+				db_killarcinst(ai);
+			}
 			continue;
 		}
 
@@ -2193,21 +2491,45 @@ INTSML moveportproto(NODEPROTO *facet, PORTPROTO *oldpt, NODEINST *newno, PORTPR
 
 	/* error checks */
 	if (oldpt == NOPORTPROTO)
+	{
+		db_donextchangequietly = 0;
 		return((INTSML)db_error(DBBADPROTO|DBMOVEPORTPROTO));
+	}
 	if (facet == NONODEPROTO)
+	{
+		db_donextchangequietly = 0;
 		return((INTSML)db_error(DBBADFACET|DBMOVEPORTPROTO));
+	}
 	if (oldpt->parent != facet)
+	{
+		db_donextchangequietly = 0;
 		return((INTSML)db_error(DBBADFACET|DBMOVEPORTPROTO));
+	}
 	if (newno == NONODEINST)
+	{
+		db_donextchangequietly = 0;
 		return((INTSML)db_error(DBBADINST|DBMOVEPORTPROTO));
+	}
 	if (newsubpt == NOPORTPROTO)
+	{
+		db_donextchangequietly = 0;
 		return((INTSML)db_error(DBBADSUBPORT|DBMOVEPORTPROTO));
+	}
 	if (newno->parent != oldpt->parent)
+	{
+		db_donextchangequietly = 0;
 		return((INTSML)db_error(DBBADPARENT|DBMOVEPORTPROTO));
+	}
 	if (newsubpt->parent != newno->proto)
+	{
+		db_donextchangequietly = 0;
 		return((INTSML)db_error(DBBADSUBPORT|DBMOVEPORTPROTO));
+	}
 	if (db_doesntconnect(newsubpt->connects, oldpt) != 0)
+	{
+		db_donextchangequietly = 0;
 		return((INTSML)db_error(DBPORTMM|DBMOVEPORTPROTO));
+	}
 
 	/* remember old state */
 	oldni = oldpt->subnodeinst;
@@ -2216,16 +2538,21 @@ INTSML moveportproto(NODEPROTO *facet, PORTPROTO *oldpt, NODEINST *newno, PORTPR
 	/* change the port origin */
 	db_changeport(oldpt, newno, newsubpt);
 
-	/* announce the change */
-	oldpt->changeaddr = (char *)db_change((INTBIG)oldpt, PORTPROTOMOD, (INTBIG)oldni,
-		(INTBIG)oldpp, 0, 0, 0, 0);
+	/* handle change control, constraint, and broadcast */
+	if (db_donextchangequietly == 0 && db_dochangesquietly == 0)
+	{
+		/* announce the change */
+		oldpt->changeaddr = (char *)db_change((INTBIG)oldpt, PORTPROTOMOD, (INTBIG)oldni,
+			(INTBIG)oldpp, 0, 0, 0, 0);
 
-	/* tell constraint system about modified port */
-	(*el_curconstraint->modifyportproto)(oldpt, oldni, oldpp);
+		/* tell constraint system about modified port */
+		(*el_curconstraint->modifyportproto)(oldpt, oldni, oldpp);
 
-	/* mark this as changed */
-	db_setchangefacet(oldpt->parent);
-	db_forcehierarchicalanalysis(oldpt->parent);
+		/* mark this as changed */
+		db_setchangefacet(oldpt->parent);
+		db_forcehierarchicalanalysis(oldpt->parent);
+	}
+	db_donextchangequietly = 0;
 
 	return(0);
 }
@@ -2259,9 +2586,53 @@ void db_changeport(PORTPROTO *pp, NODEINST *newsubno, PORTPROTO *newsubpt)
 
 /*
  * routine to recursively alter the "userbits" and "connects" fields of
- * port "pp", given that it changed or moved.
+ * export "pp", given that it changed or moved.  Also changes associated
+ * icon and contents facets.
  */
 void changeallports(PORTPROTO *pp)
+{
+	REGISTER NODEPROTO *np, *onp;
+	REGISTER PORTPROTO *opp;
+
+	/* look at all instances of the facet that had export motion */
+	db_changeallports(pp);
+
+	/* look at associated facets and change their ports */
+	np = pp->parent;
+	if (np->cellview == el_iconview)
+	{
+		/* changed an export on an icon: find contents and change it there */
+		onp = contentsview(np);
+		if (onp != NONODEPROTO)
+		{
+			opp = equivalentport(np, pp, onp);
+			if (opp != NOPORTPROTO)
+			{
+				opp->userbits = (opp->userbits & ~STATEBITS) | (pp->userbits & STATEBITS);
+				db_changeallports(opp);
+			}
+		}
+		return;
+	}
+
+	/* see if there is an icon to change */
+	onp = iconview(np);
+	if (onp != NONODEPROTO)
+	{
+		opp = equivalentport(np, pp, onp);
+		if (opp != NOPORTPROTO)
+		{
+			opp->userbits = (opp->userbits & ~STATEBITS) | (pp->userbits & STATEBITS);
+			db_changeallports(opp);
+		}
+	}
+}
+
+/*
+ * routine to recursively alter the "userbits" and "connects" fields of
+ * export "pp", given that it changed or moved.
+ */
+void db_changeallports(PORTPROTO *pp)
 {
 	REGISTER NODEINST *ni;
 	REGISTER PORTEXPINST *pe;
@@ -2281,7 +2652,7 @@ void changeallports(PORTPROTO *pp)
 					pp->userbits, VINTEGER);
 			}
 			pe->exportproto->connects = pp->connects;
-			changeallports(pe->exportproto);
+			db_changeallports(pe->exportproto);
 		}
 	}
 }
