@@ -32,13 +32,14 @@
 #include "database.h"
 #include "egraphics.h"
 #include "tech.h"
+#include "tecmocmos.h"
 #include "efunction.h"
 
 /*
  * when arcs are curved (with "arc curve" or "arc center") the number of
  * line segments will be between this value, and half of this value.
  */
-#define	MAXARCPIECES   16		/* maximum segments in curved arc */
+#define MAXARCPIECES   16		/* maximum segments in curved arc */
 
 /* for serpentine transistors */
 static VARIABLE *tech_serpentvar;
@@ -68,9 +69,10 @@ static VARIABLE *tech_firstvar;
 void db_adjusttree(NODEPROTO*);
 void tech_shortenmostrans(NODEINST*, POLYGON*, TECH_POLYGON*, INTBIG, INTBIG, TECH_PORTS*);
 INTSML db_stillinport(ARCINST*, INTSML, INTBIG, INTBIG);
-INTBIG tech_displayableanyvars(void);
+INTBIG tech_displayableanyvars(WINDOWPART *win, NODEPROTO *np);
 VARIABLE *tech_filldisplayableanyvar(POLYGON *poly, INTBIG lx, INTBIG hx,
-	INTBIG ly, INTBIG hy, INTBIG addr, INTBIG type, TECHNOLOGY *tech);
+	INTBIG ly, INTBIG hy, INTBIG addr, INTBIG type, TECHNOLOGY *tech,
+	WINDOWPART *win, NODEPROTO *facet, VARIABLE **varnoeval);
 
 /******************** GENERAL ********************/
 
@@ -207,7 +209,7 @@ INTSML tech_doaddportsandvars(TECHNOLOGY *tech)
 		/* floating point variables have data in a different place */
 		if ((tech->variables[i].type&VTYPE) == VFLOAT && (tech->variables[i].type&VISARRAY) == 0)
 		{
-			nextvarchangequiet();
+			nextchangequiet();
 			if (setvalkey((INTBIG)tech, VTECHNOLOGY, makekey(tech->variables[i].name),
 				castint(tech->variables[i].fvalue), tech->variables[i].type) == NOVARIABLE)
 					return(1);
@@ -215,7 +217,7 @@ INTSML tech_doaddportsandvars(TECHNOLOGY *tech)
 		}
 
 		/* normal variable setting */
-		nextvarchangequiet();
+		nextchangequiet();
 		if (setvalkey((INTBIG)tech, VTECHNOLOGY, makekey(tech->variables[i].name),
 			(INTBIG)tech->variables[i].value, tech->variables[i].type) == NOVARIABLE) return(1);
 	}
@@ -240,7 +242,7 @@ void tech_convertmocmoslib(LIBRARY *lib)
 		if (namesame(mocmostech->techname, "mocmos") == 0) break;
 	if (mocmostech == NOTECHNOLOGY)
 	{
-		ttyputmsg(_("Cannot find 'mocmos' technology"));
+		ttyputmsg(M_("Cannot find 'mocmos' technology"));
 		return;
 	}
 
@@ -274,7 +276,7 @@ void tech_convertmocmoslib(LIBRARY *lib)
 				prims++;
 				continue;
 			}
-			if (namesame(ni->proto->primname, "Metal-1-Substrate-Con") == 0)
+			if (ni->proto == mocmos_metal1pwellprim)
 			{
 				undogeom(ni->geom, np);
 				ni->lowx -= 4 * lambda;
@@ -291,7 +293,7 @@ void tech_convertmocmoslib(LIBRARY *lib)
 	/* if nothing was changed, quit now */
 	if (prims == 0)
 	{
-		ttyputmsg(_("No MOSIS CMOS objects found to alter"));
+		ttyputmsg(M_("No MOSIS CMOS objects found to alter"));
 		return;
 	}
 
@@ -301,11 +303,11 @@ void tech_convertmocmoslib(LIBRARY *lib)
 	for(np = lib->firstnodeproto; np != NONODEPROTO; np = np->nextnodeproto)
 		if (np->firstinst == NONODEINST) db_adjusttree(np);
 
-	ttyputmsg(_("%d S-active arcs, S-Active pins, Metal-1-S-Active"), prims);
-	ttyputmsg(_("  contacts, S-Transistors, and Metal-1-Substrate"));
-	ttyputmsg(_("  contacts from the MOSIS CMOS technology in this"));
-	ttyputmsg(_("  library have been adjusted."));
-	ttyputmsg(_("NOW DO A -debug check-database TO ADJUST FACET SIZES"));
+	ttyputmsg(M_("%d S-active arcs, S-Active pins, Metal-1-S-Active"), prims);
+	ttyputmsg(M_("  contacts, S-Transistors, and Metal-1-Substrate"));
+	ttyputmsg(M_("  contacts from the MOSIS CMOS technology in this"));
+	ttyputmsg(M_("  library have been adjusted."));
+	ttyputmsg(M_("NOW DO A -debug check-database TO ADJUST FACET SIZES"));
 }
 
 void db_adjusttree(NODEPROTO *np)
@@ -1194,45 +1196,53 @@ void tech_filltransport(NODEINST *ni, PORTPROTO *pp, POLYGON *poly, XARRAY trans
 /*
  * routine to compute the number of displayable polygons on nodeinst "ni"
  */
-INTBIG tech_displayablenvars(NODEINST *ni)
+INTBIG tech_displayablenvars(NODEINST *ni, WINDOWPART *win)
 {
 	tech_numvar = ni->numvar;
 	tech_firstvar = ni->firstvar;
-	return(tech_displayableanyvars());
+	return(tech_displayableanyvars(win, ni->parent));
 }
 
 /*
  * routine to fill polygon "poly" with the next displayable variable
  * on nodeinst "ni".  Returns the address of the variable just filled.
  */
-VARIABLE *tech_filldisplayablenvar(NODEINST *ni, POLYGON *poly)
+VARIABLE *tech_filldisplayablenvar(NODEINST *ni, POLYGON *poly, WINDOWPART *win,
+	VARIABLE **varnoeval)
 {
+	REGISTER NODEPROTO *facet;
+
+	facet = ni->parent;
 	tech_numvar = ni->numvar;
 	tech_firstvar = ni->firstvar;
+	if (facet->tech == NOTECHNOLOGY)
+		facet->tech = whattech(facet);
 	return(tech_filldisplayableanyvar(poly, ni->lowx, ni->highx,
-		ni->lowy, ni->highy, (INTBIG)ni, VNODEINST, ni->proto->tech));
+		ni->lowy, ni->highy, (INTBIG)ni, VNODEINST, facet->tech, win, facet, varnoeval));
 }
 
 /*
  * routine to compute the number of displayable polygons on nodeproto "np"
  */
-INTBIG tech_displayablefacetvars(NODEPROTO *np)
+INTBIG tech_displayablefacetvars(NODEPROTO *np, WINDOWPART *win)
 {
 	tech_numvar = np->numvar;
 	tech_firstvar = np->firstvar;
-	return(tech_displayableanyvars());
+	return(tech_displayableanyvars(win, np));
 }
 
 /*
  * routine to fill polygon "poly" with the next displayable variable
  * on nodeproto "np".  Returns the address of the variable just filled.
  */
-VARIABLE *tech_filldisplayablefacetvar(NODEPROTO *np, POLYGON *poly)
+VARIABLE *tech_filldisplayablefacetvar(NODEPROTO *np, POLYGON *poly, WINDOWPART *win,
+	VARIABLE **varnoeval)
 {
 	tech_numvar = np->numvar;
 	tech_firstvar = np->firstvar;
+	if (np->tech == NOTECHNOLOGY) np->tech = whattech(np);
 	return(tech_filldisplayableanyvar(poly, np->lowx, np->highx,
-		np->lowy, np->highy, (INTBIG)np, VNODEPROTO, np->tech));
+		np->lowy, np->highy, (INTBIG)np, VNODEPROTO, np->tech, win, np, varnoeval));
 }
 
 /******************** PORTINST DESCRIPTION ********************/
@@ -1321,23 +1331,26 @@ INTSML db_stillinport(ARCINST *ai, INTSML e, INTBIG x, INTBIG y)
 /*
  * routine to compute the number of displayable polygons on portproto "pp"
  */
-INTBIG tech_displayableportvars(PORTPROTO *pp)
+INTBIG tech_displayableportvars(PORTPROTO *pp, WINDOWPART *win)
 {
 	tech_numvar = pp->numvar;
 	tech_firstvar = pp->firstvar;
-	return(tech_displayableanyvars());
+	return(tech_displayableanyvars(win, pp->parent));
 }
 
 /*
  * routine to fill polygon "poly" with the next displayable variable
  * on nodeproto "np".  Returns the address of the variable just filled.
  */
-VARIABLE *tech_filldisplayableportvar(PORTPROTO *pp, POLYGON *poly)
+VARIABLE *tech_filldisplayableportvar(PORTPROTO *pp, POLYGON *poly, WINDOWPART *win,
+	VARIABLE **varnoeval)
 {
 	INTBIG x, y;
 	REGISTER INTSML saverot, savetrn;
 	REGISTER NODEINST *ni;
+	REGISTER NODEPROTO *facet;
 
+	facet = pp->parent;
 	ni = pp->subnodeinst;
 	saverot = ni->rotation;   savetrn = ni->transpose;
 	ni->rotation = ni->transpose = 0;
@@ -1345,8 +1358,10 @@ VARIABLE *tech_filldisplayableportvar(PORTPROTO *pp, POLYGON *poly)
 	ni->rotation = saverot;   ni->transpose = savetrn;
 	tech_numvar = pp->numvar;
 	tech_firstvar = pp->firstvar;
-	return(tech_filldisplayableanyvar(poly, x, x,
-		y, y, (INTBIG)pp, VPORTPROTO, pp->parent->tech));
+	if (facet->tech == NOTECHNOLOGY)
+		facet->tech = whattech(facet);
+	return(tech_filldisplayableanyvar(poly, x, x, y, y,
+		(INTBIG)pp, VPORTPROTO, facet->tech, win, facet, varnoeval));
 }
 
 /******************** ARCINST DESCRIPTION ********************/
@@ -1681,23 +1696,30 @@ void tech_resetnegated(ARCINST *ai)
 /*
  * routine to compute the number of displayable polygons on arcinst "ai"
  */
-INTBIG tech_displayableavars(ARCINST *ai)
+INTBIG tech_displayableavars(ARCINST *ai, WINDOWPART *win)
 {
 	tech_numvar = ai->numvar;
 	tech_firstvar = ai->firstvar;
-	return(tech_displayableanyvars());
+	return(tech_displayableanyvars(win, ai->parent));
 }
 
 /*
  * routine to fill polygon "poly" with the next displayable variable
  * on arcinst "ai".  Returns the address of the variable just filled.
  */
-VARIABLE *tech_filldisplayableavar(ARCINST *ai, POLYGON *poly)
+VARIABLE *tech_filldisplayableavar(ARCINST *ai, POLYGON *poly, WINDOWPART *win,
+	VARIABLE **varnoeval)
 {
+	REGISTER NODEPROTO *facet;
+
+	facet = ai->parent;
 	tech_numvar = ai->numvar;
 	tech_firstvar = ai->firstvar;
+	if (facet->tech == NOTECHNOLOGY)
+		facet->tech = whattech(facet);
 	return(tech_filldisplayableanyvar(poly, ai->geom->lowx, ai->geom->highx,
-		ai->geom->lowy, ai->geom->highy, (INTBIG)ai, VARCINST, ai->proto->tech));
+		ai->geom->lowy, ai->geom->highy, (INTBIG)ai, VARCINST, facet->tech,
+			win, facet, varnoeval));
 }
 
 /******************** SUPPORT ********************/
@@ -1706,16 +1728,21 @@ VARIABLE *tech_filldisplayableavar(ARCINST *ai, POLYGON *poly)
  * routine to compute the number of displayable polygons in the pair "tech_numvar/
  * tech_firstvar"
  */
-INTBIG tech_displayableanyvars(void)
+INTBIG tech_displayableanyvars(WINDOWPART *win, NODEPROTO *facet)
 {
 	REGISTER INTBIG i, total;
 	REGISTER VARIABLE *var;
 
 	tech_ndisplayindex = tech_ndisplaysubindex = total = 0;
+	if (win == NOWINDOWPART) win = el_curwindowpart;
 	for(i = 0; i < tech_numvar; i++)
 	{
 		var = &tech_firstvar[i];
 		if ((var->type&VDISPLAY) == 0) continue;
+		if ((var->textdescript&VTINTERIOR) != 0)
+		{
+			if (win != NOWINDOWPART && facet != win->curnodeproto) continue;
+		}
 		if ((var->type&VISARRAY) == 0) total++; else
 			total += getlength(var);
 	}
@@ -1727,15 +1754,20 @@ INTBIG tech_displayableanyvars(void)
  * on nodeinst "ni".  Returns the address of the variable just filled.
  */
 VARIABLE *tech_filldisplayableanyvar(POLYGON *poly, INTBIG lx, INTBIG hx,
-	INTBIG ly, INTBIG hy, INTBIG addr, INTBIG type, TECHNOLOGY *tech)
+	INTBIG ly, INTBIG hy, INTBIG addr, INTBIG type, TECHNOLOGY *tech,
+		WINDOWPART *win, NODEPROTO *facet, VARIABLE **varnoeval)
 {
 	REGISTER INTBIG descript, len, language;
 	REGISTER INTSML whichindex;
+	REGISTER char *query;
 	REGISTER VARIABLE *var;
 	REGISTER NODEINST *ni;
 	static VARIABLE retvar;
 
 	/* get next displayable variable */
+	if (varnoeval != 0)
+		(*varnoeval) = NOVARIABLE;
+	if (win == NOWINDOWPART) win = el_curwindowpart;
 	poly->count = 1;
 	poly->xv[0] = (lx+hx)/2;
 	poly->yv[0] = (ly+hy)/2;
@@ -1750,37 +1782,45 @@ VARIABLE *tech_filldisplayableanyvar(POLYGON *poly, INTBIG lx, INTBIG hx,
 			break;
 		}
 		var = &tech_firstvar[tech_ndisplayindex];
-		if ((var->type&VDISPLAY) != 0)
+		if (varnoeval != 0)
+			(*varnoeval) = var;
+		if ((var->type&VDISPLAY) != 0 &&
+			((var->textdescript&VTINTERIOR) == 0 || win == NOWINDOWPART ||
+				facet == win->curnodeproto))
 		{
-			language = var->type & (VCODE1|VCODE2);
-			if (language != 0)
+			descript = var->textdescript;
+			if ((descript&VTDISPLAYPART) != VTDISPLAYNAME)
 			{
-				retvar.key = var->key;
-				retvar.type = var->type & ~(VCODE1|VCODE2);
-				retvar.textdescript = var->textdescript;
-				if (doquerry((char *)var->addr, language, retvar.type, &retvar.addr) == 0)
-					var = &retvar;
+				language = var->type & (VCODE1|VCODE2);
+				if (language != 0)
+				{
+					if ((var->type&VISARRAY) == 0) query = (char *)var->addr; else
+						query = ((char **)var->addr)[0];
+					retvar.key = var->key;
+					retvar.type = var->type & ~(VCODE1|VCODE2|VISARRAY|VLENGTH);
+					retvar.textdescript = descript;
+					if (doquerry(query, language, retvar.type, &retvar.addr) == 0)
+						var = &retvar;
+				}
 			}
 			if ((var->type&VISARRAY) == 0)
 			{
 				tech_ndisplayindex++;
 				tech_ndisplaysubindex = 0;
-				if ((var->textdescript&VTPOSITION) == VTPOSBOXED)
+				if ((descript&VTPOSITION) == VTPOSBOXED)
 				{
 					makerectpoly(lx, hx, ly, hy, poly);
 					poly->style = CLOSED;
 				}
-				descript = var->textdescript;
-				poly->string = describevariable(var, -1, -1);
+				poly->string = describedisplayedvariable(var, -1, -1);
 				break;
 			}
 			len = getlength(var);
 			if (tech_ndisplaysubindex < len)
 			{
 				whichindex = (INTSML)tech_ndisplaysubindex++;
-				poly->string = describevariable(var, whichindex, -1);
-				descript = var->textdescript;
-				if ((var->textdescript&VTPOSITION) == VTPOSBOXED)
+				poly->string = describedisplayedvariable(var, whichindex, -1);
+				if ((descript&VTPOSITION) == VTPOSBOXED)
 				{
 					makerectpoly(lx, hx, ly, hy, poly);
 					poly->style = CLOSED;
@@ -1790,7 +1830,8 @@ VARIABLE *tech_filldisplayableanyvar(POLYGON *poly, INTBIG lx, INTBIG hx,
 					{
 						/* multiline variables only work on nodes! */
 						ni = (NODEINST *)addr;
-						getdisparrayvarlinepos((INTBIG)ni, VNODEINST, tech, el_curwindowpart, var,
+						if (win == NOWINDOWPART) win = el_curwindowpart;
+						getdisparrayvarlinepos((INTBIG)ni, VNODEINST, tech, win, var,
 							whichindex, &poly->xv[0], &poly->yv[0], 0);
 					}
 				}
