@@ -33,6 +33,7 @@
 #include "egraphics.h"
 #include "edialogs.h"
 #include "usr.h"
+#include "tecgen.h"
 #include <time.h>
 #ifdef TM_IN_SYS_TIME
 #  include <sys/time.h>
@@ -83,9 +84,9 @@ static INTBIG     db_filetypecount = 0;
 static FILETYPES *db_filetypeinfo;
 
 /* the infinite string package */
-#define	INFSTRCOUNT     8			/* number of infinite strings */
-#define	INFSTRDEFAULT 200			/* default infinite string length */
-#define	NOINFSTR ((INFSTR *)-1)
+#define INFSTRCOUNT     8			/* number of infinite strings */
+#define INFSTRDEFAULT 200			/* default infinite string length */
+#define NOINFSTR ((INFSTR *)-1)
 typedef struct
 {
 	char  *infstr;					/* the string address */
@@ -251,6 +252,74 @@ INTBIG myatoi(char *pp)
 	return(num * sign);
 }
 
+char *explainduration(float duration)
+{
+	static char elapsed[200];
+	char temp[50];
+	INTBIG hours, minutes;
+
+	elapsed[0] = 0;
+	if (duration >= 3600.0)
+	{
+		hours = (INTBIG)(duration / 3600.0f);
+		duration -= (float)(hours * 3600);
+		sprintf(temp, "%ld hours, ", hours);
+		strcat(elapsed, temp);
+	}
+	if (duration >= 60.0)
+	{
+		minutes = (INTBIG)(duration / 60.0f);
+		duration -= (float)(minutes * 60);
+		sprintf(temp, "%ld minutes, ", minutes);
+		strcat(elapsed, temp);
+	}
+	sprintf(temp, "%g seconds", duration);
+	strcat(elapsed, temp);
+	return(elapsed);
+}
+
+/*
+ * Routine to parse the version of Electric in "version" into three fields:
+ * the major version number, minor version, and a detail version number.
+ * The detail version number can be letters.  If it is omitted, it is
+ * assumed to be 999.  If it is a number, it is beyond 1000.  For example:
+ *    "6.02a"     major=6, minor=2, detail=1       (a Prerelease)
+ *    "6.02z"     major=6, minor=2, detail=26      (a Prerelease)
+ *    "6.02aa"    major=6, minor=2, detail=27      (a Prerelease)
+ *    "6.02az"    major=6, minor=2, detail=52      (a Prerelease)
+ *    "6.02ba"    major=6, minor=2, detail=53      (a Prerelease)
+ *    "6.02"      major=6, minor=2, detail=999     (a Release)
+ *    "6.02.1"    major=6, minor=2, detail=1001    (a PostRelease, update)
+ */
+void parseelectricversion(char *version, INTBIG *major, INTBIG *minor, INTBIG *detail)
+{
+	REGISTER char *pt;
+
+	/* parse the version fields */
+	pt = version;
+	*major = atoi(pt);
+	while (isdigit(*pt) != 0) pt++;
+	if (*pt++ != '.') { *minor = *detail = 0;   return; }
+	*minor = atoi(pt);
+	while (isdigit(*pt) != 0) pt++;
+	if (*pt == 0) { *detail = 999;   return; }
+	if (*pt == '.')
+	{
+		*detail = atoi(&pt[1]) + 1000;
+	} else
+	{
+		*detail = 0;
+		while (isalpha(*pt))
+		{
+			*detail = (*detail * 26) + tolower(*pt) - 'a' + 1;
+			pt++;
+		}
+	}
+#if 0		/* for debugging the version number */
+	ttyputmsg("Version '%s' is numbered %ld.%ld.%ld", version, *major, *minor, *detail);
+#endif
+}
+
 /*
  * routine to determine which node prototype is referred to by "line"
  * and return that nodeproto.  The routine returns NONODEPROTO if the
@@ -304,8 +373,12 @@ NODEPROTO *getnodeproto(char *initline)
 	for(pt = line; *pt != 0; pt++) if (*pt == ';' || *pt == '{') break;
 	save = *pt;
 	*pt = 0;
+#ifdef HASHCELLNAMES
+	c = db_findcellname(line, lib);
+#else
 	for(c = lib->firstcell; c != NOCELL; c = c->nextcell)
 		if (namesame(line, c->cellname) == 0) break;
+#endif
 	*pt = (char)save;
 	if (c == NOCELL) return(NONODEPROTO);
 
@@ -364,18 +437,23 @@ NODEPROTO *getnodeproto(char *initline)
 /*
  * routine to find cell "cellname".  Returns NOCELL if it cannot be found
  */
-static COMCOMP db_cellp = {NOKEYWORD, topofcells, nextcells, NOPARAMS,
-	NOBACKUP, 0, " \t", M_("cell"), 0};
 CELL *getcell(char *cellname)
 {
-	REGISTER INTSML i, j;
 	REGISTER CELL *cell;
+#ifdef HASHCELLNAMES
+	cell = db_findcellname(cellname, el_curlib);
+	return(cell);
+#else
+	static COMCOMP db_cellp = {NOKEYWORD, topofcells, nextcells, NOPARAMS,
+		NOBACKUP, 0, " \t", M_("cell"), 0};
+	REGISTER INTSML i, j;
 
 	i = parse(cellname, &db_cellp, 0);
 	if (i < 0) return(NOCELL);
 	for(j=0, cell = el_curlib->firstcell; cell != NOCELL; cell = cell->nextcell, j++)
 		if (j == i) return(cell);
 	return(NOCELL);
+#endif
 }
 
 /*
@@ -435,6 +513,75 @@ NETWORK *getnetwork(char *netname, NODEPROTO *facet)
 	return(NONETWORK);
 }
 
+/*
+ * routine to find network "netname" in facet "facet".  Returns NONETWORK
+ * if it cannot be found.  This routine allows for variations of the network
+ * name that may occur due to simulation, VHDL compilation, etc.
+ */
+NETWORK *getcomplexnetwork(char *name, NODEPROTO *np)
+{
+	REGISTER INTBIG len, i, l, k, c1, c2, addr;
+	REGISTER NETWORK *net;
+	REGISTER char *pt;
+
+	/* try the direct approach */
+	net = getnetwork(name, np);
+	if (net != NONETWORK) return(net);
+
+	if (name[0] == 'N' && name[1] == 'E' && name[2] == 'T')
+	{
+		addr = atoi(&name[3]);
+		for(net = np->firstnetwork; net != NONETWORK; net = net->nextnetwork)
+			if (net == (NETWORK *)addr) return(net);
+	}
+
+	/* if the name ends with "NV", try removing that */
+	len = strlen(name);
+	if (name[len-2] == 'N' && name[len-1] == 'V')
+	{
+		name[len-2] = 0;
+		net = getnetwork(name, np);
+		name[len-2] = 'N';
+		if (net != NONETWORK) return(net);
+	}
+
+	/* check for prefix "v(" and "l(" HSPICE prefixes (see also "simwindow.c:sim_window_findtrace()")*/
+	if ((name[0] == 'l' || name[0] == 'v') && name[1] == '(')
+	{
+		net = getnetwork(&name[2], np);
+		if (net != NONETWORK) return(net);
+	}
+
+	/* if there are underscores, see if they match original names */
+	for(pt = name; *pt != 0; pt++) if (*pt == '_') break;
+	if (*pt != 0)
+	{
+		len = strlen(name);
+		for(net = np->firstnetwork; net != NONETWORK; net = net->nextnetwork)
+		{
+			pt = net->netname;
+			for(k=0; k<net->namecount; k++)
+			{
+				l = strlen(pt);
+				if (l == len)
+				{
+					for(i=0; i<len; i++)
+					{
+						c1 = tolower(name[i]);
+						c2 = tolower(pt[i]);
+						if (c1 == c2) continue;
+						if (c1 == '_' && !isalnum(c2)) continue;
+						break;
+					}
+					if (i >= len) return(net);
+				}
+				pt += strlen(pt) + 1;
+			}
+		}
+	}
+	return(NONETWORK);
+}
+
 /*
  * routine to determine which arc prototype is referred to by "line"
  * and return that arcproto.  The routine returns NOARCPROTO if the prototype
@@ -1163,7 +1310,7 @@ char *geomname(GEOM *geom)
 	if (geom == NOGEOM) return("***NOGEOM***");
 	if (geom->entrytype == OBJNODEINST) return(describenodeinst(geom->entryaddr.ni));
 	if (geom->entrytype == OBJARCINST) return(describearcinst(geom->entryaddr.ai));
-	return(_("dummy"));
+	return(M_("dummy"));
 }
 
 /*
@@ -1195,10 +1342,32 @@ char *describenetwork(NETWORK *net)
 	return(returninfstr());
 }
 
+char *describeportbits(PORTPROTO *pp)
+{
+	switch (pp->userbits&STATEBITS)
+	{
+		case INPORT:     return(_("Input"));
+		case OUTPORT:    return(_("Output"));
+		case BIDIRPORT:  return(_("Bidirectional"));
+		case PWRPORT:    return(_("Power"));
+		case GNDPORT:    return(_("Ground"));
+		case CLKPORT:    return(_("Clock"));
+		case C1PORT:     return(_("Clock Phase 1"));
+		case C2PORT:     return(_("Clock Phase 2"));
+		case C3PORT:     return(_("Clock Phase 3"));
+		case C4PORT:     return(_("Clock Phase 4"));
+		case C5PORT:     return(_("Clock Phase 5"));
+		case C6PORT:     return(_("Clock Phase 6"));
+		case REFOUTPORT: return(_("Reference Output"));
+		case REFINPORT:  return(_("Reference Input"));
+	}
+	return("Unknown");
+}
+
 /*
  * routine to convert a lambda number to ascii
  */
-#define	OUTBUFS 8
+#define OUTBUFS 8
 char *latoa(INTBIG i)
 {
 	static INTSML latoaswitch = 0;
@@ -1231,17 +1400,19 @@ char *latoa(INTBIG i)
 /*
  * routine to convert a fractional number to ascii
  */
+#define FRTOANUMBUFS 10
+
 char *frtoa(INTBIG i)
 {
 	static INTSML latoaswitch = 0;
-	static char output[5][20];
+	static char output[FRTOANUMBUFS][30];
 	REGISTER INTBIG fra;
 	char temp[3];
 	REGISTER char *pp, *cur, *start;
 
 	/* get output buffer */
 	start = cur = &output[latoaswitch++][0];
-	if (latoaswitch >= 5) latoaswitch = 0;
+	if (latoaswitch >= FRTOANUMBUFS) latoaswitch = 0;
 
 	/* handle negative values */
 	if (i < 0)
@@ -1309,58 +1480,44 @@ INTSML isanumber(char *pp)
 	return(0);
 }
 
-#define	IDEALPIXPER10LAM   150
-#define	PIXINCREMENT        50
-
 /*
  * routine to convert relative or absolute font values to absolute, in the
  * window "w"
  */
-INTSML truefontsize(INTSML font, WINDOWPART *w, TECHNOLOGY *tech)
+INTBIG truefontsize(INTBIG font, WINDOWPART *w, TECHNOLOGY *tech)
 {
-	REGISTER INTSML inifont, minf, maxf;
-	REGISTER INTBIG pixper10lam, lambda;
+	REGISTER INTBIG pixperlam, lambda, height;
 	REGISTER LIBRARY *lib;
 
-	/* absolute fonts need no adjustment */
-	if ((font >= TXT4P && font <= TXT20P) || font >= TXTEDITOR) return(font);
+	/* keep special font codes */
+	if (font < 0) return(font);
 
 	/* detemine default, min, and max size of font */
-	switch (font)
-	{
-		case TXTSMALL:  inifont = TXT10P;  minf = TXT4P;  maxf = TXT16P;  break;
-		default:        ttyputmsg(_("Warning: font size is %d"), font);
-		case TXTMEDIUM: inifont = TXT14P;  minf = TXT6P;  maxf = TXT18P;  break;
-		case TXTLARGE:  inifont = TXT16P;  minf = TXT8P;  maxf = TXT20P;  break;
-	}
-	if (w == NOWINDOWPART) return(inifont);
-
-	/* determine scale of this window */
 	if (w->curnodeproto == NONODEPROTO) lib = el_curlib; else
 		lib = w->curnodeproto->cell->lib;
 	lambda = lib->lambda[tech->techindex];
-	pixper10lam = applyxscale(w, lambda*10);
-
-	/* shrink down as window gets tight */
-	if (pixper10lam < IDEALPIXPER10LAM)
+	switch (font)
 	{
-		while (pixper10lam < IDEALPIXPER10LAM)
-		{
-			if (inifont <= minf) break;
-			inifont--;
-			pixper10lam += PIXINCREMENT;
-		}
-	} else
-	{
-		pixper10lam -= PIXINCREMENT;
-		while (pixper10lam > IDEALPIXPER10LAM)
-		{
-			if (inifont >= maxf) break;
-			inifont++;
-			pixper10lam -= PIXINCREMENT;
-		}
+		case TXT4P:  return(4);
+		case TXT6P:  return(6);
+		case TXT8P:  return(8);
+		case TXT10P: return(10);
+		case TXT12P: return(12);
+		case TXT14P: return(14);
+		case TXT16P: return(16);
+		case TXT18P: return(18);
+		case TXT20P: return(20);
+		case TXTHL:  height = lambda/2;   break;
+		case TXT1L:  height = lambda;     break;
+		case TXT2L:  height = lambda*2;   break;
+		case TXT3L:  height = lambda*3;   break;
+		case TXT4L:  height = lambda*4;   break;
+		case TXT5L:  height = lambda*5;   break;
+		case TXT6L:  height = lambda*6;   break;
+		default:     return(12);
 	}
-	return(inifont);
+	pixperlam = applyyscale(w, height);
+	return(pixperlam);
 }
 
 /*
@@ -1373,24 +1530,34 @@ INTBIG defaulttextdescript(GEOM *geom)
 	REGISTER INTBIG descript, dx, dy, goleft, goright, goup, godown;
 	REGISTER NODEINST *ni;
 	REGISTER ARCINST *ai;
-	static INTBIG user_default_text_style = 0, user_default_text_size = 0,
-		user_default_text_smart_style = 0;
+	static INTBIG user_default_text_style = 0, user_default_text_smart_style = 0;
 
 	if (user_default_text_style == 0)
 		user_default_text_style = makekey("USER_default_text_style");
-	if (user_default_text_size == 0)
-		user_default_text_size = makekey("USER_default_text_size");
 	if (user_default_text_smart_style == 0)
 		user_default_text_smart_style = makekey("USER_default_text_smart_style");
 
 	txtvar = getvalkey((INTBIG)us_aid, VAID, VINTEGER, user_default_text_style);
 	if (txtvar == NOVARIABLE) descript = VTPOSCENT; else
 		descript = txtvar->addr;
-	txtvar = getvalkey((INTBIG)us_aid, VAID, VINTEGER, user_default_text_size);
-	if (txtvar == NOVARIABLE) descript |= TXTSMALL << VTSIZESH; else
-		descript |= txtvar->addr << VTSIZESH;
 	if (geom != NOGEOM)
 	{
+		/* set text size */
+		if (geom->entrytype == OBJARCINST)
+		{
+			descript |= defaulttextsize(4) << VTSIZESH;
+		} else
+		{
+			ni = geom->entryaddr.ni;
+			if (ni->proto == gen_invispinprim)
+			{
+				descript |= defaulttextsize(2) << VTSIZESH;
+			} else
+			{
+				descript |= defaulttextsize(3) << VTSIZESH;
+			}
+		}
+
 		/* handle smart text placement relative to attached object */
 		txtvar = getvalkey((INTBIG)us_aid, VAID, VINTEGER, user_default_text_smart_style);
 		if (txtvar != NOVARIABLE)
@@ -1525,6 +1692,130 @@ INTBIG defaulttextdescript(GEOM *geom)
 	return(descript);
 }
 
+#define DEFEXPORTSIZE      TXT2L
+#define DEFINVPINTEXTSIZE  TXT1L
+#define DEFNODETEXTSIZE    TXT1L
+#define DEFARCTEXTSIZE     TXT1L
+#define DEFFACETTEXTSIZE   TXT4L
+
+/*
+ * Routine to determine the size of text to use for a particular type of text:
+ * 1:  export label
+ * 2:  nonlayout text (text on an invisible pin)
+ * 3:  node name
+ * 4:  arc name
+ * 5:  facet name
+ */
+INTBIG defaulttextsize(INTBIG texttype)
+{
+	INTBIG size;
+	REGISTER VARIABLE *var;
+	static INTBIG export_size = 0, node_size = 0, arc_size = 0, facet_size = 0,
+		nonlayout_size = 0;
+
+	if (export_size == 0) export_size = makekey("USER_default_export_text_size");
+	if (node_size == 0) node_size = makekey("USER_default_node_text_size");
+	if (arc_size == 0) arc_size = makekey("USER_default_arc_text_size");
+	if (facet_size == 0) facet_size = makekey("USER_default_facet_text_size");
+	if (nonlayout_size == 0) nonlayout_size = makekey("USER_default_nonlayout_text_size");
+
+	size = 0;
+	switch (texttype)
+	{
+		case 1:
+			var = getvalkey((INTBIG)us_aid, VAID, VINTEGER, export_size);
+			if (var != NOVARIABLE) size = var->addr; else
+				size = DEFEXPORTSIZE;
+			break;
+		case 2:
+			var = getvalkey((INTBIG)us_aid, VAID, VINTEGER, nonlayout_size);
+			if (var != NOVARIABLE) size = var->addr; else
+				size = DEFINVPINTEXTSIZE;
+			break;
+		case 3:
+			var = getvalkey((INTBIG)us_aid, VAID, VINTEGER, node_size);
+			if (var != NOVARIABLE) size = var->addr; else
+				size = DEFNODETEXTSIZE;
+			break;
+		case 4:
+			var = getvalkey((INTBIG)us_aid, VAID, VINTEGER, arc_size);
+			if (var != NOVARIABLE) size = var->addr; else
+				size = DEFARCTEXTSIZE;
+			break;
+		case 5:
+			var = getvalkey((INTBIG)us_aid, VAID, VINTEGER, facet_size);
+			if (var != NOVARIABLE) size = var->addr; else
+				size = DEFFACETTEXTSIZE;
+			break;
+	}
+	return(size);
+}
+
+/*
+ * routine to make a printable string from variable "val", array index
+ * "aindex".  If "aindex" is negative, print the entire array.  If "purpose" is
+ * zero, the conversion is for human reading and should be easy to understand.
+ * If "purpose" is positive, the conversion is for machine reading and should
+ * be easy to parse.  If "purpose" is negative, the conversion is for
+ * parameter substitution and should be easy to understand but not hard to
+ * parse (a combination of the two).  If the variable is displayable and the name
+ * is to be displayed, that part is added.
+ */
+char *describedisplayedvariable(VARIABLE *var, INTSML aindex, INTSML purpose)
+{
+	REGISTER char *name;
+
+	if (var == NOVARIABLE)
+	{
+		if (purpose <= 0) return("***UNKNOWN***"); else return("");
+	}
+
+	(void)initinfstr();
+
+	/* add variable name if requested */
+	if ((var->type&VDISPLAY) != 0)
+	{
+		name = truevariablename(var);
+		switch (var->textdescript&VTDISPLAYPART)
+		{
+			case VTDISPLAYNAME:
+				(void)addstringtoinfstr(name);
+				break;
+			case VTDISPLAYVALUE:
+				(void)addstringtoinfstr(describevariable(var, aindex, purpose));
+				break;
+			case VTDISPLAYNAMEVALUE:
+				(void)formatinfstr("%s=%s", name, describevariable(var, aindex, purpose));
+				break;
+		}
+	} else
+	{
+		(void)addstringtoinfstr(describevariable(var, aindex, purpose));
+	}
+	return(returninfstr());
+}
+
+/*
+ * Routine to return the true name of variable "var".  Leading "ATTR_" or "ATTRP_"
+ * markers are removed.
+ */
+char *truevariablename(VARIABLE *var)
+{
+	REGISTER char *name;
+	REGISTER INTBIG i, len;
+
+	name = makename(var->key);
+	if (strncmp(name, "ATTR_", 5) == 0)
+		return(name + 5);
+	if (strncmp(name, "ATTRP_", 6) == 0)
+	{
+		len = strlen(name);
+		for(i=len-1; i>=0; i--) if (name[i] == '_') break;
+		return(name + i);
+	}
+	return(name);
+}
+
 /*
  * routine to make a printable string from variable "val", array index
  * "aindex".  If "aindex" is negative, print the entire array.  If "purpose" is
@@ -1546,27 +1837,6 @@ char *describevariable(VARIABLE *var, INTSML aindex, INTSML purpose)
 
 	err = initinfstr();
 
-	/* add variable name if requested */
-	if ((var->type&VDISPLAY) != 0 &&
-		(var->textdescript&VTDISPLAYPART) == VTDISPLAYNAMEVALUE &&
-		(var->type&VISARRAY) == 0)
-	{
-		char *name;
-
-		name = makename(var->key);
-		if (strncmp(name, "ATTR_", 5) == 0)
-		{
-			name += 5;
-		} else if (strncmp(name, "ATTRP_", 6) == 0)
-		{
-			len = strlen(name);
-			for(i=len-1; i>=0; i--) if (name[i] == '_') break;
-			name += i;
-		}
-		(void)addstringtoinfstr(name);
-		(void)addtoinfstr('=');
-	}
-
 	if ((var->type & (VCODE1|VCODE2)) != 0)
 	{
 		/* special case for code: it is a string, the type applies to the result */
@@ -1862,6 +2132,32 @@ INTSML db_addstring(char *str)
 	return(err);
 }
 
+/*
+ * Routine to describe a simple variable "var".  There can be no arrays, code,
+ * or pointers to Electric objects in this variable.
+ */
+char *describesimplevariable(VARIABLE *var)
+{
+	static char line[50];
+
+	switch (var->type&VTYPE)
+	{
+		case VSTRING:
+			return((char *)var->addr);
+		case VINTEGER:
+			sprintf(line, "%ld", var->addr);
+			return(line);
+		case VSHORT:
+			sprintf(line, "%ld", var->addr&0xFFFF);
+			return(line);
+		case VFLOAT:
+		case VDOUBLE:
+			sprintf(line, "%f", castfloat(var->addr));
+			return(line);
+	}
+	return("");
+}
+
 /*
  * routine to make an abbreviation for the string "pt" in upper case if
  * "upper" is nonzero
@@ -1942,7 +2238,8 @@ char *unitsname(INTBIG units)
 #endif
 #if SIMAID
 	extern DIALOG sim_optionsdialog, sim_alsclockdialog, sim_spiceoptdialog,
-		sim_verilogoptdialog, sim_fasthenrydialog, sim_fasthenryarcdialog;
+		sim_verilogoptdialog, sim_fasthenrydialog, sim_fasthenryarcdialog,
+		sim_widedialog;
 #endif
 #if VHDLAID
 	extern DIALOG vhdl_optionsdialog;
@@ -1964,7 +2261,7 @@ char *unitsname(INTBIG units)
 	io_gdsoptionsdialog,
 #endif
 #if IOSKILL
-	io_skilloptionsdialog,
+	io_skilloptionsdialog, io_skilllibdialog,
 #endif
 	net_optionsdialog, io_techadjlamdialog, io_liboptdialog,
 	us_movetodialog, us_eprogressdialog, us_progressdialog, us_3ddepthdialog,
@@ -1978,14 +2275,15 @@ char *unitsname(INTBIG units)
 	us_helpdialog, us_iconoptdialog, us_highlightlayerdialog, us_librarypathdialog,
 	us_chglibrarydialog, us_annringdialog, us_spelldialog, us_defnodedialog,
 	us_resistancedialog, us_capacitancedialog, us_inductancedialog, us_vccsdialog,
-	us_areadialog, us_widlendialog, us_gaindialog, us_sourcedialog,
+	us_areadialog, us_widlendialog, us_gaindialog, us_sourcedialog, us_txtsardialog,
 	us_nodesizedialog, us_optionsavingdialog, us_outlinedialog, us_portdialog,
 	us_portdisplaydialog, us_printingoptdialog, us_quickkeydialog, us_quitdialog,
 	us_rendialog, us_seloptdialog, us_selnamedialog, us_spreaddialog,
 	us_tecedlibtotechdialog, us_dependentlibdialog, us_techvarsdialog,
 	us_techselectdialog, us_techsetdialog, us_deftextdialog, us_unitsdialog,
 	us_variabledialog, us_newviewdialog, us_visiblelaydialog, us_windowviewdialog,
-	us_listdialog, us_ttyinputdialog, us_ttyfulldialog, us_purelayerdialog;
+	us_listdialog, us_ttyinputdialog, us_ttyfulldialog, us_purelayerdialog,
+	us_txtmodsizedialog, us_viewseldialog, us_rulesdialogdesc, us_techdifoptdialog;
 
 
 DIALOG *db_alldialogs[] =
@@ -2025,7 +2323,7 @@ DIALOG *db_alldialogs[] =
 #endif
 #if SIMAID
 	&sim_optionsdialog, &sim_alsclockdialog, &sim_spiceoptdialog, &sim_verilogoptdialog,
-	&sim_fasthenrydialog, &sim_fasthenryarcdialog,
+	&sim_fasthenrydialog, &sim_fasthenryarcdialog, &sim_widedialog,
 #endif
 #if VHDLAID
 	&vhdl_optionsdialog,
@@ -2047,7 +2345,7 @@ DIALOG *db_alldialogs[] =
 	&io_gdsoptionsdialog,
 #endif
 #if IOSKILL
-	&io_skilloptionsdialog,
+	&io_skilloptionsdialog, &io_skilllibdialog,
 #endif
 	&net_optionsdialog, &io_techadjlamdialog, &io_liboptdialog, &us_purelayerdialog,
 	&us_movetodialog, &us_eprogressdialog, &us_progressdialog, &us_3ddepthdialog, &us_aboutgnudialog,
@@ -2064,7 +2362,8 @@ DIALOG *db_alldialogs[] =
 	&us_rendialog, &us_seloptdialog, &us_selnamedialog, &us_spreaddialog, &us_tecedlibtotechdialog,
 	&us_dependentlibdialog, &us_techvarsdialog, &us_techselectdialog, &us_techsetdialog, &us_deftextdialog,
 	&us_unitsdialog, &us_variabledialog, &us_newviewdialog, &us_visiblelaydialog, &us_windowviewdialog,
-	&us_listdialog, &us_ttyinputdialog, &us_ttyfulldialog,
+	&us_listdialog, &us_ttyinputdialog, &us_ttyfulldialog, &us_txtsardialog, &us_txtmodsizedialog,
+	&us_viewseldialog, &us_rulesdialogdesc, &us_techdifoptdialog,
 0};
 
 char *db_stoppingreason[] =
@@ -2545,6 +2844,7 @@ void addtostringarray(void *vsa, char *string)
 	{
 		newlimit = sa->stringlimit * 2;
 		if (newlimit <= 0) newlimit = 10;
+		if (newlimit < sa->stringcount) newlimit = sa->stringcount;
 		newbuf = (char **)emalloc(newlimit * (sizeof (char *)), sa->cluster);
 		if (newbuf == 0) return;
 		for(i=0; i<sa->stringcount; i++) newbuf[i] = sa->strings[i];
@@ -2706,7 +3006,7 @@ INTBIG getfiletype(char *shortname)
 {
 	REGISTER INTBIG i;
 
-	for(i=0; db_filetypeinfo[i].extension != 0; i++)
+	for(i=0; i<db_filetypecount; i++)
 		if (namesame(shortname, db_filetypeinfo[i].shortname) == 0) return(i);
 	return(0);
 }
