@@ -37,9 +37,9 @@
 #include "egraphics.h"
 
 #define MAX_DRC_FLATIGNORE 100				/* max facets to ignore */
-#define	FLATDRCNAME        "ffindshort"		/* fast short program */
-#define	SFLATDRCNAME       "findshort"		/* slow short program */
-#define	SORTNAME           "sort"			/* sort program */
+#define FLATDRCNAME        "ffindshort"		/* fast short program */
+#define SFLATDRCNAME       "findshort"		/* slow short program */
+#define SORTNAME           "sort"			/* sort program */
 #define MAXSHORTLAYER	   9				/* Max. layer index  */
 
 static NODEPROTO  *dr_flatignored[MAX_DRC_FLATIGNORE];
@@ -162,9 +162,9 @@ INTSML dr_flatwrite(NODEPROTO *simnp)
 
 	/* re-assign net-list values, starting at each port of the top facet */
 	dr_index = 1;
-	for(pp=simnp->firstportproto; pp!=NOPORTPROTO; pp=pp->nextportproto)
+	for(pp = simnp->firstportproto; pp != NOPORTPROTO; pp = pp->nextportproto)
 	{
-		for(spp=simnp->firstportproto; spp!=pp; spp=spp->nextportproto)
+		for(spp = simnp->firstportproto; spp != pp; spp = spp->nextportproto)
 		{
 			if (spp->network != pp->network) continue;
 
@@ -197,8 +197,7 @@ INTSML dr_flatwrite(NODEPROTO *simnp)
 	/* no point continuing if nothing to do */
 	if (dr_flat_boxcount == 0)
 	{
-		ttyputmsg(_("Zero boxes written"));
-		ttyputmsg(_("DRC short detection complete"));
+		ttyputmsg(_("No boxes written, DRC short detection complete"));
 		return(0);
 	}
 
@@ -349,7 +348,7 @@ INTSML dr_readshorterrors(INTBIG where)
 				(void)sscanf(line, "L %s B %ld %ld %ld %ld", layer, &sx, &sy, &cx, &cy);
 				lx = cx-sx/2;   hx = cx+sx/2;
 				ly = cy-sy/2;   hy = cy+sy/2;
-				layname = layername(dr_maintech, (INTSML)atoi(layer));
+				layname = layername(dr_maintech, atoi(layer));
 				ttyputmsg(_("Error on %s layer from X(%s...%s) Y(%s...%s)"),
 					layname, latoa(lx), latoa(hx), latoa(ly), latoa(hy));
 				(void)askaid(us_aid, "show-area", lx, hx, ly, hy, el_curwindowpart->curnodeproto);
@@ -419,7 +418,7 @@ void dr_flatprint(NODEPROTO *facet, XARRAY mytrans)
 	for(ai = facet->firstarcinst; ai != NOARCINST; ai = ai->nextarcinst)
 	{
 		if (ai->proto->tech != dr_maintech) continue;
-		n = arcpolys(ai);
+		n = arcpolys(ai, NOWINDOWPART);
 		for(i = 0; i < n; i++)
 		{
 			shapearcpoly(ai, i, poly);
@@ -463,13 +462,12 @@ void dr_flatprint(NODEPROTO *facet, XARRAY mytrans)
 		}
 
 		/* if it is primitive, write the information */
-		downhierarchy(ni);
 		if (ni->proto->primindex != 0)
 		{
 			if (ni->proto->tech == dr_maintech)
 			{
 				makerot(ni, subtrans);
-				n = nodeEpolys(ni, 0);
+				n = nodeEpolys(ni, 0, NOWINDOWPART);
 				for(i = 0; i < n; i ++)
 				{
 					shapeEnodepoly(ni, i, poly);
@@ -486,9 +484,10 @@ void dr_flatprint(NODEPROTO *facet, XARRAY mytrans)
 			transmult(temptrans1, mytrans, temptrans2);
 			maketrans(ni, temptrans1);
 			transmult(temptrans1, temptrans2, subtrans);
+			downhierarchy(ni);
 			dr_flatprint(ni->proto, subtrans);
+			uphierarchy();
 		}
-		uphierarchy();
 	}
 }
 
@@ -566,7 +565,8 @@ void dr_transistor_hack(POLYGON *poly, NODEINST *ni)
 		st = &thistn->ele[i];
 		active_overlap = st->lwidth + st->rwidth;
 		i=0; do lay = &thistn->ele[i++].basics; while (lay->portnum != 0);
-		gate_min_distance = (INTSML)drcmindistance(ni->proto->tech, ni->parent->cell->lib, lay->layernum, lay->layernum, 0);
+		gate_min_distance = (INTSML)drcmindistance(ni->proto->tech, ni->parent->cell->lib,
+			lay->layernum, 0, lay->layernum, 0, 0, 0, 0);
 		if (active_overlap > gate_min_distance)
 		{
 			correction = ((active_overlap - gate_min_distance) / WHOLE) * lambda;
