@@ -65,6 +65,7 @@ BEGIN_MESSAGE_MAP(CChildFrame, CMDIChildWnd)
 	ON_WM_KEYDOWN()
 	ON_WM_MOVE()
 	ON_WM_SETCURSOR()
+	ON_WM_MOUSEWHEEL()
 	//}}AFX_MSG_MAP
 END_MESSAGE_MAP()
 
@@ -73,6 +74,7 @@ END_MESSAGE_MAP()
 
 CChildFrame::CChildFrame()
 {
+	m_wndFloating = 0;
 }
 
 CChildFrame::~CChildFrame()
@@ -81,6 +83,10 @@ CChildFrame::~CChildFrame()
 
 BOOL CChildFrame::PreCreateWindow(CREATESTRUCT& cs)
 {
+	if (m_wndFloating != 0)
+	{
+		cs.dwExStyle |= WS_EX_TOOLWINDOW|WS_EX_PALETTEWINDOW; //|WS_EX_DLGMODALFRAME;
+	}
 	return CMDIChildWnd::PreCreateWindow(cs);
 }
 
@@ -105,13 +111,15 @@ void CChildFrame::Dump(CDumpContext& dc) const
 
 void gra_buttonaction(int state, UINT nFlags, CPoint point, CWnd *frm);
 void gra_mouseaction(UINT nFlags, CPoint point, CWnd *frm);
+void gra_mousewheelaction(UINT nFlags, short zDelta, CPoint point, CWnd *frm);
 void gra_keyaction(UINT nChar, UINT nFlags, UINT nRepCnt);
 void gra_repaint(CChildFrame *frame, CPaintDC *dc);
 void gra_resize(CChildFrame *frame, int cx, int cy);
-int gra_closeframe(CChildFrame *frame);
+int  gra_closeframe(CChildFrame *frame);
 void gra_activateframe(CChildFrame *frame, BOOL bActivate);
 void gra_movedwindow(CChildFrame *frame, int x, int y);
 void gra_setdefaultcursor(void);
+int gra_setpropercursor(CWnd *wnd, int x, int y);
 
 void CChildFrame::OnLButtonDown(UINT nFlags, CPoint point)
 {
@@ -169,6 +177,12 @@ void CChildFrame::OnMouseMove(UINT nFlags, CPoint point)
 	gra_mouseaction(nFlags, point, this);
 }
 
+BOOL CChildFrame::OnMouseWheel(UINT nFlags, short zDelta, CPoint point) 
+{
+	gra_mousewheelaction(nFlags, zDelta, point, this);
+	return CMDIChildWnd::OnMouseWheel(nFlags, zDelta, point);
+}
+
 void CChildFrame::OnPaint()
 {
 	CPaintDC dc(this); // device context for painting
@@ -224,12 +238,13 @@ void CChildFrame::OnMDIActivate(BOOL bActivate, CWnd* pActivateWnd, CWnd* pDeact
 
 void CChildFrame::OnKeyDown(UINT nChar, UINT nRepCnt, UINT nFlags)
 {
-	int controlkeydown;
+	int controlkeydown, shiftkeydown;
 
 	/* handle numeric keypad with control key pressed */
-	if ((GetKeyState(VK_CONTROL) & 0x8000) != 0)
-		controlkeydown = 0200; else
-			controlkeydown = 0;
+	if ((GetKeyState(VK_CONTROL) & 0x8000) != 0) controlkeydown = 1; else
+		controlkeydown = 0;
+	if ((GetKeyState(VK_SHIFT) & 0x8000) != 0) shiftkeydown = 2; else
+		shiftkeydown = 0;
 	if (nChar == VK_NUMPAD0 && controlkeydown != 0)
 		{ gra_keyaction('0'+0200, nFlags, nRepCnt); return; }
 	if (nChar == VK_NUMPAD1 && controlkeydown != 0)
@@ -268,22 +283,60 @@ void CChildFrame::OnKeyDown(UINT nChar, UINT nRepCnt, UINT nFlags)
 	if ((nFlags&0x100) != 0)
 	{
 		if (nChar == VK_DELETE)
-			gra_keyaction(010,                 nFlags, nRepCnt);
+			gra_keyaction(DELETEKEY, nFlags, nRepCnt);
 		if (nChar == VK_UP)
-			gra_keyaction(0x1E+controlkeydown, nFlags, nRepCnt);
+		{
+			switch (controlkeydown+shiftkeydown)
+			{
+				case 0: gra_keyaction( 036, nFlags, nRepCnt);   break;
+				case 1: gra_keyaction(0236, nFlags, nRepCnt);   break;
+				case 2: gra_keyaction(  06, nFlags, nRepCnt);   break;
+				case 3: gra_keyaction(0206, nFlags, nRepCnt);   break;
+			}
+		}
 		if (nChar == VK_DOWN)
-			gra_keyaction(0x1F+controlkeydown, nFlags, nRepCnt);
+		{
+			switch (controlkeydown+shiftkeydown)
+			{
+				case 0: gra_keyaction( 037, nFlags, nRepCnt);   break;
+				case 1: gra_keyaction(0237, nFlags, nRepCnt);   break;
+				case 2: gra_keyaction(  07, nFlags, nRepCnt);   break;
+				case 3: gra_keyaction(0207, nFlags, nRepCnt);   break;
+			}
+		}
 		if (nChar == VK_RIGHT)
-			gra_keyaction(0x1D+controlkeydown, nFlags, nRepCnt);
+		{
+			switch (controlkeydown+shiftkeydown)
+			{
+				case 0: gra_keyaction( 035, nFlags, nRepCnt);   break;
+				case 1: gra_keyaction(0235, nFlags, nRepCnt);   break;
+				case 2: gra_keyaction(  05, nFlags, nRepCnt);   break;
+				case 3: gra_keyaction(0205, nFlags, nRepCnt);   break;
+			}
+		}
 		if (nChar == VK_LEFT)
-			gra_keyaction(0x1C+controlkeydown, nFlags, nRepCnt);
+		{
+			switch (controlkeydown+shiftkeydown)
+			{
+				case 0: gra_keyaction( 034, nFlags, nRepCnt);   break;
+				case 1: gra_keyaction(0234, nFlags, nRepCnt);   break;
+				case 2: gra_keyaction(  02, nFlags, nRepCnt);   break;
+				case 3: gra_keyaction(0202, nFlags, nRepCnt);   break;
+			}
+		}
 	}
 	CMDIChildWnd::OnKeyDown(nChar, nRepCnt, nFlags);
 }
 
 BOOL CChildFrame::OnSetCursor(CWnd* pWnd, UINT nHitTest, UINT message)
 {
+	POINT p, p2;
+
+	GetCursorPos(&p);
+	p2.x = p2.y = 0;
+	MapWindowPoints(0, &p2, 1);
+	if (gra_setpropercursor(pWnd, p.x-p2.x, p.y-p2.y) != 0) return(1);
+
 	if (CMDIChildWnd::OnSetCursor(pWnd, nHitTest, message)) return(1);
-	gra_setdefaultcursor();
 	return(1);
 }
