@@ -102,6 +102,27 @@ BEGIN_MESSAGE_MAP(CElectricDialog, CDialog)
 	ON_LBN_DBLCLK(ID_DIALOGITEM_37, OnDialogDoubleClick37)
 	ON_LBN_DBLCLK(ID_DIALOGITEM_38, OnDialogDoubleClick38)
 	ON_LBN_DBLCLK(ID_DIALOGITEM_39, OnDialogDoubleClick39)
+	ON_LBN_DBLCLK(ID_DIALOGITEM_40, OnDialogDoubleClick40)
+	ON_LBN_DBLCLK(ID_DIALOGITEM_41, OnDialogDoubleClick41)
+	ON_LBN_DBLCLK(ID_DIALOGITEM_42, OnDialogDoubleClick42)
+	ON_LBN_DBLCLK(ID_DIALOGITEM_43, OnDialogDoubleClick43)
+	ON_LBN_DBLCLK(ID_DIALOGITEM_44, OnDialogDoubleClick44)
+	ON_LBN_DBLCLK(ID_DIALOGITEM_45, OnDialogDoubleClick45)
+	ON_LBN_DBLCLK(ID_DIALOGITEM_46, OnDialogDoubleClick46)
+	ON_LBN_DBLCLK(ID_DIALOGITEM_47, OnDialogDoubleClick47)
+	ON_LBN_DBLCLK(ID_DIALOGITEM_48, OnDialogDoubleClick48)
+	ON_LBN_DBLCLK(ID_DIALOGITEM_49, OnDialogDoubleClick49)
+	ON_LBN_DBLCLK(ID_DIALOGITEM_50, OnDialogDoubleClick50)
+	ON_LBN_DBLCLK(ID_DIALOGITEM_51, OnDialogDoubleClick51)
+	ON_LBN_DBLCLK(ID_DIALOGITEM_52, OnDialogDoubleClick52)
+	ON_LBN_DBLCLK(ID_DIALOGITEM_53, OnDialogDoubleClick53)
+	ON_LBN_DBLCLK(ID_DIALOGITEM_54, OnDialogDoubleClick54)
+	ON_LBN_DBLCLK(ID_DIALOGITEM_55, OnDialogDoubleClick55)
+	ON_LBN_DBLCLK(ID_DIALOGITEM_56, OnDialogDoubleClick56)
+	ON_LBN_DBLCLK(ID_DIALOGITEM_57, OnDialogDoubleClick57)
+	ON_LBN_DBLCLK(ID_DIALOGITEM_58, OnDialogDoubleClick58)
+	ON_LBN_DBLCLK(ID_DIALOGITEM_59, OnDialogDoubleClick59)
+	ON_LBN_DBLCLK(ID_DIALOGITEM_60, OnDialogDoubleClick60)
 	ON_EN_CHANGE(ID_DIALOGITEM_0, OnDialogTextChanged0)
 	ON_EN_CHANGE(ID_DIALOGITEM_1, OnDialogTextChanged1)
 	ON_EN_CHANGE(ID_DIALOGITEM_2, OnDialogTextChanged2)
@@ -142,6 +163,27 @@ BEGIN_MESSAGE_MAP(CElectricDialog, CDialog)
 	ON_EN_CHANGE(ID_DIALOGITEM_37, OnDialogTextChanged37)
 	ON_EN_CHANGE(ID_DIALOGITEM_38, OnDialogTextChanged38)
 	ON_EN_CHANGE(ID_DIALOGITEM_39, OnDialogTextChanged39)
+	ON_EN_CHANGE(ID_DIALOGITEM_40, OnDialogTextChanged40)
+	ON_EN_CHANGE(ID_DIALOGITEM_41, OnDialogTextChanged41)
+	ON_EN_CHANGE(ID_DIALOGITEM_42, OnDialogTextChanged42)
+	ON_EN_CHANGE(ID_DIALOGITEM_43, OnDialogTextChanged43)
+	ON_EN_CHANGE(ID_DIALOGITEM_44, OnDialogTextChanged44)
+	ON_EN_CHANGE(ID_DIALOGITEM_45, OnDialogTextChanged45)
+	ON_EN_CHANGE(ID_DIALOGITEM_46, OnDialogTextChanged46)
+	ON_EN_CHANGE(ID_DIALOGITEM_47, OnDialogTextChanged47)
+	ON_EN_CHANGE(ID_DIALOGITEM_48, OnDialogTextChanged48)
+	ON_EN_CHANGE(ID_DIALOGITEM_49, OnDialogTextChanged49)
+	ON_EN_CHANGE(ID_DIALOGITEM_50, OnDialogTextChanged50)
+	ON_EN_CHANGE(ID_DIALOGITEM_51, OnDialogTextChanged51)
+	ON_EN_CHANGE(ID_DIALOGITEM_52, OnDialogTextChanged52)
+	ON_EN_CHANGE(ID_DIALOGITEM_53, OnDialogTextChanged53)
+	ON_EN_CHANGE(ID_DIALOGITEM_54, OnDialogTextChanged54)
+	ON_EN_CHANGE(ID_DIALOGITEM_55, OnDialogTextChanged55)
+	ON_EN_CHANGE(ID_DIALOGITEM_56, OnDialogTextChanged56)
+	ON_EN_CHANGE(ID_DIALOGITEM_57, OnDialogTextChanged57)
+	ON_EN_CHANGE(ID_DIALOGITEM_58, OnDialogTextChanged58)
+	ON_EN_CHANGE(ID_DIALOGITEM_59, OnDialogTextChanged59)
+	ON_EN_CHANGE(ID_DIALOGITEM_60, OnDialogTextChanged60)
 	ON_WM_PAINT()
 	ON_WM_MOUSEMOVE()
 	ON_WM_LBUTTONDOWN()
@@ -162,6 +204,14 @@ int gra_dodialoglistkey(UINT nKey, CListBox* pListBox, UINT nIndex);
 
 BOOL CElectricDialog::OnCmdMsg(UINT nID, int nCode, void* pExtra, AFX_CMDHANDLERINFO* pHandlerInfo)
 {
+	void gra_onint(void);
+
+	if (((GetKeyState(VK_LWIN) | GetKeyState(VK_RWIN) & GetKeyState('c')) & 0x8000) != 0)
+	{
+		gra_onint();
+		return(1);
+	}
+
 	switch (nCode)
 	{
 		case 1:
@@ -235,6 +285,27 @@ void CElectricDialog::OnDialogDoubleClick36() { gra_itemdoubleclicked(36); }
 void CElectricDialog::OnDialogDoubleClick37() { gra_itemdoubleclicked(37); }
 void CElectricDialog::OnDialogDoubleClick38() { gra_itemdoubleclicked(38); }
 void CElectricDialog::OnDialogDoubleClick39() { gra_itemdoubleclicked(39); }
+void CElectricDialog::OnDialogDoubleClick40() { gra_itemdoubleclicked(40); }
+void CElectricDialog::OnDialogDoubleClick41() { gra_itemdoubleclicked(41); }
+void CElectricDialog::OnDialogDoubleClick42() { gra_itemdoubleclicked(42); }
+void CElectricDialog::OnDialogDoubleClick43() { gra_itemdoubleclicked(43); }
+void CElectricDialog::OnDialogDoubleClick44() { gra_itemdoubleclicked(44); }
+void CElectricDialog::OnDialogDoubleClick45() { gra_itemdoubleclicked(45); }
+void CElectricDialog::OnDialogDoubleClick46() { gra_itemdoubleclicked(46); }
+void CElectricDialog::OnDialogDoubleClick47() { gra_itemdoubleclicked(47); }
+void CElectricDialog::OnDialogDoubleClick48() { gra_itemdoubleclicked(48); }
+void CElectricDialog::OnDialogDoubleClick49() { gra_itemdoubleclicked(49); }
+void CElectricDialog::OnDialogDoubleClick50() { gra_itemdoubleclicked(50); }
+void CElectricDialog::OnDialogDoubleClick51() { gra_itemdoubleclicked(51); }
+void CElectricDialog::OnDialogDoubleClick52() { gra_itemdoubleclicked(52); }
+void CElectricDialog::OnDialogDoubleClick53() { gra_itemdoubleclicked(53); }
+void CElectricDialog::OnDialogDoubleClick54() { gra_itemdoubleclicked(54); }
+void CElectricDialog::OnDialogDoubleClick55() { gra_itemdoubleclicked(55); }
+void CElectricDialog::OnDialogDoubleClick56() { gra_itemdoubleclicked(56); }
+void CElectricDialog::OnDialogDoubleClick57() { gra_itemdoubleclicked(57); }
+void CElectricDialog::OnDialogDoubleClick58() { gra_itemdoubleclicked(58); }
+void CElectricDialog::OnDialogDoubleClick59() { gra_itemdoubleclicked(59); }
+void CElectricDialog::OnDialogDoubleClick60() { gra_itemdoubleclicked(60); }
 
 void CElectricDialog::OnDialogTextChanged0() { gra_dodialogtextchange(0); }
 void CElectricDialog::OnDialogTextChanged1() { gra_dodialogtextchange(1); }
@@ -276,6 +347,27 @@ void CElectricDialog::OnDialogTextChanged36() { gra_dodialogtextchange(36); }
 void CElectricDialog::OnDialogTextChanged37() { gra_dodialogtextchange(37); }
 void CElectricDialog::OnDialogTextChanged38() { gra_dodialogtextchange(38); }
 void CElectricDialog::OnDialogTextChanged39() { gra_dodialogtextchange(39); }
+void CElectricDialog::OnDialogTextChanged40() { gra_dodialogtextchange(40); }
+void CElectricDialog::OnDialogTextChanged41() { gra_dodialogtextchange(41); }
+void CElectricDialog::OnDialogTextChanged42() { gra_dodialogtextchange(42); }
+void CElectricDialog::OnDialogTextChanged43() { gra_dodialogtextchange(43); }
+void CElectricDialog::OnDialogTextChanged44() { gra_dodialogtextchange(44); }
+void CElectricDialog::OnDialogTextChanged45() { gra_dodialogtextchange(45); }
+void CElectricDialog::OnDialogTextChanged46() { gra_dodialogtextchange(46); }
+void CElectricDialog::OnDialogTextChanged47() { gra_dodialogtextchange(47); }
+void CElectricDialog::OnDialogTextChanged48() { gra_dodialogtextchange(48); }
+void CElectricDialog::OnDialogTextChanged49() { gra_dodialogtextchange(49); }
+void CElectricDialog::OnDialogTextChanged50() { gra_dodialogtextchange(50); }
+void CElectricDialog::OnDialogTextChanged51() { gra_dodialogtextchange(51); }
+void CElectricDialog::OnDialogTextChanged52() { gra_dodialogtextchange(52); }
+void CElectricDialog::OnDialogTextChanged53() { gra_dodialogtextchange(53); }
+void CElectricDialog::OnDialogTextChanged54() { gra_dodialogtextchange(54); }
+void CElectricDialog::OnDialogTextChanged55() { gra_dodialogtextchange(55); }
+void CElectricDialog::OnDialogTextChanged56() { gra_dodialogtextchange(56); }
+void CElectricDialog::OnDialogTextChanged57() { gra_dodialogtextchange(57); }
+void CElectricDialog::OnDialogTextChanged58() { gra_dodialogtextchange(58); }
+void CElectricDialog::OnDialogTextChanged59() { gra_dodialogtextchange(59); }
+void CElectricDialog::OnDialogTextChanged60() { gra_dodialogtextchange(60); }
 
 void CElectricDialog::OnPaint()
 {
