@@ -147,6 +147,16 @@ int CMainFrame::OnCreate(LPCREATESTRUCT lpCreateStruct)
 		return -1;      // fail to create
 	}
 
+#if 0	/* make am floating window */
+	RECT r;
+	r.left = 1150;   r.right = 1250;
+	r.top = 20;    r.bottom = 600;
+	m_wndComponentMenu = new CFrameWnd();
+	m_wndComponentMenu->Create(NULL, "Components",
+		WS_OVERLAPPED|WS_CAPTION, r, this, NULL, WS_EX_TOOLWINDOW, NULL);
+	m_wndComponentMenu->ShowWindow(SW_SHOW);
+#endif
+
 	/* count off tenths of a second */
 	SetTimer(1, 100, 0);
 
