@@ -57,7 +57,7 @@ void cla_layconmodifynodeinst(NODEINST*, INTBIG, INTBIG, INTBIG, INTBIG, INTSML,
 void cla_layconmodifyarcinst(ARCINST*, INTBIG, INTBIG, INTBIG, INTBIG, INTBIG, INTBIG);
 void cla_layconmodifyportproto(PORTPROTO*, NODEINST*, PORTPROTO*);
 void cla_layconmodifynodeproto(NODEPROTO*);
-void cla_layconmodifydescript(INTBIG, INTBIG, VARIABLE*, INTBIG);
+void cla_layconmodifydescript(INTBIG, INTBIG, INTBIG, INTBIG);
 void cla_layconnewlib(LIBRARY*);
 void cla_layconkilllib(LIBRARY*);
 void cla_layconnewvariable(INTBIG, INTBIG, INTBIG, INTBIG);
