@@ -33,50 +33,52 @@ extern "C"
 {
 #endif
 
+#define HASHCELLNAMES 1					/* define to use hash table for cell names */
+
 /*************************** FLAG BIT SETTINGS ****************************/
 
-#define	MARKN             040				/* NODEINST->userbits: nodeinst is marked */
-#define	TOUCHN           0100				/* NODEINST->userbits: nodeinst is touched */
+#define MARKN             040				/* NODEINST->userbits: nodeinst is marked */
+#define TOUCHN           0100				/* NODEINST->userbits: nodeinst is touched */
 
-#define	FIXEDMOD   0100000000				/* ARCINST->userbits: fixed arc was changed */
-#define	FACETMOD      0400000				/* NODEPROTO->userbits: facet is changed */
-#define	FACETNOMOD   01000000				/* NODEPROTO->userbits: facet is not changed */
+#define FIXEDMOD   0100000000				/* ARCINST->userbits: fixed arc was changed */
+#define FACETMOD      0400000				/* NODEPROTO->userbits: facet is changed */
+#define FACETNOMOD   01000000				/* NODEPROTO->userbits: facet is not changed */
 
 /*************************** CHANGES ****************************/
 
-#define	NOCHANGE	((CHANGE *)-1)
+#define NOCHANGE	((CHANGE *)-1)
 
 /* meaning of "changetype" */
-#define	NODEINSTNEW      0					/* new nodeinst */
-#define	NODEINSTKILL     1					/* killed nodeinst */
-#define	NODEINSTMOD      2					/* modified nodeinst */
+#define NODEINSTNEW      0					/* new nodeinst */
+#define NODEINSTKILL     1					/* killed nodeinst */
+#define NODEINSTMOD      2					/* modified nodeinst */
 
-#define	ARCINSTCHANGE    3					/* base of ARCINST changes */
-#define	ARCINSTNEW       3					/* new arcinst */
-#define	ARCINSTKILL      4					/* deleted arcinst */
-#define	ARCINSTMOD       5					/* modified arcinst */
+#define ARCINSTCHANGE    3					/* base of ARCINST changes */
+#define ARCINSTNEW       3					/* new arcinst */
+#define ARCINSTKILL      4					/* deleted arcinst */
+#define ARCINSTMOD       5					/* modified arcinst */
 
-#define	PORTPROTOCHANGE  6					/* base of PORTPROTO changes */
-#define	PORTPROTONEW     6					/* new portinst on a nodeproto */
-#define	PORTPROTOKILL    7					/* deleted portinst on a nodeproto */
-#define	PORTPROTOMOD     8					/* motified portinst on a nodeproto */
+#define PORTPROTOCHANGE  6					/* base of PORTPROTO changes */
+#define PORTPROTONEW     6					/* new portinst on a nodeproto */
+#define PORTPROTOKILL    7					/* deleted portinst on a nodeproto */
+#define PORTPROTOMOD     8					/* motified portinst on a nodeproto */
 
-#define	NODEPROTOCHANGE  9					/* base of NODEPROTO changes */
-#define	NODEPROTONEW     9					/* new nodeproto */
-#define	NODEPROTOKILL   10					/* killed nodeproto */
-#define	NODEPROTOMOD    11					/* modified nodeproto */
+#define NODEPROTOCHANGE  9					/* base of NODEPROTO changes */
+#define NODEPROTONEW     9					/* new nodeproto */
+#define NODEPROTOKILL   10					/* killed nodeproto */
+#define NODEPROTOMOD    11					/* modified nodeproto */
 
-#define	VARIABLECHANGE  12					/* base of VARIABLE changes */
-#define	OBJECTSTART     12					/* start of object change */
-#define	OBJECTEND       13					/* end of object change */
-#define	OBJECTNEW       14					/* new object */
-#define	OBJECTKILL      15					/* deleted object */
-#define	VARIABLENEW     16					/* new variable */
-#define	VARIABLEKILL    17					/* deleted variable */
-#define	VARIABLEMOD     18					/* changed single entry in array variable */
-#define	VARIABLEINS     19					/* inserted single entry in array variable */
-#define	VARIABLEDEL     20					/* deleted single entry in array variable */
-#define	DESCRIPTMOD     21					/* changed text descriptor */
+#define VARIABLECHANGE  12					/* base of VARIABLE changes */
+#define OBJECTSTART     12					/* start of object change */
+#define OBJECTEND       13					/* end of object change */
+#define OBJECTNEW       14					/* new object */
+#define OBJECTKILL      15					/* deleted object */
+#define VARIABLENEW     16					/* new variable */
+#define VARIABLEKILL    17					/* deleted variable */
+#define VARIABLEMOD     18					/* changed single entry in array variable */
+#define VARIABLEINS     19					/* inserted single entry in array variable */
+#define VARIABLEDEL     20					/* deleted single entry in array variable */
+#define DESCRIPTMOD     21					/* changed text descriptor */
 
 typedef struct Ichange
 {
@@ -89,7 +91,7 @@ typedef struct Ichange
 
 /*************************** CHANGE BATCHES ****************************/
 
-#define	NOCHANGEFACET	((CHANGEFACET *)-1)
+#define NOCHANGEFACET	((CHANGEFACET *)-1)
 
 typedef struct Ichangefacet
 {
@@ -99,7 +101,7 @@ typedef struct Ichangefacet
 } CHANGEFACET;
 
 
-#define	NOCHANGEBATCH	((CHANGEBATCH *)-1)
+#define NOCHANGEBATCH	((CHANGEBATCH *)-1)
 
 typedef struct Ichangebatch
 {
@@ -191,98 +193,110 @@ extern POLYGON     *db_staticpolygons;
 /*************************** ERROR CODES ****************************/
 
 /* error messages */
-#define	DBNOERROR       0					/* no error */
-#define	DBNOMEM	        1					/* no memory */
-#define	DBBADTRANS      2					/* bad transposition */
-#define	DBBADROT        3					/* bad rotation */
-#define	DBBADPROTO      4					/* bad prototype */
-#define	DBBADPARENT     5					/* bad parent */
-#define	DBBADINST       6					/* invalid instance */
-#define	DBBADNAME       7					/* invalid name */
-#define	DBBADWIDTH      8					/* bad width */
-#define	DBBADENDAN      9					/* bad end A node/port */
-#define	DBBADENDBN     10					/* bad end B node/port */
-#define	DBBADENDAC     11					/* bad end A connection */
-#define	DBBADENDBC     12					/* bad end B connection */
-#define	DBBADENDAP     13					/* bad end A position */
-#define	DBBADENDBP     14					/* bad end B position */
-#define	DBBADNEWWID    15					/* bad new width */
-#define	DBBADFACET     16					/* bad facet */
-#define	DBBADLIB       17					/* bad library */
-#define	DBBADSIZE      18					/* bad size */
-#define	DBBADOBJECT    19					/* bad object type */
-#define	DBBADSUBPORT   20					/* bad sub port */
-#define	DBHASARCS      21					/* still has arcs */
-#define	DBHASPORTS     22					/* still has ports */
-#define	DBHASINSTANCES 23					/* facet has instances */
-#define	DBRECURSIVE    24					/* recursive call */
-#define	DBNOTINPORT    25					/* arc not in port */
-#define	DBCONFLICT     26					/* conflicts with primitive */
-#define	DBPORTMM       27					/* port mismatch */
-#define	DBDUPLICATE    28					/* duplicate name */
-#define	DBPRIMITIVE    29					/* primitive prototype */
-#define	DBBADTMAT      30					/* bad transformation matrix */
-#define	DBNOVAR        31					/* variable does not exist */
-#define	DBVARFIXED     32					/* variable cannot be set */
-#define	DBVARARRDIS    33					/* variable cannot be displayable array */
-#define	DBLASTECH      34					/* this is the last technology */
-#define	DBTECINUSE     35					/* technology is still in use */
-#define	DBNOSLIDING    36					/* sliding not allowed */
+#define DBNOERROR       0					/* no error */
+#define DBNOMEM	        1					/* no memory */
+#define DBBADTRANS      2					/* bad transposition */
+#define DBBADROT        3					/* bad rotation */
+#define DBBADPROTO      4					/* bad prototype */
+#define DBBADPARENT     5					/* bad parent */
+#define DBBADINST       6					/* invalid instance */
+#define DBBADNAME       7					/* invalid name */
+#define DBBADWIDTH      8					/* bad width */
+#define DBBADENDAN      9					/* bad end A node/port */
+#define DBBADENDBN     10					/* bad end B node/port */
+#define DBBADENDAC     11					/* bad end A connection */
+#define DBBADENDBC     12					/* bad end B connection */
+#define DBBADENDAP     13					/* bad end A position */
+#define DBBADENDBP     14					/* bad end B position */
+#define DBBADNEWWID    15					/* bad new width */
+#define DBBADFACET     16					/* bad facet */
+#define DBBADLIB       17					/* bad library */
+#define DBBADSIZE      18					/* bad size */
+#define DBBADOBJECT    19					/* bad object type */
+#define DBBADSUBPORT   20					/* bad sub port */
+#define DBHASARCS      21					/* still has arcs */
+#define DBHASPORTS     22					/* still has ports */
+#define DBHASINSTANCES 23					/* facet has instances */
+#define DBRECURSIVE    24					/* recursive call */
+#define DBNOTINPORT    25					/* arc not in port */
+#define DBCONFLICT     26					/* conflicts with primitive */
+#define DBPORTMM       27					/* port mismatch */
+#define DBDUPLICATE    28					/* duplicate name */
+#define DBPRIMITIVE    29					/* primitive prototype */
+#define DBBADTMAT      30					/* bad transformation matrix */
+#define DBNOVAR        31					/* variable does not exist */
+#define DBVARFIXED     32					/* variable cannot be set */
+#define DBVARARRDIS    33					/* variable cannot be displayable array */
+#define DBLASTECH      34					/* this is the last technology */
+#define DBTECINUSE     35					/* technology is still in use */
+#define DBNOSLIDING    36					/* sliding not allowed */
 
-#define	DBADDSTRINGTOINFSTR		(1<<16)
-#define	DBADDTECHNOLOGY			(2<<16)
-#define	DBADDTOINFSTR			(3<<16)
-#define	DBALLOCARCINST			(4<<16)
-#define	DBALLOCCELL				(5<<16)
-#define	DBALLOCGEOM				(6<<16)
-#define	DBALLOCLIBRARY			(7<<16)
-#define	DBALLOCNODEINST			(8<<16)
-#define	DBALLOCNODEPROTO		(9<<16)
-#define	DBALLOCPOLYGON			(10<<16)
-#define	DBALLOCPORTARCINST		(11<<16)
-#define	DBALLOCPORTEXPINST		(12<<16)
-#define	DBALLOCPORTPROTO		(13<<16)
-#define	DBALLOCSTRING			(14<<16)
-#define	DBALLOCTECHNOLOGY		(15<<16)
-#define	DBALLOCVIEW				(16<<16)
-#define	DBCOPYNODEPROTO			(17<<16)
-#define	DBDELIND				(18<<16)
-#define	DBDELINDKEY				(19<<16)
-#define	DBDELVALKEY				(20<<16)
-#define	DBDESCRIBEVARIABLE		(21<<16)
-#define	DBEXTENDPOLYGON			(22<<16)
-#define	DBINITINFSTR			(23<<16)
-#define	DBINITOBJLIST			(24<<16)
-#define	DBINSIND				(25<<16)
-#define	DBINSINDKEY				(26<<16)
-#define	DBKILLARCINST			(27<<16)
-#define	DBKILLLIBRARY			(28<<16)
-#define	DBKILLNODEINST			(29<<16)
-#define	DBKILLNODEPROTO			(30<<16)
-#define	DBKILLPORTPROTO			(31<<16)
-#define	DBKILLTECHNOLOGY		(32<<16)
-#define	DBMAKEKEY				(33<<16)
-#define	DBMODIFYARCINST			(34<<16)
-#define	DBMOVEPORTPROTO			(35<<16)
-#define	DBNEWARCINST			(36<<16)
-#define	DBNEWLIBRARY			(37<<16)
-#define	DBNEWNODEINST			(38<<16)
-#define	DBNEWNODEPROTO			(39<<16)
-#define	DBNEWPORTPROTO			(40<<16)
-#define	DBNEWVIEW				(41<<16)
-#define	DBREPLACEARCINST		(42<<16)
-#define	DBREPLACENODEINST		(43<<16)
-#define	DBRETURNINFSTR			(44<<16)
-#define	DBSETIND				(45<<16)
-#define	DBSETINDKEY				(46<<16)
-#define	DBSETVAL				(47<<16)
-#define	DBSETVALKEY				(48<<16)
-#define	DBTRANSMULT				(49<<16)
-#define	DBXFORM					(50<<16)
+#define DBADDSTRINGTOINFSTR		(1<<16)
+#define DBADDTECHNOLOGY			(2<<16)
+#define DBADDTOINFSTR			(3<<16)
+#define DBALLOCARCINST			(4<<16)
+#define DBALLOCCELL				(5<<16)
+#define DBALLOCGEOM				(6<<16)
+#define DBALLOCLIBRARY			(7<<16)
+#define DBALLOCNODEINST			(8<<16)
+#define DBALLOCNODEPROTO		(9<<16)
+#define DBALLOCPOLYGON			(10<<16)
+#define DBALLOCPORTARCINST		(11<<16)
+#define DBALLOCPORTEXPINST		(12<<16)
+#define DBALLOCPORTPROTO		(13<<16)
+#define DBALLOCSTRING			(14<<16)
+#define DBALLOCTECHNOLOGY		(15<<16)
+#define DBALLOCVIEW				(16<<16)
+#define DBCOPYNODEPROTO			(17<<16)
+#define DBDELIND				(18<<16)
+#define DBDELINDKEY				(19<<16)
+#define DBDELVALKEY				(20<<16)
+#define DBDESCRIBEVARIABLE		(21<<16)
+#define DBEXTENDPOLYGON			(22<<16)
+#define DBINITINFSTR			(23<<16)
+#define DBINITOBJLIST			(24<<16)
+#define DBINSIND				(25<<16)
+#define DBINSINDKEY				(26<<16)
+#define DBKILLARCINST			(27<<16)
+#define DBKILLLIBRARY			(28<<16)
+#define DBKILLNODEINST			(29<<16)
+#define DBKILLNODEPROTO			(30<<16)
+#define DBKILLPORTPROTO			(31<<16)
+#define DBKILLTECHNOLOGY		(32<<16)
+#define DBMAKEKEY				(33<<16)
+#define DBMODIFYARCINST			(34<<16)
+#define DBMOVEPORTPROTO			(35<<16)
+#define DBNEWARCINST			(36<<16)
+#define DBNEWLIBRARY			(37<<16)
+#define DBNEWNODEINST			(38<<16)
+#define DBNEWNODEPROTO			(39<<16)
+#define DBNEWPORTPROTO			(40<<16)
+#define DBNEWVIEW				(41<<16)
+#define DBREPLACEARCINST		(42<<16)
+#define DBREPLACENODEINST		(43<<16)
+#define DBRETURNINFSTR			(44<<16)
+#define DBSETIND				(45<<16)
+#define DBSETINDKEY				(46<<16)
+#define DBSETVAL				(47<<16)
+#define DBSETVALKEY				(48<<16)
+#define DBTRANSMULT				(49<<16)
+#define DBXFORM					(50<<16)
 
 extern INTBIG       db_lasterror;			/* last error message */
 extern INTSML       db_printerrors;			/* flag for printing internal errors */
 
+/*************************** VARIABLE KEYS ****************************/
+
+extern INTBIG  db_tech_node_width_offset;		/* variable "TECH_node_width_offset" */
+extern INTBIG  db_tech_layer_function;			/* variable "TECH_layer_function" */
+extern INTBIG  db_tech_layer_names;				/* variable "TECH_layer_names" */
+extern INTBIG  db_tech_arc_width_offset;		/* variable "TECH_arc_width_offset" */
+
+/*************************** CHANGE CONTROL ****************************/
+
+extern INTSML  db_donextchangequietly;			/* nonzero to do next change quietly */
+extern INTSML  db_dochangesquietly;				/* nonzero to do changes quietly */
+
 /*************************** PROTOTYPES ****************************/
 
 /* database prototypes */
@@ -290,15 +304,18 @@ void         db_addportarcinst(NODEINST*, PORTARCINST*);
 void         db_addportexpinst(NODEINST*, PORTEXPINST*);
 INTSML       db_addtortnode(UINTBIG, RTNODE*, NODEPROTO*);
 void         db_boundfacet(NODEPROTO*, INTBIG*, INTBIG*, INTBIG*, INTBIG*);
+void         db_buildcellhashtable(LIBRARY*);
 CHANGE      *db_change(INTBIG, INTSML, INTBIG, INTBIG, INTBIG, INTBIG, INTBIG, INTBIG);
 void         db_changeport(PORTPROTO*, NODEINST*, PORTPROTO*);
 void         db_checkallmemoryfree(void);
+void         db_clearportcache(NODEPROTO *facet);
 void         db_endbatch(void);
 void         db_enterarcinst(ARCINST*);
 void         db_enternodeinst(NODEINST*);
 void         db_enterportproto(PORTPROTO*);
 INTSML       db_enterwindowpart(WINDOWPART *w);
 INTBIG       db_error(INTBIG);
+CELL        *db_findcellname(char*, LIBRARY*);
 void         db_forcehierarchicalanalysis(NODEPROTO*);
 void         db_freearcproto(ARCPROTO *ap);
 void         db_freechangememory(void);
