@@ -83,37 +83,37 @@ extern "C"
 {
 #endif
 
-#define	OK           1
-#define	CANCEL       2
+#define OK           1
+#define CANCEL       2
 
 #define ITEMTYPE   017
-#define	BUTTON       0			/* standard push-button */
-#define	DEFBUTTON    1			/* default push-button */
-#define	CHECK        2			/* standard check-box */
-#define	RADIO        3			/* standard radio-button */
-#define	SCROLL       4			/* scrolling list of text */
-#define	MESSAGE      5			/* non-editable text */
-#define	EDITTEXT     6			/* editable text */
-#define	ICON         7			/* 32x32 icon */
-#define	USERDRAWN  010			/* open area for user graphics */
-#define	POPUP      011			/* popup menu */
-#define	PROGRESS   012			/* progress bar (0-100%) */
-#define	DIVIDELINE 013			/* dividing line */
-#define	INACTIVE  0200
+#define BUTTON       0			/* standard push-button */
+#define DEFBUTTON    1			/* default push-button */
+#define CHECK        2			/* standard check-box */
+#define RADIO        3			/* standard radio-button */
+#define SCROLL       4			/* scrolling list of text */
+#define MESSAGE      5			/* non-editable text */
+#define EDITTEXT     6			/* editable text */
+#define ICON         7			/* 32x32 icon */
+#define USERDRAWN  010			/* open area for user graphics */
+#define POPUP      011			/* popup menu */
+#define PROGRESS   012			/* progress bar (0-100%) */
+#define DIVIDELINE 013			/* dividing line */
+#define INACTIVE  0200
 
 /* the last parameter to "DiaInitTextDialog" */
-#define	SCSELMOUSE    1			/* mouse clicks select in scroll area */
-#define	SCSELKEY      2			/* key strokes select in scroll area */
-#define	SCDOUBLEQUIT  4			/* double-click in scroll area exits dialog */
-#define	SCREPORT      8			/* report selections in the scroll area */
-#define	SCHORIZBAR   16			/* want horizontal scroll-bar in scroll area */
-#define	SCSMALLFONT  32			/* want small text in scroll area */
-#define	SCFIXEDWIDTH 64			/* want fixed-width text in scroll area */
+#define SCSELMOUSE    1			/* mouse clicks select in scroll area */
+#define SCSELKEY      2			/* key strokes select in scroll area */
+#define SCDOUBLEQUIT  4			/* double-click in scroll area exits dialog */
+#define SCREPORT      8			/* report selections in the scroll area */
+#define SCHORIZBAR   16			/* want horizontal scroll-bar in scroll area */
+#define SCSMALLFONT  32			/* want small text in scroll area */
+#define SCFIXEDWIDTH 64			/* want fixed-width text in scroll area */
 
 /* the mode to "DiaDrawLine" */
-#define	DLMODEON      1			/* draw the line on */
-#define	DLMODEOFF     2			/* draw the line off */
-#define	DLMODEINVERT  3			/* draw the line inverted */
+#define DLMODEON      1			/* draw the line on */
+#define DLMODEOFF     2			/* draw the line off */
+#define DLMODEINVERT  3			/* draw the line inverted */
 
 typedef struct
 {
@@ -174,6 +174,9 @@ void   DiaStuffLine(INTBIG item, char *line);
 void   DiaUnDimItem(INTBIG item);
 INTSML DiaValidEntry(INTBIG item);
 
+/* a modeless dialog */
+void   DiaMessageInDialog(char*, ...);
+
 INTSML DiaNullDlogList(char**);
 char  *DiaNullDlogItem(void);
 void   DiaNullDlogDone(void);
