@@ -35,82 +35,86 @@ extern "C"
 
 #include <setjmp.h>
 
+/* #define REPORTCONVERSION 1 */		/* uncomment to report on library conversion */
+
 /* successive versions of the binary file have smaller numbers */
-#define	MAGIC10              -1591		/* current magic number: version 10 */
-#define	MAGIC9               -1589		/* older magic number: version 9 */
-#define	MAGIC8               -1587		/* older magic number: version 8 */
-#define	MAGIC7               -1585		/* older magic number: version 7 */
-#define	MAGIC6               -1583		/* older magic number: version 6 */
-#define	MAGIC5               -1581		/* older magic number: version 5 */
-#define	MAGIC4               -1579		/* older magic number: version 4 */
-#define	MAGIC3               -1577		/* older magic number: version 3 */
-#define	MAGIC2               -1575		/* older magic number: version 2 */
-#define	MAGIC1               -1573		/* oldest magic number: version 1 */
+#define MAGIC10              -1591		/* current magic number: version 10 */
+#define MAGIC9               -1589		/* older magic number: version 9 */
+#define MAGIC8               -1587		/* older magic number: version 8 */
+#define MAGIC7               -1585		/* older magic number: version 7 */
+#define MAGIC6               -1583		/* older magic number: version 6 */
+#define MAGIC5               -1581		/* older magic number: version 5 */
+#define MAGIC4               -1579		/* older magic number: version 4 */
+#define MAGIC3               -1577		/* older magic number: version 3 */
+#define MAGIC2               -1575		/* older magic number: version 2 */
+#define MAGIC1               -1573		/* oldest magic number: version 1 */
 
 /* I/O formats: */
-#define	FBINARY                  0		/* Binary */
-#define	FBINARYNOBACKUP          1		/* Binary without backup */
-#define	FCIF                     2		/* CIF */
-#define	FDXF                     3		/* DXF format */
-#define	FEDIF                    4		/* Electronic Design Interchange Format */
-#define	FGDS                     5		/* Calma GDS-II format */
-#define	FHPGL                    6		/* HPGL format (plotting) */
-#define	FL                       7		/* L format (Silicon Compilers) */
-#define	FPOSTSCRIPT              8		/* PostScript (plotting) */
-#define	FPRINTEDPOSTSCRIPT       9		/* Printed PostScript (plotting) */
-#define	FQUICKDRAW              10		/* Quickdraw (plotting) */
+#define FBINARY                  0		/* Binary */
+#define FBINARYNOBACKUP          1		/* Binary without backup */
+#define FCIF                     2		/* CIF */
+#define FDXF                     3		/* DXF format */
+#define FEDIF                    4		/* Electronic Design Interchange Format */
+#define FGDS                     5		/* Calma GDS-II format */
+#define FHPGL                    6		/* HPGL format (plotting) */
+#define FL                       7		/* L format (Silicon Compilers) */
+#define FPOSTSCRIPT              8		/* PostScript (plotting) */
+#define FPRINTEDPOSTSCRIPT       9		/* Printed PostScript (plotting) */
+#define FQUICKDRAW              10		/* Quickdraw (plotting) */
 #define FSDF                    11		/* SDF */
-#define	FTEXT                   12		/* Text */
-#define	FVHDL                   13		/* VHDL */
-#define	FSUE                    14		/* SUE */
-#define	FLEF                    15		/* LEF (Library Exchange Format) */
-#define	FDEF                    16		/* DEF (Design Exchange Format) */
-#define	FSKILL                  17		/* SKILL (CADENCE command language) */
-#define	FEAGLE                  18		/* EAGLE (CadSoft) netlist */
-#define	FPADS                   19		/* PADS netlist */
-#define	FECAD                   20		/* ECAD netlist */
+#define FTEXT                   12		/* Text */
+#define FVHDL                   13		/* VHDL */
+#define FSUE                    14		/* SUE */
+#define FLEF                    15		/* LEF (Library Exchange Format) */
+#define FDEF                    16		/* DEF (Design Exchange Format) */
+#define FSKILL                  17		/* SKILL (CADENCE command language) */
+#define FEAGLE                  18		/* EAGLE (CadSoft) netlist */
+#define FPADS                   19		/* PADS netlist */
+#define FECAD                   20		/* ECAD netlist */
 
 /* aid:inout.IO_state: */
 #define NUMIOSTATEBITWORDS       2		/* number of words for these bits */
 
-#define	CIFINSQUARE             01		/* bit set for CIF input to square wires */
-#define	CIFOUTEXACT             02		/* bit set for CIF output to mimic screen */
-#define	CIFOUTMERGE             04		/* bit set for CIF output to merge polygons */
-#define	CIFOUTADDDRC           010		/* bit set for CIF output to include DRC layer */
-#define	IODEBUG                020		/* bit set to debug input */
-#define	PLOTFOCUS              040		/* bit set to focus plot output */
-#define	PLOTDATES             0100		/* bit set to include dates in plot output */
-#define CIFOUTNOTCEN          0200		/* bit set for non-centered CIF */
-#define GDSOUTMERGE           0400		/* bit set for GDS output to merge polgons */
-#define GDSOUTADDDRC         01000		/* bit set for GDS output to include DRC layer */
-#define EPSPSCRIPT           02000		/* bit set for Encapsulated PostScript output */
-#define HPGL2                04000		/* bit set for HPGL/2 output instead of HPGL */
-#define EDIFSCHEMATIC       010000		/* bit set for EDIF write schematic instead of netlist */
-#define DXFALLLAYERS        020000		/* bit set for DXF to read all layers instead of technology list */
-#define DXFFLATTENINPUT     040000		/* bit set for DXF to flatten input */
-#define DEFNOPHYSICAL      0100000		/* bit set to ignore DEF physical data */
-#define GDSINTEXT          0200000		/* bit set for GDS input of text */
-#define GDSINEXPAND        0400000		/* bit set for GDS expansion of cells */
-#define GDSINARRAYS	      01000000		/* bit set for GDS array instantiation */
-#define PSCOLOR           02000000		/* bit set for color PostScript output */
-#define PSPLOTTER         04000000		/* bit set for PostScript plotting (continuous roll) */
-#define PSROTATE         010000000		/* bit set for PostScript rotation (by 90 degrees) */
-#define	CIFOUTNOTOPCALL  020000000		/* bit set for CIF output to not call top cell */
-#define	GDSINIGNOREUKN   040000000		/* bit set for GDS input to ignore unknown layers */
-#define DEFNOLOGICAL    0100000000		/* bit set to ignore DEF logical data */
-#define BINOUTBACKUP    0600000000		/* bits that determine binary output backups */
+#define CIFINSQUARE             01		/* 0: bit set for CIF input to square wires */
+#define CIFOUTEXACT             02		/* 0: bit set for CIF output to mimic screen */
+#define CIFOUTMERGE             04		/* 0: bit set for CIF output to merge polygons */
+#define CIFOUTADDDRC           010		/* 0: bit set for CIF output to include DRC layer */
+#define IODEBUG                020		/* 0: bit set to debug input */
+#define PLOTFOCUS              040		/* 0: bit set to focus plot output */
+#define PLOTDATES             0100		/* 0: bit set to include dates in plot output */
+#define CIFOUTNOTCEN          0200		/* 0: bit set for non-centered CIF */
+#define GDSOUTMERGE           0400		/* 0: bit set for GDS output to merge polgons */
+#define GDSOUTADDDRC         01000		/* 0: bit set for GDS output to include DRC layer */
+#define EPSPSCRIPT           02000		/* 0: bit set for Encapsulated PostScript output */
+#define HPGL2                04000		/* 0: bit set for HPGL/2 output instead of HPGL */
+#define EDIFSCHEMATIC       010000		/* 0: bit set for EDIF write schematic instead of netlist */
+#define DXFALLLAYERS        020000		/* 0: bit set for DXF to read all layers instead of technology list */
+#define DXFFLATTENINPUT     040000		/* 0: bit set for DXF to flatten input */
+#define DEFNOPHYSICAL      0100000		/* 0: bit set to ignore DEF physical data */
+#define GDSINTEXT          0200000		/* 0: bit set for GDS input of text */
+#define GDSINEXPAND        0400000		/* 0: bit set for GDS expansion of cells */
+#define GDSINARRAYS	      01000000		/* 0: bit set for GDS array instantiation */
+#define PSCOLOR           02000000		/* 0: bit set for color PostScript output */
+#define PSPLOTTER         04000000		/* 0: bit set for PostScript plotting (continuous roll) */
+#define PSROTATE         010000000		/* 0: bit set for PostScript rotation (by 90 degrees) */
+#define CIFOUTNOTOPCALL  020000000		/* 0: bit set for CIF output to not call top cell */
+#define GDSINIGNOREUKN   040000000		/* 0: bit set for GDS input to ignore unknown layers */
+#define DEFNOLOGICAL    0100000000		/* 0: bit set to ignore DEF logical data */
+#define BINOUTBACKUP    0600000000		/* 0: bits that determine binary output backups */
 #define BINOUTNOBACK    0000000000		/*   for no backup of binary output files */
 #define BINOUTONEBACK   0200000000		/*   for one backup of binary output files */
 #define BINOUTFULLBACK  0400000000		/*   for full backup of binary output files */
-#define CHECKATWRITE   01000000000		/* bit set for database check before write */
-#define PSCOLORSTIP    02000000000		/* bit set for stippled color PostScript */
-#define PSHIERARCHICAL 04000000000		/* bit set for hierarchical PostScript */
-#define	PLOTFOCUSDPY  010000000000		/* bit set to focus plot output on display */
-#define	SKILLNOHIER   020000000000		/* bit set to ignore subfacets in SKILL */
+#define CHECKATWRITE   01000000000		/* 0: bit set for database check before write */
+#define PSCOLORSTIP    02000000000		/* 0: bit set for stippled color PostScript */
+#define PSHIERARCHICAL 04000000000		/* 0: bit set for hierarchical PostScript */
+#define PLOTFOCUSDPY  010000000000		/* 0: bit set to focus plot output on display */
+#define SKILLNOHIER   020000000000		/* 0: bit set to ignore subfacets in SKILL */
+#define CIFRESHIGH              01		/* 1: bit set for CIF resolution errors to highlight */
+#define SKILLFLATHIER           02		/* 1: bit set to flatten hierarchy in SKILL */
 
-#define	DEFAULTPSWIDTH         638		/* default PostScript page width (8.5" at 75dpi) */
-#define	DEFAULTPSHEIGHT        825		/* default PostScript page height (11.0" at 75dpi) */
-#define	DEFAULTPSMARGIN         56		/* default PostScript page height (0.75" at 75dpi) */
+#define DEFAULTPSWIDTH         638		/* default PostScript page width (8.5" at 75dpi) */
+#define DEFAULTPSHEIGHT        825		/* default PostScript page height (11.0" at 75dpi) */
+#define DEFAULTPSMARGIN         56		/* default PostScript page height (0.75" at 75dpi) */
 
 /* miscellaneous */
 extern FILE        *io_fileout;			/* channel for output */
@@ -153,11 +157,13 @@ void       io_setstatebits(INTBIG *bits);
 PORTPROTO *io_convertoldportname(char *portname, NODEPROTO *np);
 void       io_fixnewlib(LIBRARY *lib, INTBIG statusitem);
 void       io_setoutputbloat(char *layer, INTBIG amount);
+NODEPROTO *io_convertoldprimitives(TECHNOLOGY *tech, char *name);
 INTSML     io_setuptechorder(TECHNOLOGY *tech);
 INTSML     io_nextplotlayer(INTBIG i);
 INTBIG     io_getoutputbloat(char *layer);
 INTSML     io_doreadbinlibrary(LIBRARY *lib, INTSML newprogress);
 INTSML     io_doreadtextlibrary(LIBRARY *lib, INTSML newprogress);
+INTSML     io_getareatoprint(NODEPROTO *np, INTBIG *lx, INTBIG *hx, INTBIG *ly, INTBIG *hy, INTSML reduce);
 INTSML     io_readbinlibrary(LIBRARY *lib);
 INTSML     io_readciflibrary(LIBRARY *lib);
 INTSML     io_readdeflibrary(LIBRARY *lib);
@@ -185,6 +191,7 @@ INTSML     io_writequickdrawlibrary(LIBRARY *lib);
 INTSML     io_writeskilllibrary(LIBRARY *lib);
 INTSML     io_writetextlibrary(LIBRARY *lib);
 void       io_freebininmemory(void);
+void       io_freebinoutmemory(void);
 void       io_freecifinmemory(void);
 void       io_freecifparsmemory(void);
 void       io_freecifoutmemory(void);
