@@ -284,7 +284,7 @@ INTBIG io_tlistlength(tlist);
 char *io_storename(char*);
 INTBIG io_findlayernum(char*);
 void io_outputusercommand(INTSML, char*);
-void io_outputds(INTSML, char*, INTBIG, INTBIG, INTBIG, INTBIG);
+void io_outputds(INTBIG, char*, INTBIG, INTBIG, INTBIG, INTBIG);
 void io_outputpolygon(INTSML, char*);
 void io_outputwire(INTSML, INTBIG, char*);
 void io_outputwire(INTSML, INTBIG, char*);
@@ -292,5 +292,5 @@ void io_outputbox(INTSML, INTBIG, INTBIG, point, INTSML, INTSML);
 void io_outputflash(INTSML, INTBIG, point);
 void io_outputgeoname(char*, point, INTSML);
 void io_outputlabel(char*, point);
-void io_outputcall(INTSML, char*, char*);
+void io_outputcall(INTBIG, char*, char*);
 void io_outputdf(void);
