@@ -39,7 +39,7 @@ extern "C"
 */
 #define VERSION			"1.00"
 #define DATE			"Feb 24, 1987"
-#define	PARS			10
+#define PARS			10
 #define TRUE			1
 #define FALSE			0
 #define MAXLINE         80
@@ -143,7 +143,7 @@ extern "C"
 #define SC_NO_LAYER1_NODE			88
 #define SC_NO_LAYER2_NODE			89
 #define SC_NO_VIA					90
-#define	SC_NO_LAYER1_ARC			91
+#define SC_NO_LAYER1_ARC			91
 #define SC_NO_LAYER2_ARC			92
 #define SC_NO_LAYER_PWELL			93
 #define SC_NOCREATE_PWELL			94
@@ -166,14 +166,17 @@ extern "C"
 #define SC_PARAM_MAKE_L2_WIDTH           4
 #define SC_PARAM_MAKE_PWR_WIDTH          5
 #define SC_PARAM_MAKE_MAIN_PWR_WIDTH     6
-#define SC_PARAM_MAKE_PWELL_SIZE         7
-#define SC_PARAM_MAKE_PWELL_OFFSET       8
-#define SC_PARAM_MAKE_VIA_SIZE           9
-#define SC_PARAM_MAKE_MIN_SPACING       10
-#define SC_PARAM_ROUTE_FEEDTHRU_SIZE    11
-#define SC_PARAM_ROUTE_PORT_X_MIN_DIST  12
-#define SC_PARAM_ROUTE_ACTIVE_DIST      13
-#define SC_PARAM_PLACE_NUM_ROWS         14
+#define SC_PARAM_MAKE_MAIN_PWR_RAIL      7
+#define SC_PARAM_MAKE_PWELL_SIZE         8
+#define SC_PARAM_MAKE_PWELL_OFFSET       9
+#define SC_PARAM_MAKE_NWELL_SIZE        10
+#define SC_PARAM_MAKE_NWELL_OFFSET      11
+#define SC_PARAM_MAKE_VIA_SIZE          12
+#define SC_PARAM_MAKE_MIN_SPACING       13
+#define SC_PARAM_ROUTE_FEEDTHRU_SIZE    14
+#define SC_PARAM_ROUTE_PORT_X_MIN_DIST  15
+#define SC_PARAM_ROUTE_ACTIVE_DIST      16
+#define SC_PARAM_PLACE_NUM_ROWS         17
 
 /* default values for maker */
 #define DEFAULT_MIN_SPACING			2400		/* minimum metal spacing */
@@ -184,8 +187,11 @@ extern "C"
 #define DEFAULT_L1_TRACK_WIDTH		1600		/* layer 1 track width */
 #define DEFAULT_POWER_TRACK_WIDTH	2000		/* power track width */
 #define DEFAULT_MAIN_POWER_WIDTH	3200		/* main power buses width */
+#define DEFAULT_MAIN_POWER_RAIL     0			/* main power on horizontal */
 #define DEFAULT_PWELL_SIZE			0			/* P-well size */
 #define DEFAULT_PWELL_OFFSET		0			/* P-well offset from bottom */
+#define DEFAULT_NWELL_SIZE			0			/* N-well size */
+#define DEFAULT_NWELL_OFFSET		0			/* N-well offset from bottom */
 
 /* default values for router */
 #define DEFAULT_FEED_THROUGH_SIZE	6400	/* feed through size */
@@ -402,7 +408,7 @@ typedef struct Iscrowlist
 	struct Iscrowlist	*last;		/* last in row list */
 } SCROWLIST;
 
-#define	NOSCNBPLACE	((SCNBPLACE *)-1)
+#define NOSCNBPLACE	((SCNBPLACE *)-1)
 typedef struct Iscnbplace
 {
 	struct Iscnitree	*cell;		/* pointer to cell */
@@ -738,6 +744,7 @@ extern INTBIG      sc_filetypesctab;	/* Silicon compiler table file descriptor *
 void sc_init(INTBIG*, char*[], AIDENTRY*);
 void sc_done(void);
 INTSML sc_set(INTSML, char*[]);
+INTBIG sc_request(char*, va_list);
 void sc_slice(void);
 
 /* prototypes for intratool interface */
@@ -769,6 +776,7 @@ SCNITREE **Sc_findni(SCNITREE**, char*);
 char *Sc_find_leaf_port(char*, char*);
 int *Sc_leaf_port_bits_address(char*);
 int Sc_library_read(char*);
+int Sc_library_use(char*);
 void Sc_leaf_port_set_next(char*, char*);
 void Sc_leaf_port_set_first(char*, char*);
 void Sc_remove_inst_from_itree(SCNITREE**, SCNITREE*);
@@ -791,6 +799,7 @@ char *Sc_create_leaf_instance(char*, char*, int, int, int, int, int, int, char*)
 char *Sc_create_layer2_node(int, int, int, int, char*);
 char *Sc_create_via(int, int, char*);
 char *Sc_create_layer1_node(int, int, int, int, char*);
+char *Sc_create_nwell(int, int, int, int, char*);
 char *Sc_create_pwell(int, int, int, int, char*);
 char *Sc_create_track_layer1(char*, char*, char*, char*, int, char*);
 char *Sc_create_track_layer2(char*, char*, char*, char*, int, char*);
