@@ -33,10 +33,10 @@ extern "C"
 {
 #endif
 
-#define	ELLIPSEPOINTS         30		/* number of lines in an ellipse */
+#define ELLIPSEPOINTS         30		/* number of lines in an ellipse */
 
 /* the state bits */
-#define	ARTWORKFILLARROWHEADS  1		/* set to fill arrow heads */
+#define ARTWORKFILLARROWHEADS  1		/* set to fill arrow heads */
 
 /* all of the artwork primitives */
 extern TECHNOLOGY *art_tech;						/* the technology */
@@ -54,7 +54,7 @@ extern NODEPROTO  *art_arrowprim;					/* Arrow */
 extern NODEPROTO  *art_openedpolygonprim;			/* Opened-Polygon */
 extern NODEPROTO  *art_openeddottedpolygonprim;		/* Opened-Dotted-Polygon */
 extern NODEPROTO  *art_openeddashedpolygonprim;		/* Opened-Dashed-Polygon */
-extern NODEPROTO  *art_openedfardottepolygonprim;	/* Opened-FarDotted-Polygon */
+extern NODEPROTO  *art_openedthickerpolygonprim;	/* Opened-Thicker-Polygon */
 extern NODEPROTO  *art_closedpolygonprim;			/* Closed-Polygon */
 extern NODEPROTO  *art_filledpolygonprim;			/* Filled-Polygon */
 
@@ -72,12 +72,12 @@ extern INTBIG      art_patternkey;					/* key for "ART_pattern" */
 INTSML art_initprocess(TECHNOLOGY*, INTSML);
 void art_setmode(INTSML, char*[]);
 INTBIG art_request(char*, va_list);
-INTBIG art_nodepolys(NODEINST*, INTBIG*);
-INTBIG art_nodeEpolys(NODEINST*, INTBIG*);
+INTBIG art_nodepolys(NODEINST*, INTBIG*, WINDOWPART*);
+INTBIG art_nodeEpolys(NODEINST*, INTBIG*, WINDOWPART*);
 void art_shapenodepoly(NODEINST*, INTBIG, POLYGON*);
 void art_shapeEnodepoly(NODEINST*, INTBIG, POLYGON*);
 void art_shapeportpoly(NODEINST*, PORTPROTO*, POLYGON*, XARRAY, INTSML);
-INTBIG art_arcpolys(ARCINST*);
+INTBIG art_arcpolys(ARCINST*, WINDOWPART*);
 void art_shapearcpoly(ARCINST*, INTBIG, POLYGON*);
 
 #if defined(__cplusplus) && !defined(ALLCPLUSPLUS)
