@@ -40,12 +40,12 @@ extern NODEPROTO  *fpga_repeaterprim;		/* repeater */
 
 /* prototypes for technology routines */
 void fpga_shapearcpoly(ARCINST*, INTBIG, POLYGON*);
-INTBIG fpga_arcpolys(ARCINST*);
+INTBIG fpga_arcpolys(ARCINST*, WINDOWPART*);
 void fpga_shapeportpoly(NODEINST*, PORTPROTO*, POLYGON*, XARRAY, INTSML);
-INTBIG fpga_nodeEpolys(NODEINST*, INTBIG*);
+INTBIG fpga_nodeEpolys(NODEINST*, INTBIG*, WINDOWPART*);
 void fpga_shapeEnodepoly(NODEINST*, INTBIG, POLYGON*);
 void fpga_shapenodepoly(NODEINST*, INTBIG, POLYGON*);
-INTBIG fpga_nodepolys(NODEINST*, INTBIG*);
+INTBIG fpga_nodepolys(NODEINST*, INTBIG*, WINDOWPART*);
 void fpga_setmode(INTSML, char*[]);
 INTSML fpga_initprocess(TECHNOLOGY*, INTSML);
 void fpga_termprocess(void);
