@@ -49,8 +49,8 @@ extern ARCPROTO   *gen_unroutedarc;			/* unrouted arc */
 /* prototypes for interface routines */
 INTSML gen_initprocess(TECHNOLOGY*, INTSML);
 void gen_termprocess(void);
-INTBIG gen_nodepolys(NODEINST*, INTBIG*);
-INTBIG gen_arcpolys(ARCINST*);
+INTBIG gen_nodepolys(NODEINST*, INTBIG*, WINDOWPART*);
+INTBIG gen_arcpolys(ARCINST*, WINDOWPART*);
 void gen_shapenodepoly(NODEINST*, INTBIG, POLYGON*);
 void gen_shapeportpoly(NODEINST*, PORTPROTO*, POLYGON*, XARRAY, INTSML);
 void gen_shapearcpoly(ARCINST*, INTBIG, POLYGON*);
