@@ -35,7 +35,7 @@ extern "C"
 
 /* prototypes for technology routines */
 INTSML rcmos_initprocess(TECHNOLOGY*, INTSML);
-INTBIG rcmos_arcpolys(ARCINST*);
+INTBIG rcmos_arcpolys(ARCINST*, WINDOWPART*);
 void rcmos_shapearcpoly(ARCINST*, INTBIG, POLYGON*);
 
 #if defined(__cplusplus) && !defined(ALLCPLUSPLUS)
