@@ -35,84 +35,93 @@ extern "C"
 
 /******************** CONSTANTS ********************/
 
-/* the meaning of NODEINST->userbits: (bit 0 to 6 (0177) used globally) */
-#define	KILLN                   0200		/* if on, this nodeinst is marked for death */
-#define	REWANTN                 0400		/* nodeinst re-drawing is scheduled */
-#define	RELOCLN                01000		/* only local nodeinst re-drawing desired */
-#define	RETDONN                02000		/* transparent nodeinst re-draw is done */
+/* the meaning of NODEINST->userbits: (other bit (037400177) used globally) */
+#define KILLN                   0200		/* if on, this nodeinst is marked for death */
+#define REWANTN                 0400		/* nodeinst re-drawing is scheduled */
+#define RELOCLN                01000		/* only local nodeinst re-drawing desired */
+#define RETDONN                02000		/* transparent nodeinst re-draw is done */
 #define REODONN                04000		/* opaque nodeinst re-draw is done */
-#define	SPREADN               010000		/* if on, nodeinst has been spread */
-#define	WANTEXP               020000		/* if on, nodeinst wants to be (un)expanded */
-#define	TEMPFLG               040000		/* temporary flag for nodeinst display */
-#define	HARDSELECTN          0100000		/* set if hard to select */
+#define SPREADN               010000		/* if on, nodeinst has been spread */
+#define WANTEXP               020000		/* if on, nodeinst wants to be (un)expanded */
+#define TEMPFLG               040000		/* temporary flag for nodeinst display */
+#define HARDSELECTN          0100000		/* set if hard to select */
+#define HIGHLIGHTNVER        0200000		/* set if part of verbose highlighting */
 
-/* the meaning of ARCINST->userbits: (bits 0 to 24 (0177777777) used globally) */
-#define	KILLA             0200000000		/* if on, this arcinst is marked for death */
-#define	REWANTA           0400000000		/* arcinst re-drawing is scheduled */
-#define	RELOCLA          01000000000		/* only local arcinst re-drawing desired */
-#define	RETDONA          02000000000		/* transparent arcinst re-draw is done */
-#define	REODONA          04000000000		/* opaque arcinst re-draw is done */
-#define	SPREADA         010000000000		/* if on, arcinst streches in spread */
-#define	HARDSELECTA     020000000000		/* set if hard to select */
+/* the meaning of ARCINST->userbits: (bits 0-1, 4-24 (0177777763) used globally) */
+#define KILLA             0200000000		/* if on, this arcinst is marked for death */
+#define REWANTA           0400000000		/* arcinst re-drawing is scheduled */
+#define RELOCLA          01000000000		/* only local arcinst re-drawing desired */
+#define RETDONA          02000000000		/* transparent arcinst re-draw is done */
+#define REODONA          04000000000		/* opaque arcinst re-draw is done */
+#define SPREADA         010000000000		/* if on, arcinst streches in spread */
+#define HARDSELECTA     020000000000		/* set if hard to select */
+#define HIGHLIGHTAVER             04		/* set if part of verbose highlighting */
 
-/* the meaning of NODEPROTO->userbits: (bits 0 to 23 (077777777) used globally) */
-#define	NHASOPA           0100000000		/* set if prototype has opaque graphics */
-#define	NFIRSTOPA       017600000000		/* index of first opaque layer (if any) */
+/* the meaning of NODEPROTO->userbits: (bits 0-23 (077777777) used globally) */
+#define NHASOPA           0100000000		/* set if prototype has opaque graphics */
+#define NFIRSTOPA       017600000000		/* index of first opaque layer (if any) */
 #define NFIRSTOPASH               25		/* right shift of NFIRSTOPA */
 
-/* the meaning of ARCPROTO->userbits: (bits 0 to 22 (037777777) used globally) */
-#define	CANCONNECT         040000000		/* temporary for finding port connections */
-#define	AHASOPA           0100000000		/* set if prototype has opaque graphics */
-#define	AFIRSTOPA       017600000000		/* index of first opaque layer (if any) */
+/* the meaning of ARCPROTO->userbits: (bits 0-22 (037777777) used globally) */
+#define CANCONNECT         040000000		/* temporary for finding port connections */
+#define AHASOPA           0100000000		/* set if prototype has opaque graphics */
+#define AFIRSTOPA       017600000000		/* index of first opaque layer (if any) */
 #define AFIRSTOPASH               25		/* right shift of AFIRSTOPA */
 
-/* the meaning of PORTPROTO->userbits: (bits 0 to 31 used globally) */
+/* the meaning of PORTPROTO->userbits: (bits 0-31 used globally) */
 
-/* the meaning of "us_aid->aidstate" (bits 0 to 6 (0177) used globally) */
-#define	NOKEYLOCK            0200000		/* set to ignore key lockout after errors */
-#define	JUSTTHEFACTS         0400000		/* set to print only informative messages */
-#define	NODETAILS           01000000		/* set to make no mention of specific commands */
-#define	USEDIALOGS          02000000		/* set to use dialogs where appropriate */
-#define	ONESHOTMENU         04000000		/* set to have fixed menus highlight in one-shot mode */
-#define	INTERACTIVE        010000000		/* set if cursor commands drag */
-#define	INTEGRAL           020000000		/* set if windows must be integral pixels */
-#define	ECHOBIND           040000000		/* set to echo bound commands in full */
-#define	MENUON            0100000000		/* set if menu is on */
-#define	SHOWXY            0200000000		/* set to display cursor rather than TECH/LAMBDA */
-#define	TTYAUDIT          0400000000		/* set to log messages window */
+/* the meaning of "us_aid->aidstate" (bits 0-6 (0177) used globally) */
+#define NOKEYLOCK            0200000		/* set to ignore key lockout after errors */
+#define JUSTTHEFACTS         0400000		/* set to print only informative messages */
+#define NODETAILS           01000000		/* set to make no mention of specific commands */
+#define USEDIALOGS          02000000		/* set to use dialogs where appropriate */
+#define ONESHOTMENU         04000000		/* set to have fixed menus highlight in one-shot mode */
+#define INTERACTIVE        010000000		/* set if cursor commands drag */
+#define INTEGRAL           020000000		/* set if windows must be integral pixels */
+#define ECHOBIND           040000000		/* set to echo bound commands in full */
+#define MENUON            0100000000		/* set if menu is on */
+#define SHOWXY            0200000000		/* set to display cursor rather than TECH/LAMBDA */
+#define TTYAUDIT          0400000000		/* set to log messages window */
 #define TERMBEEP         01000000000		/* set to enable terminal beep */
 
 /* the meaning of "aid:user.USER_optionflags" */
-#define	NOTEXTSELECT              01		/* set to disable selection of ALL text unless "special" option used */
-#define	DRAWTINYFACETS            02		/* clear to "hash-out" tiny facets */
-#define	CHECKDATE                 04		/* set to check facet dates */
-#define	NOFACETCHANGES           010		/* set to lock facet changes */
-#define	NOPRIMCHANGES            020		/* set to lock lockable primitives */
+#define NOTEXTSELECT              01		/* set to disable selection of ALL text unless "special" option used */
+#define DRAWTINYFACETS            02		/* clear to "hash-out" tiny facets */
+#define CHECKDATE                 04		/* set to check facet dates */
+#define NOFACETCHANGES           010		/* set to lock facet changes */
+#define NOPRIMCHANGES            020		/* set to lock lockable primitives */
 #define PORTLABELS              0700		/* bits for port labels */
-#define PORTSFULL                  0		/* ports not drawn */
+#define PORTLABELSSH               6		/* right shift for port labels */
+#define PORTSFULL                  0		/* ports as complete text */
 #define PORTSCROSS              0100		/* ports drawn as crosses */
-#define PORTSOFF                0200		/* ports drawn as complete text */
+#define PORTSOFF                0200		/* ports not drawn */
 #define PORTSSHORT              0300		/* ports drawn as shortened text */
 #define PUTFACETCENTERINICON   01000		/* set to put facet centers in icons */
-#define	NOINSTANCESELECT       02000		/* set to disable selection of instances unless "special" option used */
-#define	NOMOVEAFTERDUP         04000		/* set to disable moving of objects after duplication */
-#define	DUPCOPIESPORTS        010000		/* set to have duplication/array/extract copy ports */
-#define	NO3DPERSPECTIVE       020000		/* set to turn off perspective in 3D view */
-#define	FLIPINTERACTIVECON    040000		/* set to flip the sense of the interactive constraint */
-#define	CENTEREDPRIMITIVES   0100000		/* set to place primitives by centers */
-#define	AUTOSWITCHTECHNOLOGY 0200000		/* set to switch technology when facet changes */
+#define NOINSTANCESELECT       02000		/* set to disable selection of instances unless "special" option used */
+#define NOMOVEAFTERDUP         04000		/* set to disable moving of objects after duplication */
+#define DUPCOPIESPORTS        010000		/* set to have duplication/array/extract copy ports */
+#define NO3DPERSPECTIVE       020000		/* set to turn off perspective in 3D view */
+#define FLIPINTERACTIVECON    040000		/* set to flip the sense of the interactive constraint */
+#define CENTEREDPRIMITIVES   0100000		/* set to place primitives by centers */
+#define AUTOSWITCHTECHNOLOGY 0200000		/* set to switch technology when facet changes */
+#define EXPORTLABELS        01400000		/* bits for export labels */
+#define EXPORTLABELSSH            17		/* right shift for export labels */
+#define EXPORTSFULL                0		/* exports drawn as complete text */
+#define EXPORTSCROSS         0400000		/* exports drawn as crosses */
+#define EXPORTSOFF          01000000		/* exports not drawn */
+#define EXPORTSSHORT        01400000		/* exports drawn as shortened text */
 
 /* special keys */
 #define CTRLCKEY                  03		/* the Control-C key */
 #define CTRLDKEY                  04		/* the Control-D key */
-#define	DELETEKEY                010		/* the DELETE key */
-#define	TABKEY                   011		/* the TAB key */
-#define	ESCKEY                   033		/* the Escape key */
-#define	LEFTARROWKEY             034		/* the Left Arrow key */
-#define	RIGHTARROWKEY            035		/* the Right Arrow key */
-#define	UPARROWKEY               036		/* the Up Arrow key */
-#define	DOWNARROWKEY             037		/* the Down Arrow key */
-#define	BACKSPACEKEY            0177		/* the BACKSPACE key */
+#define BACKSPACEKEY             010		/* the BACKSPACE key */
+#define TABKEY                   011		/* the TAB key */
+#define ESCKEY                   033		/* the Escape key */
+#define LEFTARROWKEY             034		/* the Left Arrow key */
+#define RIGHTARROWKEY            035		/* the Right Arrow key */
+#define UPARROWKEY               036		/* the Up Arrow key */
+#define DOWNARROWKEY             037		/* the Down Arrow key */
+#define DELETEKEY               0177		/* the DELETE key */
 #define FUNCTIONF1              0216
 #define FUNCTIONF2              0217
 #define FUNCTIONF3              0220
@@ -153,16 +162,19 @@ extern "C"
 #define ICONSTYLEPORTSTYLESH       16			/* shift for ICONSTYLEPORTSTYLE */
 #define ICONSTYLEDRAWNOLEADS 01000000			/* nonzero to remove leads */
 #define ICONSTYLEDRAWNOBODY  02000000			/* nonzero to remove body */
+#define ICONSTYLETECH       014000000			/* technology of ports  0=generic, 1=schematic */
+#define ICONSTYLETECHSH            20			/* shift for ICONSTYLETECH */
 
 #define ICONSTYLEEFAULT (0<<ICONSTYLESIDEINSH) | (1<<ICONSTYLESIDEOUTSH) | \
 						(2<<ICONSTYLESIDEBIDIRSH) | (3<<ICONSTYLESIDEPOWERSH) | \
 						(3<<ICONSTYLESIDEGROUNDSH) | (0<<ICONSTYLESIDECLOCKSH) | \
-						(1<<ICONSTYLEPORTLOCSH) | (0<<ICONSTYLEPORTSTYLESH)
+						(1<<ICONSTYLEPORTLOCSH) | (0<<ICONSTYLEPORTSTYLESH) | \
+						(0<<ICONSTYLETECHSH)
 
 /* the "style" parameter to "us_getcolormap()" */
-#define	COLORSEXISTING             0		/* reload existing colors */
+#define COLORSEXISTING             0		/* reload existing colors */
 #define COLORSDEFAULT              1		/* load default colors (gray background) */
-#define	COLORSBLACK                2		/* load black background colors */
+#define COLORSBLACK                2		/* load black background colors */
 #define COLORSWHITE                3		/* load white background colors */
 
 /* editor defaults in the user interface */
@@ -173,40 +185,44 @@ extern "C"
 #define DEFAULTREDISPHANDLER     us_redisplay
 
 /* the meaning of "us_state" */
-#define	COMMANDFAILED             01		/* set if current command has error */
+#define COMMANDFAILED             01		/* set if current command has error */
 #define SKIPKEYS                  02		/* set to ignore further single-key commands */
-#define	GOTXY                     04		/* set if X/Y co-ordinates are known */
-#define	LANGLOOP                 010		/* set if interpreting language code */
-#define	HIGHLIGHTSET             020		/* set if highlighting turned on in broadcast */
-#define	DIDINPUT                 040		/* set if just did input but haven't executed yet */
-#define	MACROTERMINATED         0100		/* set if macro has terminated early */
-#define	SNAPMODE               03600		/* snapping mode bits */
-#define	SNAPMODENONE               0		/* snapping mode: none */
-#define	SNAPMODECENTER          0200		/* snapping mode: center (circles, arcs) */
-#define	SNAPMODEMIDPOINT        0400		/* snapping mode: midpoint (lines) */
-#define	SNAPMODEENDPOINT        0600		/* snapping mode: endpoint (lines, arcs) */
-#define	SNAPMODETANGENT        01000		/* snapping mode: tangent (circles, arcs) */
-#define	SNAPMODEPERP           01200		/* snapping mode: perpendicular (lines, arcs, circles) */
-#define	SNAPMODEQUAD           01400		/* snapping mode: quadrant (circles, arcs) */
-#define	SNAPMODEINTER          01600		/* snapping mode: any intersection */
-#define	NONOVERLAPPABLEDISPLAY 04000		/* set to disallow overlappable layers in display */
-#define	NONPERSISTENTCURNODE  010000		/* clear to have current node be persistent */
-#define	CURFACETCHANGED       020000		/* set if current facet changed */
+#define GOTXY                     04		/* set if X/Y co-ordinates are known */
+#define LANGLOOP                 010		/* set if interpreting language code */
+#define HIGHLIGHTSET             020		/* set if highlighting turned on in broadcast */
+#define DIDINPUT                 040		/* set if just did input but haven't executed yet */
+#define MACROTERMINATED         0100		/* set if macro has terminated early */
+#define SNAPMODE               03600		/* snapping mode bits */
+#define SNAPMODENONE               0		/* snapping mode: none */
+#define SNAPMODECENTER          0200		/* snapping mode: center (circles, arcs) */
+#define SNAPMODEMIDPOINT        0400		/* snapping mode: midpoint (lines) */
+#define SNAPMODEENDPOINT        0600		/* snapping mode: endpoint (lines, arcs) */
+#define SNAPMODETANGENT        01000		/* snapping mode: tangent (circles, arcs) */
+#define SNAPMODEPERP           01200		/* snapping mode: perpendicular (lines, arcs, circles) */
+#define SNAPMODEQUAD           01400		/* snapping mode: quadrant (circles, arcs) */
+#define SNAPMODEINTER          01600		/* snapping mode: any intersection */
+#define NONOVERLAPPABLEDISPLAY 04000		/* set to disallow overlappable layers in display */
+#define NONPERSISTENTCURNODE  010000		/* clear to have current node be persistent */
+#define CURFACETCHANGED       020000		/* set if current facet changed */
 
 /* cursor definitions */
-#define	NORMALCURSOR               0		/* a normal arrow cursor */
-#define	WANTTTYCURSOR              1		/* a "use the TTY" cursor */
-#define	PENCURSOR                  2		/* a "pen" cursor */
-#define	NULLCURSOR                 3		/* a null cursor */
-#define	MENUCURSOR                 4		/* a menu selection cursor */
-#define	HANDCURSOR                 5		/* a hand cursor (for dragging) */
-#define	TECHCURSOR                 6		/* the technology edit cursor ("T") */
-#define	IBEAMCURSOR                7		/* the text edit cursor ("I") */
-#define	WAITCURSOR                 8		/* the hourglass cursor */
-#define	LRCURSOR                   9		/* a left/right pointing cursor */
-#define	UDCURSOR                  10		/* an up/down pointing cursor */
+#define NORMALCURSOR               0		/* a normal arrow cursor */
+#define WANTTTYCURSOR              1		/* a "use the TTY" cursor */
+#define PENCURSOR                  2		/* a "pen" cursor */
+#define NULLCURSOR                 3		/* a null cursor */
+#define MENUCURSOR                 4		/* a menu selection cursor */
+#define HANDCURSOR                 5		/* a hand cursor (for dragging) */
+#define TECHCURSOR                 6		/* the technology edit cursor ("T") */
+#define IBEAMCURSOR                7		/* the text edit cursor ("I") */
+#define WAITCURSOR                 8		/* the hourglass cursor */
+#define LRCURSOR                   9		/* a left/right pointing cursor */
+#define UDCURSOR                  10		/* an up/down pointing cursor */
 
 #define MESSAGESWIDTH             80		/* width of messages window */
+#define DEFAULTEXPLORERTEXTSIZE   12		/* default size of text in explorer window */
+
+#define DISPLAYSMALLSHIFT         20		/* fraction of screen that shifts when arrows are clicked */
+#define DISPLAYLARGESHIFT          2		/* fraction of screen that shifts when scrollbar is clicked */
 
 /******************** VARIABLES ********************/
 
@@ -249,6 +265,7 @@ extern INTBIG       us_quickkeys;			/* cached key for "USER_quick_keys" */
 extern INTBIG       us_interactiveangle;	/* cached key for "USER_interactive_angle" */
 extern INTBIG       us_tinylambdaperpixel;	/* cached key for "USER_tiny_lambda_per_pixel" */
 extern INTBIG       us_techstate;			/* cached key for "TECH_state" */
+extern INTBIG       us_techlaststate;		/* cached key for "TECH_last_state" */
 extern INTBIG       us_ignoreoptionchanges;	/* cached key for "USER_ignore_option_changes" */
 extern INTBIG       us_displayunits;		/* cached key for "USER_display_units" */
 
@@ -310,23 +327,23 @@ extern char       **us_quickkeylist;		/* quick keys in initial setup */
  * HIGHTEXT  arcinst   ---------  ---------  variable  nodeproto  displayable variable on arc
  * HIGHTEXT  nodeinst  portproto  ---------  variable  nodeproto  displayable variable on port
  * HIGHTEXT  --------  ---------  ---------  variable  nodeproto  displayable variable on facet
- * HIGHTEXT  --------  portproto  ---------  --------  nodeproto  port name
+ * HIGHTEXT  nodeinst  portproto  ---------  --------  nodeproto  port name
  * HIGHTEXT  nodeinst  ---------  ---------  --------  nodeproto  node name
  * HIGHBBOX  --------  ---------  ---------  --------  nodeproto  bounding box
  * HIGHLINE  --------  ---------  ---------  --------  nodeproto  line
  */
 
-#define	NOHIGHLIGHT ((HIGHLIGHT *)-1)
-#define	HIGHFROM                  01		/* set to display "from" object */
-#define	HIGHBBOX                  04		/* set to display bounding box */
-#define	HIGHLINE                 010		/* set to display a line */
-#define	HIGHTEXT                 020		/* set to display text */
-#define	HIGHTYPE                 037		/* the above highlight types */
-#define	HIGHEXTRA                040		/* set to display extra information */
-#define	HIGHSNAP                0100		/* set if snap point selected */
-#define	HIGHSNAPTAN             0200		/* set if tangent snap point selected */
-#define	HIGHSNAPPERP            0400		/* set if perpendicular snap point selected */
-#define	HIGHNOBOX              01000		/* set to disable basic selection box */
+#define NOHIGHLIGHT ((HIGHLIGHT *)-1)
+#define HIGHFROM                  01		/* set to display "from" object */
+#define HIGHBBOX                  04		/* set to display bounding box */
+#define HIGHLINE                 010		/* set to display a line */
+#define HIGHTEXT                 020		/* set to display text */
+#define HIGHTYPE                 037		/* the above highlight types */
+#define HIGHEXTRA                040		/* set to display extra information */
+#define HIGHSNAP                0100		/* set if snap point selected */
+#define HIGHSNAPTAN             0200		/* set if tangent snap point selected */
+#define HIGHSNAPPERP            0400		/* set if perpendicular snap point selected */
+#define HIGHNOBOX              01000		/* set to disable basic selection box */
 
 typedef struct Ihighlight
 {
@@ -337,6 +354,7 @@ typedef struct Ihighlight
 	PORTPROTO        *fromport;				/* "from" object: port on node */
 	INTSML            frompoint;			/* "from" object: polygon vertex */
 	VARIABLE         *fromvar;				/* "from" object: variable with text */
+	VARIABLE         *fromvarnoeval;		/* "from" object: unevaluated variable with text */
 	INTBIG            snapx, snapy;			/* HIGHSNAP: coordinate of snap point */
 	INTBIG            stalx,staly;			/* HIGHBBOX: lower corner HIGHLINE: point 1 */
 	INTBIG            stahx,stahy;			/* HIGHBBOX: upper corner HIGHLINE: point 2 */
@@ -344,7 +362,7 @@ typedef struct Ihighlight
 
 /******************** MACRO PACKAGE MODULES ********************/
 
-#define	NOMACROPACK ((MACROPACK *)-1)
+#define NOMACROPACK ((MACROPACK *)-1)
 
 typedef struct Imacropack
 {
@@ -358,9 +376,9 @@ extern MACROPACK *us_curmacropack;			/* current macro package */
 
 /******************** USER COMMAND MODULES ********************/
 
-#define	NOUSERCOM     ((USERCOM *)-1)
-#define	NUMKEYS                  256		/* maximum keys on keyboard */
-#define	NUMBUTS                   27		/* maximum buttons on tablet/mouse */
+#define NOUSERCOM     ((USERCOM *)-1)
+#define NUMKEYS                  256		/* maximum keys on keyboard */
+#define NUMBUTS                   45		/* maximum buttons on tablet/mouse */
 
 typedef struct Iusercom
 {
@@ -380,7 +398,7 @@ extern USERCOM *us_lastcom;					/* last command and arguments */
 
 /******************** POP-UP MENU MODULES ********************/
 
-#define	NOPOPUPMENUITEM ((POPUPMENUITEM *)-1)
+#define NOPOPUPMENUITEM ((POPUPMENUITEM *)-1)
 
 typedef struct
 {
@@ -392,7 +410,7 @@ typedef struct
 	INTSML   changed;						/* nonzero if value is changed */
 } POPUPMENUITEM;
 
-#define	NOPOPUPMENU ((POPUPMENU *)-1)
+#define NOPOPUPMENU ((POPUPMENU *)-1)
 
 typedef struct Ipopupmenu
 {
@@ -407,7 +425,7 @@ extern POPUPMENU *us_firstpopupmenu;		/* list of existing pop-up menus */
 
 /******************** PULLDOWN MENUS ********************/
 
-#define	MENUBARHEIGHT             25		/* size in pixels */
+#define MENUBARHEIGHT             25		/* size in pixels */
 
 extern INTSML      us_menubarsize;			/* actual size (0 if no menu bar) */
 extern INTSML      us_pulldownmenucount;	/* number of pulldown menus */
@@ -479,7 +497,7 @@ typedef struct
 	void    (*addline)(WINDOWPART*, INTBIG, char*);		/* add line to editor buffer */
 	void    (*replaceline)(WINDOWPART*, INTBIG, char*);	/* replace line in editor */
 	void    (*deleteline)(WINDOWPART*, INTBIG);			/* delete line in editor */
-	void    (*highlightline)(WINDOWPART*, INTBIG);		/* highlight line in editor */
+	void    (*highlightline)(WINDOWPART*, INTBIG, INTBIG);	/* highlight line in editor */
 	void    (*suspendgraphics)(WINDOWPART*);			/* suspend display changes in editor */
 	void    (*resumegraphics)(WINDOWPART*);				/* resume display changes in editor */
 	void    (*writetextfile)(WINDOWPART*, char*);		/* write editer buffer to file */
@@ -491,7 +509,8 @@ typedef struct
 	void    (*copy)(WINDOWPART*);						/* text copy routine */
 	void    (*paste)(WINDOWPART*);						/* text paste routine */
 	void    (*undo)(WINDOWPART*);						/* text undo routine */
-	void    (*search)(WINDOWPART*, char*, INTSML);		/* text search routine */
+	void    (*search)(WINDOWPART*, char*, char*, INTBIG);	/* text search/replace routine */
+	void    (*pan)(WINDOWPART*, INTBIG, INTBIG);		/* pan window in direction */
 } EDITORTABLE;
 
 extern EDITORTABLE us_editortable[];
@@ -508,22 +527,13 @@ struct commandinfo
 	COMCOMP *par[TEMPLATEPARS];				/* parameter types */
 };
 
-#define	MAXKEYWORD               100		/* maximum keywords on input */
-
-extern INTSML   us_pacurchar;				/* current character on current line */
-extern char    *us_pattyline;				/* current line image */
-extern char    *us_paambiguous;				/* used by "us_pagetword" for "?" message */
-extern INTSML   us_patruelength;			/* size of above two buffers */
-extern INTSML   us_paparamcount;			/* number of parameters read in */
-extern char    *us_palinestart;				/* first character position on line */
-extern COMCOMP *us_paparamtype[MAXKEYWORD]; /* expected type of each parameter */
-extern INTSML   us_pakeycount;				/* number of keywords allowed in parse */
-extern char   **us_paparamstart;			/* character position of each parameter */
-extern KEYWORD *us_pathiskey;				/* the current keyword list */
-
 /* defined in "usercom.c" */
 extern struct commandinfo us_lcommand[];
 
+#define MAXKEYWORD               100		/* maximum keywords on input */
+
+extern KEYWORD *us_pathiskey;				/* the current keyword list */
+
 /******************** PROTOTYPES ********************/
 
 /* prototypes for tool interface */
@@ -614,7 +624,7 @@ char          *us_getline(WINDOWPART*, INTBIG);
 void           us_addline(WINDOWPART*, INTBIG, char*);
 void           us_replaceline(WINDOWPART*, INTBIG, char*);
 void           us_deleteline(WINDOWPART*, INTBIG);
-void           us_highlightline(WINDOWPART*, INTBIG);
+void           us_highlightline(WINDOWPART*, INTBIG, INTBIG);
 void           us_suspendgraphics(WINDOWPART*);
 void           us_resumegraphics(WINDOWPART*);
 void           us_writetextfile(WINDOWPART*, char*);
@@ -626,7 +636,8 @@ void           us_cuttext(WINDOWPART*);
 void           us_copytext(WINDOWPART*);
 void           us_pastetext(WINDOWPART*);
 void           us_undotext(WINDOWPART*);
-void           us_searchtext(WINDOWPART*, char*, INTSML);
+void           us_searchtext(WINDOWPART*, char*, char*, INTBIG);
+void           us_pantext(WINDOWPART*, INTBIG, INTBIG);
 
 /* prototypes for intratool interface */
 void           us_3dbuttonhandler(WINDOWPART *w, INTSML but, INTSML x, INTSML y);
@@ -640,6 +651,7 @@ void           us_3dstartdrawing(WINDOWPART*);
 void           us_3dzoomview(WINDOWPART *win, float z);
 void           us_abortcommand(char*, ...);
 void           us_abortedmsg(void);
+void           us_addfacetcenter(NODEINST *ni);
 INTSML         us_addhighlight(HIGHLIGHT*);
 void           us_adjustdisplayabletext(NODEINST*);
 void           us_adjustlambda(INTBIG, INTBIG);
@@ -667,13 +679,15 @@ void           us_cleanupfacet(NODEPROTO*, INTSML);
 void           us_clearhighlightcount(void);
 void           us_clearwindow(WINDOWPART*);
 char          *us_commandvarname(INTSML);
+UINTBIG        us_computewipestate(NODEINST *ni);
 NODEPROTO     *us_convertfacet(NODEPROTO*, TECHNOLOGY*);
 void           us_copylisttofacet(GEOM**, NODEPROTO*, NODEPROTO*, INTSML, INTSML);
 void           us_copyobjects(WINDOWPART*);
-NODEPROTO     *us_copyrecursively(NODEPROTO*, char*, LIBRARY*, VIEW*, INTSML, INTSML, char*);
+NODEPROTO     *us_copyrecursively(NODEPROTO*, char*, LIBRARY*, VIEW*, INTSML, INTSML, char*, INTSML);
 INTSML         us_copyvars(INTBIG fromaddr, INTBIG fromtype, INTBIG toaddr, INTBIG totype);
 void           us_coverimplant(void);
 void           us_createexplorerstruct(void);
+void           us_createqueuedexports(void);
 NODEPROTO     *us_currentexplorernode(void);
 INTSML         us_cursoroverhigh(HIGHLIGHT*, INTBIG, INTBIG, WINDOWPART*);
 void           us_cutobjects(WINDOWPART*);
@@ -684,9 +698,9 @@ INTSML         us_demandxy(INTBIG*, INTBIG*);
 void           us_describecontents(NODEINST*);
 char          *us_describefont(INTBIG);
 char          *us_describemenunode(COMMANDBINDING *commandbinding);
-char          *us_describeportbits(PORTPROTO*);
 char          *us_describestyle(INTBIG);
 char          *us_describetextdescript(INTBIG);
+char          *us_displayedportname(PORTPROTO *pp, INTBIG dispstyle);
 void           us_docommands(FILE*, INTSML, char*);
 void           us_doexpand(NODEINST*, INTBIG, INTBIG);
 void           us_dofillet(void);
@@ -716,6 +730,7 @@ void           us_ensurepropertechnology(NODEPROTO *facet, char *forcedtechnolog
 void           us_ensurewindow(NODEPROTO*);
 void           us_erasegeometry(NODEPROTO*);
 void           us_erasenodeinst(NODEINST*);
+void           us_eraseobjectsinlist(NODEPROTO *np, GEOM **list);
 INTSML         us_erasepassthru(NODEINST*, INTSML);
 void           us_erasescreen(WINDOWFRAME*);
 void           us_erasewindow(WINDOWPART*);
@@ -725,6 +740,7 @@ INTSML         us_expandaddrtypearray(INTBIG*, INTBIG**, INTBIG**, INTBIG);
 void           us_expandttybuffers(INTSML);
 void           us_explorebuttonhandler(WINDOWPART*, INTSML, INTSML, INTSML);
 INTSML         us_explorecharhandler(WINDOWPART*, INTSML);
+void           us_explorepan(WINDOWPART*, INTBIG);
 void           us_exploreredisphandler(WINDOWPART*);
 INTSML         us_facetedithandler(WINDOWPART*, INTSML);
 INTBIG         us_figuregrabpoint(INTBIG, INTBIG, INTBIG, INTBIG, INTBIG, INTBIG, INTBIG);
@@ -732,7 +748,7 @@ void           us_figuretechopaque(TECHNOLOGY*);
 INTBIG         us_figurevariableplace(INTBIG, INTSML, char*[]);
 INTSML         us_fillcomcomp(USERCOM*, COMCOMP*[]);
 void           us_findlowerport(NODEINST**, PORTPROTO**);
-void           us_findobject(INTBIG, INTBIG, WINDOWPART*, HIGHLIGHT*, INTSML, INTSML, INTSML, INTSML, INTSML);
+void           us_findobject(INTBIG, INTBIG, WINDOWPART*, HIGHLIGHT*, INTBIG, INTBIG, INTBIG, INTBIG, INTBIG);
 void           us_forceeditchanges(void);
 void           us_freebindingparse(COMMANDBINDING *commandbinding);
 void           us_freecomekmemory(void);
@@ -748,24 +764,26 @@ void           us_freenetmemory(void);
 void           us_freeparsememory(void);
 void           us_freewindowmemory(void);
 void           us_freestatusmemory(void);
-void           us_freetrackmemory(void);
 void           us_freeusercom(USERCOM*);
 void           us_fullview(NODEPROTO*, INTBIG*, INTBIG*, INTBIG*, INTBIG*);
 INTBIG        *us_getallhighlighted(void);
+void           us_getarcbounds(ARCINST *ai, INTBIG *nlx, INTBIG *nhx, INTBIG *nly, INTBIG *nhy);
 NODEPROTO     *us_getareabounds(INTBIG*, INTBIG*, INTBIG*, INTBIG*);
-GEOM          *us_getclosest(INTBIG, INTBIG, NODEPROTO*);
+GEOM          *us_getclosest(INTBIG, INTBIG, INTBIG, NODEPROTO*);
 void           us_getcolormap(TECHNOLOGY*, INTSML, INTSML);
 INTSML         us_getcolormapentry(char*, INTSML);
 COMCOMP       *us_getcomcompfromkeyword(char *keyword);
 void           us_gethighaddrtype(HIGHLIGHT *high, INTBIG *addr, INTBIG *type);
 INTBIG         us_gethighdescript(HIGHLIGHT *high);
-GEOM         **us_gethighlighted(INTSML);
+GEOM         **us_gethighlighted(INTSML type, INTBIG *textcount, char ***textinfo);
 NODEINST      *us_gethighnodeinst(HIGHLIGHT *high);
 char          *us_gethighstring(HIGHLIGHT *high);
 void           us_gethightextcenter(HIGHLIGHT*, INTBIG*, INTBIG*, INTSML*);
 void           us_getlowleft(NODEINST*, INTBIG*, INTBIG*);
 VARIABLE      *us_getmacro(char*);
 MACROPACK     *us_getmacropack(char*);
+void           us_getnodebounds(NODEINST *ni, INTBIG *nlx, INTBIG *nhx, INTBIG *nly, INTBIG *nhy);
+void           us_getnodedisplayposition(NODEINST *ni, INTBIG *xpos, INTBIG *ypos);
 NODEINST      *us_getnodeonarcinst(GEOM**, PORTPROTO**, GEOM*, PORTPROTO*, INTBIG, INTBIG);
 UINTBIG        us_getobject(INTSML, INTSML);
 TECHNOLOGY    *us_getobjectinfo(INTBIG addr, INTBIG type, INTBIG *lx, INTBIG *hx, INTBIG *ly, INTBIG *hy);
@@ -774,7 +792,8 @@ INTSML         us_getonesnappoint(INTBIG *x, INTBIG *y);
 INTSML         us_getplacementangle(NODEPROTO *np);
 POPUPMENU     *us_getpopupmenu(char*);
 void           us_getquickkeylist(INTBIG *quickkeycount, char ***quickkeylist);
-void           us_getslide(INTSML, INTBIG, INTBIG, INTBIG, INTBIG, INTBIG*, INTBIG*);
+void           us_getsimpletype(char *pt, INTBIG *type, INTBIG *addr);
+void           us_getslide(INTBIG, INTBIG, INTBIG, INTBIG, INTBIG, INTBIG*, INTBIG*);
 void           us_getsnappoint(HIGHLIGHT*, INTBIG*, INTBIG*);
 INTBIG         us_gettextposition(char*);
 INTBIG         us_gettextsize(char*, INTBIG);
@@ -789,19 +808,20 @@ void           us_highlightmenu(INTBIG, INTBIG, INTSML);
 void           us_highlightwindow(WINDOWPART*, INTSML);
 TECHNOLOGY    *us_hightech(HIGHLIGHT *high);
 void           us_hsvtorgb(float, float, float, INTBIG*, INTBIG*, INTBIG*);
-void           us_identifyports(NODEINST*, NODEPROTO*, INTSML);
+void           us_identifyports(NODEINST*, NODEPROTO*, INTSML, INTSML);
 void           us_illustratecommandset(void);
 void           us_inheritattributes(NODEINST *ni);
 INTSML         us_initialbinding(void);
 void           us_initlayerletters(void);
 void           us_initpulldownmenu(POPUPMENU*);
+void           us_initqueuedexports(void);
 void           us_initstatus(void);
 INTSML         us_islasteval(INTSML);
 char          *us_keyname(INTSML);
 void           us_killcurrentwindow(INTSML);
 void           us_killwindowpickanother(WINDOWPART*);
 char          *us_layerletters(TECHNOLOGY*, INTSML);
-void           us_layouttext(char *layer, char *font, INTBIG scale, char *msg);
+void           us_layouttext(char *layer, INTBIG tsize, INTBIG scale, char *msg);
 INTBIG         us_makearcuserbits(ARCPROTO*);
 USERCOM       *us_makecommand(char*);
 char          *us_makefacetline(NODEPROTO*, INTSML);
@@ -815,15 +835,15 @@ void           us_manhattantravel(NODEINST*, INTSML);
 void           us_manymove(GEOM**, NODEINST**, INTBIG, INTBIG, INTBIG);
 INTSML         us_modarcbits(INTSML, INTSML, char*, GEOM**);
 void           us_modifytextdescript(HIGHLIGHT*, INTBIG);
-void           us_movetext(HIGHLIGHT*, INTSML, char*[]);
+void           us_moveselectedtext(INTBIG numtexts, char **textlist, INTBIG odx, INTBIG ody);
 NODEPROTO     *us_needfacet(void);
 INTSML         us_needwindow(void);
 void           us_nettravel(NODEINST*, INTBIG);
 MACROPACK     *us_newmacropack(char*);
 char          *us_nextvars(void);
+INTSML         us_nodemoveswithtext(HIGHLIGHT *high);
 NODEPROTO     *us_nodetocreate(INTSML getcontents, NODEPROTO *facet);
 INTSML         us_nulldisplayroutine(POLYGON*, WINDOWPART*);
-INTSML         us_pagetword(INTSML(*)(COMCOMP**, INTSML*), void(*)(char*), void(*)(void), void(*)(void), INTSML);
 void           us_parsebinding(char*, COMMANDBINDING*);
 INTSML         us_parsecommand(char*, char*[]);
 void           us_pasteobjects(WINDOWPART*);
@@ -841,6 +861,7 @@ void           us_printnodeaidinfo(NODEINST*);
 void           us_printtechnology(TECHNOLOGY*);
 void           us_pushhighlight(void);
 char          *us_putintoinfstr(char *str);
+INTSML         us_queuenewexport(NODEINST *ni, PORTPROTO *pp, PORTPROTO *origpp);
 void           us_queueopaque(GEOM*, INTSML);
 void           us_queueredraw(GEOM*, INTSML);
 void           us_redisplay(WINDOWPART*);
@@ -850,10 +871,9 @@ void           us_redostatus(WINDOWFRAME*);
 INTSML         us_reexportport(PORTPROTO*, NODEINST*);
 void           us_regridfacet(void);
 char          *us_removeampersand(char *name);
-void           us_removegetfacets(void);
 void           us_removeubchange(NODEPROTO*);
 void           us_renameport(PORTPROTO*, char*);
-NODEINST      *us_replacenodeinst(NODEINST *oldni, NODEPROTO *newnp, INTSML ignoreportnames);
+NODEINST      *us_replacenodeinst(NODEINST *oldni, NODEPROTO *newnp, INTSML ignoreportnames, INTSML allowdeletedports);
 void           us_rgbtohsv(INTSML, INTSML, INTSML, float*, float*, float*);
 INTBIG         us_rotatedescript(GEOM*, INTBIG);
 INTBIG         us_rotatedescriptI(GEOM*, INTBIG);
@@ -872,7 +892,7 @@ void           us_setcursorpos(WINDOWFRAME*, INTBIG, INTBIG);
 INTBIG         us_setdescriptoffset(INTBIG, INTBIG, INTBIG);
 void           us_setfacetname(WINDOWPART*);
 void           us_setfacetsize(WINDOWPART*);
-void           us_setfind(HIGHLIGHT*, INTSML, INTSML, INTSML, INTSML);
+void           us_setfind(HIGHLIGHT*, INTBIG, INTBIG, INTBIG, INTBIG);
 void           us_setgridsize(WINDOWPART*);
 void           us_sethighlight(HIGHLIGHT*, INTSML);
 void           us_setkeyequiv(POPUPMENU*, INTSML);
@@ -886,8 +906,9 @@ void           us_setnodeprotocenter(INTBIG, INTBIG, NODEPROTO*);
 NODEPROTO     *us_setscrolltocurrentfacet(INTBIG scrollitem, INTSML curinstance,
 					INTSML forcecurlib, INTSML justcells);
 void           us_settechname(WINDOWFRAME*);
-void           us_settrace(NODEINST*, INTBIG*, INTSML);
+void           us_settrace(NODEINST*, INTBIG*, INTBIG);
 INTBIG         us_setunexpand(NODEINST*, INTBIG);
+void           us_setvariablevalue(NODEINST *ni, INTBIG key, char *newvalue, INTBIG oldtype);
 void           us_setwindowsimulation(WINDOWPART*, INTSML);
 INTSML         us_setxy(INTSML, INTSML);
 void           us_shadowarcproto(WINDOWFRAME*, ARCPROTO*);
@@ -897,16 +918,18 @@ void           us_showcurrentarcproto(void);
 void           us_showcurrentnodeproto(void);
 void           us_shownetworkname(WINDOWFRAME*);
 INTSML         us_showpoly(POLYGON*, WINDOWPART*);
+void           us_skeletonize(NODEPROTO *np, char *newname, LIBRARY *newlib, INTSML quiet);
 void           us_slideleft(INTBIG);
 void           us_slideup(INTBIG);
 NODEPROTO    **us_sortlib(LIBRARY*, char*, INTSML);
 WINDOWPART    *us_splitcurrentwindow(INTSML, INTSML);
+INTBIG         us_spreadaround(NODEINST *ni, INTBIG amount, char *direction);
 void           us_spreadrotateconnection(NODEINST *theni);
 void           us_squarescreen(WINDOWPART*, WINDOWPART*, INTSML, INTBIG*, INTBIG*, INTBIG*, INTBIG*);
 INTBIG         us_stretchtonodes(ARCPROTO *typ, INTBIG wid, NODEINST *ni1, PORTPROTO *pp1, INTBIG otherx, INTBIG othery);
 char          *us_stripampersand(char*);
 WINDOWPART    *us_subwindow(INTBIG, INTBIG, INTBIG, INTBIG, WINDOWPART*);
-void           us_switchtofacet(NODEPROTO*, INTBIG, INTBIG, INTBIG, INTBIG, NODEINST*, PORTPROTO*, INTSML, INTSML);
+void           us_switchtofacet(NODEPROTO*, INTBIG, INTBIG, INTBIG, INTBIG, NODEINST*, PORTPROTO*, INTBIG, INTBIG);
 void           us_switchtolibrary(LIBRARY*);
 void           us_tecedaddfunstring(INTBIG);
 void           us_tecedentry(INTSML, char*[]);
@@ -941,6 +964,7 @@ void           us_undrawfacetvariable(VARIABLE *var, NODEPROTO *np);
 char          *us_uniqueportname(char*, NODEPROTO*);
 void           us_unknowncommand(void);
 void           us_unqueueredraw(LIBRARY *lib);
+INTSML         us_validname(char *name, INTBIG type);
 void           us_varchanges(WINDOWPART*, INTSML, char*, char*, INTBIG);
 char          *us_variableattributes(VARIABLE*, INTSML);
 char          *us_variabletypename(INTBIG);
@@ -950,14 +974,11 @@ void           us_wanttoread(char*);
 INTBIG         us_widestarcinst(ARCPROTO*, NODEINST*, PORTPROTO*);
 INTSML         us_windowcansplit(WINDOWPART*, WINDOWPART*, WINDOWPART*);
 void           us_windowfit(WINDOWFRAME*, INTSML, INTSML);
-void           us_writeprotoname(PORTPROTO*, INTSML, XARRAY, INTSML, INTSML, WINDOWPART*, INTBIG, INTBIG, INTBIG, INTBIG);
+void           us_writeprotoname(PORTPROTO*, INTSML, XARRAY, INTSML, INTSML, WINDOWPART*, INTBIG, INTBIG, INTBIG, INTBIG, INTBIG);
 void           us_yankonenode(NODEINST*);
 
 /* terminal interface */
-void           us_ttyclose(void);
-void           us_ttyinit(void);
 void           us_ttyprint(INTSML, char*, va_list);
-INTSML         us_specialparse(char*, COMCOMP*, INTSML, char *paramstart[]);
 
 /* the status bar */
 STATUSFIELD   *ttydeclarestatusfield(INTSML line, INTSML startper, INTSML endper, char *label);
