@@ -54,10 +54,10 @@ INTSML us_editfacetdlog(char*);
 INTSML us_facetdlog(void);
 INTSML us_facetlist(char*, char*[], INTSML);
 INTSML us_filedlog(COMCOMP*, char*[], char*, INTSML);
+INTSML us_findtextdlog(void);
 INTSML us_frameoptionsdlog(void);
 INTSML us_gaindlog(char*, char*[], char*);
 INTSML us_griddlog(void);
-void   us_givemessagedialog(char*);
 INTSML us_helpdlog(char*);
 INTSML us_highlayerlog(void);
 INTSML us_iconstyledlog(void);
@@ -66,6 +66,7 @@ INTSML us_lambdadlog(void);
 INTSML us_librarypathdlog(void);
 INTSML us_libtotechnologydlog(void);
 INTSML us_menudlog(char*[]);
+INTSML us_modtextsizedlog(void);
 INTSML us_newfacetdlog(char*[]);
 INTSML us_newviewdlog(char*[]);
 INTSML us_nodesizedlog(char*[]);
@@ -87,14 +88,15 @@ INTSML us_resistancedlog(char*, char*[], char*);
 INTSML us_selectoptdlog(void);
 INTSML us_selectportnodenetdlog(INTSML);
 INTSML us_showdlog(INTSML);
-INTSML us_sourcedlog(char*[], char*);
-INTSML us_spreaddlog(char*[]);
+INTSML us_sourcedlog(char*[], NODEINST*, VARIABLE*);
+INTSML us_spreaddlog(void);
 INTSML us_technologydlog(char*, char*[]);
 INTSML us_techoptdlog(void);
 INTSML us_techvarsdlog(void);
 INTSML us_tracedlog(void);
 INTSML us_variablesdlog(void);
 INTSML us_vccsdlog(char*[], char*);
+INTSML us_viewdlog(INTBIG);
 INTSML us_visiblelayersdlog(char*);
 INTSML us_widlendlog(char*[], char*);
 INTSML us_windowviewdlog(void);
