@@ -35,7 +35,7 @@ char *us_editemacsgetline(WINDOWPART*, INTBIG);
 void us_editemacsaddline(WINDOWPART*, INTBIG, char*);
 void us_editemacsreplaceline(WINDOWPART*, INTBIG, char*);
 void us_editemacsdeleteline(WINDOWPART*, INTBIG);
-void us_editemacshighlightline(WINDOWPART*, INTBIG);
+void us_editemacshighlightline(WINDOWPART*, INTBIG, INTBIG);
 void us_editemacssuspendgraphics(WINDOWPART*);
 void us_editemacsresumegraphics(WINDOWPART*);
 void us_editemacswritetextfile(WINDOWPART*, char*);
@@ -47,4 +47,5 @@ void us_editemacscut(WINDOWPART*);
 void us_editemacscopy(WINDOWPART*);
 void us_editemacspaste(WINDOWPART*);
 void us_editemacsundo(WINDOWPART*);
-void us_editemacssearch(WINDOWPART*, char*, INTSML);
+void us_editemacssearch(WINDOWPART*, char*, char*, INTBIG);
+void us_editemacspan(WINDOWPART*, INTBIG, INTBIG);
