@@ -34,36 +34,36 @@ extern "C"
 #endif
 
 /* the meaning of "EDTEC_option" on nodes */
-#define	LAYERCOLOR      1					/* color (layer facet) */
-#define	LAYERSTYLE      2					/* style (layer facet) */
-#define	LAYERCIF        3					/* CIF name (layer facet) */
-#define	LAYERFUNCTION   4					/* function (layer facet) */
-#define	LAYERLETTERS    5					/* letters (layer facet) */
-#define	LAYERPATTERN    6					/* pattern (layer facet) */
-#define	LAYERPATCH      8					/* patch of layer (node/arc facet) */
-#define	ARCFUNCTION     9					/* function (arc facet) */
-#define	NODEFUNCTION   10					/* function (node facet) */
-#define	ARCFIXANG      11					/* fixed-angle (arc facet) */
-#define	ARCWIPESPINS   12					/* wipes pins (arc facet) */
-#define	ARCNOEXTEND    13					/* end extension (arc facet) */
-#define	TECHLAMBDA     14					/* lambda (info facet) */
-#define	TECHDESCRIPT   15					/* description (info facet) */
-#define	NODESERPENTINE 16					/* serpentine MOS trans (node facet) */
-#define	LAYERDRCMINWID 17					/* DRC minimum width (layer facet) */
-#define	PORTOBJ        18					/* port object (node facet) */
-#define	HIGHLIGHTOBJ   19					/* highlight object (node/arc facet) */
-#define	LAYERGDS       20					/* Calma GDS-II layer (layer facet) */
-#define	NODESQUARE     21					/* square node (node facet) */
-#define	NODEWIPES      22					/* pin node can disappear (node facet) */
-#define	ARCINC         23					/* increment for arc angles (arc facet) */
-#define	NODELOCKABLE   25					/* lockable primitive (node facet) */
-#define	CENTEROBJ      26					/* grab point object (node facet) */
-#define	LAYERSPIRES    27					/* SPICE resistance (layer facet) */
-#define	LAYERSPICAP    28					/* SPICE capacitance (layer facet) */
-#define	LAYERSPIECAP   29					/* SPICE edge capacitance (layer facet) */
-#define	LAYERDXF       30					/* DXF layer (layer facet) */
-#define	LAYER3DHEIGHT  31					/* 3D height (layer facet) */
-#define	LAYER3DTHICK   32					/* 3D thickness (layer facet) */
+#define LAYERCOLOR      1					/* color (layer facet) */
+#define LAYERSTYLE      2					/* style (layer facet) */
+#define LAYERCIF        3					/* CIF name (layer facet) */
+#define LAYERFUNCTION   4					/* function (layer facet) */
+#define LAYERLETTERS    5					/* letters (layer facet) */
+#define LAYERPATTERN    6					/* pattern (layer facet) */
+#define LAYERPATCH      8					/* patch of layer (node/arc facet) */
+#define ARCFUNCTION     9					/* function (arc facet) */
+#define NODEFUNCTION   10					/* function (node facet) */
+#define ARCFIXANG      11					/* fixed-angle (arc facet) */
+#define ARCWIPESPINS   12					/* wipes pins (arc facet) */
+#define ARCNOEXTEND    13					/* end extension (arc facet) */
+#define TECHLAMBDA     14					/* lambda (info facet) */
+#define TECHDESCRIPT   15					/* description (info facet) */
+#define NODESERPENTINE 16					/* serpentine MOS trans (node facet) */
+#define LAYERDRCMINWID 17					/* DRC minimum width (layer facet, OBSOLETE) */
+#define PORTOBJ        18					/* port object (node facet) */
+#define HIGHLIGHTOBJ   19					/* highlight object (node/arc facet) */
+#define LAYERGDS       20					/* Calma GDS-II layer (layer facet) */
+#define NODESQUARE     21					/* square node (node facet) */
+#define NODEWIPES      22					/* pin node can disappear (node facet) */
+#define ARCINC         23					/* increment for arc angles (arc facet) */
+#define NODELOCKABLE   25					/* lockable primitive (node facet) */
+#define CENTEROBJ      26					/* grab point object (node facet) */
+#define LAYERSPIRES    27					/* SPICE resistance (layer facet) */
+#define LAYERSPICAP    28					/* SPICE capacitance (layer facet) */
+#define LAYERSPIECAP   29					/* SPICE edge capacitance (layer facet) */
+#define LAYERDXF       30					/* DXF layer (layer facet) */
+#define LAYER3DHEIGHT  31					/* 3D height (layer facet) */
+#define LAYER3DTHICK   32					/* 3D thickness (layer facet) */
 
 /* the strings that appear in the technology editor */
 #define TECEDNODETEXTCOLOR      "Color: "
@@ -89,7 +89,7 @@ typedef struct Ilist
 } LIST;
 
 
-#define	NOSAMPLE ((SAMPLE *)-1)
+#define NOSAMPLE ((SAMPLE *)-1)
 
 typedef struct Isample
 {
@@ -103,7 +103,7 @@ typedef struct Isample
 } SAMPLE;
 
 
-#define	NOEXAMPLE ((EXAMPLE *)-1)
+#define NOEXAMPLE ((EXAMPLE *)-1)
 
 typedef struct Iexample
 {
@@ -115,7 +115,7 @@ typedef struct Iexample
 
 
 /* port connections */
-#define	NOPCON ((PCON *)-1)
+#define NOPCON ((PCON *)-1)
 
 typedef struct Ipcon
 {
@@ -128,7 +128,7 @@ typedef struct Ipcon
 
 
 /* rectangle rules */
-#define	NORULE ((RULE *)-1)
+#define NORULE ((RULE *)-1)
 
 typedef struct Irule
 {
@@ -145,22 +145,33 @@ typedef struct Irule
 
 
 /* the meaning of "us_tecflags" */
-#define	HASUNCONDRC       01				/* has unconnected design rules */
-#define	HASCONDRC         02				/* has connected design rules */
-#define	HASCOLORMAP       04				/* has color map */
-#define	HASARCWID        010				/* has arc width offset factors */
-#define	HASCIF           020				/* has CIF layers */
-#define	HASDXF           040				/* has DXF layers */
-#define	HASGDS          0100				/* has Calma GDS-II layers */
-#define	HASGRAB         0200				/* has grab point information */
-#define	HASSPIRES       0400				/* has SPICE resistance information */
-#define	HASSPICAP      01000				/* has SPICE capacitance information */
-#define	HASSPIECAP     02000				/* has SPICE edge capacitance information */
-#define	HASDRCMINWID   04000				/* has DRC minimum width information */
-#define	HAS3DINFO     010000				/* has 3D height/thickness information */
+#define HASDRCMINWID         01				/* has DRC minimum width information */
+#define HASDRCMINWIDR        02				/* has DRC minimum width information */
+#define HASCOLORMAP          04				/* has color map */
+#define HASARCWID           010				/* has arc width offset factors */
+#define HASCIF              020				/* has CIF layers */
+#define HASDXF              040				/* has DXF layers */
+#define HASGDS             0100				/* has Calma GDS-II layers */
+#define HASGRAB            0200				/* has grab point information */
+#define HASSPIRES          0400				/* has SPICE resistance information */
+#define HASSPICAP         01000				/* has SPICE capacitance information */
+#define HASSPIECAP        02000				/* has SPICE edge capacitance information */
+#define HAS3DINFO         04000				/* has 3D height/thickness information */
+#define HASCONDRC        010000				/* has connected design rules */
+#define HASCONDRCR       020000				/* has connected design rules reasons */
+#define HASUNCONDRC      040000				/* has unconnected design rules */
+#define HASUNCONDRCR    0100000				/* has unconnected design rules reasons */
+#define HASCONDRCW      0200000				/* has connected wide design rules */
+#define HASCONDRCWR     0400000				/* has connected wide design rules reasons */
+#define HASUNCONDRCW   01000000				/* has unconnected wide design rules */
+#define HASUNCONDRCWR  02000000				/* has unconnected wide design rules reasons */
+#define HASCONDRCM     04000000				/* has connected multicut design rules */
+#define HASCONDRCMR   010000000				/* has connected multicut design rules reasons */
+#define HASUNCONDRCM  020000000				/* has unconnected multicut design rules */
+#define HASUNCONDRCMR 040000000				/* has unconnected multicut design rules reasons */
 
 /* additional technology variables */
-#define	NOTECHVAR ((TECHVAR *)-1)
+#define NOTECHVAR ((TECHVAR *)-1)
 
 typedef struct Itechvar
 {
@@ -177,7 +188,7 @@ typedef struct Itechvar
 extern TECHVAR us_knownvars[];
 
 /* the globals that define a technology */
-extern INTSML           us_tecflags;
+extern INTBIG           us_tecflags;
 extern INTBIG           us_teclayer_count;
 extern char           **us_teclayer_iname;
 extern char           **us_teclayer_names;
@@ -190,9 +201,7 @@ extern float           *us_tecspice_ecap;
 extern INTBIG          *us_teclayer_function;
 extern INTBIG          *us_tecgds_layers;
 extern char           **us_teclayer_letters;
-extern INTBIG          *us_tecdrc_min_unconnected_dist;
-extern INTBIG          *us_tecdrc_min_connected_dist;
-extern INTBIG          *us_tecdrc_min_width;
+extern DRCRULES        *us_tecdrc_rules;
 extern INTSML           us_teceddrclayers;
 extern char           **us_teceddrclayernames;
 extern INTBIG          *us_tec3d_height;
@@ -211,8 +220,9 @@ extern LIST us_tecnode_functions[];
 
 /* prototypes for intramodule routines */
 void     us_tecedgetbbox(NODEINST *ni, INTBIG *lx, INTBIG *hx, INTBIG *ly, INTBIG *hy);
-INTSML   us_tecedgetdrc(char *str, INTSML *connected, INTBIG *amt, INTSML *layer1,
-			INTSML *layer2, INTSML maxlayers, char **layernames);
+INTSML   us_tecedgetdrc(char *str, INTSML *connected, INTSML *wide, INTSML *multi,
+			INTSML *widrule, INTBIG *amt, INTSML *layer1, INTSML *layer2, char **rule,
+			INTSML maxlayers, char **layernames);
 EXAMPLE *us_tecedgetexamples(NODEPROTO *np, INTSML isnode);
 void     us_teceditdumparcs(FILE *f, TECHNOLOGY *tech, char *techname);
 void     us_teceditdumplayers(FILE *f, TECHNOLOGY *tech, char *techname);
@@ -230,7 +240,7 @@ void     us_tecedmakearc(NODEPROTO *np, INTBIG func, INTBIG fixang, INTBIG wipes
 void     us_tecedmakeinfo(NODEPROTO *np, INTBIG lambda, char *description);
 void     us_tecedmakelayer(NODEPROTO *np, INTSML colorindex, UINTSML stip[8], INTSML style,
 			char *ciflayer, INTBIG functionindex, char *layerletters, char *dxf,
-			INTBIG gds, float spires, float spicap, float spiecap, INTBIG drcminwid,
+			INTBIG gds, float spires, float spicap, float spiecap,
 			INTBIG height3d, INTBIG thick3d);
 LIBRARY *us_tecedmakelibfromtech(TECHNOLOGY *tech);
 void     us_tecedmakenode(NODEPROTO *np, INTBIG func, INTSML serp, INTSML square, INTSML wipes,
@@ -240,6 +250,10 @@ char    *us_tecedsamplename(NODEPROTO *layernp);
 void     us_tecedswapports(INTSML *p1, INTSML *p2, TECH_NODES *tlist);
 void     us_tecfromlibinit(LIBRARY *lib, char *techname, INTSML dumpc);
 void     us_tecedfreeexamples(EXAMPLE *ne);
+char    *us_tecedstripdash(char *str);
+void     us_tecedgetlayernamelist(void);
+void     us_tecedloaddrcmessage(DRCRULES *rules, LIBRARY *lib);
+INTSML   us_teceditgetdrcarrays(VARIABLE *var, DRCRULES *rules);
 
 #if defined(__cplusplus) && !defined(ALLCPLUSPLUS)
 }
