@@ -49,7 +49,7 @@ void us_sizeainit(ARCINST*);
 void us_sizeabegin(void);
 INTSML us_sizeadown(INTBIG, INTBIG);
 void us_sizeinit(NODEINST*);
-INTSML us_sizeterm(void);
+INTBIG us_sizeterm(void);
 void us_sizebegin(void);
 INTSML us_sizecdown(INTBIG, INTBIG);
 INTSML us_sizedown(INTBIG, INTBIG);
@@ -82,7 +82,7 @@ void us_multidragup(void);
 void us_pointinit(NODEINST*, INTSML);
 void us_findpointbegin(void);
 INTSML us_movepdown(INTBIG, INTBIG);
-void us_findiinit(INTSML, INTSML, INTSML, INTSML, INTSML, INTSML, INTSML);
+void us_findiinit(INTBIG, INTBIG, INTBIG, INTBIG, INTBIG, INTBIG, INTBIG);
 void us_findibegin(void);
 INTSML us_findidown(INTBIG, INTBIG);
 void us_findiup(void);
