@@ -37,7 +37,8 @@ extern "C"
 
 #define TRUE				1
 #define FALSE				0
-#define	FNAMESIZE			200				/* maximum size of file names */
+/* #define VHDL50           1 */			/* uncomment to enable VHDL 5.0 */
+#define FNAMESIZE			200				/* maximum size of file names */
 #define MAXVHDLLINE			256				/* maximum line length */
 #define NOTARGET			0				/* no output target, compile only */
 #define TARGET_ALS			1				/* output target is ALS file */
@@ -167,6 +168,8 @@ extern AIDENTRY *vhdl_aid;
 #define KEY_ALL				68
 #endif
 
+#define KEY_LIBRARY			69
+
 /********** Keyword Structures *****************************************/
 
 #define NOVKEYWORD	((VKEYWORD *)NULL)
@@ -439,7 +442,7 @@ typedef struct Idbnamelist
  *  Allan G. Jost, Technical University of Nova Scotia
  */
 
-#define	PARSE_ERR		-1					/* parsing error */
+#define PARSE_ERR		-1					/* parsing error */
 
 /******** Parser Constants and Structures ******************************/
 
