@@ -31,11 +31,14 @@
 #include "global.h"
 #include "database.h"
 #include "eio.h"
+#include "tech.h"
 #include "tecgen.h"
 #include "tecschem.h"
+#include "tecmocmos.h"
+#include "tecmocmossub.h"
 #include "edialogs.h"
 
-#define	REPORTINC   2000			/* bytes between graphical updates */
+#define REPORTINC   2000			/* bytes between graphical updates */
 
 typedef struct
 {
@@ -196,13 +199,13 @@ INTSML io_doreadbinlibrary(LIBRARY *lib, INTSML newprogress)
 	if (io_verbose < 0 && io_binindata.filelength > 0 && newprogress != 0)
 		DiaDoneDialog();
 	if (io_binindata.swapped_floats != 0)
-		ttyputverbose(_("Read %ld swapped float variables; may be incorrect"),
+		ttyputverbose(M_("Read %ld swapped float variables; may be incorrect"),
 			io_binindata.swapped_floats);
 	if (io_binindata.swapped_doubles != 0)
-		ttyputverbose(_("Read %ld swapped double variables; may be incorrect"),
+		ttyputverbose(M_("Read %ld swapped double variables; may be incorrect"),
 			io_binindata.swapped_doubles);
 	if (io_binindata.clipped_integers != 0)
-		ttyputverbose(_("Read %ld clipped integers; may be incorrect"),
+		ttyputverbose(M_("Read %ld clipped integers; may be incorrect"),
 			io_binindata.clipped_integers);
 	if (*result == 0) return(0);
 	ttyputerr("%s", result);
@@ -212,9 +215,10 @@ INTSML io_doreadbinlibrary(LIBRARY *lib, INTSML newprogress)
 char *io_doreadlibrary(LIBRARY *lib)
 {
 	REGISTER INTSML te, imosconv;
-	REGISTER INTBIG i, thisone, *geomtype, *geommoreup, top,
-		bot, look, nodeindex, arcindex, geomindex, oldunit;
-	INTBIG num, den, j, count, cou, arcinstpos, nodeinstpos, portprotopos;
+	REGISTER INTBIG i, thisone, *geomtype, *geommoreup, top, advise,
+		bot, look, nodeindex, arcindex, geomindex, oldunit, convertmosiscmostechnologies;
+	INTBIG num, den, j, count, cou, arcinstpos, nodeinstpos, portprotopos,
+		emajor, eminor, edetail;
 	REGISTER char *name, *sname;
 	REGISTER NODEPROTO *np, *onp, *lastnp, *ent;
 	REGISTER NODEINST *ni;
@@ -225,13 +229,14 @@ char *io_doreadlibrary(LIBRARY *lib)
 	REGISTER TECHNOLOGY *tech;
 	REGISTER PORTEXPINST **portexpinstlist;
 	REGISTER PORTARCINST **portarcinstlist;
-	char *version;
+	char *version, *errormsg;
 	unsigned char chmagic[4];
 	REGISTER VIEW *v;
 	REGISTER CELL *c, *lastcell;
 	REGISTER LIBRARY *savelib;
 
-	if (setjmp(io_binindata.filerror)) return(_("error reading file"));
+	errormsg = (char *)setjmp(io_binindata.filerror);
+	if (errormsg != 0) return(errormsg);
 
 	/* read magic number */
 	io_getin(&io_binindata.magic, 4, SIZEOFINTBIG, 0);
@@ -256,20 +261,20 @@ char *io_doreadlibrary(LIBRARY *lib)
 				return(_("Bad file format")); else
 		{
 			io_binindata.swap_bytes = 1;
-			ttyputverbose(_("File has swapped bytes in the header. Will attempt to read it."));
+			ttyputverbose(M_("File has swapped bytes in the header. Will attempt to read it."));
 		}
 	}
 	if (io_verbose > 0) switch (io_binindata.magic)
 	{
-		case MAGIC1: ttyputmsg(_("This library is 9 versions old"));   break;
-		case MAGIC2: ttyputmsg(_("This library is 8 versions old"));   break;
-		case MAGIC3: ttyputmsg(_("This library is 7 versions old"));   break;
-		case MAGIC4: ttyputmsg(_("This library is 6 versions old"));   break;
-		case MAGIC5: ttyputmsg(_("This library is 5 versions old"));   break;
-		case MAGIC6: ttyputmsg(_("This library is 4 versions old"));   break;
-		case MAGIC7: ttyputmsg(_("This library is 3 versions old"));   break;
-		case MAGIC8: ttyputmsg(_("This library is 2 versions old"));   break;
-		case MAGIC9: ttyputmsg(_("This library is 1 version old"));    break;
+		case MAGIC1: ttyputmsg(M_("This library is 9 versions old"));   break;
+		case MAGIC2: ttyputmsg(M_("This library is 8 versions old"));   break;
+		case MAGIC3: ttyputmsg(M_("This library is 7 versions old"));   break;
+		case MAGIC4: ttyputmsg(M_("This library is 6 versions old"));   break;
+		case MAGIC5: ttyputmsg(M_("This library is 5 versions old"));   break;
+		case MAGIC6: ttyputmsg(M_("This library is 4 versions old"));   break;
+		case MAGIC7: ttyputmsg(M_("This library is 3 versions old"));   break;
+		case MAGIC8: ttyputmsg(M_("This library is 2 versions old"));   break;
+		case MAGIC9: ttyputmsg(M_("This library is 1 version old"));    break;
 	}
 
 	/* determine size of "big" and "small" integers on disk */
@@ -302,12 +307,12 @@ char *io_doreadlibrary(LIBRARY *lib)
 	io_getinbig(&io_binindata.curnodeproto);
 	if (io_verbose > 0)
 	{
-		ttyputmsg(_("Reading %ld aids, %ld technologies"), io_binindata.aacount,
+		ttyputmsg(M_("Reading %ld aids, %ld technologies"), io_binindata.aacount,
 			io_binindata.techcount);
-		ttyputmsg(_("        %ld prim nodes, %ld prim ports, %ld arc protos"),
+		ttyputmsg(M_("        %ld prim nodes, %ld prim ports, %ld arc protos"),
 			io_binindata.nodepprotoindex, io_binindata.portpprotoindex,
 			io_binindata.arcprotoindex);
-		ttyputmsg(_("        %ld cells, %ld facets, %ld ports, %ld nodes, %ld arcs"),
+		ttyputmsg(M_("        %ld cells, %ld facets, %ld ports, %ld nodes, %ld arcs"),
 			io_binindata.cellindex, io_binindata.nodeprotoindex,
 			io_binindata.portprotoindex, io_binindata.nodeindex,
 			io_binindata.arcindex);
@@ -316,6 +321,23 @@ char *io_doreadlibrary(LIBRARY *lib)
 	/* get the Electric version (version 8 and later) */
 	if (io_binindata.magic <= MAGIC8) version = io_getstring(el_tempcluster); else
 		(void)allocstring(&version, "3.35", el_tempcluster);
+	parseelectricversion(version, &emajor, &eminor, &edetail);
+
+	/* for versions before 6.03q, convert MOSIS CMOS technology names */
+	convertmosiscmostechnologies = 0;
+	if (emajor < 6 ||
+		(emajor == 6 && eminor < 3) ||
+		(emajor == 6 && eminor == 3 && edetail < 17))
+	{
+		if ((asktech(mocmossub_tech, "get-state")&MOCMOSSUBNOCONV) == 0)
+			convertmosiscmostechnologies = 1;
+	}
+
+#ifdef REPORTCONVERSION
+	ttyputmsg("Library is version %s (%ld.%ld.%ld)", version, emajor, eminor, edetail);
+	if (convertmosiscmostechnologies != 0)
+		ttyputmsg("   Converting MOSIS CMOS technologies (mocmossub => mocmos)");
+#endif
 
 	/* get the newly created views (version 9 and later) */
 	for(v = el_views; v != NOVIEW; v = v->nextview) v->temp1 = 0;
@@ -373,7 +395,7 @@ char *io_doreadlibrary(LIBRARY *lib)
 	}
 
 	/* erase the current database */
-	if (io_verbose > 0) ttyputmsg(_("Erasing old library"));
+	if (io_verbose > 0) ttyputmsg(M_("Erasing old library"));
 	eraselibrary(lib);
 
 	/* allocate pointers */
@@ -486,6 +508,8 @@ char *io_doreadlibrary(LIBRARY *lib)
 
 	/* allocate all cells in the library */
 	lib->firstcell = NOCELL;
+	lib->tailcell = NOCELL;
+	lib->numcells = 0;
 	if (io_binindata.magic <= MAGIC9)
 	{
 		/* versions 9 and later allocate cells now */
@@ -496,6 +520,8 @@ char *io_doreadlibrary(LIBRARY *lib)
 			io_binindata.celllist[i]->lib = lib;
 			if (i > 0) io_binindata.celllist[i-1]->nextcell = io_binindata.celllist[i]; else
 				lib->firstcell = io_binindata.celllist[0];
+			lib->tailcell = io_binindata.celllist[i];
+			lib->numcells++;
 		}
 	}
 
@@ -523,6 +549,8 @@ char *io_doreadlibrary(LIBRARY *lib)
 				c->lib = lib;
 				if (i > 0) io_binindata.nodeprotolist[i-1]->cell->nextcell = c; else
 					lib->firstcell = io_binindata.nodeprotolist[0]->cell;
+				lib->tailcell = c;
+				lib->numcells++;
 			}
 		}
 	}
@@ -614,7 +642,7 @@ char *io_doreadlibrary(LIBRARY *lib)
 		arcinstpos += io_binindata.arccount[i];
 
 		if (io_verbose > 0)
-			ttyputmsg(_("Allocated %ld arcs, %ld nodes, %ld ports for facet %ld"),
+			ttyputmsg(M_("Allocated %ld arcs, %ld nodes, %ld ports for facet %ld"),
 				io_binindata.arccount[i], io_binindata.nodecount[i], io_binindata.portcount[i], i);
 	}
 
@@ -630,7 +658,13 @@ char *io_doreadlibrary(LIBRARY *lib)
 		/* associate the technology name with the true technology */
 		name = io_gettempstring();
 		if (name == 0) return(_("No memory"));
-		tech = gettechnology(name);
+		tech = NOTECHNOLOGY;
+		if (convertmosiscmostechnologies != 0)
+		{
+			if (namesame(name, "mocmossub") == 0) tech = gettechnology("mocmos"); else
+				if (namesame(name, "mocmos") == 0) tech = gettechnology("mocmosold");
+		}
+		if (tech == NOTECHNOLOGY) tech = gettechnology(name);
 
 		/* conversion code for old technologies */
 		imosconv = 0;
@@ -662,9 +696,19 @@ char *io_doreadlibrary(LIBRARY *lib)
 			for(np = tech->firstnodeproto; np != NONODEPROTO; np = np->nextnodeproto)
 				if (strcmp(np->primname, name) == 0) break;
 			if (np == NONODEPROTO)
+			{
+				/* automatic conversion of "Active-Node" in to "P-Active-Node" (MOSIS CMOS) */
+				if (strcmp(name, "Active-Node") == 0)
+				{
+					for(np = tech->firstnodeproto; np != NONODEPROTO; np = np->nextnodeproto)
+						if (strcmp(np->primname, "P-Active-Node") == 0) break;
+				}
+			}
+			if (np == NONODEPROTO)
 			{
 				if (io_verbose > 0)
-					ttyputmsg(_("No node exactly named %s in technology %s"), name, tech->techname);
+					ttyputmsg(M_("No node exactly named %s in technology %s"), name, tech->techname);
+				advise = 1;
 
 				/* look for substring name match at start of name */
 				i = strlen(name);
@@ -686,24 +730,24 @@ char *io_doreadlibrary(LIBRARY *lib)
 				/* special cases: convert "message" and "cell-center" nodes */
 				if (np == NONODEPROTO)
 				{
-					if (namesame(name, "Cell-Center") == 0) np = gen_facetcenterprim; else
-					if (namesame(name, "Message") == 0 ||
-						namesame(name, "Centered-Message") == 0 ||
-						namesame(name, "Left-Message") == 0 ||
-						namesame(name, "Right-Message") == 0) np = gen_invispinprim;
+					np = io_convertoldprimitives(tech, name);
+					if (np != NONODEPROTO) advise = 0;
 				}
 
 				/* give up and use first primitive in this technology */
 				if (np == NONODEPROTO) np = tech->firstnodeproto;
 
 				/* construct the error message */
-				(void)initinfstr();
-				if (io_binindata.techerror[te] != 0) (void)addstringtoinfstr(io_binindata.techerror[te]); else
-					(void)addstringtoinfstr(tech->techname);
-				(void)addtoinfstr(':');
-				(void)addstringtoinfstr(name);
-				(void)allocstring(&io_binindata.nodepprotoorig[io_binindata.nodepprotoindex], returninfstr(), el_tempcluster);
-				io_binindata.nodepprotoerror[io_binindata.nodepprotoindex] = 1;
+				if (advise != 0)
+				{
+					(void)initinfstr();
+					if (io_binindata.techerror[te] != 0) (void)addstringtoinfstr(io_binindata.techerror[te]); else
+						(void)addstringtoinfstr(tech->techname);
+					(void)addtoinfstr(':');
+					(void)addstringtoinfstr(name);
+					(void)allocstring(&io_binindata.nodepprotoorig[io_binindata.nodepprotoindex], returninfstr(), el_tempcluster);
+					io_binindata.nodepprotoerror[io_binindata.nodepprotoindex] = 1;
+				}
 			}
 			io_binindata.nodepprototech[io_binindata.nodepprotoindex] = te;
 			io_binindata.nodepprotolist[io_binindata.nodepprotoindex] = np;
@@ -842,11 +886,11 @@ char *io_doreadlibrary(LIBRARY *lib)
 	if (io_readnamespace() != 0) return(_("No memory"));
 
 	/* read the library variables */
-	if (io_verbose > 0) ttyputmsg(_("Reading library variables"));
+	if (io_verbose > 0) ttyputmsg(M_("Reading library variables"));
 	if (io_readvariables((INTBIG)lib, VLIBRARY) < 0) return(_("No memory"));
 
 	/* read the aid variables */
-	if (io_verbose > 0) ttyputmsg(_("Reading aid variables"));
+	if (io_verbose > 0) ttyputmsg(M_("Reading aid variables"));
 	for(i=0; i<io_binindata.aacount; i++)
 	{
 		j = io_binindata.aidlist[i];
@@ -856,7 +900,7 @@ char *io_doreadlibrary(LIBRARY *lib)
 	}
 
 	/* read the technology variables */
-	if (io_verbose > 0) ttyputmsg(_("Reading technology/primitive variables"));
+	if (io_verbose > 0) ttyputmsg(M_("Reading technology/primitive variables"));
 	for(te=0; te<io_binindata.techcount; te++)
 	{
 		tech = io_binindata.techlist[te];
@@ -950,6 +994,7 @@ char *io_doreadlibrary(LIBRARY *lib)
 		lastnp = np;
 		np->nextnodeproto = NONODEPROTO;
 	}
+	lib->tailnodeproto = lastnp;
 
 	/* add in external facets */
 	for(i=0; i<io_binindata.nodeprotoindex; i++)
@@ -969,7 +1014,7 @@ char *io_doreadlibrary(LIBRARY *lib)
 			io_convertportproto(&pp->subportproto);
 		}
 	}
-	if (io_verbose > 0) ttyputmsg(_("Done reading facets"));
+	if (io_verbose > 0) ttyputmsg(M_("Done reading facets"));
 
 	/* read the facet contents: arcs and nodes */
 	nodeindex = arcindex = geomindex = 0;
@@ -1105,8 +1150,10 @@ char *io_doreadlibrary(LIBRARY *lib)
 					lastcell->nextcell = c->nextcell;
 				efree((char *)c->cellname);
 				efree((char *)c);
+				lib->numcells--;
 			}
 		}
+		lib->tailcell = lastcell;
 	}
 
 	/* link up the nodes and arcs to their geometry modules */
@@ -1137,7 +1184,7 @@ char *io_doreadlibrary(LIBRARY *lib)
 	io_ensureallports(lib);
 
 	/* store the version number in the library */
-	nextvarchangequiet();
+	nextchangequiet();
 	(void)setvalkey((INTBIG)lib, VLIBRARY, makekey("LIB_former_version"),
 		(INTBIG)version, VSTRING|VDONTSAVE);
 	efree(version);
@@ -1152,7 +1199,7 @@ char *io_doreadlibrary(LIBRARY *lib)
 		if (io_binindata.techerror[te] != 0)
 	{
 		if (io_verbose > 0)
-			ttyputmsg(_("Unknown technology %s not used so all is well"),
+			ttyputmsg(M_("Unknown technology %s not used so all is well"),
 				io_binindata.techerror[te]);
 		efree(io_binindata.techerror[te]);
 	}
@@ -1160,7 +1207,7 @@ char *io_doreadlibrary(LIBRARY *lib)
 		if (io_binindata.aiderror[i] != 0)
 	{
 		if (io_verbose > 0)
-			ttyputmsg(_("Unknown aid %s not used so all is well"),
+			ttyputmsg(M_("Unknown aid %s not used so all is well"),
 				io_binindata.aiderror[i]);
 		efree(io_binindata.aiderror[i]);
 	}
@@ -1168,25 +1215,43 @@ char *io_doreadlibrary(LIBRARY *lib)
 		if (io_binindata.nodepprotoorig[i] != 0)
 	{
 		if (io_verbose > 0 && io_binindata.nodepprotoerror[i] != 0)
-			ttyputmsg(_("Unknown node %s not used so all is well"),
+			ttyputmsg(M_("Unknown node %s not used so all is well"),
 				io_binindata.nodepprotoorig[i]);
 		efree(io_binindata.nodepprotoorig[i]);
 	}
 	for(i=0; i<io_binindata.portpprotoindex; i++)
 		if (io_binindata.portpprotoerror[i] != 0)
 	{
-		if (io_verbose > 0) ttyputmsg(_("Unknown port %s not used so all is well"),
+		if (io_verbose > 0) ttyputmsg(M_("Unknown port %s not used so all is well"),
 			io_binindata.portpprotoerror[i]);
 		efree(io_binindata.portpprotoerror[i]);
 	}
 	for(i=0; i<io_binindata.arcprotoindex; i++)
 		if (io_binindata.arcprotoerror[i] != 0)
 	{
-		if (io_verbose > 0) ttyputmsg(_("Unknown arc %s not used so all is well"),
+		if (io_verbose > 0) ttyputmsg(M_("Unknown arc %s not used so all is well"),
 			io_binindata.arcprotoerror[i]);
 		efree(io_binindata.arcprotoerror[i]);
 	}
 
+	/* warn if the MOSIS CMOS technologies were converted */
+	if (convertmosiscmostechnologies != 0)
+	{
+		for(np = lib->firstnodeproto; np != NONODEPROTO; np = np->nextnodeproto)
+		{
+			for(ni = np->firstnodeinst; ni != NONODEINST; ni = ni->nextnodeinst)
+				if (ni->proto->primindex != 0 && ni->proto->tech == mocmos_tech) break;
+			if (ni != NONODEINST) break;
+			for(ai = np->firstarcinst; ai != NOARCINST; ai = ai->nextarcinst)
+				if (ai->proto->tech == mocmos_tech) break;
+			if (ai != NOARCINST) break;
+		}
+		if (np != NONODEPROTO)
+			DiaMessageInDialog(
+				_("Warning: library %s has older 'mocmossub' technology, converted to new 'mocmos'"),
+					lib->libname);
+	}
+
 	/* free the memory used here */
 	if (io_binindata.realname != 0)
 	{
@@ -1582,6 +1647,23 @@ void io_readexternalnodeproto(LIBRARY *lib, INTBIG i)
 			if (pp->temp1 != 0) continue;
 			ttyputerr(_("Error: facet %s in library %s, should not have port %s"),
 				describenodeproto(np), elib->libname, pp->protoname);
+#if 0		/* debug the error */
+			initinfstr();  addstringtoinfstr("Top level lists ports:");
+			for(j=0; j<portcount; j++)
+			{
+				addtoinfstr(' ');
+				addstringtoinfstr(localportnames[j]);
+			}
+			ttyputmsg("%s", returninfstr());
+
+			initinfstr();  addstringtoinfstr("Actual facet has ports:");
+			for(pp = np->firstportproto; pp != NOPORTPROTO; pp = pp->nextportproto)
+			{
+				addtoinfstr(' ');
+				addstringtoinfstr(pp->protoname);
+			}
+			ttyputmsg("%s", returninfstr());
+#endif
 			np = NONODEPROTO;
 			break;
 		}
@@ -1634,12 +1716,13 @@ void io_readexternalnodeproto(LIBRARY *lib, INTBIG i)
 			if (onp->cellview == v) np->version = onp->version + 1;
 
 		/* insert in the library and facet structures */
-		if (i != 0)
+		if (i != 0)		/* why not always? */
 		{
 			np->lastnodeproto = NONODEPROTO;
 			np->nextnodeproto = lib->firstnodeproto;
 			if (lib->firstnodeproto != NONODEPROTO)
-				lib->firstnodeproto->lastnodeproto = np;
+				lib->firstnodeproto->lastnodeproto = np; else
+					lib->tailnodeproto = np;
 			lib->firstnodeproto = np;
 		}
 
@@ -1725,7 +1808,7 @@ INTSML io_readnodeinst(NODEINST *ni)
 	if (inst != 0)
 	{
 		if (orignodename == 0) orignodename = makekey("NODE_original_name");
-		nextvarchangequiet();
+		nextchangequiet();
 		(void)setvalkey((INTBIG)ni, VNODEINST, orignodename, (INTBIG)inst, VSTRING|VDONTSAVE);
 	}
 
@@ -1749,7 +1832,7 @@ INTSML io_readnodeinst(NODEINST *ni)
 		if (inst == 0) return(1);
 		if (*inst != 0)
 		{
-			nextvarchangequiet();
+			nextchangequiet();
 			if (setvalkey((INTBIG)ni, VNODEINST, el_node_name,
 				(INTBIG)inst, VSTRING|VDISPLAY) == NOVARIABLE) return(1);
 		}
@@ -1881,7 +1964,7 @@ INTSML io_readarcinst(ARCINST *ai)
 		if (inst == 0) return(1);
 		if (*inst != 0)
 		{
-			nextvarchangequiet();
+			nextchangequiet();
 			if (setvalkey((INTBIG)ai, VARCINST, el_arc_name,
 				(INTBIG)inst, VSTRING|VDISPLAY) == NOVARIABLE) return(1);
 		}
@@ -1940,10 +2023,10 @@ INTSML io_readarcinst(ARCINST *ai)
 /* routine to read the global namespace.  returns nonzero upon error */
 INTSML io_readnamespace(void)
 {
-	REGISTER INTSML i;
+	REGISTER INTBIG i;
 
 	io_getinbig(&io_binindata.namecount);
-	if (io_verbose > 0) ttyputmsg(_("Reading %ld variable names"), io_binindata.namecount);
+	if (io_verbose > 0) ttyputmsg(M_("Reading %ld variable names"), io_binindata.namecount);
 	if (io_binindata.namecount == 0) return(0);
 
 	/* read in the namespace */
@@ -1980,15 +2063,14 @@ void io_ignorevariables(void)
  */
 INTSML io_readvariables(INTBIG addr, INTBIG type)
 {
-	REGISTER INTSML i, j, datasize;
-	REGISTER INTBIG ty;
+	REGISTER INTBIG i, j, datasize, ty, keyval;
 	REGISTER VARIABLE *var;
 	INTBIG count, newtype, len, newaddr, cou, newdescript;
 	INTSML key;
 
 	io_getinbig(&count);
 	if (io_verbose > 0 && count > 0)
-		 ttyputmsg(_("Reading %ld variables on %ld object"), count, type);
+		 ttyputmsg(M_("Reading %ld variables on %ld object"), count, type);
 	for(i=0; i<count; i++)
 	{
 		io_getin(&key, io_binindata.sizeofsmall, SIZEOFINTSML, 0);
@@ -2030,11 +2112,16 @@ INTSML io_readvariables(INTBIG addr, INTBIG type)
 
 		/* copy this variable into database */
 		if (io_binindata.newnames[key] == 0) io_binindata.newnames[key] = makekey(io_binindata.realname[key]);
-		nextvarchangequiet();
-		var = setvalkey(addr, type, io_binindata.newnames[key], newaddr, newtype);
+		keyval = io_binindata.newnames[key];
+		nextchangequiet();
+		var = setvalkey(addr, type, keyval, newaddr, newtype);
 		if (var == NOVARIABLE) return(-1);
 		var->textdescript = newdescript;
 
+		/* handle updating of technology caches */
+		if (type == VTECHNOLOGY)
+			changedtechnologyvariable(keyval);
+
 		/* free the memory allocated for the creation of this variable */
 		if ((ty&VTYPE) == VSTRING || (ty&(VCODE1|VCODE2)) != 0)
 		{
@@ -2398,6 +2485,8 @@ void io_ensureallports(LIBRARY *lib)
 			pp->subportproto = pe->proto;
 			pp->userbits = pe->proto->userbits;
 			pp->textdescript = defaulttextdescript(NOGEOM);
+			pp->textdescript = (pp->textdescript & ~VTSIZE) |
+				(defaulttextsize(1) << VTSIZESH);
 		}
 	}
 }
@@ -2544,7 +2633,7 @@ TECHNOLOGY *io_gettechlist(INTSML i)
 		io_binindata.techerror[i] = 0;
 		count = ttygetparam(_("Is this OK? [n] "), &us_noyesp, 2, pars);
 		if (count <= 0 || namesamen(pars[0], "yes", strlen(pars[0])) != 0)
-			longjmp(io_binindata.filerror, 1);
+			longjmp(io_binindata.filerror, (INTBIG)_("Aborted"));
 		ttyputerr(_("WARNING: saving this library with a substitute technology may corrupt it!"));
 	}
 	return(io_binindata.techlist[i]);
@@ -2587,12 +2676,12 @@ void io_getin(void *data, INTSML disksize, INTSML memorysize, INTSML signextend)
 	{
 		/* just peel it off the disk */
 		ret = xfread((char *)data, 1, disksize, io_binindata.filein);
-		if (ret == 0) longjmp(io_binindata.filerror, 1);
+		if (ret == 0) longjmp(io_binindata.filerror, (INTBIG)_("ERROR: premature end of file"));
 	} else
 	{
 		/* not a simple read, use a buffer */
 		ret = xfread((char *)buf, 1, disksize, io_binindata.filein);
-		if (ret == 0) longjmp(io_binindata.filerror, 1);
+		if (ret == 0) longjmp(io_binindata.filerror, (INTBIG)_("ERROR: premature end of file"));
 		if (io_binindata.swap_bytes > 0)
 		{
 			switch (disksize)
