@@ -31,9 +31,11 @@
 #include "global.h"
 #include "eio.h"
 #include "usr.h"
+#include "edialogs.h"
 
 static INTSML  io_outputstage;		/* progress of database (aborting) */
 static char   *io_namesave;			/* for saving the global namespace */
+static char   *io_outfilename = 0;	/* the name of the file being written */
 
 /* prototypes for local routines */
 void io_cleanup(void);
@@ -48,12 +50,20 @@ void io_putout(void*, INTSML);
 void io_putoutbig(void*);
 void io_putstring(char*);
 
+/*
+ * Routine to free all memory associated with this module.
+ */
+void io_freebinoutmemory(void)
+{
+	if (io_outfilename != 0) efree(io_outfilename);
+}
+
 INTSML io_writebinlibrary(LIBRARY *lib, INTSML nobackup)
 {
 	INTBIG i, a, n, p, magic, aacount, techcount, nodepprotoindex,
 		portpprotoindex, arcprotoindex, nodeprotoindex, nodeindex,
 		portprotoindex, arcindex, geomindex, curnodeproto, cellindex,
-		*curstate;
+		*curstate, facetshere;
 	UINTBIG olddate;
 	INTSML save, year, month, mday, hour, minute, second;
 	REGISTER NODEPROTO *np;
@@ -63,29 +73,39 @@ INTSML io_writebinlibrary(LIBRARY *lib, INTSML nobackup)
 	REGISTER ARCINST *ai;
 	REGISTER LIBRARY *olib;
 	REGISTER TECHNOLOGY *tech;
-	REGISTER char *rename, *name, *istrue, *dotaddr, *msg;
-	char *truename, *arg[3], datesuffix[30];
+	REGISTER char *rename, *istrue, *dotaddr, *msg, *ext;
+	char *truename, *name, *arg[3], datesuffix[30];
 	unsigned char wordsize;
 	REGISTER CELL *c;
 	REGISTER VIEW *v;
 
 	io_outputstage = 0;
-	i = initinfstr();
-	istrue = &lib->libfile[strlen(lib->libfile)-5];
-	if (strcmp(istrue, ".elib") == 0)
+	ext = &lib->libfile[strlen(lib->libfile)-5];
+	if (strcmp(ext, ".elib") == 0)
 	{
-		dotaddr = istrue;
-		*istrue = 0;
+		dotaddr = ext;
+		*ext = 0;
 	} else dotaddr = 0;
 	istrue = truepath(lib->libfile);
+	i = initinfstr();
 	i += addstringtoinfstr(istrue);
 	i += addstringtoinfstr(".elib");
+	if (io_outfilename == 0) 
+	{
+		i += allocstring(&io_outfilename, returninfstr(), io_aid->cluster);
+	} else
+	{
+		i += reallocstring(&io_outfilename, returninfstr(), io_aid->cluster);
+	}
 	if (dotaddr != 0) *dotaddr = '.';
 	if (i != 0)
 	{
-		ttyputerr(_("No memory! saving to 'PANIC'"));
+		DiaMessageInDialog(_("No memory! saving to 'PANIC'"));
 		name = _("PANIC");
-	} else name = returninfstr();
+	} else
+	{
+		name = io_outfilename;
+	}
 
 	curstate = io_getstatebits();
 	if ((curstate[0]&CHECKATWRITE) != 0)
@@ -150,7 +170,7 @@ INTSML io_writebinlibrary(LIBRARY *lib, INTSML nobackup)
 	io_fileout = xcreate(name, io_filetypeblib, msg, &truename);
 	if (io_fileout == NULL)
 	{
-		if (truename != 0) ttyputerr(_("Cannot write %s"), truename);
+		if (truename != 0) DiaMessageInDialog(_("Cannot write %s"), truename);
 		if (rename != 0) efree(rename);
 		return(1);
 	}
@@ -162,12 +182,19 @@ INTSML io_writebinlibrary(LIBRARY *lib, INTSML nobackup)
 		arg[1] = truename;
 		arg[2] = "library";
 		us_rename(3, arg);
-		name = truename;
+		if (io_outfilename == 0) 
+		{
+			(void)allocstring(&io_outfilename, truename, io_aid->cluster);
+		} else
+		{
+			(void)reallocstring(&io_outfilename, truename, io_aid->cluster);
+		}
+		name = io_outfilename;
 	}
 
-	if (setjmp(io_filerror))
+	if (setjmp(io_filerror) != 0)
 	{
-		ttyputerr(_("Error writing %s"), name);
+		DiaMessageInDialog(_("Error writing %s"), name);
 		xclose(io_fileout);
 		io_cleanup();
 		if (rename != 0)
@@ -226,6 +253,7 @@ INTSML io_writebinlibrary(LIBRARY *lib, INTSML nobackup)
 		for(ni = np->firstnodeinst; ni != NONODEINST; ni = ni->nextnodeinst)
 			ni->temp1 = nodeindex++;
 	}
+	facetshere = nodeprotoindex;
 
 	/* count and number the facets and cells in other libraries */
 	for(olib = el_curlib; olib != NOLIBRARY; olib = olib->nextlibrary)
@@ -275,10 +303,10 @@ INTSML io_writebinlibrary(LIBRARY *lib, INTSML nobackup)
 
 	if (io_verbose > 0)
 	{
-		ttyputmsg(_("Writing %ld aids, %ld technologies"), aacount, techcount);
-		ttyputmsg(_("        %ld prim nodes, %ld prim ports, %ld arc protos"),
+		ttyputmsg(M_("Writing %ld aids, %ld technologies"), aacount, techcount);
+		ttyputmsg(M_("        %ld prim nodes, %ld prim ports, %ld arc protos"),
 			nodepprotoindex, portpprotoindex, arcprotoindex);
-		ttyputmsg(_("        %ld cells %ld facets, %ld ports, %ld nodes, %ld arcs"),
+		ttyputmsg(M_("        %ld cells %ld facets, %ld ports, %ld nodes, %ld arcs"),
 			cellindex, nodeprotoindex, portprotoindex, nodeindex, arcindex);
 	}
 
@@ -323,7 +351,7 @@ INTSML io_writebinlibrary(LIBRARY *lib, INTSML nobackup)
 		io_putoutbig(&n);
 		for(p = 0, pp = np->firstportproto; pp != NOPORTPROTO; pp = pp->nextportproto) p++;
 		io_putoutbig(&p);
-		if (io_verbose > 0) ttyputmsg(_("Facet %s has %ld arcs, %ld nodes, %ld ports"),
+		if (io_verbose > 0) ttyputmsg(M_("Facet %s has %ld arcs, %ld nodes, %ld ports"),
 			describenodeproto(np), a, n, p);
 	}
 
@@ -433,7 +461,7 @@ INTSML io_writebinlibrary(LIBRARY *lib, INTSML nobackup)
 		/* write variable information */
 		io_writevariables(c->firstvar, c->numvar);
 
-		if (io_verbose > 0) ttyputmsg(_("Wrote cell %s"), c->cellname);
+		if (io_verbose > 0) ttyputmsg(M_("Wrote cell %s"), c->cellname);
 	}
 
 	/* write all of the cell names in external libraries */
@@ -487,7 +515,7 @@ INTSML io_writebinlibrary(LIBRARY *lib, INTSML nobackup)
 			io_writenodeinst(ni);
 			n++;
 		}
-		if (io_verbose > 0) ttyputmsg(_("Wrote %ld arcs and %ld nodes in facet %s"),
+		if (io_verbose > 0) ttyputmsg(M_("Wrote %ld arcs and %ld nodes in facet %s"),
 			a, n, describenodeproto(np));
 	}
 
@@ -500,11 +528,11 @@ INTSML io_writebinlibrary(LIBRARY *lib, INTSML nobackup)
 	/* if the file was renamed, kill the backup copy */
 	if (rename != 0)
 	{
-		if (eunlink(rename) < 0) ttyputerr(_("Cannot delete backup file %s"), rename);
+		if (eunlink(rename) < 0) ttyputerr(_("Error: cannot delete backup file %s"), rename);
 		efree(rename);
 	}
 
-	ttyputmsg(_("%s written (%ld %s)"), name, nodeprotoindex,
+	ttyputmsg(_("%s written (%ld %s)"), name, facetshere,
 		makeplural(_("facet"), nodeprotoindex));
 	lib->userbits = (lib->userbits & ~(LIBCHANGEDMAJOR | LIBCHANGEDMINOR)) | READFROMDISK;
 	return(0);
@@ -597,7 +625,7 @@ void io_writenodeproto(NODEPROTO *np, INTSML thislib)
 		io_writevariables(np->firstvar, np->numvar);
 	}
 
-	if (io_verbose > 0) ttyputmsg(_("Wrote facet %s"), describenodeproto(np));
+	if (io_verbose > 0) ttyputmsg(M_("Wrote facet %s"), describenodeproto(np));
 }
 
 void io_writenodeinst(NODEINST *ni)
@@ -696,7 +724,7 @@ void io_writearcinst(ARCINST *arcinst)
 /* routine to write the global namespace */
 void io_writenamespace(void)
 {
-	REGISTER INTSML i;
+	REGISTER INTBIG i;
 	INTBIG val;
 
 	val = el_numnames;
@@ -710,18 +738,18 @@ void io_writenamespace(void)
 	{
 		ttyputerr(_("No memory! destroying internal variables to save file"));
 		ttyputerr(_("Saved file is good, memory NOT: quit when save finishes!"));
-		for(i=0; i<el_numnames; i++) *((INTSML *)el_namespace[i]) = i;
+		for(i=0; i<el_numnames; i++) *((INTSML *)el_namespace[i]) = (INTSML)i;
 	} else for(i=0; i<el_numnames; i++)
 	{
 		io_namesave[2*i] = el_namespace[i][0];
 		io_namesave[2*i+1] = el_namespace[i][1];
-		*((INTSML *)el_namespace[i]) = i;
+		*((INTSML *)el_namespace[i]) = (INTSML)i;
 	}
 }
 
 void io_restorenamespace(void)
 {
-	REGISTER INTSML i;
+	REGISTER INTBIG i;
 
 	if (el_numnames == 0) return;
 	for(i=0; i<el_numnames; i++)
