@@ -32,12 +32,13 @@
 #if IOCIF
 
 #include "global.h"
+#include "database.h"
 #include "eio.h"
 #include "iocifpars.h"
 #include "edialogs.h"
 #include <math.h>
 
-#define	RTOD (45. / atan(1.))
+#define RTOD (45. / atan(1.))
 
 /**************************** CIF CELLS ****************************/
 
@@ -45,13 +46,15 @@
 
 typedef struct Icifcell
 {
-	INTSML           cindex;		/* cell index given in the define statement */
+	INTBIG           cindex;		/* cell index given in the define statement */
 	INTBIG           l, r, t, b;	/* bounding box of cell */
 	NODEPROTO       *addr;			/* the address of the cif cell */
 	struct Icifcell *nextcell;		/* the next cif cell in the list */
 } CIFCELL;
 
-CIFCELL *io_cifcells = NOCIFCELL;	/* head of list of 'cifcells' */
+CIFCELL **io_cifcells = (CIFCELL **)NOSTRING; /* hash table of 'cifcells' */
+static INTBIG io_cifcells_size = 0; 	/* allocated size of io_cifcells */
+static INTBIG io_cifcells_num = 0;	/* number of used entries in io_cifcells */
 CIFCELL *io_curcell;				/* the current cell */
 static INTSML io_cifroundwires;		/* nonzero if wires should be rounded */
 
@@ -85,7 +88,7 @@ CIFLIST *io_curlist;				/* current location in list */
 
 typedef struct Icfstart
 {
-	INTSML cindex;					/* cell index */
+	INTBIG cindex;					/* cell index */
 	char  *name;					/* cell name */
 	INTBIG l, r, t, b;				/* bounding box of cell */
 } CFSTART;
@@ -120,7 +123,7 @@ typedef struct Iclabel
 
 typedef struct Iccall
 {
-	INTSML          cindex;			/* index of cell called */
+	INTBIG          cindex;			/* index of cell called */
 	char           *name;			/* name of cell called */
 	struct Ictrans *list;			/* list of transformations */
 } CCALL;
@@ -163,8 +166,9 @@ INTSML     io_nodes_call(void);
 void       io_rotatelayer(INTBIG*, INTBIG*, INTSML);
 INTSML     io_initfind(void);
 NODEPROTO *io_findprotonode(INTSML);
-CIFCELL   *io_findcifcell(INTSML);
-CIFCELL   *io_newcifcell(void);
+CIFCELL   *io_findcifcell(INTBIG);
+void       io_insertcifcell(CIFCELL *cifcell);
+CIFCELL   *io_newcifcell(INTBIG);
 CIFLIST   *io_newciflist(INTSML);
 void       io_placeciflist(INTSML);
 CTRANS    *io_newctrans(void);
@@ -244,7 +248,7 @@ INTSML io_interpret(FILE *file)
 	comcount = io_parsefile();		/* read in the cif */
 	(void)io_doneparser();
 	if (stopping(STOPREASONCIF)) return(1);
-	ttyputverbose(_("Total CIF commands: %ld"), comcount);
+	ttyputverbose(M_("Total CIF commands: %ld"), comcount);
 
 	if (io_fatalerrors() > 0) return(1);
 
@@ -287,7 +291,12 @@ INTSML io_listtonodes(LIBRARY *lib)
 				break;
 			case C_END:
 				/* make cell size right */
+#if 0
 				(*el_curconstraint->solve)(io_incell);
+#else
+				db_boundfacet(io_incell, &io_incell->lowx, &io_incell->highx,
+					&io_incell->lowy, &io_incell->highy);
+#endif
 				lib->curnodeproto = io_incell;
 				io_incell = NONODEPROTO;
 				break;
@@ -324,9 +333,8 @@ NODEPROTO *io_nodes_start(LIBRARY *lib)
 	CFSTART *cs;
 	char *opt;
 
-	if ((cifcell = io_newcifcell()) == NOCIFCELL) return(NONODEPROTO);
 	cs = (CFSTART *)io_curlist->member;
-	cifcell->cindex = cs->cindex;
+	if ((cifcell = io_newcifcell(cs->cindex)) == NOCIFCELL) return(NONODEPROTO);
 	cifcell->l = cs->l;   cifcell->r = cs->r;
 	cifcell->b = cs->b;   cifcell->t = cs->t;
 	io_curnodeprotoname = cs->name;
@@ -570,7 +578,7 @@ INTSML io_initfind(void)
 		nodeinst->highx = np->highx;
 		nodeinst->lowy  = np->lowy;
 		nodeinst->highy = np->highy;
-		i = nodepolys(nodeinst, 0);
+		i = nodepolys(nodeinst, 0, NOWINDOWPART);
 		if (i != 1) continue;
 		shapenodepoly(nodeinst, 0, poly);
 		if (poly->layer < 0) continue;
@@ -596,7 +604,7 @@ INTSML io_initfind(void)
 			nodeinst->highx = np->highx;
 			nodeinst->lowy  = np->lowy;
 			nodeinst->highy = np->highy;
-			(void)nodepolys(nodeinst, 0);
+			(void)nodepolys(nodeinst, 0, NOWINDOWPART);
 			shapenodepoly(nodeinst, 0, poly);
 			ch2 = ((char **)io_cifnames->addr)[poly->layer];
 			if (strcmp(ch, ch2) != 0) continue;
@@ -614,7 +622,7 @@ INTSML io_initfind(void)
 			nodeinst->highx = np->highx;
 			nodeinst->lowy  = np->lowy;
 			nodeinst->highy = np->highy;
-			j = nodepolys(nodeinst, 0);
+			j = nodepolys(nodeinst, 0, NOWINDOWPART);
 			for(k=0; k<j; k++)
 			{
 				shapenodepoly(nodeinst, 0, poly);
@@ -646,6 +654,7 @@ INTBIG io_findlayernum(char *name)
 	{
 		newlimit = io_cifnotfoundlimit * 2;
 		if (newlimit <= 0) newlimit = 10;
+		if (newlimit < io_cifnotfoundcount) newlimit = io_cifnotfoundcount;
 		newlist = (char **)emalloc(newlimit * (sizeof (char *)), io_aid->cluster);
 		if (newlist == 0) return(-1);
 		for(i=0; i<io_cifnotfoundcount; i++)
@@ -665,18 +674,47 @@ NODEPROTO *io_findprotonode(INTSML num)
 	return(io_cifnodes[num]);
 }
 
-CIFCELL *io_findcifcell(INTSML cindex)
+CIFCELL *io_findcifcell(INTBIG cindex)
 {
 	CIFCELL *cifcell;
+	REGISTER INTBIG i, j;
 
-	for(cifcell = io_cifcells; cifcell->cindex != cindex; cifcell = cifcell->nextcell)
-		if (cifcell->nextcell == NOCIFCELL) return(NOCIFCELL);
-	return(cifcell);
+	i = cindex % io_cifcells_size;
+	for(j=1; j<=io_cifcells_size; j += 2)
+	{
+		cifcell = io_cifcells[i];
+		if (cifcell == NOCIFCELL) break;
+		if (cindex == cifcell->cindex) return(cifcell);
+		i += j;
+		if (i >= io_cifcells_size) i -= io_cifcells_size;
+	}
+	return(NOCIFCELL);
 }
 
-CIFCELL *io_newcifcell(void)
+void io_insertcifcell(CIFCELL *cifcell)
 {
-	CIFCELL *newcc, *endcc;
+	REGISTER INTBIG i, j;
+
+	i = cifcell->cindex % io_cifcells_size;
+	for(j=1; j<=io_cifcells_size; j += 2)
+	{
+		if (io_cifcells[i] == NOCIFCELL)
+		{
+			io_cifcells[i] = cifcell;
+			break;
+		}
+		i += j;
+		if (i >= io_cifcells_size) i -= io_cifcells_size;
+	}
+}
+
+CIFCELL *io_newcifcell(INTBIG cindex)
+{
+	CIFCELL *newcc, *cc;
+	CIFCELL **old_cifcells;
+	INTBIG old_cifcells_size;
+	INTBIG new_cifcells_size;
+	INTBIG i;
 
 	newcc = (CIFCELL *)emalloc((sizeof (CIFCELL)), io_aid->cluster);
 	if (newcc == (CIFCELL *) 0)
@@ -684,29 +722,50 @@ CIFCELL *io_newcifcell(void)
 		ttyputerr(_("Not enough memory allocated for CIFCELL"));
 		return(NOCIFCELL);
 	}
-	newcc->nextcell = NOCIFCELL;
-	if (io_cifcells == NOCIFCELL) io_cifcells = newcc; else
+	io_cifcells_num++;
+	if (io_cifcells_num >= io_cifcells_size/2)
 	{
-		endcc = io_cifcells;
-		while(endcc->nextcell != NOCIFCELL) endcc = endcc->nextcell;
-		endcc->nextcell = newcc;
+		old_cifcells = io_cifcells;
+		old_cifcells_size = io_cifcells_size;
+		new_cifcells_size = pickprime(io_cifcells_num * 4);
+		io_cifcells = (CIFCELL **)emalloc(new_cifcells_size * (sizeof (CIFCELL *)), io_aid->cluster);
+		if (io_cifcells == (CIFCELL **) 0)
+		{
+			ttyputerr(_("Not enough memory allocated for CIFCELL"));
+			return(NOCIFCELL);
+		}
+		for (i = 0; i < new_cifcells_size; i++) io_cifcells[i] = NOCIFCELL;
+		io_cifcells_size = new_cifcells_size;
+		for (i = 0; i < old_cifcells_size; i++)
+		{
+			cc = old_cifcells[i];
+			if (cc != NOCIFCELL) io_insertcifcell(cc);	
+		}
+		if (old_cifcells_size > 0) efree((char *) old_cifcells);
 	}
-	io_curcell = newcc;
 	newcc->addr = NONODEPROTO;
-	newcc->cindex = 0;
+	newcc->cindex = cindex;
+	io_insertcifcell(newcc);
+	
+	io_curcell = newcc;
 	return(newcc);
 }
 
 void io_ciffreecifcells(void)
 {
 	CIFCELL *cc;
+	INTBIG i;
 
-	while (io_cifcells != NOCIFCELL)
+	for (i = 0; i < io_cifcells_size; i++)
 	{
-		cc = io_cifcells;
-		io_cifcells = io_cifcells->nextcell;
-		efree((char *)cc);
+		cc = io_cifcells[i];
+		if (cc != NOCIFCELL)
+			efree((char *)cc);
 	}
+	if (io_cifcells_size > 0) efree((char *)io_cifcells);
+	io_cifcells_size = 0;
+	io_cifcells_num = 0;
+	io_cifcells = (CIFCELL **)NOSTRING;
 }
 
 CIFLIST *io_newciflist(INTSML id)
@@ -981,7 +1040,7 @@ void io_outputlabel(char *name, point pt)
 	cl->x = pt.x;   cl->y = pt.y;
 }
 
-void io_outputcall(INTSML number, char *name, char *list)
+void io_outputcall(INTBIG number, char *name, char *list)
 {
 	CCALL *cc;
 	tentry temp;
@@ -1016,7 +1075,7 @@ void io_outputcall(INTSML number, char *name, char *list)
 	}
 }
 
-void io_outputds(INTSML number, char *name, INTBIG l, INTBIG r, INTBIG b, INTBIG t)
+void io_outputds(INTBIG number, char *name, INTBIG l, INTBIG r, INTBIG b, INTBIG t)
 {
 	CFSTART *cs;
 
