@@ -40,7 +40,7 @@
 
 #define FULLPOLYMERGE 1		/* uncomment to use older manhattan merging */
 
-#define	NMCHR	80
+#define NMCHR	80
 
 static INTBIG     io_prev_chr_sep, io_nchar;	/* for CRC checksum */
 static UINTBIG    io_check_sum;					/* for CRC checksum */
@@ -52,6 +52,7 @@ static INTBIG     io_cifoutoffx;				/* offset for current facet - for highlight
 static INTBIG     io_cifoutoffy;				/* offset for current facet - for highlight */
 static NODEPROTO *io_cifoutcurnp;				/* node proto of facet being written */
 static INTSML     io_zero_offset;				/* Set if there is no centering */
+static INTSML     io_cif_highlightreserr;		/* Set to highlight resolution errors */
 static INTSML     io_reserr_count;				/* count the resolution errors */
 static char       io_cifoutcurlayer[200];		/* current layer name written out */
 static INTBIG     io_cifoutfacetnumber;			/* count of facets being written out */
@@ -83,6 +84,7 @@ void io_freelayerreq(LAYERREQ *req);
 void io_includeciflayer(TECHNOLOGY *tech, INTBIG layer);
 void io_writeciflayer(char *layername);
 void io_cifoptionsdlog(void);
+INTBIG io_ciffindhighestlayer(NODEPROTO *np, INTBIG x, INTBIG y);
 
 /*
  * Routine to free all memory associated with this module.
@@ -150,18 +152,21 @@ INTSML io_writeciflibrary(LIBRARY *lib)
 	/* get form of output */
 	if ((io_cifoutputstate[0]&CIFOUTNOTCEN) != 0) io_zero_offset = 1; else
 		io_zero_offset = 0;
+	if ((io_cifoutputstate[1]&CIFRESHIGH) != 0) io_cif_highlightreserr = 1; else
+		io_cif_highlightreserr = 0;
 
 	/* get cif resolution */
 	var = getval((INTBIG)el_curtech, VTECHNOLOGY, VINTEGER, "IO_cif_resolution");
 	if (var == NOVARIABLE) io_cif_resolution = 0; else
 	{
 		io_cif_resolution = (INTSML)var->addr;
-		if (io_cif_resolution != 0) (void)askaid(us_aid, "clear");
+		if (io_cif_resolution != 0 && io_cif_highlightreserr != 0)
+			(void)askaid(us_aid, "clear");
 	}
 
 	/* get the number of allowable points on a polygon */
 	var = getval((INTBIG)el_curtech, VTECHNOLOGY, VINTEGER, "IO_cif_polypoints");
-	if (var == NOVARIABLE) io_cif_polypoints = HUGEINT; else
+	if (var == NOVARIABLE) io_cif_polypoints = MAXINTBIG; else
 		io_cif_polypoints = var->addr;
 
 	/* initialize cif merging facility if needed */
@@ -241,8 +246,15 @@ INTSML io_writeciflibrary(LIBRARY *lib)
 
 	if (io_cif_resolution != 0 && io_reserr_count != 0)
 	{
-		ttyputerr(_("WARNING: Highlighted %d resolution %s"), io_reserr_count,
-			makeplural(_("error"), io_reserr_count));
+		if (io_cif_highlightreserr != 0)
+		{
+			ttyputerr(_("WARNING: Highlighted %d resolution %s"), io_reserr_count,
+				makeplural(_("error"), io_reserr_count));
+		} else
+		{
+			ttyputerr(_("WARNING: Found %d resolution %s"), io_reserr_count,
+				makeplural(_("error"), io_reserr_count));
+		}
 	}
 
 	/* complete the checksum accumulation */
@@ -272,7 +284,7 @@ void io_cifwritefacet(NODEPROTO *np, INTSML fake)
 	REGISTER NODEPROTO *subnt, *bodysubnt, *onp;
 	REGISTER PORTPROTO *pp, *subpp;
 	REGISTER PORTARCINST *pi;
-	REGISTER INTBIG i, j, sorted, total, validport;
+	REGISTER INTBIG i, j, sorted, total, validport, toplayer, toplayerfound;
 	REGISTER char *rname, *nname;
 	REGISTER INTBIG offx, offy, fun;
 	REGISTER LAYERREQ *req, *lastreq, *nextreq;
@@ -349,7 +361,7 @@ void io_cifwritefacet(NODEPROTO *np, INTSML fake)
 			if ((subni->userbits&WIPED) != 0) continue;
 			subnt = subni->proto;
 			if (subnt->primindex == 0) continue;
-			i = nodepolys(subni, 0);
+			i = nodepolys(subni, 0, NOWINDOWPART);
 			for(j=0; j<i; j++)
 			{
 				shapenodepoly(subni, j, poly);
@@ -358,7 +370,7 @@ void io_cifwritefacet(NODEPROTO *np, INTSML fake)
 		}
 		for(ai = np->firstarcinst; ai != NOARCINST; ai = ai->nextarcinst)
 		{
-			i = arcpolys(ai);
+			i = arcpolys(ai, NOWINDOWPART);
 			for(j=0; j<i; j++)
 			{
 				shapearcpoly(ai, j, poly);
@@ -415,7 +427,7 @@ void io_cifwritefacet(NODEPROTO *np, INTSML fake)
 				makerot(subni, trans);
 
 				/* write a primitive nodeinst */
-				i = nodepolys(subni, 0);
+				i = nodepolys(subni, 0, NOWINDOWPART);
 				for(j=0; j<i; j++)
 				{
 					shapenodepoly(subni, j, poly);
@@ -429,7 +441,7 @@ void io_cifwritefacet(NODEPROTO *np, INTSML fake)
 			for(ai = np->firstarcinst; ai != NOARCINST; ai = ai->nextarcinst)
 			{
 				if (ai->proto->tech != req->tech) continue;
-				i = arcpolys(ai);
+				i = arcpolys(ai, NOWINDOWPART);
 				for(j=0; j<i; j++)
 				{
 					/* write the box describing the layer */
@@ -460,7 +472,7 @@ void io_cifwritefacet(NODEPROTO *np, INTSML fake)
 			makerot(subni, trans);
 
 			/* write a primitive nodeinst */
-			i = nodepolys(subni, 0);
+			i = nodepolys(subni, 0, NOWINDOWPART);
 			for(j=0; j<i; j++)
 			{
 				shapenodepoly(subni, j, poly);
@@ -472,7 +484,7 @@ void io_cifwritefacet(NODEPROTO *np, INTSML fake)
 		/* write all of the arcs in the facet */
 		for(ai = np->firstarcinst; ai != NOARCINST; ai = ai->nextarcinst)
 		{
-			i = arcpolys(ai);
+			i = arcpolys(ai, NOWINDOWPART);
 			for(j=0; j<i; j++)
 			{
 				/* write the box describing the layer */
@@ -610,9 +622,13 @@ void io_cifwritefacet(NODEPROTO *np, INTSML fake)
 			transmult(localtran, temp2, subrot);
 		}
 
+		/* get highest layer at this point */
+		toplayer = io_ciffindhighestlayer(np, xpos, ypos);
+		toplayerfound = 0;
+
 		/* find valid layers on this node that surround the port */
 		validport = 0;
-		total = nodepolys(subni, 0);
+		total = nodepolys(subni, 0, NOWINDOWPART);
 		for(i=0; i<total; i++)
 		{
 			shapenodepoly(subni, i, poly);
@@ -624,34 +640,55 @@ void io_cifwritefacet(NODEPROTO *np, INTSML fake)
 			if (isinside(xpos+1, ypos, poly) == 0) continue;
 			if (isinside(xpos, ypos-1, poly) == 0) continue;
 			if (isinside(xpos, ypos+1, poly) == 0) continue;
+			if (poly->layer == toplayer) toplayerfound = 1;
 			validport = 1;
 			break;
 		}
 		if (validport == 0)
 		{
 			/* look for connected arcs */
-			for(pi = pp->subnodeinst->firstportarcinst; pi != NOPORTARCINST; pi = pi->nextportarcinst)
+			subpp = pp;
+			transid(subrot);
+			for(;;)
 			{
-				ai = pi->conarcinst;
-				total = arcpolys(ai);
-				for(i=0; i<total; i++)
+				/* look for layers at this level of hierarchy */
+				for(pi = subpp->subnodeinst->firstportarcinst; pi != NOPORTARCINST; pi = pi->nextportarcinst)
 				{
-					shapearcpoly(ai, i, poly);
-					if (poly->layer < 0) continue;
-					fun = layerfunction(poly->tech, poly->layer);
-					if ((fun&LFPSEUDO) != 0) continue;
-					if (isinside(xpos-1, ypos, poly) == 0) continue;
-					if (isinside(xpos+1, ypos, poly) == 0) continue;
-					if (isinside(xpos, ypos-1, poly) == 0) continue;
-					if (isinside(xpos, ypos+1, poly) == 0) continue;
-					validport = 1;
-					break;
+					ai = pi->conarcinst;
+					total = arcpolys(ai, NOWINDOWPART);
+					for(i=0; i<total; i++)
+					{
+						shapearcpoly(ai, i, poly);
+						if (poly->layer < 0) continue;
+						fun = layerfunction(poly->tech, poly->layer);
+						if ((fun&LFPSEUDO) != 0) continue;
+						xformpoly(poly, subrot);
+						if (isinside(xpos-1, ypos, poly) == 0) continue;
+						if (isinside(xpos+1, ypos, poly) == 0) continue;
+						if (isinside(xpos, ypos-1, poly) == 0) continue;
+						if (isinside(xpos, ypos+1, poly) == 0) continue;
+						if (poly->layer == toplayer) toplayerfound = 1;
+						validport = 1;
+						break;
+					}
+					if (validport != 0) break;
 				}
 				if (validport != 0) break;
+
+				/* not found on this level: descend and look again */
+				if (subpp->subnodeinst->proto->primindex != 0) break;
+				maketrans(subpp->subnodeinst, localtran);
+				transmult(localtran, subrot, temp2);
+				makerot(subpp->subnodeinst, localtran);
+				transmult(localtran, temp2, subrot);
+				subpp = subpp->subportproto;
 			}
 		}
 		if (validport != 0)
 		{
+			if (toplayerfound == 0)
+				ttyputmsg(_("Warning: facet %s, export '%s' obscured by higher layer"),
+					describenodeproto(np), pp->protoname);
 			io_cifprintf("94 %s %ld %ld;\n", pp->protoname, io_cif_outscale(xpos-offx),
 				io_cif_outscale(ypos-offy));
 		}
@@ -660,6 +697,63 @@ void io_cifwritefacet(NODEPROTO *np, INTSML fake)
 	io_cifprintf("DF;\n");
 }
 
+INTBIG io_ciffindhighestlayer(NODEPROTO *np, INTBIG x, INTBIG y)
+{
+	REGISTER INTBIG sea, i, tot, first, bestheight, bestlayer;
+	INTBIG height, thickness;
+	REGISTER GEOM *geom;
+	REGISTER NODEINST *ni;
+	REGISTER ARCINST *ai;
+	static POLYGON *poly = NOPOLYGON;
+
+	if (poly == NOPOLYGON) poly = allocstaticpolygon(4, io_aid->cluster);
+	sea = initsearch(x, x, y, y, np);
+	first = 1;
+	for(;;)
+	{
+		geom = nextobject(sea);
+		if (geom == NOGEOM) break;
+		if (geom->entrytype == OBJNODEINST)
+		{
+			ni = geom->entryaddr.ni;
+			if (ni->proto->primindex == 0) continue;
+			tot = nodepolys(ni, 0, NOWINDOWPART);
+			for(i=0; i<tot; i++)
+			{
+				shapenodepoly(ni, i, poly);
+				if (isinside(x, y, poly) == 0) continue;
+				if (get3dfactors(poly->tech, poly->layer, &height, &thickness) != 0)
+					continue;
+				if (first != 0 || height > bestheight)
+				{
+					bestheight = height;
+					bestlayer = poly->layer;
+					first = 0;
+				}
+			}
+		} else
+		{
+			ai = geom->entryaddr.ai;
+			tot = arcpolys(ai, NOWINDOWPART);
+			for(i=0; i<tot; i++)
+			{
+				shapearcpoly(ai, i, poly);
+				if (isinside(x, y, poly) == 0) continue;
+				if (get3dfactors(poly->tech, poly->layer, &height, &thickness) != 0)
+					continue;
+				if (first != 0 || height > bestheight)
+				{
+					bestheight = height;
+					bestlayer = poly->layer;
+					first = 0;
+				}
+			}
+		}
+	}
+	if (first != 0) return(0);
+	return(bestlayer);
+}
+
 /*
  * Routine to dump the layer name (if it has changed).
  */
@@ -804,8 +898,7 @@ void io_outputcifpoly(TECHNOLOGY *tech, POLYGON *poly, INTBIG rx, INTBIG ry,
 			return;
 	}
 
-#if 0
-	/* certain polygons without valid CIF layers can still be written */
+#if 0		/* code to handle certain polygons without valid CIF layers */
 	switch (poly->style)
 	{
 		case CLOSEDRECT:
@@ -892,14 +985,15 @@ void io_cifreserror(GEOM *pos, INTBIG length, INTBIG width, INTBIG xc, INTBIG yc
 	bot = yc - (width/2);
 
 	/* if there is a geometry module */
-	if (pos != NOGEOM && io_cif_resolution != 0)
+	if (io_cif_resolution != 0 && pos != NOGEOM)
 	{
 		/* see if it resolves to grid */
 		if ((left % io_cif_resolution) != 0 || (right % io_cif_resolution) != 0 ||
 			(top % io_cif_resolution) != 0 || (bot % io_cif_resolution) != 0)
 		{
 			/* highlight the object */
-			(void)askaid(us_aid, "show-object", (INTBIG)pos);
+			if (io_cif_highlightreserr != 0)
+				(void)askaid(us_aid, "show-object", (INTBIG)pos);
 			io_reserr_count++;
 		}
 	}
@@ -954,21 +1048,6 @@ void io_cif_write_polygon(INTSML layer, TECHNOLOGY *tech, INTBIG *xbuf, INTBIG *
 
 	for(i=0; i<count; i++)
 	{
-#if 0
-		/* skip points that are colinear */
-		if (i > 0 && i < count-1)
-		{
-			float w, p;
-			w = (float)(xbuf[i+1] - xbuf[i-1]) / (float)(ybuf[i+1] - ybuf[i-1]);
-			p = (float)(xbuf[i] - xbuf[i-1]) / (float)(ybuf[i] - ybuf[i-1]);
-			if (w == p)
-			{
-				ttyputmsg("Collinear points that pass through (%ld,%ld)",
-					io_cif_outscale(xbuf[i]), io_cif_outscale(ybuf[i]));
-			}
-		}
-#endif
-
 		(void)sprintf(prbuf, " %ld %ld", io_cif_outscale(xbuf[i]),
 			io_cif_outscale(ybuf[i]));
 		prlen = strlen(prbuf); /* get length of coord. pair */
@@ -1009,6 +1088,8 @@ void io_cif_show_reserror(INTBIG x1, INTBIG y1, INTBIG x2, INTBIG y2)
 {
 	REGISTER NODEPROTO *np;
 
+	if (io_cif_highlightreserr == 0) return;
+
 	/* show resolution error if it is in the current window */
 	np = getcurfacet();
 	if (np == io_cifoutcurnp)
@@ -1069,41 +1150,49 @@ void io_freelayerreq(LAYERREQ *req)
 /* CIF Options */
 DIALOGITEM io_cifoptionsdialogitems[] =
 {
- /*  1 */ {0, {168,380,192,452}, BUTTON, N_("OK")},
- /*  2 */ {0, {168,240,192,312}, BUTTON, N_("Cancel")},
- /*  3 */ {0, {8,8,192,223}, SCROLL, ""},
+ /*  1 */ {0, {180,380,204,452}, BUTTON, N_("OK")},
+ /*  2 */ {0, {180,240,204,312}, BUTTON, N_("Cancel")},
+ /*  3 */ {0, {8,8,204,223}, SCROLL, ""},
  /*  4 */ {0, {8,232,24,312}, MESSAGE, N_("CIF Layer:")},
  /*  5 */ {0, {8,316,24,454}, EDITTEXT, ""},
  /*  6 */ {0, {32,232,48,454}, CHECK, N_("Output Mimics Display")},
  /*  7 */ {0, {56,232,72,454}, CHECK, N_("Output Merges Boxes")},
  /*  8 */ {0, {104,232,120,454}, CHECK, N_("Input Squares Wires")},
  /*  9 */ {0, {80,232,96,454}, CHECK, N_("Output Instantiates Top Level")},
- /* 10 */ {0, {128,232,144,376}, MESSAGE, N_("Output resolution:")},
- /* 11 */ {0, {128,380,144,454}, EDITTEXT, ""}
+ /* 10 */ {0, {148,240,164,384}, MESSAGE, N_("Output resolution:")},
+ /* 11 */ {0, {148,388,164,454}, EDITTEXT, ""},
+ /* 12 */ {0, {128,232,144,436}, POPUP, ""}
 };
-DIALOG io_cifoptionsdialog = {{50,75,251,538}, N_("CIF Options"), 0, 11, io_cifoptionsdialogitems};
+DIALOG io_cifoptionsdialog = {{50,75,263,538}, N_("CIF Options"), 0, 12, io_cifoptionsdialogitems};
+
+/* special items for the "CIF Options" dialog: */
+#define DCFO_LAYERLIST         3		/* Layer list (scroll list) */
+#define DCFO_NEWLAYER          5		/* New layer (edit text) */
+#define DCFO_OMIMICDISPLAY     6		/* Output Mimics Display (check) */
+#define DCFO_OMERGEBOXES       7		/* Output Merges Boxes (check) */
+#define DCFO_ISQUAREWIRES      8		/* Input Squares Wires (check) */
+#define DCFO_OINSTANTIATETOP   9		/* Output Instantiates top (check) */
+#define DCFO_ORESOLUTION_L    10		/* Output Resolution label (stat text) */
+#define DCFO_ORESOLUTION      11		/* Output resolution (edit text) */
+#define DCFO_ORESOLUTIONMODE  12		/* Output resolution mode (popup) */
 
-/*
- * special case for the "CIF Options" dialog
- * Layer list              = 3 (scroll list)
- * New layer               = 5 (edit text)
- * Output Mimics Display   = 6 (check)
- * Output Merges Boxes     = 7 (check)
- * Input Squares Wires     = 8 (check)
- * Output Instantiates top = 9 (check)
- * Output resolution       = 11 (edit text)
- */
 void io_cifoptionsdlog(void)
 {
 	INTBIG newstate[NUMIOSTATEBITWORDS];
 	REGISTER INTBIG *curstate, oldresolution, i, itemHit, nameschanged;
 	REGISTER char **layernames, *pt;
+	char *newlang[3];
 	REGISTER VARIABLE *var, *cifvar;
+	static char *whattodisplay[] = {N_("No Resolution Check"),
+		N_("Report Resolution Errors"), N_("Show Resolution Errors")};
 
 	if (DiaInitDialog(&io_cifoptionsdialog) != 0) return;
-	DiaInitTextDialog(3, DiaNullDlogList, DiaNullDlogItem, DiaNullDlogDone, 0,
-		SCSELMOUSE|SCREPORT);
-	layernames = (char **)emalloc(el_curtech->layercount * (sizeof (char *)), el_tempcluster);
+	for(i=0; i<3; i++) newlang[i] = _(whattodisplay[i]);
+	DiaSetPopup(DCFO_ORESOLUTIONMODE, 3, newlang);
+	DiaInitTextDialog(DCFO_LAYERLIST, DiaNullDlogList, DiaNullDlogItem,
+		DiaNullDlogDone, 0, SCSELMOUSE|SCREPORT);
+	layernames = (char **)emalloc(el_curtech->layercount * (sizeof (char *)),
+		el_tempcluster);
 	cifvar = getval((INTBIG)el_curtech, VTECHNOLOGY, VSTRING|VISARRAY,
 		"IO_cif_layer_names");
 	for(i=0; i<el_curtech->layercount; i++)
@@ -1112,75 +1201,120 @@ void io_cifoptionsdlog(void)
 			pt = ((char **)cifvar->addr)[i];
 		(void)allocstring(&layernames[i], pt, el_tempcluster);
 		(void)initinfstr();
-		(void)addstringtoinfstr(layername(el_curtech, (INTSML)i));
+		(void)addstringtoinfstr(layername(el_curtech, i));
 		(void)addstringtoinfstr(" (");
 		(void)addstringtoinfstr(pt);
 		(void)addstringtoinfstr(")");
-		DiaStuffLine(3, returninfstr());
+		DiaStuffLine(DCFO_LAYERLIST, returninfstr());
 	}
-	DiaSelectLine(3, 0);
-	DiaSetText(5, layernames[0]);
+	DiaSelectLine(DCFO_LAYERLIST, 0);
+	DiaSetText(DCFO_NEWLAYER, layernames[0]);
 	curstate = io_getstatebits();
 	for(i=0; i<NUMIOSTATEBITWORDS; i++) newstate[i] = curstate[i];
-	if ((curstate[0]&CIFOUTEXACT) != 0) DiaSetControl(6, 1);
-	if ((curstate[0]&CIFOUTMERGE) != 0) DiaSetControl(7, 1);
-	if ((curstate[0]&CIFINSQUARE) != 0) DiaSetControl(8, 1);
-	if ((curstate[0]&CIFOUTNOTOPCALL) == 0) DiaSetControl(9, 1);
+	if ((curstate[0]&CIFOUTEXACT) != 0) DiaSetControl(DCFO_OMIMICDISPLAY, 1);
+	if ((curstate[0]&CIFOUTMERGE) != 0) DiaSetControl(DCFO_OMERGEBOXES, 1);
+	if ((curstate[0]&CIFINSQUARE) != 0) DiaSetControl(DCFO_ISQUAREWIRES, 1);
+	if ((curstate[0]&CIFOUTNOTOPCALL) == 0) DiaSetControl(DCFO_OINSTANTIATETOP, 1);
 	var = getval((INTBIG)el_curtech, VTECHNOLOGY, VINTEGER, "IO_cif_resolution");
 	if (var == NOVARIABLE) oldresolution = 0; else oldresolution = var->addr;
-	DiaSetText(11, latoa(oldresolution));
+	if (oldresolution == 0)
+	{
+		DiaSetText(DCFO_ORESOLUTION, "");
+		DiaSetPopupEntry(DCFO_ORESOLUTIONMODE, 0);
+		DiaDimItem(DCFO_ORESOLUTION_L);
+		DiaDimItem(DCFO_ORESOLUTION);
+	} else
+	{
+		DiaUnDimItem(DCFO_ORESOLUTION_L);
+		DiaUnDimItem(DCFO_ORESOLUTION);
+		DiaSetText(DCFO_ORESOLUTION, latoa(oldresolution));
+		if ((curstate[1]&CIFRESHIGH) != 0)
+			DiaSetPopupEntry(DCFO_ORESOLUTIONMODE, 2); else
+				DiaSetPopupEntry(DCFO_ORESOLUTIONMODE, 1);
+	}
 
 	/* loop until done */
 	nameschanged = 0;
 	for(;;)
 	{
 		itemHit = DiaNextHit();
-		if (itemHit == OK || itemHit == CANCEL) break;
-		if (itemHit >= 6 && itemHit <= 9)
+		if (itemHit == CANCEL) break;
+		if (itemHit == OK)
+		{
+			i = DiaGetPopupEntry(DCFO_ORESOLUTIONMODE);
+			if (i != 0 && atola(DiaGetText(DCFO_ORESOLUTION)) == 0)
+			{
+				DiaMessageInDialog(_("Must set nonzero output resolution"));
+				continue;
+			}
+			break;
+		}
+		if (itemHit == DCFO_OMIMICDISPLAY || itemHit == DCFO_OMERGEBOXES ||
+			itemHit == DCFO_ISQUAREWIRES || itemHit == DCFO_OINSTANTIATETOP)
 		{
 			DiaSetControl(itemHit, 1 - DiaGetControl(itemHit));
 			continue;
 		}
-		if (itemHit == 3)
+		if (itemHit == DCFO_LAYERLIST)
 		{
-			i = DiaGetCurLine(3);
-			DiaSetText(5, layernames[i]);
+			i = DiaGetCurLine(DCFO_LAYERLIST);
+			DiaSetText(DCFO_NEWLAYER, layernames[i]);
 			continue;
 		}
-		if (itemHit == 5)
+		if (itemHit == DCFO_ORESOLUTIONMODE)
 		{
-			i = DiaGetCurLine(3);
-			pt = DiaGetText(5);
+			i = DiaGetPopupEntry(DCFO_ORESOLUTIONMODE);
+			if (i == 0)
+			{
+				DiaDimItem(DCFO_ORESOLUTION_L);
+				DiaDimItem(DCFO_ORESOLUTION);
+				DiaSetText(DCFO_ORESOLUTION, "");
+			} else
+			{
+				DiaUnDimItem(DCFO_ORESOLUTION_L);
+				DiaUnDimItem(DCFO_ORESOLUTION);
+				if (atola(DiaGetText(DCFO_ORESOLUTION)) == 0)
+					DiaSetText(DCFO_ORESOLUTION, "1");
+			}
+			DiaSetText(DCFO_NEWLAYER, layernames[i]);
+			continue;
+		}
+		if (itemHit == DCFO_NEWLAYER)
+		{
+			i = DiaGetCurLine(DCFO_LAYERLIST);
+			pt = DiaGetText(DCFO_NEWLAYER);
 			if (strcmp(pt, layernames[i]) == 0) continue;
 			nameschanged++;
 			(void)reallocstring(&layernames[i], pt, el_tempcluster);
 			(void)initinfstr();
-			(void)addstringtoinfstr(layername(el_curtech, (INTSML)i));
+			(void)addstringtoinfstr(layername(el_curtech, i));
 			(void)addstringtoinfstr(" (");
 			(void)addstringtoinfstr(layernames[i]);
 			(void)addstringtoinfstr(")");
-			DiaSetScrollLine(3, i, returninfstr());
+			DiaSetScrollLine(DCFO_LAYERLIST, i, returninfstr());
 			continue;
 		}
 	}
 
 	if (itemHit != CANCEL)
 	{
-		if (DiaGetControl(6) != 0) newstate[0] |= CIFOUTEXACT; else
+		if (DiaGetControl(DCFO_OMIMICDISPLAY) != 0) newstate[0] |= CIFOUTEXACT; else
 			newstate[0] &= ~CIFOUTEXACT;
-		if (DiaGetControl(7) != 0) newstate[0] |= CIFOUTMERGE; else
+		if (DiaGetControl(DCFO_OMERGEBOXES) != 0) newstate[0] |= CIFOUTMERGE; else
 			newstate[0] &= ~CIFOUTMERGE;
-		if (DiaGetControl(8) != 0) newstate[0] |= CIFINSQUARE; else
+		if (DiaGetControl(DCFO_ISQUAREWIRES) != 0) newstate[0] |= CIFINSQUARE; else
 			newstate[0] &= ~CIFINSQUARE;
-		if (DiaGetControl(9) == 0) newstate[0] |= CIFOUTNOTOPCALL; else
+		if (DiaGetControl(DCFO_OINSTANTIATETOP) == 0) newstate[0] |= CIFOUTNOTOPCALL; else
 			newstate[0] &= ~CIFOUTNOTOPCALL;
+		if (DiaGetPopupEntry(DCFO_ORESOLUTIONMODE) == 2) newstate[1] |= CIFRESHIGH; else
+			newstate[1] &= ~CIFRESHIGH;
 		for(i=0; i<NUMIOSTATEBITWORDS; i++) if (curstate[i] != newstate[i]) break;
 		if (i < NUMIOSTATEBITWORDS) io_setstatebits(newstate);
 		if (nameschanged != 0)
 			setval((INTBIG)el_curtech, VTECHNOLOGY, "IO_cif_layer_names",
 				(INTBIG)layernames, VSTRING|VISARRAY|
 					(el_curtech->layercount<<VLENGTHSH));
-		i = atola(DiaGetText(11));
+		i = atola(DiaGetText(DCFO_ORESOLUTION));
 		if (i != oldresolution)
 			setval((INTBIG)el_curtech, VTECHNOLOGY, "IO_cif_resolution", i, VINTEGER);
 	}
