@@ -1641,7 +1641,7 @@ static void io_shipcontents(stentry *sym)
 	struct kludge *ro;
 
 	ro = (struct kludge *) sym->st_guts;
-	io_outputds((INTSML)(sym->st_symnumber), sym->st_name,
+	io_outputds(sym->st_symnumber, sym->st_name,
 		sym->st_bb.l, sym->st_bb.r, sym->st_bb.b, sym->st_bb.t);
 	while (ro != NIL)
 	{
@@ -1687,7 +1687,7 @@ static void io_shipcontents(stentry *sym)
 			case CALL:
 				t_list = io_maketlist();
 				io_dupetlist((tlist)((symcall *)ro)->sy_tlist, (tlist)t_list);
-				io_outputcall((INTSML)(((symcall *)ro)->sy_symnumber),
+				io_outputcall(((symcall *)ro)->sy_symnumber,
 					((symcall *)ro)->sy_unid->st_name, t_list);
 				io_freetlist((tlist)t_list);
 				break;
@@ -1840,7 +1840,7 @@ static void io_outitem(itementry *thing)		/* spit out an item */
 			t_list = io_maketlist();
 			io_dupetlist((tlist)((symcall *) (thing->it_what))->sy_tlist, (tlist)t_list);
 			io_dumpdef(((symcall *) (thing->it_what))->sy_unid);
-			io_outputcall((INTSML)(((symcall *) (thing->it_what))->sy_symnumber),
+			io_outputcall(((symcall *) (thing->it_what))->sy_symnumber,
 				((symcall *) (thing->it_what))->sy_unid->st_name, t_list);
 			io_freetlist((tlist)t_list);
 			(void)io_swapcontext(thing->it_context);
