@@ -383,6 +383,7 @@ INTSML io_defreadcomponent(FILE *f, NODEPROTO *facet)
 	REGISTER INTBIG lx, hx, ly, hy;
 	REGISTER NODEINST *ni;
 	REGISTER NODEPROTO *np;
+	REGISTER VARIABLE *var;
 	INTSML rot, trans;
 	char compname[200], modelname[200];
 
@@ -429,8 +430,11 @@ INTSML io_defreadcomponent(FILE *f, NODEPROTO *facet)
 					return(1);
 				}
 				endobjectchange((INTBIG)ni, VNODEINST);
-				(void)setvalkey((INTBIG)ni, VNODEINST, el_node_name,
+				var = setvalkey((INTBIG)ni, VNODEINST, el_node_name,
 					(INTBIG)compname, VSTRING|VDISPLAY);
+				if (var != NOVARIABLE)
+					var->textdescript = (var->textdescript & ~VTSIZE) |
+						(defaulttextsize(3) << VTSIZESH);
 				continue;
 			}
 			continue;
@@ -1200,7 +1204,7 @@ NODEPROTO *io_defgetnodeproto(char *name, LIBRARY *curlib)
 		if (np != NONODEPROTO)
 		{
 			/* must copy the facet */
-			newnp = us_copyrecursively(np, np->cell->cellname, curlib, np->cellview, 0, 0, "");
+			newnp = us_copyrecursively(np, np->cell->cellname, curlib, np->cellview, 0, 0, "", 0);
 			return(newnp);
 		}
 	}
@@ -1265,7 +1269,7 @@ void io_defgetlayernodes(char *name, NODEPROTO **pin, NODEPROTO **pure, ARCPROTO
 			node->proto = np;
 			node->lowx = 0;   node->highx = 1000;
 			node->lowy = 0;   node->highy = 1000;
-			j = nodepolys(node, 0);
+			j = nodepolys(node, 0, NOWINDOWPART);
 			for(i=0; i<j; i++)
 			{
 				shapenodepoly(node, i, poly);
@@ -1458,11 +1462,10 @@ DIALOGITEM io_defoptionsdialogitems[] =
 };
 DIALOG io_defoptionsdialog = {{50,75,147,314}, N_("DEF Options"), 0, 4, io_defoptionsdialogitems};
 
-/*
- * special case for the "DEF Options" dialog
- * Place logical interconnect  = 3 (check)
- * Place physical interconnect = 4 (check)
- */
+/* special items for the "DEF Options" dialog: */
+#define DDFO_PLACELOGICAL    3	/* Place logical interconnect (check) */
+#define DDFO_PLACEPHYSICAL   4	/* Place physical interconnect (check) */
+
 void io_defoptionsdlog(void)
 {
 	REGISTER INTBIG itemHit, *curstate, i;
@@ -1471,15 +1474,15 @@ void io_defoptionsdlog(void)
 	if (DiaInitDialog(&io_defoptionsdialog) != 0) return;
 	curstate = io_getstatebits();
 	for(i=0; i<NUMIOSTATEBITWORDS; i++) newstate[i] = curstate[i];
-	if ((curstate[0]&DEFNOLOGICAL) == 0) DiaSetControl(3, 1);
-	if ((curstate[0]&DEFNOPHYSICAL) == 0) DiaSetControl(4, 1);
+	if ((curstate[0]&DEFNOLOGICAL) == 0) DiaSetControl(DDFO_PLACELOGICAL, 1);
+	if ((curstate[0]&DEFNOPHYSICAL) == 0) DiaSetControl(DDFO_PLACEPHYSICAL, 1);
 
 	/* loop until done */
 	for(;;)
 	{
 		itemHit = DiaNextHit();
 		if (itemHit == OK || itemHit == CANCEL) break;
-		if (itemHit == 3 || itemHit == 4)
+		if (itemHit == DDFO_PLACELOGICAL || itemHit == DDFO_PLACEPHYSICAL)
 		{
 			DiaSetControl(itemHit, 1 - DiaGetControl(itemHit));
 			continue;
@@ -1488,9 +1491,9 @@ void io_defoptionsdlog(void)
 
 	if (itemHit != CANCEL)
 	{
-		if (DiaGetControl(3) == 0) newstate[0] |= DEFNOLOGICAL; else
+		if (DiaGetControl(DDFO_PLACELOGICAL) == 0) newstate[0] |= DEFNOLOGICAL; else
 			newstate[0] &= ~DEFNOLOGICAL;
-		if (DiaGetControl(4) == 0) newstate[0] |= DEFNOPHYSICAL; else
+		if (DiaGetControl(DDFO_PLACEPHYSICAL) == 0) newstate[0] |= DEFNOPHYSICAL; else
 			newstate[0] &= ~DEFNOPHYSICAL;
 		for(i=0; i<NUMIOSTATEBITWORDS; i++) if (curstate[i] != newstate[i]) break;
 		if (i < NUMIOSTATEBITWORDS) io_setstatebits(newstate);
