@@ -43,7 +43,7 @@
 
 /* #define PRESERVEDXF     1 */		/* uncomment to save original DXF with nodes */
 
-#define	MAXCHARSPERLINE	256		/* maximum line length in DXF file */
+#define MAXCHARSPERLINE	256		/* maximum line length in DXF file */
 
 typedef struct Idxflayer
 {
@@ -323,7 +323,7 @@ void io_dxfwritefacet(NODEPROTO *np, INTSML subfacets)
 				{
 					/* get polygon */
 					if (poly == NOPOLYGON) poly = allocstaticpolygon(4, io_aid->cluster);
-					i = nodepolys(ni, 0);
+					i = nodepolys(ni, 0, NOWINDOWPART);
 					for(j=0; j<i; j++)
 					{
 						shapenodepoly(ni, j, poly);
@@ -1671,7 +1671,7 @@ INTSML io_dxfreadtextentity(FILE *io, INTBIG *lx, INTBIG *hx, INTBIG *ly, INTBIG
 	{
 		*lx = mini(x, xalign);	*hx = *lx + abs(xalign-x) * 2;
 		*ly = y;		*hy = y + height;
-		*descript = VTPOSBOXED | (TXTLARGE << VTSIZESH);
+		*descript = VTPOSBOXED | (defaulttextsize(2) << VTSIZESH);
 	} else
 	{
 		if (*saveMsg == 0)
@@ -1684,7 +1684,7 @@ INTSML io_dxfreadtextentity(FILE *io, INTBIG *lx, INTBIG *hx, INTBIG *ly, INTBIG
 			*lx = x;	*hx = x + height*px/py;
 			*ly = y;	*hy = y + height;
 		}
-		*descript = VTPOSBOXED | (TXTLARGE << VTSIZESH);
+		*descript = VTPOSBOXED | (defaulttextsize(2) << VTSIZESH);
 	}
 	*msg = saveMsg;
 	return(0);
@@ -2192,12 +2192,11 @@ DIALOGITEM io_dxfoptionsdialogitems[] =
 };
 DIALOG io_dxfoptionsdialog = {{50,75,228,514}, N_("DXF Options"), 0, 6, io_dxfoptionsdialogitems};
 
-/*
- * special case for the "DXF Options" dialog
- * Layer list               = 3 (scroll list)
- * New layer                = 5 (edit text)
- * Input flattens hierarchy = 6 (check)
- */
+/* special items for the "DXF Options" dialog: */
+#define DDXO_LAYERLIST    3		/* Layer list (scroll) */
+#define DDXO_NEWLAYER     5		/* New layer (edit text) */
+#define DDXO_FLATTENHIER  6		/* Input flattens hierarchy (check) */
+
 void io_dxfoptionsdlog(void)
 {
 	REGISTER INTBIG i, itemHit, *curstate, nameschanged;
@@ -2206,9 +2205,10 @@ void io_dxfoptionsdlog(void)
 	REGISTER VARIABLE *dxfvar;
 
 	if (DiaInitDialog(&io_dxfoptionsdialog) != 0) return;
-	DiaInitTextDialog(3, DiaNullDlogList, DiaNullDlogItem, DiaNullDlogDone, 0,
-		SCSELMOUSE|SCREPORT);
-	layernames = (char **)emalloc(el_curtech->layercount * (sizeof (char *)), el_tempcluster);
+	DiaInitTextDialog(DDXO_LAYERLIST, DiaNullDlogList, DiaNullDlogItem,
+		DiaNullDlogDone, 0, SCSELMOUSE|SCREPORT);
+	layernames = (char **)emalloc(el_curtech->layercount * (sizeof (char *)),
+		el_tempcluster);
 	dxfvar = getval((INTBIG)el_curtech, VTECHNOLOGY, VSTRING|VISARRAY,
 		"IO_dxf_layer_names");
 	for(i=0; i<el_curtech->layercount; i++)
@@ -2217,17 +2217,17 @@ void io_dxfoptionsdlog(void)
 			pt = ((char **)dxfvar->addr)[i];
 		(void)allocstring(&layernames[i], pt, el_tempcluster);
 		(void)initinfstr();
-		(void)addstringtoinfstr(layername(el_curtech, (INTSML)i));
+		(void)addstringtoinfstr(layername(el_curtech, i));
 		(void)addstringtoinfstr(" (");
 		(void)addstringtoinfstr(pt);
 		(void)addstringtoinfstr(")");
-		DiaStuffLine(3, returninfstr());
+		DiaStuffLine(DDXO_LAYERLIST, returninfstr());
 	}
-	DiaSelectLine(3, 0);
-	DiaSetText(5, layernames[0]);
+	DiaSelectLine(DDXO_LAYERLIST, 0);
+	DiaSetText(DDXO_NEWLAYER, layernames[0]);
 	curstate = io_getstatebits();
 	for(i=0; i<NUMIOSTATEBITWORDS; i++) newstate[i] = curstate[i];
-	if ((curstate[0]&DXFFLATTENINPUT) != 0) DiaSetControl(6, 1);
+	if ((curstate[0]&DXFFLATTENINPUT) != 0) DiaSetControl(DDXO_FLATTENHIER, 1);
 
 	/* loop until done */
 	nameschanged = 0;
@@ -2235,28 +2235,28 @@ void io_dxfoptionsdlog(void)
 	{
 		itemHit = DiaNextHit();
 		if (itemHit == OK || itemHit == CANCEL) break;
-		if (itemHit == 3)
+		if (itemHit == DDXO_LAYERLIST)
 		{
-			i = DiaGetCurLine(3);
-			DiaSetText(5, layernames[i]);
+			i = DiaGetCurLine(DDXO_LAYERLIST);
+			DiaSetText(DDXO_NEWLAYER, layernames[i]);
 			continue;
 		}
-		if (itemHit == 5)
+		if (itemHit == DDXO_NEWLAYER)
 		{
-			i = DiaGetCurLine(3);
-			pt = DiaGetText(5);
+			i = DiaGetCurLine(DDXO_LAYERLIST);
+			pt = DiaGetText(DDXO_NEWLAYER);
 			if (strcmp(pt, layernames[i]) == 0) continue;
 			nameschanged++;
 			(void)reallocstring(&layernames[i], pt, el_tempcluster);
 			(void)initinfstr();
-			(void)addstringtoinfstr(layername(el_curtech, (INTSML)i));
+			(void)addstringtoinfstr(layername(el_curtech, i));
 			(void)addstringtoinfstr(" (");
 			(void)addstringtoinfstr(layernames[i]);
 			(void)addstringtoinfstr(")");
-			DiaSetScrollLine(3, i, returninfstr());
+			DiaSetScrollLine(DDXO_LAYERLIST, i, returninfstr());
 			continue;
 		}
-		if (itemHit == 6)
+		if (itemHit == DDXO_FLATTENHIER)
 		{
 			DiaSetControl(itemHit, 1 - DiaGetControl(itemHit));
 			continue;
@@ -2269,7 +2269,7 @@ void io_dxfoptionsdlog(void)
 			setval((INTBIG)el_curtech, VTECHNOLOGY, "IO_dxf_layer_names",
 				(INTBIG)layernames, VSTRING|VISARRAY|
 					(el_curtech->layercount<<VLENGTHSH));
-		if (DiaGetControl(6) != 0) newstate[0] |= DXFFLATTENINPUT; else
+		if (DiaGetControl(DDXO_FLATTENHIER) != 0) newstate[0] |= DXFFLATTENINPUT; else
 			newstate[0] &= ~DXFFLATTENINPUT;
 		for(i=0; i<NUMIOSTATEBITWORDS; i++) if (curstate[i] != newstate[i]) break;
 		if (i < NUMIOSTATEBITWORDS) io_setstatebits(newstate);
