@@ -184,7 +184,7 @@ INTSML io_writeeaglelibrary(LIBRARY *lib)
  */
 void io_eagledumpfacet(NODEPROTO *curfacet, char *prefix)
 {
-	REGISTER char *name;
+	char *name;
 	char line[50], *newprefix, *tempname;
 	REGISTER NODEPROTO *np, *subnp, *subcnp;
 	REGISTER NETWORK *net;
@@ -219,7 +219,7 @@ void io_eagledumpfacet(NODEPROTO *curfacet, char *prefix)
 		{
 			/* see if this node has a part name */
 			if (ni->proto->primindex != 0) continue;
-			varpart = getval((INTBIG)ni, VNODEINST, VSTRING, "ATTR_part");
+			varpart = getval((INTBIG)ni, VNODEINST, -1, "ATTR_part");
 			if (varpart == NOVARIABLE)
 			{
 				subnp = ni->proto;
@@ -267,20 +267,20 @@ void io_eagledumpfacet(NODEPROTO *curfacet, char *prefix)
 			} else
 			{
 				/* found package at bottom of hierarchy */
-				varpackagetype = getval((INTBIG)ni, VNODEINST, VSTRING, "ATTR_pkg_type");
+				(void)allocstring(&name, describevariable(varpart, -1, -1), el_tempcluster);
+				varpackagetype = getval((INTBIG)ni, VNODEINST, -1, "ATTR_pkg_type");
 				if (varpackagetype == NOVARIABLE)
 				{
 					xprintf(io_fileout, "ADD %s '%s%s'", ni->proto->cell->cellname,
-						prefix, (char *)varpart->addr);
+						prefix, name);
 				} else
 				{
-					xprintf(io_fileout, "ADD %s '%s%s'", (char *)varpackagetype->addr,
-						prefix, (char *)varpart->addr);
+					xprintf(io_fileout, "ADD %s '%s%s'", describevariable(varpackagetype, -1, -1),
+						prefix, name);
 				}
 				xprintf(io_fileout, " (0 0)\n");
 
 				/* record all networks */
-				name = (char *)varpart->addr;
 				for(pi = ni->firstportarcinst; pi != NOPORTARCINST; pi = pi->nextportarcinst)
 				{
 					/* get "Pad" information */
@@ -288,7 +288,7 @@ void io_eagledumpfacet(NODEPROTO *curfacet, char *prefix)
 					(void)addstringtoinfstr("ATTRP_");
 					(void)addstringtoinfstr(pi->proto->protoname);
 					(void)addstringtoinfstr("_pad");
-					var = getval((INTBIG)ni, VNODEINST, VSTRING, returninfstr());
+					var = getval((INTBIG)ni, VNODEINST, -1, returninfstr());
 					if (var == NOVARIABLE) continue;
 
 					/* create a new net connection */
@@ -314,10 +314,11 @@ void io_eagledumpfacet(NODEPROTO *curfacet, char *prefix)
 					(void)addstringtoinfstr(prefix);
 					(void)addstringtoinfstr(name);
 					(void)allocstring(&en->packagename, returninfstr(), io_aid->cluster);
-					(void)allocstring(&en->pinname, (char *)var->addr, io_aid->cluster);
+					(void)allocstring(&en->pinname, describevariable(var, -1, -1), io_aid->cluster);
 					en->nexteaglenet = io_eaglefirstnet;
 					io_eaglefirstnet = en;
 				}
+				efree((char *)name);
 			}
 		}
 	}
