@@ -89,15 +89,15 @@ static EO_STREAM_PTR edif_stream;
 
 /* prototypes */
 INTBIG io_edif_scale(INTBIG val);
-INTSML io_edifsearch(NODEPROTO *np);
+INTBIG io_edifsearch(NODEPROTO *np);
 void io_edifextsearch(NODEPROTO *np);
 char *io_ediffind_path(NODEPROTO *np);
 void io_edifwriteprim(NODEPROTO *np, INTSML i, INTSML fun);
 char *io_edif_orientation(NODEINST *ni);
-char *io_edifwritecompname(NODEINST *ni, INTSML fun, INTSML serindex);
+char *io_edifwritecompname(NODEINST *ni, INTBIG fun, INTSML serindex);
 void io_edif_pt(INTBIG x, INTBIG y);
-INTSML io_edifwritefacet(NODEPROTO *np, INTSML external);
-char *io_edifdescribepriminst(NODEINST *ni, INTSML fun);
+INTBIG io_edifwritefacet(NODEPROTO *np, INTSML external);
+char *io_edifdescribepriminst(NODEINST *ni, INTBIG fun);
 INTSML io_edifisglobal(PORTPROTO *pp);
 INTSML io_edifmarknetports(NODEPROTO *np);
 INTSML io_edifwritelibext(NODEPROTO *np);
@@ -213,7 +213,6 @@ INTSML io_writeediflibrary(LIBRARY *lib)
 	(void)sprintf(fname, "%s.edif", name);
 	if ((edif_stream = EO_open_stream(fname, 0)) == NOEO_STREAM)
 	{
-		/* ttyputerr(_("Cannot write %s"), fname); */
 		return(1);
 	}
 
@@ -351,13 +350,12 @@ INTSML io_writeediflibrary(LIBRARY *lib)
 /*
  * routine to count the usage of primitives hierarchically below facet "np"
  */
-INTSML io_edifsearch(NODEPROTO *np)
+INTBIG io_edifsearch(NODEPROTO *np)
 {
 	REGISTER NODEINST *ni;
 	REGISTER NODEPROTO *onp;
 	REGISTER PORTARCINST *pi;
-	REGISTER INTSML i, fun, primcount;
-	char *extra;
+	REGISTER INTBIG i, fun, primcount;
 
 	/* do not search this facet if it is an icon */
 	if (np->cellview == el_iconview) return(0);
@@ -369,7 +367,7 @@ INTSML io_edifsearch(NODEPROTO *np)
 	{
 		if (ni->proto->primindex != 0)
 		{
-			fun = nodefunction(ni, &extra);
+			fun = nodefunction(ni, 0);
 			i = 1;
 			if (fun == NPGATEAND || fun == NPGATEOR || fun == NPGATEXOR)
 			{
@@ -605,7 +603,7 @@ char *io_edif_orientation(NODEINST *ni)
 /*
  * Helper name builder
  */
-char *io_edifwritecompname(NODEINST *ni, INTSML fun, INTSML serindex)
+char *io_edifwritecompname(NODEINST *ni, INTBIG fun, INTSML serindex)
 {
 	REGISTER VARIABLE *var;
 	REGISTER char *okname;
@@ -658,10 +656,10 @@ void io_edif_pt(INTBIG x, INTBIG y)
  * 'external' if 'external' is non-zero.
  * Returns nonzero if back-annotation was added.
  */
-INTSML io_edifwritefacet(NODEPROTO *np, INTSML external)
+INTBIG io_edifwritefacet(NODEPROTO *np, INTSML external)
 {
 	INTBIG i, netcount, displaytotal;
-	INTSML fun, backannotate;
+	INTBIG fun, backannotate;
 	NETWORK *net, *cnet, *onet;
 	VARIABLE *var;
 	NODEINST *ni;
@@ -670,7 +668,7 @@ INTSML io_edifwritefacet(NODEPROTO *np, INTSML external)
 	PORTPROTO *pp, *cpp;
 	NODEPROTO *onp, *cnp;
 	ARCINST *ai;
-	char *pt, *extra, *iname, *oname, line[WORD+1], *direction;
+	char *pt, *iname, *oname, line[WORD+1], *direction;
 	INTSML globalport;
 	INTSML contents, schematic;
 	INTBIG netindex, pageindex, is_array, sx, mx, rx, sy, my, ry;
@@ -837,7 +835,7 @@ INTSML io_edifwritefacet(NODEPROTO *np, INTSML external)
 				}
 				for (ni = np->firstnodeinst; ni != NONODEINST; ni = ni->nextnodeinst)
 				{
-					fun = nodefunction(ni, &extra);
+					fun = nodefunction(ni, 0);
 					if (ni->proto->primindex != 0)
 					{
 						if (fun == NPUNKNOWN || fun == NPPIN || fun == NPCONTACT ||
@@ -953,10 +951,10 @@ INTSML io_edifwritefacet(NODEPROTO *np, INTSML external)
 					if (schematic_view)
 					{
 						/* do all display variables first */
-						displaytotal = tech_displayablenvars(ni);
+						displaytotal = tech_displayablenvars(ni, NOWINDOWPART);
 						for (i = 0; i < displaytotal; i++)
 						{
-							var = tech_filldisplayablenvar(ni, poly);
+							var = tech_filldisplayablenvar(ni, poly, NOWINDOWPART, 0);
 							xformpoly(poly, trans);
 							/* check for name */
 							if (namesame((pt = makename(var->key)), "EDIF_annotate"))
@@ -983,7 +981,7 @@ INTSML io_edifwritefacet(NODEPROTO *np, INTSML external)
 				{
 					if (ni->proto->primindex != 0)
 					{
-						fun = nodefunction(ni, &extra);
+						fun = nodefunction(ni, 0);
 						if (fun != NPUNKNOWN && fun != NPPIN && fun != NPCONTACT &&
 							fun != NPNODE && fun != NPCONNECT && fun != NPART) continue;
 					} else if (namesame(ni->proto->cell->cellname, "monitor_probe") == 0) continue;
@@ -1117,7 +1115,7 @@ INTSML io_edifwritefacet(NODEPROTO *np, INTSML external)
 							}
 							if (ni->proto->primindex == 0) break;
 
-							fun = nodefunction(ni, &extra);
+							fun = nodefunction(ni, 0);
 							if (fun != NPUNKNOWN && fun != NPPIN && fun != NPCONTACT &&
 								fun != NPNODE && fun != NPCONNECT && fun != NPART) break;
 						}
@@ -1174,7 +1172,7 @@ INTSML io_edifwritefacet(NODEPROTO *np, INTSML external)
 						if (ni->proto->primindex != 0)
 						{
 							/* ignore passive components */
-							fun = nodefunction(ni, &extra);
+							fun = nodefunction(ni, 0);
 							if (fun == NPUNKNOWN || fun == NPPIN || fun == NPCONTACT ||
 								fun == NPNODE || fun == NPCONNECT || fun == NPART) continue;
 							cnp = ni->proto;
@@ -1381,7 +1379,7 @@ INTSML io_edifwritefacet(NODEPROTO *np, INTSML external)
  * procedure to properly identify an instance of a primitive node
  * for ASPECT netlists
  */
-char *io_edifdescribepriminst(NODEINST *ni, INTSML fun)
+char *io_edifdescribepriminst(NODEINST *ni, INTBIG fun)
 {
 	REGISTER VARIABLE *var;
 	char *model;
@@ -1934,7 +1932,7 @@ void io_edifsymbol_nodeinst(NODEINST *ni, XARRAY prevtrans)
 	/* primitive nodeinst: ask the technology how to draw it */
 	if (np->primindex != 0)
 	{
-		high = nodepolys(ni, 0);
+		high = nodepolys(ni, 0, NOWINDOWPART);
 
 		/* don't draw invisible pins */
 		if (np == gen_invispinprim) low = 1; else
@@ -1996,11 +1994,11 @@ void io_edifsymbol_nodeinst(NODEINST *ni, XARRAY prevtrans)
 		}
 
 		/* see if there are displayable variables on the facet */
-		if ((displaytotal = tech_displayablenvars(ni)) != 0)
+		if ((displaytotal = tech_displayablenvars(ni, NOWINDOWPART)) != 0)
 			io_edifsetgraphic(EGUNKNOWN);
 		for (i = 0; i < displaytotal; i++)
 		{
-			var = tech_filldisplayablenvar(ni, poly);
+			var = tech_filldisplayablenvar(ni, poly, NOWINDOWPART, 0);
 			xformpoly(poly, prevtrans);
 			/* check for name */
 			if (namesame((name = makename(var->key)), "EDIF_annotate"))
@@ -2047,7 +2045,7 @@ void io_edifsymbol_arcinst(ARCINST *ai, XARRAY trans)
 	/* get polygon */
 	if (poly == NOPOLYGON) poly = allocstaticpolygon(4, io_aid->cluster);
 
-	i = arcpolys(ai);
+	i = arcpolys(ai, NOWINDOWPART);
 
 	/* get the endpoints of the arcinst */
 	for (j = 0; j < i; j++)
@@ -2069,10 +2067,10 @@ void io_edifsymbol_arcinst(ARCINST *ai, XARRAY trans)
 	}
 
 	/* now get the variables */
-	if ((displaytotal = tech_displayableavars(ai)) != 0) io_edifsetgraphic(EGUNKNOWN);
+	if ((displaytotal = tech_displayableavars(ai, NOWINDOWPART)) != 0) io_edifsetgraphic(EGUNKNOWN);
 	for (i = 0; i < displaytotal; i++)
 	{
-		var = tech_filldisplayableavar(ai, poly);
+		var = tech_filldisplayableavar(ai, poly, NOWINDOWPART, 0);
 		xformpoly(poly, trans);
 
 		/* check for name */
@@ -2747,32 +2745,42 @@ char *EO_get_exp(double val)
 /* EDIF Options */
 DIALOGITEM io_edifoptionsdialogitems[] =
 {
- /*  1 */ {0, {32,120,56,192}, BUTTON, N_("OK")},
- /*  2 */ {0, {32,16,56,88}, BUTTON, N_("Cancel")},
- /*  3 */ {0, {8,8,24,200}, CHECK, N_("Use Schematic View")}
+ /*  1 */ {0, {68,120,92,192}, BUTTON, N_("OK")},
+ /*  2 */ {0, {68,16,92,88}, BUTTON, N_("Cancel")},
+ /*  3 */ {0, {8,8,24,200}, CHECK, N_("Use Schematic View")},
+ /*  4 */ {0, {36,8,52,96}, MESSAGE, N_("Input scale:")},
+ /*  5 */ {0, {36,100,52,188}, EDITTEXT, ""}
 };
-DIALOG io_edifoptionsdialog = {{50,75,115,284}, N_("EDIF Options"), 0, 3, io_edifoptionsdialogitems};
+DIALOG io_edifoptionsdialog = {{50,75,151,284}, N_("EDIF Options"), 0, 5, io_edifoptionsdialogitems};
+
+/* special items for the "EDIF Options" dialog: */
+#define DEDO_USESCHEMATIC  3		/* Use Schematic View (check) */
+#define DEDO_INPUTSCALE    5		/* Input scale (edit text) */
 
-/*
- * special case for the "EDIF Options" dialog
- * Use Schematic View   = 3 (check)
- */
 void io_edifoptionsdlog(void)
 {
 	REGISTER INTBIG itemHit, i, *curstate;
 	INTBIG newstate[NUMIOSTATEBITWORDS];
+	REGISTER VARIABLE *var;
+	char line[50];
+	float inputscale, newscale;
 
 	if (DiaInitDialog(&io_edifoptionsdialog) != 0) return;
 	curstate = io_getstatebits();
 	for(i=0; i<NUMIOSTATEBITWORDS; i++) newstate[i] = curstate[i];
-	if ((curstate[0]&EDIFSCHEMATIC) != 0) DiaSetControl(3, 1);
+	if ((curstate[0]&EDIFSCHEMATIC) != 0) DiaSetControl(DEDO_USESCHEMATIC, 1);
+	var = getval((INTBIG)io_aid, VAID, VFLOAT, "IO_edif_input_scale");
+	if (var == NOVARIABLE) inputscale = 1.0; else
+		inputscale = castfloat(var->addr);
+	sprintf(line, "%g", inputscale);
+	DiaSetText(DEDO_INPUTSCALE, line);
 
 	/* loop until done */
 	for(;;)
 	{
 		itemHit = DiaNextHit();
 		if (itemHit == OK || itemHit == CANCEL) break;
-		if (itemHit == 3)
+		if (itemHit == DEDO_USESCHEMATIC)
 		{
 			DiaSetControl(itemHit, 1 - DiaGetControl(itemHit));
 			continue;
@@ -2781,10 +2789,13 @@ void io_edifoptionsdlog(void)
 
 	if (itemHit != CANCEL)
 	{
-		if (DiaGetControl(3) != 0) newstate[0] |= EDIFSCHEMATIC; else
+		if (DiaGetControl(DEDO_USESCHEMATIC) != 0) newstate[0] |= EDIFSCHEMATIC; else
 			newstate[0] &= ~EDIFSCHEMATIC;
 		for(i=0; i<NUMIOSTATEBITWORDS; i++) if (curstate[i] != newstate[i]) break;
 		if (i < NUMIOSTATEBITWORDS) io_setstatebits(newstate);
+		newscale = (float)atof(DiaGetText(DEDO_INPUTSCALE));
+		if (newscale != inputscale)
+			setval((INTBIG)io_aid, VAID, "IO_edif_input_scale", castint(newscale), VFLOAT);
 	}
 	DiaDoneDialog();
 }
