@@ -72,6 +72,7 @@
 #include "edialogs.h"
 #include <math.h>
 
+#define NEWPATH 1		/* path and facet caching optimizations */
 /* #define PLACEFACETCENTERS  1 */	/* uncomment to place facet centers */
 
 /* electric specific globals */
@@ -270,6 +271,7 @@ void            io_gdsIdetermine_text(void);
 void            io_gdsIdetermine_time(char*);
 void            io_gdsIelement(INTSML*);
 void            io_gdsIerror(INTBIG);
+NODEPROTO      *io_gdsIfindproto(char *name);
 void            io_gdsIfonts(void);
 void            io_gdsIgetdouble(double*);
 void            io_gdsIgetfloat(float*);
@@ -863,6 +865,7 @@ void io_gdsItext(char *charstring, INTBIG vjust, INTBIG hjust, INTBIG angle,
 {
 	NODEINST *ni;
 	VARIABLE *var;
+	INTBIG size;
 
 	/* no text */
 	if (!io_gdslayerused || !(io_gdscurstate[0]&GDSINTEXT)) return;
@@ -883,19 +886,10 @@ void io_gdsItext(char *charstring, INTBIG vjust, INTBIG hjust, INTBIG angle,
 			longjmp(io_gdsenv, (INTBIG)_("Could not create text string"));
 
 	/* set the text size and orientation */
-	switch (rounddouble (scale))
-	{
-		case 0:
-		case 1:  var->textdescript = (TXT4P  << VTSIZESH); break;
-		case 2:  var->textdescript = (TXT6P  << VTSIZESH); break;
-		case 3:  var->textdescript = (TXT8P  << VTSIZESH); break;
-		case 4:  var->textdescript = (TXT10P << VTSIZESH); break;
-		case 5:  var->textdescript = (TXT12P << VTSIZESH); break;
-		case 6:  var->textdescript = (TXT14P << VTSIZESH); break;
-		case 7:  var->textdescript = (TXT16P << VTSIZESH); break;
-		case 8:  var->textdescript = (TXT18P << VTSIZESH); break;
-		default: var->textdescript = (TXT20P << VTSIZESH); break;
-	}
+	size = rounddouble(scale);
+	if (size < TXT4P) size = TXT4P;
+	if (size > TXT20P) size = TXT20P;
+	var->textdescript = (size << VTSIZESH);
 
 	/* determine the presentation */
 	var->textdescript = (var->textdescript & ~VTPOSITION);
@@ -976,7 +970,7 @@ void io_gdsIOnceinit(void)
 					ni->highx = np->highx;
 					ni->lowy  = np->lowy;
 					ni->highy = np->highy;
-					j = nodepolys(ni, 0);
+					j = nodepolys(ni, 0, NOWINDOWPART);
 					if (j != 1) continue;
 					shapenodepoly(ni, 0, poly);
 					if (i == poly->layer)
@@ -996,7 +990,7 @@ void io_gdsIOnceinit(void)
 				for(j=i+1; j<count; j++) if (which == layer_numbers[j]) break;
 				if (j < count) continue;
 				ttyputmsg(_("Cannot read GDS layer %ld (%s)"), which,
-					layername(el_curtech, (INTSML)i));
+					layername(el_curtech, i));
 			}
 		}
 	} else ttyputmsg(_("Technology has no GDSII layer numbers"));
@@ -1291,6 +1285,18 @@ void io_gdsIdetermine_layer(INTBIG *layer, INTBIG *sublayer)
 	}
 }
 
+NODEPROTO *io_gdsIfindproto(char *name)
+{
+	CELL *c;
+
+	c = db_findcellname(name, io_gdslibrary);
+	if (c == NOCELL) return(NONODEPROTO);
+	if (strcmp(c->cellname, name))
+		ttyputmsg(_("Cell %s is used instead of cell %s"),
+			c->cellname, name);
+	return(c->firstincell);
+}
+
 void io_gdsIbeginstructure(void)
 {
 	char create_time[IDSTRING+1], mod_time[IDSTRING+1];
@@ -1306,11 +1312,7 @@ void io_gdsIbeginstructure(void)
 			if (io_gdstoken == GDS_IDENT)
 			{
 				/* look for this nodeproto */
-				for (io_gdsstructure = io_gdslibrary->firstnodeproto;
-					io_gdsstructure != NONODEPROTO;
-						io_gdsstructure = io_gdsstructure->nextnodeproto)
-							if (!strcmp(io_gdsstructure->cell->cellname, io_gdstokenstring))
-								break;
+				io_gdsstructure = io_gdsIfindproto(io_gdstokenstring);
 				if (io_gdsstructure == NONODEPROTO)
 				{
 					/* create the proto */
@@ -1489,7 +1491,7 @@ void io_gdsIdetermine_shape(void)
 void io_gdsIdetermine_path(void)
 {
 	INTBIG endcode, n, layer, sublayer, off[8], thisAngle, lastAngle, nextAngle,
-		fextend, textend, ang;
+		bgnextend, endextend, fextend, textend, ang;
 	INTBIG width, length, lx, ly, hx, hy, i, j, fx, fy, tx, ty, cx, cy;
 	REGISTER NODEINST *ni;
 	static POLYGON *poly = NOPOLYGON;
@@ -1514,7 +1516,23 @@ void io_gdsIdetermine_path(void)
 	{
 		width = 0;
 	}
-
+#ifdef NEWPATH
+	bgnextend = endextend = (endcode == 0 || endcode == 4 ? 0 : width/2);
+	if (io_gdstoken == GDS_BGNEXTN)
+	{
+		io_gdsIgettoken();
+		if (endcode == 4)
+			bgnextend = rounddouble((double)io_gdstokenvalue32 * io_gdsscale);
+		io_gdsIgettoken();
+	}
+	if (io_gdstoken == GDS_ENDEXTN)
+	{
+		io_gdsIgettoken();
+		if (endcode == 4)
+			endextend = rounddouble((double)io_gdstokenvalue32 * io_gdsscale);
+		io_gdsIgettoken();
+	}
+#endif
 	if (io_gdstoken == GDS_XY)
 	{
 		io_gdsIgettoken();
@@ -1542,6 +1560,11 @@ void io_gdsIdetermine_path(void)
 					if (ang > 90) ang = 180 - ang;
 					fextend = tech_getextendfactor(width, ang);
 				}
+#ifdef NEWPATH
+			} else
+			{
+				fextend = bgnextend;
+#endif
 			}
 			if (i+1 < n-1)
 			{
@@ -1554,62 +1577,12 @@ void io_gdsIdetermine_path(void)
 					if (ang > 90) ang = 180 - ang;
 					textend = tech_getextendfactor(width, ang);
 				}
-			}
-
-#if 0
-			/* handle manhattan segment */
-			if (fx == tx || fy == ty)
+#ifdef NEWPATH
+			} else
 			{
-				lx = hx = fx;
-				ly = hy = fy;
-				if (lx > tx) lx = tx;
-				if (hx < tx) hx = tx;
-				if (ly > ty) ly = ty;
-				if (hy < ty) hy = ty;
-
-				/* handle all paths, except two segment paths with flush end type */
-				if (n != 3 || endcode == 2)
-				{
-					if (ly == hy || endcode == 2 || (endcode == 0 && i != 0 && i+1 < n-1))
-					{
-						ly -= width/2;
-						hy += width/2;
-					}
-					if (lx == hx || endcode == 2 || (endcode == 0 && i != 0 && i+1 < n-1))
-					{
-						lx -= width/2;
-						hx += width/2;
-					}
-				} else
-				{
-					/* special case for a two segment path with a flush end type */
-					if (endcode == 0)
-					{
-						if (io_gdsvertex[0].px < io_gdsvertex[1].px && i == 0) hx += width/2;
-						if (io_gdsvertex[1].px < io_gdsvertex[2].px && i == 1) lx -= width/2;
-						if (io_gdsvertex[1].px > io_gdsvertex[2].px && i == 1) hx += width/2;
-						if (io_gdsvertex[0].px > io_gdsvertex[1].px && i == 0) lx -= width/2;
-						if (lx == hx)
-						{
-							lx -= width/2;
-							hx += width/2;
-						}
-						if (ly == hy)
-						{
-							ly -= width/2;
-							hy += width/2;
-						}
-					}
-				}
-				if (io_gdslayerused)
-				{
-					io_gdslayerpoly[io_gdslayerwhich]++;
-					if (newnodeinst(io_gdslayernode, lx, hx, ly, hy, 0, 0, io_gdsstructure) == NONODEINST)
-						longjmp(io_gdsenv, (INTBIG)_("failed to create outline"));
-				}
-				continue;
-			}
+				textend = endextend;
 #endif
+			}
 
 			/* handle arbitrary angle path segment */
 			if (io_gdslayerused)
@@ -1621,7 +1594,9 @@ void io_gdsIdetermine_path(void)
 
 				/* determine shape of segment */
 				length = computedistance(fx, fy, tx, ty);
-				if (endcode == 0) fextend = textend = 0; /* the 0 was width/2 */
+#ifndef NEWPATH
+				if (endcode == 0) fextend = textend = 0;
+#endif
 				j = figureangle(fx, fy, tx, ty);
 				tech_makeendpointpoly(length, width, j, fx, fy, fextend,
 					tx, ty, textend, poly);
@@ -1664,8 +1639,7 @@ void io_gdsIgetproto(char *name)
 	NODEPROTO *np;
 
 	/* scan for this proto */
-	for (np = io_gdslibrary->firstnodeproto; np != NONODEPROTO; np = np->nextnodeproto)
-		if (!strcmp(np->cell->cellname, name)) break;
+	np = io_gdsIfindproto(name);
 	if (np == NONODEPROTO)
 	{
 		/* FILO order, create this nodeproto */
