@@ -42,54 +42,54 @@
 
 #define OLDGDS 1
 
-#define	GDSVERSION            3
-#define	BYTEMASK           0xFF
-#define	NAMSIZE             100
-#define	DSIZE               512		/* data block */
+#define GDSVERSION            3
+#define BYTEMASK           0xFF
+#define NAMSIZE             100
+#define DSIZE               512		/* data block */
 #define MAXPOINTS           510		/* maximum points in a polygon */
 
 /* GDSII bit assignments in STRANS record */
-#define	STRANS_REFLX     0x8000
-#define	STRANS_ABSA         0x2
+#define STRANS_REFLX     0x8000
+#define STRANS_ABSA         0x2
 
 /* data type codes */
-#define	DTYP_NONE             0
+#define DTYP_NONE             0
 
 /* header codes */
-#define	HDR_HEADER       0x0002
-#define	HDR_BGNLIB       0x0102
-#define	HDR_LIBNAME      0x0206
-#define	HDR_UNITS        0x0305
-#define	HDR_ENDLIB       0x0400
-#define	HDR_BGNSTR       0x0502
-#define	HDR_STRNAME      0x0606
-#define	HDR_ENDSTR       0x0700
-#define	HDR_BOUNDARY     0x0800
-#define	HDR_PATH         0x0900
-#define	HDR_SREF         0x0A00
+#define HDR_HEADER       0x0002
+#define HDR_BGNLIB       0x0102
+#define HDR_LIBNAME      0x0206
+#define HDR_UNITS        0x0305
+#define HDR_ENDLIB       0x0400
+#define HDR_BGNSTR       0x0502
+#define HDR_STRNAME      0x0606
+#define HDR_ENDSTR       0x0700
+#define HDR_BOUNDARY     0x0800
+#define HDR_PATH         0x0900
+#define HDR_SREF         0x0A00
 #define HDR_AREF	     0x0B00
 #define HDR_TEXT	     0x0C00
-#define	HDR_LAYER        0x0D02
-#define	HDR_DATATYPE     0x0E02
-#define	HDR_XY           0x1003
-#define	HDR_ENDEL        0x1100
-#define	HDR_SNAME        0x1206
+#define HDR_LAYER        0x0D02
+#define HDR_DATATYPE     0x0E02
+#define HDR_XY           0x1003
+#define HDR_ENDEL        0x1100
+#define HDR_SNAME        0x1206
 #define HDR_TEXTTYPE     0x1602
 #define HDR_PRESENTATION 0x1701
 #define HDR_STRING	     0x1906
-#define	HDR_STRANS       0x1A01
+#define HDR_STRANS       0x1A01
 #define HDR_MAG          0x1B05
-#define	HDR_ANGLE        0x1C05
+#define HDR_ANGLE        0x1C05
 
 /* Header byte counts */
-#define	HDR_N_BGNLIB         28
-#define	HDR_N_UNITS          20
-#define	HDR_N_ANGLE          12
+#define HDR_N_BGNLIB         28
+#define HDR_N_UNITS          20
+#define HDR_N_ANGLE          12
 
 /* Maximum string sizes  */
-#define	HDR_M_SNAME          32
-#define	HDR_M_STRNAME        32
-#define	HDR_M_ASCII         256
+#define HDR_M_SNAME          32
+#define HDR_M_STRNAME        32
+#define HDR_M_ASCII         256
 
 /* contour gathering thresholds for polygon accumulation */
 #define BESTTHRESH           0.001			/* 1/1000 of a millimeter */
@@ -383,7 +383,7 @@ void io_gdswritecell(NODEPROTO *np)
 			if ((subno->userbits&WIPED) != 0) continue;
 
 			/* check for displayable variables */
-			displaytotal = tech_displayablenvars(subno);
+			displaytotal = tech_displayablenvars(subno, NOWINDOWPART);
 			if (displaytotal != 0)
 			{
 				/* get the first polygon off the node, this is the text layer */
@@ -396,7 +396,7 @@ void io_gdswritecell(NODEPROTO *np)
 
 				for (i=0; i<displaytotal; i++)
 				{
-					(void)tech_filldisplayablenvar(subno, poly);
+					(void)tech_filldisplayablenvar(subno, poly, NOWINDOWPART, 0);
 					/* dump this text field */
 					/* note that a presentation of 8 is bottomleft */
 					io_gds_header(HDR_TEXT, 0);
@@ -433,7 +433,7 @@ void io_gdswritecell(NODEPROTO *np)
 
 					/* write out the node the text is attached to */
 					/* RLW - Queen's */
-					i = nodepolys(subno, 0);
+					i = nodepolys(subno, 0, NOWINDOWPART);
 					for (j=0; j<i; j++)
 					{
 						shapenodepoly(subno, j, poly);
@@ -443,7 +443,7 @@ void io_gdswritecell(NODEPROTO *np)
 			} else
 			{
 				/* write a primitive nodeinst */
-				i = nodepolys(subno, 0);
+				i = nodepolys(subno, 0, NOWINDOWPART);
 				for(j=0; j<i; j++)
 				{
 					shapenodepoly(subno, j, poly);
@@ -518,7 +518,7 @@ void io_gdswritecell(NODEPROTO *np)
 	for(subar = np->firstarcinst; subar != NOARCINST; subar = subar->nextarcinst)
 	{
 		/* arcinst: ask the technology how to draw it */
-		i = arcpolys(subar);
+		i = arcpolys(subar, NOWINDOWPART);
 		layervar = getvalkey((INTBIG)subar, VARCINST, VINTEGER, io_gds_layerkey);
 
 		/* plot each layer of the arcinst */
@@ -1112,18 +1112,17 @@ DIALOGITEM io_gdsoptionsdialogitems[] =
 };
 DIALOG io_gdsoptionsdialog = {{50,75,307,548}, N_("GDS Options"), 0, 15, io_gdsoptionsdialogitems};
 
-/*
- * special case for the "GDS Options" dialog
- * Layer list                   = 3 (scroll list)
- * New layer                    = 5 (edit text)
- * Input Includes Text          = 6 (check)
- * Input Expands Facets         = 7 (check)
- * Input Instantiates Arrays    = 8 (check)
- * Input Ignores unknown layers = 15 (check)
- * Output Merges Boxes          = 9 (check)
- * Output Arc Angle             = 12 (edit text)
- * Output Arc Sag               = 14 (edit text)
- */
+/* special items for the "GDS Options" dialog: */
+#define DGDO_LAYERLIST       3		/* Layer list (scroll list) */
+#define DGDO_NEWLAYER        5		/* New layer (edit text) */
+#define DGDO_IINCLUDETEXT    6		/* Input Includes Text (check) */
+#define DGDO_IEXPANDFACET    7		/* Input Expands Facets (check) */
+#define DGDO_IINSTARRAY      8		/* Input Instantiates Arrays (check) */
+#define DGDO_OMERGEBOX       9		/* Output Merges Boxes (check) */
+#define DGDO_OARCANGLE      12		/* Output Arc Angle (edit text) */
+#define DGDO_OARCSAG        14		/* Output Arc Sag (edit text) */
+#define DGDO_IIGNOREUNKNOWN 15		/* Input Ignores unknown layers (check) */
+
 void io_gdsoptionsdlog(void)
 {
 	REGISTER INTBIG i, itemHit, *layernumbers, *curstate, val, newres, newsag, numberschanged;
@@ -1132,7 +1131,7 @@ void io_gdsoptionsdlog(void)
 	REGISTER VARIABLE *gdsvar;
 
 	if (DiaInitDialog(&io_gdsoptionsdialog) != 0) return;
-	DiaInitTextDialog(3, DiaNullDlogList, DiaNullDlogItem, DiaNullDlogDone, 0,
+	DiaInitTextDialog(DGDO_LAYERLIST, DiaNullDlogList, DiaNullDlogItem, DiaNullDlogDone, 0,
 		SCSELMOUSE|SCREPORT);
 	layernumbers = (INTBIG *)emalloc(el_curtech->layercount * SIZEOFINTBIG, el_tempcluster);
 	gdsvar = getval((INTBIG)el_curtech, VTECHNOLOGY, VINTEGER|VISARRAY,
@@ -1143,24 +1142,24 @@ void io_gdsoptionsdlog(void)
 			val = ((INTBIG *)gdsvar->addr)[i];
 		layernumbers[i] = val;
 		(void)initinfstr();
-		(void)addstringtoinfstr(layername(el_curtech, (INTSML)i));
+		(void)addstringtoinfstr(layername(el_curtech, i));
 		sprintf(buf, " (%ld)", val);
 		(void)addstringtoinfstr(buf);
-		DiaStuffLine(3, returninfstr());
+		DiaStuffLine(DGDO_LAYERLIST, returninfstr());
 	}
-	DiaSelectLine(3, 0);
+	DiaSelectLine(DGDO_LAYERLIST, 0);
 	sprintf(buf, "%ld", layernumbers[0]);
-	DiaSetText(5, buf);
+	DiaSetText(DGDO_NEWLAYER, buf);
 	curstate = io_getstatebits();
 	for(i=0; i<NUMIOSTATEBITWORDS; i++) newstate[i] = curstate[i];
-	if ((curstate[0]&GDSINTEXT) != 0) DiaSetControl(6, 1);
-	if ((curstate[0]&GDSINEXPAND) != 0) DiaSetControl(7, 1);
-	if ((curstate[0]&GDSINARRAYS) != 0) DiaSetControl(8, 1);
-	if ((curstate[0]&GDSOUTMERGE) != 0) DiaSetControl(9, 1);
-	if ((curstate[0]&GDSINIGNOREUKN) != 0) DiaSetControl(15, 1);
+	if ((curstate[0]&GDSINTEXT) != 0) DiaSetControl(DGDO_IINCLUDETEXT, 1);
+	if ((curstate[0]&GDSINEXPAND) != 0) DiaSetControl(DGDO_IEXPANDFACET, 1);
+	if ((curstate[0]&GDSINARRAYS) != 0) DiaSetControl(DGDO_IINSTARRAY, 1);
+	if ((curstate[0]&GDSOUTMERGE) != 0) DiaSetControl(DGDO_OMERGEBOX, 1);
+	if ((curstate[0]&GDSINIGNOREUKN) != 0) DiaSetControl(DGDO_IIGNOREUNKNOWN, 1);
 	getcontoursegmentparameters(&arcres, &arcsag);
-	DiaSetText(12, frtoa(arcres*WHOLE/10));
-	DiaSetText(14, latoa(arcsag));
+	DiaSetText(DGDO_OARCANGLE, frtoa(arcres*WHOLE/10));
+	DiaSetText(DGDO_OARCSAG, latoa(arcsag));
 
 	/* loop until done */
 	numberschanged = 0;
@@ -1168,45 +1167,47 @@ void io_gdsoptionsdlog(void)
 	{
 		itemHit = DiaNextHit();
 		if (itemHit == OK || itemHit == CANCEL) break;
-		if ((itemHit >= 6 && itemHit <= 9) || itemHit == 15)
+		if (itemHit == DGDO_IINCLUDETEXT || itemHit == DGDO_IEXPANDFACET ||
+			itemHit == DGDO_IINSTARRAY || itemHit == DGDO_OMERGEBOX ||
+			itemHit == DGDO_IIGNOREUNKNOWN)
 		{
 			DiaSetControl(itemHit, 1 - DiaGetControl(itemHit));
 			continue;
 		}
-		if (itemHit == 3)
+		if (itemHit == DGDO_LAYERLIST)
 		{
-			i = DiaGetCurLine(3);
+			i = DiaGetCurLine(DGDO_LAYERLIST);
 			sprintf(buf, "%ld", layernumbers[i]);
-			DiaSetText(5, buf);
+			DiaSetText(DGDO_NEWLAYER, buf);
 			continue;
 		}
-		if (itemHit == 5)
+		if (itemHit == DGDO_NEWLAYER)
 		{
-			i = DiaGetCurLine(3);
-			val = atoi(DiaGetText(5));
+			i = DiaGetCurLine(DGDO_LAYERLIST);
+			val = atoi(DiaGetText(DGDO_NEWLAYER));
 			if (val == layernumbers[i]) continue;
 			numberschanged++;
 			layernumbers[i] = val;
 			(void)initinfstr();
-			(void)addstringtoinfstr(layername(el_curtech, (INTSML)i));
+			(void)addstringtoinfstr(layername(el_curtech, i));
 			sprintf(buf, " (%ld)", val);
 			(void)addstringtoinfstr(buf);
-			DiaSetScrollLine(3, i, returninfstr());
+			DiaSetScrollLine(DGDO_LAYERLIST, i, returninfstr());
 			continue;
 		}
 	}
 
 	if (itemHit != CANCEL)
 	{
-		if (DiaGetControl(6) != 0) newstate[0] |= GDSINTEXT; else
+		if (DiaGetControl(DGDO_IINCLUDETEXT) != 0) newstate[0] |= GDSINTEXT; else
 			newstate[0] &= ~GDSINTEXT;
-		if (DiaGetControl(7) != 0) newstate[0] |= GDSINEXPAND; else
+		if (DiaGetControl(DGDO_IEXPANDFACET) != 0) newstate[0] |= GDSINEXPAND; else
 			newstate[0] &= ~GDSINEXPAND;
-		if (DiaGetControl(8) != 0) newstate[0] |= GDSINARRAYS; else
+		if (DiaGetControl(DGDO_IINSTARRAY) != 0) newstate[0] |= GDSINARRAYS; else
 			newstate[0] &= ~GDSINARRAYS;
-		if (DiaGetControl(9) != 0) newstate[0] |= GDSOUTMERGE; else
+		if (DiaGetControl(DGDO_OMERGEBOX) != 0) newstate[0] |= GDSOUTMERGE; else
 			newstate[0] &= ~GDSOUTMERGE;
-		if (DiaGetControl(15) != 0) newstate[0] |= GDSINIGNOREUKN; else
+		if (DiaGetControl(DGDO_IIGNOREUNKNOWN) != 0) newstate[0] |= GDSINIGNOREUKN; else
 			newstate[0] &= ~GDSINIGNOREUKN;
 		for(i=0; i<NUMIOSTATEBITWORDS; i++) if (curstate[i] != newstate[i]) break;
 		if (i < NUMIOSTATEBITWORDS) io_setstatebits(newstate);
@@ -1214,8 +1215,8 @@ void io_gdsoptionsdlog(void)
 			setval((INTBIG)el_curtech, VTECHNOLOGY, "IO_gds_layer_numbers",
 				(INTBIG)layernumbers, VINTEGER|VISARRAY|
 					(el_curtech->layercount<<VLENGTHSH));
-		newres = atofr(DiaGetText(12)) * 10 / WHOLE;
-		newsag = atola(DiaGetText(14));
+		newres = atofr(DiaGetText(DGDO_OARCANGLE)) * 10 / WHOLE;
+		newsag = atola(DiaGetText(DGDO_OARCSAG));
 		if (newres != arcres || newsag != arcsag)
 			setcontoursegmentparameters(newres, newsag);
 	}
