@@ -55,19 +55,6 @@ typedef struct
 
 PENCOLOR pen_color_table[256];
 
-static float io_hpfontheight[] =
-{
-	0.0711f,		/*  4 pt */
-	0.1411f,		/*  6 pt */
-	0.2222f,		/*  8 pt */
-	0.3028f,		/* 10 pt */
-	0.3833f,		/* 12 pt */
-	0.4639f,		/* 14 pt */
-	0.5444f,		/* 16 pt */
-	0.6250f,		/* 18 pt */
-	0.7056f		/* 20 pt */
-};
-
 /* prototypes for local routines */
 void io_hpwritepolygon(INTSML, TECHNOLOGY*, INTBIG*, INTBIG*, INTSML);
 INTSML io_hppoly(POLYGON*, WINDOWPART*);
@@ -76,7 +63,7 @@ void io_hparc(INTBIG, INTBIG, INTBIG, INTBIG, INTBIG, INTBIG, INTSML);
 void io_hpcircle(INTBIG, INTBIG, INTBIG, INTBIG, INTSML);
 void io_hpdisc(INTBIG, INTBIG, INTBIG, INTBIG, GRAPHICS*);
 void io_hpfilledpolygon(INTBIG*, INTBIG*, INTSML, INTSML);
-void io_hptext(INTSML, INTBIG, INTBIG, INTBIG, INTBIG, INTSML, char*, INTSML);
+void io_hptext(INTSML, INTBIG, INTBIG, INTBIG, INTBIG, INTBIG, char*, INTSML);
 void io_hpparsepenfile(void);
 INTSML io_hpchoosecolorentry(INTSML);
 void io_hppenselect(INTSML);
@@ -95,8 +82,8 @@ void io_hpwritepolygon(INTSML layer, TECHNOLOGY *tech, INTBIG *x, INTBIG *y, INT
 INTSML io_writehpgllibrary(LIBRARY *lib)
 {
 	char file[100], buff[MAXSTR], *truename;
-	REGISTER NODEPROTO *np, *onp;
-	INTBIG lx, hx, ly, hy, hlx, hhx, hly, hhy, i, *curstate,
+	REGISTER NODEPROTO *np;
+	INTBIG lx, hx, ly, hy, i, *curstate,
 		sulx, suhx, suly, suhy, sslx, sshx, ssly, sshy;
 	REGISTER VARIABLE *varred, *vargreen, *varblue, *varscale;
 	REGISTER float scale;
@@ -130,37 +117,7 @@ INTSML io_writehpgllibrary(LIBRARY *lib)
 	}
 
 	/* determine area to plot */
-	if (framesize(&hx, &hy, np) == 0)
-	{
-		hx /= 2;   lx = -hx;
-		hy /= 2;   ly = -hy;
-	} else
-	{
-		lx = np->lowx;   hx = np->highx;
-		ly = np->lowy;   hy = np->highy;
-	}
-
-	/* see if focusing on highlighted area */
-	if ((curstate[0]&PLOTFOCUS) != 0)
-	{
-		if ((curstate[0]&PLOTFOCUSDPY) != 0)
-		{
-			lx = el_curwindowpart->screenlx;
-			hx = el_curwindowpart->screenhx;
-			ly = el_curwindowpart->screenly;
-			hy = el_curwindowpart->screenhy;
-		} else
-		{
-			onp = (NODEPROTO *)askaid(us_aid, "get-highlighted-area", (INTBIG)&hlx,
-				(INTBIG)&hhx, (INTBIG)&hly, (INTBIG)&hhy);
-			if (onp == NONODEPROTO)
-				ttyputerr(_("Warning: no highlighted area; plotting entire facet")); else
-			{
-				lx = hlx;   hx = hhx;
-				ly = hly;   hy = hhy;
-			}
-		}
-	}
+	if (io_getareatoprint(np, &lx, &hx, &ly, &hy, 1) != 0) return(1);
 
 	/* build pseudowindow */
 	io_hpwindow.uselx = 0;       io_hpwindow.screenlx = lx;
@@ -286,8 +243,8 @@ INTSML io_writehpgllibrary(LIBRARY *lib)
  */
 INTSML io_hppoly(POLYGON *poly, WINDOWPART *win)
 {
-	REGISTER INTSML k, font;
-	INTBIG xl, xh, yl, yh, x, y;
+	REGISTER INTSML k;
+	INTBIG xl, xh, yl, yh, x, y, font;
 	INTSML color;
 	REGISTER TECHNOLOGY *tech;
 
@@ -501,12 +458,12 @@ void io_hpfilledpolygon(INTBIG *xlist, INTBIG *ylist, INTSML count, INTSML color
 	if (ftype != 1) io_hpcmd("EP;");
 }
 
-void io_hptext(INTSML type, INTBIG xl, INTBIG xh, INTBIG yl, INTBIG yh, INTSML font,
+void io_hptext(INTSML type, INTBIG xl, INTBIG xh, INTBIG yl, INTBIG yh, INTBIG font,
 	char *text, INTSML color)
 {
 	char s[MAXSTR];
 
-	(void)sprintf(s, "SI %f,%f;", io_hpfontheight[font]/1.3, io_hpfontheight[font]);
+	(void)sprintf(s, "SI %f,%f;", font*0.02f/1.3f, font*0.02f);
 	io_hpcmd(s);
 	io_hppenselect(color);
 
