@@ -921,8 +921,8 @@ INTSML io_lefreadport(FILE *f, NODEPROTO *facet, char *portname, INTBIG portbits
 			if (pin == NONODEPROTO)
 			{
 				ttyputerr(_("Line %d: No Via in current technology connects %s to %s"),
-					io_leflineno, arcfunctionname((INTSML)afunc1),
-					arcfunctionname((INTSML)afunc2));
+					io_leflineno, arcfunctionname(afunc1),
+					arcfunctionname(afunc2));
 				return(1);
 			}
 			if (io_lefignoretosemicolon(f, "VIA") != 0) return(1);
@@ -1247,7 +1247,7 @@ ARCPROTO *io_lefgetarc(INTBIG afunc)
 	if (ap == NOARCPROTO)
 	{
 		ttyputerr(_("Line %d: Layer %s not in current technology"),
-			io_leflineno, arcfunctionname((INTSML)afunc));
+			io_leflineno, arcfunctionname(afunc));
 		return(NOARCPROTO);
 	}
 	return(ap);
