@@ -146,7 +146,7 @@ INTSML io_writeleflibrary(LIBRARY *lib)
 		}
 		xprintf(io_fileout, "    PORT\n");
 		io_lefoutcurlayer[0] = 0;
-		tot = nodeEpolys(rni, 0);
+		tot = nodeEpolys(rni, 0, NOWINDOWPART);
 		for(i=0; i<tot; i++)
 		{
 			shapeEnodepoly(rni, i, poly);
@@ -188,7 +188,6 @@ void io_lefoutspread(FILE *out, NODEPROTO *facet, NETWORK *net, NODEINST *ignore
 	REGISTER ARCINST *ai;
 	REGISTER PORTARCINST *pi;
 	REGISTER INTBIG i, tot, fun;
-	char *extra;
 	XARRAY trans;
 	static POLYGON *poly = NOPOLYGON;
 
@@ -198,7 +197,7 @@ void io_lefoutspread(FILE *out, NODEPROTO *facet, NETWORK *net, NODEINST *ignore
 	{
 		if (ni->proto->primindex == 0) continue;
 		if (ni == ignore) continue;
-		fun = nodefunction(ni, &extra);
+		fun = nodefunction(ni, 0);
 		if (fun != NPPIN && fun != NPCONTACT && fun != NPNODE && fun != NPCONNECT)
 			continue;
 		for(pi = ni->firstportarcinst; pi != NOPORTARCINST; pi = pi->nextportarcinst)
@@ -208,7 +207,7 @@ void io_lefoutspread(FILE *out, NODEPROTO *facet, NETWORK *net, NODEINST *ignore
 		/* write all layers on this node */
 		ni->temp1 = 1;
 		makerot(ni, trans);
-		tot = nodepolys(ni, 0);
+		tot = nodepolys(ni, 0, NOWINDOWPART);
 		for(i=0; i<tot; i++)
 		{
 			shapenodepoly(ni, i, poly);
@@ -221,7 +220,7 @@ void io_lefoutspread(FILE *out, NODEPROTO *facet, NETWORK *net, NODEINST *ignore
 	{
 		if (ai->network != net) continue;
 		ai->temp1 = 1;
-		tot = arcpolys(ai);
+		tot = arcpolys(ai, NOWINDOWPART);
 		for(i=0; i<tot; i++)
 		{
 			shapearcpoly(ai, i, poly);
@@ -257,7 +256,7 @@ void io_lefoutfacet(FILE *out, NODEPROTO *facet, INTSML top, XARRAY trans)
 		} else
 		{
 			/* primitive: find the metal layers */
-			tot = nodepolys(ni, 0);
+			tot = nodepolys(ni, 0, NOWINDOWPART);
 			for(i=0; i<tot; i++)
 			{
 				shapenodepoly(ni, i, poly);
@@ -270,7 +269,7 @@ void io_lefoutfacet(FILE *out, NODEPROTO *facet, INTSML top, XARRAY trans)
 	for(ai = facet->firstarcinst; ai != NOARCINST; ai = ai->nextarcinst)
 	{
 		if (top != 0 && ai->temp1 != 0) continue;
-		tot = arcpolys(ai);
+		tot = arcpolys(ai, NOWINDOWPART);
 		for(i=0; i<tot; i++)
 		{
 			shapearcpoly(ai, i, poly);
