@@ -36,10 +36,10 @@
 #include "efunction.h"
 
 /* node types returned by "io_l_nodetype" */
-#define	TRUEPIN    1
-#define	TRANSISTOR 2
-#define	INSTANCE   3
-#define	OTHERNODE  4
+#define TRUEPIN    1
+#define TRANSISTOR 2
+#define INSTANCE   3
+#define OTHERNODE  4
 
 /* table that associates arc functionality with port types */
 struct
@@ -73,7 +73,7 @@ struct
 };
 
 /* prototypes for local routines */
-INTSML io_l_writefacet(NODEPROTO*);
+INTBIG io_l_writefacet(NODEPROTO*);
 char *io_l_name(NODEINST*);
 char *io_l_fixname(char*);
 INTSML io_l_nodetype(NODEINST*);
@@ -122,7 +122,7 @@ INTSML io_writellibrary(LIBRARY *lib)
 /*
  * Routine to write "L" for facet "np".  Returns nonzero if back-annotation was added.
  */
-INTSML io_l_writefacet(NODEPROTO *np)
+INTBIG io_l_writefacet(NODEPROTO *np)
 {
 	REGISTER NODEINST *subno, *oni;
 	REGISTER ARCINST *subar, *oar;
@@ -130,10 +130,10 @@ INTSML io_l_writefacet(NODEPROTO *np)
 	REGISTER PORTPROTO *pp;
 	PORTPROTO *gateleft, *gateright, *activetop, *activebottom;
 	REGISTER PORTARCINST *pi, *opi;
-	REGISTER INTSML i, j, k, l, r, thisend, thatend, tot, nature, enature, ot, segcount, backannotate;
+	REGISTER INTBIG i, j, k, l, r, thisend, thatend, tot, nature, enature, ot, segcount, backannotate;
 	REGISTER INTBIG segdist;
 	REGISTER char *type, *lay, *dir, *lastdir;
-	char *extra, line[50];
+	char line[50];
 	INTBIG xpos, ypos, len, wid, lx, hx, ly, hy;
 	REGISTER ARCPROTO **aplist;
 
@@ -217,7 +217,7 @@ INTSML io_l_writefacet(NODEPROTO *np)
 			xprintf(io_fileout, "\tINST %s %s", type, io_l_name(subno));
 		} else
 		{
-			i = nodefunction(subno, &extra);
+			i = nodefunction(subno, 0);
 			if (i == NPPIN)
 			{
 				/* if pin is an export, don't write separate node statement */
@@ -328,7 +328,7 @@ INTSML io_l_writefacet(NODEPROTO *np)
 				xprintf(io_fileout, " W=%s", latoa(subar->width-arcwidthoffset(subar)));
 
 			/* write the starting node name (use port name if pin is an export) */
-			if (subno->firstportexpinst != NOPORTEXPINST && nodefunction(subno, &extra) == NPPIN)
+			if (subno->firstportexpinst != NOPORTEXPINST && nodefunction(subno, 0) == NPPIN)
 				xprintf(io_fileout, " %s",
 						io_l_fixname(subno->firstportexpinst->exportproto->protoname)); else
 							xprintf(io_fileout, " %s", io_l_name(subno));
@@ -407,7 +407,7 @@ INTSML io_l_writefacet(NODEPROTO *np)
 			} else xprintf(io_fileout, " TO");
 
 			/* write the terminating node name (use port name if pin is an export) */
-			if (oni->firstportexpinst != NOPORTEXPINST && nodefunction(oni, &extra) == NPPIN)
+			if (oni->firstportexpinst != NOPORTEXPINST && nodefunction(oni, 0) == NPPIN)
 				xprintf(io_fileout, " %s",
 					io_l_fixname(oni->firstportexpinst->exportproto->protoname)); else
 						xprintf(io_fileout, " %s", io_l_name(oni));
@@ -490,12 +490,11 @@ char *io_l_fixname(char *name)
  */
 INTSML io_l_nodetype(NODEINST *ni)
 {
-	REGISTER INTSML i;
-	char *extra;
+	REGISTER INTBIG i;
 	REGISTER PORTARCINST *pi;
 
 	if (ni->proto->primindex == 0) return(INSTANCE);
-	i = nodefunction(ni, &extra);
+	i = nodefunction(ni, 0);
 	if (i == NPTRANMOS || i == NPTRADMOS || i == NPTRAPMOS) return(TRANSISTOR);
 	if (i != NPPIN) return(OTHERNODE);
 	i = 0;
