@@ -54,6 +54,7 @@
 /* #define DEBUGHIERPS 1	/* uncomment to debug hierarchical PostScript (shows memory usage) */
 
 #define PSSCALE             4
+#define CORNERDATESIZE     14			/* size of date information in corner */
 
 #define PSHEADERDOT         1			/* write macros for dot drawing */
 #define PSHEADERLINE        2			/* write macros for line drawing */
@@ -85,7 +86,6 @@ static char       io_psprefix[10];			/* indentation for PostScript */
 
 GRAPHICS io_psblack = {LAYERO, BLACK, SOLIDC, SOLIDC,
 	{0xFFFF,0xFFFF,0xFFFF,0xFFFF,0xFFFF,0xFFFF,0xFFFF,0xFFFF}, NOVARIABLE, 0};
-static INTBIG     io_psconvert[9] = {4, 6, 8, 10, 12, 14, 16, 18, 20};
 
 /* prototypes for local routines */
 void   io_psarc(INTBIG, INTBIG, INTBIG, INTBIG, INTBIG, INTBIG);
@@ -203,12 +203,12 @@ INTSML io_writepostscriptlibrary(LIBRARY *lib, INTBIG printit)
  */
 INTSML io_pswritefacet(NODEPROTO *np, char *synchronize, INTBIG printit)
 {
-	char file[100], *facetname, *truename, *execpars[5], *printer, *defprinter;
-	INTBIG lx, hx, ly, hy, hlx, hhx, hly, hhy, gridlx, gridly, gridx, gridy,
+	char file[100], *truename, *execpars[5], *printer, *defprinter;
+	INTBIG lx, hx, ly, hy, gridlx, gridly, gridx, gridy,
 		cx, cy, i, j, psulx, psuhx, psuly, psuhy, sulx, suhx,
 		suly, suhy, sslx, sshx, ssly, sshy, oldoptions, ac;
 	REGISTER INTBIG pagewid, pagehei, pagemarginps, pagemargin, *curstate;
-	REGISTER INTSML *layarr, plotdates, plotfocus, plotfocusdisplay, rotateplot,
+	REGISTER INTSML *layarr, plotdates, rotateplot,
 		useplotter, hierpostscript, epsformat;
 	INTBIG bblx, bbhx, bbly, bbhy, t1, t2, unitsx, unitsy, printprocess;
 	UINTBIG curdate;
@@ -228,7 +228,7 @@ INTSML io_pswritefacet(NODEPROTO *np, char *synchronize, INTBIG printit)
 
 	/* get control options */
 	io_psusecolor = io_psusecolorstip = 0;
-	epsformat = hierpostscript = plotdates = plotfocus = plotfocusdisplay = 0;
+	epsformat = hierpostscript = plotdates = 0;
 	rotateplot = useplotter = 0;
 	io_psfake = 0;
 	curstate = io_getstatebits();
@@ -242,8 +242,6 @@ INTSML io_pswritefacet(NODEPROTO *np, char *synchronize, INTBIG printit)
 	if ((curstate[0]&PSPLOTTER) != 0) useplotter = 1;
 	if ((curstate[0]&PSROTATE) != 0) rotateplot = 1;
 	if ((curstate[0]&PLOTDATES) != 0) plotdates = 1;
-	if ((curstate[0]&PLOTFOCUS) != 0) plotfocus = 1;
-	if ((curstate[0]&PLOTFOCUSDPY) != 0) plotfocusdisplay = 1;
 
 	var = getval((INTBIG)io_aid, VAID, VFRACT, "IO_postscript_width");
 	if (var == NOVARIABLE) pagewid = DEFAULTPSWIDTH; else
@@ -279,32 +277,7 @@ INTSML io_pswritefacet(NODEPROTO *np, char *synchronize, INTBIG printit)
 	}
 
 	/* determine the area of interest */
-	us_fullview(np, &lx, &hx, &ly, &hy);
-	if (plotfocus != 0)
-	{
-		if (plotfocusdisplay != 0)
-		{
-			lx = el_curwindowpart->screenlx;
-			hx = el_curwindowpart->screenhx;
-			ly = el_curwindowpart->screenly;
-			hy = el_curwindowpart->screenhy;
-		} else
-		{
-			onp = (NODEPROTO *)askaid(us_aid, "get-highlighted-area",
-				(INTBIG)&hlx, (INTBIG)&hhx, (INTBIG)&hly, (INTBIG)&hhy);
-			if (onp == NONODEPROTO)
-				ttyputerr(_("Warning: no highlighted area; plotting entire facet")); else
-			{
-				if (hhx == hlx || hhy == hly)
-				{
-					ttyputerr(_("Warning: no highlighted area; highlight area and reissue command"));
-					return(1);
-				}
-				lx = hlx;   hx = hhx;
-				ly = hly;   hy = hhy;
-			}
-		}
-	}
+	if (io_getareatoprint(np, &lx, &hx, &ly, &hy, 0) != 0) return(1);
 
 	/* create the PostScript file */
 	if (printit != 0)
@@ -374,42 +347,39 @@ INTSML io_pswritefacet(NODEPROTO *np, char *synchronize, INTBIG printit)
 			unitsy = muldiv(pagehei-pagemarginps*2, var->addr, WHOLE);
 		} else
 		{
-			unitsx = (pagewid-pagemarginps*2)*2;
-			unitsy = (pagehei-pagemarginps*2)*2;
+			unitsx = (pagewid-pagemarginps*2) * 2;
+			unitsy = (pagehei-pagemarginps*2) * 2;
 		}
 	} else
 	{
-		unitsx = (pagewid-pagemarginps*2)*4;
-		unitsy = (pagehei-pagemarginps*2)*4;
+		unitsx = (pagewid-pagemarginps*2) * PSSCALE;
+		unitsy = (pagehei-pagemarginps*2) * PSSCALE;
 	}
 	if (useplotter != 0)
 	{
-		i = (unitsx << 16) / (hx - lx);
-		j = (unitsx << 16) / (hy - ly);
+		i = muldiv(unitsx, 65536, hx - lx);
+		j = muldiv(unitsx, 65536, hy - ly);
 	} else
 	{
-		i = mini((unitsx << 16) / (hx - lx), (unitsy << 16) / (hy - ly));
-		j = mini((unitsx << 16) / (hy - ly), (unitsy << 16) / (hx - lx));
+		i = mini(muldiv(unitsx, 65536, hx - lx), muldiv(unitsy, 65536, hy - ly));
+		j = mini(muldiv(unitsx, 65536, hy - ly), muldiv(unitsy, 65536, hx - lx));
 	}
 	if (rotateplot != 0) i = j;
 	io_psmatrix[0][0] = i;   io_psmatrix[0][1] = 0;
 	io_psmatrix[1][0] = 0;   io_psmatrix[1][1] = i;
-	io_psmatrix[2][0] = - (i*cx >> 16) + unitsx / 2 + pagemarginps * PSSCALE;
+	io_psmatrix[2][0] = - muldiv(i, cx, 65536) + unitsx / 2 + pagemarginps * PSSCALE;
 	if (useplotter != 0)
 	{
-		io_psmatrix[2][1] = - (i*ly >> 16) + pagemarginps * PSSCALE;
+		io_psmatrix[2][1] = - muldiv(i, ly, 65536) + pagemarginps * PSSCALE;
 	} else
 	{
-		io_psmatrix[2][1] = - (i*cy >> 16) + unitsy / 2 + pagemarginps * PSSCALE;
+		io_psmatrix[2][1] = - muldiv(i, cy, 65536) + unitsy / 2 + pagemarginps * PSSCALE;
 	}
 
 	/* write PostScript header */
 	if (epsformat) io_pswrite("%%!PS-Adobe-2.0 EPSF-2.0\n"); else
 		io_pswrite("%%!PS-Adobe-1.0\n");
-	(void)initinfstr();
-	(void)addstringtoinfstr(describenodeproto(np));
-	facetname = returninfstr();
-	io_pswrite("%%%%Title: %s\n", facetname);
+	io_pswrite("%%%%Title: %s\n", describenodeproto(np));
 	io_pswrite("%%%%Creator: Electric VLSI Design System version %s\n", el_version);
 	curdate = getcurrenttime();
 	io_pswrite("%%%%CreationDate: %s\n", timetostring(curdate));
@@ -442,18 +412,14 @@ INTSML io_pswritefacet(NODEPROTO *np, char *synchronize, INTBIG printit)
 	bbhy = roundfloat(((float)bbhy) / (PSSCALE * 75.0f) * 72.0f) * (bbhy>=0 ? 1 : -1);
 
 	/*
-	 * SRP920115 Write in EPS format:
-	 * if the image is rotated on the page then subtract the offset from bblx and bbhx,
-	 * otherwise add the offset.  Also increase the size of the bbox by one "pixel" to
+	 * Increase the size of the bbox by one "pixel" to
 	 * prevent the edges from being obscured by some drawing tools
 	 */
-	if (rotateplot != 0)
-		io_pswrite("%%%%BoundingBox: %ld %ld %ld %ld\n",bblx-1, bbly-1, bbhx+1, bbhy+1); else
-			io_pswrite("%%%%BoundingBox: %ld %ld %ld %ld\n",bblx-1, bbly-1, bbhx+1, bbhy+1);
+	io_pswrite("%%%%BoundingBox: %ld %ld %ld %ld\n", bblx-1, bbly-1, bbhx+1, bbhy+1);
 	io_pswrite("%%%%DocumentFonts: Times-Roman\n");
 	io_pswrite("%%%%EndComments\n");
 
-	/* PostScript: add some debugging and EPS info */
+	/* PostScript: add some debugging info */
 	if (np != NONODEPROTO)
 	{
 		io_pswrite("%% facet dimensions: %ld wide x %ld high (database units)\n",
@@ -562,9 +528,15 @@ INTSML io_pswritefacet(NODEPROTO *np, char *synchronize, INTBIG printit)
 			io_pspolygontotal *= (io_maxpslayer+1);
 			for(i=0; i<io_maxpslayer; i++)
 			{
+				if (stopping(STOPREASONDECK)) break;
 				io_whichlayer = io_nextplotlayer(i) + 1;
 				(void)askaid(us_aid, "display-to-routine", io_pspoly);
 			}
+			if (stopping(STOPREASONDECK))
+			{
+				xclose(io_psout);
+				return(0);
+			}
 			io_whichlayer = 0;
 			(void)askaid(us_aid, "display-to-routine", io_pspoly);
 		} else
@@ -682,8 +654,8 @@ INTSML io_pswritefacet(NODEPROTO *np, char *synchronize, INTBIG printit)
 		var = getvalkey((INTBIG)us_aid, VAID, VINTEGER, us_gridfloats);
 		if (var == NOVARIABLE || var->addr == 0)
 		{
-			gridlx = np->lowx  / gridx * gridx;
-			gridly = np->lowy  / gridy * gridy;
+			gridlx = lx  / gridx * gridx;
+			gridly = ly  / gridy * gridy;
 		} else
 		{
 			grabpoint(np, &gridlx, &gridly);
@@ -694,14 +666,11 @@ INTSML io_pswritefacet(NODEPROTO *np, char *synchronize, INTBIG printit)
 		}
 
 		/* adjust to ensure that the first point is inside the range */
+		if (gridlx > lx) gridlx -= (gridlx - lx) / gridx * gridx;
+		if (gridly > ly) gridly -= (gridly - ly) / gridy * gridy;
 		while (gridlx < lx) gridlx += gridx;
 		while (gridly < ly) gridly += gridy;
 
-		/* plot the loop to the printer */
-		for (cx = gridlx; cx <= hx; cx += gridx)
-			for (cy = gridly; cy <= hy; cy += gridy)
-				io_psdot(cx, cy);
-
 		/* PostScript: write the grid loop */
 		io_pswrite("%ld %ld %ld\n{\n", gridlx, gridx, hx);
 		io_pswrite("    %ld %ld %ld\n    {\n", gridly, gridy, hy);	/* x y */
@@ -735,34 +704,25 @@ INTSML io_pswritefacet(NODEPROTO *np, char *synchronize, INTBIG printit)
 	/* put out dates if requested */
 	if (plotdates != 0)
 	{
-		/* create the polygon if it doesn't exist */
-		if (poly == NOPOLYGON) poly = allocstaticpolygon(1, io_aid->cluster);
+		(void)initinfstr();
+		(void)formatinfstr(_("Facet: %s"), describenodeproto(np));
+		io_pswrite("%s%ld %ld ", io_psprefix, io_pswindow.uselx,
+			io_pswindow.usely + 2 * CORNERDATESIZE * PSSCALE);
+		io_pswritestring(returninfstr());
+		io_pswrite(" %ld Botleftstring\n", CORNERDATESIZE * PSSCALE);
 
-		/* plot facet name */
-		io_whichlayer = -1;
-		poly->string = facetname;
-		poly->xv[0] = np->highx;
-		poly->yv[0] = np->lowy;
-		poly->count = 1;
-		poly->style = TEXTBOTRIGHT;
-		poly->font = TXTMEDIUM;
-		poly->tech = el_curtech;
-		poly->desc = &io_psblack;
-		(void)io_pspoly(poly, el_curwindowpart);
-
-		/* plot creation date */
 		(void)initinfstr();
 		(void)formatinfstr(_("Created: %s"), timetostring(np->creationdate));
-		poly->string = returninfstr();
-		poly->yv[0] = np->lowy + (np->highy-np->lowy) / 20;
-		(void)io_pspoly(poly, el_curwindowpart);
+		io_pswrite("%s%ld %ld ", io_psprefix, io_pswindow.uselx,
+			io_pswindow.usely + CORNERDATESIZE * PSSCALE);
+		io_pswritestring(returninfstr());
+		io_pswrite(" %ld Botleftstring\n", CORNERDATESIZE * PSSCALE);
 
-		/* plot revision date */
 		(void)initinfstr();
 		(void)formatinfstr(_("Revised: %s"), timetostring(np->revisiondate));
-		poly->string = returninfstr();
-		poly->yv[0] = np->lowy + (np->highy-np->lowy) / 10;
-		(void)io_pspoly(poly, el_curwindowpart);
+		io_pswrite("%s%ld %ld ", io_psprefix, io_pswindow.uselx, io_pswindow.usely);
+		io_pswritestring(returninfstr());
+		io_pswrite(" %ld Botleftstring\n", CORNERDATESIZE * PSSCALE);
 	}
 
 	io_pswrite("showpage\n");
@@ -847,7 +807,7 @@ char *io_psfacetname(NODEPROTO *np)
 		(void)addtoinfstr('_');
 		if (io_whichlayer == 0) (void)addstringtoinfstr("REST"); else
 		{
-			ch = layername(el_curtech, (INTSML)(io_whichlayer-1));
+			ch = layername(el_curtech, io_whichlayer-1);
 			(void)addstringtoinfstr(ch);
 		}
 	}
@@ -862,6 +822,7 @@ void io_psdumpfacets(NODEPROTO *np)
 	REGISTER NODEINST *ni;
 	REGISTER INTBIG i;
 
+	if (stopping(STOPREASONDECK)) return;
 	if (np->temp1 != 0) return;
 	np->temp1 = 1;
 	for(ni = np->firstnodeinst; ni != NONODEINST; ni = ni->nextnodeinst)
@@ -916,6 +877,7 @@ void io_psdumpcontents(NODEPROTO *np)
 	INTSML (*savepolyroutine)(POLYGON*, WINDOWPART*);
 	INTBIG savestate;
 
+	if (stopping(STOPREASONDECK)) return;
 	if (io_psfake != 0) io_psopcount += 2; else
 		io_pswrite("/%s {\n", io_psfacetname(np));
 	if (poly == NOPOLYGON) poly = allocstaticpolygon(4, io_aid->cluster);
@@ -932,8 +894,8 @@ void io_psdumpcontents(NODEPROTO *np)
 		/* expanded: invoke lower level facet */
 		tx = (ni->lowx + ni->highx - ni->proto->lowx - ni->proto->highx)/2;
 		ty = (ni->lowy + ni->highy - ni->proto->lowy - ni->proto->highy)/2;
-		ptx = ((tx * io_psmatrix[0][0] + ty * io_psmatrix[1][0]) >> 16);
-		pty = ((tx * io_psmatrix[0][1] + ty * io_psmatrix[1][1]) >> 16);
+		ptx = muldiv(tx, io_psmatrix[0][0], 65536) + muldiv(ty, io_psmatrix[1][0], 65536);
+		pty = muldiv(tx, io_psmatrix[0][1], 65536) + muldiv(ty, io_psmatrix[1][1], 65536);
 
 		if (ni->rotation == 0 && ni->transpose == 0)
 		{
@@ -991,7 +953,7 @@ void io_psdumpcontents(NODEPROTO *np)
 				io_pspoly(poly, el_curwindowpart);
 
 				poly->style = TEXTCENT;
-				poly->font = TXTLARGE;
+				poly->font = TXT4L;
 				poly->string = describenodeproto(ni->proto);
 				io_pspoly(poly, el_curwindowpart);
 			}
@@ -999,7 +961,7 @@ void io_psdumpcontents(NODEPROTO *np)
 		{
 			/* primitive */
 			makerot(ni, trans);
-			tot = nodepolys(ni, 0);
+			tot = nodepolys(ni, 0, NOWINDOWPART);
 			for(i=0; i<tot; i++)
 			{
 				shapenodepoly(ni, i, poly);
@@ -1018,7 +980,7 @@ void io_psdumpcontents(NODEPROTO *np)
 	/* now write arcs */
 	for(ai = np->firstarcinst; ai != NOARCINST; ai = ai->nextarcinst)
 	{
-		tot = arcpolys(ai);
+		tot = arcpolys(ai, NOWINDOWPART);
 		for(i=0; i<tot; i++)
 		{
 			shapearcpoly(ai, (INTSML)i, poly);
@@ -1049,7 +1011,8 @@ void io_psdumpcontents(NODEPROTO *np)
 		{
 			makerot(pp->subnodeinst, trans);
 			us_writeprotoname(pp, LAYERA, trans, LAYERO,
-				el_colfacettxt, el_curwindowpart, 0, 0, 0, 0);
+				el_colfacettxt, el_curwindowpart, 0, 0, 0, 0,
+					(us_useroptions&PORTLABELS) >> PORTLABELSSH);
 			us_drawportprotovariables(pp, 1, trans, el_curwindowpart);
 		}
 
@@ -1242,7 +1205,7 @@ INTSML io_pspoly(POLYGON *poly, WINDOWPART *win)
 		case TEXTBOTRIGHT:
 		case TEXTBOX:
 			tech = win->curnodeproto->tech;
-			font = truefontsize(poly->font, el_curwindowpart, tech);
+			font = truefontsize(poly->font, win, tech);
 			getbbox(poly, &xl, &xh, &yl, &yh);
 			io_pstext(poly->style, xl, xh, yl, yh, font, poly->string);
 			break;
@@ -1330,7 +1293,7 @@ void io_psputheader(INTBIG which)
 	static char *stringheader[] =
 	{
 		/*
-		 * CS901126  Added code to do super and subscripts:
+		 * Code to do super and subscripts:
 		 *
 		 * example:
 		 *	"NORMAL\dSUB\}   NORMAL\uSUP\}"
@@ -1433,7 +1396,7 @@ void io_psputheader(INTBIG which)
 		"} def",
 
 		"/Centerstring {",								/* x y str sca */
-		"    dup /TSize exch def",						/* CS901126 save size */
+		"    dup /TSize exch def",						/* save size */
 		"    dup scaleFont exch dup TSize StringLength", /* x y sca str xw yw */
 		"    pop 3 -1 roll .8 mul",						/* x y str xw sca*.8 */
 		"    exch 5 -1 roll exch 2 div sub",			/* y str sca*.8 x-xw/2 */
@@ -1442,7 +1405,7 @@ void io_psputheader(INTBIG which)
 		"} def",
 
 		"/Topstring {",									/* x y str sca */
-		"    dup /TSize exch def",						/* CS901126 save size */
+		"    dup /TSize exch def",						/* save size */
 		"    dup scaleFont exch dup TSize StringLength", /* x y sca str xw yw */
 		"    pop 3 -1 roll .8 mul",						/* x y str xw sca*.8 */
 		"    exch 5 -1 roll exch 2 div sub",			/* y str sca*.8 x-xw/2 */
@@ -1451,14 +1414,14 @@ void io_psputheader(INTBIG which)
 		"} def",
 
 		"/Botstring {",									/* x y str sca */
-		"    dup /TSize exch def",						/* CS901126 save size */
+		"    dup /TSize exch def",						/* save size */
 		"    scaleFont dup TSize StringLength pop",		/* x y str xw */
 		"    4 -1 roll exch 2 div sub",					/* y str x-xw/2 */
 		"    3 -1 roll moveto TSize StringShow",
 		"} def",
 
 		"/Leftstring {",								/* x y str sca */
-		"    dup /TSize exch def",						/* CS901126 save size */
+		"    dup /TSize exch def",						/* save size */
 		"    dup scaleFont .4 mul",						/* x y str sca*.4 */
 		"    3 -1 roll exch sub",						/* x str y-sca*.4 */
 		"    3 -1 roll exch",							/* str x y-sca*.4 */
@@ -1466,7 +1429,7 @@ void io_psputheader(INTBIG which)
 		"} def",
 
 		"/Rightstring {",								/* x y str sca */
-		"    dup /TSize exch def",						/* CS901126 save size */
+		"    dup /TSize exch def",						/* save size */
 		"    dup scaleFont exch dup TSize StringLength", /* x y sca str xw yw */
 		"    pop 3 -1 roll .4 mul",						/* x y str xw sca*.4 */
 		"    exch 5 -1 roll exch sub",					/* y str sca*.4 x-xw */
@@ -1475,7 +1438,7 @@ void io_psputheader(INTBIG which)
 		"} def",
 
 		"/Topleftstring {",								/* x y str sca */
-		"    dup /TSize exch def",						/* CS901126 save size */
+		"    dup /TSize exch def",						/* save size */
 		"    dup scaleFont .8 mul",						/* x y str sca*.8 */
 		"    3 -1 roll exch sub",						/* x str y-sca*.8 */
 		"    3 -1 roll exch",							/* str x y-sca*.8 */
@@ -1483,7 +1446,7 @@ void io_psputheader(INTBIG which)
 		"} def",
 
 		"/Toprightstring {",							/* x y str sca */
-		"    dup /TSize exch def",						/* CS901126 save size */
+		"    dup /TSize exch def",						/* save size */
 		"    dup scaleFont exch dup TSize StringLength", /* x y sca str xw yw */
 		"    pop 3 -1 roll .8 mul",						/* x y str xw sca*.8 */
 		"    exch 5 -1 roll exch sub",					/* y str sca*.8 x-xw */
@@ -1492,12 +1455,12 @@ void io_psputheader(INTBIG which)
 		"} def",
 
 		"/Botleftstring {",								/* x y str sca */
-		"    dup /TSize exch def",						/* CS901126 save size */
+		"    dup /TSize exch def",						/* save size */
 		"    scaleFont 3 1 roll moveto TSize StringShow",
 		"} def",
 
 		"/Botrightstring {",							/* x y str sca */
-		"    dup /TSize exch def",						/* CS901126 save size */
+		"    dup /TSize exch def",						/* save size */
 		"    scaleFont dup TSize StringLength",
 		"    pop 4 -1 roll exch",
 		"    sub 3 -1 roll",
@@ -1511,7 +1474,7 @@ void io_psputheader(INTBIG which)
 		"} def",
 
 		"/Boxstring {",									/* x y mx my str sca */
-		"    dup /TSize exch def",						/* CS 901126 save size */
+		"    dup /TSize exch def",						/* save size */
 		"    dup scaleFont",							/* x y mx my str sca */
 		"    exch dup TSize StringLength pop",			/* x y mx my sca str xw */
 		"    3 -1 roll dup",							/* x y mx my str xw sca sca */
@@ -1589,13 +1552,25 @@ void io_psline(INTBIG x1, INTBIG y1, INTBIG x2, INTBIG y2, INTBIG pattern)
 	i = PSSCALE / 2;
 	switch (pattern)
 	{
-		case 1: io_pswrite("%s[%ld %ld] 0 setdash ", io_psprefix, i, i*3);    break;
-		case 2: io_pswrite("%s[%ld %ld] 6 setdash ", io_psprefix, i*6, i*3);  break;
-		case 3: io_pswrite("%s[%ld %ld] 0 setdash ", io_psprefix, i, i*7);    break;
+		case 0:
+			io_pswrite("%s%ld %ld %ld %ld Drawline\n", io_psprefix, psx1, psy1, psx2, psy2);
+			break;
+		case 1:
+			io_pswrite("%s[%ld %ld] 0 setdash ", io_psprefix, i, i*3);
+			io_pswrite("%s%ld %ld %ld %ld Drawline ", io_psprefix, psx1, psy1, psx2, psy2);
+			io_pswrite("%s [] 0 setdash\n", io_psprefix);
+			break;
+		case 2:
+			io_pswrite("%s[%ld %ld] 6 setdash ", io_psprefix, i*6, i*3);
+			io_pswrite("%s%ld %ld %ld %ld Drawline ", io_psprefix, psx1, psy1, psx2, psy2);
+			io_pswrite("%s [] 0 setdash\n", io_psprefix);
+			break;
+		case 3:
+			io_pswrite("%ld setlinewidth ", PSSCALE);
+			io_pswrite("%s%ld %ld %ld %ld Drawline ", io_psprefix, psx1, psy1, psx2, psy2);
+			io_pswrite("%ld setlinewidth\n", PSSCALE/2);
+			break;
 	}
-	io_pswrite("%s%ld %ld %ld %ld Drawline", io_psprefix, psx1, psy1, psx2, psy2);
-	if (pattern != 0) io_pswrite("%s [] 0 setdash", io_psprefix);
-	io_pswrite("\n");
 }
 
 /* draw an arc of a circle */
@@ -1756,19 +1731,20 @@ INTSML io_pspattern(GRAPHICS *col)
 	}
 	io_psgraphicsseen[io_psgraphicsseencount++] = col;
 
-	/* CS900228 Generate filled polygons by defining a stipple font,
-				and then tiling the polygon to fill with 128x128 pixel
-				characters, clipping to the polygon edge.
-
-				Take ELECTRIC's 16x8 bit images, double each bit,
-				and then output 4 times to get 128 bit wide image.
-				Double vertically by outputting each row twice.
-				Note that full vertical size need not be generated,
-				as PostScript will just reuse the lines until the 128
-				size is reached.
-
-				ref Making a User Defined Font:  PostScript Cookbook
-	*/
+	/*
+	 * Generate filled polygons by defining a stipple font,
+	 * and then tiling the polygon to fill with 128x128 pixel
+	 * characters, clipping to the polygon edge.
+	 *
+	 * Take Electric's 16x8 bit images, double each bit,
+	 * and then output 4 times to get 128 bit wide image.
+	 * Double vertically by outputting each row twice.
+	 * Note that full vertical size need not be generated,
+	 * as PostScript will just reuse the lines until the 128
+	 * size is reached.
+	 *
+	 * see: "Making a User Defined Font", PostScript Cookbook
+	 */
 	if (io_psfake != 0) io_psopcount += 96; else
 	{
 		io_pswrite("%sStippleFont1 begin\n", io_psprefix);
@@ -1815,7 +1791,10 @@ void io_pstext(INTSML type, INTBIG lx, INTBIG ux, INTBIG ly, INTBIG uy, INTBIG f
 	char *text)
 {
 	char *pt;
-	INTBIG pslx, pshx, psly, pshy;
+	INTBIG pslx, pshx, psly, pshy, fontsize;
+
+	fontsize = fnt * PSSCALE;
+	if (fontsize <= 0) return;
 
 	if (io_psfake != 0)
 	{
@@ -1837,53 +1816,53 @@ void io_pstext(INTSML type, INTBIG lx, INTBIG ux, INTBIG ly, INTBIG uy, INTBIG f
 		case TEXTCENT:
 			io_pswrite("%s%ld %ld ", io_psprefix, (pslx+pshx)/2, (psly+pshy)/2);
 			io_pswritestring(text);
-			io_pswrite(" %ld Centerstring\n", io_psconvert[fnt]*PSSCALE);
+			io_pswrite(" %ld Centerstring\n", fontsize);
 			break;
 		case TEXTTOP:
 			io_pswrite("%s%ld %ld ", io_psprefix, (pslx+pshx)/2, pshy);
 			io_pswritestring(text);
-			io_pswrite(" %ld Topstring\n", io_psconvert[fnt]*PSSCALE);
+			io_pswrite(" %ld Topstring\n", fontsize);
 			break;
 		case TEXTBOT:
 			io_pswrite("%s%ld %ld ", io_psprefix, (pslx+pshx)/2, psly);
 			io_pswritestring(text);
-			io_pswrite(" %ld Botstring\n", io_psconvert[fnt]*PSSCALE);
+			io_pswrite(" %ld Botstring\n", fontsize);
 			break;
 		case TEXTLEFT:
 			io_pswrite("%s%ld %ld ", io_psprefix, pslx, (psly+pshy)/2);
 			io_pswritestring(text);
-			io_pswrite(" %ld Leftstring\n", io_psconvert[fnt]*PSSCALE);
+			io_pswrite(" %ld Leftstring\n", fontsize);
 			break;
 		case TEXTRIGHT:
 			io_pswrite("%s%ld %ld ", io_psprefix, pshx, (psly+pshy)/2);
 			io_pswritestring(text);
-			io_pswrite(" %ld Rightstring\n", io_psconvert[fnt]*PSSCALE);
+			io_pswrite(" %ld Rightstring\n", fontsize);
 			break;
 		case TEXTTOPLEFT:
 			io_pswrite("%s%ld %ld ", io_psprefix, pslx, pshy);
 			io_pswritestring(text);
-			io_pswrite(" %ld Topleftstring\n", io_psconvert[fnt]*PSSCALE);
+			io_pswrite(" %ld Topleftstring\n", fontsize);
 			break;
 		case TEXTTOPRIGHT:
 			io_pswrite("%s%ld %ld ", io_psprefix, pshx, pshy);
 			io_pswritestring(text);
-			io_pswrite(" %ld Toprightstring\n", io_psconvert[fnt]*PSSCALE);
+			io_pswrite(" %ld Toprightstring\n", fontsize);
 			break;
 		case TEXTBOTLEFT:
 			io_pswrite("%s%ld %ld ", io_psprefix, pslx, psly);
 			io_pswritestring(text);
-			io_pswrite(" %ld Botleftstring\n", io_psconvert[fnt]*PSSCALE);
+			io_pswrite(" %ld Botleftstring\n", fontsize);
 			break;
 		case TEXTBOTRIGHT:
 			io_pswrite("%s%ld %ld ", io_psprefix, pshx, psly);
 			io_pswritestring(text);
-			io_pswrite(" %ld Botrightstring\n", io_psconvert[fnt]*PSSCALE);
+			io_pswrite(" %ld Botrightstring\n", fontsize);
 			break;
 		case TEXTBOX:
 			io_pswrite("%s%ld %ld %ld %ld ", io_psprefix, (pslx+pshx)/2,
 				(psly+pshy)/2, pshx-pslx, pshy-psly);
 			io_pswritestring(text);
-			io_pswrite(" %ld Boxstring\n", io_psconvert[fnt]*PSSCALE);
+			io_pswrite(" %ld Boxstring\n", fontsize);
 			break;
 	}
 }
@@ -1895,8 +1874,8 @@ void io_pstext(INTSML type, INTBIG lx, INTBIG ux, INTBIG ly, INTBIG uy, INTBIG f
  */
 void io_psxform(INTBIG x, INTBIG y, INTBIG *psx, INTBIG *psy)
 {
-	*psx = ((x * io_psmatrix[0][0] + y * io_psmatrix[1][0]) >> 16) + io_psmatrix[2][0];
-	*psy = ((x * io_psmatrix[0][1] + y * io_psmatrix[1][1]) >> 16) + io_psmatrix[2][1];
+	*psx = muldiv(x, io_psmatrix[0][0], 65536) + muldiv(y, io_psmatrix[1][0], 65536) + io_psmatrix[2][0];
+	*psy = muldiv(x, io_psmatrix[0][1], 65536) + muldiv(y, io_psmatrix[1][1], 65536) + io_psmatrix[2][1];
 }
 
 void io_pswritestring(char *str)
