@@ -42,7 +42,7 @@
 #include <Scrap.h>
 #include <QDOffscreen.h>
 
-#define	THRESH	         2000000
+#define THRESH	         2000000
 #ifdef __MWERKS__
 #  if __MWERKS__ >= 0x2300
 #    define NEWCODEWARRIOR    1
@@ -64,7 +64,6 @@ static INTBIG      io_slx, io_shx, io_sly, io_shy;
 static INTBIG      io_ulx, io_uhx, io_uly, io_uhy;
 static WindowPtr   io_plotwin;
 static INTSML      io_thislayer, io_maxlayer;
-static INTSML      io_fontconvert[9] = {6, 7, 9, 10, 12, 14, 18, 20, 24};
 
 void io_setplotlayer(INTBIG);
 void io_setbounds(INTBIG, INTBIG, INTBIG, INTBIG, INTBIG, INTBIG, INTBIG, INTBIG, INTBIG, WindowPtr);
@@ -72,7 +71,7 @@ INTSML io_plotpoly(POLYGON*, WINDOWPART*);
 void io_plotdot(INTBIG, INTBIG);
 void io_plotline(INTBIG, INTBIG, INTBIG, INTBIG, INTSML);
 void io_plotpolygon(INTBIG*, INTBIG*, INTSML, INTSML, GRAPHICS*);
-void io_plottext(INTSML, INTBIG, INTBIG, INTBIG, INTBIG, INTSML, char*);
+void io_plottext(INTSML, INTBIG, INTBIG, INTBIG, INTBIG, INTBIG, char*);
 void io_xform(INTBIG*, INTBIG*);
 void io_maccopyhighlighted(void);
 
@@ -81,7 +80,7 @@ void io_maccopyhighlighted(void);
  */
 INTSML io_writequickdrawlibrary(LIBRARY *lib)
 {
-	REGISTER NODEPROTO *np, *onp;
+	REGISTER NODEPROTO *np;
 	INTSML err, ix, swid, shei;
 	INTBIG hlx, hhx, hly, hhy, gridlx, gridly, gridx, gridy, cx, cy, prod1, prod2, i,
 		revy, slx, shx, sly, shy, ulx, uhx, uly, uhy, sxs, sys, dxs, dys, saver, saveg,
@@ -102,39 +101,11 @@ INTSML io_writequickdrawlibrary(LIBRARY *lib)
 		return(1);
 	}
 
-	/* determine area to plot */
-	if (framesize(&shx, &shy, np) == 0)
-	{
-		shx /= 2;   slx = -shx;
-		shy /= 2;   sly = -shy;
-	} else
-	{
-		slx = np->lowx;   shx = np->highx;
-		sly = np->lowy;   shy = np->highy;
-	}
-
-	/* see if focusing on highlighted area */
+	/* get printing control bits */
 	curstate = io_getstatebits();
-	if ((curstate[0]&PLOTFOCUS) != 0)
-	{
-		if ((curstate[0]&PLOTFOCUSDPY) != 0)
-		{
-			slx = el_curwindowpart->screenlx;
-			shx = el_curwindowpart->screenhx;
-			sly = el_curwindowpart->screenly;
-			shy = el_curwindowpart->screenhy;
-		} else
-		{
-			onp = (NODEPROTO *)askaid(us_aid, "get-highlighted-area", (INTBIG)&hlx,
-				(INTBIG)&hhx, (INTBIG)&hly, (INTBIG)&hhy);
-			if (onp == NONODEPROTO)
-				ttyputerr(_("Warning: no highlighted area; plotting entire facet")); else
-			{
-				slx = hlx;   shx = hhx;
-				sly = hly;   shy = hhy;
-			}
-		}
-	}
+
+	/* determine area to plot */
+	if (io_getareatoprint(np, &slx, &shx, &sly, &shy, 1) != 0) return(1);
 
 	/* open the window (to the printer) */
 	PrOpen();
@@ -356,7 +327,7 @@ INTSML io_writequickdrawlibrary(LIBRARY *lib)
 		poly->yv[0] = np->lowy;
 		poly->count = 1;
 		poly->style = TEXTBOTRIGHT;
-		poly->font = TXTSMALL;
+		poly->font = TXT1L;
 		poly->tech = el_curtech;
 		poly->desc = &io_black;
 		(void)io_plotpoly(poly, el_curwindowpart);
@@ -429,8 +400,8 @@ void io_setbounds(INTBIG slx, INTBIG shx, INTBIG sly, INTBIG shy, INTBIG ulx, IN
  */
 INTSML io_plotpoly(POLYGON *poly, WINDOWPART *win)
 {
-	REGISTER INTSML k, type, font;
-	INTBIG xl, xh, yl, yh, x, y, listx[4], listy[4],
+	REGISTER INTSML k, type;
+	INTBIG xl, xh, yl, yh, x, y, listx[4], listy[4], font,
 		centerx, centery, x1p, y1p, x2p, y2p, xa, ya, xb, yb, radius;
 	INTSML startangle, endangle, amt, style;
 	unsigned char pat[8];
@@ -717,10 +688,8 @@ void io_plotline(INTBIG x1, INTBIG y1, INTBIG x2, INTBIG y2, INTSML pattern)
 	static unsigned char pat1v[] = {0x88, 0x88, 0x88, 0x88, 0x88, 0x88, 0x88, 0x88};
 	static unsigned char pat2h[] = {0xFF, 0xFF, 0xFF, 0, 0, 0xFF, 0xFF, 0xFF};
 	static unsigned char pat2v[] = {0xE7, 0xE7, 0xE7, 0xE7, 0xE7, 0xE7, 0xE7, 0xE7};
-	static unsigned char pat3h[] = {0xFF, 0, 0, 0, 0, 0, 0, 0};
-	static unsigned char pat3v[] = {0x80, 0x80, 0x80, 0x80, 0x80, 0x80, 0x80, 0x80};
-	static unsigned char *patlisth[] = {pat0, pat1h, pat2h, pat3h};
-	static unsigned char *patlistv[] = {pat0, pat1v, pat2v, pat3v};
+	static unsigned char *patlisth[] = {pat0, pat1h, pat2h, pat0};
+	static unsigned char *patlistv[] = {pat0, pat1v, pat2v, pat0};
 
 	io_xform(&x1, &y1);
 	io_xform(&x2, &y2);
@@ -730,8 +699,10 @@ void io_plotline(INTBIG x1, INTBIG y1, INTBIG x2, INTBIG y2, INTSML pattern)
 	PenMode(patCopy);
 	if (abs(x1-x2) > abs(y1-y2)) PenPat((PatPtr)patlistv[pattern]); else
 		PenPat((PatPtr)patlisth[pattern]);
+	if (pattern == 3) PenSize(3, 3);
 	MoveTo(x1, io_plotrevy-y1);
 	LineTo(x2, io_plotrevy-y2);
+	if (pattern == 3) PenSize(1, 1);
 }
 
 void io_plotpolygon(INTBIG *x, INTBIG *y, INTSML count, INTSML style, GRAPHICS *desc)
@@ -775,11 +746,11 @@ void io_plotpolygon(INTBIG *x, INTBIG *y, INTSML count, INTSML style, GRAPHICS *
 	KillPoly(polyhandle);
 }
 
-void io_plottext(INTSML type, INTBIG lx, INTBIG ux, INTBIG ly, INTBIG uy, INTSML fnt,
+void io_plottext(INTSML type, INTBIG lx, INTBIG ux, INTBIG ly, INTBIG uy, INTBIG fnt,
 	char *text)
 {
 	char localstring[256];
-	INTSML len, i, tsx, tsy, px, py, font;
+	INTBIG len, i, tsx, tsy, px, py, font;
 	FontInfo fontinfo;
 
 	io_xform(&lx, &ly);
@@ -791,9 +762,9 @@ void io_plottext(INTSML type, INTBIG lx, INTBIG ux, INTBIG ly, INTBIG uy, INTSML
 	if (type == TEXTBOX)
 	{
 		/* scan for a font that fits */
-		for(font=fnt; font>=TXT4P; font--)
+		for(font = fnt; font >= 4; font--)
 		{
-			TextSize(io_fontconvert[font]);
+			TextSize(font);
 			GetFontInfo(&fontinfo);
 			tsx = TextWidth(localstring, 0, len);
 			tsy = fontinfo.ascent + fontinfo.descent + fontinfo.leading;
@@ -821,7 +792,7 @@ void io_plottext(INTSML type, INTBIG lx, INTBIG ux, INTBIG ly, INTBIG uy, INTSML
 		}
 	} else
 	{
-		TextSize(io_fontconvert[fnt]);
+		TextSize(fnt);
 		GetFontInfo(&fontinfo);
 		tsx = TextWidth(localstring, 0, len);
 		tsy = fontinfo.ascent + fontinfo.descent + fontinfo.leading;
