@@ -44,11 +44,11 @@
 
 /* #define SHOWORIGINAL 1 */		/* uncomment to show original nodes */
 
-#define	E0     (WHOLE/8)	/*  0.125 */
-#define	S1     (T1+E0)		/*  1.875 */
+#define E0     (WHOLE/8)	/*  0.125 */
+#define S1     (T1+E0)		/*  1.875 */
 #define F3     (K3+H0+E0)	/*  3.625 */
-#define	S6     (T6+E0)		/*  6.875 */
-#define	Q11    (K11+Q0)		/* 11.25 */
+#define S6     (T6+E0)		/*  6.875 */
+#define Q11    (K11+Q0)		/* 11.25 */
 
 /*************** SUE EQUIVALENCES ***************/
 
@@ -67,8 +67,7 @@ typedef struct
 	INTSML        rotation;
 	INTBIG        xoffset;
 	INTBIG        yoffset;
-	char         *detailvar;
-	char         *detailvalue;
+	INTBIG        detailbits;
 	INTBIG        numextrawires;
 	SUEEXTRAWIRE *extrawires;
 } SUEEQUIV;
@@ -108,24 +107,24 @@ SUEEXTRAWIRE io_suetwoportwires[] =
 
 SUEEQUIV io_sueequivs[] =
 {
-	/* name         primitive          NEG ANG    X   Y  VAR                      VALUE  WIR EXTRA-WIRES */
-	{"pmos10",     &sch_transistorprim, 0, 900, -K2,  0, "SCHEM_transistor_type", "pmos", 3, io_suetransistorwires},
-	{"pmos4",      &sch_transistorprim, 0, 900, -K2,  0, "SCHEM_transistor_type", "pmos", 3, io_suetransistorwires},
-	{"pmos",       &sch_transistorprim, 0, 900, -K2,  0, "SCHEM_transistor_type", "pmos", 3, io_suetransistorwires},
-	{"nmos10",     &sch_transistorprim, 0, 900, -K2,  0, "SCHEM_transistor_type", "nmos", 3, io_suetransistorwires},
-	{"nmos4",      &sch_transistorprim, 0, 900, -K2,  0, "SCHEM_transistor_type", "nmos", 3, io_suetransistorwires},
-	{"nmos",       &sch_transistorprim, 0, 900, -K2,  0, "SCHEM_transistor_type", "nmos", 3, io_suetransistorwires},
-	{"capacitor",  &sch_capacitorprim,  0,   0,   0,  0, 0,                       0,      2, io_suecapacitorwires},
-	{"resistor",   &sch_resistorprim,   0, 900,   0,  0, 0,                       0,      2, io_sueresistorwires},
-	{"inductor",   &sch_inductorprim,   0,   0,   0,  0, 0,                       0,      0, 0},
-	{"dc_supply",  &sch_sourceprim,     0,   0,   0, H0, "SCHEM_source",          "v",    2, io_suesourcewires},
-	{"dc_Isupply", &sch_sourceprim,     0,   0,   0, H0, "SCHEM_source",          "c",    2, io_suesourcewires},
-	{"ammeter",    &sch_sourceprim,     0,   0,   0,  0, "SCHEM_source",          "cm",   0, 0},
-	{"cccs",       &sch_twoportprim,    0,   0,  Q1,-S6, "SCHEM_twoport_type",    "f",    4, io_suetwoportwires},
-	{"ccvs",       &sch_twoportprim,    0,   0,  Q1,-S6, "SCHEM_twoport_type",    "h",    4, io_suetwoportwires},
-	{"vcvs",       &sch_twoportprim,    0,   0,  Q1,-S6, "SCHEM_twoport_type",    "u",    4, io_suetwoportwires},
-	{"vccs",       &sch_twoportprim,    0,   0, -S1,-K5, "SCHEM_twoport_type",    "g",    0, 0},
-	{0, 0, 0, 0, 0, 0, 0, 0, 0}
+	/* name         primitive          NEG ANG    X   Y  BITS         WIR EXTRA-WIRES */
+	{"pmos10",     &sch_transistorprim, 0, 900, -K2,  0, TRANPMOS,     3, io_suetransistorwires},
+	{"pmos4",      &sch_transistorprim, 0, 900, -K2,  0, TRANPMOS,     3, io_suetransistorwires},
+	{"pmos",       &sch_transistorprim, 0, 900, -K2,  0, TRANPMOS,     3, io_suetransistorwires},
+	{"nmos10",     &sch_transistorprim, 0, 900, -K2,  0, TRANNMOS,     3, io_suetransistorwires},
+	{"nmos4",      &sch_transistorprim, 0, 900, -K2,  0, TRANNMOS,     3, io_suetransistorwires},
+	{"nmos",       &sch_transistorprim, 0, 900, -K2,  0, TRANNMOS,     3, io_suetransistorwires},
+	{"capacitor",  &sch_capacitorprim,  0,   0,   0,  0, 0,            2, io_suecapacitorwires},
+	{"resistor",   &sch_resistorprim,   0, 900,   0,  0, 0,            2, io_sueresistorwires},
+	{"inductor",   &sch_inductorprim,   0,   0,   0,  0, 0,            0, 0},
+	{"dc_supply",  &sch_sourceprim,     0,   0,   0, H0, SOURCEVS,     2, io_suesourcewires},
+	{"dc_Isupply", &sch_sourceprim,     0,   0,   0, H0, SOURCECURSRC, 2, io_suesourcewires},
+	{"ammeter",    &sch_sourceprim,     0,   0,   0,  0, SOURCECURMTR, 0, 0},
+	{"cccs",       &sch_twoportprim,    0,   0,  Q1,-S6, TWOPCCCS,     4, io_suetwoportwires},
+	{"ccvs",       &sch_twoportprim,    0,   0,  Q1,-S6, TWOPCCVS,     4, io_suetwoportwires},
+	{"vcvs",       &sch_twoportprim,    0,   0,  Q1,-S6, TWOPVCVS,     4, io_suetwoportwires},
+	{"vccs",       &sch_twoportprim,    0,   0, -S1,-K5, TWOPVCCS,     0, 0},
+	{0, 0, 0, 0, 0, 0, 0, 0}
 };
 
 /*************** SUE WIRES ***************/
@@ -327,13 +326,12 @@ INTSML io_readsuelibrary(LIBRARY *lib)
 NODEPROTO *io_suereadfile(LIBRARY *lib, char *facetname)
 {
 	char *keywords[MAXKEYWORDS], *thename, *thelabel, *thetext, *namestring;
-	char suevarname[200], *detailvar, *detailvalue,
-		*pt, *cpt, *startkey, save;
+	char suevarname[200], *pt, *cpt, *startkey, save;
 	char **argnames, **argvalues, **newargnames, **newargvalues;
 	INTBIG outline[MAXICONPOINTS*2], x, y, type, dx, dy, px, py, pinx, piny, xoff, yoff,
 		numargs, namestrlen;
 	REGISTER INTBIG count, i, lx, hx, ly, hy, curly, xshrink, yshrink,
-		varcount, varindex, varoffset, pos, invertoutput, lambda,
+		varcount, varindex, varoffset, pos, invertoutput, lambda, detailbits,
 		keycount, cx, cy, p1x, p1y, p2x, p2y, start, extent, numextrawires, len;
 	INTSML rot, trn, rotation;
 	XARRAY trans;
@@ -468,8 +466,7 @@ NODEPROTO *io_suereadfile(LIBRARY *lib, char *facetname)
 			rotation = 0;
 			xoff = yoff = 0;
 			xshrink = yshrink = 0;
-			detailvar = 0;
-			detailvalue = 0;
+			detailbits = 0;
 			numextrawires = 0;
 			extrawires = 0;
 			if (namesame(keywords[1], "inout") == 0)
@@ -538,8 +535,7 @@ NODEPROTO *io_suereadfile(LIBRARY *lib, char *facetname)
 					yoff = muldiv(yoff, lambda, WHOLE);
 					rot += rotation;
 					if (rot >= 3600) rot -= 3600;
-					detailvar = io_sueequivs[i].detailvar;
-					detailvalue = io_sueequivs[i].detailvalue;
+					detailbits = io_sueequivs[i].detailbits;
 					numextrawires = io_sueequivs[i].numextrawires;
 					extrawires = io_sueequivs[i].extrawires;
 #ifdef SHOWORIGINAL
@@ -548,16 +544,13 @@ NODEPROTO *io_suereadfile(LIBRARY *lib, char *facetname)
 					ly = y - py/2 + yoff;   hy = ly + py;
 					ni = newnodeinst(proto, lx, hx, ly, hy, trn, rot, facet);
 					if (ni == NONODEINST) continue;
-					if (detailvalue != 0)
-						setval((INTBIG)ni, VNODEINST, detailvar,
-							(INTBIG)detailvalue, VSTRING);
+					ni->userbits |= detailbits;
 					endobjectchange((INTBIG)ni, VNODEINST);
 					proto = NONODEPROTO;
 					invertoutput = 0;
 					rotation = 0;
 					xoff = yoff = 0;
-					detailvar = 0;
-					detailvalue = 0;
+					detailbits = 0;
 					numextrawires = 0;
 					extrawires = 0;
 #endif
@@ -623,9 +616,7 @@ NODEPROTO *io_suereadfile(LIBRARY *lib, char *facetname)
 			ly = y - py/2 + yoff;   hy = ly + py;
 			ni = newnodeinst(proto, lx, hx, ly, hy, trn, rot, facet);
 			if (ni == NONODEINST) continue;
-			if (detailvalue != 0)
-				setval((INTBIG)ni, VNODEINST, detailvar,
-					(INTBIG)detailvalue, VSTRING);
+			ni->userbits |= detailbits;
 			ni->temp1 = invertoutput;
 			if (proto->primindex == 0 && proto->cellview == el_iconview)
 				ni->userbits |= NEXPAND;
@@ -679,12 +670,17 @@ NODEPROTO *io_suereadfile(LIBRARY *lib, char *facetname)
 					{
 						endobjectchange((INTBIG)ppt, VPORTPROTO);
 						ppt->textdescript = defaulttextdescript(NOGEOM);
+						ppt->textdescript = (ppt->textdescript & ~VTSIZE) |
+							(defaulttextsize(1) << VTSIZESH);
 					}
 				} else
 				{
 					/* just name the node */
-					(void)setvalkey((INTBIG)ni, VNODEINST, el_node_name, (INTBIG)thename,
+					var = setvalkey((INTBIG)ni, VNODEINST, el_node_name, (INTBIG)thename,
 						VSTRING|VDISPLAY);
+					if (var != NOVARIABLE)
+						var->textdescript = (var->textdescript & ~VTSIZE) |
+							(defaulttextsize(3) << VTSIZESH);
 				}
 			}
 
@@ -727,7 +723,7 @@ NODEPROTO *io_suereadfile(LIBRARY *lib, char *facetname)
 					VSTRING|VDISPLAY);
 				if (var != NOVARIABLE)
 				{
-					var->textdescript = defaulttextdescript(NOGEOM);
+					var->textdescript = defaulttextdescript(ni->geom);
 					varoffset = (ni->highy - ni->lowy) / (varcount+1);
 					pos = (ni->highy - ni->lowy) / 2 - varindex * varoffset;
 					varindex++;
@@ -779,6 +775,8 @@ NODEPROTO *io_suereadfile(LIBRARY *lib, char *facetname)
 			} else
 			{
 				ppt->textdescript = defaulttextdescript(NOGEOM);
+				ppt->textdescript = (ppt->textdescript & ~VTSIZE) |
+					(defaulttextsize(1) << VTSIZESH);
 				ppt->userbits = (ppt->userbits & ~STATEBITS) | type;
 				endobjectchange((INTBIG)ppt, VPORTPROTO);
 			}
@@ -951,7 +949,7 @@ NODEPROTO *io_suereadfile(LIBRARY *lib, char *facetname)
 			ni = newnodeinst(gen_invispinprim, x, x, y, y, 0, 0, facet);
 			if (ni == NONODEINST) continue;
 			var = setvalkey((INTBIG)ni, VNODEINST, art_messagekey, (INTBIG)thelabel, VSTRING|VDISPLAY);
-			if (var != NOVARIABLE) var->textdescript = (TXTSMALL << VTSIZESH) | VTPOSCENT;
+			if (var != NOVARIABLE) var->textdescript = (defaulttextsize(2) << VTSIZESH) | VTPOSCENT;
 			endobjectchange((INTBIG)ni, VNODEINST);
 			continue;
 		}
@@ -967,7 +965,7 @@ NODEPROTO *io_suereadfile(LIBRARY *lib, char *facetname)
 			ni = newnodeinst(gen_invispinprim, x, x, y, y, 0, 0, facet);
 			if (ni == NONODEINST) continue;
 			var = setvalkey((INTBIG)ni, VNODEINST, art_messagekey, (INTBIG)thetext, VSTRING|VDISPLAY);
-			if (var != NOVARIABLE) var->textdescript = (TXTSMALL << VTSIZESH) | VTPOSCENT;
+			if (var != NOVARIABLE) var->textdescript = (defaulttextsize(2) << VTSIZESH) | VTPOSCENT;
 			endobjectchange((INTBIG)ni, VNODEINST);
 			continue;
 		}
@@ -1690,16 +1688,21 @@ NODEPROTO *io_suecreatepseudo(NODEPROTO *proto, INTBIG count)
 			expandedname = io_sueexpandname(pp->protoname, count, i);
 			var = setvalkey((INTBIG)newai, VARCINST, el_arc_name,
 				(INTBIG)expandedname, VSTRING|VDISPLAY);
-			switch (side)
+			if (var != NOVARIABLE)
 			{
-				case 0:		/* left side */
-					var->textdescript = (var->textdescript & ~VTPOSITION) | VTPOSUPLEFT;   break;
-				case 1:		/* right side */
-					var->textdescript = (var->textdescript & ~VTPOSITION) | VTPOSUPRIGHT;  break;
-				case 2:		/* bottom side */
-					var->textdescript = (var->textdescript & ~VTPOSITION) | VTPOSRIGHT;    break;
-				case 3:		/* top side */
-					var->textdescript = (var->textdescript & ~VTPOSITION) | VTPOSRIGHT;    break;
+				var->textdescript = (var->textdescript & ~VTSIZE) |
+					(defaulttextsize(4) << VTSIZESH);
+				switch (side)
+				{
+					case 0:		/* left side */
+						var->textdescript = (var->textdescript & ~VTPOSITION) | VTPOSUPLEFT;   break;
+					case 1:		/* right side */
+						var->textdescript = (var->textdescript & ~VTPOSITION) | VTPOSUPRIGHT;  break;
+					case 2:		/* bottom side */
+						var->textdescript = (var->textdescript & ~VTPOSITION) | VTPOSRIGHT;    break;
+					case 3:		/* top side */
+						var->textdescript = (var->textdescript & ~VTPOSITION) | VTPOSRIGHT;    break;
+				}
 			}
 			endobjectchange((INTBIG)newai, VARCINST);
 		}
@@ -1912,7 +1915,7 @@ PORTPROTO *io_suewiredport(NODEINST *ni, INTBIG *x, INTBIG *y, INTBIG ox, INTBIG
 		(ni->lowy+ni->highy) / 2 != *y) return(NOPORTPROTO);
 
 	/* find port that is closest to OTHER end */
-	bestdist = HUGEINT;
+	bestdist = MAXINTBIG;
 	bestpp = NOPORTPROTO;
 	for(pp = ni->proto->firstportproto; pp != NOPORTPROTO; pp = pp->nextportproto)
 	{
