@@ -33,29 +33,31 @@
 #include "eio.h"
 #include "tecgen.h"
 #include "tecschem.h"
+#include "tecmocmos.h"
+#include "tecmocmossub.h"
 #include "edialogs.h"
 
-#define	REPORTRATE  100		/* number of elements between graphical update */
+#define REPORTRATE  100		/* number of elements between graphical update */
 
 /* state of input (value of "io_txtindata.textlevel") */
-#define	INLIB       1
-#define	INFACET     2
-#define	INPORTPROTO 3
-#define	INNODEINST  4
-#define	INPOR       5
-#define	INARCINST   6
-#define	INARCEND    7
+#define INLIB       1
+#define INFACET     2
+#define INPORTPROTO 3
+#define INNODEINST  4
+#define INPOR       5
+#define INARCINST   6
+#define INARCEND    7
 
 /* state of variable reading (value of "io_txtindata.varpos") */
-#define	INVAID         1
-#define	INVTECHNOLOGY  2
-#define	INVLIBRARY     3
-#define	INVNODEPROTO   4
-#define	INVNODEINST    5
-#define	INVPORTARCINST 6
-#define	INVPORTEXPINST 7
-#define	INVPORTPROTO   8
-#define	INVARCINST     9
+#define INVAID         1
+#define INVTECHNOLOGY  2
+#define INVLIBRARY     3
+#define INVNODEPROTO   4
+#define INVNODEINST    5
+#define INVPORTARCINST 6
+#define INVPORTEXPINST 7
+#define INVPORTPROTO   8
+#define INVARCINST     9
 
 typedef struct
 {
@@ -94,6 +96,7 @@ typedef struct
 	INTBIG      geomerror;
 	INTBIG      rtnodeerror;
 	INTBIG      libraryerror;
+	INTBIG      convertmosiscmostechnologies;
 } TXTINPUTDATA;
 
 static TXTINPUTDATA io_txtindata;			/* all data pertaining to reading a file */
@@ -355,6 +358,8 @@ INTSML io_doreadtextlibrary(LIBRARY *lib, INTSML newprogress)
 	REGISTER INTBIG oldunit;
 	REGISTER NODEPROTO *np, **list, *onp;
 	REGISTER TECHNOLOGY *tech;
+	REGISTER NODEINST *ni;
+	REGISTER ARCINST *ai;
 	REGISTER CELL *c;
 	extern DIALOG us_eprogressdialog;
 	char *filename;
@@ -472,7 +477,7 @@ INTSML io_doreadtextlibrary(LIBRARY *lib, INTSML newprogress)
 
 	/* set the former version number on the library */
 	version = atoi(io_txtindata.version);
-	nextvarchangequiet();
+	nextchangequiet();
 	(void)setvalkey((INTBIG)io_txtindata.curlib, VLIBRARY, makekey("LIB_former_version"),
 		(INTBIG)io_txtindata.version, VSTRING|VDONTSAVE);
 	efree(io_txtindata.version);
@@ -484,11 +489,19 @@ INTSML io_doreadtextlibrary(LIBRARY *lib, INTSML newprogress)
 	for(tech = el_technologies; tech != NOTECHNOLOGY; tech = tech->nexttechnology)
 	{
 		lambda = io_txtindata.curlib->lambda[tech->techindex];
-		if (lambda == 0) lambda = el_curlib->lambda[tech->techindex]; else
-			lambda = muldiv(lambda, den, num);
+		if (lambda != 0) lambda = muldiv(lambda, den, num); else
+		{
+			lambda = el_curlib->lambda[tech->techindex];
+			if (lambda == 0) lambda = tech->deflambda;
+		}			
 		io_txtindata.curlib->lambda[tech->techindex] = lambda;
 	}
 
+	/* if converting MOSIS CMOS technologies, store lambda in the right place */
+	if (io_txtindata.convertmosiscmostechnologies != 0)
+		io_txtindata.curlib->lambda[mocmos_tech->techindex] =
+			io_txtindata.curlib->lambda[mocmossub_tech->techindex];
+
 	/* if this is to be the current library, adjust technologies */
 	if (io_txtindata.curlib == el_curlib)
 		for(tech = el_technologies; tech != NOTECHNOLOGY; tech = tech->nexttechnology)
@@ -566,6 +579,24 @@ INTSML io_doreadtextlibrary(LIBRARY *lib, INTSML newprogress)
 		efree((char *)list);
 	}
 
+	/* warn if the MOSIS CMOS technologies were converted */
+	if (io_txtindata.convertmosiscmostechnologies != 0)
+	{
+		for(np = lib->firstnodeproto; np != NONODEPROTO; np = np->nextnodeproto)
+		{
+			for(ni = np->firstnodeinst; ni != NONODEINST; ni = ni->nextnodeinst)
+				if (ni->proto->primindex != 0 && ni->proto->tech == mocmos_tech) break;
+			if (ni != NONODEINST) break;
+			for(ai = np->firstarcinst; ai != NOARCINST; ai = ai->nextarcinst)
+				if (ai->proto->tech == mocmos_tech) break;
+			if (ai != NOARCINST) break;
+		}
+		if (np != NONODEPROTO)
+			DiaMessageInDialog(
+				_("Warning: library %s has older 'mocmossub' technology, converted to new 'mocmos'"),
+					lib->libname);
+	}
+
 	/* clean up the library */
 	io_fixnewlib(io_txtindata.curlib, 0);
 
@@ -748,7 +779,25 @@ void io_newlib(void)
  */
 void io_versn(void)
 {
+	INTBIG emajor, eminor, edetail;
+
 	(void)reallocstring(&io_txtindata.version, io_keyword, el_tempcluster);
+
+	/* for versions before 6.03q, convert MOSIS CMOS technology names */
+	parseelectricversion(io_txtindata.version, &emajor, &eminor, &edetail);
+	io_txtindata.convertmosiscmostechnologies = 0;
+	if (emajor < 6 ||
+		(emajor == 6 && eminor < 3) ||
+		(emajor == 6 && eminor == 3 && edetail < 17))
+	{
+		if ((asktech(mocmossub_tech, "get-state")&MOCMOSSUBNOCONV) == 0)
+			io_txtindata.convertmosiscmostechnologies = 1;
+	}
+#ifdef REPORTCONVERSION
+	ttyputmsg("Library is version %s (%ld.%ld.%ld)", io_txtindata.version, emajor, eminor, edetail);
+	if (io_txtindata.convertmosiscmostechnologies != 0)
+		ttyputmsg("   Converting MOSIS CMOS technologies (mocmossub => mocmos)");
+#endif
 }
 
 /*
@@ -986,10 +1035,14 @@ void io_getvar(void)
 			}
 			if (naddr != -1)
 			{
-				nextvarchangequiet();
+				nextchangequiet();
 				var = setvalkey(naddr, ntype, key, (INTBIG)store, type);
 				if (var == NOVARIABLE) longjmp(io_filerror, (INTBIG)_("No memory"));
 				var->textdescript = descript;
+
+				/* handle updating of technology caches */
+				if (ntype == VTECHNOLOGY)
+					changedtechnologyvariable(key);
 			}
 			efree((char *)store);
 		} else
@@ -997,10 +1050,14 @@ void io_getvar(void)
 			ret = io_decode(io_keyword, type);
 			if (naddr != -1)
 			{
-				nextvarchangequiet();
+				nextchangequiet();
 				var = setvalkey(naddr, ntype, key, ret, type);
 				if (var == NOVARIABLE) longjmp(io_filerror, (INTBIG)_("No memory"));
 				var->textdescript = descript;
+
+				/* handle updating of technology caches */
+				if (ntype == VTECHNOLOGY)
+					changedtechnologyvariable(key);
 			}
 		}
 	}
@@ -1105,7 +1162,7 @@ INTBIG io_decode(char *name, INTBIG type)
  */
 void io_libcc(void)
 {
-	REGISTER INTSML i;
+	REGISTER INTBIG i;
 	REGISTER CELL *c;
 
 	io_txtindata.varpos = INVLIBRARY;
@@ -1114,6 +1171,10 @@ void io_libcc(void)
 	if (io_txtindata.facetcount == 0)
 	{
 		io_txtindata.curlib->firstnodeproto = NONODEPROTO;
+		io_txtindata.curlib->tailnodeproto = NONODEPROTO;
+		io_txtindata.curlib->firstcell = NOCELL;
+		io_txtindata.curlib->tailcell = NOCELL;
+		io_txtindata.curlib->numcells = 0;
 		return;
 	}
 
@@ -1144,8 +1205,15 @@ void io_libcc(void)
 		if (geomstructure(io_txtindata.nodeprotolist[i]) != 0)
 			longjmp(io_filerror, (INTBIG)_("No memory for geometry structure"));
 	}
-	if (io_txtindata.facetcount == 0) io_txtindata.curlib->firstcell = NOCELL; else
+	if (io_txtindata.facetcount == 0)
+	{
+		io_txtindata.curlib->firstcell = NOCELL;
+	} else
+	{
 		io_txtindata.curlib->firstcell = io_txtindata.nodeprotolist[0]->cell;
+		io_txtindata.curlib->tailcell = io_txtindata.nodeprotolist[io_txtindata.facetcount-1]->cell;
+	}
+	io_txtindata.curlib->numcells = io_txtindata.facetcount;
 
 	/* create the linked list of node prototypes in this library */
 	for(i=0; i<io_txtindata.facetcount; i++)
@@ -1157,6 +1225,7 @@ void io_libcc(void)
 				io_txtindata.nodeprotolist[i]->nextnodeproto = NONODEPROTO;
 	}
 	io_txtindata.curlib->firstnodeproto = io_txtindata.nodeprotolist[0];
+	io_txtindata.curlib->tailnodeproto = io_txtindata.nodeprotolist[io_txtindata.facetcount-1];
 }
 
 /*
@@ -1418,6 +1487,8 @@ void io_celext(void)
 		{
 			if (lastnp == NONODEPROTO) io_txtindata.curlib->firstnodeproto = onp->nextnodeproto; else
 				lastnp->nextnodeproto = onp->nextnodeproto;
+			if (io_txtindata.curlib->tailnodeproto == onp)
+				io_txtindata.curlib->tailnodeproto = lastnp;
 		}
 		db_freertree(onp->rtree);
 		freenodeproto(onp);
@@ -1432,6 +1503,8 @@ void io_celext(void)
 		{
 			if (lastcell == NOCELL) io_txtindata.curlib->firstcell = cell->nextcell; else
 				lastcell->nextcell = cell->nextcell;
+			if (c == io_txtindata.curlib->tailcell) io_txtindata.curlib->tailcell = lastcell;
+			io_txtindata.curlib->numcells--;
 		}
 		efree(cell->cellname);
 		efree((char *)cell);
@@ -1751,16 +1824,25 @@ void io_nodtyp(void)
 			return;
 		}
 
+		/* convert "Active-Node" to "P-Active-Node" (MOSIS CMOS) */
+		if (strcmp(line, "Active-Node") == 0)
+		{
+			for(np = tech->firstnodeproto; np != NONODEPROTO; np = np->nextnodeproto)
+				if (strcmp(np->primname, "P-Active-Node") == 0)
+			{
+				io_txtindata.curnodeinst->proto = np;
+				return;
+			}
+		}
+
 		/* convert "message" and "cell-center" nodes */
-		if (namesame(line, "Cell-Center") == 0) io_txtindata.curnodeinst->proto = gen_facetcenterprim; else
-		if (namesame(line, "Message") == 0 || namesame(line, "Centered-Message") == 0 ||
-			namesame(line, "Left-Message") == 0 || namesame(line, "Right-Message") == 0)
+		io_txtindata.curnodeinst->proto = io_convertoldprimitives(tech, line);
+		if (io_txtindata.curnodeinst->proto == gen_invispinprim)
 		{
-			io_txtindata.curnodeinst->proto = gen_invispinprim;
 			(void)strcpy(orig, "artwork:");
 			(void)strcat(orig, line);
 			if (orignodename == 0) orignodename = makekey("NODE_original_name");
-			nextvarchangequiet();
+			nextchangequiet();
 			(void)setvalkey((INTBIG)io_txtindata.curnodeinst, VNODEINST,
 				orignodename, (INTBIG)orig, VSTRING|VDONTSAVE);
 		}
@@ -1798,7 +1880,7 @@ void io_nodhy(void)
  */
 void io_nodnam(void)
 {
-	nextvarchangequiet();
+	nextchangequiet();
 	(void)setvalkey((INTBIG)io_txtindata.curnodeinst, VNODEINST, el_node_name,
 		(INTBIG)io_keyword, VSTRING|VDISPLAY);
 }
@@ -1984,7 +2066,7 @@ void io_arctyp(void)
  */
 void io_arcnam(void)
 {
-	nextvarchangequiet();
+	nextchangequiet();
 	(void)setvalkey((INTBIG)io_txtindata.curarcinst, VARCINST, el_arc_name,
 		(INTBIG)io_keyword, VSTRING|VDISPLAY);
 }
@@ -2169,7 +2251,13 @@ TECHNOLOGY *io_gettechnology(char *line)
 {
 	REGISTER TECHNOLOGY *tech;
 
-	tech = gettechnology(line);
+	tech = NOTECHNOLOGY;
+	if (io_txtindata.convertmosiscmostechnologies != 0)
+	{
+		if (namesame(line, "mocmossub") == 0) tech = gettechnology("mocmos"); else
+			if (namesame(line, "mocmos") == 0) tech = gettechnology("mocmosold");
+	}
+	if (tech == NOTECHNOLOGY) tech = gettechnology(line);
 	if (tech != NOTECHNOLOGY) return(tech);
 	if (namesame(line, "logic") == 0) tech = sch_tech;
 	return(tech);
