@@ -31,8 +31,7 @@
 #include "global.h"
 #include "eio.h"
 
-INTSML io_facetnumber;
-INTBIG io_nodeinsterror, io_portarcinsterror, io_portexpinsterror,
+INTBIG io_facetnumber, io_nodeinsterror, io_portarcinsterror, io_portexpinsterror,
 	  io_portprotoerror, io_arcinsterror, io_geomerror, io_rtnodeerror, io_libraryerror;
 
 /* prototypes for local routines */
