@@ -111,11 +111,11 @@ void io_vhdlreadfile(LIBRARY *lib, FILE *io)
 			pt = text;
 			while (*pt == ' ' || *pt == '\t') pt++;
 			sectiontype = SECTIONNONE;
-			if (strncmp(pt, "component", (sectionnamelen = 9)) == 0)
+			if (namesamen(pt, "component", (sectionnamelen = 9)) == 0)
 				sectiontype = SECTIONCOMPONENT;
-			else if (strncmp(pt, "entity", (sectionnamelen = 6)) == 0)
+			else if (namesamen(pt, "entity", (sectionnamelen = 6)) == 0)
 				sectiontype = SECTIONENTITY;
-			else if (strncmp(pt, "architecture", (sectionnamelen = 12)) == 0)
+			else if (namesamen(pt, "architecture", (sectionnamelen = 12)) == 0)
 				sectiontype = SECTIONARCHITECTURE;
 			if (sectiontype != SECTIONNONE)
 			{
@@ -134,7 +134,7 @@ void io_vhdlreadfile(LIBRARY *lib, FILE *io)
 					if (sectiontype == SECTIONARCHITECTURE)
 					{
 						while (*pt == ' ' || *pt == '\t') pt++;
-						if (strncmp(pt, "of", 2) == 0)
+						if (namesamen(pt, "of", 2) == 0)
 						{
 							pt += 2;
 							while (*pt == ' ' || *pt == '\t') pt++;
@@ -188,7 +188,7 @@ void io_vhdlreadfile(LIBRARY *lib, FILE *io)
 		/* see if this is the last line */
 		pt = text;
 		while (*pt == ' ' || *pt == '\t') pt++;
-		if (strncmp(pt, "end", 3) == 0)
+		if (namesamen(pt, "end", 3) == 0)
 		{
 			pt += 3;
 			if (*pt == ' ' || *pt == '\t')
@@ -197,7 +197,7 @@ void io_vhdlreadfile(LIBRARY *lib, FILE *io)
 				switch (sectiontype)
 				{
 					case SECTIONCOMPONENT:
-						if (strncmp(pt, "component", 9) == 0)
+						if (namesamen(pt, "component", 9) == 0)
 						{
 							stringarraytotextfacet(stringarray, storagefacet, 1);
 							killstringarray(stringarray);
