@@ -51,7 +51,7 @@
 #define VERTICAL            1
 
 
-#define	NOPOLYLIST ((POLYLIST *)-1)
+#define NOPOLYLIST ((POLYLIST *)-1)
 
 typedef struct Ipolylist
 {
@@ -62,7 +62,7 @@ typedef struct Ipolylist
 } POLYLIST;
 
 
-#define	NOOBJECT ((OBJECT *)-1)
+#define NOOBJECT ((OBJECT *)-1)
 
 typedef struct Iobject
 {
@@ -78,7 +78,7 @@ typedef struct Iobject
 } OBJECT;
 
 
-#define	NOLINE ((LINE *)-1)
+#define NOLINE ((LINE *)-1)
 
 typedef struct Iline
 {
@@ -91,7 +91,7 @@ typedef struct Iline
 } LINE;
 
 
-#define	NOTECHARRAY ((TECH_ARRAY *)-1)
+#define NOTECHARRAY ((TECH_ARRAY *)-1)
 
 typedef struct Itecharray
 {
@@ -101,16 +101,6 @@ typedef struct Itecharray
 } TECH_ARRAY;
 TECH_ARRAY *com_conv_layer;
 
-/* Compaction Options */
-DIALOGITEM com_optionsdialogitems[] =
-{
- /*  1 */ {0, {64,92,88,156}, BUTTON, N_("OK")},
- /*  2 */ {0, {64,8,88,72}, BUTTON, N_("Cancel")},
- /*  3 */ {0, {8,8,24,154}, CHECK, N_("Allow spreading")},
- /*  4 */ {0, {32,8,48,154}, CHECK, N_("Verbose")}
-};
-DIALOG com_optionsdialog = {{50,75,147,243}, N_("Compaction Options"), 0, 4, com_optionsdialogitems};
-
 /* prototypes for local routines */
 void com_buildtechnologies(void);
 OBJECT *com_make_ni_object(NODEINST*, OBJECT*, XARRAY, INTSML, INTBIG, INTBIG, INTBIG, INTBIG);
@@ -193,12 +183,12 @@ void com_init(INTBIG *argc, char *argv[], AIDENTRY *thisaid)
 	if (thisaid == NOAID)
 	{
 		com_spread = makekey("COM_spread");
-		nextvarchangequiet();
+		nextchangequiet();
 		(void)setvalkey((INTBIG)com_aid, VAID, com_spread, 0, VINTEGER|VDONTSAVE);
 		com_debug = 0;
 		com_do_axis = VERTICAL;
 		com_conv_layer = NOTECHARRAY;
-		registertechnologycache(com_buildtechnologies);
+		registertechnologycache(com_buildtechnologies, 0, 0);
 		DiaDeclareHook("compopt", &com_compactp, comp_optionsdlog);
 		return;
 	}
@@ -316,8 +306,8 @@ INTSML com_set(INTSML count, char *par[])
 		var = getvalkey((INTBIG)com_aid, VAID, VINTEGER, com_spread);
 		if (var != NOVARIABLE)
 		{
-			if (var->addr != 0) ttyputmsg(_("Compactor will spread if necessary")); else
-				ttyputmsg(_("Compactor will not spread designs"));
+			if (var->addr != 0) ttyputmsg(M_("Compactor will spread if necessary")); else
+				ttyputmsg(M_("Compactor will not spread designs"));
 		}
 		return(0);
 	}
@@ -327,11 +317,11 @@ INTSML com_set(INTSML count, char *par[])
 	{
 		if (count <= 1)
 		{
-			count = ttygetparam(_("COMPACTOR negate option:"), &comnotp, MAXPARS-1,
+			count = ttygetparam(M_("COMPACTOR negate option:"), &comnotp, MAXPARS-1,
 				&par[1]) + 1;
 			if (count <= 1)
 			{
-				ttyputerr(_("Aborted"));
+				ttyputerr(M_("Aborted"));
 				return(1);
 			}
 		}
@@ -339,7 +329,7 @@ INTSML com_set(INTSML count, char *par[])
 		if (namesamen(pp, "spread", l) == 0)
 		{
 			(void)setvalkey((INTBIG)com_aid, VAID, com_spread, 0, VINTEGER|VDONTSAVE);
-			ttyputverbose(_("Compactor will not spread designs"));
+			ttyputverbose(M_("Compactor will not spread designs"));
 			return(0);
 		}
 		ttyputbadusage("tellaid compaction not");
@@ -349,7 +339,7 @@ INTSML com_set(INTSML count, char *par[])
 	if (namesamen(pp, "spread", l) == 0)
 	{
 		(void)setvalkey((INTBIG)com_aid, VAID, com_spread, 1, VINTEGER|VDONTSAVE);
-		ttyputverbose(_("Compactor will spread if necessary"));
+		ttyputverbose(M_("Compactor will spread if necessary"));
 		return(0);
 	}
 
@@ -358,20 +348,20 @@ INTSML com_set(INTSML count, char *par[])
 		/* make sure network tool is on */
 		if ((net_aid->aidstate&AIDON) == 0)
 		{
-			ttyputerr(_("Network tool must be running...turning it on"));
+			ttyputerr(M_("Network tool must be running...turning it on"));
 			aidturnon(net_aid, 0);
-			ttyputerr(_("...now reissue the compaction command"));
+			ttyputerr(M_("...now reissue the compaction command"));
 			return(1);
 		}
 
 		if (count <= 1)
 		{
-			ttyputmsg(_("Do you want horizontal or vertical compaction? "));
+			ttyputmsg(M_("Do you want horizontal or vertical compaction? "));
 			i = (INTSML)ttygetchar();
 			if (i == (INTSML)'v' || i == (INTSML)'V') com_do_axis = VERTICAL; else
 				if (i == (INTSML)'h' || i == (INTSML)'H') com_do_axis = HORIZONTAL; else
 			{
-				ttyputerr(_("Unknown compaction axis: '%c', expected 'v' or 'h'"), i);
+				ttyputerr(M_("Unknown compaction axis: '%c', expected 'v' or 'h'"), i);
 				return(1);
 			}
 		} else
@@ -390,12 +380,12 @@ INTSML com_set(INTSML count, char *par[])
 			np = getnodeproto(par[2]);
 			if (np == NONODEPROTO)
 			{
-				ttyputerr(_("No facet named %s"), par[2]);
+				ttyputerr(M_("No facet named %s"), par[2]);
 				return(1);
 			}
 			if (np->primindex != 0)
 			{
-				ttyputerr(_("Can only compact facets, not primitives"));
+				ttyputerr(M_("Can only compact facets, not primitives"));
 				return(1);
 			}
 		} else
@@ -418,11 +408,11 @@ INTSML com_set(INTSML count, char *par[])
 		if (com_debug == 0)
 		{
 			com_debug = 1;
-			ttyputmsg(_("Compactor will print debugging information"));
+			ttyputmsg(M_("Compactor will print debugging information"));
 		} else
 		{
 			com_debug = 0;
-			ttyputmsg(_("Compactor will not print debugging information"));
+			ttyputmsg(M_("Compactor will not print debugging information"));
 		}
 		return(0);
 	}
@@ -475,7 +465,7 @@ INTSML com_examineonefacet(NODEPROTO *np, INTSML axis, INTSML verbose)
 	for(pp = np->firstportproto; pp != NOPORTPROTO; pp = pp->nextportproto)
 	{
 		/* see if this port is on the same net as previously examined one */
-		for(opp = np->firstportproto; opp != pp; opp=opp->nextportproto)
+		for(opp = np->firstportproto; opp != pp; opp = opp->nextportproto)
 			if (opp->network == pp->network) break;
 		if (opp != pp) pp->temp1 = opp->temp1; else
 			pp->temp1 = com_flatindex++;
@@ -523,7 +513,7 @@ INTSML com_examineonefacet(NODEPROTO *np, INTSML axis, INTSML verbose)
 				(void)addstringtoinfstr(describearcinst(cur_object->inst.ai)); else
 					(void)addstringtoinfstr(describenodeinst(cur_object->inst.ni));
 		}
-		ttyputmsg(_("Arcs that stretch:%s"), returninfstr());
+		ttyputmsg(M_("Arcs that stretch:%s"), returninfstr());
 	}
 
 	/* sort the compacting line of objects */
@@ -532,10 +522,10 @@ INTSML com_examineonefacet(NODEPROTO *np, INTSML axis, INTSML verbose)
 	/* print the merged structure */
 	if (com_debug != 0)
 	{
-		if (axis == VERTICAL) ttyputmsg(_("Bottom-to-top:")); else
-			ttyputmsg(_("Left-to-right:"));
+		if (axis == VERTICAL) ttyputmsg(M_("Bottom-to-top:")); else
+			ttyputmsg(M_("Left-to-right:"));
 		for(cur_line = linecomp; cur_line != NOLINE; cur_line = cur_line->nextline)
-			ttyputmsg(_("  Line: %s"), com_describeline(cur_line));
+			ttyputmsg(M_("  Line: %s"), com_describeline(cur_line));
 	}
 
 	/* compute bounds for each line */
@@ -661,7 +651,7 @@ OBJECT *com_make_ai_object(ARCINST *ai, OBJECT *object, XARRAY newtrans,
 		new_object->highy = ai->geom->highy;
 	} else new_object = object;
 
-	tot = arcpolys(ai);
+	tot = arcpolys(ai, NOWINDOWPART);
 	for(j=0; j<tot; j++)
 	{
 		poly = allocpolygon(4, com_aid->cluster);
@@ -785,7 +775,7 @@ OBJECT *com_make_ni_object(NODEINST *ni, OBJECT *object, XARRAY newtrans,
 	{
 		makerot(ni, temp);
 		transmult(temp, newtrans, trans);
-		tot = nodeEpolys(ni, 0);
+		tot = nodeEpolys(ni, 0, NOWINDOWPART);
 		for(j=0; j<tot; j++)
 		{
 			poly = allocpolygon(4, com_aid->cluster);
@@ -992,7 +982,7 @@ INTSML com_lineup_firstrow(LINE *line, LINE *other_line, INTSML axis, INTBIG low
 	if (i > 0)
 	{
 		if (com_debug != 0)
-			ttyputmsg(_("Lining up the first row to low edge %s"), latoa(lowbound));
+			ttyputmsg(M_("Lining up the first row to low edge %s"), latoa(lowbound));
 		if (axis == HORIZONTAL) change = com_move(line, i, 0, change); else
 			change = com_move(line, 0, i, change);
 		com_fixed_nonfixed(line, other_line);
@@ -1086,13 +1076,10 @@ INTSML com_compact(LINE *line, LINE *other_line, INTSML axis, INTSML change,
 		{
 			/* no constraints: allow overlap */
 			best_motion = cur_line->low - com_lowbound;
-#if 0
-			if (com_debug != 0) ttyputmsg(_("Default motion %s"), latoa(best_motion));
-#endif
 		}
 		if (com_debug != 0)
 		{
-			ttyputmsg(_("Moving object '%s' by %s"), com_describeline(cur_line),
+			ttyputmsg(M_("Moving object '%s' by %s"), com_describeline(cur_line),
 				latoa(best_motion));
 			if (thisoreason != NOOBJECT && otheroreason != NOOBJECT)
 			{
@@ -1100,7 +1087,7 @@ INTSML com_compact(LINE *line, LINE *other_line, INTSML axis, INTSML change,
 				if (thispreason->poly->tech != otherpreason->poly->tech ||
 					thispreason->poly->layer != otherpreason->poly->layer)
 				{
-					(void)formatinfstr(_("  Limit is %s, layer %s to %s, layer %s"),
+					(void)formatinfstr(M_("  Limit is %s, layer %s to %s, layer %s"),
 						com_describeobject(thisoreason),
 							layername(thispreason->poly->tech, thispreason->poly->layer),
 								com_describeobject(otheroreason),
@@ -1108,7 +1095,7 @@ INTSML com_compact(LINE *line, LINE *other_line, INTSML axis, INTSML change,
 										otherpreason->poly->layer));
 				} else
 				{
-					(void)formatinfstr(_("  Limit is %s to %s, layer %s"),
+					(void)formatinfstr(M_("  Limit is %s to %s, layer %s"),
 						com_describeobject(thisoreason), com_describeobject(otheroreason),
 							layername(thispreason->poly->tech, thispreason->poly->layer));
 				}
@@ -1175,7 +1162,7 @@ INTBIG com_minseparate(OBJECT *object, INTSML nlayer, POLYLIST *npolys,
 	NODEPROTO *facet)
 {
 	REGISTER INTBIG bound, dist, con, geom_lo, ai_hi, ni_hi,
-		best_motion, this_motion, fun;
+		best_motion, this_motion, fun, minsize, nminsize;
 	REGISTER INTSML pindex, nindex, layer;
 	REGISTER POLYGON *poly, *npoly;
 	INTBIG lx, hx, ly, hy, xl, xh, yl, yh;
@@ -1185,6 +1172,7 @@ INTBIG com_minseparate(OBJECT *object, INTSML nlayer, POLYLIST *npolys,
 
 	*oreason = NOOBJECT;
 	npoly = npolys->poly;
+	nminsize = polyminsize(npoly);
 	tech = npolys->tech;
 	nindex = npolys->networknum;
 
@@ -1243,8 +1231,9 @@ INTBIG com_minseparate(OBJECT *object, INTSML nlayer, POLYLIST *npolys,
 					}
 
 					/* see how close they can get */
-					dist = drcmindistance(tech, facet->cell->lib, nlayer,
-						layer, (INTSML)con);
+					minsize = polyminsize(poly);
+					dist = drcmindistance(tech, facet->cell->lib, nlayer, nminsize,
+						layer, minsize, con, 0, 0);
 					if (dist < 0) continue;
 
 					/*
@@ -1325,8 +1314,9 @@ INTBIG com_minseparate(OBJECT *object, INTSML nlayer, POLYLIST *npolys,
 					}
 
 					/* see how close they can get */
-					dist = drcmindistance(tech, facet->cell->lib, nlayer,
-						poly->layer, (INTSML)con);
+					minsize = polyminsize(poly);
+					dist = drcmindistance(tech, facet->cell->lib, nlayer, nminsize,
+						poly->layer, minsize, con, 0, 0);
 					if (dist < 0) continue;
 
 					/*
@@ -1795,11 +1785,20 @@ char *com_describeobject(OBJECT *cur_object)
 
 /****************************** DIALOG ******************************/
 
-/*
- * special case for the "Compaction Options" dialog
- * Allow spreading = 3 (check)
- * Verbose         = 4 (check)
- */
+/* Compaction Options */
+DIALOGITEM com_optionsdialogitems[] =
+{
+ /*  1 */ {0, {64,92,88,156}, BUTTON, N_("OK")},
+ /*  2 */ {0, {64,8,88,72}, BUTTON, N_("Cancel")},
+ /*  3 */ {0, {8,8,24,154}, CHECK, N_("Allow spreading")},
+ /*  4 */ {0, {32,8,48,154}, CHECK, N_("Verbose")}
+};
+DIALOG com_optionsdialog = {{50,75,147,243}, N_("Compaction Options"), 0, 4, com_optionsdialogitems};
+
+/* special items for the "Compaction Options" dialog: */
+#define DCMO_CANSPREAD   3		/* Allow spreading (check) */
+#define DCMO_VERBOSE     4		/* Verbose (check) */
+
 void comp_optionsdlog(void)
 {
 	REGISTER INTBIG itemHit, spread, origspread;
@@ -1808,15 +1807,15 @@ void comp_optionsdlog(void)
 	if (DiaInitDialog(&com_optionsdialog) != 0) return;
 	var = getval((INTBIG)com_aid, VAID, VINTEGER, "COM_spread");
 	if (var == NOVARIABLE) origspread = 0; else origspread = var->addr;
-	if (origspread != 0) DiaSetControl(3, 1);
-	if (com_debug != 0) DiaSetControl(4, 1);
+	if (origspread != 0) DiaSetControl(DCMO_CANSPREAD, 1);
+	if (com_debug != 0) DiaSetControl(DCMO_VERBOSE, 1);
 
 	/* loop until done */
 	for(;;)
 	{
 		itemHit = DiaNextHit();
 		if (itemHit == OK || itemHit == CANCEL) break;
-		if (itemHit == 3 || itemHit == 4)
+		if (itemHit == DCMO_CANSPREAD || itemHit == DCMO_VERBOSE)
 		{
 			DiaSetControl(itemHit, 1 - DiaGetControl(itemHit));
 			continue;
@@ -1825,11 +1824,11 @@ void comp_optionsdlog(void)
 
 	if (itemHit != CANCEL)
 	{
-		if (DiaGetControl(3) != 0) spread = 1; else
+		if (DiaGetControl(DCMO_CANSPREAD) != 0) spread = 1; else
 			spread = 0;
 		if (spread != origspread)
 			(void)setval((INTBIG)com_aid, VAID, "COM_spread", spread, VINTEGER);
-		if (DiaGetControl(4) != 0) com_debug = 1; else
+		if (DiaGetControl(DCMO_VERBOSE) != 0) com_debug = 1; else
 			com_debug = 0;
 	}
 	DiaDoneDialog();
