@@ -46,47 +46,31 @@ static KEYWORD ercopt[] =
 COMCOMP erc_tablep = {ercopt, NOTOPLIST, NONEXTLIST, NOPARAMS, NOBACKUP,
 	0, " \t", M_("Electrical Rules action"), M_("show defaults")};
 
-/* ERC: Options Dialog */
-DIALOGITEM erc_optionsdialogitems[] =
+/* well contacts */
+#define NOWELLCON ((WELLCON *)-1)
+
+typedef struct Iwellcon
 {
- /*  1 */ {0, {108,396,132,460}, BUTTON, N_("OK")},
- /*  2 */ {0, {108,296,132,360}, BUTTON, N_("Cancel")},
- /*  3 */ {0, {32,8,48,228}, RADIO, N_("Must have contact in every area")},
- /*  4 */ {0, {56,8,72,228}, RADIO, N_("Must have at least 1 contact")},
- /*  5 */ {0, {80,8,96,228}, RADIO, N_("Do not check for contacts")},
- /*  6 */ {0, {8,44,24,144}, MESSAGE, N_("For well:")},
- /*  7 */ {0, {32,248,48,480}, RADIO, N_("Must have contact in every area")},
- /*  8 */ {0, {56,248,72,480}, RADIO, N_("Must have at least 1 contact")},
- /*  9 */ {0, {80,248,96,480}, RADIO, N_("Do not check for contacts")},
- /* 10 */ {0, {8,284,24,384}, MESSAGE, N_("For substrate:")}
-};
-DIALOG erc_optionsdialog = {{50,75,191,564}, N_("ERC Options"), 0, 10, erc_optionsdialogitems};
+	INTBIG           x, y;
+	INTBIG           netnum;
+	INTBIG           fun;
+	NODEPROTO       *np;
+	struct Iwellcon *nextwellcon;
+} WELLCON;
 
-/* substrate/well contacts */
-#define NOSUBWELLCON ((SUBWELLCON *)-1)
+WELLCON    *erc_wellconfree = NOWELLCON;
+WELLCON    *erc_firstwellcon = NOWELLCON;
+INTBIG      erc_worstpwelldist;
+INTBIG      erc_worstpwellconx, erc_worstpwellcony;
+INTBIG      erc_worstpwelledgex, erc_worstpwelledgey;
+INTBIG      erc_worstnwelldist;
+INTBIG      erc_worstnwellconx, erc_worstnwellcony;
+INTBIG      erc_worstnwelledgex, erc_worstnwelledgey;
 
-typedef struct Isubwellcon
-{
-	INTBIG              x, y;
-	INTBIG              netnum;
-	INTBIG              fun;
-	NODEPROTO          *np;
-	struct Isubwellcon *nextsubwellcon;
-} SUBWELLCON;
+/* well areas */
+#define NOWELLAREA ((WELLAREA *)-1)
 
-SUBWELLCON *erc_subwellconfree = NOSUBWELLCON;
-SUBWELLCON *erc_firstsubwellcon = NOSUBWELLCON;
-INTBIG      erc_worstwelldist;
-INTBIG      erc_worstwellconx, erc_worstwellcony;
-INTBIG      erc_worstwelledgex, erc_worstwelledgey;
-INTBIG      erc_worstsubdist;
-INTBIG      erc_worstsubconx, erc_worstsubcony;
-INTBIG      erc_worstsubedgex, erc_worstsubedgey;
-
-/* substrate/well areas */
-#define NOSUBWELLAREA ((SUBWELLAREA *)-1)
-
-typedef struct Isubwellarea
+typedef struct Iwellarea
 {
 	INTBIG            lx, hx, ly, hy;
 	INTBIG            netnum;
@@ -94,23 +78,23 @@ typedef struct Isubwellarea
 	INTSML            layer;
 	TECHNOLOGY        *tech;
 	NODEPROTO         *facet;
-	struct Isubwellarea *nextsubwellarea;
-} SUBWELLAREA;
+	struct Iwellarea  *nextwellarea;
+} WELLAREA;
 
-SUBWELLAREA *erc_subwellareafree = NOSUBWELLAREA;
-SUBWELLAREA *erc_firstsubwellarea = NOSUBWELLAREA;
+WELLAREA *erc_wellareafree = NOWELLAREA;
+WELLAREA *erc_firstwellarea = NOWELLAREA;
 
 /* miscellaneous */
 
 /* the meaning of "ERC_options" */
-#define WELLCONOPTIONS  03
-#define WELLCONPERAREA   0
-#define WELLCONONCE      1
-#define WELLCONIGNORE    2
-#define SUBCONOPTIONS  014
-#define SUBCONPERAREA    0
-#define SUBCONONCE      04
-#define SUBCONIGNORE   010
+#define PWELLCONOPTIONS   03
+#define PWELLCONPERAREA    0
+#define PWELLCONONCE       1
+#define PWELLCONIGNORE     2
+#define NWELLCONOPTIONS  014
+#define NWELLCONPERAREA    0
+#define NWELLCONONCE      04
+#define NWELLCONIGNORE   010
 
 AIDENTRY   *erc_aid;						/* this tool */
 NODEPROTO  *erc_facet;
@@ -123,19 +107,16 @@ INTBIG      erc_polyscrunched;				/* number of polygons analyzed */
 INTBIG      erc_realrun;					/* nonzero if this is the real analysis */
 
 /* prototypes for local routines */
-SUBWELLAREA *erc_allocsubwellarea(void);
-SUBWELLCON  *erc_allocsubwellcon(void);
+WELLAREA    *erc_allocwellarea(void);
+WELLCON     *erc_allocwellcon(void);
 void         erc_analyzefacet(NODEPROTO *facet);
-void         erc_freeallsubwellareas(void);
-void         erc_freeallsubwellcons(void);
-void         erc_freesubwellarea(SUBWELLAREA *swa);
-void         erc_freesubwellcon(SUBWELLCON *swc);
-void         erc_gathersubandwell(NODEPROTO *np, XARRAY trans);
+void         erc_freeallwellareas(void);
+void         erc_freeallwellcons(void);
+void         erc_freewellarea(WELLAREA *wa);
+void         erc_freewellcon(WELLCON *wc);
+void         erc_gatherwells(NODEPROTO *np, XARRAY trans);
 void         erc_getpolygon(INTSML layer, TECHNOLOGY *tech, INTBIG *x, INTBIG *y, INTSML count);
 void         erc_optionsdlog(void);
-INTSML       erc_substratelayer(TECHNOLOGY *tech, INTSML layer);
-INTSML       erc_truesubstratelayer(TECHNOLOGY *tech, INTSML layer);
-INTSML       erc_truewelllayer(TECHNOLOGY *tech, INTSML layer);
 INTSML       erc_welllayer(TECHNOLOGY *tech, INTSML layer);
 
 /************************ CONTROL ***********************/
@@ -156,8 +137,8 @@ void erc_init(INTBIG *argc, char *argv[], AIDENTRY *thisaid)
 
 void erc_done(void)
 {
-	erc_freeallsubwellcons();
-	erc_freeallsubwellareas();
+	erc_freeallwellcons();
+	erc_freeallwellareas();
 }
 
 /*
@@ -197,14 +178,15 @@ INTSML erc_set(INTSML count, char *par[])
  */
 void erc_analyzefacet(NODEPROTO *facet)
 {
-	REGISTER INTBIG errors, minsep, dist;
+	REGISTER INTBIG errors, minsep, dist, layertype;
 	REGISTER NODEPROTO *np;
 	REGISTER TECHNOLOGY *tech;
 	REGISTER char *abortseq, *errormsg;
-	REGISTER SUBWELLCON *swc, *oswc;
+	char errmsg[200];
+	REGISTER WELLCON *wc, *owc;
 	REGISTER VARIABLE *var;
 	REGISTER PORTPROTO *pp, *opp;
-	REGISTER SUBWELLAREA *swa, *oswa;
+	REGISTER WELLAREA *wa, *owa;
 	REGISTER void *err;
 	extern DIALOG us_progressdialog;
 
@@ -229,9 +211,9 @@ void erc_analyzefacet(NODEPROTO *facet)
 
 	/* initialize to analyze facet */
 	initerrorlogging(_("ERC"));
-	erc_freeallsubwellcons();
-	erc_freeallsubwellareas();
-	erc_worstwelldist = erc_worstsubdist = 0;
+	erc_freeallwellcons();
+	erc_freeallwellareas();
+	erc_worstpwelldist = erc_worstnwelldist = 0;
 	erc_facet = facet;
 	var = getvalkey((INTBIG)erc_aid, VAID, VINTEGER, erc_optionskey);
 	if (var == NOVARIABLE) erc_options = 0; else
@@ -261,7 +243,7 @@ void erc_analyzefacet(NODEPROTO *facet)
 	/* run through the work and count the number of polygons */
 	erc_polyscrunched = 0;
 	erc_realrun = 0;
-	erc_gathersubandwell(facet, el_matid);
+	erc_gatherwells(facet, el_matid);
 	erc_jobsize = erc_polyscrunched;
 
 	/* now gather all of the geometry into the polygon merging system */
@@ -269,7 +251,7 @@ void erc_analyzefacet(NODEPROTO *facet)
 	mergeinit();
 	erc_realrun = 1;
 	erc_polyscrunched = 0;
-	erc_gathersubandwell(facet, el_matid);
+	erc_gatherwells(facet, el_matid);
 
 	/* determine the number of polygon points in the analysis */
 	DiaSetText(2, _("Analyzing..."));
@@ -285,91 +267,133 @@ void erc_analyzefacet(NODEPROTO *facet)
 	DiaPercent(1, 100);
 
 	/* make sure all of the contacts are on the same net */
-	for(swc = erc_firstsubwellcon; swc != NOSUBWELLCON; swc = swc->nextsubwellcon)
+	for(wc = erc_firstwellcon; wc != NOWELLCON; wc = wc->nextwellcon)
 	{
-		if (swc->netnum == 0)
+		if (wc->netnum == 0)
 		{
-			if (swc->fun == NPWELL) errormsg = _("Well contact is floating"); else
-				errormsg = _("Substrate contact is floating");
+			if (wc->fun == NPWELL) errormsg = _("P-Well contact is floating"); else
+				errormsg = _("N-Well contact is floating");
 			err = logerror(errormsg, erc_facet, 0);
-			(void)addpointtoerror(err, swc->x, swc->y);
+			(void)addpointtoerror(err, wc->x, wc->y);
 			continue;
 		}
-		for(oswc = swc->nextsubwellcon; oswc != NOSUBWELLCON; oswc = oswc->nextsubwellcon)
+		for(owc = wc->nextwellcon; owc != NOWELLCON; owc = owc->nextwellcon)
 		{
-			if (oswc->netnum == 0) continue;
-			if (oswc->fun != swc->fun) continue;
-			if (oswc->netnum == swc->netnum) continue;
-			if (swc->fun == NPWELL) errormsg = _("Well contacts are not connected"); else
-				errormsg = _("Substrate contacts are not connected");
+			if (owc->netnum == 0) continue;
+			if (owc->fun != wc->fun) continue;
+			if (owc->netnum == wc->netnum) continue;
+			if (wc->fun == NPWELL) errormsg = _("P-Well contacts are not connected"); else
+				errormsg = _("N-Well contacts are not connected");
 			err = logerror(errormsg, erc_facet, 0);
-			(void)addpointtoerror(err, swc->x, swc->y);
-			(void)addpointtoerror(err, oswc->x, oswc->y);
+			(void)addpointtoerror(err, wc->x, wc->y);
+			(void)addpointtoerror(err, owc->x, owc->y);
 			break;
 		}
 	}
 
-	/* if just 1 substrate contact is needed, see if it is there */
-	if ((erc_options&SUBCONOPTIONS) == SUBCONONCE)
+	/* if just 1 N-Well contact is needed, see if it is there */
+	if ((erc_options&NWELLCONOPTIONS) == NWELLCONONCE)
 	{
-		for(swc = erc_firstsubwellcon; swc != NOSUBWELLCON; swc = swc->nextsubwellcon)
-			if (swc->fun == NPSUBSTRATE) break;
-		if (swc == NOSUBWELLCON)
+		for(wc = erc_firstwellcon; wc != NOWELLCON; wc = wc->nextwellcon)
+			if (wc->fun == NPSUBSTRATE) break;
+		if (wc == NOWELLCON)
 		{
-			(void)logerror(_("No substrate contact found in this facet"), erc_facet, 0);
+			(void)logerror(_("No N-Well contact found in this facet"), erc_facet, 0);
 		}
 	}
 
-	/* if just 1 well contact is needed, see if it is there */
-	if ((erc_options&WELLCONOPTIONS) == WELLCONONCE)
+	/* if just 1 P-Well contact is needed, see if it is there */
+	if ((erc_options&PWELLCONOPTIONS) == PWELLCONONCE)
 	{
-		for(swc = erc_firstsubwellcon; swc != NOSUBWELLCON; swc = swc->nextsubwellcon)
-			if (swc->fun == NPWELL) break;
-		if (swc == NOSUBWELLCON)
+		for(wc = erc_firstwellcon; wc != NOWELLCON; wc = wc->nextwellcon)
+			if (wc->fun == NPWELL) break;
+		if (wc == NOWELLCON)
 		{
-			(void)logerror(_("No well contact found in this facet"), erc_facet, 0);
+			(void)logerror(_("No P-well contact found in this facet"), erc_facet, 0);
 		}
 	}
 
-	/* make sure the substrates and wells are separated properly */
-	for(swa = erc_firstsubwellarea; swa != NOSUBWELLAREA; swa = swa->nextsubwellarea)
+	/* make sure the wells are separated properly */
+	for(wa = erc_firstwellarea; wa != NOWELLAREA; wa = wa->nextwellarea)
 	{
-		for(oswa = swa->nextsubwellarea; oswa != NOSUBWELLAREA; oswa = oswa->nextsubwellarea)
+		for(owa = wa->nextwellarea; owa != NOWELLAREA; owa = owa->nextwellarea)
 		{
-			if (swa->tech != oswa->tech || swa->layer != oswa->layer) continue;
-			if (swa->netnum == oswa->netnum && swa->netnum >= 0)
-				minsep = drcmindistance(swa->tech, facet->cell->lib, swa->layer, swa->layer, 1); else
-					minsep = drcmindistance(swa->tech, facet->cell->lib, swa->layer, swa->layer, 0);
+			if (wa->tech != owa->tech || wa->layer != owa->layer) continue;
+			if (wa->netnum == owa->netnum && wa->netnum >= 0)
+				minsep = drcmindistance(wa->tech, facet->cell->lib, wa->layer, 0, wa->layer, 0, 1, 0, 0); else
+					minsep = drcmindistance(wa->tech, facet->cell->lib, wa->layer, 0, wa->layer, 0, 0, 0, 0);
 			if (minsep < 0) continue;
-			dist = polyseparation(swa->poly, oswa->poly);
+			dist = polyseparation(wa->poly, owa->poly);
 			if (dist < minsep)
 			{
-				if (erc_welllayer(swa->tech, swa->layer) != 0) errormsg = _("Well areas too close"); else
-					errormsg = _("Substrate areas too close");
-				err = logerror(errormsg, erc_facet, 0);
-				(void)addpolytoerror(err, swa->poly);
-				(void)addpolytoerror(err, oswa->poly);
+				layertype = erc_welllayer(wa->tech, wa->layer);
+				if (layertype == 0) continue;
+				switch (layertype)
+				{
+					case 1:
+						sprintf(errmsg, _("P-Well areas too close (are %s, should be %s)"),
+							latoa(dist), latoa(minsep));
+						break;
+					case 2:
+						sprintf(errmsg, _("N-Well areas too close (are %s, should be %s)"),
+							latoa(dist), latoa(minsep));
+						break;
+					case 3:
+						sprintf(errmsg, _("P-Select areas too close (are %s, should be %s)"),
+							latoa(dist), latoa(minsep));
+						break;
+					case 4:
+						sprintf(errmsg, _("N-Select areas too close (are %s, should be %s)"),
+							latoa(dist), latoa(minsep));
+						break;
+				}
+				err = logerror(errmsg, erc_facet, 0);
+				(void)addpolytoerror(err, wa->poly);
+				(void)addpolytoerror(err, owa->poly);
 			}
 		}
 	}
 
-	/* show the farthest distance from a well/substrate contact */
-	if (erc_worstwelldist > 0 || erc_worstsubdist > 0)
+#if 0			/* debugging to show all well areas */
+	for(wa = erc_firstwellarea; wa != NOWELLAREA; wa = wa->nextwellarea)
+	{
+		layertype = erc_welllayer(wa->tech, wa->layer);
+		if (layertype == 0) continue;
+		switch (layertype)
+		{
+			case 1:
+				err = logerror("P-Well area", erc_facet, 0);
+				break;
+			case 2:
+				err = logerror("N-Well area", erc_facet, 0);
+				break;
+			case 3:
+				err = logerror("P-Select area", erc_facet, 0);
+				break;
+			case 4:
+				err = logerror("N-Select area", erc_facet, 0);
+		}
+		(void)addpolytoerror(err, wa->poly);
+	}
+#endif
+
+	/* show the farthest distance from a well contact */
+	if (erc_worstpwelldist > 0 || erc_worstnwelldist > 0)
 	{
 		(void)askaid(us_aid, "clear");
-		if (erc_worstwelldist > 0)
+		if (erc_worstpwelldist > 0)
 		{
-			(void)askaid(us_aid, "show-line", erc_worstwellconx, erc_worstwellcony,
-				erc_worstwelledgex, erc_worstwelledgey, erc_facet);
-			ttyputmsg(_("Farthest distance from a well contact is %s"),
-				latoa(erc_worstwelldist));
+			(void)askaid(us_aid, "show-line", erc_worstpwellconx, erc_worstpwellcony,
+				erc_worstpwelledgex, erc_worstpwelledgey, erc_facet);
+			ttyputmsg(_("Farthest distance from a P-Well contact is %s"),
+				latoa(erc_worstpwelldist));
 		}
-		if (erc_worstsubdist > 0)
+		if (erc_worstnwelldist > 0)
 		{
-			(void)askaid(us_aid, "show-line", erc_worstsubconx, erc_worstsubcony,
-				erc_worstsubedgex, erc_worstsubedgey, erc_facet);
-			ttyputmsg(_("Farthest distance from a substrate contact is %s"),
-				latoa(erc_worstsubdist));
+			(void)askaid(us_aid, "show-line", erc_worstnwellconx, erc_worstnwellcony,
+				erc_worstnwelledgex, erc_worstnwelledgey, erc_facet);
+			ttyputmsg(_("Farthest distance from a N-Well contact is %s"),
+				latoa(erc_worstnwelldist));
 		}
 	}
 
@@ -381,16 +405,15 @@ void erc_analyzefacet(NODEPROTO *facet)
 			if (np->temp1 == 0) continue;
 			switch ((np->userbits&NFUNCTION) >> NFUNCTIONSH)
 			{
-				case NPTRANMOS:
-				case NPTRADMOS:
-				case NPTRAPMOS:
-				case NPTRANPN:
-				case NPTRAPNP:
-				case NPTRANJFET:
-				case NPTRAPJFET:
-				case NPTRADMES:
-				case NPTRAEMES:
-				case NPTRANE2L:
+				case NPTRANMOS:   case NPTRA4NMOS:
+				case NPTRADMOS:   case NPTRA4DMOS:
+				case NPTRAPMOS:   case NPTRA4PMOS:
+				case NPTRANPN:    case NPTRA4NPN:
+				case NPTRAPNP:    case NPTRA4PNP:
+				case NPTRANJFET:  case NPTRA4NJFET:
+				case NPTRAPJFET:  case NPTRA4PJFET:
+				case NPTRADMES:   case NPTRA4DMES:
+				case NPTRAEMES:   case NPTRA4EMES:
 				case NPTRANSREF:
 				case NPTRANS:
 					ttyputmsg(_("%ld nodes of type %s"), np->temp1, describenodeproto(np));
@@ -412,10 +435,10 @@ void erc_analyzefacet(NODEPROTO *facet)
 }
 
 /*
- * Recursive helper routine to gather all substrate and well areas in facet "np"
+ * Recursive helper routine to gather all well areas in facet "np"
  * (transformed by "trans").
  */
-void erc_gathersubandwell(NODEPROTO *np, XARRAY trans)
+void erc_gatherwells(NODEPROTO *np, XARRAY trans)
 {
 	REGISTER NODEINST *ni;
 	REGISTER NODEPROTO *subnp;
@@ -425,12 +448,10 @@ void erc_gathersubandwell(NODEPROTO *np, XARRAY trans)
 	REGISTER PORTPROTO *pp, *opp;
 	REGISTER NETWORK *net;
 	REGISTER INTBIG i, tot, nfun;
-	REGISTER INTSML layer;
-	char *extra;
 	static INTBIG checkstop = 0;
 	XARRAY localrot, localtran, final, subrot;
 	static POLYGON *poly = NOPOLYGON;
-	REGISTER SUBWELLCON *swc;
+	REGISTER WELLCON *wc;
 
 	if (poly == NOPOLYGON) poly = allocstaticpolygon(4, erc_aid->cluster);
 
@@ -482,7 +503,7 @@ void erc_gathersubandwell(NODEPROTO *np, XARRAY trans)
 				maketrans(ni, localtran);
 				transmult(localtran, final, subrot);
 			}
-			erc_gathersubandwell(subnp, subrot);
+			erc_gatherwells(subnp, subrot);
 		} else
 		{
 			if (erc_realrun != 0)
@@ -492,47 +513,39 @@ void erc_gathersubandwell(NODEPROTO *np, XARRAY trans)
 				{
 					if (stopping(STOPREASONERC)) return;
 				}
-				nfun = nodefunction(ni, &extra);
+				nfun = nodefunction(ni, 0);
 				ni->proto->temp1++;
 
 				if (nfun == NPWELL || nfun == NPSUBSTRATE)
 				{
-					swc = erc_allocsubwellcon();
-					if (swc == NOSUBWELLCON) return;
+					wc = erc_allocwellcon();
+					if (wc == NOWELLCON) return;
 					xform((ni->lowx + ni->highx) / 2, (ni->lowy + ni->highy) / 2,
-						&swc->x, &swc->y, final);
-					swc->np = subnp;
-					swc->fun = nfun;
+						&wc->x, &wc->y, final);
+					wc->np = subnp;
+					wc->fun = nfun;
 					if (ni->firstportarcinst != NOPORTARCINST)
 					{
-						swc->netnum = ni->firstportarcinst->conarcinst->network->temp1;
+						wc->netnum = ni->firstportarcinst->conarcinst->network->temp1;
 					} else if (ni->firstportexpinst != NOPORTEXPINST)
 					{
-						swc->netnum = ni->firstportexpinst->exportproto->network->temp1;
-					} else swc->netnum = 0;
-					swc->nextsubwellcon = erc_firstsubwellcon;
-					erc_firstsubwellcon = swc;
+						wc->netnum = ni->firstportexpinst->exportproto->network->temp1;
+					} else wc->netnum = 0;
+					wc->nextwellcon = erc_firstwellcon;
+					erc_firstwellcon = wc;
 				}
 			}
 
-			tot = nodepolys(ni, 0);
+			tot = nodepolys(ni, 0, NOWINDOWPART);
 			erc_polyscrunched += tot;
 			if (erc_realrun != 0)
 			{
 				for(i=0; i<tot; i++)
 				{
 					shapenodepoly(ni, i, poly);
-					if (nfun == NPWELL)
-					{
-						layer = erc_truewelllayer(poly->tech, poly->layer);
-					} else if (nfun == NPSUBSTRATE)
-					{
-						layer = erc_truesubstratelayer(poly->tech, poly->layer);
-					} else layer = poly->layer;
-					if (erc_welllayer(poly->tech, layer) == 0 &&
-						erc_substratelayer(poly->tech, layer) == 0) continue;
+					if (erc_welllayer(poly->tech, poly->layer) == 0) continue;
 					xformpoly(poly, final);
-					mergestorepolygon(layer, poly->tech, poly);
+					mergestorepolygon(poly->layer, poly->tech, poly);
 				}
 			}
 		}
@@ -545,15 +558,14 @@ void erc_gathersubandwell(NODEPROTO *np, XARRAY trans)
 		{
 			if (stopping(STOPREASONERC)) return;
 		}
-		tot = arcpolys(ai);
+		tot = arcpolys(ai, NOWINDOWPART);
 		erc_polyscrunched += tot;
 		if (erc_realrun != 0)
 		{
 			for(i=0; i<tot; i++)
 			{
 				shapearcpoly(ai, i, poly);
-				if (erc_welllayer(poly->tech, poly->layer) == 0 &&
-					erc_substratelayer(poly->tech, poly->layer) == 0) continue;
+				if (erc_welllayer(poly->tech, poly->layer) == 0) continue;
 				xformpoly(poly, trans);
 				mergestorepolygon(poly->layer, poly->tech, poly);
 			}
@@ -569,8 +581,8 @@ void erc_getpolygon(INTSML layer, TECHNOLOGY *tech, INTBIG *x, INTBIG *y, INTSML
 {
 	REGISTER INTSML i, first, welltype;
 	REGISTER INTBIG dist, bestdist, desiredcontact, perareabit, prev, xp, yp;
-	REGISTER SUBWELLCON *swc, *bestswc;
-	REGISTER SUBWELLAREA *swa;
+	REGISTER WELLCON *wc, *bestwc;
+	REGISTER WELLAREA *wa;
 	REGISTER void *err;
 	REGISTER char *nocontacterror;
 	static INTBIG checkstop = 0;
@@ -589,55 +601,55 @@ void erc_getpolygon(INTSML layer, TECHNOLOGY *tech, INTBIG *x, INTBIG *y, INTSML
 	DiaPercent(1, erc_polyscrunched * 100 / erc_jobsize);
 
 	/* save the polygon */
-	swa = erc_allocsubwellarea();
-	swa->poly = allocpolygon(count, erc_aid->cluster);
+	wa = erc_allocwellarea();
+	wa->poly = allocpolygon(count, erc_aid->cluster);
 	for(i=0; i<count; i++)
 	{
-		swa->poly->xv[i] = x[i];
-		swa->poly->yv[i] = y[i];
+		wa->poly->xv[i] = x[i];
+		wa->poly->yv[i] = y[i];
 	}
-	swa->poly->count = count;
-	swa->poly->style = FILLED;
-	swa->layer = layer;
-	swa->netnum = 0;
-	swa->tech = tech;
-	getbbox(swa->poly, &swa->lx, &swa->hx, &swa->ly, &swa->hy);
-	swa->nextsubwellarea = erc_firstsubwellarea;
-	erc_firstsubwellarea = swa;
+	wa->poly->count = count;
+	wa->poly->style = FILLED;
+	wa->layer = layer;
+	wa->netnum = 0;
+	wa->tech = tech;
+	getbbox(wa->poly, &wa->lx, &wa->hx, &wa->ly, &wa->hy);
+	wa->nextwellarea = erc_firstwellarea;
+	erc_firstwellarea = wa;
 
-	/* special checks for the well/substrate areas */
+	/* special checks for the well areas */
 	welltype = erc_welllayer(tech, layer);
-	if (welltype == 0) return;
-	if (welltype > 0)
+	if (welltype != 1 && welltype != 2) return;
+	if (welltype == 1)
 	{
 		/* P-well */
 		desiredcontact = NPWELL;
-		perareabit = WELLCONPERAREA;
-		nocontacterror = _("No well contact found in this area");
+		perareabit = PWELLCONPERAREA;
+		nocontacterror = _("No P-Well contact found in this area");
 	} else
 	{
-		/* Select */
+		/* N-Well */
 		desiredcontact = NPSUBSTRATE;
-		perareabit = SUBCONPERAREA;
-		nocontacterror = _("No substrate contact found in this area");
+		perareabit = NWELLCONPERAREA;
+		nocontacterror = _("No N-Well contact found in this area");
 	}
 
 	/* find a contact in the area */
-	for(swc = erc_firstsubwellcon; swc != NOSUBWELLCON; swc = swc->nextsubwellcon)
+	for(wc = erc_firstwellcon; wc != NOWELLCON; wc = wc->nextwellcon)
 	{
-		if (swc->fun != desiredcontact) continue;
-		if (isinside(swc->x, swc->y, swa->poly) == 0) continue;
-		swa->netnum = swc->netnum;
+		if (wc->fun != desiredcontact) continue;
+		if (isinside(wc->x, wc->y, wa->poly) == 0) continue;
+		wa->netnum = wc->netnum;
 		break;
 	}
 
 	/* if no contact, issue appropriate errors */
-	if (swc == NOSUBWELLCON)
+	if (wc == NOWELLCON)
 	{
-		if ((erc_options&WELLCONOPTIONS) == perareabit)
+		if ((erc_options&PWELLCONOPTIONS) == perareabit)
 		{
 			err = logerror(nocontacterror, erc_facet, 0);
-			(void)addpolytoerror(err, swa->poly);
+			(void)addpolytoerror(err, wa->poly);
 		}
 		return;
 	}
@@ -660,48 +672,52 @@ void erc_getpolygon(INTSML layer, TECHNOLOGY *tech, INTBIG *x, INTBIG *y, INTSML
 		/* find the closest contact to this point */
 		first = 1;
 		bestdist = 0;
-		for(swc = erc_firstsubwellcon; swc != NOSUBWELLCON; swc = swc->nextsubwellcon)
+		for(wc = erc_firstwellcon; wc != NOWELLCON; wc = wc->nextwellcon)
 		{
-			if (swc->fun != desiredcontact) continue;
-			if (isinside(swc->x, swc->y, swa->poly) == 0) continue;
-			dist = computedistance(swc->x, swc->y, xp, yp);
+			if (wc->fun != desiredcontact) continue;
+			if (isinside(wc->x, wc->y, wa->poly) == 0) continue;
+			dist = computedistance(wc->x, wc->y, xp, yp);
 			if (first != 0 || dist < bestdist)
 			{
 				bestdist = dist;
-				bestswc = swc;
+				bestwc = wc;
 			}
 			first = 0;
 		}
 		if (first != 0) continue;
 
 		/* accumulate worst distances to edges */
-		if (welltype > 0)
+		if (welltype == 1)
 		{
-			if (bestdist > erc_worstwelldist)
+			if (bestdist > erc_worstpwelldist)
 			{
-				erc_worstwelldist = bestdist;
-				erc_worstwellconx = bestswc->x;
-				erc_worstwellcony = bestswc->y;
-				erc_worstwelledgex = xp;
-				erc_worstwelledgey = yp;
+				erc_worstpwelldist = bestdist;
+				erc_worstpwellconx = bestwc->x;
+				erc_worstpwellcony = bestwc->y;
+				erc_worstpwelledgex = xp;
+				erc_worstpwelledgey = yp;
 			}
 		} else
 		{
-			if (bestdist > erc_worstsubdist)
+			if (bestdist > erc_worstnwelldist)
 			{
-				erc_worstsubdist = bestdist;
-				erc_worstsubconx = bestswc->x;
-				erc_worstsubcony = bestswc->y;
-				erc_worstsubedgex = xp;
-				erc_worstsubedgey = yp;
+				erc_worstnwelldist = bestdist;
+				erc_worstnwellconx = bestwc->x;
+				erc_worstnwellcony = bestwc->y;
+				erc_worstnwelledgex = xp;
+				erc_worstnwelledgey = yp;
 			}
 		}
 	}
 }
 
 /*
- * Routine to return nonzero if layer "layer" in technology "tech" is a well layer.
- * (returns 1 if it is P-Well, -1 if it is N-Well).
+ * Routine to return nonzero if layer "layer" in technology "tech" is a well/select layer.
+ * Returns:
+ *   1: P-Well
+ *   2: N-Well
+ *   3: P-Select
+ *   4: N-Select
  */
 INTSML erc_welllayer(TECHNOLOGY *tech, INTSML layer)
 {
@@ -709,153 +725,114 @@ INTSML erc_welllayer(TECHNOLOGY *tech, INTSML layer)
 
 	fun = layerfunction(tech, layer);
 	funpure = fun & LFTYPE;
-	if (funpure != LFWELL) return(0);
-	if ((fun&LFPTYPE) != 0) return(1);
-	return(-1);
-}
-
-/*
- * Routine to return nonzero if layer "layer" in technology "tech" is a substrate layer.
- */
-INTSML erc_substratelayer(TECHNOLOGY *tech, INTSML layer)
-{
-	REGISTER INTBIG fun, funpure;
-
-	fun = layerfunction(tech, layer);
-	funpure = fun & LFTYPE;
-	if (funpure != LFSUBSTRATE && funpure != LFIMPLANT) return(0);
-	if ((fun&LFPTYPE) == 0) return(0);
-	return(1);
-}
-
-/*
- * Routine to convert layer "layer" to an appropriate layer, given that this layer
- * is on a well contact.  Converts the select layer to an unknown layer so that
- * it will be ignored.
- */
-INTSML erc_truewelllayer(TECHNOLOGY *tech, INTSML layer)
-{
-	REGISTER INTBIG fun;
-
-	fun = layerfunction(tech, layer) & LFTYPE;
-	if (fun == LFSUBSTRATE || fun == LFIMPLANT) return(-1);
-	return(layer);
-}
-
-/*
- * Routine to convert layer "layer" to an appropriate layer, given that this layer
- * is on a substrate contact.  Converts the select layer to P type.
- */
-INTSML erc_truesubstratelayer(TECHNOLOGY *tech, INTSML layer)
-{
-	REGISTER INTBIG fun, funpure, cfun;
-	REGISTER INTSML i;
-	static INTSML altselectlayer;
-	static TECHNOLOGY *cachedtech = NOTECHNOLOGY;
-
-	fun = layerfunction(tech, layer);
-	funpure = fun & LFTYPE;
+	if (funpure == LFWELL)
+	{
+		if ((fun&LFPTYPE) != 0) return(1);
+		return(2);
+	}
 	if (funpure == LFSUBSTRATE || funpure == LFIMPLANT)
 	{
-		if (cachedtech != tech)
-		{
-			for(i=0; i<tech->layercount; i++)
-			{
-				cfun = layerfunction(tech, i);
-				if ((cfun&LFTYPE) == funpure && (cfun&LFPTYPE) != (fun&LFPTYPE))
-					break;
-			}
-			if (i < tech->layercount)
-			{
-				cachedtech = tech;
-				altselectlayer = i;
-			}
-		}
-		if (cachedtech == tech) layer = altselectlayer;
+		if ((fun&LFPTYPE) != 0) return(3);
+		return(4);
 	}
-	return(layer);
+	return(0);
 }
 
-void erc_freeallsubwellcons(void)
+void erc_freeallwellcons(void)
 {
-	REGISTER SUBWELLCON *swc;
+	REGISTER WELLCON *wc;
 
-	while (erc_firstsubwellcon != NOSUBWELLCON)
+	while (erc_firstwellcon != NOWELLCON)
 	{
-		swc = erc_firstsubwellcon;
-		erc_firstsubwellcon = swc->nextsubwellcon;
-		efree((char *)swc);
+		wc = erc_firstwellcon;
+		erc_firstwellcon = wc->nextwellcon;
+		efree((char *)wc);
 	}
 }
 
-SUBWELLCON *erc_allocsubwellcon(void)
+WELLCON *erc_allocwellcon(void)
 {
-	REGISTER SUBWELLCON *swc;
+	REGISTER WELLCON *wc;
 
-	if (erc_subwellconfree != NOSUBWELLCON)
+	if (erc_wellconfree != NOWELLCON)
 	{
-		swc = erc_subwellconfree;
-		erc_subwellconfree = swc->nextsubwellcon;
+		wc = erc_wellconfree;
+		erc_wellconfree = wc->nextwellcon;
 	} else
 	{
-		swc = (SUBWELLCON *)emalloc(sizeof (SUBWELLCON), erc_aid->cluster);
-		if (swc == 0) return(NOSUBWELLCON);
+		wc = (WELLCON *)emalloc(sizeof (WELLCON), erc_aid->cluster);
+		if (wc == 0) return(NOWELLCON);
 	}
-	return(swc);
+	return(wc);
 }
 
-void erc_freesubwellcon(SUBWELLCON *swc)
+void erc_freewellcon(WELLCON *wc)
 {
-	swc->nextsubwellcon = erc_subwellconfree;
-	erc_subwellconfree = swc;
+	wc->nextwellcon = erc_wellconfree;
+	erc_wellconfree = wc;
 }
 
-void erc_freeallsubwellareas(void)
+void erc_freeallwellareas(void)
 {
-	REGISTER SUBWELLAREA *swa;
+	REGISTER WELLAREA *wa;
 
-	while (erc_firstsubwellarea != NOSUBWELLAREA)
+	while (erc_firstwellarea != NOWELLAREA)
 	{
-		swa = erc_firstsubwellarea;
-		erc_firstsubwellarea = swa->nextsubwellarea;
-		freepolygon(swa->poly);
-		efree((char *)swa);
+		wa = erc_firstwellarea;
+		erc_firstwellarea = wa->nextwellarea;
+		freepolygon(wa->poly);
+		efree((char *)wa);
 	}
 }
 
-SUBWELLAREA *erc_allocsubwellarea(void)
+WELLAREA *erc_allocwellarea(void)
 {
-	REGISTER SUBWELLAREA *swa;
+	REGISTER WELLAREA *wa;
 
-	if (erc_subwellareafree != NOSUBWELLAREA)
+	if (erc_wellareafree != NOWELLAREA)
 	{
-		swa = erc_subwellareafree;
-		erc_subwellareafree = swa->nextsubwellarea;
+		wa = erc_wellareafree;
+		erc_wellareafree = wa->nextwellarea;
 	} else
 	{
-		swa = (SUBWELLAREA *)emalloc(sizeof (SUBWELLAREA), erc_aid->cluster);
-		if (swa == 0) return(NOSUBWELLAREA);
+		wa = (WELLAREA *)emalloc(sizeof (WELLAREA), erc_aid->cluster);
+		if (wa == 0) return(NOWELLAREA);
 	}
-	return(swa);
+	return(wa);
 }
 
-void erc_freesubwellarea(SUBWELLAREA *swa)
+void erc_freewellarea(WELLAREA *wa)
 {
-	swa->nextsubwellarea = erc_subwellareafree;
-	erc_subwellareafree = swa;
+	wa->nextwellarea = erc_wellareafree;
+	erc_wellareafree = wa;
 }
 
 /******************** OPTIONS DIALOG ********************/
 
-/*
- * special case for the "ERC Options" dialog
- * Well contact in each area  = 3 (radio)
- * Well contact once          = 4 (radio)
- * Well contact not needed    = 5 (radio)
- * Sub. contact in each area  = 7 (radio)
- * Sub. contact once          = 8 (radio)
- * Sub. contact not needed    = 9 (radio)
- */
+/* ERC: Options Dialog */
+DIALOGITEM erc_optionsdialogitems[] =
+{
+ /*  1 */ {0, {108,396,132,460}, BUTTON, N_("OK")},
+ /*  2 */ {0, {108,296,132,360}, BUTTON, N_("Cancel")},
+ /*  3 */ {0, {32,8,48,228}, RADIO, N_("Must have contact in every area")},
+ /*  4 */ {0, {56,8,72,228}, RADIO, N_("Must have at least 1 contact")},
+ /*  5 */ {0, {80,8,96,228}, RADIO, N_("Do not check for contacts")},
+ /*  6 */ {0, {8,44,24,144}, MESSAGE, N_("For P-Well:")},
+ /*  7 */ {0, {32,248,48,480}, RADIO, N_("Must have contact in every area")},
+ /*  8 */ {0, {56,248,72,480}, RADIO, N_("Must have at least 1 contact")},
+ /*  9 */ {0, {80,248,96,480}, RADIO, N_("Do not check for contacts")},
+ /* 10 */ {0, {8,284,24,384}, MESSAGE, N_("For N-Well:")}
+};
+DIALOG erc_optionsdialog = {{50,75,191,564}, N_("ERC Options"), 0, 10, erc_optionsdialogitems};
+
+/* special items for the "ERC Options" dialog: */
+#define DERO_PWELLEACH   3		/* PWell contact in each area (radio) */
+#define DERO_PWELLONCE   4		/* PWell contact once (radio) */
+#define DERO_PWELLNOT    5		/* PWell contact not needed (radio) */
+#define DERO_NWELLEACH   7		/* NWell contact in each area (radio) */
+#define DERO_NWELLONCE   8		/* NWell contact once (radio) */
+#define DERO_NWELLNOT    9		/* NWell contact not needed (radio) */
+
 void erc_optionsdlog(void)
 {
 	REGISTER INTBIG itemHit, ercoptions, oldoptions;
@@ -866,17 +843,17 @@ void erc_optionsdlog(void)
 	if (var == NOVARIABLE) ercoptions = 0; else
 		ercoptions = var->addr;
 	oldoptions = ercoptions;
-	switch (ercoptions & WELLCONOPTIONS)
+	switch (ercoptions & PWELLCONOPTIONS)
 	{
-		case WELLCONPERAREA: DiaSetControl(3, 1);   break;
-		case WELLCONONCE:    DiaSetControl(4, 1);   break;
-		case WELLCONIGNORE:  DiaSetControl(5, 1);   break;
+		case PWELLCONPERAREA: DiaSetControl(DERO_PWELLEACH, 1);   break;
+		case PWELLCONONCE:    DiaSetControl(DERO_PWELLONCE, 1);   break;
+		case PWELLCONIGNORE:  DiaSetControl(DERO_PWELLNOT, 1);    break;
 	}
-	switch (ercoptions & SUBCONOPTIONS)
+	switch (ercoptions & NWELLCONOPTIONS)
 	{
-		case SUBCONPERAREA: DiaSetControl(7, 1);   break;
-		case SUBCONONCE:    DiaSetControl(8, 1);   break;
-		case SUBCONIGNORE:  DiaSetControl(9, 1);   break;
+		case NWELLCONPERAREA: DiaSetControl(DERO_NWELLEACH, 1);   break;
+		case NWELLCONONCE:    DiaSetControl(DERO_NWELLONCE, 1);   break;
+		case NWELLCONIGNORE:  DiaSetControl(DERO_NWELLNOT, 1);    break;
 	}
 
 	/* loop until done */
@@ -884,19 +861,21 @@ void erc_optionsdlog(void)
 	{
 		itemHit = DiaNextHit();
 		if (itemHit == OK || itemHit == CANCEL) break;
-		if (itemHit == 3 || itemHit == 4 || itemHit == 5)
+		if (itemHit == DERO_PWELLEACH || itemHit == DERO_PWELLONCE ||
+			itemHit == DERO_PWELLNOT)
 		{
-			DiaSetControl(3, 0);
-			DiaSetControl(4, 0);
-			DiaSetControl(5, 0);
+			DiaSetControl(DERO_PWELLEACH, 0);
+			DiaSetControl(DERO_PWELLONCE, 0);
+			DiaSetControl(DERO_PWELLNOT, 0);
 			DiaSetControl(itemHit, 1);
 			continue;
 		}
-		if (itemHit == 7 || itemHit == 8 || itemHit == 9)
+		if (itemHit == DERO_NWELLEACH || itemHit == DERO_NWELLONCE ||
+			itemHit == DERO_NWELLNOT)
 		{
-			DiaSetControl(7, 0);
-			DiaSetControl(8, 0);
-			DiaSetControl(9, 0);
+			DiaSetControl(DERO_NWELLEACH, 0);
+			DiaSetControl(DERO_NWELLONCE, 0);
+			DiaSetControl(DERO_NWELLNOT, 0);
 			DiaSetControl(itemHit, 1);
 			continue;
 		}
@@ -906,12 +885,12 @@ void erc_optionsdlog(void)
 	{
 		/* change state of ERC */
 		ercoptions = 0;
-		if (DiaGetControl(3) != 0) ercoptions |= WELLCONPERAREA; else
-			if (DiaGetControl(4) != 0) ercoptions |= WELLCONONCE; else
-				ercoptions |= WELLCONIGNORE;
-		if (DiaGetControl(7) != 0) ercoptions |= SUBCONPERAREA; else
-			if (DiaGetControl(8) != 0) ercoptions |= SUBCONONCE; else
-				ercoptions |= SUBCONIGNORE;
+		if (DiaGetControl(DERO_PWELLEACH) != 0) ercoptions |= PWELLCONPERAREA; else
+			if (DiaGetControl(DERO_PWELLONCE) != 0) ercoptions |= PWELLCONONCE; else
+				ercoptions |= PWELLCONIGNORE;
+		if (DiaGetControl(DERO_NWELLEACH) != 0) ercoptions |= NWELLCONPERAREA; else
+			if (DiaGetControl(DERO_NWELLONCE) != 0) ercoptions |= NWELLCONONCE; else
+				ercoptions |= NWELLCONIGNORE;
 		if (ercoptions != oldoptions)
 			(void)setvalkey((INTBIG)erc_aid, VAID, erc_optionskey, ercoptions, VINTEGER);
 	}
