@@ -39,6 +39,7 @@
 #include "efunction.h"
 #include "edialogs.h"
 #include "logeffort.h"
+#include "network.h"
 #include "usr.h"
 #include <math.h>
 
@@ -47,6 +48,7 @@ static KEYWORD leopt[] =
 {
 	{"analyze-path",       0,{NOKEY,NOKEY,NOKEY,NOKEY,NOKEY}},
 	{"analyze-facet",      0,{NOKEY,NOKEY,NOKEY,NOKEY,NOKEY}},
+	{"estimate-delay",     0,{NOKEY,NOKEY,NOKEY,NOKEY,NOKEY}},
 	{"set-options",        0,{NOKEY,NOKEY,NOKEY,NOKEY,NOKEY}},
 	{"set-capacitance",    0,{NOKEY,NOKEY,NOKEY,NOKEY,NOKEY}},
 	{"set-node-effort",    0,{NOKEY,NOKEY,NOKEY,NOKEY,NOKEY}},
@@ -94,38 +96,6 @@ typedef struct Ilenode
 	INTBIG           numoutputnodes;	/* size of array of output nodes */
 } LENODE;
 
-/* Logical Effort Options dialog */
-DIALOGITEM le_leoptionsdialogitems[] =
-{
- /*  1 */ {0, {96,180,120,244}, BUTTON, N_("OK")},
- /*  2 */ {0, {96,12,120,76}, BUTTON, N_("Cancel")},
- /*  3 */ {0, {8,8,24,160}, MESSAGE, N_("Maximum Stage Gain")},
- /*  4 */ {0, {8,168,24,220}, EDITTEXT, ""},
- /*  5 */ {0, {36,8,52,244}, CHECK, N_("Display intermediate capacitances")},
- /*  6 */ {0, {64,8,80,192}, CHECK, N_("Highlight components")}
-};
-DIALOG le_leoptionsdialog = {{75,75,206,329}, N_("Logical Effort Options"), 0, 6, le_leoptionsdialogitems};
-
-/* Logical Effort Capacitance dialog */
-DIALOGITEM le_capacitancedialogitems[] =
-{
- /*  1 */ {0, {36,132,60,200}, BUTTON, N_("OK")},
- /*  2 */ {0, {36,8,60,76}, BUTTON, N_("Cancel")},
- /*  3 */ {0, {8,8,24,100}, MESSAGE, N_("Capacitance:")},
- /*  4 */ {0, {8,108,24,200}, EDITTEXT, ""}
-};
-DIALOG le_capacitancedialog = {{75,75,146,286}, N_("Logical Effort Capacitance"), 0, 4, le_capacitancedialogitems};
-
-/* Logical Effort effort dialog */
-DIALOGITEM le_logeffortdialogitems[] =
-{
- /*  1 */ {0, {40,128,64,192}, BUTTON, N_("OK")},
- /*  2 */ {0, {40,12,64,76}, BUTTON, N_("Cancel")},
- /*  3 */ {0, {8,8,24,116}, MESSAGE, N_("Logical Effort:")},
- /*  4 */ {0, {8,128,24,192}, EDITTEXT, ""}
-};
-DIALOG le_logeffortdialog = {{75,75,149,281}, N_("Logical Effort"), 0, 4, le_logeffortdialogitems};
-
 #define DISPLAYCAPACITANCE  01
 #define HIGHLIGHTCOMPONENTS 02
 #define DEFAULTSTATE        DISPLAYCAPACITANCE
@@ -147,6 +117,7 @@ LENODE   *le_alloclenode(void);
 void      le_analyzefacet(void);
 void      le_analyzepath(void);
 char     *le_describenode(LENODE *le);
+void      le_estimatedelay(NODEPROTO *facet);
 void      le_figurebranching(void);
 void      le_freealllenodes(void);
 void      le_freelenode(LENODE *le);
@@ -213,6 +184,7 @@ INTSML le_set(INTSML count, char *par[])
 {
 	REGISTER INTSML l;
 	REGISTER VARIABLE *var;
+	REGISTER NODEPROTO *np;
 	REGISTER INTBIG state;
 	REGISTER char *pp;
 
@@ -246,6 +218,15 @@ INTSML le_set(INTSML count, char *par[])
 		return(0);
 	}
 
+	if (namesamen(pp, "estimate-delay", l) == 0)
+	{
+		/* analyze facet */
+		np = getcurfacet();
+		if (np == NONODEPROTO) return(0);
+		le_estimatedelay(np);
+		return(0);
+	}
+
 	if (namesamen(pp, "set-options", l) == 0 && l >= 5)
 	{
 		le_setoptions();
@@ -266,6 +247,23 @@ INTSML le_set(INTSML count, char *par[])
 	return(1);
 }
 
+/* Logical Effort Options dialog */
+DIALOGITEM le_leoptionsdialogitems[] =
+{
+ /*  1 */ {0, {96,180,120,244}, BUTTON, N_("OK")},
+ /*  2 */ {0, {96,12,120,76}, BUTTON, N_("Cancel")},
+ /*  3 */ {0, {8,8,24,160}, MESSAGE, N_("Maximum Stage Gain")},
+ /*  4 */ {0, {8,168,24,220}, EDITTEXT, ""},
+ /*  5 */ {0, {36,8,52,244}, CHECK, N_("Display intermediate capacitances")},
+ /*  6 */ {0, {64,8,80,192}, CHECK, N_("Highlight components")}
+};
+DIALOG le_leoptionsdialog = {{75,75,206,329}, N_("Logical Effort Options"), 0, 6, le_leoptionsdialogitems};
+
+/* special items for the "Logical Effort Options" dialog: */
+#define DLEO_MAXGAIN    4		/* Maximum Stage Gain (edit text) */
+#define DLEO_INTERCAP   5		/* Show intermediate capacitances (check) */
+#define DLEO_HIGHCOMP   6		/* Highlight components (check) */
+
 /*
  * Routine to interactively set the logical effort options.
  */
@@ -281,19 +279,19 @@ void le_setoptions(void)
 	var = getvalkey((INTBIG)le_aid, VAID, VINTEGER, le_state);
 	if (var == NOVARIABLE) state = DEFAULTSTATE; else
 		state = var->addr;
-	if ((state&DISPLAYCAPACITANCE) != 0) DiaSetControl(5, 1);
-	if ((state&HIGHLIGHTCOMPONENTS) != 0) DiaSetControl(6, 1);
+	if ((state&DISPLAYCAPACITANCE) != 0) DiaSetControl(DLEO_INTERCAP, 1);
+	if ((state&HIGHLIGHTCOMPONENTS) != 0) DiaSetControl(DLEO_HIGHCOMP, 1);
 	var = getvalkey((INTBIG)le_aid, VAID, VFLOAT, le_maximumstageeffort);
 	if (var == NOVARIABLE) maxstageeffort = MAXSTAGEEFFORT; else
 		maxstageeffort = castfloat(var->addr);
 
 	sprintf(line, "%g", maxstageeffort);
-	DiaSetText(-4, line);
+	DiaSetText(-DLEO_MAXGAIN, line);
 	for(;;)
 	{
 		itemHit = DiaNextHit();
 		if (itemHit == OK || itemHit == CANCEL) break;
-		if (itemHit == 5 || itemHit == 6)
+		if (itemHit == DLEO_INTERCAP || itemHit == DLEO_HIGHCOMP)
 		{
 			DiaSetControl(itemHit, (INTSML)(1 - DiaGetControl(itemHit)));
 			continue;
@@ -302,11 +300,11 @@ void le_setoptions(void)
 	if (itemHit == OK)
 	{
 		newstate = 0;
-		if (DiaGetControl(5) != 0) newstate |= DISPLAYCAPACITANCE;
-		if (DiaGetControl(6) != 0) newstate |= HIGHLIGHTCOMPONENTS;
+		if (DiaGetControl(DLEO_INTERCAP) != 0) newstate |= DISPLAYCAPACITANCE;
+		if (DiaGetControl(DLEO_HIGHCOMP) != 0) newstate |= HIGHLIGHTCOMPONENTS;
 		if (newstate != state)
 			(void)setvalkey((INTBIG)le_aid, VAID, le_state, newstate, VINTEGER);
-		newmaxstageeffort = (float)atof(DiaGetText(4));
+		newmaxstageeffort = (float)atof(DiaGetText(DLEO_MAXGAIN));
 		if (newmaxstageeffort != maxstageeffort)
 			(void)setvalkey((INTBIG)le_aid, VAID, le_maximumstageeffort,
 				castint(newmaxstageeffort), VFLOAT);
@@ -314,6 +312,19 @@ void le_setoptions(void)
 	DiaDoneDialog();
 }
 
+/* Logical Effort Capacitance dialog */
+DIALOGITEM le_capacitancedialogitems[] =
+{
+ /*  1 */ {0, {36,132,60,200}, BUTTON, N_("OK")},
+ /*  2 */ {0, {36,8,60,76}, BUTTON, N_("Cancel")},
+ /*  3 */ {0, {8,8,24,100}, MESSAGE, N_("Capacitance:")},
+ /*  4 */ {0, {8,108,24,200}, EDITTEXT, ""}
+};
+DIALOG le_capacitancedialog = {{75,75,146,286}, N_("Logical Effort Capacitance"), 0, 4, le_capacitancedialogitems};
+
+/* special items for the "Logical Effort capacitance" dialog: */
+#define DLEC_CAPVALUE    4		/* Capacitance (edit text) */
+
 /*
  * Routine to interactively set the capacitance value on the selected arc.
  */
@@ -333,10 +344,10 @@ void le_setcapacitance(void)
 	}
 	if (DiaInitDialog(&le_capacitancedialog) != 0) return;
 	c = le_getcapacitance(ai);
-	if (c == 0.0) DiaSetText(-4, "1.0"); else
+	if (c == 0.0) DiaSetText(-DLEC_CAPVALUE, "1.0"); else
 	{
 		sprintf(line, "%g", c);
-		DiaSetText(-4, line);
+		DiaSetText(-DLEC_CAPVALUE, line);
 	}
 	for(;;)
 	{
@@ -345,7 +356,7 @@ void le_setcapacitance(void)
 	}
 	if (itemHit == OK)
 	{
-		c = atof(DiaGetText(4));
+		c = atof(DiaGetText(DLEC_CAPVALUE));
 		if (c == 0.0)
 		{
 			var = getvalkey((INTBIG)ai, VARCINST, VSTRING, le_capacitance);
@@ -363,6 +374,19 @@ void le_setcapacitance(void)
 	DiaDoneDialog();
 }
 
+/* Logical Effort effort dialog */
+DIALOGITEM le_logeffortdialogitems[] =
+{
+ /*  1 */ {0, {40,128,64,192}, BUTTON, N_("OK")},
+ /*  2 */ {0, {40,12,64,76}, BUTTON, N_("Cancel")},
+ /*  3 */ {0, {8,8,24,116}, MESSAGE, N_("Logical Effort:")},
+ /*  4 */ {0, {8,128,24,192}, EDITTEXT, ""}
+};
+DIALOG le_logeffortdialog = {{75,75,149,281}, N_("Logical Effort"), 0, 4, le_logeffortdialogitems};
+
+/* special items for the "Logical Effort effort" dialog: */
+#define DLEE_EFFVALUE    4		/* Effort (edit text) */
+
 /*
  * Routine to interactively set the logical effort value on the selected node.
  */
@@ -391,7 +415,7 @@ void le_setlogicaleffort(void)
 	if (DiaInitDialog(&le_logeffortdialog) != 0) return;
 
 	sprintf(line, "%g", e);
-	DiaSetText(-4, line);
+	DiaSetText(-DLEE_EFFVALUE, line);
 	for(;;)
 	{
 		itemHit = DiaNextHit();
@@ -399,7 +423,7 @@ void le_setlogicaleffort(void)
 	}
 	if (itemHit == OK)
 	{
-		pt = DiaGetText(4);
+		pt = DiaGetText(DLEE_EFFVALUE);
 		startobjectchange((INTBIG)ni, VNODEINST);
 		sprintf(line, "g=%s", pt);
 		setvalkey((INTBIG)ni, VNODEINST, le_nodeeffort, (INTBIG)line, VSTRING|VDISPLAY);
@@ -523,6 +547,103 @@ void le_analyzepath(void)
 	}
 }
 
+/******************** DELAY ESTIMATION ********************/
+
+#define NONETDELAY ((NETDELAY *)-1)
+
+typedef struct Inetdelay
+{
+	float             numerator;
+	float             pdenominator;
+	float             ndenominator;
+	NETWORK          *net;
+	struct Inetdelay *nextnetdelay;
+} NETDELAY;
+
+/*
+ * Routine to analyze facet "facet" and build a list of ERC errors.
+ */
+void le_estimatedelay(NODEPROTO *facet)
+{
+	REGISTER NETWORK *net;
+	TRANSISTORINFO *p_gate, *n_gate, *p_active, *n_active;
+	REGISTER AREAPERIM *ap, *aplist, *nextap;
+	REGISTER INTBIG fun;
+	REGISTER NETDELAY *nd, *firstnd;
+	float numerator, coefficient, flambda;
+
+	firstnd = NONETDELAY;
+	flambda = (float)lambdaoffacet(facet);
+	for(net = facet->firstnetwork; net != NONETWORK; net = net->nextnetwork)
+	{
+		aplist = net_gathergeometry(net, &p_gate, &n_gate, &p_active, &n_active);
+
+		/* numerator has the area of all transistors connected by their gate */
+		numerator = (float)(p_gate->area + n_gate->area);
+ttyputmsg("Network %s has numerator:", describenetwork(net));
+ttyputmsg("      N Gates = %g, P Gates = %g", n_gate->area/flambda/flambda, p_gate->area/flambda/flambda);
+
+		/* numerator also sums area of each layer */
+		for(ap = aplist; ap != NOAREAPERIM; ap = nextap)
+		{
+			nextap = ap->nextareaperim;
+			fun = layerfunction(ap->tech, ap->layer);
+			if ((fun&LFTYPE) != LFDIFF && layerismetal(fun) == 0 && layerispoly(fun) == 0) continue;
+			coefficient = 1.0f;
+			if (layerismetal(fun) != 0) coefficient = 0.1f;
+			if (layerispoly(fun) != 0) coefficient = 0.1f;
+ttyputmsg("      Layer %s has %g x %g = %g", layername(ap->tech, ap->layer),
+	ap->area/flambda/flambda, coefficient, ap->area*coefficient/flambda/flambda);
+			numerator += (float)ap->area * coefficient;
+			efree((char *)ap);
+		}
+
+		/* save the results */
+		nd = (NETDELAY *)emalloc(sizeof (NETDELAY), le_aid->cluster);
+		if (nd == 0) break;
+		nd->pdenominator = (float)p_active->width / flambda;
+		nd->ndenominator = (float)n_active->width / flambda;
+		nd->numerator = numerator / flambda / flambda;
+if (n_active->width == 0) ttyputmsg("   N denominator undefined"); else
+  ttyputmsg("   N denominator = %g, ratio = %g", nd->ndenominator, nd->numerator/nd->ndenominator);
+if (p_active->width == 0) ttyputmsg("   P denominator undefined"); else
+  ttyputmsg("   P denominator = %g, ratio = %g", nd->pdenominator, nd->numerator/nd->pdenominator);
+if (n_active->width+p_active->width == 0) ttyputmsg("   Denominator undefined"); else
+  ttyputmsg("   Denominator = %g, ratio = %g", nd->ndenominator+nd->pdenominator,
+	nd->numerator/(nd->ndenominator+nd->pdenominator));
+		nd->net = net;
+		nd->nextnetdelay = firstnd;
+		firstnd = nd;
+	}
+
+	/* now sort by delay size */
+
+#if 0	/* code to show the results */
+	for(nd = firstnd; nd != NONETDELAY; nd = nd->nextnetdelay)
+	{
+		ttyputmsg(M_("Network %s:"), describenetwork(nd->net));
+		(void)initinfstr();
+		(void)formatinfstr(M_("Network %s has N delay "), describenetwork(nd->net));
+		if (nd->ndenominator == 0.0)
+		{
+			ttyputmsg(M_("   N delay UNDEFINED"));
+		} else
+		{
+			ttyputmsg(M_("   N delay is %g/%g = %g"), nd->numerator, nd->ndenominator,
+				nd->numerator / nd->ndenominator);
+		}
+		if (nd->pdenominator == 0.0)
+		{
+			ttyputmsg(M_("   P delay UNDEFINED"));
+		} else
+		{
+			ttyputmsg(M_("   P delay is %g/%g = %g"), nd->numerator, nd->pdenominator,
+				nd->numerator / nd->pdenominator);
+		}
+	}
+#endif
+}
+
 /******************** FACET EXTRACTION ********************/
 
 /*
@@ -1113,14 +1234,13 @@ void le_unwind(NODEINST *start, NODEINST *prev)
 	REGISTER PORTARCINST *pi;
 	REGISTER ARCINST *ai;
 	REGISTER INTBIG otherend;
-	char *extra;
 	double capacitance;
-	REGISTER INTSML func;
+	REGISTER INTBIG func;
 	LENODE *le;
 
 	/* add this node to the chain, if it can be analyzed */
 	le = NOLENODE;
-	func = nodefunction(start, &extra);
+	func = nodefunction(start, 0);
 	switch (func)
 	{
 		case NPPIN:
@@ -1241,10 +1361,9 @@ void le_propagatebranch(PORTARCINST *tpi, LENODE *final, double capacitance,
 	REGISTER ARCINST *ai;
 	REGISTER PORTARCINST *pi;
 	REGISTER INTBIG otherend;
-	REGISTER INTSML func;
+	REGISTER INTBIG func;
 	double cap;
 	REGISTER NODEINST *oni;
-	char *extra;
 
 	/* see if this arc has been examined */
 	ai = tpi->conarcinst;
@@ -1258,7 +1377,7 @@ void le_propagatebranch(PORTARCINST *tpi, LENODE *final, double capacitance,
 	/* see if the other end of the arc is a terminal point */
 	if (ai->end[0].portarcinst == tpi) otherend = 1; else otherend = 0;
 	oni = ai->end[otherend].nodeinst;
-	func = nodefunction(oni, &extra);
+	func = nodefunction(oni, 0);
 	if (func != NPPIN && func != NPCONTACT && func != NPNODE && func != NPCONNECT)
 	{
 		/* end of this chain: add capacitance into one of the path accumulators */
@@ -1282,16 +1401,15 @@ void le_propagatebranch(PORTARCINST *tpi, LENODE *final, double capacitance,
  */
 INTBIG le_getgatetype(NODEINST *ni, INTBIG *inputs)
 {
-	INTSML func;
-	REGISTER INTBIG inputnegates, outputnegates, thisend, isinput, gatetype;
-	char *extra, *inputport, *outputport1, *outputport2;
+	REGISTER INTBIG inputnegates, outputnegates, thisend, isinput, gatetype, func;
+	char *inputport, *outputport1, *outputport2;
 	REGISTER ARCINST *ai;
 	REGISTER PORTARCINST *pi;
 
 	/* determine the number of inputs to the gate */
 	*inputs = 0;
 	inputnegates = outputnegates = 0;
-	func = nodefunction(ni, &extra);
+	func = nodefunction(ni, 0);
 	inputport = "a";   outputport1 = outputport2 = "y";
 	if (func == NPTRANMOS || func == NPTRAPMOS)
 	{
