@@ -67,68 +67,9 @@ static KEYWORD projopt[] =
 COMCOMP proj_projp = {projopt, NOTOPLIST, NONEXTLIST, NOPARAMS, NOBACKUP,
 	0, " \t", M_("Project management tool action"), ""};
 
-/***** dialogs *****/
-
-/* Project User */
-DIALOGITEM proj_usersdialogitems[] =
-{
- /*  1 */ {0, {28,184,52,312}, BUTTON, N_("Select User")},
- /*  2 */ {0, {188,212,212,276}, BUTTON, N_("Cancel")},
- /*  3 */ {0, {28,4,220,172}, SCROLL, ""},
- /*  4 */ {0, {8,60,24,112}, MESSAGE, N_("Users:")},
- /*  5 */ {0, {136,184,160,312}, BUTTON, N_("Delete User")},
- /*  6 */ {0, {64,184,88,312}, BUTTON, N_("Change Password")},
- /*  7 */ {0, {100,184,124,312}, BUTTON, N_("New User")}
-};
-DIALOG proj_usersdialog = {{50,75,279,398}, N_("User Name"), 0, 7, proj_usersdialogitems};
-
-/* Project Password */
-DIALOGITEM proj_passworddialogitems[] =
-{
- /*  1 */ {0, {72,12,96,76}, BUTTON, N_("OK")},
- /*  2 */ {0, {72,132,96,196}, BUTTON, N_("Cancel")},
- /*  3 */ {0, {12,8,28,205}, MESSAGE, N_("Password for")},
- /*  4 */ {0, {40,48,56,165}, EDITTEXT, ""}
-};
-DIALOG proj_passworddialog = {{298,75,404,292}, N_("User / Password"), 0, 4, proj_passworddialogitems};
-
-/* Project List */
-DIALOGITEM proj_listdialogitems[] =
-{
- /*  1 */ {0, {204,312,228,376}, BUTTON, N_("Done")},
- /*  2 */ {0, {4,4,196,376}, SCROLL, ""},
- /*  3 */ {0, {260,4,324,376}, MESSAGE, ""},
- /*  4 */ {0, {240,4,256,86}, MESSAGE, N_("Comments:")},
- /*  5 */ {0, {204,120,228,220}, BUTTON, N_("Check It Out")},
- /*  6 */ {0, {236,4,237,376}, DIVIDELINE, ""},
- /*  7 */ {0, {204,8,228,108}, BUTTON, N_("Check It In")},
- /*  8 */ {0, {204,232,228,296}, BUTTON, N_("Update")}
-};
-DIALOG proj_listdialog = {{50,75,383,462}, N_("Project Management"), 0, 8, proj_listdialogitems};
-
-/* Project Comments */
-DIALOGITEM proj_commentsdialogitems[] =
-{
- /*  1 */ {0, {104,244,128,308}, BUTTON, N_("OK")},
- /*  2 */ {0, {104,48,128,112}, BUTTON, N_("Cancel")},
- /*  3 */ {0, {4,8,20,363}, MESSAGE, N_("Reason for checking out facet")},
- /*  4 */ {0, {28,12,92,358}, EDITTEXT, ""}
-};
-DIALOG proj_commentsdialog = {{108,75,248,447}, N_("Project Comments"), 0, 4, proj_commentsdialogitems};
-
-/* Project Old Version */
-DIALOGITEM proj_oldversdialogitems[] =
-{
- /*  1 */ {0, {176,224,200,288}, BUTTON, N_("OK")},
- /*  2 */ {0, {176,16,200,80}, BUTTON, N_("Cancel")},
- /*  3 */ {0, {4,8,20,308}, MESSAGE, N_("Old version of facet")},
- /*  4 */ {0, {28,12,166,312}, SCROLL, ""}
-};
-DIALOG proj_oldversdialog = {{108,75,318,396}, N_("Get Old Version of Facet"), 0, 4, proj_oldversdialogitems};
-
 /***** facet checking queue *****/
 
-#define	NOFCHECK ((FCHECK *)-1)
+#define NOFCHECK ((FCHECK *)-1)
 
 typedef struct Ifcheck
 {
@@ -142,7 +83,7 @@ static FCHECK *proj_fcheckfree = NOFCHECK;
 
 /***** user database information *****/
 
-#define	NOPUSER ((PUSER *)-1)
+#define NOPUSER ((PUSER *)-1)
 #define PUSERFILE "projectusers"
 
 typedef struct Ipuser
@@ -156,7 +97,7 @@ PUSER *proj_users = NOPUSER;
 
 /***** project file information *****/
 
-#define	NOPROJECTFACET ((PROJECTFACET *)-1)
+#define NOPROJECTFACET ((PROJECTFACET *)-1)
 
 typedef struct Iprojectfacet
 {
@@ -481,6 +422,17 @@ void proj_modifyportproto(PORTPROTO *pp, NODEINST *oni, PORTPROTO *opp)
 	proj_queuecheck(pp->parent);
 }
 
+void proj_modifydescript(INTBIG addr, INTBIG type, INTBIG key, INTBIG old)
+{
+	if (proj_ignorechanges != 0 || proj_source == proj_aid) return;
+	switch (type&VTYPE)
+	{
+		case VNODEINST:  proj_queuecheck(((NODEINST *)addr)->parent);   break;
+		case VARCINST:   proj_queuecheck(((ARCINST *)addr)->parent);    break;
+		case VPORTPROTO: proj_queuecheck(((PORTPROTO *)addr)->parent);  break;
+	}
+}
+
 void proj_newobject(INTBIG addr, INTBIG type)
 {
 	if (proj_ignorechanges != 0 || proj_source == proj_aid) return;
@@ -503,6 +455,73 @@ void proj_killobject(INTBIG addr, INTBIG type)
 	}
 }
 
+void proj_newvariable(INTBIG addr, INTBIG type, INTBIG key, INTBIG newtype)
+{
+	if (proj_ignorechanges != 0 || proj_source == proj_aid) return;
+	switch (type&VTYPE)
+	{
+		case VNODEINST:  proj_queuecheck(((NODEINST *)addr)->parent);   break;
+		case VARCINST:   proj_queuecheck(((ARCINST *)addr)->parent);    break;
+		case VPORTPROTO: proj_queuecheck(((PORTPROTO *)addr)->parent);  break;
+		case VNODEPROTO:
+			if (key != proj_lockedkey) proj_queuecheck((NODEPROTO *)addr);
+            break;
+	}
+}
+
+void proj_killvariable(INTBIG addr, INTBIG type, INTBIG key, INTBIG oldaddr, INTBIG oldtype,
+	INTBIG olddescript)
+{
+	if (proj_ignorechanges != 0 || proj_source == proj_aid) return;
+	switch (type&VTYPE)
+	{
+		case VNODEINST:  proj_queuecheck(((NODEINST *)addr)->parent);   break;
+		case VARCINST:   proj_queuecheck(((ARCINST *)addr)->parent);    break;
+		case VPORTPROTO: proj_queuecheck(((PORTPROTO *)addr)->parent);  break;
+		case VNODEPROTO:
+			if (key != proj_lockedkey) proj_queuecheck((NODEPROTO *)addr);
+            break;
+	}
+}
+
+void proj_modifyvariable(INTBIG addr, INTBIG type, INTBIG key, INTBIG vartype,
+	INTBIG aindex, INTBIG oldvalue)
+{
+	if (proj_ignorechanges != 0 || proj_source == proj_aid) return;
+	switch (type&VTYPE)
+	{
+		case VNODEINST:  proj_queuecheck(((NODEINST *)addr)->parent);   break;
+		case VARCINST:   proj_queuecheck(((ARCINST *)addr)->parent);    break;
+		case VPORTPROTO: proj_queuecheck(((PORTPROTO *)addr)->parent);  break;
+		case VNODEPROTO: proj_queuecheck((NODEPROTO *)addr);            break;
+	}
+}
+
+void proj_insertvariable(INTBIG addr, INTBIG type, INTBIG key, INTBIG vartype, INTBIG index)
+{
+	if (proj_ignorechanges != 0 || proj_source == proj_aid) return;
+	switch (type&VTYPE)
+	{
+		case VNODEINST:  proj_queuecheck(((NODEINST *)addr)->parent);   break;
+		case VARCINST:   proj_queuecheck(((ARCINST *)addr)->parent);    break;
+		case VPORTPROTO: proj_queuecheck(((PORTPROTO *)addr)->parent);  break;
+		case VNODEPROTO: proj_queuecheck((NODEPROTO *)addr);            break;
+	}
+}
+
+void proj_deletevariable(INTBIG addr, INTBIG type, INTBIG key, INTBIG vartype, INTBIG index,
+	INTBIG oldvalue)
+{
+	if (proj_ignorechanges != 0 || proj_source == proj_aid) return;
+	switch (type&VTYPE)
+	{
+		case VNODEINST:  proj_queuecheck(((NODEINST *)addr)->parent);   break;
+		case VARCINST:   proj_queuecheck(((ARCINST *)addr)->parent);    break;
+		case VPORTPROTO: proj_queuecheck(((PORTPROTO *)addr)->parent);  break;
+		case VNODEPROTO: proj_queuecheck((NODEPROTO *)addr);            break;
+	}
+}
+
 void proj_readlibrary(LIBRARY *lib)
 {
 	REGISTER NODEPROTO *np;
@@ -863,6 +882,21 @@ void proj_checkout(char *facetname, INTSML showfacet)
 	}
 }
 
+/* Project Old Version */
+DIALOGITEM proj_oldversdialogitems[] =
+{
+ /*  1 */ {0, {176,224,200,288}, BUTTON, N_("OK")},
+ /*  2 */ {0, {176,16,200,80}, BUTTON, N_("Cancel")},
+ /*  3 */ {0, {4,8,20,140}, MESSAGE, N_("Old version of facet")},
+ /*  4 */ {0, {28,12,166,312}, SCROLL, ""},
+ /*  5 */ {0, {4,140,20,312}, MESSAGE, ""}
+};
+DIALOG proj_oldversdialog = {{108,75,318,396}, N_("Get Old Version of Facet"), 0, 5, proj_oldversdialogitems};
+
+/* special items for the "get old version" dialog: */
+#define DPRV_FACETLIST  4		/* Facet list (scroll) */
+#define DPRV_FACETNAME  5		/* Facet name (stat text) */
+
 void proj_getoldversion(char *facetname)
 {
 	PROJECTFACET *pf, statpf;
@@ -908,10 +942,12 @@ void proj_getoldversion(char *facetname)
 	sep[0] = DIRSEP;   sep[1] = 0;
 	strcat(projectpath, sep);
 	strcat(projectpath, pf->cellname);
+	strcat(projectpath, sep);
 	count = filesindirectory(projectpath, &filelist);
 
 	if (DiaInitDialog(&proj_oldversdialog) != 0) return;
-	DiaInitTextDialog(4, DiaNullDlogList, DiaNullDlogItem, DiaNullDlogDone, -1,
+	DiaSetText(DPRV_FACETNAME, facetname);
+	DiaInitTextDialog(DPRV_FACETLIST, DiaNullDlogList, DiaNullDlogItem, DiaNullDlogDone, -1,
 		SCSELMOUSE);
 	for(i=0; i<count; i++)
 	{
@@ -932,19 +968,19 @@ void proj_getoldversion(char *facetname)
 		strcat(projectfile, filelist[i]);
 		date = filedate(projectfile);
 		sprintf(line, _("Version %ld, %s"), version, timetostring(date));
-		DiaStuffLine(4, line);
+		DiaStuffLine(DPRV_FACETLIST, line);
 	}
-	DiaSelectLine(4, -1);
+	DiaSelectLine(DPRV_FACETLIST, -1);
 	for(;;)
 	{
 		itemHit = DiaNextHit();
 		if (itemHit == OK || itemHit == CANCEL) break;
 	}
 	version = -1;
-	i = DiaGetCurLine(4);
+	i = DiaGetCurLine(DPRV_FACETLIST);
 	if (i >= 0)
 	{
-		pt = DiaGetScrollLine(4, i);
+		pt = DiaGetScrollLine(DPRV_FACETLIST, i);
 		if (strlen(pt) > 8) version = atoi(&pt[8]);
 	}
 	DiaDoneDialog();
@@ -960,6 +996,7 @@ void proj_getoldversion(char *facetname)
 		ttyputerr(_("Error retrieving old version of facet"));
 	proj_marklocked(np, 0);
 	(*el_curconstraint->solve)(np);
+	ttyputmsg(_("Facet %s is now in this library"), describenodeproto(np));
 }
 
 void proj_update(LIBRARY *lib)
@@ -1384,6 +1421,25 @@ void proj_deletefacet(char *facetname)
 
 /************************ USER DATABASE ***********************/
 
+/* Project User */
+DIALOGITEM proj_usersdialogitems[] =
+{
+ /*  1 */ {0, {28,184,52,312}, BUTTON, N_("Select User")},
+ /*  2 */ {0, {188,212,212,276}, BUTTON, N_("Cancel")},
+ /*  3 */ {0, {28,4,220,172}, SCROLL, ""},
+ /*  4 */ {0, {8,60,24,112}, MESSAGE, N_("Users:")},
+ /*  5 */ {0, {136,184,160,312}, BUTTON, N_("Delete User")},
+ /*  6 */ {0, {64,184,88,312}, BUTTON, N_("Change Password")},
+ /*  7 */ {0, {100,184,124,312}, BUTTON, N_("New User")}
+};
+DIALOG proj_usersdialog = {{50,75,279,398}, N_("User Name"), 0, 7, proj_usersdialogitems};
+
+/* special items for the "User selection" dialog: */
+#define DPRU_USERLIST  3		/* User list (scroll) */
+#define DPRU_DELUSER   5		/* Delete user (button) */
+#define DPRU_CHPASS    6		/* Change password (button) */
+#define DPRU_NEWUSER   7		/* New user (button) */
+
 /*
  * Routine to obtain the user's name.  If "newone" is nonzero, force input of a name,
  * otherwise, only ask if unknown.  Returns nonzero if user name is invalid.
@@ -1459,7 +1515,7 @@ INTSML proj_getusername(INTSML newone, LIBRARY *lib)
 	/* show the users dialog */
 	writeback = 0;
 	if (DiaInitDialog(&proj_usersdialog) != 0) return(1);
-	DiaInitTextDialog(3, proj_initusers, proj_nextuser, DiaNullDlogDone, 0,
+	DiaInitTextDialog(DPRU_USERLIST, proj_initusers, proj_nextuser, DiaNullDlogDone, 0,
 		SCSELMOUSE | SCSELKEY | SCDOUBLEQUIT);
 	for(;;)
 	{
@@ -1468,9 +1524,9 @@ INTSML proj_getusername(INTSML newone, LIBRARY *lib)
 		if (itemHit == OK)
 		{
 			/* validate the user name */
-			i = DiaGetCurLine(3);
+			i = DiaGetCurLine(DPRU_USERLIST);
 			if (i < 0) continue;
-			pt = DiaGetScrollLine(3, i);
+			pt = DiaGetScrollLine(DPRU_USERLIST, i);
 			for(pu = proj_users; pu != NOPUSER; pu = pu->nextpuser)
 				if (strcmp(pt, pu->username) == 0) break;
 			if (pu == NOPUSER) continue;
@@ -1478,18 +1534,19 @@ INTSML proj_getusername(INTSML newone, LIBRARY *lib)
 			if (strcmp(pwd, pu->userpassword) == 0)
 			{
 				strcpy(proj_username, pu->username);
-				(void)setvalkey((INTBIG)lib, VLIBRARY, proj_userkey, (INTBIG)proj_username, VSTRING);
+				(void)setvalkey((INTBIG)lib, VLIBRARY, proj_userkey,
+					(INTBIG)proj_username, VSTRING);
 				break;
 			}
 			ttyputmsg(_("Incorrect password"));
 			continue;
 		}
-		if (itemHit == 5)
+		if (itemHit == DPRU_DELUSER)
 		{
 			/* delete user */
-			i = DiaGetCurLine(3);
+			i = DiaGetCurLine(DPRU_USERLIST);
 			if (i < 0) continue;
-			pt = DiaGetScrollLine(3, i);
+			pt = DiaGetScrollLine(DPRU_USERLIST, i);
 			lastpu = NOPUSER;
 			for(pu = proj_users; pu != NOPUSER; pu = pu->nextpuser)
 			{
@@ -1508,16 +1565,17 @@ INTSML proj_getusername(INTSML newone, LIBRARY *lib)
 			efree(pu->username);
 			efree(pu->userpassword);
 			efree((char *)pu);
-			DiaLoadTextDialog(3, proj_initusers, proj_nextuser, DiaNullDlogDone, 0);
+			DiaLoadTextDialog(DPRU_USERLIST, proj_initusers, proj_nextuser,
+				DiaNullDlogDone, 0);
 			writeback = 1;
 			continue;
 		}
-		if (itemHit == 6)
+		if (itemHit == DPRU_CHPASS)
 		{
 			/* change password */
-			i = DiaGetCurLine(3);
+			i = DiaGetCurLine(DPRU_USERLIST);
 			if (i < 0) continue;
-			pt = DiaGetScrollLine(3, i);
+			pt = DiaGetScrollLine(DPRU_USERLIST, i);
 			for(pu = proj_users; pu != NOPUSER; pu = pu->nextpuser)
 				if (strcmp(pt, pu->username) == 0) break;
 			if (pu == NOPUSER) continue;
@@ -1539,7 +1597,7 @@ INTSML proj_getusername(INTSML newone, LIBRARY *lib)
 			writeback = 1;
 			continue;
 		}
-		if (itemHit == 7)
+		if (itemHit == DPRU_NEWUSER)
 		{
 			/* new user */
 			pt = proj_wantpassword(0, "");
@@ -1559,7 +1617,8 @@ INTSML proj_getusername(INTSML newone, LIBRARY *lib)
 			(void)allocstring(&pu->userpassword, pwdline, proj_aid->cluster);
 			pu->nextpuser = proj_users;
 			proj_users = pu;
-			DiaLoadTextDialog(3, proj_initusers, proj_nextuser, DiaNullDlogDone, 0);
+			DiaLoadTextDialog(DPRU_USERLIST, proj_initusers, proj_nextuser,
+				DiaNullDlogDone, 0);
 			writeback = 1;
 			continue;
 		}
@@ -1602,6 +1661,20 @@ INTSML proj_getusername(INTSML newone, LIBRARY *lib)
 	return(0);
 }
 
+/* Project Password */
+DIALOGITEM proj_passworddialogitems[] =
+{
+ /*  1 */ {0, {72,12,96,76}, BUTTON, N_("OK")},
+ /*  2 */ {0, {72,132,96,196}, BUTTON, N_("Cancel")},
+ /*  3 */ {0, {12,8,28,205}, MESSAGE, N_("Password for")},
+ /*  4 */ {0, {40,48,56,165}, EDITTEXT, ""}
+};
+DIALOG proj_passworddialog = {{298,75,404,292}, N_("User / Password"), 0, 4, proj_passworddialogitems};
+
+/* special items for the "Password prompt" dialog: */
+#define DPRP_USERNAME  3		/* User name (stat text) */
+#define DPRP_PASSWORD  4		/* Password (edit text) */
+
 /*
  * Routine to prompt for a user name/password, according to the "mode" and return
  * the string.  "mode" is:
@@ -1619,28 +1692,28 @@ char *proj_wantpassword(INTSML mode, char *username)
 	switch (mode)
 	{
 		case 0:
-			DiaSetText(3, _("New User Name"));
+			DiaSetText(DPRP_USERNAME, _("New User Name"));
 			break;
 		case 1:
 			sprintf(typedstuff, _("Password for %s"), username);
-			DiaSetText(3, typedstuff);
+			DiaSetText(DPRP_USERNAME, typedstuff);
 			break;
 		case 2:
 			sprintf(typedstuff, _("New Password for %s"), username);
-			DiaSetText(3, typedstuff);
+			DiaSetText(DPRP_USERNAME, typedstuff);
 			break;
 		case 3:
 			sprintf(typedstuff, _("Verify Password for %s"), username);
-			DiaSetText(3, typedstuff);
+			DiaSetText(DPRP_USERNAME, typedstuff);
 			break;
 	}
-	if (mode != 0) DiaOpaqueEdit(4);
+	if (mode != 0) DiaOpaqueEdit(DPRP_PASSWORD);
 	for(;;)
 	{
 		itemHit = DiaNextHit();
 		if (itemHit == CANCEL || itemHit == OK) break;
 	}
-	strcpy(typedstuff, DiaGetText(4));
+	strcpy(typedstuff, DiaGetText(DPRP_PASSWORD));
 	DiaDoneDialog();
 	if (mode != 0)
 		myencrypt(typedstuff, "BicIsSchediwy");
@@ -1786,6 +1859,27 @@ INTSML proj_getprojinfo(LIBRARY *lib, char *path, char *projfile)
 	return(0);
 }
 
+/* Project List */
+DIALOGITEM proj_listdialogitems[] =
+{
+ /*  1 */ {0, {204,312,228,376}, BUTTON, N_("Done")},
+ /*  2 */ {0, {4,4,196,376}, SCROLL, ""},
+ /*  3 */ {0, {260,4,324,376}, MESSAGE, ""},
+ /*  4 */ {0, {240,4,256,86}, MESSAGE, N_("Comments:")},
+ /*  5 */ {0, {204,120,228,220}, BUTTON, N_("Check It Out")},
+ /*  6 */ {0, {236,4,237,376}, DIVIDELINE, ""},
+ /*  7 */ {0, {204,8,228,108}, BUTTON, N_("Check It In")},
+ /*  8 */ {0, {204,232,228,296}, BUTTON, N_("Update")}
+};
+DIALOG proj_listdialog = {{50,75,383,462}, N_("Project Management"), 0, 8, proj_listdialogitems};
+
+/* special items for the "Project list" dialog: */
+#define DPRL_PROJLIST  2		/* Project list (scroll) */
+#define DPRL_COMMENTS  3		/* Comments (stat text) */
+#define DPRL_CHECKOUT  5		/* Check out (button) */
+#define DPRL_CHECKIN   7		/* Check in (button) */
+#define DPRL_UPDATE    8		/* Update (button) */
+
 void proj_showlistdialog(LIBRARY *lib)
 {
 	REGISTER INTBIG itemHit, i, j;
@@ -1799,8 +1893,8 @@ void proj_showlistdialog(LIBRARY *lib)
 
 	/* show the dialog */
 	if (DiaInitDialog(&proj_listdialog) != 0) return;
-	DiaInitTextDialog(2, DiaNullDlogList, DiaNullDlogItem, DiaNullDlogDone, -1,
-		SCSELMOUSE | SCREPORT);
+	DiaInitTextDialog(DPRL_PROJLIST, DiaNullDlogList, DiaNullDlogItem, DiaNullDlogDone,
+		-1, SCSELMOUSE | SCREPORT);
 
 	/* load project information into the scroll area */
 	if (proj_loadlist(lib) != 0)
@@ -1813,23 +1907,25 @@ void proj_showlistdialog(LIBRARY *lib)
 	{
 		itemHit = DiaNextHit();
 		if (itemHit == OK) break;
-		if (itemHit == 8)
+		if (itemHit == DPRL_UPDATE)
 		{
 			/* update project */
 			proj_update(lib);
 			(void)proj_loadlist(lib);
+			us_endbatch();
 			continue;
 		}
-		if (itemHit == 5 || itemHit == 7 || itemHit == 2)
+		if (itemHit == DPRL_CHECKOUT || itemHit == DPRL_CHECKIN ||
+			itemHit == DPRL_PROJLIST)
 		{
 			/* figure out which facet is selected */
-			i = DiaGetCurLine(2);
+			i = DiaGetCurLine(DPRL_PROJLIST);
 			if (i < 0) continue;
 			for(j=0, pf = proj_firstprojectfacet; pf != NOPROJECTFACET; pf = pf->nextprojectfacet, j++)
 				if (i == j) break;
 			if (pf == NOPROJECTFACET) continue;
 
-			if (itemHit == 5)
+			if (itemHit == DPRL_CHECKOUT)
 			{
 				/* check it out */
 				(void)initinfstr();
@@ -1842,9 +1938,10 @@ void proj_showlistdialog(LIBRARY *lib)
 				}
 				proj_checkout(returninfstr(), 0);
 				(void)proj_loadlist(lib);
+				us_endbatch();
 				continue;
 			}
-			if (itemHit == 7)
+			if (itemHit == DPRL_CHECKIN)
 			{
 				/* check it in */
 				(void)initinfstr();
@@ -1859,11 +1956,13 @@ void proj_showlistdialog(LIBRARY *lib)
 				(void)proj_loadlist(lib);
 				continue;
 			}
-			if (itemHit == 2)
+			if (itemHit == DPRL_PROJLIST)
 			{
-				if (*pf->comment != 0) DiaSetText(3, pf->comment);
-				if (*pf->owner == 0) DiaUnDimItem(5); else DiaDimItem(5);
-				if (strcmp(pf->owner, proj_username) == 0) DiaUnDimItem(7); else DiaDimItem(7);
+				if (*pf->comment != 0) DiaSetText(DPRL_COMMENTS, pf->comment);
+				if (*pf->owner == 0) DiaUnDimItem(DPRL_CHECKOUT); else
+					DiaDimItem(DPRL_CHECKOUT);
+				if (strcmp(pf->owner, proj_username) == 0) DiaUnDimItem(DPRL_CHECKIN); else
+					DiaDimItem(DPRL_CHECKIN);
 				continue;
 			}
 		}
@@ -1871,7 +1970,6 @@ void proj_showlistdialog(LIBRARY *lib)
 	DiaDoneDialog();
 }
 
-
 /*
  * Routine to display the current project information in the list dialog.
  * Returns nonzero on error.
@@ -1914,7 +2012,7 @@ INTSML proj_loadlist(LIBRARY *lib)
 	curfacet = getcurfacet();
 
 	/* show what is in the project file */
-	DiaLoadTextDialog(2, DiaNullDlogList, DiaNullDlogItem, DiaNullDlogDone, -1);
+	DiaLoadTextDialog(DPRL_PROJLIST, DiaNullDlogList, DiaNullDlogItem, DiaNullDlogDone, -1);
 	whichline = -1;
 	thisline = 0;
 	uptodate = 1;
@@ -1965,29 +2063,45 @@ INTSML proj_loadlist(LIBRARY *lib)
 				strcat(line, pf->owner);
 			}
 		}
-		DiaStuffLine(2, line);
+		DiaStuffLine(DPRL_PROJLIST, line);
 		thisline++;
 	}
-	DiaSelectLine(2, whichline);
+	DiaSelectLine(DPRL_PROJLIST, whichline);
 
-	DiaDimItem(5);
-	DiaDimItem(7);
+	DiaDimItem(DPRL_CHECKOUT);
+	DiaDimItem(DPRL_CHECKIN);
 	if (whichline >= 0)
 	{
-		if (*curpf->comment != 0) DiaSetText(3, curpf->comment);
-		if (*curpf->owner == 0) DiaUnDimItem(5); else DiaDimItem(5);
-		if (strcmp(curpf->owner, proj_username) == 0) DiaUnDimItem(7); else DiaDimItem(7);
+		if (*curpf->comment != 0) DiaSetText(DPRL_COMMENTS, curpf->comment);
+		if (*curpf->owner == 0) DiaUnDimItem(DPRL_CHECKOUT); else
+			DiaDimItem(DPRL_CHECKOUT);
+		if (strcmp(curpf->owner, proj_username) == 0) DiaUnDimItem(DPRL_CHECKIN); else
+			DiaDimItem(DPRL_CHECKIN);
 	}
 
-	if (uptodate != 0) 	DiaDimItem(8); else
+	if (uptodate != 0) 	DiaDimItem(DPRL_UPDATE); else
 	{
 		ttyputmsg(_("Your library does not contain the most recent additions to the project."));
 		ttyputmsg(_("You should do an 'Update' to make it current."));
-		DiaUnDimItem(8);
+		DiaUnDimItem(DPRL_UPDATE);
 	}
 	return(0);
 }
 
+/* Project Comments */
+DIALOGITEM proj_commentsdialogitems[] =
+{
+ /*  1 */ {0, {104,244,128,308}, BUTTON, N_("OK")},
+ /*  2 */ {0, {104,48,128,112}, BUTTON, N_("Cancel")},
+ /*  3 */ {0, {4,8,20,363}, MESSAGE, N_("Reason for checking out facet")},
+ /*  4 */ {0, {28,12,92,358}, EDITTEXT, ""}
+};
+DIALOG proj_commentsdialog = {{108,75,248,447}, N_("Project Comments"), 0, 4, proj_commentsdialogitems};
+
+/* special items for the "Project list" dialog: */
+#define DPRC_COMMENT_L  3		/* Comment label (stat text) */
+#define DPRC_COMMENT    4		/* Comment (edit text) */
+
 /*
  * Routine to obtain comments about a checkin/checkout for facet "pf".  "direction" is
  * either "in" or "out".  Returns nonzero if the dialog is cancelled.
@@ -1999,15 +2113,15 @@ INTSML proj_getcomments(PROJECTFACET *pf, char *direction)
 
 	if (DiaInitDialog(&proj_commentsdialog) != 0) return(1);
 	sprintf(line, _("Reason for checking-%s facet %s"), direction, pf->cellname);
-	DiaSetText(3, line);
-	DiaSetText(-4, pf->comment);
+	DiaSetText(DPRC_COMMENT_L, line);
+	DiaSetText(-DPRC_COMMENT, pf->comment);
 	for(;;)
 	{
 		itemHit = DiaNextHit();
 		if (itemHit == CANCEL || itemHit == OK) break;
 	}
 	if (itemHit == OK)
-		(void)reallocstring(&pf->comment, DiaGetText(4), proj_aid->cluster);
+		(void)reallocstring(&pf->comment, DiaGetText(DPRC_COMMENT), proj_aid->cluster);
 	DiaDoneDialog();
 	if (itemHit == CANCEL) return(1);
 	return(0);
@@ -2453,7 +2567,7 @@ INTSML proj_usenewestversion(NODEPROTO *oldnp, NODEPROTO *newnp)
 	for(ni = oldnp->firstinst; ni != NONODEINST; ni = nextni)
 	{
 		nextni = ni->nextinst;
-		newni = replacenodeinst(ni, newnp, 0);
+		newni = replacenodeinst(ni, newnp, 0, 0);
 		if (newni == NONODEINST)
 		{
 			ttyputerr(_("Failed to update instance of %s in %s"), describenodeproto(newnp),
@@ -2692,7 +2806,7 @@ NODEPROTO *copyskeleton(NODEPROTO *fromnp, LIBRARY *tolib)
 	INTBIG newx, newy;
 	XARRAY trans, localtrans, ntrans;
 	REGISTER NODEPROTO *np;
-	REGISTER PORTPROTO *pp, *rpp;
+	REGISTER PORTPROTO *pp, *rpp, *newpp;
 	REGISTER NODEINST *ni, *newni;
 
 	/* cannot skeletonize text-only views */
@@ -2744,7 +2858,7 @@ NODEPROTO *copyskeleton(NODEPROTO *fromnp, LIBRARY *tolib)
 		endobjectchange((INTBIG)newni, VNODEINST);
 
 		/* export the port from the node */
-		(void)newportproto(np, newni, rpp, pp->protoname);
+		newpp = newportproto(np, newni, rpp, pp->protoname);
 	}
 
 	/* make sure facet is the same size */
