@@ -134,10 +134,10 @@ INTSML pla_set(INTSML count, char *par[])
 
 	if (count == 0)
 	{
-		count = ttygetparam(_("PLA option:"), &pla_plap, MAXPARS, par);
+		count = ttygetparam(M_("PLA option:"), &pla_plap, MAXPARS, par);
 		if (count == 0)
 		{
-			ttyputerr(_("Aborted"));
+			ttyputerr(M_("Aborted"));
 			return(1);
 		}
 	}
@@ -158,10 +158,10 @@ INTSML pla_set(INTSML count, char *par[])
 	{
 		if (count == 1)
 		{
-			count = ttygetparam(_("CMOS PLA option:"), &placmosp, MAXPARS-1, &par[1]) + 1;
+			count = ttygetparam(M_("CMOS PLA option:"), &placmosp, MAXPARS-1, &par[1]) + 1;
 			if (count == 1)
 			{
-				ttyputerr(_("Aborted"));
+				ttyputerr(M_("Aborted"));
 				return(1);
 			}
 		}
