@@ -48,6 +48,21 @@ NODEINST *plac_make_Pin(NODEPROTO *facet, INTBIG X, INTBIG Y, INTBIG size, char
 
 	node = getnodeproto(pin_type);
 	if (node == NONODEPROTO)
+	{
+		if (namesame(pin_type, "Metal-1-Well-Con") == 0)
+			node = getnodeproto("Metal-1-P-Well-Con");
+		if (namesame(pin_type, "Metal-1-Substrate-Con") == 0)
+			node = getnodeproto("Metal-1-N-Well-Con");
+
+		if (namesame(pin_type, "Metal-1-D-Active-Con") == 0)
+			node = getnodeproto("Metal-1-N-Active-Con");
+		if (namesame(pin_type, "Metal-1-S-Active-Con") == 0)
+			node = getnodeproto("Metal-1-P-Active-Con");
+
+		if (namesame(pin_type, "Metal-1-Polysilicon-Con") == 0)
+			node = getnodeproto("Metal-1-Polysilicon-1-Con");
+	}
+	if (node == NONODEPROTO)
 	{
 		ttyputerr(_("No %s contact for technology %s"), pin_type, el_curtech->techname);
 		node = gen_univpinprim;
@@ -86,6 +101,15 @@ void plac_wire(char *type_arc, INTBIG width, NODEINST *fromnodeinst,
 	portposition(tonodeinst, toportproto, &X2, &Y2);
 	typ = getarcproto(type_arc);
 	if (typ == NOARCPROTO)
+	{
+		if (namesame(type_arc, "Polysilicon") == 0)
+			typ = getarcproto("Polysilicon-1");
+		if (namesame(type_arc, "D-active") == 0)
+			typ = getarcproto("N-Active");
+		if (namesame(type_arc, "S-active") == 0)
+			typ = getarcproto("P-Active");
+	}
+	if (typ == NOARCPROTO)
 	{
 		ttyputerr(_("Attempting to wire with unknown arc layer"));
 		return;
