@@ -164,29 +164,29 @@ void pla_Flag(INTSML option)
 			case VERBOSE:
 				pla_verbose = onoroff;
 				if (pla_verbose == ON)
-					ttyputmsg(_("'verbose' set to %s"), pla_onoffmessage(pla_verbose == ON));
+					ttyputmsg(M_("'verbose' set to %s"), pla_onoffmessage(pla_verbose == ON));
 				break;
 			case FLEXIBLE:
 				if (onoroff == ON) pla_userbits &= ~FIXED; else
 					pla_userbits |= FIXED;
 				if (pla_verbose == ON)
-					ttyputmsg(_("'flexible' set to %s"), pla_onoffmessage(pla_userbits&FIXED));
+					ttyputmsg(M_("'flexible' set to %s"), pla_onoffmessage(pla_userbits&FIXED));
 				break;
 			case FIXEDANGLE:
 				if (onoroff == ON) pla_userbits |= FIXANG; else
 					pla_userbits &= ~FIXANG;
 				if (pla_verbose == ON)
-					ttyputmsg(_("'fixedangle' set to %s"), pla_onoffmessage(pla_userbits&FIXANG));
+					ttyputmsg(M_("'fixedangle' set to %s"), pla_onoffmessage(pla_userbits&FIXANG));
 				break;
 			case BUTTINGCONTACT:
 				pla_buttingcontact = onoroff;
 				if (pla_verbose == ON)
-					ttyputmsg(_("'buttingcontact' set to %s"), pla_onoffmessage(pla_buttingcontact == ON));
+					ttyputmsg(M_("'buttingcontact' set to %s"), pla_onoffmessage(pla_buttingcontact == ON));
 				break;
 			case SAMESIDEOUTPUT:
 				pla_samesideoutput = ON;
 				if (pla_verbose == ON)
-					ttyputmsg(_("'samesideoutput' set to %s"), pla_onoffmessage(pla_samesideoutput == ON));
+					ttyputmsg(M_("'samesideoutput' set to %s"), pla_onoffmessage(pla_samesideoutput == ON));
 				break;
 		}
 	}
@@ -331,29 +331,29 @@ void pla_SetFlag(void)
 				break;
 			case INPUTS:
 				if (pla_ReadOffEqualSign() != ERRORRET) pla_inputs = pla_GetVal();
-				if (pla_verbose == ON) ttyputmsg(_("'inputs' set to %d"), pla_inputs);
+				if (pla_verbose == ON) ttyputmsg(M_("'inputs' set to %d"), pla_inputs);
 				break;
 			case OUTPUTS:
 				if (pla_ReadOffEqualSign() != ERRORRET) pla_outputs = pla_GetVal();
-				if (pla_verbose == ON) ttyputmsg(_("'outputs' set to %d"), pla_outputs);
+				if (pla_verbose == ON) ttyputmsg(M_("'outputs' set to %d"), pla_outputs);
 				break;
 			case PTERMS:
 				if (pla_ReadOffEqualSign() != ERRORRET) pla_pterms = pla_GetVal();
-				if (pla_verbose == ON) ttyputmsg(_("'pterms' set to %d"), pla_pterms);
+				if (pla_verbose == ON) ttyputmsg(M_("'pterms' set to %d"), pla_pterms);
 				break;
 			case VDDWIDTH:
 				if (pla_ReadOffEqualSign() != ERRORRET)
 					pla_VddWidth = pla_lam * pla_GetVal();
 				if (pla_VddWidth < 4 * pla_lam) pla_VddWidth = 4 * pla_lam;
 				pla_halfVdd = pla_VddWidth / 2;
-				if (pla_verbose == ON) ttyputmsg(_("'Vdd Width' set to %s"), latoa(pla_VddWidth));
+				if (pla_verbose == ON) ttyputmsg(M_("'Vdd Width' set to %s"), latoa(pla_VddWidth));
 				break;
 			case GNDWIDTH:
 				if (pla_ReadOffEqualSign() != ERRORRET)
 					pla_GndWidth = pla_lam * pla_GetVal();
 				if (pla_GndWidth < 4 * pla_lam) pla_GndWidth = 4 * pla_lam;
 				pla_halfGnd = pla_GndWidth / 2;
-				if (pla_verbose == ON) ttyputmsg(_("'Gnd Width' set to %s"), latoa(pla_GndWidth));
+				if (pla_verbose == ON) ttyputmsg(M_("'Gnd Width' set to %s"), latoa(pla_GndWidth));
 				break;
 			case VERBOSE:
 			case FLEXIBLE:
@@ -387,7 +387,7 @@ void pla_SetName(void)
 
 	if (pla_GetKeyWord(keyword) != ERRORRET) (void)strcpy(pla_name, keyword);
 	if (pla_verbose == ON)
-		ttyputmsg(_("Name of the PLA set to '%s'"), pla_name);
+		ttyputmsg(M_("Name of the PLA set to '%s'"), pla_name);
 }
 
 INTSML pla_SkipChars(void)
@@ -425,19 +425,19 @@ INTSML pla_SkipChars(void)
 
 void pla_displayoptions(void)
 {
-	ttyputmsg(_("nMOS PLA options and current values:"));
-	ttyputmsg(_("       Input file = %s"), pla_infile);
-	ttyputmsg(_(" Number of Inputs = %d"), pla_inputs);
-	ttyputmsg(_("Number of Outputs = %d"), pla_outputs);
-	ttyputmsg(_(" Number of Pterms = %d"), pla_pterms);
-	ttyputmsg(_("	       Verbose = %s"), pla_onoffmessage(pla_verbose == ON));
-	ttyputmsg(_("	      Flexible = %s"), pla_onoffmessage(pla_userbits&FIXED));
-	ttyputmsg(_("       Fixedangle = %s"), pla_onoffmessage(pla_userbits&FIXANG));
-	ttyputmsg(_("   Buttingcontact = %s"), pla_onoffmessage(pla_buttingcontact == ON));
-	ttyputmsg(_("   Samesideoutput = %s"), pla_onoffmessage(pla_samesideoutput == ON));
-	ttyputmsg(_("	     VDD Width = %s"), latoa(pla_VddWidth));
-	ttyputmsg(_("	     Gnd Width = %s"), latoa(pla_GndWidth));
-	ttyputmsg(_("  Name of the PLA = %s"), pla_name);
+	ttyputmsg(M_("nMOS PLA options and current values:"));
+	ttyputmsg(M_("       Input file = %s"), pla_infile);
+	ttyputmsg(M_(" Number of Inputs = %d"), pla_inputs);
+	ttyputmsg(M_("Number of Outputs = %d"), pla_outputs);
+	ttyputmsg(M_(" Number of Pterms = %d"), pla_pterms);
+	ttyputmsg(M_("	       Verbose = %s"), pla_onoffmessage(pla_verbose == ON));
+	ttyputmsg(M_("	      Flexible = %s"), pla_onoffmessage(pla_userbits&FIXED));
+	ttyputmsg(M_("       Fixedangle = %s"), pla_onoffmessage(pla_userbits&FIXANG));
+	ttyputmsg(M_("   Buttingcontact = %s"), pla_onoffmessage(pla_buttingcontact == ON));
+	ttyputmsg(M_("   Samesideoutput = %s"), pla_onoffmessage(pla_samesideoutput == ON));
+	ttyputmsg(M_("	     VDD Width = %s"), latoa(pla_VddWidth));
+	ttyputmsg(M_("	     Gnd Width = %s"), latoa(pla_GndWidth));
+	ttyputmsg(M_("  Name of the PLA = %s"), pla_name);
 }
 
 char *pla_onoffmessage(INTBIG onoff)
@@ -481,25 +481,25 @@ void pla_setoption(char *option, char *value)
 		{
 			case VERBOSE:
 				pla_verbose = val;
-				ttyputverbose(_("'verbose' set to %s"), pla_onoffmessage(pla_verbose == ON));
+				ttyputverbose(M_("'verbose' set to %s"), pla_onoffmessage(pla_verbose == ON));
 				break;
 			case FLEXIBLE:
 				if (val == ON) pla_userbits &= ~FIXED; else
 					pla_userbits |= FIXED;
-				ttyputverbose(_("'flexible' set to %s"), pla_onoffmessage(pla_userbits&FIXED));
+				ttyputverbose(M_("'flexible' set to %s"), pla_onoffmessage(pla_userbits&FIXED));
 				break;
 			case FIXEDANGLE:
 				if (val == ON) pla_userbits |= FIXANG; else
 					pla_userbits &= ~FIXANG;
-				ttyputverbose(_("'fixedangle' set to %s"), pla_onoffmessage(pla_userbits&FIXANG));
+				ttyputverbose(M_("'fixedangle' set to %s"), pla_onoffmessage(pla_userbits&FIXANG));
 				break;
 			case BUTTINGCONTACT:
 				pla_buttingcontact = val;
-				ttyputverbose(_("'buttingcontact' set to %s"), pla_onoffmessage(pla_buttingcontact == ON));
+				ttyputverbose(M_("'buttingcontact' set to %s"), pla_onoffmessage(pla_buttingcontact == ON));
 				break;
 			case SAMESIDEOUTPUT:
 				pla_samesideoutput = val;
-				ttyputverbose(_("'samesideoutput' set to %s"), pla_onoffmessage(pla_samesideoutput == ON));
+				ttyputverbose(M_("'samesideoutput' set to %s"), pla_onoffmessage(pla_samesideoutput == ON));
 				break;
 		}
 		return;
@@ -512,22 +512,22 @@ void pla_setoption(char *option, char *value)
 		{
 			case INPUTS:
 				pla_inputs = val;
-				ttyputverbose(_("Inputs set to %d"), pla_inputs);
+				ttyputverbose(M_("Inputs set to %d"), pla_inputs);
 				break;
 			case OUTPUTS:
 				pla_outputs = val;
-				ttyputverbose(_("oOutputs set to %d"), pla_inputs);
+				ttyputverbose(M_("Outputs set to %d"), pla_inputs);
 				break;
 			case PTERMS:
 				pla_pterms = val;
-				ttyputverbose(_("Product terms set to %d"), pla_inputs);
+				ttyputverbose(M_("Product terms set to %d"), pla_inputs);
 				break;
 			case VDDWIDTH:
 				if (val < 4) ttyputerr(_("Vdd Width must be at least 4 lambda")); else
 				{
 					pla_VddWidth = val*pla_lam;
 					pla_halfVdd  = pla_VddWidth / 2;
-					ttyputverbose(_("VDD width set to %s"), latoa(pla_VddWidth));
+					ttyputverbose(M_("VDD width set to %s"), latoa(pla_VddWidth));
 				}
 				break;
 			case GNDWIDTH:
@@ -535,7 +535,7 @@ void pla_setoption(char *option, char *value)
 				{
 					pla_GndWidth = val*pla_lam;
 					pla_halfGnd  = pla_GndWidth / 2;
-					ttyputverbose(_("Gnd width set to %s"), latoa(pla_GndWidth));
+					ttyputverbose(M_("Gnd width set to %s"), latoa(pla_GndWidth));
 				}
 				break;
 		}
@@ -544,12 +544,12 @@ void pla_setoption(char *option, char *value)
 	if (k == FILENAME)
 	{
 		(void)strncpy(pla_infile, value, 100);
-		ttyputverbose(_("File set to %s"), pla_infile);
+		ttyputverbose(M_("File set to %s"), pla_infile);
 	}
 	if (k == NAME)
 	{
 		(void)strncpy(pla_name, value, 32);
-		ttyputverbose(_("Name set to %s"), pla_name);
+		ttyputverbose(M_("Name set to %s"), pla_name);
 	}
 }
 
