@@ -167,13 +167,13 @@ INTSML pla_Make(void)
 
 	if (pla_verbose == ON)
 	{
-		ttyputmsg(_("Facets have been defined with sizes given below:"));
-		ttyputmsg(_("Pullup  = %sx%s"), latoa(puX), latoa(puY));
-		ttyputmsg(_("Input   = %sx%s"), latoa(inX), latoa(inY));
-		ttyputmsg(_("Output  = %sx%s"), latoa(outX), latoa(outY));
-		ttyputmsg(_("Program = %sx%s"), latoa(progX), latoa(progY));
-		ttyputmsg(_("Connect = %sx%s"), latoa(connectX), latoa(connectY));
-		ttyputmsg(_("VDD Width = %s, Gnd Width = %s"), latoa(pla_VddWidth), latoa(pla_GndWidth));
+		ttyputmsg(M_("Facets have been defined with sizes given below:"));
+		ttyputmsg(M_("Pullup  = %sx%s"), latoa(puX), latoa(puY));
+		ttyputmsg(M_("Input   = %sx%s"), latoa(inX), latoa(inY));
+		ttyputmsg(M_("Output  = %sx%s"), latoa(outX), latoa(outY));
+		ttyputmsg(M_("Program = %sx%s"), latoa(progX), latoa(progY));
+		ttyputmsg(M_("Connect = %sx%s"), latoa(connectX), latoa(connectY));
+		ttyputmsg(M_("VDD Width = %s, Gnd Width = %s"), latoa(pla_VddWidth), latoa(pla_GndWidth));
 	}
 
 	/* Create the pla prototype with these dimensions.
@@ -214,7 +214,7 @@ INTSML pla_Make(void)
 
 		ttyputmsg(_("Defining '%s'"), pla_name);
 		if (pla_verbose == ON)
-			ttyputmsg(_("Beginning to program the AND and OR planes."));
+			ttyputmsg(M_("Beginning to program the AND and OR planes."));
 		if (pla_ProgramPLA(inX, inY, puX, puY, outX, outY, connectX,
 			connectY, pla_facet) == ERRORRET)
 				ttyputerr(_("PLA '%s' is incompletely defined"), pla_name);
@@ -276,11 +276,11 @@ FACETIST *pla_PlaceInputs(INTBIG puX, INTBIG inX, INTBIG inY, NODEPROTO *pla)
 	firstptr= NOFACETIST;
 	cicv = picv = cicg = picg = NOVDDGND;
 
-	if (pla_verbose == ON) ttyputmsg(_("Placing the input facets"));
+	if (pla_verbose == ON) ttyputmsg(M_("Placing the input facets"));
 
 	for(i = 1; i <= pla_inputs; i++)
 	{
-		if (pla_verbose == ON) ttyputmsg(_("Placing input facet %d"), i);
+		if (pla_verbose == ON) ttyputmsg(M_("Placing input facet %d"), i);
 		ptr = (FACETIST *)emalloc(sizeof(FACETIST), pla_aid->cluster);
 		if (ptr == NOFACETIST)
 		{
@@ -352,11 +352,11 @@ FACETIST *pla_PlaceOutputs(INTBIG orLeft, INTBIG outX, INTBIG outY, NODEPROTO *p
 	prevptr = firstptr = NOFACETIST;
 	cocv = pocv = cocg = pocg = NOVDDGND;
 
-	if (pla_verbose == ON) ttyputmsg(_("Placing the output facets"));
+	if (pla_verbose == ON) ttyputmsg(M_("Placing the output facets"));
 
 	for(o = 1; o <= (pla_outputs+pla_outputs%2)/2; o++)
 	{
-		if (pla_verbose == ON) ttyputmsg(_("Placing output facet %d"), o);
+		if (pla_verbose == ON) ttyputmsg(M_("Placing output facet %d"), o);
 
 		ptr = (FACETIST *)emalloc(sizeof(FACETIST), pla_aid->cluster);
 		if (ptr == NOFACETIST)
@@ -435,12 +435,12 @@ FACETIST *pla_PlacePullupsonAND(INTBIG puX, INTBIG puY, INTBIG inY, NODEPROTO *p
 	calv = palv = NOVDDGND;
 
 	if (pla_verbose == ON)
-		ttyputmsg(_("Placing the pullup facets on the left edge of the AND-plane"));
+		ttyputmsg(M_("Placing the pullup facets on the left edge of the AND-plane"));
 
 	for(p = 1; p <= pla_pterms; p++)
 	{
 		if (pla_verbose == ON)
-			ttyputmsg(_("Placing pullup facet %d in AND-plane"), p);
+			ttyputmsg(M_("Placing pullup facet %d in AND-plane"), p);
 
 		ptr = (FACETIST *)emalloc(sizeof(FACETIST), pla_aid->cluster);
 		if (ptr == NOFACETIST)
@@ -502,13 +502,13 @@ FACETIST *pla_PlacePullupsonOR(INTBIG andY, INTBIG orLeft, INTBIG inY, INTBIG pu
 	cotv = potv = NOVDDGND;
 
 	if (pla_verbose == ON)
-		ttyputmsg(_("Placing the pullups on the %s edge of the OR-plane"),
-			(pla_samesideoutput == ON) ? _("top") : _("bottom"));
+		ttyputmsg(M_("Placing the pullups on the %s edge of the OR-plane"),
+			(pla_samesideoutput == ON) ? M_("top") : M_("bottom"));
 
 	for(i = 1; i <= pla_outputs; i++)
 	{
 		if (pla_verbose == ON)
-			ttyputmsg(_("Placing pullup facet %d in OR-plane"), i);
+			ttyputmsg(M_("Placing pullup facet %d in OR-plane"), i);
 
 		ptr = (FACETIST *)emalloc(sizeof(FACETIST), pla_aid->cluster);
 		if (ptr == NOFACETIST)
@@ -737,10 +737,10 @@ INTSML pla_ProgramPLA(INTBIG inX, INTBIG inY, INTBIG puX, INTBIG puY, INTBIG out
 
 	if (pla_verbose == ON)
 	{
-		ttyputmsg(_("Dimensions of the planes:"));
-		ttyputmsg(_("  andX = %s,    andY = %s"), latoa(andX), latoa(andY));
-		ttyputmsg(_("   orX = %s,     orY = %s"), latoa(orX), latoa(andY));
-		ttyputmsg(_("orLeft = %s, orRight = %s"), latoa(orLeft), latoa(orRight));
+		ttyputmsg(M_("Dimensions of the planes:"));
+		ttyputmsg(M_("  andX = %s,    andY = %s"), latoa(andX), latoa(andY));
+		ttyputmsg(M_("   orX = %s,     orY = %s"), latoa(orX), latoa(andY));
+		ttyputmsg(M_("orLeft = %s, orRight = %s"), latoa(orLeft), latoa(orRight));
 	}
 
 	/* Place all the input, output and pullup facets */
@@ -764,7 +764,7 @@ INTSML pla_ProgramPLA(INTBIG inX, INTBIG inY, INTBIG puX, INTBIG puY, INTBIG out
 			return(ERRORRET);
 		}
 
-		if (pla_verbose == ON) ttyputmsg(_("Programming row %ld."), p);
+		if (pla_verbose == ON) ttyputmsg(M_("Programming row %ld."), p);
 
 		CurRow   = ptermptr->findex;
 		if (CurRow != p)
@@ -819,12 +819,12 @@ INTSML pla_ProgramPLA(INTBIG inX, INTBIG inY, INTBIG puX, INTBIG puY, INTBIG out
 			if (CurTrans == NOPROGTRANS)
 			{
 				if (pla_verbose == ON)
-					ttyputmsg(_("row=%ld, ANDcolumn=%ld not programmed."), CurRow, CurCol);
+					ttyputmsg(M_("row=%ld, ANDcolumn=%ld not programmed."), CurRow, CurCol);
 				inptr = inptr->nextfacet;
 				continue;
 			}
 
-			if (pla_verbose == ON) ttyputmsg(_("row=%ld, ANDcolumn=%ld programmed."), CurRow, CurCol);
+			if (pla_verbose == ON) ttyputmsg(M_("row=%ld, ANDcolumn=%ld programmed."), CurRow, CurCol);
 
 			CurTrans->row = (INTSML)CurRow;
 			CurTrans->column = (INTSML)CurCol;
@@ -964,12 +964,12 @@ INTSML pla_ProgramPLA(INTBIG inX, INTBIG inY, INTBIG puX, INTBIG puY, INTBIG out
 					{
 						/* This point is not being programmed */
 						if (pla_verbose == ON)
-							ttyputmsg(_("row=%ld, ORcolumn=%ld not programmed."), CurRow, CurCol);
+							ttyputmsg(M_("row=%ld, ORcolumn=%ld not programmed."), CurRow, CurCol);
 						continue;
 					}
 
 					if (pla_verbose == ON)
-						ttyputmsg(_("row=%ld, ORcolumn=%ld programmed."), CurRow, CurCol);
+						ttyputmsg(M_("row=%ld, ORcolumn=%ld programmed."), CurRow, CurCol);
 
 					CurTrans->row = (INTSML)CurRow;
 					CurTrans->column = (INTSML)CurCol;
