@@ -88,7 +88,7 @@ void pla_CHfromPuAND(INTBIG orRight, FACETIST *PullupAND, NODEPROTO *pla)
 	for(currow = PullupAND; currow != NOFACETIST; currow = currow->nextfacet)
 	{
 		if (pla_verbose == ON)
-			ttyputmsg(_("connecting right from pullup# %d"), currow->findex);
+			ttyputmsg(M_("connecting right from pullup# %d"), currow->findex);
 
 		if (pla_ConnectFirstFacet(PULLUPSIDE, currow, pla) == ERRORRET)
 			ttyputerr(_("CHfromPuAND: cannot connect to first trans"));
@@ -202,7 +202,7 @@ void pla_CVfromInput(INTBIG inY, INTBIG andY, FACETIST *inptr, NODEPROTO *pla)
 			ttyputmsg(_("*** Warning. Input %d is never used in unnegated form."), inptr->findex);
 
 		if (pla_verbose == ON)
-			ttyputmsg(_("connecting up from input# %d"), inptr->findex);
+			ttyputmsg(M_("connecting up from input# %d"), inptr->findex);
 
 		/* Place a contact to ground at the top of the PLA right above the
 		 * current input facet.
@@ -268,7 +268,7 @@ void pla_CVfromOutput(FACETIST *outptr, FACETIST *puptr, NODEPROTO *pla)
 			}
 		}
 		if (pla_verbose == ON)
-			ttyputmsg(_("connecting up from output facet# %d"), outptr->findex);
+			ttyputmsg(M_("connecting up from output facet# %d"), outptr->findex);
 
 		pla_DrawArcs(OUTPUTSIDE, endtrans, outptr, METAL, pla);
 
@@ -649,7 +649,7 @@ void pla_RunVddGnd(INTBIG puX, INTBIG andX, NODEPROTO *pla)
 	VDDGND turnpin, turnpin1, forkpin;
 	INTBIG lowx, highx, lowy, highy, portx, porty;
 
-	if (pla_verbose == ON) ttyputmsg(_("Running the Vdd and Gnd lines"));
+	if (pla_verbose == ON) ttyputmsg(M_("Running the Vdd and Gnd lines"));
 
 	pla_ConnectThruFacets(VDD, pla_otv, _("RunVddGnd: cannot connect on Vdd at Or Top"), pla);
 
