@@ -156,23 +156,6 @@ static PORTPROTO **ro_findnetlistpp;
 static INTBIG     *ro_findnetlistx;
 static INTBIG     *ro_findnetlisty;
 
-/* Routing Options */
-DIALOGITEM ro_optionsdialogitems[] =
-{
- /*  1 */ {0, {216,208,240,272}, BUTTON, N_("OK")},
- /*  2 */ {0, {216,12,240,76}, BUTTON, N_("Cancel")},
- /*  3 */ {0, {32,88,48,272}, POPUP, ""},
- /*  4 */ {0, {32,8,48,88}, MESSAGE, N_("Currently:")},
- /*  5 */ {0, {8,8,24,220}, MESSAGE, N_("Arc to use in stitching routers:")},
- /*  6 */ {0, {68,8,84,268}, CHECK, N_("Mimic stitching can unstitch")},
- /*  7 */ {0, {116,24,132,268}, CHECK, N_("Ports must match")},
- /*  8 */ {0, {188,8,204,268}, CHECK, N_("Mimic stitching runs interactively")},
- /*  9 */ {0, {92,8,108,268}, MESSAGE, N_("Mimic stitching restrictions:")},
- /* 10 */ {0, {164,24,180,268}, CHECK, N_("Node types must match")},
- /* 11 */ {0, {140,24,156,268}, CHECK, N_("Nodes sizes must match")}
-};
-DIALOG ro_optionsdialog = {{50,75,299,356}, N_("Routing Options"), 0, 11, ro_optionsdialogitems};
-
 /* prototypes for local routines */
 RCHECK *ro_allocrcheck(void);
 void ro_freercheck(RCHECK*);
@@ -245,7 +228,7 @@ void ro_init(INTBIG *argc, char *argv[], AIDENTRY *thisaid)
 		ro_statekey = makekey("ROUT_state");
 		ro_optionskey = makekey("ROUT_options");
 		ro_preferedkey = makekey("ROUT_prefered_arc");
-		nextvarchangequiet();
+		nextchangequiet();
 		ro_state = NOSTITCH;
 		(void)setvalkey((INTBIG)ro_aid, VAID, ro_statekey, ro_state, VINTEGER|VDONTSAVE);
 
@@ -308,15 +291,15 @@ INTSML ro_set(INTSML count, char *par[])
 		switch (ro_state&STITCHMODE)
 		{
 			case NOSTITCH:
-				ttyputmsg(_("No stitching being done"));          break;
+				ttyputmsg(M_("No stitching being done"));          break;
 			case AUTOSTITCH:
-				ttyputmsg(_("Default is auto-stitch routing"));   break;
+				ttyputmsg(M_("Default is auto-stitch routing"));   break;
 			case MIMICSTITCH:
-				ttyputmsg(_("Default is mimic-stitch routing"));  break;
+				ttyputmsg(M_("Default is mimic-stitch routing"));  break;
 		}
 		var = getvalkey((INTBIG)ro_aid, VAID, VARCPROTO, ro_preferedkey);
 		if (var != NOVARIABLE)
-			ttyputmsg(_("Default arc for river routing is %s"),
+			ttyputmsg(M_("Default arc for river routing is %s"),
 				describearcproto(((ARCPROTO *)var->addr)));
 		return(0);
 	}
@@ -352,7 +335,7 @@ INTSML ro_set(INTSML count, char *par[])
 				}
 				if (np->primindex != 0)
 				{
-					ttyputerr(_("Can only stitch facets, not primitives"));
+					ttyputerr(M_("Can only stitch facets, not primitives"));
 					return(1);
 				}
 				if (np->cell->lib != el_curlib)
@@ -382,7 +365,7 @@ INTSML ro_set(INTSML count, char *par[])
 
 			/* restore routing state */
 			ro_state = savestate;
-			ttyputverbose(_("Facet %s stitched"), describenodeproto(np));
+			ttyputverbose(M_("Facet %s stitched"), describenodeproto(np));
 			return(0);
 		}
 		if (namesamen(pp, "highlighted-stitch-now", l) == 0)
@@ -405,7 +388,7 @@ INTSML ro_set(INTSML count, char *par[])
 
 			/* restore routing state */
 			ro_state = savestate;
-			ttyputverbose(_("Stitching complete"));
+			ttyputverbose(M_("Stitching complete"));
 			return(0);
 		}
 		ttyputbadusage("tellaid routing auto-stitch");
@@ -435,28 +418,28 @@ INTSML ro_set(INTSML count, char *par[])
 		{
 			(void)setvalkey((INTBIG)ro_aid, VAID, ro_optionskey,
 				ro_getoptions() | MIMICUNROUTES, VINTEGER);
-			ttyputverbose(_("Mimic stitcher will route and unroute"));
+			ttyputverbose(M_("Mimic stitcher will route and unroute"));
 			return(0);
 		}
 		if (namesamen(pp, "stitch-only", l) == 0 && l >= 8)
 		{
 			(void)setvalkey((INTBIG)ro_aid, VAID, ro_optionskey,
 				ro_getoptions() & ~MIMICUNROUTES, VINTEGER);
-			ttyputverbose(_("Mimic stitcher will only route, not unroute"));
+			ttyputverbose(M_("Mimic stitcher will only route, not unroute"));
 			return(0);
 		}
 		if (namesamen(pp, "port-general", l) == 0 && l >= 8)
 		{
 			(void)setvalkey((INTBIG)ro_aid, VAID, ro_optionskey,
 				ro_getoptions() | MIMICIGNOREPORTS, VINTEGER);
-			ttyputverbose(_("Mimic stitcher will route to any port"));
+			ttyputverbose(M_("Mimic stitcher will route to any port"));
 			return(0);
 		}
 		if (namesamen(pp, "port-specific", l) == 0 && l >= 8)
 		{
 			(void)setvalkey((INTBIG)ro_aid, VAID, ro_optionskey,
 				ro_getoptions() & ~MIMICIGNOREPORTS, VINTEGER);
-			ttyputverbose(_("Mimic stitcher will rout only to similar ports"));
+			ttyputverbose(M_("Mimic stitcher will rout only to similar ports"));
 			return(0);
 		}
 		ttyputbadusage("tellaid routing mimic-stitching");
@@ -475,7 +458,7 @@ INTSML ro_set(INTSML count, char *par[])
 			}
 			if (np->primindex != 0)
 			{
-				ttyputerr(_("Can only route facets, not primitives"));
+				ttyputerr(M_("Can only route facets, not primitives"));
 				return(1);
 			}
 		} else
@@ -489,7 +472,7 @@ INTSML ro_set(INTSML count, char *par[])
 		}
 
 		if (ro_river(np) == 0) ttyputmsg(_("Routing not successful")); else
-			ttyputverbose(_("Facet %s river-routed"), describenodeproto(np));
+			ttyputverbose(M_("Facet %s river-routed"), describenodeproto(np));
 		return(0);
 	}
 
@@ -517,9 +500,9 @@ INTSML ro_set(INTSML count, char *par[])
 		{
 			if (count <= 2)
 			{
-				ttyputmsg(_("Routing grid: %sx%s"), latoa(ro_mazegridx), latoa(ro_mazegridy));
-				ttyputmsg(_("Routing offset: (%s,%s)"), latoa(ro_mazeoffsetx), latoa(ro_mazeoffsety));
-				ttyputmsg(_("Routing boundary: %s"), latoa(ro_mazeboundary));
+				ttyputmsg(M_("Routing grid: %sx%s"), latoa(ro_mazegridx), latoa(ro_mazegridy));
+				ttyputmsg(M_("Routing offset: (%s,%s)"), latoa(ro_mazeoffsetx), latoa(ro_mazeoffsety));
+				ttyputmsg(M_("Routing boundary: %s"), latoa(ro_mazeboundary));
 				return(0);
 			}
 
@@ -528,20 +511,20 @@ INTSML ro_set(INTSML count, char *par[])
 			{
 				ro_mazegridx = atola(par[3]);
 				ro_mazegridy = atola(par[4]);
-				ttyputmsg(_("Routing grid: %sx%s"), latoa(ro_mazegridx), latoa(ro_mazegridy));
+				ttyputmsg(M_("Routing grid: %sx%s"), latoa(ro_mazegridx), latoa(ro_mazegridy));
 				return 0;
 			}
 			if (namesamen(pp, "offset", l) == 0)
 			{
 				ro_mazeoffsetx = atola(par[3]);
 				ro_mazeoffsety = atola(par[4]);
-				ttyputmsg(_("Routing offset: (%s,%s)"), latoa(ro_mazeoffsetx), latoa(ro_mazeoffsety));
+				ttyputmsg(M_("Routing offset: (%s,%s)"), latoa(ro_mazeoffsetx), latoa(ro_mazeoffsety));
 				return 0;
 			}
 			if (namesamen(pp, "boundary", l) == 0)
 			{
 				ro_mazeboundary = atola(par[3]);
-				ttyputmsg(_("Routing boundary: %s"), latoa(ro_mazeboundary));
+				ttyputmsg(M_("Routing boundary: %s"), latoa(ro_mazeboundary));
 				return 0;
 			}
 
@@ -569,16 +552,16 @@ INTSML ro_set(INTSML count, char *par[])
 		{
 			if (getvalkey((INTBIG)ro_aid, VAID, VARCPROTO, ro_preferedkey) != NOVARIABLE)
 				(void)delvalkey((INTBIG)ro_aid, VAID, ro_preferedkey);
-			ttyputverbose(_("No arc will be presumed for stitching"));
+			ttyputverbose(M_("No arc will be presumed for stitching"));
 			return(0);
 		}
 		ap = getarcproto(par[1]);
 		if (ap == NOARCPROTO)
 		{
-			ttyputerr(_("No arc prototype called %s"), par[1]);
+			ttyputerr(M_("No arc prototype called %s"), par[1]);
 			return(1);
 		}
-		ttyputverbose(_("Default stitching arc will be %s"), describearcproto(ap));
+		ttyputverbose(M_("Default stitching arc will be %s"), describearcproto(ap));
 		(void)setvalkey((INTBIG)ro_aid, VAID, ro_preferedkey, (INTBIG)ap, VARCPROTO);
 		return(0);
 	}
@@ -601,7 +584,7 @@ INTSML ro_set(INTSML count, char *par[])
 		{
 			(void)setvalkey((INTBIG)ro_aid, VAID, ro_statekey,
 				ro_state & ~SELECT, VINTEGER|VDONTSAVE);
-			ttyputverbose(_("Wire placement not subject to approval"));
+			ttyputverbose(M_("Wire placement not subject to approval"));
 			return(0);
 		}
 		ttyputbadusage("tellaid routing not");
@@ -612,7 +595,7 @@ INTSML ro_set(INTSML count, char *par[])
 	{
 		(void)setvalkey((INTBIG)ro_aid, VAID, ro_statekey,
 			ro_state | SELECT, VINTEGER|VDONTSAVE);
-		ttyputverbose(_("Wire placement subject to approval"));
+		ttyputverbose(M_("Wire placement subject to approval"));
 		return(0);
 	}
 
@@ -745,7 +728,7 @@ void ro_unroutecurrent(void)
 	REGISTER INTSML found;
 
 	/* see what is highlighted */
-	list = us_gethighlighted(OBJARCINST);
+	list = us_gethighlighted(OBJARCINST, 0, 0);
 	for(selectcount=0; list[selectcount] != NOGEOM; selectcount++) ;
 	if (selectcount == 0)
 	{
@@ -988,15 +971,31 @@ INTBIG ro_getoptions(void)
 
 /****************************** ROUTING OPTIONS DIALOG ******************************/
 
-/*
- * special case for the "Routing Options" dialog
- * Arc list                 = 3 (user item)
- * Mimic can unstitch       = 6 (check)
- * Mimic port-specific      = 7 (check)
- * Mimic node-specific      = 10 (check)
- * Mimic node-size-specific = 11 (check)
- * Mimic is interactive     = 8 (check)
- */
+/* Routing Options */
+DIALOGITEM ro_optionsdialogitems[] =
+{
+ /*  1 */ {0, {216,208,240,272}, BUTTON, N_("OK")},
+ /*  2 */ {0, {216,12,240,76}, BUTTON, N_("Cancel")},
+ /*  3 */ {0, {32,88,48,272}, POPUP, ""},
+ /*  4 */ {0, {32,8,48,88}, MESSAGE, N_("Currently:")},
+ /*  5 */ {0, {8,8,24,220}, MESSAGE, N_("Arc to use in stitching routers:")},
+ /*  6 */ {0, {68,8,84,268}, CHECK, N_("Mimic stitching can unstitch")},
+ /*  7 */ {0, {116,24,132,268}, CHECK, N_("Ports must match")},
+ /*  8 */ {0, {188,8,204,268}, CHECK, N_("Mimic stitching runs interactively")},
+ /*  9 */ {0, {92,8,108,268}, MESSAGE, N_("Mimic stitching restrictions:")},
+ /* 10 */ {0, {164,24,180,268}, CHECK, N_("Node types must match")},
+ /* 11 */ {0, {140,24,156,268}, CHECK, N_("Nodes sizes must match")}
+};
+DIALOG ro_optionsdialog = {{50,75,299,356}, N_("Routing Options"), 0, 11, ro_optionsdialogitems};
+
+/* special items for the "Routing Options" dialog: */
+#define DROO_ARCLIST         3		/* Arc list (popup) */
+#define DROO_MUNSTITCH       6		/* Mimic can unstitch (check) */
+#define DROO_MPORTSPECIFIC   7		/* Mimic port-specific (check) */
+#define DROO_MINTERACTIVE    8		/* Mimic is interactive (check) */
+#define DROO_MNODEPSECIFIC  10		/* Mimic node-specific (check) */
+#define DROO_MNODESIZESPEC  11		/* Mimic node-size-specific (check) */
+
 void ro_optionsdlog(void)
 {
 	INTBIG itemHit, numarcnames, initialindex, i, options, oldoptions;
@@ -1028,47 +1027,48 @@ void ro_optionsdlog(void)
 
 	/* display the router arc dialog box */
 	if (DiaInitDialog(&ro_optionsdialog) != 0) return;
-	DiaSetPopup(3, numarcnames, arcnames);
-	DiaSetPopupEntry(3, initialindex);
+	DiaSetPopup(DROO_ARCLIST, numarcnames, arcnames);
+	DiaSetPopupEntry(DROO_ARCLIST, initialindex);
 	efree((char *)arcnames);
 	ap = arcs[initialindex];
 	oldoptions = options = ro_getoptions();
-	if ((options&MIMICUNROUTES) != 0) DiaSetControl(6, 1);
-	if ((options&MIMICIGNOREPORTS) == 0) DiaSetControl(7, 1);
-	if ((options&MIMICIGNORENODETYPE) == 0) DiaSetControl(10, 1);
-	if ((options&MIMICIGNORENODESIZE) == 0) DiaSetControl(11, 1);
-	if ((options&MIMICINTERACTIVE) != 0) DiaSetControl(8, 1);
+	if ((options&MIMICUNROUTES) != 0) DiaSetControl(DROO_MUNSTITCH, 1);
+	if ((options&MIMICIGNOREPORTS) == 0) DiaSetControl(DROO_MPORTSPECIFIC, 1);
+	if ((options&MIMICIGNORENODETYPE) == 0) DiaSetControl(DROO_MNODEPSECIFIC, 1);
+	if ((options&MIMICIGNORENODESIZE) == 0) DiaSetControl(DROO_MNODESIZESPEC, 1);
+	if ((options&MIMICINTERACTIVE) != 0) DiaSetControl(DROO_MINTERACTIVE, 1);
 
 	/* loop until done */
 	for(;;)
 	{
 		itemHit = DiaNextHit();
 		if (itemHit == OK || itemHit == CANCEL) break;
-		if (itemHit == 6 || itemHit == 7 || itemHit == 8 ||
-			itemHit == 10 || itemHit == 11)
+		if (itemHit == DROO_MUNSTITCH || itemHit == DROO_MPORTSPECIFIC ||
+			itemHit == DROO_MINTERACTIVE || itemHit == DROO_MNODEPSECIFIC ||
+			itemHit == DROO_MNODESIZESPEC)
 		{
 			DiaSetControl(itemHit, 1 - DiaGetControl(itemHit));
 			continue;
 		}
-		if (itemHit == 3)
+		if (itemHit == DROO_ARCLIST)
 		{
 			/* find this arc */
-			i = DiaGetPopupEntry(3);
+			i = DiaGetPopupEntry(DROO_ARCLIST);
 			ap = arcs[i];
 		}
 	}
 
 	if (itemHit != CANCEL)
 	{
-		if (DiaGetControl(6) != 0) options |= MIMICUNROUTES; else
+		if (DiaGetControl(DROO_MUNSTITCH) != 0) options |= MIMICUNROUTES; else
 			options &= ~MIMICUNROUTES;
-		if (DiaGetControl(7) == 0) options |= MIMICIGNOREPORTS; else
+		if (DiaGetControl(DROO_MPORTSPECIFIC) == 0) options |= MIMICIGNOREPORTS; else
 			options &= ~MIMICIGNOREPORTS;
-		if (DiaGetControl(10) == 0) options |= MIMICIGNORENODETYPE; else
+		if (DiaGetControl(DROO_MNODEPSECIFIC) == 0) options |= MIMICIGNORENODETYPE; else
 			options &= ~MIMICIGNORENODETYPE;
-		if (DiaGetControl(11) == 0) options |= MIMICIGNORENODESIZE; else
+		if (DiaGetControl(DROO_MNODESIZESPEC) == 0) options |= MIMICIGNORENODESIZE; else
 			options &= ~MIMICIGNORENODESIZE;
-		if (DiaGetControl(8) != 0) options |= MIMICINTERACTIVE; else
+		if (DiaGetControl(DROO_MINTERACTIVE) != 0) options |= MIMICINTERACTIVE; else
 			options &= ~MIMICINTERACTIVE;
 		if (options != oldoptions)
 			(void)setvalkey((INTBIG)ro_aid, VAID, ro_optionskey, options, VINTEGER);
