@@ -191,7 +191,7 @@ INTBIG ro_checkstitching(NODEINST *ni)
 	REGISTER NODEINST *oni, *rni;
 	REGISTER PORTPROTO *pp, *rpp, *bestpp;
 	REGISTER PORTARCINST *pi;
-	REGISTER ARCPROTO *ap, *couldbe;
+	REGISTER ARCPROTO *ap;
 	static POLYGON *poly = NOPOLYGON;
 
 	/* get polygon */
@@ -263,7 +263,7 @@ INTBIG ro_checkstitching(NODEINST *ni)
 				}
 
 				/* look at all polygons on this nodeinst */
-				tot = nodeEpolys(rni, 0);
+				tot = nodeEpolys(rni, 0, NOWINDOWPART);
 				for(j=0; j<tot; j++)
 				{
 					shapeEnodepoly(rni, j, poly);
@@ -280,27 +280,46 @@ INTBIG ro_checkstitching(NODEINST *ni)
 					/* if the polygon layer is pseudo, substitute real layer */
 					ro_substituteforpseudo(rni, poly);
 
-					/* check all possible arcs */
-					for(i=0; pp->connects[i] != NOARCPROTO; i++)
-						if (pp->connects[i] == ro_preferedarc) break;
-					couldbe = pp->connects[i];
+					/* first see if the prefered arc is possible */
+					ap = ro_preferedarc;
+					stitched = 0;
 					for(i=0; pp->connects[i] != NOARCPROTO; i++)
 					{
-						ap = pp->connects[i];
-						if (couldbe != NOARCPROTO && couldbe != ap) continue;
+						if (pp->connects[i] != ap) continue;
 
 						/* arc must be in the same technology */
-						if (ap->tech != rni->proto->tech) continue;
+						if (ap->tech != rni->proto->tech) break;
 
 						/* this polygon must be the smallest arc layer */
-						if (ap->temp1 != poly->layer) continue;
+						if (ap->temp1 != poly->layer) break;
 
 						/* pass it on to the next test */
 						stitched = ro_testpoly(ni, pp, ap, poly, oni);
 						count += stitched;
-						if (stitched != 0) break;
+						break;
 					}
-					if (pp->connects[i] != NOARCPROTO) break;
+
+					/* now look for any arc */
+					if (stitched == 0)
+					{
+						for(i=0; pp->connects[i] != NOARCPROTO; i++)
+						{
+							ap = pp->connects[i];
+							if (ap == ro_preferedarc) continue;
+
+							/* arc must be in the same technology */
+							if (ap->tech != rni->proto->tech) continue;
+
+							/* this polygon must be the smallest arc layer */
+							if (ap->temp1 != poly->layer) continue;
+
+							/* pass it on to the next test */
+							stitched = ro_testpoly(ni, pp, ap, poly, oni);
+							count += stitched;
+							if (stitched != 0) break;
+						}
+					}
+					if (stitched != 0) break;
 				}
 			}
 		} else
@@ -313,7 +332,7 @@ INTBIG ro_checkstitching(NODEINST *ni)
 			oy = (oni->lowy + oni->highy) / 2;
 
 			/* look at all polygons on this nodeinst */
-			tot = nodeEpolys(ni, 0);
+			tot = nodeEpolys(ni, 0, NOWINDOWPART);
 			for(j=0; j<tot; j++)
 			{
 				shapeEnodepoly(ni, j, poly);
@@ -352,28 +371,48 @@ INTBIG ro_checkstitching(NODEINST *ni)
 				/* transformed the polygon */
 				xformpoly(poly, trans);
 
-				/* check all possible arcs */
-				for(i=0; rpp->connects[i] != NOARCPROTO; i++)
-					if (rpp->connects[i] == ro_preferedarc) break;
-				couldbe = rpp->connects[i];
+				/* first see if the prefered arc is possible */
+				ap = ro_preferedarc;
+				stitched = 0;
 				for(i=0; rpp->connects[i] != NOARCPROTO; i++)
 				{
-					ap = rpp->connects[i];
-					if (couldbe != NOARCPROTO && couldbe != ap) continue;
+					if (rpp->connects[i] != ap) continue;
 
 					/* arc must be in the same technology */
-					if (ap->tech != ni->proto->tech) continue;
+					if (ap->tech != ni->proto->tech) break;
 
 					/* this polygon must be the smallest arc layer */
 					ro_findsmallestlayer(ap);
-					if (ap->temp1 != poly->layer) continue;
+					if (ap->temp1 != poly->layer) break;
 
 					/* pass it on to the next test */
 					stitched = ro_testpoly(ni, rpp, ap, poly, oni);
 					count += stitched;
-					if (stitched != 0) break;
+					break;
 				}
-				if (rpp->connects[i] != NOARCPROTO) break;
+
+				/* now look for any arc */
+				if (stitched == 0)
+				{
+					for(i=0; rpp->connects[i] != NOARCPROTO; i++)
+					{
+						ap = rpp->connects[i];
+						if (ap == ro_preferedarc) continue;
+
+						/* arc must be in the same technology */
+						if (ap->tech != ni->proto->tech) continue;
+
+						/* this polygon must be the smallest arc layer */
+						ro_findsmallestlayer(ap);
+						if (ap->temp1 != poly->layer) continue;
+
+						/* pass it on to the next test */
+						stitched = ro_testpoly(ni, rpp, ap, poly, oni);
+						count += stitched;
+						if (stitched != 0) break;
+					}
+				}
+				if (stitched != 0) break;
 			}
 		}
 	}
@@ -446,7 +485,7 @@ INTBIG ro_testpoly(NODEINST *ni, PORTPROTO *pp, ARCPROTO *ap, POLYGON *poly, NOD
 			}
 
 			/* look at all polygons on this nodeinst */
-			tot = nodeEpolys(rni, 0);
+			tot = nodeEpolys(rni, 0, NOWINDOWPART);
 			for(j=0; j<tot; j++)
 			{
 				shapeEnodepoly(rni, j, opoly);
@@ -481,7 +520,7 @@ INTBIG ro_testpoly(NODEINST *ni, PORTPROTO *pp, ARCPROTO *ap, POLYGON *poly, NOD
 		oy = (ni->lowy + ni->highy) / 2;
 
 		/* look at all polygons on this nodeinst */
-		tot = nodeEpolys(oni, 0);
+		tot = nodeEpolys(oni, 0, NOWINDOWPART);
 		for(j=0; j<tot; j++)
 		{
 			shapeEnodepoly(oni, j, opoly);
@@ -682,7 +721,7 @@ void ro_findsmallestlayer(ARCPROTO *ap)
 
 	/* find the smallest layer */
 	bestfound = 0;
-	j = arcpolys(ai);
+	j = arcpolys(ai, NOWINDOWPART);
 	for(i=0; i<j; i++)
 	{
 		shapearcpoly(ai, i, poly);
@@ -718,7 +757,7 @@ void ro_substituteforpseudo(NODEINST *ni, POLYGON *poly)
 		for(pi = ni->firstportarcinst; pi != NOPORTARCINST; pi = pi->nextportarcinst)
 		{
 			ai = pi->conarcinst;
-			j = arcpolys(ai);
+			j = arcpolys(ai, NOWINDOWPART);
 			for(i=0; i<j; i++)
 			{
 				shapearcpoly(ai, i, apoly);
