@@ -341,7 +341,7 @@ void ro_mazerouteselected(void)
 	REGISTER NETWORK *net;
 
 	/* see what is selected */
-	list = us_gethighlighted(OBJARCINST);
+	list = us_gethighlighted(OBJARCINST, 0, 0);
 	for(selectcount=0; list[selectcount] != NOGEOM; selectcount++) ;
 	if (selectcount == 0)
 	{
@@ -929,7 +929,7 @@ void ro_mazedraw_arcinst(ARCINST *ai, XARRAY trans, NETWORK *net, SRREGION *regi
 	}
 
 	/* get the polygons of the arcinst */
-	i = arcpolys(ai);
+	i = arcpolys(ai, NOWINDOWPART);
 	for (j = 0; j < i; j++)
 	{
 		/* generate a polygon, force line for path generation */
@@ -980,7 +980,7 @@ void ro_mazedraw_nodeinst(NODEINST *ni, XARRAY prevtrans, NETWORK *net, SRREGION
 	if (np->primindex != 0)
 	{
 		/* primitive nodeinst: ask the technology how to draw it */
-		high = nodepolys(ni, 0);
+		high = nodepolys(ni, 0, NOWINDOWPART);
 		for (j = 0; j < high; j++)
 		{
 			/* get description of this layer */
