@@ -119,7 +119,7 @@ void ro_mimicstitch(INTSML forced)
 		}
 		elapsed = endtimer();
 		if ((options&MIMICINTERACTIVE) == 0 && elapsed >= 2.0)
-			ttyputmsg(_("Mimicing took %g seconds"), elapsed);
+			ttyputmsg(_("Mimicing took %s"), explainduration(elapsed));
 	}
 }
 
