@@ -86,7 +86,7 @@
 
 /******** RPOINT ********/
 
-#define	NOSIDE -1
+#define NOSIDE -1
 #define SIDE1   1
 #define SIDE2   2
 #define SIDE3   3
@@ -367,7 +367,7 @@ RDESC *ro_newroutedesc(INTBIG fx, INTBIG fy, INTSML fside, INTBIG sx, INTBIG sy,
 	if (rd->from == 0) return(0);
 	rd->to = ro_newrvpoint(sx, sy, sside);
 	if (rd->to == 0) return(0);
-	rd->sortval = -HUGEINT;
+	rd->sortval = -MAXINTBIG;
 	rd->unroutedwire1 = ai1;   rd->unroutedend1 = ae1;
 	rd->unroutedwire2 = ai2;   rd->unroutedend2 = ae2;
 	rd->path = NULLRPATH;
@@ -404,9 +404,9 @@ void ro_initialize(void)
 	ro_rrinfo.rightp = NULLRDESC;   ro_rrinfo.leftp = NULLRDESC;
 	ro_rrinfo.origmatrix = NULLTRANSFORM;
 	ro_rrinfo.invmatrix = NULLTRANSFORM;
-	ro_rrinfo.fromline = ro_rrinfo.toline = -HUGEINT;
-	ro_rrinfo.startright = -HUGEINT;   ro_rrinfo.startleft = -HUGEINT;
-	ro_rrinfo.height = -HUGEINT;   ro_rrinfo.xx = ILLEGALROUTE;
+	ro_rrinfo.fromline = ro_rrinfo.toline = -MAXINTBIG;
+	ro_rrinfo.startright = -MAXINTBIG;   ro_rrinfo.startleft = -MAXINTBIG;
+	ro_rrinfo.height = -MAXINTBIG;   ro_rrinfo.xx = ILLEGALROUTE;
 
 	if (ro_initialized == 0)
 	{
@@ -1000,7 +1000,7 @@ INTSML ro_calculate_height_and_process(RDESC *right, RDESC *left, INTBIG width,
 	RDESC *rright, *lleft;
 	RPOINT *lastp;
 
-	maxheight = -HUGEINT;
+	maxheight = -MAXINTBIG;
 	for(rright = right; rright; rright = rright->next)
 		maxheight = maxi(maxheight, rright->path->lastp->second);
 
@@ -1044,7 +1044,7 @@ void ro_calculate_bb(INTBIG *llx, INTBIG *lly, INTBIG *urx, INTBIG *ury, RDESC *
 	RPOINT *rvp;
 	INTBIG lx, ly, ux, uy;
 
-	lx = ly = HUGEINT; ux = uy = -HUGEINT;
+	lx = ly = MAXINTBIG; ux = uy = -MAXINTBIG;
 	for(rright = right; rright; rright = rright->next)
 	{
 		for(rvp = rright->path->pathdesc; rvp; rvp = rvp->next)
@@ -1736,7 +1736,7 @@ RDESC *ro_findnormalwires(NODEPROTO *np, INTSML *tot)
 	INTBIG llx, lly, urx, ury;
 	INTSML wireadded;
 
-	thelist = NULLRDESC;   llx = lly = HUGEINT;   urx = ury = -HUGEINT;
+	thelist = NULLRDESC;   llx = lly = MAXINTBIG;   urx = ury = -MAXINTBIG;
 	total = 0;
 
 	for(ai = np->firstarcinst; ai != NOARCINST; ai = ai->nextarcinst) ai->temp1 = 0;
@@ -1838,9 +1838,10 @@ INTSML ro_findwires(NODEPROTO *np)
 	ai->proto = wantap;
 	ai->width = defaultarcwidth(wantap);
 	ai->length = 10000;
-	(void)arcpolys(ai);
+	(void)arcpolys(ai, NOWINDOWPART);
 	shapearcpoly(ai, 0, poly);
-	amt = drcmindistance(ai->proto->tech, np->cell->lib, poly->layer, poly->layer, 0);
+	amt = drcmindistance(ai->proto->tech, np->cell->lib, poly->layer, ai->width,
+		poly->layer, ai->width, 0, 0, 0);
 	if (amt < 0) amt = lambdaofarc(ai);
 	return(ro_unsorted_rivrot(wantap, thelist, ai->width, amt, amt, amt, 0));
 }
