@@ -92,18 +92,18 @@ void Sc_main(void)
 	int	   count, l, err;
 	char   *pars[PARS];
 
-	ttyputmsg(_("QUISC - Queen's University Interactive Silicon Compiler"));
-	ttyputmsg(_("Version %s, %s"), VERSION, DATE);
+	ttyputmsg(M_("QUISC - Queen's University Interactive Silicon Compiler"));
+	ttyputmsg(M_("Version %s, %s"), VERSION, DATE);
 
 	if (setjmp(sc_please_stop))
 	{
 		Sc_clear_stop();
-		ttyputmsg(_("Command Aborted..."));
+		ttyputmsg(M_("Command Aborted..."));
 	}
 
 	for (;;)
 	{
-		count = ttygetfullparam(_("Enter command> "), &sc_silcomp, PARS, pars);
+		count = ttygetfullparam(M_("Enter command> "), &sc_silcomp, PARS, pars);
 		if (count < 0) break;
 		if (count == 0) continue;
 		l = strlen(pars[0]);
@@ -136,7 +136,7 @@ void Sc_one_command(int count, char *pars[])
 	if (setjmp(sc_please_stop))
 	{
 		Sc_clear_stop();
-		ttyputmsg(_("Command Aborted..."));
+		ttyputmsg(M_("Command Aborted..."));
 		return;
 	}
 
@@ -351,13 +351,13 @@ int Sc_compile(int count, char *pars[])
 		if ((err = Sc_parse(icount, par)))
 		{
 			ttyputmsg(_("ERROR line %d : SCERR %d - %s"), lnum, err, sc_errmsg);
-			ttyputmsg(inbuf);
+			ttyputverbose(inbuf);
 			ttyputmsg(_("Aborting compile..."));
 			break;
 		}
 	}
 	(void)askaid(vhdl_aid, "end-input");
-	ttyputmsg(_("Done (time = %s seconds)"), Sc_cpu_time(TIME_ABS));
+	ttyputmsg(_("Done (time = %s)"), Sc_cpu_time(TIME_ABS));
 	return(SC_NOERROR);
 }
 
@@ -392,14 +392,14 @@ int Sc_show(int count, char *pars[])
 	l = maxi(l, 2);
 	if (namesamen(sptr, "cells", l) == 0)
 	{
-		ttyputmsg(_("***** List of cells *****"));
+		ttyputmsg(M_("***** List of cells *****"));
 		for (cells = sc_cells; cells; cells = cells->next)
 		{
 			if (cells == sc_curcell)
 				ttyputmsg(" -->%s", cells->name); else
 					ttyputmsg("    %s", cells->name);
 		}
-		ttyputmsg(_("***** End of cell list *****"));
+		ttyputmsg(M_("***** End of cell list *****"));
 		return(0);
 	}
 
@@ -428,7 +428,7 @@ int Sc_show(int count, char *pars[])
 		}
 
 		/* list all exports on the cell */
-		ttyputmsg(_("Ports for cell %s:"), sptr);
+		ttyputmsg(M_("Ports for cell %s:"), sptr);
 		if (cells != NULL)
 		{
 			for (port = cells->ports; port; port = port->next)
@@ -436,36 +436,36 @@ int Sc_show(int count, char *pars[])
 				switch (port->bits & SCPORTTYPE)
 				{
 					case SCGNDPORT:
-						ptype = _("Ground port");
+						ptype = M_("Ground port");
 						break;
 					case SCPWRPORT:
-						ptype = _("Power port");
+						ptype = M_("Power port");
 						break;
 					case SCBIDIRPORT:
-						ptype = _("Bidirectional port");
+						ptype = M_("Bidirectional port");
 						break;
 					case SCOUTPORT:
-						ptype = _("Output port");
+						ptype = M_("Output port");
 						break;
 					case SCINPORT:
-						ptype = _("Input port");
+						ptype = M_("Input port");
 						break;
 					default:
-						ptype = _("Unknown type");
+						ptype = M_("Unknown type");
 						break;
 				}
 				*pdir = 0;
 				bits = port->bits;
 				if (bits & SCPORTDIRUP)
-					(void)strcat(pdir, _(" up"));
+					(void)strcat(pdir, M_(" up"));
 				if (bits & SCPORTDIRDOWN)
-					(void)strcat(pdir, _(" down"));
+					(void)strcat(pdir, M_(" down"));
 				if (bits & SCPORTDIRRIGHT)
-					(void)strcat(pdir, _(" right"));
+					(void)strcat(pdir, M_(" right"));
 				if (bits & SCPORTDIRLEFT)
-					(void)strcat(pdir, _(" left"));
+					(void)strcat(pdir, M_(" left"));
 				if (!(bits & SCPORTDIRMASK))
-					(void)strcat(pdir, _(" none"));
+					(void)strcat(pdir, M_(" none"));
 				ttyputmsg("  %-20s    %-20s    %-20s", port->name,
 					ptype, pdir);
 			}
@@ -477,41 +477,41 @@ int Sc_show(int count, char *pars[])
 				switch (Sc_leaf_port_type(leafport))
 				{
 					case SCGNDPORT:
-						ptype = _("Ground port");
+						ptype = M_("Ground port");
 						break;
 					case SCPWRPORT:
-						ptype = _("Power port");
+						ptype = M_("Power port");
 						break;
 					case SCBIDIRPORT:
-						ptype = _("Bidirectional port");
+						ptype = M_("Bidirectional port");
 						break;
 					case SCOUTPORT:
-						ptype = _("Output port");
+						ptype = M_("Output port");
 						break;
 					case SCINPORT:
-						ptype = _("Input port");
+						ptype = M_("Input port");
 						break;
 					default:
-						ptype = _("Unknown type");
+						ptype = M_("Unknown type");
 						break;
 				}
 				*pdir = 0;
 				bits = Sc_leaf_port_bits(leafport);
 				if (bits & SCPORTDIRUP)
-					(void)strcat(pdir, _(" up"));
+					(void)strcat(pdir, M_(" up"));
 				if (bits & SCPORTDIRDOWN)
-					(void)strcat(pdir, _(" down"));
+					(void)strcat(pdir, M_(" down"));
 				if (bits & SCPORTDIRRIGHT)
-					(void)strcat(pdir, _(" right"));
+					(void)strcat(pdir, M_(" right"));
 				if (bits & SCPORTDIRLEFT)
-					(void)strcat(pdir, _(" left"));
+					(void)strcat(pdir, M_(" left"));
 				if (!(bits & SCPORTDIRMASK))
-					(void)strcat(pdir, _(" none"));
+					(void)strcat(pdir, M_(" none"));
 				ttyputmsg("  %-20s    %-20s    %-20s",
 					Sc_leaf_port_name(leafport), ptype, pdir);
 			}
 		}
-		ttyputmsg(_("End of port list."));
+		ttyputmsg(M_("End of port list."));
 		return(0);
 	}
 
@@ -523,13 +523,13 @@ int Sc_show(int count, char *pars[])
 		}
 		Sc_leaf_facet_get_nums(leafcell, &cnums);
 		ttyputmsg("****************************************");
-		ttyputmsg(_("**      LEAF CELL MAGIC NUMBERS       **"));
+		ttyputmsg(M_("**      LEAF CELL MAGIC NUMBERS       **"));
 		ttyputmsg("****************************************");
 		ttyputmsg(" ");
-		ttyputmsg(_("Top active distance      =  %d"), cnums.top_active);
-		ttyputmsg(_("Bottom active distance   =  %d"), cnums.bottom_active);
-		ttyputmsg(_("Left active distance     =  %d"), cnums.left_active);
-		ttyputmsg(_("Right active distance    =  %d"), cnums.right_active);
+		ttyputmsg(M_("Top active distance      =  %d"), cnums.top_active);
+		ttyputmsg(M_("Bottom active distance   =  %d"), cnums.bottom_active);
+		ttyputmsg(M_("Left active distance     =  %d"), cnums.left_active);
+		ttyputmsg(M_("Right active distance    =  %d"), cnums.right_active);
 		ttyputmsg("----------------------------------------");
 		return(SC_NOERROR);
 	}
@@ -562,7 +562,7 @@ int Sc_show(int count, char *pars[])
 		{
 			all_flag = FALSE;
 		}
-		ttyputmsg(_("Instances for cell %s:"), cells->name);
+		ttyputmsg(M_("Instances for cell %s:"), cells->name);
 		Sc_nitreelist(cells->niroot, all_flag);
 		return(0);
 	}
@@ -571,7 +571,7 @@ int Sc_show(int count, char *pars[])
 	{
 		if (sc_curcell == NULL)
 			return(Sc_seterrmsg(SC_NOCELL));
-		ttyputmsg(_("List of extracted nodes for cell %s:"), sc_curcell->name);
+		ttyputmsg(M_("List of extracted nodes for cell %s:"), sc_curcell->name);
 		Sc_extract_print_nodes(sc_curcell);
 		return(SC_NOERROR);
 	}
@@ -597,10 +597,10 @@ int Sc_show(int count, char *pars[])
 		}
 		if (cells->placement == NULL)
 			return(Sc_seterrmsg(SC_CELL_NO_PLACE, cells->name));
-		ttyputmsg(_("*****  PLACEMENT for cell %s *****"), sptr);
+		ttyputmsg(M_("*****  PLACEMENT for cell %s *****"), sptr);
 		for (row = cells->placement->rows; row; row = row->next)
 		{
-			ttyputmsg(_("For Row #%d, size %d:"), row->row_num, row->row_size);
+			ttyputmsg(M_("For Row #%d, size %d:"), row->row_num, row->row_size);
 			for (place = row->start; place; place = place->next)
 				ttyputmsg("    %8d    %-s", place->xpos, place->cell->name);
 		}
@@ -629,7 +629,7 @@ int Sc_show(int count, char *pars[])
 		if (cells->route == NULL)
 			return(Sc_seterrmsg(SC_CELL_NO_ROUTE, cells->name));
 		ttyputmsg("****************************************");
-		ttyputmsg(_("**     CHANNEL ROUTING INFORMATION    **"));
+		ttyputmsg(M_("**     CHANNEL ROUTING INFORMATION    **"));
 		ttyputmsg("****************************************");
 		ttyputmsg(" ");
 		Sc_route_print_channel(cells->route->channels);
@@ -884,6 +884,13 @@ int Sc_library(int count, char *pars[])
 
 	l = strlen((sptr = pars[0]));
 	if (count == 0 || l == 0) return(Sc_seterrmsg(SC_NLIBRARY));
+	if (namesamen(sptr, "use", l) == 0)
+	{
+		l = strlen(sptr = pars[1]);
+		if (--count == 0 || l == 0) return(Sc_seterrmsg(SC_NLIBRARYUSE));
+		err = Sc_library_use(sptr);
+		return(err);
+	}
 	if (namesamen(sptr, "read", l) == 0)
 	{
 		l = strlen(sptr = pars[1]);
@@ -931,7 +938,7 @@ int Sc_order(int count, char *pars[])
 	{
 		if (fp == NULL)
 		{
-			sptr = ttygetline(_("Port>  "));
+			sptr = ttygetline(M_("Port>  "));
 			if (sptr == 0 || strlen(sptr) == 0) break;
 			(void)strcpy(order, sptr);
 		} else
@@ -1403,7 +1410,7 @@ seconds		*char		Pointer to string giving time in seconds.
 
 char  *Sc_cpu_time(int action)
 {
-	static char		sbuf[40];
+	static char		sbuf[200];
 	static float	sofar;
 	float thisone;
 
@@ -1412,12 +1419,12 @@ char  *Sc_cpu_time(int action)
 		case TIME_ABS:
 			thisone = endtimer();
 			sofar += thisone;
-			(void)sprintf(sbuf, "%.2f", sofar);
+			(void)strcpy(sbuf, explainduration(sofar));
 			return(sbuf);
 		case TIME_REL:
 			thisone = endtimer();
 			sofar += thisone;
-			(void)sprintf(sbuf, "%.2f", thisone);
+			(void)strcpy(sbuf, explainduration(thisone));
 			return(sbuf);
 		case TIME_RESET:
 			starttimer();
