@@ -42,10 +42,10 @@
 /*>>>>>>>>>>>>>>>>>>>> CONNECT command <<<<<<<<<<<<<<<<<<<<*/
 
 static COMCOMP sc_connap = {NOKEYWORD,NOTOPLIST,NONEXTLIST,NOPARAMS,NOBACKUP,
-	0, " \t", "name of node a", 0};
+	0, " \t", M_("name of node a"), 0};
 
 static COMCOMP sc_conpap = {NOKEYWORD,NOTOPLIST,NONEXTLIST,NOPARAMS,NOBACKUP,
-	0, " \t", "port on node a", 0};
+	0, " \t", M_("port on node a"), 0};
 
 static KEYWORD sc_conb[] = {
 	{"power",		0,{NOKEY,NOKEY,NOKEY,NOKEY,NOKEY}},
@@ -54,22 +54,22 @@ static KEYWORD sc_conb[] = {
 };
 
 static COMCOMP sc_connbp = {sc_conb,NOTOPLIST,NONEXTLIST,NOPARAMS,NOBACKUP,
-	0, " \t", "name of node b", 0};
+	0, " \t", M_("name of node b"), 0};
 
 static COMCOMP sc_conpbp = {NOKEYWORD,NOTOPLIST,NONEXTLIST,NOPARAMS,NOBACKUP,
-	0, " \t", "port on node b", 0};
+	0, " \t", M_("port on node b"), 0};
 
 
 /*>>>>>>>>>>>>>>>>>>>> CREATE command <<<<<<<<<<<<<<<<<<<<*/
 
 static COMCOMP sc_cellp = {NOKEYWORD,NOTOPLIST,NONEXTLIST,NOPARAMS,NOBACKUP,
-	0, " \t", "name of cell", 0};
+	0, " \t", M_("name of cell"), 0};
 
 static COMCOMP sc_crinstnp = {NOKEYWORD,NOTOPLIST,NONEXTLIST,NOPARAMS,NOBACKUP,
-	0, " \t", "instance name", 0};
+	0, " \t", M_("instance name"), 0};
 
 static COMCOMP sc_crinsttp = {NOKEYWORD,NOTOPLIST,NONEXTLIST,NOPARAMS,NOBACKUP,
-	0, " \t", "instance type", 0};
+	0, " \t", M_("instance type"), 0};
 
 static KEYWORD sc_createopt[] =
 {
@@ -79,19 +79,19 @@ static KEYWORD sc_createopt[] =
 };
 
 static COMCOMP sc_createp = {sc_createopt,NOTOPLIST,NONEXTLIST,NOPARAMS,
-	NOBACKUP, 0, " \t", "create option", 0};
+	NOBACKUP, 0, " \t", M_("create option"), 0};
 
 
 /*>>>>>>>>>>>>>>>>>>>> EXPORT command <<<<<<<<<<<<<<<<<<<<*/
 
 static COMCOMP sc_exportnp = {NOKEYWORD,NOTOPLIST,NONEXTLIST,NOPARAMS,NOBACKUP,
-	0, " \t", "name of instance", 0};
+	0, " \t", M_("name of instance"), 0};
 
 static COMCOMP sc_exportpp = {NOKEYWORD,NOTOPLIST,NONEXTLIST,NOPARAMS,NOBACKUP,
-	0, " \t", "name of port on instance", 0};
+	0, " \t", M_("name of port on instance"), 0};
 
 static COMCOMP sc_exportpnp = {NOKEYWORD,NOTOPLIST,NONEXTLIST,NOPARAMS,NOBACKUP,
-	0, " \t", "name of port", 0};
+	0, " \t", M_("name of port"), 0};
 
 static KEYWORD sc_exportopt[] =
 {
@@ -102,7 +102,7 @@ static KEYWORD sc_exportopt[] =
 };
 
 static COMCOMP sc_exporttp = {sc_exportopt,NOTOPLIST,NONEXTLIST,NOPARAMS,
-	NOBACKUP,0, " \t", "type of port", 0};
+	NOBACKUP,0, " \t", M_("type of port"), 0};
 
 
 /*>>>>>>>>>>>>>>>>>>>> EXTRACT command <<<<<<<<<<<<<<<<<<<<*/
@@ -114,13 +114,13 @@ static KEYWORD sc_echo[] =
 };
 
 static COMCOMP sc_extractp = {sc_echo,NOTOPLIST,NONEXTLIST,NOPARAMS,NOBACKUP,
-	0, " \t", "optional echo flag", 0};
+	0, " \t", M_("optional echo flag"), 0};
 
 
 /*>>>>>>>>>>>>>>>>>>>> LIBRARY command <<<<<<<<<<<<<<<<<<<<*/
 
 static COMCOMP sc_libusep = {NOKEYWORD,NOTOPLIST,NONEXTLIST,NOPARAMS,NOBACKUP,
-	0, " \t", "name of library", 0};
+	0, " \t", M_("name of library"), 0};
 
 static KEYWORD sc_libopt[] =
 {
@@ -129,40 +129,40 @@ static KEYWORD sc_libopt[] =
 };
 
 static COMCOMP sc_libp = {sc_libopt,NOTOPLIST,NONEXTLIST,NOPARAMS,NOBACKUP,
-	0, " \t", "library option", 0};
+	0, " \t", M_("library option"), 0};
 
 
 /*>>>>>>>>>>>>>>>>>>>> MAKE command <<<<<<<<<<<<<<<<<<<<*/
 
 static COMCOMP sc_makemsp = {NOKEYWORD,NOTOPLIST,NONEXTLIST,NOPARAMS,NOBACKUP,
-	0, " \t", "Minimum spacing (in internal units)", 0};
+	0, " \t", M_("Minimum spacing (in internal units)"), 0};
 
 static COMCOMP sc_makevsp = {NOKEYWORD,NOTOPLIST,NONEXTLIST,NOPARAMS,NOBACKUP,
-	0, " \t", "Via Size (internal units)", 0};
+	0, " \t", M_("Via Size (internal units)"), 0};
 
 static COMCOMP sc_makeavp = {NOKEYWORD,topofarcs,nextarcs,NOPARAMS,
-	NOBACKUP, 0, " \t", "Arc type to use for vertical wires", 0};
+	NOBACKUP, 0, " \t", M_("Arc type to use for vertical wires"), 0};
 
 static COMCOMP sc_makeahp = {NOKEYWORD,topofarcs,nextarcs,NOPARAMS,
-	NOBACKUP, 0, " \t", "Arc type to use for horizontal wires", 0};
+	NOBACKUP, 0, " \t", M_("Arc type to use for horizontal wires"), 0};
 
 static COMCOMP sc_makevtp = {NOKEYWORD,NOTOPLIST,NONEXTLIST,NOPARAMS,NOBACKUP,
-	0, " \t", "Layer2 Track width (internal units)", 0};
+	0, " \t", M_("Layer2 Track width (internal units)"), 0};
 
 static COMCOMP sc_makehtp = {NOKEYWORD,NOTOPLIST,NONEXTLIST,NOPARAMS,NOBACKUP,
-	0, " \t", "Layer1 Track width (internal units)", 0};
+	0, " \t", M_("Layer1 Track width (internal units)"), 0};
 
 static COMCOMP sc_makeptp = {NOKEYWORD,NOTOPLIST,NONEXTLIST,NOPARAMS,NOBACKUP,
-	0, " \t", "Power Track width (internal units)", 0};
+	0, " \t", M_("Power Track width (internal units)"), 0};
 
 static COMCOMP sc_makemtp = {NOKEYWORD,NOTOPLIST,NONEXTLIST,NOPARAMS,NOBACKUP,
-	0, " \t", "Main Power Bus width (internal units)", 0};
+	0, " \t", M_("Main Power Bus width (internal units)"), 0};
 
 static COMCOMP sc_makepwsp = {NOKEYWORD,NOTOPLIST,NONEXTLIST,NOPARAMS,NOBACKUP,
-	0, " \t", "P-well Size (internal units)", 0};
+	0, " \t", M_("P-well Size (internal units)"), 0};
 
 static COMCOMP sc_makepwop = {NOKEYWORD,NOTOPLIST,NONEXTLIST,NOPARAMS,NOBACKUP,
-	0, " \t", "P-well Offset from bottom (internal units)", 0};
+	0, " \t", M_("P-well Offset from bottom (internal units)"), 0};
 
 static KEYWORD sc_makesetopt[] =
 {
@@ -183,7 +183,7 @@ static KEYWORD sc_makesetopt[] =
 };
 
 static COMCOMP sc_makesetp = {sc_makesetopt,NOTOPLIST,NONEXTLIST,NOPARAMS,
-	NOBACKUP, 0, " \t", "Make set-control option", 0};
+	NOBACKUP, 0, " \t", M_("Make set-control option"), 0};
 
 static KEYWORD sc_makeopt[] =
 {
@@ -195,28 +195,28 @@ static KEYWORD sc_makeopt[] =
 };
 
 COMCOMP sc_makep = {sc_makeopt,NOTOPLIST,NONEXTLIST,NOPARAMS,NOBACKUP,
-	0, " \t", "Make option", 0};
+	0, " \t", M_("Make option"), 0};
 
 
 /*>>>>>>>>>>>>>>>>>>>> ORDER command <<<<<<<<<<<<<<<<<<<<*/
 
 static COMCOMP sc_orderp = {NOKEYWORD,NOTOPLIST,NONEXTLIST,NOPARAMS,NOBACKUP,
-	0, " \t", "leaf cell to order ports", 0};
+	0, " \t", M_("leaf cell to order ports"), 0};
 
 static COMCOMP sc_orderfp = {NOKEYWORD,topoffile,nextfile,NOPARAMS,NOBACKUP,
-	NOFILL, " \t", "Optional input file", 0};
+	NOFILL, " \t", M_("Optional input file"), 0};
 
 
 /*>>>>>>>>>>>>>>>>>>>> PLACE command <<<<<<<<<<<<<<<<<<<<*/
 
 static COMCOMP sc_placerp = {NOKEYWORD,NOTOPLIST,NONEXTLIST,NOPARAMS,NOBACKUP,
-	0, " \t", "Number of rows", 0};
+	0, " \t", M_("Number of rows"), 0};
 
 static COMCOMP sc_placevp = {NOKEYWORD,NOTOPLIST,NONEXTLIST,NOPARAMS,NOBACKUP,
-	0, " \t", "Vertical cost scaling factor", 0};
+	0, " \t", M_("Vertical cost scaling factor"), 0};
 
 static COMCOMP sc_placenp = {NOKEYWORD,NOTOPLIST,NONEXTLIST,NOPARAMS,NOBACKUP,
-	0, " \t", "Movement limit for Net Balancing", 0};
+	0, " \t", M_("Movement limit for Net Balancing"), 0};
 
 static KEYWORD sc_placesetopt[] =
 {
@@ -234,7 +234,7 @@ static KEYWORD sc_placesetopt[] =
 };
 
 static COMCOMP sc_placesetp = {sc_placesetopt,NOTOPLIST,NONEXTLIST,NOPARAMS,
-	NOBACKUP, 0, " \t", "Placement set-control option", 0};
+	NOBACKUP, 0, " \t", M_("Placement set-control option"), 0};
 
 static KEYWORD sc_placeopt[] =
 {
@@ -245,7 +245,7 @@ static KEYWORD sc_placeopt[] =
 };
 
 static COMCOMP sc_placep = {sc_placeopt,NOTOPLIST,NONEXTLIST,NOPARAMS,NOBACKUP,
-	0, " \t", "placement algorithm", 0};
+	0, " \t", M_("placement algorithm"), 0};
 
 
 /*>>>>>>>>>>>>>>>>>>>> PULL command <<<<<<<<<<<<<<<<<<<<*/
@@ -257,22 +257,22 @@ static KEYWORD sc_pullopt[] =
 };
 
 static COMCOMP sc_pullp = {sc_pullopt,NOTOPLIST,NONEXTLIST,NOPARAMS,NOBACKUP,
-	0, " \t", "pull (flatten) cell", 0};
+	0, " \t", M_("pull (flatten) cell"), 0};
 
 
 /*>>>>>>>>>>>>>>>>>>>> ROUTE command <<<<<<<<<<<<<<<<<<<<*/
 
 static COMCOMP sc_routeftp = {NOKEYWORD,NOTOPLIST,NONEXTLIST,NOPARAMS,NOBACKUP,
-	0, " \t", "Size of Feed Throughs (internal units)", 0};
+	0, " \t", M_("Size of Feed Throughs (internal units)"), 0};
 
 static COMCOMP sc_routepdp = {NOKEYWORD,NOTOPLIST,NONEXTLIST,NOPARAMS,NOBACKUP,
-	0, " \t", "Minimum Distance between ports (internal units)", 0};
+	0, " \t", M_("Minimum Distance between ports (internal units)"), 0};
 
 static COMCOMP sc_routefwp = {NOKEYWORD,NOTOPLIST,NONEXTLIST,NOPARAMS,NOBACKUP,
-	0, " \t", "Fuzzy Window Limit (internal units)", 0};
+	0, " \t", M_("Fuzzy Window Limit (internal units)"), 0};
 
 static COMCOMP sc_routeadp = {NOKEYWORD,NOTOPLIST,NONEXTLIST,NOPARAMS,NOBACKUP,
-	0, " \t", "Minimum Active distance (internal units)", 0};
+	0, " \t", M_("Minimum Active distance (internal units)"), 0};
 
 static KEYWORD sc_routesetopt[] =
 {
@@ -287,7 +287,7 @@ static KEYWORD sc_routesetopt[] =
 };
 
 static COMCOMP sc_routesetp = {sc_routesetopt,NOTOPLIST,NONEXTLIST,NOPARAMS,
-	NOBACKUP, 0, " \t", "Route set-control option", 0};
+	NOBACKUP, 0, " \t", M_("Route set-control option"), 0};
 
 static KEYWORD sc_routeopt[] =
 {
@@ -298,28 +298,28 @@ static KEYWORD sc_routeopt[] =
 };
 
 static COMCOMP sc_routep = {sc_routeopt,NOTOPLIST,NONEXTLIST,NOPARAMS,NOBACKUP,
-	0, " \t", "Routing option", 0};
+	0, " \t", M_("Routing option"), 0};
 
 
 /*>>>>>>>>>>>>>>>>>>>> SELECT command <<<<<<<<<<<<<<<<<<<<*/
 
 static COMCOMP sc_selectp = {NOKEYWORD,NOTOPLIST,NONEXTLIST,NOPARAMS,
-	NOBACKUP, 0, " \t", "name of cell", 0};
+	NOBACKUP, 0, " \t", M_("name of cell"), 0};
 
 
 /*>>>>>>>>>>>>>>>>>>>> SET command <<<<<<<<<<<<<<<<<<<<*/
 
 static COMCOMP sc_setpdp = {NOKEYWORD,NOTOPLIST,NONEXTLIST,NOPARAMS,
-	NOBACKUP, 0, " \t", "direction(s) to attach to port (u d r l)", 0};
+	NOBACKUP, 0, " \t", M_("direction(s) to attach to port (u d r l)"), 0};
 
 static COMCOMP sc_setpdpp = {NOKEYWORD,NOTOPLIST,NONEXTLIST,NOPARAMS,NOBACKUP,
-	0, " \t", "name of port", 0};
+	0, " \t", M_("name of port"), 0};
 
 static COMCOMP sc_setpdcp = {NOKEYWORD,NOTOPLIST,NONEXTLIST,NOPARAMS,NOBACKUP,
-	0, " \t", "name of cell", 0};
+	0, " \t", M_("name of cell"), 0};
 
 static COMCOMP sc_setnumvp = {NOKEYWORD,NOTOPLIST,NONEXTLIST,NOPARAMS,NOBACKUP,
-	0, " \t", "From edge (internal units)", 0};
+	0, " \t", M_("From edge (internal units)"), 0};
 
 static KEYWORD sc_setnumopt[] =
 {
@@ -331,16 +331,16 @@ static KEYWORD sc_setnumopt[] =
 };
 
 static COMCOMP sc_setnump = {sc_setnumopt,NOTOPLIST,NONEXTLIST,NOPARAMS,
-	NOBACKUP, 0, " \t", "cell number option", 0};
+	NOBACKUP, 0, " \t", M_("cell number option"), 0};
 
 static COMCOMP sc_setnnip = {NOKEYWORD,NOTOPLIST,NONEXTLIST,NOPARAMS,
-	NOBACKUP, 0, " \t", "instance name", 0};
+	NOBACKUP, 0, " \t", M_("instance name"), 0};
 
 static COMCOMP sc_setnnpp = {NOKEYWORD,NOTOPLIST,NONEXTLIST,NOPARAMS,
-	NOBACKUP, 0, " \t", "port name", 0};
+	NOBACKUP, 0, " \t", M_("port name"), 0};
 
 static COMCOMP sc_setnnnp = {NOKEYWORD,NOTOPLIST,NONEXTLIST,NOPARAMS,
-	NOBACKUP, 0, " \t", "node name", 0};
+	NOBACKUP, 0, " \t", M_("node name"), 0};
 
 static KEYWORD sc_setopt[] =
 {
@@ -352,13 +352,13 @@ static KEYWORD sc_setopt[] =
 };
 
 static COMCOMP sc_setp = {sc_setopt,NOTOPLIST,NONEXTLIST,NOPARAMS,NOBACKUP,
-	0, " \t", "set option", 0};
+	0, " \t", M_("set option"), 0};
 
 
 /*>>>>>>>>>>>>>>>>>>>> SHOW command <<<<<<<<<<<<<<<<<<<<*/
 
 static COMCOMP sc_showpp = {NOKEYWORD,NOTOPLIST,NONEXTLIST,NOPARAMS,NOBACKUP,
-	0, " \t", "cell name", 0};
+	0, " \t", M_("cell name"), 0};
 
 static KEYWORD sc_shownopt[] =
 {
@@ -367,10 +367,10 @@ static KEYWORD sc_shownopt[] =
 };
 
 static COMCOMP sc_shownp = {sc_shownopt,NOTOPLIST,NONEXTLIST,NOPARAMS,
-	NOBACKUP, 0, " \t", "show instances option", 0};
+	NOBACKUP, 0, " \t", M_("show instances option"), 0};
 
 static COMCOMP sc_shownump = {NOKEYWORD,NOTOPLIST,NONEXTLIST,NOPARAMS,
-	NOBACKUP, 0, " \t", "leaf cell name", 0};
+	NOBACKUP, 0, " \t", M_("leaf cell name"), 0};
 
 static KEYWORD sc_showopt[] =
 {
@@ -385,19 +385,19 @@ static KEYWORD sc_showopt[] =
 };
 
 static COMCOMP sc_showp = {sc_showopt,NOTOPLIST,NONEXTLIST,NOPARAMS,
-	NOBACKUP, 0, " \t", "show option", 0};
+	NOBACKUP, 0, " \t", M_("show option"), 0};
 
 
 /*>>>>>>>>>>>>>>>>>>>> SIMULATION command <<<<<<<<<<<<<<<<<<<<*/
 
 static COMCOMP sc_simsetp = {NOKEYWORD,NOTOPLIST,NONEXTLIST,NOPARAMS,NOBACKUP,
-	0, " \t", "name of cell", 0};
+	0, " \t", M_("name of cell"), 0};
 
 static COMCOMP sc_simwrfilep = {NOKEYWORD,NOTOPLIST,NONEXTLIST,NOPARAMS,
-	NOBACKUP, 0, " \t", "name of output file", 0};
+	NOBACKUP, 0, " \t", M_("name of output file"), 0};
 
 static COMCOMP sc_simformatp = {NOKEYWORD,NOTOPLIST,NONEXTLIST,NOPARAMS,
-	NOBACKUP, 0, " \t",  "Output format: als or silos", 0};
+	NOBACKUP, 0, " \t",  M_("Output format: als or silos"), 0};
 
 static KEYWORD sc_simopt[] =
 {
@@ -409,13 +409,13 @@ static KEYWORD sc_simopt[] =
 };
 
 static COMCOMP sc_simp = {sc_simopt,NOTOPLIST,NONEXTLIST,NOPARAMS,NOBACKUP,
-	0, " \t", "set option", 0};
+	0, " \t", M_("set option"), 0};
 
 
 /*>>>>>>>>>>>>>>>>>>>> HELP command <<<<<<<<<<<<<<<<<<<<*/
 
 static COMCOMP sc_helpp = {NOKEYWORD,NOTOPLIST,NONEXTLIST,NOPARAMS,NOBACKUP,
-	0, " \t", "help object", 0};
+	0, " \t", M_("help object"), 0};
 
 
 /*>>>>>>>>>>>>>>>>>>>> SILICON COMPILER commands <<<<<<<<<<<<<<<<<<<<*/
@@ -446,6 +446,6 @@ static KEYWORD sc_opt[] =
 };
 
 COMCOMP sc_silcomp = {sc_opt,NOTOPLIST,NONEXTLIST,NOPARAMS,NOBACKUP,
-	0, " \t", "Silicon Compiler action ('quit' to terminate)", 0};
+	0, " \t", M_("Silicon Compiler action"), 0};
 
 #endif  /* SCAID - at top */
