@@ -52,49 +52,15 @@ static NODEPROTO *sc_layer1proto = NONODEPROTO;
 static NODEPROTO *sc_layer2proto = NONODEPROTO;
 static NODEPROTO *sc_viaproto = NONODEPROTO;
 static NODEPROTO *sc_pwellproto = NONODEPROTO;
+static NODEPROTO *sc_nwellproto = NONODEPROTO;
 static ARCPROTO	 *sc_layer1arc = NOARCPROTO;
 static ARCPROTO	 *sc_layer2arc = NOARCPROTO;
+static LIBRARY   *sc_celllibrary = NOLIBRARY;
 
 extern char      *sc_errmsg;
 extern int        sc_sim_format;
 extern COMCOMP    sc_makep;
 
-/* Silicon Compiler Options */
-DIALOGITEM sc_optionsdialogitems[] =
-{
- /*  1 */ {0, {428,272,452,330}, BUTTON, N_("OK")},
- /*  2 */ {0, {428,40,452,98}, BUTTON, N_("Cancel")},
- /*  3 */ {0, {60,208,76,388}, POPUP, ""},
- /*  4 */ {0, {8,208,24,388}, POPUP, ""},
- /*  5 */ {0, {8,8,24,180}, MESSAGE, N_("Horizontal routing arc:")},
- /*  6 */ {0, {60,8,76,180}, MESSAGE, N_("Vertical routing arc:")},
- /*  7 */ {0, {32,8,48,174}, MESSAGE, N_("Horizontal wire width:")},
- /*  8 */ {0, {32,208,48,282}, EDITTEXT, "2.5"},
- /*  9 */ {0, {84,8,100,174}, MESSAGE, N_("Vertical wire width:")},
- /* 10 */ {0, {84,208,100,282}, EDITTEXT, "2.5"},
- /* 11 */ {0, {120,8,136,174}, MESSAGE, N_("Power wire width:")},
- /* 12 */ {0, {120,208,136,282}, EDITTEXT, "2.5"},
- /* 13 */ {0, {144,8,160,174}, MESSAGE, N_("Main power wire width:")},
- /* 14 */ {0, {144,208,160,282}, EDITTEXT, "2.5"},
- /* 15 */ {0, {184,12,200,203}, MESSAGE, N_("P-Well height (0 for none):")},
- /* 16 */ {0, {184,208,200,282}, EDITTEXT, "2.5"},
- /* 17 */ {0, {208,12,224,203}, MESSAGE, N_("P-Well offset from bottom:")},
- /* 18 */ {0, {208,208,224,282}, EDITTEXT, "2.5"},
- /* 19 */ {0, {248,12,264,203}, MESSAGE, N_("Via size:")},
- /* 20 */ {0, {248,208,264,282}, EDITTEXT, "2.5"},
- /* 21 */ {0, {272,12,288,203}, MESSAGE, N_("Minimum metal spacing:")},
- /* 22 */ {0, {272,208,288,282}, EDITTEXT, "2.5"},
- /* 23 */ {0, {312,12,328,203}, MESSAGE, N_("Routing: feed-through size:")},
- /* 24 */ {0, {312,208,328,282}, EDITTEXT, "2.5"},
- /* 25 */ {0, {336,12,352,203}, MESSAGE, N_("Routing: min. port distance:")},
- /* 26 */ {0, {336,208,352,282}, EDITTEXT, "2.5"},
- /* 27 */ {0, {360,12,376,203}, MESSAGE, N_("Routing: min. active dist.:")},
- /* 28 */ {0, {360,208,376,282}, EDITTEXT, "2.5"},
- /* 29 */ {0, {400,12,416,203}, MESSAGE, N_("Number of rows of cells:")},
- /* 30 */ {0, {400,208,416,282}, EDITTEXT, "5"}
-};
-DIALOG sc_optionsdialog = {{50,75,512,472}, N_("Silicon Compiler Options"), 0, 30, sc_optionsdialogitems};
-
 /* prototypes for local routines */
 void Sc_set_leaf_port_type(PORTPROTO*, int);
 void Sc_create_connection(NODEINST **ni, PORTPROTO **pp, INTBIG x, INTBIG y,
@@ -158,6 +124,21 @@ INTSML sc_set(INTSML count, char *pars[])
 	return(1);
 }
 
+/***********************************************************************
+Module:  sc_request
+------------------------------------------------------------------------
+Description:
+	Module invoked by the askaid command.
+------------------------------------------------------------------------
+*/
+
+INTBIG sc_request(char *command, va_list ap)
+{
+	if (namesame(command, "cell-lib") == 0)
+		return((INTBIG)sc_celllibrary);
+	return(0);
+}
+
 /***********************************************************************
 Default Silicon Compiler aid routines.
 ------------------------------------------------------------------------
@@ -217,6 +198,10 @@ INTBIG ScGetParameter(INTBIG paramnum)
 			var = getval((INTBIG)sc_aid, VAID, -1, "SC_main_pwr_width");
 			if (var == NOVARIABLE) return(DEFAULT_MAIN_POWER_WIDTH);
 			break;
+		case SC_PARAM_MAKE_MAIN_PWR_RAIL:
+			var = getval((INTBIG)sc_aid, VAID, -1, "SC_main_pwr_rail");
+			if (var == NOVARIABLE) return(DEFAULT_MAIN_POWER_RAIL);
+			break;
 		case SC_PARAM_MAKE_PWELL_SIZE:
 			var = getval((INTBIG)sc_aid, VAID, -1, "SC_pwell_size");
 			if (var == NOVARIABLE) return(DEFAULT_PWELL_SIZE);
@@ -225,6 +210,14 @@ INTBIG ScGetParameter(INTBIG paramnum)
 			var = getval((INTBIG)sc_aid, VAID, -1, "SC_pwell_offset");
 			if (var == NOVARIABLE) return(DEFAULT_PWELL_OFFSET);
 			break;
+		case SC_PARAM_MAKE_NWELL_SIZE:
+			var = getval((INTBIG)sc_aid, VAID, -1, "SC_nwell_size");
+			if (var == NOVARIABLE) return(DEFAULT_NWELL_SIZE);
+			break;
+		case SC_PARAM_MAKE_NWELL_OFFSET:
+			var = getval((INTBIG)sc_aid, VAID, -1, "SC_nwell_offset");
+			if (var == NOVARIABLE) return(DEFAULT_NWELL_OFFSET);
+			break;
 		case SC_PARAM_MAKE_VIA_SIZE:
 			var = getval((INTBIG)sc_aid, VAID, -1, "SC_via_size");
 			if (var == NOVARIABLE) return(DEFAULT_VIA_SIZE);
@@ -275,12 +268,21 @@ void ScSetParameter(INTBIG paramnum, INTBIG addr)
 		case SC_PARAM_MAKE_MAIN_PWR_WIDTH:
 			setval((INTBIG)sc_aid, VAID, "SC_main_pwr_width", addr, VINTEGER);
 			break;
+		case SC_PARAM_MAKE_MAIN_PWR_RAIL:
+			setval((INTBIG)sc_aid, VAID, "SC_main_pwr_rail", addr, VINTEGER);
+			break;
 		case SC_PARAM_MAKE_PWELL_SIZE:
 			setval((INTBIG)sc_aid, VAID, "SC_pwell_size", addr, VINTEGER);
 			break;
 		case SC_PARAM_MAKE_PWELL_OFFSET:
 			setval((INTBIG)sc_aid, VAID, "SC_pwell_offset", addr, VINTEGER);
 			break;
+		case SC_PARAM_MAKE_NWELL_SIZE:
+			setval((INTBIG)sc_aid, VAID, "SC_nwell_size", addr, VINTEGER);
+			break;
+		case SC_PARAM_MAKE_NWELL_OFFSET:
+			setval((INTBIG)sc_aid, VAID, "SC_nwell_offset", addr, VINTEGER);
+			break;
 		case SC_PARAM_MAKE_VIA_SIZE:
 			setval((INTBIG)sc_aid, VAID, "SC_via_size", addr, VINTEGER);
 			break;
@@ -328,13 +330,25 @@ leaffacet	*char		Generic pointer to found facet,
 
 char  *Sc_find_leaf_facet(char *name)
 {
-	NODEPROTO	*np, *laynp;
+	NODEPROTO *np, *laynp;
 
 	np = getnodeproto(name);
+	if (np == NONODEPROTO)
+	{
+		(void)initinfstr();
+		if (sc_celllibrary != NOLIBRARY)
+		{
+			(void)formatinfstr("%s:%s", sc_celllibrary->libname, name);
+		} else
+		{
+			(void)formatinfstr("%s", name);
+		}
+		np = getnodeproto(returninfstr());
+		if (np == NONODEPROTO) return((char *)NULL);
+	}
 	laynp = layoutview(np);
 	if (laynp != NONODEPROTO) np = laynp;
-	if (np != NONODEPROTO && np->primindex == 0)
-		return((char *)np);
+	if (np->primindex == 0) return((char *)np);
 	return((char *)NULL);
 }
 
@@ -1111,6 +1125,36 @@ int Sc_library_read(char *libname)
 	return(0);
 }
 
+/***********************************************************************
+Module:  Sc_library_use
+------------------------------------------------------------------------
+Description:
+	Use the specified library as a cell library.  Set error codes.
+------------------------------------------------------------------------
+Calling Sequence:  err = Sc_library_use(libname);
+
+Name		Type		Description
+----		----		-----------
+libname		*char		String giving name of library.
+err			int			Returned error code, 0 = no error.
+------------------------------------------------------------------------
+*/
+
+int Sc_library_use(char *libname)
+{
+	LIBRARY    *lib;
+
+	/* check to see if library already exists */
+	for (lib = el_curlib; lib != NOLIBRARY; lib = lib->nextlibrary)
+		if (namesame(libname, lib->libname) == 0) break;
+	if (lib == NOLIBRARY)
+	{
+		return(Sc_seterrmsg(SC_NOLIBRARY, libname));
+	}
+	sc_celllibrary = lib;
+	return(0);
+}
+
 /***********************************************************************
 Module:  Sc_setup_for_maker
 ------------------------------------------------------------------------
@@ -1190,9 +1234,14 @@ int Sc_setup_for_maker(ARCPROTO *layer1, ARCPROTO *layer2)
 	 */
 	if (ScGetParameter(SC_PARAM_MAKE_PWELL_SIZE) == 0) sc_pwellproto = NONODEPROTO; else
 	{
-		sc_pwellproto = getpurelayernode(el_curtech, -1, LFWELL);
+		sc_pwellproto = getpurelayernode(el_curtech, -1, LFWELL|LFPTYPE);
 		if (sc_pwellproto == NONODEPROTO) return(Sc_seterrmsg(SC_NO_LAYER_PWELL));
 	}
+	if (ScGetParameter(SC_PARAM_MAKE_NWELL_SIZE) == 0) sc_nwellproto = NONODEPROTO; else
+	{
+		sc_nwellproto = getpurelayernode(el_curtech, -1, LFWELL|LFNTYPE);
+		if (sc_nwellproto == NONODEPROTO) return(Sc_seterrmsg(SC_NO_LAYER_PWELL));
+	}
 
 	return(SC_NOERROR);
 }
@@ -1264,14 +1313,21 @@ char  *Sc_create_leaf_instance(char *name, char *type, int minx, int maxx, int m
 	int maxy, int transpose, int rotation, char *parent)
 {
 	NODEINST	*binst;
+	REGISTER VARIABLE *var;
 
 	binst = newnodeinst((NODEPROTO *)type, minx, maxx, miny, maxy, (INTSML)transpose,
 		(INTSML)rotation, (NODEPROTO *)parent);
 	if (binst == NONODEINST) return((char *)NULL);
 
-	if (setvalkey((INTBIG)binst, VNODEINST, el_node_name, (INTBIG)name,
-		VSTRING|VDISPLAY) == NOVARIABLE)
-			ttyputmsg(_("ERROR creating name of instance '%s'."), name);
+	var = setvalkey((INTBIG)binst, VNODEINST, el_node_name, (INTBIG)name, VSTRING|VDISPLAY);
+	if (var == NOVARIABLE)
+	{
+		ttyputmsg(_("ERROR creating name of instance '%s'."), name);
+	} else
+	{
+		var->textdescript = (var->textdescript & ~VTSIZE) |
+			(defaulttextsize(3) << VTSIZESH);
+	}
 	return((char *)binst);
 }
 
@@ -1415,6 +1471,43 @@ char  *Sc_create_pwell(int xpos, int ypos, int xsize, int ysize, char *bfacet)
 	return((char *)binst);
 }
 
+/***********************************************************************
+Module:  Sc_create_nwell
+------------------------------------------------------------------------
+Description:
+	Create a nwell in the indicated facet at the given position.
+	Added 2/11/01 David_Harris@hmc.edu
+------------------------------------------------------------------------
+Calling Sequence:  binst = Sc_create_nwell(xpos, ypos, xsize, ysize, bfacet);
+
+Name		Type		Description
+----		----		-----------
+xpos		int			Center X position.
+ypos		int			Center Y position.
+xsize		int			Size in X.
+ysize		int			Size in Y.
+bfacet		*char		Facet in which to create feed.
+binst		*char		Returned pointer to created instance.
+------------------------------------------------------------------------
+*/
+
+char  *Sc_create_nwell(int xpos, int ypos, int xsize, int ysize, char *bfacet)
+{
+	NODEINST	*binst;
+	int		lowx, highx, lowy, highy;
+
+	if (sc_nwellproto == NONODEPROTO) return((char *)NULL);
+
+	lowx = xpos - (xsize >> 1);
+	highx = lowx + xsize;
+	lowy = ypos - (ysize >> 1);
+	highy = lowy + ysize;
+	binst = newnodeinst(sc_nwellproto, lowx, highx, lowy, highy, 0, 0,
+		(NODEPROTO *)bfacet);
+	if (binst == NONODEINST) return((char *)NULL);
+	return((char *)binst);
+}
+
 /***********************************************************************
 Module:  Sc_create_track_layer1
 ------------------------------------------------------------------------
@@ -1556,30 +1649,28 @@ void Sc_create_connection(NODEINST **ni, PORTPROTO **pp, INTBIG x, INTBIG y,
 	REGISTER ARCINST *zeroarc;
 
 	niarclist = (*pp)->connects;
-	for(via = el_curtech->firstnodeproto; via != NONODEPROTO;
-		via = via->nextnodeproto)
-	{
-		fun = (via->userbits&NFUNCTION) >> NFUNCTIONSH;
-		if (fun != NPCONTACT && fun != NPCONNECT) continue;
-		arclist = via->firstportproto->connects;
 
-		/* make sure that this contact connects to the desired arc */
-		for(i=0; arclist[i] != NOARCPROTO; i++)
-			if (arclist[i] == arc) break;
-		if (arclist[i] == NOARCPROTO) continue;
-
-		/* make sure that this contact connects to the initial node */
-		for(i=0; arclist[i] != NOARCPROTO; i++)
-		{
-			for(j=0; niarclist[j] != NOARCPROTO; j++)
-			{
-				if (arclist[i] == niarclist[j]) break;
-			}
-			if (niarclist[j] != NOARCPROTO) break;
-		}
-		if (arclist[i] != NOARCPROTO) break;
-	}
+	/* always use the standard via (David Harris) */
+	via = sc_viaproto;
 	if (via == NONODEPROTO) return;
+	fun = (via->userbits&NFUNCTION) >> NFUNCTIONSH;
+	if (fun != NPCONTACT && fun != NPCONNECT) return;
+	arclist = via->firstportproto->connects;
+
+	/* make sure that this contact connects to the desired arc */
+	for(i=0; arclist[i] != NOARCPROTO; i++)
+		if (arclist[i] == arc) break;
+	if (arclist[i] == NOARCPROTO) return;
+
+	/* make sure that this contact connects to the initial node */
+	for(i=0; arclist[i] != NOARCPROTO; i++)
+	{
+		for(j=0; niarclist[j] != NOARCPROTO; j++)
+		{
+			if (arclist[i] == niarclist[j]) break;
+		}
+		if (niarclist[j] != NOARCPROTO) break;
+	}
 
 	/* use this via to make the connection */
 	defaultnodesize(via, &sx, &sy);
@@ -1642,31 +1733,77 @@ void Sc_schematic(void)
 
 /****************************** DIALOG ******************************/
 
-/*
- * special case for the "Silicon Compiler Options" dialog
- * Vertical arc list           =  3 (popup)
- * Horizontal arc list         =  4 (popup)
- * Horizontal arc width        =  8 (edit text)
- * Vertical arc width          = 10 (edit text)
- * Power arc width             = 12 (edit text)
- * Main power arc width        = 14 (edit text)
- * P-Well height               = 16 (edit text)
- * P-Well offset               = 18 (edit text)
- * Via size                    = 20 (edit text)
- * Minimum spacing             = 22 (edit text)
- * Routing feed-through size   = 24 (edit text)
- * Routing minimum port dist   = 26 (edit text)
- * Routing minimum active dist = 28 (edit text)
- * Number of rows of cells     = 30 (edit text)
- */
+/* Silicon Compiler Options */
+DIALOGITEM sc_optionsdialogitems[] =
+{
+ /*  1 */ {0, {500,268,524,326}, BUTTON, N_("OK")},
+ /*  2 */ {0, {500,36,524,94}, BUTTON, N_("Cancel")},
+ /*  3 */ {0, {60,208,76,388}, POPUP, ""},
+ /*  4 */ {0, {8,208,24,388}, POPUP, ""},
+ /*  5 */ {0, {8,8,24,180}, MESSAGE, N_("Horizontal routing arc:")},
+ /*  6 */ {0, {60,8,76,180}, MESSAGE, N_("Vertical routing arc:")},
+ /*  7 */ {0, {32,8,48,174}, MESSAGE, N_("Horizontal wire width:")},
+ /*  8 */ {0, {32,208,48,282}, EDITTEXT, "2.5"},
+ /*  9 */ {0, {84,8,100,174}, MESSAGE, N_("Vertical wire width:")},
+ /* 10 */ {0, {84,208,100,282}, EDITTEXT, "2.5"},
+ /* 11 */ {0, {120,8,136,174}, MESSAGE, N_("Power wire width:")},
+ /* 12 */ {0, {120,208,136,282}, EDITTEXT, "2.5"},
+ /* 13 */ {0, {144,8,160,174}, MESSAGE, N_("Main power wire width:")},
+ /* 14 */ {0, {144,208,160,282}, EDITTEXT, "2.5"},
+ /* 15 */ {0, {204,8,220,199}, MESSAGE, N_("P-Well height (0 for none):")},
+ /* 16 */ {0, {204,204,220,278}, EDITTEXT, "2.5"},
+ /* 17 */ {0, {228,8,244,199}, MESSAGE, N_("P-Well offset from bottom:")},
+ /* 18 */ {0, {228,204,244,278}, EDITTEXT, "2.5"},
+ /* 19 */ {0, {320,8,336,199}, MESSAGE, N_("Via size:")},
+ /* 20 */ {0, {320,204,336,278}, EDITTEXT, "2.5"},
+ /* 21 */ {0, {344,8,360,199}, MESSAGE, N_("Minimum metal spacing:")},
+ /* 22 */ {0, {344,204,360,278}, EDITTEXT, "2.5"},
+ /* 23 */ {0, {384,8,400,199}, MESSAGE, N_("Routing: feed-through size:")},
+ /* 24 */ {0, {384,204,400,278}, EDITTEXT, "2.5"},
+ /* 25 */ {0, {408,8,424,199}, MESSAGE, N_("Routing: min. port distance:")},
+ /* 26 */ {0, {408,204,424,278}, EDITTEXT, "2.5"},
+ /* 27 */ {0, {432,8,448,199}, MESSAGE, N_("Routing: min. active dist.:")},
+ /* 28 */ {0, {432,204,448,278}, EDITTEXT, "2.5"},
+ /* 29 */ {0, {472,8,488,199}, MESSAGE, N_("Number of rows of cells:")},
+ /* 30 */ {0, {472,204,488,278}, EDITTEXT, "5"},
+ /* 31 */ {0, {168,208,184,356}, POPUP, ""},
+ /* 32 */ {0, {168,8,184,180}, MESSAGE, N_("Main power arc:")},
+ /* 33 */ {0, {256,8,272,199}, MESSAGE, N_("N-Well height (0 for none):")},
+ /* 34 */ {0, {256,204,272,278}, EDITTEXT, "2.5"},
+ /* 35 */ {0, {280,8,296,199}, MESSAGE, N_("N-Well offset from top:")},
+ /* 36 */ {0, {280,204,296,278}, EDITTEXT, "2.5"}
+};
+DIALOG sc_optionsdialog = {{50,75,583,472}, N_("Silicon Compiler Options"), 0, 36, sc_optionsdialogitems};
+
+/* special items for the "Silicon Compiler Options" dialog: */
+#define DSCO_VERTARCS      3		/* Vertical arc list (popup) */
+#define DSCO_HORIZARCS     4		/* Horizontal arc list (popup) */
+#define DSCO_HORIZWIDTH    8		/* Horizontal arc width (edit text) */
+#define DSCO_VERTWIDTH    10		/* Vertical arc width (edit text) */
+#define DSCO_POWERWIDTH   12		/* Power arc width (edit text) */
+#define DSCO_MPOWERWIDTH  14		/* Main power arc width (edit text) */
+#define DSCO_PWELLHEIGHT  16		/* P-Well height (edit text) */
+#define DSCO_PWELLOFFSET  18		/* P-Well offset (edit text) */
+#define DSCO_VIASIZE      20		/* Via size (edit text) */
+#define DSCO_MINSPACING   22		/* Minimum spacing (edit text) */
+#define DSCO_FEEDTHRU     24		/* Routing feed-through size (edit text) */
+#define DSCO_PORTDIST     26		/* Routing minimum port dist (edit text) */
+#define DSCO_ACTIVEDIST   28		/* Routing minimum active dist (edit text) */
+#define DSCO_NUMROWS      30		/* Number of rows of cells (edit text) */
+#define DSCO_MPOWERARC    31		/* Main power arc (popup) */
+#define DSCO_NWELLHEIGHT  34		/* N-Well height (edit text) */
+#define DSCO_NWELLOFFSET  36		/* N-Well offset (edit text) */
+
 void sc_optionsdlog(void)
 {
 	INTBIG itemHit, numarcnames, horizindex, vertindex, i, l1width, l2width,
-		pwrwidth, mainpwrwidth, pwellsize, pwelloffset, viasize, minspacing,
-		feedthrusize, portxmindist, activedist, numrows;
+		pwrwidth, mainpwrwidth, pwellsize, pwelloffset, nwellsize, nwelloffset,
+		viasize, minspacing, feedthrusize, portxmindist, activedist, numrows,
+		mainpwrrail;
 	extern AIDENTRY *ro_aid;
 	REGISTER ARCPROTO *ap, *aphoriz, *apvert;
-	char **arcnames, numstring[20];
+	char **arcnames, numstring[20], *transnames[2];
+	static char *powerarcnames[2] = {N_("Horizontal Arc"), N_("Vertical Arc")};
 
 	/* get parameters */
 	aphoriz      = (ARCPROTO *)ScGetParameter(SC_PARAM_MAKE_HORIZ_ARC);
@@ -1675,8 +1812,11 @@ void sc_optionsdlog(void)
 	l2width      = ScGetParameter(SC_PARAM_MAKE_L2_WIDTH);
 	pwrwidth     = ScGetParameter(SC_PARAM_MAKE_PWR_WIDTH);
 	mainpwrwidth = ScGetParameter(SC_PARAM_MAKE_MAIN_PWR_WIDTH);
+	mainpwrrail  = ScGetParameter(SC_PARAM_MAKE_MAIN_PWR_RAIL);
 	pwellsize    = ScGetParameter(SC_PARAM_MAKE_PWELL_SIZE);
 	pwelloffset  = ScGetParameter(SC_PARAM_MAKE_PWELL_OFFSET);
+	nwellsize    = ScGetParameter(SC_PARAM_MAKE_NWELL_SIZE);
+	nwelloffset  = ScGetParameter(SC_PARAM_MAKE_NWELL_OFFSET);
 	viasize      = ScGetParameter(SC_PARAM_MAKE_VIA_SIZE);
 	minspacing   = ScGetParameter(SC_PARAM_MAKE_MIN_SPACING);
 	feedthrusize = ScGetParameter(SC_PARAM_ROUTE_FEEDTHRU_SIZE);
@@ -1704,23 +1844,28 @@ void sc_optionsdlog(void)
 
 	/* display the silicon compiler options dialog box */
 	if (DiaInitDialog(&sc_optionsdialog) != 0) return;
-	DiaSetPopup(3, numarcnames, arcnames);
-	DiaSetPopup(4, numarcnames, arcnames);
-	DiaSetPopupEntry(3, vertindex);
-	DiaSetPopupEntry(4, horizindex);
-	DiaSetText(8, latoa(l1width));
-	DiaSetText(10, latoa(l2width));
-	DiaSetText(12, latoa(pwrwidth));
-	DiaSetText(14, latoa(mainpwrwidth));
-	DiaSetText(16, latoa(pwellsize));
-	DiaSetText(18, latoa(pwelloffset));
-	DiaSetText(20, latoa(viasize));
-	DiaSetText(22, latoa(minspacing));
-	DiaSetText(24, latoa(feedthrusize));
-	DiaSetText(26, latoa(portxmindist));
-	DiaSetText(28, latoa(activedist));
+	for(i=0; i<2; i++) transnames[i] = _(powerarcnames[i]);
+	DiaSetPopup(DSCO_MPOWERARC, 2, transnames);
+	DiaSetPopup(DSCO_VERTARCS, numarcnames, arcnames);
+	DiaSetPopup(DSCO_HORIZARCS, numarcnames, arcnames);
+	DiaSetPopupEntry(DSCO_VERTARCS, vertindex);
+	DiaSetPopupEntry(DSCO_HORIZARCS, horizindex);
+	DiaSetText(DSCO_HORIZWIDTH, latoa(l1width));
+	DiaSetText(DSCO_VERTWIDTH, latoa(l2width));
+	DiaSetText(DSCO_POWERWIDTH, latoa(pwrwidth));
+	DiaSetText(DSCO_MPOWERWIDTH, latoa(mainpwrwidth));
+	DiaSetText(DSCO_PWELLHEIGHT, latoa(pwellsize));
+	DiaSetText(DSCO_PWELLOFFSET, latoa(pwelloffset));
+	DiaSetText(DSCO_VIASIZE, latoa(viasize));
+	DiaSetText(DSCO_MINSPACING, latoa(minspacing));
+	DiaSetText(DSCO_FEEDTHRU, latoa(feedthrusize));
+	DiaSetText(DSCO_PORTDIST, latoa(portxmindist));
+	DiaSetText(DSCO_ACTIVEDIST, latoa(activedist));
 	sprintf(numstring, "%ld", numrows);
-	DiaSetText(30, numstring);
+	DiaSetText(DSCO_NUMROWS, numstring);
+	DiaSetPopupEntry(DSCO_MPOWERARC, mainpwrrail);
+	DiaSetText(DSCO_NWELLHEIGHT, latoa(nwellsize));
+	DiaSetText(DSCO_NWELLOFFSET, latoa(nwelloffset));
 
 	/* loop until done */
 	for(;;)
@@ -1731,39 +1876,45 @@ void sc_optionsdlog(void)
 
 	if (itemHit != CANCEL)
 	{
-		i = DiaGetPopupEntry(3);
+		i = DiaGetPopupEntry(DSCO_VERTARCS);
 		if (i == 0) ap = NOARCPROTO; else
 			ap = getarcproto(arcnames[i]);
 		if (ap != apvert) ScSetParameter(SC_PARAM_MAKE_VERT_ARC, (INTBIG)ap);
-		i = DiaGetPopupEntry(4);
+		i = DiaGetPopupEntry(DSCO_HORIZARCS);
 		if (i == 0) ap = NOARCPROTO; else
 			ap = getarcproto(arcnames[i]);
 		if (ap != aphoriz) ScSetParameter(SC_PARAM_MAKE_HORIZ_ARC, (INTBIG)ap);
 
-		i = atola(DiaGetText(8));
+		i = atola(DiaGetText(DSCO_HORIZWIDTH));
 		if (i != l1width) ScSetParameter(SC_PARAM_MAKE_L1_WIDTH, i);
-		i = atola(DiaGetText(10));
+		i = atola(DiaGetText(DSCO_VERTWIDTH));
 		if (i != l2width) ScSetParameter(SC_PARAM_MAKE_L2_WIDTH, i);
-		i = atola(DiaGetText(12));
+		i = atola(DiaGetText(DSCO_POWERWIDTH));
 		if (i != pwrwidth) ScSetParameter(SC_PARAM_MAKE_PWR_WIDTH, i);
-		i = atola(DiaGetText(14));
+		i = atola(DiaGetText(DSCO_MPOWERWIDTH));
 		if (i != mainpwrwidth) ScSetParameter(SC_PARAM_MAKE_MAIN_PWR_WIDTH, i);
-		i = atola(DiaGetText(16));
+		i = atola(DiaGetText(DSCO_PWELLHEIGHT));
 		if (i != pwellsize) ScSetParameter(SC_PARAM_MAKE_PWELL_SIZE, i);
-		i = atola(DiaGetText(18));
+		i = atola(DiaGetText(DSCO_PWELLOFFSET));
 		if (i != pwelloffset) ScSetParameter(SC_PARAM_MAKE_PWELL_OFFSET, i);
-		i = atola(DiaGetText(20));
+		i = atola(DiaGetText(DSCO_VIASIZE));
 		if (i != viasize) ScSetParameter(SC_PARAM_MAKE_VIA_SIZE, i);
-		i = atola(DiaGetText(22));
+		i = atola(DiaGetText(DSCO_MINSPACING));
 		if (i != minspacing) ScSetParameter(SC_PARAM_MAKE_MIN_SPACING, i);
-		i = atola(DiaGetText(24));
+		i = atola(DiaGetText(DSCO_FEEDTHRU));
 		if (i != feedthrusize) ScSetParameter(SC_PARAM_ROUTE_FEEDTHRU_SIZE, i);
-		i = atola(DiaGetText(26));
+		i = atola(DiaGetText(DSCO_PORTDIST));
 		if (i != portxmindist) ScSetParameter(SC_PARAM_ROUTE_PORT_X_MIN_DIST, i);
-		i = atola(DiaGetText(28));
+		i = atola(DiaGetText(DSCO_ACTIVEDIST));
 		if (i != activedist) ScSetParameter(SC_PARAM_ROUTE_ACTIVE_DIST, i);
-		i = atoi(DiaGetText(30));
+		i = atoi(DiaGetText(DSCO_NUMROWS));
 		if (i != numrows) ScSetParameter(SC_PARAM_PLACE_NUM_ROWS, i);
+		i = DiaGetPopupEntry(DSCO_MPOWERARC);
+		if (i != mainpwrrail) ScSetParameter(SC_PARAM_MAKE_MAIN_PWR_RAIL, i);
+		i = atola(DiaGetText(DSCO_NWELLHEIGHT));
+		if (i != nwellsize) ScSetParameter(SC_PARAM_MAKE_NWELL_SIZE, i);
+		i = atola(DiaGetText(DSCO_NWELLOFFSET));
+		if (i != nwelloffset) ScSetParameter(SC_PARAM_MAKE_NWELL_OFFSET, i);
 	}
 	efree((char *)arcnames);
 	DiaDoneDialog();
