@@ -439,13 +439,13 @@ void Sc_extract_print_nodes(SCCELL *vars)
 	i = 0;
 	if (vars->ground)
 	{
-		ttyputmsg(_("Node %d  %s:"), i, vars->ground->name);
+		ttyputmsg(M_("Node %d  %s:"), i, vars->ground->name);
 		for (plist = vars->ground->firstport; plist; plist = plist->next)
 		{
 			switch (plist->node->type)
 			{
 				case SCSPECIALCELL:
-					portname = _("Special");
+					portname = M_("Special");
 					break;
 				case SCCOMPLEXCELL:
 					portname = ((SCPORT *)(plist->port->port))->name;
@@ -454,7 +454,7 @@ void Sc_extract_print_nodes(SCCELL *vars)
 					portname = Sc_leaf_port_name(plist->port->port);
 					break;
 				default:
-					portname = _("Unknown");
+					portname = M_("Unknown");
 					break;
 			}
 			ttyputmsg("    %-20s    %s", plist->node->name, portname);
@@ -464,13 +464,13 @@ void Sc_extract_print_nodes(SCCELL *vars)
 
 	if (vars->power)
 	{
-		ttyputmsg(_("Node %d  %s:"), i, vars->power->name);
+		ttyputmsg(M_("Node %d  %s:"), i, vars->power->name);
 		for (plist = vars->power->firstport; plist; plist = plist->next)
 		{
 			switch (plist->node->type)
 			{
 				case SCSPECIALCELL:
-					portname = _("Special");
+					portname = M_("Special");
 					break;
 				case SCCOMPLEXCELL:
 					portname = ((SCPORT *)(plist->port->port))->name;
@@ -479,7 +479,7 @@ void Sc_extract_print_nodes(SCCELL *vars)
 					portname = Sc_leaf_port_name(plist->port->port);
 					break;
 				default:
-					portname = _("Unknown");
+					portname = M_("Unknown");
 					break;
 			}
 			ttyputmsg("    %-20s    %s", plist->node->name, portname);
@@ -489,13 +489,13 @@ void Sc_extract_print_nodes(SCCELL *vars)
 
 	for (simnode = vars->ex_nodes; simnode; simnode = simnode->next)
 	{
-		ttyputmsg(_("Node %d  %s:"), i, simnode->name);
+		ttyputmsg(M_("Node %d  %s:"), i, simnode->name);
 		for (plist = simnode->firstport; plist; plist = plist->next)
 		{
 			switch (plist->node->type)
 			{
 				case SCSPECIALCELL:
-					portname = _("Special");
+					portname = M_("Special");
 					break;
 				case SCCOMPLEXCELL:
 					portname = ((SCPORT *)(plist->port->port))->name;
@@ -504,7 +504,7 @@ void Sc_extract_print_nodes(SCCELL *vars)
 					portname = Sc_leaf_port_name(plist->port->port);
 					break;
 				default:
-					portname = _("Unknown");
+					portname = M_("Unknown");
 					break;
 			}
 			ttyputmsg("    %-20s    %s", plist->node->name, portname);
