@@ -31,6 +31,8 @@
 #include "config.h"
 #if SCAID
 
+#define MAINPWRLAYER2 1
+
 #include "global.h"
 #include "sc1.h"
 #include <math.h>
@@ -39,14 +41,14 @@
 
 static char *sc_makerblerb[] =
 {
-	N_("*********** QUISC MAKER MODULE - VERSION 1.03"),
+	M_("*********** QUISC MAKER MODULE - VERSION 1.03"),
 	" ",
-	N_("    o  Determination of final position"),
-	N_("    o  Include squeezing rows in vertical direction"),
-	N_("    o  Squeeze tracks together if nonadjacent via"),
-	N_("    o  Creating ties to power and ground"),
-	N_("    o  Routing Power and Ground buses"),
-	N_("    o  Creation in VLSI Layout Tool's database"),
+	M_("    o  Determination of final position"),
+	M_("    o  Include squeezing rows in vertical direction"),
+	M_("    o  Squeeze tracks together if nonadjacent via"),
+	M_("    o  Creating ties to power and ground"),
+	M_("    o  Routing Power and Ground buses"),
+	M_("    o  Creation in VLSI Layout Tool's database"),
 	"----------------------------------------------------------",
 	0
 } ;
@@ -156,7 +158,7 @@ int Sc_maker(int count, char *pars[])
 
 	Sc_free_maker_data(make_data);
 
-	ttyputmsg(_("Done (time = %s seconds)"), Sc_cpu_time(TIME_ABS));
+	ttyputmsg(_("Done (time = %s)"), Sc_cpu_time(TIME_ABS));
 
 	return(SC_NOERROR);
 }
@@ -217,6 +219,12 @@ int Sc_maker_set_control(int count, char *pars[])
 					ScSetParameter(SC_PARAM_MAKE_MAIN_PWR_WIDTH, atoi(pars[numcmd]));
 				continue;
 			}
+			if (namesamen(pars[numcmd], "main-power-rail", n) == 0)
+			{
+				if (++numcmd < count)
+					ScSetParameter(SC_PARAM_MAKE_MAIN_PWR_RAIL, atoi(pars[numcmd]));
+				continue;
+			}
 			if (namesamen(pars[numcmd], "via-size", n) == 0)
 			{
 				if (++numcmd < count)
@@ -240,8 +248,11 @@ int Sc_maker_set_control(int count, char *pars[])
 				ScSetParameter(SC_PARAM_MAKE_L1_WIDTH, DEFAULT_L1_TRACK_WIDTH);
 				ScSetParameter(SC_PARAM_MAKE_PWR_WIDTH, DEFAULT_POWER_TRACK_WIDTH);
 				ScSetParameter(SC_PARAM_MAKE_MAIN_PWR_WIDTH, DEFAULT_MAIN_POWER_WIDTH);
+				ScSetParameter(SC_PARAM_MAKE_MAIN_PWR_RAIL, DEFAULT_MAIN_POWER_RAIL);
 				ScSetParameter(SC_PARAM_MAKE_PWELL_SIZE, DEFAULT_PWELL_SIZE);
 				ScSetParameter(SC_PARAM_MAKE_PWELL_OFFSET, DEFAULT_PWELL_OFFSET);
+				ScSetParameter(SC_PARAM_MAKE_NWELL_SIZE, DEFAULT_NWELL_SIZE);
+				ScSetParameter(SC_PARAM_MAKE_NWELL_OFFSET, DEFAULT_NWELL_OFFSET);
 				continue;
 			}
 			n = maxi(l, 5);
@@ -281,6 +292,18 @@ int Sc_maker_set_control(int count, char *pars[])
 					ScSetParameter(SC_PARAM_MAKE_PWELL_OFFSET, atoi(pars[numcmd]));
 				continue;
 			}
+			if (namesamen(pars[numcmd], "n-well-size", n) == 0)
+			{
+				if (++numcmd < count)
+					ScSetParameter(SC_PARAM_MAKE_NWELL_SIZE, atoi(pars[numcmd]));
+				continue;
+			}
+			if (namesamen(pars[numcmd], "n-well-offset", n) == 0)
+			{
+				if (++numcmd < count)
+					ScSetParameter(SC_PARAM_MAKE_NWELL_OFFSET, atoi(pars[numcmd]));
+				continue;
+			}
 			return(Sc_seterrmsg(SC_MAKE_SET_XCMD, pars[numcmd]));
 		}
 	} else
@@ -303,31 +326,38 @@ Calling Sequence:  Sc_maker_show_control();
 void Sc_maker_show_control(void)
 {
 	char *str1;
+	static char *hv[3] = {N_("Horizontal"), N_("Vertical")};
 
-	ttyputverbose(_("************ MAKER CONTROL STRUCTURE"));
-	if (sc_makerverbose) str1 = _("TRUE"); else
-		str1 = _("FALSE");
-	ttyputverbose(_("Verbose output           =  %s"), str1);
-	ttyputverbose(_("Minimum metal spacing    =  %d"),
+	ttyputverbose(M_("************ MAKER CONTROL STRUCTURE"));
+	if (sc_makerverbose) str1 = M_("TRUE"); else
+		str1 = M_("FALSE");
+	ttyputverbose(M_("Verbose output           =  %s"), str1);
+	ttyputverbose(M_("Minimum metal spacing    =  %d"),
 		ScGetParameter(SC_PARAM_MAKE_MIN_SPACING));
-	ttyputverbose(_("Via Size                 =  %d"),
+	ttyputverbose(M_("Via Size                 =  %d"),
 		ScGetParameter(SC_PARAM_MAKE_VIA_SIZE));
-	ttyputverbose(_("Horizontal (layer 1) arc =  '%s'"),
+	ttyputverbose(M_("Horizontal (layer 1) arc =  '%s'"),
 		describearcproto((ARCPROTO *)ScGetParameter(SC_PARAM_MAKE_HORIZ_ARC)));
-	ttyputverbose(_("Vertical (layer 2) arc   =  '%s'"),
+	ttyputverbose(M_("Vertical (layer 2) arc   =  '%s'"),
 		describearcproto((ARCPROTO *)ScGetParameter(SC_PARAM_MAKE_VERT_ARC)));
-	ttyputverbose(_("Layer 1 Track width      =  %d"),
+	ttyputverbose(M_("Layer 1 Track width      =  %d"),
 		ScGetParameter(SC_PARAM_MAKE_L1_WIDTH));
-	ttyputverbose(_("Layer 2 Track width      =  %d"),
+	ttyputverbose(M_("Layer 2 Track width      =  %d"),
 		ScGetParameter(SC_PARAM_MAKE_L2_WIDTH));
-	ttyputverbose(_("Power Track width        =  %d"),
+	ttyputverbose(M_("Power Track width        =  %d"),
 		ScGetParameter(SC_PARAM_MAKE_PWR_WIDTH));
-	ttyputverbose(_("Main Power Bus width     =  %d"),
+	ttyputverbose(M_("Main Power Bus width     =  %d"),
 		ScGetParameter(SC_PARAM_MAKE_MAIN_PWR_WIDTH));
-	ttyputverbose(_("P-well size (0 for none) =  %d"),
+	ttyputverbose(M_("Main Power Bus rail      =  %s"),
+		hv[ScGetParameter(SC_PARAM_MAKE_MAIN_PWR_RAIL)]);
+	ttyputverbose(M_("P-well size (0 for none) =  %d"),
 		ScGetParameter(SC_PARAM_MAKE_PWELL_SIZE));
-	ttyputverbose(_("P-well offset (bottom)   =  %d"),
+	ttyputverbose(M_("P-well offset (bottom)   =  %d"),
 		ScGetParameter(SC_PARAM_MAKE_PWELL_OFFSET));
+	ttyputverbose(M_("N-well size (0 for none) =  %d"),
+		ScGetParameter(SC_PARAM_MAKE_NWELL_SIZE));
+	ttyputverbose(M_("N-well offset (bottom)   =  %d"),
+		ScGetParameter(SC_PARAM_MAKE_NWELL_OFFSET));
 }
 
 /***********************************************************************
@@ -370,8 +400,8 @@ int Sc_maker_create_info(SCCELL *cell)
 		ScGetParameter(SC_PARAM_MAKE_MIN_SPACING);
 	row_to_track = (ScGetParameter(SC_PARAM_MAKE_VIA_SIZE) >> 1) +
 		ScGetParameter(SC_PARAM_MAKE_MIN_SPACING);
-	min_xpos = HUGEINT;
-	max_xpos = -HUGEINT;
+	min_xpos = MAXINTBIG;
+	max_xpos = -MAXINTBIG;
 	ysize = 0;
 	num_leaf_cells = 0;
 	num_feeds = 0;
@@ -435,25 +465,25 @@ int Sc_maker_create_info(SCCELL *cell)
 	info->num_tracks = num_tracks;
 	info->track_length = track_length;
 
-	ttyputmsg(_("************ MAKER INFORMATION"));
-	ttyputmsg(_("Size:"));
-	ttyputmsg(_("    X size             = %10d  (microns)"), info->x_size / 2000);
-	ttyputmsg(_("    Y size             = %10d  (microns)"), info->y_size / 2000);
-	ttyputmsg(_("    Area               = %10d  (square microns)"), info->area);
+	ttyputmsg(M_("************ MAKER INFORMATION"));
+	ttyputmsg(M_("Size:"));
+	ttyputmsg(M_("    X size             = %10d  (microns)"), info->x_size / 2000);
+	ttyputmsg(M_("    Y size             = %10d  (microns)"), info->y_size / 2000);
+	ttyputmsg(M_("    Area               = %10d  (square microns)"), info->area);
 	if (sc_makerverbose)
 	{
-		ttyputmsg(_("Limits (in microns):"));
-		ttyputmsg(_("    Minimum X position =  %d"), info->min_x / 2000);
-		ttyputmsg(_("    Maximim X position =  %d"), info->max_x / 2000);
-		ttyputmsg(_("    Minimum Y position =  %d"), info->min_y / 2000);
-		ttyputmsg(_("    Maximim Y position =  %d"), info->max_y / 2000);
-		ttyputmsg(_("Numbers of Elements:"));
-		ttyputmsg(_("    Base Cells         =  %d"), info->num_leaf_cells);
-		ttyputmsg(_("    Feed Throughs      =  %d"), info->num_feeds);
-		ttyputmsg(_("    Rows of cells      =  %d"), info->num_rows);
-		ttyputmsg(_("    Routing Channels   =  %d"), info->num_channels);
-		ttyputmsg(_("    Routing Tracks     =  %d"), info->num_tracks);
-		ttyputmsg(_("    Total Track Length =  %d  (microns)"),
+		ttyputmsg(M_("Limits (in microns):"));
+		ttyputmsg(M_("    Minimum X position =  %d"), info->min_x / 2000);
+		ttyputmsg(M_("    Maximim X position =  %d"), info->max_x / 2000);
+		ttyputmsg(M_("    Minimum Y position =  %d"), info->min_y / 2000);
+		ttyputmsg(M_("    Maximim Y position =  %d"), info->max_y / 2000);
+		ttyputmsg(M_("Numbers of Elements:"));
+		ttyputmsg(M_("    Base Cells         =  %d"), info->num_leaf_cells);
+		ttyputmsg(M_("    Feed Throughs      =  %d"), info->num_feeds);
+		ttyputmsg(M_("    Rows of cells      =  %d"), info->num_rows);
+		ttyputmsg(M_("    Routing Channels   =  %d"), info->num_channels);
+		ttyputmsg(M_("    Routing Tracks     =  %d"), info->num_tracks);
+		ttyputmsg(M_("    Total Track Length =  %d  (microns)"),
 			info->track_length / 2000);
 	}
 
@@ -512,10 +542,10 @@ int Sc_maker_set_up(SCCELL *cell, SCMAKERDATA **make_data)
 	data->channels = NULL;
 	data->power = NULL;
 	data->ground = NULL;
-	data->minx = HUGEINT;
-	data->maxx = -HUGEINT;
-	data->miny = HUGEINT;
-	data->maxy = -HUGEINT;
+	data->minx = MAXINTBIG;
+	data->maxx = -MAXINTBIG;
+	data->miny = MAXINTBIG;
+	data->maxy = -MAXINTBIG;
 
 	/* create Maker Channel and Track data structures */
 	row_to_track = (ScGetParameter(SC_PARAM_MAKE_VIA_SIZE) >> 1) +
@@ -631,10 +661,10 @@ int Sc_maker_set_up(SCCELL *cell, SCMAKERDATA **make_data)
 			return(Sc_seterrmsg(SC_NOMEMORY));
 		mrow->number = row->row_num;
 		mrow->members = NULL;
-		mrow->minx = HUGEINT;
-		mrow->maxx = -HUGEINT;
-		mrow->miny = HUGEINT;
-		mrow->maxy = -HUGEINT;
+		mrow->minx = MAXINTBIG;
+		mrow->maxx = -MAXINTBIG;
+		mrow->miny = MAXINTBIG;
+		mrow->maxy = -MAXINTBIG;
 		mrow->flags = 0;
 		mrow->next = NULL;
 		mrow->last = last_mrow;
@@ -648,8 +678,8 @@ int Sc_maker_set_up(SCCELL *cell, SCMAKERDATA **make_data)
 		last_mrow = mrow;
 
 		/* determine permissible top and bottom overlap */
-		toffset = -HUGEINT;
-		boffset = HUGEINT;
+		toffset = -MAXINTBIG;
+		boffset = MAXINTBIG;
 		for (place = row->start; place; place = place->next)
 		{
 			if (place->cell->type != SCLEAFCELL) continue;
@@ -973,7 +1003,7 @@ int Sc_maker_create_layout(SCMAKERDATA *data)
 	SCNBPLACE		*place;
 	SCROUTEPORT		*rport;
 	int				transpose, rotation, err, xpos, ypos;
-	int				xsize, ysize;
+	int				xsize, ysize, mainpwrrail;
 	int				row_to_track, track_to_track;
 
 	row_to_track = (ScGetParameter(SC_PARAM_MAKE_VIA_SIZE) >> 1) +
@@ -1262,25 +1292,45 @@ int Sc_maker_create_layout(SCMAKERDATA *data)
 	xpos = data->minx - row_to_track -
 		(ScGetParameter(SC_PARAM_MAKE_MAIN_PWR_WIDTH) >> 1);
 
+	mainpwrrail = ScGetParameter(SC_PARAM_MAKE_MAIN_PWR_RAIL);
+
 	for (plist = data->power; plist; plist = plist->next)
 	{
 		ypos = plist->ypos;
 
 		/* create main power bus node */
-		if ((binst = Sc_create_layer1_node(xpos, ypos,
-			ScGetParameter(SC_PARAM_MAKE_MAIN_PWR_WIDTH),
-			ScGetParameter(SC_PARAM_MAKE_MAIN_PWR_WIDTH), bcell)) == NULL)
+		if (mainpwrrail == 0)
 		{
-			return(Sc_seterrmsg(SC_MAKER_NOCREATE_VIA));
+			binst = Sc_create_layer1_node(xpos, ypos,
+				ScGetParameter(SC_PARAM_MAKE_MAIN_PWR_WIDTH),
+					ScGetParameter(SC_PARAM_MAKE_MAIN_PWR_WIDTH), bcell);
+		} else
+		{
+			binst = Sc_create_layer2_node(xpos, ypos,
+				ScGetParameter(SC_PARAM_MAKE_MAIN_PWR_WIDTH),
+					ScGetParameter(SC_PARAM_MAKE_MAIN_PWR_WIDTH), bcell);
 		}
+		if (binst == NULL)
+			return(Sc_seterrmsg(SC_MAKER_NOCREATE_VIA));
 		if (lastpower)
 		{
 			/* join to previous */
-			if (Sc_create_track_layer1(binst, (char *)NULL,
-				(char *)lastpower, (char *)NULL,
-				ScGetParameter(SC_PARAM_MAKE_MAIN_PWR_WIDTH), bcell) == NULL)
+			if (mainpwrrail == 0)
 			{
-				return(Sc_seterrmsg(SC_MAKER_NOCREATE_LAYER1));
+				if (Sc_create_track_layer1(binst, (char *)NULL,
+					(char *)lastpower, (char *)NULL,
+					ScGetParameter(SC_PARAM_MAKE_MAIN_PWR_WIDTH), bcell) == NULL)
+				{
+					return(Sc_seterrmsg(SC_MAKER_NOCREATE_LAYER1));
+				}
+			} else
+			{
+				if (Sc_create_track_layer2(binst, (char *)NULL,
+					(char *)lastpower, (char *)NULL,
+					ScGetParameter(SC_PARAM_MAKE_MAIN_PWR_WIDTH), bcell) == NULL)
+				{
+					return(Sc_seterrmsg(SC_MAKER_NOCREATE_LAYER1));
+				}
 			}
 		}
 		lastpower = (SCMAKERINST *)binst;
@@ -1322,20 +1372,38 @@ int Sc_maker_create_layout(SCMAKERDATA *data)
 		ypos = plist->ypos;
 
 		/* create main ground bus node */
-		if ((binst = Sc_create_layer1_node(xpos, ypos,
-			ScGetParameter(SC_PARAM_MAKE_MAIN_PWR_WIDTH),
-			ScGetParameter(SC_PARAM_MAKE_MAIN_PWR_WIDTH), bcell)) == NULL)
+		if (mainpwrrail == 0)
 		{
-			return(Sc_seterrmsg(SC_MAKER_NOCREATE_VIA));
+			binst = Sc_create_layer1_node(xpos, ypos,
+				ScGetParameter(SC_PARAM_MAKE_MAIN_PWR_WIDTH),
+					ScGetParameter(SC_PARAM_MAKE_MAIN_PWR_WIDTH), bcell);
+		} else
+		{
+			binst = Sc_create_layer2_node(xpos, ypos,
+				ScGetParameter(SC_PARAM_MAKE_MAIN_PWR_WIDTH),
+					ScGetParameter(SC_PARAM_MAKE_MAIN_PWR_WIDTH), bcell);
 		}
+		if (binst == NULL)
+			return(Sc_seterrmsg(SC_MAKER_NOCREATE_VIA));
 		if (lastground)
 		{
 			/* join to previous */
-			if (Sc_create_track_layer1(binst, (char *)NULL,
-				(char *)lastground, (char *)NULL,
-				ScGetParameter(SC_PARAM_MAKE_MAIN_PWR_WIDTH), bcell) == NULL)
+			if (mainpwrrail == 0)
 			{
-				return(Sc_seterrmsg(SC_MAKER_NOCREATE_LAYER1));
+				if (Sc_create_track_layer1(binst, (char *)NULL,
+					(char *)lastground, (char *)NULL,
+					ScGetParameter(SC_PARAM_MAKE_MAIN_PWR_WIDTH), bcell) == NULL)
+				{
+					return(Sc_seterrmsg(SC_MAKER_NOCREATE_LAYER1));
+				}
+			} else
+			{
+				if (Sc_create_track_layer2(binst, (char *)NULL,
+					(char *)lastground, (char *)NULL,
+					ScGetParameter(SC_PARAM_MAKE_MAIN_PWR_WIDTH), bcell) == NULL)
+				{
+					return(Sc_seterrmsg(SC_MAKER_NOCREATE_LAYER1));
+				}
 			}
 		} else
 		{
@@ -1402,17 +1470,30 @@ int Sc_maker_create_layout(SCMAKERDATA *data)
 			}
 			if (lastinst)
 			{
-				xpos = (firstinst->xpos + (firstinst->xsize >> 1) +
-					lastinst->xpos + (lastinst->xsize >> 1)) >> 1;
-				ypos = firstinst->ypos + ScGetParameter(SC_PARAM_MAKE_PWELL_OFFSET) +
-					(ScGetParameter(SC_PARAM_MAKE_PWELL_SIZE) >> 1);
-				xsize = (lastinst->xpos + (lastinst->xsize >> 1)) -
-					(firstinst->xpos + (firstinst->xsize >> 1));
+				xpos = (firstinst->xpos + lastinst->xpos + lastinst->xsize) >> 1;
+				xsize = (lastinst->xpos + lastinst->xsize) - firstinst->xpos;
 				ysize = ScGetParameter(SC_PARAM_MAKE_PWELL_SIZE);
-				if ((binst = Sc_create_pwell(xpos, ypos, xsize, ysize, bcell))
-					== NULL)
+				if (ysize > 0)
 				{
-					return(Sc_seterrmsg(SC_NOCREATE_PWELL));
+					ypos = firstinst->ypos + ScGetParameter(SC_PARAM_MAKE_PWELL_OFFSET) +
+						(ScGetParameter(SC_PARAM_MAKE_PWELL_SIZE) >> 1);
+					if ((binst = Sc_create_pwell(xpos, ypos, xsize, ysize, bcell))
+						== NULL)
+					{
+						return(Sc_seterrmsg(SC_NOCREATE_PWELL));
+					}
+				}
+
+				ysize = ScGetParameter(SC_PARAM_MAKE_NWELL_SIZE);
+				if (ysize > 0)
+				{
+					ypos = firstinst->ypos + firstinst->ysize - ScGetParameter(SC_PARAM_MAKE_NWELL_OFFSET) -
+						(ScGetParameter(SC_PARAM_MAKE_NWELL_SIZE) >> 1);
+					if ((binst = Sc_create_nwell(xpos, ypos, xsize, ysize, bcell))
+						== NULL)
+					{
+						return(Sc_seterrmsg(SC_NOCREATE_PWELL));
+					}
 				}
 			}
 		}
