@@ -40,17 +40,17 @@
 
 static char *sc_placeblerb[] =
 {
-	N_("************ QUISC PLACEMENT MODULE - VERSION 1.00"),
-	N_("Cluster Tree Creation:"),
-	N_("    o  Maxi-cut Algorithm"),
-	N_("    o  Minimum use for best pair choice for equal weight"),
-	N_("Cluster Tree Sorting:"),
-	N_("    o  Top-down cluster tree sort"),
-	N_("    o  Bottom-up cluster tree sort"),
-	N_("Net Balancing:"),
-	N_("    o  Independent routing channels"),
-	N_("    o  Cross refencing of trunks to improve speed"),
-	N_("    o  Effective calculation of rows in costing"),
+	M_("************ QUISC PLACEMENT MODULE - VERSION 1.00"),
+	M_("Cluster Tree Creation:"),
+	M_("    o  Maxi-cut Algorithm"),
+	M_("    o  Minimum use for best pair choice for equal weight"),
+	M_("Cluster Tree Sorting:"),
+	M_("    o  Top-down cluster tree sort"),
+	M_("    o  Bottom-up cluster tree sort"),
+	M_("Net Balancing:"),
+	M_("    o  Independent routing channels"),
+	M_("    o  Cross refencing of trunks to improve speed"),
+	M_("    o  Effective calculation of rows in costing"),
 	0
 } ;
 
@@ -215,11 +215,11 @@ int Sc_place(int count, char *pars[])
 		sc_curcell)))
 			return(err);
 	efree((char *)cllist);
-	ttyputverbose(_("    Time to create Cluster Tree  =  %s seconds."),
+	ttyputverbose(M_("    Time to create Cluster Tree  =  %s."),
 		Sc_cpu_time(TIME_REL));
 	if (sc_placecontrol.stats_flag)
 	{
-		ttyputmsg(_("************ Initial placement of Clusters"));
+		ttyputmsg(M_("************ Initial placement of Clusters"));
 		Sc_place_print_cluster_tree(sc_gcluster_tree, 0);
 	}
 	place->size_rows = place->size_inst / place->num_rows;
@@ -231,7 +231,7 @@ int Sc_place(int count, char *pars[])
 			return(err);
 		if (sc_placecontrol.stats_flag)
 		{
-			ttyputmsg(_("************ Placement of Clusters after Sorting"));
+			ttyputmsg(M_("************ Placement of Clusters after Sorting"));
 			Sc_place_print_cluster_tree(sc_gcluster_tree, 0);
 		}
 	}
@@ -256,7 +256,7 @@ int Sc_place(int count, char *pars[])
 	Sc_place_number_placement(sc_curcell->placement->rows);
 	if (sc_placecontrol.stats_flag)
 	{
-		ttyputmsg(_("************ Placement before Net Balancing"));
+		ttyputmsg(M_("************ Placement before Net Balancing"));
 		Sc_place_show_placement(sc_curcell->placement->rows);
 	}
 	Sc_free_cluster_tree(sc_gcluster_tree);
@@ -267,18 +267,18 @@ int Sc_place(int count, char *pars[])
 		if ((err = Sc_place_net_balance(sc_curcell->placement->rows,
 			sc_curcell)))
 				return(err);
-		ttyputmsg(_("    Time to do Net Balancing =  %s seconds."),
+		ttyputmsg(M_("    Time to do Net Balancing =  %s."),
 			Sc_cpu_time(TIME_REL));
 		if (sc_placecontrol.stats_flag)
 		{
-			ttyputmsg(_("************ Placement after Net Balancing"));
+			ttyputmsg(M_("************ Placement after Net Balancing"));
 			Sc_place_show_placement(sc_curcell->placement->rows);
 		}
 	}
 
 	/* print process time for placement */
 	Sc_place_reorder_rows(sc_curcell->placement->rows);
-	ttyputmsg(_("Done (time = %s seconds)"), Sc_cpu_time(TIME_ABS));
+	ttyputmsg(_("Done (time = %s)"), Sc_cpu_time(TIME_ABS));
 
 	return(SC_NOERROR);
 }
@@ -398,35 +398,35 @@ void Sc_place_show_control(void)
 {
 	char	*str1;
 
-	ttyputverbose(_("************ PLACEMENT CONTROL STRUCTURE"));
+	ttyputverbose(M_("************ PLACEMENT CONTROL STRUCTURE"));
 	if (sc_placecontrol.stats_flag)
 	{
-		str1 = _("TRUE");
+		str1 = M_("TRUE");
 	} else
 	{
-		str1 = _("FALSE");
+		str1 = M_("FALSE");
 	}
-	ttyputverbose(_("Print statistics  =  %-s"), str1);
+	ttyputverbose(M_("Print statistics  =  %-s"), str1);
 	if (sc_placecontrol.sort_flag)
 	{
-		str1 = _("TRUE");
+		str1 = M_("TRUE");
 	} else
 	{
-		str1 = _("FALSE");
+		str1 = M_("FALSE");
 	}
-	ttyputverbose(_("Sort cluster tree =  %-s"), str1);
-	ttyputverbose(_("Number of rows    =  %d"), ScGetParameter(SC_PARAM_PLACE_NUM_ROWS));
+	ttyputverbose(M_("Sort cluster tree =  %-s"), str1);
+	ttyputverbose(M_("Number of rows    =  %d"), ScGetParameter(SC_PARAM_PLACE_NUM_ROWS));
 	if (sc_placecontrol.net_balance_flag)
 	{
-		str1 = _("TRUE");
+		str1 = M_("TRUE");
 	} else
 	{
-		str1 = _("FALSE");
+		str1 = M_("FALSE");
 	}
-	ttyputverbose(_("Net balancing     =  %-s"), str1);
-	ttyputverbose(_("Net balace limit  =  %d"),
+	ttyputverbose(M_("Net balancing     =  %-s"), str1);
+	ttyputverbose(M_("Net balace limit  =  %d"),
 		sc_placecontrol.net_balance_limit);
-	ttyputverbose(_("Vertical cost X   =  %d"),
+	ttyputverbose(M_("Vertical cost X   =  %d"),
 		sc_placecontrol.vertical_cost);
 }
 
@@ -465,11 +465,11 @@ int Sc_place_create_clusters(SCCLUSTER **clusters, SCCELL *cell)
 	avg_height = height / num;
 	if (sc_placecontrol.stats_flag)
 	{
-		ttyputmsg(_("************ Cell Statistics"));
-		ttyputmsg(_("    Number of cells         = %d"), num);
-		ttyputmsg(_("    Total length            = %d"), size);
-		ttyputmsg(_("    Average size of cells   = %d"), avg_size);
-		ttyputmsg(_("    Average height of cells = %d"), avg_height);
+		ttyputmsg(M_("************ Cell Statistics"));
+		ttyputmsg(M_("    Number of cells         = %d"), num);
+		ttyputmsg(M_("    Total length            = %d"), size);
+		ttyputmsg(M_("    Average size of cells   = %d"), avg_size);
+		ttyputmsg(M_("    Average height of cells = %d"), avg_height);
 	}
 	cell->placement->num_inst = num;
 	cell->placement->size_inst = size;
@@ -1229,23 +1229,23 @@ int Sc_place_sort_cluster_tree(SCCLUSTERTREE *ctree, SCCELL *cell)
 
 	sc_currentcost = Sc_place_cost_cluster_tree(sc_gcluster_tree, trunks, cell);
 	if (sc_placecontrol.stats_flag)
-		ttyputverbose(_("***** Cost of placement before cluster sorting = %d"),
+		ttyputverbose(M_("***** Cost of placement before cluster sorting = %d"),
 			sc_currentcost);
 
 	/* call top-down swapper */
 	if ((err = Sc_place_sort_swapper_top_down(ctree, trunks, cell)))
 		return(err);
-	ttyputverbose(_("    Time for Top-Down Sort       =  %s seconds."),
+	ttyputverbose(M_("    Time for Top-Down Sort       =  %s."),
 		Sc_cpu_time(TIME_REL));
 
 	/* call bottom-up swapper */
 	if ((err = Sc_place_sort_swapper_bottom_up(ctree, trunks, cell)))
 		return(err);
-	ttyputverbose(_("    Time for Bottom_Up Sort      =  %s seconds."),
+	ttyputverbose(M_("    Time for Bottom_Up Sort      =  %s."),
 		Sc_cpu_time(TIME_REL));
 
 	if (sc_placecontrol.stats_flag)
-		ttyputverbose(_("***** Cost of placement after cluster sorting = %d"),
+		ttyputverbose(M_("***** Cost of placement after cluster sorting = %d"),
 			sc_currentcost);
 
 	for (newtrunk = trunks; newtrunk; newtrunk = nexttrunk)
@@ -1765,7 +1765,7 @@ int Sc_place_net_balance(SCROWLIST *row, SCCELL *cell)
 
 	/* report current placement evaluation */
 	if (sc_placecontrol.stats_flag)
-		ttyputmsg(_("Evaluation before Net-Balancing  = %d"),
+		ttyputmsg(M_("Evaluation before Net-Balancing  = %d"),
 			Sc_place_nb_cost(cell->placement->rows, channel, cell));
 
 	/* do the net balance for each cell */
@@ -1778,7 +1778,7 @@ int Sc_place_net_balance(SCROWLIST *row, SCCELL *cell)
 
 	/* report new evaluation */
 	if (sc_placecontrol.stats_flag)
-		ttyputmsg(_("Evaluation after Net-Balancing   = %d"),
+		ttyputmsg(M_("Evaluation after Net-Balancing   = %d"),
 			Sc_place_nb_cost(cell->placement->rows, channel, cell));
 
 	for (newchan = channel; newchan; newchan = nextnewchan)
@@ -1973,8 +1973,8 @@ int Sc_place_nb_cost(SCROWLIST *rows, SCCHANNEL *channels, SCCELL *cell)
 	{
 		for (ntrunk = nchan->trunks; ntrunk; ntrunk = ntrunk->next)
 		{
-			ntrunk->minx = HUGEINT;
-			ntrunk->maxx = -HUGEINT;
+			ntrunk->minx = MAXINTBIG;
+			ntrunk->maxx = -MAXINTBIG;
 		}
 	}
 
@@ -2023,7 +2023,7 @@ int Sc_place_nb_cost(SCROWLIST *rows, SCCHANNEL *channels, SCCELL *cell)
 				continue;
 			for (i = nchan->number; i; i--)
 				ntrunk = ntrunk->same;
-			if (ntrunk->minx == HUGEINT)
+			if (ntrunk->minx == MAXINTBIG)
 			{
 				if (!above && ntrunk->same)
 					ntrunk = ntrunk->same;
@@ -2055,7 +2055,7 @@ int Sc_place_nb_cost(SCROWLIST *rows, SCCHANNEL *channels, SCCELL *cell)
 	{
 		for (ntrunk = nchan->trunks; ntrunk; ntrunk = ntrunk->next)
 		{
-			if (ntrunk->minx != HUGEINT)
+			if (ntrunk->minx != MAXINTBIG)
 				cost += abs(ntrunk->maxx - ntrunk->minx);
 		}
 	}
@@ -2067,7 +2067,7 @@ int Sc_place_nb_cost(SCROWLIST *rows, SCCHANNEL *channels, SCCELL *cell)
 		fcount = count = 0;
 		for (strunk = ntrunk; strunk; strunk = strunk->same)
 		{
-			if (strunk->minx != HUGEINT)
+			if (strunk->minx != MAXINTBIG)
 			{
 				if (ftrunk == NULL)
 				{
@@ -2380,7 +2380,7 @@ void Sc_place_show_placement(SCROWLIST *rows)
 
 	for (row = rows; row; row = row->next)
 	{
-		ttyputmsg(_("For Row #%d, size %d:"), row->row_num, row->row_size);
+		ttyputmsg(M_("For Row #%d, size %d:"), row->row_num, row->row_size);
 		for (inst = row->start; inst != row->end;)
 		{
 			ttyputmsg("    %8d    %-s", inst->xpos, inst->cell->name);
