@@ -51,7 +51,7 @@ char      **simals_rowptr1, **simals_ioptr1, *simals_instbuf = 0;
 INTBIG      simals_loop_count, simals_pseq, simals_nseq, *simals_instptr = 0,
 			simals_ibufsize, simals_iptrsize, simals_trakfull, simals_trakptr;
 INTBIG      simals_no_update;		/* cached key for "SIM_als_no_update" */
-INTSML      simals_seed_flag = 1;
+INTSML      simals_seed_flag = 1, simals_trace_all_nodes = 0;
 float       simals_time_abs, simals_delta_def, simals_linear_def,
 			simals_exp_def, simals_random_def, simals_abs_def;
 NODEPROTO  *simals_mainproto = NONODEPROTO;
@@ -1074,7 +1074,7 @@ INTSML simals_parse_node(void)
 			ttyputerr(_("Node declaration: EOF unexpectedly found"));
 			return(1);
 		}
-		simals_ioptr2->strength = (INTSML)j;
+		simals_ioptr2->strength = (INTSML)j*2;
 	}
 	return(0);
 }
@@ -1701,7 +1701,7 @@ void simals_compute_node_name(NODEPTR nodehead, char *sp)
 	EXPTR   exhead;
 
 	cellhead = nodehead->cellptr;
-	simals_compute_path_name(cellhead, sp);
+	strcpy(sp, simals_compute_path_name(cellhead));
 
 	for (exhead = cellhead->exptr; 1; exhead = exhead->next)
 	{
@@ -1725,16 +1725,12 @@ void simals_compute_node_name(NODEPTR nodehead, char *sp)
  *	cellhead = pointer to desired level of hierarchy
  *	sp	= pointer to char string where path name is to be saved
  */
-void simals_compute_path_name(CONPTR cellhead, char *sp)
+char *simals_compute_path_name(CONPTR cellhead)
 {
-	char  s1[256];
-
-	for (s1[0] = 0; cellhead; cellhead = cellhead->parent)
-	{
-		(void)sprintf(sp,".%s", cellhead->inst_name);
-		(void)strcat(sp, s1);
-		(void)strcpy(s1, sp);
-	}
+	(void)initinfstr();
+	for ( ; cellhead; cellhead = cellhead->parent)
+		(void)formatinfstr(".%s", cellhead->inst_name);
+	return(returninfstr());
 }
 
 /*
@@ -1805,12 +1801,11 @@ INTSML simals_topofinstances(char **c)
 
 char *simals_nextinstance(void)
 {
-	char *nextname;
-
 	if (sim_als_cellptr == 0) return(0);
-	nextname = sim_als_cellptr->inst_name;
+	(void)initinfstr();
+	(void)formatinfstr("%s (%s)", sim_als_cellptr->inst_name, sim_als_cellptr->model_name);
 	sim_als_cellptr = sim_als_cellptr->next;
-	return(nextname);
+	return(returninfstr());
 }
 
 /*
