@@ -101,7 +101,7 @@ void simals_build_actel_command(INTSML count, char *par[])
 	filecount = filesindirectory(par[0], &filelist);
 	if (filecount == 0)
 	{
-		ttyputerr(_("There are no files in directory '%s'"), par[0]);
+		ttyputerr(M_("There are no files in directory '%s'"), par[0]);
 		return;
 	}
 	for(i=0; i<filecount; i++)
@@ -112,11 +112,11 @@ void simals_build_actel_command(INTSML count, char *par[])
 		(void)addstringtoinfstr(filelist[i]);
 		pt = returninfstr();
 		if (filetypeacteltab < 0)
-			filetypeacteltab = setupfiletype("tab", "*.tab", MACFSTAG('TEXT'), 0, "actel", _("QuickPartTable"));
+			filetypeacteltab = setupfiletype("tab", "*.tab", MACFSTAG('TEXT'), 0, "actel", M_("QuickPartTable"));
 		io = xopen(pt, filetypeacteltab, "", &filename);
 		if (io == 0)
 		{
-			ttyputerr(_("Cannot find Actel file %s"), pt);
+			ttyputerr(M_("Cannot find Actel file %s"), pt);
 			break;
 		}
 		for(;;)
@@ -203,7 +203,7 @@ void simals_makeactelmodel(char *name, FILE *io, LIBRARY *lib)
 								if (strcmp(ap->paramname, start) == 0) break;
 							if (ap == 0)
 							{
-								ttyputerr(_("In model %s, symbol %s not found in parameter lists"),
+								ttyputerr(M_("In model %s, symbol %s not found in parameter lists"),
 									name, start);
 								return;
 							}
@@ -234,7 +234,7 @@ void simals_makeactelmodel(char *name, FILE *io, LIBRARY *lib)
 								if (strcmp(ap->paramname, start) == 0) break;
 							if (ap == 0)
 							{
-								ttyputerr(_("In model %s, symbol %s not found in parameter lists"),
+								ttyputerr(M_("In model %s, symbol %s not found in parameter lists"),
 									name, start);
 								return;
 							}
@@ -261,7 +261,7 @@ void simals_makeactelmodel(char *name, FILE *io, LIBRARY *lib)
 					if (*pt == ';') break;
 					if (*pt == ':') { pt += 2;   doinginput = 0;   continue; }
 					if (*pt == ',') { pt++;   continue; }
-					ttyputerr(_("In model %s, missing comma in Actel symbol list"), name);
+					ttyputerr(M_("In model %s, missing comma in Actel symbol list"), name);
 					return;
 				}
 
@@ -355,7 +355,7 @@ void simals_makeactelmodel(char *name, FILE *io, LIBRARY *lib)
 				while (*pt == ' ' || *pt == '\t') pt++;
 				if (*pt == ';') break;
 				if (*pt == ',') { pt++;   continue; }
-				ttyputerr(_("In model %s, missing comma in Actel input list"), name);
+				ttyputerr(M_("In model %s, missing comma in Actel input list"), name);
 				return;
 			}
 			continue;
@@ -382,7 +382,7 @@ void simals_makeactelmodel(char *name, FILE *io, LIBRARY *lib)
 				while (*pt == ' ' || *pt == '\t') pt++;
 				if (*pt == ';') break;
 				if (*pt == ',') { pt++;   continue; }
-				ttyputerr(_("In model %s, missing comma in Actel output list"), name);
+				ttyputerr(M_("In model %s, missing comma in Actel output list"), name);
 				return;
 			}
 			continue;
@@ -410,7 +410,7 @@ void simals_makeactelmodel(char *name, FILE *io, LIBRARY *lib)
 				while (*pt == ' ' || *pt == '\t') pt++;
 				if (*pt == ';') break;
 				if (*pt == ',') { pt++;   continue; }
-				ttyputerr(_("In model %s, missing comma in Actel biput list"), name);
+				ttyputerr(M_("In model %s, missing comma in Actel biput list"), name);
 				return;
 			}
 			continue;
@@ -447,34 +447,34 @@ void simals_makeactelmodel(char *name, FILE *io, LIBRARY *lib)
 	np = newnodeproto(facetname, lib);
 	if (np == NONODEPROTO)
 	{
-		ttyputerr(_("Cannot create facet %s"), facetname);
+		ttyputerr(M_("Cannot create facet %s"), facetname);
 		return;
 	}
 	stringarraytotextfacet(sa, np, 1);
 	killstringarray(sa);
-	ttyputmsg(_("Created %s"), describenodeproto(np));
+	ttyputmsg(M_("Created %s"), describenodeproto(np));
 }
 
 INTSML simals_fgets(char *line, INTSML limit, FILE *file)
 {
-	REGISTER char *pp;
+	REGISTER char *pt;
 	REGISTER INTSML c, total;
 
-	pp = line;
+	pt = line;
 	total = 1;
 	for(;;)
 	{
 		c = xgetc(file);
 		if (c == EOF)
 		{
-			if (pp == line) return(1);
+			if (pt == line) return(1);
 			break;
 		}
 		if (c == '\n' || c == '\r') break;
-		*pp++ = (char)c;
+		*pt++ = (char)c;
 		if ((++total) >= limit) break;
 	}
-	return(*pp = 0);
+	return(*pt = 0);
 }
 
 /****************************** CLOCK ******************************/
@@ -502,7 +502,7 @@ void simals_clock_command(INTSML count, char *par[])
 
 	if (sim_window_isactive(&np) == 0)
 	{
-		ttyputerr(_("No simulator active"));
+		ttyputerr(M_("No simulator active"));
 		return;
 	}
 	if (count < 1)
@@ -515,7 +515,7 @@ void simals_clock_command(INTSML count, char *par[])
 	nodehead = simals_find_node(par[0]);
 	if (! nodehead)
 	{
-		ttyputerr(_("ERROR: Unable to find node %s"), par[0]);
+		ttyputerr(M_("ERROR: Unable to find node %s"), par[0]);
 		return;
 	}
 
@@ -537,7 +537,7 @@ void simals_clock_command(INTSML count, char *par[])
 		time = (float)atof(par[2]);
 		if (time <= 0.0)
 		{
-			ttyputerr(_("Clock timing parameter must be greater than 0"));
+			ttyputerr(M_("Clock timing parameter must be greater than 0"));
 			return;
 		}
 
@@ -603,7 +603,7 @@ void simals_clock_command(INTSML count, char *par[])
 	}
 
 	linear = (float)atof(par[2]);
-	strength = (INTSML)simals_atoi(par[3]);
+	strength = (INTSML)simals_atoi(par[3])*2;
 	num = simals_atoi(par[4]);
 
 	totaltime = 0.0;
@@ -918,7 +918,7 @@ void simals_erase_submod(CONPTR conhead)
 	}
 	if (conhead->display_page != 0)
 	{
-		chn = simals_levelptr->num_chn + 1;
+		chn = conhead->num_chn + 1;
 		for (i = 1; i < chn; i++)
 			efree((char *)conhead->display_page[i].name);
 		efree((char *)conhead->display_page);
@@ -942,18 +942,18 @@ void simals_go_command(INTSML count, char *par[])
 
 	if (sim_window_isactive(&np) == 0)
 	{
-		ttyputerr(_("No simulator active"));
+		ttyputerr(M_("No simulator active"));
 		return;
 	}
 	if (count < 1)
 	{
-		ttyputerr(_("Must specify simulation time"));
+		ttyputerr(M_("Must specify simulation time"));
 		return;
 	}
 	max = (float)atof(par[0]);
 	if (max <= 0.0)
 	{
-		ttyputerr(_("Simulation time must be greater than 0 seconds"));
+		ttyputerr(M_("Simulation time must be greater than 0 seconds"));
 		return;
 	}
 	sim_window_settimerange(0.0, max);
@@ -987,7 +987,8 @@ void simals_level_command(INTSML count, char *par[])
 {
 	CONPTR  cellptr;
 	INTSML   l;
-	char    *pp;
+	float mintime, maxtime, maintime, exttime;
+	char    *pt;
 	NODEPROTO *np;
 	extern COMCOMP sim_alslevelsp;
 
@@ -1002,8 +1003,8 @@ void simals_level_command(INTSML count, char *par[])
 		return;
 	}
 
-	l = strlen(pp = par[0]);
-	if (namesamen(pp, "up", l) == 0)
+	l = strlen(pt = par[0]);
+	if (namesamen(pt, "up", l) == 0)
 	{
 		if (simals_levelptr == 0)
 		{
@@ -1016,18 +1017,20 @@ void simals_level_command(INTSML count, char *par[])
 			ttyputerr(_("ERROR: Currently at top level of hierarchy"));
 			return;
 		}
-	} else if (namesamen(pp, "set", l) == 0)
+	} else if (namesamen(pt, "set", l) == 0)
 	{
 		if (count < 2)
 		{
-			count = ttygetparam(_("Instance to simulate: "), &sim_alslevelsp, 3, &par[1]) + 1;
+			count = ttygetparam(M_("Instance to simulate: "), &sim_alslevelsp, 3, &par[1]) + 1;
 			if (count < 2) return;
 		}
+		for(pt = par[1]; *pt != 0; pt++) if (*pt == ' ') break;
+		*pt = 0;
 		simals_convert_to_upper(par[1]);
 		cellptr = simals_find_level(par[1]);
 		if (! cellptr)
 		{
-			ttyputerr(_("ERROR: Unable to find level %s"), par[1]);
+			ttyputerr(M_("ERROR: Unable to find level %s"), par[1]);
 			return;
 		}
 	} else
@@ -1037,9 +1040,19 @@ void simals_level_command(INTSML count, char *par[])
 	}
 
 	simals_levelptr = cellptr;
+
+	/* reinitialize simulator while preserving time information */
+	sim_window_gettimerange(&mintime, &maxtime);
+	maintime = sim_window_getmaincursor();
+	exttime = sim_window_getextensioncursor();
 	if (simals_set_current_level() != 0) return;
+	sim_window_settimerange(mintime, maxtime);
+	sim_window_setmaincursor(maintime);
+	sim_window_setextensioncursor(exttime);
+
 	l = simals_levelptr->num_chn;
 	sim_window_setlines(l);
+	sim_window_settopline(0);
 	simals_initialize_simulator(1);
 }
 
@@ -1051,7 +1064,7 @@ void simals_level_command(INTSML count, char *par[])
 void simals_print_command(INTSML count, char *par[])
 {
 	INTSML l;
-	char *pp, s1[256];
+	char *pt, s1[256];
 	LINKPTR  linkhead;
 	NODEPTR  nodehead;
 	STATPTR  stathead;
@@ -1066,19 +1079,19 @@ void simals_print_command(INTSML count, char *par[])
 		ttyputusage("tellaid simulation als print OPTION");
 		return;
 	}
-	l = strlen(pp = par[0]);
+	l = strlen(pt = par[0]);
 
-	if (namesamen(pp, "size", l) == 0 && l >= 2)
+	if (namesamen(pt, "size", l) == 0 && l >= 2)
 	{
-		ttyputmsg(_("Number of Primitive Elements in Database = %ld"), simals_pseq);
-		ttyputmsg(_("Number of Nodes in Database = %ld"), simals_nseq);
+		ttyputmsg(M_("Number of Primitive Elements in Database = %ld"), simals_pseq);
+		ttyputmsg(M_("Number of Nodes in Database = %ld"), simals_nseq);
 		return;
 	}
 
-	if (namesamen(pp, "vector", l) == 0 && l >= 1)
+	if (namesamen(pt, "vector", l) == 0 && l >= 1)
 	{
 		linkhead = simals_setroot;
-		ttyputmsg(_("** VECTOR LINKLIST **"));
+		ttyputmsg(M_("** VECTOR LINKLIST **"));
 		while (linkhead)
 		{
 			switch (linkhead->type)
@@ -1086,7 +1099,7 @@ void simals_print_command(INTSML count, char *par[])
 				case 'N':
 					nodehead = (NODEPTR)linkhead->ptr;
 					simals_trans_number_to_state(linkhead->state, s1);
-					ttyputmsg(_("***** vector: $N%ld, state = %s, strength = %s, time = %g, priority = %d"),
+					ttyputmsg(M_("***** vector: $N%ld, state = %s, strength = %s, time = %g, priority = %d"),
 						nodehead->num, s1, simals_strengthstring(linkhead->strength),
 							linkhead->time, linkhead->priority);
 					break;
@@ -1094,16 +1107,16 @@ void simals_print_command(INTSML count, char *par[])
 					stathead = (STATPTR)linkhead->ptr;
 					nodehead = stathead->nodeptr;
 					simals_trans_number_to_state(linkhead->state, s1);
-					ttyputmsg(_("***** function: $N%ld, state = %s, strength = %s, time = %g, priority = %d"),
+					ttyputmsg(M_("***** function: $N%ld, state = %s, strength = %s, time = %g, priority = %d"),
 						nodehead->num, s1, simals_strengthstring(linkhead->strength),
 							linkhead->time, linkhead->priority);
 					break;
 				case 'R':
-					ttyputmsg(_("***** rowptr = %ld, time = %g, priority = %d"),
+					ttyputmsg(M_("***** rowptr = %ld, time = %g, priority = %d"),
 						linkhead->ptr, linkhead->time, linkhead->priority);
 					break;
 				case 'C':
-					ttyputmsg(_("***** clokptr = %ld, time = %g, priority = %d"),
+					ttyputmsg(M_("***** clokptr = %ld, time = %g, priority = %d"),
 						linkhead->ptr, linkhead->time, linkhead->priority);
 			}
 			linkhead = linkhead->right;
@@ -1111,9 +1124,9 @@ void simals_print_command(INTSML count, char *par[])
 		return;
 	}
 
-	if (namesamen(pp, "netlist", l) == 0 && l >= 1)
+	if (namesamen(pt, "netlist", l) == 0 && l >= 1)
 	{
-		ttyputmsg(_("** NETWORK DESCRIPTION **"));
+		ttyputmsg(M_("** NETWORK DESCRIPTION **"));
 		for (primhead = simals_primroot; primhead; primhead = primhead->next)
 		{
 			if (stopping(STOPREASONDISPLAY)) return;
@@ -1121,61 +1134,62 @@ void simals_print_command(INTSML count, char *par[])
 			{
 				case 'F':
 					(void)initinfstr();
-					(void)formatinfstr(_("FUNCTION %ld%s: %s("), primhead->num, s1,
-						primhead->name);
+					(void)formatinfstr(M_("FUNCTION %ld: %s (instance %s) ["), primhead->num, primhead->name,
+						(primhead->level == NULL) ? M_("null") : primhead->level);
 					for (exhead = primhead->exptr; exhead; exhead=exhead->next)
 					{
 						if (exhead != primhead->exptr) (void)addstringtoinfstr(", ");
 						(void)formatinfstr("N%ld", exhead->nodeptr->num);
 					}
+					(void)addstringtoinfstr("]");
 					ttyputmsg("%s", returninfstr());
 					(void)initinfstr();
-					(void)addstringtoinfstr(_("  Event Driving Inputs:"));
+					(void)addstringtoinfstr(M_("  Event Driving Inputs:"));
 					funchead = (FUNCPTR)primhead->ptr;
 					for (exhead = funchead->inptr; exhead; exhead=exhead->next)
 						(void)formatinfstr(" N%ld", exhead->nodeptr->num);
 					ttyputmsg("%s", returninfstr());
 					(void)initinfstr();
-					(void)addstringtoinfstr(_("  Output Ports:"));
+					(void)addstringtoinfstr(M_("  Output Ports:"));
 					for (exhead = primhead->exptr; exhead; exhead=exhead->next)
 					{
 						if (exhead->node_name)
 							(void)formatinfstr(" N%ld", ((STATPTR)exhead->node_name)->nodeptr->num);
 					}
 					ttyputmsg("%s", returninfstr());
-					ttyputmsg(_("  Timing: D=%g, L=%g, E=%g, R=%g, A=%g"), funchead->delta,
+					ttyputmsg(M_("  Timing: D=%g, L=%g, E=%g, R=%g, A=%g"), funchead->delta,
 						funchead->linear, funchead->exp, funchead->random, funchead->abs);
-					ttyputmsg(_("  Firing Priority = %d"), primhead->priority);
+					ttyputmsg(M_("  Firing Priority = %d"), primhead->priority);
 					break;
 				case 'G':
-					ttyputmsg(_("GATE %ld: %s  LEVEL: %s"), primhead->num, primhead->name,
-						(primhead->level == NULL) ? _("null") : primhead->level);
+					ttyputmsg(M_("GATE %ld: %s (instance %s)"), primhead->num, primhead->name,
+						(primhead->level == NULL) ? M_("null") : primhead->level);
 					for (rowhead = (ROWPTR)primhead->ptr; rowhead; rowhead=rowhead->next)
 					{
-						ttyputmsg(_("  Timing: D=%g, L=%g, E=%g, R=%g, A=%g"), rowhead->delta,
+						ttyputmsg(M_("  Timing: D=%g, L=%g, E=%g, R=%g, A=%g"), rowhead->delta,
 							rowhead->linear, rowhead->exp, rowhead->random, rowhead->abs);
-						ttyputmsg(_("  Delay type: %s"), (rowhead->delay == NULL) ? _("null") : rowhead->delay);
+						ttyputmsg(M_("  Delay type: %s"), (rowhead->delay == NULL) ? M_("null") : rowhead->delay);
 						simals_print_in_entry(rowhead);
 						simals_print_out_entry(rowhead);
 					}
-					ttyputmsg(_("  Firing Priority = %d"), primhead->priority);
+					ttyputmsg(M_("  Firing Priority = %d"), primhead->priority);
 					break;
 				default:
-					ttyputerr(_("Illegal primitive type '%c', database is bad"), primhead->type);
+					ttyputerr(M_("Illegal primitive type '%c', database is bad"), primhead->type);
 					break;
 			}
 		}
 		return;
 	}
 
-	if (namesamen(pp, "xref", l) == 0 && l >= 1)
+	if (namesamen(pt, "xref", l) == 0 && l >= 1)
 	{
-		ttyputmsg(_("** CROSS REFERENCE TABLE **"));
+		ttyputmsg(M_("** CROSS REFERENCE TABLE **"));
 		simals_print_xref_entry(simals_levelptr, 0);
 		return;
 	}
 
-	if (namesamen(pp, "state", l) == 0 && l >= 2)
+	if (namesamen(pt, "state", l) == 0 && l >= 2)
 	{
 		if (count < 2)
 		{
@@ -1186,32 +1200,31 @@ void simals_print_command(INTSML count, char *par[])
 		nodehead = simals_find_node(par[1]);
 		if (nodehead == 0)
 		{
-			ttyputerr(_("ERROR: Unable to find node %s"), par[1]);
+			ttyputerr(M_("ERROR: Unable to find node %s"), par[1]);
 			return;
 		}
 
 		simals_trans_number_to_state(nodehead->new_state, s1);
-		ttyputmsg(_("Node %s: State = %s, Strength = %s"), par[1], s1,
+		ttyputmsg(M_("Node %s: State = %s, Strength = %s"), par[1], s1,
 			simals_strengthstring(nodehead->new_strength));
 		stathead = nodehead->statptr;
 		while (stathead)
 		{
 			simals_trans_number_to_state(stathead->new_state, s1);
-			ttyputmsg(_("Primitive %ld:    State = %s, Strength = %s"), stathead->primptr->num,
+			ttyputmsg(M_("Primitive %ld:    State = %s, Strength = %s"), stathead->primptr->num,
 				s1, simals_strengthstring(stathead->new_strength));
 			stathead = stathead->next;
 		}
 		return;
 	}
 
-	if (namesamen(pp, "instances", l) == 0 && l >= 1)
+	if (namesamen(pt, "instances", l) == 0 && l >= 1)
 	{
-		simals_compute_path_name(simals_levelptr, s1);
-		ttyputmsg(_("Instances at level: %s"), s1);
+		ttyputmsg(M_("Instances at level: %s"), simals_compute_path_name(simals_levelptr));
 		for (cellhead = simals_levelptr->child; cellhead; cellhead = cellhead->next)
 		{
 			if (stopping(STOPREASONDISPLAY)) break;
-			ttyputmsg(_("Name: %s, Model: %s"), cellhead->inst_name, cellhead->model_name);
+			ttyputmsg(M_("Name: %s, Model: %s"), cellhead->inst_name, cellhead->model_name);
 		}
 		return;
 	}
@@ -1220,13 +1233,10 @@ void simals_print_command(INTSML count, char *par[])
 
 char *simals_strengthstring(INTSML strength)
 {
-	switch (strength)
-	{
-		case NODE_STRENGTH: return(_("node"));
-		case GATE_STRENGTH: return(_("gate"));
-		case VDD_STRENGTH:  return(_("power"));
-	}
-	return(_("off"));
+	if (strength == OFF_STRENGTH) return(_("off"));
+	if (strength <= NODE_STRENGTH) return(_("node"));
+	if (strength <= GATE_STRENGTH) return(_("gate"));
+	return(_("power"));
 }
 
 /*
@@ -1250,7 +1260,7 @@ void simals_print_in_entry(ROWPTR rowhead)
 
 	flag = 0;
 	(void)initinfstr();
-	(void)addstringtoinfstr(_("  Input: "));
+	(void)addstringtoinfstr(M_("  Input: "));
 
 	for (iohead = rowhead->inptr; iohead; iohead = iohead->next)
 	{
@@ -1302,7 +1312,7 @@ void simals_print_out_entry(ROWPTR rowhead)
 
 	flag = 0;
 	(void)initinfstr();
-	(void)addstringtoinfstr(_("  Output: "));
+	(void)addstringtoinfstr(M_("  Output: "));
 
 	for (iohead = rowhead->outptr; iohead; iohead = iohead->next)
 	{
@@ -1318,7 +1328,7 @@ void simals_print_out_entry(ROWPTR rowhead)
 		{
 			operatr = iohead->operatr - 128;
 			nodehead = (NODEPTR) iohead->operand;
-			(void)sprintf(s1, "%cN%ld@%d ", operatr, nodehead->num, iohead->strength);
+			(void)sprintf(s1, "%cN%ld@%d ", operatr, nodehead->num, (iohead->strength+1)/2);
 			(void)addstringtoinfstr(s1);
 			continue;
 		}
@@ -1328,7 +1338,7 @@ void simals_print_out_entry(ROWPTR rowhead)
 		num = (INTBIG)iohead->operand;
 		simals_trans_number_to_state(num, s1);
 		(void)addstringtoinfstr(s1);
-		(void)sprintf(s1, "@%d ", iohead->strength);
+		(void)sprintf(s1, "@%d ", (iohead->strength+1)/2);
 		(void)addstringtoinfstr(s1);
 	}
 	ttyputmsg("%s", returninfstr());
@@ -1350,15 +1360,15 @@ void simals_print_out_entry(ROWPTR rowhead)
 void simals_print_xref_entry(CONPTR cellhead, INTBIG tab)
 {
 	INTBIG  i, k, delay;
-	char    s1[256], tabsp[200], ts[256];
+	char    tabsp[200], ts[256];
 	EXPTR   exhead;
 	CONPTR  subcell;
 
 	if (stopping(STOPREASONDISPLAY)) return;
-	simals_compute_path_name(cellhead, s1);
 	for (i = 0; i < tab; ++i) tabsp[i] = ' ';
 	tabsp[tab] = 0;
-	ttyputmsg(_("%sLevel: %s, Model: %s"), tabsp, s1, cellhead->model_name);
+	ttyputmsg(M_("%sLevel: %s, Model: %s"), tabsp, simals_compute_path_name(cellhead),
+		cellhead->model_name);
 
 	for (exhead = cellhead->exptr; exhead; exhead = exhead->next)
 	{
@@ -1442,7 +1452,7 @@ void simals_set_command(INTSML count, char *par[])
 	}
 
 	state = simals_trans_state_to_number(par[1]);
-	strength = (INTSML)simals_atoi(par[2]);
+	strength = (INTSML)simals_atoi(par[2])*2;
 	time = (float)atof(par[3]);
 
 	sethead = simals_alloc_link_mem();
@@ -1456,7 +1466,7 @@ void simals_set_command(INTSML count, char *par[])
 	sethead->right = 0;
 	simals_insert_set_list(sethead);
 
-	ttyputmsg(_("Node '%s' scheduled, state = %s, strength = %s, time = %g"), par[0], par[1],
+	ttyputmsg(M_("Node '%s' scheduled, state = %s, strength = %s, time = %g"), par[0], par[1],
 		simals_strengthstring(strength), time);
 	simals_initialize_simulator(0);
 }
@@ -1473,20 +1483,20 @@ void simals_set_command(INTSML count, char *par[])
 void simals_trace_command(INTSML count, char *par[])
 {
 	INTSML l;
-	char *pp;
+	char *pt;
 
 	if (count < 1)
 	{
 		ttyputusage("tellaid simulation als trace OPTION");
 		return;
 	}
-	l = strlen(pp = par[0]);
-	if (namesamen(pp, "on", l) == 0 && l >= 2)
+	l = strlen(pt = par[0]);
+	if (namesamen(pt, "on", l) == 0 && l >= 2)
 	{
 		(void)setvalkey((INTBIG)sim_aid, VAID, simals_no_update, 0, VINTEGER);
 		return;
 	}
-	if (namesamen(pp, "off", l) == 0 && l >= 2)
+	if (namesamen(pt, "off", l) == 0 && l >= 2)
 	{
 		(void)setvalkey((INTBIG)sim_aid, VAID, simals_no_update, 1, VINTEGER);
 		return;
@@ -1500,7 +1510,7 @@ void simals_vector_command(INTSML count, char *par[])
 {
 	INTBIG	 i, flag;
 	INTSML   l, strength;
-	char     s1[256], s2[80], *pp, **vectptr1, **backptr;
+	char     s1[256], s2[80], *pt, **vectptr1, **backptr;
 	char    *filename, *truename;
 	FILE    *vin, *vout;
 	LINKPTR  sethead, vecthead, vectptr2, nextvec;
@@ -1519,9 +1529,9 @@ void simals_vector_command(INTSML count, char *par[])
 		ttyputusage("tellaid simulation als vector OPTION");
 		return;
 	}
-	l = strlen(pp = par[0]);
+	l = strlen(pt = par[0]);
 
-	if (namesamen(pp, "load", l) == 0)
+	if (namesamen(pt, "load", l) == 0)
 	{
 		if (count < 2)
 		{
@@ -1580,7 +1590,7 @@ void simals_vector_command(INTSML count, char *par[])
 					flag = 1;
 					continue;
 				}
-				strength = atoi(&(simals_instbuf[simals_instptr[9]]));
+				strength = atoi(&(simals_instbuf[simals_instptr[9]]))*2;
 
 				sethead = simals_alloc_link_mem();
 				if (sethead == 0) return;
@@ -1649,7 +1659,7 @@ void simals_vector_command(INTSML count, char *par[])
 				sethead->type = 'N';
 				sethead->ptr = (char *) nodehead;
 				sethead->state = simals_trans_state_to_number(&(simals_instbuf[simals_instptr[2]]));
-				sethead->strength = atoi(&(simals_instbuf[simals_instptr[3]]));
+				sethead->strength = atoi(&(simals_instbuf[simals_instptr[3]]))*2;
 				sethead->priority = 2;
 				sethead->time = (float)atof(&(simals_instbuf[simals_instptr[5]]));
 				sethead->right = 0;
@@ -1660,7 +1670,7 @@ void simals_vector_command(INTSML count, char *par[])
 		return;
 	}
 
-	if (namesamen(pp, "new", l) == 0)
+	if (namesamen(pt, "new", l) == 0)
 	{
 		/* clear all vectors */
 		simals_clearallvectors(0);
@@ -1668,14 +1678,14 @@ void simals_vector_command(INTSML count, char *par[])
 		return;
 	}
 
-	if (namesamen(pp, "save", l) == 0)
+	if (namesamen(pt, "save", l) == 0)
 	{
 		if (count < 2)
 		{
 			ttyputusage("tellaid simulation als vector save FILENAME");
 			return;
 		}
-		vout = xcreate(par[1], sim_filetypealsvec, _("Vector File"), &truename);
+		vout = xcreate(par[1], sim_filetypealsvec, 0, &truename);
 		if (vout == 0)
 		{
 			if (truename != 0) ttyputerr(_("ERROR: Can't create %s"), truename);
@@ -1692,7 +1702,7 @@ void simals_vector_command(INTSML count, char *par[])
 					simals_compute_node_name((NODEPTR)vecthead->ptr, s1);
 					xprintf(vout, "CLOCK %s D=%g L=%g E=%g ", s1, clokhead->delta,
 						clokhead->linear, clokhead->exp);
-					xprintf(vout, "STRENGTH=%d TIME=%g CYCLES=%ld\n", vecthead->strength,
+					xprintf(vout, "STRENGTH=%d TIME=%g CYCLES=%ld\n", vecthead->strength/2,
 						sethead->time, sethead->state);
 					for (; vecthead; vecthead = vecthead->right)
 					{
@@ -1703,7 +1713,7 @@ void simals_vector_command(INTSML count, char *par[])
 				case 'N':
 					simals_compute_node_name((NODEPTR)sethead->ptr, s1);
 					simals_trans_number_to_state(sethead->state, s2);
-					xprintf(vout, "SET %s=%s@%d TIME=%g\n", s1, s2, sethead->strength,
+					xprintf(vout, "SET %s=%s@%d TIME=%g\n", s1, s2, sethead->strength/2,
 						sethead->time);
 			}
 		}
@@ -1711,7 +1721,7 @@ void simals_vector_command(INTSML count, char *par[])
 		return;
 	}
 
-	if (namesamen(pp, "delete", l) == 0)
+	if (namesamen(pt, "delete", l) == 0)
 	{
 		if (count < 3)
 		{
@@ -1836,7 +1846,7 @@ void simals_annotate_command(INTSML count, char *par[])
 
 	if (simals_levelptr == 0)
 	{
-		ttyputerr(_("Must start simulator before annotating delay information"));
+		ttyputerr(M_("Must start simulator before annotating delay information"));
 		return;
 	}
 
@@ -1851,7 +1861,7 @@ void simals_annotate_command(INTSML count, char *par[])
 
 	simals_sdfannotate(simals_levelptr);
 	simals_update_netlist();
-	ttyputmsg(_("Completed annotation of SDF %s delay values"), par[0]);
+	ttyputmsg(M_("Completed annotation of SDF %s delay values"), par[0]);
 }
 
 /*
@@ -1859,12 +1869,12 @@ void simals_annotate_command(INTSML count, char *par[])
  */
 void simals_sdfannotate(CONPTR cellhead)
 {
-	char    s1[256];
+	char    *s1;
 	CONPTR  subcell;
 	NODEINST *ni;
 
 	if (stopping(STOPREASONDISPLAY)) return;
-	simals_compute_path_name(cellhead, s1);
+	s1 = simals_compute_path_name(cellhead);
 
 	ni = simals_getcellinstance(cellhead->model_name, s1);
 	if (ni != NONODEINST)
@@ -2113,7 +2123,7 @@ void simals_order_save(void)
 
 void simals_order_restore(char *list)
 {
-	INTSML tc = 0, i, found, lines, thispos, pos, fromlib = 0;
+	INTBIG tc = 0, i, found, lines, thispos, pos, fromlib = 0;
 	INTBIG tr, trl;
 	NODEPTR node;
 	VARIABLE *var;
@@ -2151,7 +2161,7 @@ void simals_order_restore(char *list)
 	}
 
 	/* delete traces not in restored list */
-	sim_window_sethighlighttrace(0);
+	sim_window_cleartracehighlight();
 	sim_window_inittraceloop();
 	while ((tr = sim_window_nexttraceloop()) != 0)
 	{
@@ -2169,10 +2179,10 @@ void simals_order_restore(char *list)
 				trl = sim_window_nexttraceloop2();
 				if (trl == 0) break;
 				pos = sim_window_gettraceline(trl);
-				if (pos > thispos) sim_window_settraceline(trl, (INTSML)(pos-1));
+				if (pos > thispos) sim_window_settraceline(trl, pos-1);
 			}
 			lines = sim_window_getlines();
-			if (lines > 1) sim_window_setlines((INTSML)(lines-1));
+			if (lines > 1) sim_window_setlines(lines-1);
 
 			/* remove from the simulator's list */
 			for(i=0; i<simals_levelptr->num_chn; i++)
@@ -2198,7 +2208,7 @@ void simals_order_restore(char *list)
 	{
 		for (i=0; i<tc; i++)
 		   if (namesame(tl[i], sim_window_gettracename(tr)) == 0) break;
-		if (fromlib) sim_window_settraceline(tr, (INTSML)(tc-i-1)); else  /* order from library save is bottom to top */
+		if (fromlib) sim_window_settraceline(tr, tc-i-1); else  /* order from library save is bottom to top */
 			sim_window_settraceline(tr, i);
 	}
 
@@ -2242,9 +2252,9 @@ void simals_update_netlist(void)
 						{
 							rowhead->abs = (float)max_delay * 1.0e-12f;
 						}
-						ttyputmsg(_("*** DEBUG *** gate: %s, level: %s, delay: %g(%s)"),
+						ttyputmsg(M_("*** DEBUG *** gate: %s, level: %s, delay: %g(%s)"),
 							primhead->name, primhead->level, (float)max_delay * 1.0e-12, rowhead->delay);
-						ttyputmsg(_("  Timing: D=%g, L=%g, E=%g, R=%g, A=%g"),
+						ttyputmsg(M_("  Timing: D=%g, L=%g, E=%g, R=%g, A=%g"),
 							rowhead->delta, rowhead->linear, rowhead->exp, rowhead->random, rowhead->abs);
 						simals_print_in_entry(rowhead);
 						simals_print_out_entry(rowhead);
@@ -2253,7 +2263,7 @@ void simals_update_netlist(void)
 				break;
 
 			default:
-				ttyputerr(_("Illegal primitive type '%c', database is bad"), primhead->type);
+				ttyputerr(M_("Illegal primitive type '%c', database is bad"), primhead->type);
 				break;
 		}
 	}
