@@ -349,6 +349,7 @@ EXPTR simals_find_xref_entry(CONPTR cellhead, char *name)
 	nodeptr2->pinptr = 0;
 	nodeptr2->load = -1.0;
 	nodeptr2->visit = 0;
+	nodeptr2->tracenode = 0;
 	nodeptr2->next = simals_noderoot;
 	xrefptr2->nodeptr = simals_noderoot = nodeptr2;
 	return(xrefptr2);
@@ -371,7 +372,6 @@ EXPTR simals_find_xref_entry(CONPTR cellhead, char *name)
 INTSML simals_process_gate(CONPTR cellhead, MODPTR modhead)
 {
 	ROWPTR  rowhead;
-	char s1[256];
 
 	simals_primptr2 = (MODPTR)simals_alloc_mem((INTBIG)sizeof(MODEL));
 	if (simals_primptr2 == 0) return(1);
@@ -386,8 +386,7 @@ INTSML simals_process_gate(CONPTR cellhead, MODPTR modhead)
 	simals_primptr2->fanout = modhead->fanout;
 	simals_primptr2->priority = modhead->priority;
 	simals_primptr2->next = 0;
-	simals_compute_path_name(cellhead, s1);
-	(void)allocstring(&(simals_primptr2->level), s1, sim_aid->cluster);
+	(void)allocstring(&(simals_primptr2->level), simals_compute_path_name(cellhead), sim_aid->cluster);
 	*simals_primptr1 = simals_primptr2;
 	simals_primptr1 = &simals_primptr2->next;
 
@@ -669,6 +668,8 @@ INTSML simals_process_function(CONPTR cellhead, MODPTR modhead)
 	simals_primptr2->fanout = 0;
 	simals_primptr2->priority = modhead->priority;
 	simals_primptr2->next = 0;
+	(void)allocstring(&(simals_primptr2->level), simals_compute_path_name(cellhead),
+		sim_aid->cluster);
 	*simals_primptr1 = simals_primptr2;
 	simals_primptr1 = &simals_primptr2->next;
 
