@@ -37,15 +37,17 @@
 #include "simals.h"
 #include "usr.h"
 #include "network.h"
+#include "edialogs.h"
 
 static INTSML sim_window_iter;
-static INTSML sim_als_wantweak = 0, sim_als_wantstrong = 0;
+static INTSML sim_als_wantweak = 0, sim_als_wantstrong = 0, sim_als_wantfull = 0;
 
 /* prototypes for local routines */
 INTSML sim_window_topofdsignals(char**);
 char *sim_window_nextdsignals(void);
 INTBIG simals_count_channels(CONPTR);
 void simals_helpwindow(INTSML waveform);
+INTSML simals_getwidevalue(INTBIG *value);
 
 INTSML sim_window_topofdsignals(char **c)
 {
@@ -76,8 +78,9 @@ static COMCOMP sim_window_pickdtrace = {NOKEYWORD, sim_window_topofdsignals,
  */
 INTSML simals_charhandlerwave(WINDOWPART *w, INTSML chr)
 {
-	REGISTER INTSML i, j, traces, pos, thispos, lines, strength;
-	REGISTER INTBIG tr, trl, nexttr, prevtr, state, highsig, windowstate;
+	REGISTER INTBIG i, j, traces, pos, thispos, lines, strength, *tracelist,
+		tr, trl, nexttr, prevtr, highsig, *highsigs, windowstate, bit, fulltrace;
+	INTBIG state;
 	REGISTER VARIABLE *var;
 	char *par[30], *name;
 	NODEPTR node, nodehead;
@@ -91,7 +94,7 @@ INTSML simals_charhandlerwave(WINDOWPART *w, INTSML chr)
 	if (chr == 'p')
 	{
 		sim_window_savegraph();
-		sim_als_wantweak = sim_als_wantstrong = 0;
+		sim_als_wantweak = sim_als_wantstrong = sim_als_wantfull = 0;
 		return(0);
 	}
 
@@ -99,7 +102,7 @@ INTSML simals_charhandlerwave(WINDOWPART *w, INTSML chr)
 	if (chr == '?')
 	{
 		simals_helpwindow(1);
-		sim_als_wantweak = sim_als_wantstrong = 0;
+		sim_als_wantweak = sim_als_wantstrong = sim_als_wantfull = 0;
 		return(0);
 	}
 
@@ -107,6 +110,7 @@ INTSML simals_charhandlerwave(WINDOWPART *w, INTSML chr)
 	if (sim_window_isactive(&np) == 0)
 		return(us_charhandler(w, chr));
 
+	fulltrace = 0;
 	switch (chr)
 	{
 		/* update display */
@@ -117,6 +121,11 @@ INTSML simals_charhandlerwave(WINDOWPART *w, INTSML chr)
 			if ((windowstate&ADVANCETIME) != 0) sim_window_setmaincursor(endtime);
 			return(0);
 
+		/* convert busses */
+		case 'b':
+			(void)sim_window_buscommand();
+			return(0);
+
 		/* add trace */
 		case 'a':
 			i = ttygetparam(_("Signal to add"), &sim_window_pickdtrace, 3, par);
@@ -134,7 +143,7 @@ INTSML simals_charhandlerwave(WINDOWPART *w, INTSML chr)
 			if (i >= simals_levelptr->num_chn) return(0);
 
 			/* ready to add: remove highlighting */
-			sim_window_sethighlighttrace(0);
+			sim_window_cleartracehighlight();
 
 			/* count the number of traces */
 			sim_window_inittraceloop();
@@ -145,7 +154,7 @@ INTSML simals_charhandlerwave(WINDOWPART *w, INTSML chr)
 			{
 				/* other traces exist, make a new line in the plot */
 				j = sim_window_getlines();
-				sim_window_setlines((INTSML)(j+1));
+				sim_window_setlines(j+1);
 			}
 
 			/* create a new trace in the last slot */
@@ -154,8 +163,8 @@ INTSML simals_charhandlerwave(WINDOWPART *w, INTSML chr)
 			simals_levelptr->display_page[i+1].displayptr = tr;
 			simals_fill_display_arrays();
 			sim_window_redraw();
-			sim_window_sethighlighttrace(tr);
-			sim_als_wantweak = sim_als_wantstrong = 0;
+			sim_window_addhighlighttrace(tr);
+			sim_als_wantweak = sim_als_wantstrong = sim_als_wantfull = 0;
 			return(0);
 
 		/* handle weak and strong prefixes */
@@ -167,6 +176,10 @@ INTSML simals_charhandlerwave(WINDOWPART *w, INTSML chr)
 			if (sim_als_wantweak != 0 || sim_als_wantstrong != 0) ttybeep();
 			sim_als_wantstrong = 1;
 			return(0);
+		case 'f':
+			if (sim_als_wantfull != 0) ttybeep();
+			sim_als_wantfull = 1;
+			return(0);
 
 		/* different flavors of low values */
 		case '0':
@@ -191,6 +204,37 @@ INTSML simals_charhandlerwave(WINDOWPART *w, INTSML chr)
 				if (sim_als_wantstrong != 0) strength = VDD_STRENGTH;
 			break;
 
+		/* different flavors of wide numeric values */
+		case 'v':
+			if (simals_getwidevalue(&state) != 0) return(0);
+			strength = GATE_STRENGTH;
+			if (sim_als_wantweak != 0) strength = NODE_STRENGTH; else
+				if (sim_als_wantstrong != 0) strength = VDD_STRENGTH;
+			break;
+
+		/* signal tracing */
+		case 't':
+			if (sim_als_wantfull != 0)
+			{
+				simals_trace_all_nodes = 1;
+				(void)simals_initialize_simulator(1);
+				simals_trace_all_nodes = 0;
+			} else
+			{
+				highsigs = sim_window_gethighlighttraces();
+				if (highsigs[0] == 0)
+				{
+					ttyputerr(_("Select signal names first"));
+					return(0);
+				}
+				for(i=0; highsigs[i] != 0; i++)
+					((NODEPTR)sim_window_gettracedata(highsigs[i]))->tracenode = 1;
+				(void)simals_initialize_simulator(1);
+				for(i=0; highsigs[i] != 0; i++)
+					((NODEPTR)sim_window_gettracedata(highsigs[i]))->tracenode = 0;
+			}
+			return(0);
+
 		/* signal clock setting, info, erasing, removing (all handled later) */
 		case 'c':
 		case 'i':
@@ -199,21 +243,33 @@ INTSML simals_charhandlerwave(WINDOWPART *w, INTSML chr)
 			break;
 
 		default:
-			sim_als_wantweak = sim_als_wantstrong = 0;
+			sim_als_wantweak = sim_als_wantstrong = sim_als_wantfull = 0;
 			return(us_charhandler(w, chr));
 	}
-	sim_als_wantweak = sim_als_wantstrong = 0;
+	sim_als_wantweak = sim_als_wantstrong = sim_als_wantfull = 0;
 
 	/* the following commands demand a current trace...get it */
-	highsig = sim_window_gethighlighttrace();
-	if (highsig == 0)
+	highsigs = sim_window_gethighlighttraces();
+	if (highsigs[0] == 0)
 	{
 		ttyputerr(_("Select a signal name first"));
 		return(0);
 	}
+	if (highsigs[1] != 0)
+	{
+		ttyputerr(_("Select just one signal name first"));
+		return(0);
+	}
+	highsig = highsigs[0];
 
 	if (chr == 'c')		/* set clock waveform */
 	{
+		tracelist = sim_window_getbustraces(highsig);
+		if (tracelist != 0 && tracelist[0] != 0)
+		{
+			ttyputerr(_("Cannot set clock waveform on a bus signal"));
+			return(0);
+		}
 		par[0] = sim_window_gettracename(highsig);
 		simals_clock_command(1, par);
 		return(0);
@@ -227,15 +283,28 @@ INTSML simals_charhandlerwave(WINDOWPART *w, INTSML chr)
 	}
 	if (chr == 'e')		/* clear signal vectors */
 	{
-		par[0] = "delete";
-		par[1] = sim_window_gettracename(highsig);
-		par[2] = "all";
-		simals_vector_command(3, par);
+		tracelist = sim_window_getbustraces(highsig);
+		if (tracelist != 0 && tracelist[0] != 0)
+		{
+			for(i=0; tracelist[i] != 0; i++)
+			{
+				par[0] = "delete";
+				par[1] = sim_window_gettracename(tracelist[i]);
+				par[2] = "all";
+				simals_vector_command(3, par);
+			}
+		} else
+		{
+			par[0] = "delete";
+			par[1] = sim_window_gettracename(highsig);
+			par[2] = "all";
+			simals_vector_command(3, par);
+		}
 		return(0);
 	}
 	if (chr == 'r')		/* remove trace */
 	{
-		sim_window_sethighlighttrace(0);
+		sim_window_cleartracehighlight();
 
 		/* delete it */
 		thispos = sim_window_gettraceline(highsig);
@@ -250,11 +319,11 @@ INTSML simals_charhandlerwave(WINDOWPART *w, INTSML chr)
 			{
 				pos--;
 				if (pos == thispos) nexttr = trl;
-				sim_window_settraceline(trl, (INTSML)pos);
+				sim_window_settraceline(trl, pos);
 			} else if (pos == thispos-1) prevtr = trl;
 		}
 		lines = sim_window_getlines();
-		if (lines > 1) sim_window_setlines((INTSML)(lines-1));
+		if (lines > 1) sim_window_setlines(lines-1);
 
 		/* remove from the simulator's list */
 		for(i=0; i<simals_levelptr->num_chn; i++)
@@ -270,25 +339,55 @@ INTSML simals_charhandlerwave(WINDOWPART *w, INTSML chr)
 
 		/* kill trace, redraw */
 		sim_window_killtrace(highsig);
-		if (nexttr != 0) sim_window_sethighlighttrace(nexttr); else
-			if (prevtr != 0) sim_window_sethighlighttrace(prevtr);
+		if (nexttr != 0) sim_window_addhighlighttrace(nexttr); else
+			if (prevtr != 0) sim_window_addhighlighttrace(prevtr);
 		sim_window_redraw();
 		return(0);
 	}
 
 	/* handle setting of values on signals */
-	nodehead = simals_find_node(sim_window_gettracename(highsig));
-	if (! nodehead) return(0);
-	sethead = simals_alloc_link_mem();
-	if (sethead == 0) return(0);
-	sethead->type = 'N';
-	sethead->ptr = (char *)nodehead;
-	sethead->state = state;
-	sethead->strength = strength;
-	sethead->priority = 2;
-	sethead->time = sim_window_getmaincursor();
-	sethead->right = 0;
-	simals_insert_set_list(sethead);
+	if (chr == 'v')
+	{
+		tracelist = sim_window_getbustraces(highsig);
+		if (tracelist == 0 || tracelist[0] == 0)
+		{
+			ttyputerr(_("Select a bus signal before setting numeric values on it"));
+			return(0);
+		}
+		for(i=0; tracelist[i] != 0; i++) ;
+		bit = 1 << (i-1);
+		for(i=0; tracelist[i] != 0; i++)
+		{
+			nodehead = simals_find_node(sim_window_gettracename(tracelist[i]));
+			if (! nodehead) return(0);
+			sethead = simals_alloc_link_mem();
+			if (sethead == 0) return(0);
+			sethead->type = 'N';
+			sethead->ptr = (char *)nodehead;
+			if ((state & bit) == 0) sethead->state = LOGIC_LOW; else
+				sethead->state = LOGIC_HIGH;
+			bit >>= 1;
+			sethead->strength = (INTSML)strength;
+			sethead->priority = 2;
+			sethead->time = sim_window_getmaincursor();
+			sethead->right = 0;
+			simals_insert_set_list(sethead);
+		}
+	} else
+	{
+		nodehead = simals_find_node(sim_window_gettracename(highsig));
+		if (! nodehead) return(0);
+		sethead = simals_alloc_link_mem();
+		if (sethead == 0) return(0);
+		sethead->type = 'N';
+		sethead->ptr = (char *)nodehead;
+		sethead->state = state;
+		sethead->strength = (INTSML)strength;
+		sethead->priority = 2;
+		sethead->time = sim_window_getmaincursor();
+		sethead->right = 0;
+		simals_insert_set_list(sethead);
+	}
 
 	endtime = simals_initialize_simulator(0);
 	var = getvalkey((INTBIG)sim_aid, VAID, VINTEGER, sim_window_state);
@@ -351,6 +450,7 @@ INTSML simals_charhandlerschem(WINDOWPART *w, INTSML chr)
 			break;
 		case 'c':
 		case 'i':
+		case 't':
 		case 'e':
 			break;
 		default:
@@ -364,15 +464,14 @@ INTSML simals_charhandlerschem(WINDOWPART *w, INTSML chr)
 	if (net == NONETWORK) return(0);
 	if (net->namecount > 0) pt = net->netname; else
 		pt = describenetwork(net);
-	sim_window_sethighlighttrace(sim_window_findtrace(pt));
-
-	/* get waveform trace and make changes */
-	highsig = sim_window_gethighlighttrace();
+	highsig = sim_window_findtrace(pt);
 	if (highsig == 0)
 	{
-		ttyputerr(_("Select a signal name first"));
+		ttyputerr(_("Cannot find network %s in simulation"), pt);
 		return(0);
 	}
+	sim_window_cleartracehighlight();
+	sim_window_addhighlighttrace(highsig);
 
 	if (chr == 'c')
 	{
@@ -387,6 +486,14 @@ INTSML simals_charhandlerschem(WINDOWPART *w, INTSML chr)
 		simals_print_command(2, par);
 		return(0);
 	}
+	if (chr == 't')
+	{
+		nodehead = (NODEPTR)sim_window_gettracedata(highsig);
+		nodehead->tracenode = 1;
+		(void)simals_initialize_simulator(1);
+		nodehead->tracenode = 0;
+		return(0);
+	}
 	if (chr == 'e')
 	{
 		par[0] = "delete";
@@ -419,40 +526,82 @@ INTSML simals_charhandlerschem(WINDOWPART *w, INTSML chr)
 	return(0);
 }
 
+/* Simulation: Wide Value */
+DIALOGITEM sim_widedialogitems[] =
+{
+ /*  1 */ {0, {36,148,60,228}, BUTTON, N_("OK")},
+ /*  2 */ {0, {4,4,20,148}, MESSAGE, N_("Value to set on bus:")},
+ /*  3 */ {0, {36,16,60,96}, BUTTON, N_("Cancel")},
+ /*  4 */ {0, {4,152,20,240}, EDITTEXT, ""}
+};
+DIALOG sim_widedialog = {{75,75,144,324}, N_("Set Wide Value"), 0, 4, sim_widedialogitems};
+
+/* special items for the "wide value" dialog: */
+#define DSWV_VALUE     4		/* Width (edit text) */
+
+/*
+ * Routine to prompt for a wide value and return it in "value".
+ * Returns nonzero on abort/error.
+ */
+INTSML simals_getwidevalue(INTBIG *value)
+{
+	INTBIG itemHit;
+
+	if (DiaInitDialog(&sim_widedialog) != 0) return(1);
+	DiaSetText(-DSWV_VALUE, "0");
+	for(;;)
+	{
+		itemHit = DiaNextHit();
+		if (itemHit == OK || itemHit == CANCEL) break;
+	}
+	*value = myatoi(DiaGetText(DSWV_VALUE));
+	DiaDoneDialog();
+	if (itemHit == OK) return(0);
+	return(1);
+}
+
 void simals_helpwindow(INTSML waveform)
 {
-	REGISTER INTSML active;
+	REGISTER INTBIG active;
 	NODEPROTO *np;
 
 	active = sim_window_isactive(&np);
+	if (active == 0)
+	{
+		ttyputmsg(_("There is no current simulation"));
+		return;
+	}
 
 	if (waveform != 0)
 		ttyputmsg(_("These keys may be typed in the ALS Waveform window:")); else
 			ttyputmsg(_("These keys may be typed in the ALS Schematic window:"));
-	if (active != 0)
+	ttyputinstruction("L,0", 4, _("Set signal low (normal strength)"));
+	ttyputinstruction("WL",  4, _("Set signal low (weak strength)"));
+	ttyputinstruction("SL",  4, _("Set signal low (strong strength)"));
+	ttyputinstruction("H,1", 4, _("Set signal high (normal strength)"));
+	ttyputinstruction("WH",  4, _("Set signal high (weak strength)"));
+	ttyputinstruction("SH",  4, _("Set signal high (strong strength)"));
+	ttyputinstruction(" X",  4, _("Set signal undefined (normal strength)"));
+	ttyputinstruction("WX",  4, _("Set signal undefined (weak strength)"));
+	ttyputinstruction("SX",  4, _("Set signal undefined (strong strength)"));
+	if (waveform != 0)
 	{
-		ttyputinstruction("L,0", 4, _("Set signal low (gate strength)"));
-		ttyputinstruction("WL",  4, _("Set signal weak low (node strength)"));
-		ttyputinstruction("SL",  4, _("Set signal strong low (power strength)"));
-		ttyputinstruction("H,1", 4, _("Set signal high (gate strength)"));
-		ttyputinstruction("WH",  4, _("Set signal weak high (node strength)"));
-		ttyputinstruction("SH",  4, _("Set signal strong high (power strength)"));
-		ttyputinstruction(" X",  4, _("Set signal undefined (gate strength)"));
-		ttyputinstruction("WX",  4, _("Set signal weak undefined (node strength)"));
-		ttyputinstruction("SX",  4, _("Set signal strong undefined (power strength)"));
-		ttyputinstruction(" C",  4, _("Set signal to be a clock"));
-		ttyputinstruction(" E",  4, _("Delete all vectors on signal"));
-		ttyputinstruction(" I",  4, _("Print information about signal"));
-		ttyputinstruction(" U",  4, _("Update display (resimulate)"));
+		ttyputinstruction(" V",  4, _("Set bus to wide value (normal strength)"));
+		ttyputinstruction("WV",  4, _("Set bus to wide value (weak strength)"));
+		ttyputinstruction("SV",  4, _("Set bus to wide value (strong strength)"));
 	}
+	ttyputinstruction(" C",  4, _("Set signal to be a clock"));
+	ttyputinstruction(" E",  4, _("Delete all vectors on signal"));
+	ttyputinstruction(" I",  4, _("Print information about signal"));
+	ttyputinstruction(" T",  4, _("Trace events on signal(s)"));
+	ttyputinstruction(" U",  4, _("Update display (resimulate)"));
 
 	if (waveform != 0)
 	{
-		if (active != 0)
-		{
-			ttyputinstruction(" A", 4, _("Add signal to simulation window"));
-			ttyputinstruction(" R", 4, _("Remove signal from the window"));
-		}
+		ttyputinstruction(" A", 4, _("Add signal to simulation window"));
+		ttyputinstruction(" R", 4, _("Remove signal from the window"));
+		ttyputinstruction(" B", 4, _("Combine selected signals into a bus"));
+		ttyputinstruction("FT", 4, _("Full Trace of all events"));
 		ttyputinstruction(" P", 4, _("Preserve snapshot of simulation window in database"));
 	}
 }
@@ -484,12 +633,14 @@ INTBIG simals_count_channels(CONPTR level_pointer)
  */
 INTSML simals_set_current_level(void)
 {
-	INTBIG i, chn, windowstate;
+	INTBIG i, j, buscount, oldsigcount, chn, windowstate, bussignals[32];
+	char **oldsignames;
 	REGISTER VARIABLE *var;
 	NODEPTR nodehead;
 	NODE *node;
 	EXPTR exhead;
-	char *name;
+	char *name, *pt, *start, save;
+	void *sa;
 	NODEPROTO *np;
 
 	for (nodehead = simals_noderoot; nodehead; nodehead = nodehead->next)
@@ -532,6 +683,7 @@ INTSML simals_set_current_level(void)
 	}
 
 	/* prepare the simulation window */
+	oldsigcount = 0;
 	if (sim_window_isactive(&np) == 0)
 	{
 		/* no simulation running: start it up */
@@ -539,25 +691,91 @@ INTSML simals_set_current_level(void)
 
 		var = getvalkey((INTBIG)sim_aid, VAID, VINTEGER, sim_window_state);
 		if (var == NOVARIABLE) windowstate = 0; else windowstate = var->addr;
+		var = getvalkey((INTBIG)simals_mainproto, VNODEPROTO, VSTRING|VISARRAY, sim_window_signal_order);
+		if (var != NOVARIABLE)
+		{
+			oldsigcount = getlength(var);
+			if (oldsigcount > 0)
+			{
+				sa = newstringarray(sim_aid->cluster);
+				for(i=0; i<oldsigcount; i++)
+					addtostringarray(sa, ((char **)var->addr)[i]);
+				oldsignames = getstringarray(sa, &oldsigcount);
+			}
+		}
 		if (sim_window_create(simals_levelptr->num_chn, simals_mainproto,
 			((windowstate&SHOWWAVEFORM) != 0 ? simals_charhandlerwave : 0),
-				simals_charhandlerschem) != 0) return(1);
+				simals_charhandlerschem) != 0)
+		{
+			if (oldsigcount > 0) killstringarray(sa);
+			return(1);
+		}
 	}
 
 	/* load the traces */
 	sim_window_killalltraces();
 	for(i=0; i<simals_levelptr->num_chn; i++)
+		simals_levelptr->display_page[i+1].displayptr = 0;
+
+	for(j=0; j<oldsigcount; j++)
 	{
+		/* see if the name is a bus */
+		for(pt = oldsignames[j]; *pt != 0; pt++) if (*pt == '\t') break;
+		if (*pt == '\t')
+		{
+			/* a bus */
+			pt = oldsignames[j];
+			for(buscount = 0; ; )
+			{
+				for(start = pt; *pt != 0; pt++) if (*pt == '\t') break;
+				save = *pt;
+				*pt = 0;
+				for(i=0; i<simals_levelptr->num_chn; i++)
+				{
+					node = simals_levelptr->display_page[i+1].nodeptr;
+					if (node == 0) continue;
+					name = simals_levelptr->display_page[i+1].name;
+					if (namesame(name, start) != 0) continue;
+					bussignals[buscount] = sim_window_newtrace(-1, name, (INTBIG)node);
+					simals_levelptr->display_page[i+1].displayptr = bussignals[buscount];
+					buscount++;
+					break;
+				}
+				*pt++ = save;
+				if (save == 0) break;
+			}
+
+			/* create the bus */
+			(void)sim_window_makebus(buscount, bussignals);
+		} else
+		{
+			/* a single signal */
+			for(i=0; i<simals_levelptr->num_chn; i++)
+			{
+				node = simals_levelptr->display_page[i+1].nodeptr;
+				if (node == 0) continue;
+				name = simals_levelptr->display_page[i+1].name;
+				if (namesame(name, oldsignames[j]) != 0) continue;
+				simals_levelptr->display_page[i+1].displayptr = sim_window_newtrace(-1, name,
+					(INTBIG)node);
+				break;
+			}
+		}
+	}
+	for(i=0; i<simals_levelptr->num_chn; i++)
+	{
+		if (simals_levelptr->display_page[i+1].displayptr != 0) continue;
 		node = simals_levelptr->display_page[i+1].nodeptr;
 		if (node == 0) continue;
 		name = simals_levelptr->display_page[i+1].name;
-		simals_levelptr->display_page[i+1].displayptr = sim_window_newtrace((INTSML)i, name,
+		simals_levelptr->display_page[i+1].displayptr = sim_window_newtrace(-1, name,
 			(INTBIG)node);
 	}
-	sim_window_sethighlighttrace(0);
+	sim_window_cleartracehighlight();
 	sim_window_settimerange(0.0, 0.0000005f);
 	sim_window_setmaincursor(0.0000002f);
 	sim_window_setextensioncursor(0.0000003f);
+	if (oldsigcount > 0) killstringarray(sa);
 	return(0);
 }
 
@@ -615,7 +833,7 @@ void simals_fill_display_arrays(void)
 		if (nodelist[i] == 0) continue;
 		numsteps[i] = 1;
 		timearrays[i][0] = min;
-		statearrays[i][0] = (LOGIC_LOW << 8) | 0;
+		statearrays[i][0] = (LOGIC_LOW << 8) | OFF_STRENGTH;
 	}
 	if (simals_trakfull == 0) i = 0; else i = (INTSML)simals_trakptr;
 	for( ; i < simals_trakptr && i < simals_trace_size; i++)
