@@ -37,10 +37,14 @@
 #include "simals.h"
 
 LINKPTR simals_freelink = 0;
+INTSML  simals_tracing = 0;
+char *simals_statedesc[] = {N_("High"), N_("Undefined"), N_("Low")};
+char *simals_strengthdesc[] = {N_("Off-"), N_("Weak-"), N_("Weak-"), "", "",
+	N_("Strong-"), N_("Strong-")};
 
 /* prototypes for local routines */
 INTSML simals_fire_event(void);
-void simals_create_check_list(NODEPTR);
+void simals_create_check_list(NODEPTR, LINKPTR);
 INTSML simals_schedule_new_events(void);
 void simals_calculate_clock_time(LINKPTR, ROWPTR);
 INTSML simals_calculate_event_time(MODPTR, ROWPTR);
@@ -107,15 +111,16 @@ float simals_initialize_simulator(INTSML force)
 		linkptr2->strength = linkhead->strength;
 		linkptr2->priority = linkhead->priority;
 		linkptr2->time = linkhead->time;
+		linkptr2->primhead = 0;
 		simals_insert_link_list(linkptr2);
 	}
 
 	for (nodehead = simals_noderoot; nodehead; nodehead = nodehead->next)
 	{
 		nodehead->sum_state = LOGIC_LOW;
-		nodehead->sum_strength = 0;
+		nodehead->sum_strength = OFF_STRENGTH;
 		nodehead->new_state = LOGIC_LOW;
-		nodehead->new_strength = 0;
+		nodehead->new_strength = OFF_STRENGTH;
 		nodehead->maxsize = 0;
 		nodehead->arrive = 0;
 		nodehead->depart = 0;
@@ -124,7 +129,7 @@ float simals_initialize_simulator(INTSML force)
 		for (stathead = nodehead->statptr; stathead; stathead = stathead->next)
 		{
 			stathead->new_state = LOGIC_LOW;
-			stathead->new_strength = 0;
+			stathead->new_strength = OFF_STRENGTH;
 			stathead->sched_op = 0;
 		}
 	}
@@ -153,7 +158,7 @@ float simals_initialize_simulator(INTSML force)
 		sim_window_redraw();
 		sim_window_updatelayoutwindow();
 		sim_windowconvertengineeringnotation(simals_time_abs, s1);
-		ttyputverbose(_("Simulation completed at time %s"), s1);
+		ttyputverbose(M_("Simulation completed at time %s"), s1);
 	}
 
 	return(simals_time_abs);
@@ -178,6 +183,7 @@ INTSML simals_fire_event(void)
 	NODEPTR	nodehead;
 	ROWPTR rowhead;
 	STATPTR	stathead;
+	char s1[30], s2[300];
 
 	simals_time_abs = simals_linkfront->time;
 	linkhead = simals_linkfront;
@@ -206,24 +212,34 @@ INTSML simals_fire_event(void)
 		}
 	}
 
+	simals_tracing = 0;
 	switch (linkhead->type)
 	{
 		case 'G':
-			stathead = (STATPTR) linkhead->ptr;
-			if ((stathead->sched_op != linkhead->operatr)
-				|| (stathead->sched_state != linkhead->state)
-				|| (stathead->sched_strength != linkhead->strength)) break;
+			stathead = (STATPTR)linkhead->ptr;
+			if (simals_trace_all_nodes != 0 || stathead->nodeptr->tracenode != 0)
+			{
+				sim_windowconvertengineeringnotation(simals_time_abs, s1);
+				simals_compute_node_name(stathead->nodeptr, s2);
+				ttyputmsg("%s: Firing gate %s%s, net %s", s1, stathead->primptr->name, stathead->primptr->level, s2);
+				simals_tracing = 1;
+			}
+			if (stathead->sched_state != linkhead->state ||
+				stathead->sched_op != linkhead->operatr ||
+				stathead->sched_strength != linkhead->strength)
+			{
+				break;
+			}
 			stathead->sched_op = 0;
 
-			/* ttyputmsg(_("Fire: $N%ld"), stathead->nodeptr->num); */
 			operatr = linkhead->operatr;
 			if (operatr < 128)
 			{
-				operand = (INTBIG) linkhead->state;
+				operand = (INTBIG)linkhead->state;
 			} else
 			{
 				operatr -= 128;
-				nodehead = (NODEPTR) linkhead->state;
+				nodehead = (NODEPTR)linkhead->state;
 				operand = nodehead->sum_state;
 			}
 
@@ -252,28 +268,38 @@ INTSML simals_fire_event(void)
 					return(1);
 			}
 
-			if (stathead->new_state == state && stathead->new_strength == linkhead->strength)
+			if (state == stathead->new_state &&
+				linkhead->strength == stathead->new_strength)
+			{
 				break;
+			}
 			stathead->new_state = state;
 			stathead->new_strength = linkhead->strength;
-			simals_create_check_list(stathead->nodeptr);
+			simals_create_check_list(stathead->nodeptr, linkhead);
 			break;
 
 		case 'N':
-			nodehead = (NODEPTR) linkhead->ptr;
-			if (linkhead->state == nodehead->new_state && linkhead->strength == nodehead->new_strength)
-				break;
+			nodehead = (NODEPTR)linkhead->ptr;
+			if (simals_trace_all_nodes != 0 || nodehead->tracenode != 0)
+			{
+				sim_windowconvertengineeringnotation(simals_time_abs, s1);
+				simals_compute_node_name(nodehead, s2);
+				ttyputmsg("%s: Changed state of net %s", s1, s2);
+				simals_tracing = 1;
+			}
+			if (linkhead->state == nodehead->new_state &&
+				linkhead->strength == nodehead->new_strength)
+					break;
 
-			/* ttyputmsg(_("Fire: $N%ld"), nodehead->num);*/
 			nodehead->new_state = linkhead->state;
 			nodehead->new_strength = linkhead->strength;
-			simals_create_check_list(nodehead);
+			simals_create_check_list(nodehead, linkhead);
 			break;
 
 		case 'C':
 			time = simals_time_abs;
-			rowhead = (ROWPTR) linkhead->ptr;
-			for (vecthead = (LINKPTR) rowhead->inptr; vecthead; vecthead = vecthead->right)
+			rowhead = (ROWPTR)linkhead->ptr;
+			for (vecthead = (LINKPTR)rowhead->inptr; vecthead; vecthead = vecthead->right)
 			{
 				linkptr2 = simals_alloc_link_mem();
 				if (linkptr2 == 0) return(1);
@@ -283,6 +309,7 @@ INTSML simals_fire_event(void)
 				linkptr2->strength = vecthead->strength;
 				linkptr2->priority = vecthead->priority;
 				linkptr2->time = time;
+				linkptr2->primhead = 0;
 				simals_insert_link_list(linkptr2);
 				time += vecthead->time;
 			}
@@ -314,54 +341,78 @@ INTSML simals_fire_event(void)
  * state for a node because it is possible to have nodes that have more than
  * one transition driving it.
  */
-void simals_create_check_list(NODEPTR nodehead)
+
+void simals_create_check_list(NODEPTR nodehead, LINKPTR linkhead)
 {
-	INTBIG	state;
-	INTSML	strength;
+	REGISTER INTBIG state, thisstate;
+	INTSML	strength, thisstrength;
 	STATPTR stathead;
 	TRAKPTR trakhead;
 
+	/* get initial state of the node */
 	state = nodehead->new_state;
 	strength = nodehead->new_strength;
 
+	/* print state of signal if this signal is being traced */
+	if (simals_tracing != 0)
+	{
+		ttyputmsg("  Formerly %s%s, starts at %s%s",
+			_(simals_strengthdesc[nodehead->sum_strength]), _(simals_statedesc[nodehead->sum_state+3]),
+			_(simals_strengthdesc[strength]), _(simals_statedesc[state+3]));
+	}
+
+	/* look at all factors affecting the node */
 	for (stathead = nodehead->statptr; stathead; stathead = stathead->next)
 	{
-		if (stathead->new_strength > strength)
+		thisstate = stathead->new_state;
+		thisstrength = stathead->new_strength;
+		if (simals_tracing != 0)
+			ttyputmsg("    %s%s from %s%s", _(simals_strengthdesc[thisstrength]),
+			_(simals_statedesc[thisstate+3]), stathead->primptr->name, stathead->primptr->level);
+
+		/* higher strength overrides previous node state */
+		if (thisstrength > strength)
 		{
-			state = stathead->new_state;
-			strength = stathead->new_strength;
+			state = thisstate;
+			strength = thisstrength;
 			continue;
 		}
-		if (stathead->new_strength == strength)
+
+		/* same strength: must arbitrate */
+		if (thisstrength == strength)
 		{
-			if (stathead->new_state != state)
-			{
+			if (thisstate != state)
 				state = LOGIC_X;
-			}
 		}
 	}
 
+	/* if the node has nothing driving it, set it to the old value */
 	if (strength == OFF_STRENGTH)
 	{
 		state = nodehead->sum_state;
 		strength = NODE_STRENGTH;
 	}
 
-	if (nodehead->sum_state == state && nodehead->sum_strength == strength) return;
+	/* stop now if node state did not change */
+	if (nodehead->sum_state == state && nodehead->sum_strength == strength)
+	{
+		if (simals_tracing != 0) ttyputmsg("    NO CHANGE");
+		return;
+	}
 
 	if (nodehead->plot_node)
 	{
 		trakhead = &(simals_trakroot[simals_trakptr]);
-		trakhead->ptr = (NODEPTR) nodehead;
+		trakhead->ptr = (NODEPTR)nodehead;
 		trakhead->state = state;
 		trakhead->strength = strength;
 		trakhead->time = simals_time_abs;
 		simals_trakptr = (simals_trakptr + 1) % simals_trace_size;
 		if (! simals_trakptr)
-		{
 			simals_trakfull = 1;
-		}
 	}
+	if (simals_tracing != 0)
+		ttyputmsg("    BECOMES %s%s", _(simals_strengthdesc[strength]), _(simals_statedesc[state+3]));
 
 	nodehead->sum_state = state;
 	nodehead->sum_strength = strength;
@@ -393,16 +444,16 @@ INTSML simals_schedule_new_events(void)
 
 	for (chekhead = simals_chekroot; chekhead; chekhead = chekhead->next)
 	{
-		primhead = (MODPTR) chekhead->ptr;
+		primhead = (MODPTR)chekhead->ptr;
 
 		if (primhead->type == 'F')
 		{
-			funchead = (FUNCPTR) primhead->ptr;
+			funchead = (FUNCPTR)primhead->ptr;
 			(*(funchead->procptr))(primhead);
 			continue;
 		}
 
-		for (rowhead = (ROWPTR) primhead->ptr; rowhead; rowhead = rowhead->next)
+		for (rowhead = (ROWPTR)primhead->ptr; rowhead; rowhead = rowhead->next)
 		{
 			flag = 1;
 			for (iohead = rowhead->inptr; iohead; iohead = iohead->next)
@@ -410,11 +461,11 @@ INTSML simals_schedule_new_events(void)
 				operatr = iohead->operatr;
 				if (operatr < 128)
 				{
-					operand = (INTBIG) iohead->operand;
+					operand = (INTBIG)iohead->operand;
 				} else
 				{
 					operatr -= 128;
-					nodehead = (NODEPTR) iohead->operand;
+					nodehead = (NODEPTR)iohead->operand;
 					operand = nodehead->sum_state;
 				}
 
@@ -471,17 +522,17 @@ void simals_calculate_clock_time(LINKPTR linkhead, ROWPTR rowhead)
 	if (rowhead->delta) time += rowhead->delta;
 	if (rowhead->linear)
 	{
-		prob = (float)(rand() / HUGEINT);
+		prob = (float)(rand() / MAXINTBIG);
 		time += 2.0f * prob * rowhead->linear;
 	}
 
 	/*
-	if (rowhead->exp)
-	{
-		prob = rand() / HUGEINT;
-		time += (-log(prob) * (rowhead->exp));
-	}
-	*/
+	 * if (rowhead->exp)
+	 * {
+	 * 	prob = rand() / MAXINTBIG;
+	 * 	time += (-log(prob) * (rowhead->exp));
+	 * }
+	 */
 
 	linkhead->time = time;
 	simals_insert_link_list(linkhead);
@@ -508,6 +559,7 @@ INTSML simals_calculate_event_time(MODPTR primhead, ROWPTR rowhead)
 	STATPTR	stathead;
 	IOPTR iohead;
 	LINKPTR	linkptr2;
+	char s1[30];
 
 	time = 0.0;
 	priority = primhead->priority;
@@ -516,21 +568,21 @@ INTSML simals_calculate_event_time(MODPTR primhead, ROWPTR rowhead)
 	if (rowhead->abs) time += rowhead->abs;
 	if (rowhead->linear)
 	{
-		prob = (float)(rand() / HUGEINT);
+		prob = (float)(rand() / MAXINTBIG);
 		time += 2.0f * prob * rowhead->linear;
 	}
 
 	/*
-	if (rowhead->exp)
-	{
-		prob = rand() / HUGEINT;
-		time += (-log(prob) * (rowhead->exp));
-	}
-	*/
+	 * if (rowhead->exp)
+	 * {
+	 * 	prob = rand() / MAXINTBIG;
+	 * 	time += (-log(prob) * (rowhead->exp));
+	 * }
+	 */
 
 	if (rowhead->random)
 	{
-		prob = (float)(rand() / HUGEINT);
+		prob = (float)(rand() / MAXINTBIG);
 		if (prob <= rowhead->random)
 		{
 			priority = -1;
@@ -539,17 +591,17 @@ INTSML simals_calculate_event_time(MODPTR primhead, ROWPTR rowhead)
 
 	if (primhead->fanout)
 	{
-		stathead = (STATPTR) rowhead->outptr->nodeptr;
+		stathead = (STATPTR)rowhead->outptr->nodeptr;
 		time *= stathead->nodeptr->load;
 	}
 	time += simals_time_abs;
 
 	for (iohead = rowhead->outptr; iohead; iohead = iohead->next)
 	{
-		stathead = (STATPTR) iohead->nodeptr;
-		if ((stathead->sched_op == iohead->operatr)
-			&& (stathead->sched_state == (INTBIG)iohead->operand)
-				&& (stathead->sched_strength == iohead->strength))
+		stathead = (STATPTR)iohead->nodeptr;
+		if (stathead->sched_op == iohead->operatr &&
+			stathead->sched_state == (INTBIG)iohead->operand &&
+				stathead->sched_strength == iohead->strength)
 		{
 			continue;
 		}
@@ -557,12 +609,19 @@ INTSML simals_calculate_event_time(MODPTR primhead, ROWPTR rowhead)
 		linkptr2 = simals_alloc_link_mem();
 		if (linkptr2 == 0) return(1);
 		linkptr2->type = 'G';
-		linkptr2->ptr = (char *) stathead;
+		linkptr2->ptr = (char *)stathead;
 		linkptr2->operatr = stathead->sched_op = iohead->operatr;
-		linkptr2->state = stathead->sched_state = (INTBIG) iohead->operand;
+		linkptr2->state = stathead->sched_state = (INTBIG)iohead->operand;
 		linkptr2->strength = stathead->sched_strength = iohead->strength;
 		linkptr2->time = time;
 		linkptr2->priority = priority;
+		linkptr2->primhead = primhead;
+		if (simals_tracing != 0)
+		{
+			sim_windowconvertengineeringnotation(time, s1);
+			ttyputmsg(_("      Schedule(G): %s%s at %s"), stathead->primptr->name,
+				stathead->primptr->level, s1);
+		}
 		simals_insert_link_list(linkptr2);
 	}
 	return(0);
@@ -626,23 +685,23 @@ char **simals_insert_set_list(LINKPTR linkhead)
 	char **linkptr1;
 	LINKPTR linkptr2;
 
-	linkptr1 = (char**) &simals_setroot;
+	linkptr1 = (char **)&simals_setroot;
 	while (1)
 	{
 		if ((*linkptr1) == 0)
 		{
-			*linkptr1 = (char*)linkhead;
+			*linkptr1 = (char *)linkhead;
 			return(linkptr1);
 		}
-		linkptr2 = (LINKPTR) *linkptr1;
-		if ((linkptr2->time > linkhead->time) || ((linkptr2->time == linkhead->time) &&
-			(linkptr2->priority > linkhead->priority)))
+		linkptr2 = (LINKPTR)*linkptr1;
+		if (linkptr2->time > linkhead->time || (linkptr2->time == linkhead->time &&
+			linkptr2->priority > linkhead->priority))
 		{
 			linkhead->right = linkptr2;
-			*linkptr1 = (char*) linkhead;
+			*linkptr1 = (char *)linkhead;
 			return(linkptr1);
 		}
-		linkptr1 = (char**) &(linkptr2->right);
+		linkptr1 = (char **)&(linkptr2->right);
 	}
 }
 
@@ -662,17 +721,14 @@ void simals_insert_link_list(LINKPTR linkhead)
 	char **linkptr1;
 	LINKPTR linkptr2, linkptr3;
 
-	/* if (linkhead->type == 'G')
-		ttyputmsg(_("Schedule: $N%ld"), ((STATPTR) linkhead->ptr)->nodeptr->num); */
-
-	linkptr1 = (char**) &simals_linkback;
+	linkptr1 = (char **)&simals_linkback;
 	linkptr2 = simals_linkback;
 	linkptr3 = 0;
 	while (1)
 	{
 		if (! linkptr2)
 		{
-			simals_linkfront = (LINKPTR)(*linkptr1 = (char*)linkhead);
+			simals_linkfront = (LINKPTR)(*linkptr1 = (char *)linkhead);
 			linkhead->left = 0;
 			linkhead->right = linkptr3;
 			linkhead->up = linkhead;
@@ -682,7 +738,7 @@ void simals_insert_link_list(LINKPTR linkhead)
 
 		if (linkptr2->time < linkhead->time)
 		{
-			*linkptr1 = (char*)(linkptr2->right = linkhead);
+			*linkptr1 = (char *)(linkptr2->right = linkhead);
 			linkhead->left = linkptr2;
 			linkhead->right = linkptr3;
 			linkhead->up = linkhead;
@@ -698,7 +754,7 @@ void simals_insert_link_list(LINKPTR linkhead)
 				linkhead->right = linkptr2->right;
 				linkhead->down = linkptr2;
 				linkhead->up = linkptr2->up;
-				linkptr2->up = (LINKPTR)(*linkptr1 = (char*)linkhead);
+				linkptr2->up = (LINKPTR)(*linkptr1 = (char *)linkhead);
 				if (linkhead->left)
 				{
 					linkhead->left->right = linkhead;
@@ -709,27 +765,27 @@ void simals_insert_link_list(LINKPTR linkhead)
 				return;
 			}
 
-			linkptr1 = (char**) &(linkptr2->up);
+			linkptr1 = (char **)&(linkptr2->up);
 			linkptr2 = linkptr2->up;
 			linkptr3 = 0;
 			while (1)
 			{
 				if (linkptr2->priority <= linkhead->priority)
 				{
-					*linkptr1 = (char*)(linkptr2->down = linkhead);
+					*linkptr1 = (char *)(linkptr2->down = linkhead);
 					linkhead->up = linkptr2;
 					linkhead->down = linkptr3;
 					return;
 				}
 
 				linkptr3 = linkptr2;
-				linkptr1 = (char**) &(linkptr2->up);
+				linkptr1 = (char **)&(linkptr2->up);
 				linkptr2 = linkptr2->up;
 			}
 		}
 
 		linkptr3 = linkptr2;
-		linkptr1 = (char**) &(linkptr2->left);
+		linkptr1 = (char **)&(linkptr2->left);
 		linkptr2 = linkptr2->left;
 	}
 }
