@@ -435,7 +435,7 @@ char *sim_fasthenrygetarcoptions(ARCINST *ai, INTBIG *thickness, INTBIG *widsubd
 	if (var != NOVARIABLE) *heisubdiv = var->addr; else *heisubdiv = -1;
 
 	/* get z depth and any overrides */
-	total = arcpolys(ai);
+	total = arcpolys(ai, NOWINDOWPART);
 	for(i=0; i<total; i++)
 	{
 		shapearcpoly(ai, i, poly);
@@ -495,26 +495,24 @@ DIALOGITEM sim_fasthenrydialogitems[] =
 };
 DIALOG sim_fasthenrydialog = {{75,75,276,564}, N_("FastHenry Options"), 0, 24, sim_fasthenrydialogitems};
 
-/*
- * special case for the "FastHenry options" dialog
- *
- * frequency start       =  4 (edit text)
- * frequency end label   =  5 (static text)
- * frequency end         =  6 (edit text)
- * runs per decade label =  5 (static text)
- * runs per decade       =  8 (edit text)
- * use single frequency  =  9 (check)
- * number of poles label = 10 (static text)
- * number of poles       = 11 (edit text)
- * make multipole subckt = 12 (check)
- * make PostScript view  = 13 (check)
- * max seg length        = 15 (edit text)
- * default width subdiv  = 17 (edit text)
- * default height subdiv = 19 (edit text)
- * make SPICE subckt     = 20 (check)
- * default thickness     = 22 (edit text)
- * after deck writing    = 23 (popup)
- */
+/* special items for the "FastHenry options" dialog: */
+#define DFHO_FREQSTART      4		/* frequency start (edit text) */
+#define DFHO_FREQEND_L      5		/* frequency end label (stat text) */
+#define DFHO_FREQEND        6		/* frequency end (edit text) */
+#define DFHO_RUNSPERDEC_L   7		/* runs per decade label (stat text) */
+#define DFHO_RUNSPERDEC     8		/* runs per decade (edit text) */
+#define DFHO_USESINGLEFREQ  9		/* use single frequency (check) */
+#define DFHO_NUMPOLES_L    10		/* number of poles label (stat text) */
+#define DFHO_NUMPOLES      11		/* number of poles (edit text) */
+#define DFHO_MULTIPOLE     12		/* make multipole subckt (check) */
+#define DFHO_POSTSCRIPT    13		/* make PostScript view (check) */
+#define DFHO_MAXSEGLEN     15		/* max seg length (edit text) */
+#define DFHO_DEFWIDSUB     17		/* default width subdiv (edit text) */
+#define DFHO_DEFHEISUB     19		/* default height subdiv (edit text) */
+#define DFHO_MAKESPICE     20		/* make SPICE subckt (check) */
+#define DFHO_DEFTHICK      22		/* default thickness (edit text) */
+#define DFHO_AFTERWRITE    23		/* after deck writing (popup) */
+
 void sim_fasthenrydlog(void)
 {
 	REGISTER INTBIG itemHit, value, canexecute, i;
@@ -534,97 +532,104 @@ void sim_fasthenrydlog(void)
 	if (DiaInitDialog(&sim_fasthenrydialog) != 0) return;
 
 	/* set popup */
-	for(i=0; i<3; i++) newlang[i] = _(exechoices[i]);   DiaSetPopup(23, 3, newlang);
+	for(i=0; i<3; i++) newlang[i] = _(exechoices[i]);
+	DiaSetPopup(DFHO_AFTERWRITE, 3, newlang);
 	canexecute = graphicshas(CANRUNPROCESS);
 	if (canexecute != 0)
 	{
-		if ((options&FHEXECUTETYPE) == FHEXECUTERUNFH) DiaSetPopupEntry(23, 1); else
-			if ((options&FHEXECUTETYPE) == FHEXECUTERUNFHMUL) DiaSetPopupEntry(23, 2);
+		if ((options&FHEXECUTETYPE) == FHEXECUTERUNFH)
+		{
+			DiaSetPopupEntry(DFHO_AFTERWRITE, 1);
+		} else if ((options&FHEXECUTETYPE) == FHEXECUTERUNFHMUL)
+		{
+			DiaSetPopupEntry(DFHO_AFTERWRITE, 2);
+		}
 	} else
 	{
-		DiaDimItem(23);
+		DiaDimItem(DFHO_AFTERWRITE);
 	}
 
 	/* set checkboxes */
-	if ((options&FHUSESINGLEFREQ) != 0) DiaSetControl(9, 1);
-	if ((options&FHMAKEMULTIPOLECKT) != 0) DiaSetControl(12, 1);
-	if ((options&FHMAKEPOSTSCRIPTVIEW) != 0) DiaSetControl(13, 1);
-	if ((options&FHMAKESPICESUBCKT) != 0) DiaSetControl(20, 1);
+	if ((options&FHUSESINGLEFREQ) != 0) DiaSetControl(DFHO_USESINGLEFREQ, 1);
+	if ((options&FHMAKEMULTIPOLECKT) != 0) DiaSetControl(DFHO_MULTIPOLE, 1);
+	if ((options&FHMAKEPOSTSCRIPTVIEW) != 0) DiaSetControl(DFHO_POSTSCRIPT, 1);
+	if ((options&FHMAKESPICESUBCKT) != 0) DiaSetControl(DFHO_MAKESPICE, 1);
 
 	/* load default frequency range */
-	sprintf(line, "%g", startfreq);   DiaSetText(4, line);
-	sprintf(line, "%g", endfreq);   DiaSetText(6, line);
-	sprintf(line, "%ld", runsperdecade);   DiaSetText(8, line);
+	sprintf(line, "%g", startfreq);   DiaSetText(DFHO_FREQSTART, line);
+	sprintf(line, "%g", endfreq);   DiaSetText(DFHO_FREQEND, line);
+	sprintf(line, "%ld", runsperdecade);   DiaSetText(DFHO_RUNSPERDEC, line);
 
 	/* load segment limits */
-	DiaSetText(15, latoa(seglenlimit));
-	DiaSetText(22, latoa(thickness));
-	sprintf(line, "%ld", widsubdiv);   DiaSetText(17, line);
-	sprintf(line, "%ld", heisubdiv);   DiaSetText(19, line);
+	DiaSetText(DFHO_MAXSEGLEN, latoa(seglenlimit));
+	DiaSetText(DFHO_DEFTHICK, latoa(thickness));
+	sprintf(line, "%ld", widsubdiv);   DiaSetText(DFHO_DEFWIDSUB, line);
+	sprintf(line, "%ld", heisubdiv);   DiaSetText(DFHO_DEFHEISUB, line);
 
 	/* load other numeric options */
-	sprintf(line, "%ld", numpoles);   DiaSetText(11, line);
+	sprintf(line, "%ld", numpoles);   DiaSetText(DFHO_NUMPOLES, line);
 
 	if ((options&FHMAKEMULTIPOLECKT) != 0)
 	{
-		DiaUnDimItem(10);
-		DiaEditControl(11);
+		DiaUnDimItem(DFHO_NUMPOLES_L);
+		DiaEditControl(DFHO_NUMPOLES);
 	} else
 	{
-		DiaDimItem(10);
-		DiaNoEditControl(11);
+		DiaDimItem(DFHO_NUMPOLES_L);
+		DiaNoEditControl(DFHO_NUMPOLES);
 	}
 	if ((options&FHUSESINGLEFREQ) != 0)
 	{
-		DiaDimItem(5);
-		DiaNoEditControl(6);
-		DiaDimItem(7);
-		DiaNoEditControl(8);
+		DiaDimItem(DFHO_FREQEND_L);
+		DiaNoEditControl(DFHO_FREQEND);
+		DiaDimItem(DFHO_RUNSPERDEC_L);
+		DiaNoEditControl(DFHO_RUNSPERDEC);
 	} else
 	{
-		DiaUnDimItem(5);
-		DiaEditControl(6);
-		DiaUnDimItem(7);
-		DiaEditControl(8);
+		DiaUnDimItem(DFHO_FREQEND_L);
+		DiaEditControl(DFHO_FREQEND);
+		DiaUnDimItem(DFHO_RUNSPERDEC_L);
+		DiaEditControl(DFHO_RUNSPERDEC);
 	}
-	DiaDimItem(13);
-	DiaNoEditControl(15);
-	DiaDimItem(20);
+	DiaDimItem(DFHO_POSTSCRIPT);
+	DiaNoEditControl(DFHO_MAXSEGLEN);
+	DiaDimItem(DFHO_MAKESPICE);
 
 	for(;;)
 	{
 		itemHit = DiaNextHit();
 		if (itemHit == OK || itemHit == CANCEL) break;
-		if (itemHit == 9 || itemHit == 12 || itemHit == 13 || itemHit == 20)
+		if (itemHit == DFHO_USESINGLEFREQ || itemHit == DFHO_MULTIPOLE ||
+			itemHit == DFHO_POSTSCRIPT || itemHit == DFHO_MAKESPICE)
 		{
 			value = 1 - DiaGetControl(itemHit);
 			DiaSetControl(itemHit, value);
-			if (itemHit == 9)
+			if (itemHit == DFHO_USESINGLEFREQ)
 			{
 				if (value != 0)
 				{
-					DiaDimItem(5);
-					DiaNoEditControl(6);
-					DiaDimItem(7);
-					DiaNoEditControl(8);
+					DiaDimItem(DFHO_FREQEND_L);
+					DiaNoEditControl(DFHO_FREQEND);
+					DiaDimItem(DFHO_RUNSPERDEC_L);
+					DiaNoEditControl(DFHO_RUNSPERDEC);
 				} else
 				{
-					DiaUnDimItem(5);
-					DiaEditControl(6);
-					DiaUnDimItem(7);
-					DiaEditControl(8);
+					DiaUnDimItem(DFHO_FREQEND_L);
+					DiaEditControl(DFHO_FREQEND);
+					DiaUnDimItem(DFHO_RUNSPERDEC_L);
+					DiaEditControl(DFHO_RUNSPERDEC);
 				}
 			}
-			if (itemHit == 12)
+			if (itemHit == DFHO_MULTIPOLE)
 			{
 				if (value != 0)
 				{
-					DiaUnDimItem(10);
-					DiaEditControl(11);
+					DiaUnDimItem(DFHO_NUMPOLES_L);
+					DiaEditControl(DFHO_NUMPOLES);
 				} else
 				{
-					DiaDimItem(10);
-					DiaNoEditControl(11);
+					DiaDimItem(DFHO_NUMPOLES_L);
+					DiaNoEditControl(DFHO_NUMPOLES);
 				}
 			}
 			continue;
@@ -635,39 +640,39 @@ void sim_fasthenrydlog(void)
 	{
 		/* save options */
 		value = 0;
-		if (DiaGetControl(9) != 0) value |= FHUSESINGLEFREQ;
-		if (DiaGetControl(12) != 0) value |= FHMAKEMULTIPOLECKT;
-		if (DiaGetControl(13) != 0) value |= FHMAKEPOSTSCRIPTVIEW;
-		if (DiaGetControl(20) != 0) value |= FHMAKESPICESUBCKT;
-		i = DiaGetPopupEntry(23);
+		if (DiaGetControl(DFHO_USESINGLEFREQ) != 0) value |= FHUSESINGLEFREQ;
+		if (DiaGetControl(DFHO_MULTIPOLE) != 0) value |= FHMAKEMULTIPOLECKT;
+		if (DiaGetControl(DFHO_POSTSCRIPT) != 0) value |= FHMAKEPOSTSCRIPTVIEW;
+		if (DiaGetControl(DFHO_MAKESPICE) != 0) value |= FHMAKESPICESUBCKT;
+		i = DiaGetPopupEntry(DFHO_AFTERWRITE);
 		if (i == 1) value |= FHEXECUTERUNFH; else
 			if (i == 2) value |= FHEXECUTERUNFHMUL;
 		if (value != options)
 			(void)setvalkey((INTBIG)sim_aid, VAID, sim_fasthenrystatekey, value, VINTEGER);
 
 		/* save other limits */
-		fvalue = (float)atof(DiaGetText(4));
+		fvalue = (float)atof(DiaGetText(DFHO_FREQSTART));
 		if (fvalue != startfreq)
 			(void)setvalkey((INTBIG)sim_aid, VAID, sim_fasthenryfreqstartkey, castint(fvalue), VFLOAT);
-		fvalue = (float)atof(DiaGetText(6));
+		fvalue = (float)atof(DiaGetText(DFHO_FREQEND));
 		if (fvalue != endfreq)
 			(void)setvalkey((INTBIG)sim_aid, VAID, sim_fasthenryfreqendkey, castint(fvalue), VFLOAT);
-		value = atoi(DiaGetText(8));
+		value = atoi(DiaGetText(DFHO_RUNSPERDEC));
 		if (value != runsperdecade)
 			(void)setvalkey((INTBIG)sim_aid, VAID, sim_fasthenryrunsperdecadekey, value, VINTEGER);
-		value = atola(DiaGetText(15));
+		value = atola(DiaGetText(DFHO_MAXSEGLEN));
 		if (value != seglenlimit)
 			(void)setvalkey((INTBIG)sim_aid, VAID, sim_fasthenryseglimitkey, value, VINTEGER);
-		value = atola(DiaGetText(22));
+		value = atola(DiaGetText(DFHO_DEFTHICK));
 		if (value != thickness)
 			(void)setvalkey((INTBIG)sim_aid, VAID, sim_fasthenrythicknesskey, value, VINTEGER);
-		value = atoi(DiaGetText(17));
+		value = atoi(DiaGetText(DFHO_DEFWIDSUB));
 		if (value != widsubdiv)
 			(void)setvalkey((INTBIG)sim_aid, VAID, sim_fasthenrywidthsubdivkey, value, VINTEGER);
-		value = atoi(DiaGetText(19));
+		value = atoi(DiaGetText(DFHO_DEFHEISUB));
 		if (value != heisubdiv)
 			(void)setvalkey((INTBIG)sim_aid, VAID, sim_fasthenryheightsubdivkey, value, VINTEGER);
-		value = atoi(DiaGetText(11));
+		value = atoi(DiaGetText(DFHO_NUMPOLES));
 		if (value != numpoles)
 			(void)setvalkey((INTBIG)sim_aid, VAID, sim_fasthenrynumpoleskey, value, VINTEGER);
 	}
@@ -712,31 +717,28 @@ DIALOGITEM sim_fasthenryarcdialogitems[] =
 };
 DIALOG sim_fasthenryarcdialog = {{75,75,404,401}, N_("FastHenry Arc Properties"), 0, 32, sim_fasthenryarcdialogitems};
 
-/*
- * special case for the "FastHenry arc" dialog
- *
- * arc thickness label   =  3 (static text)
- * arc thickness         =  4 (edit text)
- * arc width             =  6 (static text)
- * width subdiv label    =  7 (edit text)
- * width subdivisions    =  8 (edit text)
- * height subdiv label   =  9 (edit text)
- * height subdivisions   = 10 (edit text)
- * arc head X            = 12 (static text)
- * arc head Y            = 15 (static text)
- * arc head Z label      = 16 (static text)
- * arc head Z            = 17 (edit text)
- * arc tail X            = 19 (static text)
- * arc tail Y            = 22 (static text)
- * arc tail Z label      = 23 (static text)
- * arc tail Z            = 24 (edit text)
- * group name label      = 25 (static text)
- * group name            = 26 (popup)
- * make new group        = 27 (button)
- * include in FastHenry  = 30 (check)
- * default z label       = 31 (static text)
- * default z             = 32 (static text)
- */
+/* special items for the "FastHenry arc" dialog: */
+#define DFHA_ARCTHICK_L      3		/* arc thickness label (stat text) */
+#define DFHA_ARCTHICK        4		/* arc thickness (edit text) */
+#define DFHA_ARCWIDTH        6		/* arc width (stat text) */
+#define DFHA_WIDTHSUBDIV_L   7		/* width subdiv label (edit text) */
+#define DFHA_WIDTHSUBDIV     8		/* width subdivisions (edit text) */
+#define DFHA_HEIGHTSUBDIV_L  9		/* height subdiv label (edit text) */
+#define DFHA_HEIGHTSUBDIV   10		/* height subdivisions (edit text) */
+#define DFHA_ARCHEADX       12		/* arc head X (stat text) */
+#define DFHA_ARCHEADY       15		/* arc head Y (stat text) */
+#define DFHA_ARCHEADZ_L     16		/* arc head Z label (stat text) */
+#define DFHA_ARCHEADZ       17		/* arc head Z (edit text) */
+#define DFHA_ARCTAILX       19		/* arc tail X (stat text) */
+#define DFHA_ARCTAILY       22		/* arc tail Y (stat text) */
+#define DFHA_ARCTAILZ_L     23		/* arc tail Z label (stat text) */
+#define DFHA_ARCTAILZ       24		/* arc tail Z (edit text) */
+#define DFHA_GROUPNAME_L    25		/* group name label (stat text) */
+#define DFHA_GROUPNAME      26		/* group name (popup) */
+#define DFHA_NEWGROUP       27		/* make new group (button) */
+#define DFHA_INCLUDE        30		/* include in FastHenry (check) */
+#define DFHA_DEFZ           32		/* default z (stat text) */
+
 void sim_fasthenryarcdlog(void)
 {
 	REGISTER INTBIG itemHit, value, groupnamesize, i, which, val;
@@ -777,10 +779,12 @@ void sim_fasthenryarcdlog(void)
 			if (i < groupnamesize) continue;
 
 			/* add to the list */
-			newgroupnames = (char **)emalloc((groupnamesize+1) * (sizeof (char *)), sim_aid->cluster);
+			newgroupnames = (char **)emalloc((groupnamesize+1) * (sizeof (char *)),
+				sim_aid->cluster);
 			for(i=0; i<groupnamesize; i++)
 				newgroupnames[i] = groupnames[i];
-			(void)allocstring(&newgroupnames[groupnamesize], (char *)var->addr, sim_aid->cluster);
+			(void)allocstring(&newgroupnames[groupnamesize], (char *)var->addr,
+				sim_aid->cluster);
 			if (groupnamesize > 0) efree((char *)groupnames);
 			groupnames = newgroupnames;
 			groupnamesize++;
@@ -792,123 +796,124 @@ void sim_fasthenryarcdlog(void)
 		(void)allocstring(&groupnames[0], _("Group 1"), sim_aid->cluster);
 		groupnamesize++;
 	}
-	DiaSetPopup(26, groupnamesize, groupnames);
+	DiaSetPopup(DFHA_GROUPNAME, groupnamesize, groupnames);
 
 	/* select group name */
 	if (groupname != 0)
 	{
 		for(i=0; i<groupnamesize; i++)
 			if (namesame(groupname, groupnames[i]) == 0) break;
-		if (i < groupnamesize) DiaSetPopupEntry(26, i);
+		if (i < groupnamesize) DiaSetPopupEntry(DFHA_GROUPNAME, i);
 	}
 
 	/* show arc parameters */
-	sprintf(line, _("Thickness (%s):"), latoa(defthickness));   DiaSetText(3, line);
+	sprintf(line, _("Thickness (%s):"), latoa(defthickness));
+	DiaSetText(DFHA_ARCTHICK_L, line);
 	if (thickness >= 0)
-		DiaSetText(4, latoa(thickness));
-	DiaSetText(6, latoa(ai->width));
+		DiaSetText(DFHA_ARCTHICK, latoa(thickness));
+	DiaSetText(DFHA_ARCWIDTH, latoa(ai->width));
 
 	/* show subdivision overrides */
-	sprintf(line, _("Width subdivisions (%ld):"), defwidsubdiv);   DiaSetText(7, line);
+	sprintf(line, _("Width subdivisions (%ld):"), defwidsubdiv);
+	DiaSetText(DFHA_WIDTHSUBDIV_L, line);
 	if (widsubdiv > 0)
 	{
 		sprintf(line, "%ld", widsubdiv);
-		DiaSetText(8, line);
+		DiaSetText(DFHA_WIDTHSUBDIV, line);
 	}
-	sprintf(line, _("Height subdivisions (%ld):"), defheisubdiv);   DiaSetText(9, line);
+	sprintf(line, _("Height subdivisions (%ld):"), defheisubdiv);
+	DiaSetText(DFHA_HEIGHTSUBDIV_L, line);
 	if (heisubdiv > 0)
 	{
 		sprintf(line, "%ld", heisubdiv);
-		DiaSetText(10, line);
+		DiaSetText(DFHA_HEIGHTSUBDIV, line);
 	}
 
 	/* show end coordinates */
-	DiaSetText(32, latoa(defz));
-	DiaSetText(12, latoa(ai->end[0].xpos));
-	DiaSetText(15, latoa(ai->end[0].ypos));
-	if (zhover != 0)
-		DiaSetText(17, latoa(z_head));
-	DiaSetText(19, latoa(ai->end[1].xpos));
-	DiaSetText(22, latoa(ai->end[1].ypos));
-	if (ztover != 0)
-		DiaSetText(24, latoa(z_tail));
+	DiaSetText(DFHA_DEFZ, latoa(defz));
+	DiaSetText(DFHA_ARCHEADX, latoa(ai->end[0].xpos));
+	DiaSetText(DFHA_ARCHEADY, latoa(ai->end[0].ypos));
+	if (zhover != 0) DiaSetText(DFHA_ARCHEADZ, latoa(z_head));
+	DiaSetText(DFHA_ARCTAILX, latoa(ai->end[1].xpos));
+	DiaSetText(DFHA_ARCTAILY, latoa(ai->end[1].ypos));
+	if (ztover != 0) DiaSetText(DFHA_ARCTAILZ, latoa(z_tail));
 
 	if (groupname != 0)
 	{
-		DiaUnDimItem(3);
-		DiaEditControl(4);
-		DiaUnDimItem(7);
-		DiaEditControl(8);
-		DiaUnDimItem(9);
-		DiaEditControl(10);
-		DiaUnDimItem(16);
-		DiaEditControl(17);
-		DiaUnDimItem(23);
-		DiaEditControl(24);
-		DiaUnDimItem(25);
-		DiaUnDimItem(26);
-		DiaUnDimItem(27);
-		DiaSetControl(30, 1);
+		DiaUnDimItem(DFHA_ARCTHICK_L);
+		DiaEditControl(DFHA_ARCTHICK);
+		DiaUnDimItem(DFHA_WIDTHSUBDIV_L);
+		DiaEditControl(DFHA_WIDTHSUBDIV);
+		DiaUnDimItem(DFHA_HEIGHTSUBDIV_L);
+		DiaEditControl(DFHA_HEIGHTSUBDIV);
+		DiaUnDimItem(DFHA_ARCHEADZ_L);
+		DiaEditControl(DFHA_ARCHEADZ);
+		DiaUnDimItem(DFHA_ARCTAILZ_L);
+		DiaEditControl(DFHA_ARCTAILZ);
+		DiaUnDimItem(DFHA_GROUPNAME_L);
+		DiaUnDimItem(DFHA_GROUPNAME);
+		DiaUnDimItem(DFHA_NEWGROUP);
+		DiaSetControl(DFHA_INCLUDE, 1);
 	} else
 	{
-		DiaDimItem(3);
-		DiaNoEditControl(4);
-		DiaDimItem(7);
-		DiaNoEditControl(8);
-		DiaDimItem(9);
-		DiaNoEditControl(10);
-		DiaDimItem(16);
-		DiaNoEditControl(17);
-		DiaDimItem(23);
-		DiaNoEditControl(24);
-		DiaDimItem(25);
-		DiaDimItem(26);
-		DiaDimItem(27);
+		DiaDimItem(DFHA_ARCTHICK_L);
+		DiaNoEditControl(DFHA_ARCTHICK);
+		DiaDimItem(DFHA_WIDTHSUBDIV_L);
+		DiaNoEditControl(DFHA_WIDTHSUBDIV);
+		DiaDimItem(DFHA_HEIGHTSUBDIV_L);
+		DiaNoEditControl(DFHA_HEIGHTSUBDIV);
+		DiaDimItem(DFHA_ARCHEADZ_L);
+		DiaNoEditControl(DFHA_ARCHEADZ);
+		DiaDimItem(DFHA_ARCTAILZ_L);
+		DiaNoEditControl(DFHA_ARCTAILZ);
+		DiaDimItem(DFHA_GROUPNAME_L);
+		DiaDimItem(DFHA_GROUPNAME);
+		DiaDimItem(DFHA_NEWGROUP);
 	}
 
 	for(;;)
 	{
 		itemHit = DiaNextHit();
 		if (itemHit == OK || itemHit == CANCEL) break;
-		if (itemHit == 30)
+		if (itemHit == DFHA_INCLUDE)
 		{
 			value = 1 - DiaGetControl(itemHit);
 			DiaSetControl(itemHit, value);
 			if (value != 0)
 			{
-				DiaUnDimItem(3);
-				DiaEditControl(4);
-				DiaUnDimItem(7);
-				DiaEditControl(8);
-				DiaUnDimItem(9);
-				DiaEditControl(10);
-				DiaUnDimItem(16);
-				DiaEditControl(17);
-				DiaUnDimItem(23);
-				DiaEditControl(24);
-				DiaUnDimItem(25);
-				DiaUnDimItem(26);
-				DiaUnDimItem(27);
-				DiaSetControl(30, 1);
+				DiaUnDimItem(DFHA_ARCTHICK_L);
+				DiaEditControl(DFHA_ARCTHICK);
+				DiaUnDimItem(DFHA_WIDTHSUBDIV_L);
+				DiaEditControl(DFHA_WIDTHSUBDIV);
+				DiaUnDimItem(DFHA_HEIGHTSUBDIV_L);
+				DiaEditControl(DFHA_HEIGHTSUBDIV);
+				DiaUnDimItem(DFHA_ARCHEADZ_L);
+				DiaEditControl(DFHA_ARCHEADZ);
+				DiaUnDimItem(DFHA_ARCTAILZ_L);
+				DiaEditControl(DFHA_ARCTAILZ);
+				DiaUnDimItem(DFHA_GROUPNAME_L);
+				DiaUnDimItem(DFHA_GROUPNAME);
+				DiaUnDimItem(DFHA_NEWGROUP);
+				DiaSetControl(DFHA_INCLUDE, 1);
 			} else
 			{
-				DiaDimItem(3);
-				DiaNoEditControl(4);
-				DiaDimItem(7);
-				DiaNoEditControl(8);
-				DiaDimItem(9);
-				DiaNoEditControl(10);
-				DiaDimItem(16);
-				DiaNoEditControl(17);
-				DiaDimItem(23);
-				DiaNoEditControl(24);
-				DiaDimItem(25);
-				DiaDimItem(26);
-				DiaDimItem(27);
+				DiaDimItem(DFHA_ARCTHICK_L);
+				DiaNoEditControl(DFHA_ARCTHICK);
+				DiaDimItem(DFHA_WIDTHSUBDIV_L);
+				DiaNoEditControl(DFHA_WIDTHSUBDIV);
+				DiaDimItem(DFHA_HEIGHTSUBDIV_L);
+				DiaNoEditControl(DFHA_HEIGHTSUBDIV);
+				DiaDimItem(DFHA_ARCHEADZ_L);
+				DiaNoEditControl(DFHA_ARCHEADZ);
+				DiaDimItem(DFHA_ARCTAILZ_L);
+				DiaNoEditControl(DFHA_ARCTAILZ);
+				DiaDimItem(DFHA_GROUPNAME_L);
+				DiaDimItem(DFHA_GROUPNAME);
+				DiaDimItem(DFHA_NEWGROUP);
 			}
 			continue;
 		}
-		if (itemHit == 27)		/* new group name */
+		if (itemHit == DFHA_NEWGROUP)		/* new group name */
 		{
 			newname = ttygetline(_("New group name:"));
 			if (newname == 0 || *newname == 0) continue;
@@ -924,9 +929,9 @@ void sim_fasthenryarcdlog(void)
 			if (groupnamesize > 0) efree((char *)groupnames);
 			groupnames = newgroupnames;
 			groupnamesize++;
-			which = DiaGetPopupEntry(26);
-			DiaSetPopup(26, groupnamesize, groupnames);
-			DiaSetPopupEntry(26, groupnamesize-1);
+			which = DiaGetPopupEntry(DFHA_GROUPNAME);
+			DiaSetPopup(DFHA_GROUPNAME, groupnamesize, groupnames);
+			DiaSetPopupEntry(DFHA_GROUPNAME, groupnamesize-1);
 			continue;
 		}
 	}
@@ -935,14 +940,14 @@ void sim_fasthenryarcdlog(void)
 	{
 		startobjectchange((INTBIG)ai, VARCINST);
 		var = getvalkey((INTBIG)ai, VARCINST, VSTRING, sim_fasthenrygroupnamekey);
-		if (DiaGetControl(30) != 0)
+		if (DiaGetControl(DFHA_INCLUDE) != 0)
 		{
 			/* active in FastHenry */
-			i = DiaGetPopupEntry(26);
+			i = DiaGetPopupEntry(DFHA_GROUPNAME);
 			groupname = groupnames[i];
 			if (var == NOVARIABLE || strcmp(groupname, (char *)var->addr) != 0)
-				setvalkey((INTBIG)ai, VARCINST, sim_fasthenrygroupnamekey, (INTBIG)groupname,
-					VSTRING|VDISPLAY);
+				setvalkey((INTBIG)ai, VARCINST, sim_fasthenrygroupnamekey,
+					(INTBIG)groupname, VSTRING|VDISPLAY);
 		} else
 		{
 			if (var != NOVARIABLE)
@@ -950,7 +955,7 @@ void sim_fasthenryarcdlog(void)
 		}
 
 		/* save thickness */
-		pt = DiaGetText(4);
+		pt = DiaGetText(DFHA_ARCTHICK);
 		var = getvalkey((INTBIG)ai, VARCINST, VINTEGER, sim_fasthenrythicknesskey);
 		if (*pt != 0)
 		{
@@ -964,7 +969,7 @@ void sim_fasthenryarcdlog(void)
 		}
 
 		/* save width and height subdivisions */
-		pt = DiaGetText(8);
+		pt = DiaGetText(DFHA_WIDTHSUBDIV);
 		var = getvalkey((INTBIG)ai, VARCINST, VINTEGER, sim_fasthenrywidthsubdivkey);
 		if (*pt != 0)
 		{
@@ -976,7 +981,7 @@ void sim_fasthenryarcdlog(void)
 			if (var != NOVARIABLE)
 				delvalkey((INTBIG)ai, VARCINST, sim_fasthenrywidthsubdivkey);
 		}
-		pt = DiaGetText(10);
+		pt = DiaGetText(DFHA_HEIGHTSUBDIV);
 		var = getvalkey((INTBIG)ai, VARCINST, VINTEGER, sim_fasthenryheightsubdivkey);
 		if (*pt != 0)
 		{
@@ -990,7 +995,7 @@ void sim_fasthenryarcdlog(void)
 		}
 
 		/* save z overrides */
-		pt = DiaGetText(17);
+		pt = DiaGetText(DFHA_ARCHEADZ);
 		var = getvalkey((INTBIG)ai, VARCINST, VINTEGER, sim_fasthenryzheadkey);
 		if (*pt != 0)
 		{
@@ -1002,7 +1007,7 @@ void sim_fasthenryarcdlog(void)
 			if (var != NOVARIABLE)
 				delvalkey((INTBIG)ai, VARCINST, sim_fasthenryzheadkey);
 		}
-		pt = DiaGetText(24);
+		pt = DiaGetText(DFHA_ARCTAILZ);
 		var = getvalkey((INTBIG)ai, VARCINST, VINTEGER, sim_fasthenryztailkey);
 		if (*pt != 0)
 		{
