@@ -72,7 +72,7 @@ INTBIG sim_silos_model_key = 0;		/* for model on cell */
 /* ************************************************************* */
 
 /* prototypes for local routines */
-INTSML sim_writesilfacet(NODEPROTO*, INTSML);
+INTBIG sim_writesilfacet(NODEPROTO*, INTSML);
 void sim_writesilinstances(NODEPROTO*, INTSML);
 char *sim_silportname(INTSML, NODEINST*, PORTPROTO*, NODEPROTO*);
 char *sim_silname(NODEINST*);
@@ -85,7 +85,7 @@ char *sim_silspecials(char*);
 INTSML sim_silostype(NODEINST*, char**);
 char *sim_silrisetime(NODEINST*);
 char *sim_silfalltime(NODEINST*);
-INTSML sim_silgetcapacitance(NODEINST*);
+INTBIG sim_silgetcapacitance(NODEINST*);
 void sim_silwriteflipflop(INTSML, NODEINST*, NODEPROTO*, INTSML);
 char *sim_sil_isaport(NODEPROTO*, char*);
 static char *sim_convertsub(char*);
@@ -179,18 +179,18 @@ void sim_writesilnetlist(NODEPROTO *np)
  * The description is treated as the top-level facet if "top" is nonzero
  * np is the current nodeproto
  */
-INTSML sim_writesilfacet(NODEPROTO *np, INTSML top)
+INTBIG sim_writesilfacet(NODEPROTO *np, INTSML top)
 {
 	char buf[256];
-	REGISTER INTSML i, j, nodetype, clktype, count, backannotate;
+	REGISTER INTBIG i, j, nodetype, clktype, count, backannotate;
 	REGISTER NODEINST *ni;
 	REGISTER PORTPROTO *pp;
 	REGISTER NODEPROTO *onp;
 	REGISTER NETWORK *net;
 	REGISTER VARIABLE *var;
 	REGISTER FILE *f;
-	char line[100], *filename;
-	char *extra, **model, *name;
+	char line[100], *filename, *extra;
+	char **model, *name;
 
 	/* stop if requested */
 	if (el_pleasestop != 0)
@@ -227,19 +227,19 @@ INTSML sim_writesilfacet(NODEPROTO *np, INTSML top)
 				sprintf(line,"$$$$$ Unspecified source: \n");
 			} else	/* There is more */
 			{
-				clktype = 0;	/* Extra data required if 1 */
+				clktype = 0;	/* Extra data required if variable there */
 				for(extra = (char *)var->addr; *extra != 0 && *extra != '/'; extra++)
 				{
-					switch(*extra)
+					switch (*extra)
 					{
-						case 'g':	/* a global clock */
+						case 'g':	/* a global clock (THIS IS WRONG!!!) */
 							(void)sprintf(line, "%s .CLK ", name);
 							clktype = 1;
 							break;
-						case 'h':	/* a fixed high source */
+						case 'h':	/* a fixed high source (THIS IS WRONG!!!) */
 							(void)sprintf(line, "%s .CLK 0 S1 $ HIGH LEVEL", name);
 							break;
-						case 'l':	/* a fixed low source */
+						case 'l':	/* a fixed low source (THIS IS WRONG!!!) */
 							(void)sprintf(line, "%s .CLK 0 S0 $ LOW LEVEL", name);
 							break;
 					}
@@ -380,7 +380,8 @@ INTSML sim_writesilfacet(NODEPROTO *np, INTSML top)
  */
 void sim_writesilinstances(NODEPROTO *np, INTSML top)
 {
-	REGISTER INTSML j, nodetype, schem_ref;
+	REGISTER INTBIG j, schem_ref;
+	REGISTER INTSML nodetype;
 	REGISTER NODEINST *ni;
 	REGISTER PORTPROTO *pp, *lastpp, *outpp;
 	REGISTER PORTARCINST *pi;
@@ -706,16 +707,16 @@ char *sim_silproto(NODEINST *ni,INTSML neg)
 		case SILOSCAP:
 			return(".CAP");
 		case SILOSJKFF:
-			if (namesamen(extra, "jkn", 3) == 0) return(".JKNEFF");
+			if ((ni->userbits&FFCLOCK) == FFCLOCKN) return(".JKNEFF");
 			return(".JKPEFF");
 		case SILOSDFF:
-			if (namesamen(extra, "dn", 2) == 0) return(".DNEFF");
+			if ((ni->userbits&FFCLOCK) == FFCLOCKN) return(".DNEFF");
 			return(".DPEFF");
 		case SILOSSRFF:
-			if (namesamen(extra, "rsn", 3) == 0) return(".SRNEFF");
+			if ((ni->userbits&FFCLOCK) == FFCLOCKN) return(".SRNEFF");
 			return(".SRPEFF");
 		case SILOSTFF:
-			if (namesamen(extra, "tn", 2) == 0) return(".TNEFF");
+			if ((ni->userbits&FFCLOCK) == FFCLOCKN) return(".TNEFF");
 			return(".TPEFF");
 	}
 	return(sim_siltext(ni->proto->primname));
@@ -806,12 +807,12 @@ INTSML sim_silnegated(PORTARCINST *pi)
 
 char *sim_silspecials(char *str)
 {
-	if (namesamen(str, "vdd",3) == 0) return(".VDD");
-	if (namesamen(str, "vss",3) == 0) return(".VSS");
-	if (namesamen(str, "vcc",3) == 0) return(".VCC");
-	if (namesamen(str, "gnd",3) == 0) return(".GND");
-	if (namesamen(str, "low",3) == 0) return(".GND");
-	if (namesamen(str, "hig",3) == 0) return(".VDD");
+	if (namesamen(str, "vdd", 3) == 0) return(".VDD");
+	if (namesamen(str, "vss", 3) == 0) return(".VSS");
+	if (namesamen(str, "vcc", 3) == 0) return(".VCC");
+	if (namesamen(str, "gnd", 3) == 0) return(".GND");
+	if (namesamen(str, "low", 3) == 0) return(".GND");
+	if (namesamen(str, "hig", 3) == 0) return(".VDD");
 	return(str);
 }
 
@@ -822,7 +823,7 @@ char *sim_silspecials(char *str)
  */
 INTSML sim_silostype(NODEINST *ni, char **info)
 {
-	INTSML func;
+	INTBIG func;
 	REGISTER VARIABLE *var;
 	char *string;
 
@@ -833,8 +834,10 @@ INTSML sim_silostype(NODEINST *ni, char **info)
 	switch(func)
 	{
 		case NPTRAPMOS:
+		case NPTRA4PMOS:
 			return(SILOSPMOS);
 		case NPTRANMOS:
+		case NPTRA4NMOS:
 			return(SILOSNMOS);
 		case NPGATEAND:
 			return(SILOSAND);
@@ -849,16 +852,13 @@ INTSML sim_silostype(NODEINST *ni, char **info)
 		case NPCAPAC:
 			return(SILOSCAP);
 		case NPFLIPFLOP:	/* We will decode further here */
-			var = getvalkey((INTBIG)ni, VNODEINST, VSTRING, sch_flipfloptypekey);
-			if (var != NOVARIABLE)
+			switch (ni->userbits&FFTYPE)
 			{
-				string = (char *)var->addr;
-				*info = string;
+				case FFTYPERS: return(SILOSSRFF);
+				case FFTYPEJK: return(SILOSJKFF);
+				case FFTYPED:  return(SILOSDFF);
+				case FFTYPET:  return(SILOSTFF);
 			}
-			if (*string == 'd') return(SILOSDFF);
-			if (*string == 'j') return(SILOSJKFF);
-			if (*string == 'r') return(SILOSSRFF);
-			if (*string == 't') return(SILOSTFF);
 			return(SILOSDFF);
 		case NPSOURCE:
 			var = getvalkey((INTBIG)ni, VNODEINST, VSTRING, sim_silos_global_name);
@@ -945,12 +945,12 @@ char *sim_silfalltime(NODEINST *ni)
  * by "SCHEM_capacitance" variable on an instance of a capacitor.
  * Return -1 if nothing found.
  */
-INTSML sim_silgetcapacitance(NODEINST *ni)
+INTBIG sim_silgetcapacitance(NODEINST *ni)
 {
 	REGISTER VARIABLE *var;
 
-	var = getvalkey((INTBIG)ni, VNODEINST, VINTEGER, sch_capacitancekey);
-	if (var != NOVARIABLE) return((INTSML)var->addr);
+	var = getvalkey((INTBIG)ni, VNODEINST, -1, sch_capacitancekey);
+	if (var != NOVARIABLE) return(myatoi(describesimplevariable(var)));
 	return(-1);
 }
 
