@@ -54,7 +54,7 @@ void sim_prop(ARCINST*, INTBIG, INTBIG*, INTBIG*, INTBIG*, INTBIG*, INTBIG*, INT
 
 void sim_writesim(NODEPROTO *simnt, INTSML format)
 {
-	REGISTER INTBIG save0, save1, save2;
+	REGISTER INTBIG save0, save1, save2, filetype;
 	REGISTER INTSML i;
 	REGISTER VARIABLE *var;
 	REGISTER FILE *f;
@@ -96,7 +96,9 @@ void sim_writesim(NODEPROTO *simnt, INTSML format)
 	(void)allocstring(&sim_inputfile, netfile, sim_aid->cluster);
 
 	/* create the simulation file */
-	f = xcreate(netfile, sim_filetypeesim, _("Netlist File"), &truename);
+	if (format == RSIM) filetype = sim_filetypersim; else
+		filetype = sim_filetypeesim;
+	f = xcreate(netfile, filetype, _("Netlist File"), &truename);
 	if (f == NULL)
 	{
 		if (truename != 0) ttyputerr(_("Cannot write %s"), truename);
@@ -127,7 +129,7 @@ void sim_writesim(NODEPROTO *simnt, INTSML format)
 	for(pp = simnt->firstportproto; pp != NOPORTPROTO; pp = pp->nextportproto)
 	{
 		/* see if this port is on the same net number as some other port */
-		for(spt=simnt->firstportproto; spt!=pp; spt=spt->nextportproto)
+		for(spt = simnt->firstportproto; spt != pp; spt = spt->nextportproto)
 			if (spt->network == pp->network) break;
 
 		/* this port is the same as another one, use same net */
@@ -268,7 +270,7 @@ void sim_writesim(NODEPROTO *simnt, INTSML format)
 		{
 			if (count_unconn != 0)
 				ttyputmsg(_("Warning: %ld unconnected nodes were found"), count_unconn);
-			ttyputverbose(_("Created simulation file %s"), netfile);
+			ttyputverbose(M_("Created simulation file %s"), netfile);
 			return;
 		}
 
@@ -329,7 +331,7 @@ void sim_writesim(NODEPROTO *simnt, INTSML format)
 	{
 		if (count_unconn != 0)
 			ttyputmsg(_("Warning: %ld unconnected nodes were found"), count_unconn);
-		ttyputverbose(_("Created simulation file %s"), netfile);
+		ttyputverbose(M_("Created simulation file %s"), netfile);
 		return;
 	}
 
@@ -403,9 +405,9 @@ INTSML sim_simpointout(char *nonum, INTSML format)
 	sim_index = 1;
 	sim_vdd = sim_gnd = 0;
 	sim_phi1h = sim_phi1l = sim_phi2h = sim_phi2l = 0;
-	for(pp=sim_simnt->firstportproto; pp!=NOPORTPROTO; pp=pp->nextportproto)
+	for(pp = sim_simnt->firstportproto; pp != NOPORTPROTO; pp = pp->nextportproto)
 	{
-		for(spt=sim_simnt->firstportproto; spt!=pp; spt=spt->nextportproto)
+		for(spt = sim_simnt->firstportproto; spt != pp; spt = spt->nextportproto)
 		{
 			if (spt->network != pp->network) continue;
 
@@ -486,7 +488,7 @@ INTSML sim_simsearch(INTSML want, NODEPROTO *facet, INTSML format)
 		if (ai->temp1 != 0) continue;
 
 		/* see if this arcinst is connected to a port */
-		for(pp=facet->firstportproto; pp!=NOPORTPROTO; pp=pp->nextportproto)
+		for(pp = facet->firstportproto; pp != NOPORTPROTO; pp = pp->nextportproto)
 		{
 			if (pp->network != ai->network) continue;
 			if (pp->temp1 == want)
@@ -731,8 +733,6 @@ void sim_simprint(NODEPROTO *facet, FILE *f, INTSML format)
 	REGISTER PORTEXPINST *pe;
 	REGISTER NODEINST *ni;
 	REGISTER NODEPROTO *np, *snp;
-	REGISTER VARIABLE *var;
-	REGISTER char *str;
 
 	/* stop if requested */
 	if (stopping(STOPREASONDECK)) return;
@@ -747,7 +747,7 @@ void sim_simprint(NODEPROTO *facet, FILE *f, INTSML format)
 		if (ai->temp1 != 0) continue;
 
 		/* see if this arcinst is connected to a port */
-		for(pp=facet->firstportproto; pp!=NOPORTPROTO; pp=pp->nextportproto)
+		for(pp = facet->firstportproto; pp != NOPORTPROTO; pp = pp->nextportproto)
 		{
 			if (pp->network != ai->network) continue;
 			darea = dperim = parea = pperim = marea = mperim = 0;
@@ -790,7 +790,7 @@ void sim_simprint(NODEPROTO *facet, FILE *f, INTSML format)
 		/* ignore artwork */
 		if (type == NPART) continue;
 
-		for(pp=np->firstportproto; pp!=NOPORTPROTO; pp=pp->nextportproto)
+		for(pp = np->firstportproto; pp != NOPORTPROTO; pp = pp->nextportproto)
 			pp->temp1 = 0;
 
 		/* set network numbers from arcs and exports */
@@ -824,7 +824,7 @@ void sim_simprint(NODEPROTO *facet, FILE *f, INTSML format)
 		}
 
 		/* look for unconnected ports and assign new network numbers */
-		for(pp=np->firstportproto; pp!=NOPORTPROTO; pp=pp->nextportproto)
+		for(pp = np->firstportproto; pp != NOPORTPROTO; pp = pp->nextportproto)
 		{
 			if (pp->temp1 != 0) continue;
 
@@ -858,14 +858,7 @@ void sim_simprint(NODEPROTO *facet, FILE *f, INTSML format)
 			if (type == NPTRANS)
 			{
 				/* undefined transistor: look at its transistor type */
-				var = getvalkey((INTBIG)ni, VNODEINST, VSTRING, sch_transistortypekey);
-				if (var != NOVARIABLE)
-				{
-					str = (char *)var->addr;
-					if (namesamen(str, "nmos", 4) == 0) type=NPTRANMOS; else
-						if (namesamen(str, "dmos", 4) == 0) type=NPTRADMOS; else
-							if (namesamen(str, "pmos", 4) == 0) type=NPTRAPMOS;
-				}
+				type = nodefunction(ni, 0);
 
 				/* gate is port 0, source is port 1, drain is port 2 */
 				p = np->firstportproto;    gate = p->temp1;
@@ -879,10 +872,19 @@ void sim_simprint(NODEPROTO *facet, FILE *f, INTSML format)
 				p = p->nextportproto->nextportproto;   drain = p->temp1;
 			}
 
-			if (type == NPTRANMOS) xprintf(f, "e"); else
-				if (type == NPTRADMOS) xprintf(f, "d"); else
-					if (type == NPTRAPMOS) xprintf(f, "p"); else
-						xprintf(f, "U");
+			if (type == NPTRANMOS)
+			{
+				xprintf(f, "e");
+			} else if (type == NPTRADMOS)
+			{
+				xprintf(f, "d");
+			} else if (type == NPTRAPMOS)
+			{
+				xprintf(f, "p");
+			} else
+			{
+				xprintf(f, "U");
+			}
 			xprintf(f, " %s", sim_makenodename(gate, format));
 			xprintf(f, " %s", sim_makenodename(source, format));
 			xprintf(f, " %s", sim_makenodename(drain, format));
@@ -926,7 +928,6 @@ void sim_simprintcosmos(NODEPROTO *facet, FILE *f, INTSML format)
 	REGISTER NODEINST *ni;
 	REGISTER NODEPROTO *np, *snp;
 	REGISTER VARIABLE *var;
-	REGISTER char *str;
 
 	/* stop if requested */
 	if (stopping(STOPREASONDECK)) return;
@@ -946,7 +947,7 @@ void sim_simprintcosmos(NODEPROTO *facet, FILE *f, INTSML format)
 		if (ai->temp1 != 0) continue;
 
 		/* see if this arcinst is connected to a port */
-		for(pp=facet->firstportproto; pp!=NOPORTPROTO; pp=pp->nextportproto)
+		for(pp = facet->firstportproto; pp != NOPORTPROTO; pp = pp->nextportproto)
 		{
 			if (pp->network != ai->network) continue;
 			darea = dperim = parea = pperim = marea = mperim = 0;
@@ -1001,7 +1002,7 @@ void sim_simprintcosmos(NODEPROTO *facet, FILE *f, INTSML format)
 		/* ignore artwork */
 		if (type == NPART) continue;
 
-		for(pp=np->firstportproto; pp!=NOPORTPROTO; pp=pp->nextportproto)
+		for(pp = np->firstportproto; pp != NOPORTPROTO; pp = pp->nextportproto)
 			pp->temp1 = 0;
 
 		/* set network numbers from arcs and exports */
@@ -1035,7 +1036,7 @@ void sim_simprintcosmos(NODEPROTO *facet, FILE *f, INTSML format)
 		}
 
 		/* look for unconnected ports and assign new network numbers */
-		for(pp=np->firstportproto; pp!=NOPORTPROTO; pp=pp->nextportproto)
+		for(pp = np->firstportproto; pp != NOPORTPROTO; pp = pp->nextportproto)
 		{
 			if (pp->temp1 != 0) continue;
 
@@ -1070,14 +1071,7 @@ void sim_simprintcosmos(NODEPROTO *facet, FILE *f, INTSML format)
 			if (type == NPTRANS)
 			{
 				/* undefined transistor: look at its transistor type */
-				var = getvalkey((INTBIG)ni, VNODEINST, VSTRING, sch_transistortypekey);
-				if (var != NOVARIABLE)
-				{
-					str = (char *)var->addr;
-					if (namesamen(str, "nmos", 4) == 0) type=NPTRANMOS; else
-						if (namesamen(str, "dmos", 4) == 0) type=NPTRADMOS; else
-							if (namesamen(str, "pmos", 4) == 0) type=NPTRAPMOS;
-				}
+				type = nodefunction(ni, 0);
 
 				/* gate is port 0, source is port 1, drain is port 2 */
 				gate = np->firstportproto;
