@@ -206,12 +206,6 @@
  *    circuit. The character '/' is reserved as a separator for length
  *    and width values, and if present, causes the default type to be
  *    invoked.)
- *    If you create the transistor manually, then the variable
- *    "SCHEM_transistor_type" will be of the form:  tttNNNNNNN
- *    where: ttt is one of "npn", "pnp", "pjfet", or "njfet"
- *           NNNNNNNN is your model name, ex: LARGE_NPN
- *    Note: you can also define model names for the types "dmes", "emes",
- *    "nmos" "pmos" and  "dmos", which are helpful in schematic designs.
  *
  * (11) You can use the contents of a file to replace the extracted description
  *    of any facet, by attaching the name of the file as a variable
@@ -276,41 +270,50 @@
 #define SPICEFACET     -1	/* sub-facet */
 #define SPICEUNKNOWN    0	/* unimportant node (all connected) */
 #define SPICENMOS       1	/* nMOS transistor */
-#define SPICEDMOS       2	/* DMOD transistor */
-#define SPICEPMOS       3	/* PMOS transistor */
-#define SPICENPN        4	/* NPN transistor */
-#define SPICEPNP        5	/* PNP transistor */
-#define SPICENJFET      6	/* N-JFET transistor */
-#define SPICEPJFET      7	/* P-JFET transistor */
-#define SPICEDMES       8	/* Depletion MESFET transistor */
-#define SPICEEMES       9	/* Enhancement MESFET transistor */
-#define SPICEREF        10	/* Self-referential transistor */
-#define SPICEMETER      11	/* meter */
-#define SPICESOURCEV    12	/* voltage source */
-#define SPICESOURCEC    13	/* current source */
-#define SPICESOURCECM   14	/* current meter */
-#define SPICESOURCET    15	/* transient analysis specification */
-#define SPICESOURCEDC   16	/* DC analysis specification */
-#define SPICESOURCEAC   17	/* AC analysis specification */
-#define SPICESOURCEN    18	/* NodeSet source */
-#define SPICESOURCEX    19	/* Extension source */
-#define SPICESOURCEB    20	/* Bulk source */
-#define SPICESOURCES    21	/* Special source */
-#define SPICERESISTOR   22	/* resistor */
-#define SPICECAPACITOR  23	/* capacitor */
-#define SPICEINDUCTOR   24	/* inductor */
-#define SPICEDIODE      25	/* diode */
-#define SPICEDIODEZ     26	/* Zener diode */
-#define SPICEARRAY      27	/* Multiport/function device */
-#define SPICESUBSTRATE  28	/* connection to substrate */
-#define SPICETRANS      29	/* generic (special) transistor */
-#define SPICEGROUND     30	/* A node connected to ground */
-#define SPICEPOWER      31	/* A node connected to power */
-#define SPICECCCS       32	/* Current Controlled Current Source */
-#define SPICECCVS       33	/* Current Controlled Voltage Source */
-#define SPICEVCCS       34	/* Voltage Controlled Current Source */
-#define SPICEVCVS       35	/* Voltage Controlled Voltage Source */
-#define SPICETLINE      36	/* Transmission Line */
+#define SPICENMOS4      2	/* nMOS 4-port transistor */
+#define SPICEDMOS       3	/* DMOS transistor */
+#define SPICEDMOS4      4	/* DMOS 4-port transistor */
+#define SPICEPMOS       5	/* PMOS transistor */
+#define SPICEPMOS4      6	/* PMOS 4-port transistor */
+#define SPICENPN        7	/* NPN transistor */
+#define SPICENPN4       8	/* NPN 4-port transistor */
+#define SPICEPNP        9	/* PNP transistor */
+#define SPICEPNP4       10	/* PNP 4-port transistor */
+#define SPICENJFET      11	/* N-JFET transistor */
+#define SPICENJFET4     12	/* N-JFET 4-port transistor */
+#define SPICEPJFET      13	/* P-JFET transistor */
+#define SPICEPJFET4     14	/* P-JFET 4-port transistor */
+#define SPICEDMES       15	/* Depletion MESFET transistor */
+#define SPICEDMES4      16	/* Depletion MESFET 4-port transistor */
+#define SPICEEMES       17	/* Enhancement MESFET transistor */
+#define SPICEEMES4      18	/* Enhancement MESFET 4-port transistor */
+#define SPICEREF        19	/* Self-referential transistor */
+#define SPICEMETER      20	/* meter */
+#define SPICESOURCEV    21	/* voltage source */
+#define SPICESOURCEC    22	/* current source */
+#define SPICESOURCECM   23	/* current meter */
+#define SPICESOURCET    24	/* transient analysis specification */
+#define SPICESOURCEDC   25	/* DC analysis specification */
+#define SPICESOURCEAC   26	/* AC analysis specification */
+#define SPICESOURCEN    27	/* NodeSet source */
+#define SPICESOURCEX    28	/* Extension source */
+#define SPICESOURCEB    29	/* Bulk source */
+#define SPICESOURCES    30	/* Special source */
+#define SPICERESISTOR   31	/* resistor */
+#define SPICECAPACITOR  32	/* capacitor */
+#define SPICEINDUCTOR   33	/* inductor */
+#define SPICEDIODE      34	/* diode */
+#define SPICEDIODEZ     35	/* Zener diode */
+#define SPICEARRAY      36	/* Multiport/function device */
+#define SPICESUBSTRATE  37	/* connection to substrate */
+#define SPICETRANS      38	/* generic (special) transistor */
+#define SPICEGROUND     39	/* A node connected to ground */
+#define SPICEPOWER      40	/* A node connected to power */
+#define SPICECCCS       41	/* Current Controlled Current Source */
+#define SPICECCVS       42	/* Current Controlled Voltage Source */
+#define SPICEVCCS       43	/* Voltage Controlled Current Source */
+#define SPICEVCVS       44	/* Voltage Controlled Voltage Source */
+#define SPICETLINE      45	/* Transmission Line */
 
 /* Types of spice sources */
 #define SPICENORMAL     0	/* normal source */
@@ -714,11 +717,12 @@ INTSML sim_spicewritefacet(NODEPROTO *np, FILE *f, INTSML top)
 {
 	REGISTER NODEINST *ni;
 	REGISTER NODEPROTO *subnt, *tnp, *cnp;
-	REGISTER PORTPROTO *pp, *posport, *negport, *tpp, *otpp;
+	REGISTER PORTPROTO *pp, *posport, *negport, *tpp, *otpp, *biasport;
 	PORTPROTO *gate, *source, *drain, *gatedummy;
 	REGISTER PORTARCINST *pi;
 	REGISTER ARCINST *ai;
-	REGISTER NET *net, *gaten, *gatedn, *sourcen, *drainn, *nnet, *posnet, *negnet, *subnet, *tempnet;
+	REGISTER NET *net, *gaten, *gatedn, *sourcen, *drainn, *nnet, *posnet, *negnet, *subnet, *tempnet,
+		*biasn;
 	float a, b;
 	REGISTER INTSML netindex, state, j, first, found, meterit, i, nodetype, backannotate, retval;
 	INTSML subfacetindex, nodeindex, resistnum, capacnum, inductnum, diodenum;
@@ -789,10 +793,8 @@ INTSML sim_spicewritefacet(NODEPROTO *np, FILE *f, INTSML top)
 				(void)xputc(c, f);
 				c = (char)xgetc(modelfile);
 			}
-#if 1
 			for(i=1, pp = np->firstportproto; pp != NOPORTPROTO; pp = pp->nextportproto)
 				pp->temp1 = i++;
-#endif
 			return(backannotate);
 		} else ttyputerr(_("Warning: cannot open model file: %s"), (char *)var->addr);
 	}
@@ -834,7 +836,7 @@ INTSML sim_spicewritefacet(NODEPROTO *np, FILE *f, INTSML top)
 		net->nextnet = sim_spice_firstnet;
 		sim_spice_firstnet = net;
 
-		if (sim_spice_debug != 0) ttyputmsg(_("Starting net %ld"), net->netnumber);
+		if (sim_spice_debug != 0) ttyputmsg(M_("Starting net %ld"), net->netnumber);
 		sim_spice_arcarea(net, ai); /* builds areas, etc */
 
 		/* get merged polygons so far */
@@ -887,11 +889,16 @@ INTSML sim_spicewritefacet(NODEPROTO *np, FILE *f, INTSML top)
 		switch (state)
 		{
 			case SPICEEMES:
+			case SPICEEMES4:
 			case SPICEDMES:
+			case SPICEDMES4:
 			case SPICEDMOS:
-			case SPICENMOS: nodetype = ISNTYPE; break;
-			case SPICEPMOS: nodetype = ISPTYPE; break;
-			default:        nodetype = ISNONE;  break;
+			case SPICEDMOS4:
+			case SPICENMOS:
+			case SPICENMOS4: nodetype = ISNTYPE; break;
+			case SPICEPMOS:
+			case SPICEPMOS4: nodetype = ISPTYPE; break;
+			default:         nodetype = ISNONE;  break;
 		}
 
 		/* Only count drains and source connections in counting transistors */
@@ -901,7 +908,9 @@ INTSML sim_spicewritefacet(NODEPROTO *np, FILE *f, INTSML top)
 		{
 			transistorports(ni, &gate, &gatedummy, &source, &drain);
 			if (pp->subportproto == source || pp->subportproto == drain)
+			{
 				net->components[nodetype]++;
+			}
 		}
 		net->nextnet = sim_spice_firstnet;
 		sim_spice_firstnet = net;
@@ -961,19 +970,26 @@ INTSML sim_spicewritefacet(NODEPROTO *np, FILE *f, INTSML top)
 		switch (state)
 		{
 			case SPICENPN:
+			case SPICENPN4:
 			case SPICEPNP:
+			case SPICEPNP4:
 			case SPICETRANS:
 				nodetype = ISNONE;
 				bipolars++;
 				break;
 			case SPICEEMES:
+			case SPICEEMES4:
 			case SPICEDMES:
+			case SPICEDMES4:
 			case SPICEDMOS:
+			case SPICEDMOS4:
 			case SPICENMOS:
+			case SPICENMOS4:
 				nodetype = ISNTYPE;
 				nmostrans++;
 				break;
 			case SPICEPMOS:
+			case SPICEPMOS4:
 				nodetype = ISPTYPE;
 				pmostrans++;
 				break;
@@ -997,7 +1013,9 @@ INTSML sim_spicewritefacet(NODEPROTO *np, FILE *f, INTSML top)
 					{
 						transistorports(ni, &gate, &gatedummy, &source, &drain);
 						if (pi->proto == source || pi->proto == drain)
+						{
 							net->components[nodetype]++;
+						}
 					}
 				}
 			}
@@ -1082,7 +1100,7 @@ INTSML sim_spicewritefacet(NODEPROTO *np, FILE *f, INTSML top)
 		{
 			/* do not write the ground net */
 			if (pp->temp1 == 0) continue;
-			for (tpp = np->firstportproto; tpp!=pp; tpp=tpp->nextportproto)
+			for (tpp = np->firstportproto; tpp != pp; tpp = tpp->nextportproto)
 				if (pp->temp1 == tpp->temp1) break;	/* Already done */
 			if (tpp != pp) continue;
 			sim_spice_xprintf(f, "** PORT %s", sim_spice_pp_name(pp));
@@ -1231,7 +1249,7 @@ INTSML sim_spicewritefacet(NODEPROTO *np, FILE *f, INTSML top)
 
 		switch (state)
 		{
-			case SPICEREF:	    /* self-referential transistor */
+			case SPICEREF:			/* self-referential transistor */
 				sim_spice_xputs(sim_spice_element_name(ni, 'X', &subfacetindex), f);
 				sim_spice_xputs(sim_spice_node_name(drainn), f);
 				sim_spice_xputs(sim_spice_node_name(gaten), f);
@@ -1239,7 +1257,7 @@ INTSML sim_spicewritefacet(NODEPROTO *np, FILE *f, INTSML top)
 				sim_spice_xputs(sim_spice_node_name(negnet), f);
 				sim_spice_xprintf(f, " %s", ni->proto->primname);
 				break;
-			case SPICEVCVS:	    /* current controlled Current source */
+			case SPICEVCVS:			/* current controlled Current source */
 				sim_spice_xputs(sim_spice_element_name(ni, 'E', &nodeindex), f);
 				sim_spice_xputs(sim_spice_node_name(gatedn), f);
 				sim_spice_xputs(sim_spice_node_name(drainn), f);
@@ -1247,7 +1265,7 @@ INTSML sim_spicewritefacet(NODEPROTO *np, FILE *f, INTSML top)
 				sim_spice_xputs(sim_spice_node_name(sourcen), f);
 				if (info != NULL) sim_spice_xprintf(f, " %s",info);
 				break;
-			case SPICEVCCS:	    /* current controlled Current source */
+			case SPICEVCCS:			/* current controlled Current source */
 				sim_spice_xputs(sim_spice_element_name(ni, 'G', &nodeindex), f);
 				sim_spice_xputs(sim_spice_node_name(gatedn), f);
 				sim_spice_xputs(sim_spice_node_name(drainn), f);
@@ -1255,7 +1273,7 @@ INTSML sim_spicewritefacet(NODEPROTO *np, FILE *f, INTSML top)
 				sim_spice_xputs(sim_spice_node_name(sourcen), f);
 				if (info != NULL) sim_spice_xprintf(f, " %s",info);
 				break;
-			case SPICECCVS:	    /* current controlled Current source */
+			case SPICECCVS:			/* current controlled Current source */
 				sim_spice_xputs("*CCVS\n", f); 
 				sim_spice_xputs(sim_spice_element_name(ni, 'V', &nodeindex), f);
 				sim_spice_xputs(sim_spice_node_name(gaten), f);
@@ -1269,7 +1287,7 @@ INTSML sim_spicewritefacet(NODEPROTO *np, FILE *f, INTSML top)
 				if (info != NULL) sim_spice_xprintf(f, " %s",info);
 				sim_spice_xputs("\n*", f);
 				break;
-			case SPICECCCS:	    /* current controlled Current source */
+			case SPICECCCS:			/* current controlled Current source */
 				sim_spice_xputs("*CCCS\n", f); 
 				sim_spice_xputs(sim_spice_element_name(ni, 'V', &nodeindex), f);
 				sim_spice_xputs(sim_spice_node_name(gaten), f);
@@ -1291,7 +1309,7 @@ INTSML sim_spicewritefacet(NODEPROTO *np, FILE *f, INTSML top)
 				sim_spice_xputs(sim_spice_node_name(drainn), f);
 				if (info != NULL) sim_spice_xprintf(f, " %s",info);
 				break;
-			case SPICENMOS:	  /* NMOS (Enhancement) transistor */
+			case SPICENMOS:			/* NMOS (Enhancement) transistor */
 				sim_spice_xputs(sim_spice_element_name(ni, 'M', &nodeindex), f);
 				sim_spice_xputs(sim_spice_node_name(drainn), f);
 				sim_spice_xputs(sim_spice_node_name(gaten), f);
@@ -1300,7 +1318,19 @@ INTSML sim_spicewritefacet(NODEPROTO *np, FILE *f, INTSML top)
 				if (info != NULL) sim_spice_xprintf(f, " %s",info); else
 					sim_spice_xprintf(f, " N");
 				break;
-			case SPICEDMOS:	  /* DMOS (Depletion) transistor */
+			case SPICENMOS4:		/* NMOS (Complementary) 4-port transistor */
+				sim_spice_xputs(sim_spice_element_name(ni, 'M', &nodeindex), f);
+				sim_spice_xputs(sim_spice_node_name(drainn), f);
+				sim_spice_xputs(sim_spice_node_name(gaten), f);
+				sim_spice_xputs(sim_spice_node_name(sourcen), f);
+				for(pp = ni->proto->firstportproto; pp != NOPORTPROTO; pp = pp->nextportproto)
+					biasport = pp;
+				biasn = sim_spice_getnet(ni, biasport);
+				sim_spice_xputs(sim_spice_node_name(biasn), f);
+				if (info != NULL) sim_spice_xprintf(f, " %s", info); else
+					sim_spice_xprintf(f, " N");
+				break;
+			case SPICEDMOS:			/* DMOS (Depletion) transistor */
 				sim_spice_xputs(sim_spice_element_name(ni, 'M', &nodeindex), f);
 				sim_spice_xputs(sim_spice_node_name(drainn), f);
 				sim_spice_xputs(sim_spice_node_name(gaten), f);
@@ -1309,7 +1339,19 @@ INTSML sim_spicewritefacet(NODEPROTO *np, FILE *f, INTSML top)
 				if (info != NULL) sim_spice_xprintf(f, " %s", info); else
 					sim_spice_xprintf(f, " D");
 				break;
-			case SPICEPMOS:	  /* PMOS (Complementary) transistor */
+			case SPICEDMOS4:		/* DMOS (Depletion) 4-port transistor */
+				sim_spice_xputs(sim_spice_element_name(ni, 'M', &nodeindex), f);
+				sim_spice_xputs(sim_spice_node_name(drainn), f);
+				sim_spice_xputs(sim_spice_node_name(gaten), f);
+				sim_spice_xputs(sim_spice_node_name(sourcen), f);
+				for(pp = ni->proto->firstportproto; pp != NOPORTPROTO; pp = pp->nextportproto)
+					biasport = pp;
+				biasn = sim_spice_getnet(ni, biasport);
+				sim_spice_xputs(sim_spice_node_name(biasn), f);
+				if (info != NULL) sim_spice_xprintf(f, " %s", info); else
+					sim_spice_xprintf(f, " D");
+				break;
+			case SPICEPMOS:			/* PMOS (Complementary) transistor */
 				sim_spice_xputs(sim_spice_element_name(ni, 'M', &nodeindex), f);
 				sim_spice_xputs(sim_spice_node_name(drainn), f);
 				sim_spice_xputs(sim_spice_node_name(gaten), f);
@@ -1318,7 +1360,19 @@ INTSML sim_spicewritefacet(NODEPROTO *np, FILE *f, INTSML top)
 				if (info != NULL) sim_spice_xprintf(f, " %s", info); else
 					sim_spice_xprintf(f, " P");
 				break;
-			case SPICENPN:	    /* NPN (Junction) transistor */
+			case SPICEPMOS4:		/* PMOS (Complementary) 4-port transistor */
+				sim_spice_xputs(sim_spice_element_name(ni, 'M', &nodeindex), f);
+				sim_spice_xputs(sim_spice_node_name(drainn), f);
+				sim_spice_xputs(sim_spice_node_name(gaten), f);
+				sim_spice_xputs(sim_spice_node_name(sourcen), f);
+				for(pp = ni->proto->firstportproto; pp != NOPORTPROTO; pp = pp->nextportproto)
+					biasport = pp;
+				biasn = sim_spice_getnet(ni, biasport);
+				sim_spice_xputs(sim_spice_node_name(biasn), f);
+				if (info != NULL) sim_spice_xprintf(f, " %s", info); else
+					sim_spice_xprintf(f, " P");
+				break;
+			case SPICENPN:			/* NPN (Junction) transistor */
 				sim_spice_xputs(sim_spice_element_name(ni, 'Q', &nodeindex), f);
 				sim_spice_xputs(sim_spice_node_name(drainn), f);
 				sim_spice_xputs(sim_spice_node_name(gaten), f);
@@ -1327,7 +1381,19 @@ INTSML sim_spicewritefacet(NODEPROTO *np, FILE *f, INTSML top)
 				if (info != NULL) sim_spice_xprintf(f, " %s", info); else
 					sim_spice_xprintf(f, " NBJT");
 				break;
-			case SPICEPNP:	    /* PNP (Junction) transistor */
+			case SPICENPN4:			/* NPN (Junction) 4-port transistor */
+				sim_spice_xputs(sim_spice_element_name(ni, 'Q', &nodeindex), f);
+				sim_spice_xputs(sim_spice_node_name(drainn), f);
+				sim_spice_xputs(sim_spice_node_name(gaten), f);
+				sim_spice_xputs(sim_spice_node_name(sourcen), f);
+				for(pp = ni->proto->firstportproto; pp != NOPORTPROTO; pp = pp->nextportproto)
+					biasport = pp;
+				biasn = sim_spice_getnet(ni, biasport);
+				sim_spice_xputs(sim_spice_node_name(biasn), f);
+				if (info != NULL) sim_spice_xprintf(f, " %s", info); else
+					sim_spice_xprintf(f, " NBJT");
+				break;
+			case SPICEPNP:			/* PNP (Junction) transistor */
 				sim_spice_xputs(sim_spice_element_name(ni, 'Q', &nodeindex), f);
 				sim_spice_xputs(sim_spice_node_name(drainn), f);
 				sim_spice_xputs(sim_spice_node_name(gaten), f);
@@ -1336,7 +1402,19 @@ INTSML sim_spicewritefacet(NODEPROTO *np, FILE *f, INTSML top)
 				if (info != NULL) sim_spice_xprintf(f, " %s", info); else
 					sim_spice_xprintf(f, " PBJT");
 				break;
-			case SPICENJFET:	/* NJFET (N Channel) transistor */
+			case SPICEPNP4:			/* PNP (Junction) 4-port transistor */
+				sim_spice_xputs(sim_spice_element_name(ni, 'Q', &nodeindex), f);
+				sim_spice_xputs(sim_spice_node_name(drainn), f);
+				sim_spice_xputs(sim_spice_node_name(gaten), f);
+				sim_spice_xputs(sim_spice_node_name(sourcen), f);
+				for(pp = ni->proto->firstportproto; pp != NOPORTPROTO; pp = pp->nextportproto)
+					biasport = pp;
+				biasn = sim_spice_getnet(ni, biasport);
+				sim_spice_xputs(sim_spice_node_name(biasn), f);
+				if (info != NULL) sim_spice_xprintf(f, " %s", info); else
+					sim_spice_xprintf(f, " PBJT");
+				break;
+			case SPICENJFET:		/* NJFET (N Channel) transistor */
 				sim_spice_xputs(sim_spice_element_name(ni, 'J', &nodeindex), f);
 				sim_spice_xputs(sim_spice_node_name(drainn), f);
 				sim_spice_xputs(sim_spice_node_name(gaten), f);
@@ -1344,7 +1422,19 @@ INTSML sim_spicewritefacet(NODEPROTO *np, FILE *f, INTSML top)
 				if (info != NULL) sim_spice_xprintf(f,  " %s", info); else
 					sim_spice_xprintf(f, " NJFET");
 				break;
-			case SPICEPJFET:	/* PJFET (P Channel) transistor */
+			case SPICENJFET4:		/* NJFET (N Channel) 4-port transistor */
+				sim_spice_xputs(sim_spice_element_name(ni, 'J', &nodeindex), f);
+				sim_spice_xputs(sim_spice_node_name(drainn), f);
+				sim_spice_xputs(sim_spice_node_name(gaten), f);
+				sim_spice_xputs(sim_spice_node_name(sourcen), f);
+				for(pp = ni->proto->firstportproto; pp != NOPORTPROTO; pp = pp->nextportproto)
+					biasport = pp;
+				biasn = sim_spice_getnet(ni, biasport);
+				sim_spice_xputs(sim_spice_node_name(biasn), f);
+				if (info != NULL) sim_spice_xprintf(f, " %s", info); else
+					sim_spice_xprintf(f, " NJFET");
+				break;
+			case SPICEPJFET:		/* PJFET (P Channel) transistor */
 				sim_spice_xputs(sim_spice_element_name(ni, 'J', &nodeindex), f);
 				sim_spice_xputs(sim_spice_node_name(drainn), f);
 				sim_spice_xputs(sim_spice_node_name(gaten), f);
@@ -1352,21 +1442,35 @@ INTSML sim_spicewritefacet(NODEPROTO *np, FILE *f, INTSML top)
 				if (info != NULL) sim_spice_xprintf(f, " %s", info); else
 					sim_spice_xprintf(f, " PJFET");
 				break;
-			case SPICEDMES:	  /* DMES (Depletion) transistor */
+			case SPICEPJFET4:		/* PJFET (P Channel) 4-port transistor */
+				sim_spice_xputs(sim_spice_element_name(ni, 'J', &nodeindex), f);
+				sim_spice_xputs(sim_spice_node_name(drainn), f);
+				sim_spice_xputs(sim_spice_node_name(gaten), f);
+				sim_spice_xputs(sim_spice_node_name(sourcen), f);
+				for(pp = ni->proto->firstportproto; pp != NOPORTPROTO; pp = pp->nextportproto)
+					biasport = pp;
+				biasn = sim_spice_getnet(ni, biasport);
+				sim_spice_xputs(sim_spice_node_name(biasn), f);
+				if (info != NULL) sim_spice_xprintf(f, " %s", info); else
+					sim_spice_xprintf(f, " PJFET");
+				break;
+			case SPICEDMES:			/* DMES (Depletion) transistor */
+			case SPICEDMES4:		/* DMES (Depletion) 4-port transistor */
 				sim_spice_xputs(sim_spice_element_name(ni, 'Z', &nodeindex), f);
 				sim_spice_xputs(sim_spice_node_name(drainn), f);
 				sim_spice_xputs(sim_spice_node_name(gaten), f);
 				sim_spice_xputs(sim_spice_node_name(sourcen), f);
 				sim_spice_xprintf(f, " DMES");
 				break;
-			case SPICEEMES:	  /* EMES (Enhancement) transistor */
+			case SPICEEMES:			/* EMES (Enhancement) transistor */
+			case SPICEEMES4:		/* EMES (Enhancement) 4-port transistor */
 				sim_spice_xputs(sim_spice_element_name(ni, 'Z', &nodeindex), f);
 				sim_spice_xputs(sim_spice_node_name(drainn), f);
 				sim_spice_xputs(sim_spice_node_name(gaten), f);
 				sim_spice_xputs(sim_spice_node_name(sourcen), f);
 				sim_spice_xprintf(f, " EMES");
 				break;
-			case SPICETRANS:	 /* special transistor */
+			case SPICETRANS:		/* special transistor */
 				sim_spice_xputs(sim_spice_element_name(ni, 'Q', &nodeindex), f);
 				sim_spice_xputs(sim_spice_node_name(drainn), f);
 				sim_spice_xputs(sim_spice_node_name(gaten), f);
@@ -1386,7 +1490,8 @@ INTSML sim_spicewritefacet(NODEPROTO *np, FILE *f, INTSML top)
 		{
 			a = sim_spice_mask_scale*lx;
 			b = sim_spice_mask_scale*ly;
-			if (state == SPICENMOS || state == SPICEDMOS || state == SPICEPMOS)
+			if (state == SPICENMOS  || state == SPICEDMOS  || state == SPICEPMOS ||
+				state == SPICENMOS4 || state == SPICEDMOS4 || state == SPICEPMOS4)
 			{
 				sim_spice_xprintf(f, " L=%3.2fU W=%3.2fU", scaletodispunit((INTBIG)a, DISPUNITMIC),
 					scaletodispunit((INTBIG)b, DISPUNITMIC));
@@ -1408,14 +1513,18 @@ INTSML sim_spicewritefacet(NODEPROTO *np, FILE *f, INTSML top)
 		}
 
 		/* compute area of source and drain */
-		if (state == SPICENMOS || state == SPICEDMOS || state == SPICEPMOS)
+		if (state == SPICENMOS  || state == SPICEDMOS  || state == SPICEPMOS ||
+			state == SPICENMOS4 || state == SPICEDMOS4 || state == SPICEPMOS4)
 		{
 			switch (state)
 			{
 				case SPICEDMOS:
-				case SPICENMOS: i = ISNTYPE; break;
-				case SPICEPMOS: i = ISPTYPE; break;
-				default:        i = ISNONE;  break;
+				case SPICEDMOS4:
+				case SPICENMOS:
+				case SPICENMOS4: i = ISNTYPE; break;
+				case SPICEPMOS:
+				case SPICEPMOS4: i = ISPTYPE; break;
+				default:         i = ISNONE;  break;
 			}
 
 			/* we should not look at the ISNONE entry of components[],
@@ -1753,14 +1862,14 @@ INTSML sim_spicewritefacet(NODEPROTO *np, FILE *f, INTSML top)
 		}
 
 		/* make sure meter is connected to exports */
-		for(pp=np->firstportproto; pp!=NOPORTPROTO; pp=pp->nextportproto)
+		for(pp = np->firstportproto; pp != NOPORTPROTO; pp = pp->nextportproto)
 			if (pp->temp1 == sourcen->netnumber) break;
 		if (pp == NOPORTPROTO)
 			ttyputerr(_("Warning: top of %s component in facet %s not connected to an export"),
 				describenodeinst(ni), describenodeproto(np));
 		if (drainn != NONET)
 		{
-			for(pp=np->firstportproto; pp!=NOPORTPROTO; pp=pp->nextportproto)
+			for(pp = np->firstportproto; pp != NOPORTPROTO; pp = pp->nextportproto)
 				if (pp->temp1 == sourcen->netnumber) break;
 			if (pp == NOPORTPROTO)
 				ttyputerr(_("Warning: bottom of %s component in facet %s not connected to anexport"),
@@ -1784,8 +1893,8 @@ INTSML sim_spicewritefacet(NODEPROTO *np, FILE *f, INTSML top)
 		}
 
 		/* see what kind of meter it is */
-		var = getvalkey((INTBIG)ni, VNODEINST, VSTRING, sch_meterkey);
-		if (var != NOVARIABLE) extra = (char *)var->addr; else extra = "";
+		var = getvalkey((INTBIG)ni, VNODEINST, -1, sch_meterkey);
+		if (var != NOVARIABLE) extra = describesimplevariable(var); else extra = "";
 		if (drainn == NONET || drainn->netnumber == 0)
 		{
 			/* no space in V(NName), because HSPICE can't handle line break after ")" */
@@ -1836,8 +1945,11 @@ INTSML sim_spicewritefacet(NODEPROTO *np, FILE *f, INTSML top)
 	for(net = sim_spice_firstnet; net != NONET; net = net->nextnet)
 	{
 		for (i = 0; i < DIFFTYPES; i++)
-			if (net->diffarea[i] != 0.0 && net->components[i] == 0)
 		{
+			if (net->diffarea[i] == 0.0 || net->components[i] != 0) continue;
+
+			/* do not issue errors for active area on supply rails (probably well contacts) */
+			if (net == sim_spice_power || net == sim_spice_ground) continue;
 			switch (i)
 			{
 				case ISNTYPE:
@@ -1871,7 +1983,7 @@ INTSML sim_spicewritefacet(NODEPROTO *np, FILE *f, INTSML top)
  */
 INTSML sim_spice_nodetype(NODEINST *ni, char **extra)
 {
-	REGISTER INTSML nodetype;
+	REGISTER INTBIG nodetype;
 
 	if (ni->proto->primindex == 0) return(SPICEFACET);
 	nodetype = nodefunction(ni, extra);
@@ -1888,6 +2000,15 @@ INTSML sim_spice_nodetype(NODEINST *ni, char **extra)
 		case NPTRAEMES:     return(SPICEEMES);
 		case NPTRANSREF:    return(SPICEREF);
 		case NPTRANS:       return(SPICETRANS);
+		case NPTRA4NMOS:    return(SPICENMOS4);
+		case NPTRA4DMOS:    return(SPICEDMOS4);
+		case NPTRA4PMOS:    return(SPICEPMOS4);
+		case NPTRA4NPN:     return(SPICENPN4);
+		case NPTRA4PNP:     return(SPICEPNP4);
+		case NPTRA4NJFET:   return(SPICENJFET4);
+		case NPTRA4PJFET:   return(SPICEPJFET4);
+		case NPTRA4DMES:    return(SPICEDMES4);
+		case NPTRA4EMES:    return(SPICEEMES4);
 		case NPMETER:       return(SPICEMETER);
 		case NPSOURCEV:     return(SPICESOURCEV);
 		case NPSOURCEC:     return(SPICESOURCEC);
@@ -1993,12 +2114,16 @@ void sim_spice_nodearea(NET *net, NODEINST *ni, PORTPROTO *pp)
 		 * to substrate of their top plate polysilicon  or metal
 		 */
 		if (function == SPICECAPACITOR) dominant = -1; else dominant = -2;
-		if (function == SPICENMOS || function == SPICEPMOS || function == SPICEDMOS ||
-			function == SPICEEMES || function == SPICEDMES) function = SPICENMOS;   /* One will do */
+		if (function == SPICENMOS || function == SPICENMOS4 ||
+			function == SPICEPMOS || function == SPICEPMOS4 ||
+			function == SPICEDMOS || function == SPICEDMOS4 ||
+			function == SPICEEMES || function == SPICEEMES4 ||
+			function == SPICEDMES || function == SPICEDMES4)
+				function = SPICENMOS;   /* One will do */
 
 		/* make linked list of polygons */
 		lastpoly = firstpoly = NOPOLYGON;
-		tot = nodeEpolys(ni, 0);
+		tot = nodeEpolys(ni, 0, NOWINDOWPART);
 		for(i=0; i<tot; i++)
 		{
 			poly = sim_allocpolygon();
@@ -2072,13 +2197,13 @@ void sim_spice_nodearea(NET *net, NODEINST *ni, PORTPROTO *pp)
 			if (tech != el_curtech || j == 0)
 			{
 				if (sim_spice_debug != 0)
-					ttyputmsg(_("%s POLYGON ON %s NODE IGNORED IN %s TECHNOLOGY"),
+					ttyputmsg(M_("%s POLYGON ON %s NODE IGNORED IN %s TECHNOLOGY"),
 						layername(tech, poly->layer), ni->proto->cell->cellname, tech->techname);
 				continue;
 			}
 			sim_spice_store_box(tech, poly->layer, hx-lx, hy-ly, (lx+hx)/2, (ly+hy)/2);
 			if (sim_spice_debug != 0)
-				ttyputmsg(_("BOX: %ld to %ld; %ld to %ld; layer %s"), lx, hx, ly, hy,
+				ttyputmsg(M_("BOX: %ld to %ld; %ld to %ld; layer %s"), lx, hx, ly, hy,
 					layername(tech,poly->layer));
 			if (sim_spice_layerisdiff(tech, poly->layer) == ISNONE &&
 				poly->layer != dominant)
@@ -2124,7 +2249,7 @@ void sim_spice_arcarea(NET *net, ARCINST *ai)
 
 	if (poly == NOPOLYGON) poly = allocstaticpolygon(4, sim_aid->cluster);
 
-	tot = arcpolys(ai);
+	tot = arcpolys(ai, NOWINDOWPART);
 	tech = ai->proto->tech;
 	isdiffarc = sim_spice_arcisdiff(ai);    /* Check arc function */
 	for(i=0; i<tot; i++)
@@ -2134,7 +2259,7 @@ void sim_spice_arcarea(NET *net, ARCINST *ai)
 		if (tech != el_curtech || j == 0)
 		{
 			if (sim_spice_debug != 0)
-				ttyputmsg(_("%s POLYGON ON %s ARC IGNORED IN %s TECHNOLOGY"),
+				ttyputmsg(M_("%s POLYGON ON %s ARC IGNORED IN %s TECHNOLOGY"),
 					layername(tech, poly->layer), describearcinst(ai), tech->techname);
 			continue;
 		}
@@ -2143,7 +2268,7 @@ void sim_spice_arcarea(NET *net, ARCINST *ai)
 			(!isdiffarc && sim_spice_capacitance(tech, poly->layer) > 0.0))
 				sim_spice_store_box(tech, poly->layer, hx-lx, hy-ly, (lx+hx)/2, (ly+hy)/2);
 		if (sim_spice_debug != 0)
-			ttyputmsg(_("BOX: %ld to %ld; %ld to %ld; layer %s"), lx, hx, ly, hy,
+			ttyputmsg(M_("BOX: %ld to %ld; %ld to %ld; layer %s"), lx, hx, ly, hy,
 				layername(tech,poly->layer));
 	}
 
@@ -2338,16 +2463,13 @@ void sim_spice_xputs(char *s, FILE *stream)
 char *sim_spice_describesource(NODEINST *ni)
 {
 	REGISTER VARIABLE *var;
-	REGISTER char *str;
 
-	var = getvalkey((INTBIG)ni, VNODEINST, VSTRING, sch_sourcekey);
+	var = getvalkey((INTBIG)ni, VNODEINST, -1, sch_sourcekey);
 	if (var == NOVARIABLE) return(describenodeinst(ni));
-	str = (char *)var->addr;
-	while (*str != 0 && *str != '/') str++;
-	if (*str == 0) return(describenodeinst(ni));
 	(void)initinfstr();
 	(void)addstringtoinfstr(describenodeinst(ni));
-	(void)addstringtoinfstr(str);
+	(void)addtoinfstr('/');
+	(void)addstringtoinfstr(describesimplevariable(var));
 	return(returninfstr());
 }
 
@@ -2380,7 +2502,7 @@ void sim_spice_evalpolygon(INTSML layer, TECHNOLOGY *tech, INTBIG *xbuf, INTBIG
 
 	if (sim_spice_debug != 0)
 	{
-		ttyputmsg(_("** Evaluating polygon on Layer: %s; Net: %ld"),
+		ttyputmsg(M_("** Evaluating polygon on Layer: %s; Net: %ld"),
 			layername(tech, layer), sim_spice_cur_net->netnumber);
 	}
 
@@ -2415,7 +2537,7 @@ void sim_spice_evalpolygon(INTSML layer, TECHNOLOGY *tech, INTBIG *xbuf, INTBIG
 				sim_spice_mask_scale;
 	}
 	if (sim_spice_debug != 0)
-		ttyputmsg(_("Area: %f , Perim: %ld Cap: %f"), area, perim, sim_spice_cur_net->capacitance);
+		ttyputmsg(M_("Area: %f , Perim: %ld Cap: %f"), area, perim, sim_spice_cur_net->capacitance);
 }
 
 /******************** ROUTINES TO ACCESS VARIABLES ********************/
