@@ -706,7 +706,7 @@ void sim_spice_terminate(FILE *outputfile, char *outfile, char *infile)
 	if (outputfile != NULL)
 	{
 		xclose(outputfile);
-		ttyputverbose(_("%s written"), outfile);
+		ttyputverbose(M_("%s written"), outfile);
 	}
 	if (*infile != 0)
 	{
@@ -855,6 +855,28 @@ INTSML sim_spice_plotvalues(INTSML important)
 	/* set the globals with the range of the simulation window */
 	sim_window_lowy = tl;   sim_window_highy = th;
 
+#if 0		/* don't think signal order makes sense when all overlaid */
+	REGISTER VARIABLE *var;
+	INTBIG oldsigcount;
+	void *sa;
+	char **oldsignames;
+
+	/* find former signal order */
+	oldsigcount = 0;
+	var = getvalkey((INTBIG)plotnp, VNODEPROTO, VSTRING|VISARRAY, sim_window_signal_order);
+	if (var != NOVARIABLE)
+	{
+		oldsigcount = getlength(var);
+		if (oldsigcount > 0)
+		{
+			sa = newstringarray(sim_aid->cluster);
+			for(i=0; i<oldsigcount; i++)
+				addtostringarray(sa, ((char **)var->addr)[i]);
+			oldsignames = getstringarray(sa, &oldsigcount);
+		}
+	}
+#endif
+
 	/* make the simulation window */
 	if (sim_window_create(1, plotnp, sim_spice_charhandlerwave,
 		sim_spice_charhandlerschem) != 0) return(1);
@@ -939,7 +961,7 @@ INTSML sim_spice_charhandlerwave(WINDOWPART *w, INTSML chr)
 	char *par[3], s2[20];
 	float v, time, low, high;
 	REGISTER INTBIG tr, trl;
-	REGISTER INTSML i, j, thispos, pos, lines, traces;
+	REGISTER INTBIG i, j, thispos, pos, lines, traces;
 	REGISTER NUMBERS *num;
 
 	/* see if there are special functions for SPICE waveform simulation */
@@ -956,7 +978,7 @@ INTSML sim_spice_charhandlerwave(WINDOWPART *w, INTSML chr)
 			ttyputinstruction(" a", 6, _("add signal to simulation window"));
 			ttyputinstruction(" o", 6, _("overlay signal on top of current signal line"));
 			ttyputinstruction(" r", 6, _("remove selected signal from simulation window"));
-			ttyputinstruction(" s", 6, _("save snapshot of simulation window in database"));
+			ttyputinstruction(" p", 6, _("preserve snapshot of simulation window in database"));
 			return(0);
 		case '9':		/* show entire vertical range */
 			sim_window_getanaextents(&low, &high);
@@ -1003,7 +1025,7 @@ INTSML sim_spice_charhandlerwave(WINDOWPART *w, INTSML chr)
 			sim_windowconvertengineeringnotation(time, s2);
 			ttyputmsg(_("Signal %s has value %g at time %s"), sim_window_gettracename(tr), v, s2);
 			return(0);
-		case 's':		/* save image in {simulation_snapshot} view */
+		case 'p':		/* preserve image in {simulation_snapshot} view */
 			sim_window_savegraph();
 			return(0);
 		case 'a':		/* add trace */
@@ -1021,7 +1043,7 @@ INTSML sim_spice_charhandlerwave(WINDOWPART *w, INTSML chr)
 			{
 				/* other traces exist, make a new line in the plot */
 				j = sim_window_getlines();
-				sim_window_setlines((INTSML)(j+1));
+				sim_window_setlines(j+1);
 			}
 
 			/* create a new trace in the last slot */
@@ -1031,7 +1053,8 @@ INTSML sim_spice_charhandlerwave(WINDOWPART *w, INTSML chr)
 				sim_window_val[j++] = (num->list[i] - sim_window_lowy) /
 					(sim_window_highy - sim_window_lowy);
 			sim_window_loadanatrace(tr, j, sim_window_time, sim_window_val);
-			sim_window_sethighlighttrace(tr);
+			sim_window_cleartracehighlight();
+			sim_window_addhighlighttrace(tr);
 			sim_window_redraw();
 			return(0);
 		case 'o':		/* overlap trace */
@@ -1050,13 +1073,14 @@ INTSML sim_spice_charhandlerwave(WINDOWPART *w, INTSML chr)
 				sim_window_val[j++] = (num->list[i] - sim_window_lowy) /
 					(sim_window_highy - sim_window_lowy);
 			sim_window_loadanatrace(tr, j, sim_window_time, sim_window_val);
-			sim_window_sethighlighttrace(tr);
+			sim_window_cleartracehighlight();
+			sim_window_addhighlighttrace(tr);
 			sim_window_redraw();
 			return(0);
 		case 'r':		/* remove trace */
 			tr = sim_window_gethighlighttrace();
 			if (tr == 0) return(0);
-			sim_window_sethighlighttrace(0);
+			sim_window_cleartracehighlight();
 
 			/* see if any other traces use this line */
 			thispos = sim_window_gettraceline(tr);
@@ -1079,9 +1103,9 @@ INTSML sim_spice_charhandlerwave(WINDOWPART *w, INTSML chr)
 					trl = sim_window_nexttraceloop();
 					if (trl == 0) break;
 					pos = sim_window_gettraceline(trl);
-					if (pos > thispos) sim_window_settraceline(trl, (INTSML)(pos-1));
+					if (pos > thispos) sim_window_settraceline(trl, pos-1);
 				}
-				sim_window_setlines((INTSML)(lines-1));
+				sim_window_setlines(lines-1);
 			}
 
 			/* kill trace, redraw */
@@ -1148,12 +1172,12 @@ void sim_window_addhighlightednet(INTSML overlay)
 		{
 			/* other traces exist, make a new line in the plot */
 			traceline = sim_window_getlines();
-			sim_window_setlines((INTSML)(traceline+1));
+			sim_window_setlines(traceline+1);
 		}
 	} else
 	{
 		tr = sim_window_gethighlighttrace();
-		sim_window_sethighlighttrace(0);
+		sim_window_cleartracehighlight();
 		if (tr != 0)
 		{
 			traceline = sim_window_gettraceline(tr);
@@ -1169,20 +1193,21 @@ void sim_window_addhighlightednet(INTSML overlay)
 			{
 				/* no traces: create a new line */
 				traceline = 0;
-				sim_window_setlines((INTSML)(traceline+1));
+				sim_window_setlines(traceline+1);
 			}
 		}
 	}
 
 	/* create a new trace in this slot */
-	tr = sim_window_newtrace((INTSML)traceline, sim_window_signames[i], 0);
+	tr = sim_window_newtrace(traceline, sim_window_signames[i], 0);
 	j = 0;
 	for(num = sim_window_numbers; num != NONUMBERS; num = num->nextnumbers)
 		sim_window_val[j++] = (num->list[i] - sim_window_lowy) /
 			(sim_window_highy - sim_window_lowy);
-	sim_window_loadanatrace(tr, (INTSML)j, sim_window_time, sim_window_val);
+	sim_window_loadanatrace(tr, j, sim_window_time, sim_window_val);
 	sim_window_redraw();
-	sim_window_sethighlighttrace(tr);
+	sim_window_cleartracehighlight();
+	sim_window_addhighlighttrace(tr);
 }
 
 /*
