@@ -144,7 +144,7 @@ void sim_writetexcell(NODEPROTO *np, INTBIG top)
 	PORTPROTO *pp, *xpp;
 	NODEINST *ni, *nextni;
 	ARCINST *ai;
-	char str1[MAXLENGTH], *extra, *ptr;
+	char str1[MAXLENGTH], *ptr;
 	INTBIG inputs, nextniinputs, ninegated, nextninegated, f, x, separator,
 		count, length, tempa, tempb, module_err=0;
 	INTBIG pwr=0, gnd=0;
@@ -172,7 +172,7 @@ void sim_writetexcell(NODEPROTO *np, INTBIG top)
 		(void)initinfstr();
 		addstringtoinfstr("INPUTS:\n");
 
-		for(pp=np->firstportproto; pp!=NOPORTPROTO; pp=pp->nextportproto)
+		for(pp = np->firstportproto; pp != NOPORTPROTO; pp = pp->nextportproto)
 		{
 			if (sim_texcheckname(sim_texname(pp->protoname)) == 0)
 				sim_texerror(11, &module_err, sim_texname(pp->protoname));
@@ -193,7 +193,7 @@ void sim_writetexcell(NODEPROTO *np, INTBIG top)
 		(void)initinfstr();
 		addstringtoinfstr("OUTPUTS:\n");
 
-		for(pp=np->firstportproto; pp!=NOPORTPROTO; pp=pp->nextportproto)
+		for(pp = np->firstportproto; pp != NOPORTPROTO; pp = pp->nextportproto)
 		{
 			if ((pp->userbits&STATEBITS) == OUTPORT)
 			{
@@ -212,11 +212,11 @@ void sim_writetexcell(NODEPROTO *np, INTBIG top)
 	(void)initinfstr();
 	(void)addstringtoinfstr("USE:\n\n");  /* write USE: */
 
-	for(ni=np->firstnodeinst, ni->temp2=0; ni!=NONODEINST; ni=ni->nextnodeinst)
+	for(ni = np->firstnodeinst, ni->temp2=0; ni != NONODEINST; ni = ni->nextnodeinst)
 	{
 		if (ni->temp1 != 0) continue;
 		ni->temp1++;	     /* mark node seen */
-		f = nodefunction(ni, &extra);
+		f = nodefunction(ni, 0);
 
 		switch(f)
 		{
@@ -237,7 +237,7 @@ void sim_writetexcell(NODEPROTO *np, INTBIG top)
 				/* Count number of inputs */
 				if ((inputs = sim_texcountwires(ni)) < 2)
 					sim_texerror(9, &module_err, sim_texname(describenodeproto(ni->proto)));
-				for(nextni=ni->nextnodeinst; nextni!=NONODEINST; nextni=nextni->nextnodeinst)
+				for(nextni = ni->nextnodeinst; nextni != NONODEINST; nextni = nextni->nextnodeinst)
 				{
 					/* look at similar nodes */
 					if (nextni->proto != ni->proto) continue;
@@ -252,7 +252,7 @@ void sim_writetexcell(NODEPROTO *np, INTBIG top)
 
 					/* Compare current node and next node, If next node is the same  */
 					/* mark it seen */
-					if (nextniinputs == inputs && ninegated==nextninegated)
+					if (nextniinputs == inputs && ninegated == nextninegated)
 						nextni->temp1++;
 				}
 
@@ -320,7 +320,7 @@ void sim_writetexcell(NODEPROTO *np, INTBIG top)
 				(void)addstringtoinfstr(",\n");
 
 				/* if another node is the same as this one mark it seen */
-				for(nextni=ni->nextnodeinst; nextni!=NONODEINST; nextni=nextni->nextnodeinst)
+				for(nextni = ni->nextnodeinst; nextni != NONODEINST; nextni = nextni->nextnodeinst)
 					if (nextni->proto == ni->proto) nextni->temp1++;
 
 				break; /* from  switch */
@@ -345,12 +345,12 @@ void sim_writetexcell(NODEPROTO *np, INTBIG top)
 		if ((ai->userbits&ISNEGATED) == 0) continue;
 		if ((ai->end[0].portarcinst->proto->userbits&STATEBITS) == OUTPORT)
 		{
-			f = nodefunction(ai->end[0].nodeinst, &extra);
+			f = nodefunction(ai->end[0].nodeinst, 0);
 			if (f == NPGATEAND || f == NPGATEOR || f == NPGATEXOR || f == NPBUFFER)
 				ai->temp1 = -1; else ai->temp1 = count++;
-		} else if ((ai->end[1].portarcinst->proto->userbits&STATEBITS)==OUTPORT)
+		} else if ((ai->end[1].portarcinst->proto->userbits&STATEBITS) == OUTPORT)
 		{
-			f = nodefunction(ai->end[1].nodeinst,&extra);
+			f = nodefunction(ai->end[1].nodeinst, 0);
 			if (f == NPGATEAND || f == NPGATEOR || f == NPGATEXOR || f == NPBUFFER)
 				ai->temp1 = -1; else ai->temp1 = count++;
 		} else ai->temp1 = count++;
@@ -364,10 +364,10 @@ void sim_writetexcell(NODEPROTO *np, INTBIG top)
 			sim_texerror(3, &module_err, NULLSTR);
 	}
 
-	for (ni=np->firstnodeinst, count=1; ni!=NONODEINST; ni=ni->nextnodeinst)
+	for (ni = np->firstnodeinst, count=1; ni != NONODEINST; ni = ni->nextnodeinst)
 	{
 		ni->temp1 = 0;
-		f = nodefunction(ni,&extra);
+		f = nodefunction(ni, 0);
 
 		/* handle nodeinst descriptions */
 		switch(f)
@@ -406,10 +406,10 @@ void sim_writetexcell(NODEPROTO *np, INTBIG top)
 
 	/* any ports on the same network in the cell have to be made equivalent points */
 	/* in the tdl description */
-	for(pp=np->firstportproto; pp!= NOPORTPROTO; pp=pp->nextportproto)
+	for(pp = np->firstportproto; pp != NOPORTPROTO; pp = pp->nextportproto)
 	{
 		if (pp->temp2 != 0) continue;
-		for(xpp=pp->nextportproto; xpp!=NOPORTPROTO; xpp=xpp->nextportproto)
+		for(xpp = pp->nextportproto; xpp != NOPORTPROTO; xpp = xpp->nextportproto)
 		{
 			if (xpp->network != pp->network) continue;
 			if (xpp->temp2 != 0) continue;
@@ -465,10 +465,10 @@ INTSML sim_texcountwires(NODEINST *ni)
 	PORTARCINST *pai;
 	PORTEXPINST *pei;
 
-	for(inputs=0, pai=ni->firstportarcinst; pai!=NOPORTARCINST; pai=pai->nextportarcinst)
+	for(inputs=0, pai = ni->firstportarcinst; pai != NOPORTARCINST; pai = pai->nextportarcinst)
 		if (pai->proto == ni->proto->firstportproto) inputs++;
 
-	for(pei=ni->firstportexpinst; pei!=NOPORTEXPINST; pei=pei->nextportexpinst)
+	for(pei = ni->firstportexpinst; pei != NOPORTEXPINST; pei = pei->nextportexpinst)
 		if (pei->proto == ni->proto->firstportproto) inputs++;
 
 	return(inputs);
@@ -484,7 +484,7 @@ char *sim_texsrcname(NODEINST *ni, PORTARCINST *pai)
 	INTBIG exp_found=0;
 	static char str[MAXNAMECHARS + 1];
 
-	for(pp=ni->parent->firstportproto; pp != NOPORTPROTO; pp=pp->nextportproto)
+	for(pp = ni->parent->firstportproto; pp != NOPORTPROTO; pp = pp->nextportproto)
 	{
 		if (pp->network == pai->conarcinst->network)
 		{
@@ -517,7 +517,7 @@ char *sim_texwritesignals(NODEINST *ni, INTBIG f, INTBIG *module_err)
 		(void)addstringtoinfstr("(");
 	} else return(NULLSTR);
 
-	for(pp=ni->proto->firstportproto; pp != NOPORTPROTO; pp = pp->nextportproto)
+	for(pp = ni->proto->firstportproto; pp != NOPORTPROTO; pp = pp->nextportproto)
 	{
 		if((pp->userbits&STATEBITS) != INPORT) continue;
 
@@ -541,7 +541,7 @@ char *sim_texwritesignals(NODEINST *ni, INTBIG f, INTBIG *module_err)
 				if ((pp->userbits&PORTISOLATED) == 0) break;
 			}
 		}
-		for(pei=ni->firstportexpinst; pei != NOPORTEXPINST; pei=pei->nextportexpinst)
+		for(pei = ni->firstportexpinst; pei != NOPORTEXPINST; pei = pei->nextportexpinst)
 		{
 			if (pei->proto == pp)
 			{
@@ -575,7 +575,6 @@ char *sim_texwriteffsignals(NODEINST *ni, INTBIG *module_err)
 	PORTPROTO *pp;
 	PORTARCINST *pai;
 	PORTEXPINST *pei;
-	VARIABLE *var;
 	char signals[5][MAXNAMECHARS + 1], *string;
 	char *ptr;
 	INTBIG x, length;
@@ -595,7 +594,7 @@ char *sim_texwriteffsignals(NODEINST *ni, INTBIG *module_err)
 				break;
 			}
 		}
-		for(pei=ni->firstportexpinst; pei != NOPORTEXPINST; pei=pei->nextportexpinst)
+		for(pei = ni->firstportexpinst; pei != NOPORTEXPINST; pei = pei->nextportexpinst)
 		{
 			if (pei->proto == pp)
 			{
@@ -609,15 +608,13 @@ char *sim_texwriteffsignals(NODEINST *ni, INTBIG *module_err)
 	/* We now have the signals in 5 arrays ready to be output in the correct */
 	/* order which is 2,0,1,3,4.If the flip flop is a D or T type don't put */
 	/* out array[1][]. */
-	var = getvalkey((INTBIG)ni, VNODEINST, VSTRING, sch_flipfloptypekey);
-
 	(void)initinfstr();
 	(void)addstringtoinfstr("(");
 	(void)addstringtoinfstr(signals[2]);
 	(void)addstringtoinfstr(signals[0]);
 
 	/* JK and SR have one input more than D or T flip flops */
-	if (namesamen((char *)var->addr, "jk", 2) == 0 || namesamen((char *)var->addr, "rs", 2) == 0)
+	if ((ni->userbits&FFTYPE) == FFTYPERS || (ni->userbits&FFTYPE) == FFTYPEJK)
 		(void)addstringtoinfstr(signals[1]);
 
 	(void)addstringtoinfstr(signals[3]);
@@ -683,13 +680,13 @@ char *sim_textdlinst(NODEINST *ni, INTBIG f)
 	(void)addstringtoinfstr(str1);
 	(void)addstringtoinfstr("(");
 
-	for(pp=ni->proto->firstportproto; pp != NOPORTPROTO; pp = pp->nextportproto)
+	for(pp = ni->proto->firstportproto; pp != NOPORTPROTO; pp = pp->nextportproto)
 	{
 		if (((pp->userbits&STATEBITS) == INPORT) && f != NPTRANS) continue;
 		if (f == NPTRANS && namesamen(pp->protoname, "g", 1) == 0) continue;
 
 		portcount++;
-		for(pai=ni->firstportarcinst; pai!=NOPORTARCINST; pai=pai->nextportarcinst)
+		for(pai = ni->firstportarcinst; pai != NOPORTARCINST; pai = pai->nextportarcinst)
 		{
 			if (pai->proto != pp) continue;
 			if (portcount == instcount) continue;
@@ -751,7 +748,7 @@ char *sim_textdlinst(NODEINST *ni, INTBIG f)
 			}/* end of else*/
 		} /* end of for */
 
-		for(pei=ni->firstportexpinst; pei!=NOPORTEXPINST; pei=pei->nextportexpinst)
+		for(pei = ni->firstportexpinst; pei != NOPORTEXPINST; pei = pei->nextportexpinst)
 		{
 			if (pei->proto != pp) continue;
 			if (portcount == instcount) continue;
@@ -945,54 +942,25 @@ char *sim_texproto(NODEINST *ni, INTBIG f, INTBIG *module_err)
 /* routine to return the TDL primitive name of a flip-flop in electric */
 char *sim_texffname(NODEINST *ni, INTBIG *module_err)
 {
-	VARIABLE *var;
-	char *str;
 	static char name[5];
 
-	var = getvalkey((INTBIG)ni, VNODEINST, VSTRING, sch_flipfloptypekey);
-
-	for (str = (char *)var->addr; str != 0; str++)
+	name[0] = 0;
+	switch (ni->userbits&FFTYPE)
 	{
-		if (namesamen(str, "rs", 2)== 0)
-		{
-			strcpy(name, "SR");
-			str++;
-			continue;
-		}
-		if (namesamen(str, "jk", 2)== 0)
-		{
-			strcpy(name, "JK");
-			str++;
-			continue;
-		}
-		if (namesamen(str, "t", 1)== 0)
-		{
-			str++;
-			if (*str == 'p' || *str == 'n') sim_texerror(10, module_err, NULLSTR);
+		case FFTYPERS: strcpy(name, "SR");   break;
+		case FFTYPEJK: strcpy(name, "JK");   break;
+		case FFTYPED:  strcpy(name, "D");    break;
+		case FFTYPET:
+			if ((ni->userbits&FFCLOCK) != FFCLOCKMS)
+				sim_texerror(10, module_err, NULLSTR);
 			strcpy(name, "TMNE");
 			break;
-		}
-		if (namesamen(str, "d", 1)== 0)
-		{
-			strcpy(name, "D");
-			str++;
-			continue;
-		}
-		if (namesamen(str, "p", 1)== 0)
-		{
-			strcat(name, "EPE");
-			break;
-		}
-		if (namesamen(str, "n", 1)== 0)
-		{
-			strcat(name, "ENE");
-			break;
-		}
-		if (namesamen(str, "ms", 2)== 0)
-		{
-			strcat(name, "MNE");
-			break;
-		}
+	}
+	switch (ni->userbits&FFCLOCK)
+	{
+		case FFCLOCKMS: strcat(name, "MNE");   break;
+		case FFCLOCKP:  strcat(name, "EPE");   break;
+		case FFCLOCKN:  strcat(name, "ENE");   break;
 	}
 	return(name);
 }
@@ -1038,7 +1006,7 @@ INTSML sim_texnegatedoutput(NODEINST *ni)
 {
 	PORTARCINST *pai;
 
-	for(pai=ni->firstportarcinst; pai!=NOPORTARCINST; pai=pai->nextportarcinst)
+	for(pai = ni->firstportarcinst; pai != NOPORTARCINST; pai = pai->nextportarcinst)
 	{
 		if ((pai->conarcinst->userbits&ISNEGATED) == 0) continue;
 		if ((pai->proto->userbits&STATEBITS) == OUTPORT) return(1);
