@@ -35,7 +35,7 @@
 #include "efunction.h"
 #include <math.h>
 
-#define	SPLINEGRAIN   20		/* number of line segments in each spline section */
+#define SPLINEGRAIN   20		/* number of line segments in each spline section */
 
 /* the options table */
 static KEYWORD artopt[] =
@@ -58,7 +58,7 @@ NODEPROTO  *art_pinprim, *art_openedpolygonprim, *art_boxprim, *art_crossedboxpr
 NODEPROTO  *art_filledboxprim, *art_circleprim, *art_filledcircleprim, *art_splineprim;
 NODEPROTO  *art_triangleprim, *art_filledtriangleprim, *art_arrowprim;
 NODEPROTO  *art_openeddottedpolygonprim, *art_openeddashedpolygonprim;
-NODEPROTO  *art_openedfardottepolygonprim, *art_closedpolygonprim, *art_filledpolygonprim;
+NODEPROTO  *art_openedthickerpolygonprim, *art_closedpolygonprim, *art_filledpolygonprim;
 ARCPROTO   *art_solidarc, *art_dash1arc, *art_dash2arc, *art_dash3arc;
 
 /* prototypes for local routines */
@@ -69,8 +69,8 @@ void art_setstate(INTBIG newstate);
 
 /******************** LAYERS ********************/
 
-#define	MAXLAYERS  1		/* total layers below */
-#define	LOPAQUE    0		/* solid graphics     */
+#define MAXLAYERS  1		/* total layers below */
+#define LOPAQUE    0		/* solid graphics     */
 
 static GRAPHICS art_o_lay = {LAYERO, MENGLY, SOLIDC, SOLIDC,
 /* graphics layer */		{0xFFFF,0xFFFF,0xFFFF,0xFFFF,
@@ -89,11 +89,11 @@ static INTBIG art_gds_layers[MAXLAYERS] = {1};
 
 /******************** ARCS ********************/
 
-#define	ARCPROTOCOUNT  4
-#define	ASOLID         0
-#define	ADASH1         1
-#define	ADASH2         2
-#define	ADASH3         3
+#define ARCPROTOCOUNT  4
+#define ASOLID         0
+#define ADASH1         1
+#define ADASH2         2
+#define ADASH3         3
 
 /* solid arc */
 static TECH_ARCLAY art_al_s[] = {{LOPAQUE, 0, FILLED}};
@@ -102,21 +102,21 @@ static TECH_ARCS art_a_s = {
 	1,art_al_s,														/* layers */
 	(APUNKNOWN<<AFUNCTIONSH)|CANWIPE|CANCURVE|(0<<AANGLEINCSH)};	/* userbits */
 
-/* dash 1 arc */
+/* dotted arc */
 static TECH_ARCLAY art_al_d1[] = {{LOPAQUE, 0, FILLED}};
 static TECH_ARCS art_a_d1 = {
 	"Dash-1",0,ADASH1,												/* name */
 	1,art_al_d1,													/* layers */
 	(APUNKNOWN<<AFUNCTIONSH)|CANWIPE|CANCURVE|(0<<AANGLEINCSH)};	/* userbits */
 
-/* dash 2 arc */
+/* dashed arc */
 static TECH_ARCLAY art_al_d2[] = {{LOPAQUE, 0, FILLED}};
 static TECH_ARCS art_a_d2 = {
 	"Dash-2",0,ADASH2,												/* name */
 	1,art_al_d2,													/* layers */
 	(APUNKNOWN<<AFUNCTIONSH)|CANWIPE|CANCURVE|(0<<AANGLEINCSH)};	/* userbits */
 
-/* dash 3 arc */
+/* thicker arc */
 static TECH_ARCLAY art_al_d3[] = {{LOPAQUE, 0, FILLED}};
 static TECH_ARCS art_a_d3 = {
 	"Dash-3",0,ADASH3,												/* name */
@@ -133,25 +133,25 @@ static INTBIG art_pc_d[] = {-1, ASOLID, ADASH1, ADASH2, ADASH3, ALLGEN, -1};
 
 /******************** NODES ********************/
 
-#define	NODEPROTOCOUNT 16
-#define	NPIN            1		/* point */
-#define	NBOX            2		/* rectangular box */
-#define	NCBOX           3		/* crossed rectangular box */
-#define	NFBOX           4		/* filled rectangular box */
-#define	NCIRCLE         5		/* circle */
-#define	NFCIRCLE        6		/* filled circle */
-#define	NSPLINE         7		/* spline curve */
-#define	NTRIANGLE       8		/* triangle */
-#define	NFTRIANGLE      9		/* filled triangle */
-#define	NARROW         10		/* arrow head */
-#define	NOPENED        11		/* opened solid polygon */
-#define	NOPENEDT1      12		/* opened dotted polygon */
-#define	NOPENEDT2      13		/* opened dashed polygon */
-#define	NOPENEDT3      14		/* opened far dotted polygon */
-#define	NCLOSED        15		/* closed polygon */
-#define	NFILLED        16		/* filled polygon */
+#define NODEPROTOCOUNT 16
+#define NPIN            1		/* point */
+#define NBOX            2		/* rectangular box */
+#define NCBOX           3		/* crossed rectangular box */
+#define NFBOX           4		/* filled rectangular box */
+#define NCIRCLE         5		/* circle */
+#define NFCIRCLE        6		/* filled circle */
+#define NSPLINE         7		/* spline curve */
+#define NTRIANGLE       8		/* triangle */
+#define NFTRIANGLE      9		/* filled triangle */
+#define NARROW         10		/* arrow head */
+#define NOPENED        11		/* opened solid polygon */
+#define NOPENEDT1      12		/* opened dotted polygon */
+#define NOPENEDT2      13		/* opened dashed polygon */
+#define NOPENEDT3      14		/* opened thicker polygon */
+#define NCLOSED        15		/* closed polygon */
+#define NFILLED        16		/* filled polygon */
 
-#define	SX   (WHOLE/6)
+#define SX   (WHOLE/6)
 static INTBIG art_fullbox[8]  = {LEFTEDGE, LEFTEDGE, RIGHTEDGE, TOPEDGE};
 static INTBIG art_circ[8]     = {CENTER, CENTER,     RIGHTEDGE, CENTER};
 static INTBIG art_opnd[16]    = {LEFTEDGE, BOTEDGE,  -SX,0,TOPEDGE,
@@ -350,14 +350,14 @@ static TECH_NODES art_freeformdash = {
 	(NPART<<NFUNCTIONSH)|HOLDSTRACE|NEDGESELECT,	/* userbits */
 	0,0,0,0,0,0,0,0,0};								/* characteristics */
 
-/* Opened Far Dotted Polygon */
+/* Opened Thicker Polygon */
 static TECH_PORTS art_freeformfdot_p[] = {			/* ports */
 	{art_pc_d, "site", NOPORTPROTO, (180<<PORTARANGESH),
 		CENTER, CENTER, CENTER, CENTER}};
 static TECH_POLYGON art_freeformfdot_l[] = {		/* layers */
 	{LOPAQUE, -1, 4, OPENEDT3, POINTS, art_opnd}};
 static TECH_NODES art_freeformfdot = {
-	"Opened-FarDotted-Polygon",NOPENEDT3,NONODEPROTO,/* name */
+	"Opened-Thicker-Polygon",NOPENEDT3,NONODEPROTO,	/* name */
 	K4,K4,											/* size */
 	1,art_freeformfdot_p,							/* ports */
 	1,art_freeformfdot_l,							/* layers */
@@ -446,7 +446,7 @@ INTSML art_initprocess(TECHNOLOGY *tech, INTSML pass)
 		art_openedpolygonprim = getnodeproto("artwork:Opened-Polygon");
 		art_openeddottedpolygonprim = getnodeproto("artwork:Opened-Dotted-Polygon");
 		art_openeddashedpolygonprim = getnodeproto("artwork:Opened-Dashed-Polygon");
-		art_openedfardottepolygonprim = getnodeproto("artwork:Opened-FarDotted-Polygon");
+		art_openedthickerpolygonprim = getnodeproto("artwork:Opened-Thicker-Polygon");
 		art_closedpolygonprim = getnodeproto("artwork:Closed-Polygon");
 		art_filledpolygonprim = getnodeproto("artwork:Filled-Polygon");
 
@@ -472,8 +472,8 @@ void art_setmode(INTSML count, char *par[])
 	if (count == 0)
 	{
 		if (art_arrow.layercount == 2)
-			ttyputmsg(_("Artwork arrow heads are filled")); else
-				ttyputmsg(_("Artwork arrow heads are outlines"));
+			ttyputmsg(M_("Artwork arrow heads are filled")); else
+				ttyputmsg(M_("Artwork arrow heads are outlines"));
 		return;
 	}
 
@@ -482,14 +482,14 @@ void art_setmode(INTSML count, char *par[])
 	{
 		/* set the arrow heads to be fancy */
 		art_setstate(art_state | ARTWORKFILLARROWHEADS);
-		ttyputverbose(_("Artwork arrow heads will be filled"));
+		ttyputverbose(M_("Artwork arrow heads will be filled"));
 		return;
 	}
 	if (namesamen(pp, "arrows-outline", l) == 0)
 	{
 		/* set the arrow heads to be simple */
 		art_setstate(art_state & ~ARTWORKFILLARROWHEADS);
-		ttyputverbose(_("Artwork arrow heads will be outline"));
+		ttyputverbose(M_("Artwork arrow heads will be outline"));
 		return;
 	}
 	ttyputbadusage("technology tell artwork");
@@ -497,6 +497,7 @@ void art_setmode(INTSML count, char *par[])
 
 INTBIG art_request(char *command, va_list ap)
 {
+	if (namesame(command, "has-state") == 0) return(1);
 	if (namesame(command, "get-state") == 0)
 	{
 		return(art_state);
@@ -528,7 +529,7 @@ void art_setstate(INTBIG newstate)
 	}
 }
 
-INTBIG art_nodepolys(NODEINST *ni, INTBIG *reasonable)
+INTBIG art_nodepolys(NODEINST *ni, INTBIG *reasonable, WINDOWPART *win)
 {
 	REGISTER INTBIG pindex, count;
 
@@ -544,12 +545,12 @@ INTBIG art_nodepolys(NODEINST *ni, INTBIG *reasonable)
 
 	/* add in displayable variables */
 	tech_realpolys = count;
-	count += tech_displayablenvars(ni);
+	count += tech_displayablenvars(ni, tech_curwindowpart);
 	if (reasonable != 0) *reasonable = count;
 	return(count);
 }
 
-INTBIG art_nodeEpolys(NODEINST *ni, INTBIG *reasonable)
+INTBIG art_nodeEpolys(NODEINST *ni, INTBIG *reasonable, WINDOWPART *win)
 {
 	if (reasonable != 0) *reasonable = 0;
 	return(0);
@@ -566,7 +567,7 @@ void art_shapenodepoly(NODEINST *ni, INTBIG box, POLYGON *poly)
 	/* handle displayable variables */
 	if (box >= tech_realpolys)
 	{
-		(void)tech_filldisplayablenvar(ni, poly);
+		(void)tech_filldisplayablenvar(ni, poly, tech_curwindowpart, 0);
 	} else
 	{
 		pindex = ni->proto->primindex;
@@ -655,12 +656,12 @@ void art_shapeportpoly(NODEINST *ni, PORTPROTO *pp, POLYGON *poly, XARRAY trans,
 	}
 
 	/* just use first graphic polygon as the port */
-	(void)art_nodepolys(ni, 0);
+	(void)art_nodepolys(ni, 0, NOWINDOWPART);
 	art_shapenodepoly(ni, 0, poly);
 	xformpoly(poly, trans);
 }
 
-INTBIG art_arcpolys(ARCINST *ai)
+INTBIG art_arcpolys(ARCINST *ai, WINDOWPART *win)
 {
 	REGISTER INTBIG def;
 
@@ -670,7 +671,7 @@ INTBIG art_arcpolys(ARCINST *ai)
 
 	/* add in displayable variables */
 	tech_realpolys = def;
-	def += tech_displayableavars(ai);
+	def += tech_displayableavars(ai, win);
 	return(def);
 }
 
@@ -683,7 +684,7 @@ void art_shapearcpoly(ARCINST *ai, INTBIG box, POLYGON *poly)
 	/* handle displayable variables */
 	if (box >= tech_realpolys)
 	{
-		(void)tech_filldisplayableavar(ai, poly);
+		(void)tech_filldisplayableavar(ai, poly, tech_curwindowpart, 0);
 		return;
 	}
 
