@@ -42,8 +42,8 @@
 
 /******************** TREE STRUCTURE FOR ARCHITECTURE FILE ********************/
 
-#define	MAXLINE	   500		/* max characters on FPGA input line */
-#define	MAXDEPTH	50		/* max depth of FPGA nesting */
+#define MAXLINE	   500		/* max characters on FPGA input line */
+#define MAXDEPTH	50		/* max depth of FPGA nesting */
 
 #define PARAMBRANCH  1		/* parameter is a subtree */
 #define PARAMATOM    2		/* parameter is atomic */
@@ -65,7 +65,7 @@ LISPTREE *fpga_freelisptree = 0;
 
 /******************** ADDITIONAL INFORMATION ABOUT PRIMITIVES ********************/
 
-#define	ACTIVEPART   1			/* set if segment or pip is active */
+#define ACTIVEPART   1			/* set if segment or pip is active */
 #define ACTIVESAVE   2			/* saved area for segment/pip activity */
 
 typedef struct
@@ -201,12 +201,12 @@ void       fpga_setrepeater(char *name);
 
 /******************** LAYERS ********************/
 
-#define	MAXLAYERS   4		/* total layers below      */
+#define MAXLAYERS   4		/* total layers below      */
 
-#define	LWIRE       0		/* wire                    */
-#define	LCOMP       1		/* component               */
-#define	LPIP        2		/* pip                     */
-#define	LREPEATER   3		/* repeater                */
+#define LWIRE       0		/* wire                    */
+#define LCOMP       1		/* component               */
+#define LPIP        2		/* pip                     */
+#define LREPEATER   3		/* repeater                */
 
 static GRAPHICS fpga_w_lay = {LAYERO, RED, SOLIDC, SOLIDC,
 	{0xFFFF,0xFFFF,0xFFFF,0xFFFF,0xFFFF,0xFFFF,0xFFFF,0xFFFF}, NOVARIABLE, 0};
@@ -225,9 +225,9 @@ static char *fpga_layer_letters[MAXLAYERS] = {"w", "c", "p", "r"};
 
 /******************** ARCS ********************/
 
-#define	ARCPROTOCOUNT   1
+#define ARCPROTOCOUNT   1
 
-#define	AWIRE           0	/* wire               */
+#define AWIRE           0	/* wire               */
 
 /* wire arc */
 static TECH_ARCLAY fpga_al_w[] = { {LWIRE,0,FILLED} };
@@ -245,7 +245,7 @@ INTBIG fpga_pc_wire[]	= {-1, AWIRE, ALLGEN, -1};
 
 /******************** NODES ********************/
 
-#define	NODEPROTOCOUNT	3
+#define NODEPROTOCOUNT	3
 
 #define NWIREPIN	 1		/* wire pin */
 #define NPIP		 2		/* pip */
@@ -464,19 +464,19 @@ void fpga_setmode(INTSML count, char *par[])
 		if (namesamen(pp, "empty", l) == 0)
 		{
 			fpga_internaldisplay = (fpga_internaldisplay & ~DISPLAYLEVEL) | NOPRIMDISPLAY;
-			ttyputverbose(_("No internal display"));
+			ttyputverbose(M_("No internal display"));
 			return;
 		}
 		if (namesamen(pp, "full", l) == 0)
 		{
 			fpga_internaldisplay = (fpga_internaldisplay & ~DISPLAYLEVEL) | FULLPRIMDISPLAY;
-			ttyputverbose(_("Full internal display"));
+			ttyputverbose(M_("Full internal display"));
 			return;
 		}
 		if (namesamen(pp, "active", l) == 0)
 		{
 			fpga_internaldisplay = (fpga_internaldisplay & ~DISPLAYLEVEL) | ACTIVEPRIMDISPLAY;
-			ttyputverbose(_("Active internal display"));
+			ttyputverbose(M_("Active internal display"));
 			return;
 		}
 		if (namesamen(pp, "text", l) == 0)
@@ -484,21 +484,21 @@ void fpga_setmode(INTSML count, char *par[])
 			if (count == 2)
 			{
 				if ((fpga_internaldisplay & TEXTDISPLAY) == 0)
-					ttyputmsg(_("Text not displayed")); else
-						ttyputmsg(_("Text is displayed"));
+					ttyputmsg(M_("Text not displayed")); else
+						ttyputmsg(M_("Text is displayed"));
 				return;
 			}
 			l = strlen(pp = par[2]);
 			if (namesamen(pp, "on", l) == 0 && l >= 2)
 			{
 				fpga_internaldisplay |= TEXTDISPLAY;
-				ttyputverbose(_("Text is displayed"));
+				ttyputverbose(M_("Text is displayed"));
 				return;
 			}
 			if (namesamen(pp, "off", l) == 0 && l >= 2)
 			{
 				fpga_internaldisplay &= ~TEXTDISPLAY;
-				ttyputverbose(_("Text not displayed"));
+				ttyputverbose(M_("Text not displayed"));
 				return;
 			}
 			ttyputbadusage("technology tell fpga display-level text");
@@ -595,7 +595,7 @@ void fpga_setmode(INTSML count, char *par[])
 	ttyputbadusage("technology tell fpga");
 }
 
-INTBIG fpga_nodepolys(NODEINST *ni, INTBIG *reasonable)
+INTBIG fpga_nodepolys(NODEINST *ni, INTBIG *reasonable, WINDOWPART *win)
 {
 	REGISTER INTBIG total, i;
 	REGISTER INTBIG pindex, j;
@@ -700,7 +700,7 @@ INTBIG fpga_nodepolys(NODEINST *ni, INTBIG *reasonable)
 	/* add in displayable variables */
 	tech_realpolys = total;
 	if ((fpga_internaldisplay&TEXTDISPLAY) != 0)
-		total += tech_displayablenvars(ni);
+		total += tech_displayablenvars(ni, tech_curwindowpart);
 	if (reasonable != 0) *reasonable = total;
 	return(total);
 }
@@ -715,7 +715,7 @@ void fpga_shapenodepoly(NODEINST *ni, INTBIG box, POLYGON *poly)
 	/* handle displayable variables */
 	if (box >= tech_realpolys)
 	{
-		(void)tech_filldisplayablenvar(ni, poly);
+		(void)tech_filldisplayablenvar(ni, poly, tech_curwindowpart, 0);
 		return;
 	}
 
@@ -754,7 +754,7 @@ void fpga_shapenodepoly(NODEINST *ni, INTBIG box, POLYGON *poly)
 				poly->count = 4;
 				poly->style = TEXTBOX;
 				poly->string = ni->proto->primname;
-				poly->font = TXTLARGE;
+				poly->font = TXT4L;
 				poly->tech = fpga_tech;
 				poly->layer = LCOMP;
 				break;
@@ -830,7 +830,7 @@ void fpga_shapenodepoly(NODEINST *ni, INTBIG box, POLYGON *poly)
 	poly->desc = fpga_layers[poly->layer];
 }
 
-INTBIG fpga_nodeEpolys(NODEINST *ni, INTBIG *reasonable)
+INTBIG fpga_nodeEpolys(NODEINST *ni, INTBIG *reasonable, WINDOWPART *win)
 {
 	if (reasonable != 0) *reasonable = 0;
 	return(0);
@@ -868,7 +868,7 @@ void fpga_shapeportpoly(NODEINST *ni, PORTPROTO *pp, POLYGON *poly, XARRAY trans
 	xform(poly->xv[0], poly->yv[0], &poly->xv[0], &poly->yv[0], trans);
 }
 
-INTBIG fpga_arcpolys(ARCINST *ai)
+INTBIG fpga_arcpolys(ARCINST *ai, WINDOWPART *win)
 {
 	REGISTER INTBIG i, aindex;
 
@@ -885,7 +885,7 @@ INTBIG fpga_arcpolys(ARCINST *ai)
 	/* add in displayable variables */
 	tech_realpolys = i;
 	if ((fpga_internaldisplay&TEXTDISPLAY) != 0)
-		i += tech_displayableavars(ai);
+		i += tech_displayableavars(ai, win);
 	return(i);
 }
 
@@ -897,7 +897,7 @@ void fpga_shapearcpoly(ARCINST *ai, INTBIG box, POLYGON *poly)
 	/* handle displayable variables */
 	if (box >= tech_realpolys)
 	{
-		(void)tech_filldisplayableavar(ai, poly);
+		(void)tech_filldisplayableavar(ai, poly, tech_curwindowpart, 0);
 		return;
 	}
 
@@ -1062,7 +1062,7 @@ INTSML fpga_arcactive(ARCINST *ai)
 		((NODEINST **)ptr)[0] = nilist[i];   ptr += (sizeof (NODEINST *));
 	}
 	((INTSML *)ptr)[0] = value;
-	nextvarchangequiet();
+	nextchangequiet();
 	setvalkey((INTBIG)ai, VARCINST, fpga_arcactivecachekey, (INTBIG)fpga_arcbuf,
 		VCHAR|VISARRAY|(size<<VLENGTHSH)|VDONTSAVE);
 	return(value);
@@ -1166,7 +1166,7 @@ void fpga_reevaluatepips(NODEINST *ni, FPGANODE *fn)
 			((INTSML *)ptr)[0] = 0;
 		ptr += SIZEOFINTSML;
 	}
-	nextvarchangequiet();
+	nextchangequiet();
 	setvalkey((INTBIG)ni, VNODEINST, fpga_nodepipcachekey, (INTBIG)fpga_pipbuf,
 		VCHAR|VISARRAY|(size<<VLENGTHSH)|VDONTSAVE);
 }
@@ -1633,7 +1633,6 @@ INTSML fpga_makeprimitive(LISPTREE *lt)
 	REGISTER FPGAPIP *fpipblock;
 	REGISTER FPGAPORT *fpblock, *fp;
 	REGISTER char *primname, *primsizex, *primsizey;
-	void tech_initnodesizeoffset(void);
 
 	/* find all of the pieces of this primitive */
 	ltattribute = ltnets = ltports = ltcomponents = 0;
@@ -1771,7 +1770,6 @@ INTSML fpga_makeprimitive(LISPTREE *lt)
 		setval((INTBIG)fpga_tech, VTECHNOLOGY, "TECH_node_width_offset", (INTBIG)newnodewidoff,
 			VFRACT|VDONTSAVE|VISARRAY|((primnp->primindex*4)<<VLENGTHSH));
 		efree((char *)newnodewidoff);
-		tech_initnodesizeoffset();
 	}
 
 	/* add any unrecognized attributes */
@@ -2627,7 +2625,7 @@ INTSML fpga_makeblockinstance(NODEPROTO *facet, LISPTREE *lt)
 		{
 			yoff = (ni->highy-ni->lowy) / lambda;
 			var->textdescript = (var->textdescript & ~(VTYOFF|VTSIZE|VTPOSITION)) |
-				(yoff << VTYOFFSH) | (TXTLARGE << VTSIZESH) | VTPOSBOXED;
+				(yoff << VTYOFFSH) | (defaulttextsize(3) << VTSIZESH) | VTPOSBOXED;
 		}
 	}
 	return(0);
@@ -2719,6 +2717,7 @@ INTSML fpga_makeblockrepeater(NODEPROTO *facet, LISPTREE *lt)
 	REGISTER INTSML j, angle;
 	REGISTER INTBIG portax, portay, portbx, portby, ctrx, ctry, length, lambda;
 	REGISTER NODEINST *ni;
+	REGISTER VARIABLE *var;
 	REGISTER LISPTREE *ltname, *ltporta, *ltportb, *scanlt;
 
 	ltname = ltporta = ltportb = 0;
@@ -2796,7 +2795,10 @@ INTSML fpga_makeblockrepeater(NODEPROTO *facet, LISPTREE *lt)
 			ttyputerr(_("Need one atom in 'name' of block repeater (line %ld)"), ltname->lineno);
 			return(1);
 		}
-		setvalkey((INTBIG)ni, VNODEINST, el_node_name, (INTBIG)ltname->paramvalue[0], VSTRING);
+		var = setvalkey((INTBIG)ni, VNODEINST, el_node_name, (INTBIG)ltname->paramvalue[0], VSTRING);
+		if (var != NOVARIABLE)
+			var->textdescript = (var->textdescript & ~VTSIZE) |
+				(defaulttextsize(3) << VTSIZESH);
 	}
 	return(0);
 }
