@@ -47,15 +47,15 @@ static TECHNOLOGY *gem_tech;
 
 /****************************** LAYERS **************************************/
 
-#define	MAXLAYERS  7		/* total layers below */
+#define MAXLAYERS  7		/* total layers below */
 
-#define	LELEGROUP  0		/* elements and groups */
-#define	LGENARC    1		/* general arcs */
-#define	LTEMPARC   2		/* temporal arcs */
-#define	LCAUSEARC  3		/* causal arcs */
-#define	LPREARC    4		/* prerequisite arcs */
-#define	LNONARC    5		/* nondeterministic arcs */
-#define	LFORKARC   6		/* fork arcs */
+#define LELEGROUP  0		/* elements and groups */
+#define LGENARC    1		/* general arcs */
+#define LTEMPARC   2		/* temporal arcs */
+#define LCAUSEARC  3		/* causal arcs */
+#define LPREARC    4		/* prerequisite arcs */
+#define LNONARC    5		/* nondeterministic arcs */
+#define LFORKARC   6		/* fork arcs */
 
 static GRAPHICS gem_e_lay = {LAYERO, RED, SOLIDC, SOLIDC,
 /* node */		{0, 0, 0, 0, 0, 0, 0, 0}, NOVARIABLE, 0};
@@ -84,13 +84,13 @@ static char *gem_layer_letters[MAXLAYERS] = {"e", "g", "t", "c", "p", "n", "f"};
 
 /********************************* ARCS *************************************/
 
-#define	ARCPROTOCOUNT  6
-#define	AGENERAL       0		/* general arc */
-#define	ATEMP          1		/* temporal arc */
-#define	ACAUSE         2		/* causal arc */
-#define	APREREQ        3		/* prerequisite arc */
-#define	ANONDET        4		/* nondeterministic arc */
-#define	AFORK          5		/* fork arc */
+#define ARCPROTOCOUNT  6
+#define AGENERAL       0		/* general arc */
+#define ATEMP          1		/* temporal arc */
+#define ACAUSE         2		/* causal arc */
+#define APREREQ        3		/* prerequisite arc */
+#define ANONDET        4		/* nondeterministic arc */
+#define AFORK          5		/* fork arc */
 
 /* General arc */
 static TECH_ARCLAY gem_gw[] = {{LGENARC,0,VECTORS}};
@@ -168,15 +168,15 @@ static char *gem_event4name[]  = {(char *)-H0, (char *)K2,  (char *)0,(char *)-K
 
 /********************************** NODES **********************************/
 
-#define	NODEPROTOCOUNT	 8
-#define	NELEMENT         1		/* element */
-#define	NGROUP           2		/* group */
-#define	NGENERALP        3		/* general arc pin */
-#define	NTEMPP           4		/* temporal arc pin */
-#define	NCAUSEP          5		/* causal arc pin */
-#define	NPREREQP         6		/* prerequisite arc pin */
-#define	NNONDETP         7		/* nondeterministic arc pin */
-#define	NFORKP           8		/* fork arc PIN */
+#define NODEPROTOCOUNT	 8
+#define NELEMENT         1		/* element */
+#define NGROUP           2		/* group */
+#define NGENERALP        3		/* general arc pin */
+#define NTEMPP           4		/* temporal arc pin */
+#define NCAUSEP          5		/* causal arc pin */
+#define NPREREQP         6		/* prerequisite arc pin */
+#define NNONDETP         7		/* nondeterministic arc pin */
+#define NFORKP           8		/* fork arc PIN */
 
 /* element */
 static TECH_PORTS gem_e_p[] = {				/* ports */
@@ -338,7 +338,7 @@ void gem_shapenodepoly(NODEINST *ni, INTBIG box, POLYGON *poly)
 	/* handle displayable variables */
 	if (box >= tech_realpolys)
 	{
-		(void)tech_filldisplayablenvar(ni, poly);
+		(void)tech_filldisplayablenvar(ni, poly, tech_curwindowpart, 0);
 		return;
 	}
 
@@ -350,14 +350,14 @@ void gem_shapenodepoly(NODEINST *ni, INTBIG box, POLYGON *poly)
 	if (box == 0) return;
 
 	/* get element message */
-	poly->font = TXTSMALL;
+	poly->font = TXT1L;
 	poly->tech = gem_tech;
 	switch (box)
 	{
 		case 1:
 			if (element == 0) element = makekey("GEM_element");
 			var = getvalkey((INTBIG)ni, VNODEINST, VSTRING, element);
-			poly->font = TXTLARGE;
+			poly->font = TXT4L;
 			break;
 		case 2:
 			if (event1 == 0) event1 = makekey("GEM_event1");
@@ -389,7 +389,7 @@ void gem_shapearcpoly(ARCINST *ai, INTBIG box, POLYGON *poly)
 	/* handle displayable variables */
 	if (box >= tech_realpolys)
 	{
-		(void)tech_filldisplayableavar(ai, poly);
+		(void)tech_filldisplayableavar(ai, poly, tech_curwindowpart, 0);
 		return;
 	}
 
