@@ -46,12 +46,12 @@ void gen_makeunivlist(void);
 
 /******************** LAYERS ********************/
 
-#define	MAXLAYERS       5		/* total layers below */
-#define	LUNIV           0		/* universal layer (connects all) */
-#define	LINVIS          1		/* invisible layer (nonelectrical) */
-#define	LUNROUTED       2		/* unrouted layer (for routers) */
-#define	LGLYPH          3		/* glyph layer (for menu icons) */
-#define	LDRC            4		/* drc ignore layer */
+#define MAXLAYERS       5		/* total layers below */
+#define LUNIV           0		/* universal layer (connects all) */
+#define LINVIS          1		/* invisible layer (nonelectrical) */
+#define LUNROUTED       2		/* unrouted layer (for routers) */
+#define LGLYPH          3		/* glyph layer (for menu icons) */
+#define LDRC            4		/* drc ignore layer */
 
 static GRAPHICS gen_un_lay = {LAYERO, MENTXT, SOLIDC, SOLIDC,
 /* univ */					{0,0,0,0,0,0,0,0}, NOVARIABLE, 0};
@@ -78,7 +78,7 @@ static char *gen_layer_letters[MAXLAYERS] = {"u", "i", "r", "g", "d"};
 
 /******************** ARCS ********************/
 
-#define	ARCPROTOCOUNT     3
+#define ARCPROTOCOUNT     3
 #define GENAUNIV          0
 #define GENAINVIS         1
 #define GENAUNROUTED      2
@@ -115,13 +115,13 @@ static INTBIG gen_pc_riu[] = {-1, GENAUNROUTED, GENAINVIS, GENAUNIV, -1};
 
 /******************** NODES ********************/
 
-#define	NODEPROTOCOUNT     6
-#define	NUNIV              1		/* universal pin */
-#define	NINVIS             2		/* invisible pin */
-#define	NUNROUTED          3		/* unrouted pin */
-#define	NCENTER            4		/* facet center */
-#define	NPORT              5		/* port declaration (for tech edit) */
-#define	NDRC               6		/* drc ignore mask (for ECAD's DRC) */
+#define NODEPROTOCOUNT     6
+#define NUNIV              1		/* universal pin */
+#define NINVIS             2		/* invisible pin */
+#define NUNROUTED          3		/* unrouted pin */
+#define NCENTER            4		/* facet center */
+#define NPORT              5		/* port declaration (for tech edit) */
+#define NDRC               6		/* drc ignore mask (for ECAD's DRC) */
 
 static INTBIG gen_fullbox[]    = {LEFTEDGE, BOTEDGE, RIGHTEDGE, TOPEDGE};
 static INTBIG gen_disccenter[] = {CENTER,   CENTER,  RIGHTEDGE, CENTER};
@@ -257,7 +257,7 @@ INTSML gen_initprocess(TECHNOLOGY *tech, INTSML pass)
 
 		case 1:
 			/* create list of ALL arcs (now and when technologies change) */
-			registertechnologycache(gen_makeunivlist);
+			registertechnologycache(gen_makeunivlist, 0, 0);
 
 			gen_univpinprim = getnodeproto("Generic:Universal-Pin");
 			gen_invispinprim = getnodeproto("Generic:Invisible-Pin");
@@ -334,7 +334,7 @@ void gen_makeunivlist(void)
 	first = 0;
 }
 
-INTBIG gen_nodepolys(NODEINST *ni, INTBIG *reasonable)
+INTBIG gen_nodepolys(NODEINST *ni, INTBIG *reasonable, WINDOWPART *win)
 {
 	REGISTER INTBIG pindex, count;
 	REGISTER INTSML start, end;
@@ -403,7 +403,7 @@ INTBIG gen_nodepolys(NODEINST *ni, INTBIG *reasonable)
 
 	/* add in displayable variables */
 	tech_realpolys = count;
-	count += tech_displayablenvars(ni);
+	count += tech_displayablenvars(ni, tech_curwindowpart);
 	if (reasonable != 0) *reasonable = count;
 	return(count);
 }
@@ -419,7 +419,7 @@ void gen_shapenodepoly(NODEINST *ni, INTBIG box, POLYGON *poly)
 	pindex = ni->proto->primindex;
 	if (box >= tech_realpolys)
 	{
-		(void)tech_filldisplayablenvar(ni, poly);
+		(void)tech_filldisplayablenvar(ni, poly, tech_curwindowpart, 0);
 		if (pindex == NINVIS) poly->desc = &gen_int_lay;
 		return;
 	}
@@ -489,7 +489,7 @@ void gen_shapeportpoly(NODEINST *ni, PORTPROTO *pp, POLYGON *poly, XARRAY trans,
 	tech_fillportpoly(ni, pp, poly, trans, gen_nodeprotos[pindex-1], OPENED);
 }
 
-INTBIG gen_arcpolys(ARCINST *ai)
+INTBIG gen_arcpolys(ARCINST *ai, WINDOWPART *win)
 {
 	REGISTER INTBIG i;
 
@@ -497,7 +497,7 @@ INTBIG gen_arcpolys(ARCINST *ai)
 
 	/* add in displayable variables */
 	tech_realpolys = i;
-	i += tech_displayableavars(ai);
+	i += tech_displayableavars(ai, win);
 	return(i);
 }
 
@@ -509,7 +509,7 @@ void gen_shapearcpoly(ARCINST *ai, INTBIG box, POLYGON *poly)
 	/* handle displayable variables */
 	if (box >= tech_realpolys)
 	{
-		(void)tech_filldisplayableavar(ai, poly);
+		(void)tech_filldisplayableavar(ai, poly, tech_curwindowpart, 0);
 		return;
 	}
 
