@@ -42,26 +42,26 @@ static TECHNOLOGY *rcmos_tech;
 
 /******************** LAYERS ********************/
 
-#define	MAXLAYERS 19		/* total layers below         */
-#define	LMETAL1    0		/* metal layer 1              */
-#define	LMETAL2    1		/* metal layer 2              */
-#define	LPOLY      2		/* polysilicon                */
-#define	LSACT      3		/* S active (diffusion)       */
-#define	LDACT      4		/* D active (diffusion)       */
-#define	LSELECT    5		/* Select                     */
-#define	LWELL      6		/* Well                       */
-#define	LCUT       7		/* contact cut                */
-#define	LVIA       8		/* metal-to-metal via         */
-#define	LPASS      9		/* passivation (overglass)    */
-#define	LPOLYCUT  10		/* poly contact cut           */
-#define	LACTCUT   11		/* active contact cut         */
-#define	LMET1P    12		/* pseudo metal 1             */
-#define	LMET2P    13		/* pseudo metal 2             */
-#define	LPOLYP    14		/* pseudo polysilicon         */
-#define	LSACTP    15		/* pseudo S active            */
-#define	LDACTP    16		/* pseudo D active            */
-#define	LSELECTP  17		/* pseudo Select              */
-#define	LWELLP    18		/* pseudo Well                */
+#define MAXLAYERS 19		/* total layers below         */
+#define LMETAL1    0		/* metal layer 1              */
+#define LMETAL2    1		/* metal layer 2              */
+#define LPOLY      2		/* polysilicon                */
+#define LSACT      3		/* S active (diffusion)       */
+#define LDACT      4		/* D active (diffusion)       */
+#define LSELECT    5		/* Select                     */
+#define LWELL      6		/* Well                       */
+#define LCUT       7		/* contact cut                */
+#define LVIA       8		/* metal-to-metal via         */
+#define LPASS      9		/* passivation (overglass)    */
+#define LPOLYCUT  10		/* poly contact cut           */
+#define LACTCUT   11		/* active contact cut         */
+#define LMET1P    12		/* pseudo metal 1             */
+#define LMET2P    13		/* pseudo metal 2             */
+#define LPOLYP    14		/* pseudo polysilicon         */
+#define LSACTP    15		/* pseudo S active            */
+#define LDACTP    16		/* pseudo D active            */
+#define LSELECTP  17		/* pseudo Select              */
+#define LWELLP    18		/* pseudo Well                */
 
 static GRAPHICS rcmos_m1_lay = {LAYERT1,COLORT1, SOLIDC, PATTERNED,
 /* metal-1 layer */		{0x2222, /*   X   X   X   X  */
@@ -285,16 +285,16 @@ static TECH_COLORMAP rcmos_colmap[32] =
 
 /******************** ARCS ********************/
 
-#define	ARCPROTOCOUNT  9
-#define	AMETAL1        0	/* metal-1                   */
-#define	AMETAL2        1	/* metal-2                   */
-#define	APOLY          2	/* polysilicon               */
-#define	ASACT          3	/* S-active                  */
-#define	ADACT          4	/* D-active                  */
-#define	ASUACT         5	/* Substrate active          */
-#define	AWEACT         6	/* Well active               */
-#define	ASTRANS        7	/* S-transistor              */
-#define	ADTRANS        8	/* D-transistor              */
+#define ARCPROTOCOUNT  9
+#define AMETAL1        0	/* metal-1                   */
+#define AMETAL2        1	/* metal-2                   */
+#define APOLY          2	/* polysilicon               */
+#define ASACT          3	/* S-active                  */
+#define ADACT          4	/* D-active                  */
+#define ASUACT         5	/* Substrate active          */
+#define AWEACT         6	/* Well active               */
+#define ASTRANS        7	/* S-transistor              */
+#define ADTRANS        8	/* D-transistor              */
 
 /* metal 1 arc */
 static TECH_ARCLAY rcmos_al_m1[] = {{LMETAL1,0,FILLED }};
@@ -390,33 +390,33 @@ static INTBIG rcmos_pc_null[] = {-1, ALLGEN, -1};
 
 /******************** NODES ********************/
 
-#define	NODEPROTOCOUNT 26
-#define	NMETAL1P        1	/* metal-1 pin */
-#define	NMETAL2P        2	/* metal-2 pin */
-#define	NPOLYP          3	/* polysilicon pin */
-#define	NSACTP          4	/* S-active pin */
-#define	NDACTP          5	/* D-active pin */
-#define	NSUACTP         6	/* Substrate active pin */
-#define	NWEACTP         7	/* Well active pin */
-#define	NSTRANSP        8	/* S-transistor pin */
-#define	NDTRANSP        9	/* D-transistor pin */
-#define	NMETSACTC      10	/* metal-1-S-active contact */
-#define	NMETDACTC      11	/* metal-1-D-active contact */
-#define	NMETPOLYC      12	/* metal-1-polysilicon contact */
-#define	NVIA           13	/* metal-1-metal-2 contact */
-#define	NWBUT          14	/* metal-1-Well contact */
-#define	NSBUT          15	/* metal-1-Substrate contact */
-#define	NMETAL1N       16	/* metal-1 node */
-#define	NMETAL2N       17	/* metal-2 node */
-#define	NPOLYN         18	/* polysilicon node */
-#define	NACTIVEN       19	/* active node */
-#define	NSELECTN       20	/* select node */
-#define	NCUTN          21	/* cut node */
-#define	NPCUTN         22	/* poly cut node */
-#define	NACUTN         23	/* active cut node */
-#define	NVIAN          24	/* via node */
-#define	NWELLN         25	/* well node */
-#define	NPASSN         26	/* passivation node */
+#define NODEPROTOCOUNT 26
+#define NMETAL1P        1	/* metal-1 pin */
+#define NMETAL2P        2	/* metal-2 pin */
+#define NPOLYP          3	/* polysilicon pin */
+#define NSACTP          4	/* S-active pin */
+#define NDACTP          5	/* D-active pin */
+#define NSUACTP         6	/* Substrate active pin */
+#define NWEACTP         7	/* Well active pin */
+#define NSTRANSP        8	/* S-transistor pin */
+#define NDTRANSP        9	/* D-transistor pin */
+#define NMETSACTC      10	/* metal-1-S-active contact */
+#define NMETDACTC      11	/* metal-1-D-active contact */
+#define NMETPOLYC      12	/* metal-1-polysilicon contact */
+#define NVIA           13	/* metal-1-metal-2 contact */
+#define NWBUT          14	/* metal-1-Well contact */
+#define NSBUT          15	/* metal-1-Substrate contact */
+#define NMETAL1N       16	/* metal-1 node */
+#define NMETAL2N       17	/* metal-2 node */
+#define NPOLYN         18	/* polysilicon node */
+#define NACTIVEN       19	/* active node */
+#define NSELECTN       20	/* select node */
+#define NCUTN          21	/* cut node */
+#define NPCUTN         22	/* poly cut node */
+#define NACUTN         23	/* active cut node */
+#define NVIAN          24	/* via node */
+#define NWELLN         25	/* well node */
+#define NPASSN         26	/* passivation node */
 
 /* for geometry */
 static INTBIG rcmos_fullbox[8] = {LEFTEDGE, BOTEDGE, RIGHTEDGE, TOPEDGE};
@@ -847,8 +847,8 @@ static INTBIG rcmos_node_widoff[NODEPROTOCOUNT*4] = {
 /******************** SIMULATION VARIABLES ********************/
 
 /* for SPICE simulation */
-#define	RCMOS_MIN_RESIST	50.0f	/* minimum resistance consider */
-#define	RCMOS_MIN_CAPAC	     0.04f	/* minimum capacitance consider */
+#define RCMOS_MIN_RESIST	50.0f	/* minimum resistance consider */
+#define RCMOS_MIN_CAPAC	     0.04f	/* minimum capacitance consider */
 float rcmos_sim_spice_resistance[MAXLAYERS] = {  /* per square micron */
 	0.03f /* METAL1 */,    0.03f /* METAL2 */,    50.0f /* POLY */,
 	10.0f /* SACT */,      10.0f /* DACT */,
@@ -937,7 +937,7 @@ INTSML rcmos_initprocess(TECHNOLOGY *tech, INTSML pass)
 	return(0);
 }
 
-INTBIG rcmos_arcpolys(ARCINST *ai)
+INTBIG rcmos_arcpolys(ARCINST *ai, WINDOWPART *win)
 {
 	REGISTER INTBIG widestp, widestd, i, lambda, pwid, dwid, orip, orid, amt;
 	REGISTER INTBIG total, aindex;
@@ -987,7 +987,7 @@ INTBIG rcmos_arcpolys(ARCINST *ai)
 
 	/* add in displayable variables */
 	tech_realpolys = total;
-	total += tech_displayableavars(ai);
+	total += tech_displayableavars(ai, win);
 	return(total);
 }
 
@@ -999,7 +999,7 @@ void rcmos_shapearcpoly(ARCINST *ai, INTBIG box, POLYGON *poly)
 	/* handle displayable variables */
 	if (box >= tech_realpolys)
 	{
-		(void)tech_filldisplayableavar(ai, poly);
+		(void)tech_filldisplayableavar(ai, poly, tech_curwindowpart, 0);
 		return;
 	}
 
