@@ -97,7 +97,7 @@ ARCINST *aconnect(GEOM *fromgeom, PORTPROTO *frompp, GEOM *togeom, PORTPROTO *to
 		if (ni != NONODEINST && ni->proto->primindex == 0)
 		{
 			/* found facet connected to arcinst: search for closest portinst */
-			bestdist = HUGEINT;
+			bestdist = MAXINTBIG;
 			for(fpt = ni->proto->firstportproto; fpt != NOPORTPROTO; fpt = fpt->nextportproto)
 			{
 				/* see if the arcinst can connect at this portinst */
@@ -113,7 +113,7 @@ ARCINST *aconnect(GEOM *fromgeom, PORTPROTO *frompp, GEOM *togeom, PORTPROTO *to
 			}
 
 			/* adjust prefered position to closest port (manhattan only!!!) */
-			if (bestdist < HUGEINT)
+			if (bestdist < MAXINTBIG)
 			{
 				if (ai->end[0].xpos == ai->end[1].xpos) prefy = besty;
 				if (ai->end[0].ypos == ai->end[1].ypos) prefx = bestx;
@@ -277,7 +277,7 @@ ARCINST *us_makeconnection(NODEINST *fromnodeinst, PORTPROTO *fromportproto,
 	INTBIG x, y, hx, hy, lx, ly;
 
 	/* see if the cursor is near a port on a facet */
-	bestdist = HUGEINT;
+	bestdist = MAXINTBIG;
 	if (fromportproto == NOPORTPROTO && fromnodeinst->proto->primindex == 0)
 		for(fpt = fromnodeinst->proto->firstportproto; fpt != NOPORTPROTO; fpt = fpt->nextportproto)
 	{
@@ -322,16 +322,8 @@ ARCINST *us_makeconnection(NODEINST *fromnodeinst, PORTPROTO *fromportproto,
 		return(newai);
 	}
 
-#if 0
-	/* now try a zig-zag if in Manhattan and port angles are right */
-	if (ang == 900 && nozigzag == 0 && (fromportproto == NOPORTPROTO ||
-		((fromportproto->userbits&PORTARANGE) >> PORTARANGESH) == 180) &&
-			(toportproto == NOPORTPROTO ||
-				((toportproto->userbits&PORTARANGE) >> PORTARANGESH) == 180))
-#else
 	/* now try a zig-zag if in Manhattan allowed */
 	if (ang == 900 && nozigzag == 0)
-#endif
 	{
 		/*
 		 * check location of cursor with respect to fromnode and tonode.
@@ -424,20 +416,30 @@ ARCINST *us_directarcinst(NODEINST *fromnodeinst, PORTPROTO *fromppt,
 	/* precompute better positions if ports were specified */
 	if (toppt != NOPORTPROTO)
 	{
+		portposition(tonodeinst, toppt, &tpx, &tpy);
 		tpoly->xv[0] = prefx;   tpoly->yv[0] = prefy;   tpoly->count = 1;
 		shapeportpoly(tonodeinst, toppt, tpoly, 1);
 		reduceportpoly(tpoly, tonodeinst, toppt, rwid);
-		tpx = prefx;   tpy = prefy;
-		closestpoint(tpoly, &tpx, &tpy);
+		if ((toppt->userbits&PORTISOLATED) != 0)
+		{
+			/* use prefered location on isolated ports */
+			tpx = prefx;   tpy = prefy;
+			closestpoint(tpoly, &tpx, &tpy);
+		}
 		tstart = toppt;
 	} else tstart = tonodeinst->proto->firstportproto;
 	if (fromppt != NOPORTPROTO)
 	{
+		portposition(fromnodeinst, fromppt, &fpx, &fpy);
 		fpoly->xv[0] = prefx;   fpoly->yv[0] = prefy;   fpoly->count = 1;
 		shapeportpoly(fromnodeinst, fromppt, fpoly, 1);
 		reduceportpoly(fpoly, fromnodeinst, fromppt, rwid);
-		fpx = prefx;   fpy = prefy;
-		closestpoint(fpoly, &fpx, &fpy);
+		if ((fromppt->userbits&PORTISOLATED) != 0)
+		{
+			/* use prefered location on isolated ports */
+			fpx = prefx;   fpy = prefy;
+			closestpoint(fpoly, &fpx, &fpy);
+		}
 		fstart = fromppt;
 	} else fstart = fromnodeinst->proto->firstportproto;
 
@@ -452,9 +454,14 @@ ARCINST *us_directarcinst(NODEINST *fromnodeinst, PORTPROTO *fromppt,
 		fpx = tpx;   fpy = tpy;
 		closestpoint(fpoly, &fpx, &fpy);
 	}
+	if (toppt != NOPORTPROTO)
+	{
+		tpx = fpx;   tpy = fpy;
+		closestpoint(tpoly, &tpx, &tpy);
+	}
 
 	/* search every possible port on the "from" node */
-	bestdist = bestpdist = HUGEINT;
+	bestdist = bestpdist = MAXINTBIG;
 	for(fpt = fstart; fpt != NOPORTPROTO; fpt = fpt->nextportproto)
 	{
 		if (fromppt != NOPORTPROTO && fromppt != fpt) break;
@@ -644,7 +651,7 @@ ARCINST *us_directarcinst(NODEINST *fromnodeinst, PORTPROTO *fromppt,
 		}
 	}
 
-	if (bestdist < HUGEINT)
+	if (bestdist < MAXINTBIG)
 	{
 		ai = us_runarcinst(fromnodeinst, fromportproto, bestfx, bestfy, 
 			tonodeinst, toportproto, besttx, bestty, typ, wid);
@@ -688,7 +695,7 @@ ARCINST *us_onebend(NODEINST *fromnodeinst, PORTPROTO *fromppt, NODEINST *tonode
 	wid = us_figurearcwidth(typ, wid, fromnodeinst, fromppt, prefx, prefy, tonodeinst, toppt, prefx, prefy);
 	rwid = wid - arcprotowidthoffset(typ);
 
-	found = 0;    bestdist = HUGEINT;
+	found = 0;    bestdist = MAXINTBIG;
 	for(fpt = fromnodeinst->proto->firstportproto; fpt != NOPORTPROTO; fpt = fpt->nextportproto)
 	{
 		if (fromppt != NOPORTPROTO && fromppt != fpt) continue;
@@ -699,9 +706,15 @@ ARCINST *us_onebend(NODEINST *fromnodeinst, PORTPROTO *fromppt, NODEINST *tonode
 		if (fpt->connects[i] == NOARCPROTO) continue;
 
 		/* potential port: get information about its position */
-		(void)us_portdistance(fromnodeinst, fpt, prefx, prefy, &fx, &fy, rwid, 1);
+		if ((fpt->userbits&PORTISOLATED) != 0)
+		{
+			(void)us_portdistance(fromnodeinst, fpt, prefx, prefy, &fx, &fy, rwid, 1);
+		} else
+		{
+			portposition(fromnodeinst, fpt, &fx, &fy);
+		}
 
-		for(tpt = tonodeinst->proto->firstportproto; tpt !=NOPORTPROTO; tpt = tpt->nextportproto)
+		for(tpt = tonodeinst->proto->firstportproto; tpt != NOPORTPROTO; tpt = tpt->nextportproto)
 		{
 			if (toppt != NOPORTPROTO && toppt != tpt) continue;
 
@@ -714,7 +727,13 @@ ARCINST *us_onebend(NODEINST *fromnodeinst, PORTPROTO *fromppt, NODEINST *tonode
 			if (tpt->connects[i] == NOARCPROTO) continue;
 
 			/* potential portinst: get information about its position */
-			(void)us_portdistance(tonodeinst, tpt, prefx, prefy, &tx, &ty, rwid, 1);
+			if ((tpt->userbits&PORTISOLATED) != 0)
+			{
+				(void)us_portdistance(tonodeinst, tpt, prefx, prefy, &tx, &ty, rwid, 1);
+			} else
+			{
+				portposition(tonodeinst, tpt, &tx, &ty);
+			}
 
 			if (abs(tx-prefx) + abs(fy-prefy) < abs(fx-prefx) + abs(ty-prefy))
 			{
@@ -882,7 +901,7 @@ ARCINST *us_twobend(NODEINST *fromnodeinst, PORTPROTO *fromppt, NODEINST *tonode
 	rwid = wid - arcprotowidthoffset(typ);
 
 	/* find the "from" port */
-	bestdist = HUGEINT;   fromportproto = NOPORTPROTO;
+	bestdist = MAXINTBIG;   fromportproto = NOPORTPROTO;
 	for(fpt = fromnodeinst->proto->firstportproto; fpt != NOPORTPROTO; fpt = fpt->nextportproto)
 	{
 		if (fromppt != NOPORTPROTO && fromppt != fpt) continue;
@@ -903,7 +922,7 @@ ARCINST *us_twobend(NODEINST *fromnodeinst, PORTPROTO *fromppt, NODEINST *tonode
 	if (fromportproto == NOPORTPROTO) return(NOARCINST);
 
 	/* find the "to" port */
-	bestdist = HUGEINT;   toportproto = NOPORTPROTO;
+	bestdist = MAXINTBIG;   toportproto = NOPORTPROTO;
 	for(tpt = tonodeinst->proto->firstportproto; tpt != NOPORTPROTO; tpt = tpt->nextportproto)
 	{
 		if (toppt != NOPORTPROTO && toppt != tpt) continue;
@@ -947,6 +966,7 @@ ARCINST *us_twobend(NODEINST *fromnodeinst, PORTPROTO *fromppt, NODEINST *tonode
 		if (abs(bestfx-besttx) > abs(bestfy-bestty)) i = 0; else i = 900;
 	}
 	i = (i+450) % 1800;   if (i < 0) i += 1800;
+	gridalign(&prefx, &prefx, us_alignment);
 	if (i < 900)
 	{
 		xi1 = xi2 = prefx;
@@ -1078,8 +1098,7 @@ INTBIG us_widestarcinst(ARCPROTO *ap, NODEINST *ni, PORTPROTO *por)
  */
 INTBIG us_stretchtonodes(ARCPROTO *typ, INTBIG wid, NODEINST *ni1, PORTPROTO *pp1, INTBIG otherx, INTBIG othery)
 {
-	/* this turns out to be a bad idea.  So it is turned off. */
-#if 0
+#if 0		/* this turns out to be a bad idea.  So it is turned off. */
 	REGISTER INTBIG xstretch, ystretch, i, rot, cx, cy;
 	INTBIG ni1x, ni1y;
 
@@ -1141,7 +1160,6 @@ INTBIG us_figurearcwidth(ARCPROTO *typ, INTBIG wid, NODEINST *ni1, PORTPROTO *pp
 	NODEINST *ni2, PORTPROTO *pp2, INTBIG ox2, INTBIG oy2)
 {
 	INTBIG wid1, wid2, stretchwid, stretch1, stretch2, fun;
-	char *extra;
 	REGISTER NODEINST *rni;
 	REGISTER PORTPROTO *rpp;
 
@@ -1154,7 +1172,7 @@ INTBIG us_figurearcwidth(ARCPROTO *typ, INTBIG wid, NODEINST *ni1, PORTPROTO *pp
 		rni = rpp->subnodeinst;
 		rpp = rpp->subportproto;
 	}
-	fun = nodefunction(rni, &extra);
+	fun = nodefunction(rni, 0);
 	if (fun == NPPIN) stretch1 = 0;
 
 	/* see if node 2 is a pin */
@@ -1164,7 +1182,7 @@ INTBIG us_figurearcwidth(ARCPROTO *typ, INTBIG wid, NODEINST *ni1, PORTPROTO *pp
 		rni = rpp->subnodeinst;
 		rpp = rpp->subportproto;
 	}
-	fun = nodefunction(rni, &extra);
+	fun = nodefunction(rni, 0);
 	if (fun == NPPIN) stretch2 = 0;
 
 	if (stretch1 == 0 && stretch2 == 0) return(wid);
@@ -1257,11 +1275,13 @@ NODEINST *us_getnodeonarcinst(GEOM **ipos, PORTPROTO **ipp, GEOM *othergeom,
 	if (prefx == ai->end[0].xpos && prefy == ai->end[0].ypos)
 	{
 		*ipp = ai->end[0].portarcinst->proto;
+		*ipos = ai->end[0].nodeinst->geom;
 		return(ai->end[0].nodeinst);
 	}
 	if (prefx == ai->end[1].xpos && prefy == ai->end[1].ypos)
 	{
 		*ipp = ai->end[1].portarcinst->proto;
+		*ipos = ai->end[1].nodeinst->geom;
 		return(ai->end[1].nodeinst);
 	}
 
