@@ -37,9 +37,9 @@
 extern ARCPROTO **gen_upconn;
 
 /* prototypes for local routines */
-void us_checklibrary(LIBRARY*, INTSML, INTSML*, INTSML*);
+void us_checklibrary(LIBRARY*, INTSML, INTBIG*, INTBIG*);
 INTSML us_checkvariables(INTSML*, VARIABLE*, LIBRARY*);
-INTBIG us_checkrtree(RTNODE*, RTNODE*, NODEPROTO*, INTSML*, INTSML*);
+INTBIG us_checkrtree(RTNODE*, RTNODE*, NODEPROTO*, INTBIG*, INTBIG*);
 
 void us_checkdatabase(INTSML verbose)
 {
@@ -48,13 +48,14 @@ void us_checkdatabase(INTSML verbose)
 	REGISTER NODEINST *ni;
 	REGISTER LIBRARY *lib;
 	REGISTER INTBIG co, i, found;
+	INTBIG warnings, errors;
 	REGISTER CELL *c, *oc;
 	REGISTER TECHNOLOGY *tech;
 	REGISTER AIDENTRY *aid;
 	REGISTER ARCPROTO *ap;
 	REGISTER VIEW *v;
 	REGISTER char *ch;
-	INTSML warnings, errors;
+	char *cellname;
 
 	errors = warnings = 0;
 
@@ -217,7 +218,7 @@ void us_checkdatabase(INTSML verbose)
 		if (*c->cellname == 0)
 		{
 			ttyputmsg(_("Library %s: null cell name; renamed to 'NULL'"), lib->libname);
-			(void)allocstring(&c->cellname, "NULL", c->cluster);
+			(void)setval((INTBIG)c, VCELL, "cellname", (INTBIG)"NULL", VSTRING);
 			warnings++;
 		}
 
@@ -237,26 +238,26 @@ void us_checkdatabase(INTSML verbose)
 		}
 		if (found != 0)
 		{
-			warnings++;
 			ttyputmsg(_("...renamed to '%s'"), c->cellname);
+			db_buildcellhashtable(lib);
+			warnings++;
 		}
 
 		/* make sure cell name is unique */
 		for(;;)
 		{
-			for(oc = lib->firstcell; oc != NOCELL; oc = oc->nextcell)
-				if (oc != c)
-			{
-				if (namesame(oc->cellname, c->cellname) != 0) continue;
-				ttyputmsg(_("Library %s, cell %s: duplicate name"), lib->libname, c->cellname);
-				(void)initinfstr();
-				(void)addstringtoinfstr(c->cellname);
-				(void)addstringtoinfstr(".0");
-				(void)reallocstring(&c->cellname, returninfstr(), c->cluster);
-				warnings++;
-				break;
-			}
+			cellname = c->cellname;
+			c->cellname = "";
+			oc = db_findcellname(cellname, lib);
+			c->cellname = cellname;
 			if (oc == NOCELL) break;
+			ttyputmsg(_("Library %s, cell %s: duplicate name"), lib->libname, c->cellname);
+			(void)initinfstr();
+			(void)addstringtoinfstr(c->cellname);
+			(void)addstringtoinfstr(".0");
+			nextchangequiet();
+			(void)setval((INTBIG)c, VCELL, "cellname", (INTBIG)returninfstr(), VSTRING);
+			warnings++;
 		}
 
 		/* make sure the cell has facets */
@@ -317,6 +318,14 @@ void us_checkdatabase(INTSML verbose)
 			warnings++;
 		}
 
+		/* check that cell is in cell hash table */
+		if (lib->cellhashtablesize > 0 && db_findcellname (c->cellname, c->lib) != c)
+		{
+			ttyputmsg(_("Library %c, cell %s: not in cell hash"), lib->libname, c->cellname);
+			db_buildcellhashtable(lib);
+			warnings++;
+		}
+
 		/* check variables in the cell */
 		if (us_checkvariables(&c->numvar, c->firstvar, lib) != 0)
 		{
@@ -325,6 +334,67 @@ void us_checkdatabase(INTSML verbose)
 		}
 	}
 
+	/* check nodeproto and cell lists in the libraries */
+	for(lib = el_curlib; lib != NOLIBRARY; lib = lib->nextlibrary)
+	{
+		lastnp = NONODEPROTO;
+		for(np = lib->firstnodeproto; np != NONODEPROTO; np = np->nextnodeproto)
+		{
+			if (np->lastnodeproto != lastnp)
+			{
+				ttyputmsg(_("Library %s: invalid lastnodeproto"), lib->libname);
+				np->lastnodeproto = lastnp;
+				warnings++;
+			}
+			lastnp = np;
+		}
+		if (lib->tailnodeproto != lastnp)
+		{
+			ttyputmsg(_("Library %s: invalid tailnodeproto"), lib->libname);
+			lib->tailnodeproto = lastnp;
+			warnings++;
+		}
+
+		/* check cell list */
+		c = lib->firstcell;
+		co = 0;
+		if (c != NOCELL)
+		{
+			while(c->nextcell != NOCELL)
+			{
+				c = c->nextcell;
+				co++;
+			}
+			co++;
+		}
+		if (c != lib->tailcell)
+		{
+			ttyputmsg(_("Library %s: invalid tailcell"), lib->libname);
+			lib->tailcell = c;
+			warnings++;
+		}
+		if (co != lib->numcells)
+		{
+			ttyputmsg(_("Library %s: numcell reported=%d actual=%d"),
+				lib->libname, co, lib->numcells);
+			db_buildcellhashtable(lib);
+			warnings++;
+		}
+		if (lib->cellhashtablesize > 0)
+		{
+			co = 0;
+			for (i = 0; i < lib->cellhashtablesize; i++)
+				if (lib->cellhashtable[i] != NOCELL) co++;
+			if (co != lib->numcells)
+			{
+				ttyputmsg(_("Library %s: numcell=%d in hash=%d"),
+					lib->libname, lib->numcells, co);
+				db_buildcellhashtable(lib);
+				warnings++;
+			}
+		}
+	}
+  
 	/* make sure every facet is both in the library and in a facet */
 	for(lib = el_curlib; lib != NOLIBRARY; lib = lib->nextlibrary)
 	{
@@ -438,7 +508,7 @@ void us_checkdatabase(INTSML verbose)
 	ttyputmsg(_("...Re-issue the check command to make sure"));
 }
 
-void us_checklibrary(LIBRARY *curlib, INTSML verbose, INTSML *warnings, INTSML *errors)
+void us_checklibrary(LIBRARY *curlib, INTSML verbose, INTBIG *warnings, INTBIG *errors)
 {
 	REGISTER NODEPROTO *np, *onp;
 	REGISTER PORTPROTO *pp, *opp, *subpp, *lpp;
@@ -450,10 +520,11 @@ void us_checklibrary(LIBRARY *curlib, INTSML verbose, INTSML *warnings, INTSML *
 	REGISTER TECHNOLOGY *tech;
 	REGISTER NETWORK *net;
 	REGISTER VIEW *v;
-	REGISTER INTSML found, foundtot, foundtrue, facetcenters, neterrors, len, accumulatesize;
-	REGISTER INTBIG totlx, tothx, totly, tothy, truelx, truehx, truely, truehy, i, j, *arr,
-		problems;
+	REGISTER INTBIG found, foundtot, foundtrue, facetcenters, neterrors, len, accumulatesize,
+		totlx, tothx, totly, tothy, truelx, truehx, truely, truehy, i, j, *arr, problems;
+	REGISTER UINTBIG wiped;
 	INTBIG lx, hx, ly, hy, position[2];
+	char newframeinfo[10];
 	REGISTER char *ch;
 	static POLYGON *poly = NOPOLYGON;
 
@@ -483,6 +554,38 @@ void us_checklibrary(LIBRARY *curlib, INTSML verbose, INTSML *warnings, INTSML *
 			ttyputmsg(_("    Facet %s"), describenodeproto(np));
 		}
 
+		/* check specific variables */
+		var = getvalkey((INTBIG)np, VNODEPROTO, VSTRING, el_schematic_page_size);
+		if (var != NOVARIABLE)
+		{
+			strncpy(newframeinfo, (char *)var->addr, 9);
+			i = 0;
+			if (newframeinfo[0] < 'a' || newframeinfo[0] > 'e')
+			{
+				i = 1;
+				newframeinfo[0] = 'a';				
+			}
+			if (newframeinfo[1] != 0 && newframeinfo[1] != 'v')
+			{
+				i = 1;
+				newframeinfo[1] = 0;
+			}
+			if (newframeinfo[1] == 'v' && newframeinfo[2] != 0)
+			{
+				i = 1;
+				newframeinfo[2] = 0;
+			}
+			if (i != 0)
+			{
+				(*warnings)++;
+				ttyputmsg(_("Facet %s: bad frame size info (was '%s', now '%s')"),
+					describenodeproto(np), (char *)var->addr, newframeinfo);
+				setvalkey((INTBIG)np, VNODEPROTO, el_schematic_page_size,
+					(INTBIG)newframeinfo, VSTRING);
+			}
+		}
+
+
 		/* make sure the "facet" primindex is zero */
 		if (np->primindex != 0)
 		{
@@ -575,12 +678,24 @@ void us_checklibrary(LIBRARY *curlib, INTSML verbose, INTSML *warnings, INTSML *
 			if ((subpp->userbits&i) != (pp->userbits&i))
 			{
 				j = (pp->userbits & ~i) | (subpp->userbits & i);
-				ttyputmsg(_("Facet %s, port %s: state bits are 0%o, should be 0%o"),
+				ttyputmsg(_("Facet %s, export %s: state bits are 0%o, should be 0%o"),
 					describenodeproto(np), pp->protoname, pp->userbits, j);
 				pp->userbits = j;
 				(*warnings)++;
 			}
 
+			/* ports on invisible pins must not have an offset */
+			if (pp->subnodeinst->proto == gen_invispinprim)
+			{
+				if ((pp->textdescript&(VTXOFF|VTXOFFNEG|VTYOFF|VTYOFFNEG)) != 0)
+				{
+					pp->textdescript &= ~(VTXOFF|VTXOFFNEG|VTYOFF|VTYOFFNEG);
+					ttyputmsg(_("Facet %s, export %s: should not have offset (on invisible pin)"),
+						describenodeproto(np), pp->protoname);
+					(*warnings)++;
+				}
+			}
+
 			/* make sure subnodeinst of this port exists in facet */
 			for(ni = np->firstnodeinst; ni != NONODEINST; ni = ni->nextnodeinst)
 				if (ni == pp->subnodeinst) break;
@@ -783,7 +898,7 @@ void us_checklibrary(LIBRARY *curlib, INTSML verbose, INTSML *warnings, INTSML *
 						ttyputmsg(_("Facet %s, node %s: cannot hold outline information"),
 							describenodeproto(np), describenodeinst(ni));
 						(*warnings)++;
-						nextvarchangequiet();
+						nextchangequiet();
 						(void)delvalkey((INTBIG)ni, VNODEINST, el_trace);
 					} else
 					{
@@ -797,11 +912,21 @@ void us_checklibrary(LIBRARY *curlib, INTSML verbose, INTSML *warnings, INTSML *
 							ttyputmsg(_("Facet %s, node %s: zero-size outline (has %ld points)"),
 								describenodeproto(np), describenodeinst(ni), len);
 							(*warnings)++;
-							nextvarchangequiet();
+							nextchangequiet();
 							(void)delvalkey((INTBIG)ni, VNODEINST, el_trace);
 						}
 					}
 				}
+
+				/* check "WIPED" bit in userbits */
+				wiped = us_computewipestate(ni);
+				if (wiped != (ni->userbits&WIPED))
+				{
+					ttyputmsg(_("Facet %s, node %s: incorrect visibility"),
+						describenodeproto(np), describenodeinst(ni));
+					(*warnings)++;
+					ni->userbits = (ni->userbits & ~WIPED) | wiped;
+				}
 			}
 
 			/* check orientation of this nodeinst */
@@ -1561,8 +1686,7 @@ void us_checklibrary(LIBRARY *curlib, INTSML verbose, INTSML *warnings, INTSML *
 
 INTSML us_checkvariables(INTSML *numvar, VARIABLE *firstvar, LIBRARY *olib)
 {
-	REGISTER INTSML i, j, errors;
-	REGISTER INTBIG ty;
+	REGISTER INTBIG i, j, errors, ty;
 	REGISTER LIBRARY *lib;
 	REGISTER NODEPROTO *np, *vnp;
 	REGISTER TECHNOLOGY *tech;
@@ -1646,7 +1770,7 @@ INTSML us_checkvariables(INTSML *numvar, VARIABLE *firstvar, LIBRARY *olib)
  * is "facet"
  */
 INTBIG us_checkrtree(RTNODE *expectedparent, RTNODE *rtn, NODEPROTO *facet,
-	INTSML *warnings, INTSML *errors)
+	INTBIG *warnings, INTBIG *errors)
 {
 	REGISTER INTSML j;
 	INTBIG lx, hx, ly, hy, lowestxv, highestxv, lowestyv, highestyv, problems;
