@@ -47,7 +47,7 @@ void us_arc(INTSML count, char *par[])
 
 	if (count == 0)
 	{
-		count = ttygetparam(_("ARC command: "), &us_arcp, MAXPARS, par);
+		count = ttygetparam(M_("ARC command: "), &us_arcp, MAXPARS, par);
 		if (count == 0)
 		{
 			us_abortedmsg();
@@ -61,7 +61,7 @@ void us_arc(INTSML count, char *par[])
 	{
 		if (count == 1)
 		{
-			count = ttygetparam(_("Negation option: "), &us_arcnotp, MAXPARS-1, &par[1]) + 1;
+			count = ttygetparam(M_("Negation option: "), &us_arcnotp, MAXPARS-1, &par[1]) + 1;
 			if (count == 1)
 			{
 				us_abortedmsg();
@@ -76,7 +76,7 @@ void us_arc(INTSML count, char *par[])
 	{
 		if (count == 1)
 		{
-			count = ttygetparam(_("Toggle option: "), &us_arctogp, MAXPARS-1, &par[1]) + 1;
+			count = ttygetparam(M_("Toggle option: "), &us_arctogp, MAXPARS-1, &par[1]) + 1;
 			if (count == 1)
 			{
 				us_abortedmsg();
@@ -90,7 +90,7 @@ void us_arc(INTSML count, char *par[])
 	}
 
 	/* get the list of all highlighted arcs */
-	list = us_gethighlighted(OBJARCINST);
+	list = us_gethighlighted(OBJARCINST, 0, 0);
 	if (list[0] == NOGEOM)
 	{
 		us_abortcommand(_("No arcs are selected"));
@@ -152,7 +152,7 @@ void us_arc(INTSML count, char *par[])
 			return;
 		}
 		i = us_modarcbits((INTSML)((negate != 0 ? 7 : 6)), 0, "", list);
-		if (i >= 0) ttyputverbose(_("%d %s made temporarily %srigid"), i, makeplural(_("arc"), i),
+		if (i >= 0) ttyputverbose(M_("%d %s made temporarily %srigid"), i, makeplural(_("arc"), i),
 			(negate!=0 ? _("un-") : ""));
 	} else if (namesamen(pp, "ends-extend", l) == 0 && l >= 1)
 	{
@@ -185,7 +185,7 @@ void us_arc(INTSML count, char *par[])
 	{
 		if (negate != 0) ttyputerr(_("'arc reverse' is a relative notion, 'not' ignored"));
 		i = us_modarcbits(18, 1, "", list);
-		if (i >= 0) ttyputverbose(_("%d %s direction reversed"), i, makeplural(_("arc"), i));
+		if (i >= 0) ttyputverbose(M_("%d %s direction reversed"), i, makeplural(_("arc"), i));
 	} else if (namesamen(pp, "constraint", l) == 0 && l >= 2)
 	{
 		if (namesame(el_curconstraint->conname, "linear") != 0)
@@ -218,7 +218,7 @@ void us_arc(INTSML count, char *par[])
 		{
 			if (count < 2)
 			{
-				count = ttygetparam(_("ARC name: "), &us_arcnamep, MAXPARS-1, &par[1]) + 1;
+				count = ttygetparam(M_("ARC name: "), &us_arcnamep, MAXPARS-1, &par[1]) + 1;
 				if (count == 1)
 				{
 					us_abortedmsg();
@@ -234,7 +234,7 @@ void us_arc(INTSML count, char *par[])
 			{
 				if (ai->network == net)
 					ttyputmsg(_("Network %s is already on this arc"), describenetwork(net));
-			} else ttyputverbose(_("Network %s defined"), pp);
+			} else ttyputverbose(M_("Network %s defined"), pp);
 			us_setarcname(ai, pp);
 		} else
 		{
@@ -246,7 +246,7 @@ void us_arc(INTSML count, char *par[])
 				return;
 			}
 			if (var != NOVARIABLE) us_setarcname(ai, (char *)0);
-			ttyputverbose(_("Arc name removed"));
+			ttyputverbose(M_("Arc name removed"));
 		}
 	} else if (namesamen(pp, "center", l) == 0 && l >= 2)
 	{
@@ -388,17 +388,17 @@ void us_arc(INTSML count, char *par[])
 
 void us_array(INTSML count, char *par[])
 {
-	REGISTER INTSML xsize, ysize, flipx, flipy, staggerx, staggery, ro, tr, noname, l,
-		diagonal, first;
+	REGISTER INTSML xsize, ysize, flipx, flipy, staggerx, staggery, centerx, centery,
+		ro, tr, noname, l, diagonal, first;
 	REGISTER INTBIG x, y, xpos, ypos, nodecount, arccount, i, lx, hx, ly, hy, cx, cy,
-		cx0, cy0, cx1, cy1, xoff0, xoff1, yoff0, yoff1, xoff, yoff, keepbits;
+		cx0, cy0, cx1, cy1, xoff0, xoff1, yoff0, yoff1, xoff, yoff, keepbits, xindex,
+		yindex, originalx, originaly;
 	REGISTER GEOM **list, *geom;
 	INTBIG xoverlap, yoverlap, plx, ply, phx, phy;
-	REGISTER char *pt1, *pt2, *portname;
+	REGISTER char *pt1, *pt2;
 	char totalname[200], *objname;
 	REGISTER NODEINST *ni, *newno, **nodelist, *ni0, *ni1;
 	REGISTER ARCINST *ai, **arclist, *newai;
-	REGISTER PORTPROTO *pp;
 	REGISTER NODEPROTO *np;
 	REGISTER PORTEXPINST *pe;
 	REGISTER VARIABLE *var;
@@ -411,7 +411,7 @@ void us_array(INTSML count, char *par[])
 	}
 
 	/* get the objects to be arrayed */
-	list = us_gethighlighted(OBJNODEINST|OBJARCINST);
+	list = us_gethighlighted(OBJNODEINST|OBJARCINST, 0, 0);
 	if (list[0] == NOGEOM)
 	{
 		us_abortcommand(_("Must select circuitry before arraying it"));
@@ -450,28 +450,30 @@ void us_array(INTSML count, char *par[])
 	/* get the X extent of the array */
 	if (count == 0)
 	{
-		ttyputusage("array XREP[f][s] [YREP[f][s] [XOVRLP [YOVRLP [no-names][diagonal]]]]");
+		ttyputusage("array XREP[f][s][c] [YREP[f][s][c] [XOVRLP [YOVRLP [no-names][diagonal]]]]");
 		return;
 	}
 	pt1 = par[0];
 	xsize = (INTSML)myatoi(pt1);
-	flipx = staggerx = 0;
+	flipx = staggerx = centerx = 0;
 	for(i = strlen(pt1)-1; i > 0; i--)
 	{
 		if (pt1[i] == 'f') flipx = 1; else
 			if (pt1[i] == 's') staggerx = 1; else
-				break;
+				if (pt1[i] == 'c') centerx = 1; else
+					break;
 	}
 
 	/* get the Y extent of the array */
 	if (count == 1) pt2 = "1"; else pt2 = par[1];
 	ysize = atoi(pt2);
-	flipy = staggery = 0;
+	flipy = staggery = centery = 0;
 	for(i = strlen(pt2)-1; i > 0; i--)
 	{
 		if (pt2[i] == 'f') flipy = 1; else
 			if (pt2[i] == 's') staggery = 1; else
-				break;
+				if (pt2[i] == 'c') centery = 1; else
+					break;
 	}
 
 	/* check for nonsense */
@@ -485,6 +487,13 @@ void us_array(INTSML count, char *par[])
 		us_abortcommand(_("Diagonal arrays need one dimension to be 1"));
 		return;
 	}
+#if 0
+	if ((centerx != 0 && (xsize&1) == 0) || (centery != 0 && (ysize&1) == 0))
+	{
+		us_abortcommand(_("When centering the array, the size must be odd"));
+		return;
+	}
+#endif
 
 	/* mark the list of nodes and arcs in the facet that will be arrayed */
 	for(ni = np->firstnodeinst; ni != NONODEINST; ni = ni->nextnodeinst)
@@ -578,45 +587,51 @@ void us_array(INTSML count, char *par[])
 	}
 
 	/* check to ensure the array doesn't overflow the address space */
-	for(x=0; x<xsize; x++)
+	if (centerx == 0)
 	{
-		plx = lx + xoverlap * x;
-		phx = hx + xoverlap * x;
-		if (staggery != 0) { plx += xoverlap/2;   phx += xoverlap/2; }
-		if (xoverlap > 0)
+		for(x=0; x<xsize; x++)
 		{
-			if (phx < lx) break;
-		} else
+			plx = lx + xoverlap * x;
+			phx = hx + xoverlap * x;
+			if (staggery != 0) { plx += xoverlap/2;   phx += xoverlap/2; }
+			if (xoverlap > 0)
+			{
+				if (phx < lx) break;
+			} else
+			{
+				if (plx > hx) break;
+			}
+		}
+		if (x < xsize)
 		{
-			if (plx > hx) break;
+			us_abortcommand(_("Array is too wide...place elements closer or use fewer"));
+			if (nodecount > 0) efree((char *)nodelist);
+			if (arccount > 0) efree((char *)arclist);
+			return;
 		}
 	}
-	if (x < xsize)
+	if (centery == 0)
 	{
-		us_abortcommand(_("Array is too wide...place elements closer or use fewer"));
-		if (nodecount > 0) efree((char *)nodelist);
-		if (arccount > 0) efree((char *)arclist);
-		return;
-	}
-	for(y=0; y<ysize; y++)
-	{
-		ply = ly + yoverlap * y;
-		phy = hy + yoverlap * y;
-		if (staggerx != 0) { ply += yoverlap/2;   phy += yoverlap/2; }
-		if (yoverlap > 0)
+		for(y=0; y<ysize; y++)
 		{
-			if (phy < ly) break;
-		} else
+			ply = ly + yoverlap * y;
+			phy = hy + yoverlap * y;
+			if (staggerx != 0) { ply += yoverlap/2;   phy += yoverlap/2; }
+			if (yoverlap > 0)
+			{
+				if (phy < ly) break;
+			} else
+			{
+				if (ply > hy) break;
+			}
+		}
+		if (y < ysize)
 		{
-			if (ply > hy) break;
+			us_abortcommand(_("Array is too tall...place elements closer or use fewer"));
+			if (nodecount > 0) efree((char *)nodelist);
+			if (arccount > 0) efree((char *)arclist);
+			return;
 		}
-	}
-	if (y < ysize)
-	{
-		us_abortcommand(_("Array is too tall...place elements closer or use fewer"));
-		if (nodecount > 0) efree((char *)nodelist);
-		if (arccount > 0) efree((char *)arclist);
-		return;
 	}
 
 	/* save highlighting */
@@ -627,29 +642,41 @@ void us_array(INTSML count, char *par[])
 	for(x=0; x<xsize; x++) for(y=0; y<ysize; y++)
 	{
 		if (stopping(STOPREASONARRAY)) break;
-		if (x == 0 && y == 0) continue;
+		if (centerx == 0) xindex = x; else
+			xindex = x - (xsize-1)/2;
+		if (centery == 0) yindex = y; else
+			yindex = y - (ysize-1)/2;
+		if (xindex == 0 && yindex == 0)
+		{
+			originalx = x;
+			originaly = y;
+			continue;
+		}
+
+		/* initialize for queueing creation of new exports */
+		us_initqueuedexports();
 
 		/* first replicate the nodes */
 		for(i=0; i<nodecount; i++)
 		{
 			ni = nodelist[i];
-			if (diagonal != 0 && xsize == 1) xpos = cx + xoverlap * y; else
-				xpos = cx + xoverlap * x;
-			if (diagonal != 0 && ysize == 1) ypos = cy + yoverlap * x; else
-				ypos = cy + yoverlap * y;
+			if (diagonal != 0 && xsize == 1) xpos = cx + xoverlap * yindex; else
+				xpos = cx + xoverlap * xindex;
+			if (diagonal != 0 && ysize == 1) ypos = cy + yoverlap * xindex; else
+				ypos = cy + yoverlap * yindex;
 			xoff = (ni->lowx + ni->highx) / 2 - cx;
 			yoff = (ni->lowy + ni->highy) / 2 - cy;
-			if ((x&1) != 0 && staggerx != 0) ypos += yoverlap/2;
-			if ((y&1) != 0 && staggery != 0) xpos += xoverlap/2;
+			if ((xindex&1) != 0 && staggerx != 0) ypos += yoverlap/2;
+			if ((yindex&1) != 0 && staggery != 0) xpos += xoverlap/2;
 			ro = ni->rotation;  tr = ni->transpose;
-			if ((x&1) != 0 && flipx)
+			if ((xindex&1) != 0 && flipx)
 			{
 				if (tr) ro += 2700; else ro += 900;
 				ro %= 3600;
 				tr = 1 - tr;
 				xoff = -xoff;
 			}
-			if ((y&1) != 0 && flipy)
+			if ((yindex&1) != 0 && flipy)
 			{
 				if (tr) ro += 900; else ro += 2700;
 				ro %= 3600;
@@ -685,12 +712,17 @@ void us_array(INTSML count, char *par[])
 						(void)sprintf(totalname, "%s%ld-%ld", objname, x, y);
 				var = setvalkey((INTBIG)newno, VNODEINST, el_node_name,
 					(INTBIG)totalname, VSTRING|VDISPLAY);
-
-				/* shift text down if on a facet instance */
-				if (var != NOVARIABLE && ni->proto->primindex == 0)
+				if (var != NOVARIABLE)
 				{
-					var->textdescript = us_setdescriptoffset(var->textdescript,
-						0, (ni->highy-ni->lowy) / el_curlib->lambda[el_curtech->techindex]);
+					var->textdescript = (var->textdescript & ~VTSIZE) |
+						(defaulttextsize(3) << VTSIZESH);
+
+					/* shift text down if on a facet instance */
+					if (ni->proto->primindex == 0)
+					{
+						var->textdescript = us_setdescriptoffset(var->textdescript,
+							0, (ni->highy-ni->lowy) / el_curlib->lambda[el_curtech->techindex]);
+					}
 				}
 			}
 
@@ -699,17 +731,7 @@ void us_array(INTSML count, char *par[])
 			{
 				for(pe = ni->firstportexpinst; pe != NOPORTEXPINST; pe = pe->nextportexpinst)
 				{
-					portname = us_uniqueportname(pe->exportproto->protoname, np);
-					pp = newportproto(np, newno, pe->proto, portname);
-					if (pp == NOPORTPROTO)
-					{
-						us_abortcommand(_("Cannot create port %s"), portname);
-						return;
-					}
-					pp->userbits = pe->exportproto->userbits;
-					pp->textdescript = pe->exportproto->textdescript;
-					if (copyvars((INTBIG)pe->exportproto, VPORTPROTO, (INTBIG)pp, VPORTPROTO) != 0)
-						return;
+					if (us_queuenewexport(newno, pe->proto, pe->exportproto) != 0) return;
 				}
 			}
 
@@ -717,6 +739,9 @@ void us_array(INTSML count, char *par[])
 			ni->temp1 = (UINTBIG)newno;
 		}
 
+		/* create any queued exports */
+		us_createqueuedexports();
+
 		/* next replicate the arcs */
 		for(i=0; i<arccount; i++)
 		{
@@ -730,12 +755,12 @@ void us_array(INTSML count, char *par[])
 			cy1 = (ai->end[1].nodeinst->lowy + ai->end[1].nodeinst->highy) / 2;
 			xoff1 = ai->end[1].xpos - cx1;
 			yoff1 = ai->end[1].ypos - cy1;
-			if ((x&1) != 0 && flipx)
+			if ((xindex&1) != 0 && flipx)
 			{
 				xoff0 = -xoff0;
 				xoff1 = -xoff1;
 			}
-			if ((y&1) != 0 && flipy)
+			if ((yindex&1) != 0 && flipy)
 			{
 				yoff0 = -yoff0;
 				yoff1 = -yoff1;
@@ -772,6 +797,9 @@ void us_array(INTSML count, char *par[])
 						(void)sprintf(totalname, "%s%ld-%ld", objname, x, y);
 				var = setvalkey((INTBIG)newai, VARCINST, el_arc_name,
 					(INTBIG)totalname, VSTRING|VDISPLAY);
+				if (var != NOVARIABLE)
+					var->textdescript = (var->textdescript & ~VTSIZE) |
+						(defaulttextsize(4) << VTSIZESH);
 			}
 			endobjectchange((INTBIG)newai, VARCINST);
 		}
@@ -793,15 +821,20 @@ void us_array(INTSML count, char *par[])
 				if (strcmp((char *)var->addr, "0") != 0 && strcmp((char *)var->addr, "0-0") != 0)
 					objname = (char *)var->addr;
 			}
-			if (xsize <= 1 || ysize <= 1) (void)sprintf(totalname, "%s0", objname); else
-				(void)sprintf(totalname, "%s0-0", objname);
+			if (xsize <= 1 || ysize <= 1) (void)sprintf(totalname, "%s%ld", objname, originalx+originaly); else
+				(void)sprintf(totalname, "%s%ld-%ld", objname, originalx, originaly);
 			var = setvalkey((INTBIG)ni, VNODEINST, el_node_name, (INTBIG)totalname, VSTRING|VDISPLAY);
-
-			/* shift text down if on a facet instance */
-			if (var != NOVARIABLE && ni->proto->primindex == 0)
+			if (var != NOVARIABLE)
 			{
-				var->textdescript = us_setdescriptoffset(var->textdescript,
-					0, (ni->highy-ni->lowy) / el_curlib->lambda[el_curtech->techindex]);
+				var->textdescript = (var->textdescript & ~VTSIZE) |
+					(defaulttextsize(3) << VTSIZESH);
+
+				/* shift text down if on a facet instance */
+				if (ni->proto->primindex == 0)
+				{
+					var->textdescript = us_setdescriptoffset(var->textdescript,
+						0, (ni->highy-ni->lowy) / el_curlib->lambda[el_curtech->techindex]);
+				}
 			}
 			endobjectchange((INTBIG)ni, VNODEINST);
 		}
@@ -818,9 +851,12 @@ void us_array(INTSML count, char *par[])
 				if (strcmp((char *)var->addr, "0") != 0 && strcmp((char *)var->addr, "0-0") != 0)
 					objname = (char *)var->addr;
 			}
-			if (xsize <= 1 || ysize <= 1) (void)sprintf(totalname, "%s0", objname); else
-				(void)sprintf(totalname, "%s0-0", objname);
+			if (xsize <= 1 || ysize <= 1) (void)sprintf(totalname, "%s%ld", objname, originalx+originaly); else
+				(void)sprintf(totalname, "%s%ld-%ld", objname, originalx, originaly);
 			var = setvalkey((INTBIG)ai, VARCINST, el_arc_name, (INTBIG)totalname, VSTRING|VDISPLAY);
+			if (var != NOVARIABLE)
+				var->textdescript = (var->textdescript & ~VTSIZE) |
+					(defaulttextsize(4) << VTSIZESH);
 			endobjectchange((INTBIG)ai, VARCINST);
 		}
 	}
@@ -850,7 +886,7 @@ void us_bind(INTSML count, char *par[])
 
 	if (count == 0)
 	{
-		count = ttygetparam(_("BIND option: "), &us_bindp, MAXPARS, par);
+		count = ttygetparam(M_("BIND option: "), &us_bindp, MAXPARS, par);
 		if (count == 0)
 		{
 			us_abortedmsg();
@@ -876,11 +912,11 @@ void us_bind(INTSML count, char *par[])
 		if (negated == 0)
 		{
 			us_aid->aidstate |= ECHOBIND;
-			ttyputverbose(_("Key/menu/button commands echoed"));
+			ttyputverbose(M_("Key/menu/button commands echoed"));
 		} else
 		{
 			us_aid->aidstate &= ~ECHOBIND;
-			ttyputverbose(_("Key/menu/button commands not echoed"));
+			ttyputverbose(M_("Key/menu/button commands not echoed"));
 		}
 		return;
 	}
@@ -1018,7 +1054,7 @@ void us_bind(INTSML count, char *par[])
 	{
 		if (count < 2)
 		{
-			count = ttygetparam(_("Key, Menu, Button, or Popup: "), &us_bindsetp, MAXPARS-1, &par[1])+1;
+			count = ttygetparam(M_("Key, Menu, Button, or Popup: "), &us_bindsetp, MAXPARS-1, &par[1])+1;
 			if (count == 1)
 			{
 				us_abortedmsg();
@@ -1032,13 +1068,13 @@ void us_bind(INTSML count, char *par[])
 		arcglyph = NOARCPROTO;
 		backgroundcolor = 0;
 		menumessage = 0;
-		menumessagesize = TXTLARGE;
+		menumessagesize = TXT20P;
 
 		if (namesamen(pp, "button", l) == 0 && l >= 1)
 		{
 			if (count < 3)
 			{
-				count = ttygetparam(_("Button name: "), &us_bindbuttonp, MAXPARS-2, &par[2])+2;
+				count = ttygetparam(M_("Button name: "), &us_bindbuttonp, MAXPARS-2, &par[2])+2;
 				if (count == 2)
 				{
 					us_abortedmsg();
@@ -1069,7 +1105,7 @@ void us_bind(INTSML count, char *par[])
 		{
 			if (count < 3)
 			{
-				count = ttygetparam(_("Key name: "), &us_bindkeyp, MAXPARS-2, &par[2])+2;
+				count = ttygetparam(M_("Key name: "), &us_bindkeyp, MAXPARS-2, &par[2])+2;
 				if (count == 2)
 				{
 					us_abortedmsg();
@@ -1090,7 +1126,7 @@ void us_bind(INTSML count, char *par[])
 		{
 			if (count < 3)
 			{
-				count = ttygetparam(_("Popup menu name: "), &us_bindpopnp, MAXPARS-2, &par[2])+2;
+				count = ttygetparam(M_("Popup menu name: "), &us_bindpopnp, MAXPARS-2, &par[2])+2;
 				if (count == 2)
 				{
 					us_abortedmsg();
@@ -1119,7 +1155,7 @@ void us_bind(INTSML count, char *par[])
 
 			if (count < 4)
 			{
-				count = ttygetparam(_("Popup menu entry: "), &us_bindpoprep, MAXPARS-3, &par[3])+3;
+				count = ttygetparam(M_("Popup menu entry: "), &us_bindpoprep, MAXPARS-3, &par[3])+3;
 				if (count == 3)
 				{
 					us_abortedmsg();
@@ -1162,7 +1198,7 @@ void us_bind(INTSML count, char *par[])
 				}
 				if (namesamen(pp, "textsize", l) == 0)
 				{
-					menumessagesize = (INTSML)us_gettextsize(par[3], TXTLARGE);
+					menumessagesize = (INTSML)us_gettextsize(par[3], TXT4L);
 					count -= 2;
 					par += 2;
 					continue;
@@ -1180,7 +1216,7 @@ void us_bind(INTSML count, char *par[])
 			/* get the menu row number */
 			if (count < 3)
 			{
-				count = ttygetparam(_("Menu row: "), &us_bindmenuryp, MAXPARS-2, &par[2]) + 2;
+				count = ttygetparam(M_("Menu row: "), &us_bindmenuryp, MAXPARS-2, &par[2]) + 2;
 				if (count == 2)
 				{
 					us_abortedmsg();
@@ -1190,7 +1226,7 @@ void us_bind(INTSML count, char *par[])
 			y = (INTSML)myatoi(pp = par[2]);
 			if (count < 4)
 			{
-				count = ttygetparam(_("Menu column: "), &us_bindmenuxp, MAXPARS+3, &par[3])+3;
+				count = ttygetparam(M_("Menu column: "), &us_bindmenuxp, MAXPARS+3, &par[3])+3;
 				if (count == 3)
 				{
 					us_abortedmsg();
@@ -1233,7 +1269,7 @@ void us_bind(INTSML count, char *par[])
 		}
 
 		if (count <= important)
-			count = ttygetparam(_("Command: "), &us_userp, (INTSML)(MAXPARS-important),
+			count = ttygetparam(M_("Command: "), &us_userp, (INTSML)(MAXPARS-important),
 				&par[important]) + important;
 		(void)initinfstr();
 		for(i=important; i<count; i++)
