@@ -40,7 +40,7 @@
 #include "usrdiacom.h"
 #include <math.h>
 
-#define	MAXLINE	   200		/* max characters on color map input line */
+#define MAXLINE	   200		/* max characters on color map input line */
 struct {
 	INTSML bits;
 	INTSML set;
@@ -69,7 +69,7 @@ void us_color(INTSML count, char *par[])
 
 	if (count == 0)
 	{
-		count = ttygetparam(_("COLOR option: "), &us_colorp, MAXPARS, par);
+		count = ttygetparam(M_("COLOR option: "), &us_colorp, MAXPARS, par);
 		if (count == 0)
 		{
 			us_abortedmsg();
@@ -81,21 +81,21 @@ void us_color(INTSML count, char *par[])
 	if (namesamen(pp, "black-background", l) == 0 && l >= 1)
 	{
 		us_getcolormap(el_curtech, COLORSBLACK, 1);
-		ttyputverbose(_("Black-background colormap loaded"));
+		ttyputverbose(M_("Black-background colormap loaded"));
 		return;
 	}
 
 	if (namesamen(pp, "white-background", l) == 0 && l >= 1)
 	{
 		us_getcolormap(el_curtech, COLORSWHITE, 1);
-		ttyputverbose(_("White-background colormap loaded"));
+		ttyputverbose(M_("White-background colormap loaded"));
 		return;
 	}
 
 	if (namesamen(pp, "default", l) == 0 && l >= 1)
 	{
 		us_getcolormap(el_curtech, COLORSDEFAULT, 1);
-		ttyputverbose(_("Default colormap loaded"));
+		ttyputverbose(M_("Default colormap loaded"));
 		return;
 	}
 
@@ -203,7 +203,7 @@ void us_color(INTSML count, char *par[])
 		/* get the entry number */
 		if (count < 2)
 		{
-			count = ttygetparam(_("Entry: "), &us_colorentryp, MAXPARS-1, &par[1]) + 1;
+			count = ttygetparam(M_("Entry: "), &us_colorentryp, MAXPARS-1, &par[1]) + 1;
 			if (count == 1)
 			{
 				us_abortedmsg();
@@ -362,7 +362,7 @@ void us_color(INTSML count, char *par[])
 		}
 		pp = &line[11];
 		if (strcmp(pp, el_curtech->techname) != 0)
-			ttyputverbose(_("Warning: color map is for %s technology"), pp);
+			ttyputverbose(M_("Warning: color map is for %s technology"), pp);
 
 		max = 0;
 		for(;;)
@@ -498,7 +498,7 @@ void us_color(INTSML count, char *par[])
 			xprintf(f, "\n");
 		}
 		xclose(f);
-		ttyputverbose(_("%s written"), truepath(orig));
+		ttyputverbose(M_("%s written"), truepath(orig));
 		return;
 	}
 
@@ -587,7 +587,7 @@ void us_constraint(INTSML count, char *par[])
 		us_clearhighlightcount();
 		(void)setvalkey((INTBIG)us_aid, VAID, us_current_constraint,
 			(INTBIG)&el_constraints[c], VCONSTRAINT|VDONTSAVE);
-		ttyputmsg(_("Switching to %s"), _(el_constraints[c].condesc));
+		ttyputmsg(M_("Switching to %s"), _(el_constraints[c].condesc));
 		return;
 	}
 
@@ -600,18 +600,15 @@ void us_copyfacet(INTSML count, char *par[])
 	REGISTER LIBRARY *fromlib, *tolib;
 	REGISTER char *pt, *ppt, *lastch;
 	extern COMCOMP us_copyfacetp;
-	REGISTER PORTPROTO *pp, *rpp, *npp;
-	REGISTER NODEINST *ni, *newni;
+	REGISTER NODEINST *ni;
 	REGISTER VIEW *nview;
-	REGISTER INTBIG xc, yc, lowx, highx, lowy, highy, i;
-	REGISTER INTSML newang, newtran, save, quiet, move;
-	XARRAY localtrans, ntrans, trans;
-	INTBIG newx, newy;
+	REGISTER INTBIG i;
+	REGISTER INTSML save, quiet, move, norelatedviews;
 
 	/* get name of old facet */
 	if (count == 0)
 	{
-		count = ttygetparam(_("Old facet name: "), &us_copyfacetp, MAXPARS, par);
+		count = ttygetparam(M_("Old facet name: "), &us_copyfacetp, MAXPARS, par);
 		if (count == 0)
 		{
 			us_abortedmsg();
@@ -633,7 +630,7 @@ void us_copyfacet(INTSML count, char *par[])
 	tolib = fromlib;
 
 	/* get name of new facet */
-	quiet = move = 0;
+	quiet = move = norelatedviews = 0;
 	if (count <= 1) pt = np->cell->cellname; else
 	{
 		pt = par[1];
@@ -661,6 +658,7 @@ void us_copyfacet(INTSML count, char *par[])
 			i = strlen(ppt = par[2]);
 			if (namesamen(ppt, "quiet", i) == 0) quiet = 1; else
 			if (namesamen(ppt, "move", i) == 0) move = 1; else
+			if (namesamen(ppt, "no-related-views", i) == 0) norelatedviews = 1; else
 			{
 				ttyputbadusage("copyfacet");
 				return;
@@ -701,138 +699,7 @@ void us_copyfacet(INTSML count, char *par[])
 	i = strlen(pt);
 	if (count > 1 && i > 4 && namesame(&pt[i-4], "{sk}") == 0 && np->cellview != el_skeletonview)
 	{
-		/* cannot skeletonize text-only views */
-		if ((np->cellview->viewstate&TEXTVIEW) != 0)
-		{
-			us_abortcommand(_("Cannot skeletonize textual views: only layout"));
-			return;
-		}
-
-		/* warn if skeletonizing nonlayout views */
-		if (np->cellview != el_unknownview && np->cellview != el_layoutview)
-			ttyputmsg(_("Warning: skeletonization only makes sense for layout facets, not %s"),
-				np->cellview->viewname);
-
-		onp = np;
-		np = newnodeproto(pt, tolib);
-		if (np == NONODEPROTO)
-		{
-			us_abortcommand(_("Cannot create %s"), pt);
-			return;
-		}
-
-		/* place all exports in the new facet */
-		lowx = highx = (onp->lowx + onp->highx) / 2;
-		lowy = highy = (onp->lowy + onp->highy) / 2;
-		for(pp = onp->firstportproto; pp != NOPORTPROTO; pp = pp->nextportproto)
-		{
-			/* make a transformation matrix for the node that is an export */
-			ni = pp->subnodeinst;
-			rpp = pp->subportproto;
-			newang = ni->rotation;
-			newtran = ni->transpose;
-			makerot(ni, trans);
-			while (ni->proto->primindex == 0)
-			{
-				maketrans(ni, localtrans);
-				transmult(localtrans, trans, ntrans);
-				ni = rpp->subnodeinst;
-				rpp = rpp->subportproto;
-				if (ni->transpose == 0) newang = ni->rotation + newang; else
-					newang = ni->rotation + 3600 - newang;
-				newtran = (newtran + ni->transpose) & 1;
-				makerot(ni, localtrans);
-				transmult(localtrans, ntrans, trans);
-			}
-
-			/* create this node */
-			xc = (ni->lowx + ni->highx) / 2;   yc = (ni->lowy + ni->highy) / 2;
-			xform(xc, yc, &newx, &newy, trans);
-			newx -= (ni->highx - ni->lowx) / 2;
-			newy -= (ni->highy - ni->lowy) / 2;
-			newang = newang % 3600;   if (newang < 0) newang += 3600;
-			newni = newnodeinst(ni->proto, newx, newx+ni->highx-ni->lowx,
-				newy, newy+ni->highy-ni->lowy, newtran, newang, np);
-			if (newni == NONODEINST)
-			{
-				us_abortcommand(_("Cannot create node in this facet"));
-				return;
-			}
-			endobjectchange((INTBIG)newni, VNODEINST);
-			lowx = mini(lowx, newx);
-			highx = maxi(highx, newx+ni->highx-ni->lowx);
-			lowy = mini(lowy, newy);
-			highy = maxi(highy, newy+ni->highy-ni->lowy);
-
-			/* export the port from the node */
-			npp = newportproto(np, newni, rpp, pp->protoname);
-			if (npp == NOPORTPROTO)
-			{
-				us_abortcommand(_("Could not create port %s"), pp->protoname);
-				return;
-			}
-			npp->userbits = pp->userbits;
-			npp->textdescript = pp->textdescript;
-			if (copyvars((INTBIG)pp, VPORTPROTO, (INTBIG)npp, VPORTPROTO) != 0)
-				return;
-		}
-
-		/* copy the facet center node if it exists */
-		for(ni = onp->firstnodeinst; ni != NONODEINST; ni = ni->nextnodeinst)
-		{
-			if (ni->proto != gen_facetcenterprim) continue;
-			newni = newnodeinst(ni->proto, ni->lowx, ni->highx,
-				ni->lowy, ni->highy, ni->transpose, ni->rotation, np);
-			if (newni == NONODEINST)
-			{
-				us_abortcommand(_("Cannot create node in this facet"));
-				return;
-			}
-			endobjectchange((INTBIG)newni, VNODEINST);
-			lowx = mini(lowx, ni->lowx);
-			highx = maxi(highx, ni->highx);
-			lowy = mini(lowy, ni->lowy);
-			highy = maxi(highy, ni->highy);
-			break;
-		}
-
-		/* make sure facet is the same size */
-		if (lowx > onp->lowx)
-		{
-			i = (onp->highy+onp->lowy)/2 - (gen_invispinprim->highy-gen_invispinprim->lowy)/2;
-			(void)newnodeinst(gen_invispinprim, onp->lowx, onp->lowx+gen_invispinprim->highx-gen_invispinprim->lowx,
-				i, i+gen_invispinprim->highy-gen_invispinprim->lowy, 0, 0, np);
-		}
-		if (highx < onp->highx)
-		{
-			i = (onp->highy+onp->lowy)/2 - (gen_invispinprim->highy-gen_invispinprim->lowy)/2;
-			(void)newnodeinst(gen_invispinprim, onp->highx-(gen_invispinprim->highx-gen_invispinprim->lowx), onp->highx,
-				i, i+gen_invispinprim->highy-gen_invispinprim->lowy, 0, 0, np);
-		}
-		if (lowy > onp->lowy)
-		{
-			i = (onp->highx+onp->lowx)/2 - (gen_invispinprim->highx-gen_invispinprim->lowx)/2;
-			(void)newnodeinst(gen_invispinprim, i, i+gen_invispinprim->highx-gen_invispinprim->lowx,
-				onp->lowy, onp->lowy+gen_invispinprim->highy-gen_invispinprim->lowy, 0, 0, np);
-		}
-		if (highy < onp->highy)
-		{
-			i = (onp->highx+onp->lowx)/2 - (gen_invispinprim->highx-gen_invispinprim->lowx)/2;
-			(void)newnodeinst(gen_invispinprim, i, i+gen_invispinprim->highx-gen_invispinprim->lowx,
-				onp->highy-(gen_invispinprim->highy-gen_invispinprim->lowy), onp->highy, 0, 0,np);
-		}
-
-		/* place the actual origin of the facet inside */
-		(void)initinfstr();
-		(void)addstringtoinfstr(onp->cell->lib->libfile);
-		(void)addtoinfstr(':');
-		(void)addstringtoinfstr(nldescribenodeproto(onp));
-		(void)setval((INTBIG)np, VNODEPROTO, "FACET_original_facet",
-			(INTBIG)returninfstr(), VSTRING);
-
-		if (quiet == 0)
-			ttyputmsg(_("Facet %s created with a skeletal representation of %s"),
-				describenodeproto(np), describenodeproto(onp));
+		us_skeletonize(np, pt, tolib, quiet);
 		return;
 	}
 
@@ -894,7 +761,7 @@ void us_copyfacet(INTSML count, char *par[])
 
 	save = *ppt;
 	*ppt = 0;
-	onp = us_copyrecursively(np, pt, tolib, nview, 1, move, "");
+	onp = us_copyrecursively(np, pt, tolib, nview, 1, move, "", norelatedviews);
 	*ppt = (char)save;
 }
 
@@ -911,9 +778,10 @@ void us_create(INTSML count, char *par[])
 	REGISTER INTSML ang, l, remainhighlighted, ishor, isver, getcontents,
 		pangle, join, waitfordown, angledcreate;
 	REGISTER INTBIG bestdist, bestdist1, bestdist2, wid, truewid, i, j, k,
-		bits1, bits2, dist, lx, hx, ly, hy, overcount;
+		bits1, bits2, dist, overcount;
 	INTBIG px, py, nx, ny, cx, cy, x, y, xp[2], yp[2], xcur, ycur, otheralign, pxs, pys,
-		cornerx[4], cornery[4], overx[2], overy[2];
+		cornerx[4], cornery[4], overx[2], overy[2], lx, hx, ly, hy, flx, fhx, fly, fhy,
+		tlx, tly, thx, thy;
 	PORTPROTO *app[2];
 	REGISTER VARIABLE *var;
 	static POLYGON *poly = NOPOLYGON;
@@ -1103,7 +971,7 @@ void us_create(INTSML count, char *par[])
 		var = getvalkey((INTBIG)us_aid, VAID, VSTRING|VISARRAY, us_highlighted);
 		if (var != NOVARIABLE)
 			(void)us_makehighlight(((char **)var->addr)[0], &newhigh);
-		bestdist1 = bestdist2 = HUGEINT;
+		bestdist1 = bestdist2 = MAXINTBIG;
 		ppt = fpt = NOPORTPROTO;
 		for(pp = ni->proto->firstportproto; pp != NOPORTPROTO; pp = pp->nextportproto)
 		{
@@ -1306,6 +1174,43 @@ void us_create(INTSML count, char *par[])
 						}
 						pxs = cornerx[j]*2 - (lx+hx)/2;
 						pys = cornery[j]*2 - (ly+hy)/2;
+
+						/* set coordinate in the center if the ports overlap in either axis */
+						if (newhigh.fromport != NOPORTPROTO)
+						{
+							shapeportpoly(ni, newhigh.fromport, poly, 0);
+							getbbox(poly, &flx, &fhx, &fly, &fhy);
+							shapeportpoly(togeom->entryaddr.ni, toport, poly, 0);
+							getbbox(poly, &tlx, &thx, &tly, &thy);
+							lx = maxi(flx, tlx);   hx = mini(fhx, thx);
+							ly = maxi(fly, tly);   hy = mini(fhy, thy);
+							if (lx <= hx || ly <= hy)
+							{
+								pxs = (lx + hx) / 2;
+								pys = (ly + hy) / 2;
+							}
+						}
+					}
+
+					/* isolated ports simply use cursor location */
+					if ((newhigh.fromport != NOPORTPROTO &&
+						(newhigh.fromport->userbits&PORTISOLATED) != 0) ||
+							(toport->userbits&PORTISOLATED) != 0)
+					{
+						portposition(ni, newhigh.fromport, &lx, &ly);
+						portposition(togeom->entryaddr.ni, toport, &hx, &hy);
+						if (hx < lx)
+						{
+							x = lx;   lx = hx;   hx = x;
+						}
+						if (hy < ly)
+						{
+							y = ly;   ly = hy;   hy = y;
+						}
+
+						/* if cursor location is within range of ports, use it */
+						if (xcur >= lx && xcur <= hx) pxs = xcur;
+						if (ycur >= ly && ycur <= hy) pys = ycur;
 					}
 
 					ai = aconnect(fromgeom, ppt, togeom, toport, us_curarcproto,
@@ -1322,7 +1227,7 @@ void us_create(INTSML count, char *par[])
 					newhigh.status = HIGHFROM;
 					newhigh.facet = geomparent(togeom);
 					newhigh.fromgeom = togeom;
-					newhigh.fromport = NOPORTPROTO;
+					newhigh.fromport = toport;
 					(void)us_addhighlight(&newhigh);
 					return;
 				}
@@ -1338,12 +1243,12 @@ void us_create(INTSML count, char *par[])
 
 		/* find the object under the cursor */
 		if (join == 0) foundgeom = NOGEOM; else
-			foundgeom = us_getclosest(nx+px, ny+py, ni->parent);
+			foundgeom = us_getclosest(nx+px, ny+py, us_alignment/2, ni->parent);
 		if (foundgeom != NOGEOM)
 		{
 			/* found another object: connect to it */
 			fpt = ppt = NOPORTPROTO;
-			bestdist = bestdist2 = HUGEINT;
+			bestdist = bestdist2 = MAXINTBIG;
 			if (foundgeom->entrytype == OBJNODEINST)
 			{
 				/* find the closest port */
@@ -1705,7 +1610,10 @@ void us_create(INTSML count, char *par[])
 		if (oldarcname != 0 && newai != NOARCINST)
 		{
 			(void)askaid(net_aid, "setname", (INTBIG)newai, (INTBIG)oldarcname);
-			(void)setvalkey((INTBIG)newai, VARCINST, el_arc_name, (INTBIG)oldarcname, VSTRING|VDISPLAY);
+			var = setvalkey((INTBIG)newai, VARCINST, el_arc_name, (INTBIG)oldarcname, VSTRING|VDISPLAY);
+			if (var != NOVARIABLE)
+				var->textdescript = (var->textdescript & ~VTSIZE) |
+					(defaulttextsize(4) << VTSIZESH);
 			efree(oldarcname);
 		}
 		if (newai != NOARCINST) endobjectchange((INTBIG)newai, VARCINST);
@@ -1838,7 +1746,10 @@ void us_create(INTSML count, char *par[])
 		if (oldarcname != 0 && newai != NOARCINST)
 		{
 			(void)askaid(net_aid, "setname", (INTBIG)newai, (INTBIG)oldarcname);
-			(void)setvalkey((INTBIG)newai, VARCINST, el_arc_name, (INTBIG)oldarcname, VSTRING|VDISPLAY);
+			var = setvalkey((INTBIG)newai, VARCINST, el_arc_name, (INTBIG)oldarcname, VSTRING|VDISPLAY);
+			if (var != NOVARIABLE)
+				var->textdescript = (var->textdescript & ~VTSIZE) |
+					(defaulttextsize(4) << VTSIZESH);
 			efree(oldarcname);
 		}
 		if (newai != NOARCINST) endobjectchange((INTBIG)newai, VARCINST);
@@ -1915,7 +1826,6 @@ void us_debug(INTSML count, char *par[])
 	REGISTER NODEPROTO *np;
 	REGISTER PORTPROTO *pp;
 	REGISTER NODEINST *ni;
-	REGISTER PORTARCINST *pi;
 	REGISTER ARCINST *ai;
 	REGISTER LIBRARY *lib;
 	REGISTER VARIABLE *var;
@@ -1927,6 +1837,111 @@ void us_debug(INTSML count, char *par[])
 	if (count == 0) return;
 	l = strlen(par[0]);
 
+	if (namesamen(par[0], "mudd", l) == 0)
+	{
+#include "tecmocmos.h"
+		INTBIG lam;
+		void io_fixupnodeinst(NODEINST *ni, INTBIG dlx, INTBIG dly, INTBIG dhx, INTBIG dhy);
+
+		for(np = el_curlib->firstnodeproto; np != NONODEPROTO; np = np->nextnodeproto)
+		{
+			for(ni = np->firstnodeinst; ni != NONODEINST; ni = ni->nextnodeinst)
+			{
+				if (ni->proto->primindex == 0) continue;
+				if (ni->proto->tech == sch_tech)
+				{
+					lam = el_curlib->lambda[sch_tech->techindex];
+					if (ni->proto == sch_pwrprim)
+					{
+						io_fixupnodeinst(ni, lam/2, lam/2, -lam/2, -lam/2);
+					} else if (ni->proto == sch_gndprim)
+					{
+						io_fixupnodeinst(ni, lam/2, 0, -lam/2, 0);
+					} else if (ni->proto == sch_capacitorprim)
+					{
+						io_fixupnodeinst(ni, lam/2, 0, -lam/2, 0);
+					} else if (ni->proto == sch_resistorprim)
+					{
+						io_fixupnodeinst(ni, 0, lam/2, 0, -lam/2);
+					}
+				} else if (ni->proto->tech == mocmos_tech)
+				{
+					lam = el_curlib->lambda[mocmos_tech->techindex];
+					if (ni->proto == mocmos_metal1metal2prim)
+					{
+						io_fixupnodeinst(ni, -lam/2, -lam/2, lam/2, lam/2);
+					} else if (ni->proto == mocmos_metal4metal5prim)
+					{
+						io_fixupnodeinst(ni, -lam/2, -lam/2, lam/2, lam/2);
+					} else if (ni->proto == mocmos_metal5metal6prim)
+					{
+						io_fixupnodeinst(ni, -lam, -lam, lam, lam);
+					} else if (ni->proto == mocmos_ptransistorprim)
+					{
+						io_fixupnodeinst(ni, 0, -lam, 0, lam);
+					} else if (ni->proto == mocmos_ntransistorprim)
+					{
+						io_fixupnodeinst(ni, 0, -lam, 0, lam);
+					}
+				}
+			}
+		}
+		return;
+	}
+
+#if 0		/* "debug million" */
+	if (namesamen(par[0], "million", l) == 0)
+	{
+#include "edialogs.h"
+		NODEPROTO *item;
+		extern AIDENTRY *dr_aid;
+		REGISTER ARCPROTO *apwire;
+		REGISTER PORTPROTO *ppy, *ppa;
+		float duration;
+		NODEINST *nilast;
+		INTBIG lx, hx, ly, hy, wid, hei, maxy, maxx, nilastx, nilasty, nix, niy;
+		extern DIALOG us_progressdialog;
+
+		aidturnoff(dr_aid, 0);
+		aidturnoff(us_aid, 0);
+		apwire = getarcproto("schematic:wire");
+		item = getnodeproto("schematic:Buffer");
+		if (item == NONODEPROTO) return;
+		ppy = getportproto(item, "y");
+		ppa = getportproto(item, "a");
+		wid = item->highx - item->lowx;
+		hei = item->highy - item->lowy;
+		np = newnodeproto("million", el_curlib);
+		(void)DiaInitDialog(&us_progressdialog);
+		maxy = 1000;    maxx = 1000;
+		starttimer();
+		for(i=0; i<maxy; i++)
+		{
+			DiaPercent(1, i*100/maxy);
+			nilast = NONODEINST;
+			for(j=0; j<maxx; j++)
+			{
+				lx = wid*j;   hx = lx + wid;
+				ly = hei*i;   hy = ly + hei;
+				ni = newnodeinst(item, lx, hx, ly, hy, 0, 0, np);
+				if (nilast != NONODEINST)
+				{
+					portposition(nilast, ppy, &nilastx, &nilasty);
+					portposition(ni, ppa, &nix, &niy);
+					newarcinst(apwire, 0, 0, nilast, ppy, nilastx, nilasty,
+						ni, ppa, nix, niy, np);
+				}
+				nilast = ni;
+			}
+		}
+		duration = endtimer();
+		ttyputmsg("Took %s", explainduration(duration));
+		DiaDoneDialog();
+		aidturnon(us_aid, 0);
+		return;
+	}
+#endif
+
 	/********** THESE ARE SPECIAL CASES **********/
 	if (namesamen(par[0], "convert-dashes", l) == 0)
 	{
@@ -2002,7 +2017,7 @@ void us_debug(INTSML count, char *par[])
 		for(l=0; l<total; l++)
 		{
 			ni = nilist[l];
-			j = nodepolys(ni, 0);
+			j = nodepolys(ni, 0, NOWINDOWPART);
 			for(i=0; i<j; i++)
 			{
 				shapenodepoly(ni, i, poly);
@@ -2121,21 +2136,11 @@ void us_debug(INTSML count, char *par[])
 					ni->userbits &= NEXPAND;
 
 					/* compute WIPED bit on node instance */
-					if ((ni->proto->userbits&ARCSWIPE) != 0)
-					{
-						for(pi = ni->firstportarcinst; pi != NOPORTARCINST; pi = pi->nextportarcinst)
-						{
-							if ((pi->conarcinst->proto->userbits&CANWIPE) != 0)
-							{
-								ni->userbits |= WIPED;
-								break;
-							}
-						}
-					}
+					ni->userbits |= us_computewipestate(ni);
 				}
 
 				/* clear the arc instance bits */
-				for(ai=np->firstarcinst; ai!=NOARCINST; ai=ai->nextarcinst)
+				for(ai = np->firstarcinst; ai != NOARCINST; ai = ai->nextarcinst)
 				{
 					ai->temp1 = ai->temp2 = 0;
 					ai->userbits &= (FIXED|FIXANG|ISNEGATED|NOEXTEND|NOTEND0|NOTEND1|REVERSEEND|CANTSLIDE);
@@ -2158,7 +2163,10 @@ void us_debug(INTSML count, char *par[])
 		{
 			line[0] = (char)let1;   line[1] = (char)let2;
 			line[2] = (char)let3;   line[3] = 0;
-			(void)setvalkey((INTBIG)ni, VNODEINST, el_node_name, (INTBIG)line, VSTRING|VDISPLAY);
+			var = setvalkey((INTBIG)ni, VNODEINST, el_node_name, (INTBIG)line, VSTRING|VDISPLAY);
+			if (var != NOVARIABLE)
+				var->textdescript = (var->textdescript & ~VTSIZE) |
+					(defaulttextsize(3) << VTSIZESH);
 			let1++;
 			if (let1 > 'Z')
 			{
@@ -2176,8 +2184,11 @@ void us_debug(INTSML count, char *par[])
 			ai = ai->nextarcinst)
 		{
 			(void)sprintf(line, "%d", i++);
-			(void)setvalkey((INTBIG)ai, VARCINST, el_arc_name, (INTBIG)line,
+			var = setvalkey((INTBIG)ai, VARCINST, el_arc_name, (INTBIG)line,
 				VSTRING|VDISPLAY);
+			if (var != NOVARIABLE)
+				var->textdescript = (var->textdescript & ~VTSIZE) |
+					(defaulttextsize(4) << VTSIZESH);
 		}
 		us_redisplay(el_curwindowpart);
 		return;
@@ -2190,8 +2201,8 @@ void us_debug(INTSML count, char *par[])
 		i = 0;
 		for(ni = np->firstnodeinst; ni != NONODEINST; ni = ni->nextnodeinst)
 		{
-			if (ni->geom->lowx > -HUGEINT/2 && ni->geom->highx < HUGEINT/2 &&
-				ni->geom->lowy > -HUGEINT/2 && ni->geom->highy < HUGEINT/2) continue;
+			if (ni->geom->lowx > -MAXINTBIG/2 && ni->geom->highx < MAXINTBIG/2 &&
+				ni->geom->lowy > -MAXINTBIG/2 && ni->geom->highy < MAXINTBIG/2) continue;
 			if (i == 0) us_clearhighlightcount();
 			newhigh.status = HIGHFROM;
 			newhigh.facet = np;
@@ -2203,8 +2214,8 @@ void us_debug(INTSML count, char *par[])
 		}
 		for(ai = np->firstarcinst; ai != NOARCINST; ai = ai->nextarcinst)
 		{
-			if (ai->geom->lowx > -HUGEINT/2 && ai->geom->highx < HUGEINT/2 &&
-				ai->geom->lowy > -HUGEINT/2 && ai->geom->highy < HUGEINT/2) continue;
+			if (ai->geom->lowx > -MAXINTBIG/2 && ai->geom->highx < MAXINTBIG/2 &&
+				ai->geom->lowy > -MAXINTBIG/2 && ai->geom->highy < MAXINTBIG/2) continue;
 			newhigh.status = HIGHFROM;
 			newhigh.facet = np;
 			newhigh.fromgeom = ai->geom;
@@ -2419,11 +2430,11 @@ void us_defarc(INTSML count, char *par[])
 		if (ap != NOARCPROTO)
 		{
 			(void)delvalkey((INTBIG)ap, VARCPROTO, us_arcstyle);
-			ttyputverbose(_("%s arcs drawn in default style"), describearcproto(ap));
+			ttyputverbose(M_("%s arcs drawn in default style"), describearcproto(ap));
 			return;
 		}
 		(void)delvalkey((INTBIG)us_aid, VAID, us_arcstyle);
-		ttyputverbose(_("All arcs drawn in default style"));
+		ttyputverbose(M_("All arcs drawn in default style"));
 		return;
 	}
 
@@ -2444,8 +2455,8 @@ void us_defarc(INTSML count, char *par[])
 			if (negate != 0) style &= ~WANTFIXANG; else style |= WANTFIXANG;
 			(void)setvalkey((INTBIG)us_aid, VAID, us_arcstyle, style, VINTEGER|VDONTSAVE);
 		}
-		if (negate != 0) ttyputverbose(_("%s arcs drawn not fixed-angle"), aname); else
-			ttyputverbose(_("%s arcs drawn fixed-angle"), aname);
+		if (negate != 0) ttyputverbose(M_("%s arcs drawn not fixed-angle"), aname); else
+			ttyputverbose(M_("%s arcs drawn fixed-angle"), aname);
 		return;
 	}
 
@@ -2464,8 +2475,8 @@ void us_defarc(INTSML count, char *par[])
 			if (negate != 0) style &= ~WANTFIX; else style |= WANTFIX;
 			(void)setvalkey((INTBIG)us_aid, VAID, us_arcstyle, style, VINTEGER|VDONTSAVE);
 		}
-		if (negate != 0) ttyputverbose(_("%s arcs drawn un-rigid"), aname); else
-			ttyputverbose(_("%s arcs drawn rigid"), aname);
+		if (negate != 0) ttyputverbose(M_("%s arcs drawn un-rigid"), aname); else
+			ttyputverbose(M_("%s arcs drawn rigid"), aname);
 		return;
 	}
 
@@ -2486,8 +2497,8 @@ void us_defarc(INTSML count, char *par[])
 				style |= WANTDIRECTIONAL;
 			(void)setvalkey((INTBIG)us_aid, VAID, us_arcstyle, style, VINTEGER|VDONTSAVE);
 		}
-		if (negate != 0) ttyputverbose(_("%s arcs drawn nondirectional"), aname); else
-			ttyputverbose(_("%s arcs drawn directional"), aname);
+		if (negate != 0) ttyputverbose(M_("%s arcs drawn nondirectional"), aname); else
+			ttyputverbose(M_("%s arcs drawn directional"), aname);
 		return;
 	}
 
@@ -2508,8 +2519,8 @@ void us_defarc(INTSML count, char *par[])
 				style |= WANTCANTSLIDE;
 			(void)setvalkey((INTBIG)us_aid, VAID, us_arcstyle, style, VINTEGER|VDONTSAVE);
 		}
-		if (negate != 0) ttyputverbose(_("%s arcs can not slide in their ports"), aname); else
-			ttyputverbose(_("%s arcs can slide in their ports"), aname);
+		if (negate != 0) ttyputverbose(M_("%s arcs can not slide in their ports"), aname); else
+			ttyputverbose(M_("%s arcs can slide in their ports"), aname);
 		return;
 	}
 
@@ -2528,8 +2539,8 @@ void us_defarc(INTSML count, char *par[])
 			if (negate != 0) style &= ~WANTNEGATED; else style |= WANTNEGATED;
 			(void)setvalkey((INTBIG)us_aid, VAID, us_arcstyle, style, VINTEGER|VDONTSAVE);
 		}
-		if (negate != 0) ttyputverbose(_("%s arcs drawn un-negated"), aname); else
-			ttyputverbose(_("%s arcs drawn negated"), aname);
+		if (negate != 0) ttyputverbose(M_("%s arcs drawn un-negated"), aname); else
+			ttyputverbose(M_("%s arcs drawn negated"), aname);
 		return;
 	}
 
@@ -2548,8 +2559,8 @@ void us_defarc(INTSML count, char *par[])
 			if (negate != 0) style |= WANTNOEXTEND; else style &= ~WANTNOEXTEND;
 			(void)setvalkey((INTBIG)us_aid, VAID, us_arcstyle, style, VINTEGER|VDONTSAVE);
 		}
-		if (negate != 0) ttyputverbose(_("%s arcs drawn with ends not extended"), aname); else
-			ttyputverbose(_("%s arcs drawn with ends extended by half-width"), aname);
+		if (negate != 0) ttyputverbose(M_("%s arcs drawn with ends not extended"), aname); else
+			ttyputverbose(M_("%s arcs drawn with ends extended by half-width"), aname);
 		return;
 	}
 
@@ -2578,8 +2589,9 @@ void us_defarc(INTSML count, char *par[])
 			return;
 		}
 		wid = arcprotowidthoffset(ap) + i;
-		(void)setvalkey((INTBIG)ap, VARCPROTO, el_arc_default_width, wid, VINTEGER);
-		ttyputverbose(_("%s arcs default to %s wide"), aname, latoa(i));
+		wid = wid * WHOLE / el_curlib->lambda[ap->tech->techindex];
+		(void)setvalkey((INTBIG)ap, VARCPROTO, el_arc_width_default, wid, VINTEGER);
+		ttyputverbose(M_("%s arcs default to %s wide"), aname, latoa(i));
 		return;
 	}
 
@@ -2608,7 +2620,7 @@ void us_defarc(INTSML count, char *par[])
 		}
 		bits = (ap->userbits & ~AANGLEINC) | (i << AANGLEINCSH);
 		(void)setval((INTBIG)ap, VARCPROTO, "userbits", bits, VINTEGER);
-		ttyputverbose(_("%s arcs will run at %d degree angle increments"), aname, i);
+		ttyputverbose(M_("%s arcs will run at %d degree angle increments"), aname, i);
 		return;
 	}
 
@@ -2644,7 +2656,7 @@ void us_defarc(INTSML count, char *par[])
 			return;
 		}
 		(void)setval((INTBIG)ap, VARCPROTO, "ARC_Default_Pin", (INTBIG)np, VNODEPROTO|VDONTSAVE);
-		ttyputverbose(_("Default pins for arc %s will be %s"), aname, describenodeproto(np));
+		ttyputverbose(M_("Default pins for arc %s will be %s"), aname, describenodeproto(np));
 		return;
 	}
 
@@ -2757,11 +2769,11 @@ void us_defnode(INTSML count, char *par[])
 			{
 				if ((onp->userbits&INCELLLIBRARY) != 0)
 				{
-					ttyputverbose(_("Facet %s is already part of a cell library"),
+					ttyputverbose(M_("Facet %s is already part of a cell library"),
 						describenodeproto(onp));
 				} else
 				{
-					ttyputverbose(_("Facet %s is now part of a cell library"),
+					ttyputverbose(M_("Facet %s is now part of a cell library"),
 						describenodeproto(onp));
 				}
 				(void)setval((INTBIG)onp, VNODEPROTO, "userbits",
@@ -2770,11 +2782,11 @@ void us_defnode(INTSML count, char *par[])
 			{
 				if ((onp->userbits&INCELLLIBRARY) == 0)
 				{
-					ttyputverbose(_("Facet %s is already not part of a cell library"),
+					ttyputverbose(M_("Facet %s is already not part of a cell library"),
 						describenodeproto(onp));
 				} else
 				{
-					ttyputverbose(_("Facet %s is now not part of a cell library"),
+					ttyputverbose(M_("Facet %s is now not part of a cell library"),
 						describenodeproto(onp));
 				}
 				(void)setval((INTBIG)onp, VNODEPROTO, "userbits",
@@ -2794,15 +2806,15 @@ void us_defnode(INTSML count, char *par[])
 		if (negated == 0)
 		{
 			if ((us_useroptions&CHECKDATE) != 0)
-				ttyputverbose(_("Facet revision dates already being checked")); else
-					ttyputverbose(_("Facet revision date errors will be identified"));
+				ttyputverbose(M_("Facet revision dates already being checked")); else
+					ttyputverbose(M_("Facet revision date errors will be identified"));
 			(void)setvalkey((INTBIG)us_aid, VAID, us_optionflags,
 				us_useroptions | CHECKDATE, VINTEGER);
 		} else
 		{
 			if ((us_useroptions&CHECKDATE) == 0)
-				ttyputverbose(_("Facet revision dates already being ignored")); else
-					ttyputverbose(_("Facet revision date errors will be ignored"));
+				ttyputverbose(M_("Facet revision dates already being ignored")); else
+					ttyputverbose(M_("Facet revision date errors will be ignored"));
 			(void)setvalkey((INTBIG)us_aid, VAID, us_optionflags,
 				us_useroptions & ~CHECKDATE, VINTEGER);
 		}
@@ -2819,15 +2831,15 @@ void us_defnode(INTSML count, char *par[])
 		if (negated == 0)
 		{
 			if ((us_useroptions&CENTEREDPRIMITIVES) != 0)
-				ttyputverbose(_("Primitives are already being placed by their center")); else
-					ttyputverbose(_("Primitives will be placed by their center"));
+				ttyputverbose(M_("Primitives are already being placed by their center")); else
+					ttyputverbose(M_("Primitives will be placed by their center"));
 			(void)setvalkey((INTBIG)us_aid, VAID, us_optionflags,
 				us_useroptions | CENTEREDPRIMITIVES, VINTEGER);
 		} else
 		{
 			if ((us_useroptions&CENTEREDPRIMITIVES) == 0)
-				ttyputverbose(_("Primitives are already being placed by edges")); else
-					ttyputverbose(_("Primitives will be placed by edges"));
+				ttyputverbose(M_("Primitives are already being placed by edges")); else
+					ttyputverbose(M_("Primitives will be placed by edges"));
 			(void)setvalkey((INTBIG)us_aid, VAID, us_optionflags,
 				us_useroptions & ~CENTEREDPRIMITIVES, VINTEGER);
 		}
@@ -2844,15 +2856,15 @@ void us_defnode(INTSML count, char *par[])
 		if (negated == 0)
 		{
 			if ((us_useroptions&DUPCOPIESPORTS) != 0)
-				ttyputverbose(_("Ports are already being copied by duplicate/array/yanknode")); else
-					ttyputverbose(_("Ports will be copied by duplicate/array/yanknode"));
+				ttyputverbose(M_("Ports are already being copied by duplicate/array/yanknode")); else
+					ttyputverbose(M_("Ports will be copied by duplicate/array/yanknode"));
 			(void)setvalkey((INTBIG)us_aid, VAID, us_optionflags,
 				us_useroptions | DUPCOPIESPORTS, VINTEGER);
 		} else
 		{
 			if ((us_useroptions&DUPCOPIESPORTS) == 0)
-				ttyputverbose(_("Ports are already not being copied by duplicate/array/yanknode")); else
-					ttyputverbose(_("Ports will not be copied by duplicate/array/yanknode"));
+				ttyputverbose(M_("Ports are already not being copied by duplicate/array/yanknode")); else
+					ttyputverbose(M_("Ports will not be copied by duplicate/array/yanknode"));
 			(void)setvalkey((INTBIG)us_aid, VAID, us_optionflags,
 				us_useroptions & ~DUPCOPIESPORTS, VINTEGER);
 		}
@@ -2868,11 +2880,11 @@ void us_defnode(INTSML count, char *par[])
 			{
 				if ((np->userbits&NLOCKED) == 0)
 				{
-					ttyputverbose(_("Contents of facet %s are already able to be freely altered"),
+					ttyputverbose(M_("Contents of facet %s are already able to be freely altered"),
 						describenodeproto(np));
 				} else
 				{
-					ttyputverbose(_("Contents of facet %s may now be freely edited"),
+					ttyputverbose(M_("Contents of facet %s may now be freely edited"),
 						describenodeproto(np));
 				}
 				(void)setval((INTBIG)np, VNODEPROTO, "userbits",
@@ -2881,11 +2893,11 @@ void us_defnode(INTSML count, char *par[])
 			{
 				if ((np->userbits&NLOCKED) != 0)
 				{
-					ttyputverbose(_("Contents of facet %s are already locked"),
+					ttyputverbose(M_("Contents of facet %s are already locked"),
 						describenodeproto(np));
 				} else
 				{
-					ttyputverbose(_("Contents of facet %s will not be allowed to change"),
+					ttyputverbose(M_("Contents of facet %s will not be allowed to change"),
 						describenodeproto(np));
 				}
 				(void)setval((INTBIG)np, VNODEPROTO, "userbits",
@@ -2907,11 +2919,11 @@ void us_defnode(INTSML count, char *par[])
 			{
 				if ((np->userbits&NILOCKED) == 0)
 				{
-					ttyputverbose(_("Instances in facet %s are already able to be freely altered"),
+					ttyputverbose(M_("Instances in facet %s are already able to be freely altered"),
 						describenodeproto(np));
 				} else
 				{
-					ttyputverbose(_("Instances in facet %s may now be freely edited"),
+					ttyputverbose(M_("Instances in facet %s may now be freely edited"),
 						describenodeproto(np));
 				}
 				(void)setval((INTBIG)np, VNODEPROTO, "userbits",
@@ -2920,11 +2932,11 @@ void us_defnode(INTSML count, char *par[])
 			{
 				if ((np->userbits&NILOCKED) != 0)
 				{
-					ttyputverbose(_("Instances in facet %s are already locked"),
+					ttyputverbose(M_("Instances in facet %s are already locked"),
 						describenodeproto(np));
 				} else
 				{
-					ttyputverbose(_("Instances in facet %s will not be allowed to change"),
+					ttyputverbose(M_("Instances in facet %s will not be allowed to change"),
 						describenodeproto(np));
 				}
 				(void)setval((INTBIG)np, VNODEPROTO, "userbits",
@@ -2936,15 +2948,15 @@ void us_defnode(INTSML count, char *par[])
 			if (negated != 0)
 			{
 				if ((us_useroptions&NOFACETCHANGES) == 0)
-					ttyputverbose(_("Facets are already able to be freely altered")); else
-						ttyputverbose(_("Facets may now be freely edited"));
+					ttyputverbose(M_("Facets are already able to be freely altered")); else
+						ttyputverbose(M_("Facets may now be freely edited"));
 				(void)setvalkey((INTBIG)us_aid, VAID, us_optionflags,
 					us_useroptions & ~NOFACETCHANGES, VINTEGER);
 			} else
 			{
 				if ((us_useroptions&NOFACETCHANGES) != 0)
-					ttyputverbose(_("Facet changes are already being disallowed")); else
-						ttyputverbose(_("Facet changes will not be allowed"));
+					ttyputverbose(M_("Facet changes are already being disallowed")); else
+						ttyputverbose(M_("Facet changes will not be allowed"));
 				(void)setvalkey((INTBIG)us_aid, VAID, us_optionflags,
 					us_useroptions | NOFACETCHANGES, VINTEGER);
 			}
@@ -2962,15 +2974,15 @@ void us_defnode(INTSML count, char *par[])
 		if (negated != 0)
 		{
 			if ((us_useroptions&NOPRIMCHANGES) == 0)
-				ttyputverbose(_("Locked primitives are already able to be freely altered")); else
-					ttyputverbose(_("Locked primitives may now be freely edited"));
+				ttyputverbose(M_("Locked primitives are already able to be freely altered")); else
+					ttyputverbose(M_("Locked primitives may now be freely edited"));
 			(void)setvalkey((INTBIG)us_aid, VAID, us_optionflags,
 				us_useroptions & ~NOPRIMCHANGES, VINTEGER);
 		} else
 		{
 			if ((us_useroptions&NOPRIMCHANGES) != 0)
-				ttyputverbose(_("Locked primitive changes are already being disallowed")); else
-					ttyputverbose(_("Locked primitive changes will not be allowed"));
+				ttyputverbose(M_("Locked primitive changes are already being disallowed")); else
+					ttyputverbose(M_("Locked primitive changes will not be allowed"));
 			(void)setvalkey((INTBIG)us_aid, VAID, us_optionflags,
 				us_useroptions | NOPRIMCHANGES, VINTEGER);
 		}
@@ -3001,7 +3013,7 @@ void us_defnode(INTSML count, char *par[])
 					(void)setval((INTBIG)np, VNODEPROTO, "userbits",
 						np->userbits & ~WANTNEXPAND, VINTEGER);
 		}
-		ttyputverbose(_("%s node prototypes will be %sexpanded on creation"), nname, (negated==0 ? "" : _("un-")));
+		ttyputverbose(M_("%s node prototypes will be %sexpanded on creation"), nname, (negated==0 ? "" : _("un-")));
 		return;
 	}
 
@@ -3049,9 +3061,11 @@ void us_defnode(INTSML count, char *par[])
 		nodeprotosizeoffset(np, &plx, &ply, &phx, &phy);
 		nodesize[0] = i+plx+phx;
 		nodesize[1] = j+ply+phy;
-		(void)setvalkey((INTBIG)np, VNODEPROTO, el_node_default_size,
+		nodesize[0] = nodesize[0] * WHOLE / el_curlib->lambda[np->tech->techindex];
+		nodesize[1] = nodesize[1] * WHOLE / el_curlib->lambda[np->tech->techindex];
+		(void)setvalkey((INTBIG)np, VNODEPROTO, el_node_size_default,
 			(INTBIG)nodesize, VINTEGER|VISARRAY|(2<<VLENGTHSH));
-		ttyputverbose(_("%s nodes will be created %sx%s in size"), describenodeproto(np),
+		ttyputverbose(M_("%s nodes will be created %sx%s in size"), describenodeproto(np),
 			latoa(i), latoa(j));
 		return;
 	}
@@ -3102,7 +3116,7 @@ void us_duplicate(INTSML count, char *par[])
 	/* get object */
 	np = us_needfacet();
 	if (np == NONODEPROTO) return;
-	list = us_gethighlighted(OBJNODEINST | OBJARCINST);
+	list = us_gethighlighted(OBJNODEINST | OBJARCINST, 0, 0);
 	if (list[0] == NOGEOM)
 	{
 		us_abortcommand(_("First select objects to duplicate"));
@@ -3137,13 +3151,13 @@ void us_debugjohn(void)
 	if (facet == NONODEPROTO) return;
 
 	/* get the objects we need */
-	tra = getnodeproto("mocmossub:N-Transistor");
+	tra = getnodeproto("mocmos:N-Transistor");
 	if (tra == NONODEPROTO) return;
-	contact = getnodeproto("mocmossub:Metal-1-N-Active-Con");
+	contact = getnodeproto("mocmos:Metal-1-N-Active-Con");
 	conport = contact->firstportproto;
 	topport = getportproto(tra, "n-trans-diff-top");
 	botport = getportproto(tra, "n-trans-diff-bot");
-	active = getarcproto("mocmossub:N-Active");
+	active = getarcproto("mocmos:N-Active");
 
 	/* get information for layout */
 	sizex = tra->highx - tra->lowx;
