@@ -42,7 +42,7 @@
 #  include <fcntl.h>
 #endif
 
-#define	MAXLINE 500		/* maximum length of news/help file input lines */
+#define MAXLINE 500		/* maximum length of news/help file input lines */
 
 /* working memory for "us_iterate()" */
 static INTBIG *us_iterateaddr, *us_iteratetype;
@@ -62,7 +62,7 @@ void us_freecomekmemory(void)
 
 void us_echo(INTSML count, char *par[])
 {
-	REGISTER INTSML lastquiet, i, j;
+	REGISTER INTBIG lastquiet, i, j;
 
 	lastquiet = ttyquiet(0);
 	i = initinfstr();
@@ -82,7 +82,7 @@ void us_echo(INTSML count, char *par[])
 
 void us_editfacet(INTSML count, char *par[])
 {
-	REGISTER INTSML implicit, i, len, newwindow, nonredundant, newframe, push;
+	REGISTER INTBIG implicit, i, len, newwindow, nonredundant, newframe, push;
 	INTBIG lx, hx, ly, hy;
 	REGISTER INTBIG lambda;
 	REGISTER NODEPROTO *np, *onp, *np1, *curfacet;
@@ -162,7 +162,7 @@ void us_editfacet(INTSML count, char *par[])
 
 			/* icon facets should always be expanded */
 			if (np->cellview == el_iconview) np->userbits |= WANTNEXPAND;
-			ttyputverbose(_("Editing new facet: %s"), par[0]);
+			ttyputverbose(M_("Editing new facet: %s"), par[0]);
 		}
 
 		/* look through window for instances of this facet */
@@ -264,10 +264,9 @@ void us_editfacet(INTSML count, char *par[])
 
 void us_erase(INTSML count, char *par[])
 {
-	REGISTER INTSML i, j, l, otherend, foundnode;
-	REGISTER NODEINST *ni, *nextni, *ni1, *ni2;
+	REGISTER INTSML i, j, l, foundnode;
+	REGISTER NODEINST *ni;
 	REGISTER ARCINST *ai;
-	REGISTER PORTARCINST *pi;
 	REGISTER NODEPROTO *np;
 	HIGHLIGHT high;
 	REGISTER char *pt;
@@ -369,7 +368,7 @@ void us_erase(INTSML count, char *par[])
 		us_abortcommand(_("No current facet"));
 		return;
 	}
-	list = us_gethighlighted(OBJARCINST|OBJNODEINST);
+	list = us_gethighlighted(OBJARCINST|OBJNODEINST, 0, 0);
 	if (list[0] == NOGEOM)
 	{
 		us_abortcommand(_("Find an object to erase"));
@@ -430,104 +429,17 @@ void us_erase(INTSML count, char *par[])
 
 	/* handle simple erasing */
 	us_clearhighlightcount();
-
-	/* mark all nodes touching arcs that are killed */
-	for(ni = np->firstnodeinst; ni != NONODEINST; ni = ni->nextnodeinst) ni->temp1 = 0;
-	for(i=0; list[i] != NOGEOM; i++)
-	{
-		if (list[i]->entrytype != OBJARCINST) continue;
-		ai = list[i]->entryaddr.ai;
-		ai->end[0].nodeinst->temp1 = 1;
-		ai->end[1].nodeinst->temp1 = 1;
-	}
-
-	/* also mark all nodes on arcs that will be erased */
-	for(i=0; list[i] != NOGEOM; i++)
-	{
-		if (list[i]->entrytype != OBJNODEINST) continue;
-		ni = list[i]->entryaddr.ni;
-		if (ni->temp1 != 0) ni->temp1 = 2;
-	}
-
-	/* also mark all nodes on the other end of arcs connected to erased nodes */
-	for(i=0; list[i] != NOGEOM; i++)
-	{
-		if (list[i]->entrytype != OBJNODEINST) continue;
-		ni = list[i]->entryaddr.ni;
-		for(pi = ni->firstportarcinst; pi != NOPORTARCINST; pi = pi->nextportarcinst)
-		{
-			ai = pi->conarcinst;
-			if (ai->end[0].portarcinst == pi) otherend = 1; else
-				otherend = 0;
-			if (ai->end[otherend].nodeinst->temp1 == 0)
-				ai->end[otherend].nodeinst->temp1 = 1;
-		}
-	}
-
-	/* now kill all of the arcs */
-	for(i=0; list[i] != NOGEOM; i++)
-	{
-		if (list[i]->entrytype != OBJARCINST) continue;
-		ai = list[i]->entryaddr.ai;
-
-		/* see if nodes need to be undrawn to account for "Steiner Point" changes */
-		ni1 = ai->end[0].nodeinst;   ni2 = ai->end[1].nodeinst;
-		if (ni1->temp1 == 1 && (ni1->proto->userbits&WIPEON1OR2) != 0)
-			startobjectchange((INTBIG)ni1, VNODEINST);
-		if (ni2->temp1 == 1 && (ni2->proto->userbits&WIPEON1OR2) != 0)
-			startobjectchange((INTBIG)ni2, VNODEINST);
-
-		startobjectchange((INTBIG)ai, VARCINST);
-		if (killarcinst(ai)) ttyputerr(_("Error killing arc"));
-
-		/* see if nodes need to be redrawn to account for "Steiner Point" changes */
-		if (ni1->temp1 == 1 && (ni1->proto->userbits&WIPEON1OR2) != 0)
-			endobjectchange((INTBIG)ni1, VNODEINST);
-		if (ni2->temp1 == 1 && (ni2->proto->userbits&WIPEON1OR2) != 0)
-			endobjectchange((INTBIG)ni2, VNODEINST);
-	}
-
-	/* next kill all of the nodes */
-	for(i=0; list[i] != NOGEOM; i++)
-	{
-		if (list[i]->entrytype != OBJNODEINST) continue;
-		ni = list[i]->entryaddr.ni;
-		us_erasenodeinst(ni);
-	}
-
-	/* kill all pin nodes that touched an arc and no longer do */
-	for(ni = np->firstnodeinst; ni != NONODEINST; ni = nextni)
-	{
-		nextni = ni->nextnodeinst;
-		if (ni->temp1 == 0) continue;
-		if (ni->proto->primindex == 0) continue;
-		if (((ni->proto->userbits&NFUNCTION) >> NFUNCTIONSH) != NPPIN) continue;
-		if (ni->firstportarcinst != NOPORTARCINST || ni->firstportexpinst != NOPORTEXPINST)
-			continue;
-		us_erasenodeinst(ni);
-	}
-
-	/* kill all unexported pin or bus nodes left in the middle of arcs */
-	for(ni = np->firstnodeinst; ni != NONODEINST; ni = nextni)
-	{
-		nextni = ni->nextnodeinst;
-		if (ni->temp1 == 0) continue;
-		if (ni->proto->primindex == 0) continue;
-		if (((ni->proto->userbits&NFUNCTION) >> NFUNCTIONSH) != NPPIN) continue;
-		if (ni->firstportexpinst != NOPORTEXPINST) continue;
-		(void)us_erasepassthru(ni, 0);
-	}
+	us_eraseobjectsinlist(np, list);
 }
 
 void us_find(INTSML count, char *par[])
 {
-	REGISTER INTSML findport, l, findpoint, findexclusively, findmore, findangle,
-		findwithin, findstill, extrainfo, findnobox, findspecial, size, findconstrained,
-		findeasy, findhard;
-	REGISTER INTBIG i, total, type, addr;
+	REGISTER INTBIG i, j, l, findport, findpoint, findexclusively, findangle,
+		findwithin, findstill, findspecial, size, findconstrained,
+		findeasy, findhard, total, type, addr, len;
+	REGISTER INTBIG areasizex, areasizey, x, y, *newlist, extrainfo, findmore, findnobox;
 	INTBIG xcur, ycur;
 	XARRAY trans;
-	REGISTER INTBIG areasizex, areasizey, x, y, *newlist;
 	REGISTER NODEINST *ni;
 	REGISTER NODEPROTO *np, *onp;
 	REGISTER LIBRARY *lib;
@@ -554,10 +466,10 @@ void us_find(INTSML count, char *par[])
 				findangle = (INTSML)var->addr;
 			if (findangle == 0)
 			{
-				ttyputverbose(_("Interactive dragging done with no constraints"));
+				ttyputverbose(M_("Interactive dragging done with no constraints"));
 			} else
 			{
-				ttyputverbose(_("Interactive dragging constrained to %d degree angles"),
+				ttyputverbose(M_("Interactive dragging constrained to %d degree angles"),
 					findangle);
 			}
 			return;
@@ -580,10 +492,10 @@ void us_find(INTSML count, char *par[])
 			}
 			if ((us_useroptions&FLIPINTERACTIVECON) != 0)
 			{
-				ttyputverbose(_("Interactive selection constrains to one axis"));
+				ttyputverbose(M_("Interactive selection constrains to one axis"));
 			} else
 			{
-				ttyputverbose(_("Interactive selection allows motion in either axis"));
+				ttyputverbose(M_("Interactive selection allows motion in either axis"));
 			}
 			return;
 		}
@@ -594,6 +506,14 @@ void us_find(INTSML count, char *par[])
 	if (np == NONODEPROTO) return;
 	if ((np->cellview->viewstate&TEXTVIEW) != 0)
 	{
+		l = strlen(pt = par[0]);
+		if (namesamen(pt, "all", l) == 0 && l >= 2)
+		{
+			/* special case: "find all" selects all text */
+			i = us_totallines(el_curwindowpart);
+			us_highlightline(el_curwindowpart, 0, i-1);
+			return;
+		}
 		us_abortcommand(_("There are no components to select in a text-only facet"));
 		return;
 	}
@@ -612,6 +532,7 @@ void us_find(INTSML count, char *par[])
 		newhigh.fromgeom = NOGEOM;
 		newhigh.fromport = NOPORTPROTO;
 		newhigh.fromvar = NOVARIABLE;
+		newhigh.fromvarnoeval = NOVARIABLE;
 		newhigh.frompoint = 0;
 	}
 
@@ -716,11 +637,12 @@ void us_find(INTSML count, char *par[])
 			list = (char **)emalloc((total * (sizeof (char *))), el_tempcluster);
 			if (list == 0) return;
 
-			newhigh.status = HIGHFROM | extrainfo;
+			newhigh.status = (INTSML)(HIGHFROM | extrainfo);
 			if (findnobox != 0) newhigh.status |= HIGHNOBOX;
 			newhigh.facet = np;
 			newhigh.frompoint = 0;
 			newhigh.fromvar = NOVARIABLE;
+			newhigh.fromvarnoeval = NOVARIABLE;
 			total = 0;
 			for(ni = np->firstnodeinst; ni != NONODEINST; ni = ni->nextnodeinst)
 			{
@@ -947,7 +869,7 @@ void us_find(INTSML count, char *par[])
 				if (var == NOVARIABLE) findangle = 0; else
 					findangle = (INTSML)var->addr;
 			}				
-			us_findiinit(findport, extrainfo, findangle, (INTSML)(1-findmore), findstill,
+			us_findiinit(findport, extrainfo, findangle, 1-findmore, findstill,
 				findnobox, findspecial);
 			trackcursor(0, us_ignoreup, us_findcibegin, us_findidown, us_stoponchar,
 				us_findiup, TRACKDRAGGING);
@@ -956,14 +878,31 @@ void us_find(INTSML count, char *par[])
 
 		if (namesamen(pt, "deselect-arcs", l) == 0 && l >= 2)
 		{
-			glist = us_gethighlighted(OBJARCINST);
-			for(i=0; glist[i] != NOGEOM; i++)
+			var = getvalkey((INTBIG)us_aid, VAID, VSTRING|VISARRAY, us_highlighted);
+			if (var == NOVARIABLE) return;
+			len = getlength(var);
+			list = (char **)emalloc(len * (sizeof (char *)), el_tempcluster);
+			if (list == 0) return;
+			j = 0;
+			for(i=0; i<len; i++)
 			{
-				newhigh.status = HIGHFROM;
-				newhigh.fromgeom = glist[i];
-				newhigh.fromport = NOPORTPROTO;
-				us_delhighlight(&newhigh);
+				if (us_makehighlight(((char **)var->addr)[i], &newhigh) != 0) break;
+				if ((newhigh.status&HIGHTYPE) == HIGHFROM &&
+					newhigh.fromgeom->entrytype == OBJARCINST) continue;
+				(void)allocstring(&list[j], ((char **)var->addr)[i], el_tempcluster);
+				j++;
 			}
+			if (j == 0)
+			{
+				(void)delvalkey((INTBIG)us_aid, VAID, us_highlighted);
+			} else
+			{
+				(void)setvalkey((INTBIG)us_aid, VAID, us_highlighted, (INTBIG)list,
+					VSTRING|VISARRAY|(j<<VLENGTHSH)|VDONTSAVE);
+			}
+			for(i=0; i<j; i++)
+				efree((char *)list[i]);
+			efree((char *)list);
 			return;
 		}
 
@@ -976,7 +915,7 @@ void us_find(INTSML count, char *par[])
 				return;
 			}
 			us_pushhighlight();
-			ttyputverbose(_("Pushed"));
+			ttyputverbose(M_("Pushed"));
 			return;
 		}
 
@@ -1066,7 +1005,7 @@ void us_find(INTSML count, char *par[])
 				if (var == NOVARIABLE) findangle = 0; else
 					findangle = (INTSML)var->addr;
 			}				
-			us_findiinit(findport, extrainfo, findangle, (INTSML)(1-findmore), findstill,
+			us_findiinit(findport, extrainfo, findangle, 1-findmore, findstill,
 				findnobox, findspecial);
 			trackcursor(0, us_ignoreup, us_findibegin, us_findidown, us_stoponchar,
 				us_findiup, TRACKDRAGGING);
@@ -1377,13 +1316,13 @@ void us_find(INTSML count, char *par[])
 			(void)addstringtoinfstr("USER_highlight_");
 			(void)addstringtoinfstr(par[1]);
 			(void)setval((INTBIG)us_aid, VAID, returninfstr(), var->addr, var->type|VDONTSAVE);
-			ttyputverbose(_("%s saved"), par[1]);
+			ttyputverbose(M_("%s saved"), par[1]);
 			return;
 		}
 
 		if (namesamen(pt, "set-easy-selection", l) == 0 && l >= 5)
 		{
-			glist = us_gethighlighted(OBJARCINST|OBJNODEINST);
+			glist = us_gethighlighted(OBJARCINST|OBJNODEINST, 0, 0);
 			if (glist[0] == NOGEOM)
 			{
 				us_abortcommand(_("Select something before making it easy-to-select"));
@@ -1407,7 +1346,7 @@ void us_find(INTSML count, char *par[])
 
 		if (namesamen(pt, "set-hard-selection", l) == 0 && l >= 5)
 		{
-			glist = us_gethighlighted(OBJARCINST|OBJNODEINST);
+			glist = us_gethighlighted(OBJARCINST|OBJNODEINST, 0, 0);
 			if (glist[0] == NOGEOM)
 			{
 				us_abortcommand(_("Select something before making it easy-to-select"));
@@ -1442,14 +1381,14 @@ void us_find(INTSML count, char *par[])
 			{
 				switch (us_state&SNAPMODE)
 				{
-					case SNAPMODENONE:     ttyputmsg(_("Snapping mode: none"));              break;
-					case SNAPMODECENTER:   ttyputmsg(_("Snapping mode: center"));            break;
-					case SNAPMODEMIDPOINT: ttyputmsg(_("Snapping mode: midpoint"));          break;
-					case SNAPMODEENDPOINT: ttyputmsg(_("Snapping mode: end point"));         break;
-					case SNAPMODETANGENT:  ttyputmsg(_("Snapping mode: tangent"));           break;
-					case SNAPMODEPERP:     ttyputmsg(_("Snapping mode: perpendicular"));     break;
-					case SNAPMODEQUAD:     ttyputmsg(_("Snapping mode: quadrant"));          break;
-					case SNAPMODEINTER:    ttyputmsg(_("Snapping mode: any intersection"));  break;
+					case SNAPMODENONE:     ttyputmsg(M_("Snapping mode: none"));              break;
+					case SNAPMODECENTER:   ttyputmsg(M_("Snapping mode: center"));            break;
+					case SNAPMODEMIDPOINT: ttyputmsg(M_("Snapping mode: midpoint"));          break;
+					case SNAPMODEENDPOINT: ttyputmsg(M_("Snapping mode: end point"));         break;
+					case SNAPMODETANGENT:  ttyputmsg(M_("Snapping mode: tangent"));           break;
+					case SNAPMODEPERP:     ttyputmsg(M_("Snapping mode: perpendicular"));     break;
+					case SNAPMODEQUAD:     ttyputmsg(M_("Snapping mode: quadrant"));          break;
+					case SNAPMODEINTER:    ttyputmsg(M_("Snapping mode: any intersection"));  break;
 				}
 				return;
 			}
@@ -1457,49 +1396,49 @@ void us_find(INTSML count, char *par[])
 			if (namesamen(pt, "none", l) == 0)
 			{
 				us_state = (us_state & ~SNAPMODE) | SNAPMODENONE;
-				ttyputverbose(_("Snapping mode: none"));
+				ttyputverbose(M_("Snapping mode: none"));
 				return;
 			}
 			if (namesamen(pt, "center", l) == 0)
 			{
 				us_state = (us_state & ~SNAPMODE) | SNAPMODECENTER;
-				ttyputverbose(_("Snapping mode: center"));
+				ttyputverbose(M_("Snapping mode: center"));
 				return;
 			}
 			if (namesamen(pt, "midpoint", l) == 0)
 			{
 				us_state = (us_state & ~SNAPMODE) | SNAPMODEMIDPOINT;
-				ttyputverbose(_("Snapping mode: midpoint"));
+				ttyputverbose(M_("Snapping mode: midpoint"));
 				return;
 			}
 			if (namesamen(pt, "endpoint", l) == 0)
 			{
 				us_state = (us_state & ~SNAPMODE) | SNAPMODEENDPOINT;
-				ttyputverbose(_("Snapping mode: endpoint"));
+				ttyputverbose(M_("Snapping mode: endpoint"));
 				return;
 			}
 			if (namesamen(pt, "tangent", l) == 0)
 			{
 				us_state = (us_state & ~SNAPMODE) | SNAPMODETANGENT;
-				ttyputverbose(_("Snapping mode: tangent"));
+				ttyputverbose(M_("Snapping mode: tangent"));
 				return;
 			}
 			if (namesamen(pt, "perpendicular", l) == 0)
 			{
 				us_state = (us_state & ~SNAPMODE) | SNAPMODEPERP;
-				ttyputverbose(_("Snapping mode: perpendicular"));
+				ttyputverbose(M_("Snapping mode: perpendicular"));
 				return;
 			}
 			if (namesamen(pt, "quadrant", l) == 0)
 			{
 				us_state = (us_state & ~SNAPMODE) | SNAPMODEQUAD;
-				ttyputverbose(_("Snapping mode: quadrant"));
+				ttyputverbose(M_("Snapping mode: quadrant"));
 				return;
 			}
 			if (namesamen(pt, "intersection", l) == 0)
 			{
 				us_state = (us_state & ~SNAPMODE) | SNAPMODEINTER;
-				ttyputverbose(_("Snapping mode: any intersection"));
+				ttyputverbose(M_("Snapping mode: any intersection"));
 				return;
 			}
 			us_abortcommand(_("Unknown snapping mode: %s"), pt);
@@ -1547,7 +1486,7 @@ void us_getproto(INTSML count, char *par[])
 
 	if (count == 0)
 	{
-		count = ttygetparam(_("Getproto option: "), &us_getproto1p, MAXPARS, par);
+		count = ttygetparam(M_("Getproto option: "), &us_getproto1p, MAXPARS, par);
 		if (count == 0)
 		{
 			us_abortedmsg();
@@ -1705,7 +1644,7 @@ void us_grid(INTSML count, char *par[])
 			}
 			(void)setvalkey((INTBIG)us_aid, VAID, us_alignment_obj, i, VINTEGER|VDONTSAVE);
 		}
-		ttyputverbose(_("Cursor alignment is %s lambda"), latoa(us_alignment));
+		ttyputverbose(M_("Cursor alignment is %s lambda"), latoa(us_alignment));
 		return;
 	}
 
@@ -1721,8 +1660,8 @@ void us_grid(INTSML count, char *par[])
 			}
 			(void)setvalkey((INTBIG)us_aid, VAID, us_alignment_edge, i, VINTEGER|VDONTSAVE);
 		}
-		if (us_edgealignment == 0) ttyputverbose(_("No edge alignment done")); else
-			ttyputverbose(_("Edge alignment is %s lambda"), latoa(us_edgealignment));
+		if (us_edgealignment == 0) ttyputverbose(M_("No edge alignment done")); else
+			ttyputverbose(M_("Edge alignment is %s lambda"), latoa(us_edgealignment));
 		return;
 	}
 
@@ -1773,15 +1712,14 @@ void us_grid(INTSML count, char *par[])
 	ttyputbadusage("grid");
 }
 
-#define	NEWSFILE   "newsfile"			/* file with news */
+#define NEWSFILE   "newsfile"			/* file with news */
 
-#define	NEWSDATE   ".electricnews"		/* file with date of last news */
+#define NEWSDATE   ".electricnews"		/* file with date of last news */
 
 void us_help(INTSML count, char *par[])
 {
-	REGISTER INTSML len;
 	REGISTER char *pp;
-	REGISTER INTSML lastquiet, nday, nmonth, nyear, on;
+	REGISTER INTBIG len, lastquiet, nday, nmonth, nyear, on;
 	INTSML day, month, year, hour, minute, second;
 	FILE *in;
 	char line[256], sep[2], *filename, *hd;
@@ -2246,7 +2184,7 @@ void us_killfacet(INTSML count, char *par[])
 
 	if (count == 0)
 	{
-		count = ttygetparam(_("Facet name: "), &us_showdp, MAXPARS, par);
+		count = ttygetparam(M_("Facet name: "), &us_showdp, MAXPARS, par);
 		if (count == 0)
 		{
 			us_abortedmsg();
