@@ -81,29 +81,29 @@ void us_lambda(INTSML count, char *par[])
 			switch (el_units&DISPLAYUNITS)
 			{
 				case DISPUNITLAMBDA:
-					ttyputverbose(_("Distance expressed in lambda (currently %ld %ss)"),
+					ttyputverbose(M_("Distance expressed in lambda (currently %ld %ss)"),
 						el_curlib->lambda[el_curtech->techindex], unitsname(el_units));
 					break;
 				case DISPUNITINCH:
-					ttyputverbose(_("Distance expressed in inches"));
+					ttyputverbose(M_("Distance expressed in inches"));
 					break;
 				case DISPUNITCM:
-					ttyputverbose(_("Distance expressed in centimeters"));
+					ttyputverbose(M_("Distance expressed in centimeters"));
 					break;
 				case DISPUNITMM:
-					ttyputverbose(_("Distance expressed in millimeters"));
+					ttyputverbose(M_("Distance expressed in millimeters"));
 					break;
 				case DISPUNITMIL:
-					ttyputverbose(_("Distance expressed in mils"));
+					ttyputverbose(M_("Distance expressed in mils"));
 					break;
 				case DISPUNITMIC:
-					ttyputverbose(_("Distance expressed in microns"));
+					ttyputverbose(M_("Distance expressed in microns"));
 					break;
 				case DISPUNITCMIC:
-					ttyputverbose(_("Distance expressed in centimicrons"));
+					ttyputverbose(M_("Distance expressed in centimicrons"));
 					break;
 				case DISPUNITMMIC:
-					ttyputverbose(_("Distance expressed in millimicrons"));
+					ttyputverbose(M_("Distance expressed in millimicrons"));
 					break;
 			}
 			return;
@@ -127,7 +127,7 @@ void us_lambda(INTSML count, char *par[])
 					return;
 				}
 			}
-			ttyputverbose(_("Smallest internal unit is %s"), unitsname(el_units));
+			ttyputverbose(M_("Smallest internal unit is %s"), unitsname(el_units));
 			return;
 		}
 
@@ -141,7 +141,7 @@ void us_lambda(INTSML count, char *par[])
 
 		if (count <= 1)
 		{
-			count = ttygetparam(_("Lambda value: "), &us_lambdachp, MAXPARS-1, &par[1]) + 1;
+			count = ttygetparam(M_("Lambda value: "), &us_lambdachp, MAXPARS-1, &par[1]) + 1;
 			if (count == 1)
 			{
 				us_abortedmsg();
@@ -161,7 +161,7 @@ void us_lambda(INTSML count, char *par[])
 				&us_noyesp, MAXPARS, par);
 			if (count <= 0 || namesamen(par[0], "yes", strlen(par[0])) != 0)
 			{
-				ttyputverbose(_("No change made"));
+				ttyputverbose(M_("No change made"));
 				return;
 			}
 		}
@@ -203,7 +203,7 @@ void us_lambda(INTSML count, char *par[])
 	for(tech = el_technologies; tech != NOTECHNOLOGY; tech = tech->nexttechnology)
 	{
 		if (tech->temp1 == 0) continue;
-		ttyputverbose(_("Lambda size for %s is %s"), tech->techname,
+		ttyputverbose(M_("Lambda size for %s is %s"), tech->techname,
 			latoa(el_curlib->lambda[tech->techindex]));
 	}
 }
@@ -226,7 +226,7 @@ void us_library(INTSML count, char *par[])
 
 	if (count == 0)
 	{
-		count = ttygetparam(_("Library option: "), &us_libraryp, MAXPARS, par);
+		count = ttygetparam(M_("Library option: "), &us_libraryp, MAXPARS, par);
 		if (count == 0)
 		{
 			us_abortedmsg();
@@ -243,7 +243,7 @@ void us_library(INTSML count, char *par[])
 			return;
 		}
 		setlibdir(par[1]);
-		ttyputverbose(_("Default library path is now '%s'"), el_libdir);
+		ttyputverbose(M_("Default library path is now '%s'"), el_libdir);
 		return;
 	}
 
@@ -317,8 +317,8 @@ void us_library(INTSML count, char *par[])
 		if (lib == el_curlib)
 		{
 			us_switchtolibrary(el_curlib->nextlibrary);
-			ttyputverbose(_("Current library is now %s"), el_curlib->libname);
-		} else ttyputverbose(_("Library %s killed"), lib->libname);
+			ttyputverbose(M_("Current library is now %s"), el_curlib->libname);
+		} else ttyputverbose(M_("Library %s killed"), lib->libname);
 
 		/* remove display of all facets in this library */
 		for(w = el_topwindowpart; w != NOWINDOWPART; w = w->nextwindowpart)
@@ -464,9 +464,6 @@ void us_library(INTSML count, char *par[])
 						us_redisplaynow(w, 1);
 					}
 				}
-
-				/* remove any GETPROTOs to facets in this library */
-				us_removegetfacets();
 			}
 
 			/* read the library */
@@ -476,8 +473,10 @@ void us_library(INTSML count, char *par[])
 				if (i == 0) arg = 1; else
 					if (i == multifile) arg = 3;
 			}
+			if (mergelib == NOLIBRARY) changesquiet(1);
 			if (askaid(io_aid, "read", (INTBIG)lib, (INTBIG)style, arg) != 0)
 				retval = 1;
+			if (mergelib == NOLIBRARY) changesquiet(0);
 			if (multifile != 0 || mergelib != NOLIBRARY) lib->libfile = oldlibfile;
 
 			/* advance to the next file in the list */
@@ -494,12 +493,12 @@ void us_library(INTSML count, char *par[])
 				/* these operations are done directly to prevent undo */
 				us_alignment = muldiv(us_alignment, lib->lambda[el_curtech->techindex],
 					el_curtech->deflambda);
-				nextvarchangequiet();
+				nextchangequiet();
 				(void)setvalkey((INTBIG)us_aid, VAID, us_alignment_obj, us_alignment,
 					VINTEGER|VDONTSAVE);
 				us_edgealignment = muldiv(us_edgealignment, lib->lambda[el_curtech->techindex],
 					el_curtech->deflambda);
-				nextvarchangequiet();
+				nextchangequiet();
 				(void)setvalkey((INTBIG)us_aid, VAID, us_alignment_edge, us_edgealignment,
 					VINTEGER|VDONTSAVE);
 				us_setalignment(NOWINDOWFRAME);
@@ -511,7 +510,7 @@ void us_library(INTSML count, char *par[])
 				us_setlambda(NOWINDOWFRAME);
 				if (us_curnodeproto == NONODEPROTO || us_curnodeproto->primindex == 0)
 				{
-					nextvarchangequiet();
+					nextchangequiet();
 					(void)setvalkey((INTBIG)us_aid, VAID, us_current_node,
 						(INTBIG)el_curtech->firstnodeproto, VNODEPROTO|VDONTSAVE);
 					us_shadownodeproto(NOWINDOWFRAME, el_curtech->firstnodeproto);
@@ -708,7 +707,7 @@ void us_library(INTSML count, char *par[])
 
 		/* make this library the current one */
 		us_switchtolibrary(lib);
-		ttyputverbose(_("New library: %s"), libn);
+		ttyputverbose(M_("New library: %s"), libn);
 		return;
 	}
 
@@ -738,7 +737,7 @@ void us_library(INTSML count, char *par[])
 			return;
 		}
 
-		ttyputverbose(_("New library: %s"), libn);
+		ttyputverbose(M_("New library: %s"), libn);
 		return;
 	}
 
@@ -747,6 +746,7 @@ void us_library(INTSML count, char *par[])
 		/* save every library that has been modified */
 		for(lib = el_curlib; lib != NOLIBRARY; lib = lib->nextlibrary)
 		{
+			if ((lib->userbits&HIDDENLIBRARY) != 0) continue;
 			if ((lib->userbits&(LIBCHANGEDMAJOR | LIBCHANGEDMINOR)) == 0) continue;
 
 			/* save the library in binary format */
@@ -883,7 +883,7 @@ void us_macbegin(INTSML count, char *par[])
 	/* get macro name */
 	if (count == 0)
 	{
-		count = ttygetparam(_("Macro name: "), &us_macbeginnp, MAXPARS, par);
+		count = ttygetparam(M_("Macro name: "), &us_macbeginnp, MAXPARS, par);
 		if (count == 0)
 		{
 			us_abortedmsg();
@@ -950,7 +950,7 @@ void us_macbegin(INTSML count, char *par[])
 	(void)setvalkey((INTBIG)us_aid, VAID, us_macrobuilding, (INTBIG)pp, VSTRING|VDONTSAVE);
 	(void)setvalkey((INTBIG)us_aid, VAID, us_macrorunning, (INTBIG)pp, VSTRING|VDONTSAVE);
 
-	ttyputverbose(_("Remembering %s..."), pp);
+	ttyputverbose(M_("Remembering %s..."), pp);
 }
 
 void us_macdone(INTSML count, char *par[])
@@ -976,7 +976,7 @@ void us_macend(INTSML count, char *par[])
 		us_abortcommand(_("Not in a macro"));
 		return;
 	}
-	ttyputverbose(_("%s defined"), (char *)var->addr);
+	ttyputverbose(M_("%s defined"), (char *)var->addr);
 	(void)delvalkey((INTBIG)us_aid, VAID, us_macrobuilding);
 	if (getvalkey((INTBIG)us_aid, VAID, VSTRING, us_macrorunning) != NOVARIABLE)
 		(void)delvalkey((INTBIG)us_aid, VAID, us_macrorunning);
@@ -1519,10 +1519,11 @@ void us_menu(INTSML count, char *par[])
 	if (x == us_menux && y == us_menuy && position == us_menupos &&
 		(us_aid->aidstate&MENUON) != 0 && autoset == 0)
 	{
-		ttyputverbose(_("Menu has not changed"));
+		ttyputverbose(M_("Menu has not changed"));
 		return;
 	}
 
+	if (position != us_menupos) resetpaletteparameters();
 	startobjectchange((INTBIG)us_aid, VAID);
 	if (x != us_menux || y != us_menuy || position != us_menupos || autoset != 0)
 		(void)us_setmenusize((INTSML)x, (INTSML)y, (INTSML)position, autoset);
@@ -1547,7 +1548,7 @@ void us_mirror(INTSML count, char *par[])
 	INTBIG gx, gy, cx, cy, x, y, thex, they;
 	extern COMCOMP us_mirrorp;
 
-	list = us_gethighlighted(OBJARCINST|OBJNODEINST);
+	list = us_gethighlighted(OBJARCINST|OBJNODEINST, 0, 0);
 	if (list[0] == NOGEOM)
 	{
 		us_abortcommand(_("Must select objects to mirror"));
@@ -1613,7 +1614,7 @@ void us_mirror(INTSML count, char *par[])
 
 	if (count == 0)
 	{
-		count = ttygetparam(_("Horizontally or vertically? "), &us_mirrorp, MAXPARS, par);
+		count = ttygetparam(M_("Horizontally or vertically? "), &us_mirrorp, MAXPARS, par);
 		if (count == 0)
 		{
 			us_abortedmsg();
@@ -1801,8 +1802,8 @@ void us_mirror(INTSML count, char *par[])
 	/* restore highlighting */
 	(void)us_pophighlight(1);
 
-	if (*pt == 'h') ttyputverbose(_("Node mirrored horizontally")); else
-		ttyputverbose(_("Node mirrored vertically"));
+	if (*pt == 'h') ttyputverbose(M_("Node mirrored horizontally")); else
+		ttyputverbose(M_("Node mirrored vertically"));
 }
 
 static DIALOGITEM us_movetodialogitems[] =
@@ -1816,6 +1817,12 @@ static DIALOGITEM us_movetodialogitems[] =
 };
 DIALOG us_movetodialog = {{50,75,147,244}, N_("Move To Location"), 0, 6, us_movetodialogitems};
 
+/* special items for the "move to" dialog: */
+#define DMVT_XPOS       3		/* X position (edit text) */
+#define DMVT_XPOS_L     4		/* X position label (stat text) */
+#define DMVT_YPOS       5		/* Y position (edit text) */
+#define DMVT_YPOS_L     6		/* Y position label (stat text) */
+
 void us_move(INTSML count, char *par[])
 {
 	REGISTER NODEINST *ni, **nodelist;
@@ -1824,11 +1831,11 @@ void us_move(INTSML count, char *par[])
 	REGISTER INTSML ang;
 	REGISTER INTBIG l, total, i, wid, len, itemHit, amt;
 	INTBIG dx, dy, xcur, ycur, lx, hx, ly, hy, bestlx, bestly, p1x, p1y, p2x, p2y,
-		snapx, snapy;
-	HIGHLIGHT high;
+		snapx, snapy, numtexts, addr, type;
 	REGISTER GEOM **list;
 	HIGHLIGHT thishigh, otherhigh;
 	REGISTER char *pp;
+	char **textlist;
 	static POLYGON *poly = NOPOLYGON;
 	REGISTER VARIABLE *var;
 
@@ -1840,25 +1847,9 @@ void us_move(INTSML count, char *par[])
 	if (np == NONODEPROTO) return;
 
 	/* get the objects to be moved */
-	list = us_gethighlighted(OBJNODEINST | OBJARCINST);
+	list = us_gethighlighted(OBJNODEINST | OBJARCINST, &numtexts, &textlist);
 
-	/* see if a text object is highlighted */
-	var = getvalkey((INTBIG)us_aid, VAID, VSTRING|VISARRAY, us_highlighted);
-	if (var != NOVARIABLE)
-	{
-		if (getlength(var) == 1)
-		{
-			(void)us_makehighlight(((char **)var->addr)[0], &high);
-			if ((high.status&HIGHTYPE) == HIGHTEXT)
-			{
-				us_movetext(&high, count, par);
-				return;
-			}
-			if ((high.status&HIGHTYPE) == HIGHBBOX) list = us_gethighlighted(OBJNODEINST);
-		}
-	}
-
-	if (list[0] == NOGEOM)
+	if (list[0] == NOGEOM && numtexts == 0)
 	{
 		us_abortcommand(_("First select objects to move"));
 		return;
@@ -1882,6 +1873,25 @@ void us_move(INTSML count, char *par[])
 	for(i=0; list[i] != NOGEOM; i++) if (list[i]->entrytype == OBJNODEINST)
 		list[i]->entryaddr.ni->temp1 = 1;
 
+	/* remove nodes listed as text objects */
+	for(i=0; i<numtexts; i++)
+	{
+		(void)us_makehighlight(textlist[i], &thishigh);
+		if (us_nodemoveswithtext(&thishigh) == 0) continue;
+		us_gethighaddrtype(&thishigh, &addr, &type);
+
+		/* undraw the text */
+		if (type == VPORTPROTO)
+		{
+			ni = ((PORTPROTO *)addr)->subnodeinst;
+			ni->temp1 = 0;
+		} else if (type == VNODEINST)
+		{
+			ni = (NODEINST *)addr;
+			ni->temp1 = 0;
+		}
+	}
+
 	/* count the number of nodes */
 	for(total=0, ni = np->firstnodeinst; ni != NONODEINST; ni = ni->nextnodeinst)
 	{
@@ -1889,16 +1899,26 @@ void us_move(INTSML count, char *par[])
 		if (us_canedit(np, ni->proto, 1) != 0) return;
 		total++;
 	}
-	if (total == 0) return;
+	if (total == 0 && numtexts == 0) return;
 
 	/* build a list that includes all nodes touching moved arcs */
-	nodelist = (NODEINST **)emalloc((total * (sizeof (NODEINST *))), el_tempcluster);
-	if (nodelist == 0) return;
-	for(i=0, ni = np->firstnodeinst; ni != NONODEINST; ni = ni->nextnodeinst)
-		if (ni->temp1 != 0) nodelist[i++] = ni;
+	if (total > 0)
+	{
+		nodelist = (NODEINST **)emalloc((total * (sizeof (NODEINST *))), el_tempcluster);
+		if (nodelist == 0) return;
+		for(i=0, ni = np->firstnodeinst; ni != NONODEINST; ni = ni->nextnodeinst)
+			if (ni->temp1 != 0) nodelist[i++] = ni;
+	}
 
 	/* figure out lower-left corner of this collection of objects */
-	us_getlowleft(nodelist[0], &bestlx, &bestly);
+	if (total == 0)
+	{
+		bestlx = (np->lowx + np->highx) / 2;
+		bestly = (np->lowy + np->highy) / 2;
+	} else
+	{
+		us_getlowleft(nodelist[0], &bestlx, &bestly);
+	}
 	for(i=1; i<total; i++)
 	{
 		us_getlowleft(nodelist[i], &lx, &ly);
@@ -1916,7 +1936,7 @@ void us_move(INTSML count, char *par[])
 	}
 
 	/* special case when moving one node: account for facet center */
-	if (total == 1 && list[1] == NOGEOM)
+	if (total == 1 && list[1] == NOGEOM && numtexts == 0)
 	{
 		/* get standard corner offset */
 		ni = nodelist[0];
@@ -2002,8 +2022,11 @@ void us_move(INTSML count, char *par[])
 		gridalign(&xcur, &ycur, us_alignment);
 
 		/* make the move if it is valid */
-		if (xcur == bestlx && ycur == bestly) ttyputverbose(_("Null motion")); else
+		if (xcur == bestlx && ycur == bestly) ttyputverbose(M_("Null motion")); else
+		{
 			us_manymove(list, nodelist, total, xcur-bestlx, ycur-bestly);
+			us_moveselectedtext(numtexts, textlist, xcur-bestlx, ycur-bestly);
+		}
 		efree((char *)nodelist);
 		(void)us_pophighlight(1);
 		return;
@@ -2018,16 +2041,19 @@ void us_move(INTSML count, char *par[])
 		if (count == 2 && namesamen(par[1], "dialog", strlen(par[1])) == 0)
 		{
 			/* get coordinates from dialog */
+			us_movetodialogitems[DMVT_XPOS_L-1].msg = _("X:");
+			us_movetodialogitems[DMVT_YPOS_L-1].msg = _("Y:");
+			us_movetodialog.movable = _("Move To Location");
 			if (DiaInitDialog(&us_movetodialog) != 0) return;
-			DiaSetText(3, latoa(bestlx));
-			DiaSetText(5, latoa(bestly));
+			DiaSetText(DMVT_XPOS, latoa(bestlx));
+			DiaSetText(DMVT_YPOS, latoa(bestly));
 			for(;;)
 			{
 				itemHit = DiaNextHit();
 				if (itemHit == CANCEL || itemHit == OK) break;
 			}
-			xcur = atola(DiaGetText(3));
-			ycur = atola(DiaGetText(5));
+			xcur = atola(DiaGetText(DMVT_XPOS));
+			ycur = atola(DiaGetText(DMVT_YPOS));
 			DiaDoneDialog();
 			if (itemHit == CANCEL) return;
 		} else
@@ -2045,8 +2071,58 @@ void us_move(INTSML count, char *par[])
 		}
 
 		/* make the move if it is valid */
-		if (xcur == bestlx && ycur == bestly) ttyputverbose(_("Null motion")); else
+		if (xcur == bestlx && ycur == bestly) ttyputverbose(M_("Null motion")); else
+		{
 			us_manymove(list, nodelist, total, xcur-bestlx, ycur-bestly);
+			us_moveselectedtext(numtexts, textlist, xcur-bestlx, ycur-bestly);
+		}
+		efree((char *)nodelist);
+		(void)us_pophighlight(1);
+		return;
+	}
+
+	/* handle relative motion option "move by X Y" */
+	if (namesamen(pp, "by", l) == 0 && l >= 1)
+	{
+		/* get absolute position to place object */
+		if (count == 2 && namesamen(par[1], "dialog", strlen(par[1])) == 0)
+		{
+			/* get coordinates from dialog */
+			us_movetodialogitems[DMVT_XPOS_L-1].msg = _("dX:");
+			us_movetodialogitems[DMVT_YPOS_L-1].msg = _("dY:");
+			us_movetodialog.movable = _("Move By Amount");
+			if (DiaInitDialog(&us_movetodialog) != 0) return;
+			DiaSetText(DMVT_XPOS, "0");
+			DiaSetText(DMVT_YPOS, "0");
+			for(;;)
+			{
+				itemHit = DiaNextHit();
+				if (itemHit == CANCEL || itemHit == OK) break;
+			}
+			xcur = atola(DiaGetText(DMVT_XPOS));
+			ycur = atola(DiaGetText(DMVT_YPOS));
+			DiaDoneDialog();
+			if (itemHit == CANCEL) return;
+		} else
+		{
+			/* get coordinates from command line */
+			if (count < 3)
+			{
+				ttyputusage("move by X Y");
+				efree((char *)nodelist);
+				(void)us_pophighlight(0);
+				return;
+			}
+			xcur = atola(par[1]);
+			ycur = atola(par[2]);
+		}
+
+		/* make the move if it is valid */
+		if (xcur == 0 && ycur == 0) ttyputverbose(M_("Null motion")); else
+		{
+			us_manymove(list, nodelist, total, xcur, ycur);
+			us_moveselectedtext(numtexts, textlist, xcur-bestlx, ycur-bestly);
+		}
 		efree((char *)nodelist);
 		(void)us_pophighlight(1);
 		return;
@@ -2094,8 +2170,11 @@ void us_move(INTSML count, char *par[])
 		us_getslide(ang, bestlx, bestly, xcur, ycur, &dx, &dy);
 
 		/* make the move if it is nonnull */
-		if (dx == bestlx && dy == bestly) ttyputverbose(_("Null motion")); else
+		if (dx == bestlx && dy == bestly) ttyputverbose(M_("Null motion")); else
+		{
 			us_manymove(list, nodelist, total, dx-bestlx, dy-bestly);
+			us_moveselectedtext(numtexts, textlist, xcur-bestlx, ycur-bestly);
+		}
 		efree((char *)nodelist);
 		(void)us_pophighlight(1);
 		return;
@@ -2152,8 +2231,12 @@ void us_move(INTSML count, char *par[])
 	}
 
 	/* now move the object */
-	if (dx != 0 || dy != 0) us_manymove(list, nodelist, total, dx, dy);
-	efree((char *)nodelist);
+	if (dx != 0 || dy != 0)
+	{
+		us_manymove(list, nodelist, total, dx, dy);
+		us_moveselectedtext(numtexts, textlist, dx, dy);
+	}
+	if (total > 0) efree((char *)nodelist);
 	(void)us_pophighlight(1);
 }
 
@@ -2179,7 +2262,7 @@ void us_node(INTSML count, char *par[])
 
 	if (count == 0)
 	{
-		count = ttygetparam(_("Node option: "), &us_nodep, MAXPARS, par);
+		count = ttygetparam(M_("Node option: "), &us_nodep, MAXPARS, par);
 		if (count == 0)
 		{
 			us_abortedmsg();
@@ -2199,8 +2282,8 @@ void us_node(INTSML count, char *par[])
 
 	if (namesamen(pt, "expand", l) == 0 && l >= 1)
 	{
-		if (count < 2) amt = HUGEINT; else amt = atoi(par[1]);
-		list = us_gethighlighted(OBJNODEINST);
+		if (count < 2) amt = MAXINTBIG; else amt = atoi(par[1]);
+		list = us_gethighlighted(OBJNODEINST, 0, 0);
 		if (list[0] == NOGEOM)
 		{
 			us_abortcommand(_("Select nodes to be expanded"));
@@ -2268,12 +2351,17 @@ void us_node(INTSML count, char *par[])
 		if (negated != 0) (void)delvalkey((INTBIG)ni, VNODEINST, el_node_name); else
 		{
 			var = setvalkey((INTBIG)ni, VNODEINST, el_node_name, (INTBIG)ch, VSTRING|VDISPLAY);
-
-			/* shift text down if on a facet instance */
-			if (var != NOVARIABLE && ni->proto->primindex == 0)
+			if (var != NOVARIABLE)
 			{
-				var->textdescript = us_setdescriptoffset(var->textdescript,
-					0, (ni->highy-ni->lowy) / el_curlib->lambda[el_curtech->techindex]);
+				var->textdescript = (var->textdescript & ~VTSIZE) |
+					(defaulttextsize(3) << VTSIZESH);
+
+				/* shift text down if on a facet instance */
+				if (ni->proto->primindex == 0)
+				{
+					var->textdescript = us_setdescriptoffset(var->textdescript,
+						0, (ni->highy-ni->lowy) / el_curlib->lambda[el_curtech->techindex]);
+				}
 			}
 		}
 		endobjectchange((INTBIG)ni, VNODEINST);
@@ -2282,8 +2370,8 @@ void us_node(INTSML count, char *par[])
 		(void)us_pophighlight(0);
 
 		/* report results and quit */
-		if (negated != 0) ttyputverbose(_("Node name removed")); else
-			ttyputverbose(_("Node name is '%s'"), ch);
+		if (negated != 0) ttyputverbose(M_("Node name removed")); else
+			ttyputverbose(M_("Node name is '%s'"), ch);
 		return;
 	}
 
@@ -2318,7 +2406,7 @@ void us_node(INTSML count, char *par[])
 				ttyputusage("node trace place-text LAYER FONT SCALE MESSAGE");
 				return;
 			}
-			us_layouttext(par[2], par[3], atoi(par[4]), par[5]);
+			us_layouttext(par[2], atoi(par[3]), atoi(par[4]), par[5]);
 			return;
 		}
 
@@ -2384,7 +2472,7 @@ void us_node(INTSML count, char *par[])
 			/* save highlighting, set trace, restore highlighting */
 			us_pushhighlight();
 			us_clearhighlightcount();
-			us_settrace(store, newlist, (INTSML)((segs+1)*2));
+			us_settrace(store, newlist, (segs+1)*2);
 			(void)us_pophighlight(0);
 			efree((char *)newlist);
 			return;
@@ -2650,7 +2738,7 @@ void us_node(INTSML count, char *par[])
 				}
 
 				/* now re-draw this trace */
-				us_settrace(store, newlist, (INTSML)(size+1));
+				us_settrace(store, newlist, size+1);
 				whichpoint++;
 				efree((char *)newlist);
 			}
@@ -2708,7 +2796,7 @@ void us_node(INTSML count, char *par[])
 			us_clearhighlightcount();
 
 			/* now re-draw this trace */
-			us_settrace(store, newlist, (INTSML)(size-total));
+			us_settrace(store, newlist, size-total);
 			whichpoint = (INTSML)maxi(whichpoint-total, 1);
 			efree((char *)newlist);
 
