@@ -227,7 +227,7 @@ void us_package(INTSML count, char *par[])
 
 	if (count == 0)
 	{
-		count = ttygetparam(_("New facet name: "), &us_packagep, MAXPARS, par);
+		count = ttygetparam(M_("New facet name: "), &us_packagep, MAXPARS, par);
 		if (count == 0)
 		{
 			us_abortedmsg();
@@ -303,7 +303,8 @@ void us_package(INTSML count, char *par[])
 			if (ppt->subnodeinst != ni) continue;
 			npt = newportproto(np, newni, ppt->subportproto, ppt->protoname);
 			npt->userbits = (npt->userbits & ~STATEBITS) | (ppt->userbits & STATEBITS);
-			npt->textdescript = ppt->textdescript;
+			npt->textdescript = (ppt->textdescript & ~VTSIZE) |
+				(defaulttextsize(1) << VTSIZESH);
 			if (copyvars((INTBIG)ppt, VPORTPROTO, (INTBIG)npt, VPORTPROTO) != 0)
 			{
 				ttyputnomemory();
@@ -350,6 +351,7 @@ void us_package(INTSML count, char *par[])
 
 void us_port(INTSML count, char *par[])
 {
+	REGISTER INTBIG i, l, labels, total, reducehighlighting;
 	INTBIG lx, hx, ly, hy, mlx, mly, mhx, mhy, bits, mask, newbits, x, y, deleted;
 	REGISTER NODEINST *ni, *oni;
 	REGISTER NODEPROTO *np;
@@ -360,7 +362,6 @@ void us_port(INTSML count, char *par[])
 	GEOM *fromgeom, *togeom;
 	PORTPROTO *fromport, *toport;
 	REGISTER char *pt;
-	REGISTER INTSML i, l, labels, total, reducehighlighting;
 	char *portname;
 	REGISTER HIGHLIGHT *high;
 	HIGHLIGHT newhigh;
@@ -371,7 +372,7 @@ void us_port(INTSML count, char *par[])
 	/* get options */
 	if (count == 0)
 	{
-		count = ttygetparam(_("Port option: "), &us_portp, MAXPARS, par);
+		count = ttygetparam(M_("Port option: "), &us_portp, MAXPARS, par);
 		if (count == 0)
 		{
 			us_abortedmsg();
@@ -398,10 +399,36 @@ void us_port(INTSML count, char *par[])
 		}
 		switch (us_useroptions&PORTLABELS)
 		{
-			case PORTSOFF:   ttyputverbose(_("No ports are indicated"));     break;
-			case PORTSCROSS: ttyputverbose(_("Ports shown as crosses"));     break;
-			case PORTSFULL:  ttyputverbose(_("Long port names displayed"));  break;
-			case PORTSSHORT: ttyputverbose(_("Short port names displayed")); break;
+			case PORTSOFF:   ttyputverbose(M_("No ports are indicated"));     break;
+			case PORTSCROSS: ttyputverbose(M_("Ports shown as crosses"));     break;
+			case PORTSFULL:  ttyputverbose(M_("Long port names displayed"));  break;
+			case PORTSSHORT: ttyputverbose(M_("Short port names displayed")); break;
+		}
+		return;
+	}
+
+	if (namesamen(pt, "export-labels", l) == 0 && l >= 8)
+	{
+		if (count > 1)
+		{
+			l = strlen(pt = par[1]);
+			if (namesamen(pt, "none", l) == 0 && l >= 2) labels = EXPORTSOFF; else
+			if (namesamen(pt, "crosses", l) == 0 && l >= 1) labels = EXPORTSCROSS; else
+			if (namesamen(pt, "short", l) == 0 && l >= 2) labels = EXPORTSSHORT; else
+			if (namesamen(pt, "long", l) == 0 && l >= 1) labels = EXPORTSFULL; else
+			{
+				ttyputusage("port export-labels none|short|long|crosses");
+				return;
+			}
+			(void)setvalkey((INTBIG)us_aid, VAID, us_optionflags,
+				(us_useroptions & ~EXPORTLABELS) | labels, VINTEGER);
+		}
+		switch (us_useroptions&EXPORTLABELS)
+		{
+			case EXPORTSOFF:   ttyputverbose(M_("No exports are indicated"));     break;
+			case EXPORTSCROSS: ttyputverbose(M_("Exports shown as crosses"));     break;
+			case EXPORTSFULL:  ttyputverbose(M_("Long export names displayed"));  break;
+			case EXPORTSSHORT: ttyputverbose(M_("Short export names displayed")); break;
 		}
 		return;
 	}
@@ -428,17 +455,31 @@ void us_port(INTSML count, char *par[])
 		return;
 	}
 
-	if (namesamen(pt, "identify", l) == 0 && l >= 1)
+	if (namesamen(pt, "identify-facet", l) == 0 && l >= 10)
 	{
 		/* make sure there is a current facet */
 		np = us_needfacet();
 		if (np == NONODEPROTO) return;
 		if ((el_curwindowpart->state&WINDOWTYPE) == DISP3DWINDOW)
+		{
+			us_abortcommand(_("Cannot show exports in a 3D window"));
+			return;
+		}
+		us_identifyports(NONODEINST, np, LAYERA, 0);
+		return;
+	}
+
+	if (namesamen(pt, "identify-node", l) == 0 && l >= 10)
+	{
+		/* make sure there is a current node */
+		ni = (NODEINST *)us_getobject(OBJNODEINST, 0);
+		if (ni == NONODEINST) return;
+		if ((el_curwindowpart->state&WINDOWTYPE) == DISP3DWINDOW)
 		{
 			us_abortcommand(_("Cannot show ports in a 3D window"));
 			return;
 		}
-		us_identifyports(NONODEINST, np, LAYERA);
+		us_identifyports(ni, ni->proto, LAYERA, 0);
 		return;
 	}
 
@@ -504,7 +545,7 @@ void us_port(INTSML count, char *par[])
 		{
 			endobjectchange((INTBIG)ni, VNODEINST);
 			endobjectchange((INTBIG)oni, VNODEINST);
-			ttyputverbose(_("Port %s moved from node %s to node %s"), pt, describenodeinst(oni),
+			ttyputverbose(M_("Port %s moved from node %s to node %s"), pt, describenodeinst(oni),
 				describenodeinst(ni));
 		}
 
@@ -587,7 +628,7 @@ void us_port(INTSML count, char *par[])
 		if (us_canedit(np, NONODEPROTO, 1) != 0) return;
 
 		/* get list of nodes in the selected area */
-		g = us_gethighlighted(OBJNODEINST);
+		g = us_gethighlighted(OBJNODEINST, 0, 0);
 		if (g[0] == NOGEOM)
 		{
 			us_abortcommand(_("Must select some nodes for making exports"));
@@ -691,7 +732,7 @@ void us_port(INTSML count, char *par[])
 		}
 		if (count == 3 && namesame(par[2], "all") == 0)
 		{
-			g = us_gethighlighted(OBJNODEINST);
+			g = us_gethighlighted(OBJNODEINST, 0, 0);
 			if (g[0] == NOGEOM)
 			{
 				us_abortcommand(_("Must first select nodes with exports"));
@@ -773,7 +814,7 @@ void us_port(INTSML count, char *par[])
 		/* restore highlighting */
 		(void)us_pophighlight(0);
 
-		ttyputverbose(_("Port %s deleted"), pp->protoname);
+		ttyputverbose(M_("Port %s deleted"), pp->protoname);
 		return;
 	}
 
@@ -787,7 +828,7 @@ void us_port(INTSML count, char *par[])
 
 		if (count <= 1)
 		{
-			count = ttygetparam(_("Port name: "), &us_portep, MAXPARS-1, &par[1])+1;
+			count = ttygetparam(M_("Port name: "), &us_portep, MAXPARS-1, &par[1])+1;
 			if (count <= 1)
 			{
 				us_abortedmsg();
@@ -823,7 +864,7 @@ void us_port(INTSML count, char *par[])
 		/* restore highlighting */
 		(void)us_pophighlight(0);
 
-		ttyputverbose(_("Port %s created"), pp->protoname);
+		ttyputverbose(M_("Port %s created"), pp->protoname);
 		return;
 	}
 
@@ -872,7 +913,7 @@ void us_port(INTSML count, char *par[])
 		if ((pp->userbits&STATEBITS) == 0)
 			(void)addstringtoinfstr(_(" has no characteristics")); else
 		{
-			(void)formatinfstr(_(" is %s"), us_describeportbits(pp));
+			(void)formatinfstr(_(" is %s"), describeportbits(pp));
 		}
 		(void)formatinfstr(_(" (label %s"), us_describetextdescript(pp->textdescript));
 		if ((pp->userbits&PORTDRAWN) != 0) (void)addstringtoinfstr(_(",always-drawn"));
