@@ -81,7 +81,7 @@ void us_technology(INTSML count, char *par[])
 	/* ensure there is a technology option */
 	if (count == 0)
 	{
-		count = ttygetparam(_("Technology option: "), &us_technologyp, MAXPARS, par);
+		count = ttygetparam(M_("Technology option: "), &us_technologyp, MAXPARS, par);
 		if (count == 0)
 		{
 			us_abortedmsg();
@@ -109,14 +109,14 @@ void us_technology(INTSML count, char *par[])
 		{
 			(void)setvalkey((INTBIG)us_aid, VAID, us_optionflags,
 				us_useroptions | AUTOSWITCHTECHNOLOGY, VINTEGER);
-			ttyputverbose(_("Technology will automatically switch to match facet"));
+			ttyputverbose(M_("Technology will automatically switch to match facet"));
 			return;
 		}
 		if (namesamen(pp, "off", l) == 0 && l >= 2)
 		{
 			(void)setvalkey((INTBIG)us_aid, VAID, us_optionflags,
 				us_useroptions & ~AUTOSWITCHTECHNOLOGY, VINTEGER);
-			ttyputverbose(_("Automatic technology changing disabled"));
+			ttyputverbose(M_("Automatic technology changing disabled"));
 			return;
 		}
 		ttyputbadusage("technology autoswitch");
@@ -161,11 +161,11 @@ void us_technology(INTSML count, char *par[])
 	{
 		if (el_curtech == tech)
 		{
-			ttyputverbose(_("Already in %s technology"), el_curtech->techname);
+			ttyputverbose(M_("Already in %s technology"), el_curtech->techname);
 			return;
 		}
 
-		ttyputverbose(_("Switching to %s"), tech->techdescript);
+		ttyputverbose(M_("Switching to %s"), tech->techdescript);
 		us_setnodeproto(NONODEPROTO);
 		us_setarcproto(NOARCPROTO, 1);
 		oldlam = el_curlib->lambda[el_curtech->techindex];
@@ -260,7 +260,7 @@ void us_tellaid(INTSML count, char *par[])
 
 	if (count == 0)
 	{
-		count = ttygetparam(_("Which aid: "), &us_tellaidp, MAXPARS, par);
+		count = ttygetparam(M_("Which aid: "), &us_tellaidp, MAXPARS, par);
 		if (count == 0)
 		{
 			us_abortedmsg();
@@ -302,7 +302,7 @@ void us_terminal(INTSML count, char *par[])
 		us_colorhighp, us_bindgkeyp, us_gridp, us_technologytp, us_nodetcaip,
 		us_librarykp, us_findobjap, us_window3dp, us_nodetptlp, us_renamecp,
 		us_findexportp, us_findnnamep, us_showup, us_varop, us_copyfacetdp,
-		us_findnodep, us_technologycnnp, us_showep;
+		us_findnodep, us_technologycnnp, us_showep, us_textfp, us_textsp, us_viewdp;
 
 	REGISTER COMCOMP *comcomp;
 	float newfloat;
@@ -325,44 +325,44 @@ void us_terminal(INTSML count, char *par[])
 		if (namesame(pp, "lock-keys-on-error") == 0)
 		{
 			(void)setval((INTBIG)us_aid, VAID, "aidstate", us_aid->aidstate | NOKEYLOCK, VINTEGER);
-			ttyputverbose(_("Command errors will not lockout single-key commands"));
+			ttyputverbose(M_("Command errors will not lockout single-key commands"));
 			return;
 		}
 		if (namesame(pp, "only-informative-messages") == 0)
 		{
 			(void)setval((INTBIG)us_aid, VAID, "aidstate", us_aid->aidstate & ~JUSTTHEFACTS, VINTEGER);
-			ttyputverbose(_("All messages will be displayed"));
+			ttyputverbose(M_("All messages will be displayed"));
 			return;
 		}
 		if (namesame(pp, "use-electric-commands") == 0)
 		{
 			(void)setval((INTBIG)us_aid, VAID, "aidstate", us_aid->aidstate | NODETAILS, VINTEGER);
-			ttyputverbose(_("Messages will not mention specific commands"));
+			ttyputverbose(M_("Messages will not mention specific commands"));
 			return;
 		}
 		if (namesame(pp, "display-dialogs") == 0)
 		{
 			(void)setval((INTBIG)us_aid, VAID, "aidstate", us_aid->aidstate & ~USEDIALOGS, VINTEGER);
-			ttyputverbose(_("Dialogs will not be used"));
+			ttyputverbose(M_("Dialogs will not be used"));
 			return;
 		}
 		if (namesame(pp, "permanent-menu-highlighting") == 0)
 		{
 			(void)setval((INTBIG)us_aid, VAID, "aidstate", us_aid->aidstate | ONESHOTMENU, VINTEGER);
-			ttyputverbose(_("Menu highlighting will clear after each command"));
+			ttyputverbose(M_("Menu highlighting will clear after each command"));
 			return;
 		}
 		if (namesame(pp, "track-cursor-coordinates") == 0)
 		{
 			(void)setval((INTBIG)us_aid, VAID, "aidstate", us_aid->aidstate & ~SHOWXY, VINTEGER);
-			ttyputverbose(_("Technology and Lambda will be displayed where cursor coordinates are"));
+			ttyputverbose(M_("Technology and Lambda will be displayed where cursor coordinates are"));
 			us_redostatus(NOWINDOWFRAME);
 			return;
 		}
 		if (namesame(pp, "beep") == 0)
 		{
 			(void)setval((INTBIG)us_aid, VAID, "aidstate", us_aid->aidstate & ~TERMBEEP, VINTEGER);
-			ttyputverbose(_("Terminal beep disabled"));
+			ttyputverbose(M_("Terminal beep disabled"));
 			return;
 		}
 		if (namesamen(pp, "audit", l) == 0)
@@ -370,7 +370,7 @@ void us_terminal(INTSML count, char *par[])
 			if (us_termaudit != NULL) xclose(us_termaudit);
 			us_termaudit = NULL;
 			(void)setval((INTBIG)us_aid, VAID, "aidstate", us_aid->aidstate & ~TTYAUDIT, VINTEGER);
-			ttyputverbose(_("Terminal auditing off"));
+			ttyputverbose(M_("No longer saving messages"));
 			return;
 		}
 		ttyputbadusage("terminal not");
@@ -380,35 +380,35 @@ void us_terminal(INTSML count, char *par[])
 	if (namesamen(pp, "lock-keys-on-error", l) == 0 && l >= 2)
 	{
 		(void)setval((INTBIG)us_aid, VAID, "aidstate", us_aid->aidstate & ~NOKEYLOCK, VINTEGER);
-		ttyputverbose(_("Command errors will lockout single-key commands"));
+		ttyputverbose(M_("Command errors will lockout single-key commands"));
 		return;
 	}
 
 	if (namesamen(pp, "only-informative-messages", l) == 0)
 	{
 		(void)setval((INTBIG)us_aid, VAID, "aidstate", us_aid->aidstate | JUSTTHEFACTS, VINTEGER);
-		ttyputverbose(_("Nonessential messages will be supressed"));
+		ttyputverbose(M_("Nonessential messages will be supressed"));
 		return;
 	}
 
 	if (namesamen(pp, "display-dialogs", l) == 0)
 	{
 		(void)setval((INTBIG)us_aid, VAID, "aidstate", us_aid->aidstate | USEDIALOGS, VINTEGER);
-		ttyputverbose(_("Dialogs will be use where appropriate"));
+		ttyputverbose(M_("Dialogs will be use where appropriate"));
 		return;
 	}
 
 	if (namesamen(pp, "permanent-menu-highlighting", l) == 0)
 	{
 		(void)setval((INTBIG)us_aid, VAID, "aidstate", us_aid->aidstate & ~ONESHOTMENU, VINTEGER);
-		ttyputverbose(_("Menu highlighting will show current node/arc"));
+		ttyputverbose(M_("Menu highlighting will show current node/arc"));
 		return;
 	}
 
 	if (namesame(pp, "track-cursor-coordinates") == 0)
 	{
 		(void)setval((INTBIG)us_aid, VAID, "aidstate", us_aid->aidstate | SHOWXY, VINTEGER);
-		ttyputverbose(_("Cursor coordinates will be displayed where TECH/LAMBDA are"));
+		ttyputverbose(M_("Cursor coordinates will be displayed where TECH/LAMBDA are"));
 		us_redostatus(NOWINDOWFRAME);
 		return;
 	}
@@ -416,26 +416,26 @@ void us_terminal(INTSML count, char *par[])
 	if (namesamen(pp, "use-electric-commands", l) == 0)
 	{
 		(void)setval((INTBIG)us_aid, VAID, "aidstate", us_aid->aidstate & ~NODETAILS, VINTEGER);
-		ttyputverbose(_("Messages will use specific Electric commands"));
+		ttyputverbose(M_("Messages will use specific Electric commands"));
 		return;
 	}
 	if (namesamen(pp, "beep", l) == 0)
 	{
 		(void)setval((INTBIG)us_aid, VAID, "aidstate", us_aid->aidstate | TERMBEEP, VINTEGER);
-		ttyputverbose(_("Terminal beep enabled"));
+		ttyputverbose(M_("Terminal beep enabled"));
 		return;
 	}
 
 	if (namesamen(pp, "audit", l) == 0)
 	{
-		us_termaudit = xcreate("messages.txt", el_filetypetext, 0, &truename);
+		us_termaudit = xcreate("emessages.txt", el_filetypetext, 0, &truename);
 		if (us_termaudit == 0)
 		{
-			ttyputerr(_("Cannot create 'messages.txt'"));
+			ttyputerr(_("Cannot create 'emessages.txt'"));
 			return;
 		}
+		ttyputmsg(_("Saving messages to '%s'"), truename);
 		(void)setval((INTBIG)us_aid, VAID, "aidstate", us_aid->aidstate | TTYAUDIT, VINTEGER);
-		ttyputmsg(_("Terminal auditing on (saving to %s)"), truename);
 		return;
 	}
 
@@ -487,6 +487,7 @@ void us_terminal(INTSML count, char *par[])
 			if (namesame(par[3], "defnode") == 0)       comcomp = &us_defnodexsp; else
 			if (namesame(par[3], "deftext") == 0)       comcomp = &us_textdsp; else
 			if (namesame(par[3], "delfacet") == 0)      comcomp = &us_showup; else
+			if (namesame(par[3], "delview") == 0)       comcomp = &us_viewdp; else
 			if (namesame(par[3], "dependentlibs") == 0) comcomp = &us_technologyedlp; else
 			if (namesame(par[3], "depth3d") == 0)       comcomp = &us_window3dp; else
 			if (namesame(par[3], "editfacet") == 0)     comcomp = &us_editfacetp; else
@@ -496,6 +497,7 @@ void us_terminal(INTSML count, char *par[])
 			if (namesame(par[3], "facetinfo") == 0)     comcomp = &us_defnodep; else
 			if (namesame(par[3], "facet") == 0)         comcomp = &us_showdp; else
 			if (namesame(par[3], "file") == 0)          comcomp = &us_colorreadp; else
+			if (namesame(par[3], "find") == 0)          comcomp = &us_textfp; else
 			if (namesame(par[3], "frameopt") == 0)      comcomp = &us_viewfp; else
 			if (namesame(par[3], "gain") == 0)          comcomp = &us_technologyclsp; else
 			if (namesame(par[3], "grid") == 0)          comcomp = &us_gridp; else
@@ -534,6 +536,7 @@ void us_terminal(INTSML count, char *par[])
 			if (namesame(par[3], "technology") == 0)    comcomp = &us_technologyup; else
 			if (namesame(par[3], "techopt") == 0)       comcomp = &us_technologytp; else
 			if (namesame(par[3], "techvars") == 0)      comcomp = &us_technologyctdp; else
+			if (namesame(par[3], "textsize") == 0)      comcomp = &us_textsp; else
 			if (namesame(par[3], "trace") == 0)         comcomp = &us_nodetp; else
 			if (namesame(par[3], "variable") == 0)      comcomp = &us_varvep; else
 			if (namesame(par[3], "view") == 0)          comcomp = &us_viewc2p; else
@@ -639,7 +642,7 @@ void us_terminal(INTSML count, char *par[])
 				return;
 			}
 			us_logflushfreq = (INTSML)myatoi(par[2]);
-			ttyputverbose(_("Session logging will be guaranteed every %d commands"), us_logflushfreq);
+			ttyputverbose(M_("Session logging will be guaranteed every %d commands"), us_logflushfreq);
 			return;
 		}
 
@@ -797,7 +800,7 @@ void us_text(INTSML count, char *par[])
 			return;
 		}
 
-		us_searchtext(el_curwindowpart, par[1], 0);
+		us_searchtext(el_curwindowpart, par[1], 0, 0);
 		return;
 	}
 
@@ -826,7 +829,7 @@ void us_text(INTSML count, char *par[])
 			us_abortcommand(_("Unknown editor: %s"), pp);
 			return;
 		}
-		ttyputverbose(_("Now using %s editor for text"), us_editortable[i].editorname);
+		ttyputverbose(M_("Now using %s editor for text"), us_editortable[i].editorname);
 		setvalkey((INTBIG)us_aid, VAID, us_text_editor, (INTBIG)us_editortable[i].editorname,
 			VSTRING);
 		return;
@@ -835,9 +838,9 @@ void us_text(INTSML count, char *par[])
 	if (namesamen(pp, "easy-text-selection", l) == 0 && l >= 2)
 	{
 		if ((us_useroptions&NOTEXTSELECT) == 0)
-			ttyputverbose(_("Annotation text is already easy to select")); else
+			ttyputverbose(M_("Annotation text is already easy to select")); else
 		{
-			ttyputverbose(_("Annotation text will be easily selectable"));
+			ttyputverbose(M_("Annotation text will be easily selectable"));
 			(void)setvalkey((INTBIG)us_aid, VAID, us_optionflags,
 				us_useroptions | NOTEXTSELECT, VINTEGER);
 		}
@@ -847,37 +850,102 @@ void us_text(INTSML count, char *par[])
 	if (namesamen(pp, "hard-text-selection", l) == 0 && l >= 1)
 	{
 		if ((us_useroptions&NOTEXTSELECT) != 0)
-			ttyputverbose(_("Annotation text is already hard to select")); else
+			ttyputverbose(M_("Annotation text is already hard to select")); else
 		{
-			ttyputverbose(_("Annotation text will be hard to select (only when the 'find' command does NOT have the 'port' option)"));
+			ttyputverbose(M_("Annotation text will be hard to select (only when the 'find' command does NOT have the 'port' option)"));
 			(void)setvalkey((INTBIG)us_aid, VAID, us_optionflags,
 				us_useroptions & ~NOTEXTSELECT, VINTEGER);
 		}
 		return;
 	}
 
-	/* handle default size */
-	if (namesamen(pp, "default-size", l) == 0 && l >= 9)
+	/* handle default export size */
+	if (namesamen(pp, "default-export-size", l) == 0 && l >= 11)
 	{
-		/* get current value */
-		var = getval((INTBIG)us_aid, VAID, VINTEGER, "USER_default_text_size");
-		if (var == NOVARIABLE) font = TXTSMALL; else font = var->addr;
-
-		/* if no argument provided, show current value */
+		/* get new value */
 		if (count < 2)
 		{
-			ttyputmsg(_("Default text size is %s"), us_describefont(font));
+			ttyputusage("text default-export-size SIZE");
 			return;
 		}
-
-		/* get new value */
-		l = strlen(pp = par[1]);
-		font = us_gettextsize(par[1], font);
+		font = us_gettextsize(par[1], defaulttextsize(1));
 		if (font < 0) return;
 
 		/* set the default text size */
-		setval((INTBIG)us_aid, VAID, "USER_default_text_size", font, VINTEGER|VDONTSAVE);
-		ttyputverbose(_("Default text size is now %s"), us_describefont(font));
+		setval((INTBIG)us_aid, VAID, "USER_default_export_text_size", font, VINTEGER|VDONTSAVE);
+		ttyputverbose(M_("Default export text size is now %s"), us_describefont(font));
+		return;
+	}
+
+	/* handle default node size */
+	if (namesamen(pp, "default-node-size", l) == 0 && l >= 9)
+	{
+		/* get new value */
+		if (count < 2)
+		{
+			ttyputusage("text default-node-size SIZE");
+			return;
+		}
+		font = us_gettextsize(par[1], defaulttextsize(3));
+		if (font < 0) return;
+
+		/* set the default text size */
+		setval((INTBIG)us_aid, VAID, "USER_default_node_text_size", font, VINTEGER|VDONTSAVE);
+		ttyputverbose(M_("Default node text size is now %s"), us_describefont(font));
+		return;
+	}
+
+	/* handle default arc size */
+	if (namesamen(pp, "default-arc-size", l) == 0 && l >= 9)
+	{
+		/* get new value */
+		if (count < 2)
+		{
+			ttyputusage("text default-arc-size SIZE");
+			return;
+		}
+		font = us_gettextsize(par[1], defaulttextsize(4));
+		if (font < 0) return;
+
+		/* set the default text size */
+		setval((INTBIG)us_aid, VAID, "USER_default_arc_text_size", font, VINTEGER|VDONTSAVE);
+		ttyputverbose(M_("Default arc text size is now %s"), us_describefont(font));
+		return;
+	}
+
+	/* handle default facet size */
+	if (namesamen(pp, "default-facet-size", l) == 0 && l >= 9)
+	{
+		/* get new value */
+		if (count < 2)
+		{
+			ttyputusage("text default-facet-size SIZE");
+			return;
+		}
+		font = us_gettextsize(par[1], defaulttextsize(5));
+		if (font < 0) return;
+
+		/* set the default text size */
+		setval((INTBIG)us_aid, VAID, "USER_default_facet_text_size", font, VINTEGER|VDONTSAVE);
+		ttyputverbose(M_("Default facet text size is now %s"), us_describefont(font));
+		return;
+	}
+
+	/* handle default nonlayout text size */
+	if (namesamen(pp, "default-nonlayout-text-size", l) == 0 && l >= 9)
+	{
+		/* get new value */
+		if (count < 2)
+		{
+			ttyputusage("text default-nonlayout-text-size SIZE");
+			return;
+		}
+		font = us_gettextsize(par[1], defaulttextsize(2));
+		if (font < 0) return;
+
+		/* set the default text size */
+		setval((INTBIG)us_aid, VAID, "USER_default_nonlayout_text_size", font, VINTEGER|VDONTSAVE);
+		ttyputverbose(M_("Default nonlayout text size is now %s"), us_describefont(font));
 		return;
 	}
 
@@ -891,10 +959,10 @@ void us_text(INTSML count, char *par[])
 		/* set the default interior style */
 		setval((INTBIG)us_aid, VAID, "USER_default_text_style",
 			grabpoint|VTINTERIOR, VINTEGER|VDONTSAVE);
-		ttyputverbose(_("Default text is only visible inside facet"));
+		ttyputverbose(M_("Default text is only visible inside facet"));
 		return;
 	}
-	if (namesamen(pp, "default-exterior", l) == 0 && l >= 9)
+	if (namesamen(pp, "default-exterior", l) == 0 && l >= 11)
 	{
 		/* get current value */
 		var = getval((INTBIG)us_aid, VAID, VINTEGER, "USER_default_text_style");
@@ -903,7 +971,7 @@ void us_text(INTSML count, char *par[])
 		/* set the default interior style */
 		setval((INTBIG)us_aid, VAID, "USER_default_text_style",
 			grabpoint & ~VTINTERIOR, VINTEGER|VDONTSAVE);
-		ttyputverbose(_("Default text is visible outside of facet"));
+		ttyputverbose(M_("Default text is visible outside of facet"));
 		return;
 	}
 
@@ -917,10 +985,10 @@ void us_text(INTSML count, char *par[])
 		/* if no argument provided, show current value */
 		if (count < 2)
 		{
-			ttyputmsg(_("Default text style is %s"), us_describestyle(grabpoint));
+			ttyputmsg(M_("Default text style is %s"), us_describestyle(grabpoint));
 			if ((grabpoint&VTINTERIOR) != 0)
-				ttyputmsg(_("Default text is only visible inside facet")); else
-					ttyputmsg(_("Default text is visible outside of facet"));
+				ttyputmsg(M_("Default text is only visible inside facet")); else
+					ttyputmsg(M_("Default text is visible outside of facet"));
 			return;
 		}
 
@@ -932,7 +1000,7 @@ void us_text(INTSML count, char *par[])
 		/* set the default text style */
 		setval((INTBIG)us_aid, VAID, "USER_default_text_style",
 			grabpoint, VINTEGER|VDONTSAVE);
-		ttyputverbose(_("Default text style is now %s"), us_describestyle(grabpoint));
+		ttyputverbose(M_("Default text style is now %s"), us_describestyle(grabpoint));
 		return;
 	}
 
@@ -946,7 +1014,7 @@ void us_text(INTSML count, char *par[])
 		/* if no argument provided, show current value */
 		if (count < 2)
 		{
-			ttyputmsg(_("Default horizontal text style is %s"),
+			ttyputmsg(M_("Default horizontal text style is %s"),
 				_(smartstyle[grabpoint&03]));
 			return;
 		}
@@ -963,7 +1031,7 @@ void us_text(INTSML count, char *par[])
 
 		/* set the default text style */
 		setval((INTBIG)us_aid, VAID, "USER_default_text_smart_style", grabpoint, VINTEGER|VDONTSAVE);
-		ttyputverbose(_("Default horizontal text style is now %s"), _(smartstyle[grabpoint&03]));
+		ttyputverbose(M_("Default horizontal text style is now %s"), _(smartstyle[grabpoint&03]));
 		return;
 	}
 	if (namesamen(pp, "default-vertical-style", l) == 0 && l >= 9)
@@ -975,7 +1043,7 @@ void us_text(INTSML count, char *par[])
 		/* if no argument provided, show current value */
 		if (count < 2)
 		{
-			ttyputmsg(_("Default vertical text style is %s"),
+			ttyputmsg(M_("Default vertical text style is %s"),
 				_(smartstyle[(grabpoint>>2)&03]));
 			return;
 		}
@@ -992,7 +1060,7 @@ void us_text(INTSML count, char *par[])
 
 		/* set the default text style */
 		setval((INTBIG)us_aid, VAID, "USER_default_text_smart_style", grabpoint, VINTEGER|VDONTSAVE);
-		ttyputverbose(_("Default vertical text style is now %s"), _(smartstyle[(grabpoint>>2)&03]));
+		ttyputverbose(M_("Default vertical text style is now %s"), _(smartstyle[(grabpoint>>2)&03]));
 		return;
 	}
 
@@ -1037,7 +1105,7 @@ void us_text(INTSML count, char *par[])
 			if (high->fromgeom->entrytype == OBJARCINST)
 				tech = high->fromgeom->entryaddr.ai->proto->tech; else
 					tech = high->fromgeom->entryaddr.ni->proto->tech;
-			screensettextsize(el_curwindowpart, truefontsize((INTSML)((olddescript&VTSIZE)>>VTSIZESH),
+			screensettextsize(el_curwindowpart, truefontsize((olddescript&VTSIZE)>>VTSIZESH,
 				el_curwindowpart, tech));
 			screengettextsize(el_curwindowpart, str, &tsx, &tsy);
 			xw = muldiv(tsx, el_curwindowpart->screenhx-el_curwindowpart->screenlx,
@@ -1212,7 +1280,7 @@ void us_undo(INTSML count, char *par[])
 			return;
 		}
 		j = historylistsize(i);
-		ttyputverbose(_("History list size changed from %d to %d entries"), j, i);
+		ttyputverbose(M_("History list size changed from %d to %d entries"), j, i);
 		return;
 	}
 
@@ -1362,9 +1430,9 @@ INTSML us_patternmatchhere(char *pattern, char *testcase)
 void us_var(INTSML count, char *par[])
 {
 	REGISTER INTSML l, maxleng, annotate, negated, inplace;
-	REGISTER INTBIG i, j, k, function, newval, newtype, search, oldaddr, language,
+	REGISTER INTBIG i, j, k, function, newval, search, oldaddr, language,
 		*newarray, oldlen, newdescript, len1, len2, count1, count2, count3;
-	INTBIG objaddr, objaddr1, objaddr2, objtype, objtype1, objtype2;
+	INTBIG objaddr, objaddr1, objaddr2, objtype, objtype1, objtype2, newtype, newaddr;
 	INTSML aindex, dummy, comvar, but, x, y;
 	REGISTER VARIABLE *var, *tvar, *res, *var1, *var2;
 	static VARIABLE fvar;
@@ -1420,7 +1488,7 @@ void us_var(INTSML count, char *par[])
 		{
 			(void)setvalkey((INTBIG)us_aid, VAID, us_ignoreoptionchanges, 1,
 				VINTEGER|VDONTSAVE);
-			ttyputverbose(_("Option changes are being ignored"));
+			ttyputverbose(M_("Option changes are being ignored"));
 			return;
 		}
 		if (namesamen(pp, "track", l) == 0)
@@ -1428,7 +1496,7 @@ void us_var(INTSML count, char *par[])
 			var = getvalkey((INTBIG)us_aid, VAID, VINTEGER, us_ignoreoptionchanges);
 			if (var != NOVARIABLE)
 				delvalkey((INTBIG)us_aid, VAID, us_ignoreoptionchanges);
-			ttyputverbose(_("Option changes are being tracked"));
+			ttyputverbose(M_("Option changes are being tracked"));
 			return;
 		}
 		ttyputusage("var options [ignore | track]");
@@ -1437,7 +1505,7 @@ void us_var(INTSML count, char *par[])
 
 	if (namesamen(pp, "reinherit", l) == 0 && l >= 1)
 	{
-		list = us_gethighlighted(OBJNODEINST);
+		list = us_gethighlighted(OBJNODEINST, 0, 0);
 		if (list[0] == NOGEOM)
 		{
 			us_abortcommand(_("Must select nodes for reinheriting attributes"));
@@ -1458,7 +1526,7 @@ void us_var(INTSML count, char *par[])
 	{
 		if (count <= 1)
 		{
-			count = ttygetparam(_("Variable: "), &us_varvep, MAXPARS-1, &par[1]) + 1;
+			count = ttygetparam(M_("Variable: "), &us_varvep, MAXPARS-1, &par[1]) + 1;
 			if (count == 1)
 			{
 				us_abortedmsg();
@@ -1701,7 +1769,7 @@ void us_var(INTSML count, char *par[])
 	{
 		if (count <= 1)
 		{
-			count = ttygetparam(_("Variable: "), &us_varvep, MAXPARS-1, &par[1]) + 1;
+			count = ttygetparam(M_("Variable: "), &us_varvep, MAXPARS-1, &par[1]) + 1;
 			if (count == 1)
 			{
 				us_abortedmsg();
@@ -1752,7 +1820,7 @@ void us_var(INTSML count, char *par[])
 					us_abortcommand(_("Cannot create %s on the object"), qual);
 					return;
 				}
-				ttyputverbose(_("Creating %s on the object"), qual);
+				ttyputverbose(M_("Creating %s on the object"), qual);
 			}
 			fvar.addr = var->addr;   fvar.type = var->type;
 			fvar.key = var->key;     fvar.textdescript = var->textdescript;
@@ -1827,7 +1895,7 @@ void us_var(INTSML count, char *par[])
 			/* loop on input */
 			oldw = el_curwindowpart;
 			el_curwindowpart = wpop;
-			ttyputverbose(_("You are typing to the editor"));
+			ttyputverbose(M_("You are typing to the editor"));
 			for(;;)
 			{
 				if (ttydataready() != 0 && wpop->charhandler != 0)
@@ -1954,8 +2022,18 @@ void us_var(INTSML count, char *par[])
 			if (namesame(us_varqualsave, "USER_color_map") == 0) annotate = 3;
 
 			/* these variables correspond to an upper-diagonal layer table */
-			if (namesame(us_varqualsave, "DRC_min_unconnected_distances") == 0 ||
-				namesame(us_varqualsave, "DRC_min_connected_distances") == 0)
+			if (namesame(us_varqualsave, "DRC_min_connected_distances") == 0 ||
+				namesame(us_varqualsave, "DRC_min_connected_distances_rule") == 0 ||
+				namesame(us_varqualsave, "DRC_min_unconnected_distances") == 0 ||
+				namesame(us_varqualsave, "DRC_min_unconnected_distances_rule") == 0 ||
+				namesame(us_varqualsave, "DRC_min_connected_distances_wide") == 0 ||
+				namesame(us_varqualsave, "DRC_min_connected_distances_wide_rule") == 0 ||
+				namesame(us_varqualsave, "DRC_min_unconnected_distances_wide") == 0 ||
+				namesame(us_varqualsave, "DRC_min_unconnected_distances_wide_rule") == 0 ||
+				namesame(us_varqualsave, "DRC_min_connected_distances_multi") == 0 ||
+				namesame(us_varqualsave, "DRC_min_connected_distances_multi_rule") == 0 ||
+				namesame(us_varqualsave, "DRC_min_unconnected_distances_multi") == 0 ||
+				namesame(us_varqualsave, "DRC_min_unconnected_distances_multi_rule") == 0)
 			{
 				annotate = 4;
 				maxleng = ((TECHNOLOGY *)objaddr)->layercount;
@@ -2558,7 +2636,7 @@ void us_var(INTSML count, char *par[])
 	/* make sure there is a value to set/adjust */
 	if (count <= 2)
 	{
-		count = ttygetparam(_("Value: "), &us_varvalp, MAXPARS-2, &par[2]) + 2;
+		count = ttygetparam(M_("Value: "), &us_varvalp, MAXPARS-2, &par[2]) + 2;
 		if (count == 2)
 		{
 			us_abortedmsg();
@@ -2641,16 +2719,11 @@ void us_var(INTSML count, char *par[])
 		if (var == NOVARIABLE || (var->type&VISARRAY) == 0)
 		{
 			/* establish the type of this parameter if it is being set */
-			if (isanumber(par[2]) == 0) newtype = VSTRING; else
-			{
-				newtype = VINTEGER;
-				for(pp = par[2]; *pp != 0; pp++) if (*pp == '.')
-				{
-					newtype = VFLOAT;
-					break;
-				}
-			}
-			newdescript = defaulttextdescript(NOGEOM);
+			us_getsimpletype(par[2], &newtype, &newaddr);
+			if (objtype == VARCINST) geom = ((ARCINST *)objaddr)->geom; else
+				if (objtype == VNODEINST) geom = ((NODEINST *)objaddr)->geom; else
+					geom = NOGEOM;
+			newdescript = defaulttextdescript(geom);
 		}
 	}
 
@@ -2693,7 +2766,7 @@ void us_var(INTSML count, char *par[])
 		if (l > 0)
 		{
 			ttyputusage("var set VARIABLE [OPTION]");
-			ttyputmsg(_("OPTION: lisp|tcl|java|display|temporary|fractional|cannot-change"));
+			ttyputusage("OPTION: lisp|tcl|java|display|temporary|fractional|cannot-change");
 			return;
 		}
 	}
@@ -2888,7 +2961,7 @@ void us_view(INTSML count, char *par[])
 			}
 			if (nv != NOVIEW)
 			{
-				ttyputverbose(_("Schematic page %s already exists"), &pp[15]);
+				ttyputverbose(M_("Schematic page %s already exists"), &pp[15]);
 				return;
 			}
 		}
@@ -2943,7 +3016,7 @@ void us_view(INTSML count, char *par[])
 				(void)strncpy(abbrev, pp, i+1);
 				abbrev[i+1] = 0;
 			}
-			ttyputverbose(_("Using '%s' as abbreviation"), abbrev);
+			ttyputverbose(M_("Using '%s' as abbreviation"), abbrev);
 		}
 
 		/* see if the view already exists */
@@ -2969,7 +3042,7 @@ void us_view(INTSML count, char *par[])
 			pp = par[3];
 			if (namesamen(pp, "text", strlen(pp)) == 0) nv->viewstate |= TEXTVIEW;
 		}
-		ttyputverbose(_("View '%s' created"), pp);
+		ttyputverbose(M_("View '%s' created"), pp);
 
 		/* clean up */
 		if (count < 3) efree(abbrev);
@@ -3015,7 +3088,7 @@ void us_view(INTSML count, char *par[])
 
 		/* delete the view */
 		if (killview(nv) != 0) ttyputerr(_("Problem deleting view"));
-			else ttyputverbose(_("View '%s' deleted"), par[1]);
+			else ttyputverbose(M_("View '%s' deleted"), par[1]);
 		return;
 	}
 
@@ -3048,7 +3121,7 @@ void us_view(INTSML count, char *par[])
 
 		if (changefacetview(np, nv) == 0)
 		{
-			ttyputverbose(_("Facet %s is now %s"), par[1], describenodeproto(np));
+			ttyputverbose(M_("Facet %s is now %s"), par[1], describenodeproto(np));
 
 			/* set current nodeproto to itself to redisplay its name */
 			if (np == getcurfacet())
@@ -3092,7 +3165,7 @@ void us_view(INTSML count, char *par[])
 			var = getvalkey((INTBIG)np, VNODEPROTO, VSTRING, el_schematic_page_size);
 			if (var == NOVARIABLE)
 			{
-				ttyputmsg(_("This facet has no schematic frame"));
+				ttyputmsg(M_("This facet has no schematic frame"));
 				return;
 			}
 			pt = (char *)var->addr;
@@ -3105,8 +3178,8 @@ void us_view(INTSML count, char *par[])
 				case 'd': case 'D': (void)addtoinfstr('D');   break;
 				case 'e': case 'E': (void)addtoinfstr('E');   break;
 			}
-			if (pt[1] == 'v' || pt[1] == 'V') (void)addstringtoinfstr(_(", vertical"));
-			ttyputmsg(_("This facet has a frame of size %s"), returninfstr());
+			if (pt[1] == 'v' || pt[1] == 'V') (void)addstringtoinfstr(M_(", vertical"));
+			ttyputmsg(M_("This facet has a frame of size %s"), returninfstr());
 			return;
 		}
 
@@ -3284,12 +3357,12 @@ void us_visiblelayers(INTSML count, char *par[])
 	for(i=0; i<el_curtech->layercount; i++)
 	{
 		if ((el_curtech->layers[i]->colstyle&INVISIBLE) != 0) continue;
-		if (j++ == 0) ttyputmsg(_("Visible layers:"));
+		if (j++ == 0) ttyputmsg(M_("Visible layers:"));
 		ch = us_layerletters(el_curtech, i);
 		fun = layerfunction(el_curtech, i);
-		if (*ch == 0) ttyputmsg(_("Layer %s"), layername(el_curtech, i)); else
-			ttyputmsg(_("Layer %s, letters %s%s"), layername(el_curtech, i), us_layerletters(el_curtech, i),
-				(fun&(LFTRANS1|LFTRANS2|LFTRANS3|LFTRANS4|LFTRANS5)) != 0 ? _(" (overlappable)") : "");
+		if (*ch == 0) ttyputmsg(M_("Layer %s"), layername(el_curtech, i)); else
+			ttyputmsg(M_("Layer %s, letters %s%s"), layername(el_curtech, i), us_layerletters(el_curtech, i),
+				(fun&(LFTRANS1|LFTRANS2|LFTRANS3|LFTRANS4|LFTRANS5)) != 0 ? M_(" (overlappable)") : "");
 	}
-	if (j == 0) ttyputmsg(_("No visible layers"));
+	if (j == 0) ttyputmsg(M_("No visible layers"));
 }
