@@ -69,7 +69,7 @@ void us_window(INTSML count, char *par[])
 	INTBIG lx, hx, ly, hy, xcur, ycur, windowView[8];
 	static POLYGON *poly = NOPOLYGON;
 	REGISTER STATUSFIELD *sf, **whichstatus;
-	char *newpar[4], *fieldname;
+	char *newpar[4], *fieldname, line[50];
 	WINDOWFRAME *wf;
 #if SIMAID
 	extern AIDENTRY *sim_aid;
@@ -85,7 +85,7 @@ void us_window(INTSML count, char *par[])
 
 	if (count == 0)
 	{
-		count = ttygetparam(_("Window configuration: "), &us_windowp, MAXPARS, par);
+		count = ttygetparam(M_("Window configuration: "), &us_windowp, MAXPARS, par);
 		if (count == 0)
 		{
 			us_abortedmsg();
@@ -341,38 +341,36 @@ void us_window(INTSML count, char *par[])
 	if (namesamen(pp, "down", l) == 0 && l >= 2)
 	{
 		if (us_needwindow()) return;
-
-#if SIMAID
-		/* special case for waveform window */
-		if ((el_curwindowpart->state&WINDOWTYPE) == WAVEFORMWINDOW)
-		{
-			newpar[0] = "window";
-			newpar[1] = "move";
-			newpar[2] = "down";
-			(void)tellaid(sim_aid, 3, newpar);
-			return;
-		}
-#endif
-
-		/* pan a 3D window */
-		if ((el_curwindowpart->state&WINDOWTYPE) == DISP3DWINDOW)
-		{
-			us_3dpanview(el_curwindowpart, 0, 1);
-			return;
-		}
-
-		/* cannot pan a nonstandard window */
-		if ((el_curwindowpart->state&WINDOWTYPE) != DISPWINDOW)
-		{
-			us_abortcommand(_("Can only pan circuit editing windows"));
-			return;
-		}
-
-		if (us_needfacet() == NONODEPROTO) return;
 		if (count < 2) pp = "0.5"; else pp = par[1];
-		if (pp[strlen(pp)-1] == 'l') dist = atola(pp); else
-			dist = muldiv(atofr(pp), (el_curwindowpart->screenhy - el_curwindowpart->screenly), WHOLE);
-		us_slideup(-dist);
+		switch (el_curwindowpart->state&WINDOWTYPE)
+		{
+			case DISPWINDOW:
+				if (us_needfacet() == NONODEPROTO) return;
+				if (pp[strlen(pp)-1] == 'l') dist = atola(pp); else
+					dist = muldiv(atofr(pp), (el_curwindowpart->screenhy - el_curwindowpart->screenly), WHOLE);
+				us_slideup(-dist);
+				return;
+			case DISP3DWINDOW:
+				us_3dpanview(el_curwindowpart, 0, -1);
+				return;
+			case TEXTWINDOW:
+				us_pantext(el_curwindowpart, 0, -1);
+				return;
+#if SIMAID
+			case WAVEFORMWINDOW:
+				newpar[0] = "window";
+				newpar[1] = "move";
+				newpar[2] = "down";
+				sprintf(line, "%d", (INTBIG)(atof(pp)*20.0));
+				newpar[3] = line;
+				(void)tellaid(sim_aid, 4, newpar);
+				return;
+#endif
+			case EXPLORERWINDOW:
+				us_explorepan(el_curwindowpart, -1);
+				return;
+		}
+		us_abortcommand(_("Cannot pan this kind of windows"));
 		return;
 	}
 
@@ -392,8 +390,8 @@ void us_window(INTSML count, char *par[])
 			}
 		}
 		if ((us_aid->aidstate&INTERACTIVE) == 0)
-			ttyputverbose(_("Cursor-based commands will act immediately")); else
-				ttyputverbose(_("Cursor-based commands will drag their objects"));
+			ttyputverbose(M_("Cursor-based commands will act immediately")); else
+				ttyputverbose(M_("Cursor-based commands will drag their objects"));
 		return;
 	}
 
@@ -617,38 +615,33 @@ void us_window(INTSML count, char *par[])
 	if (namesamen(pp, "left", l) == 0 && l >= 1)
 	{
 		if (us_needwindow()) return;
-
-#if SIMAID
-		/* special case for waveform window */
-		if ((el_curwindowpart->state&WINDOWTYPE) == WAVEFORMWINDOW)
-		{
-			newpar[0] = "window";
-			newpar[1] = "move";
-			newpar[2] = "left";
-			(void)tellaid(sim_aid, 3, newpar);
-			return;
-		}
-#endif
-
-		/* pan a 3D window */
-		if ((el_curwindowpart->state&WINDOWTYPE) == DISP3DWINDOW)
-		{
-			us_3dpanview(el_curwindowpart, -1, 0);
-			return;
-		}
-
-		/* cannot pan a nonstandard window */
-		if ((el_curwindowpart->state&WINDOWTYPE) != DISPWINDOW)
-		{
-			us_abortcommand(_("Can only pan circuit editing windows"));
-			return;
-		}
-
-		if (us_needfacet() == NONODEPROTO) return;
 		if (count < 2) pp = "0.5"; else pp = par[1];
-		if (pp[strlen(pp)-1] == 'l') dist = atola(pp); else
-			dist = muldiv(atofr(pp), (el_curwindowpart->screenhx - el_curwindowpart->screenlx), WHOLE);
-		us_slideleft(dist);
+		switch (el_curwindowpart->state&WINDOWTYPE)
+		{
+			case DISPWINDOW:
+				if (us_needfacet() == NONODEPROTO) return;
+				if (pp[strlen(pp)-1] == 'l') dist = atola(pp); else
+					dist = muldiv(atofr(pp), (el_curwindowpart->screenhx - el_curwindowpart->screenlx), WHOLE);
+				us_slideleft(dist);
+				return;
+			case DISP3DWINDOW:
+				us_3dpanview(el_curwindowpart, 1, 0);
+				return;
+			case TEXTWINDOW:
+				us_pantext(el_curwindowpart, 1, 0);
+				return;
+#if SIMAID
+			case WAVEFORMWINDOW:
+				newpar[0] = "window";
+				newpar[1] = "move";
+				newpar[2] = "left";
+				sprintf(line, "%d", (INTBIG)(atof(pp)*20.0));
+				newpar[3] = line;
+				(void)tellaid(sim_aid, 4, newpar);
+				return;
+#endif
+		}
+		us_abortcommand(_("Cannot pan this kind of windows"));
 		return;
 	}
 
@@ -929,8 +922,8 @@ void us_window(INTSML count, char *par[])
 			}
 		}
 		if ((us_state&NONOVERLAPPABLEDISPLAY) != 0)
-			ttyputverbose(_("Overlappable layers will not be handled")); else
-				ttyputverbose(_("Overlappable layers will be drawn properly"));
+			ttyputverbose(M_("Overlappable layers will not be handled")); else
+				ttyputverbose(M_("Overlappable layers will be drawn properly"));
 		return;
 	}
 
@@ -993,38 +986,33 @@ void us_window(INTSML count, char *par[])
 	if (namesamen(pp, "right", l) == 0 && l >= 1)
 	{
 		if (us_needwindow()) return;
-
-#if SIMAID
-		/* special case for waveform window */
-		if ((el_curwindowpart->state&WINDOWTYPE) == WAVEFORMWINDOW)
-		{
-			newpar[0] = "window";
-			newpar[1] = "move";
-			newpar[2] = "right";
-			(void)tellaid(sim_aid, 3, newpar);
-			return;
-		}
-#endif
-
-		/* pan a 3D window */
-		if ((el_curwindowpart->state&WINDOWTYPE) == DISP3DWINDOW)
-		{
-			us_3dpanview(el_curwindowpart, 1, 0);
-			return;
-		}
-
-		/* cannot pan a nonstandard window */
-		if ((el_curwindowpart->state&WINDOWTYPE) != DISPWINDOW)
-		{
-			us_abortcommand(_("Can only pan circuit editing windows"));
-			return;
-		}
-
-		if (us_needfacet() == NONODEPROTO) return;
 		if (count < 2) pp = "0.5"; else pp = par[1];
-		if (pp[strlen(pp)-1] == 'l') dist = atola(pp); else
-			dist = muldiv(atofr(pp), (el_curwindowpart->screenhx - el_curwindowpart->screenlx), WHOLE);
-		us_slideleft(-dist);
+		switch (el_curwindowpart->state&WINDOWTYPE)
+		{
+			case DISPWINDOW:
+				if (us_needfacet() == NONODEPROTO) return;
+				if (pp[strlen(pp)-1] == 'l') dist = atola(pp); else
+					dist = muldiv(atofr(pp), (el_curwindowpart->screenhx - el_curwindowpart->screenlx), WHOLE);
+				us_slideleft(-dist);
+				return;
+			case DISP3DWINDOW:
+				us_3dpanview(el_curwindowpart, -1, 0);
+				return;
+			case TEXTWINDOW:
+				us_pantext(el_curwindowpart, -1, 0);
+				return;
+#if SIMAID
+			case WAVEFORMWINDOW:
+				newpar[0] = "window";
+				newpar[1] = "move";
+				newpar[2] = "right";
+				sprintf(line, "%d", (INTBIG)(atof(pp)*20.0));
+				newpar[3] = line;
+				(void)tellaid(sim_aid, 4, newpar);
+				return;
+#endif
+		}
+		us_abortcommand(_("Cannot pan this kind of windows"));
 		return;
 	}
 
@@ -1049,7 +1037,7 @@ void us_window(INTSML count, char *par[])
 		(void)addstringtoinfstr(par[1]);
 		(void)setval((INTBIG)us_aid, VAID, returninfstr(), (INTBIG)windowView,
 			VINTEGER|VISARRAY|(8<<VLENGTHSH)|VDONTSAVE);
-		ttyputverbose(_("Window view %s saved"), par[1]);
+		ttyputverbose(M_("Window view %s saved"), par[1]);
 		return;
 	}
 
@@ -1083,8 +1071,8 @@ void us_window(INTSML count, char *par[])
 				sf = *us_statusfields[i].fieldaddr;
 				if (sf == 0) continue;
 				if (sf->line == 0)
-					ttyputmsg(_("Window title has %s"), us_statusfields[i].fieldaddr); else
-						ttyputmsg(_("Line %d from %3d%% to %3d%% is %s"), sf->line, sf->startper,
+					ttyputmsg(M_("Window title has %s"), us_statusfields[i].fieldaddr); else
+						ttyputmsg(M_("Line %d from %3d%% to %3d%% is %s"), sf->line, sf->startper,
 							sf->endper, us_statusfields[i].fieldaddr);
 			}
 			return;
@@ -1095,21 +1083,21 @@ void us_window(INTSML count, char *par[])
 			if (count < 3)
 			{
 				if ((us_state&NONPERSISTENTCURNODE) != 0)
-					ttyputmsg(_("Current node displayed temporarily")); else
-						ttyputmsg(_("Current node display is persistent"));
+					ttyputmsg(M_("Current node displayed temporarily")); else
+						ttyputmsg(M_("Current node display is persistent"));
 				return;
 			}
 			l = strlen(pp = par[2]);
 			if (namesamen(pp, "persistent", l) == 0)
 			{
 				us_state &= ~NONPERSISTENTCURNODE;
-				ttyputverbose(_("Current node display is persistent"));
+				ttyputverbose(M_("Current node display is persistent"));
 				return;
 			}
 			if (namesamen(pp, "temporary", l) == 0)
 			{
 				us_state |= NONPERSISTENTCURNODE;
-				ttyputverbose(_("Current node displayed temporarily"));
+				ttyputverbose(M_("Current node displayed temporarily"));
 				return;
 			}
 			ttyputusage("window status-bar current-node [persistent|temporary]");
@@ -1213,10 +1201,10 @@ void us_window(INTSML count, char *par[])
 		}
 		if ((us_useroptions&DRAWTINYFACETS) != 0)
 		{
-			ttyputverbose(_("Tiny facets will be drawn"));
+			ttyputverbose(M_("Tiny facets will be drawn"));
 		} else
 		{
-			ttyputverbose(_("Tiny facets will be hashed-out after %s lambda per pixel"),
+			ttyputverbose(M_("Tiny facets will be hashed-out after %s lambda per pixel"),
 				frtoa(us_tinyratio));
 		}
 		return;
@@ -1290,38 +1278,36 @@ void us_window(INTSML count, char *par[])
 	if (namesamen(pp, "up", l) == 0 && l >= 2)
 	{
 		if (us_needwindow()) return;
-
-#if SIMAID
-		/* special case for waveform window */
-		if ((el_curwindowpart->state&WINDOWTYPE) == WAVEFORMWINDOW)
-		{
-			newpar[0] = "window";
-			newpar[1] = "move";
-			newpar[2] = "up";
-			(void)tellaid(sim_aid, 3, newpar);
-			return;
-		}
-#endif
-
-		/* pan a 3D window */
-		if ((el_curwindowpart->state&WINDOWTYPE) == DISP3DWINDOW)
-		{
-			us_3dpanview(el_curwindowpart, 0, -1);
-			return;
-		}
-
-		/* cannot pan a nonstandard window */
-		if ((el_curwindowpart->state&WINDOWTYPE) != DISPWINDOW)
-		{
-			us_abortcommand(_("Can only pan circuit editing windows"));
-			return;
-		}
-
-		if (us_needfacet() == NONODEPROTO) return;
 		if (count < 2) pp = "0.5"; else pp = par[1];
-		if (pp[strlen(pp)-1] == 'l') dist = atola(pp); else
-			dist = muldiv(atofr(pp), (el_curwindowpart->screenhy - el_curwindowpart->screenly), WHOLE);
-		us_slideup(dist);
+		switch (el_curwindowpart->state&WINDOWTYPE)
+		{
+			case DISPWINDOW:
+				if (us_needfacet() == NONODEPROTO) return;
+				if (pp[strlen(pp)-1] == 'l') dist = atola(pp); else
+					dist = muldiv(atofr(pp), (el_curwindowpart->screenhy - el_curwindowpart->screenly), WHOLE);
+				us_slideup(dist);
+				return;
+			case DISP3DWINDOW:
+				us_3dpanview(el_curwindowpart, 0, 1);
+				return;
+			case TEXTWINDOW:
+				us_pantext(el_curwindowpart, 0, 1);
+				return;
+#if SIMAID
+			case WAVEFORMWINDOW:
+				newpar[0] = "window";
+				newpar[1] = "move";
+				newpar[2] = "up";
+				sprintf(line, "%d", (INTBIG)(atof(pp)*20.0));
+				newpar[3] = line;
+				(void)tellaid(sim_aid, 4, newpar);
+				return;
+#endif
+			case EXPLORERWINDOW:
+				us_explorepan(el_curwindowpart, 1);
+				return;
+		}
+		us_abortcommand(_("Cannot pan this kind of windows"));
 		return;
 	}
 
@@ -1329,7 +1315,7 @@ void us_window(INTSML count, char *par[])
 	{
 		if (count <= 1)
 		{
-			count = ttygetparam(_("Window to use: "), &us_windowup, MAXPARS-1, &par[1]) + 1;
+			count = ttygetparam(M_("Window to use: "), &us_windowup, MAXPARS-1, &par[1]) + 1;
 			if (count == 1)
 			{
 				us_abortedmsg();
@@ -1371,8 +1357,8 @@ void us_window(INTSML count, char *par[])
 			}
 		}
 		if ((us_aid->aidstate&INTEGRAL) == 0)
-			ttyputverbose(_("Window scaling will be continuous")); else
-				ttyputverbose(_("Window scaling will force integral pixel alignment"));
+			ttyputverbose(M_("Window scaling will be continuous")); else
+				ttyputverbose(M_("Window scaling will force integral pixel alignment"));
 		return;
 	}
 
@@ -1453,14 +1439,24 @@ void us_window(INTSML count, char *par[])
 				us_abortcommand(_("Cannot view this window in 3 dimensions"));
 				return;
 			}
+			np = us_needfacet();
+			if (np == NONODEPROTO) return;
 
 			us_3dsetupviewing(w);
 
 			/* clear highlighting */
 			us_clearhighlightcount();
 
+			/* make the facet fill the window */
+			us_fullview(np, &lx, &hx, &ly, &hy);
+			us_squarescreen(w, NOWINDOWPART, 0, &lx, &hx, &ly, &hy);
+
 			/* set the window extents */
 			startobjectchange((INTBIG)w, VWINDOWPART);
+			(void)setval((INTBIG)el_curwindowpart, VWINDOWPART, "screenlx", lx, VINTEGER);
+			(void)setval((INTBIG)el_curwindowpart, VWINDOWPART, "screenhx", hx, VINTEGER);
+			(void)setval((INTBIG)el_curwindowpart, VWINDOWPART, "screenly", ly, VINTEGER);
+			(void)setval((INTBIG)el_curwindowpart, VWINDOWPART, "screenhy", hy, VINTEGER);
 			(void)setval((INTBIG)w, VWINDOWPART, "buttonhandler", (INTBIG)us_3dbuttonhandler,
 				VADDRESS);
 			(void)setval((INTBIG)w, VWINDOWPART, "state", (w->state & ~WINDOWTYPE) | DISP3DWINDOW,
@@ -1468,6 +1464,7 @@ void us_window(INTSML count, char *par[])
 			endobjectchange((INTBIG)w, VWINDOWPART);
 			return;
 		}
+
 		if (namesamen(pp, "end", l) == 0 && l >= 1)
 		{
 			if ((w->state&WINDOWTYPE) == DISPWINDOW)
@@ -1525,7 +1522,7 @@ void us_yanknode(INTSML count, char *par[])
 	REGISTER NODEPROTO *np;
 	REGISTER GEOM **list;
 
-	list = us_gethighlighted(OBJNODEINST);
+	list = us_gethighlighted(OBJNODEINST, 0, 0);
 	if (list[0] == NOGEOM)
 	{
 		us_abortcommand(_("Must highlight facet(s) to be yanked"));
