@@ -51,7 +51,7 @@ void us_tran_logmakearcs(NODEPROTO*, NODEPROTO*);
 void us_tran_logadjustnodes(NODEPROTO*);
 void us_tran_makemanhattan(NODEPROTO*);
 PORTPROTO *us_tranconvpp(NODEINST*, PORTPROTO*);
-INTSML us_tranismos(NODEINST*);
+INTBIG us_tranismos(NODEINST*);
 PORTPROTO *us_trangetproto(NODEINST*, INTSML);
 NODEPROTO *us_tran_makelayoutfacets(NODEPROTO*, char*, TECHNOLOGY*, TECHNOLOGY*, VIEW*);
 INTSML us_tran_makelayoutparts(NODEPROTO*, NODEPROTO*, TECHNOLOGY*, TECHNOLOGY*, VIEW*);
@@ -177,9 +177,8 @@ void us_tran_logmakenodes(NODEPROTO *facet, NODEPROTO *newfacet, TECHNOLOGY *new
 	PORTEXPINST *pexp;
 	REGISTER PORTPROTO *pp, *pp2;
 	REGISTER PORTARCINST *pi;
-	INTSML type, rotate, trans;
-	INTBIG len, wid, vartype, lambda;
-	char ttype[30];
+	INTSML rotate, trans;
+	INTBIG type, len, wid, lambda;
 
 	/*
 	 * for each node, create a new node in the newfacet, of the correct
@@ -225,56 +224,36 @@ void us_tran_logmakenodes(NODEPROTO *facet, NODEPROTO *newfacet, TECHNOLOGY *new
 				schemni = us_tran_logmakenode(sch_transistorprim, ni, trans, rotate,
 					newfacet, newtech);
 
-				/* start with the name of the transistor */
-				(void)initinfstr();
+				/* set the type of the transistor */
 				switch (type)
 				{
-					case NPTRAPMOS:
-						(void)addstringtoinfstr("pmos");    break;
-					case NPTRANMOS:
-						(void)addstringtoinfstr("nmos");    break;
-					case NPTRADMOS:
-						(void)addstringtoinfstr("dmos");    break;
-					case NPTRAPNP:
-						(void)addstringtoinfstr("pnp");     break;
-					case NPTRANPN:
-						(void)addstringtoinfstr("npn");     break;
-					case NPTRANJFET:
-						(void)addstringtoinfstr("njfet");   break;
-					case NPTRAPJFET:
-						(void)addstringtoinfstr("pjfet");   break;
-					case NPTRADMES:
-						(void)addstringtoinfstr("dmes");    break;
-					case NPTRAEMES:
-						(void)addstringtoinfstr("emes");    break;
-					case NPTRANE2L:
-						(void)addstringtoinfstr("e2l");     break;
-					default:
-						 break;
+					case NPTRAPMOS:  schemni->userbits |= TRANPMOS;    break;
+					case NPTRANMOS:  schemni->userbits |= TRANNMOS;    break;
+					case NPTRADMOS:  schemni->userbits |= TRANDMOS;    break;
+					case NPTRAPNP:   schemni->userbits |= TRANPNP;     break;
+					case NPTRANPN:   schemni->userbits |= TRANNPN;     break;
+					case NPTRANJFET: schemni->userbits |= TRANNJFET;   break;
+					case NPTRAPJFET: schemni->userbits |= TRANPJFET;   break;
+					case NPTRADMES:  schemni->userbits |= TRANDMES;    break;
+					case NPTRAEMES:  schemni->userbits |= TRANEMES;    break;
 				}
 
 				/* add in the size */
 				transistorsize(ni, &len, &wid);
 				if (len >= 0 && wid >= 0)
 				{
+					(void)initinfstr();
 					lambda = lambdaofnode(ni);
 					len /= lambda;
 					wid /= lambda;
 					if (type == NPTRAPMOS || type == NPTRANMOS || type == NPTRADMOS)
-						(void)sprintf(ttype, "%ld/%ld", wid, len); else
-							(void)sprintf(ttype, "%ld", len*wid);
-					(void)addstringtoinfstr(ttype);
+						(void)formatinfstr("%ld/%ld", wid, len); else
+							(void)formatinfstr("%ld", len*wid);
+
+					/* store this information on the transistor */
+					us_setvariablevalue(schemni, sch_transistortypekey,
+						returninfstr(), VSTRING|VDISPLAY);
 				}
-
-				/* decide whether or not to display the size information */
-				vartype = VSTRING;
-				if (ni->highx - ni->lowx != ni->proto->highx - ni->proto->lowx ||
-					ni->highy - ni->lowy != ni->proto->highy - ni->proto->lowy)
-						vartype |= VDISPLAY;
-
-				/* store this information on the transistor */
-				(void)setvalkey((INTBIG)schemni, VNODEINST, sch_transistortypekey,
-					(INTBIG)returninfstr(), vartype);
 		}
 
 		/* store the new node in the old node */
@@ -291,7 +270,8 @@ void us_tran_logmakenodes(NODEPROTO *facet, NODEPROTO *newfacet, TECHNOLOGY *new
 				if (pp2 == NOPORTPROTO) return;
 				pp2->userbits = (pp2->userbits & ~STATEBITS) |
 					(pexp->exportproto->userbits & STATEBITS);
-				pp2->textdescript = pexp->exportproto->textdescript;
+				pp2->textdescript = (pexp->exportproto->textdescript & ~VTSIZE) |
+					(defaulttextsize(1) << VTSIZESH);
 				if (copyvars((INTBIG)pexp->exportproto, VPORTPROTO, (INTBIG)pp2, VPORTPROTO) != 0)
 					return;
 			}
@@ -374,7 +354,7 @@ void us_tran_logadjustnodes(NODEPROTO *newfacet)
 	us_tran_makemanhattan(newfacet);
 }
 
-#define	MAXADJUST        5
+#define MAXADJUST        5
 
 void us_tran_makemanhattan(NODEPROTO *newfacet)
 {
@@ -393,7 +373,7 @@ void us_tran_makemanhattan(NODEPROTO *newfacet)
 
 		/* see if this pin can be adjusted so that all wires are manhattan */
 		count = 0;
-		for(pi=ni->firstportarcinst; pi!=NOPORTARCINST; pi=pi->nextportarcinst)
+		for(pi = ni->firstportarcinst; pi != NOPORTARCINST; pi = pi->nextportarcinst)
 		{
 			ai = pi->conarcinst;
 			if (ai->end[0].nodeinst == ni)
@@ -409,7 +389,7 @@ void us_tran_makemanhattan(NODEPROTO *newfacet)
 
 		/* now adjust for all these points */
 		xp = (ni->lowx + ni->highx) / 2;   yp = (ni->lowy + ni->highy) / 2;
-		bestdist = HUGEINT;
+		bestdist = MAXINTBIG;
 		for(i=0; i<count; i++) for(j=0; j<count; j++)
 		{
 			dist = abs(xp - x[i]) + abs(yp - y[j]);
@@ -419,7 +399,7 @@ void us_tran_makemanhattan(NODEPROTO *newfacet)
 		}
 
 		/* if there was a better place, move the node */
-		if (bestdist != HUGEINT)
+		if (bestdist != MAXINTBIG)
 			modifynodeinst(ni, bestx-xp, besty-yp, bestx-xp, besty-yp, 0, 0);
 	}
 }
@@ -459,28 +439,32 @@ PORTPROTO *us_tranconvpp(NODEINST *ni, PORTPROTO *mospp)
  * define from efunction.h; if it's a passive connector, return TRAN_PIN;
  * if it's a facet, return TRAN_FACET; else return NPUNKNOWN.
  */
-INTSML us_tranismos(NODEINST *ni)
+INTBIG us_tranismos(NODEINST *ni)
 {
+	INTBIG fun;
+
 	if (ni->proto->primindex == 0) return(TRAN_FACET);
-	switch((ni->proto->userbits & NFUNCTION) >> NFUNCTIONSH)
+	fun = (ni->proto->userbits & NFUNCTION) >> NFUNCTIONSH;
+	switch(fun)
 	{
-		case NPTRANMOS:   return(NPTRANMOS);
-		case NPTRADMOS:   return(NPTRADMOS);
-		case NPTRAPMOS:   return(NPTRAPMOS);
-		case NPTRANPN:    return(NPTRANPN);
-		case NPTRAPNP:    return(NPTRAPNP);
-		case NPTRANJFET:  return(NPTRANJFET);
-		case NPTRAPJFET:  return(NPTRAPJFET);
-		case NPTRADMES:   return(NPTRADMES);
-		case NPTRAEMES:   return(NPTRAEMES);
-		case NPTRANE2L:   return(NPTRANE2L);
-		case NPTRANSREF:  return(NPTRANSREF);
-		case NPPIN:       return(TRAN_PIN);
-		case NPCONTACT:   return(TRAN_PIN);
-		case NPNODE:      return(TRAN_PIN);
-		case NPCONNECT:   return(TRAN_PIN);
-		case NPSUBSTRATE: return(TRAN_PIN);
-		case NPWELL:      return(TRAN_PIN);
+		case NPTRANMOS:   case NPTRA4NMOS:
+		case NPTRADMOS:   case NPTRA4DMOS:
+		case NPTRAPMOS:   case NPTRA4PMOS:
+		case NPTRANPN:    case NPTRA4NPN:
+		case NPTRAPNP:    case NPTRA4PNP:
+		case NPTRANJFET:  case NPTRA4NJFET:
+		case NPTRAPJFET:  case NPTRA4PJFET:
+		case NPTRADMES:   case NPTRA4DMES:
+		case NPTRAEMES:   case NPTRA4EMES:
+		case NPTRANSREF:
+			return(fun);
+		case NPPIN:
+		case NPCONTACT:
+		case NPNODE:
+		case NPCONNECT:
+		case NPSUBSTRATE:
+		case NPWELL:
+			return(TRAN_PIN);
 	}
 	return(NPUNKNOWN);
 }
@@ -805,7 +789,8 @@ void us_tranplacenode(NODEINST *ni, NODEPROTO *newnp, NODEPROTO *newfacet,
 		pp2 = newportproto(newfacet, newni, pp, pexp->exportproto->protoname);
 		if (pp2 == NOPORTPROTO) return;
 		pp2->userbits = (pp2->userbits & ~STATEBITS) | (pexp->exportproto->userbits & STATEBITS);
-		pp2->textdescript = pexp->exportproto->textdescript;
+		pp2->textdescript = (pexp->exportproto->textdescript & ~VTSIZE) |
+			(defaulttextsize(1) << VTSIZESH);
 		if (copyvars((INTBIG)pexp->exportproto, VPORTPROTO, (INTBIG)pp2, VPORTPROTO) != 0)
 			return;
 	}
@@ -845,7 +830,7 @@ PORTPROTO *us_convport(NODEINST *ni, NODEINST *newni, PORTPROTO *oldpp)
 	}
 
 	/* associate by position in port list */
-	for(pp=ni->proto->firstportproto, npp=newni->proto->firstportproto;
+	for(pp = ni->proto->firstportproto, npp = newni->proto->firstportproto;
 		pp != NOPORTPROTO && npp != NOPORTPROTO;
 			pp = pp->nextportproto, npp = npp->nextportproto)
 				if (pp == oldpp) return(npp);
@@ -895,7 +880,6 @@ NODEPROTO *us_figurenewnproto(NODEINST *oldni, TECHNOLOGY *newtech)
 	REGISTER INTBIG important, funct;
 	REGISTER NODEPROTO *np, *rnp, *oldnp;
 	REGISTER NODEINST *ni;
-	char *dummy;
 	static POLYGON *poly = NOPOLYGON;
 
 	/* get a polygon */
@@ -906,11 +890,11 @@ NODEPROTO *us_figurenewnproto(NODEINST *oldni, TECHNOLOGY *newtech)
 	if (oldnp->primindex == 0 || oldnp->tech == newtech) return(oldnp);
 
 	/* if this is a layer node, check the layer functions */
-	type = nodefunction(oldni, &dummy);
+	type = nodefunction(oldni, 0);
 	if (type == NPNODE)
 	{
 		/* get the polygon describing the first box of the old node */
-		(void)nodepolys(oldni, 0);
+		(void)nodepolys(oldni, 0, NOWINDOWPART);
 		shapenodepoly(oldni, 0, poly);
 		important = LFTYPE | LFPSEUDO | LFNONELEC;
 		funct = layerfunction(oldnp->tech, poly->layer) & important;
@@ -921,7 +905,7 @@ NODEPROTO *us_figurenewnproto(NODEINST *oldni, TECHNOLOGY *newtech)
 			if (((np->userbits&NFUNCTION) >> NFUNCTIONSH) != NPNODE) continue;
 			ni = dummynode();
 			ni->proto = np;
-			(void)nodepolys(ni, 0);
+			(void)nodepolys(ni, 0, NOWINDOWPART);
 			shapenodepoly(ni, 0, poly);
 			if ((layerfunction(newtech, poly->layer)&important) == funct)
 				return(np);
@@ -971,7 +955,7 @@ NODEPROTO *us_figurenewnproto(NODEINST *oldni, TECHNOLOGY *newtech)
 	{
 		if (oldnp->temp1 == 0)
 			ttyputmsg(_("Node %s (function %s) has no equivalent in the %s technology"),
-				describenodeproto(oldnp), nodefunctionname((INTSML)type, oldni),
+				describenodeproto(oldnp), nodefunctionname(type, oldni),
 					newtech->techname);
 		oldnp->temp1 = 1;
 		return(oldnp);
@@ -985,7 +969,7 @@ extern LIST us_teclayer_functions[];
 extern LIST us_tecarc_functions[];
 extern LIST us_tecnode_functions[];
 
-#define	MAXCOLS 10
+#define MAXCOLS 10
 
 void us_printtechnology(TECHNOLOGY *tech)
 {
@@ -1024,7 +1008,7 @@ void us_printtechnology(TECHNOLOGY *tech)
 		if (truename != 0) us_abortcommand(_("Cannot write %s"), truename);
 		return;
 	}
-	ttyputverbose(_("Writing: %s"), name);
+	ttyputverbose(M_("Writing: %s"), name);
 
 	/****************************** dump layers ******************************/
 
@@ -1054,7 +1038,7 @@ void us_printtechnology(TECHNOLOGY *tech)
 	/* compute each layer */
 	for(i=0; i<tech->layercount; i++)
 	{
-		(void)allocstring(&names[i+1], layername(tech, (INTSML)i), el_tempcluster);
+		(void)allocstring(&names[i+1], layername(tech, i), el_tempcluster);
 
 		gra = tech->layers[i];
 		if (ecolorname(gra->col, &fieldname, &colorsymbol) != 0) fieldname = "?";
@@ -1122,10 +1106,10 @@ void us_printtechnology(TECHNOLOGY *tech)
 	ai->length = 4000;
 	ai->userbits |= NOEXTEND;
 	tot = 1;
-	for(ap=tech->firstarcproto; ap != NOARCPROTO; ap = ap->nextarcproto)
+	for(ap = tech->firstarcproto; ap != NOARCPROTO; ap = ap->nextarcproto)
 	{
 		ai->proto = ap;
-		if (tech->arcpolys != 0) j = (*(tech->arcpolys))(ai); else
+		if (tech->arcpolys != 0) j = (*(tech->arcpolys))(ai, NOWINDOWPART); else
 			j = tech->arcprotos[ap->arcindex]->laycount;
 		tot += j;
 	}
@@ -1149,7 +1133,7 @@ void us_printtechnology(TECHNOLOGY *tech)
 	(void)allocstring(&funcs[0], "Function", el_tempcluster);
 
 	tot = 1;
-	for(ap=tech->firstarcproto; ap != NOARCPROTO; ap = ap->nextarcproto)
+	for(ap = tech->firstarcproto; ap != NOARCPROTO; ap = ap->nextarcproto)
 	{
 		(void)allocstring(&names[tot], ap->protoname, el_tempcluster);
 
@@ -1166,7 +1150,7 @@ void us_printtechnology(TECHNOLOGY *tech)
 		(void)allocstring(&funcs[tot], us_tecarc_functions[func].name, el_tempcluster);
 
 		ai->proto = ap;
-		if (tech->arcpolys != 0) j = (*(tech->arcpolys))(ai); else
+		if (tech->arcpolys != 0) j = (*(tech->arcpolys))(ai, NOWINDOWPART); else
 			tech_realpolys = j = tech->arcprotos[ap->arcindex]->laycount;
 		for(k=0; k<j; k++)
 		{
@@ -1227,7 +1211,7 @@ void us_printtechnology(TECHNOLOGY *tech)
 	for(np = tech->firstnodeproto; np != NONODEPROTO; np = np->nextnodeproto)
 	{
 		ni->proto = np;
-		if (tech->nodepolys != 0) j = (*(tech->nodepolys))(ni, 0); else
+		if (tech->nodepolys != 0) j = (*(tech->nodepolys))(ni, 0, NOWINDOWPART); else
 			j = tech->nodeprotos[np->primindex-1]->layercount;
 		l = 0;
 		for(pp = np->firstportproto; pp != NOPORTPROTO; pp = pp->nextportproto)
@@ -1273,7 +1257,7 @@ void us_printtechnology(TECHNOLOGY *tech)
 		ni->proto = np;
 		ni->lowx = np->lowx;   ni->highx = np->highx;
 		ni->lowy = np->lowy;   ni->highy = np->highy;
-		if (tech->nodepolys != 0) j = (*(tech->nodepolys))(ni, 0); else
+		if (tech->nodepolys != 0) j = (*(tech->nodepolys))(ni, 0, NOWINDOWPART); else
 			tech_realpolys = j = tech->nodeprotos[np->primindex-1]->layercount;
 		for(k=0; k<j; k++)
 		{
