@@ -80,7 +80,7 @@ GRAPHICS us_graybox = {LAYERO, GRAY, PATTERNED, PATTERNED,
 					0x3030}, /*   XX      XX     */
 					NOVARIABLE, 0};
 
-#define	MAXGRID        75
+#define MAXGRID        75
 #define MINGRID         4	/* the minimum grid has 4 pixels spacing */
 
 /* prototypes for local routines */
@@ -163,16 +163,10 @@ void us_gettruewindowbounds(WINDOWPART *w, INTBIG *lx, INTBIG *hx, INTBIG *ly, I
 void us_fullview(NODEPROTO *np, INTBIG *screenlx, INTBIG *screenhx, INTBIG *screenly,
 	INTBIG *screenhy)
 {
-	INTBIG x, y, newxc, newyc, xp, yp, lx, hx, ly, hy;
-	REGISTER INTBIG lambda;
-	REGISTER INTSML i, first, nodexfvalid;
+	INTBIG x, y, nlx, nhx, nly, nhy;
+	REGISTER INTBIG lambda, first;
 	REGISTER NODEINST *ni;
 	REGISTER ARCINST *ai;
-	REGISTER VARIABLE *var;
-	REGISTER PORTPROTO *pp;
-	REGISTER PORTEXPINST *pe;
-	XARRAY trans;
-	static POLYGON *poly = NOPOLYGON;
 
 	if (framesize(&x, &y, np) == 0)
 	{
@@ -183,137 +177,35 @@ void us_fullview(NODEPROTO *np, INTBIG *screenlx, INTBIG *screenhx, INTBIG *scre
 		*screenhy = y/2;
 	} else
 	{
-		/* must recompute this by hand because facet bounds don't include facet-centers */
-		nodexfvalid = 0;
+		/* must recompute this by hand because facet bounds don't include facet-centers or big text */
 		first = 1;
 		for(ni = np->firstnodeinst; ni != NONODEINST; ni = ni->nextnodeinst)
 		{
+			us_getnodebounds(ni, &nlx, &nhx, &nly, &nhy);
 			if (first != 0)
 			{
-				*screenlx = ni->geom->lowx;
-				*screenhx = ni->geom->highx;
-				*screenly = ni->geom->lowy;
-				*screenhy = ni->geom->highy;
+				*screenlx = nlx;
+				*screenhx = nhx;
+				*screenly = nly;
+				*screenhy = nhy;
 				first = 0;
 			} else
 			{
-				*screenlx = mini(*screenlx, ni->geom->lowx);
-				*screenhx = maxi(*screenhx, ni->geom->highx);
-				*screenly = mini(*screenly, ni->geom->lowy);
-				*screenhy = maxi(*screenhy, ni->geom->highy);
-			}
-
-			/* include any displayable variables with an offset or array */
-			if (el_curwindowpart != NOWINDOWPART)
-			{
-				for(i=0; i<ni->numvar; i++)
-				{
-					var = &ni->firstvar[i];
-					if ((var->type&VDISPLAY) == 0) continue;
-					if ((var->type&VISARRAY) != 0 && getlength(var) > 1)
-					{
-						if (poly == NOPOLYGON) poly = allocstaticpolygon(4, us_aid->cluster);
-						makedisparrayvarpoly(ni->geom, el_curwindowpart, var, poly);
-						getbbox(poly, &lx, &hx, &ly, &hy);
-						*screenlx = mini(*screenlx, lx);
-						*screenhx = maxi(*screenhx, hx);
-						*screenly = mini(*screenly, ly);
-						*screenhy = maxi(*screenhy, hy);
-						continue;
-					}
-					if ((var->textdescript&(VTXOFF|VTYOFF)) == 0) continue;
-					if (nodexfvalid == 0)
-					{
-						nodexfvalid = 1;
-						makeangle(ni->rotation, ni->transpose, trans);
-						lambda = figurelambda(ni->geom);
-					}
-					newxc = (var->textdescript&VTXOFF)>>VTXOFFSH;
-					if ((var->textdescript&VTXOFFNEG) != 0) newxc = -newxc;
-					newxc = newxc * lambda / 4;
-					newyc = (var->textdescript&VTYOFF)>>VTYOFFSH;
-					if ((var->textdescript&VTYOFFNEG) != 0) newyc = -newyc;
-					newyc = newyc * lambda / 4;
-					xform(newxc, newyc, &newxc, &newyc, trans);
-					xp = (ni->lowx + ni->highx) / 2 + newxc;
-					yp = (ni->lowy + ni->highy) / 2 + newyc;
-					*screenlx = mini(*screenlx, xp);
-					*screenhx = maxi(*screenhx, xp);
-					*screenly = mini(*screenly, yp);
-					*screenhy = maxi(*screenhy, yp);
-				}
-			}
-
-			/* include exports that have an offset */
-			for(pe = ni->firstportexpinst; pe != NOPORTEXPINST; pe = pe->nextportexpinst)
-			{
-				pp = pe->exportproto;
-				if ((pp->textdescript&(VTXOFF|VTYOFF)) == 0) continue;
-				if (nodexfvalid == 0)
-				{
-					nodexfvalid = 1;
-					makeangle(ni->rotation, ni->transpose, trans);
-					lambda = figurelambda(ni->geom);
-				}
-				newxc = (pp->textdescript&VTXOFF)>>VTXOFFSH;
-				if ((pp->textdescript&VTXOFFNEG) != 0) newxc = -newxc;
-				newxc = newxc * lambda / 4;
-				newyc = (pp->textdescript&VTYOFF)>>VTYOFFSH;
-				if ((pp->textdescript&VTYOFFNEG) != 0) newyc = -newyc;
-				newyc = newyc * lambda / 4;
-				xform(newxc, newyc, &newxc, &newyc, trans);
-				portposition(ni, pe->proto, &xp, &yp);
-				xp += newxc;
-				yp += newyc;
-				*screenlx = mini(*screenlx, xp);
-				*screenhx = maxi(*screenhx, xp);
-				*screenly = mini(*screenly, yp);
-				*screenhy = maxi(*screenhy, yp);
+				*screenlx = mini(*screenlx, nlx);
+				*screenhx = maxi(*screenhx, nhx);
+				*screenly = mini(*screenly, nly);
+				*screenhy = maxi(*screenhy, nhy);
 			}
 		}
 
 		/* include all arcs in the facet */
 		for(ai = np->firstarcinst; ai != NOARCINST; ai = ai->nextarcinst)
 		{
-			*screenlx = mini(*screenlx, ai->geom->lowx);
-			*screenhx = maxi(*screenhx, ai->geom->highx);
-			*screenly = mini(*screenly, ai->geom->lowy);
-			*screenhy = maxi(*screenhy, ai->geom->highy);
-
-			/* include any displayable variables with an offset */
-			if (el_curwindowpart != NOWINDOWPART)
-			{
-				for(i=0; i<ai->numvar; i++)
-				{
-					var = &ai->firstvar[i];
-					if ((var->type&VDISPLAY) == 0) continue;
-					if ((var->type&VISARRAY) != 0 && getlength(var) > 1)
-					{
-						if (poly == NOPOLYGON) poly = allocstaticpolygon(4, us_aid->cluster);
-						makedisparrayvarpoly(ai->geom, el_curwindowpart, var, poly);
-						getbbox(poly, &lx, &hx, &ly, &hy);
-						*screenlx = mini(*screenlx, lx);
-						*screenhx = maxi(*screenhx, hx);
-						*screenly = mini(*screenly, ly);
-						*screenhy = maxi(*screenhy, hy);
-						continue;
-					}
-					if ((var->textdescript&(VTXOFF|VTYOFF)) == 0) continue;
-					lambda = figurelambda(ai->geom);
-					newxc = (var->textdescript&VTXOFF)>>VTXOFFSH;
-					if ((var->textdescript&VTXOFFNEG) != 0) newxc = -newxc;
-					newxc = newxc * lambda / 4;
-					newyc = (var->textdescript&VTYOFF)>>VTYOFFSH;
-					if ((var->textdescript&VTYOFFNEG) != 0) newyc = -newyc;
-					newyc = newyc * lambda / 4;
-					xp = (ai->geom->lowx + ai->geom->highx) / 2 + newxc;
-					yp = (ai->geom->lowy + ai->geom->highy) / 2 + newyc;
-					*screenlx = mini(*screenlx, xp);
-					*screenhx = maxi(*screenhx, xp);
-					*screenly = mini(*screenly, yp);
-					*screenhy = maxi(*screenhy, yp);
-				}
-			}
+			us_getarcbounds(ai, &nlx, &nhx, &nly, &nhy);
+			*screenlx = mini(*screenlx, nlx);
+			*screenhx = maxi(*screenhx, nhx);
+			*screenly = mini(*screenly, nly);
+			*screenhy = maxi(*screenhy, nhy);
 		}
 
 		/* set default size if nothing is there */
@@ -336,6 +228,157 @@ void us_fullview(NODEPROTO *np, INTBIG *screenlx, INTBIG *screenhx, INTBIG *scre
 	}
 }
 
+/*
+ * Routine to determine the bounds of node "ni" and return it in "nlx", "nhx",
+ * "nly", and "nhy".
+ */
+void us_getnodebounds(NODEINST *ni, INTBIG *nlx, INTBIG *nhx, INTBIG *nly, INTBIG *nhy)
+{
+	REGISTER INTBIG i, nodexfvalid, lambda, font, shei, swid, tf, portstyle;
+	INTBIG lx, hx, ly, hy, xp, yp, newxc, newyc;
+	INTSML wid, hei;
+	REGISTER VARIABLE *var;
+	XARRAY trans;
+	REGISTER PORTEXPINST *pe;
+	REGISTER PORTPROTO *pp;
+	static POLYGON *poly = NOPOLYGON;
+
+	*nlx = ni->geom->lowx;
+	*nhx = ni->geom->highx;
+	*nly = ni->geom->lowy;
+	*nhy = ni->geom->highy;
+
+	/* include any displayable variables with an offset or array */
+	if (el_curwindowpart != NOWINDOWPART)
+	{
+		for(i=0; i<ni->numvar; i++)
+		{
+			var = &ni->firstvar[i];
+			if ((var->type&VDISPLAY) == 0) continue;
+
+			if (poly == NOPOLYGON) poly = allocstaticpolygon(4, us_aid->cluster);
+			makedisparrayvarpoly(ni->geom, el_curwindowpart, var, poly);
+			getbbox(poly, &lx, &hx, &ly, &hy);
+			*nlx = mini(*nlx, lx);
+			*nhx = maxi(*nhx, hx);
+			*nly = mini(*nly, ly);
+			*nhy = maxi(*nhy, hy);
+		}
+	}
+
+	/* include exports */
+	nodexfvalid = 0;
+	for(pe = ni->firstportexpinst; pe != NOPORTEXPINST; pe = pe->nextportexpinst)
+	{
+		pp = pe->exportproto;
+		if (nodexfvalid == 0)
+		{
+			nodexfvalid = 1;
+			makeangle(ni->rotation, ni->transpose, trans);
+			lambda = figurelambda(ni->geom);
+		}
+		newxc = (pp->textdescript&VTXOFF)>>VTXOFFSH;
+		if ((pp->textdescript&VTXOFFNEG) != 0) newxc = -newxc;
+		newxc = newxc * lambda / 4;
+		newyc = (pp->textdescript&VTYOFF)>>VTYOFFSH;
+		if ((pp->textdescript&VTYOFFNEG) != 0) newyc = -newyc;
+		newyc = newyc * lambda / 4;
+		xform(newxc, newyc, &newxc, &newyc, trans);
+		portposition(ni, pe->proto, &xp, &yp);
+		xp += newxc;
+		yp += newyc;
+		lx = hx = xp;   ly = hy = yp;
+		portstyle = us_useroptions & EXPORTLABELS;
+		if (el_curwindowpart != NOWINDOWPART &&
+			(portstyle == EXPORTSFULL || portstyle == EXPORTSSHORT))
+		{
+			/* adjust for size of port text */
+			font = (pp->textdescript&VTSIZE)>>VTSIZESH;
+			if (font >= TXTHL && font <= TXT6L)
+			{
+				screensettextsize(el_curwindowpart, 20);
+				screengettextsize(el_curwindowpart,
+					us_displayedportname(pp, portstyle >> EXPORTLABELSSH), &wid, &hei);
+				switch (font)
+				{
+					case TXTHL: shei = lambda / 2;   break;
+					case TXT1L: shei = lambda;       break;
+					case TXT2L: shei = lambda * 2;   break;
+					case TXT3L: shei = lambda * 3;   break;
+					case TXT4L: shei = lambda * 4;   break;
+					case TXT5L: shei = lambda * 5;   break;
+					case TXT6L: shei = lambda * 6;   break;
+				}
+				swid = shei * wid / hei * 5 / 4;
+			} else
+			{
+				tf = truefontsize(font, el_curwindowpart, pp->parent->tech);
+				screensettextsize(el_curwindowpart, tf);
+				screengettextsize(el_curwindowpart,
+					us_displayedportname(pp, portstyle >> EXPORTLABELSSH), &wid, &hei);
+				swid = roundfloat((float)wid / el_curwindowpart->scalex);
+				shei = roundfloat((float)hei / el_curwindowpart->scaley);
+			}
+			switch (pp->textdescript&VTPOSITION)
+			{
+				case VTPOSCENT:     case VTPOSDOWN:       case VTPOSUP:
+					lx -= swid/2;   hx += swid/2;         break;
+				case VTPOSRIGHT:    case VTPOSDOWNRIGHT:  case VTPOSUPRIGHT:
+					hx += swid;     break;
+				case VTPOSLEFT:     case VTPOSDOWNLEFT:   case VTPOSUPLEFT:
+					lx -= swid;     break;
+			}
+			switch (pp->textdescript&VTPOSITION)
+			{
+				case VTPOSCENT:     case VTPOSRIGHT:      case VTPOSLEFT:
+					ly -= shei/2;   hy += shei/2;         break;
+				case VTPOSDOWN:     case VTPOSDOWNRIGHT:  case VTPOSDOWNLEFT:
+					ly -= shei;     break;
+				case VTPOSUP:       case VTPOSUPRIGHT:    case VTPOSUPLEFT:
+					hy += shei;     break;
+			}
+		}
+		*nlx = mini(*nlx, lx);
+		*nhx = maxi(*nhx, hx);
+		*nly = mini(*nly, ly);
+		*nhy = maxi(*nhy, hy);
+	}
+}
+
+/*
+ * Routine to determine the bounds of node "ni" and return it in "nlx", "nhx",
+ * "nly", and "nhy".
+ */
+void us_getarcbounds(ARCINST *ai, INTBIG *nlx, INTBIG *nhx, INTBIG *nly, INTBIG *nhy)
+{
+	REGISTER INTBIG i;
+	INTBIG lx, hx, ly, hy;
+	REGISTER VARIABLE *var;
+	static POLYGON *poly = NOPOLYGON;
+
+	*nlx = ai->geom->lowx;
+	*nhx = ai->geom->highx;
+	*nly = ai->geom->lowy;
+	*nhy = ai->geom->highy;
+
+	/* include any displayable variables with an offset */
+	if (el_curwindowpart != NOWINDOWPART)
+	{
+		for(i=0; i<ai->numvar; i++)
+		{
+			var = &ai->firstvar[i];
+			if ((var->type&VDISPLAY) == 0) continue;
+			if (poly == NOPOLYGON) poly = allocstaticpolygon(4, us_aid->cluster);
+			makedisparrayvarpoly(ai->geom, el_curwindowpart, var, poly);
+			getbbox(poly, &lx, &hx, &ly, &hy);
+			*nlx = mini(*nlx, lx);
+			*nhx = maxi(*nhx, hx);
+			*nly = mini(*nly, ly);
+			*nhy = maxi(*nhy, hy);
+		}
+	}
+}
+
 /*
  * routine to adjust "screenlx/hx/ly/hy" for window "w" so that it defines
  * a square screen and leaves some extra space around the values.
@@ -401,10 +444,10 @@ void us_squarescreen(WINDOWPART *w, WINDOWPART *formerw, INTSML largescale, INTB
 		}
 
 		/* put it back into the integer extent fields */
-		if (fslx < -HUGEINT/2) *screenlx = -HUGEINT/2; else *screenlx = (INTBIG)fslx;
-		if (fshx >  HUGEINT/2) *screenhx =  HUGEINT/2; else *screenhx = (INTBIG)fshx;
-		if (fsly < -HUGEINT/2) *screenly = -HUGEINT/2; else *screenly = (INTBIG)fsly;
-		if (fshy >  HUGEINT/2) *screenhy =  HUGEINT/2; else *screenhy = (INTBIG)fshy;
+		if (fslx < -MAXINTBIG/2) *screenlx = -MAXINTBIG/2; else *screenlx = (INTBIG)fslx;
+		if (fshx >  MAXINTBIG/2) *screenhx =  MAXINTBIG/2; else *screenhx = (INTBIG)fshx;
+		if (fsly < -MAXINTBIG/2) *screenly = -MAXINTBIG/2; else *screenly = (INTBIG)fsly;
+		if (fshy >  MAXINTBIG/2) *screenhy =  MAXINTBIG/2; else *screenhy = (INTBIG)fshy;
 	}
 
 	if ((us_aid->aidstate&INTEGRAL) != 0)
@@ -468,7 +511,6 @@ WINDOWPART *us_subwindow(INTBIG lx, INTBIG hx, INTBIG ly, INTBIG hy, WINDOWPART
 	return(&ww);
 }
 
-
 /*
  * routine to erase window "w" and draw everything that should be in it.
  */
@@ -1078,13 +1120,13 @@ void us_gridset(WINDOWPART *w, INTBIG state)
 	{
 		if ((w->state&GRIDTOOSMALL) == 0)
 		{
-			ttyputverbose(_("Grid too small, turned off"));
+			ttyputverbose(M_("Grid too small, turned off"));
 			(void)setval((INTBIG)w, VWINDOWPART, "state", w->state | GRIDTOOSMALL | GRIDON, VINTEGER);
 		}
 		return;
 	}
 
-	if ((w->state&GRIDTOOSMALL) != 0) ttyputverbose(_("Grid turned back on"));
+	if ((w->state&GRIDTOOSMALL) != 0) ttyputverbose(M_("Grid turned back on"));
 	(void)setval((INTBIG)w, VWINDOWPART, "state", (w->state & ~GRIDTOOSMALL) | GRIDON, VINTEGER);
 }
 
@@ -1093,7 +1135,7 @@ void us_gridset(WINDOWPART *w, INTBIG state)
 /*
  * routine to identify the portinst locations by number in a nodeinst
  */
-void us_identifyports(NODEINST *ni, NODEPROTO *np, INTSML on)
+void us_identifyports(NODEINST *ni, NODEPROTO *np, INTSML on, INTSML usenumbers)
 {
 	REGISTER PORTPROTO *pp, **portpp, *bestpp, *subpp;
 	REGISTER NODEPROTO *parent;
@@ -1148,8 +1190,15 @@ void us_identifyports(NODEINST *ni, NODEPROTO *np, INTSML on)
 	}
 	if (total == 0)
 	{
-		if (ignored <= 0) ttyputmsg(_("There are no ports on facet %s"), describenodeproto(np)); else
-			ttyputmsg(_("All %ld ports are outside of the window"), ignored);			
+		if (ignored <= 0)
+		{
+			if (ni == NONODEINST)
+				ttyputmsg(_("There are no ports on facet %s"), describenodeproto(np)); else
+					ttyputmsg(_("There are no ports on node %s"), describenodeinst(ni));
+		} else
+		{
+			ttyputmsg(_("All %ld ports are outside of the window"), ignored);
+		}
 		return;
 	}
 
@@ -1238,20 +1287,22 @@ void us_identifyports(NODEINST *ni, NODEPROTO *np, INTSML on)
 		{
 			subni = portpp[i]->subnodeinst;
 			subpp = portpp[i]->subportproto;
-			poly->string = portpp[i]->protoname;
 		} else
 		{
 			subni = ni;
 			subpp = portpp[i];
+		}
+		if (usenumbers != 0)
+		{
 			(void)sprintf(line, "%d", portused[i]);
 			poly->string = line;
-		}
+		} else poly->string = portpp[i]->protoname;
 
 		/* draw the port index number */
 		poly->xv[0] = portx[i];   poly->yv[0] = porty[i];   poly->count = 1;
 		poly->desc = &us_hbox;
 		poly->style = TEXTCENT;
-		poly->font = TXTLARGE;
+		poly->font = TXT20P;
 		poly->tech = subni->proto->tech;
 		us_hbox.col = HIGHLIT&on;
 		(void)us_showpoly(poly, w);
@@ -1327,10 +1378,10 @@ INTSML us_drawfacet(NODEINST *ni, INTSML on, XARRAY prevtrans, INTSML layers, WI
  */
 INTSML us_drawnodeinst(NODEINST *ni, INTSML on, XARRAY prevtrans, INTSML layers, WINDOWPART *w)
 {
-	REGISTER INTBIG i, j, res, displaytotal, dontdraw, low, high, cutsizex, cutsizey;
+	REGISTER INTBIG i, j, res, displaytotal, low, high, cutsizex, cutsizey, portstyle;
 	INTSML color, bits, moretodo;
 	XARRAY localtran, subrot, trans;
-	INTBIG bx, by, ux, uy, portcliplx, portcliphx, portcliply, portcliphy, drawport, saveopt,
+	INTBIG bx, by, ux, uy, portcliplx, portcliphx, portcliply, portcliphy, drawport,
 		reasonable;
 	REGISTER INTBIG ratio, swap, lambda;
 	static POLYGON *poly = NOPOLYGON;
@@ -1376,76 +1427,59 @@ INTSML us_drawnodeinst(NODEINST *ni, INTSML on, XARRAY prevtrans, INTSML layers,
 	/* primitive nodeinst: ask the technology how to draw it */
 	if (np->primindex != 0)
 	{
-		/* do not draw displayable text that is in a subfacet and only appears in its facet */
-		dontdraw = 0;
-		if (np == gen_invispinprim && ni->parent != w->curnodeproto)
+		i = nodepolys(ni, &reasonable, w);
+
+		/* if the amount of geometry can be reduced, do so at large scales */
+		if (i != reasonable)
 		{
-			for(i=0; i<ni->numvar; i++)
-			{
-				var = &ni->firstvar[i];
-				if ((var->type&VDISPLAY) != 0 &&
-					(var->textdescript&VTINTERIOR) != 0) break;
-			}
-			if (i < ni->numvar) dontdraw = 1;
+			cutsizex = applyxscale(w, ni->highx - ni->lowx);
+			cutsizey = applyyscale(w, ni->highy - ni->lowy);
+			if (cutsizex*cutsizey < i * 75) i = reasonable;
 		}
-		if (dontdraw == 0)
+		if ((us_state&NONOVERLAPPABLEDISPLAY) != 0)
 		{
-			downhierarchy(ni);
-			i = nodepolys(ni, &reasonable);
-
-			/* if the amount of geometry can be reduced, do so at large scales */
-			if (i != reasonable)
+			/* no overlappable display: just draw everything */
+			low = 0;   high = i;
+		} else
+		{
+			if ((np->userbits&NHASOPA) == 0) j = i; else
 			{
-				cutsizex = applyxscale(w, ni->highx - ni->lowx);
-				cutsizey = applyyscale(w, ni->highy - ni->lowy);
-				if (cutsizex*cutsizey < i * 75) i = reasonable;
-			}
-			if ((us_state&NONOVERLAPPABLEDISPLAY) != 0)
-			{
-				/* no overlappable display: just draw everything */
-				low = 0;   high = i;
-			} else
-			{
-				if ((np->userbits&NHASOPA) == 0) j = i; else
-				{
-					j = (INTSML)((np->userbits&NFIRSTOPA) >> NFIRSTOPASH);
-					if (i < j) j = i;
-				}
-				if ((layers&1) != 0) low = 0; else low = j;
-				if ((layers&2) != 0) high = i; else high = j;
-				if (low > 0) moretodo |= 1;
-				if (high < i) moretodo |= 2;
+				j = (INTSML)((np->userbits&NFIRSTOPA) >> NFIRSTOPASH);
+				if (i < j) j = i;
 			}
+			if ((layers&1) != 0) low = 0; else low = j;
+			if ((layers&2) != 0) high = i; else high = j;
+			if (low > 0) moretodo |= 1;
+			if (high < i) moretodo |= 2;
+		}
 
-			/* don't draw invisible pins to alternative output */
-			if (us_displayroutine != us_showpoly)
-				if (np == gen_invispinprim && low == 0) low++;
+		/* don't draw invisible pins to alternative output */
+		if (us_displayroutine != us_showpoly)
+			if (np == gen_invispinprim && low == 0) low++;
 
-			for(j=low; j<high; j++)
-			{
-				/* get description of this layer */
-				shapenodepoly(ni, j, poly);
+		for(j=low; j<high; j++)
+		{
+			/* get description of this layer */
+			shapenodepoly(ni, j, poly);
 
-				/* ignore if this layer is not being displayed */
-				gra = poly->desc;
-				if ((gra->colstyle&INVISIBLE) != 0) continue;
+			/* ignore if this layer is not being displayed */
+			gra = poly->desc;
+			if ((gra->colstyle&INVISIBLE) != 0) continue;
 
-				/* ignore if no bits are to be drawn */
-				if ((on&gra->col) == 0 && on != 0) continue;
+			/* ignore if no bits are to be drawn */
+			if ((on&gra->col) == 0 && on != 0) continue;
 
-				/* draw the nodeinst */
-				xformpoly(poly, prevtrans);
+			/* draw the nodeinst */
+			xformpoly(poly, prevtrans);
 
-				/* save the color information and update it */
-				i = gra->col;
-				gra->col &= on;
+			/* save the color information and update it */
+			i = gra->col;
+			gra->col &= on;
 
-				/* draw the nodeinst and restore the color */
-				if ((w->state&WINDOWTYPE) == DISP3DWINDOW) us_3dshowpoly(poly, w); else
-					(*us_displayroutine)(poly, w);
-				gra->col = (INTSML)i;
-			}
-			uphierarchy();
+			/* draw the nodeinst and restore the color */
+			if ((w->state&WINDOWTYPE) == DISP3DWINDOW) us_3dshowpoly(poly, w); else
+				(*us_displayroutine)(poly, w);
+			gra->col = (INTSML)i;
 		}
 	} else
 	{
@@ -1518,10 +1552,10 @@ INTSML us_drawnodeinst(NODEINST *ni, INTSML on, XARRAY prevtrans, INTSML layers,
 					(*us_displayroutine)(poly, w);
 
 				/* see if there are displayable variables on the facet */
-				displaytotal = tech_displayablenvars(ni);
+				displaytotal = tech_displayablenvars(ni, w);
 				for(i = 0; i < displaytotal; i++)
 				{
-					(void)tech_filldisplayablenvar(ni, poly);
+					(void)tech_filldisplayablenvar(ni, poly, w, 0);
 					xformpoly(poly, prevtrans);
 					if ((w->state&WINDOWTYPE) == DISP3DWINDOW) us_3dshowpoly(poly, w); else
 						(*us_displayroutine)(poly, w);
@@ -1559,16 +1593,17 @@ INTSML us_drawnodeinst(NODEINST *ni, INTSML on, XARRAY prevtrans, INTSML layers,
 				if ((bits&LAYEROE) == 0 && (layers&1) == 0)
 				{ moretodo |= 1;  continue; }
 
+				portstyle = (us_useroptions&PORTLABELS) >> PORTLABELSSH;
 				if (pp->subnodeinst->rotation == 0 && pp->subnodeinst->transpose == 0)
 				{
 					us_writeprotoname(pp, LAYERA, subrot, bits, color, w, portcliplx,
-						portcliphx, portcliply, portcliphy);
+						portcliphx, portcliply, portcliphy, portstyle);
 				} else
 				{
 					makerot(pp->subnodeinst, localtran);
 					transmult(localtran, subrot, trans);
 					us_writeprotoname(pp, LAYERA, trans, bits, color, w, portcliplx,
-						portcliphx, portcliply, portcliphy);
+						portcliphx, portcliply, portcliphy, portstyle);
 				}
 			}
 		} else
@@ -1603,8 +1638,8 @@ INTSML us_drawnodeinst(NODEINST *ni, INTSML on, XARRAY prevtrans, INTSML layers,
 				for(pp = ni->proto->firstportproto; pp != NOPORTPROTO; pp = pp->nextportproto)
 				{
 					drawport = 0;
+					portstyle = (us_useroptions&PORTLABELS) >> PORTLABELSSH;
 					if ((pp->userbits&PORTDRAWN) != 0) drawport = 1;
-					saveopt = us_useroptions;
 					for(pi = ni->firstportarcinst; pi != NOPORTARCINST; pi = pi->nextportarcinst)
 						if (pi->proto == pp) break;
 					for(pe = ni->firstportexpinst; pe != NOPORTEXPINST; pe = pe->nextportexpinst)
@@ -1612,7 +1647,7 @@ INTSML us_drawnodeinst(NODEINST *ni, INTSML on, XARRAY prevtrans, INTSML layers,
 					if (pi == NOPORTARCINST && pe == NOPORTEXPINST)
 					{
 						if (drawport == 0)
-							us_useroptions = (us_useroptions & ~PORTLABELS) | PORTSCROSS;
+							portstyle = PORTSCROSS >> PORTLABELSSH;
 						drawport = 1;
 					}
 					if (drawport != 0)
@@ -1620,23 +1655,22 @@ INTSML us_drawnodeinst(NODEINST *ni, INTSML on, XARRAY prevtrans, INTSML layers,
 						if (pp->subnodeinst->rotation == 0 && pp->subnodeinst->transpose == 0)
 						{
 							us_writeprotoname(pp, LAYERA, subrot, LAYERO, el_colfacettxt, w,
-								portcliplx, portcliphx, portcliply, portcliphy);
+								portcliplx, portcliphx, portcliply, portcliphy, portstyle);
 						} else
 						{
 							makerot(pp->subnodeinst, localtran);
 							transmult(localtran, subrot, trans);
 							us_writeprotoname(pp, LAYERA, trans, LAYERO, el_colfacettxt, w,
-								portcliplx, portcliphx, portcliply, portcliphy);
+								portcliplx, portcliphx, portcliply, portcliphy, portstyle);
 						}
-						us_useroptions = saveopt;
 					}
 				}
 
 				/* see if there are displayable variables on the facet instance */
-				displaytotal = tech_displayablenvars(ni);
+				displaytotal = tech_displayablenvars(ni, w);
 				for(i = 0; i < displaytotal; i++)
 				{
-					(void)tech_filldisplayablenvar(ni, poly);
+					(void)tech_filldisplayablenvar(ni, poly, w, 0);
 					xformpoly(poly, prevtrans);
 					if ((w->state&WINDOWTYPE) == DISP3DWINDOW) us_3dshowpoly(poly, w); else
 						(*us_displayroutine)(poly, w);
@@ -1673,7 +1707,8 @@ INTSML us_drawnodeinst(NODEINST *ni, INTSML on, XARRAY prevtrans, INTSML layers,
 			for(pe = ni->firstportexpinst; pe != NOPORTEXPINST; pe = pe->nextportexpinst)
 			{
 				us_writeprotoname(pe->exportproto, on, prevtrans, LAYERO,
-					(INTSML)(el_colfacettxt&on), w, 0, 0, 0, 0);
+					(INTSML)(el_colfacettxt&on), w, 0, 0, 0, 0,
+						(us_useroptions&EXPORTLABELS)>>EXPORTLABELSSH);
 				us_drawportprotovariables(pe->exportproto, on, prevtrans, w);
 			}
 		}
@@ -1689,10 +1724,10 @@ void us_drawnodeprotovariables(NODEPROTO *np, XARRAY trans, WINDOWPART *w)
 	/* get polygon */
 	if (poly == NOPOLYGON) poly = allocstaticpolygon(4, us_aid->cluster);
 
-	displaytotal = tech_displayablefacetvars(np);
+	displaytotal = tech_displayablefacetvars(np, w);
 	for(i = 0; i < displaytotal; i++)
 	{
-		(void)tech_filldisplayablefacetvar(np, poly);
+		(void)tech_filldisplayablefacetvar(np, poly, w, 0);
 		xformpoly(poly, trans);
 		if ((w->state&WINDOWTYPE) == DISP3DWINDOW) us_3dshowpoly(poly, w); else
 			(*us_displayroutine)(poly, w);
@@ -1707,10 +1742,10 @@ void us_drawportprotovariables(PORTPROTO *pp, INTSML on, XARRAY trans, WINDOWPAR
 	/* get polygon */
 	if (poly == NOPOLYGON) poly = allocstaticpolygon(4, us_aid->cluster);
 
-	displaytotal = tech_displayableportvars(pp);
+	displaytotal = tech_displayableportvars(pp, w);
 	for(i = 0; i < displaytotal; i++)
 	{
-		(void)tech_filldisplayableportvar(pp, poly);
+		(void)tech_filldisplayableportvar(pp, poly, w, 0);
 		xformpoly(poly, trans);
 		if (on == 0)
 		{
@@ -1774,7 +1809,7 @@ void us_combinelayers(ARCINST *ai, INTSML *bitplanes, INTSML *color)
 	/* get polygon */
 	if (poly == NOPOLYGON) poly = allocstaticpolygon(4, us_aid->cluster);
 
-	k = arcpolys(ai);
+	k = arcpolys(ai, NOWINDOWPART);
 	for(j=0; j<k; j++)
 	{
 		shapearcpoly(ai, j, poly);
@@ -1805,7 +1840,7 @@ INTSML us_drawarcinst(ARCINST *ai, INTSML on, XARRAY trans, INTSML layers, WINDO
 	if (poly == NOPOLYGON) poly = allocstaticpolygon(4, us_aid->cluster);
 
 	ap = ai->proto;
-	i = arcpolys(ai);
+	i = arcpolys(ai, NOWINDOWPART);
 	moretodo = 0;
 	if ((us_state&NONOVERLAPPABLEDISPLAY) != 0)
 	{
@@ -1854,46 +1889,40 @@ INTSML us_drawarcinst(ARCINST *ai, INTSML on, XARRAY trans, INTSML layers, WINDO
  * written if "on" is nonzero, erased if zero.  The port is written in window
  * "w".  If "portcliplx" is not equal to "portcliphx", then clip the port text
  * to within that range in X and to "portcliply/portcliphy" in Y.
- * The style of the port is taken from the "us_useroptions&PORTLABELS" field:
- * PORTSOFF       no port indication drawn
- * PORTSCROSS     port drawn as a cross
- * PORTSFULL      full port name written textually (regardless of size)
- * PORTSSHORT     short port name written textually (regardless of size)
+ * The "dispstyle" of the port is:
+ *   0  full port name written
+ *   1  port drawn as a cross
+ *   2  no port indication drawn
+ *   3  short port name written
  */
 void us_writeprotoname(PORTPROTO *pp, INTSML on, XARRAY prevtrans, INTSML bits,
 	INTSML color, WINDOWPART *w, INTBIG portcliplx, INTBIG portcliphx, INTBIG portcliply,
-	INTBIG portcliphy)
+	INTBIG portcliphy, INTBIG dispstyle)
 {
 	XARRAY localtran, tempt1, tempt2, *t1, *t2, *swapt;
 	INTBIG px, py;
 	INTSML wid, hei;
-	REGISTER INTBIG sx, sy, lambda;
-	REGISTER char *pt;
+	REGISTER INTBIG sx, sy, lambda, tf;
 	static POLYGON *poly = NOPOLYGON;
 	REGISTER NODEINST *ni;
+	REGISTER NODEPROTO *facet;
 	REGISTER PORTPROTO *rpp;
 	REGISTER TECHNOLOGY *tech;
 
 	/* quit now if labels are off */
-	if ((us_useroptions&PORTLABELS) == PORTSOFF) return;
+	if (dispstyle == 2) return;
 
 	/* get polygon */
 	if (poly == NOPOLYGON) poly = allocstaticpolygon(4, us_aid->cluster);
 
-	/* find the most primitive node to get its technology */
-	ni = pp->subnodeinst;
-	rpp = pp->subportproto;
-	while (ni->proto->primindex == 0)
-	{
-		ni = rpp->subnodeinst;
-		rpp = rpp->subportproto;
-	}
-	tech = ni->proto->tech;
-	lambda = lambdaofnode(ni);
+	/* get technology and lambda */
+	facet = pp->subnodeinst->parent;
+	tech = facet->tech;
+	lambda = facet->cell->lib->lambda[tech->techindex];
 
 	/* account for port offset */
 	t1 = (XARRAY *)tempt1;   t2 = (XARRAY *)tempt2;
-	if ((us_useroptions&PORTLABELS) != PORTSCROSS && (pp->textdescript&(VTXOFF|VTYOFF)) != 0)
+	if (dispstyle != 1 && (pp->textdescript&(VTXOFF|VTYOFF)) != 0)
 	{
 		transid(localtran);
 		localtran[2][0] = (pp->textdescript&VTXOFF)>>VTXOFFSH;
@@ -1921,8 +1950,8 @@ void us_writeprotoname(PORTPROTO *pp, INTSML on, XARRAY prevtrans, INTSML bits,
 			transmult(localtran, *t1, *t2);
 			swapt = t1;   t1 = t2;   t2 = swapt;
 		}
-		if ((us_useroptions&PORTLABELS) != PORTSCROSS &&
-			ni->proto->primindex == 0 && (rpp->textdescript&(VTXOFF|VTYOFF)) != 0)
+		if (dispstyle != 1 && ni->proto->primindex == 0 &&
+			(rpp->textdescript&(VTXOFF|VTYOFF)) != 0)
 		{
 			transid(localtran);
 			localtran[2][0] = (rpp->textdescript&VTXOFF)>>VTXOFFSH;
@@ -1938,20 +1967,18 @@ void us_writeprotoname(PORTPROTO *pp, INTSML on, XARRAY prevtrans, INTSML bits,
 
 	/* get the center position of the port */
 	shapetransportpoly(ni, rpp, poly, *t1);
-	if ((us_useroptions&PORTLABELS) == PORTSCROSS ||
-		(pp->textdescript&VTPOSITION) != VTPOSBOXED)
+	if (dispstyle == 1 || (pp->textdescript&VTPOSITION) != VTPOSBOXED)
 	{
 		getcenter(poly, &px, &py);
 		poly->xv[0] = px;   poly->yv[0] = py;   poly->count = 1;
 	}
 
 	/* simple description if only drawing ports as crosses */
-	if ((us_useroptions&PORTLABELS) == PORTSCROSS) poly->style = CROSS; else
+	if (dispstyle == 1) poly->style = CROSS; else
 	{
 		/* writing text name: setup polygon */
 		poly->font = (INTSML)((pp->textdescript & VTSIZE) >> VTSIZESH);
-		poly->font = truefontsize(poly->font, w, tech);
-		poly->tech = ni->proto->tech;
+		poly->tech = pp->parent->tech;
 
 		/* convert text description to polygon description */
 		switch (pp->textdescript&VTPOSITION)
@@ -1969,17 +1996,7 @@ void us_writeprotoname(PORTPROTO *pp, INTSML on, XARRAY prevtrans, INTSML bits,
 		}
 
 		/* get shortened name if requested */
-		poly->string = pp->protoname;
-		if ((us_useroptions&PORTLABELS) == PORTSSHORT)
-		{
-			for(pt = pp->protoname; *pt != 0; pt++) if (ispunct(*pt)) break;
-			if (*pt != 0)
-			{
-				(void)initinfstr();
-				for(pt = pp->protoname; !ispunct(*pt); pt++) (void)addtoinfstr(*pt);
-				poly->string = returninfstr();
-			}
-		}
+		poly->string = us_displayedportname(pp, dispstyle);
 
 		/* acount for node rotation, which port position */
 		poly->style = rotatelabel(poly->style, prevtrans);
@@ -1991,7 +2008,8 @@ void us_writeprotoname(PORTPROTO *pp, INTSML on, XARRAY prevtrans, INTSML bits,
 			sy = applyyscale(w, py-w->screenly) + w->usely;
 			if (sx < portcliplx || sx > portcliphx || sy < portcliply || sy > portcliphy)
 				return;
-			screensettextsize(w, poly->font);
+			tf = truefontsize(poly->font, w, tech);
+			screensettextsize(w, tf);
 			screengettextsize(w, poly->string, &wid, &hei);   wid++;   hei++;
 			switch (poly->style)
 			{
@@ -2068,6 +2086,27 @@ void us_writeprotoname(PORTPROTO *pp, INTSML on, XARRAY prevtrans, INTSML bits,
 		(*us_displayroutine)(poly, w);
 }
 
+/*
+ * routine to return the displayed port name for port "pp", given a style of "dispstyle"
+ * (0: full, 1: crosses, 2: nothing, 3: short)
+ */
+char *us_displayedportname(PORTPROTO *pp, INTBIG dispstyle)
+{
+	REGISTER char *pt;
+
+	if (dispstyle == 3)
+	{
+		for(pt = pp->protoname; *pt != 0; pt++) if (ispunct(*pt)) break;
+		if (*pt != 0)
+		{
+			(void)initinfstr();
+			for(pt = pp->protoname; !ispunct(*pt); pt++) (void)addtoinfstr(*pt);
+			return(returninfstr());
+		}
+	}
+	return(pp->protoname);
+}
+
 /******************** PEEK DISPLAY ********************/
 
 WINDOWPART *us_peekwindow;
@@ -2133,7 +2172,6 @@ INTSML us_drawall(INTBIG lx, INTBIG hx, INTBIG ly, INTBIG hy, NODEPROTO *np, XAR
 				makerot(ni, localrot);   transmult(localrot, trans, thist);
 				tthis = (XARRAY *)thist;
 			}
-			downhierarchy(ni);
 			if (ni->proto->primindex != 0)
 			{
 				/* node is primitive: draw it */
@@ -2142,6 +2180,7 @@ INTSML us_drawall(INTBIG lx, INTBIG hx, INTBIG ly, INTBIG hy, NODEPROTO *np, XAR
 			} else
 			{
 				/* node is complex: recurse */
+				downhierarchy(ni);
 				subnt = ni->proto;
 				maketransI(ni, localtrans);
 				if (ni->rotation == 0 && ni->transpose == 0)
@@ -2162,8 +2201,8 @@ INTSML us_drawall(INTBIG lx, INTBIG hx, INTBIG ly, INTBIG hy, NODEPROTO *np, XAR
 				transmult(localtrans, *tthis, subrot);
 				i = us_drawall(newlx, newhx, newly, newhy, subnt, subrot, layers);
 				if (i != -1) moretodo |= i;
+				uphierarchy();
 			}
-			uphierarchy();
 		}
 	}
 	if (stopping(STOPREASONDISPLAY)) return(-1);
@@ -2183,7 +2222,7 @@ INTSML us_drawarcinstpeek(ARCINST *ai, XARRAY trans, INTSML layers)
 
 	/* determine which polygons to draw */
 	ap = ai->proto;
-	i = arcpolys(ai);
+	i = arcpolys(ai, us_peekwindow);
 	moretodo = 0;
 	if ((us_state&NONOVERLAPPABLEDISPLAY) != 0)
 	{
@@ -2229,7 +2268,6 @@ INTSML us_drawnodeinstpeek(NODEINST *ni, XARRAY prevtrans, INTSML layers)
 	REGISTER INTBIG i, j, low, high, cutsizex, cutsizey;
 	INTSML moretodo;
 	INTBIG reasonable;
-	REGISTER VARIABLE *var;
 	static POLYGON *poly = NOPOLYGON;
 	REGISTER GRAPHICS *gra;
 	REGISTER NODEPROTO *np;
@@ -2238,18 +2276,9 @@ INTSML us_drawnodeinstpeek(NODEINST *ni, XARRAY prevtrans, INTSML layers)
 	if (poly == NOPOLYGON) poly = allocstaticpolygon(4, us_aid->cluster);
 
 	np = ni->proto;
-	if (np == gen_invispinprim && ni->parent != us_peekwindow->curnodeproto)
-	{
-		for(i=0; i<ni->numvar; i++)
-		{
-			var = &ni->firstvar[i];
-			if ((var->type&VDISPLAY) != 0 &&
-				(var->textdescript&VTINTERIOR) != 0) return(0);
-		}
-	}
 
 	/* determine which polygons to draw */
-	i = nodepolys(ni, &reasonable);
+	i = nodepolys(ni, &reasonable, us_peekwindow);
 
 	/* if the amount of geometry can be reduced, do so at large scales */
 	if (i != reasonable)
