@@ -75,38 +75,38 @@
 #include "usr.h"
 #include "usreditemacs.h"
 
-#define	CTLA   01			/* ^A */
-#define	CTLB   02			/* ^B */
-#define	CTLC   03			/* ^C */
-#define	CTLD   04			/* ^D */
-#define	CTLE   05			/* ^E */
-#define	CTLF   06			/* ^F */
-#define	CTLG   07			/* ^G */
-#define	CTLH  010			/* ^H */
-#define	CTLK  013			/* ^K */
-#define	CTLL  014			/* ^L */
-#define	CTLN  016			/* ^N */
-#define	CTLO  017			/* ^O */
-#define	CTLP  020			/* ^P */
-#define	CTLR  022			/* ^R */
-#define	CTLS  023			/* ^S */
-#define	CTLV  026			/* ^V */
-#define	CTLW  027			/* ^W */
-#define	CTLX  030			/* ^X */
-#define	CTLY  031			/* ^Y */
-#define	CTLZ  032			/* ^Z */
+#define CTLA   01			/* ^A */
+#define CTLB   02			/* ^B */
+#define CTLC   03			/* ^C */
+#define CTLD   04			/* ^D */
+#define CTLE   05			/* ^E */
+#define CTLF   06			/* ^F */
+#define CTLG   07			/* ^G */
+#define CTLH  010			/* ^H */
+#define CTLK  013			/* ^K */
+#define CTLL  014			/* ^L */
+#define CTLN  016			/* ^N */
+#define CTLO  017			/* ^O */
+#define CTLP  020			/* ^P */
+#define CTLR  022			/* ^R */
+#define CTLS  023			/* ^S */
+#define CTLV  026			/* ^V */
+#define CTLW  027			/* ^W */
+#define CTLX  030			/* ^X */
+#define CTLY  031			/* ^Y */
+#define CTLZ  032			/* ^Z */
 
 /* the bits in us_lastemacschar */
-#define	CONTROLX 1			/* ^X was last character typed */
-#define	ESCAPE   2			/* ESCAPE was last character typed */
+#define CONTROLX 1			/* ^X was last character typed */
+#define ESCAPE   2			/* ESCAPE was last character typed */
 
-#define	HEADERLINES 2
+#define HEADERLINES 2
 
 extern GRAPHICS us_ebox, us_menutext, us_menufigs;
 
 INTBIG          us_lastemacschar;		/* state of last character typed */
 static INTSML   us_twid, us_thei;		/* size of a single letter */
-static INTSML   us_editemacsfont;		/* font size for editor */
+static INTBIG   us_editemacsfont;		/* font size for editor */
 static char    *us_killbuf;				/* saved line when ^K is done */
 static INTBIG   us_killbufchars = 0;	/* characters in kill buffer */
 static char    *us_searchbuf = 0;		/* search string for ^S and ^R */
@@ -136,7 +136,7 @@ void us_editemacsclearbox(WINDOWPART*, INTBIG, INTBIG, INTBIG, INTBIG);
 void us_editemacsinvertbox(WINDOWPART*, INTBIG, INTBIG, INTBIG, INTBIG);
 void us_editemacstext(WINDOWPART*, char*, INTBIG, INTBIG);
 void us_editemacsdosearch(WINDOWPART *win, char *str, INTSML reverse,
-	INTSML fromtop, INTSML fromuser);
+	INTSML fromtop, INTSML casesensitive, INTSML fromuser);
 
 /******************** INTERFACE FOR TEXT WINDOW ********************/
 
@@ -214,8 +214,8 @@ WINDOWPART *us_editemacsmakeeditor(WINDOWPART *oriwin, char *header, INTSML *cha
 	 */
 	us_editemacsfont = TXTEDITOR;
 	var = getval((INTBIG)us_aid, VAID, VINTEGER, "USER_textedit_font");
-	if (var != NOVARIABLE && var->addr >= TXT4P && var->addr <= TXT20P)
-		us_editemacsfont = (INTSML)var->addr;
+	if (var != NOVARIABLE && var->addr >= 4)
+		us_editemacsfont = var->addr;
 	screensettextsize(win, us_editemacsfont);
 	screengettextsize(win, "X", &us_twid, &us_thei);
 
@@ -405,9 +405,9 @@ void us_editemacsdeleteline(WINDOWPART *win, INTBIG lindex)
 }
 
 /*
- * routine to highlight line "lindex" in the text window
+ * routine to highlight lines "lindex" to "hindex" in the text window
  */
-void us_editemacshighlightline(WINDOWPART *win, INTBIG lindex)
+void us_editemacshighlightline(WINDOWPART *win, INTBIG lindex, INTBIG hindex)
 {
 	REGISTER EDITOR *e;
 
@@ -415,6 +415,8 @@ void us_editemacshighlightline(WINDOWPART *win, INTBIG lindex)
 	if (e == NOEDITOR) return;
 	us_editemacsoffhighlight(win);
 	if ((e->state&EGRAPHICSOFF) != 0) return;
+	if (hindex != lindex)
+		ttyputmsg(_("EMACS can only highlight a single line"));
 	e->highlightedline = lindex;
 	us_editemacsensuretextshown(win, e->highlightedline);
 	if (e->highlightedline-e->firstline < e->screenlines &&
@@ -555,6 +557,14 @@ void us_editemacsgotbutton(WINDOWPART *win, INTSML but, INTSML x, INTSML y)
 	REGISTER EDITOR *e;
 	REGISTER INTBIG xc, yc, len;
 
+	/* changes to the mouse-wheel are handled by the user interface */
+	if (wheelbutton(but) != 0)
+	{
+		us_buttonhandler(win, but, x, y);
+		return;
+	}
+	ttynewcommand();
+
 	e = win->editor;
 	if (e == NOEDITOR) return;
 	xc = (x-e->offx) / us_twid;
@@ -634,13 +644,54 @@ void us_editemacspaste(WINDOWPART *w) { ttybeep(); }
 
 void us_editemacsundo(WINDOWPART *w) { ttybeep(); }
 
-void us_editemacssearch(WINDOWPART *w, char *str, INTSML fromtop)
+/*
+ * routine to search and/or replace text.  If "replace" is nonzero, this is
+ * a replace.  The meaning of "bits" is as follows:
+ *   1   search from top
+ *   2   replace all
+ *   4   case sensitive
+ *   8   search upwards
+ */
+void us_editemacssearch(WINDOWPART *w, char *str, char *replace, INTBIG bits)
 {
+	REGISTER INTSML fromtop, reverse, casesensitive;
+
+	if (replace != 0)
+	{
+		ttyputerr(_("EMACS cannot replace yet"));
+		return;
+	}
+	if ((bits&1) != 0) fromtop = 1; else fromtop = 0;
+	if ((bits&4) != 0) casesensitive = 1; else casesensitive = 0;
+	if ((bits&8) != 0) reverse = 1; else reverse = 0;
 	us_editemacsflashcursor(w);
-	us_editemacsdosearch(w, str, 0, fromtop, 1);
+	us_editemacsdosearch(w, str, reverse, fromtop, casesensitive, 1);
 	us_editemacsflashcursor(w);
 }
 
+void us_editemacspan(WINDOWPART *win, INTBIG dx, INTBIG dy)
+{
+	REGISTER EDITOR *e;
+
+	e = win->editor;
+	if (e == NOEDITOR) return;
+
+	/* EMACS cannot handle horizontal panning */
+	if (dx != 0) return;
+	if (dy < 0)
+	{
+		if (e->firstline <= 0) return;
+		us_editemacsflashcursor(win);
+		e->firstline--;
+	} else
+	{
+		us_editemacsflashcursor(win);
+		e->firstline++;
+	}
+	us_editemacsredrawscreen(win);
+	us_editemacsflashcursor(win);
+}
+
 /******************** EDITOR CONTROL ********************/
 
 /*
@@ -784,7 +835,7 @@ INTSML us_editemacsimplementchar(WINDOWPART *win, INTBIG i, INTSML m, INTSML fro
 			if (us_searchbuf != 0) efree(us_searchbuf);
 			(void)allocstring(&us_searchbuf, pt, us_aid->cluster);
 		}
-		us_editemacsdosearch(win, us_searchbuf, 0, 0, fromuser);
+		us_editemacsdosearch(win, us_searchbuf, 0, 0, 0, fromuser);
 		return(0);
 	}
 
@@ -814,7 +865,7 @@ INTSML us_editemacsimplementchar(WINDOWPART *win, INTBIG i, INTSML m, INTSML fro
 			if (us_searchbuf != 0) efree(us_searchbuf);
 			(void)allocstring(&us_searchbuf, pt, us_aid->cluster);
 		}
-		us_editemacsdosearch(win, us_searchbuf, 1, 0, fromuser);
+		us_editemacsdosearch(win, us_searchbuf, 1, 0, 0, fromuser);
 		return(0);
 	}
 
@@ -855,7 +906,7 @@ INTSML us_editemacsimplementchar(WINDOWPART *win, INTBIG i, INTSML m, INTSML fro
 	}
 
 	/* delete previous character on a line (cannot delete line break) */
-	if ((i == CTLH || i == BACKSPACEKEY) && m == 0)
+	if ((i == CTLH || i == DELETEKEY) && m == 0)
 	{
 		if (e->curchar <= 0)
 		{
@@ -869,7 +920,7 @@ INTSML us_editemacsimplementchar(WINDOWPART *win, INTBIG i, INTSML m, INTSML fro
 	}
 
 	/* delete previous word on a line (cannot delete line break) */
-	if ((i == 'h' || i == BACKSPACEKEY) && m == 1)
+	if ((i == 'h' || i == DELETEKEY) && m == 1)
 	{
 		if (fromuser != 0) us_editemacsworkingoncurline(e);
 
@@ -1452,10 +1503,10 @@ void us_editemacsdeletechar(WINDOWPART *win)
  * made by the user ("fromuser" nonzero).
  */
 void us_editemacsdosearch(WINDOWPART *win, char *str, INTSML reverse,
-	INTSML fromtop, INTSML fromuser)
+	INTSML fromtop, INTSML casesensitive, INTSML fromuser)
 {
 	REGISTER EDITOR *e;
-	REGISTER INTBIG i, j, k, startchr, startlne;
+	REGISTER INTBIG i, j, k, startchr, startlne, match;
 
 	/* search for the string */
 	e = win->editor;
@@ -1503,7 +1554,14 @@ void us_editemacsdosearch(WINDOWPART *win, char *str, INTSML reverse,
 		}
 
 		/* if string matches, set cursor pointers */
-		if (namesamen(str, &e->textarray[j][k], i) == 0)
+		if (casesensitive != 0)
+		{
+			match = strncmp(str, &e->textarray[j][k], i);
+		} else
+		{
+			match = namesamen(str, &e->textarray[j][k], i);
+		}
+		if (match == 0)
 		{
 			if (fromuser != 0) us_editemacsshipchanges(win);
 			if (reverse != 0) e->curchar = k; else e->curchar = k+i;
