@@ -35,13 +35,13 @@
 #include "usrtrack.h"
 #include "edialogs.h"
 
-#define	THUMBSIZE   16					/* width of the thumb area in scroll slider */
-#define	HEADERLINES  2					/* number of lines of text at top of editor */
-#define	LEFTINDENT   4					/* space between left edge and text */
-#define	DEFAULTWID   1					/* default number of characters on a line */
+#define THUMBSIZE   16					/* width of the thumb area in scroll slider */
+#define HEADERLINES  2					/* number of lines of text at top of editor */
+#define LEFTINDENT   4					/* space between left edge and text */
+#define DEFAULTWID   1					/* default number of characters on a line */
 
 static INTSML   us_twid, us_thei;		/* size of a single letter */
-static INTSML   us_editpacfont;			/* font size for editor */
+static INTBIG   us_editpacfont;			/* font size for editor */
 static char    *us_editpacbuffer = 0;	/* the cut/copy/paste buffer */
 
 /* variables for cursor tracking */
@@ -99,6 +99,10 @@ void us_editpacmovebox(WINDOWPART*, INTSML, INTSML, INTSML, INTSML, INTSML, INTS
 void us_editpacclearbox(WINDOWPART*, INTSML, INTSML, INTSML, INTSML);
 void us_editpacinvertrectframe(WINDOWPART*, INTSML, INTSML);
 void us_editpactext(WINDOWPART*, char*, INTSML, INTSML);
+void us_editpacbackupposition(EDITOR *e, INTBIG *chr, INTBIG *lne);
+void us_editpacadvanceposition(EDITOR *e, INTBIG *chr, INTBIG *lne);
+INTSML us_editpacsinglesearch(WINDOWPART *win, char *str, char *replace, INTBIG bits);
+void us_editpanwindow(WINDOWPART *w, INTBIG dx, INTBIG dy);
 
 /******************** ROUTINES IN THE EDITOR TABLE ********************/
 
@@ -125,8 +129,8 @@ WINDOWPART *us_editpacmakeeditor(WINDOWPART *oriwin, char *header, INTSML *chars
 	 */
 	us_editpacfont = TXTEDITOR;
 	var = getval((INTBIG)us_aid, VAID, VINTEGER, "USER_textedit_font");
-	if (var != NOVARIABLE && var->addr >= TXT4P && var->addr <= TXT20P)
-		us_editpacfont = (INTSML)var->addr;
+	if (var != NOVARIABLE && var->addr >= 4)
+		us_editpacfont = var->addr;
 
 	/* if no window exists, create one as a popup */
 	win = oriwin;
@@ -399,7 +403,7 @@ void us_editpacdeleteline(WINDOWPART *win, INTBIG lindex)
 	us_editpacbeginwork(win);
 
 	/* fake the deletion character */
-	(void)us_editpacimplementchar(win, DELETEKEY, 0);
+	(void)us_editpacimplementchar(win, BACKSPACEKEY, 0);
 
 	/* send out pending changes */
 	us_editpacshipchanges(win);
@@ -412,9 +416,9 @@ void us_editpacdeleteline(WINDOWPART *win, INTBIG lindex)
 }
 
 /*
- * routine to highlight line "lindex" in the text window
+ * routine to highlight lines "lindex" to "hindex" in the text window
  */
-void us_editpachighlightline(WINDOWPART *win, INTBIG lindex)
+void us_editpachighlightline(WINDOWPART *win, INTBIG lindex, INTBIG hindex)
 {
 	REGISTER EDITOR *e;
 
@@ -422,8 +426,10 @@ void us_editpachighlightline(WINDOWPART *win, INTBIG lindex)
 	if (e == NOEDITOR) return;
 	screensettextsize(win, us_editpacfont);
 	us_editpacinvertselection(win);
-	e->curline = e->endline = lindex;
-	e->curchar = 0;   e->endchar = e->maxchars[lindex];
+	e->curline = lindex;
+	e->curchar = 0;
+	e->endline = hindex;
+	e->endchar = e->maxchars[hindex];
 	us_editpacinvertselection(win);
 	us_editpacbeginwork(win);
 
@@ -660,7 +666,7 @@ void us_editpaccut(WINDOWPART *win)
 	/* remove the selection from the edit buffer */
 	screensettextsize(win, us_editpacfont);
 	us_editpacinvertselection(win);
-	(void)us_editpacimplementchar(win, DELETEKEY, 1);
+	(void)us_editpacimplementchar(win, BACKSPACEKEY, 1);
 	us_editpacinvertselection(win);
 }
 
@@ -721,7 +727,13 @@ void us_editpacpaste(WINDOWPART *win)
 
 	us_editpacinvertselection(win);
 	for(i=0; us_editpacbuffer[i] != 0; i++)
+	{
+		if (i > 0)
+		{
+			if (us_editpacbuffer[i] == '\n' && us_editpacbuffer[i-1] == '\r') continue;
+		}
 		(void)us_editpacimplementchar(win, us_editpacbuffer[i], 1);
+	}
 	us_editpacinvertselection(win);
 }
 
@@ -749,7 +761,7 @@ void us_editpacundo(WINDOWPART *win)
 	e->curline = us_editpacundonewcurline;    e->curchar = us_editpacundonewcurchar;
 	e->endline = us_editpacundonewendline;    e->endchar = us_editpacundonewendchar;
 	us_editpacbeginwork(win);
-	if (savebuffer[0] == 0) (void)us_editpacimplementchar(win, DELETEKEY, 1); else
+	if (savebuffer[0] == 0) (void)us_editpacimplementchar(win, BACKSPACEKEY, 1); else
 		for(i=0; savebuffer[i] != 0; i++)
 			(void)us_editpacimplementchar(win, savebuffer[i], 1);
 	us_editpacinvertselection(win);
@@ -757,62 +769,105 @@ void us_editpacundo(WINDOWPART *win)
 	efree(savebuffer);
 }
 
-void us_editpacsearch(WINDOWPART *win, char *str, INTSML fromtop)
+/*
+ * routine to search and/or replace text.  If "replace" is nonzero, this is
+ * a replace.  The meaning of "bits" is as follows:
+ *   1   search from top
+ *   2   replace all
+ *   4   case sensitive
+ *   8   search upwards
+ */
+void us_editpacsearch(WINDOWPART *win, char *str, char *replace, INTBIG bits)
 {
-	REGISTER INTBIG i, chr, lne, len, want, mchr, mlne, mlen, startchr, startlne;
+	if ((bits&2) != 0 && replace != 0)
+	{
+		for(;;)
+		{
+			if (us_editpacsinglesearch(win, str, replace, bits) == 0) break;
+		}
+	} else
+	{
+		us_editpacsinglesearch(win, str, replace, bits);
+	}
+}
+
+void us_editpacpan(WINDOWPART *win, INTBIG dx, INTBIG dy)
+{
+	us_editpanwindow(win, dx, dy);
+}
+
+/*
+ * Helper routine to do a search and replace.  Returns nonzero if a replacement
+ * was done.
+ */
+INTSML us_editpacsinglesearch(WINDOWPART *win, char *str, char *replace, INTBIG bits)
+{
+	REGISTER INTBIG i, want, mchr, mlne;
+	INTBIG startchr, startlne, chr, lne;
 	REGISTER EDITOR *e;
 
 	e = win->editor;
-	if (e == NOEDITOR) return;
+	if (e == NOEDITOR) return(0);
 
-	if (fromtop != 0)
+	if ((bits&1) != 0)
 	{
 		/* search from the top of the buffer */
 		startchr = 0;   startlne = 0;
 	} else
 	{
-		/* search from the end of the current selection */
-		startchr = e->endchar;   startlne = e->endline;
+		/* search from current location */
+		if ((bits&8) != 0)
+		{
+			/* search from before the start of the current selection */
+			startchr = e->curchar;   startlne = e->curline;
+			us_editpacbackupposition(e, &startchr, &startlne);
+		} else
+		{
+			/* search from the end of the current selection */
+			startchr = e->endchar;   startlne = e->endline;
+		}
 	}
 	chr = startchr;   lne = startlne;
-	len = strlen(e->textarray[lne]);
 	for(;;)
 	{
 		/* see if the string matches at this point */
-		mchr = chr;   mlne = lne;   mlen = len;
+		mchr = chr;   mlne = lne;
 		for(i=0; str[i] != 0; i++)
 		{
 			want = str[i];
 			if (want == '\n') want = 0;
-			if (tolower(e->textarray[mlne][mchr]) != tolower(str[i])) break;
+			if ((bits&4) != 0)
+			{
+				if (e->textarray[mlne][mchr] != str[i]) break;
+			} else
+			{
+				if (tolower(e->textarray[mlne][mchr]) != tolower(str[i])) break;
+			}
 
 			/* advance match pointer to the next character in the file */
-			mchr++;
-			if (mchr > mlen)
+			if (e->textarray[mlne][mchr] != 0) mchr++; else
 			{
 				mchr = 0;
 				mlne++;
-				if (mlne >= e->linecount) mlne = 0;
-				mlen = strlen(e->textarray[mlne]);
+				if (mlne >= e->linecount) break;
 			}
 		}
 		if (str[i] == 0) break;
 
 		/* advance to the next character in the file */
-		chr++;
-		if (chr > len)
+		if ((bits&8) != 0)
 		{
-			chr = 0;
-			lne++;
-			if (lne >= e->linecount) lne = 0;
-			len = strlen(e->textarray[lne]);
+			us_editpacbackupposition(e, &chr, &lne);
+		} else
+		{
+			us_editpacadvanceposition(e, &chr, &lne);
 		}
 
 		/* if it came all the way around, stop */
 		if (chr == startchr && lne == startlne)
 		{
 			ttybeep();
-			return;
+			return(0);
 		}
 	}
 
@@ -822,20 +877,25 @@ void us_editpacsearch(WINDOWPART *win, char *str, INTSML fromtop)
 	us_editpacinvertselection(win);
 	e->curline = lne;   e->curchar = chr;
 	for(i=0; str[i] != 0; i++)
-	{
-		chr++;
-		if (chr > len)
-		{
-			chr = 0;
-			lne++;
-			if (lne >= e->linecount) lne = 0;
-			len = strlen(e->textarray[lne]);
-		}
-	}
+		us_editpacadvanceposition(e, &chr, &lne);
 	e->endline = lne;   e->endchar = chr;
 	us_editpacinvertselection(win);
 	us_editpacbeginwork(win);
 
+	if (replace != 0)
+	{
+		us_editpacinvertselection(win);
+		if (replace[0] == 0)
+		{
+			(void)us_editpacimplementchar(win, BACKSPACEKEY, 1);
+		} else
+		{
+			for(i=0; replace[i] != 0; i++)
+				(void)us_editpacimplementchar(win, replace[i], 1);
+		}
+		us_editpacinvertselection(win);
+	}
+
 	/* ensure that line "e->curline" is shown in the display */
 	if (e->curline-e->firstline >= e->screenlines || e->curline < e->firstline)
 	{
@@ -844,6 +904,7 @@ void us_editpacsearch(WINDOWPART *win, char *str, INTSML fromtop)
 		if ((win->state&WINDOWTYPE) != POPTEXTWINDOW)
 			us_editpacdrawvscroll(win);
 	}
+	return(1);
 }
 
 /******************** ROUTINES IN THE WINDOW STRUCTURE ********************/
@@ -870,6 +931,14 @@ void us_editpacgotbutton(WINDOWPART *win, INTSML but, INTSML x, INTSML y)
 	REGISTER INTBIG xc, yc, len, i;
 	INTSML (*clickroutine)(INTBIG, INTBIG);
 
+	/* changes to the mouse-wheel are handled by the user interface */
+	if (wheelbutton(but) != 0)
+	{
+		us_buttonhandler(win, but, x, y);
+		return;
+	}
+	ttynewcommand();
+
 	e = win->editor;
 	if (e == NOEDITOR) return;
 	screensettextsize(win, us_editpacfont);
@@ -1136,29 +1205,13 @@ INTSML us_editpacdoclickdown(INTBIG x, INTBIG y, INTSML bywords)
 INTSML us_editpacdownarrow(INTBIG x, INTBIG y)
 {
 	REGISTER EDITOR *e;
-	REGISTER INTBIG i;
 
 	e = us_editpaccurrentwin->editor;
 	if (x <= e->offx + e->swid || x > e->offx + e->swid + DISPLAYSLIDERSIZE) return(0);
 	if (y >= e->revy - e->shei + DISPLAYSLIDERSIZE || y < e->revy - e->shei) return(0);
 	if (e->linecount-e->firstline <= e->screenlines) return(1);
 
-	us_editpacinvertselection(us_editpaccurrentwin);
-	e->firstline++;
-
-	/* shift the window contents up by 1 line (us_thei) */
-	screenmovebox(us_editpaccurrentwin, (INTSML)(e->offx-LEFTINDENT), (INTSML)(e->revy-e->shei),
-		(INTSML)(e->swid+LEFTINDENT), (INTSML)(e->shei-us_thei), (INTSML)(e->offx-LEFTINDENT),
-			(INTSML)(e->revy-e->shei+us_thei));
-
-	/* fill in a new last line */
-	screendrawbox(us_editpaccurrentwin, (INTSML)(e->offx-LEFTINDENT), (INTSML)(e->offx+e->swid),
-		(INTSML)(e->revy-e->shei), (INTSML)(e->revy-e->shei+us_thei), &us_ebox);
-	i = e->firstline + e->screenlines - 1;
-	if (i < e->linecount)
-		us_editpactext(us_editpaccurrentwin, e->textarray[i], 0, (INTSML)(i-e->firstline));
-	us_editpacinvertselection(us_editpaccurrentwin);
-	us_editpacdrawvscroll(us_editpaccurrentwin);
+	us_editpanwindow(us_editpaccurrentwin, 0, 1);
 	return(0);
 }
 
@@ -1171,19 +1224,7 @@ INTSML us_editpacuparrow(INTBIG x, INTBIG y)
 	if (y <= e->revy - DISPLAYSLIDERSIZE || y > e->revy) return(0);
 	if (e->firstline <= 0) return(1);
 
-	us_editpacinvertselection(us_editpaccurrentwin);
-	e->firstline--;
-
-	/* shift the window contents down by 1 line (us_thei) */
-	screenmovebox(us_editpaccurrentwin, (INTSML)(e->offx-LEFTINDENT), (INTSML)(e->revy-e->shei+us_thei), (INTSML)(e->swid+LEFTINDENT),
-		(INTSML)(e->shei-us_thei), (INTSML)(e->offx-LEFTINDENT), (INTSML)(e->revy-e->shei));
-
-	/* fill in a new first line */
-	screendrawbox(us_editpaccurrentwin, (INTSML)(e->offx-LEFTINDENT), (INTSML)(e->offx+e->swid),
-		(INTSML)(e->revy-us_thei), e->revy, &us_ebox);
-	us_editpactext(us_editpaccurrentwin, e->textarray[e->firstline], 0, 0);
-	us_editpacinvertselection(us_editpaccurrentwin);
-	us_editpacdrawvscroll(us_editpaccurrentwin);
+	us_editpanwindow(us_editpaccurrentwin, 0, -1);
 	return(0);
 }
 
@@ -1225,10 +1266,7 @@ INTSML us_editpacrightarrow(INTBIG x, INTBIG y)
 	e = us_editpaccurrentwin->editor;
 	if (y > e->revy - e->shei || y < e->revy - e->shei - DISPLAYSLIDERSIZE) return(0);
 	if (x <= e->offx + e->swid - DISPLAYSLIDERSIZE || x > e->offx + e->swid) return(0);
-	if (e->horizfactor >= e->screenchars*10) return(1);
-	e->horizfactor++;
-	us_editpacredrawlines(us_editpaccurrentwin);
-	us_editpacdrawhscroll(us_editpaccurrentwin);
+	us_editpanwindow(us_editpaccurrentwin, 1, 0);
 	return(0);
 }
 
@@ -1239,10 +1277,7 @@ INTSML us_editpacleftarrow(INTBIG x, INTBIG y)
 	e = us_editpaccurrentwin->editor;
 	if (y > e->revy - e->shei || y < e->revy - e->shei - DISPLAYSLIDERSIZE) return(0);
 	if (x >= e->offx + DISPLAYSLIDERSIZE - LEFTINDENT || x < e->offx - LEFTINDENT) return(0);
-	if (e->horizfactor <= 0) return(1);
-	e->horizfactor--;
-	us_editpacredrawlines(us_editpaccurrentwin);
-	us_editpacdrawhscroll(us_editpaccurrentwin);
+	us_editpanwindow(us_editpaccurrentwin, -1, 0);
 	return(0);
 }
 
@@ -1370,10 +1405,24 @@ INTSML us_editpacimplementchar(WINDOWPART *win, INTSML i, INTSML fromuser)
 	}
 
 	/* if delete key hit and nothing selected, set to erase previous character */
-	if (i == DELETEKEY || i == BACKSPACEKEY)
+	if (i == BACKSPACEKEY || i == DELETEKEY)
 	{
 		if (e->curline == e->endline && e->curchar == e->endchar)
 		{
+#ifdef WIN32
+			if (i == DELETEKEY)
+			{
+				e->curchar++;
+				len = strlen(e->textarray[e->curline]);
+				if (e->curchar > len)
+				{
+					e->curchar = 0;
+					e->curline++;
+				}
+				e->endchar = e->curchar;
+				e->endline = e->curline;
+			}
+#endif
 			if (e->curchar > 0) e->curchar--; else
 			{
 				if (e->curline > 0)
@@ -1397,10 +1446,8 @@ INTSML us_editpacimplementchar(WINDOWPART *win, INTSML i, INTSML fromuser)
 		}
 	}
 
-	/* mark a dirty edit buffer */
-	e->dirty = 1;
 	us_editpacamtyping(e);
-	if (i == DELETEKEY || i == BACKSPACEKEY) us_editpactypeddelete(e);
+	if (i == BACKSPACEKEY || i == DELETEKEY) us_editpactypeddelete(e);
 
 	/* remove the selected characters */
 	us_editpacremoveselection(win);
@@ -1408,8 +1455,11 @@ INTSML us_editpacimplementchar(WINDOWPART *win, INTSML i, INTSML fromuser)
 	e->endchar = e->curchar;
 	e->charposition = e->endchar;
 
+	/* mark a dirty edit buffer */
+	e->dirty = 1;
+
 	/* ignore delete or backspace characters */
-	if (i == DELETEKEY || i == BACKSPACEKEY) return(0);
+	if (i == BACKSPACEKEY || i == DELETEKEY) return(0);
 
 	/* self-insert of normal characters */
 	if (i == '\t') i = ' ';
@@ -1539,7 +1589,6 @@ void us_editpacremoveselection(WINDOWPART *win)
 	diff = e->endline - e->curline;
 	if (e->curline == e->endline && e->curchar == e->endchar) return;
 
-
 	/* report the deletion */
 	if (win->changehandler != 0)
 	{
@@ -1549,6 +1598,8 @@ void us_editpacremoveselection(WINDOWPART *win)
 			(*win->changehandler)(win, DELETETEXTLINE, e->textarray[e->curline+j], "",
 				e->curline+j);
 	}
+	if (diff > 0)
+		us_editpacbeginwork(win);
 
 	/* append text beyond end of selection to the start of the selection */
 	j = strlen(&e->textarray[e->endline][e->endchar]) + e->curchar + 1;
@@ -1597,7 +1648,6 @@ void us_editpacremoveselection(WINDOWPART *win)
 				us_editpaccleanupline(win, j);
 		}
 		e->linecount -= diff;
-		us_editpacbeginwork(win);
 	}
 }
 
@@ -1798,6 +1848,72 @@ void us_editpacsetheader(WINDOWPART *win, char *header)
 	}
 }
 
+/*
+ * Routine to shift the edit window by the direction in the coordinates (dx, dy)
+ */
+void us_editpanwindow(WINDOWPART *w, INTBIG dx, INTBIG dy)
+{
+	REGISTER EDITOR *e;
+	REGISTER INTBIG i;
+
+	e = w->editor;
+	us_editpacinvertselection(w);
+	if (dx == 0)
+	{
+		/* vertical panning */
+		if (dy > 0)
+		{
+			/* pan down by 1 line */
+			e->firstline++;
+
+			/* shift the window contents up by 1 line (us_thei) */
+			screenmovebox(w, (INTSML)(e->offx-LEFTINDENT), (INTSML)(e->revy-e->shei),
+				(INTSML)(e->swid+LEFTINDENT), (INTSML)(e->shei-us_thei), (INTSML)(e->offx-LEFTINDENT),
+					(INTSML)(e->revy-e->shei+us_thei));
+
+			/* fill in a new last line */
+			screendrawbox(w, (INTSML)(e->offx-LEFTINDENT), (INTSML)(e->offx+e->swid),
+				(INTSML)(e->revy-e->shei), (INTSML)(e->revy-e->shei+us_thei), &us_ebox);
+			i = e->firstline + e->screenlines - 1;
+			if (i < e->linecount)
+				us_editpactext(w, e->textarray[i], 0, (INTSML)(i-e->firstline));
+		} else
+		{
+			/* pan up by 1 line */
+			e->firstline--;
+
+			/* shift the window contents down by 1 line (us_thei) */
+			screenmovebox(w, (INTSML)(e->offx-LEFTINDENT), (INTSML)(e->revy-e->shei+us_thei), (INTSML)(e->swid+LEFTINDENT),
+				(INTSML)(e->shei-us_thei), (INTSML)(e->offx-LEFTINDENT), (INTSML)(e->revy-e->shei));
+
+			/* fill in a new first line */
+			screendrawbox(w, (INTSML)(e->offx-LEFTINDENT), (INTSML)(e->offx+e->swid),
+				(INTSML)(e->revy-us_thei), e->revy, &us_ebox);
+			us_editpactext(w, e->textarray[e->firstline], 0, 0);
+		}
+		us_editpacinvertselection(w);
+		us_editpacdrawvscroll(w);
+	} else
+	{
+		/* horizontal panning */
+		if (dx > 0)
+		{
+			/* pan to the right */
+			if (e->horizfactor >= e->screenchars*10) return;
+			e->horizfactor++;
+			us_editpacredrawlines(w);
+			us_editpacdrawhscroll(w);
+		} else
+		{
+			/* pan to the left */
+			if (e->horizfactor <= 0) return;
+			e->horizfactor--;
+			us_editpacredrawlines(w);
+			us_editpacdrawhscroll(w);
+		}
+	}
+}
+
 /*
  * routine to invert the selected text, turning it on or off
  */
@@ -1872,12 +1988,14 @@ void us_editpacamtyping(EDITOR *e)
 		if (e->curline < us_editpacundonewcurline ||
 			(e->curline == us_editpacundonewcurline && e->curchar < us_editpacundonewcurchar))
 		{
-			us_editpacundonewcurline = e->curline;    us_editpacundonewcurchar = e->curchar;
+			us_editpacundonewcurline = e->curline;
+			us_editpacundonewcurchar = e->curchar;
 		}
 		if (e->endline > us_editpacundonewendline ||
 			(e->endline == us_editpacundonewendline && e->endchar > us_editpacundonewendchar))
 		{
-			us_editpacundonewendline = e->endline;    us_editpacundonewendchar = e->endchar;
+			us_editpacundonewendline = e->endline;
+			us_editpacundonewendchar = e->endchar;
 		}
 		return;
 	}
@@ -1982,6 +2100,27 @@ void us_editpacloadselection(EDITOR *e, char *buf)
 	buf[len] = 0;
 }
 
+void us_editpacadvanceposition(EDITOR *e, INTBIG *chr, INTBIG *lne)
+{
+	if (e->textarray[*lne][*chr] != 0) (*chr)++; else
+	{
+		*chr = 0;
+		(*lne)++;
+		if (*lne >= e->linecount) *lne = 0;
+	}
+}
+
+void us_editpacbackupposition(EDITOR *e, INTBIG *chr, INTBIG *lne)
+{
+	(*chr)--;
+	if (*chr < 0)
+	{
+		(*lne)--;
+		if (*lne < 0) *lne = e->linecount - 1;
+		*chr = strlen(e->textarray[*lne]) - 1;
+	}
+}
+
 /******************** SUPPORT ********************/
 
 /*
