@@ -32,12 +32,13 @@
 #include "egraphics.h"
 #include "efunction.h"
 #include "usr.h"
+#include "drc.h"
 #include "tecgen.h"
 #include "tecart.h"
 #include "usredtec.h"
 #include "usrdiacom.h"
 
-#define	MAXNAMELEN 25		/* max chars in a new name */
+#define MAXNAMELEN 25		/* max chars in a new name */
 
 INTSML us_teceddrclayers = 0;
 char **us_teceddrclayernames = 0;
@@ -166,9 +167,18 @@ LIST us_tecnode_functions[] =
 	{"p-type-JFET-transistor",        "NPTRAPJFET",  NPTRAPJFET},
 	{"depletion-mesfet",              "NPTRADMES",   NPTRADMES},
 	{"enhancement-mesfet",            "NPTRAEMES",   NPTRAEMES},
-	{"dual-emitter-transistor",       "NPTRANE2L",   NPTRANE2L},
 	{"prototype-defined-transistor",  "NPTRANSREF",  NPTRANSREF},
 	{"transistor",                    "NPTRANS",     NPTRANS},
+	{"4-port-nMOS-transistor",        "NPTRA4NMOS",  NPTRA4NMOS},
+	{"4-port-DMOS-transistor",        "NPTRA4DMOS",  NPTRA4DMOS},
+	{"4-port-PMOS-transistor",        "NPTRA4PMOS",  NPTRA4PMOS},
+	{"4-port-NPN-transistor",         "NPTRA4NPN",   NPTRA4NPN},
+	{"4-port-PNP-transistor",         "NPTRA4PNP",   NPTRA4PNP},
+	{"4-port-n-type-JFET-transistor", "NPTRA4NJFET", NPTRA4NJFET},
+	{"4-port-p-type-JFET-transistor", "NPTRA4PJFET", NPTRA4PJFET},
+	{"4-port-depletion-mesfet",       "NPTRA4DMES",  NPTRA4DMES},
+	{"4-port-enhancement-mesfet",     "NPTRA4EMES",  NPTRA4EMES},
+	{"transistor",                    "NPTRANS4",    NPTRANS4},
 	{"resistor",                      "NPRESIST",    NPRESIST},
 	{"capacitor",                     "NPCAPAC",     NPCAPAC},
 	{"electrolytic-capacitor",        "NPECAPAC",    NPECAPAC},
@@ -211,9 +221,6 @@ LIST us_tecnode_functions[] =
 	{NULL, NULL, 0}
 };
 
-static INTSML us_teceddrc;			/* nonzero if editing DRC */
-static void (*us_edtecoldptermhandler)(WINDOWPART*);	/* former pallet term. routine */
-
 GRAPHICS us_edtechigh = {LAYERH, HIGHLIT, SOLIDC, SOLIDC,
 	{0xFFFF,0xFFFF,0xFFFF,0xFFFF,0xFFFF,0xFFFF,0xFFFF,0xFFFF}, NOVARIABLE, 0};
 
@@ -254,8 +261,6 @@ void us_tecedinfolambda(NODEINST*, INTSML, char*[]);
 void us_tecedinfodescript(NODEINST*, INTSML, char*[]);
 void us_tecedinfounits(NODEINST*, INTSML, char*[]);
 void us_tecedsetnode(NODEINST*, char*);
-void us_teceddrcchange(WINDOWPART*, INTSML, char*, char*, INTBIG);
-void us_teceddrcterm(WINDOWPART*);
 NODEPROTO *us_tecedenterfacet(char*);
 void us_tecedredolayergraphics(NODEPROTO*);
 void us_tecedloadlibmap(LIBRARY*);
@@ -286,11 +291,6 @@ void us_tecedentry(INTSML count, char *par[])
 	l = strlen(pp = par[0]);
 	if (namesamen(pp, "library-to-tech-and-c", l) == 0 && l >= 17)
 	{
-		if (us_teceddrc != 0)
-		{
-			us_abortcommand(_("Cannot generate technology while editing DRC"));
-			return;
-		}
 		if (count <= 1) pp = 0; else
 			pp = par[1];
 		us_tecfromlibinit(el_curlib, pp, 1);
@@ -298,11 +298,6 @@ void us_tecedentry(INTSML count, char *par[])
 	}
 	if (namesamen(pp, "library-to-tech", l) == 0)
 	{
-		if (us_teceddrc != 0)
-		{
-			us_abortcommand(_("Cannot generate technology while editing DRC"));
-			return;
-		}
 		if (count <= 1) pp = 0; else
 			pp = par[1];
 		us_tecfromlibinit(el_curlib, pp, 0);
@@ -426,7 +421,7 @@ void us_tecedentry(INTSML count, char *par[])
 		if (np != NONODEPROTO)
 		{
 			us_tecedmakelayer(np, COLORT1, stip, SOLIDC, "XX", LFUNKNOWN, "x", "",
-				-1, 0.0, 0.0, 0.0, -1, 0, 0);
+				-1, 0.0, 0.0, 0.0, 0, 0);
 			(*el_curconstraint->solve)(np);
 		}
 
@@ -434,7 +429,7 @@ void us_tecedentry(INTSML count, char *par[])
 		efree(facetname);
 		if (np == NONODEPROTO) return;
 		us_tecedmakelayer(np, COLORT1, stip, SOLIDC, "XX", LFUNKNOWN, "x", "",
-			-1, 0.0, 0.0, 0.0, -1, 0, 0);
+			-1, 0.0, 0.0, 0.0, 0, 0);
 		(*el_curconstraint->solve)(np);
 		(void)us_tecedenterfacet(describenodeproto(np));
 		return;
@@ -575,19 +570,42 @@ void us_tecedentry(INTSML count, char *par[])
  */
 void us_teceditdrc(void)
 {
-	REGISTER WINDOWPART *w;
-	REGISTER EDITOR *e;
-	INTSML dummy;
-	REGISTER INTSML i, l;
+	REGISTER INTSML changed;
 	REGISTER VARIABLE *var;
-	REGISTER NODEPROTO *np;
+	REGISTER DRCRULES *rules;
+	INTBIG i;
 
-	if (us_teceddrc != 0)
+	/* get the current list of layer names */
+	us_tecedgetlayernamelist();
+
+	/* create a RULES structure */
+	rules = dr_allocaterules(us_teceddrclayers, "EDITED TECHNOLOGY");
+	if (rules == NODRCRULES) return;
+	for(i=0; i<us_teceddrclayers; i++)
+		(void)allocstring(&rules->layernames[i], us_teceddrclayernames[i], el_tempcluster);
+
+	/* get the text-list of design rules and convert them into arrays */
+	var = getval((INTBIG)el_curlib, VLIBRARY, VSTRING|VISARRAY, "EDTEC_DRC");
+	if (us_teceditgetdrcarrays(var, rules) != 0) return;
+
+	/* edit the design-rule arrays */
+	changed = dr_rulesdialog(NOTECHNOLOGY, rules);
+
+	/* if changes were made, convert the arrays back into a text-list */
+	if (changed != 0)
 	{
-		ttyputerr(_("Already editing design rules"));
-		return;
+		us_tecedloaddrcmessage(rules, el_curlib);
 	}
 
+	/* free the arrays */
+	dr_freerules(rules);
+}
+
+void us_tecedgetlayernamelist(void)
+{
+	REGISTER INTBIG i;
+	REGISTER NODEPROTO *np;
+
 	/* free any former layer name information */
 	if (us_teceddrclayernames != 0)
 	{
@@ -612,30 +630,211 @@ void us_teceditdrc(void)
 	for(i = 0, np = el_curlib->firstnodeproto; np != NONODEPROTO; np = np->nextnodeproto)
 		if (namesamen(np->cell->cellname, "layer-", 6) == 0)
 			(void)allocstring(&us_teceddrclayernames[i++], &np->cell->cellname[6], us_aid->cluster);
+}
 
-	/* make the editor window */
-	w = us_wantnewwindow(0);
-	if (w == NOWINDOWPART) return;
+/*
+ * Helper routine to create arrays describing the design rules in the variable "EDTEC_DRC".
+ */
+INTSML us_teceditgetdrcarrays(VARIABLE *var, DRCRULES *rules)
+{
+	REGISTER INTBIG i, l;
+	INTBIG amt;
+	INTSML connected, wide, multi, widrule, layer1, layer2, j;
+	REGISTER char *str;
+	char *rule;
 
-	if (us_makeeditor(w, _("Design Rules (Format: [C]SPACING LAYER LAYER)"), &dummy,
-		&dummy) == NOWINDOWPART) return;
-	e = w->editor;
-	if (e == NOEDITOR) return;
-	w->termhandler = us_teceddrcterm;
-	us_teceddrc = 1;
-
-	/* load in the existing design rules */
-	var = getval((INTBIG)el_curlib, VLIBRARY, VSTRING|VISARRAY, "EDTEC_DRC");
+	/* get the design rules */
 	if (var != NOVARIABLE)
 	{
-		us_suspendgraphics(w);
-		l = (INTSML)getlength(var);
+		l = getlength(var);
 		for(i=0; i<l; i++)
-			us_addline(w, i, ((char **)var->addr)[i]);
-		us_resumegraphics(w);
+		{
+			/* parse the DRC rule */
+			str = ((char **)var->addr)[i];
+			while (*str == ' ') str++;
+			if (*str == 0) continue;
+			if (us_tecedgetdrc(str, &connected, &wide, &multi, &widrule,
+				&amt, &layer1, &layer2, &rule, (INTSML)rules->numlayers, rules->layernames) != 0)
+			{
+				ttyputmsg(_("DRC line %d is: %s"), i+1, str);
+				continue;
+			}
+
+			/* set the layer spacing */
+			if (widrule == 1)
+			{
+				rules->minwidth[layer1] = amt;
+				if (*rule != 0)
+					(void)reallocstring(&rules->minwidthR[layer1], rule, el_tempcluster);
+			} else if (widrule == 2)
+			{
+				rules->widelimit = amt;
+			} else
+			{
+				if (layer1 > layer2) { j = layer1;  layer1 = layer2;  layer2 = j; }
+				j = (layer1+1) * (layer1/2) + (layer1&1) * ((layer1+1)/2);
+				j = layer2 + rules->numlayers * layer1 - j;
+				if (wide != 0)
+				{
+					if (connected != 0)
+					{
+						rules->conlistW[j] = amt;
+						if (*rule != 0)
+							(void)reallocstring(&rules->conlistWR[j], rule, el_tempcluster);
+					} else
+					{
+						rules->unconlistW[j] = amt;
+						if (*rule != 0)
+							(void)reallocstring(&rules->unconlistWR[j], rule, el_tempcluster);
+					}
+				} else if (multi != 0)
+				{
+					if (connected != 0)
+					{
+						rules->conlistM[j] = amt;
+						if (*rule != 0)
+							(void)reallocstring(&rules->conlistMR[j], rule, el_tempcluster);
+					} else
+					{
+						rules->unconlistM[j] = amt;
+						if (*rule != 0)
+							(void)reallocstring(&rules->unconlistMR[j], rule, el_tempcluster);
+					}
+				} else
+				{
+					if (connected != 0)
+					{
+						rules->conlist[j] = amt;
+						if (*rule != 0)
+							(void)reallocstring(&rules->conlistR[j], rule, el_tempcluster);
+					} else
+					{
+						rules->unconlist[j] = amt;
+						if (*rule != 0)
+							(void)reallocstring(&rules->unconlistR[j], rule, el_tempcluster);
+					}
+				}
+			}
+		}
+	}
+	return(0);
+}
+
+/*
+ * Helper routine to examine the arrays describing the design rules and create
+ * the variable "EDTEC_DRC" on library "lib".
+ */
+void us_tecedloaddrcmessage(DRCRULES *rules, LIBRARY *lib)
+{
+	REGISTER INTBIG drccount, drcindex, i, k, j;
+	REGISTER char **drclist;
+
+	/* determine the number of lines in the text-version of the design rules */
+	drccount = 0;
+	for(i=0; i<rules->utsize; i++)
+	{
+		if (rules->conlist[i] >= 0) drccount++;
+		if (rules->unconlist[i] >= 0) drccount++;
+		if (rules->conlistW[i] >= 0) drccount++;
+		if (rules->unconlistW[i] >= 0) drccount++;
+		if (rules->conlistM[i] >= 0) drccount++;
+		if (rules->unconlistM[i] >= 0) drccount++;
+	}
+	for(i=0; i<rules->numlayers; i++)
+	{
+		if (rules->minwidth[i] >= 0) drccount++;
+	}
+
+	/* load the arrays */
+	if (drccount != 0)
+	{
+		drccount++;
+		drclist = (char **)emalloc((drccount * (sizeof (char *))), el_tempcluster);
+		if (drclist == 0) return;
+		drcindex = 0;
+
+		/* write the width limit */
+		(void)initinfstr();
+		(void)formatinfstr("l%s", frtoa(rules->widelimit));
+		(void)allocstring(&drclist[drcindex++], returninfstr(), el_tempcluster);
+
+		/* write the minimum width for each layer */
+		for(i=0; i<rules->numlayers; i++)
+		{
+			if (rules->minwidth[i] >= 0)
+			{
+				(void)initinfstr();
+				(void)formatinfstr("s%s %s %s", frtoa(rules->minwidth[i]),
+					us_tecedstripdash(rules->layernames[i]), rules->minwidthR[i]);
+				(void)allocstring(&drclist[drcindex++], returninfstr(), el_tempcluster);
+			}
+		}
+
+		/* now do the distance rules */
+		k = 0;
+		for(i=0; i<rules->numlayers; i++) for(j=i; j<rules->numlayers; j++)
+		{
+			if (rules->conlist[k] >= 0)
+			{
+				(void)initinfstr();
+				(void)formatinfstr("c%s %s %s %s", frtoa(rules->conlist[k]),
+					us_tecedstripdash(rules->layernames[i]), us_tecedstripdash(rules->layernames[j]),
+						rules->conlistR[k]);
+				(void)allocstring(&drclist[drcindex++], returninfstr(), el_tempcluster);
+			}
+			if (rules->unconlist[k] >= 0)
+			{
+				(void)initinfstr();
+				(void)formatinfstr("%s %s %s %s", frtoa(rules->unconlist[k]),
+					us_tecedstripdash(rules->layernames[i]), us_tecedstripdash(rules->layernames[j]),
+						rules->unconlistR[k]);
+				(void)allocstring(&drclist[drcindex++], returninfstr(), el_tempcluster);
+			}
+			if (rules->conlistW[k] >= 0)
+			{
+				(void)initinfstr();
+				(void)formatinfstr("cw%s %s %s %s", frtoa(rules->conlistW[k]),
+					us_tecedstripdash(rules->layernames[i]), us_tecedstripdash(rules->layernames[j]),
+						rules->conlistWR[k]);
+				(void)allocstring(&drclist[drcindex++], returninfstr(), el_tempcluster);
+			}
+			if (rules->unconlistW[k] >= 0)
+			{
+				(void)initinfstr();
+				(void)formatinfstr("w%s %s %s %s", frtoa(rules->unconlistW[k]),
+					us_tecedstripdash(rules->layernames[i]), us_tecedstripdash(rules->layernames[j]),
+						rules->unconlistWR[k]);
+				(void)allocstring(&drclist[drcindex++], returninfstr(), el_tempcluster);
+			}
+			if (rules->conlistM[k] >= 0)
+			{
+				(void)initinfstr();
+				(void)formatinfstr("cm%s %s %s %s", frtoa(rules->conlistM[k]),
+					us_tecedstripdash(rules->layernames[i]), us_tecedstripdash(rules->layernames[j]),
+						rules->conlistMR[k]);
+				(void)allocstring(&drclist[drcindex++], returninfstr(), el_tempcluster);
+			}
+			if (rules->unconlistM[k] >= 0)
+			{
+				(void)initinfstr();
+				(void)formatinfstr("m%s %s %s %s", frtoa(rules->unconlistM[k]),
+					us_tecedstripdash(rules->layernames[i]), us_tecedstripdash(rules->layernames[j]),
+						rules->unconlistMR[k]);
+				(void)allocstring(&drclist[drcindex++], returninfstr(), el_tempcluster);
+			}
+			k++;
+		}
+
+		(void)setval((INTBIG)lib, VLIBRARY, "EDTEC_DRC", (INTBIG)drclist,
+			VSTRING|VISARRAY|(drccount<<VLENGTHSH));
+		for(i=0; i<drccount; i++) efree(drclist[i]);
+		efree((char *)drclist);
+	} else
+	{
+		/* no rules: remove the variable */
+		if (getval((INTBIG)lib, VLIBRARY, VSTRING|VISARRAY, "EDTEC_DRC") != NOVARIABLE)
+			delval((INTBIG)lib, VLIBRARY, "EDTEC_DRC");
 	}
-	w->changehandler = us_teceddrcchange;
-	ttyputverbose(_("Editing design rules"));
 }
 
 /*
@@ -765,7 +964,7 @@ void us_teceditcreat(INTSML count, char *par[])
 	if (namesamen(pp, "lines-solid", l) == 0 && l >= 7) np = art_openedpolygonprim;
 	if (namesamen(pp, "lines-dotted", l) == 0 && l >= 8) np = art_openeddottedpolygonprim;
 	if (namesamen(pp, "lines-dashed", l) == 0 && l >= 8) np = art_openeddashedpolygonprim;
-	if (namesamen(pp, "lines-far-dotted", l) == 0 && l >= 7) np = art_openedfardottepolygonprim;
+	if (namesamen(pp, "lines-thicker", l) == 0 && l >= 7) np = art_openedthickerpolygonprim;
 	if (namesamen(pp, "circle-outline", l) == 0 && l >= 8) np = art_circleprim;
 	if (namesamen(pp, "circle-filled", l) == 0 && l >= 8) np = art_filledcircleprim;
 	if (namesamen(pp, "circle-half", l) == 0 && l >= 8) np = art_circleprim;
@@ -820,7 +1019,7 @@ void us_teceditcreat(INTSML count, char *par[])
 		/* a port layer */
 		if (count == 1)
 		{
-			name = ttygetline(_("Port name: "));
+			name = ttygetline(M_("Port name: "));
 			if (name == 0 || name[0] == 0)
 			{
 				us_abortedmsg();
@@ -828,7 +1027,7 @@ void us_teceditcreat(INTSML count, char *par[])
 			}
 		} else name = par[1];
 		var = setval((INTBIG)ni, VNODEINST, "EDTEC_portname", (INTBIG)name, VSTRING|VDISPLAY);
-		if (var != NOVARIABLE) var->textdescript = defaulttextdescript(NOGEOM);
+		if (var != NOVARIABLE) var->textdescript = defaulttextdescript(ni->geom);
 		if ((us_aid->aidstate&NODETAILS) == 0)
 			ttyputmsg(_("Use 'change' option to set arc connectivity and port angle"));
 	}
@@ -848,7 +1047,7 @@ void us_teceditcreat(INTSML count, char *par[])
 		if (namesamen(pp, "polygon-", 8) == 0)
 		{
 			ttyputmsg(_("Use 'change' option to set a layer for this shape"));
-			ttyputmsg(_("Use 'polygonedit' command to describe polygonal shape"));
+			ttyputmsg(_("Use 'outline edit' to describe polygonal shape"));
 		}
 		if (namesamen(pp, "lines-", 6) == 0)
 			ttyputmsg(_("Use 'change' option to set a layer for this line"));
@@ -893,7 +1092,7 @@ void us_teceditidentify(INTSML doports)
 		{
 			us_abortcommand(_("Must be editing a node to identify ports"));
 			if ((us_aid->aidstate&NODETAILS) == 0)
-				ttyputmsg(_("Use the 'edit-node' option"));
+				ttyputmsg(M_("Use the 'edit-node' option"));
 			return;
 		}
 	} else
@@ -903,7 +1102,7 @@ void us_teceditidentify(INTSML doports)
 		{
 			us_abortcommand(_("Must be editing a node or arc to identify layers"));
 			if ((us_aid->aidstate&NODETAILS) == 0)
-				ttyputmsg(_("Use 'edit-node' or 'edit-arc' options"));
+				ttyputmsg(M_("Use 'edit-node' or 'edit-arc' options"));
 			return;
 		}
 	}
@@ -991,7 +1190,7 @@ void us_teceditidentify(INTSML doports)
 	}
 
 	/* rotate through all configurations, finding least distance */
-	bestdist = HUGEINT;
+	bestdist = MAXINTBIG;
 	for(i=0; i<total; i++)
 	{
 		/* find distance from each label to its sample center */
@@ -1043,7 +1242,7 @@ void us_teceditidentify(INTSML doports)
 		} else poly->string = &ns->layer->cell->cellname[6];
 		poly->desc = &us_hbox;
 		poly->style = (INTSML)style[i];
-		poly->font = TXTSMALL;
+		poly->font = TXT1L;
 		poly->tech = el_curtech;
 		us_hbox.col = HIGHLIT;
 
@@ -1175,7 +1374,7 @@ void us_teceditinquire(void)
 			ttyputmsg(_("This object defines the DXF name(s) of %s"), describenodeproto(np));
 			break;
 		case LAYERDRCMINWID:
-			ttyputmsg(_("This object defines the minimum DRC width of %s"), describenodeproto(np));
+			ttyputmsg(_("This object defines the minimum DRC width of %s (OBSOLETE)"), describenodeproto(np));
 			break;
 		case LAYERFUNCTION:
 			ttyputmsg(_("This object defines the function of %s"), describenodeproto(np));
@@ -1577,7 +1776,7 @@ void us_tecedlayerpattern(NODEINST *ni)
 	if (ni->proto == art_boxprim)
 	{
 		startobjectchange((INTBIG)ni, VNODEINST);
-		newni = replacenodeinst(ni, art_filledboxprim, 0);
+		newni = replacenodeinst(ni, art_filledboxprim, 0, 0);
 		if (newni == NONODEINST) return;
 		endobjectchange((INTBIG)newni, VNODEINST);
 		ni = newni;
@@ -1601,6 +1800,7 @@ void us_tecedlayerpattern(NODEINST *ni)
 	high.fromport = NOPORTPROTO;
 	high.frompoint = 0;
 	high.fromvar = NOVARIABLE;
+	high.fromvarnoeval = NOVARIABLE;
 	(void)us_addhighlight(&high);
 
 	/* redraw the demo layer in this facet */
@@ -1949,76 +2149,6 @@ void us_tecedsetnode(NODEINST *ni, char *chr)
 	endobjectchange((INTBIG)ni, VNODEINST);
 }
 
-/*
- * routine to handle changes to the DRC window.
- */
-void us_teceddrcchange(WINDOWPART *w, INTSML nature, char *oldline, char *newline, INTBIG changed)
-{
-	INTSML connected, layer1, layer2;
-	INTBIG amt;
-	REGISTER INTBIG i;
-	REGISTER char **newlist;
-
-	/* ignore deletions and insertions */
-	if (nature == DELETETEXTLINE || nature == INSERTTEXTLINE) return;
-
-	/* handle entire replacements */
-	if (nature == REPLACEALLTEXT)
-	{
-		newlist = (char **)newline;
-		for(i=0; i<changed; i++)
-			us_teceddrcchange(w, REPLACETEXTLINE, "", newlist[i], i);
-		return;
-	}
-
-	/* parse the new line and print an error if found */
-	while (*newline == ' ') newline++;
-	if (*newline == 0) return;
-	if (us_tecedgetdrc(newline, &connected, &amt, &layer1, &layer2,
-		us_teceddrclayers, us_teceddrclayernames) != 0)
-			ttyputmsg(_("DRC line is '%s'"), newline);
-}
-
-/*
- * routine to handle termination of the DRC window
- */
-void us_teceddrcterm(WINDOWPART *w)
-{
-	REGISTER INTSML i, j, l;
-	REGISTER char **lines, *pt;
-	INTSML connected, layer1, layer2;
-	INTBIG amt;
-
-	us_teceddrc = 0;
-
-	/* get rules back into library */
-	l = (INTSML)us_totallines(w);
-	if (l != 0)
-	{
-		lines = (char **)emalloc(l * (sizeof (char *)), el_tempcluster);
-		if (lines != 0)
-		{
-			for(i=j=0; i<l; i++)
-			{
-				pt = us_getline(w, i);
-				while (*pt == ' ') pt++;
-				if (*pt == 0) continue;
-				if (us_tecedgetdrc(pt, &connected, &amt, &layer1, &layer2, us_teceddrclayers,
-					us_teceddrclayernames) != 0) ttyputmsg(_("DRC line is '%s'"), pt);
-				(void)allocstring(&lines[j++], pt, el_tempcluster);
-			}
-			if (j > 0)
-				(void)setval((INTBIG)el_curlib, VLIBRARY, "EDTEC_DRC", (INTBIG)lines,
-					VSTRING|VISARRAY|(j<<VLENGTHSH));
-			for(i=0; i<j; i++) efree(lines[i]);
-			efree((char *)lines);
-		}
-	}
-
-	/* now do the rest of the termination of this window */
-	us_editorterm(w);
-}
-
 /*
  * routine to call up the facet "facetname" (either create it or reedit it)
  * returns NONODEPROTO if there is an error or the facet exists
