@@ -1,4 +1,4 @@
-/*
+ /*
  * Electric(tm) VLSI Design System
  *
  * File: usredtecg.c
@@ -35,10 +35,10 @@
 #include "tecgen.h"
 #include "tecart.h"
 #include "usr.h"
+#include "drc.h"
 #include "usredtec.h"
 
 /* prototypes for local routines */
-char *us_tecedstripdash(char*);
 NODEINST *us_tecedplacegeom(POLYGON*, NODEPROTO*);
 void us_tecedsetlist(NODEINST*, POLYGON*, INTBIG, INTBIG, INTBIG, INTBIG);
 
@@ -49,16 +49,16 @@ void us_tecedsetlist(NODEINST*, POLYGON*, INTBIG, INTBIG, INTBIG, INTBIG);
 LIBRARY *us_tecedmakelibfromtech(TECHNOLOGY *tech)
 {
 	REGISTER LIBRARY *lib;
-	REGISTER char **drclist, *lay, *dxf, **sequence, **varnames, *fname;
-	REGISTER INTBIG i, j, k, l, total, e, xs, ys, oldlam, *newmap, *mapptr,
-		drccount, drcindex, tcon, func, serp, gds, nodexpos, portdescript, bits,
-		drcminwid, height3d, thick3d, lambda, min2x, min2y;
+	REGISTER char *lay, *dxf, **sequence, **varnames, *fname;
+	REGISTER INTBIG i, j, k, total, e, xs, ys, oldlam, *newmap, *mapptr,
+		tcon, func, serp, gds, nodexpos, portdescript, bits,
+		height3d, thick3d, lambda, min2x, min2y;
 	REGISTER INTSML first;
 	INTBIG lx, hx, ly, hy, xpos[4], ypos[4], xsc[4], ysc[4], lxo, hxo, lyo, hyo,
 		lxp, hxp, lyp, hyp;
 	REGISTER NODEPROTO *np, **nplist, *pnp, **aplist;
 	float spires, spicap, spiecap;
-	REGISTER VARIABLE *var, *var2, *var3, *var5, *var6, *var7, *var8, *var9,
+	REGISTER VARIABLE *var, *var2, *var3, *var5, *var6, *var7, *var8,
 		*var10, *var11, *varred, *vargreen, *varblue;
 	REGISTER NODEINST *ni, *oni, *nni;
 	REGISTER ARCINST *ai;
@@ -67,6 +67,7 @@ LIBRARY *us_tecedmakelibfromtech(TECHNOLOGY *tech)
 	REGISTER CELL *c;
 	REGISTER GRAPHICS *desc;
 	static POLYGON *poly = NOPOLYGON;
+	REGISTER DRCRULES *rules;
 	REGISTER TECH_POLYGON *ll;
 	REGISTER TECH_NODES *techn;
 	REGISTER TECH_COLORMAP *colmap;
@@ -138,7 +139,6 @@ LIBRARY *us_tecedmakelibfromtech(TECHNOLOGY *tech)
 	var5 = getval((INTBIG)tech, VTECHNOLOGY, VFLOAT|VISARRAY, "SIM_spice_resistance");
 	var6 = getval((INTBIG)tech, VTECHNOLOGY, VFLOAT|VISARRAY, "SIM_spice_capacitance");
 	var7 = getval((INTBIG)tech, VTECHNOLOGY, VFLOAT|VISARRAY, "SIM_spice_edge_capacitance");
-	var9 = getval((INTBIG)tech, VTECHNOLOGY, VFRACT|VISARRAY, "DRC_min_width");
 	var10 = getval((INTBIG)tech, VTECHNOLOGY, VINTEGER|VISARRAY, "TECH_layer_3dheight");
 	var11 = getval((INTBIG)tech, VTECHNOLOGY, VINTEGER|VISARRAY, "TECH_layer_3dthickness");
 	for(i=0; i<total; i++)
@@ -146,7 +146,7 @@ LIBRARY *us_tecedmakelibfromtech(TECHNOLOGY *tech)
 		desc = tech->layers[i];
 		(void)initinfstr();
 		(void)addstringtoinfstr("layer-");
-		(void)addstringtoinfstr(us_tecedstripdash(layername(tech, (INTSML)i)));
+		(void)addstringtoinfstr(us_tecedstripdash(layername(tech, i)));
 		fname = returninfstr();
 
 		/* make sure the layer doesn't exist */
@@ -177,10 +177,6 @@ LIBRARY *us_tecedmakelibfromtech(TECHNOLOGY *tech)
 		if (var7 == NOVARIABLE) spiecap = 0.0; else
 			spiecap = ((float *)var7->addr)[i];
 
-		/* compute the DRC information */
-		if (var9 == NOVARIABLE) drcminwid = -WHOLE; else
-			drcminwid = ((INTBIG *)var9->addr)[i];
-
 		/* compute the 3D information */
 		if (var10 == NOVARIABLE) height3d = 0; else
 			height3d = ((INTBIG *)var10->addr)[i];
@@ -189,9 +185,9 @@ LIBRARY *us_tecedmakelibfromtech(TECHNOLOGY *tech)
 
 		/* build the layer facet */
 		us_tecedmakelayer(np, desc->col, desc->raster,
-			(INTSML)(desc->colstyle&(NATURE|OUTLINEPAT)), lay, layerfunction(tech, (INTSML)i),
+			(INTSML)(desc->colstyle&(NATURE|OUTLINEPAT)), lay, layerfunction(tech, i),
 				us_layerletters(tech, (INTSML)i), dxf, gds, spires, spicap, spiecap,
-					drcminwid, height3d, thick3d);
+					height3d, thick3d);
 	}
 
 	/* save the layer sequence */
@@ -236,80 +232,64 @@ LIBRARY *us_tecedmakelibfromtech(TECHNOLOGY *tech)
 	}
 
 	/* create the design rule information */
-	var2 = getval((INTBIG)tech, VTECHNOLOGY, VFRACT|VISARRAY, "DRC_min_unconnected_distances");
-	var3 = getval((INTBIG)tech, VTECHNOLOGY, VFRACT|VISARRAY, "DRC_min_connected_distances");
-	if (var2 != NOVARIABLE || var3 != NOVARIABLE)
-	{
-		/* design rules exist: count them */
-		drccount = 0;
-		if (var2 != NOVARIABLE)
-		{
-			l = getlength(var2);
-			for(i=0; i<l; i++) if (((INTBIG *)var2->addr)[i] != XX) drccount++;
-		}
-		if (var3 != NOVARIABLE)
-		{
-			l = getlength(var3);
-			for(i=0; i<l; i++) if (((INTBIG *)var3->addr)[i] != XX) drccount++;
-		}
+	rules = dr_allocaterules(total, tech->techname);
+	if (rules == NODRCRULES) return(NOLIBRARY);
+	for(i=0; i<total; i++)
+		(void)allocstring(&rules->layernames[i], us_tecedstripdash(layername(tech, i)), el_tempcluster);
+	var = getvalkey((INTBIG)tech, VTECHNOLOGY, VFRACT|VISARRAY, dr_min_widthkey);
+	if (var != NOVARIABLE)
+		for(i=0; i<rules->numlayers; i++) rules->minwidth[i] = ((INTBIG *)var->addr)[i];
+	var = getvalkey((INTBIG)tech, VTECHNOLOGY, VSTRING|VISARRAY, dr_min_width_rulekey);
+	if (var != NOVARIABLE)
+		for(i=0; i<rules->numlayers; i++)
+			(void)reallocstring(&rules->minwidthR[i], ((char **)var->addr)[i], el_tempcluster);
+	var = getvalkey((INTBIG)tech, VTECHNOLOGY, VFRACT|VISARRAY, dr_connected_distanceskey);
+	if (var != NOVARIABLE)
+		for(i=0; i<rules->utsize; i++) rules->conlist[i] = ((INTBIG *)var->addr)[i];
+	var = getvalkey((INTBIG)tech, VTECHNOLOGY, VSTRING|VISARRAY, dr_connected_distances_rulekey);
+	if (var != NOVARIABLE)
+		for(i=0; i<rules->utsize; i++)
+			(void)reallocstring(&rules->conlistR[i], ((char **)var->addr)[i], el_tempcluster);
+	var = getvalkey((INTBIG)tech, VTECHNOLOGY, VFRACT|VISARRAY, dr_unconnected_distanceskey);
+	if (var != NOVARIABLE)
+		for(i=0; i<rules->utsize; i++) rules->unconlist[i] = ((INTBIG *)var->addr)[i];
+	var = getvalkey((INTBIG)tech, VTECHNOLOGY, VSTRING|VISARRAY, dr_unconnected_distances_rulekey);
+	if (var != NOVARIABLE)
+		for(i=0; i<rules->utsize; i++)
+			(void)reallocstring(&rules->unconlistR[i], ((char **)var->addr)[i], el_tempcluster);
+	var = getvalkey((INTBIG)tech, VTECHNOLOGY, VFRACT|VISARRAY, dr_connected_distancesWkey);
+	if (var != NOVARIABLE)
+		for(i=0; i<rules->utsize; i++) rules->conlistW[i] = ((INTBIG *)var->addr)[i];
+	var = getvalkey((INTBIG)tech, VTECHNOLOGY, VSTRING|VISARRAY, dr_connected_distancesW_rulekey);
+	if (var != NOVARIABLE)
+		for(i=0; i<rules->utsize; i++)
+			(void)reallocstring(&rules->conlistWR[i], ((char **)var->addr)[i], el_tempcluster);
+	var = getvalkey((INTBIG)tech, VTECHNOLOGY, VFRACT|VISARRAY, dr_unconnected_distancesWkey);
+	if (var != NOVARIABLE)
+		for(i=0; i<rules->utsize; i++) rules->unconlistW[i] = ((INTBIG *)var->addr)[i];
+	var = getvalkey((INTBIG)tech, VTECHNOLOGY, VSTRING|VISARRAY, dr_unconnected_distancesW_rulekey);
+	if (var != NOVARIABLE)
+		for(i=0; i<rules->utsize; i++)
+			(void)reallocstring(&rules->unconlistWR[i], ((char **)var->addr)[i], el_tempcluster);
+	var = getvalkey((INTBIG)tech, VTECHNOLOGY, VFRACT|VISARRAY, dr_connected_distancesMkey);
+	if (var != NOVARIABLE)
+		for(i=0; i<rules->utsize; i++) rules->conlistM[i] = ((INTBIG *)var->addr)[i];
+	var = getvalkey((INTBIG)tech, VTECHNOLOGY, VSTRING|VISARRAY, dr_connected_distancesM_rulekey);
+	if (var != NOVARIABLE)
+		for(i=0; i<rules->utsize; i++)
+			(void)reallocstring(&rules->conlistMR[i], ((char **)var->addr)[i], el_tempcluster);
+	var = getvalkey((INTBIG)tech, VTECHNOLOGY, VFRACT|VISARRAY, dr_unconnected_distancesMkey);
+	if (var != NOVARIABLE)
+		for(i=0; i<rules->utsize; i++) rules->unconlistM[i] = ((INTBIG *)var->addr)[i];
+	var = getvalkey((INTBIG)tech, VTECHNOLOGY, VSTRING|VISARRAY, dr_unconnected_distancesM_rulekey);
+	if (var != NOVARIABLE)
+		for(i=0; i<rules->utsize; i++)
+			(void)reallocstring(&rules->unconlistMR[i], ((char **)var->addr)[i], el_tempcluster);
+	var = getvalkey((INTBIG)tech, VTECHNOLOGY, VFRACT, dr_wide_limitkey);
+	if (var != NOVARIABLE) rules->widelimit = var->addr;
 
-		/* nonzero amount: save it */
-		if (drccount != 0)
-		{
-			drclist = (char **)emalloc((drccount * (sizeof (char *))), el_tempcluster);
-			if (drclist == 0) return(NOLIBRARY);
-
-			/* first do the unconnected list */
-			drcindex = 0;
-			if (var2 != NOVARIABLE)
-			{
-				k = 0;
-				for(i=0; i<total; i++) for(j=i; j<total; j++)
-				{
-					if (((INTBIG *)var2->addr)[k] != XX)
-					{
-						(void)initinfstr();
-						(void)addstringtoinfstr(frtoa(((INTBIG *)var2->addr)[k]));
-						(void)addtoinfstr(' ');
-						(void)addstringtoinfstr(us_tecedstripdash(layername(tech, (INTSML)i)));
-						(void)addtoinfstr(' ');
-						(void)addstringtoinfstr(us_tecedstripdash(layername(tech, (INTSML)j)));
-						(void)allocstring(&drclist[drcindex], returninfstr(), el_tempcluster);
-						drcindex++;
-					}
-					k++;
-				}
-			}
-
-			/* now do the connected list */
-			if (var3 != NOVARIABLE)
-			{
-				k = 0;
-				for(i=0; i<total; i++) for(j=i; j<total; j++)
-				{
-					if (((INTBIG *)var3->addr)[k] != XX)
-					{
-						(void)initinfstr();
-						(void)addtoinfstr('c');
-						(void)addstringtoinfstr(frtoa(((INTBIG *)var3->addr)[k]));
-						(void)addtoinfstr(' ');
-						(void)addstringtoinfstr(us_tecedstripdash(layername(tech, (INTSML)i)));
-						(void)addtoinfstr(' ');
-						(void)addstringtoinfstr(us_tecedstripdash(layername(tech, (INTSML)j)));
-						(void)allocstring(&drclist[drcindex], returninfstr(), el_tempcluster);
-						drcindex++;
-					}
-					k++;
-				}
-			}
-
-			/* save the design rules in the library object */
-			(void)setval((INTBIG)lib, VLIBRARY, "EDTEC_DRC", (INTBIG)drclist,
-				VSTRING|VISARRAY|(drccount<<VLENGTHSH));
-			for(i=0; i<drccount; i++) efree(drclist[i]);
-			efree((char *)drclist);
-		}
-	}
+	us_tecedloaddrcmessage(rules, lib);
+	dr_freerules(rules);
 
 	/* create the arc nodes */
 	ttyputmsg(_("Creating the arcs..."));
@@ -348,7 +328,7 @@ LIBRARY *us_tecedmakelibfromtech(TECHNOLOGY *tech)
 		ai->end[1].xpos = -60000 * oldlam / lambda;
 		ai->end[1].ypos = -10000 * oldlam / lambda;
 		ai->length = computedistance(ai->end[0].xpos, ai->end[0].ypos, ai->end[1].xpos, ai->end[1].ypos);
-		j = arcpolys(ai);
+		j = arcpolys(ai, NOWINDOWPART);
 		for(i=0; i<j; i++)
 		{
 			shapearcpoly(ai, (INTSML)i, poly);
@@ -448,7 +428,7 @@ LIBRARY *us_tecedmakelibfromtech(TECHNOLOGY *tech)
 			oni->highy = ypos[e] + ysc[e]/2 + hyp;
 
 			/* place the layers */
-			j = nodepolys(oni, 0);
+			j = nodepolys(oni, 0, NOWINDOWPART);
 			for(i=0; i<j; i++)
 			{
 				shapenodepoly(oni, i, poly);
@@ -509,7 +489,7 @@ LIBRARY *us_tecedmakelibfromtech(TECHNOLOGY *tech)
 				if (ll == 0) continue;
 				if (ll[i].representation != MINBOX) continue;
 				var = setval((INTBIG)ni, VNODEINST, "EDTEC_minbox", (INTBIG)"MIN", VSTRING|VDISPLAY);
-				if (var != NOVARIABLE) var->textdescript = TXTSMALL << VTSIZESH;
+				if (var != NOVARIABLE) var->textdescript = defaulttextsize(3) << VTSIZESH;
 			}
 			if (first != 0) continue;
 
@@ -629,17 +609,16 @@ LIBRARY *us_tecedmakelibfromtech(TECHNOLOGY *tech)
  * in "functionindex"; the layer letters are in "layerletters"; the DXF layer name(s)
  * are "dxf"; the Calma GDS-II layer is in "gds"; the SPICE resistance is in "spires",
  * the SPICE capacitance is in "spicap", the SPICE edge capacitance is in "spiecap",
- * the DRC minimum width is in "drcminwid", the 3D height is in "height3d", and
- * the 3D thickness is in "thick3d".
+ * the 3D height is in "height3d", and the 3D thickness is in "thick3d".
  */
 void us_tecedmakelayer(NODEPROTO *np, INTSML colorindex, UINTSML stip[8], INTSML style,
 	char *ciflayer, INTBIG functionindex, char *layerletters, char *dxf,
-	INTBIG gds, float spires, float spicap, float spiecap, INTBIG drcminwid,
+	INTBIG gds, float spires, float spicap, float spiecap,
 	INTBIG height3d, INTBIG thick3d)
 {
 	REGISTER NODEINST *nicolor, *ni, *laycolor, *laystipple, *laystyle, *laycif,
 		*laydxf, *laycalma, *layfunction, *layletters, *layspires, *layspicap,
-		*layspiecap, *laydrcminwid, *lay3dheight, *lay3dthick;
+		*layspiecap, *lay3dheight, *lay3dthick;
 	REGISTER INTSML i, x, y;
 	REGISTER VARIABLE *var;
 	char line[20], *colorname, *colorsymbol;
@@ -648,7 +627,7 @@ void us_tecedmakelayer(NODEPROTO *np, INTSML colorindex, UINTSML stip[8], INTSML
 
 	laycolor = laystipple = laystyle = laycif = laydxf = laycalma = NONODEINST;
 	layfunction = layletters = layspires = layspicap = layspiecap = NONODEINST;
-	laydrcminwid = lay3dheight = lay3dthick = NONODEINST;
+	lay3dheight = lay3dthick = NONODEINST;
 	for(ni = np->firstnodeinst; ni != NONODEINST; ni = ni->nextnodeinst)
 	{
 		var = getval((INTBIG)ni, VNODEINST, VINTEGER, "EDTEC_option");
@@ -666,7 +645,6 @@ void us_tecedmakelayer(NODEPROTO *np, INTSML colorindex, UINTSML stip[8], INTSML
 			case LAYERSPIRES:    layspires = ni;     break;
 			case LAYERSPICAP:    layspicap = ni;     break;
 			case LAYERSPIECAP:   layspiecap = ni;    break;
-			case LAYERDRCMINWID: laydrcminwid = ni;  break;
 			case LAYER3DHEIGHT:  lay3dheight = ni;   break;
 			case LAYER3DTHICK:   lay3dthick = ni;    break;
 		}
@@ -704,7 +682,7 @@ void us_tecedmakelayer(NODEPROTO *np, INTSML colorindex, UINTSML stip[8], INTSML
 		(void)addstringtoinfstr(colorname);
 		var = setvalkey((INTBIG)ni, VNODEINST, art_messagekey, (INTBIG)returninfstr(),
 			VSTRING|VDISPLAY);
-		if (var != NOVARIABLE) var->textdescript = TXTMEDIUM << VTSIZESH;
+		if (var != NOVARIABLE) var->textdescript = defaulttextsize(2) << VTSIZESH;
 		(void)setval((INTBIG)ni, VNODEINST, "EDTEC_option", LAYERCOLOR, VINTEGER);
 	}
 
@@ -732,7 +710,7 @@ void us_tecedmakelayer(NODEPROTO *np, INTSML colorindex, UINTSML stip[8], INTSML
 		if (ni == NONODEINST) return;
 		var = setvalkey((INTBIG)ni, VNODEINST, art_messagekey, (INTBIG)"Stipple Pattern",
 			VSTRING|VDISPLAY);
-		if (var != NOVARIABLE) var->textdescript = TXTMEDIUM << VTSIZESH;
+		if (var != NOVARIABLE) var->textdescript = defaulttextsize(2) << VTSIZESH;
 	}
 
 	/* create the color style object */
@@ -749,7 +727,7 @@ void us_tecedmakelayer(NODEPROTO *np, INTSML colorindex, UINTSML stip[8], INTSML
 		}
 		var = setvalkey((INTBIG)ni, VNODEINST, art_messagekey, (INTBIG)returninfstr(),
 			VSTRING|VDISPLAY);
-		if (var != NOVARIABLE) var->textdescript = TXTMEDIUM << VTSIZESH;
+		if (var != NOVARIABLE) var->textdescript = defaulttextsize(2) << VTSIZESH;
 		(void)setval((INTBIG)ni, VNODEINST, "EDTEC_option", LAYERSTYLE, VINTEGER);
 	}
 
@@ -763,7 +741,7 @@ void us_tecedmakelayer(NODEPROTO *np, INTSML colorindex, UINTSML stip[8], INTSML
 		(void)addstringtoinfstr(ciflayer);
 		var = setvalkey((INTBIG)ni, VNODEINST, art_messagekey, (INTBIG)returninfstr(),
 			VSTRING|VDISPLAY);
-		if (var != NOVARIABLE) var->textdescript = TXTMEDIUM << VTSIZESH;
+		if (var != NOVARIABLE) var->textdescript = defaulttextsize(2) << VTSIZESH;
 		(void)setval((INTBIG)ni, VNODEINST, "EDTEC_option", LAYERCIF, VINTEGER);
 	}
 
@@ -777,7 +755,7 @@ void us_tecedmakelayer(NODEPROTO *np, INTSML colorindex, UINTSML stip[8], INTSML
 		(void)addstringtoinfstr(dxf);
 		var = setvalkey((INTBIG)ni, VNODEINST, art_messagekey, (INTBIG)returninfstr(),
 			VSTRING|VDISPLAY);
-		if (var != NOVARIABLE) var->textdescript = TXTMEDIUM << VTSIZESH;
+		if (var != NOVARIABLE) var->textdescript = defaulttextsize(2) << VTSIZESH;
 		(void)setval((INTBIG)ni, VNODEINST, "EDTEC_option", LAYERDXF, VINTEGER);
 	}
 
@@ -792,7 +770,7 @@ void us_tecedmakelayer(NODEPROTO *np, INTSML colorindex, UINTSML stip[8], INTSML
 		(void)addstringtoinfstr(line);
 		var = setvalkey((INTBIG)ni, VNODEINST, art_messagekey, (INTBIG)returninfstr(),
 			VSTRING|VDISPLAY);
-		if (var != NOVARIABLE) var->textdescript = TXTMEDIUM << VTSIZESH;
+		if (var != NOVARIABLE) var->textdescript = defaulttextsize(2) << VTSIZESH;
 		(void)setval((INTBIG)ni, VNODEINST, "EDTEC_option", LAYERGDS, VINTEGER);
 	}
 
@@ -806,7 +784,7 @@ void us_tecedmakelayer(NODEPROTO *np, INTSML colorindex, UINTSML stip[8], INTSML
 		us_tecedaddfunstring(functionindex);
 		var = setvalkey((INTBIG)ni, VNODEINST, art_messagekey, (INTBIG)returninfstr(),
 			VSTRING|VDISPLAY);
-		if (var != NOVARIABLE) var->textdescript = TXTMEDIUM << VTSIZESH;
+		if (var != NOVARIABLE) var->textdescript = defaulttextsize(2) << VTSIZESH;
 		(void)setval((INTBIG)ni, VNODEINST, "EDTEC_option", LAYERFUNCTION, VINTEGER);
 	}
 
@@ -820,7 +798,7 @@ void us_tecedmakelayer(NODEPROTO *np, INTSML colorindex, UINTSML stip[8], INTSML
 		(void)addstringtoinfstr(layerletters);
 		var = setvalkey((INTBIG)ni, VNODEINST, art_messagekey, (INTBIG)returninfstr(),
 			VSTRING|VDISPLAY);
-		if (var != NOVARIABLE) var->textdescript = TXTMEDIUM << VTSIZESH;
+		if (var != NOVARIABLE) var->textdescript = defaulttextsize(2) << VTSIZESH;
 		(void)setval((INTBIG)ni, VNODEINST, "EDTEC_option", LAYERLETTERS, VINTEGER);
 	}
 
@@ -835,7 +813,7 @@ void us_tecedmakelayer(NODEPROTO *np, INTSML colorindex, UINTSML stip[8], INTSML
 		(void)addstringtoinfstr(line);
 		var = setvalkey((INTBIG)ni, VNODEINST, art_messagekey, (INTBIG)returninfstr(),
 			VSTRING|VDISPLAY);
-		if (var != NOVARIABLE) var->textdescript = TXTMEDIUM << VTSIZESH;
+		if (var != NOVARIABLE) var->textdescript = defaulttextsize(2) << VTSIZESH;
 		(void)setval((INTBIG)ni, VNODEINST, "EDTEC_option", LAYERSPIRES, VINTEGER);
 	}
 
@@ -850,7 +828,7 @@ void us_tecedmakelayer(NODEPROTO *np, INTSML colorindex, UINTSML stip[8], INTSML
 		(void)addstringtoinfstr(line);
 		var = setvalkey((INTBIG)ni, VNODEINST, art_messagekey, (INTBIG)returninfstr(),
 			VSTRING|VDISPLAY);
-		if (var != NOVARIABLE) var->textdescript = TXTMEDIUM << VTSIZESH;
+		if (var != NOVARIABLE) var->textdescript = defaulttextsize(2) << VTSIZESH;
 		(void)setval((INTBIG)ni, VNODEINST, "EDTEC_option", LAYERSPICAP, VINTEGER);
 	}
 
@@ -865,29 +843,14 @@ void us_tecedmakelayer(NODEPROTO *np, INTSML colorindex, UINTSML stip[8], INTSML
 		(void)addstringtoinfstr(line);
 		var = setvalkey((INTBIG)ni, VNODEINST, art_messagekey, (INTBIG)returninfstr(),
 			VSTRING|VDISPLAY);
-		if (var != NOVARIABLE) var->textdescript = TXTMEDIUM << VTSIZESH;
+		if (var != NOVARIABLE) var->textdescript = defaulttextsize(2) << VTSIZESH;
 		(void)setval((INTBIG)ni, VNODEINST, "EDTEC_option", LAYERSPIECAP, VINTEGER);
 	}
 
-	/* create the DRC minimum width object */
-	if (laydrcminwid == NONODEINST)
-	{
-		ni = newnodeinst(gen_invispinprim, 0, 40000, -38000, -34000, 0, 0, np);
-		if (ni == NONODEINST) return;
-		(void)initinfstr();
-		(void)addstringtoinfstr(TECEDNODETEXTDRCMINWID);
-		(void)strcpy(line, frtoa(drcminwid));
-		(void)addstringtoinfstr(line);
-		var = setvalkey((INTBIG)ni, VNODEINST, art_messagekey, (INTBIG)returninfstr(),
-			VSTRING|VDISPLAY);
-		if (var != NOVARIABLE) var->textdescript = TXTMEDIUM << VTSIZESH;
-		(void)setval((INTBIG)ni, VNODEINST, "EDTEC_option", LAYERDRCMINWID, VINTEGER);
-	}
-
 	/* create the 3D height object */
 	if (lay3dheight == NONODEINST)
 	{
-		ni = newnodeinst(gen_invispinprim, 0, 40000, -44000, -40000, 0, 0, np);
+		ni = newnodeinst(gen_invispinprim, 0, 40000, -38000, -34000, 0, 0, np);
 		if (ni == NONODEINST) return;
 		(void)initinfstr();
 		(void)addstringtoinfstr(TECEDNODETEXT3DHEIGHT);
@@ -895,14 +858,14 @@ void us_tecedmakelayer(NODEPROTO *np, INTSML colorindex, UINTSML stip[8], INTSML
 		(void)addstringtoinfstr(line);
 		var = setvalkey((INTBIG)ni, VNODEINST, art_messagekey, (INTBIG)returninfstr(),
 			VSTRING|VDISPLAY);
-		if (var != NOVARIABLE) var->textdescript = TXTMEDIUM << VTSIZESH;
+		if (var != NOVARIABLE) var->textdescript = defaulttextsize(2) << VTSIZESH;
 		(void)setval((INTBIG)ni, VNODEINST, "EDTEC_option", LAYER3DHEIGHT, VINTEGER);
 	}
 
 	/* create the 3D thickness object */
 	if (lay3dthick == NONODEINST)
 	{
-		ni = newnodeinst(gen_invispinprim, 0, 40000, -50000, -46000, 0, 0, np);
+		ni = newnodeinst(gen_invispinprim, 0, 40000, -44000, -40000, 0, 0, np);
 		if (ni == NONODEINST) return;
 		(void)initinfstr();
 		(void)addstringtoinfstr(TECEDNODETEXT3DTHICK);
@@ -910,7 +873,7 @@ void us_tecedmakelayer(NODEPROTO *np, INTSML colorindex, UINTSML stip[8], INTSML
 		(void)addstringtoinfstr(line);
 		var = setvalkey((INTBIG)ni, VNODEINST, art_messagekey, (INTBIG)returninfstr(),
 			VSTRING|VDISPLAY);
-		if (var != NOVARIABLE) var->textdescript = TXTMEDIUM << VTSIZESH;
+		if (var != NOVARIABLE) var->textdescript = defaulttextsize(2) << VTSIZESH;
 		(void)setval((INTBIG)ni, VNODEINST, "EDTEC_option", LAYER3DTHICK, VINTEGER);
 	}
 }
@@ -956,7 +919,7 @@ void us_tecedmakearc(NODEPROTO *np, INTBIG func, INTBIG fixang, INTBIG wipes, IN
 	(void)addstringtoinfstr(TECEDNODETEXTFUNCTION);
 	(void)addstringtoinfstr(us_tecarc_functions[func].name);
 	var = setvalkey((INTBIG)ni, VNODEINST, art_messagekey, (INTBIG)returninfstr(), VSTRING|VDISPLAY);
-	if (var != NOVARIABLE) var->textdescript = TXTMEDIUM << VTSIZESH;
+	if (var != NOVARIABLE) var->textdescript = defaulttextsize(2) << VTSIZESH;
 	(void)setval((INTBIG)ni, VNODEINST, "EDTEC_option", ARCFUNCTION, VINTEGER);
 
 	/* create the Fixed-angle object */
@@ -964,7 +927,7 @@ void us_tecedmakearc(NODEPROTO *np, INTBIG func, INTBIG fixang, INTBIG wipes, IN
 	if (ni == NONODEINST) return;
 	if (fixang != 0) str = "Fixed-angle: Yes"; else str = "Fixed-angle: No";
 	var = setvalkey((INTBIG)ni, VNODEINST, art_messagekey, (INTBIG)str, VSTRING|VDISPLAY);
-	if (var != NOVARIABLE) var->textdescript = TXTMEDIUM << VTSIZESH;
+	if (var != NOVARIABLE) var->textdescript = defaulttextsize(2) << VTSIZESH;
 	(void)setval((INTBIG)ni, VNODEINST, "EDTEC_option", ARCFIXANG, VINTEGER);
 
 	/* create the Wipe object */
@@ -972,7 +935,7 @@ void us_tecedmakearc(NODEPROTO *np, INTBIG func, INTBIG fixang, INTBIG wipes, IN
 	if (ni == NONODEINST) return;
 	if (wipes != 0) str = "Wipes pins: Yes"; else str = "Wipes pins: No";
 	var = setvalkey((INTBIG)ni, VNODEINST, art_messagekey, (INTBIG)str, VSTRING|VDISPLAY);
-	if (var != NOVARIABLE) var->textdescript = TXTMEDIUM << VTSIZESH;
+	if (var != NOVARIABLE) var->textdescript = defaulttextsize(2) << VTSIZESH;
 	(void)setval((INTBIG)ni, VNODEINST, "EDTEC_option", ARCWIPESPINS, VINTEGER);
 
 	/* create the Noextend object */
@@ -980,7 +943,7 @@ void us_tecedmakearc(NODEPROTO *np, INTBIG func, INTBIG fixang, INTBIG wipes, IN
 	if (ni == NONODEINST) return;
 	if (noextend == 0) str = "Extend arcs: Yes"; else str = "Extend arcs: No";
 	var = setvalkey((INTBIG)ni, VNODEINST, art_messagekey, (INTBIG)str, VSTRING|VDISPLAY);
-	if (var != NOVARIABLE) var->textdescript = TXTMEDIUM << VTSIZESH;
+	if (var != NOVARIABLE) var->textdescript = defaulttextsize(2) << VTSIZESH;
 	(void)setval((INTBIG)ni, VNODEINST, "EDTEC_option", ARCNOEXTEND, VINTEGER);
 
 	/* create the Angle Increment object */
@@ -988,7 +951,7 @@ void us_tecedmakearc(NODEPROTO *np, INTBIG func, INTBIG fixang, INTBIG wipes, IN
 	if (ni == NONODEINST) return;
 	(void)sprintf(line, "Angle increment: %ld", anginc);
 	var = setvalkey((INTBIG)ni, VNODEINST, art_messagekey, (INTBIG)line, VSTRING|VDISPLAY);
-	if (var != NOVARIABLE) var->textdescript = TXTMEDIUM << VTSIZESH;
+	if (var != NOVARIABLE) var->textdescript = defaulttextsize(2) << VTSIZESH;
 	(void)setval((INTBIG)ni, VNODEINST, "EDTEC_option", ARCINC, VINTEGER);
 }
 
@@ -1031,7 +994,7 @@ void us_tecedmakenode(NODEPROTO *np, INTBIG func, INTSML serp, INTSML square, IN
 		(void)addstringtoinfstr(us_tecnode_functions[func].name);
 		var = setvalkey((INTBIG)ni, VNODEINST, art_messagekey, (INTBIG)returninfstr(),
 			VSTRING|VDISPLAY);
-		if (var != NOVARIABLE) var->textdescript = TXTMEDIUM << VTSIZESH;
+		if (var != NOVARIABLE) var->textdescript = defaulttextsize(2) << VTSIZESH;
 		(void)setval((INTBIG)ni, VNODEINST, "EDTEC_option", NODEFUNCTION, VINTEGER);
 	}
 
@@ -1043,7 +1006,7 @@ void us_tecedmakenode(NODEPROTO *np, INTBIG func, INTSML serp, INTSML square, IN
 		if (serp != 0) str = "Serpentine transistor: yes"; else
 			str = "Serpentine transistor: no";
 		var = setvalkey((INTBIG)ni, VNODEINST, art_messagekey, (INTBIG)str, VSTRING|VDISPLAY);
-		if (var != NOVARIABLE) var->textdescript = TXTMEDIUM << VTSIZESH;
+		if (var != NOVARIABLE) var->textdescript = defaulttextsize(2) << VTSIZESH;
 		(void)setval((INTBIG)ni, VNODEINST, "EDTEC_option", NODESERPENTINE, VINTEGER);
 	}
 
@@ -1055,7 +1018,7 @@ void us_tecedmakenode(NODEPROTO *np, INTBIG func, INTSML serp, INTSML square, IN
 		if (square != 0) str = "Square node: yes"; else
 			str = "Square node: no";
 		var = setvalkey((INTBIG)ni, VNODEINST, art_messagekey, (INTBIG)str, VSTRING|VDISPLAY);
-		if (var != NOVARIABLE) var->textdescript = TXTMEDIUM << VTSIZESH;
+		if (var != NOVARIABLE) var->textdescript = defaulttextsize(2) << VTSIZESH;
 		(void)setval((INTBIG)ni, VNODEINST, "EDTEC_option", NODESQUARE, VINTEGER);
 	}
 
@@ -1067,7 +1030,7 @@ void us_tecedmakenode(NODEPROTO *np, INTBIG func, INTSML serp, INTSML square, IN
 		if (wipes != 0) str = "Invisible with 1 or 2 arcs: yes"; else
 			str = "Invisible with 1 or 2 arcs: no";
 		var = setvalkey((INTBIG)ni, VNODEINST, art_messagekey, (INTBIG)str, VSTRING|VDISPLAY);
-		if (var != NOVARIABLE) var->textdescript = TXTMEDIUM << VTSIZESH;
+		if (var != NOVARIABLE) var->textdescript = defaulttextsize(2) << VTSIZESH;
 		(void)setval((INTBIG)ni, VNODEINST, "EDTEC_option", NODEWIPES, VINTEGER);
 	}
 
@@ -1078,7 +1041,7 @@ void us_tecedmakenode(NODEPROTO *np, INTBIG func, INTSML serp, INTSML square, IN
 		if (ni == NONODEINST) return;
 		if (lockable != 0) str = "Lockable: yes"; else str = "Lockable: no";
 		var = setvalkey((INTBIG)ni, VNODEINST, art_messagekey, (INTBIG)str, VSTRING|VDISPLAY);
-		if (var != NOVARIABLE) var->textdescript = TXTMEDIUM << VTSIZESH;
+		if (var != NOVARIABLE) var->textdescript = defaulttextsize(2) << VTSIZESH;
 		(void)setval((INTBIG)ni, VNODEINST, "EDTEC_option", NODELOCKABLE, VINTEGER);
 	}
 }
@@ -1112,7 +1075,7 @@ void us_tecedmakeinfo(NODEPROTO *np, INTBIG lambda, char *description)
 		if (ni == NONODEINST) return;
 		(void)sprintf(line, "Lambda: %ld", lambda);
 		var = setvalkey((INTBIG)ni, VNODEINST, art_messagekey, (INTBIG)line, VSTRING|VDISPLAY);
-		if (var != NOVARIABLE) var->textdescript = TXTMEDIUM << VTSIZESH;
+		if (var != NOVARIABLE) var->textdescript = defaulttextsize(2) << VTSIZESH;
 		(void)setval((INTBIG)ni, VNODEINST, "EDTEC_option", TECHLAMBDA, VINTEGER);
 	}
 
@@ -1126,7 +1089,7 @@ void us_tecedmakeinfo(NODEPROTO *np, INTBIG lambda, char *description)
 		(void)addstringtoinfstr(description);
 		var = setvalkey((INTBIG)ni, VNODEINST, art_messagekey, (INTBIG)returninfstr(),
 			VSTRING|VDISPLAY);
-		if (var != NOVARIABLE) var->textdescript = TXTMEDIUM << VTSIZESH;
+		if (var != NOVARIABLE) var->textdescript = defaulttextsize(2) << VTSIZESH;
 		(void)setval((INTBIG)ni, VNODEINST, "EDTEC_option", TECHDESCRIPT, VINTEGER);
 	}
 }
@@ -1213,7 +1176,7 @@ NODEINST *us_tecedplacegeom(POLYGON *poly, NODEPROTO *np)
 			endobjectchange((INTBIG)nni, VNODEINST);
 			return(nni);
 		case OPENEDT3:
-			nni = newnodeinst(art_openedfardottepolygonprim, lx, hx, ly, hy, 0, 0, np);
+			nni = newnodeinst(art_openedthickerpolygonprim, lx, hx, ly, hy, 0, 0, np);
 			if (nni == NONODEINST) return(NONODEINST);
 			us_tecedsetlist(nni, poly, lx, hx, ly, hy);
 			endobjectchange((INTBIG)nni, VNODEINST);
