@@ -36,9 +36,14 @@
 
 /****************************** LIBRARY GRAPHING *****************************/
 
-#define	XSCALE       12000	/* horizontal distance between words */
-#define	YSCALE     (-40000)	/* vertical distance between words */
-#define	YOFFSET       6000	/* vertical offset between adjacent words */
+typedef struct
+{
+	INTBIG x, y;
+	INTBIG yoff;
+	NODEINST *pin;
+} NODEDESCR;
+
+NODEPROTO *us_graphmainview(NODEPROTO *np);
 
 void us_graphfacets(void)
 {
@@ -47,8 +52,10 @@ void us_graphfacets(void)
 	REGISTER ARCINST *ai;
 	REGISTER LIBRARY *lib;
 	REGISTER INTSML more, maxdepth, color;
-	REGISTER INTBIG *xval, i, x, y, xe, ye, clock, maxwidth, xsc;
+	REGISTER INTBIG *xval, *yoff, i, x, y, xe, ye, clock, maxwidth, xsc, lambda, xscale, yscale, yoffset;
 	REGISTER PORTPROTO *pinpp;
+	REGISTER VARIABLE *var;
+	REGISTER NODEDESCR *nd, *ndsub;
 
 	pinpp = art_pinprim->firstportproto;
 
@@ -56,8 +63,8 @@ void us_graphfacets(void)
 	graphnp = newnodeproto("FacetStructure", el_curlib);
 	if (graphnp == NONODEPROTO) return;
 	if (graphnp->lastversion != NONODEPROTO)
-		ttyputverbose(_("Creating new version of facet: FacetStructure")); else
-			ttyputverbose(_("Creating facet: FacetStructure"));
+		ttyputverbose(M_("Creating new version of facet: FacetStructure")); else
+			ttyputverbose(M_("Creating facet: FacetStructure"));
 
 	/* clear flags on all of the facets */
 	for(lib = el_curlib; lib != NOLIBRARY; lib = lib->nextlibrary)
@@ -68,9 +75,6 @@ void us_graphfacets(void)
 	for(np = el_curlib->firstnodeproto; np != NONODEPROTO; np = np->nextnodeproto)
 	{
 		if (np->firstinst == NONODEINST) np->temp1 = 0;
-		truenp = iconview(np);
-		if (truenp == NONODEPROTO) continue;
-		if (truenp->firstinst == NONODEINST) truenp->temp1 = 0;
 	}
 
 	/* now place all facets at their proper depth */
@@ -103,14 +107,17 @@ void us_graphfacets(void)
 				}
 			}
 		}
-	}
 
-	/* validity check */
-	for(np = el_curlib->firstnodeproto; np != NONODEPROTO; np = np->nextnodeproto)
-	{
-		if (np->temp1 >= 0) continue;
-		ttyputmsg(_("Warning: facet %s not found in hierarchy"), describenodeproto(np));
-		np->temp1 = 0;
+		/* add in any facets referenced from other libraries */
+		if (more == 0)
+		{
+			for(np = el_curlib->firstnodeproto; np != NONODEPROTO; np = np->nextnodeproto)
+			{
+				if (np->temp1 >= 0) continue;
+				np->temp1 = 0;
+				more++;
+			}
+		}
 	}
 
 	/* now assign X coordinates to each facet */
@@ -118,29 +125,96 @@ void us_graphfacets(void)
 	maxwidth = 0;
 	xval = emalloc((SIZEOFINTBIG * maxdepth), el_tempcluster);
 	if (xval == 0) return;
-	for(i=0; i<maxdepth; i++) xval[i] = 0;
+	yoff = emalloc((SIZEOFINTBIG * maxdepth), el_tempcluster);
+	if (yoff == 0) return;
+	for(i=0; i<maxdepth; i++) xval[i] = yoff[i] = 0;
 	for(lib = el_curlib; lib != NOLIBRARY; lib = lib->nextlibrary)
 		for(np = lib->firstnodeproto; np != NONODEPROTO; np = np->nextnodeproto)
 	{
 		/* ignore icon facets from the graph (merge with contents) */
 		if (np->temp1 == -1) continue;
-		truenp = contentsview(np);
-		if (truenp != NONODEPROTO) continue;
 
-		xval[np->temp1]++;
+		/* ignore associated facets for now */
+		truenp = us_graphmainview(np);
+		if (truenp != NONODEPROTO &&
+			(np->firstnodeinst == NONODEINST || np->cellview == el_iconview ||
+				np->cellview == el_skeletonview))
+		{
+			np->temp1 = -1;
+			continue;
+		}
+
+		nd = (NODEDESCR *)emalloc(sizeof (NODEDESCR), us_aid->cluster);
+		nd->pin = NONODEINST;
+
+		nd->x = xval[np->temp1];
+		xval[np->temp1] += strlen(describenodeproto(np));
 		if (xval[np->temp1] > maxwidth) maxwidth = xval[np->temp1];
-		np->temp1 = (np->temp1 << 16) | (xval[np->temp1] - 1);
+		nd->y = np->temp1;
+		nd->yoff = 0;
+		np->temp1 = (INTBIG)nd;
+	}
+
+	/* now center each row */
+	for(lib = el_curlib; lib != NOLIBRARY; lib = lib->nextlibrary)
+		for(np = lib->firstnodeproto; np != NONODEPROTO; np = np->nextnodeproto)
+	{
+		if (np->temp1 == -1) continue;
+		nd = (NODEDESCR *)np->temp1;
+		if (xval[nd->y] < maxwidth)
+		{
+			float spread;
+			spread = (float)maxwidth / (float)xval[nd->y];
+			nd->x = roundfloat(nd->x * spread);
+		}
+	}
+
+	/* generate accurate X/Y coordinates */
+	lambda = el_curlib->lambda[art_tech->techindex];
+	xscale = lambda * 2 / 3;
+	yscale = lambda * 20;
+	yoffset = lambda/2;
+	for(lib = el_curlib; lib != NOLIBRARY; lib = lib->nextlibrary)
+		for(np = lib->firstnodeproto; np != NONODEPROTO; np = np->nextnodeproto)
+	{
+		if (np->temp1 == -1) continue;
+		nd = (NODEDESCR *)np->temp1;
+		x = nd->x;   y = nd->y;
+		x = x * xscale;
+		y = -y * yscale + ((yoff[nd->y]++)%2) * yoffset;
+		nd->x = x;   nd->y = y;
+	}
+
+	/* make unattached facets sit with their contents view */
+	for(lib = el_curlib; lib != NOLIBRARY; lib = lib->nextlibrary)
+		for(np = lib->firstnodeproto; np != NONODEPROTO; np = np->nextnodeproto)
+	{
+		if (np->temp1 != -1) continue;
+		if (np->firstnodeinst != NONODEINST && np->cellview != el_iconview &&
+				np->cellview != el_skeletonview) continue;
+		truenp = us_graphmainview(np);
+		if (truenp == NONODEPROTO) continue;
+		if (truenp->temp1 == -1) continue;
+
+		nd = (NODEDESCR *)truenp->temp1;
+		ndsub = (NODEDESCR *)emalloc(sizeof (NODEDESCR), us_aid->cluster);
+		ndsub->pin = NONODEINST;
+		nd->yoff += yoffset*2;
+		ndsub->x = nd->x;   ndsub->y = nd->y + nd->yoff;
+		np->temp1 = (INTBIG)ndsub;
 	}
 
 	/* write the header message */
-	xsc = maxwidth * XSCALE;
-	ni = newnodeinst(gen_invispinprim, xsc/2, xsc/2, -YSCALE, -YSCALE, 0, 0, graphnp);
+	xsc = maxwidth * xscale / 2;
+	ni = newnodeinst(gen_invispinprim, xsc, xsc, yscale, yscale, 0, 0, graphnp);
 	if (ni == NONODEINST) return;
 	endobjectchange((INTBIG)ni, VNODEINST);
 	(void)initinfstr();
 	(void)formatinfstr(_("Structure of library %s"), el_curlib->libname);
-	(void)setvalkey((INTBIG)ni, VNODEINST, art_messagekey, (INTBIG)returninfstr(),
+	var = setvalkey((INTBIG)ni, VNODEINST, art_messagekey, (INTBIG)returninfstr(),
 		VSTRING|VDISPLAY);
+	if (var != NOVARIABLE)
+		var->textdescript = TXT6L << VTSIZESH;
 
 	/* place the components */
 	for(lib = el_curlib; lib != NOLIBRARY; lib = lib->nextlibrary)
@@ -148,22 +222,22 @@ void us_graphfacets(void)
 	{
 		if (np == graphnp) continue;
 		if (np->temp1 == -1) continue;
+		nd = (NODEDESCR *)np->temp1;
 
 		/* always use the contents facet, not the icon */
-		truenp = contentsview(np);
-		if (truenp == NONODEPROTO) truenp = np;
+		truenp = np;
 
-		x = (truenp->temp1 & 0xFFFF) * xsc / xval[(truenp->temp1 >> 16) & 0xFFFF];
-		y = ((truenp->temp1 >> 16) & 0xFFFF) * YSCALE;
-		if ((truenp->temp1&0xFFFF) % 2 != 0) y += YOFFSET;
+		x = nd->x;   y = nd->y;
 		ni = newnodeinst(gen_invispinprim, x, x, y, y, 0, 0, graphnp);
 		if (ni == NONODEINST) return;
 		endobjectchange((INTBIG)ni, VNODEINST);
-		np->temp2 = (INTBIG)ni;
+		nd->pin = ni;
 
 		/* write the facet name in the node */
-		(void)setvalkey((INTBIG)ni, VNODEINST, art_messagekey, (INTBIG)describenodeproto(truenp),
+		var = setvalkey((INTBIG)ni, VNODEINST, art_messagekey, (INTBIG)describenodeproto(truenp),
 			VSTRING|VDISPLAY);
+		if (var != NOVARIABLE)
+			var->textdescript = TXT1L << VTSIZESH;
 	}
 
 	/* build wires between the hierarchical levels */
@@ -181,6 +255,7 @@ void us_graphfacets(void)
 		if (truenp == NONODEPROTO) truenp = np;
 		if (truenp->temp1 == -1) continue;
 
+		nd = (NODEDESCR *)truenp->temp1;
 		toppin = NONODEINST;
 		clock++;
 		for(ni = truenp->firstnodeinst; ni != NONODEINST; ni = ni->nextnodeinst)
@@ -195,12 +270,13 @@ void us_graphfacets(void)
 			truesubnp->temp2 = clock;
 
 			/* draw a line from facet "truenp" to facet "truesubnp" */
-			x = (truenp->temp1 & 0xFFFF) * xsc / xval[(truenp->temp1 >> 16) & 0xFFFF];
-			y = ((truenp->temp1 >> 16) & 0xFFFF) * YSCALE + gen_invispinprim->lowy;
-			if ((truenp->temp1&0xFFFF) % 2 != 0) y += YOFFSET;
-			xe = (truesubnp->temp1 & 0xFFFF) * xsc / xval[(truesubnp->temp1 >> 16) & 0xFFFF];
-			ye = ((truesubnp->temp1 >> 16) & 0xFFFF) * YSCALE + gen_invispinprim->highy;
-			if ((truesubnp->temp1&0xFFFF) % 2 != 0) ye += YOFFSET;
+			x = nd->x;
+			y = nd->y + gen_invispinprim->highy;
+
+			if (truesubnp->temp1 == -1) continue;
+			ndsub = (NODEDESCR *)truesubnp->temp1;
+			xe = ndsub->x;
+			ye = ndsub->y + gen_invispinprim->highy;
 			if (toppin == NONODEINST)
 			{
 				toppin = newnodeinst(art_pinprim, x+art_pinprim->lowx, x+art_pinprim->highx,
@@ -219,12 +295,52 @@ void us_graphfacets(void)
 
 			/* set an appropriate color on the arc (red for jumps of more than 1 level of depth) */
 			color = BLUE;
-			if (((truesubnp->temp1 >> 16) & 0xFFFF) - ((truenp->temp1 >> 16) & 0xFFFF) != 1) color = RED;
+			if (nd->y - ndsub->y > yscale+yoffset+yoffset) color = RED;
 			(void)setvalkey((INTBIG)ai, VARCINST, art_colorkey, color, VINTEGER);
 		}
 	}
 
+	/* free space */
+	for(lib = el_curlib; lib != NOLIBRARY; lib = lib->nextlibrary)
+		for(np = lib->firstnodeproto; np != NONODEPROTO; np = np->nextnodeproto)
+	{
+		if (np->temp1 == -1) continue;
+		efree((char *)np->temp1);
+	}
 	efree((char *)xval);
+	efree((char *)yoff);
+}
+
+/*
+ * routine to find the main facet that "np" is associated with in the graph.  This code is
+ * essentially the same as "contentsfacet()" except that any original type is allowed.
+ * Returns NONODEPROTO if the facet is not associated.
+ */
+NODEPROTO *us_graphmainview(NODEPROTO *np)
+{
+	REGISTER NODEPROTO *rnp;
+
+	/* primitives have no contents view */
+	if (np == NONODEPROTO) return(NONODEPROTO);
+	if (np->primindex != 0) return(NONODEPROTO);
+
+	/* first check to see if there is a schematics link */
+	for(rnp = np->cell->firstincell; rnp != NONODEPROTO; rnp = rnp->nextincell)
+	{
+		if (rnp->cellview == el_schematicview) return(rnp);
+		if ((rnp->cellview->viewstate&MULTIPAGEVIEW) != 0) return(rnp);
+	}
+
+	/* now check to see if there is any layout link */
+	for(rnp = np->cell->firstincell; rnp != NONODEPROTO; rnp = rnp->nextincell)
+		if (rnp->cellview == el_layoutview) return(rnp);
+
+	/* finally check to see if there is any "unknown" link */
+	for(rnp = np->cell->firstincell; rnp != NONODEPROTO; rnp = rnp->nextincell)
+		if (rnp->cellview == el_unknownview) return(rnp);
+
+	/* no contents found */
+	return(NONODEPROTO);
 }
 
 /****************************** COMMAND GRAPHING ******************************/
@@ -232,11 +348,11 @@ void us_graphfacets(void)
 #define MAXILLUSTRATEDEPTH  100		/* maximum depth of command graph */
 #define FORCEDDEPTH           7		/* required depth of command graph */
 #define MAXILLUSTRATEWIDTH  200		/* maximum depth of command graph */
-#define	XCOMSCALE          1000		/* horizontal distance between words */
-#define	YCOMSCALE       (-10000)	/* vertical distance between words */
-#define	YCOMOFFSET         1000		/* vertical offset between words */
+#define XCOMSCALE          1000		/* horizontal distance between words */
+#define YCOMSCALE       (-10000)	/* vertical distance between words */
+#define YCOMOFFSET         1000		/* vertical offset between words */
 
-#define	NOCOMILL ((COMILL *)-1)
+#define NOCOMILL ((COMILL *)-1)
 
 typedef struct Icomill
 {
@@ -339,7 +455,7 @@ void us_illustratecommandset(void)
 
 		/* set the node name, color, font */
 		var = setvalkey((INTBIG)ni, VNODEINST, art_messagekey, (INTBIG)ci->name, VSTRING|VDISPLAY);
-		if (var != NOVARIABLE) var->textdescript = TXTMEDIUM << VTSIZESH;
+		if (var != NOVARIABLE) var->textdescript = defaulttextsize(2) << VTSIZESH;
 		(void)setvalkey((INTBIG)ni, VNODEINST, art_colorkey, RED, VINTEGER);
 	}
 
