@@ -39,12 +39,14 @@
 extern GRAPHICS us_box, us_arbit, us_hbox;
 
 /* working memory for "us_gethighlighted()" */
-static GEOM **us_gethighlist;
-static INTSML us_gethighlistsize = 0;
+static GEOM   **us_gethighlist;
+static INTSML   us_gethighlistsize = 0;
+static INTBIG   us_selectedtexttotal = 0;	/* size of selected text item list */
+static char   **us_selectedtextlist;		/* the selected text items */
 
 /* working memory for "us_getallhighlighted()" */
-static INTBIG *us_getallhighlist;
-static INTSML us_getallhighlistsize = 0;
+static INTBIG  *us_getallhighlist;
+static INTSML   us_getallhighlistsize = 0;
 
 /* prototypes for local routines */
 void us_makecurrentobject(void);
@@ -55,8 +57,13 @@ void us_highlightverbose(GEOM*, PORTPROTO*, INTSML, NODEPROTO*, POLYGON*);
  */
 void us_freehighmemory(void)
 {
+	REGISTER INTBIG i;
+
 	if (us_gethighlistsize != 0) efree((char *)us_gethighlist);
 	if (us_getallhighlistsize != 0) efree((char *)us_getallhighlist);
+	for(i=0; i<us_selectedtexttotal; i++)
+		if (us_selectedtextlist[i] != 0) efree((char *)us_selectedtextlist[i]);
+	if (us_selectedtexttotal > 0) efree((char *)us_selectedtextlist);
 }
 
 /******************** MENU AND WINDOW HIGHLIGHTING ********************/
@@ -295,12 +302,11 @@ void us_clearhighlightcount(void)
  * with extra information.  If "findmore" is nonzero, add this highlight rather
  * than replacing.  If "nobox" is nonzero, do not draw the basic box.
  */
-void us_setfind(HIGHLIGHT *newhigh, INTSML findpoint, INTSML extrainfo, INTSML findmore, INTSML nobox)
+void us_setfind(HIGHLIGHT *newhigh, INTBIG findpoint, INTBIG extrainfo, INTBIG findmore, INTBIG nobox)
 {
 	REGISTER NODEINST *ni;
 	REGISTER VARIABLE *var;
-	REGISTER INTBIG x, y, d, bestdist, bestpoint;
-	REGISTER INTSML i, total;
+	REGISTER INTBIG x, y, d, bestdist, bestpoint, i, total;
 	INTBIG xp, yp, xc, yc;
 	XARRAY trans;
 
@@ -450,23 +456,26 @@ INTSML us_gettwoobjects(GEOM **firstgeom, PORTPROTO **firstport, GEOM **secondge
  * of them, and return the array.
  * If "type" is OBJNODEINST, only the highlighted nodes are returned.
  * If "type" is OBJARCINST, only the highlighted arcs are returned.
- * By ORing the constants, any combination can be requested.  For example,
  * if "type" is OBJNODEINST|OBJARCINST, all highlighted objects are returned.
  * The list is terminated with the value NOGEOM.
+ * If "textcount" and "textinfo" are nonzero, they are set to the number of
+ * text objects that are highlighted (and the highlight descriptors for each).
  */
-GEOM **us_gethighlighted(INTSML type)
+GEOM **us_gethighlighted(INTSML type, INTBIG *textcount, char ***textinfo)
 {
 	HIGHLIGHT high;
 	REGISTER VARIABLE *var;
-	REGISTER INTBIG total, len, i, j, k;
+	REGISTER INTBIG total, len, i, j, k, newcount, numtext;
 	REGISTER INTBIG search;
 	REGISTER GEOM *geom;
 	REGISTER NODEINST *ni;
 	REGISTER ARCINST *ai;
+	REGISTER char **newtext;
 	static GEOM *nulllist[1];
 	static POLYGON *poly = NOPOLYGON;
 	XARRAY trans;
 
+	if (textcount != 0) *textcount = 0;
 	var = getvalkey((INTBIG)us_aid, VAID, VSTRING|VISARRAY, us_highlighted);
 	if (var == NOVARIABLE)
 	{
@@ -476,6 +485,7 @@ GEOM **us_gethighlighted(INTSML type)
 
 	/* first search to see how many objects there are */
 	len = (INTSML)getlength(var);
+	numtext = 0;
 	total = 0;
 	for(i=0; i<len; i++)
 	{
@@ -484,19 +494,17 @@ GEOM **us_gethighlighted(INTSML type)
 			if ((high.fromgeom->entrytype & type) != 0) total++;
 		if ((high.status&HIGHTYPE) == HIGHBBOX)
 		{
-			search = initsearch(high.stalx, high.stahx, high.staly, high.stahy, high.facet);
+			search = initsearch(high.stalx, high.stahx, high.staly, high.stahy,
+				high.facet);
 			while ((geom = nextobject(search)) != NOGEOM)
 				if ((geom->entrytype & type) != 0) total++;
 		}
 
-		/* special case: accept invisible node if it holds highlighted text */
+		/* special case: include in node list if text tied to node and no text requested */
 		if ((high.status&HIGHTYPE) == HIGHTEXT)
 		{
-			if (high.fromvar != NOVARIABLE && high.fromgeom != NOGEOM &&
-				high.fromgeom->entrytype == OBJNODEINST &&
-					high.fromgeom->entryaddr.ni->proto == gen_invispinprim &&
-						(type&OBJNODEINST) != 0)
-							total++;
+			if (us_nodemoveswithtext(&high) != 0 && textcount == 0)
+				total++;
 		}
 	}
 
@@ -504,7 +512,8 @@ GEOM **us_gethighlighted(INTSML type)
 	if (total >= us_gethighlistsize)
 	{
 		if (us_gethighlistsize != 0) efree((char *)us_gethighlist);
-		us_gethighlist = (GEOM **)emalloc(((total+1) * (sizeof (GEOM *))), us_aid->cluster);
+		us_gethighlist = (GEOM **)emalloc(((total+1) * (sizeof (GEOM *))),
+			us_aid->cluster);
 		if (us_gethighlist == 0)
 		{
 			ttyputnomemory();
@@ -541,11 +550,12 @@ GEOM **us_gethighlighted(INTSML type)
 				if (j < total) continue;
 
 				/* check carefully for "edge select" primitives */
-				if (geom->entrytype == OBJNODEINST && geom->entryaddr.ni->proto->primindex != 0 &&
+				if (geom->entrytype == OBJNODEINST &&
+					geom->entryaddr.ni->proto->primindex != 0 &&
 					(geom->entryaddr.ni->proto->userbits&NEDGESELECT) != 0)
 				{
 					ni = geom->entryaddr.ni;
-					k = nodepolys(ni, 0);
+					k = nodepolys(ni, 0, NOWINDOWPART);
 					if (poly == NOPOLYGON) poly = allocstaticpolygon(4, us_aid->cluster);
 					makerot(ni, trans);
 					for(j=0; j<k; j++)
@@ -554,7 +564,8 @@ GEOM **us_gethighlighted(INTSML type)
 						if ((poly->desc->colstyle&INVISIBLE) == 0)
 						{
 							xformpoly(poly, trans);
-							if (polyinrect(poly, high.stalx, high.stahx, high.staly, high.stahy) == 0) break;
+							if (polyinrect(poly, high.stalx, high.stahx,
+								high.staly, high.stahy) == 0) break;
 						}
 					}
 					if (j < k) continue;
@@ -562,14 +573,15 @@ GEOM **us_gethighlighted(INTSML type)
 					(geom->entryaddr.ai->proto->userbits&AEDGESELECT) != 0)
 				{
 					ai = geom->entryaddr.ai;
-					k = arcpolys(ai);
+					k = arcpolys(ai, NOWINDOWPART);
 					if (poly == NOPOLYGON) poly = allocstaticpolygon(4, us_aid->cluster);
 					for(j=0; j<k; j++)
 					{
 						shapearcpoly(ai, j, poly);
 						if ((poly->desc->colstyle&INVISIBLE) == 0)
 						{
-							if (polyinrect(poly, high.stalx, high.stahx, high.staly, high.stahy) == 0) break;
+							if (polyinrect(poly, high.stalx, high.stahx,
+								high.staly, high.stahy) == 0) break;
 						}
 					}
 					if (j < k) continue;
@@ -581,13 +593,50 @@ GEOM **us_gethighlighted(INTSML type)
 		}
 		if ((high.status&HIGHTYPE) == HIGHTEXT)
 		{
-			if (high.fromvar != NOVARIABLE && high.fromgeom != NOGEOM &&
-				high.fromgeom->entrytype == OBJNODEINST &&
-					high.fromgeom->entryaddr.ni->proto == gen_invispinprim &&
-						(type&OBJNODEINST) != 0)
-							us_gethighlist[total++] = high.fromgeom;
+			if (us_nodemoveswithtext(&high) != 0 && textcount == 0)
+			{
+				/* include in node list if text tied to node and no text requested */
+				us_gethighlist[total++] = high.fromgeom;
+			} else
+			{
+				if (textcount != 0)
+				{
+					if (numtext >= us_selectedtexttotal)
+					{
+						newcount = numtext + 5;
+						newtext = (char **)emalloc(newcount * (sizeof (char *)),
+							us_aid->cluster);
+						if (newtext == 0)
+						{
+							nulllist[0] = NOGEOM;
+							return(nulllist);
+						}
+						for(j=0; j<us_selectedtexttotal; j++)
+							newtext[j] = us_selectedtextlist[j];
+						for( ; j<newcount; j++) newtext[j] = 0;
+						if (us_selectedtexttotal > 0) efree((char *)us_selectedtextlist);
+						us_selectedtextlist = newtext;
+						us_selectedtexttotal = newcount;
+					}
+					if (us_selectedtextlist[numtext] == 0)
+					{
+						allocstring(&us_selectedtextlist[numtext],
+							((char **)var->addr)[i], us_aid->cluster);
+					} else
+					{
+						reallocstring(&us_selectedtextlist[numtext],
+							((char **)var->addr)[i], us_aid->cluster);
+					}
+					numtext++;
+				}
+			}
 		}
 	}
+	if (textcount != 0 && textinfo != 0)
+	{
+		*textcount = numtext;
+		*textinfo = us_selectedtextlist;
+	}
 	us_gethighlist[total] = NOGEOM;
 	return(us_gethighlist);
 }
@@ -690,7 +739,7 @@ INTBIG *us_getallhighlighted(void)
 					(geom->entryaddr.ni->proto->userbits&NEDGESELECT) != 0)
 				{
 					ni = geom->entryaddr.ni;
-					k = nodepolys(ni, 0);
+					k = nodepolys(ni, 0, NOWINDOWPART);
 					if (poly == NOPOLYGON) poly = allocstaticpolygon(4, us_aid->cluster);
 					makerot(ni, trans);
 					for(j=0; j<k; j++)
@@ -707,7 +756,7 @@ INTBIG *us_getallhighlighted(void)
 					(geom->entryaddr.ai->proto->userbits&AEDGESELECT) != 0)
 				{
 					ai = geom->entryaddr.ai;
-					k = arcpolys(ai);
+					k = arcpolys(ai, NOWINDOWPART);
 					if (poly == NOPOLYGON) poly = allocstaticpolygon(4, us_aid->cluster);
 					for(j=0; j<k; j++)
 					{
@@ -820,9 +869,9 @@ NODEPROTO *us_getareabounds(INTBIG *lx, INTBIG *hx, INTBIG *ly, INTBIG *hy)
 			} else if (high.fromport != NOPORTPROTO)
 			{
 				portposition(high.fromport->subnodeinst, high.fromport->subportproto, &xc, &yc);
-				us_maketextpoly(high.fromport->protoname, el_curwindowpart, xc, yc,
-					high.fromport->subnodeinst, high.fromport->subnodeinst->proto->tech,
-						high.fromport->textdescript, poly);
+				us_maketextpoly(us_displayedportname(high.fromport, (us_useroptions&EXPORTLABELS) >> EXPORTLABELSSH),
+					el_curwindowpart, xc, yc, high.fromport->subnodeinst,
+						high.fromport->subnodeinst->proto->tech, high.fromport->textdescript, poly);
 			} else if (high.fromgeom->entrytype == OBJNODEINST)
 			{
 				ni = high.fromgeom->entryaddr.ni;
@@ -839,8 +888,13 @@ NODEPROTO *us_getareabounds(INTBIG *lx, INTBIG *hx, INTBIG *ly, INTBIG *hy)
 		}
 		if ((high.status&HIGHTYPE) == HIGHFROM)
 		{
-			tlx = high.fromgeom->lowx;   thx = high.fromgeom->highx;
-			tly = high.fromgeom->lowy;   thy = high.fromgeom->highy;
+			if (high.fromgeom->entrytype == OBJNODEINST)
+			{
+				us_getnodebounds(high.fromgeom->entryaddr.ni, &tlx, &thx, &tly, &thy);
+			} else
+			{
+				us_getarcbounds(high.fromgeom->entryaddr.ai, &tlx, &thx, &tly, &thy);
+			}
 		}
 		if (i == 0)
 		{
@@ -854,6 +908,100 @@ NODEPROTO *us_getareabounds(INTBIG *lx, INTBIG *hx, INTBIG *ly, INTBIG *hy)
 	return(thisfacet);
 }
 
+/*
+ * Routine to move the "numtexts" text objects described (as highlight strings)
+ * in the array "textlist", by "odx" and "ody".
+ */
+void us_moveselectedtext(INTBIG numtexts, char **textlist, INTBIG odx, INTBIG ody)
+{
+	REGISTER INTBIG i, lambda, descript;
+	INTBIG dx, dy, addr, type;
+	REGISTER NODEINST *ni;
+	XARRAY trans;
+	HIGHLIGHT high;
+
+	for(i=0; i<numtexts; i++)
+	{
+		dx = odx;   dy = ody;
+		(void)us_makehighlight(textlist[i], &high);
+		if ((high.status&HIGHTYPE) != HIGHTEXT) continue;
+
+		/* get object and movement amount */
+		us_gethighaddrtype(&high, &addr, &type);
+
+		/* undraw the text */
+		if (type == VPORTPROTO)
+		{
+			ni = ((PORTPROTO *)addr)->subnodeinst;
+			startobjectchange((INTBIG)ni, VNODEINST);
+			if (us_nodemoveswithtext(&high) != 0)
+			{
+				modifynodeinst(ni, dx, dy, dx, dy, 0, 0);
+				endobjectchange((INTBIG)ni, VNODEINST);
+				continue;
+			}
+			if (ni->transpose != 0)
+				makeangle(ni->rotation, ni->transpose, trans); else
+					makeangle((INTSML)((3600-ni->rotation)%3600), 0, trans);
+			xform(dx, dy, &dx, &dy, trans);
+		} else
+		{
+			startobjectchange(addr, type);
+			if (type == VNODEINST)
+			{
+				ni = (NODEINST *)addr;
+				if (us_nodemoveswithtext(&high) != 0)
+				{
+					modifynodeinst(ni, dx, dy, dx, dy, 0, 0);
+					endobjectchange((INTBIG)ni, VNODEINST);
+					continue;
+				}
+				if (ni->transpose != 0)
+					makeangle(ni->rotation, ni->transpose, trans); else
+						makeangle((INTSML)((3600-ni->rotation)%3600), 0, trans);
+				xform(dx, dy, &dx, &dy, trans);
+			}
+		}
+		if (type == VNODEPROTO && high.fromvar != NOVARIABLE)
+			us_undrawfacetvariable(high.fromvar, (NODEPROTO *)addr);
+
+		/* set the new descriptor on the text */
+		lambda = el_curlib->lambda[us_hightech(&high)->techindex];
+		dx = dx*4/lambda;   dy = dy*4/lambda;
+		descript = us_gethighdescript(&high);
+		if ((descript&VTXOFFNEG) != 0) dx -= (descript&VTXOFF) >> VTXOFFSH; else
+			dx += (descript&VTXOFF) >> VTXOFFSH;
+		if ((descript&VTYOFFNEG) != 0) dy -= (descript&VTYOFF) >> VTYOFFSH; else
+			dy += (descript&VTYOFF) >> VTYOFFSH;
+		descript = us_setdescriptoffset(descript, dx, dy);
+		us_modifytextdescript(&high, descript);
+
+		/* redisplay the text */
+		if (type == VNODEPROTO && high.fromvar != NOVARIABLE)
+			us_drawfacetvariable(high.fromvar, (NODEPROTO *)addr);
+		if (type == VPORTPROTO)
+		{
+			endobjectchange((INTBIG)ni, VNODEINST);
+		} else
+		{
+			endobjectchange(addr, type);
+		}
+		(void)us_addhighlight(&high);
+
+		/* modify all higher-level nodes if port moved */
+		if (high.fromvar == NOVARIABLE && high.fromport != NOPORTPROTO)
+		{
+			for(ni = high.fromport->parent->firstinst; ni != NONODEINST; ni = ni->nextinst)
+			{
+				if ((ni->userbits&NEXPAND) != 0 &&
+					(high.fromport->userbits&PORTDRAWN) == 0) continue;
+				startobjectchange((INTBIG)ni, VNODEINST);
+				endobjectchange((INTBIG)ni, VNODEINST);
+			}
+		}
+	}
+}
+
 /*
  * Routine to find a single selected snap point and return it in (x, y).
  * Returns zero if there is not a single snap point.
@@ -975,7 +1123,7 @@ void us_sethighlight(HIGHLIGHT *high, INTSML on)
 			{
 				/* determine size from text */
 				tech = us_hightech(high);
-				screensettextsize(w, truefontsize((INTSML)((descript&VTSIZE)>>VTSIZESH), w, tech));
+				screensettextsize(w, truefontsize((descript&VTSIZE)>>VTSIZESH, w, tech));
 				screengettextsize(w, str, &tsx, &tsy);
 				xw = muldiv(tsx, w->screenhx-w->screenlx, w->usehx-w->uselx);
 				yw = muldiv(tsy, w->screenhy-w->screenly, w->usehy-w->usely);
@@ -1296,24 +1444,24 @@ void us_highlightverbose(GEOM *look, PORTPROTO *pp, INTSML color, NODEPROTO *par
 
 		hni = look->entryaddr.ni;
 		for(ni = par->firstnodeinst; ni != NONODEINST; ni = ni->nextnodeinst)
-			ni->temp1 = 0;
+			ni->userbits &= ~HIGHLIGHTNVER;
 		for(ai = par->firstarcinst; ai != NOARCINST; ai = ai->nextarcinst)
-			ai->temp1 = 0;
+			ai->userbits &= ~HIGHLIGHTAVER;
 
 		/* determine which arcs should be highlighted */
 		for(pi = hni->firstportarcinst; pi != NOPORTARCINST; pi = pi->nextportarcinst)
 		{
 			if (pi->proto->network != pp->network) continue;
 			ai = pi->conarcinst;
-			ai->temp1 = 1;
+			ai->userbits |= HIGHLIGHTAVER;
 			if (ai->network != NONETWORK)
 			{
 				for(oai = par->firstarcinst; oai != NOARCINST; oai = oai->nextarcinst)
 				{
 					if (oai->network != ai->network) continue;
-					oai->temp1 = 1;
-					oai->end[0].nodeinst->temp1 = 1;
-					oai->end[1].nodeinst->temp1 = 1;
+					oai->userbits |= HIGHLIGHTAVER;
+					oai->end[0].nodeinst->userbits |= HIGHLIGHTNVER;
+					oai->end[1].nodeinst->userbits |= HIGHLIGHTNVER;
 				}
 			}
 		}
@@ -1321,7 +1469,7 @@ void us_highlightverbose(GEOM *look, PORTPROTO *pp, INTSML color, NODEPROTO *par
 		/* draw lines along all of the arcs on the network */
 		for(ai = par->firstarcinst; ai != NOARCINST; ai = ai->nextarcinst)
 		{
-			if (ai->temp1 == 0) continue;
+			if ((ai->userbits&HIGHLIGHTAVER) == 0) continue;
 			poly->xv[0] = ai->end[0].xpos;
 			poly->yv[0] = ai->end[0].ypos;
 			poly->xv[1] = ai->end[1].xpos;
@@ -1340,7 +1488,7 @@ void us_highlightverbose(GEOM *look, PORTPROTO *pp, INTSML color, NODEPROTO *par
 		for(ni = par->firstnodeinst; ni != NONODEINST; ni = ni->nextnodeinst)
 		{
 			if (ni == hni) continue;
-			if (ni->temp1 == 0) continue;
+			if ((ni->userbits&HIGHLIGHTNVER) == 0) continue;
 			poly->xv[0] = (ni->lowx + ni->highx) / 2;
 			poly->yv[0] = (ni->lowy + ni->highy) / 2;
 			poly->xv[1] = poly->xv[0] + el_curlib->lambda[el_curtech->techindex]/4;
@@ -1360,7 +1508,7 @@ void us_highlightverbose(GEOM *look, PORTPROTO *pp, INTSML color, NODEPROTO *par
 			for(pi = ni->firstportarcinst; pi != NOPORTARCINST; pi = pi->nextportarcinst)
 			{
 				ai = pi->conarcinst;
-				if (ai->temp1 == 0) continue;
+				if ((ai->userbits&HIGHLIGHTAVER) == 0) continue;
 				if (ai->end[0].portarcinst == pi) thisend = 0; else thisend = 1;
 				if (ai->end[thisend].xpos != poly->xv[0] ||
 					ai->end[thisend].ypos != poly->yv[0])
@@ -1415,8 +1563,8 @@ void us_highlightverbose(GEOM *look, PORTPROTO *pp, INTSML color, NODEPROTO *par
 						poly->style = TEXTBOTLEFT;
 	}
 	poly->count = 1;
-	poly->font = TXTSMALL;
-	poly->tech = ai->proto->tech;
+	poly->font = TXT1L;
+	poly->tech = ai->parent->tech;
 	for(w = el_topwindowpart; w != NOWINDOWPART; w = w->nextwindowpart)
 	{
 		if ((w->state&WINDOWTYPE) != DISPWINDOW &&
@@ -1652,6 +1800,7 @@ INTSML us_makehighlight(char *str, HIGHLIGHT *high)
 		/* parse the three values: geom/port/point */
 		high->status = HIGHFROM;
 		high->fromvar = NOVARIABLE;
+		high->fromvarnoeval = NOVARIABLE;
 		keyword = getkeyword(&pt, ";");
 		if (keyword == NOSTRING) return(1);
 		high->fromgeom = (GEOM *)myatoi(keyword);
@@ -1697,24 +1846,27 @@ INTSML us_makehighlight(char *str, HIGHLIGHT *high)
 		if (tonextchar(&pt) != ';') return(1);
 		keyword = getkeyword(&pt, ";");
 		if (keyword == NOSTRING) return(1);
-		if (*keyword == '-') high->fromvar = NOVARIABLE; else
+		if (*keyword == '-') high->fromvarnoeval = high->fromvar = NOVARIABLE; else
 		{
 			if (high->fromport != NOPORTPROTO)
 			{
-				high->fromvar = getval((INTBIG)high->fromport, VPORTPROTO, -1, keyword);
+				high->fromvarnoeval = getvalnoeval((INTBIG)high->fromport, VPORTPROTO, -1, keyword);
 			} else if (high->fromgeom != NOGEOM)
 			{
 				if (high->fromgeom->entrytype == OBJNODEINST)
 				{
-					high->fromvar = getval((INTBIG)high->fromgeom->entryaddr.ni, VNODEINST, -1, keyword);
+					high->fromvarnoeval = getvalnoeval((INTBIG)high->fromgeom->entryaddr.ni,
+						VNODEINST, -1, keyword);
 				} else
 				{
-					high->fromvar = getval((INTBIG)high->fromgeom->entryaddr.ai, VARCINST, -1, keyword);
+					high->fromvarnoeval = getvalnoeval((INTBIG)high->fromgeom->entryaddr.ai,
+						VARCINST, -1, keyword);
 				}
 			} else
 			{
-				high->fromvar = getval((INTBIG)high->facet, VNODEPROTO, -1, keyword);
+				high->fromvarnoeval = getvalnoeval((INTBIG)high->facet, VNODEPROTO, -1, keyword);
 			}
+			high->fromvar = evalvar(high->fromvarnoeval);
 		}
 	} else return(1);
 
@@ -1806,6 +1958,7 @@ void us_gethightextcenter(HIGHLIGHT *high, INTBIG *xc, INTBIG *yc, INTSML *style
 	static POLYGON *poly = NOPOLYGON;
 	REGISTER NODEINST *ni;
 	REGISTER ARCINST *ai;
+	REGISTER NODEPROTO *facet;
 	XARRAY trans;
 	INTBIG newxc, newyc;
 	REGISTER INTSML saverot, savetrn;
@@ -1863,7 +2016,8 @@ void us_gethightextcenter(HIGHLIGHT *high, INTBIG *xc, INTBIG *yc, INTSML *style
 		portposition(high->fromport->subnodeinst, high->fromport->subportproto,
 			&poly->xv[0], &poly->yv[0]);
 		makeangle(ni->rotation, ni->transpose, trans);
-		lambda = figurelambda(ni->geom);
+		facet = ni->parent;
+		lambda = facet->cell->lib->lambda[facet->tech->techindex];
 		newxc = (descript&VTXOFF)>>VTXOFFSH;
 		if ((descript&VTXOFFNEG) != 0) newxc = -newxc;
 		newxc = newxc * lambda / 4;
@@ -1932,23 +2086,26 @@ INTBIG us_gethighdescript(HIGHLIGHT *high)
 char *us_gethighstring(HIGHLIGHT *high)
 {
 	REGISTER char *str;
-	REGISTER INTBIG len, slen;
+	REGISTER VARIABLE *var;
 
-	if (high->fromvar != NOVARIABLE)
+	var = high->fromvar;
+	if (var != NOVARIABLE)
 	{
-		str = describevariable(high->fromvar, -1, -1);
-		if ((high->fromvar->type&VISARRAY) == 0 &&
-			(high->fromvar->textdescript&VTDISPLAYPART) != VTDISPLAYNAMEVALUE)
+		switch (var->textdescript&VTDISPLAYPART)
 		{
-			len = (high->fromvar->type&VLENGTH) >> VLENGTHSH;
-			slen = strlen(str);
-			if (len > slen) len = slen;
-			str += len;
+			case VTDISPLAYNAME:
+				str = describedisplayedvariable(var, -1, -1);
+				break;
+			case VTDISPLAYVALUE:
+			case VTDISPLAYNAMEVALUE:
+				if (high->fromvarnoeval != NOVARIABLE) var = evalvar(high->fromvarnoeval);
+				str = describedisplayedvariable(var, -1, -1);
+				break;
 		}
 		return(str);
 	}
 	if (high->fromport != NOPORTPROTO)
-		return(high->fromport->protoname);
+		return(us_displayedportname(high->fromport, (us_useroptions&EXPORTLABELS) >> EXPORTLABELSSH));
 	if (high->fromgeom->entrytype == OBJNODEINST)
 		return(describenodeproto(high->fromgeom->entryaddr.ni->proto));
 	return("");
@@ -2000,6 +2157,31 @@ void us_gethighaddrtype(HIGHLIGHT *high, INTBIG *addr, INTBIG *type)
 	}
 }
 
+/*
+ * Routine to return nonzero if highlight "high" is text on a node that must move together.
+ * This only happens for "nonlayout text" (a variable on an invisible node) or
+ * for exports on an invisible pin.
+ */
+INTSML us_nodemoveswithtext(HIGHLIGHT *high)
+{
+	REGISTER NODEINST *ni;
+
+	if (high->status != HIGHTEXT) return(0);
+	if (high->fromgeom == NOGEOM) return(0);
+	if (high->fromgeom->entrytype == OBJARCINST) return(0);
+	ni = high->fromgeom->entryaddr.ni;
+	if (ni == NONODEINST) return(0);
+	if (ni->proto != gen_invispinprim) return(0);
+	if (high->fromvar == NOVARIABLE)
+	{
+		if (high->fromport != NOPORTPROTO) return(1);
+	} else
+	{
+		if (high->fromport == NOPORTPROTO) return(1);
+	}
+	return(0);
+}
+
 /*
  * Routine to figure out the technology to use for highlight "high".
  */
@@ -2008,18 +2190,18 @@ TECHNOLOGY *us_hightech(HIGHLIGHT *high)
 	if (high->fromvar != NOVARIABLE)
 	{
 		if (high->fromport != NOPORTPROTO)
-			return(high->fromport->subnodeinst->proto->tech);
+			return(high->fromport->subnodeinst->parent->tech);
 		if (high->fromgeom == NOGEOM) return(high->facet->tech);
 		if (high->fromgeom->entrytype == OBJNODEINST)
-			return(high->fromgeom->entryaddr.ni->proto->tech);
-		return(high->fromgeom->entryaddr.ai->proto->tech);
+			return(high->fromgeom->entryaddr.ni->parent->tech);
+		return(high->fromgeom->entryaddr.ai->parent->tech);
 	}
 	if (high->fromport != NOPORTPROTO)
-		return(high->fromgeom->entryaddr.ni->proto->tech);
+		return(high->fromgeom->entryaddr.ni->parent->tech);
 	if (high->fromgeom->entrytype == OBJNODEINST)
-		return(high->fromgeom->entryaddr.ni->proto->tech);
+		return(high->fromgeom->entryaddr.ni->parent->tech);
 	if (high->fromgeom->entrytype == OBJARCINST)
-		return(high->fromgeom->entryaddr.ai->proto->tech);
+		return(high->fromgeom->entryaddr.ai->parent->tech);
 	return(NOTECHNOLOGY);
 }
 
@@ -2036,12 +2218,12 @@ TECHNOLOGY *us_getobjectinfo(INTBIG addr, INTBIG type, INTBIG *lx, INTBIG *hx, I
 			ni = (NODEINST *)addr;
 			*lx = ni->geom->lowx;   *hx = ni->geom->highx;
 			*ly = ni->geom->lowy;   *hy = ni->geom->highy;
-			return(ni->proto->tech);
+			return(ni->parent->tech);
 		case VARCINST:
 			ai = (ARCINST *)addr;
 			*lx = ai->geom->lowx;   *hx = ai->geom->highx;
 			*ly = ai->geom->lowy;   *hy = ai->geom->highy;
-			return(ai->proto->tech);
+			return(ai->parent->tech);
 		case VPORTPROTO:
 			pp = (PORTPROTO *)addr;
 			ni = pp->subnodeinst;
@@ -2067,10 +2249,10 @@ void us_drawfacetvariable(VARIABLE *var, NODEPROTO *np)
 	/* get polygon */
 	if (poly == NOPOLYGON) poly = allocstaticpolygon(4, us_aid->cluster);
 
-	displaytotal = tech_displayablefacetvars(np);
+	displaytotal = tech_displayablefacetvars(np, NOWINDOWPART);
 	for(i = 0; i < displaytotal; i++)
 	{
-		thisvar = tech_filldisplayablefacetvar(np, poly);
+		thisvar = tech_filldisplayablefacetvar(np, poly, NOWINDOWPART, 0);
 		for(w = el_topwindowpart; w != NOWINDOWPART; w = w->nextwindowpart)
 		{
 			if (w->curnodeproto != np) continue;
@@ -2083,8 +2265,8 @@ void us_undrawfacetvariable(VARIABLE *var, NODEPROTO *np)
 {
 	INTBIG x, y;
 	INTSML tsx, tsy;
-	REGISTER INTSML tf, font;
-	REGISTER INTBIG objwid, objhei, sea, slx, shx, sly, shy, nudge;
+	REGISTER INTSML font;
+	REGISTER INTBIG tf, objwid, objhei, sea, slx, shx, sly, shy, nudge;
 	INTBIG lx, hx, ly, hy;
 	REGISTER TECHNOLOGY *tech;
 	REGISTER NODEINST *ni;
@@ -2116,7 +2298,7 @@ void us_undrawfacetvariable(VARIABLE *var, NODEPROTO *np)
 		/* erase that area */
 		lx = slx;   hx = shx;
 		ly = sly;   hy = shy;
-		(void)us_makescreen(&lx, &ly, &hx, &hy, w);
+		if (us_makescreen(&lx, &ly, &hx, &hy, w) != 0) continue;
 		screendrawbox(w, (INTSML)lx, (INTSML)hx, (INTSML)ly, (INTSML)hy, &us_ebox);
 
 		/* now redraw everything that touches this box */
