@@ -49,8 +49,8 @@ GRAPHICS us_shadowdesc = {LAYERA, MENGLY, PATTERNED, PATTERNED,
 GRAPHICS us_menutext = {LAYERA, MENTXT, SOLIDC, SOLIDC,
 	{0xFFFF,0xFFFF,0xFFFF,0xFFFF,0xFFFF,0xFFFF,0xFFFF,0xFFFF}, NOVARIABLE, 0};
 
-#define	MENUARCLENGTH   5	/* length of arcs in menu entries */
-#define	MENUSHADOW      2	/* size of shadow around popup menus */
+#define MENUARCLENGTH   5	/* length of arcs in menu entries */
+#define MENUSHADOW      2	/* size of shadow around popup menus */
 
 /* prototypes for local routines */
 INTSML us_groupfunction(NODEPROTO*);
@@ -164,13 +164,12 @@ void us_drawmenu(INTSML scaletofit, WINDOWFRAME *mw)
 void us_drawmenuentry(INTBIG x, INTBIG y, char *str)
 {
 	REGISTER INTBIG i, scaleu, scaled, x1, y1, lowx, lowy, highx, highy, xl,
-		yl, xh, yh, ofun, fun, lambda;
+		yl, xh, yh, ofun, fun, lambda, bits;
 	REGISTER NODEINST *nodeinst;
 	REGISTER ARCINST *arcinst;
 	REGISTER NODEPROTO *tnp;
 	REGISTER ARCPROTO *tap;
 	REGISTER TECHNOLOGY *tech;
-	REGISTER char *pt;
 	VARIABLE colorvar[2];
 	WINDOWPART ww;
 	XARRAY trans;
@@ -279,11 +278,11 @@ void us_drawmenuentry(INTBIG x, INTBIG y, char *str)
 			highy = nodeinst->highy = nodeinst->lowy + pys;
 
 			/*
-			 * Special hack: the Schematic Transistor does not fill its bounding box
+			 * Special hack: the Schematic Transistors do not fill their bounding box
 			 * but stops 1 lambda short of the top.  To center it in the menu square,
 			 * that amount is rotated and the node is shifted thusly.
 			 */
-			if (nodeinst->proto == sch_transistorprim)
+			if (nodeinst->proto == sch_transistorprim || nodeinst->proto == sch_transistor4prim)
 			{
 				makerot(nodeinst, trans);
 				xform(0, el_curlib->lambda[sch_tech->techindex]/2, &pxs, &pys, trans);
@@ -298,18 +297,9 @@ void us_drawmenuentry(INTBIG x, INTBIG y, char *str)
 			nodeinst->numvar = 1;
 			if (commandbinding.menumessage != 0)
 			{
-				/* add another variable to the node */
-				for(pt = commandbinding.menumessage; *pt != 0 && *pt != '.'; pt++) ;
-				if (*pt == '.')
-				{
-					*pt = 0;
-					colorvar[1].key = makekey(commandbinding.menumessage);
-					*pt++ = '.';
-					colorvar[1].type = VSTRING;
-					colorvar[1].textdescript = VTPOSCENT;
-					colorvar[1].addr = (INTBIG)pt;
-					nodeinst->numvar = 2;
-				}
+				/* add node specialization bits */
+				bits = myatoi(commandbinding.menumessage);
+				nodeinst->userbits |= bits;
 			}
 		}
 		if (commandbinding.arcglyph != NOARCPROTO)
@@ -343,7 +333,8 @@ void us_drawmenuentry(INTBIG x, INTBIG y, char *str)
 
 		/* compute the scale for graphics */
 		scaleu = scaled = maxi(highx-lowx, highy-lowy);
-#define	INSET 2
+		if (scaleu == 0) scaleu = scaled = 1;
+#define INSET 2
 		if (commandbinding.nodeglyph != NONODEPROTO)
 		{
 			/* make all nodes of the same class be the same scale */
@@ -446,16 +437,15 @@ INTSML us_groupfunction(NODEPROTO *np)
 	switch (fun)
 	{
 		/* combine all transistors */
-		case NPTRANMOS:
-		case NPTRADMOS:
-		case NPTRAPMOS:
-		case NPTRANPN:
-		case NPTRAPNP:
-		case NPTRANJFET:
-		case NPTRAPJFET:
-		case NPTRADMES:
-		case NPTRAEMES:
-		case NPTRANE2L:
+		case NPTRANMOS:   case NPTRA4NMOS:
+		case NPTRADMOS:   case NPTRA4DMOS:
+		case NPTRAPMOS:   case NPTRA4PMOS:
+		case NPTRANPN:    case NPTRA4NPN:
+		case NPTRAPNP:    case NPTRA4PNP:
+		case NPTRANJFET:  case NPTRA4NJFET:
+		case NPTRAPJFET:  case NPTRA4PJFET:
+		case NPTRADMES:   case NPTRA4DMES:
+		case NPTRAEMES:   case NPTRA4EMES:
 		case NPTRANSREF:   fun = NPTRANS;       break;
 
 		/* combine all analog parts */
@@ -506,66 +496,6 @@ INTSML us_groupfunction(NODEPROTO *np)
 
 /******************** MENU COMPUTATION ********************/
 
-/*
- * routine to search the menu for any "getproto" entries that refer to
- * facets in the current library and remove these references.  This routine
- * is called when switching libraries
- */
-void us_removegetfacets(void)
-{
-	REGISTER INTSML i, j;
-	char *newpar[5], si[10], sj[10], *colorname, *colorsymbol;
-	REGISTER VARIABLE *var;
-	COMMANDBINDING commandbinding;
-
-	var = getvalkey((INTBIG)us_aid, VAID, VSTRING|VISARRAY, us_binding_menu);
-	if (var == NOVARIABLE) return;
-	for(i=0; i<us_menuy*us_menux; i++)
-	{
-		us_parsebinding(((char **)var->addr)[i], &commandbinding);
-
-		/* must have active menu entries */
-		if (*commandbinding.command != 0)
-		{
-			/* must have menu entries with glyph */
-			if (commandbinding.nodeglyph != NONODEPROTO)
-			{
-				/* must have glyphs that are facet node prototypes */
-				if (commandbinding.nodeglyph->primindex == 0)
-				{
-					/* must have facets that are in the current library */
-					if (commandbinding.nodeglyph->cell->lib == el_curlib)
-					{
-						/* load up the strings to void the menu entry */
-						if (us_menupos <= 1)
-						{
-							(void)sprintf(si, "%d", i%us_menux);
-							(void)sprintf(sj, "%d", i/us_menux);
-						} else
-						{
-							(void)sprintf(si, "%d", i/us_menuy);
-							(void)sprintf(sj, "%d", i%us_menuy);
-						}
-						newpar[0] = "set";      newpar[1] = "menu";
-						j = 2;
-						if (commandbinding.backgroundcolor != 0)
-						{
-							newpar[j++] = "background";
-							if (ecolorname(commandbinding.backgroundcolor, &colorname,
-								&colorsymbol) != 0) colorname = "**UNKNOWN**";
-							newpar[j++] = colorname;
-						}
-						newpar[j++] = sj;         newpar[j++] = si;
-						newpar[j++] = "getproto";
-						us_bind(j, newpar);
-					}
-				}
-			}
-		}
-		us_freebindingparse(&commandbinding);
-	}
-}
-
 /*
  * routine to change the "getproto" commands in the menu to
  * properly reflect the primitive nodes and arcs in the current
@@ -731,7 +661,7 @@ INTSML us_setmenusize(INTSML x, INTSML y, INTSML position, INTSML allgetproto)
 
 /******************** POPUP MENUS ********************/
 
-#define	ATTVALGAP 20		/* distance between attribute and value columns */
+#define ATTVALGAP 20		/* distance between attribute and value columns */
 
 /* structure for popup menus */
 typedef struct
@@ -770,9 +700,9 @@ POPUPINST us_pmcur;
 POPUPMENUITEM *us_popupmenu(POPUPMENU **cmenu, INTSML *waitfordown, INTSML header,
 	INTSML left, INTSML top, INTSML quitline)
 {
-	REGISTER INTSML i, textscale, yp, waitdown, startfont;
+	REGISTER INTSML i, yp, waitdown, startfont;
 	REGISTER char *savepos, *pt;
-	REGISTER INTBIG save_pix;
+	REGISTER INTBIG textscale, save_pix;
 	INTSML x, y, chwid, swid, shei;
 	REGISTER POPUPMENU *menu;
 	REGISTER POPUPMENUITEM *mi;
@@ -823,14 +753,14 @@ POPUPMENUITEM *us_popupmenu(POPUPMENU **cmenu, INTSML *waitfordown, INTSML heade
 
 	/* loop through fonts to find one that fits the menu */
 #ifdef	sun
-	startfont = TXT20P;
+	startfont = 20;
 #else
-	startfont = TXT14P;
+	startfont = 14;
 #endif
-	for(textscale = startfont; textscale >= TXT4P; textscale--)
+	for(textscale = startfont; textscale >= 4; textscale--)
 	{
 		us_pmcur.attlen = us_pmcur.vallen = 0;
-		if (textscale == TXT14P) screensettextsize(&us_pmcur.popupwindow, TXTMENU); else
+		if (textscale == 14) screensettextsize(&us_pmcur.popupwindow, TXTMENU); else
 			screensettextsize(&us_pmcur.popupwindow, textscale);
 		screengettextsize(&us_pmcur.popupwindow, "0", &chwid, &y);
 		us_pmcur.txthei = y+2;
@@ -878,7 +808,7 @@ POPUPMENUITEM *us_popupmenu(POPUPMENU **cmenu, INTSML *waitfordown, INTSML heade
 	}
 
 	/* no text size will fit */
-	if (textscale < TXT4P) return(0);
+	if (textscale < 4) return(0);
 
 	/* determine the location of the menu */
 	readtablet(&x, &y);
