@@ -35,7 +35,7 @@
 #include "edialogs.h"
 #include "tecschem.h"
 
-#define	MAXLINE 400		/* maximum length of command file input lines */
+#define MAXLINE 400		/* maximum length of command file input lines */
 
 /* working memory for "us_parsecommand()" */
 char *us_parseownbuild[MAXPARS];
@@ -48,37 +48,34 @@ static INTBIG  us_evaluatevarlimit = 0;
 DIALOGHOOK *us_firstdialoghook = NODIALOGHOOK;
 
 /* global variables for parsing */
+KEYWORD *us_pathiskey;					/* the current keyword list */
+
+/* local variables for parsing */
 INTSML   us_pacurchar;					/* current character on current line */
-char    *us_pattyline;					/* current line image */
-char    *us_paambiguous;				/* used by "us_pagetword" for "?" message */
+char    *us_pattyline = 0;				/* current line image */
 INTSML   us_patruelength;				/* size of above two buffers */
 INTSML   us_paparamcount;				/* number of parameters read in */
 char    *us_palinestart;				/* first character position on line */
 COMCOMP *us_paparamtype[MAXKEYWORD];	/* expected type of each parameter */
 INTSML   us_pakeycount;					/* number of keywords allowed in parse */
 char   **us_paparamstart;				/* character position of each parameter */
-KEYWORD *us_pathiskey;					/* the current keyword list */
-
-/* local variables for parsing */
+INTBIG	 us_ttypamyItemHit;				/* item hit during parsing */
+char    *us_paambiguous = 0;			/* used by "us_pagetword" for "?" message */
 INTSML   us_paparamadded[MAXKEYWORD];	/* number of parameters added here */
 INTSML   us_pathisind;					/* pointer into the current keyword list */
 
-/* List Dialog */
-DIALOGITEM us_listdialogitems[] =
-{
- /*  1 */ {0, {160,216,184,280}, BUTTON, N_("OK")},
- /*  2 */ {0, {96,216,120,280}, BUTTON, N_("Cancel")},
- /*  3 */ {0, {56,8,200,206}, SCROLL, ""},
- /*  4 */ {0, {8,8,54,275}, MESSAGE, ""}
-};
-DIALOG us_listdialog = {{50,75,259,369}, "", 0, 4, us_listdialogitems};
-
 /* prototypes for local routines */
-INTSML us_addusercom(USERCOM*, char*);
+INTSML   us_addusercom(USERCOM*, char*);
 USERCOM *us_fleshcommand(USERCOM*);
-void us_dopopupmenu(POPUPMENU*);
-void us_domacro(char*, INTSML, char*[]);
-INTSML us_expandvarlist(INTBIG **varlist, INTBIG *varlimit);
+void     us_dopopupmenu(POPUPMENU*);
+void     us_domacro(char*, INTSML, char*[]);
+INTSML   us_expandvarlist(INTBIG **varlist, INTBIG *varlimit);
+INTSML   us_pagetword(INTSML(*)(COMCOMP**, INTSML*), void(*)(char*), void(*)(void), void(*)(void), INTSML);
+INTSML   us_ttyinttychar(COMCOMP**, INTSML*);
+void     us_ttyoutttychar(char*);
+void     us_ttyerasettychar(void);
+void     us_ttykillttychar(void);
+INTSML   us_specialparse(char*, COMCOMP*, INTSML, char *paramstart[]);
 
 /*
  * Routine to free all memory associated with this module.
@@ -119,6 +116,8 @@ void us_freeparsememory(void)
 		if (dh->terminputkeyword != 0) efree((char *)dh->terminputkeyword);
 		efree((char *)dh);
 	}
+	if (us_paambiguous != 0) efree(us_paambiguous);
+	if (us_pattyline != 0) efree(us_pattyline);
 }
 
 /*
@@ -211,19 +210,20 @@ INTSML us_addusercom(USERCOM *us, char *mac)
 		hasspace = 0;
 		for(pt = us->word[i]; *pt != 0; pt++)
 		{
+
 			if (*pt == ' ' || *pt == '\t') hasspace++;
 			if (*pt != '%') continue;
 			if (!isdigit(pt[1])) continue;
 
 			/* formal parameter found: determine command completion for it */
 			parnumber = pt[1] - '0';
-			count = us_parsecommand(manystrings[2], build);
-			total = (INTSML)maxi(count, parnumber);
 			if (parsed == 0)
 			{
 				tot = us_fillcomcomp(us, comarray);
 				parsed++;
 			}
+			count = us_parsecommand(manystrings[2], build);
+			total = (INTSML)maxi(count, parnumber);
 			(void)initinfstr();
 			for(j=0; j<total; j++)
 			{
@@ -263,6 +263,68 @@ INTSML us_addusercom(USERCOM *us, char *mac)
 	return(0);
 }
 
+/*
+ * routine to examine the command in "uc" and fill the command completion array
+ * "comarray" with appropriate entries for parsing the string.  Returns the
+ * number of valid entries in the array.
+ */
+INTSML us_fillcomcomp(USERCOM *uc, COMCOMP *comarray[])
+{
+	REGISTER INTSML j, k, com, pos, curtotal, count;
+	REGISTER char *pt;
+	char *build[MAXPARS];
+	COMCOMP *nextparams[MAXPARS+5];
+	REGISTER VARIABLE *var;
+
+	comarray[0] = NOCOMCOMP;
+	com = uc->active;
+
+	/* if the command is not valid, exit now */
+	if (com < 0) return(0);
+
+	/* handle normal commands */
+	if (com < us_longcount)
+	{
+		/* initialize the command completion array with the default parameters */
+		for(curtotal=0; curtotal<us_lcommand[com].params; curtotal++)
+			comarray[curtotal] = us_lcommand[com].par[curtotal];
+	} else
+	{
+		/* ignore popup menus */
+		if (uc->menu != NOPOPUPMENU) return(0);
+
+		/* handle macros */
+		var = us_getmacro(uc->comname);
+		if (var == NOVARIABLE) return(0);
+		pt = ((char **)var->addr)[2];
+		count = us_parsecommand(pt, build);
+		for(curtotal=0; curtotal<count; curtotal++)
+			comarray[curtotal] = (COMCOMP *)myatoi(build[curtotal]);
+	}
+	comarray[curtotal] = NOCOMCOMP;
+
+	/* now run through the parameters, inserting whatever is appropriate */
+	for(pos=0; comarray[pos] != NOCOMCOMP; pos++)
+	{
+		if (pos >= uc->count) break;
+
+		/* load the necessary global and get the next parameters */
+		us_pathiskey = comarray[pos]->ifmatch;
+		k = (*comarray[pos]->params)(uc->word[pos], nextparams, ' ');
+
+		/* clip to the number of allowable parameters */
+		if (k+curtotal >= MAXPARS-1) k = MAXPARS-2-curtotal;
+
+		/* spread open the list */
+		for(j = MAXPARS-k-1; j >= curtotal; j--) comarray[j+k] = comarray[j];
+
+		/* fill in the parameter types */
+		for(j = 0; j < k; j++) comarray[curtotal+j] = nextparams[j];
+		curtotal += k;
+	}
+	return(curtotal);
+}
+
 /*
  * routine to execute user command "item".  The command is echoed if "print"
  * is nonzero.  If "macro" is nonzero, the command is saved in the
@@ -1211,32 +1273,6 @@ void us_domacro(char *mac, INTSML count, char *pp[])
 	}
 }
 
-/*
- * routine to cause further single-key commands to be ignored until a
- * carriage-return is typed.
- */
-void us_unknowncommand(void)
-{
-	REGISTER INTSML i;
-
-	if ((us_aid->aidstate&NOKEYLOCK) != 0) return;
-	i = ttyquiet(0);
-	ttyputmsg(_("Single-key commands will be ignored until the next RETURN"));
-	(void)ttyquiet(i);
-	us_state |= SKIPKEYS;
-}
-
-void us_abortedmsg(void)
-{
-	if ((us_aid->aidstate & JUSTTHEFACTS) == 0) us_abortcommand(_("Aborted")); else
-	{
-		us_state |= COMMANDFAILED;
-		if (getvalkey((INTBIG)us_aid, VAID, VSTRING, us_macrobuilding) != NOVARIABLE)
-			ttyputmsg(_("Command ignored from macro definition"));
-		us_unknowncommand();
-	}
-}
-
 /*
  * routine to execute the commands in the file pointed to by stream "infile".
  * If "verbose" is zero, execute the commands silently.  "headerstring"
@@ -1245,7 +1281,7 @@ void us_abortedmsg(void)
  */
 void us_docommands(FILE *infile, INTSML verbose, char *headerstring)
 {
-	REGISTER INTSML headercount, lastquiet;
+	REGISTER INTBIG headercount, lastquiet;
 	REGISTER char *pp;
 	char line[MAXLINE], *savemacro;
 	REGISTER USERCOM *uc, *savelastc;
@@ -1362,18 +1398,6 @@ INTSML us_parsecommand(char *pp, char *build[])
 		c = *pp;   *pp = 0;
 		if (tstart != 0 && tend != 0)
 		{
-#if 0
-			(void)initinfstr();
-			(void)addtoinfstr('"');
-			for(ch = ppsave; ch != tstart; ch++) (void)addtoinfstr(*ch);
-			tstart += 4;
-			*tend = 0;
-			(void)addstringtoinfstr(_(tstart));
-			*tend = '"';
-			for(ch = tend+2; *ch != 0; ch++) (void)addtoinfstr(*ch);
-			(void)addtoinfstr('"');
-			ppsave = returninfstr();
-#else
 			tp = 0;
 			tempbuf[tp++] = '"';
 			for(ch = ppsave; ch != tstart; ch++) tempbuf[tp++] = *ch;
@@ -1385,7 +1409,6 @@ INTSML us_parsecommand(char *pp, char *build[])
 			tempbuf[tp++] = '"';
 			tempbuf[tp] = 0;
 			ppsave = tempbuf;
-#endif
 		}
 		if (count > us_parsemaxownbuild)
 		{
@@ -1533,11 +1556,11 @@ char *us_putintoinfstr(char *str)
 	return(returninfstr());
 }
 
-/********************** COMMON STUFF FROM THE TERMINAL MODULE **********************/
+/***************************** GENERAL DIALOG CONTROL *****************************/
 
 /*
  * Routine to declare a dialog routine in "routine" that will be called when
- * "us_specialparse()" is given the command-completion module "getlinecomcomp".
+ * "ttygetparam()" is given the command-completion module "getlinecomcomp".
  * Also, if "terminputkeyword" is not zero, this is the keyword for the
  * "terminal input" command that invokes this dialog routine.
  */
@@ -1571,7 +1594,24 @@ COMCOMP *us_getcomcompfromkeyword(char *keyword)
 	return(NOCOMCOMP);
 }
 
-INTSML us_specialparse(char *prompt, COMCOMP *parameter, INTSML keycount, char *paramstart[])
+/* List Dialog */
+DIALOGITEM us_listdialogitems[] =
+{
+ /*  1 */ {0, {160,216,184,280}, BUTTON, N_("OK")},
+ /*  2 */ {0, {96,216,120,280}, BUTTON, N_("Cancel")},
+ /*  3 */ {0, {56,8,200,206}, SCROLL, ""},
+ /*  4 */ {0, {8,8,54,275}, MESSAGE, ""}
+};
+DIALOG us_listdialog = {{50,75,259,369}, "", 0, 4, us_listdialogitems};
+
+/* special items for the "ttygetline" dialog: */
+#define DLST_LIST     3		/* line (scroll) */
+#define DLST_PROMPT   4		/* prompt (stat text) */
+
+/*
+ * routine to get a string with "parameter" information (does command completion)
+ */
+INTSML ttygetparam(char *prompt, COMCOMP *parameter, INTSML keycount, char *paramstart[])
 {
 	INTBIG filetype, itemHit;
 	INTSML multiplefiles;
@@ -1587,9 +1627,13 @@ INTSML us_specialparse(char *prompt, COMCOMP *parameter, INTSML keycount, char *
 		us_technologyclsrp, us_technologyclscp, us_technologyclsecp, us_technologyclip,
 		us_technologycnsp, us_technologyclgp, us_technologyclsp, us_showdp, us_createxp,
 		us_nodetcaip, us_bindgkeyp, us_sizep, us_sizewp, us_defnodep, us_showup,
-		us_librarykp, us_findobjap, us_window3dp, us_gridp, us_technologytp,
+		us_librarykp, us_findobjap, us_window3dp, us_gridp, us_technologytp, us_textfp,
 		us_renamecp, us_findnnamep, us_findexportp, us_noyesalwaysp, us_varop,
-		us_copyfacetdp, us_findnodep, us_technologycnnp, us_showep, us_purelayerp;
+		us_copyfacetdp, us_findnodep, us_technologycnnp, us_showep, us_purelayerp,
+		us_textsp, us_viewdp;
+
+	/* is this necessary?  if not, us_pathiskey does not need to be global */
+	us_pathiskey = parameter->ifmatch;
 
 	for(dh = us_firstdialoghook; dh != NODIALOGHOOK; dh = dh->nextdialoghook)
 		if (parameter == dh->getlinecomp)
@@ -1597,7 +1641,6 @@ INTSML us_specialparse(char *prompt, COMCOMP *parameter, INTSML keycount, char *
 		(*dh->routine)();
 		return(0);
 	}
-
 	if (parameter == &us_window3dp)           return(us_3ddepthdlog());
 	if (parameter == &us_showep)              return(us_aboutdlog());
 	if (parameter == &us_gridalip)            return(us_alignmentdlog());
@@ -1618,6 +1661,7 @@ INTSML us_specialparse(char *prompt, COMCOMP *parameter, INTSML keycount, char *
 	if (parameter == &us_defnodep)            return(us_facetdlog());
 	if (parameter == &us_showdp)              return(us_facetlist(prompt, paramstart, 1));
 	if (parameter == &us_showup)              return(us_facetlist(prompt, paramstart, 0));
+	if (parameter == &us_textfp)              return(us_findtextdlog());
 	if (parameter == &us_viewfp)              return(us_frameoptionsdlog());
 	if (parameter == &us_technologyclsp)      return(us_gaindlog(prompt, paramstart, 0));
 	if (parameter == &us_gridp)               return(us_griddlog());
@@ -1628,6 +1672,7 @@ INTSML us_specialparse(char *prompt, COMCOMP *parameter, INTSML keycount, char *
 	if (parameter == &us_lambdachp)           return(us_lambdadlog());
 	if (parameter == &us_librarydp)           return(us_librarypathdlog());
 	if (parameter == &us_technologycnnp)      return(us_libtotechnologydlog());
+	if (parameter == &us_textsp)              return(us_modtextsizedlog());
 	if (parameter == &us_menup)               return(us_menudlog(paramstart));
 	if (parameter == &us_viewn1p)             return(us_newviewdlog(paramstart));
 	if (parameter == &us_sizep)               return(us_nodesizedlog(paramstart));
@@ -1651,7 +1696,7 @@ INTSML us_specialparse(char *prompt, COMCOMP *parameter, INTSML keycount, char *
 	if (parameter == &us_findnodep)           return(us_selectportnodenetdlog(2));
 	if (parameter == &us_showp)               return(us_showdlog(0));
 	if (parameter == &us_showop)              return(us_showdlog(1));
-	if (parameter == &us_spreaddp)            return(us_spreaddlog(paramstart));
+	if (parameter == &us_spreaddp)            return(us_spreaddlog());
 	if (parameter == &us_technologyup)        return(us_technologydlog(prompt, paramstart));
 	if (parameter == &us_technologytp)        return(us_techoptdlog());
 	if (parameter == &us_technologyctdp)      return(us_techvarsdlog());
@@ -1659,6 +1704,7 @@ INTSML us_specialparse(char *prompt, COMCOMP *parameter, INTSML keycount, char *
 	if (parameter == &us_purelayerp)          return(us_purelayernodedlog(paramstart));
 	if (parameter == &us_varvep)              return(us_variablesdlog());
 	if (parameter == &us_technologyclip)      return(us_vccsdlog(paramstart, 0));
+	if (parameter == &us_viewdp)              return(us_viewdlog(1));
 	if (parameter == &us_visiblelayersp)      return(us_visiblelayersdlog(prompt));
 	if (parameter == &us_technologyclgp)      return(us_widlendlog(paramstart, 0));
 	if (parameter == &us_windowrnamep)        return(us_windowviewdlog());
@@ -1727,81 +1773,93 @@ INTSML us_specialparse(char *prompt, COMCOMP *parameter, INTSML keycount, char *
 
 	/* the general case: display the dialog box */
 	if (DiaInitDialog(&us_listdialog) != 0) return(0);
-	DiaInitTextDialog(3, parameter->toplist, parameter->nextcomcomp, DiaNullDlogDone,
-		0, SCSELMOUSE|SCSELKEY|SCDOUBLEQUIT);
-	DiaSetText(4, prompt);
+	DiaInitTextDialog(DLST_LIST, parameter->toplist, parameter->nextcomcomp, DiaNullDlogDone,
+		0, SCSELMOUSE|SCSELKEY|SCDOUBLEQUIT|SCHORIZBAR);
+	DiaSetText(DLST_PROMPT, prompt);
 
 	for(;;)
 	{
 		itemHit = DiaNextHit();
 		if (itemHit == OK || itemHit == CANCEL) break;
 	}
-	paramstart[0] = us_putintoinfstr(DiaGetScrollLine(3, DiaGetCurLine(3)));
+	paramstart[0] = us_putintoinfstr(DiaGetScrollLine(DLST_LIST, DiaGetCurLine(DLST_LIST)));
 	DiaDoneDialog();
 	if (itemHit == CANCEL) return(0);
 	return(1);
 }
 
-/*
- * routine to examine the command in "uc" and fill the command completion array
- * "comarray" with appropriate entries for parsing the string.  Returns the
- * number of valid entries in the array.
- */
-INTSML us_fillcomcomp(USERCOM *uc, COMCOMP *comarray[])
+/***************************** GET FULL COMMAND *****************************/
+
+/* Full Input Dialog */
+DIALOGITEM us_ttyfulldialogitems[] =
 {
-	REGISTER INTSML j, k, com, pos, curtotal, count;
-	REGISTER char *pt;
-	char *build[MAXPARS];
-	COMCOMP *nextparams[MAXPARS+5];
-	REGISTER VARIABLE *var;
+ /*  1 */ {0, {160,328,184,392}, BUTTON, N_("OK")},
+ /*  2 */ {0, {104,328,128,392}, BUTTON, N_("Cancel")},
+ /*  3 */ {0, {24,8,56,408}, EDITTEXT, ""},
+ /*  4 */ {0, {3,8,19,408}, MESSAGE, ""},
+ /*  5 */ {0, {88,8,212,294}, SCROLL, ""},
+ /*  6 */ {0, {72,56,88,270}, MESSAGE, N_("Type '?' for a list of options")}
+};
+DIALOG us_ttyfulldialog = {{50,75,271,493}, N_("Command Completion"), 0, 6, us_ttyfulldialogitems};
 
-	comarray[0] = NOCOMCOMP;
-	com = uc->active;
+/* special items for the "ttygetline" dialog: */
+#define DGTF_LINE     3		/* line (edit text) */
+#define DGTF_PROMPT   4		/* prompt (edit text) */
+#define DGTF_OPTIONS  5		/* options (scroll) */
 
-	/* if the command is not valid, exit now */
-	if (com < 0) return(0);
+/*
+ * Routine to get a string with "parameter" information (does command completion).
+ * Returns the number of strings parsed into "paramstart" (-1 if cancelled or
+ * on error).
+ */
+INTSML ttygetfullparam(char *prompt, COMCOMP *parameter, INTSML keycount,
+	char *paramstart[])
+{
+	INTBIG i;
 
-	/* handle normal commands */
-	if (com < us_longcount)
+	/* initialize for command completion */
+	us_expandttybuffers(80);
+	us_pacurchar = 0;
+	us_pattyline[0] = 0;
+	us_palinestart = us_pattyline;
+	us_paparamstart = paramstart;
+	us_pakeycount = MAXPARS;
+	us_paparamstart[us_paparamcount = 0] = us_palinestart;
+	us_paparamtype[us_paparamcount] = parameter;
+	us_paparamtype[us_paparamcount+1] = NOCOMCOMP;
+
+	/* display the dialog box */
+	if (DiaInitDialog(&us_ttyfulldialog) != 0) return(-1);
+	DiaInitTextDialog(DGTF_OPTIONS, DiaNullDlogList, DiaNullDlogItem, DiaNullDlogDone, -1, 0);
+ 	DiaCharacterEdit(DGTF_LINE);
+
+	/* load header message */
+	(void)initinfstr();
+	if (*parameter->noise != 0)
 	{
-		/* initialize the command completion array with the default parameters */
-		for(curtotal=0; curtotal<us_lcommand[com].params; curtotal++)
-			comarray[curtotal] = us_lcommand[com].par[curtotal];
-	} else
-	{
-		/* ignore popup menus */
-		if (uc->menu != NOPOPUPMENU) return(0);
+		(void)addstringtoinfstr(parameter->noise);
+		if (parameter->def != 0)
+			(void)formatinfstr(" (%s)", parameter->def);
+	} else (void)addstringtoinfstr(prompt);
+	DiaSetText(DGTF_PROMPT, returninfstr());
 
-		/* handle macros */
-		var = us_getmacro(uc->comname);
-		if (var == NOVARIABLE) return(0);
-		pt = ((char **)var->addr)[2];
-		count = us_parsecommand(pt, build);
-		for(curtotal=0; curtotal<count; curtotal++)
-			comarray[curtotal] = (COMCOMP *)myatoi(build[curtotal]);
+	for(;;)
+	{
+		us_ttypamyItemHit = 0;
+		if (us_pagetword(us_ttyinttychar, us_ttyoutttychar, us_ttyerasettychar,
+			us_ttykillttychar, 0) != 0) break;
+		if (us_ttypamyItemHit == CANCEL || us_ttypamyItemHit == OK) break;
 	}
-	comarray[curtotal] = NOCOMCOMP;
 
-	/* now run through the parameters, inserting whatever is appropriate */
-	for(pos=0; comarray[pos] != NOCOMCOMP; pos++)
+	/* parse the line if not cancelled */
+	if (us_ttypamyItemHit == CANCEL) us_paparamcount = -1; else
 	{
-		if (pos >= uc->count) break;
-
-		/* load the necessary global and get the next parameters */
-		us_pathiskey = comarray[pos]->ifmatch;
-		k = (*comarray[pos]->params)(uc->word[pos], nextparams, ' ');
-
-		/* clip to the number of allowable parameters */
-		if (k+curtotal >= MAXPARS-1) k = MAXPARS-2-curtotal;
-
-		/* spread open the list */
-		for(j = MAXPARS-k-1; j >= curtotal; j--) comarray[j+k] = comarray[j];
-
-		/* fill in the parameter types */
-		for(j = 0; j < k; j++) comarray[curtotal+j] = nextparams[j];
-		curtotal += k;
+		us_paparamcount++;
+		for(i=1; i<us_paparamcount; i++) (us_paparamstart[i])[-1] = 0;
+		if (*us_paparamstart[us_paparamcount-1] == 0) us_paparamcount--;
 	}
-	return(curtotal);
+	DiaDoneDialog();
+	return(us_paparamcount);
 }
 
 /*
@@ -2108,6 +2166,55 @@ INTSML us_pagetword(INTSML (*input)(COMCOMP**, INTSML*), void (*output)(char*),
 	return(0);
 }
 
+INTSML us_ttyinttychar(COMCOMP **newcomcomp, INTSML *reset)
+{
+	INTSML chr;
+
+	*reset = 0;
+	for(;;)
+	{
+		chr = DiaGetNextCharacter(&us_ttypamyItemHit);
+		if (chr == -1) continue;
+		if (chr == -2)
+		{
+			if (us_ttypamyItemHit == 1 || us_ttypamyItemHit == 2) return(EOF);
+			continue;
+		}
+		break;
+	}
+	if (chr == CTRLCKEY) chr = '\n';
+	return(chr);
+}
+
+void us_ttyoutttychar(char *t)
+{
+	INTBIG len;
+	char *newmsg, *line;
+
+	len = strlen(t);
+	if (len == 0) return;
+	line = DiaGetText(DGTF_LINE);
+	newmsg = (char *)emalloc(strlen(line)+len+1, el_tempcluster);
+	(void)strcpy(newmsg, line);
+	(void)strcat(newmsg, t);
+	DiaSetText(DGTF_LINE, newmsg);
+	efree(newmsg);
+}
+
+void us_ttyerasettychar(void)
+{
+	char *line;
+
+	line = DiaGetText(DGTF_LINE);
+	line[strlen(line)-1] = 0;
+	DiaSetText(DGTF_LINE, line);
+}
+
+void us_ttykillttychar(void)
+{
+	DiaSetText(DGTF_LINE, "");
+}
+
 /*
  * Routine to expand the buffers "us_pattyline" and "us_paambiguous" to "amt" long
  */
