@@ -35,6 +35,7 @@
 #include "network.h"
 #include "efunction.h"
 #include "tecgen.h"
+#include "tecschem.h"
 
 static INTBIG *us_layer_letter_data = 0;	/* cache for "us_layerletters" */
 
@@ -399,17 +400,24 @@ void us_setfacetname(WINDOWPART *w)
 	if ((w->state&WINDOWTYPE) == EXPLORERWINDOW &&
 		strcmp(w->location, "entire") != 0) return;
 
+	/* if this is part of a frame and the current window is in another part, don't write name */
+	if (el_curwindowpart != NOWINDOWPART && w != el_curwindowpart &&
+		w->frame == el_curwindowpart->frame) return;
+
 	/* write the facet name */
 	(void)initinfstr();
 	switch (w->state&WINDOWTYPE)
 	{
 		case TEXTWINDOW:
 		case POPTEXTWINDOW:
-			ed = w->editor;
-			if (ed != NOEDITOR)
+			if (w->curnodeproto == NONODEPROTO)
 			{
-				(void)addstringtoinfstr(ed->header);
-				break;
+				ed = w->editor;
+				if (ed != NOEDITOR)
+				{
+					(void)addstringtoinfstr(ed->header);
+					break;
+				}
 			}
 		case DISPWINDOW:
 		case DISP3DWINDOW:
@@ -445,13 +453,26 @@ void us_setfacetsize(WINDOWPART *w)
 {
 	REGISTER NODEPROTO *np;
 	char valstring[50];
+	REGISTER INTBIG lines;
 
 	np = w->curnodeproto;
-	if (np == NONODEPROTO) valstring[0] = 0; else
+	valstring[0] = 0;
+	if (np != NONODEPROTO)
 	{
-		(void)strcpy(valstring, latoa(np->highx - np->lowx));
-		(void)strcat(valstring, "x");
-		(void)strcat(valstring, latoa(np->highy - np->lowy));
+		switch (w->state&WINDOWTYPE)
+		{
+			case TEXTWINDOW:
+			case POPTEXTWINDOW:
+				lines = us_totallines(w);
+				sprintf(valstring, _("%ld lines"), lines);
+				break;
+			case DISPWINDOW:
+			case DISP3DWINDOW:
+				(void)strcpy(valstring, latoa(np->highx - np->lowx));
+				(void)strcat(valstring, "x");
+				(void)strcat(valstring, latoa(np->highy - np->lowy));
+				break;
+		}
 	}
 	ttysetstatusfield(w->frame, us_statusfacetsize, valstring, 0);
 }
@@ -459,13 +480,22 @@ void us_setfacetsize(WINDOWPART *w)
 /* write the grid size of window "w" from "w->gridx" and "w->gridy" */
 void us_setgridsize(WINDOWPART *w)
 {
+	REGISTER NODEPROTO *np;
 	char valstring[50];
 
-	if (w->curnodeproto == NONODEPROTO) valstring[0] = 0; else
+	np = w->curnodeproto;
+	valstring[0] = 0;
+	if (np != NONODEPROTO)
 	{
-		(void)strcpy(valstring, latoa(w->gridx));
-		(void)strcat(valstring, "x");
-		(void)strcat(valstring, latoa(w->gridy));
+		switch (w->state&WINDOWTYPE)
+		{
+			case DISPWINDOW:
+			case DISP3DWINDOW:
+				(void)strcpy(valstring, latoa(w->gridx));
+				(void)strcat(valstring, "x");
+				(void)strcat(valstring, latoa(w->gridy));
+				break;
+		}
 	}
 	ttysetstatusfield(NOWINDOWFRAME, us_statusgridsize, valstring, 0);
 }
@@ -542,11 +572,11 @@ void us_printmacros(void)
 	}
 	if (gotany == 0)
 	{
-		ttyputmsg(_("No macros defined"));
+		ttyputmsg(M_("No macros defined"));
 		return;
 	}
 
-	ttyputmsg(_("---------- Macros by Package ----------"));
+	ttyputmsg(M_("---------- Macros by Package ----------"));
 
 	/* now print the macros by package */
 	for(pack = us_macropacktop; ; pack = pack->nextmacropack)
@@ -554,14 +584,14 @@ void us_printmacros(void)
 		if (pack == NOMACROPACK)
 		{
 			if (keyboard == 0) break;
-			name = _("Keyboard-defined");
+			name = M_("Keyboard-defined");
 		} else
 		{
 			if (pack->total == 0) continue;
 			name = pack->packname;
 		}
 		(void)strcpy(line, name);
-		(void)strcat(line, _(" macros:"));
+		(void)strcat(line, M_(" macros:"));
 		len = strlen(line);
 		inter = "";
 		for(i=0; i<us_aid->numvar; i++)
@@ -606,12 +636,12 @@ void us_printmacro(VARIABLE *mac)
 	len = (INTSML)getlength(mac);
 	if (len <= 3)
 	{
-		ttyputmsg(_("Macro '%s' is not defined"), &makename(mac->key)[11]);
+		ttyputmsg(M_("Macro '%s' is not defined"), &makename(mac->key)[11]);
 		return;
 	}
 
-	ttyputmsg(_("Macro '%s' has %d %s"), &makename(mac->key)[11], len-3,
-		makeplural(_("command"), len-3));
+	ttyputmsg(M_("Macro '%s' has %d %s"), &makename(mac->key)[11], len-3,
+		makeplural(M_("command"), len-3));
 	for(j=3; j<len; j++) ttyputmsg("   %s", ((char **)mac->addr)[j]);
 }
 
@@ -766,31 +796,41 @@ void us_describecontents(NODEINST *ni)
 
 struct
 {
-	char *menumessage;
-	char *statusmessage;
+	NODEPROTO **prim;
+	INTBIG      bits;
+	char       *statusmessage;
 } us_specialstatus[] =
 {
-	{"SCHEM_transistor_type.nmos",  "nMOS"},
-	{"SCHEM_transistor_type.pmos",  "pMOS"},
-	{"SCHEM_transistor_type.dmos",  "dMOS"},
-	{"SCHEM_transistor_type.npn",   "NPN"},
-	{"SCHEM_transistor_type.pnp",   "PNP"},
-	{"SCHEM_transistor_type.emes",  "EMES"},
-	{"SCHEM_transistor_type.dmes",  "DMES"},
-	{"SCHEM_transistor_type.njfet", "NJFET"},
-	{"SCHEM_transistor_type.pjfet", "PJFET"},
-	{"SCHEM_twoport_type.h",        "CCVS"},
-	{"SCHEM_twoport_type.u",        "VCVS"},
-	{"SCHEM_twoport_type.f",        "CCCS"},
-	{"SCHEM_twoport_type.g",        "VCCS"},
-	{"SCHEM_twoport_type.l",        "Transmission"},
-	{"SCHEM_diode.z",               "Zener"},
-	{"SCHEM_capacitance.e",         "Electrolytic"},
-	{"SCHEM_flipflop_type.rs",      "RS"},
-	{"SCHEM_flipflop_type.jk",      "JK"},
-	{"SCHEM_flipflop_type.d",       "D"},
-	{"SCHEM_flipflop_type.t",       "T"},
-	{0, 0}
+	{&sch_transistorprim, TRANNMOS,   "nMOS"},
+	{&sch_transistorprim, TRANDMOS,   "dMOS"},
+	{&sch_transistorprim, TRANPMOS,   "pMOS"},
+	{&sch_transistorprim, TRANNPN,    "NPN"},
+	{&sch_transistorprim, TRANPNP,    "PNP"},
+	{&sch_transistorprim, TRANNJFET,  "NJFET"},
+	{&sch_transistorprim, TRANPJFET,  "PJFET"},
+	{&sch_transistorprim, TRANDMES,   "DMES"},
+	{&sch_transistorprim, TRANEMES,   "EMES"},
+	{&sch_transistor4prim,TRANNMOS,   "nMOS"},
+	{&sch_transistor4prim,TRANDMOS,   "dMOS"},
+	{&sch_transistor4prim,TRANPMOS,   "pMOS"},
+	{&sch_transistor4prim,TRANNPN,    "NPN"},
+	{&sch_transistor4prim,TRANPNP,    "PNP"},
+	{&sch_transistor4prim,TRANNJFET,  "NJFET"},
+	{&sch_transistor4prim,TRANPJFET,  "PJFET"},
+	{&sch_transistor4prim,TRANDMES,   "DMES"},
+	{&sch_transistor4prim,TRANEMES,   "EMES"},
+	{&sch_twoportprim,    TWOPVCCS,   "VCCS"},
+	{&sch_twoportprim,    TWOPCCVS,   "CCVS"},
+	{&sch_twoportprim,    TWOPVCVS,   "VCVS"},
+	{&sch_twoportprim,    TWOPCCCS,   "CCCS"},
+	{&sch_twoportprim,    TWOPTLINE,  "Transmission"},
+	{&sch_diodeprim,      DIODEZENER, "Zener"},
+	{&sch_capacitorprim,  CAPACELEC,  "Electrolytic"},
+	{&sch_ffprim,         FFTYPERS,   "RS"},
+	{&sch_ffprim,         FFTYPEJK,   "JK"},
+	{&sch_ffprim,         FFTYPED,    "D"},
+	{&sch_ffprim,         FFTYPET,    "T"},
+	{0, 0, 0}
 };
 
 /*
@@ -798,20 +838,20 @@ struct
  */
 char *us_describemenunode(COMMANDBINDING *commandbinding)
 {
-	REGISTER INTBIG i;
+	REGISTER INTBIG i, bits;
 
 	(void)initinfstr();
 	(void)addstringtoinfstr(describenodeproto(commandbinding->nodeglyph));
-	if (commandbinding->menumessage != 0)
+	if (commandbinding->menumessage == 0) bits = 0; else
+		bits = myatoi(commandbinding->menumessage);
+	for(i=0; us_specialstatus[i].prim != 0; i++)
+		if (us_specialstatus[i].bits == bits &&
+			*us_specialstatus[i].prim == commandbinding->nodeglyph) break;
+	if (us_specialstatus[i].prim != 0)
 	{
-		for(i=0; us_specialstatus[i].menumessage != 0; i++)
-			if (namesame(commandbinding->menumessage, us_specialstatus[i].menumessage) == 0) break;
-		if (us_specialstatus[i].menumessage != 0)
-		{
-			(void)addstringtoinfstr(" (");
-			(void)addstringtoinfstr(us_specialstatus[i].statusmessage);
-			(void)addstringtoinfstr(")");
-		}
+		(void)addstringtoinfstr(" (");
+		(void)addstringtoinfstr(us_specialstatus[i].statusmessage);
+		(void)addstringtoinfstr(")");
 	}
 	return(returninfstr());
 }
@@ -1033,7 +1073,7 @@ void us_chatportproto(NODEINST *ni, PORTPROTO *pp)
 
 	/* talk about the port prototype */
 	(void)initinfstr();
-	activity = us_describeportbits(pp);
+	activity = describeportbits(pp);
 	if (*activity != 0)
 	{
 		(void)formatinfstr(_("%s port %s connects to "), activity, pp->protoname);
@@ -1076,7 +1116,7 @@ void us_chatportproto(NODEINST *ni, PORTPROTO *pp)
 		describenodeinst(pp->subnodeinst), pp->subportproto->protoname);
 
 	/* talk about any arcs on this prototype */
-	for(pi=ni->firstportarcinst; pi!=NOPORTARCINST; pi=pi->nextportarcinst)
+	for(pi = ni->firstportarcinst; pi != NOPORTARCINST; pi = pi->nextportarcinst)
 	{
 		if (pi->proto != pp) continue;
 		ai = pi->conarcinst;
@@ -1086,12 +1126,12 @@ void us_chatportproto(NODEINST *ni, PORTPROTO *pp)
 	}
 
 	/* talk about any exports of this prototype */
-	for(pe=ni->firstportexpinst; pe!=NOPORTEXPINST; pe=pe->nextportexpinst)
+	for(pe = ni->firstportexpinst; pe != NOPORTEXPINST; pe = pe->nextportexpinst)
 		if (pe->proto == pp)
 	{
 		(void)initinfstr();
 		(void)formatinfstr(_("  Available as %s export '%s' (label %s"),
-			us_describeportbits(pe->exportproto), pe->exportproto->protoname,
+			describeportbits(pe->exportproto), pe->exportproto->protoname,
 				us_describetextdescript(pe->exportproto->textdescript));
 		if ((pe->exportproto->userbits&PORTDRAWN) != 0) (void)addstringtoinfstr(_(",always-drawn"));
 		if ((pe->exportproto->userbits&BODYONLY) != 0) (void)addstringtoinfstr(_(",only-on-body"));
@@ -1100,28 +1140,6 @@ void us_chatportproto(NODEINST *ni, PORTPROTO *pp)
 	}
 }
 
-char *us_describeportbits(PORTPROTO *pp)
-{
-	switch (pp->userbits&STATEBITS)
-	{
-		case INPORT:     return(_("Input"));
-		case OUTPORT:    return(_("Output"));
-		case BIDIRPORT:  return(_("Bidirectional"));
-		case PWRPORT:    return(_("Power"));
-		case GNDPORT:    return(_("Ground"));
-		case CLKPORT:    return(_("Clock"));
-		case C1PORT:     return(_("Clock Phase 1"));
-		case C2PORT:     return(_("Clock Phase 2"));
-		case C3PORT:     return(_("Clock Phase 3"));
-		case C4PORT:     return(_("Clock Phase 4"));
-		case C5PORT:     return(_("Clock Phase 5"));
-		case C6PORT:     return(_("Clock Phase 6"));
-		case REFOUTPORT: return(_("Reference Output"));
-		case REFINPORT:  return(_("Reference Input"));
-	}
-	return("");
-}
-
 /*
  * routine to construct a string that describes the text descriptor field
  * in "descript".  The string has the form "POSITION+(DX,DY),SIZE" if there
@@ -1185,9 +1203,13 @@ char *us_describefont(INTBIG font)
 		case TXT16P:    return(_("16-point"));
 		case TXT18P:    return(_("18-point"));
 		case TXT20P:    return(_("20-point"));
-		case TXTSMALL:  return(_("small"));
-		case TXTMEDIUM: return(_("medium"));
-		case TXTLARGE:  return(_("large"));
+		case TXTHL:     return(_("Half-lambda"));
+		case TXT1L:     return(_("1-lambda"));
+		case TXT2L:     return(_("2-lambda"));
+		case TXT3L:     return(_("3-lambda"));
+		case TXT4L:     return(_("4-lambda"));
+		case TXT5L:     return(_("5-lambda"));
+		case TXT6L:     return(_("6-lambda"));
 	}
 	return("?");
 }
@@ -1198,8 +1220,9 @@ char *us_describefont(INTBIG font)
  */
 char *us_keyname(INTSML key)
 {
-	static char line[10];
+	static char line[30];
 	REGISTER char *pt;
+	char *acceleratorstring, *acceleratorprefix;
 
 	pt = line;
 	if (key >= 0200)
@@ -1210,8 +1233,10 @@ char *us_keyname(INTSML key)
 			return(line);
 		}
 
-		*pt++ = 'M';
-		*pt++ = '-';
+		getacceleratorstrings(&acceleratorstring, &acceleratorprefix);
+		strcpy(line, acceleratorstring);
+		strcat(line, "-");
+		pt = &line[strlen(line)];
 		key -= 0200;
 	}
 	if (key < 040)
@@ -1480,7 +1505,7 @@ INTSML us_preventloss(LIBRARY *lib, char *action, INTSML cancancel)
 				(void)askaid(io_aid, "write", (INTBIG)l, (INTBIG)"binary");
 			} else
 			{
-				ttyputverbose(_("Keep working"));
+				ttyputverbose(M_("Keep working"));
 				return(1);
 			}
 		} else
@@ -1510,7 +1535,7 @@ INTSML us_preventloss(LIBRARY *lib, char *action, INTSML cancancel)
 				us_saveoptions();
 			} else
 			{
-				ttyputverbose(_("Keep working"));
+				ttyputverbose(M_("Keep working"));
 				return(1);
 			}
 		} else
@@ -1767,10 +1792,10 @@ void us_setaid(INTSML count, char *par[], INTSML state)
 
 	if (count == 0)
 	{
-		count = ttygetparam(_("Which aid: "), &us_onoffaidp, MAXPARS, par);
+		count = ttygetparam(M_("Which aid: "), &us_onoffaidp, MAXPARS, par);
 		if (count == 0)
 		{
-			us_abortcommand(_("Specify an aid to control"));
+			us_abortcommand(M_("Specify an aid to control"));
 			return;
 		}
 	}
@@ -1787,23 +1812,23 @@ void us_setaid(INTSML count, char *par[], INTSML state)
 
 	if (aid == us_aid && state <= 1)
 	{
-		us_abortcommand(_("No! I won't go!"));
+		us_abortcommand(M_("No! I won't go!"));
 		return;
 	}
 	if ((aid->aidstate&AIDON) == 0 && state <= 1)
 	{
-		ttyputverbose(_("%s already off"), pt);
+		ttyputverbose(M_("%s already off"), pt);
 		return;
 	}
 	if ((aid->aidstate&AIDON) != 0 && state > 1)
 	{
-		ttyputverbose(_("%s already on"), pt);
+		ttyputverbose(M_("%s already on"), pt);
 		return;
 	}
-	if (state <= 1) ttyputverbose(_("%s turned off"), aid->aidname); else
+	if (state <= 1) ttyputverbose(M_("%s turned off"), aid->aidname); else
 	{
 		if ((aid->aidstate&AIDINCREMENTAL) == 0 || state == 3)
-			ttyputverbose(_("%s turned on"), aid->aidname); else
+			ttyputverbose(M_("%s turned on"), aid->aidname); else
 				ttyputmsg(_("%s turned on, catching up on changes..."), aid->aidname);
 	}
 
