@@ -32,63 +32,8 @@
 #include "usr.h"
 #include "edialogs.h"
 
-INTSML   us_ttymute;					/* nonzero to supress unimportant messages */
-INTBIG	 us_ttypamyItemHit;				/* item hit during parsing */
-INTSML	 us_ttyttywriteseparator;		/* nonzero to write separator before next text */
-INTSML   us_ttyttyinited;
-
-/* ttygetline() Dialog */
-DIALOGITEM us_ttyinputdialogitems[] =
-{
- /*  1 */ {0, {96,200,120,264}, BUTTON, N_("OK")},
- /*  2 */ {0, {96,24,120,88}, BUTTON, N_("Cancel")},
- /*  3 */ {0, {58,38,74,246}, EDITTEXT, ""},
- /*  4 */ {0, {3,9,51,281}, MESSAGE, ""}
-};
-DIALOG us_ttyinputdialog = {{50,75,188,371}, "", 0, 4, us_ttyinputdialogitems};
-
-/* Full Input Dialog */
-DIALOGITEM us_ttyfulldialogitems[] =
-{
- /*  1 */ {0, {160,328,184,392}, BUTTON, N_("OK")},
- /*  2 */ {0, {104,328,128,392}, BUTTON, N_("Cancel")},
- /*  3 */ {0, {24,8,56,408}, EDITTEXT, ""},
- /*  4 */ {0, {3,8,19,408}, MESSAGE, ""},
- /*  5 */ {0, {88,8,212,294}, SCROLL, ""},
- /*  6 */ {0, {72,56,88,270}, MESSAGE, N_("Type '?' for a list of options")}
-};
-DIALOG us_ttyfulldialog = {{50,75,271,493}, N_("Command Completion"), 0, 6, us_ttyfulldialogitems};
-
-/* prototypes for local routines */
-INTSML us_ttyinttychar(COMCOMP**, INTSML*);
-void us_ttyoutttychar(char*);
-void us_ttyerasettychar(void);
-void us_ttykillttychar(void);
-
-/************************* INITIALIZATION/TERMINATION *************************/
-
-/*
- * initialize the text terminal and saves state information
- * so that we can restore its characteristics later.  This
- * routine is called once at the start of the program.
- */
-void us_ttyinit(void)
-{
-	/* allocate character arrays that are as wide as the terminal */
-	us_pattyline = us_paambiguous = 0;
-	us_ttymute = 0;
-	us_ttyttywriteseparator = 0;
-	us_ttyttyinited = 1;
-}
-
-/*
- * close the terminal (called once at the end of the program)
- */
-void us_ttyclose(void)
-{
-	if (us_pattyline != 0) efree(us_pattyline);
-	if (us_paambiguous != 0) efree(us_paambiguous);
-}
+INTBIG   us_ttymute = 0;				/* nonzero to supress unimportant messages */
+INTBIG	 us_ttyttywriteseparator = 0;	/* nonzero to write separator before next text */
 
 /***************************** TERMINAL OUTPUT *****************************/
 
@@ -96,9 +41,9 @@ void us_ttyclose(void)
  * routine to mute nonerror messages if flag is nonzero
  * (returns previous state)
  */
-INTSML ttyquiet(INTSML flag)
+INTBIG ttyquiet(INTBIG flag)
 {
-	REGISTER INTSML prev;
+	REGISTER INTBIG prev;
 
 	prev = us_ttymute;
 	us_ttymute = flag;
@@ -216,6 +161,32 @@ void us_abortcommand(char *msg, ...)
 	us_unknowncommand();
 }
 
+/*
+ * routine to cause further single-key commands to be ignored until a
+ * carriage-return is typed.
+ */
+void us_unknowncommand(void)
+{
+	REGISTER INTBIG i;
+
+	if ((us_aid->aidstate&NOKEYLOCK) != 0) return;
+	i = ttyquiet(0);
+	ttyputmsg(_("Single-key commands will be ignored until the next RETURN"));
+	(void)ttyquiet(i);
+	us_state |= SKIPKEYS;
+}
+
+void us_abortedmsg(void)
+{
+	if ((us_aid->aidstate & JUSTTHEFACTS) == 0) us_abortcommand(_("Aborted")); else
+	{
+		us_state |= COMMANDFAILED;
+		if (getvalkey((INTBIG)us_aid, VAID, VSTRING, us_macrobuilding) != NOVARIABLE)
+			ttyputmsg(_("Command ignored from macro definition"));
+		us_unknowncommand();
+	}
+}
+
 /****************************** TERMINAL INPUT ******************************/
 
 /*
@@ -227,6 +198,20 @@ INTSML ttygetchar(void)
 	return(getnxtchar());
 }
 
+/* ttygetline() Dialog */
+DIALOGITEM us_ttyinputdialogitems[] =
+{
+ /*  1 */ {0, {96,200,120,264}, BUTTON, N_("OK")},
+ /*  2 */ {0, {96,24,120,88}, BUTTON, N_("Cancel")},
+ /*  3 */ {0, {58,38,74,246}, EDITTEXT, ""},
+ /*  4 */ {0, {3,9,51,281}, MESSAGE, ""}
+};
+DIALOG us_ttyinputdialog = {{50,75,188,371}, "", 0, 4, us_ttyinputdialogitems};
+
+/* special items for the "ttygetline" dialog: */
+#define DGTL_LINE     3		/* line (edit text) */
+#define DGTL_PROMPT   4		/* prompt (stat text) */
+
 /*
  * routine to print "prompt" and then read a line of text from the terminal
  * The address of the text line is returned (0 if cancelled).
@@ -250,8 +235,8 @@ char *ttygetline(char *prompt)
 
 	/* display the dialog box */
 	if (DiaInitDialog(&us_ttyinputdialog) != 0) return(0);
-	DiaSetText(4, localprompt);
-	DiaSetText(-3, defaultval);
+	DiaSetText(DGTL_PROMPT, localprompt);
+	DiaSetText(-DGTL_LINE, defaultval);
 
 	/* loop until done */
 	for(;;)
@@ -262,7 +247,7 @@ char *ttygetline(char *prompt)
 	}
 
 	if (itemHit != CANCEL)
-		line = us_putintoinfstr(DiaGetText(3));
+		line = us_putintoinfstr(DiaGetText(DGTL_LINE));
 
 	/* terminate the dialog */
 	DiaDoneDialog();
@@ -280,95 +265,6 @@ char *ttygetlinemessages(char *prompt)
 	return(getmessagesstring(prompt));
 }
 
-/*
- * routine to get a string with "parameter" information (does command completion)
- */
-INTSML ttygetparam(char *prompt, COMCOMP *parameter, INTSML keycount, char *paramstart[])
-{
-	REGISTER INTBIG itemHit;
-	extern DIALOG us_listdialog;
-
-	/* is this necessary?  if not, us_pathiskey does not need to be global */
-	us_pathiskey = parameter->ifmatch;
-
-	/* use dialogs if requested */
-	if ((us_aid->aidstate & USEDIALOGS) != 0)
-	{
-		return(us_specialparse(prompt, parameter, keycount, paramstart));
-	}
-
-	/* the general case: display the dialog box */
-	if (DiaInitDialog(&us_listdialog) != 0) return(0);
-	DiaInitTextDialog(3, parameter->toplist, parameter->nextcomcomp, DiaNullDlogDone,
-		0, SCSELMOUSE|SCSELKEY|SCDOUBLEQUIT);
-	DiaSetText(4, prompt);
-
-	for(;;)
-	{
-		itemHit = DiaNextHit();
-		if (itemHit == OK || itemHit == CANCEL) break;
-	}
-	paramstart[0] = us_putintoinfstr(DiaGetScrollLine(3, DiaGetCurLine(3)));
-	DiaDoneDialog();
-	if (itemHit == CANCEL) return(0);
-	return(1);
-}
-
-/*
- * Routine to get a string with "parameter" information (does command completion).
- * Returns the number of strings parsed into "paramstart" (-1 if cancelled or
- * on error).
- */
-INTSML ttygetfullparam(char *prompt, COMCOMP *parameter, INTSML keycount,
-	char *paramstart[])
-{
-	INTSML i;
-
-	/* initialize for command completion */
-	us_expandttybuffers(80);
-	us_pacurchar = 0;
-	us_pattyline[0] = 0;
-	us_palinestart = us_pattyline;
-	us_paparamstart = paramstart;
-	us_pakeycount = MAXPARS;
-	us_paparamstart[us_paparamcount = 0] = us_palinestart;
-	us_paparamtype[us_paparamcount] = parameter;
-	us_paparamtype[us_paparamcount+1] = NOCOMCOMP;
-
-	/* display the dialog box */
-	if (DiaInitDialog(&us_ttyfulldialog) != 0) return(-1);
-	DiaInitTextDialog(5, DiaNullDlogList, DiaNullDlogItem, DiaNullDlogDone, -1, 0);
- 	DiaCharacterEdit(3);
-
-	/* load header message */
-	(void)initinfstr();
-	if (*parameter->noise != 0)
-	{
-		(void)addstringtoinfstr(parameter->noise);
-		if (parameter->def != 0)
-			(void)formatinfstr(" (%s)", parameter->def);
-	} else (void)addstringtoinfstr(prompt);
-	DiaSetText(4, returninfstr());
-
-	for(;;)
-	{
-		us_ttypamyItemHit = 0;
-		if (us_pagetword(us_ttyinttychar, us_ttyoutttychar, us_ttyerasettychar,
-			us_ttykillttychar, 0) != 0) break;
-		if (us_ttypamyItemHit == CANCEL || us_ttypamyItemHit == OK) break;
-	}
-
-	/* parse the line if not cancelled */
-	if (us_ttypamyItemHit == CANCEL) us_paparamcount = -1; else
-	{
-		us_paparamcount++;
-		for(i=1; i<us_paparamcount; i++) (us_paparamstart[i])[-1] = 0;
-		if (*us_paparamstart[us_paparamcount-1] == 0) us_paparamcount--;
-	}
-	DiaDoneDialog();
-	return(us_paparamcount);
-}
-
 /***************************** INTERNAL SUPPORT *****************************/
 
 /*
@@ -378,16 +274,9 @@ INTSML ttygetfullparam(char *prompt, COMCOMP *parameter, INTSML keycount,
 void us_ttyprint(INTSML important, char *msg, va_list ap)
 {
 	char line[8192];
-	REGISTER INTSML i, j, k;
+	REGISTER INTBIG i, j, k;
 	static INTBIG sepcount = 0;
 
-	if (us_ttyttyinited == 0)
-	{
-		(void)vsprintf(line, msg, ap);
-		error("%s", line);
-		return;
-	}
-
 	if (us_ttyttywriteseparator != 0)
 	{
 		sepcount++;
@@ -425,52 +314,3 @@ void us_ttyprint(INTSML important, char *msg, va_list ap)
 		if (us_termaudit != 0) xprintf(us_termaudit, "%s\n", &line[k]);
 	}
 }
-
-INTSML us_ttyinttychar(COMCOMP **newcomcomp, INTSML *reset)
-{
-	INTSML chr;
-
-	*reset = 0;
-	for(;;)
-	{
-		chr = DiaGetNextCharacter(&us_ttypamyItemHit);
-		if (chr == -1) continue;
-		if (chr == -2)
-		{
-			if (us_ttypamyItemHit == 1 || us_ttypamyItemHit == 2) return(EOF);
-			continue;
-		}
-		break;
-	}
-	if (chr == CTRLCKEY) chr = '\n';
-	return(chr);
-}
-
-void us_ttyoutttychar(char *t)
-{
-	INTBIG len;
-	char *newmsg, *line;
-
-	len = strlen(t);
-	if (len == 0) return;
-	line = DiaGetText(3);
-	newmsg = (char *)emalloc(strlen(line)+len+1, el_tempcluster);
-	(void)strcpy(newmsg, line);
-	(void)strcat(newmsg, t);
-	DiaSetText(3, newmsg);
-	efree(newmsg);
-}
-
-void us_ttyerasettychar(void)
-{
-	char *line;
-
-	line = DiaGetText(3);
-	line[strlen(line)-1] = 0;
-	DiaSetText(3, line);
-}
-
-void us_ttykillttychar(void)
-{
-	DiaSetText(3, "");
-}
