@@ -41,7 +41,7 @@
 #include "tecgen.h"
 #include "sim.h"
 
-#define	MAXTRACE 400
+#define MAXTRACE 400
 #define MINSLIDEDELAY 60			/* ticks between shifts of window when dragging to edge */
 static INTSML us_tracelist;
 static INTBIG us_tracedata[MAXTRACE];
@@ -51,7 +51,7 @@ static GRAPHICS us_highl = {LAYERH, 0, SOLIDC, SOLIDC,
 static INTBIG us_dragx, us_dragy, us_dragox, us_dragoy, us_dragoffx,
 	us_dragoffy, us_lastcurx, us_lastcury, us_dragpoint, us_dragjoinfactor,
 	us_dragnodetotal, us_dragdescript, us_cantdrag, us_draglowval, us_draghighval;
-static INTSML us_dragshown, us_dragangle, us_dragstate, us_dragfport, us_dragextra,
+static INTBIG us_dragshown, us_dragangle, us_dragstate, us_dragfport, us_dragextra,
 	us_dragnobox, us_dragstayonhigh, us_dragstill, us_dragcorner, us_dragspecial;
 static INTBIG      us_arrowamount;				/* for slider tracking */
 static INTBIG      us_arrowlx, us_arrowhx, us_arrowly, us_arrowhy;
@@ -68,8 +68,8 @@ static char        us_dragmessage[100];
 static UINTBIG     us_beforepantime = 0;
 static INTBIG      us_initthumbcoord;
 static WINDOWPART  us_trackww;
-static INTBIG      us_dragnumtext, us_dragtotaltext = 0;
-static char      **us_dragtextlist;
+static INTBIG      us_dragtextcount;
+static char      **us_dragtexts;
 
 /* prototypes for local routines */
 void us_finddoibegin(INTSML);
@@ -79,14 +79,6 @@ void us_wanttoinvert(INTBIG fx, INTBIG fy, INTBIG tx, INTBIG ty, WINDOWPART *w);
 INTSML us_geominrect(GEOM *geom, INTBIG slx, INTBIG shx, INTBIG sly, INTBIG shy);
 void us_drawdistance(void);
 
-/*
- * Routine to free all memory associated with this module.
- */
-void us_freetrackmemory(void)
-{
-	if (us_dragtotaltext > 0) efree((char *)us_dragtextlist);
-}
-
 /************************* NODE TRACE *************************/
 
 /* initialization routine when reading a cursor trace */
@@ -550,7 +542,7 @@ INTSML us_rotatedown(INTBIG x, INTBIG y)
 	saver = us_dragnodeinst->rotation;
 	savet = us_dragnodeinst->transpose;
 	us_dragnodeinst->transpose = 0;
-	us_dragnodeinst->rotation = newangle-us_dragangle;
+	us_dragnodeinst->rotation = (INTSML)(newangle-us_dragangle);
 	while (us_dragnodeinst->rotation < 0) us_dragnodeinst->rotation += 3600;
 	while (us_dragnodeinst->rotation > 3600) us_dragnodeinst->rotation -= 3600;
 	makerot(us_dragnodeinst, trans);
@@ -585,7 +577,7 @@ void us_sizebegin(void)
 	us_dragcorner = -1;
 }
 
-INTSML us_sizeterm(void)
+INTBIG us_sizeterm(void)
 {
 	return(us_dragcorner);
 }
@@ -653,7 +645,7 @@ INTSML us_sizedown(INTBIG x, INTBIG y)
 {
 	REGISTER INTBIG lx, hx, ly, hy, otherx, othery, dist, bestdist;
 	INTBIG xl, xh, yl, yh, rx, ry;
-	REGISTER INTSML corner;
+	REGISTER INTBIG corner;
 	XARRAY trans;
 
 	/* grid align the cursor value */
@@ -848,8 +840,8 @@ void us_finddterm(INTBIG *x, INTBIG *y)
 
 /************************* FIND INTERACTIVE *************************/
 
-void us_findiinit(INTSML findport, INTSML extrainfo, INTSML findangle, INTSML stayonhighlighted,
-	INTSML findstill, INTSML findnobox, INTSML findspecial)
+void us_findiinit(INTBIG findport, INTBIG extrainfo, INTBIG findangle, INTBIG stayonhighlighted,
+	INTBIG findstill, INTBIG findnobox, INTBIG findspecial)
 {
 	us_dragfport = findport;
 	us_dragextra = (extrainfo != 0 ? 1 : 0);
@@ -880,10 +872,9 @@ void us_finddoibegin(INTSML compl)
 	REGISTER ARCINST *ai;
 	REGISTER PORTPROTO *pp;
 	REGISTER INTSML len, another;
-	REGISTER char **newtext;
-	HIGHLIGHT newhigh, oldhigh;
+	HIGHLIGHT newhigh, oldhigh, high;
 	INTSML i, j;
-	INTBIG xcur, ycur, dist, bestdist, numhigh, newcount;
+	INTBIG xcur, ycur, dist, bestdist, numhigh, addr, type;
 
 	/* remember initial time of this action */
 	us_initialtime = ticktime();
@@ -894,7 +885,7 @@ void us_finddoibegin(INTSML compl)
 	us_dragpoly->desc = &us_highl;
 	us_dragstate = -1;
 	us_dragshown = 0;
-	us_dragnumtext = 0;
+	us_dragtextcount = 0;
 	us_dragwindow = el_curwindowpart;
 
 	/* get cursor coordinates */
@@ -916,6 +907,7 @@ void us_finddoibegin(INTSML compl)
 		newhigh.fromgeom = NOGEOM;
 		newhigh.fromport = NOPORTPROTO;
 		newhigh.fromvar = NOVARIABLE;
+		newhigh.fromvarnoeval = NOVARIABLE;
 		newhigh.frompoint = 0;
 	}
 
@@ -1020,8 +1012,8 @@ void us_finddoibegin(INTSML compl)
 		if (i >= len || us_dragpoint >= 0)
 		{
 			/* object is not highlighted: select only it and move */
-			us_setfind(&newhigh, (INTSML)((us_dragpoint < 0 ? 0 : 1)),
-				(INTSML)((us_dragextra != 0 ? HIGHEXTRA : 0)), 0, us_dragnobox);
+			us_setfind(&newhigh, (us_dragpoint < 0 ? 0 : 1),
+				(us_dragextra != 0 ? HIGHEXTRA : 0), 0, us_dragnobox);
 		} else
 		{
 			/* object is already highlighted: rehighlight and move */
@@ -1036,7 +1028,7 @@ void us_finddoibegin(INTSML compl)
 			/* object not highlighted: add it and move */
 			/* newhigh.fromport = NOPORTPROTO; */
 			newhigh.facet = geomparent(newhigh.fromgeom);
-			us_setfind(&newhigh, 0, (INTSML)((us_dragextra != 0 ? HIGHEXTRA : 0)), 1, us_dragnobox);
+			us_setfind(&newhigh, 0, (us_dragextra != 0 ? HIGHEXTRA : 0), 1, us_dragnobox);
 		} else
 		{
 			/* object highlighted: unhighlight it and quit */
@@ -1060,8 +1052,9 @@ void us_finddoibegin(INTSML compl)
 	if (highvar == NOVARIABLE) return;
 	numhigh = getlength(highvar);
 
-	/* get the selected nodes and arcs */
-	us_dragobjectlist = us_gethighlighted(OBJNODEINST | OBJARCINST);
+	/* get the selected nodes, arcs, and text */
+	us_dragobjectlist = us_gethighlighted(OBJNODEINST | OBJARCINST,
+		&us_dragtextcount, &us_dragtexts);
 	for(i=0; us_dragobjectlist[i] != NOGEOM; i++)
 		if (us_dragnodeproto != geomparent(us_dragobjectlist[i])) return;
 
@@ -1078,42 +1071,27 @@ void us_finddoibegin(INTSML compl)
 		if (us_dragobjectlist[i]->entrytype == OBJNODEINST)
 			us_dragobjectlist[i]->entryaddr.ni->temp1 = 1;
 
-	/* build list of selected text, removing invisible nodes from node list */
-	us_dragnumtext = 0;
-	for(i=0; i<numhigh; i++)
+	/* stop if nothing selected */
+	if (us_dragobjectlist[0] == NOGEOM && us_dragtextcount == 0) return;
+
+	/* remove nodes listed as text objects */
+	for(i=0; i<us_dragtextcount; i++)
 	{
-		(void)us_makehighlight(((char **)highvar->addr)[i], &us_draghigh);
-		if ((us_draghigh.status&HIGHTYPE) != HIGHTEXT) continue;
-		us_gethighaddrtype(&us_draghigh, &us_dragaddr, &us_dragtype);
-		if (us_dragtype == VNODEINST)
+		(void)us_makehighlight(us_dragtexts[i], &high);
+		if (us_nodemoveswithtext(&high) == 0) continue;
+		us_gethighaddrtype(&high, &addr, &type);
+
+		/* undraw the text */
+		if (type == VPORTPROTO)
 		{
-			ni = (NODEINST *)us_dragaddr;
-			if (ni->proto == gen_invispinprim) ni->temp1 = 0;
+			ni = ((PORTPROTO *)addr)->subnodeinst;
+			ni->temp1 = 0;
+		} else if (type == VNODEINST)
+		{
+			ni = (NODEINST *)addr;
+			ni->temp1 = 0;
 		}
-		if (us_dragnumtext >= us_dragtotaltext)
-		{
-			newcount = us_dragnumtext + 5;
-			newtext = (char **)emalloc(newcount * (sizeof (char *)), us_aid->cluster);
-			if (newtext == 0) return;
-			for(j=0; j<us_dragtotaltext; j++)
-				newtext[j] = us_dragtextlist[j];
-			for( ; j<newcount; j++) newtext[j] = 0;
-			if (us_dragtotaltext > 0) efree((char *)us_dragtextlist);
-			us_dragtextlist = newtext;
-			us_dragtotaltext = newcount;
-		}
-		if (us_dragtextlist[us_dragnumtext] == 0)
-		{
-			allocstring(&us_dragtextlist[us_dragnumtext], ((char **)highvar->addr)[i],
-				us_aid->cluster);
-		} else
-		{
-			reallocstring(&us_dragtextlist[us_dragnumtext], ((char **)highvar->addr)[i],
-				us_aid->cluster);
-		}
-		us_dragnumtext++;
 	}
-	if (us_dragobjectlist[0] == NOGEOM && us_dragnumtext == 0) return;
 
 	/* count the number of nodes, excluding locked ones */
 	us_dragnodetotal = 0;
@@ -1127,7 +1105,7 @@ void us_finddoibegin(INTSML compl)
 		}
 		us_dragnodetotal++;
 	}
-	if (us_dragnodetotal == 0 && us_dragnumtext == 0) us_cantdrag = 1; else
+	if (us_dragnodetotal == 0 && us_dragtextcount == 0) us_cantdrag = 1; else
 		us_cantdrag = 0;
 
 	/* remove from list all arcs touching locked nodes */
@@ -1244,10 +1222,10 @@ INTSML us_findidown(INTBIG x, INTBIG y)
 
 void us_findiup(void)
 {
-	INTBIG xcur, ycur, search, dx, dy;
+	INTBIG xcur, ycur, search;
 	XARRAY trans;
 	REGISTER INTBIG j;
-	REGISTER INTBIG i, len, k, slx, shx, sly, shy, addtotal, lambda;
+	REGISTER INTBIG i, len, k, slx, shx, sly, shy, addtotal;
 	REGISTER GEOM *geom;
 	REGISTER NODEINST *ni;
 	REGISTER ARCINST *ai;
@@ -1270,84 +1248,11 @@ void us_findiup(void)
 			us_setcursorpos(NOWINDOWFRAME, xcur, ycur);
 			us_getslide(us_dragangle, us_dragox, us_dragoy, xcur, ycur, &us_dragx, &us_dragy);
 			if (us_dragx != us_dragox || us_dragy != us_dragoy)
+			{
 				us_manymove(us_dragobjectlist, us_dragnodelist, us_dragnodetotal,
 					us_dragx-us_dragox, us_dragy-us_dragoy);
-
-			/* adjust text that moved */
-			for(i=0; i<us_dragnumtext; i++)
-			{
-				(void)us_makehighlight(us_dragtextlist[i], &us_draghigh);
-				if ((us_draghigh.status&HIGHTYPE) != HIGHTEXT) continue;
-
-				/* get object and movement amount */
-				us_gethighaddrtype(&us_draghigh, &us_dragaddr, &us_dragtype);
-				dx = us_dragx - us_dragox;
-				dy = us_dragy - us_dragoy;
-
-				/* undraw the text */
-				if (us_dragtype == VPORTPROTO)
-				{
-					ni = ((PORTPROTO *)us_dragaddr)->subnodeinst;
-					startobjectchange((INTBIG)ni, VNODEINST);
-					if (ni->transpose != 0)
-						makeangle(ni->rotation, ni->transpose, trans); else
-							makeangle((INTSML)((3600-ni->rotation)%3600), 0, trans);
-					xform(dx, dy, &dx, &dy, trans);
-				} else
-				{
-					startobjectchange(us_dragaddr, us_dragtype);
-					if (us_dragtype == VNODEINST)
-					{
-						ni = (NODEINST *)us_dragaddr;
-						if (ni->proto == gen_invispinprim)
-						{
-							modifynodeinst(ni, dx, dy, dx, dy, 0, 0);
-							endobjectchange((INTBIG)ni, VNODEINST);
-							continue;
-						}
-						if (ni->transpose != 0)
-							makeangle(ni->rotation, ni->transpose, trans); else
-								makeangle((INTSML)((3600-ni->rotation)%3600), 0, trans);
-						xform(dx, dy, &dx, &dy, trans);
-					}
-				}
-				if (us_dragtype == VNODEPROTO && us_draghigh.fromvar != NOVARIABLE)
-					us_undrawfacetvariable(us_draghigh.fromvar, (NODEPROTO *)us_dragaddr);
-
-				/* set the new descriptor on the text */
-				lambda = el_curlib->lambda[us_hightech(&us_draghigh)->techindex];
-				dx = dx*4/lambda;   dy = dy*4/lambda;
-				us_dragdescript = us_gethighdescript(&us_draghigh);
-				if ((us_dragdescript&VTXOFFNEG) != 0) dx -= (us_dragdescript&VTXOFF) >> VTXOFFSH; else
-					dx += (us_dragdescript&VTXOFF) >> VTXOFFSH;
-				if ((us_dragdescript&VTYOFFNEG) != 0) dy -= (us_dragdescript&VTYOFF) >> VTYOFFSH; else
-					dy += (us_dragdescript&VTYOFF) >> VTYOFFSH;
-				us_dragdescript = us_setdescriptoffset(us_dragdescript, dx, dy);
-				us_modifytextdescript(&us_draghigh, us_dragdescript);
-
-				/* redisplay the text */
-				if (us_dragtype == VNODEPROTO && us_draghigh.fromvar != NOVARIABLE)
-					us_drawfacetvariable(us_draghigh.fromvar, (NODEPROTO *)us_dragaddr);
-				if (us_dragtype == VPORTPROTO)
-				{
-					endobjectchange((INTBIG)ni, VNODEINST);
-				} else
-				{
-					endobjectchange(us_dragaddr, us_dragtype);
-				}
-				(void)us_addhighlight(&us_draghigh);
-
-				/* modify all higher-level nodes if port moved */
-				if (us_draghigh.fromvar == NOVARIABLE && us_draghigh.fromport != NOPORTPROTO)
-				{
-					for(ni = us_draghigh.fromport->parent->firstinst; ni != NONODEINST; ni = ni->nextinst)
-					{
-						if ((ni->userbits&NEXPAND) != 0 &&
-							(us_draghigh.fromport->userbits&PORTDRAWN) == 0) continue;
-						startobjectchange((INTBIG)ni, VNODEINST);
-						endobjectchange((INTBIG)ni, VNODEINST);
-					}
-				}
+				us_moveselectedtext(us_dragtextcount, us_dragtexts,
+					us_dragx - us_dragox, us_dragy - us_dragoy);
 			}
 		}
 		if (us_dragnodetotal > 0) efree((char *)us_dragnodelist);
@@ -1406,7 +1311,7 @@ void us_findiup(void)
 				if (ni->proto->primindex != 0 &&
 					(ni->proto->userbits&NEDGESELECT) != 0)
 				{
-					i = nodepolys(ni, 0);
+					i = nodepolys(ni, 0, NOWINDOWPART);
 					makerot(ni, trans);
 					for(j=0; j<i; j++)
 					{
@@ -1430,7 +1335,7 @@ void us_findiup(void)
 				if ((ai->proto->userbits&AEDGESELECT) != 0)
 				{
 					ai = geom->entryaddr.ai;
-					i = arcpolys(ai);
+					i = arcpolys(ai, NOWINDOWPART);
 					if (poly == NOPOLYGON) poly = allocstaticpolygon(4, us_aid->cluster);
 					for(j=0; j<i; j++)
 					{
@@ -1450,6 +1355,7 @@ void us_findiup(void)
 			newhigh.fromgeom = geom;
 			newhigh.fromport = NOPORTPROTO;
 			newhigh.fromvar = NOVARIABLE;
+			newhigh.fromvarnoeval = NOVARIABLE;
 			newhigh.frompoint = 0;
 
 			/* add snapping information if requested */
@@ -1469,17 +1375,25 @@ void us_findiup(void)
 			if (geom->entrytype == OBJNODEINST &&
 				geom->entryaddr.ni->proto == gen_invispinprim)
 			{
-				/* if the pin has a displayable variable, highlight the text */
+				/* if the pin has a displayable variable or export, highlight the text */
 				for(i=0; i<geom->entryaddr.ni->numvar; i++)
 				{
 					var = &geom->entryaddr.ni->firstvar[i];
 					if ((var->type&VDISPLAY) != 0)
 					{
 						newhigh.fromvar = var;
+						newhigh.fromvarnoeval = NOVARIABLE;
 						newhigh.status = HIGHTEXT;
 						break;
 					}
 				}
+				if (geom->entryaddr.ni->firstportexpinst != NOPORTEXPINST)
+				{
+					newhigh.fromvar = NOVARIABLE;
+					newhigh.fromvarnoeval = NOVARIABLE;
+					newhigh.fromport = geom->entryaddr.ni->firstportexpinst->exportproto;
+					newhigh.status = HIGHTEXT;
+				}
 			}
 			(void)allocstring(&highlist[addtotal], us_makehighlightstring(&newhigh), el_tempcluster);
 			addtotal++;
@@ -1519,7 +1433,7 @@ void us_findiup(void)
 				(geom->entryaddr.ni->proto->userbits&NEDGESELECT) != 0)
 			{
 				ni = geom->entryaddr.ni;
-				i = nodepolys(ni, 0);
+				i = nodepolys(ni, 0, NOWINDOWPART);
 				if (poly == NOPOLYGON) poly = allocstaticpolygon(4, us_aid->cluster);
 				makerot(ni, trans);
 				for(j=0; j<i; j++)
@@ -1536,7 +1450,7 @@ void us_findiup(void)
 				(geom->entryaddr.ai->proto->userbits&AEDGESELECT) != 0)
 			{
 				ai = geom->entryaddr.ai;
-				i = arcpolys(ai);
+				i = arcpolys(ai, NOWINDOWPART);
 				if (poly == NOPOLYGON) poly = allocstaticpolygon(4, us_aid->cluster);
 				for(j=0; j<i; j++)
 				{
@@ -1555,6 +1469,7 @@ void us_findiup(void)
 			newhigh.fromgeom = geom;
 			newhigh.fromport = NOPORTPROTO;
 			newhigh.fromvar = NOVARIABLE;
+			newhigh.fromvarnoeval = NOVARIABLE;
 			newhigh.frompoint = 0;
 
 			/* add snapping information if requested */
@@ -1580,10 +1495,18 @@ void us_findiup(void)
 					if ((var->type&VDISPLAY) != 0)
 					{
 						newhigh.fromvar = var;
+						newhigh.fromvarnoeval = NOVARIABLE;
 						newhigh.status = HIGHTEXT;
 						break;
 					}
 				}
+				if (geom->entryaddr.ni->firstportexpinst != NOPORTEXPINST)
+				{
+					newhigh.fromvar = NOVARIABLE;
+					newhigh.fromvarnoeval = NOVARIABLE;
+					newhigh.fromport = geom->entryaddr.ni->firstportexpinst->exportproto;
+					newhigh.status = HIGHTEXT;
+				}
 			}
 
 			/* see if this object is already highlighted */
@@ -1591,8 +1514,19 @@ void us_findiup(void)
 			{
 				if (us_makehighlight(((char **)highvar->addr)[k], &oldhigh) != 0) continue;
 				if (oldhigh.facet != newhigh.facet) continue;
-				if ((oldhigh.status&HIGHTYPE) != HIGHFROM) continue;
-				if (oldhigh.fromgeom != newhigh.fromgeom) continue;
+				if ((oldhigh.status&HIGHTYPE) != (newhigh.status&HIGHTYPE)) continue;
+				if ((oldhigh.status&HIGHTYPE) == HIGHFROM)
+				{
+					if (oldhigh.fromgeom != newhigh.fromgeom) continue;
+				} else if ((oldhigh.status&HIGHTYPE) == HIGHTEXT)
+				{
+					if (oldhigh.fromvar != newhigh.fromvar) continue;
+					if (oldhigh.fromvar == NOVARIABLE)
+					{
+						if (oldhigh.fromgeom != newhigh.fromgeom) continue;
+						if (oldhigh.fromport != newhigh.fromport) continue;
+					}
+				}
 				break;
 			}
 			if (k < len) (void)us_delhighlight(&newhigh); else
@@ -1780,7 +1714,7 @@ INTSML us_createadown(INTBIG x, INTBIG y)
 	{
 		defaultnodesize(us_dragnodeproto, &pxs, &pys);
 		foundgeom = us_getclosest(us_lastcurx-us_dragoffx+pxs/2, us_lastcury-us_dragoffy+pys/2,
-			us_dragwindow->curnodeproto);
+			us_alignment/2, us_dragwindow->curnodeproto);
 
 		/* determine closest port if a node was found */
 		if (foundgeom != NOGEOM && foundgeom->entrytype == OBJNODEINST)
@@ -2048,12 +1982,11 @@ void us_multidragup(void)
 void us_multidragdraw(INTBIG col, INTBIG dx, INTBIG dy)
 {
 	REGISTER INTSML i, thisend, portcount, endfixed;
-	REGISTER INTBIG wid, lambda, xcur, ycur, xw, yw, len;
+	REGISTER INTBIG wid, xw, yw, len;
 	INTBIG lx, ly, hx, hy, xc, yc, tlx, tly, thx, thy;
 	XARRAY trans;
 	INTSML tsx, tsy, j;
 	REGISTER char *str;
-	REGISTER VARIABLE *var;
 	REGISTER NODEINST *ni;
 	REGISTER ARCINST *ai;
 	REGISTER PORTARCINST *pi;
@@ -2067,67 +2000,12 @@ void us_multidragdraw(INTBIG col, INTBIG dx, INTBIG dy)
 		if (us_dragobjectlist[i]->entrytype != OBJNODEINST) continue;
 		ni = us_dragobjectlist[i]->entryaddr.ni;
 
-		/* if node is an invisible pin with text, show the text highlight */
+		/* if node is an invisible pin with text, ignore the pin */
 		if (ni->proto == gen_invispinprim)
 		{
-			var = NOVARIABLE;
 			for(j=0; j<ni->numvar; j++)
-			{
-				var = &ni->firstvar[j];
-				if ((var->type&VDISPLAY) != 0) break;
-				var = NOVARIABLE;
-			}
-			if (var != NOVARIABLE)
-			{
-				/* determine number of lines of text and text size */
-				str = describevariable(var, -1, -1);
-				len = 1;
-				if ((var->type&VISARRAY) != 0) len = (INTSML)getlength(var);
-				if (len > 1)
-				{
-					xw = ni->geom->highx - ni->geom->lowx;
-					yw = ni->geom->highy - ni->geom->lowy;
-				} else
-				{
-					screensettextsize(el_curwindowpart, truefontsize((INTSML)((var->textdescript&VTSIZE)>>VTSIZESH),
-						el_curwindowpart, ni->proto->tech));
-					screengettextsize(el_curwindowpart, str, &tsx, &tsy);
-					xw = muldiv(tsx, el_curwindowpart->screenhx-el_curwindowpart->screenlx,
-						el_curwindowpart->usehx-el_curwindowpart->uselx);
-					yw = muldiv(tsy, el_curwindowpart->screenhy-el_curwindowpart->screenly,
-						el_curwindowpart->usehy-el_curwindowpart->usely);
-				}
-
-				/* get center of highlight, and also a style */
-				us_dragpoly->count = 1;
-				us_dragpoly->style = FILLED;
-				us_dragpoly->xv[0] = (ni->lowx + ni->highx) / 2;
-				us_dragpoly->yv[0] = (ni->lowy + ni->highy) / 2;
-				adjustdisoffset((INTBIG)ni, VNODEINST, ni->proto->tech, us_dragpoly, var->textdescript);
-				makerot(ni, trans);
-				xformpoly(us_dragpoly, trans);
-				getcenter(us_dragpoly, &xc, &yc);
-
-				/* clip range of text descriptor */
-				lambda = figurelambda(ni->geom);
-				xcur = (xc + dx) * 4 / lambda;
-				xc += dx;
-				if (xcur > 1023) xc = 1023 * lambda / 4; else
-					if (xcur < -1023) xc = -1023 * lambda / 4;
-				ycur = (yc + dy) * 4 / lambda;
-				yc += dy;
-				if (ycur > 1023) yc = 1023 * lambda / 4; else
-					if (ycur < -1023) yc = -1023 * lambda / 4;
-
-				/* draw the descriptor */
-				us_buildtexthighpoly(ni->geom->lowx, ni->geom->highx,
-					ni->geom->lowy, ni->geom->highy, xc, yc, xw, yw,
-						us_dragpoly->style, us_dragpoly);
-
-				/* draw the text highlight */
-				(void)us_showpoly(us_dragpoly, us_dragwindow);
-				continue;
-			}
+				if ((ni->firstvar[j].type&VDISPLAY) != 0) break;
+			if (j < ni->numvar) continue;
 		}
 
 		makerot(ni, trans);
@@ -2246,9 +2124,9 @@ void us_multidragdraw(INTBIG col, INTBIG dx, INTBIG dy)
 	}
 
 	/* update selected text, too */
-	for(i=0; i<us_dragnumtext; i++)
+	for(i=0; i<us_dragtextcount; i++)
 	{
-		(void)us_makehighlight(us_dragtextlist[i], &us_draghigh);
+		(void)us_makehighlight(us_dragtexts[i], &us_draghigh);
 		if ((us_draghigh.status&HIGHTYPE) != HIGHTEXT) continue;
 
 		/* get object and extent */
@@ -2270,9 +2148,8 @@ void us_multidragdraw(INTBIG col, INTBIG dx, INTBIG dy)
 		{
 			str = us_gethighstring(&us_draghigh);
 			us_dragdescript = us_gethighdescript(&us_draghigh);
-			screensettextsize(el_curwindowpart,
-				truefontsize((INTSML)((us_dragdescript&VTSIZE)>>VTSIZESH),
-					el_curwindowpart, tech));
+			screensettextsize(el_curwindowpart, truefontsize((us_dragdescript&VTSIZE)>>VTSIZESH,
+				el_curwindowpart, tech));
 			screengettextsize(el_curwindowpart, str, &tsx, &tsy);
 			xw = muldiv(tsx, el_curwindowpart->screenhx-el_curwindowpart->screenlx,
 				el_curwindowpart->usehx-el_curwindowpart->uselx);
@@ -2365,7 +2242,7 @@ INTSML us_distancedown(INTBIG x, INTBIG y)
 	/* draw crosses at the end */
 	us_highl.col = HIGHLIT;
 	us_lastcurx = us_dragx;   us_lastcury = us_dragy;
-	us_dragpoly->font = TXTMEDIUM;
+	us_dragpoly->font = TXT2L;
 	us_dragpoly->tech = el_curtech;
 	dist = computedistance(us_dragox, us_dragoy, us_dragx, us_dragy);
 	(void)strcpy(us_dragmessage, _("Distance: "));
