@@ -37,7 +37,7 @@
 
 /***** REDRAW MODULES *****/
 
-#define	NOREDRAW ((REDRAW *)-1)
+#define NOREDRAW ((REDRAW *)-1)
 
 typedef struct Iredraw
 {
@@ -103,8 +103,8 @@ POLY3D *us_3dgetnextpoly(INTBIG count);
 #define ZUP(x)       ((x) << ZSHIFT)
 #define ZDN(x)       (((x) + 1) >> ZSHIFT)
 
-#define	CROSSSIZE      2				/* size in pixels of half a cross */
-#define	BIGCROSSSIZE   5				/* size in pixels of half a big cross */
+#define CROSSSIZE      2				/* size in pixels of half a cross */
+#define BIGCROSSSIZE   5				/* size in pixels of half a big cross */
 #define MINDISCSIZE    1				/* min radius of a disc */
 
 extern GRAPHICS us_ebox;
@@ -116,7 +116,7 @@ INTSML us_showthispoly(POLYGON*, WINDOWPART*);
 void us_wanttodraw(INTBIG, INTBIG, INTBIG, INTBIG, WINDOWPART*, GRAPHICS*, INTSML);
 void us_wanttodrawo(INTBIG, INTSML, INTBIG, INTSML, INTBIG, INTSML, INTBIG, INTSML, WINDOWPART*, GRAPHICS*, INTSML);
 void us_drawextendedpolyline(POLYGON*, WINDOWPART*);
-INTSML us_writetext(INTBIG, INTBIG, INTBIG, INTBIG, GRAPHICS*, char*, INTSML, WINDOWPART*);
+INTSML us_writetext(INTBIG, INTBIG, INTBIG, INTBIG, GRAPHICS*, char*, INTBIG, WINDOWPART*);
 void us_reverse(POLYGON*);
 
 /*
@@ -191,10 +191,10 @@ void us_erasewindow(WINDOWPART *w)
  * create a window for this facet, otherwise reuse the current one.
  */
 void us_switchtofacet(NODEPROTO *cur, INTBIG lx, INTBIG hx, INTBIG ly, INTBIG hy,
-	NODEINST *focusinst, PORTPROTO *facetinstport, INTSML newframe, INTSML pushpop)
+	NODEINST *focusinst, PORTPROTO *facetinstport, INTBIG newframe, INTBIG pushpop)
 {
 	HIGHLIGHT newhigh;
-	REGISTER INTSML i, l, resethandlers, turnoffsim;
+	REGISTER INTBIG i, l, resethandlers, turnoffsim;
 	INTSML dummy;
 	REGISTER VARIABLE *var;
 	char *one[1];
@@ -247,6 +247,7 @@ void us_switchtofacet(NODEPROTO *cur, INTBIG lx, INTBIG hx, INTBIG ly, INTBIG hy
 		oldstate = DISPWINDOW;
 		location = "Entire";
 	}
+	el_curwindowpart = neww;
 
 	if ((cur->cellview->viewstate&TEXTVIEW) != 0)
 	{
@@ -356,6 +357,7 @@ void us_switchtofacet(NODEPROTO *cur, INTBIG lx, INTBIG hx, INTBIG ly, INTBIG hy
 			newhigh.fromport = facetinstport;
 			newhigh.frompoint = 0;
 			newhigh.fromvar = NOVARIABLE;
+			newhigh.fromvarnoeval = NOVARIABLE;
 			(void)us_addhighlight(&newhigh);
 		}
 	}
@@ -1008,17 +1010,17 @@ INTSML us_showpoly(POLYGON *obj, WINDOWPART *w)
 				us_wanttodraw(obj->xv[i-1], obj->yv[i-1], obj->xv[i], obj->yv[i], w, gra, 0);
 			return(0);
 
-		case OPENEDT1:		/* opened polygon outline, texture 1 */
+		case OPENEDT1:		/* opened polygon outline, dotted */
 			for(i=1; i<obj->count; i++)
 				us_wanttodraw(obj->xv[i-1], obj->yv[i-1], obj->xv[i], obj->yv[i], w, gra, 1);
 			return(0);
 
-		case OPENEDT2:		/* opened polygon outline, texture 2 */
+		case OPENEDT2:		/* opened polygon outline, dashed */
 			for(i=1; i<obj->count; i++)
 				us_wanttodraw(obj->xv[i-1], obj->yv[i-1], obj->xv[i], obj->yv[i], w, gra, 2);
 			return(0);
 
-		case OPENEDT3:		/* opened polygon outline, texture 3 */
+		case OPENEDT3:		/* opened polygon outline, thicker */
 			for(i=1; i<obj->count; i++)
 				us_wanttodraw(obj->xv[i-1], obj->yv[i-1], obj->xv[i], obj->yv[i], w, gra, 3);
 			return(0);
@@ -1059,7 +1061,7 @@ INTSML us_showpoly(POLYGON *obj, WINDOWPART *w)
 			ux = applyxscale(w, ux-w->screenlx) + w->uselx;
 			uy = applyyscale(w, uy-w->screenly) + w->usely;
 			i = truefontsize(obj->font, w, obj->tech);
-			screensettextsize(w, (INTSML)i);
+			screensettextsize(w, i);
 			screengettextsize(w, obj->string, &tsx, &tsy);
 			switch (obj->style)
 			{
@@ -1301,17 +1303,26 @@ void us_drawextendedpolyline(POLYGON *obj, WINDOWPART *w)
  * will fit.
  */
 INTSML us_writetext(INTBIG lx, INTBIG ux, INTBIG ly, INTBIG uy, GRAPHICS *desc, char *txt,
-	INTSML initialfont, WINDOWPART *win)
+	INTBIG initialfont, WINDOWPART *win)
 {
-	REGISTER INTSML stop, save, font;
+	REGISTER INTBIG stop, save, font, xfont, yfont;
 	INTSML six, siy;
 
 	/* scan for a font that fits */
-	for(font=initialfont; font>=TXT4P; font--)
+	if (initialfont < 4) initialfont = 4;
+	for(font = initialfont; font >= 4; font--)
 	{
 		screensettextsize(win, font);
 		screengettextsize(win, txt, &six, &siy);
 		if (six <= ux-lx && siy <= uy-ly) break;
+
+		/* jump quickly to the proper font size */
+		if (six <= ux-lx) xfont = font; else
+			xfont = font * (ux-lx) / six;
+		if (siy <= uy-ly) yfont = font; else
+			yfont = font * (uy-ly) / siy;
+		font = mini(xfont, yfont);
+		if (font < 4) font = 4;
 	}
 
 	/* if the text doesn't fit in Y, quit */
@@ -1776,6 +1787,12 @@ void us_3dbuttonhandler(WINDOWPART *w, INTSML but, INTSML x, INTSML y)
 {
 	REGISTER XFORM3D *xf3;
 
+	/* changes to the mouse-wheel are handled by the user interface */
+	if (wheelbutton(but) != 0)
+	{
+		us_buttonhandler(w, but, x, y);
+		return;
+	}
 	xf3 = &w->xf3;
 	us_3dlastbutx = us_3dinitialbutx = x;
 	us_3dlastbuty = us_3dinitialbuty = y;
@@ -2081,6 +2098,7 @@ POLY3D *us_3dgetnextpoly(INTBIG count)
 	{
 		newtotal = us_3dpolytotal * 2;
 		if (newtotal <= 0) newtotal = 50;
+		if (us_3dpolycount > newtotal) newtotal = us_3dpolycount;
 		newlist = (POLY3D **)emalloc(newtotal * (sizeof (POLY3D *)), us_aid->cluster);
 		if (newlist == 0) return(0);
 		for(i=0; i<us_3dpolytotal; i++)
