@@ -38,10 +38,11 @@
 #include "vhdl.h"
 #include "sim.h"
 #include "eio.h"
+#include "network.h"
 #include "tecschem.h"
 #include "edialogs.h"
 
-#define	MAXINPUTS	30		/* maximum inputs to logic gate */
+#define MAXINPUTS	30		/* maximum inputs to logic gate */
 
 /* special codes during VHDL generation */
 #define BLOCKNORMAL   0		/* ordinary block */
@@ -143,6 +144,7 @@ static VKEYWORD vhdl_keywords[] =
 	{"in",				KEY_IN},
 	{"inout",			KEY_INOUT},
 	{"is",				KEY_IS},
+	{"library",			KEY_LIBRARY},
 	{"linkage",			KEY_LINKAGE},
 	{"loop",			KEY_LOOP},
 #ifndef VHDL50
@@ -206,23 +208,6 @@ void              *vhdl_componentarray = 0;
 void              *vhdl_stringarray;
 UNRESLIST         *vhdl_unresolved_list = 0;
 
-/* VHDL Options */
-DIALOGITEM vhdl_optionsdialogitems[] =
-{
- /*  1 */ {0, {336,228,360,292}, BUTTON, N_("OK")},
- /*  2 */ {0, {300,228,324,292}, BUTTON, N_("Cancel")},
- /*  3 */ {0, {308,4,324,166}, CHECK, N_("VHDL stored in facet")},
- /*  4 */ {0, {332,4,348,179}, CHECK, N_("Netlist stored in facet")},
- /*  5 */ {0, {28,8,224,295}, SCROLL, ""},
- /*  6 */ {0, {236,4,252,179}, MESSAGE, N_("VHDL for primitive:")},
- /*  7 */ {0, {260,4,276,179}, MESSAGE, N_("VHDL for negated primitive:")},
- /*  8 */ {0, {236,184,252,295}, EDITTEXT, ""},
- /*  9 */ {0, {260,184,276,295}, EDITTEXT, ""},
- /* 10 */ {0, {8,8,24,183}, MESSAGE, N_("Schematics primitives:")},
- /* 11 */ {0, {292,8,293,295}, DIVIDELINE, ""}
-};
-DIALOG vhdl_optionsdialog = {{50,75,419,379}, N_("VHDL Options"), 0, 11, vhdl_optionsdialogitems};
-
 /* prototypes for local routines */
 void      vhdl_addportlist(NODEINST*, NODEPROTO*, INTSML);
 void      vhdl_addrealports(NODEINST *ni, INTSML special);
@@ -302,7 +287,7 @@ INTSML vhdl_set(INTSML count, char *par[])
 			}
 			if (np->primindex != 0)
 			{
-				ttyputerr(_("Can only convert facets to VHDL, not primitives"));
+				ttyputerr(M_("Can only convert facets to VHDL, not primitives"));
 				return(1);
 			}
 		} else
@@ -346,42 +331,42 @@ INTSML vhdl_set(INTSML count, char *par[])
 			ttyputerr(_("Cannot find library %s"), par[1]);
 			return(0);
 		}
-		ttyputverbose(_("Library %s will be used for behavioral descriptions"), par[1]);
+		ttyputverbose(M_("Library %s will be used for behavioral descriptions"), par[1]);
 		return(0);
 	}
 
 	if (namesamen(pp, "vhdl-on-disk", l) == 0)
 	{
 		setvalkey((INTBIG)vhdl_aid, VAID, vhdl_vhdlondisk, 1, VINTEGER);
-		ttyputverbose(_("VHDL will be kept in separate disk files"));
+		ttyputverbose(M_("VHDL will be kept in separate disk files"));
 		return(0);
 	}
 
 	if (namesamen(pp, "netlist-on-disk", l) == 0)
 	{
 		setvalkey((INTBIG)vhdl_aid, VAID, vhdl_netlistondisk, 1, VINTEGER);
-		ttyputverbose(_("Netlists will be kept in separate disk files"));
+		ttyputverbose(M_("Netlists will be kept in separate disk files"));
 		return(0);
 	}
 
 	if (namesamen(pp, "warn", l) == 0)
 	{
 		vhdl_warnflag = TRUE;
-		ttyputverbose(_("VHDL compiler will display warnings"));
+		ttyputverbose(M_("VHDL compiler will display warnings"));
 		return(0);
 	}
 
 	if (namesamen(pp, "external", l) == 0)
 	{
 		vhdl_externentities = TRUE;
-		ttyputverbose(_("VHDL compiler will allow external references"));
+		ttyputverbose(M_("VHDL compiler will allow external references"));
 		return(0);
 	}
 
 	if (namesamen(pp, "generate-equations", l) == 0)
 	{
 		vhdl_genequations = TRUE;
-		ttyputverbose(_("VHDL generation will use equations instead of logic block calls"));
+		ttyputverbose(M_("VHDL generation will use equations instead of logic block calls"));
 		return(0);
 	}
 
@@ -396,31 +381,31 @@ INTSML vhdl_set(INTSML count, char *par[])
 		if (namesamen(pp, "warn", l) == 0)
 		{
 			vhdl_warnflag = FALSE;
-			ttyputverbose(_("VHDL compiler will not display warnings"));
+			ttyputverbose(M_("VHDL compiler will not display warnings"));
 			return(0);
 		}
 		if (namesamen(pp, "external", l) == 0)
 		{
 			vhdl_externentities = FALSE;
-			ttyputverbose(_("VHDL compiler will not allow external references"));
+			ttyputverbose(M_("VHDL compiler will not allow external references"));
 			return(0);
 		}
 		if (namesamen(pp, "vhdl-on-disk", l) == 0)
 		{
 			setvalkey((INTBIG)vhdl_aid, VAID, vhdl_vhdlondisk, 0, VINTEGER);
-			ttyputverbose(_("VHDL will be kept in facets"));
+			ttyputverbose(M_("VHDL will be kept in facets"));
 			return(0);
 		}
 		if (namesamen(pp, "netlist-on-disk", l) == 0)
 		{
 			setvalkey((INTBIG)vhdl_aid, VAID, vhdl_netlistondisk, 0, VINTEGER);
-			ttyputverbose(_("Netlists will be kept in facets"));
+			ttyputverbose(M_("Netlists will be kept in facets"));
 			return(0);
 		}
 		if (namesamen(pp, "generate-equations", l) == 0)
 		{
 			vhdl_genequations = FALSE;
-			ttyputverbose(_("VHDL generation will use logic block calls instead of equations"));
+			ttyputverbose(M_("VHDL generation will use logic block calls instead of equations"));
 			return(0);
 		}
 		ttyputbadusage("tellaid vhdl-compiler not");
@@ -438,31 +423,31 @@ INTSML vhdl_set(INTSML count, char *par[])
 		if (namesamen(pp, "netlisp", l) == 0)
 		{
 			vhdl_target = TARGET_NETLISP;
-			ttyputverbose(_("VHDL compiles to net lisp"));
+			ttyputverbose(M_("VHDL compiles to net lisp"));
 			return(0);
 		}
 		if (namesamen(pp, "als", l) == 0 && l >= 1)
 		{
 			vhdl_target = TARGET_ALS;
-			ttyputverbose(_("VHDL compiles to ALS simulation"));
+			ttyputverbose(M_("VHDL compiles to ALS simulation"));
 			return(0);
 		}
 		if (namesamen(pp, "quisc", l) == 0 && l >= 3)
 		{
 			vhdl_target = TARGET_QUISC;
-			ttyputverbose(_("VHDL compiles to QUISC place-and-route"));
+			ttyputverbose(M_("VHDL compiles to QUISC place-and-route"));
 			return(0);
 		}
 		if (namesamen(pp, "silos", l) == 0)
 		{
 			vhdl_target = TARGET_SILOS;
-			ttyputverbose(_("VHDL compiles to SILOS simulation"));
+			ttyputverbose(M_("VHDL compiles to SILOS simulation"));
 			return(0);
 		}
 		if (namesamen(pp, "rsim", l) == 0)
 		{
 			vhdl_target = TARGET_RSIM;
-			ttyputverbose(_("VHDL compiles to RSIM simulation"));
+			ttyputverbose(M_("VHDL compiles to RSIM simulation"));
 			return(0);
 		}
 		ttyputbadusage("tellaid vhdl-compiler output-format");
@@ -508,10 +493,10 @@ void vhdl_slice(void)
  */
 INTBIG vhdl_request(char *command, va_list ap)
 {
-	INTBIG i;
+	INTBIG i, len;
 	INTSML vhdlondisk, netlistondisk;
 	UINTBIG netlistdate, vhdldate;
-	char *intended;
+	char *intended, *desired, buffer[MAXVHDLLINE];
 	REGISTER VARIABLE *var;
 	REGISTER INTBIG arg1, arg2, arg3;
 	REGISTER NODEPROTO *np, *orignp;
@@ -595,6 +580,20 @@ INTBIG vhdl_request(char *command, va_list ap)
 					&vhdldate) == 0)
 				{
 					if (netlistdate < vhdldate) i = 1;
+
+					/* if original is not VHDL, ensure that VHDL comes from original */
+					if (orignp->cellview != el_vhdlview)
+					{
+						if (vhdl_getnextline(buffer) != 0)
+						{
+							desired = "-- VHDL automatically generated from facet ";
+							len = strlen(desired);
+							if (strncmp(buffer, desired, len) == 0)
+							{
+								if (strcmp(&buffer[len], describenodeproto(orignp)) != 0) i = 1;
+							}
+						}
+					}
 				}
 				vhdl_endinput();
 
@@ -617,6 +616,20 @@ INTBIG vhdl_request(char *command, va_list ap)
 		{
 			/* no valid netlist: look for VHDL */
 			i = vhdl_startinput(orignp, io_filetypevhdl, vhdlondisk, &intended, &vhdldate);
+
+			/* if original is not VHDL, ensure that VHDL comes from original */
+			if (i == 0 && orignp->cellview != el_vhdlview)
+			{
+				if (vhdl_getnextline(buffer) != 0)
+				{
+					desired = "-- VHDL automatically generated from facet ";
+					len = strlen(desired);
+					if (strncmp(buffer, desired, len) == 0)
+					{
+						if (strcmp(&buffer[len], describenodeproto(orignp)) != 0) i = 1;
+					}
+				}
+			}
 			vhdl_endinput();
 			if (i == 0)
 			{
@@ -1051,14 +1064,15 @@ INTSML vhdl_generatevhdl(NODEPROTO *np)
 {
 	REGISTER NODEINST *ni;
 	REGISTER NODEPROTO *onp, *cnp;
-	REGISTER INTSML first, i, j, instnum, linelen, thisend, gotinverters, backannotate;
+	REGISTER INTBIG count, first, i, j, instnum, linelen, thisend, gotinverters;
+	REGISTER INTSML backannotate;
 	INTBIG componentcount;
 	INTSML special;
 	REGISTER NETWORK *net;
 	REGISTER ARCINST *ai;
 	REGISTER VARIABLE *var;
 	char line[30], *pt, gotnand[MAXINPUTS+1], gotxnor[MAXINPUTS+1],
-		gotnor[MAXINPUTS+1], **componentlist;
+		gotnor[MAXINPUTS+1], **componentlist, **strings;
 
 	/* make sure that all nodes have names on them */
 	backannotate = 0;
@@ -1246,29 +1260,33 @@ INTSML vhdl_generatevhdl(NODEPROTO *np)
 	first = 0;
 	for(net = np->firstnetwork; net != NONETWORK; net = net->nextnetwork)
 	{
-		if (net->portcount != 0) continue;
-		if (first == 0)
-		{
-			(void)initinfstr();
-			(void)addstringtoinfstr("  signal ");
-			linelen = 9;
-		} else
-		{
-			(void)addstringtoinfstr(", ");
-			linelen += 2;
-		}
-		first = 1;
+		if (net->portcount != 0 || net->buslinkcount != 0) continue;
 		if (net->namecount > 0) pt = net->netname; else
 			pt = describenetwork(net);
-		if (linelen + strlen(pt) > 80)
+		count = net_evalbusname(APBUS, pt, &strings, NOARCINST, np, 0);
+		for(i=0; i<count; i++)
 		{
-			vhdl_print(returninfstr());
-			(void)initinfstr();
-			(void)addstringtoinfstr("    ");
-			linelen = 4;
+			if (first == 0)
+			{
+				(void)initinfstr();
+				(void)addstringtoinfstr("  signal ");
+				linelen = 9;
+			} else
+			{
+				(void)addstringtoinfstr(", ");
+				linelen += 2;
+			}
+			first = 1;
+			if (linelen + strlen(strings[i]) > 80)
+			{
+				vhdl_print(returninfstr());
+				(void)initinfstr();
+				(void)addstringtoinfstr("    ");
+				linelen = 4;
+			}
+			vhdl_addstring(strings[i]);
+			linelen += strlen(strings[i]);
 		}
-		vhdl_addstring(pt);
-		linelen += strlen(pt);
 	}
 	if (first != 0)
 	{
@@ -1545,6 +1563,8 @@ void vhdl_addrealports(NODEINST *ni, INTSML special)
 			/* port is not connected or an export */
 			if (first != 0) (void)addstringtoinfstr(", ");   first = 1;
 			(void)addstringtoinfstr("open");
+			ttyputmsg(_("Warning: port %s of node %s is not connected"),
+				cpp->protoname, describenodeinst(ni));
 		}
 	}
 }
@@ -1568,9 +1588,8 @@ void vhdl_addrealports(NODEINST *ni, INTSML special)
  */
 char *vhdl_primname(NODEINST *ni, INTSML *special)
 {
-	REGISTER INTSML k, inport, isneg;
+	REGISTER INTBIG k, inport, isneg;
 	static INTBIG tech_vhdl_names = 0;
-	char *dummy;
 	REGISTER char *str, *ptr;
 	REGISTER VARIABLE *var;
 	REGISTER PORTARCINST *pi;
@@ -1587,39 +1606,57 @@ char *vhdl_primname(NODEINST *ni, INTSML *special)
 	if (ni->proto->primindex == 0) return(ni->proto->cell->cellname);
 
 	/* get the primitive function */
-	k = nodefunction(ni, &dummy);
+	k = nodefunction(ni, 0);
 	pt[0] = 0;
 	switch (k)
 	{
-		case NPTRANMOS:   (void)strcpy(pt, "nMOStran");   *special = BLOCKMOSTRAN;   break;
-		case NPTRADMOS:   (void)strcpy(pt, "DMOStran");   *special = BLOCKMOSTRAN;   break;
-		case NPTRAPMOS:   (void)strcpy(pt, "PMOStran");   *special = BLOCKMOSTRAN;   break;
-		case NPTRANPN:    (void)strcpy(pt, "NPNtran");    break;
-		case NPTRAPNP:    (void)strcpy(pt, "PNPtran");    break;
-		case NPTRANJFET:  (void)strcpy(pt, "NJFET");      break;
-		case NPTRAPJFET:  (void)strcpy(pt, "PJFET");      break;
-		case NPTRADMES:   (void)strcpy(pt, "DMEStran");   break;
-		case NPTRAEMES:   (void)strcpy(pt, "EMEStran");   break;
-		case NPTRANE2L:   (void)strcpy(pt, "E2Ltran");    break;
+		case NPTRANMOS:   case NPTRA4NMOS:
+			var = getval((INTBIG)ni, VNODEINST, -1, "SIM_weak_node");
+			if (var == NOVARIABLE) (void)strcpy(pt, "nMOStran"); else
+				(void)strcpy(pt, "nMOStranWeak");
+			*special = BLOCKMOSTRAN;
+			break;
+		case NPTRADMOS:   case NPTRA4DMOS:
+			(void)strcpy(pt, "DMOStran");
+			*special = BLOCKMOSTRAN;
+			break;
+		case NPTRAPMOS:   case NPTRA4PMOS:
+			var = getval((INTBIG)ni, VNODEINST, -1, "SIM_weak_node");
+			if (var == NOVARIABLE) (void)strcpy(pt, "PMOStran"); else
+				(void)strcpy(pt, "pMOStranWeak");
+			*special = BLOCKMOSTRAN;
+			break;
+		case NPTRANPN:    case NPTRA4NPN:
+			(void)strcpy(pt, "NPNtran");
+			break;
+		case NPTRAPNP:    case NPTRA4PNP:
+			(void)strcpy(pt, "PNPtran");
+			break;
+		case NPTRANJFET:  case NPTRA4NJFET:
+			(void)strcpy(pt, "NJFET");
+			break;
+		case NPTRAPJFET:  case NPTRA4PJFET:
+			(void)strcpy(pt, "PJFET");
+			break;
+		case NPTRADMES:   case NPTRA4DMES:
+			(void)strcpy(pt, "DMEStran");
+			break;
+		case NPTRAEMES:   case NPTRA4EMES:
+			(void)strcpy(pt, "EMEStran");
+			break;
 
 		case NPFLIPFLOP:
-			var = getvalkey((INTBIG)ni, VNODEINST, VSTRING, sch_flipfloptypekey);
-			if (var != NOVARIABLE)
+			switch (ni->userbits&FFTYPE)
 			{
-				if (namesamen((char *)var->addr, "rs", 2) == 0)
-				{
+				case FFTYPERS:
 					(void)strcpy(pt, "rsff");
 					*special = BLOCKFLOP;
 					break;
-				}
-				if (namesamen((char *)var->addr, "jk", 2) == 0)
-				{
+				case FFTYPEJK:
 					(void)strcpy(pt, "jkff");
 					*special = BLOCKFLOP;
 					break;
-				}
-				if (namesamen((char *)var->addr, "d", 1) == 0)
-				{
+				case FFTYPED:
 					(void)strcpy(pt, "dsff");
 					*special = BLOCKFLOPDS;
 					for(pi = ni->firstportarcinst; pi != NOPORTARCINST; pi = pi->nextportarcinst)
@@ -1632,9 +1669,7 @@ char *vhdl_primname(NODEINST *ni, INTSML *special)
 						}
 					}
 					break;
-				}
-				if (namesamen((char *)var->addr, "t", 1) == 0)
-				{
+				case FFTYPET:
 					(void)strcpy(pt, "tsff");
 					*special = BLOCKFLOPTS;
 					for(pi = ni->firstportarcinst; pi != NOPORTARCINST; pi = pi->nextportarcinst)
@@ -1647,10 +1682,11 @@ char *vhdl_primname(NODEINST *ni, INTSML *special)
 						}
 					}
 					break;
-				}
+				default:
+					(void)strcpy(pt, "fflop");
+					*special = BLOCKFLOP;
+					break;
 			}
-			(void)strcpy(pt, "fflop");
-			*special = BLOCKFLOP;
 			break;
 
 		case NPBUFFER:
@@ -1658,7 +1694,7 @@ char *vhdl_primname(NODEINST *ni, INTSML *special)
 			if (var == NOVARIABLE) str = "buffer/inverter"; else
 				str = ((char **)var->addr)[ni->proto->primindex-1];
 			*special = BLOCKBUFFER;
-			for(pi=ni->firstportarcinst; pi != NOPORTARCINST; pi = pi->nextportarcinst)
+			for(pi = ni->firstportarcinst; pi != NOPORTARCINST; pi = pi->nextportarcinst)
 			{
 				if (strcmp(pi->proto->protoname, "y") != 0) continue;
 				ai = pi->conarcinst;
@@ -1825,6 +1861,9 @@ void vhdl_addstring(char *orig)
 	/* check for VHDL keyword clashes */
 	len = strlen(orig);
 	if (vhdl_iskeyword(orig, len) != NOVKEYWORD) (void)addstringtoinfstr("NV");
+
+	/* "bit" isn't a keyword, but the compiler can't handle it */
+	if (namesame(orig, "bit") == 0) (void)addstringtoinfstr("NV");
 }
 
 /*
@@ -1906,7 +1945,8 @@ void vhdl_addtheseports(NODEINST *ni, NODEPROTO *np, INTBIG bits, char *before)
 	REGISTER PORTPROTO *pp;
 	REGISTER PORTARCINST *pi;
 	REGISTER INTSML didsome, inst;
-	char instname[10];
+	REGISTER INTBIG i, count;
+	char instname[10], **strings;
 
 	didsome = 0;
 	for(pp = np->firstportproto; pp != NOPORTPROTO; pp = pp->nextportproto)
@@ -1927,19 +1967,28 @@ void vhdl_addtheseports(NODEINST *ni, NODEPROTO *np, INTBIG bits, char *before)
 			for(pi = ni->firstportarcinst; pi != NOPORTARCINST; pi = pi->nextportarcinst)
 			{
 				if (pi->proto != pp) continue;
-				(void)addstringtoinfstr(before);
-				(void)strcpy(before, ", ");
-				vhdl_addstring(pp->protoname);
-				(void)sprintf(instname, "%d", inst++);
-				(void)addstringtoinfstr(instname);
-				didsome++;
+				count = net_evalbusname(APBUS, pp->protoname, &strings, NOARCINST, np, 0);
+				for(i=0; i<count; i++)
+				{
+					(void)addstringtoinfstr(before);
+					(void)strcpy(before, ", ");
+					vhdl_addstring(strings[i]);
+					(void)sprintf(instname, "%d", inst);
+					(void)addstringtoinfstr(instname);
+					didsome++;
+				}
+				inst++;
 			}
 		} else
 		{
-			(void)addstringtoinfstr(before);
-			(void)strcpy(before, ", ");
-			vhdl_addstring(pp->protoname);
-			didsome++;
+			count = net_evalbusname(APBUS, pp->protoname, &strings, NOARCINST, np, 0);
+			for(i=0; i<count; i++)
+			{
+				(void)addstringtoinfstr(before);
+				(void)strcpy(before, ", ");
+				vhdl_addstring(strings[i]);
+				didsome++;
+			}
 		}
 	}
 	if (didsome != 0)
@@ -2879,14 +2928,30 @@ void vhdl_unresolved(IDENTTABLE *iname, UNRESLIST **lstart)
 
 /****************************** VHDL OPTIONS DIALOG ******************************/
 
-/*
- * special case for the "VHDL Options" dialog
- * VHDL stored in facet     = 3 (check)
- * Netlist stored in facet  = 4 (check)
- * List of schematics prims = 5 (scroll)
- * Unnegated name           = 8 (edit text)
- * Negated name             = 9 (edit text)
- */
+/* VHDL Options */
+DIALOGITEM vhdl_optionsdialogitems[] =
+{
+ /*  1 */ {0, {336,228,360,292}, BUTTON, N_("OK")},
+ /*  2 */ {0, {300,228,324,292}, BUTTON, N_("Cancel")},
+ /*  3 */ {0, {308,4,324,166}, CHECK, N_("VHDL stored in facet")},
+ /*  4 */ {0, {332,4,348,179}, CHECK, N_("Netlist stored in facet")},
+ /*  5 */ {0, {28,8,224,295}, SCROLL, ""},
+ /*  6 */ {0, {236,4,252,179}, MESSAGE, N_("VHDL for primitive:")},
+ /*  7 */ {0, {260,4,276,179}, MESSAGE, N_("VHDL for negated primitive:")},
+ /*  8 */ {0, {236,184,252,295}, EDITTEXT, ""},
+ /*  9 */ {0, {260,184,276,295}, EDITTEXT, ""},
+ /* 10 */ {0, {8,8,24,183}, MESSAGE, N_("Schematics primitives:")},
+ /* 11 */ {0, {292,8,293,295}, DIVIDELINE, ""}
+};
+DIALOG vhdl_optionsdialog = {{50,75,419,379}, N_("VHDL Options"), 0, 11, vhdl_optionsdialogitems};
+
+/* special items for the "VHDL Options" dialog: */
+#define DVHO_VHDLINFACET   3		/* VHDL stored in facet (check) */
+#define DVHO_NETINFACET    4		/* Netlist stored in facet (check) */
+#define DVHO_PRIMLIST      5		/* List of schematics prims (scroll) */
+#define DVHO_UNNEGATEDNAME 8		/* Unnegated name (edit text) */
+#define DVHO_NEGATEDNAME   9		/* Negated name (edit text) */
+
 void vhdl_optionsdlog(void)
 {
 	INTBIG itemHit;
@@ -2906,7 +2971,7 @@ void vhdl_optionsdlog(void)
 
 	/* setup for generated names in the technology */
 	tech = sch_tech;
-	DiaInitTextDialog(5, DiaNullDlogList, DiaNullDlogItem, DiaNullDlogDone, 0,
+	DiaInitTextDialog(DVHO_PRIMLIST, DiaNullDlogList, DiaNullDlogItem, DiaNullDlogDone, 0,
 		SCSELMOUSE|SCREPORT);
 	var = getval((INTBIG)tech, VTECHNOLOGY, VSTRING|VISARRAY, "TECH_vhdl_names");
 	if (var == NOVARIABLE)
@@ -2942,12 +3007,12 @@ void vhdl_optionsdlog(void)
 			(void)addstringtoinfstr(varnames[i]);
 			(void)addstringtoinfstr(")");
 		}
-		DiaStuffLine(5, returninfstr());
+		DiaStuffLine(DVHO_PRIMLIST, returninfstr());
 	}
-	DiaSelectLine(5, 0);
+	DiaSelectLine(DVHO_PRIMLIST, 0);
 
-	if (vhdlondisk == 0) DiaSetControl(3, 1);
-	if (netlistondisk == 0) DiaSetControl(4, 1);
+	if (vhdlondisk == 0) DiaSetControl(DVHO_VHDLINFACET, 1);
+	if (netlistondisk == 0) DiaSetControl(DVHO_NETINFACET, 1);
 
 	/* loop until done */
 	technameschanged = 0;
@@ -2955,38 +3020,38 @@ void vhdl_optionsdlog(void)
 	{
 		itemHit = DiaNextHit();
 		if (itemHit == OK || itemHit == CANCEL) break;
-		if (itemHit >= 3 && itemHit <= 4)
+		if (itemHit == DVHO_VHDLINFACET || itemHit == DVHO_NETINFACET)
 		{
 			DiaSetControl(itemHit, 1 - DiaGetControl(itemHit));
 			continue;
 		}
-		if (itemHit == 5)
+		if (itemHit == DVHO_PRIMLIST)
 		{
-			DiaSetText(8, "");
-			DiaSetText(9, "");
-			i = DiaGetCurLine(5);
+			DiaSetText(DVHO_UNNEGATEDNAME, "");
+			DiaSetText(DVHO_NEGATEDNAME, "");
+			i = DiaGetCurLine(DVHO_PRIMLIST);
 			pt = varnames[i];
 			for(opt = pt; *opt != 0; opt++) if (*opt == '/') break;
 			if (*opt == '/')
 			{
 				*opt = 0;
-				DiaSetText(8, pt);
-				DiaSetText(9, &opt[1]);
+				DiaSetText(DVHO_UNNEGATEDNAME, pt);
+				DiaSetText(DVHO_NEGATEDNAME, &opt[1]);
 				*opt = '/';
 			} else
 			{
-				DiaSetText(8, pt);
-				DiaSetText(9, pt);
+				DiaSetText(DVHO_UNNEGATEDNAME, pt);
+				DiaSetText(DVHO_NEGATEDNAME, pt);
 			}
 			continue;
 		}
-		if (itemHit == 8 || itemHit == 9)
+		if (itemHit == DVHO_UNNEGATEDNAME || itemHit == DVHO_NEGATEDNAME)
 		{
-			i = DiaGetCurLine(5);
+			i = DiaGetCurLine(DVHO_PRIMLIST);
 			(void)initinfstr();
-			(void)addstringtoinfstr(DiaGetText(8));
+			(void)addstringtoinfstr(DiaGetText(DVHO_UNNEGATEDNAME));
 			(void)addtoinfstr('/');
-			(void)addstringtoinfstr(DiaGetText(9));
+			(void)addstringtoinfstr(DiaGetText(DVHO_NEGATEDNAME));
 			pt = returninfstr();
 			if (strcmp(varnames[i], pt) == 0) continue;
 
@@ -3001,17 +3066,17 @@ void vhdl_optionsdlog(void)
 				(void)addstringtoinfstr(varnames[i]);
 				(void)addstringtoinfstr(")");
 			}
-			DiaSetScrollLine(5, i, returninfstr());
+			DiaSetScrollLine(DVHO_PRIMLIST, i, returninfstr());
 			continue;
 		}
 	}
 
 	if (itemHit != CANCEL)
 	{
-		if (DiaGetControl(3) != 0) i = 0; else i = 1;
+		if (DiaGetControl(DVHO_VHDLINFACET) != 0) i = 0; else i = 1;
 		if (i != vhdlondisk)
 			setval((INTBIG)vhdl_aid, VAID, "VHDL_vhdl_on_disk", i, VINTEGER);
-		if (DiaGetControl(4) != 0) i = 0; else i = 1;
+		if (DiaGetControl(DVHO_NETINFACET) != 0) i = 0; else i = 1;
 		if (i != netlistondisk)
 			setval((INTBIG)vhdl_aid, VAID, "VHDL_netlist_on_disk", i, VINTEGER);
 		if (technameschanged != 0)
