@@ -78,7 +78,7 @@ void vhdl_genals(LIBRARY *lib, NODEPROTO *basenp)
 		"t: delta=0",
 		0
 	};
-	static char *PMOStran[] =
+	static char *pMOStran[] =
 	{
 		"function PMOStran(g, a1, a2)",
 		"i: g, a1, a2",
@@ -86,6 +86,14 @@ void vhdl_genals(LIBRARY *lib, NODEPROTO *basenp)
 		"t: delta=1e-8",
 		0
 	};
+	static char *pMOStranWeak[] =
+	{
+		"function pMOStranWeak(g, a1, a2)",
+		"i: g, a1, a2",
+		"o: a1, a2",
+		"t: delta=1e-8",
+		0
+	};
 	static char *nMOStran[] =
 	{
 		"function nMOStran(g, a1, a2)",
@@ -94,6 +102,14 @@ void vhdl_genals(LIBRARY *lib, NODEPROTO *basenp)
 		"t: delta=1e-8",
 		0
 	};
+	static char *nMOStranWeak[] =
+	{
+		"function nMOStranWeak(g, a1, a2)",
+		"i: g, a1, a2",
+		"o: a1, a2",
+		"t: delta=1e-8",
+		0
+	};
 	static char *inverter[] =
 	{
 		"gate inverter(a,z)",
@@ -226,7 +242,12 @@ void vhdl_genals(LIBRARY *lib, NODEPROTO *basenp)
 		/* now see if this is a reference to a function primitive */
 		if (strcmp(gate, "PMOStran") == 0)
 		{
-			vhdl_dumpfunction(gate, PMOStran);
+			vhdl_dumpfunction(gate, pMOStran);
+			ulist->numref = 0;
+			total--;
+		} else if (strcmp(gate, "pMOStranWeak") == 0)
+		{
+			vhdl_dumpfunction(gate, pMOStranWeak);
 			ulist->numref = 0;
 			total--;
 		} else if (strcmp(gate, "nMOStran") == 0)
@@ -234,6 +255,11 @@ void vhdl_genals(LIBRARY *lib, NODEPROTO *basenp)
 			vhdl_dumpfunction(gate, nMOStran);
 			ulist->numref = 0;
 			total--;
+		} else if (strcmp(gate, "nMOStranWeak") == 0)
+		{
+			vhdl_dumpfunction(gate, nMOStranWeak);
+			ulist->numref = 0;
+			total--;
 		} else if (strcmp(gate, "inverter") == 0)
 		{
 			vhdl_dumpfunction(gate, inverter);
