@@ -38,7 +38,6 @@
 
 extern TOKENLIST	*vhdl_nexttoken;
 extern SYMBOLLIST	*vhdl_symbols;
-extern AIDENTRY		*vhdl_aid;
 
 /* prototypes for local routines */
 MRELATIONS *vhdl_parsemorerelations(INTBIG, INTBIG);
@@ -95,7 +94,8 @@ EXPRESSION  *vhdl_parseexpression(void)
 	logop = NOLOGOP;
 	if (vhdl_nexttoken->token == TOKEN_KEYWORD)
 	{
-		switch ( key = ((VKEYWORD *)(vhdl_nexttoken->pointer))->num )
+		key = ((VKEYWORD *)(vhdl_nexttoken->pointer))->num;
+		switch (key = ((VKEYWORD *)(vhdl_nexttoken->pointer))->num)
 		{
 			case KEY_AND:
 				logop = LOGOP_AND;
@@ -151,7 +151,7 @@ MRELATIONS  *vhdl_parsemorerelations(INTBIG key, INTBIG logop)
 
 	more = 0;
 
-	if ( vhdl_keysame(vhdl_nexttoken, key) )
+	if (vhdl_keysame(vhdl_nexttoken, key))
 	{
 		vhdl_getnexttoken();
 		more = (MRELATIONS *)emalloc((INTBIG)sizeof(MRELATIONS), vhdl_aid->cluster);
@@ -492,7 +492,7 @@ PRIMARY  *vhdl_parseprimary(void)
 			/* should be at right bracket */
 			if (vhdl_nexttoken->token != TOKEN_RIGHTBRACKET)
 			{
-				vhdl_reporterrormsg(vhdl_nexttoken, "Expecting a right bracket");
+				vhdl_reporterrormsg(vhdl_nexttoken, _("Expecting a right bracket"));
 			}
 			vhdl_getnexttoken();
 			break;
@@ -902,9 +902,9 @@ INTBIG vhdl_evalname(NAME *name)
 
 	if (name == 0) return(0);
 	value = 0;
-	if ((symbol = vhdl_searchsymbol( vhdl_getnameident(name), vhdl_symbols )) == 0)
+	if ((symbol = vhdl_searchsymbol(vhdl_getnameident(name), vhdl_symbols)) == 0)
 	{
-		vhdl_reporterrormsg(vhdl_getnametoken(name), "Symbol is undefined");
+		vhdl_reporterrormsg(vhdl_getnametoken(name), _("Symbol is undefined"));
 		return(value);
 	}
 	if (symbol->type == SYMBOL_VARIABLE)
@@ -915,7 +915,7 @@ INTBIG vhdl_evalname(NAME *name)
 		value = (INTBIG)symbol->pointer;
 	} else
 	{
-		vhdl_reporterrormsg(vhdl_getnametoken(name), "Cannot evaluate value of symbol");
+		vhdl_reporterrormsg(vhdl_getnametoken(name), _("Cannot evaluate value of symbol"));
 		return(value);
 	}
 	return(value);
@@ -1081,7 +1081,7 @@ SINGLENAME  *vhdl_parsesinglename(void)
 				/* should be at right bracket */
 				if (vhdl_nexttoken->token != TOKEN_RIGHTBRACKET)
 				{
-					vhdl_reporterrormsg(vhdl_nexttoken, "Expecting a right bracket");
+					vhdl_reporterrormsg(vhdl_nexttoken, _("Expecting a right bracket"));
 				}
 				vhdl_getnexttoken();
 				break;
@@ -1125,7 +1125,7 @@ SIMPLENAME  *vhdl_parsesimplename(void)
 	sname = NULL;
 	if (vhdl_nexttoken->token != TOKEN_IDENTIFIER)
 	{
-		vhdl_reporterrormsg(vhdl_nexttoken, "Expecting an identifier");
+		vhdl_reporterrormsg(vhdl_nexttoken, _("Expecting an identifier"));
 		vhdl_getnexttoken();
 		return(sname);
 	}
@@ -1499,12 +1499,12 @@ DBNAME  *vhdl_semindexedname(INDEXEDNAME *name)
 	type = vhdl_gettype(vhdl_getprefixident(name->prefix));
 	if (type == NULL)
 	{
-		vhdl_reporterrormsg(vhdl_getprefixtoken(name->prefix), "No type specified");
+		vhdl_reporterrormsg(vhdl_getprefixtoken(name->prefix), _("No type specified"));
 		return(dbname);
 	}
 	if (type->type != DBTYPE_ARRAY)
 	{
-		vhdl_reporterrormsg(vhdl_getprefixtoken(name->prefix), "Must be of constrained array type");
+		vhdl_reporterrormsg(vhdl_getprefixtoken(name->prefix), _("Must be of constrained array type"));
 		return(dbname);
 	}
 	dbname = (DBNAME *)emalloc((INTBIG)sizeof(DBNAME), vhdl_aid->cluster);
@@ -1519,9 +1519,9 @@ DBNAME  *vhdl_semindexedname(INDEXEDNAME *name)
 	for (expr = name->expr_list; expr && indexr; expr = expr->next)
 	{
 		value = vhdl_evalexpression(expr->expression);
-		if ( !vhdl_indiscreterange((INTBIG)value, indexr->drange) )
+		if (!vhdl_indiscreterange((INTBIG)value, indexr->drange))
 		{
-			vhdl_reporterrormsg(vhdl_getprefixtoken(name->prefix), "Index is out of range");
+			vhdl_reporterrormsg(vhdl_getprefixtoken(name->prefix), _("Index is out of range"));
 			return(dbname);
 		}
 		nexpr = (DBEXPRLIST *)emalloc((INTBIG)sizeof(DBEXPRLIST), vhdl_aid->cluster);
