@@ -67,6 +67,7 @@ INTBIG vhdl_parseport_mode(void);
 APORTLIST *vhdl_parseactual_port_list(void);
 PACKAGE *vhdl_parsepackage(void);
 PACKAGEDPART *vhdl_parsepackage_declare_part(void);
+void vhdl_parsetosemicolon(void);
 #ifdef VHDL50
   WITH *vhdl_parsewith(void);
 #endif
@@ -124,6 +125,9 @@ INTSML vhdl_parser(TOKENLIST *tlist)
 			type = NOUNIT;
 			switch (((VKEYWORD *)(vhdl_nexttoken->pointer))->num)
 			{
+				case KEY_LIBRARY:
+					vhdl_parsetosemicolon();
+					break;
 				case KEY_ENTITY:
 					type = UNIT_INTERFACE;
 					pointer = (char *)vhdl_parseinterface();
@@ -238,7 +242,7 @@ void vhdl_reporterrormsg(TOKENLIST *tlist, char *err_msg)
 			default:
 				if (tstart->pointer != 0)
 				{
-					sprintf(stemp, "%s", tstart->pointer);
+					sprintf(stemp, "%ld", tstart->pointer);
 					strcat(buffer, stemp);
 				}
 				break;
@@ -1643,6 +1647,7 @@ APORTLIST  *vhdl_parseactual_port_list(void)
 	APORTLIST *aplist, *lastport, *newport;
 
 	aplist = lastport = NULL;
+
 	/* should be at least one port association */
 	aplist = (APORTLIST *)emalloc((INTBIG)sizeof(APORTLIST), vhdl_aid->cluster);
 	aplist->type = APORTLIST_NAME;
@@ -1650,14 +1655,21 @@ APORTLIST  *vhdl_parseactual_port_list(void)
 		vhdl_nexttoken->token != TOKEN_RIGHTBRACKET)
 	{
 #ifndef VHDL50
-		if (vhdl_keysame(vhdl_nexttoken,(INTBIG)KEY_OPEN))
+		if (vhdl_keysame(vhdl_nexttoken, (INTBIG)KEY_OPEN))
 		{
-				aplist->pointer = NULL;
-				vhdl_getnexttoken();
-		}
-		else
+			aplist->pointer = NULL;
+			vhdl_getnexttoken();
+		} else
 #endif
-		aplist->pointer = (char *)vhdl_parsename();
+		{
+			aplist->pointer = (char *)vhdl_parsename();
+			if (vhdl_nexttoken->token == TOKEN_ARROW)
+			{
+				vhdl_getnexttoken();
+				aplist->pointer = (char *)vhdl_parsename();
+			}
+		}
+
 	}
 #ifdef VHDL50
 	else
@@ -1683,10 +1695,16 @@ APORTLIST  *vhdl_parseactual_port_list(void)
 			{
 				newport->pointer = NULL;
 				vhdl_getnexttoken();
-			}
-			else
+			} else
 #endif
-			newport->pointer = (char *)vhdl_parsename();
+			{
+				newport->pointer = (char *)vhdl_parsename();
+				if (vhdl_nexttoken->token == TOKEN_ARROW)
+				{
+					vhdl_getnexttoken();
+					newport->pointer = (char *)vhdl_parsename();
+				}
+			}
 		}
 #ifdef VHDL50
 		else
@@ -1704,6 +1722,22 @@ APORTLIST  *vhdl_parseactual_port_list(void)
 	return(aplist);
 }
 
+/*
+ * ignore up to the next semicolon.
+ */
+void vhdl_parsetosemicolon(void)
+{
+	for(;;)
+	{
+		vhdl_getnexttoken();
+		if (vhdl_nexttoken->token == TOKEN_SEMICOLON)
+		{
+			vhdl_getnexttoken();
+			break;
+		}
+	}
+}
+
 /*
 Module:  vhdl_parsepackage
 ------------------------------------------------------------------------
@@ -1976,13 +2010,27 @@ USE *vhdl_parseuse(void)
 	vhdl_getnexttoken();
 #ifndef VHDL50
 	/* IEEE version uses form unit.ALL only */
-	if (vhdl_nexttoken->token == TOKEN_PERIOD)
+	for(;;)
+	{
+		if (vhdl_nexttoken->token != TOKEN_PERIOD)
+		{
+			vhdl_reporterrormsg(vhdl_nexttoken, _("Expecting period"));
+			break;
+		}
 		vhdl_getnexttoken();
-	else vhdl_reporterrormsg(vhdl_nexttoken, _("Expecting period"));
 
-	if (vhdl_keysame(vhdl_nexttoken,(INTBIG)KEY_ALL))
+		if (vhdl_keysame(vhdl_nexttoken, (INTBIG)KEY_ALL))
+		{
+			vhdl_getnexttoken();
+			break;
+		}
+		if (vhdl_nexttoken->token != TOKEN_IDENTIFIER)
+		{
+			vhdl_reporterrormsg(vhdl_nexttoken, _("Bad unit name for use clause"));
+			break;
+		}
 		vhdl_getnexttoken();
-	else vhdl_reporterrormsg(vhdl_nexttoken, _("Expecting keyword ALL"));
+	}
 #endif
 
 	while (vhdl_nexttoken->token == TOKEN_COMMA)
