@@ -34,6 +34,7 @@
 
 #include "global.h"
 #include "vhdl.h"
+#include "sc1.h"
 
 extern INTBIG		vhdl_externentities, vhdl_warnflag;
 extern DBUNITS		*vhdl_units;
@@ -91,6 +92,7 @@ void vhdl_genquisc(void)
 	char *strtime, *pn, *key;
 	UINTBIG tsec;
 	INTSML total;
+	REGISTER LIBRARY *celllib;
 	REGISTER CELL *cell;
 	DBINTERFACE *top_interface, *interfacef;
 	UNRESLIST *ulist;
@@ -124,11 +126,12 @@ void vhdl_genquisc(void)
 	vhdl_printoneline("!********* End of command file *******************");
 
 	/* scan unresolved references for reality inside of Electric */
+	celllib = (LIBRARY *)askaid(sc_aid, "cell-lib");
+	if (celllib == 0) celllib = NOLIBRARY;
 	total = 0;
 	for (ulist = vhdl_unresolved_list; ulist != NULL; ulist = ulist->next)
 	{
 		/* see if this is a reference to a cell in the current library */
-		total++;
 		for(cell = el_curlib->firstcell; cell != NOCELL; cell = cell->nextcell)
 		{
 			(void)initinfstr();
@@ -138,9 +141,24 @@ void vhdl_genquisc(void)
 			key = returninfstr();
 			if (namesame(ulist->interfacef->string, key) == 0) break;
 		}
-		if (cell == NOCELL) continue;
-		ulist->numref = 0;
-		total--;
+		if (cell == NOCELL && celllib != NOLIBRARY)
+		{
+			for(cell = celllib->firstcell; cell != NOCELL; cell = cell->nextcell)
+			{
+				(void)initinfstr();
+				for(pn = cell->cellname; *pn != 0; pn++)
+					if (isalnum(*pn)) (void)addtoinfstr(*pn); else
+						(void)addtoinfstr('_');
+				key = returninfstr();
+				if (namesame(ulist->interfacef->string, key) == 0) break;
+			}
+		}
+		if (cell != NOCELL)
+		{
+			ulist->numref = 0;
+			continue;
+		}
+		total++;
 	}
 
 	/* print unresolved reference list */
