@@ -360,6 +360,16 @@ void vhdl_createdefaulttype(SYMBOLLIST *symbols)
 		if (ikey == 0) return;
 	} else efree(bitname);
 	vhdl_addsymbol(ikey, (INTBIG)SYMBOL_TYPE, (char *)0, symbols);
+
+	/* type "std_logic" */
+	bitname = (char *)emalloc(10, vhdl_aid->cluster);
+	(void)strcpy(bitname, "std_logic");
+	if ((ikey = vhdl_findidentkey(bitname)) == 0)
+	{
+		ikey = vhdl_makeidentkey(bitname);
+		if (ikey == 0) return;
+	} else efree(bitname);
+	vhdl_addsymbol(ikey, (INTBIG)SYMBOL_TYPE, (char *)0, symbols);
 }
 
 /*
@@ -1159,7 +1169,7 @@ DBSTATEMENTS *vhdl_semgenerate(GENERATE *gen)
 		{
 			vhdl_addsymbol((IDENTTABLE *)gen->label->pointer, (INTBIG)SYMBOL_LABEL, (char *)0,
 				vhdl_symbols);
-			strcpy(vhdl_default_name,((IDENTTABLE *)(gen->label->pointer))->string);
+			strcpy(vhdl_default_name, ((IDENTTABLE *)(gen->label->pointer))->string);
 #ifdef VHDL50
 			vhdl_default_num = 0;
 #endif
@@ -1320,7 +1330,9 @@ void vhdl_semuse(USE *use)
 		symbol = vhdl_searchsymbol((IDENTTABLE *)use->unit->pointer, vhdl_gsymbols);
 		if (symbol == 0)
 		{
+#if 0		/* allow undefined "use" keywords */
 			vhdl_reporterrormsg(use->unit, _("Symbol is undefined"));
+#endif
 			continue;
 		}
 		if (symbol->type != SYMBOL_PACKAGE)
